﻿(function cloudPanoIFrameInit() {
"use strict";
// Get local compartment info
var localCompartment = null;
var localCompartmentOrigin = "ms-appx://" + ((document && document.location && document.location.host) ? document.location.host : "");
var localCompartmentId = null;
// Helper method to send message to local context
function sendMessageToLocalContext (msg) {
// Validate
if (document && localCompartment && localCompartmentOrigin && localCompartmentId) {
// Add top element in message
msg.id = localCompartmentId;
try {
// Post message to local compartment
localCompartment.postMessage(msg, localCompartmentOrigin);
} catch (error) {
// Failed to post message to local compartment
// should only happen if we're already disposed
}
}
}
// Helper method to append text css element
function appendTextCss(head, textCss) {
// Validate
if (head && textCss) {
var stylesElement = document.createElement("style");
stylesElement.type = "text/css";
if (stylesElement.styleSheet) {
stylesElement.styleSheet.cssText = textCss;
} else {
stylesElement.appendChild(document.createTextNode(textCss));
}
head.appendChild(stylesElement);
}
}
// Helper method to send loaded event
function sendLoadedEvent() {
// Send click message
var msg = { type: "loaded" };
sendMessageToLocalContext(msg);
}
// Helper method to send interaction events
function sendInteractionEvent(event) {
// Send interaction message
var msg = {
type: "interaction",
event : {
type: event.type,
detail: event.detail,
screenX: event.screenX,
screenY: event.screenY,
clientX: event.clientX,
clientY: event.clientY,
altKey: event.altKey,
ctrlKey: event.ctrlKey,
shiftKey: event.shiftKey,
metaKey: event.metaKey,
button: event.button
}
};
sendMessageToLocalContext(msg);
}
// Helper method to send click event
function sendClickEvent(event) {
// Send click message
var msg = { type: "click"};
sendMessageToLocalContext(msg);
}
// Helper method to setup content div events
function setupContentEvents(element) {
// Validate
if (element) {
// Setup interaction events
element.addEventListener("MSLostPointerCapture", sendInteractionEvent, false);
element.addEventListener("MSPointerCancel", sendInteractionEvent, false);
element.addEventListener("MSPointerUp", sendInteractionEvent, false);
element.addEventListener("MSPointerDown", sendInteractionEvent, false);
element.addEventListener("MSPointerOut", sendInteractionEvent, false);
// Setup click event
element.addEventListener("click", sendClickEvent, false);
}
}
// Helper method to clear content div events
function clearContentEvents(element) {
// Validate
if (element) {
// Setup interaction events
element.removeEventListener("MSLostPointerCapture", sendInteractionEvent);
element.removeEventListener("MSPointerCancel", sendInteractionEvent);
element.removeEventListener("MSPointerUp", sendInteractionEvent);
element.removeEventListener("MSPointerDown", sendInteractionEvent);
element.removeEventListener("MSPointerOut", sendInteractionEvent);
// Setup click event
element.removeEventListener("click", sendClickEvent);
}
}
// Method to load iframe
function load(message) {
// Validate message
if (message && message.html) {
// Get local compartment ID
localCompartmentId = message.id;
// Validate
if (document && document.head && document.body) {
// Get link elements
var headElement = document.head;
var linkElements = headElement.getElementsByTagName("link");
// Get content div
var contentDiv = document.getElementById("cloudPanoIFrameContent");
if (linkElements && contentDiv) {
// Set ui theme
var uiStylesElement = document.createElement("link");
if (uiStylesElement) {
uiStylesElement.setAttribute("rel", "stylesheet");
uiStylesElement.setAttribute("type", "text/css");
if (message.uiTheme &&
message.uiTheme === "light") {
uiStylesElement.setAttribute("href", "//Microsoft.WinJS.1.0/css/ui-light.css");
}
else {
uiStylesElement.setAttribute("href", "//Microsoft.WinJS.1.0/css/ui-dark.css");
}
if (linkElements.length > 0) {
headElement.insertBefore(uiStylesElement, linkElements[0]);
} else {
headElement.appendChild(uiStylesElement);
}
}
// Append common css if any
appendTextCss(headElement, message.commonCss);
// Append custom css if any
appendTextCss(headElement, message.customCss);
// Set html
contentDiv.innerHTML = toStaticHTML(message.html);
// Set content inline css if any
if (message.elementCss) {
document.body.style.cssText = message.elementCss;
}
// Send loaded event to local compartment
sendLoadedEvent();
// Setup content events
setupContentEvents(contentDiv);
}
}
}
}
// Method to handle messages from host
function handleMessage(message) {
// Validate sender
if (localCompartmentOrigin &&
message &&
message.origin &&
message.origin === localCompartmentOrigin) {
// If local compartment isn't yet set
if (!localCompartment) {
localCompartment = message.source;
}
// Validate message
if (message.data && message.data.type) {
// If this is a load message
var messageType = message.data.type;
if (messageType === "load") {
// Load iframe based on message content
load(message.data);
}
else if (messageType === "dispose") {
// Dispose of iframe
dispose(message.data);
}
}
}
}
// Method to dispose of iframe
function dispose(message) {
// Remove message handler
window.removeEventListener("message", handleMessage);
// Get content div
var contentDiv = document.getElementById("cloudPanoIFrameContent");
if (contentDiv) {
// Remove content div event listeners
clearContentEvents(contentDiv);
}
}
// Listen on messages from host
window.addEventListener("message", handleMessage, false);
})();