﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: AdsConfig.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformAds_ConfigInit() {
"use strict";
WinJS.Namespace.define("PlatformJS.Ads", {
Config: WinJS.Class.define(
function adsConfig_ctor() {
/// <summary>Class for retrieving the ads configuration</summary>
var adsConfig = PlatformJS.Services.configuration.getDictionary("Ads2");
var that = this;
this.applicationId = adsConfig.getString("ApplicationID", "");
if (this.applicationId === "") {
this.applicationId = null;
}
this._populateAdsData();
var haveResetOnSuspend = false;
haveResetOnSuspend |= this._populateAdClusters(adsConfig);
if (haveResetOnSuspend) {
Windows.UI.WebUI.WebUIApplication.addEventListener("suspending", function ads_onSuspending(event) {
that.resetAdsCounter(that.adClusters);
});
}
// Read ad settings
var adSettings = adsConfig.getDictionary("AdSettings");
if (adSettings) {
// Set enable ad label flag
this._enableAdLabel = adSettings.getBool("EnableAdLabel", false);
// Read refresh interval in seconds
this._refreshPeriodInSeconds = adSettings.getInt32("RefreshPeriodInSeconds", 120);
this._allowDisablingArticleReaderAds = adSettings.getBool("CanDisableArticleReaderAds", true);
}
},
{
//#region Private Fields
_enableAdLabel: false,
_refreshPeriodInSeconds: 60,
_hasVerifiedArticleReaderAds: false,
_allowDisablingArticleReaderAds: false,
_articleReaderVerificationDelay: 2000,
//#endregion Private Fields
//#region Public variables and getters
disableArticleReaderAds: false,
pageAdsConfigs: {},
adsData: null,
/// <field>Dictionary of ad clusters keyed on channel ids</field>
adClusters: null,
/// <field>Ad SDK application ID</field>
applicationId: null,
// Flag that specifies whether ad label should be shown or not
enableAdLabel: {
get: function get() {
return this._enableAdLabel;
}
},
// Controls how often the ad refreshes
refreshPeriodInSeconds: {
get: function get() {
return this._refreshPeriodInSeconds;
}
},
//#endregion Public Variables and Getters
//#region Public Functions
resetAdsCounter: function resetAdsCounter(adConfigs) {
/// <summary>Reset the ads config</summary>
/// <param name="adConfigs">The ad config list</param>
for (var adClusterId in adConfigs) {
var adCluster = adConfigs[adClusterId];
if (adCluster.resetOnSuspend) {
adCluster.adDisplayController.reset();
}
}
},
saveAdsData: function saveAdsData() {
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
localSettings.values["adsData"] = this.adsData;
},
verifyArticleReaderAds: function verifyArticleReaderAds(pageId) {
var that = this;
WinJS.Promise.timeout(this._articleReaderVerificationDelay).done(function configureArticleReaderConfig() {
if (that._allowDisablingArticleReaderAds && that._hasVerifiedArticleReaderAds === false) {
var pageConfig = that.pageAdsConfigs[pageId];
if (pageConfig) {
that._hasVerifiedArticleReaderAds = true; // if CMS data has already come once, then we don't want to verify again
var articleReaderAdsConfig = pageConfig.articleReaderAds;
if (Array.isArray(articleReaderAdsConfig)) {
var tileConfig = null;
for (var i = 0; i < articleReaderAdsConfig.length; i++) {
if (articleReaderAdsConfig[i].type === "tile") {
tileConfig = articleReaderAdsConfig[i];
break;
}
}
if (tileConfig) {
var adcontrol = new PlatformJS.Ads.AdsFetchProxyControl({
adUnitId: tileConfig.adUnitId,
className: tileConfig.adType
});
adcontrol.errorHandler = function() {
that.disableArticleReaderAds = true;
adcontrol.dispose();
};
adcontrol.readyHandler = function() {
adcontrol.dispose();
};
}
}
}
}
});
},
//#endregion Public Functions
//#region Private Functions
_populateAdsData: function _populateAdsData() {
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
this.adsData = localSettings.values["adsData"];
if (this.adsData) {
var iterator = this.adsData.first();
while (iterator.hasCurrent) {
try {
this.pageAdsConfigs[iterator.current.key] = new PlatformJS.Ads.PageAdsConfig(JSON.parse(iterator.current.value));
} catch(e) {
// Skip as the error is not recoverable when data saved in the localSettings is corrupt
} 
iterator.moveNext();
}
} else {
this.adsData = new Windows.Storage.ApplicationDataCompositeValue();
}
},
_populateAdClusters: function _populateAdClusters(adsConfig) {
var haveResetOnSuspend = false, resetOnSuspend = null, id = null, adCounter = null, adUnitId = null;
var fragment = null, page = null, clusterKey = null, clusterPosition = null, clusterId = null, adType = null, channelId = null;
var adClusters = adsConfig.getList("AdClusters", Platform.Configuration.ListConfigurationItem.empty);
this.adClusters = {};
for (var i = 0; i < adClusters.length; i++) {
channelId = adClusters[i].getString("ChannelID", "");
adUnitId = adClusters[i].getString("AdUnitID", "");
adType = adClusters[i].getString("AdType", "");
clusterId = adClusters[i].getString("AdClusterID", "");
clusterPosition = adClusters[i].getString("AdClusterPosition", "");
clusterKey = adClusters[i].getString("AdClusterKey", "");
page = adClusters[i].getString("Page", "");
fragment = adClusters[i].getString("Fragment", "");
adCounter = adClusters[i].getInt32("AdCounter", 1);
resetOnSuspend = adClusters[i].getBool("ResetOnSuspend", false);
haveResetOnSuspend |= resetOnSuspend;
// Do some validation
if (adUnitId !== "" && adType !== "") {
id = null;
if (channelId !== "") {
id = channelId;
} else if (page !== "" && fragment !== "") {
id = page + "-" + fragment;
}
if (id) {
//// Supporting CSV for adClusterID and clusterPosition.
//// This is to prevent config bloat if same ad needs to be put multiple times on a page.
var clusterIds = clusterId.split(",");
var clusterPositions = clusterPosition.split(",");
var clusterKeys = clusterKey.split(",");
for (var j = 0; j < clusterIds.length; j++) {
var currentClusterId = clusterIds[j];
var currentClusterPosition = clusterPositions[j];
var currentClusterKey = clusterKeys[j];
this.adClusters[id + "-" + currentClusterId] = {
clusterPosition: currentClusterPosition,
clusterId: currentClusterId,
clusterKey: currentClusterKey,
adUnitId: adUnitId,
adType: adType,
resetOnSuspend: resetOnSuspend,
adDisplayController: new PlatformJS.Ads.DisplayController(adCounter)
};
}
}
}
}
return haveResetOnSuspend;
},
//#endregion Private Functions
},
{
_instance: null,
/// <field>Singleton instance</field>
instance: {
get: function get() {
if (!PlatformJS.Ads.Config._instance) {
PlatformJS.Ads.Config._instance = new PlatformJS.Ads.Config();
}
return PlatformJS.Ads.Config._instance;
}
},
clearAdsConfig: function clearAdsConfig() {
PlatformJS.Ads.Config._instance = null;
}
}
),
//#region Public Static Functions
addOrderedAdsClusterConfig: function addOrderedAdsClusterConfig(bindingList, clusterConfig, channelId) {
///<summary>Find ads using pageUId and add the ads into the bindinglist</summary>
if (!bindingList) {
return;
}
var adsOnPage = PlatformJS.Ads.getClusterAdsConfig(channelId);
if (!adsOnPage) {
return;
}
var getClusterAt = function(position) {
var cluster = null;
if (bindingList.getAt) {
cluster = bindingList.getAt(position);
} else { // handle the case where an array is sent instead of a binding list.
cluster = bindingList[position];
}
return cluster;
};
var isAdCluster = function (position) {
var cluster = getClusterAt(position);
if (cluster && cluster.clusterContent && cluster.clusterContent.contentControl === "PlatformJS.Ads.AdsWrapperControl") {
return true;
}
return false;
};
var isTempCluster = function (position) {
var cluster = getClusterAt(position);
if (cluster && cluster.isTempCluster === true) {
return true;
}
return false;
};
var skipTempClusters = function(position) {
for (var index = 0; index <= position; index++) {
if (isTempCluster(index) || isAdCluster(index)) {
position++;
}
}
return position;
};
for (var i = 0; i < adsOnPage.length; i++) {
var currentAd = adsOnPage[i];
if (!currentAd) { // Temporary solution when having old and new data format comingled.
currentAd = adsOnPage.getItem(i).data;
}
var adCluster = PlatformJS.Ads._prepareAdsClusterConfig(bindingList, clusterConfig, currentAd.clusterId, channelId);
if (adCluster) {
var clusterPosition = bindingList.length;
if (adCluster.clusterPosition && adCluster.clusterPosition < bindingList.length) {
clusterPosition = skipTempClusters(adCluster.clusterPosition);
}
if (clusterPosition >= bindingList.length) {
clusterPosition = bindingList.length;
var lastCluster = getClusterAt(clusterPosition - 1);
// If the ad cluster is going to be the last cluster and the last cluster has isLastCluster set to true, 
// insert the ad cluster before the last cluster.
if (lastCluster && lastCluster.isLastCluster === true) {
clusterPosition = clusterPosition - 1;
}
}
// only add ad cluster if we have no adjacent ad cluster.
if (!isAdCluster(clusterPosition) && !isAdCluster(clusterPosition - 1)) {
bindingList.splice(clusterPosition, 0, adCluster);
if (clusterConfig) {
clusterConfig = JSON.parse(JSON.stringify(clusterConfig)); // clone clusterConfig in case there are more than one ads cluster
}
}
}
}
if (clusterConfig) {
clusterConfig = null; // avoid memory leak
}
},
addAdsClusterConfig: function addAdsClusterConfig(bindingList, clusterConfig, adClusterId) {
/// <summary>Add all ads cluster to the binding list if adClusterId is not defined.
///          Add the ads cluster if adClusterId is defined
///          Assume sorting is done by the caller
/// </summary>
/// <param name="bindingList">List to add ads cluster to</param>
/// <param name="clusterConfig" optional="true">Cluster to config to extend</param>
/// <param name="adClusterId" optional="true">Ad cluster ID</param>
var addSingleCluster = function(id) {
var adsCluster = PlatformJS.Ads._prepareAdsClusterConfig(bindingList, clusterConfig, id);
if (adsCluster) {
bindingList.push(adsCluster);
}
};
var pageAdsConfig = PlatformJS.Ads.getCMSConfig();
if (!pageAdsConfig) {
addSingleCluster(adClusterId);
} else if (pageAdsConfig.clusterAds && Array.isArray(pageAdsConfig.clusterAds)) {
for (var clusterId in pageAdsConfig.clusterAds) {
addSingleCluster(clusterId);
if (clusterConfig) {
clusterConfig = JSON.parse(JSON.stringify(clusterConfig)); // duplicate clusterConfig in case there are more than one ads cluster
}
}
if (clusterConfig) {
clusterConfig = null; // avoid memory leak
}
}
},
setAdConfig: function setAdConfig(ads) {
var pageId = WinJS.Navigation.location.channelId;
if (!pageId) {
pageId = WinJS.Navigation.location.page + "-" + WinJS.Navigation.location.fragment;
}
var adsConfig = PlatformJS.Ads.Config.instance;
if (ads) {
var stringVal = JSON.stringify(ads);
if (adsConfig.adsData[pageId] !== stringVal) {
adsConfig.pageAdsConfigs[pageId] = new PlatformJS.Ads.PageAdsConfig(ads);
adsConfig.adsData[pageId] = stringVal;
adsConfig.saveAdsData();
}
} else {
if (adsConfig.adsData[pageId]) {
delete adsConfig.adsData[pageId];
adsConfig.saveAdsData();
}
}
adsConfig.verifyArticleReaderAds(pageId);
},
getCMSConfig: function getCMSConfig(channelId) {
var pageId = channelId || WinJS.Navigation.location.channelId;
if (!pageId) {
pageId = 0;
}
var config = PlatformJS.Ads.Config.instance.pageAdsConfigs[pageId];
if (!config) {
pageId = WinJS.Navigation.location.page + "-" + WinJS.Navigation.location.fragment;
config = PlatformJS.Ads.Config.instance.pageAdsConfigs[pageId];
}
return config;
},
getVideoAdsConfigByProvider: function getVideoAdsConfigByProvider(provider) {
var config = null;
var pageConfig = PlatformJS.Ads.getCMSConfig();
if (pageConfig) {
if (pageConfig.videoAds) {
config = pageConfig.videoAds[provider.toLowerCase().trim()];
if (config) {
config.appID = PlatformJS.Ads.Config.instance.applicationId; // AD SDK currently does not support multiple appID's in an app.
}
}
} else {
config = PlatformJS.Ads._getLegacyVideoAdsConfig(provider);
}
return config;
},
getClusterAdsConfig: function getClusterAdsConfig(channelId) {
var pageConfig = PlatformJS.Ads.getCMSConfig(channelId);
if (pageConfig) {
if (!pageConfig.clusterAds || !Array.isArray(pageConfig.clusterAds)) {
return null;
}
var clusters = new WinJS.Binding.List();
for (var key in pageConfig.clusterAds) {
clusters.push(pageConfig.clusterAds[key]);
}
clusters.sort(function sortClusterPosition(l, r) {
if (l.clusterPosition && r.clusterPosition) {
if (l.clusterPosition <= 0 || r.clusterPosition <= 0) {
if (r.clusterPosition > 0) {
return 1;
} else if (l.clusterPosition > 0) {
return -1;
} else {
return 0;
}
} else {
if (l.clusterPosition === r.clusterPosition) {
return 0;
}
if (l.clusterPosition > r.clusterPosition) { 
return 1;
} else {
return -1;
}
}
} else { // handle case when clusterPosition isn't specified. Rare
if (l.clusterPosition) {
return -1;
} else if (r.clusterPosition) {
return 1;
} else {
return 0;
}
}
});
return clusters;
} else {
return PlatformJS.Ads._oldGetAdsOnPage();
}
},
getAdsConfig: function getAdsConfig(adClusterId, channelId) {
var adConfig = null;
var pageConfig = PlatformJS.Ads.getCMSConfig(channelId);
if (pageConfig) {
if (pageConfig.clusterAds) {
if (adClusterId) {
adConfig = pageConfig.clusterAds[adClusterId];
} else {
// resolve to the first element in the list
var keys = Object.keys(pageConfig.clusterAds);
if (keys && keys.length) {
adConfig = pageConfig.clusterAds[keys[0]];
}
}
}
} else {
adClusterId = adClusterId ? adClusterId : "";
adConfig = PlatformJS.Ads._getAdsConfigForCluster(adClusterId);
}
if (!adConfig) {
return null;
}
adConfig.adDisplayController.increment();
if (!adConfig.adDisplayController.isVisible()) {
return null;
}
adConfig.adDisplayController.reset();
return adConfig;
},
getAdsClusterConfig: function getAdsClusterConfig(clusterConfig, adClusterId, channelId) {
/// <summary>Get the ads cluster configuration</summary>
/// <param name="clusterConfig" optional="true">Cluster to config to extend</param>
/// <param name="adClusterId" optional="true">Ad cluster ID</param>
/// <param name="channelId" optional="true">Target channel ID, this will override current page's channel ID</param>
var adConfig = PlatformJS.Ads.getAdsConfig(adClusterId, channelId);
if (!adConfig) {
return null;
}
if (!clusterConfig) {
clusterConfig = {};
}
clusterConfig.clusterContent = {
contentControl: "PlatformJS.Ads.AdsWrapperControl",
contentOptions: {
className: adConfig.adType,
otherAdOptions: clusterConfig.otherAdOptions,
adUnitId: adConfig.adUnitId
}
};
var clusterContentOptions = clusterConfig.clusterContent.contentOptions;
// Setup adTags for additional information such as lat/long, if otherAdOptions already
// has adTags specified, it takes precedence
if (adConfig.adTags) {
if (clusterContentOptions.otherAdOptions) {
if (clusterContentOptions.otherAdOptions.adTags) {
// save adTag only when otherAdOptions hasn't already specified the same tag
adConfig.adTags.forEach(function addOrUpdateTag(tag) {
if (tag.key) {
var foundMatchingKey = false;
clusterContentOptions.otherAdOptions.adTags.forEach(function findAndUpdateTag(targetTag) {
if (targetTag.key && targetTag.key === tag.key) {
foundMatchingKey = true;
}
});
if (!foundMatchingKey) {
clusterContentOptions.otherAdOptions.adTags.push(tag);
}
}
});
} else {
clusterContentOptions.otherAdOptions.adTags = adConfig.adTags;
}
} else {
clusterContentOptions.otherAdOptions = {
adTags: adConfig.adTags
};
}
}
if (typeof adConfig.clusterPosition === "number") {
clusterConfig.clusterPosition = adConfig.clusterPosition;
clusterConfig.clusterIndex = adConfig.clusterPosition; // Travel app uses clusterIndex to sort
} else if (adConfig.clusterPosition) {
clusterConfig.clusterPosition = adConfig.clusterPosition - 1;
clusterConfig.clusterIndex = adConfig.clusterPosition - 1; // Travel app uses clusterIndex to sort
}
// TODO: This change was removed due to FRE boot perf regressions (jitting of ads cs code)
// need to investigate better way to to do this so that we won't regress app load time
// Prefetch ad (this will trigger no instrumentation beacons
// since we make sure the ad doesn't actually render)
//clusterConfig.clusterContent.contentOptions.fetchedAd =
//    new PlatformJS.Ads.AdsFetchProxyControl(clusterConfig.clusterContent.contentOptions);
clusterConfig.supressTitleInHeader = true;
if (adConfig.getClusterConfigKey) {
clusterConfig.clusterKey = adConfig.getClusterConfigKey();
} else { // TODO: remove when ads config is fully migrated to CMS
if (adConfig.clusterKey) {
clusterConfig.clusterKey = adConfig.clusterKey;
}
if (!clusterConfig.clusterKey) {
clusterConfig.clusterKey = "AdsCluster";
if (adClusterId) {
clusterConfig.clusterKey += "_" + adClusterId;
}
}
}
if (!clusterConfig.clusterTitle) {
clusterConfig.clusterTitle = PlatformJS.Services.resourceLoader.getString("/platform/advertisement");
}
clusterConfig.platformInvisibleCluster = true;
return clusterConfig;
},
//#endregion Public Static Functions
//#region Private Static Helper Functions
_prepareAdsClusterConfig: function _prepareAdsClusterConfig(bindingList, clusterConfig, adClusterId, channelId) {
/// <summary>Add an ads cluster to the binding list</summary>
/// <param name="bindingList">List to add ads cluster to</param>
/// <param name="clusterConfig" optional="true">Cluster to config to extend</param>
/// <param name="adClusterId" optional="true">Ad cluster ID</param>
/// <param name="channelId" optional="true">Target channel ID, this will override current page's channel ID</param>
// Get the desired ads cluster
var adsCluster = PlatformJS.Ads.getAdsClusterConfig(clusterConfig, adClusterId, channelId);
var hasAd = false;
// If we have an ad to add to this cluster, check if we already have an ad
if (adsCluster) {
// Look for the cluster with the same clusterKey as the ad cluster
var foundIndex = 0;
var foundMatch = false;
var item = null;
for (; foundIndex < bindingList.length; foundIndex++) {
if (bindingList.getAt) {
item = bindingList.getAt(foundIndex);
} else {
item = bindingList[foundIndex];
}
if (item && item.clusterKey === adsCluster.clusterKey) {
foundMatch = true;
break;
}
}
if (foundMatch) {
bindingList.splice(foundIndex, 1);
}
}
return adsCluster;
},
_getAdsConfigForCluster: function _getAdsConfigForCluster(adClusterId) {
var adConfig = null;
var pageId = WinJS.Navigation.location.channelId;
adClusterId = adClusterId ? adClusterId : "";
if (pageId) {
// Make sure we are suppose to show the ad for this channel
adConfig = PlatformJS.Ads.Config.instance.adClusters[pageId + "-" + adClusterId];
}
// Couldn't find an ad based on channel id, so let's try the page/fragment combo
if (!adConfig) {
pageId = WinJS.Navigation.location.page + "-" + WinJS.Navigation.location.fragment;
adConfig = PlatformJS.Ads.Config.instance.adClusters[pageId + "-" + adClusterId];
}
return adConfig;
},
_oldGetAdsOnPage: function _oldGetAdsOnPage() {
var pageId = null;
var adsOnPage = null;
/// Try getting ads using channelId combo
if (WinJS.Navigation.location.channelId) {
pageId = WinJS.Navigation.location.channelId + "-";
adsOnPage = PlatformJS.Ads._getAdsOnPage(pageId);
}
/// Try getting ads using pageId + fragment combo
if (!adsOnPage || adsOnPage.length === 0) {
pageId = WinJS.Navigation.location.page + "-" + WinJS.Navigation.location.fragment + "-";
adsOnPage = PlatformJS.Ads._getAdsOnPage(pageId);
}
return adsOnPage;
},
_getAdsOnPage: function _getAdsOnPage(pageId) {
if (!pageId) {
return null;
}
var adsOnPage = [];
var ads = Object.keys(PlatformJS.Ads.Config.instance.adClusters);
for (var i = 0; i < ads.length; i++) {
var currentAd = ads[i];
if (currentAd.indexOf(pageId) === 0) {
adsOnPage.push(PlatformJS.Ads.Config.instance.adClusters[currentAd]);
}
}
return adsOnPage;
},
_getLegacyVideoAdsConfig: function _getLegacyVideoAdsConfig(provider) {
var configuration = PlatformJS.Services.configuration;
var videoAds = configuration.getDictionary("VideoAds");
if (videoAds) {
var adUnitId = videoAds.getString(provider);
if (adUnitId) {
return new PlatformJS.Ads.VideoAdUnit({
adUnit: {
id: adUnitId,
appID: PlatformJS.Ads.Config.instance.applicationId
},
adStartFrequency: configuration.getInt32("VideoAdsPlayTimeAtInterval", 0),
adRepeatFrequency: configuration.getInt32("VideoAdsInterval", 5),
adProvider: provider,
adServer: "mmppf"
});
}
}
return null;
},
//#endregion Private Static Helper Functions
});
})();
/* File: AdsDisplayController.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformAds_DisplayControllerInit() {
"use strict";
WinJS.Namespace.define("PlatformJS.Ads", {
/// <summary>
/// Ad display controller class, keeps counter to determine visiblity of ad
/// </summary>
DisplayController: WinJS.Class.define(function (maxCount) {
this._counter = 0;
this._maxCount = maxCount;
},
{
//#region Private Variables
_counter: 0,
_maxCount: 0,
//#endregion Private Variables
//#region Public Functions
increment: function increment() {
this._counter++;
},
reset: function reset() {
this._counter = 0;
},
isVisible: function isVisible() {
if (!MicrosoftNSJS) {
return false;
}
if (!PlatformJS.Ads.Config.instance.applicationId) {
return false;
}
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
return false;
}
var costSuggestion = Platform.Networking.NetworkManager.instance.networkCostSuggestion;
if (costSuggestion === Platform.Networking.NetworkCostSuggestion.highCost) {
return false;
}
// never visible
if (this._maxCount === 0) {
return false;
}
// always visible
if (this._maxCount === 1) {
return true;
}
if (this._counter > this._maxCount) {
return false;
}
return true;
},
//#endregion Public Functions
})
});
})();
/* File: AdsFetchProxyControl.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformAds_AdsFetchProxyControlInit() {
"use strict";
WinJS.Namespace.define("PlatformJS.Ads", {
AdsFetchProxyControl: WinJS.Class.define(function adsPrefetchProxyControl_ctor(options) {
var that = this;
this.element = document.createElement("div");
this.element.winControl = this;
this._adElement = document.createElement("div");
WinJS.Utilities.addClass(this._adElement, "platformAd");
WinJS.Utilities.addClass(this.element, "platformAdWrapper");
this.element.appendChild(this._adElement);
this._renderPromise = new WinJS.Promise(function (complete) {
that._renderPromiseSignal = complete;
});
this._setupEvents();
WinJS.UI.setOptions(this, options);
this._hideInSemanticZoom();
this.element.style.visibility = "hidden";
document.body.appendChild(this.element);
this._create();
},
{
//#region Public Variables
element: null,
otherAdOptions: null,
//#endregion Public Variables
//#region Private Variables
_adElement: null,
_adControl: null,
_backupImage: null, // Backup image from options
_adShown: false,
_handlerInvoked: false, // Bool indicating if either a completion or error handler has been invoked
_renderPromiseSignal: null,
_className: null,
_adUnitID: null,
_completionHandler: null,
_errorHandler: null,
_readyHandler: null,
_semanticZoom: null,
_zoomChanged: null,
//#endregion Private Variables
//#region Public getters/setters
className: {
get: function get() {
return this._className;
},
set: function set(value) {
this._className = value;
WinJS.Utilities.addClass(this._adElement, value);
}
},
adUnitId: {
set: function set(value) {
this._adUnitID = value;
}
},
completionHandler: {
set: function set(value) {
this._completionHandler = value;
}
},
errorHandler: {
set: function set(value) {
this._errorHandler = value;
}
},
readyHandler: {
set: function set(value) {
this._readyHandler = value;
}
},
//#endregion Public getters/setters
//#region Public Functions
render: function render() {
return WinJS.Promise.wrap({});
},
dispose: function dispose() {
if (this._adControl) {
this._adControl.dispose();
}
this._adControl = null;
if (this.element) {
try {
if (this.element.parentNode === document.body) {
document.body.removeChild(this.element);
}
} catch (error) {
// ignore error since the element is no longer part of the document
}
this.element.onmouseout = null;
this.element.onmouseover = null;
this.element = null;
}
this._backupImage = null;
if (this._semanticZoom && this._zoomChanged) {
this._semanticZoom.removeEventListener("zoomchanged", this._zoomChanged);
this._semanticZoom = null;
this._zoomChanged = null;
}
},
refresh: function refresh() {
if (this._adControl) {
this._adControl.dispose();
this._adControl = null;
}
this._adShown = false;
this._create();
},
//#endregion Public Functions
//#region Private Functions
_hideInSemanticZoom: function _hideInSemanticZoom() {
var that = this;
var semanticZoom = WinJS.Utilities.query(".win-semanticzoom");
if (semanticZoom.length > 0) {
var zoomControl = this._semanticZoom = semanticZoom[0].winControl;
var zoomChanged = this._zoomChanged = function () {
if (zoomControl.zoomedOut) {
WinJS.Utilities.addClass(that.element, "adTypeHidden");
} else {
WinJS.Utilities.removeClass(that.element, "adTypeHidden");
}
};
zoomControl.addEventListener("zoomchanged", zoomChanged);
}
},
_setupEvents: function _setupEvents() {
var that = this;
this.element.onmouseover = function () {
// If there is no backup image, then do the hover state stuff
if (!that._backupImage) {
WinJS.Utilities.addClass(that.element, "platformAdWrapperHover");
} else {
// Make sure the outline is gone
WinJS.Utilities.removeClass(that.element, "platformAdWrapperHover");
}
};
this.element.onmouseout = function () {
WinJS.Utilities.removeClass(that.element, "platformAdWrapperHover");
};
},
// Method to create ad control, must be called after all control options are set
_create: function _create() {
// Validate and create ad control
var that = this;
if (this._adUnitID && PlatformJS.Ads.Config.instance.applicationId && MicrosoftNSJS) {
var adsOptions = {
applicationId: PlatformJS.Ads.Config.instance.applicationId,
adUnitId: that._adUnitID,
onErrorOccurred: function onErrorOccurred(adControl, error) {
that._onErrorOccurred(adControl, error);
},
onAdRefreshed: function onAdRefreshed(adControl) {
that._onAdRefreshed(adControl);
},
onBeforeAdRender: function onBeforeAdRender(adControl) {
return that._onBeforeAdRender(adControl);
},
onAdRendered: function onAdRendered(adControl) {
that._onAdRendered(adControl);
},
onPointerUp: function onPointerUp() {
WinJS.UI.Animation.pointerUp(that.element).then();
},
onPointerDown: function onPointerDown() {
WinJS.UI.Animation.pointerDown(that.element).then();
}
};
if (this.otherAdOptions) {
for (var option in this.otherAdOptions) {
adsOptions[option] = this.otherAdOptions[option];
}
}
// Hide ad element
if (that._adElement && that._adElement.style) {
that._adElement.style.opacity = 0;
}
// Set ad refresh frequency in ad options
if (PlatformJS.Ads.Config._instance && PlatformJS.Ads.Config._instance.refreshPeriodInSeconds) {
adsOptions.refreshPeriodSeconds = PlatformJS.Ads.Config._instance.refreshPeriodInSeconds;
}
// Create ad control
that._adControl = new MicrosoftNSJS.Advertising.AdControl(that._adElement, adsOptions);
// Check if we need to disable ad label
if (PlatformJS.Ads.Config._instance && PlatformJS.Ads.Config._instance.enableAdLabel === false) {
that._adControl.addAdTag("adLabel", false);
}
if (this.otherAdOptions && this.otherAdOptions.adTags && this.otherAdOptions.adTags.length) {
var adTags = this.otherAdOptions.adTags;
for (var i = 0; i < adTags.length; i++) {
that._adControl.addAdTag(adTags[i].key, adTags[i].value);
}
}
}
},
_onAdRefreshed: function _onAdRefreshed(adControl) {
///<summary>Refresh handler. Destroy the backup image if created</summary>
///<param name="adControl">Ad control instance</param>
if (this._backupImage && this.element.contains(this._backupImage)) {
this.element.removeChild(this._backupImage);
this._backupImage = null;
}
if (!this._adShown) {
// On slow connections, this helps create some visual indication
// that ad is loading without being too visually disturbing
this._adElement.style.opacity = 0.5;
}
// If no handler was invoked yet
if (!this._handlerInvoked) {
// Invoke completion handler
if (this._completionHandler) {
this._completionHandler();
}
this._handlerInvoked = true;
}
},
_onBeforeAdRender: function _onBeforeAdRender(adControl) {
///<summary>Return promise that completes only when client is ready to display ad</summary>
///<param name="adControl">Ad control instance</param>
var that = this;
msSetImmediate(function () {
if (that._readyHandler) {
that._readyHandler();
}
});
if (this && this._renderPromise) {
return this._renderPromise;
}
else {
WinJS.Promise.wrap(null);
}
},
_onAdRendered: function _onAdRendered(adControl) {
if (this && this._adElement && !this._adShown) {
this._adShown = true;
WinJS.UI.executeTransition(this._adElement,
{
property: "opacity",
delay: 0,
duration: 100,
timing: "linear",
from: 0.5,
to: 1
});
}
},
_onErrorOccurred: function _onErrorOccurred(adControl, error) {
///<summary>Error handler. Will show backup static image</summary>
///<param name="adControl">Ad control instance</param>
///<param name="error">Error message</param>
// If we have already shown an ad, then ad control will show the old ad.
// So we don't need to show the backup image.
if (!this._backupImage && !this._adShown) {
this._backupImage = document.createElement("div");
WinJS.Utilities.addClass(this._backupImage, "platformBackupAd");
WinJS.Utilities.addClass(this._backupImage, this.className);
this.element.appendChild(this._backupImage);
//HACK: IE renderer does not always draw background images on screen (Windows 8 bug: #685585). Setting the opacity seems to force it to draw. (naeims)
//[BEGIN]
var imgStyle = window.getComputedStyle(this._backupImage);
this._backupImage.style.opacity = imgStyle ? imgStyle.opacity : "1";
//[END]
}
// If no handler was invoked yet
if (!this._handlerInvoked) {
// Don't invoke error handler if 
// request is still in progress
if (error.errorMessage.indexOf("request is already in progress") < 0) {
// Invoke error handler
if (this._errorHandler) {
this._errorHandler();
}
this._handlerInvoked = true;
}
}
},
//#endregion Private Functions
})
});
})();
/* File: AdsWrapperControl.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformAds_AdsWrapperControlInit() {
"use strict";
WinJS.Namespace.define("PlatformJS.Ads", {
AdsWrapperControl: WinJS.Class.define(function adsWrapperControl_ctor(div, options) {
this.element = div ? div : document.createElement("div");
this.element.winControl = this;
WinJS.UI.setOptions(this, options);
// Verify we have a fetched ad
if (options.fetchedAd) {
// Set fetched ad
this._fetchedAd = options.fetchedAd;
options.fetchedAd = null;
}
else {
// Fetch ad now
this._fetchedAd = new PlatformJS.Ads.AdsFetchProxyControl(options);
}
// Validate fetched ad element
if (this._fetchedAd.element) {
// Attach ad to current div
this.element.appendChild(this._fetchedAd.element);
// Show fetched ad element
if (this._fetchedAd.element.style) {
this._fetchedAd.element.style.visibility = "visible";
}
}
// Signal ad control to start rendering the ad
if (this._fetchedAd._renderPromiseSignal) {
this._fetchedAd._renderPromiseSignal();
}
},
{
element: null,
//#region Private Variables
_className: null,
_fetchedAd: null,
//#endregion Private Variables
/// <field>CSS class to ad to the ad control</field>
className: {
get: function get() {
return this._className;
},
set: function set(value) {
this._className = value;
WinJS.Utilities.addClass(this.element, value);
}
},
//#region Public Functions
render: function render() {
if (this._fetchedAd) {
return this._fetchedAd.render();
} else {
return WinJS.Promise.wrap({});
}
},
dispose: function dispose() {
if (this._fetchedAd) {
this._fetchedAd.dispose();
}
this.element = null;
},
refresh: function refresh() {
if (this._fetchedAd) {
this._fetchedAd.refresh();
}
}
//#endregion Public Functions
})
});
})();
/* File: RotatingAdsWrapperControl.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformAds_RotatingAdsWrapperControlInit() {
"use strict";
WinJS.Namespace.define("PlatformJS.Ads", {
RotatingAdsWrapperControl: WinJS.Class.define(function rotatingadswrapperctrl_ctor(element, options) {
/// <summary>Ads wrapper control that switches ads depending on rotation</summary>
this.element = element || document.createElement("div");
WinJS.Utilities.addClass(this.element, "platformRotatingAdsWrapper");
this.element.winControl = this;
WinJS.UI.setOptions(this, options);
},
{
//#region Public Variables
element: null,
landscapeOptions: null,
portraitOptions: null,
currentViewState: null,
//#endregion Public Variables
//#region Private Variables
_landscapeAd: null,
_portraitAd: null,
//#endregion Private Variables
//#region Public Getters/Setters
adOptions: {
set: function set(value) {
this.landscapeOptions = value.landscapeOptions;
this.portraitOptions = value.portraitOptions;
this._createAd(this.isLandscape(Windows.UI.ViewManagement.ApplicationView.value));
}
},
//#endregion Public Getters/Setters
//#region Public Functions
dispose: function dispose() {
this._disposeAd();
this.element = null;
},
onOrientationChange: function onOrientationChange(viewState) {
this._createAd(this.isLandscape(viewState));
},
isLandscape: function isLandscape(orientation) {
var viewState = Windows.UI.ViewManagement.ApplicationViewState;
return orientation === viewState.fullScreenLandscape ||
orientation === viewState.filled;
},
//#endregion Public Functions
//#region Private Functions
_disposeAd: function _disposeAd() {
if (this._landscapeAd) {
this.element.removeChild(this._landscapeAd.element);
this._landscapeAd.dispose();
this._landscapeAd = null;
}
if (this._portraitAd) {
this.element.removeChild(this._portraitAd.element);
this._portraitAd.dispose();
this._portraitAd = null;
}
},
_createAd: function _createAd(isLandscape) {
this.currentViewState = isLandscape;
if (isLandscape) {
if (!this._landscapeAd) {
var landscapeDiv = document.createElement("div");
WinJS.Utilities.addClass(landscapeDiv, "platformLandscapeAd");
this.element.appendChild(landscapeDiv);
this._landscapeAd = new PlatformJS.Ads.AdsWrapperControl(landscapeDiv, this.landscapeOptions);
}
} else {
if (!this._portraitAd) {
var portraitDiv = document.createElement("div");
WinJS.Utilities.addClass(portraitDiv, "platformPortraitAd");
this.element.appendChild(portraitDiv);
this._portraitAd = new PlatformJS.Ads.AdsWrapperControl(portraitDiv, this.portraitOptions);
}
}
}
//#endregion Private Functions
})
});
})();
/* File: AdUnit.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformAds_AdUnitInit() {
"use strict";
var _adTypes = Object.freeze({
thinTypeClass: "adTypeThin",
tileTypeClass: "adTypeTile",
thin: "thin",
tile: "tile",
getClassType: function getClassType(specType) {
switch (specType) {
case this.thin:
return this.thinTypeClass;
case this.tile:
return this.tileTypeClass;
default:
return specType;
}
}
});
// class that understands adUnit data object from CMS, also used as ad unit specification for article reader ads
// sample data contract: {id: "12345", appID: "0ca8c02e-2c04-439a-8fa3-7c4576c05bdb", height: 600, width: 300, adTags: [{key: "key1", value: "value1"}, {key: "key2", value: "value2"}, ...]}
WinJS.Namespace.define("PlatformJS.Ads", {
AdUnit: WinJS.Class.define(
function adunit_ctor(adSpec) {
if (adSpec.id) {
this.adUnitId = adSpec.id;
}
if (adSpec.appID) {
this.appID = adSpec.appID;
}
this.adHeight = adSpec.height;
this.adWidth = adSpec.width;
if (adSpec.type) {
var type = adSpec.type.toLowerCase().trim();
this.adType = _adTypes.getClassType(type);
this.type = type;
}
if (adSpec.adTags) {
this._setupAdTags(adSpec.adTags);
}
},
{
type: null,
adUnitId: null,
appID: null,
adHeight: 0,
adWidth: 0,
adType: null,
adTags: null,
_setupAdTags: function _setupAdTags(tags) {
if (tags && tags.length) {
this.adTags = [];
for (var i = 0; i < tags.length; i++) {
if (tags[i].key && tags[i].value) {
this.adTags.push({
key: tags[i].key,
value: tags[i].value
});
}
}
}
}
}
)
});
// class that understands video ads units from CMS
// sample data contract: {adUnit: <AdUnit object>, adProvider: "Reuters", adStartFrequency: 2, adRepeatFrequency: 3, adServer: "MMPPF"}
WinJS.Namespace.define("PlatformJS.Ads", {
VideoAdUnit: WinJS.Class.derive(
PlatformJS.Ads.AdUnit,
function videoadunit_ctor(adSpec) {
if (adSpec.adUnit) {
PlatformJS.Ads.AdUnit.call(this, adSpec.adUnit);
}
if (adSpec.adProvider) {
this.adProvider = adSpec.adProvider.toLowerCase().trim();
}
if (adSpec.adStartFrequency) {
this.adStartFrequency = adSpec.adStartFrequency;
}
if (adSpec.adRepeatFrequency) {
this.adRepeatFrequency = adSpec.adRepeatFrequency;
}
if (adSpec.adServer) {
this.adServer = adSpec.adServer.toLowerCase().trim();
}
},
{
adProvider: null,
adStartFrequency: 0,
adRepeatFrequency: 0,
adServer: null,
}
)
});
// class that understands cluster ads units from CMS
// sample data contract: {adUnit: <AdUnit object>, adClusterID: "newsAdCluster1", adClusterPosition: 2, adClusterKey: "adCluster1"}
WinJS.Namespace.define("PlatformJS.Ads", {
ClusterAdUnit: WinJS.Class.derive(
PlatformJS.Ads.AdUnit,
function clusteradunit_ctor(adSpec) {
if (adSpec.adUnit) {
PlatformJS.Ads.AdUnit.call(this, adSpec.adUnit);
}
if (adSpec.adClusterId) {
this.clusterId = adSpec.adClusterId;
}
if (typeof adSpec.adClusterPosition === "number") {
this.clusterPosition = adSpec.adClusterPosition - 1;
}
if (adSpec.adClusterKey) {
this.clusterKey = adSpec.adClusterKey;
}
var adCounter = 1; // TODO: how is this set up?
this.adDisplayController = new PlatformJS.Ads.DisplayController(adCounter);
},
{
clusterId: "",
clusterPosition: 0,
clusterKey: null,
resetOnSuspend: false, // TODO: used? getting config? deprecating it?
adDisplayController: null,
getClusterConfigKey: function getClusterConfigKey() {
var key = null;
if (this.clusterKey) {
key = this.clusterKey;
}
if (!key) {
key = "AdsCluster";
if (this.clusterId) {
key += "_" + this.clusterId;
}
}
return key;
}
}
)
});
})();
/* File: PageAdsConfig.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformAds_PageAdsConfigInit() {
"use strict";
// Class that understands the adsData data contract from CMS:
// adsData: { articleReaderAds: <list of article reader ads spec>, videoAds: <list of video ads spec>, clusterAds: <list of cluster ads spec> }
WinJS.Namespace.define("PlatformJS.Ads", {
PageAdsConfig: WinJS.Class.define(
function pageadsconfig_ctor(pageads) {
var i;
if (pageads.articleReaderAds && Array.isArray(pageads.articleReaderAds)) {
this.articleReaderAds = [];
for (i = 0; i < pageads.articleReaderAds.length; i++) {
if (pageads.articleReaderAds[i]) {
var articleReaderUnit = new PlatformJS.Ads.AdUnit(pageads.articleReaderAds[i]);
this.articleReaderAds.push(articleReaderUnit);
}
}
}
if (pageads.videoAds && Array.isArray(pageads.videoAds)) {
this.videoAds = [];
for (i = 0; i < pageads.videoAds.length; i++) {
if (pageads.videoAds[i]) {
var videoUnit = new PlatformJS.Ads.VideoAdUnit(pageads.videoAds[i]);
if (videoUnit.type !== "noad") {
this.videoAds[videoUnit.adProvider] = videoUnit; // for duplicate adProvider entry, last entry overwrites
}
}
}
}
if (pageads.clusterAds && Array.isArray(pageads.clusterAds)) {
this.clusterAds = [];
for (i = 0; i < pageads.clusterAds.length; i++) {
if (pageads.clusterAds[i]) {
var clusterUnit = new PlatformJS.Ads.ClusterAdUnit(pageads.clusterAds[i]);
// if clusterPosition is specified, it has to be greater than 1, otherwise, no-op
if (clusterUnit.type && clusterUnit.type !== "noad" && ((!clusterUnit.clusterPosition && clusterUnit.clusterPosition !== 0) || clusterUnit.clusterPosition >= 1)) {
this.clusterAds[clusterUnit.clusterId] = clusterUnit; // for duplicate clusterId entry, last entry overwrites
}
}
}
}
},
{
articleReaderAds: null,
videoAds: null,
clusterAds: null,
}
)
});
})();
