﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: BaseAuth.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexBaseAuthInit() {
WinJS.Namespace.define("CommonJS.Partners.Auth", {
BaseAuth: WinJS.Class.define(function commonjs_baseauth_ctor() {
/// <summary>
/// The basic auth manager for a partner. This class controls signing in/out users and checking entitlements.
/// Partners are expected to derive from this class and implement the empty functions.
/// </summary>
this._refreshTimer = null;
///<disable>JS2042.DoNotAssignUndefined</disable>
this._authToken = undefined;
///<enable>JS2042.DoNotAssignUndefined</enable>
// Assume we're unentitled at first
this.isEntitled = false;
// Get the auth token to refresh UX by calling the getter
this.authToken;
this._entitlements = null;
this._onlineEntitlementsChecked = false;
// Check the entitlements to get the subscription status
this.getEntitlementsAsync().then(function () { }, function () { });
this.getEntitlementsAsync(false, true).then(function () { }, function () { });
CommonJS.Partners.Auth.BaseAuth.authenticators[this._partner] = this;
var that = this;
WinJS.Navigation.addEventListener("navigated", function () {
that.closeExistingPromptControl();
});
},
{
_entitlements: null,
_partner: null,
_refreshTimer: null,
_lastEntitlementsUpdateTime: null,
_entitlementsCheckExpireTime: null,
_onlineEntitlementsChecked: false,
_serializedEntitlements: null,
_serializeEntitlements: function _serializeEntitlements() {
// This needs to be implemented by deriving class
},
_deserializeEntitlements: function _deserializeEntitlements() {
// This needs to be implemented by deriving class
},
entitlementsCheckExpireTime: {
set: function set(value) {
if (this._entitlementsCheckExpireTime !== value) {
this._entitlementsCheckExpireTime = value;
}
},
get: function get() {
return this._entitlementsCheckExpireTime;
}
},
entitlements: {
set: function set(value) {
if (this._entitlements !== value) {
this._entitlements = value;
this._serializeEntitlements();
}
},
get: function get() {
if (!this._entitlements) {
this._entitlements = this._deserializeEntitlements();
}
return this._entitlements;
}
},
getRefreshInterval: function getRefreshInterval() {
// Needs to overridden by implementing class
return 6 * 60 * 60 * 1000;
},
_scheduleNextEntitlementsRefresh: function _scheduleNextEntitlementsRefresh(lastUpdateTime) {
var timestamp = lastUpdateTime ? lastUpdateTime.getTime() : 0;
// We've already scheduled the next update
if (this._lastEntitlementsUpdateTime === timestamp) {
return;
}
this._lastEntitlementsUpdateTime = timestamp;
var refreshInterval = this.getRefreshInterval(),
nextRefresh = refreshInterval - (Date.now() - this._lastEntitlementsUpdateTime);
this.entitlementsCheckExpireTime = this._lastEntitlementsUpdateTime + refreshInterval;
// If the computed next refresh time is less than 0, then set it to next refresh interval (e.g., 6 hours)
if (nextRefresh <= 0) {
nextRefresh = refreshInterval;
}
if (this._refreshTimer) {
clearTimeout(this._refreshTimer);
}
var that = this;
this._refreshTimer = setTimeout(function refresh_handler() {
that.getEntitlementsAsync(false, true).then(function () { }, function () { });
}, nextRefresh);
},
getEntitlementsAsync: function getEntitlementsAsync(showPrompt, bypassCache) {
// This needs to be implemented by deriving class
},
showSignInPromptAsync: function showSignInPromptAsync(code) {
this.closeExistingPromptControl();
// This needs to be implemented by deriving class
},
showSubscribePromptAsync: function showSubscribePromptAsync(code) {
// This needs to be implemented by deriving class
this.closeExistingPromptControl();
},
closeExistingPromptControl: function closeExistingPromptControl() {
if (CommonJS.Partners.Auth.BaseAuthControl.promptControl) {
CommonJS.Partners.Auth.BaseAuthControl.promptControl.cancel();
}
},
getVaultValue: function getVaultValue(resource, userName) {
var passwordVault = new Windows.Security.Credentials.PasswordVault();
resource = this._partner + resource;
if (!userName) {
userName = resource;
}
try {
var creds = passwordVault.retrieve(resource, resource);
if (creds) {
creds.retrievePassword();
return creds.password;
}
} catch (e) { }
return null;
},
setVaultValue: function setVaultValue(resource, value, userName) {
var passwordVault = new Windows.Security.Credentials.PasswordVault();
resource = this._partner + resource;
if (!userName) {
userName = resource;
}
try {
var creds = passwordVault.retrieve(resource, resource);
if (creds) {
passwordVault.remove(creds);
}
} catch (e) { }
if (value) {
passwordVault.add(new Windows.Security.Credentials.PasswordCredential(resource, resource, value));
}
},
signIn: function signIn(code) {
var that = this;
return this.showSignInPromptAsync(code).then(function () {
Windows.Storage.ApplicationData.current.signalDataChanged();
}, function () { });
},
signOut: function signOut() {
this.authToken = null;
this.authTokenExpire = 0;
this.isEntitled = false;
Windows.Storage.ApplicationData.current.signalDataChanged();
return WinJS.Promise.wrap(null);
},
subscribe: function subscribe(code) {
var that = this;
this.showSubscribePromptAsync(code).then(function () {
Windows.Storage.ApplicationData.current.signalDataChanged();
}, function () { });
},
isSignedIn: {
get: function get() {
return this.authToken !== null;
}
},
_authToken: null,
authToken: {
get: function get() {
if (this._authToken === undefined) {
this._authToken = this.getVaultValue("AuthToken");
CommonJS.Partners.Debug.instance.log("AuthToken vault value: " + this._authToken);
if (this._authToken) {
WinJS.Utilities.removeClass(document.body, this._partner + "SignedOut");
WinJS.Utilities.addClass(document.body, this._partner + "SignedIn");
} else {
WinJS.Utilities.addClass(document.body, this._partner + "SignedOut");
WinJS.Utilities.removeClass(document.body, this._partner + "SignedIn");
}
}
return this._authToken;
},
set: function set(value) {
if (this._authToken !== value) {
this._authToken = value;
this.setVaultValue("AuthToken", value);
if (!value) {
this.isEntitled = false;
}
if (this._authToken) {
WinJS.Utilities.removeClass(document.body, this._partner + "SignedOut");
WinJS.Utilities.addClass(document.body, this._partner + "SignedIn");
} else {
WinJS.Utilities.addClass(document.body, this._partner + "SignedOut");
WinJS.Utilities.removeClass(document.body, this._partner + "SignedIn");
this.authTokenExpire = 0;
}
}
}
},
_authTokenExpire: null,
authTokenExpire: {
set: function set(value) {
CommonJS.Partners.Config.setConfig(this._partner, "AuthTokenExpireTime", value);
this._authTokenExpire = value;
},
get: function get(value) {
if (!this._authTokenExpire) {
this._authTokenExpire = CommonJS.Partners.Config.getConfig(this._partner, "AuthTokenExpireTime", 0);
}
return this._authTokenExpire;
}
},
_isEntitled: null,
isEntitled: {
set: function set(value) {
if (this._isEntitled !== value) {
this._isEntitled = value;
if (this._isEntitled) {
WinJS.Utilities.removeClass(document.body, this._partner + "NoEntitled");
WinJS.Utilities.addClass(document.body, this._partner + "Entitled");
} else {
WinJS.Utilities.addClass(document.body, this._partner + "NoEntitled");
WinJS.Utilities.removeClass(document.body, this._partner + "Entitled");
}
}
},
get: function get() {
return this._isEntitled;
}
}
},
{
notLoggedInError: "NotLoggedIn",
authenticators: {}
})
});
})();
/* File: BaseAuthControl.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexBaseAuthCtrlInit() {
WinJS.Namespace.define("CommonJS.Partners.Auth", {
BaseAuthControl: WinJS.Class.define(function commonjs_baseauthctrl_ctor(options) {
/// <summary>
/// The base auth control for a partner. This is the UX dialog that signs users in/out.
/// Partners are expected to derive from this class and implement the empty functions.
/// </summary>
/// <param name="options">  
/// Options object
/// </param>
var that = this;
this.isShowing = false;
this.element = document.createElement("div");
CommonJS.setAutomationId(this.element);
this.element.winControl = this;
WinJS.Utilities.addClass(this.element, "authControl");
this.container = document.createElement("div");
WinJS.Utilities.addClass(this.container, "authContainer");
this.element.appendChild(this.container);
var accentBar = document.createElement("div");
WinJS.Utilities.addClass(accentBar, "accentBar authAccentBar");
this.container.appendChild(accentBar);
var closeButton = document.createElement("button");
closeButton.tabIndex = 31;
closeButton.textContent = PlatformJS.Services.resourceLoader.getString("/platform/Cancel");
CommonJS.setAutomationId(closeButton, this.element, "closeButton");
closeButton.addEventListener("click", function auth_closeBtn_click() {
that.cancel();
if (that.instrumentCancel) {
that.instrumentCancel();
}
});
WinJS.Utilities.addClass(closeButton, "authCloseButton");
this.container.appendChild(closeButton);
var logo = document.createElement("div");
WinJS.Utilities.addClass(logo, "authLogo");
this.container.appendChild(logo);
this._progressIndicator = document.createElement("progress");
WinJS.Utilities.addClass(this._progressIndicator, "authProgress platformHide");
this.container.appendChild(this._progressIndicator);
this.container.addEventListener("keydown", this._containerHandleKey.bind(this), false);
WinJS.UI.setOptions(this, options);
},
{
element: null,
container: null,
_onerror: null,
_onsuccess: null,
isShowing: false,
_partner: null,
_containerHandleKey: function _containerHandleKey(event) {
if (event.keyCode === WinJS.Utilities.Key.escape) {
this.cancel();
}
},
instrumentCancel: function instrumentCancel() {
// Implemented in derived classes.
},
showProgress: function showProgress() {
WinJS.Utilities.removeClass(this._progressIndicator, "platformHide");
},
hideProgress: function hideProgress() {
WinJS.Utilities.addClass(this._progressIndicator, "platformHide");
},
disableUX: function disableUX(value) {
var signinButtons = document.querySelectorAll(".signInButton");
for (var i = 0; i < signinButtons.length; i++) {
signinButtons[i].disabled = value;
}
var bottomEdgy = document.querySelector(".articleReaderPage #bottomEdgy");
if (bottomEdgy) {
bottomEdgy.disabled = value;
}
var platformPage = document.getElementById("platformPageArea");
platformPage.disabled = value;
if (value) {
WinJS.Utilities.addClass(document.body, "authOpen");
} else {
WinJS.Utilities.removeClass(document.body, "authOpen");
}
try {
if (value) {
this._previousKeyboardInput = Windows.ApplicationModel.Search.SearchPane.getForCurrentView().showOnKeyboardInput;
Windows.ApplicationModel.Search.SearchPane.getForCurrentView().showOnKeyboardInput = false;
} else {
Windows.ApplicationModel.Search.SearchPane.getForCurrentView().showOnKeyboardInput = this._previousKeyboardInput;
}
} catch (e) { }
},
_previousKeyboardInput: null,
showAsync: function showAsync() {
// To be extended in derived classes.
var that = this;
this.isShowing = true;
this.disableUX(true);
document.body.appendChild(this.element);
var authCloseButton = this.element.querySelector(".authCloseButton");
if (authCloseButton) {
authCloseButton.focus();
}
var userNameInput = this.element.querySelector(".userNameInput");
if (userNameInput) {
userNameInput.focus();
}
var ApplicationView = Windows.UI.ViewManagement.ApplicationView;
var ApplicationViewState = Windows.UI.ViewManagement.ApplicationViewState;
if (ApplicationView.value === ApplicationViewState.snapped) {
ApplicationView.tryUnsnap();
}
return new WinJS.Promise(function (complete, error) {
that._onsuccess = complete;
that._onerror = error;
}).then(function (response) {
that.close();
return WinJS.Promise.wrap(response);
});
},
close: function close() {
if (this.isShowing) {
this.disableUX(false);
this.isShowing = false;
this._onerror = null;
this._onsuccess = null;
if (this.element && document.body.contains(this.element)) {
document.body.removeChild(this.element);
}
this.element = null;
CommonJS.Partners.Auth.BaseAuthControl.promptControl = null;
}
},
cancel: function cancel() {
if (this._onerror) {
this._onerror(CommonJS.Partners.Auth.BaseAuth.notLoggedInError);
}
this.close();
}
},
{
promptControl: null
}),
});
})();
/* File: BaseWebAuthControl.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexBaseAuthInit() {
WinJS.Namespace.define("CommonJS.Partners.Auth", {
BaseWebAuthControl: WinJS.Class.derive(CommonJS.Partners.Auth.BaseAuthControl, function commonjs_basewebauthctrl_ctor(options) {
/// <summary>
/// The base iframe web auth control for a partner. This is the UX dialog that signs users in/out that uses an iframe and postMessages.
/// Partners are expected to derive from this class and implement the empty functions.
/// </summary>
/// <param name="options">
/// Options object
/// </param>
this._url = null;
this._messageListener = null;
this.iframe = document.createElement("iframe");
this.iframe.tabIndex = 0;
this.iframe.name = this._partner.toLowerCase() + "_AuthFrame";
WinJS.Utilities.addClass(this.iframe, "authIFrame authMainControl");
var that = this;
CommonJS.Partners.Auth.BaseAuthControl.call(this, options);
this.container.appendChild(this.iframe);
that.showProgress();
if (this.isDirty) {
this.url = options.initializeUrl ? options.initializeUrl : options.targetUrl;
} else {
this.isDirty = true;
this.url = options.targetUrl;
this.iframe.onload = function (event) {
that.hideProgress();
};
}
},
{
iframe: null,
targetDomain: null,
partner: null,
_isDirty: null,
isDirty: {
get: function get() {
if (this._isDirty === null) {
this._isDirty = CommonJS.Partners.Config.getConfig(this.partner, "IFrameDirty", false);
}
return this._isDirty;
},
set: function set(value) {
this._isDirty = value;
CommonJS.Partners.Config.setConfig(this.partner, "IFrameDirty", value);
}
},
_url: null,
height: {
set: function set(value) {
if (value) {
this.iframe.style.height = value + "px";
}
}
},
width: {
set: function set(value) {
if (value) {
this.iframe.style.width = value + "px";
}
}
},
url: {
set: function set(value) {
this._url = value;
if (this.isShowing) {
this.iframe.src = value;
}
},
get: function get() {
return this._url;
}
},
initializeUrl: null,
targetUrl: null,
instrumentCancel: function instrumentCancel() {
// Implemented in derived classes.
},
onmessage: function onmessage(event) {
var that = this;
if (event.origin === this.targetDomain ||
event.origin === "ms-appx://" + Windows.ApplicationModel.Package.current.id.name.toLowerCase()) {
var cmd = event.data;
CommonJS.Partners.Debug.instance.log("Message from IFrame: ", cmd);
if (cmd) {
if (cmd.action === "success") {
this._onsuccess(cmd);
} else if (cmd.action === "open") {
this.url = "about:blank";
this.width = cmd.data.width;
this.height = cmd.data.height;
msSetImmediate(function () {
that.url = that.getUrl(cmd.data);
});
} else if (cmd.action === "logout") {
this.isDirty = false;
if (this._isDisposed) {
this._finalizeCleanup();
} else {
// Before we start with target url, make sure we've logged out
this.isDirty = true;
this.url = this.targetUrl;
this.iframe.onload = function (event) {
that.hideProgress();
};
}
} else if (cmd.action === "showProgress") {
this.showProgress();
} else if (cmd.action === "hideProgress") {
this.hideProgress();
}
}
}
},
getUrl: function getUrl(data) {
return data.url;
},
showAsync: function showAsync() {
this.isShowing = true;
this.iframe.src = this.url;
var that = this;
this._messageListener = function (event) {
that.onmessage(event);
};
window.addEventListener("message", this._messageListener);
return CommonJS.Partners.Auth.BaseAuthControl.prototype.showAsync.call(this);
},
_finalizeCleanup: function _finalizeCleanup() {
this.isDirty = false;
if (this.iframe) {
this.iframe.src = "about:blank";
this.iframe.onload = null;
document.body.removeChild(this.iframe);
this.iframe = null;
}
window.removeEventListener("message", this._messageListener);
this._messageListener = null;
},
close: function close() {
CommonJS.Partners.Auth.BaseAuthControl.prototype.close.call(this);
if (this.iframe) {
this.iframe.src = "about:blank";
this.iframe.onload = null;
this.iframe = null;
}
this.iframe = document.createElement("iframe");
WinJS.Utilities.addClass(this.iframe, "platformHidden");
document.body.appendChild(this.iframe);
this.iframe.src = this.initializeUrl;
this._isDisposed = true;
}
})
});
})();
/* File: BaseNativeAuthControl.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexBaseNativeAuthInit() {
WinJS.Namespace.define("CommonJS.Partners.Auth", {
BaseNativeAuthControl: WinJS.Class.derive(CommonJS.Partners.Auth.BaseAuthControl, function commonjs_basenativeauthctrol_ctor(options) {
/// <summary>
/// The base native auth control for a partner. This is the UX dialog that signs users in/out that uses a REST API.
/// Partners are expected to derive from this class and implement the empty functions.
/// </summary>
/// <param name="options">
/// Options object
/// </param>
// note on element tabIndex allotments:
//    This class (name, password, etc.): 1 - 10, 21-30
//    Derived classes (subscribe): 11 - 20
//    Parent (cancel): 31+
CommonJS.Partners.Auth.BaseAuthControl.call(this, options);
var form = document.createElement("div");
form.role = "form";
WinJS.Utilities.addClass(form, "authForm authMainControl");
this.container.appendChild(form);
var logo = document.createElement("div");
form.appendChild(logo);
WinJS.Utilities.addClass(logo, "authFormLogo");
var prompt = document.createElement("div");
prompt.innerText = this.getString("subscribePrompt");
form.appendChild(prompt);
WinJS.Utilities.addClass(prompt, "prompt");
var instructions = document.createElement("div");
instructions.innerText = this.getString("subscribeInstructions");
form.appendChild(instructions);
WinJS.Utilities.addClass(instructions, "instructions");
this._userNameInput = document.createElement("input");
this._userNameInput.tabIndex = 1;
this._userNameInput.type = "text";
WinJS.Utilities.addClass(this._userNameInput, "userNameInput inputBoxContainer");
form.appendChild(this._userNameInput);
// custom placeholder overlay that supports focus
this._userNameInputOverlay = this._createPlaceholderOverlayElement(this._userNameInput, this.getString("userNameInput"), "userNameInput");
form.appendChild(this._userNameInputOverlay);
this._passwordInput = document.createElement("input");
this._passwordInput.tabIndex = 2;
this._passwordInput.type = "password";
WinJS.Utilities.addClass(this._passwordInput, "passwordInput inputBoxContainer");
form.appendChild(this._passwordInput);
// custom placeholder overlay that supports focus
this._passwordInputOverlay = this._createPlaceholderOverlayElement(this._passwordInput, this.getString("passwordInput"), "passwordInput");
form.appendChild(this._passwordInputOverlay);
this._statusLabel = document.createElement("div");
WinJS.Utilities.addClass(this._statusLabel, "statusLabel platformHide");
form.appendChild(this._statusLabel);
if (!options.hidePrivacyPolicy) {
this._privacyPolicy = document.createElement("button");
this._privacyPolicy.tabIndex = 21;
this._privacyPolicy.innerText = this.getString("privacyPolicy");
WinJS.Utilities.addClass(this._privacyPolicy, "privacyPolicy");
form.appendChild(this._privacyPolicy);
this._privacyPolicy.onclick = this._onPrivacyPolicyClick.bind(this);
}
if (!options.hideTermsOfUse) {
this._tou = document.createElement("button");
this._tou.tabIndex = 22;
this._tou.innerText = this.getString("TermsOfUse");
WinJS.Utilities.addClass(this._tou, "tou");
form.appendChild(this._tou);
this._tou.onclick = this._onTOUClick.bind(this);
}
this._submitButton = document.createElement("button");
this._submitButton.tabIndex = 4;
WinJS.Utilities.addClass(this._submitButton, "submitButton");
this._submitButton.innerText = this.getString("submit");
form.appendChild(this._submitButton);
this._submitButton.onclick = this._onSubmitClick.bind(this);
this._passwordInput.addEventListener("keydown", this._handleKey.bind(this), false);
this._userNameInput.addEventListener("keydown", this._handleKey.bind(this), false);
if (!options.hideCreateAccount) {
this._createAccountButton = document.createElement("button");
this._createAccountButton.tabIndex = 6;
this._createAccountButton.innerText = this.getString("createAccountButton");
WinJS.Utilities.addClass(this._createAccountButton, "createAccountButton");
form.appendChild(this._createAccountButton);
this._createAccountButton.onclick = this._onCreateAccountClick.bind(this);
}
if (!options.hideForgotPassword) {
this._forgotPasswordButton = document.createElement("button");
this._forgotPasswordButton.tabIndex = 5;
this._forgotPasswordButton.innerText = this.getString("forgotPasswordButton");
WinJS.Utilities.addClass(this._forgotPasswordButton, "forgotPasswordButton");
form.appendChild(this._forgotPasswordButton);
this._forgotPasswordButton.onclick = this._onForgotPasswordClick.bind(this);
}
this.customContainer = document.createElement("div");
WinJS.Utilities.addClass(this.customContainer, "customContainer");
form.appendChild(this.customContainer);
},
{
getString: function getString(stringId) {
return PlatformJS.Services.resourceLoader.getString("/platform/" + stringId);
},
_handleKey: function _handleKey(event) {
var that = this;
if (event.keyCode === WinJS.Utilities.Key.enter) {
that._submitButton.focus();
}
},
_partner: null,
_userNameInput: null,
_userNameInputOverlay: null,
_passwordInput: null,
_passwordInputOverlay: null,
_statusLabel: null,
_partnerStringPrefix: null,
customContainer: null,
validateAsync: function validateAsync(userName, password) {
// To be implemented by derived classes
return WinJS.Promise.wrap({});
},
_onCreateAccountClick: function _onCreateAccountClick() {
// To be implemented by derived classes
},
_onForgotPasswordClick: function _onForgotPasswordClick() {
// To be implemented by derived classes
},
_onPrivacyPolicyClick: function _onPrivacyPolicyClick() {
// To be implemented by derived classes
},
_onTOUClick: function _onTOUClick() {
// To be implemented by derived classes
},
_onSubmitClick: function _onSubmitClick() {
var userName = this._userNameInput.value;
var password = this._passwordInput.value;
var that = this;
this._submitButton.disabled = true;
WinJS.Utilities.addClass(that._statusLabel, "platformHide");
this.showProgress();
that.validateAsync(userName, password).then(function (response) {
that.hideProgress();
that._onsuccess(response);
}, function (error) {
that._submitButton.disabled = false;
that.hideProgress();
WinJS.Utilities.removeClass(that._statusLabel, "platformHide");
if (typeof error === "string") {
that._statusLabel.innerText = error;
} else {
that._statusLabel.innerText = that.getString("validateError");
}
});
},
_createPlaceholderOverlayElement: function _createPlaceholderOverlayElement(input, text, className) {
// Create an element that overlays input placeholder text. Placeholder remains when focus is gained.
var overlayElement = document.createElement("div");
overlayElement.innerText = text;
overlayElement.id = "authInputBoxPlaceholderOverlay";
WinJS.Utilities.addClass(overlayElement, className); // apply class (that has auth input position)
// transfer focus to the input element if the placeholder text receives it
overlayElement.addEventListener("focus", function placeholder_onFocus () {
input.focus();
});
// setup listeners to hide and unhide overlay based on input
input.addEventListener("input", function placeholderInput_onInput () {
if (this.value.length !== 0) {
overlayElement.style.visibility = "hidden";
}
else {
overlayElement.style.visibility = "visible";
}
});
return overlayElement;
}
})
});
})();
/* File: DynamicPanoAuth.js */
(function () {
WinJS.Namespace.define("DynamicPanoJS", {
AuthControl: WinJS.Class.derive(CommonJS.Partners.Auth.BaseNativeAuthControl, function dynamic_pano_auth_ctrl_ctor(options) {
this._partner = options.authProvider._partner;
this._authProvider = options.authProvider;
this._instrumentationId = options.instrumentationId || "";
options.hidePrivacyPolicy = !options.authProvider.authInfo.privacyurl;
options.hideTermsOfUse = !options.authProvider.authInfo.termsofuseurl;
options.hideCreateAccount = !options.authProvider.authInfo.registrationurl;
options.hideForgotPassword = !options.authProvider.authInfo.forgotpasswordurl;
CommonJS.Partners.Auth.BaseNativeAuthControl.call(this, options);
if (options.authProvider.authInfo.subscriptiondescription) {
var subscribeInstructions = document.createElement("div");
subscribeInstructions.innerText = PlatformJS.Services.resourceLoader.getString("/platform/subscribeNow");
this.customContainer.appendChild(subscribeInstructions);
WinJS.Utilities.addClass(subscribeInstructions, "subscribeInstructions");
var subscribeInfo = document.createElement("div");
subscribeInfo.innerHTML = options.authProvider.authInfo.subscriptiondescription;
this.customContainer.appendChild(subscribeInfo);
WinJS.Utilities.addClass(subscribeInfo, "subscribeInfo");
}
this._subscribeButton = document.createElement("button");
this._subscribeButton.tabIndex = 11;
this._subscribeButton.innerText = PlatformJS.Services.resourceLoader.getString("/Platform/subscribeButton");
WinJS.Utilities.addClass(this._subscribeButton, "subscribeButton");
this.customContainer.appendChild(this._subscribeButton);
this._subscribeButton.onclick = this._onSubscribeClick.bind(this);
var logo = this.element.querySelector(".authFormLogo");
logo.style.backgroundImage = "url(" + this._authProvider.authInfo.logo + ")";
var dynamicPanoAccentColor = null;
var dynamicPanoAccent = document.querySelector(".dynamicPanoAccent");
if (dynamicPanoAccent) {
dynamicPanoAccentColor = window.getComputedStyle(dynamicPanoAccent).backgroundColor;
}
if (!dynamicPanoAccentColor) {
dynamicPanoAccentColor = this._authProvider.authInfo.accentColor;
}
if (dynamicPanoAccentColor) {
// Apply dynamic pano color to the accent bar of the signup dialog
var authAccentBarList = this.container.getElementsByClassName("authAccentBar");
for (var i = 0 ; i < authAccentBarList.length ; i++) {
authAccentBarList[i].style.backgroundColor = dynamicPanoAccentColor;
}
var promptControlList = this.container.getElementsByClassName("prompt");
for (var j = 0 ; j < promptControlList.length ; j++) {
promptControlList[j].style.backgroundColor = dynamicPanoAccentColor;
}
}
},
{
_authProvider:null,
instrumentCancel: function instrumentCancel() {
DynamicPanoJS.Auth.logPaywallUserAction(Platform.Telemetry.UserActionOperation.click, "Cancel", null, this._instrumentationId);
},
_onCreateAccountClick: function _onCreateAccountClick() {
DynamicPanoJS.Auth.logPaywallUserAction(Platform.Telemetry.UserActionOperation.click, "CreateAccount", null, this._instrumentationId);
var url = this._authProvider.authInfo.registrationurl;
window.open(url);
},
_onForgotPasswordClick: function _onForgotPasswordClick() {
DynamicPanoJS.Auth.logPaywallUserAction(Platform.Telemetry.UserActionOperation.click, "ForgotPassword", null, this._instrumentationId);
var url = this._authProvider.authInfo.forgotpasswordurl;
window.open(url);
},
_onPrivacyPolicyClick: function _onPrivacyPolicyClick() {
DynamicPanoJS.Auth.logPaywallUserAction(Platform.Telemetry.UserActionOperation.click, "PrivacyPolicy", null, this._instrumentationId);
var url = this._authProvider.authInfo.privacyurl;
window.open(url);
},
_onTOUClick: function _onTOUClick() {
DynamicPanoJS.Auth.logPaywallUserAction(Platform.Telemetry.UserActionOperation.click, "TermsOfUse", null, this._instrumentationId);
var url = this._authProvider.authInfo.termsofuseurl;
window.open(url);
},
validateAsync: function validateAsync(userName, password) {
var that = this;
if (!userName || !password) {
return WinJS.Promise.wrapError(PlatformJS.Services.resourceLoader.getString("/Platform/credentialsInvalid"));
}
return this._authProvider.loginAsync(userName, password).then(function (result) {
DynamicPanoJS.Auth.logPaywallAppAction("appsignin", null, that._instrumentationId);
return WinJS.Promise.as(result);
}, function (error) {
DynamicPanoJS.Auth.logPaywallAppAction("appsigninerror", { error: error }, that._instrumentationId);
throw error;
});
},
_onSubscribeClick: function _onSubscribeClick() {
DynamicPanoJS.Auth.logPaywallUserAction(Platform.Telemetry.UserActionOperation.click, "Subscribe", null, this._instrumentationId);
var url = this._authProvider.authInfo.subscriptionurl;
window.open(url);
},
}),
Auth: WinJS.Class.mix(WinJS.Class.derive(CommonJS.Partners.Auth.BaseAuth, function dynamic_pano_auth_ctor(partnerId, authInfo, instrumentationId) {
Object.defineProperties(this, WinJS.Utilities.createEventProperties("authchanged"));
this._partner = partnerId;
this._lastGatewayShowing = null;
this._entitlements = null;
this._instrumentationId = instrumentationId;
this.authInfo = authInfo;
this.authUrlAppId = authInfo.authUrl;
if (!this.authStatus.expirationTime) {
this.authStatus.expirationTime = Date.now() + this.authInfo.meteringtimems;
}
if (!this.authStatus.articlesRead) {
this.authStatus.articlesRead = {};
}
// Try to capture the accent color here since the Article Reader does not have access to it.
var dynamicPanoAccent = document.querySelector(".dynamicPanoAccent");
if (dynamicPanoAccent) {
this.authInfo.accentColor = window.getComputedStyle(dynamicPanoAccent).backgroundColor;
}
CommonJS.Partners.Auth.BaseAuth.call(this);
},
{
_instrumentationId : null,
authInfo: null,
_authStatus: null,
authStatus: {
get: function get(){
if (!this._authStatus) {
try {
var stringToParse = Windows.Storage.ApplicationData.current.localSettings.values[this._partner + "_AuthStatus"];
this._authStatus = JSON.parse(stringToParse);
} catch (e) {
this._authStatus = {};
}
}
return this._authStatus;
}
},
_isEntitled: false,
isEntitled: {
get: function get() {
return this._isEntitled;
},
set: function set(value) {
var previousValue = this._isEntitled;
this._isEntitled = value;
if (value !== previousValue) {
this.dispatchEvent("authchanged");
}
}
},
_persistAuthStatus: function _persistAuthStatus() {
var stringToSave = JSON.stringify(this.authStatus);
Windows.Storage.ApplicationData.current.localSettings.values[this._partner + "_AuthStatus"] = stringToSave;
},
_serializeEntitlements: function _serializeEntitlements() {
if (this._entitlements) {
var serializer = new XMLSerializer();
var serialized = serializer.serializeToString(this._entitlements);
if (this._serializedEntitlements !== serialized) { // check: only write data if the serialized data value has changed
CommonJS.Partners.Config.setConfig(this._partner, "Entitlements", serialized);
}
} else {
CommonJS.Partners.Config.setConfig(this._partner, "Entitlements", null);
this._serializedEntitlements = null; // clear the current serialized data value so next non-null write check will pass
}
},
_deserializeEntitlements: function _deserializeEntitlements() {
var serialized = CommonJS.Partners.Config.getConfig(this._partner, "Entitlements", "");
this._serializedEntitlements = serialized; // cache the serialized data last read
if (serialized) {
return this._parseEntitlements(serialized);
} else {
return null;
}
},
getRefreshInterval: function getRefreshInterval() {
return this.authInfo.authrefreshintervalms;
},
loginAsync: function loginAsync(userName, password) {
var url = this.authInfo.authurl;
var that = this;
url = url.format(encodeURIComponent(userName), encodeURIComponent(password));
// note: the If-Modified-Since header ensures that the request is sent again, even if it has already been cached
return WinJS.xhr({
url: url,
headers: {
"If-Modified-Since": "Mon, 27 Mar 1972 00:00:00 GMT"
},
type: "GET"
}).then(function (response) {
if (response.status === 200) {
var xml = response.responseXML; 
if (xml) {
var status = xml.querySelector("response");
var message = xml.querySelector("message");
var uuid = xml.querySelector("userid");
var errorMessage = null;
// validate response
if (status && status.textContent !== "success") {
errorMessage = PlatformJS.Services.resourceLoader.getString("/Platform/credentialsInvalid");  // bad credentials
} else if (!uuid) {
errorMessage = PlatformJS.Services.resourceLoader.getString("/Platform/validateError"); // no uuid in response (invalid format)
} else if (!that._containsActiveProduct(xml)) {
errorMessage = PlatformJS.Services.resourceLoader.getString("/Platform/entitlementsInvalid"); // no Win8 entitlement
}
if (errorMessage !== null) {
// override the error message with reponse message response if it exists
if (message) {
return WinJS.Promise.wrapError(message.textContent);
} else {
return WinJS.Promise.wrapError(errorMessage);
}
}
}
return WinJS.Promise.wrap(xml);
} else {
return WinJS.Promise.wrapError(PlatformJS.Services.resourceLoader.getString("/Platform/credentialsInvalid"));
}
});
},
signOut: function signOut(source) {
var attributes = {
source: source ? source : "Unknown"
};
DynamicPanoJS.Auth.logPaywallAppAction("appsignout", attributes, this._instrumentationId);
return CommonJS.Partners.Auth.BaseAuth.prototype.signOut.call(this);
},
showSignInPromptAsync: function showSignInPromptAsync(code) {
CommonJS.Partners.Auth.BaseAuth.prototype.showSignInPromptAsync.call(this);
var that = this;
CommonJS.Partners.Auth.BaseAuthControl.promptControl = this.promptControl = new DynamicPanoJS.AuthControl({
authProvider: this,
instrumentationId: this._instrumentationId
});
DynamicPanoJS.Auth.logPaywallAppAction(Platform.Telemetry.AppActionOperation.open, null, this._instrumentationId);
return this.promptControl.showAsync().then(function (xml) {
DynamicPanoJS.Auth.logPaywallAppAction(Platform.Telemetry.AppActionOperation.close, null, that._instrumentationId);
if (xml) {
var uuid = xml.querySelector("userid");
if (uuid) {
that.authToken = uuid.textContent;
that.entitlements = xml;
that._scheduleNextEntitlementsRefresh(new Date());
return WinJS.Promise.wrap(that._checkEntitlement(xml));
}
}
return WinJS.Promise.wrapError("AuthError");
}, function (error) {
DynamicPanoJS.Auth.logPaywallAppAction(Platform.Telemetry.AppActionOperation.close, { error: error }, that._instrumentationId);
return WinJS.Promise.wrapError("AuthError");
});
},
dismissSignInPrompt: function dismissSignInPrompt() {
if (this.promptControl) {
this.promptControl.cancel();
this.promptControl = null;
}
},
showSubscribePromptAsync: function showSubscribePromptAsync() {
// TODO Need to provide instrumentation here. This is executed when an entitlement runs out, and user clicks on "subscribe".
// But right now the instrumentationId is not accessible to this function
var url = this.authInfo.subscriptionUrl;
window.open(url);
return WinJS.Promise.wrap(true);
},
showToast: function showToast() {
var that = this,
message = PlatformJS.Services.resourceLoader.getString("/Platform/InvalidSubscriptionMessage"),
buttonLabel = PlatformJS.Services.resourceLoader.getString("/Platform/subscribeButton"),
messageBar = new CommonJS.MessageBar(message);
messageBar.addButton(buttonLabel, function () {
messageBar.hide();
that.showSubscribePromptAsync().then(function () { }, function () { });
});
messageBar.addButton(PlatformJS.Services.resourceLoader.getString("/Platform/Dismiss"), function () {
messageBar.hide();
});
messageBar.show();
},
_checkEntitlement: function _checkEntitlement(xml) {
this.isEntitled = this._containsActiveProduct(xml) ? true : false;
return this.isEntitled;
},
_containsActiveProduct: function _containsActiveProduct(xml) {
if (xml) {
var products = xml.querySelectorAll("product");
if (products.length) {
for (var i = 0; i < products.length; i++) {
if (products[i].textContent === "Win8") {
return true;
}
}
}
}
return false;
},
_parseEntitlements: function _parseEntitlements(dataString) {
var parser = new DOMParser();
return parser.parseFromString(dataString, "text/xml");
},
_getEntitlementsAsync: function _getEntitlementsAsync(authToken, bypassCache) {
var that = this;
// Otherwise, ask the entitlements server
var url = this.authInfo.authrefreshurl;
url = url.format(encodeURIComponent(authToken));
// note: the If-Modified-Since header ensures that the request is sent again, even if it has already been cached
var promise = WinJS.xhr({
url: url,
headers: {
"If-Modified-Since": "Mon, 27 Mar 1972 00:00:00 GMT"
},
type: "GET"
}).then(function (response) {
that.entitlements = null;
if (response.status === 200) {
var xml = response.responseXML;
if (xml) {
var status = xml.querySelector("response");
if (status && status.textContent !== "success") {
var error = xml.querySelector("message");
if (error) {
return WinJS.Promise.wrapError(error.textContent);
} else {
return WinJS.Promise.wrapError(PlatformJS.Services.resourceLoader.getString("/Platform/credentialsInvalid"));
}
}
// update authToken if it has changed
var uuid = xml.querySelector("userid");
if (uuid && uuid.textContent !== that.authToken) {
that.authToken = uuid.textContent;
}
}
that.entitlements = xml;
that._onlineEntitlementsChecked = true;
that._scheduleNextEntitlementsRefresh(new Date());
return WinJS.Promise.wrap(xml);
}
});
return promise.then(function () {
return WinJS.Promise.wrap(that._checkEntitlement(that.entitlements));
}, function () {
// schedule next refresh even if there was an error this time (e.g. WinJS.xhr returns 404)
try {
that._scheduleNextEntitlementsRefresh(new Date());
} catch (error) { 
// don't crash if scheduling fails...
}
return WinJS.Promise.wrap(that._checkEntitlement(that.entitlements));
});
},
getAuthTokenAsync: function getAuthTokenAsync() {
return WinJS.Promise.wrap(this.authToken);
},
getEntitlementsAsync: function getEntitlementsAsync(showPrompt, bypassCache, isBlockingPrompt, localQueryOnly) {
var that = this;
var isOnline = !localQueryOnly && PlatformJS.Utilities.hasInternetConnection();
return this.getAuthTokenAsync().then(function (authToken) {
if (authToken) {
var promise = null;
// See if the in-memory version of the entitlements is still good
if (that.entitlements &&
((Date.now() < that.entitlementsCheckExpireTime || !that._onlineEntitlementsChecked) && !bypassCache ||
!isOnline)) {
promise = WinJS.Promise.wrap(that._checkEntitlement(that.entitlements));
}
if (!promise) {
if (isOnline) {
promise = that._getEntitlementsAsync(authToken, bypassCache);
} else {
promise = WinJS.Promise.wrap(false);
}
}
return promise.then(function (isEntitled) {
if (!isEntitled) {
return that.handleEntitlementsErrorAsync(showPrompt, isBlockingPrompt);
} else {
return WinJS.Promise.wrap(isEntitled);
}
});
} else {
return that.handleEntitlementsErrorAsync(showPrompt, isBlockingPrompt);
}
}, function () {
return that.handleEntitlementsErrorAsync(showPrompt, isBlockingPrompt);
});
},
handleEntitlementsErrorAsync: function handleEntitlementsErrorAsync(showPrompt, isBlockingPrompt) {
var that = this;
if (that.isSignedIn) {
// on entitlements error, sign out
that.signOut("EntitlementsError");
}
if (showPrompt) {
if (this.isSignedIn && !isBlockingPrompt) {
this.showToast();
return WinJS.Promise.wrap(false);
} else {
return this.showSignInPromptAsync(PlatformJS.Services.resourceLoader.getString("/Platform/loginPromptContentLocked")).then(function (isEntitled) {
if (!isEntitled && showPrompt) {
that.showToast();
}
return WinJS.Promise.wrap(isEntitled);
}, function (error) {
return WinJS.Promise.wrap(false);
});
}
} else {
return WinJS.Promise.wrap(false);
}
},
isReadingAllowed: function isReadingAllowed(articleId, clusterIndex, itemIndex, paywallSetting) {
try {
switch (this.authInfo.meteringtype) {
case "TimeBased":
return this._isReadingAllowedTimeOnly(articleId);
case "CountPerTime":
return this._isReadingAllowedQuantityPerTime(articleId);
case "CountBased":
return this._isReadingAllowedQuantityOnly(articleId);
case "FlagBased":
return this._isReadingAllowedFlagBased(paywallSetting);
default:
return false;
}
} catch (error) {
return true;
}
},
_isReadingAllowedTimeOnly: function _isReadingAllowedTimeOnly(articleId) {
var expirationTime = this.authStatus.expirationTime || 0;
if (!expirationTime) {
return true;
}
if (Date.now() < expirationTime) {
return true;
}
// time has expired, so we should not allow anyone to read anything at all (ever again!)
return false;
},
_isReadingAllowedQuantityPerTime: function _isReadingAllowedQuantityPerTime(articleId) {
var expirationTime = this.authStatus.expirationTime || 0;
if (!expirationTime) {
return true;
}
if (Date.now() > expirationTime) {
// time has expired, so we can read any article
return true;
}
if (this._isArticleReadBefore(articleId)) {
return true;
}
// time has not expired, let's do a quantity check
var countOfRead = this.authStatus.countOfRead || 0;
// we can still read more!
if (countOfRead < this.authInfo.meteringquantity) {
return true;
}
return false;
},
_isReadingAllowedQuantityOnly: function _isReadingAllowedQuantityOnly(articleId) {
if (this._isArticleReadBefore(articleId)) {
return true;
}
// let's do a quantity check
var countOfRead = this.authStatus.countOfRead || 0;
// we can still read more!
if (countOfRead < this.authInfo.meteringquantity) {
return true;
}
return false;
},
_isReadingAllowedFlagBased: function _isReadingAllowedFlagBased(paywall) {
// paywall is numerical enum representing SubscriptionType. Currently: paywallState = "Free" or "Locked"   (context: DynamicPano)
// or it is a bool representing article "free" metadata  (context: ArticleReader)
if (paywall === true || (typeof paywall === "string" && paywall.toLowerCase() === "free") || paywall === undefined) {
// free
return true;
}
return false;
},
_isArticleReadBefore: function _isArticleReadBefore(articleId) {
if (!this.authStatus.articlesRead) {
return false;
}
return this.authStatus.articlesRead[articleId];
},
markAsRead: function markAsRead(articleId) {
try {
// Don't mark the article as read if we are not allowed to read it or if we have already read it.
if (!this.isReadingAllowed(articleId) || this._isArticleReadBefore(articleId)) {
return;
}
this.authStatus.articlesRead[articleId] = true;
switch (this.authInfo.meteringtype) {
case "TimeBased":
return this._markAsReadTimeOnly();
case "CountPerTime":
return this._markAsReadQuantityPerTime(articleId);
case "CountBased":
return this._markAsReadQuantityOnly();
default:
return true;
}
} catch (error) {
return true;
}
},
_markAsReadTimeOnly: function _markAsReadTimeOnly() {
var expirationTime = this.authStatus.expirationTime || 0;
var now = Date.now();
if (now > expirationTime) {
// time has expired, so if the user has purchased this, let us reset the clock
var expirationIntervalMs = this.authInfo.meteringtimems;
this.authStatus.expirationTime = now + expirationIntervalMs;
}
this._persistAuthStatus();
return true;
},
_markAsReadQuantityPerTime: function _markAsReadQuantityPerTime(articleId) {
var expirationTime = this.authStatus.expirationTime || 0;
var now = Date.now();
if (now > expirationTime) {
// time has expired, so let us reset the counter
var expirationIntervalMs = this.authInfo.meteringtimems;
this.authStatus.expirationTime = now + expirationIntervalMs;
// reset
this.authStatus.articlesRead = {};
this.authStatus.countOfRead = 0;
}
this.authStatus.articlesRead[articleId] = true;
var countOfRead = this.authStatus.countOfRead || 0;
this.authStatus.countOfRead = countOfRead + 1;
this._persistAuthStatus();
return true;
},
_markAsReadQuantityOnly: function _markAsReadQuantityOnly() {
var countOfRead = this.authStatus.countOfRead || 0;
this.authStatus.countOfRead = countOfRead + 1;
this._persistAuthStatus();
return true;
}
},
{
logPaywallAppAction: function logPaywallAppAction(operation, attributes, instrumentationId) {
if (!attributes){
attributes = { };
}
// set common event attributes
attributes.partner = instrumentationId;  // Note: partner info should be implicit from impression, so we don't really need to provide it here...
Platform.Telemetry.FlightRecorder.logAppActionWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
Platform.Telemetry.ActionContext.paywall,
null,
operation, 
0, 
JSON.stringify(attributes));
},
logPaywallUserAction: function logPaywallUserAction(operation, element, attributes, instrumentationId) {
if (!attributes){
attributes = { };
}
// set common event attributes
attributes.partner = instrumentationId;  // Note: partner info should be implicit from impression, so we don't really need to provide it here...
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
Platform.Telemetry.ActionContext.paywall,
element,
operation,
Platform.Telemetry.UserActionMethod.unknown,
0,
JSON.stringify(attributes));
},
validateAuthInfo: function validateAuthInfo(authInfo) {
// Validate authInfo to ensure that minimum required fields are present
var requiredFields = [
'meteringtype',
'paywalltype'];
var requiredNumberFields = [
'authrefreshintervalms',
'meteringquantity',
'meteringtimems'];
var requiredUrlFields = [
'authrefreshurl',
'authurl',
'logo',
'subscriptionurl'];
// Verify the paywalltype
if (authInfo.paywalltype !== "RESTAuthentication") {
return false;
}
// Check required fields are present
for (var i = 0 ; i < requiredFields.length ; i++) {
if (authInfo[requiredFields[i]] === undefined) {
return false;
}
}
// Check Numbers
for (var j = 0 ; j < requiredNumberFields.length ; j++) {
if (isNaN(parseInt(authInfo[requiredNumberFields[j]]))) {
return false;
}
}
// Check URLs
for (var k = 0 ; k < requiredUrlFields.length ; k++) {
if (authInfo[requiredUrlFields[k]] === undefined) {
return false;
}
try { 
var uri = new Windows.Foundation.Uri(authInfo[requiredUrlFields[k]]);
} catch (e) {
return false;
}
}
return true;
}
}), WinJS.Utilities.eventMixin),
PaywallManager:WinJS.Class.define(
function paywallManager_ctor() {
},
{
_authProviders: {},
getAuthProvider: function getAuthProvider(partnerId, authInfo, instrumentationId) {
var key = partnerId + "-" + instrumentationId; // user partnerId-instrumentationId pair as the lookup key
if (!this._authProviders[key]) {
// we only support RESTAuthentication
if (DynamicPanoJS.Auth.validateAuthInfo(authInfo)) {
this._authProviders[key] = new DynamicPanoJS.Auth(partnerId, authInfo, instrumentationId);
} else {
// something is wrong with authInfo...
throw "AuthInfo invalid or not supported.";
}
}
return this._authProviders[key];
}
},
{
_instance: null,
instance: {
get: function get() {
if (!DynamicPanoJS.PaywallManager._instance) {
DynamicPanoJS.PaywallManager._instance = new DynamicPanoJS.PaywallManager();
}
return DynamicPanoJS.PaywallManager._instance;
}
}
})
});
})();
