﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: AutoSuggest.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsAutoSuggestInit() {
"use strict";
WinJS.Namespace.define("CommonJS",
{
autoSuggestID: null,
autoSuggest: null,
/// <summary>
/// AutoSuggest Control
/// </summary>
AutoSuggest: WinJS.Class.define(
/// <summary>
/// constructor
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
function autoSuggest_ctor(element, options) {
var that = this;
this.element = element || document.createElement("input");
CommonJS.setAutomationId(this.element);
this.element.setAttribute("type", "text");
this.element.setAttribute("maxlength", "256"); //restrict the length of input to 256 chars
WinJS.Utilities.addClass(this.element, "platformAutoSuggestTextBox");
var previousList = document.getElementById("platformAutoSuggestList");
if (previousList) {
this._platformAutoSuggestListDom = previousList;
} else {
this._platformAutoSuggestListDom = document.createElement("div");
WinJS.Utilities.addClass(this._platformAutoSuggestListDom, "platformAutoSuggestListBase");
this._platformAutoSuggestListDom.id = "platformAutoSuggestList";
this._platformAutoSuggestListDom.setAttribute("aria-label", "platformAutoSuggestList");
this._platformAutoSuggestListDom.setAttribute("role", "listbox");
this._platformAutoSuggestListDom.setAttribute("tabIndex", "-1");
this._platformAutoSuggestListDom.setAttribute("aria-expanded", "true");
}
this.element.addEventListener("focus", function autoSuggest_onFocus(event) {
that._focus(event);
});
this.element.addEventListener("blur", function autoSuggest_onBlur(event) {
that._blur();
});
this.element.addEventListener("keydown", function autoSuggest_onKeyDown(event) {
that._keydown(event);
});
this.element.addEventListener("input",function autoSuggest_onInput(event) {
//do not request autosuggest list update when in IME
if (that._inIME) {
return;
}
//if string contains line breaks and HTML tag chars, remove them
if (escape(that.element.value).indexOf("%0A") >= 0) {
that.element.value = unescape(escape(that.element.value).replace(/%0A/ig, "%20"));
}
if (that.element.value.indexOf("<") >= 0 ||
that.element.value.indexOf(">") >= 0 ||
that.element.value.indexOf("{") >= 0 ||
that.element.value.indexOf("}") >= 0 ||
that.element.value.indexOf("[") >= 0 ||
that.element.value.indexOf("]") >= 0) {
that.element.value = that.element.value.replace(/[<>{}\[\]]/g, "");
}
//work around for a Windows issue: when textbox partly out of screen right edge, a dropdown list selection will casue display incorrect
if (that && that._panoramaID) {
var panorama = document.getElementById(CommonJS.autoSuggest._panoramaID);
if (panorama && panorama.winControl) {
if (that.element.getBoundingClientRect().right > document.body.clientWidth) { //do not need to handle "left" case, as seen, Windows already handles that
//note: setting currentClusterIndex doesn't fix this issue on HotelSearch textbox in "everything big" mode. Panorama will need to provide a way to offset per pixel in this case
panorama.winControl.currentClusterIndex = that._clusterIndex;
setTimeout(function autoSuggest_focusElementAfterClusterIndex() {
that.element.focus();
}, 50);
}
}
}
if (CommonJS.autoSuggest && this.value.length === 0) {
//add some delay here, to avoid a case that when presing backspace too fast, the autosuggest list for last letter will be updated and shown later than _hideList():
setTimeout(function autoSuggest_updateAndHideList() {
CommonJS.autoSuggest._updateList(null);
CommonJS.autoSuggest._hideList();
}, 250);
}
if (typeof that._keyup === "function") {
that._keyup(event);
}
});
this.element.addEventListener("compositionstart", function autoSuggest_onCompositionStart(event) {
if (CommonJS.autoSuggest) {
CommonJS.autoSuggest._inIME = true;
}
});
this.element.addEventListener("compositionend", function autoSuggest_onCompositionEnd(event) {
if (CommonJS.autoSuggest) {
CommonJS.autoSuggest._inIME = false;
}
});
document.body.appendChild(this._platformAutoSuggestListDom);
this.element.winControl = this;
this.itemDataSource = null;
this.keyup = null;
this._clusterIndex = 0,
this._selectionMode = CommonJS.AutoSuggest.freeForm;
this._inputPane = Windows.UI.ViewManagement.InputPane.getForCurrentView();
this._selectedIndex = -1;
this._appViewType = Windows.UI.ViewManagement.ApplicationView.value;
this._windowEventManager = CommonJS.WindowEventManager.getInstance();
//when a textBox(this.element) is focus, we should update the autosuggest list, because there might be text already in
//the textBox, or the autosuggest list contains old data for privous textBox (notice that autoSuggest is a global object
//which will be attached with different textbox dynamically
//
//but after DP6, the focus behavior is changed, a div will be focus when scorlled/touched, so scoll on the autosuggest list will 
//cause textbox to blur, which make soft keyboard to hide immediately and unexpectedly. In order to keep soft keyboard up when scrolling,
//we need to set a onfocus function to each autosuggest item so that when they are focused, we set focus back to textBox.
//but, this introduce an issue that updating the list everytime you touch a autosuggest button, the side effect is the list will be
//clearred, recreated and so reset the position during the middle of a scrolling, 
//set this variable to false so autosuggest._focus() will know not to call _updateList in this case
//MAKE SURE this variable is always reset to true in autosuggest._blur();
this._updateListWhenFocus = true;
//in case that there are exactly five dropdownItems such that DropdownListHeight === maxDropdownListHeight,
//and the autosuggest will hide when inputting a certain character for some particular strings in Flight status. See Bug 315024
//here maxDropdownListHeight: 199 (not 200) to avoid DropdownListHeight === maxDropdownListHeight will fix this problem.
this._staticVars = { dropdownItemHeight: 40, maxDropdownListHeight: 199, autosuggestBarHeight: 68 };
WinJS.UI.setOptions(this, options);
},
{
_isInvalidEntry: false,
_panoramaID: null,
_clusterIndex: 0,
_itemDataSource: null,
_keyup: null,
_userOnItemSelection: null,
_submitOnBlur: false,
_selectionMode: null,
_inputPane: null,
_isShowing: false,
_selectedIndex: -1,
_alreadyOnSelection: false, //multiple places to call _onSelection, use this to make sure only call this method once in a focus-blur cycle.
_forceDropdown: false, //true indicate that a mouse click just happened, need to show dropdown style
_inIME: false,
_appViewType: null,
_windowEventManager: null,
_updateListWhenFocus: true,
_staticVars: null,
zIndex: {
set: function set(value) {
this._platformAutoSuggestListDom.style.zIndex = value;
}
},
hide: function hide() {
this._isShowing = true;
/// <disable>JS3053</disable>
this._blur();   // JSCOPBUG: _blur takes an event parameter...
/// <enable>JS3053</enable>
},
/// <field>
/// suggestions data
/// </field>
itemDataSource: {
set: function set(value) {
if (!this._isInvalidEntry) {
this._itemDataSource = value;
this._updateList(value);
}
}
},
/// <field>
/// forceSelection or freeForm
/// </field>
selectionMode: {
set: function set(value) {
this._selectionMode = value;
}
},
/// <field>
/// user callback on item being selected
/// </field>
onItemSelection: {
set: function set(value) {
this._userOnItemSelection = value;
}
},
/// <field>
/// String to show on invalid entry
/// </field>
isInvalidEntry: {
get: function get() {
return this._isInvalidEntry;
}
},
/// <field>
/// ID of panorama this textbox belongs to, need this info to reset cluster
/// </field>
panoramaID: {
set: function set(value) {
this._panoramaID = value;
}
},
/// <field>
/// clusterIndex that need to be reset to 
/// </field>
clusterIndex: {
set: function set(value) {
this._clusterIndex = value;
}
},
/// <field>
/// callback oninput
/// </field>
keyup: {
set: function set(value) {
this._keyup = value;
}
},
_onInputPaneShowing: function _onInputPaneShowing(event) {
var that = CommonJS.autoSuggest;
if (!document.activeElement) {
console.warn("document.activeElement is null, something goes wrong, at _onInputPaneShowing()");
return;
}
if (!(that && that.element)) {
console.warn("autosuggest.element is null, something goes wrong, at _onInputPaneShowing()");
return;
}
if (document.activeElement.uniqueID === that.element.uniqueID) {
var rectangle = event.occludedRect;
var ev = null;
//calculate the offset based on inputPane hieght and element position;
CommonJS.displacement = 0;
CommonJS.offsetToEnsureInputInView = 0;
var delta = rectangle.y - that._staticVars.autosuggestBarHeight - that.element.getBoundingClientRect().bottom;
if (delta < 0) {
CommonJS.offsetToEnsureInputInView = delta;
CommonJS.displacement = CommonJS.offsetToEnsureInputInView;
// Appying the transform directly via css style causes the text box to lose focus - a reported Windows/IE bug.
// So as a workaround we are using WinJS.UI.executeTransition API to apply the transform, and setting the focus 
// back to the text box once the promise is fulfilled
WinJS.UI.executeTransition(document.body.parentNode,
{
property: "transform",
delay: 0,
duration: 200,
timing: "linear",
to: "translate(0px," + CommonJS.offsetToEnsureInputInView + "px)"
}).done(function elementTransformCompleteOnInputPaneShowing() {
if (event.srcElement) {
event.srcElement.focus();
}
});
that.element.value = that.element.value.trim();
ev = document.createEvent("HTMLEvents");
ev.initEvent("input", false, true);
that.element.dispatchEvent(ev);
event.ensuredFocusedElementInView = true;
} else {
ev = document.createEvent("HTMLEvents");
ev.initEvent("input", false, true);
that.element.dispatchEvent(ev);
}
}
that._showList();
},
_onInputPaneHiding: function _onInputPaneHiding(event) {
var that = CommonJS.autoSuggest;
if (CommonJS.displacement !== 0) {
//call _hideList() here to avoid keeping bar/dropdownList in old position when the whole page is moving back to orginal position
that._hideList();
document.body.parentNode.style.msTransform = "translate(0px,0px)";
CommonJS.displacement = 0;
event.ensuredFocusedElementInView = true;
}
that._isShowing = false;
//inputPane hide event is NOT always after textbox blur event, this code double check textbox focus/blur status
//also, after onItemSelection we dont want to show the list
//Generally, activeElement indicates the element on focus (then we show dropdown list)
//but there is an exception: 
//The active element retains focus in the parent document even when focus is shifted from the parent to another application,
//(see http://msdn.microsoft.com/en-us/library/ie/ms533065%28v=vs.85%29.aspx)
//So currently we keep showing dropdown list when snapping another app from the backstack.
//And in order to align the dropdown list in the case that textbox is onblur but dropdown list still shows up,
//we DON'T remove WINDOW_RESIZE event listener on blur.
if (document.activeElement === that.element && !that._alreadyOnSelection) {
that._forceDropdown = true;
that._showList();
} else {
that._blur();
}
if (document.activeElement !== that.element) { //remove myself if textBox is not focus
that._inputPane.removeEventListener("hiding", that._onInputPaneHiding, false);
}
},
_wireupEvents: function _wireupEvents() {
var that = this;
this._inputPane.addEventListener("showing", that._onInputPaneShowing, false);
this._inputPane.addEventListener("hiding", that._onInputPaneHiding, false);
this._windowEventManager.addEventListener(CommonJS.WindowEventManager.Events.MOUSE_BACK, that._dismissAutoSuggest);
this._windowEventManager.addEventListener(CommonJS.WindowEventManager.Events.MOUSE_FORWARD, that._dismissAutoSuggest);
//WINDOW_RESIZE event listener is removed only if no autoSuggest or no text box to which autoSuggest attached exists.
//It might be added more times than it is removed,
//but since the duplicate event handlers are discarded if you register multiple identical event handlers on the same event target, 
//and we will eventually remove it, there won't be memory leak.
//(see http://msdn.microsoft.com/en-us/library/ie/ff975245%28v=vs.85%29.aspx)
this._windowEventManager.addEventListener(CommonJS.WindowEventManager.Events.WINDOW_RESIZE, that._onWindowResize);
//monitor scroll event and hide autoSuggest dom object if user does so.
window.addEventListener("scroll", that._dismissAutoSuggest);
window.addEventListener("mousewheel", that._mousewheel);
window.addEventListener("resize", that._orientationChange); //redraw list on orientation change
},
_dismissAutoSuggest: function _dismissAutoSuggest() {
if (CommonJS.autoSuggest) {
CommonJS.autoSuggest._hideList();
}
},
_mousewheel: function _mousewheel(event) {
if (CommonJS.autoSuggest && WinJS.Utilities.hasClass(CommonJS.autoSuggest._platformAutoSuggestListDom, "platformAutoSuggestList")) {
return; //do nothing if this is a touch style autosuggest list, avoid crashing after 8321
}
//walk through the Dom tree to see if activeElement(in this case, the element which is being clicked/tapped) belongs to autosuggest List Dom
var childOfAutoSuggestDom = false;
var checkElement = event.srcElement;
while (checkElement) {
if (WinJS.Utilities.hasClass(checkElement, "platformAutoSuggestListBase")) {
childOfAutoSuggestDom = true;
break;
}
checkElement = checkElement.parentElement;
}
if (!childOfAutoSuggestDom && CommonJS.autoSuggest) {
CommonJS.autoSuggest._hideList();
}
},
_onWindowResize: function _onWindowResize() {
if(CommonJS.autoSuggest && (Windows.UI.ViewManagement.InputPane.getForCurrentView().occludedRect.height === 0 || CommonJS.autoSuggest._forceDropdown)) {
CommonJS.autoSuggest._positionList();
}
},
_orientationChange: function _orientationChange(event) {
if (CommonJS.autoSuggest) {
if (CommonJS.autoSuggest._appViewType !== Windows.UI.ViewManagement.ApplicationView.value) {
CommonJS.autoSuggest._appViewType = Windows.UI.ViewManagement.ApplicationView.value;
if (CommonJS.autoSuggest._isShowing) { //do not force showing if list is hidden
CommonJS.autoSuggest._showList();
}
}
}
},
_removeEvents: function _removeEvents() {
var that = this;
this._inputPane.removeEventListener("showing", that._onInputPaneShowing, false);
//could not remove InputPane Hiding EventListener here since InputPane Hiding Event will be raised later than textBox blur event, which will call this function
this._windowEventManager.removeEventListener(CommonJS.WindowEventManager.Events.MOUSE_BACK, that._dismissAutoSuggest);
this._windowEventManager.removeEventListener(CommonJS.WindowEventManager.Events.MOUSE_FORWARD, that._dismissAutoSuggest);
window.removeEventListener("scroll", that._dismissAutoSuggest);
window.removeEventListener("mousewheel", that._mousewheel);
window.removeEventListener("resize", that._orientationChange);
},
_updateList: function _updateList(data) {
var i;
this._platformAutoSuggestListDom.innerHTML = "";
this._selectedIndex = -1;
this._platformAutoSuggestListDom.setAttribute("onfocus", "CommonJS.autoSuggest._SetFocusBack();"); //need to set focus back when user touch non-button area of the autosuggestlist Dom
if (data && data.length > 0) {
if (this._selectionMode === CommonJS.AutoSuggest.forceSelection) {
this._selectedIndex = 0; //only set default item in forceSelection mode when there is valid data
}
var dropdownListHeight = this._staticVars.dropdownItemHeight * data.length < this._staticVars.maxDropdownListHeight ? this._staticVars.dropdownItemHeight * data.length : this._staticVars.maxDropdownListHeight;
this._platformAutoSuggestListDom.style.height = dropdownListHeight + "px";
for (i = 0; i < data.length; i++) {
if (i >= 20) {
//limit to 20 items;
break;
}
var item = this._createItem(data[i], i);
if (item) {
item.id = "CommonJS.AutoSuggest_" + i;
item.setAttribute("data-index", i);
this._platformAutoSuggestListDom.appendChild(item);
}
}
if (this._selectionMode === CommonJS.AutoSuggest.forceSelection) {
this._platformAutoSuggestListDom.children[0].setAttribute("aria-selected", true);
}
if (!this._isShowing) {
this._showList();
}
} else if (data) { //skip _updateList(null) case which is used internally
this._isShowing = false;
this._hideList();
}
msWriteProfilerMark("Platform:Autosuggest:suggestlistUpdated");
},
_createItem: function _createItem(itemData, i) {
var symbol = (itemData.Symbol) ? itemData.Symbol : "";
var title = (itemData.Title) ? itemData.Title : "";
var subtitle = (itemData.Subtitle) ? itemData.Subtitle : "";
var comment = (itemData.Comment) ? itemData.Comment : "";
if (symbol === "" && title === "" && subtitle === "" && comment === "") {
return null;
}
var buttonItem = document.createElement("button"),
listItem = document.createElement("div"),
listItemIcon = document.createElement("image"),
listItemLabel = document.createElement("div"),
listItemTitle = document.createElement("div"),
listItemSubTitle = document.createElement("div");
var listItemComment = document.createElement("div");
WinJS.Utilities.addClass(buttonItem, "win-interactive");
WinJS.Utilities.addClass(listItem, "platformAutoSuggestListItem");
WinJS.Utilities.addClass(listItemIcon, "platformAutoSuggestListItemIcon");
WinJS.Utilities.addClass(listItemLabel, "platformAutoSuggestListItemLabel");
WinJS.Utilities.addClass(listItemTitle, "platformAutoSuggestListItemTitle");
WinJS.Utilities.addClass(listItemSubTitle, "platformAutoSuggestListItemSubTitle");
WinJS.Utilities.addClass(listItemComment, "platformAutoSuggestListItemComment");
listItemLabel.innerText = symbol;
listItemTitle.innerText = title;
listItemSubTitle.innerText = subtitle;
listItemComment.innerText = comment;
if (itemData.Icon) {
listItemIcon.src = itemData.Icon;
listItem.appendChild(listItemIcon);
}
//setting back focus to textBox, this is critical for DP6, in DP6 when a div is tapped, it will be focus, which will blur TextBox
buttonItem.setAttribute("onfocus", "CommonJS.autoSuggest._SetFocusBack();");
buttonItem.setAttribute("data-displayedText", itemData.displayedText ? itemData.displayedText : symbol + " " + title + " " + subtitle);
buttonItem.setAttribute("data-value", itemData.value ? itemData.value : symbol + " " + title + " " + title);
buttonItem.setAttribute("onmspointerup", "CommonJS.autoSuggest._selectedIndex = " + i + "; CommonJS.autoSuggest._onClick(event)");
buttonItem.setAttribute("role", "option");
listItem.appendChild(listItemLabel);
listItem.appendChild(listItemTitle);
listItem.appendChild(listItemSubTitle);
listItem.appendChild(listItemComment);
buttonItem.appendChild(listItem);
return buttonItem;
},
_SetFocusBack: function _SetFocusBack() {
if (CommonJS.autoSuggest) {
CommonJS.autoSuggest._updateListWhenFocus = false;
CommonJS.autoSuggest.element.focus();
}
},
_onClick: function _onClick(event) {
CommonJS.autoSuggest._onItemSelection(CommonJS.autoSuggest._selectedIndex, event);
CommonJS.autoSuggest._hideList();
},
_keydown: function _keydown(event) {
if (this._inIME) {
return;
}
if (event.key === "Enter") {
if (this._selectionMode === CommonJS.AutoSuggest.forceSelection || this._selectedIndex > -1) {
if (this._itemDataSource && this._itemDataSource.length > 0) {
var selectedItem = this._platformAutoSuggestListDom.children[this._selectedIndex];
if (selectedItem) {
this.element.setAttribute("data-value", selectedItem.getAttribute("data-value"));
this.element.value = selectedItem.getAttribute("data-displayedText");
this.element.blur();
this._onItemSelection(this._selectedIndex, event);
} else {
this._selectedIndex = -1;
this._isInvalidEntry = true;
this.element.blur();
this._onItemSelection(-1, event);
}
} else {
this.element.setAttribute("data-value", "");
this._selectedIndex = -1;
this._isInvalidEntry = true;
this.element.blur();
this._onItemSelection(-1, event);
}
} else {
this.element.setAttribute("data-value", "");
this.element.blur();
}
} else if (event.key === "Down") {
var downItemsLength = this._platformAutoSuggestListDom.children.length;
if (downItemsLength > 0) {
this._selectedIndex++;
if (this._selectedIndex >= downItemsLength) {
if (this._selectionMode === CommonJS.AutoSuggest.forceSelection) {
this._selectedIndex = 0;
} else {
this._selectedIndex = -1;
}
}
this._updateSelectedItem(this._selectedIndex);
if (this._selectedIndex >= 0) {
this._platformAutoSuggestListDom.children[this._selectedIndex].scrollIntoView(false);
}
}
event.cancelBubble = true;
} else if (event.key === "Up") {
var upItemsLength = this._platformAutoSuggestListDom.children.length;
if (upItemsLength > 0) {
this._selectedIndex--;
if (this._selectedIndex === -1) {
if (this._selectionMode === CommonJS.AutoSuggest.forceSelection) {
this._selectedIndex = upItemsLength - 1;
}
}
if (this._selectedIndex < -1) {
this._selectedIndex = upItemsLength - 1;
}
this._updateSelectedItem(this._selectedIndex);
if (this._selectedIndex >= 0) {
this._platformAutoSuggestListDom.children[this._selectedIndex].scrollIntoView(false);
}
}
event.cancelBubble = true;
} else if (event.key === "Left" || event.key === "Right") {
event.cancelBubble = true;
}
},
_updateSelectedItem: function _updateSelectedItem(index) {
var i;
var itemsLength = CommonJS.autoSuggest._platformAutoSuggestListDom.children.length;
for (i = 0; i < itemsLength; i++) {
CommonJS.autoSuggest._platformAutoSuggestListDom.children[i].setAttribute("aria-selected", false);
}
if (index > -1) {
CommonJS.autoSuggest._platformAutoSuggestListDom.children[index].setAttribute("aria-selected", true);
//note: this text change will NOT trigger oninput event so no _updatelist is called
CommonJS.autoSuggest.element.setAttribute("data-value", CommonJS.autoSuggest._platformAutoSuggestListDom.children[index].getAttribute("data-value"));
CommonJS.autoSuggest.element.value = CommonJS.autoSuggest._platformAutoSuggestListDom.children[index].getAttribute("data-displayedText");
}
},
_blur: function _blur() {
var that = this;
//walk through the Dom tree to see if activeElement(in this case, the element which is being clicked/tapped) is belong to autosuggest List Dom
var childOfAutoSuggestDom = false;
var checkElement = document.activeElement;
while (checkElement) {
if (WinJS.Utilities.hasClass(checkElement, "platformAutoSuggestListBase")) {
childOfAutoSuggestDom = true;
break;
}
checkElement = checkElement.parentElement;
}
//hide list only when user click/tap out side autoSuggestList Dom area
if (!childOfAutoSuggestDom) {
this._removeEvents(); //wire up events when focus, remove events when blur
this._hideList();
if (this._selectedIndex < 0) {
this.element.setAttribute("data-value", "");
} else {
var selectedItem = this._platformAutoSuggestListDom.children[that._selectedIndex];
if (selectedItem) {
this.element.setAttribute("data-value", selectedItem.getAttribute("data-value"));
this.element.value = selectedItem.getAttribute("data-displayedText");
} else {
console.warn("Could not find a selectedItem with valid index " + this._selectedIndex);
}
}
//remove aria Attributes which are for Narrator
this.element.removeAttribute("role");
this.element.removeAttribute("aria-autocomplete");
this.element.removeAttribute("aria-owns");
//reset this variable so not affect a normal list update in next this._focus()
this._updateListWhenFocus = true;
if (that._submitOnBlur) {
that._onItemSelection(that._selectedIndex, event);
}
}
//remove panorama MSManipulationStateChanged event handler
if (that._panoramaID) {
var panorama = document.getElementById(that._panoramaID);
if (panorama) {
var viewport = panorama.querySelector(".platformPanoramaViewport");
if (viewport) {
viewport.removeEventListener("MSManipulationStateChanged", that._dismissAutoSuggest);
} else {
console.warn("panorama viewport could not be found, something goes wrong, please check.");
}
}
}
},
_focus: function _focus(event) {
//work around for a Windows issue when textbox partly out of screen right edge, a dropdown list selection will casue display incorrect
var that = this;
if (that._panoramaID) {
var panorama = document.getElementById(that._panoramaID);
if (panorama && panorama.winControl) {
if (that.element.getBoundingClientRect().right > document.body.clientWidth) { //do not need to handle "left" case, as seen, Windows already handle that
panorama.winControl.currentClusterIndex = that._clusterIndex;
setTimeout(function autoSuggest_focusElement() {
that.element.focus();
}, 50);
}
//dismiss autosuggest list when scrolling, need to hook up to panorama control otherwise could not receive scroll event
var viewport = panorama.querySelector(".platformPanoramaViewport");
if (viewport) {
viewport.addEventListener("MSManipulationStateChanged", that._dismissAutoSuggest);
} else {
console.warn("panorama viewport could not be found, something goes wrong, please check.");
}
}
}
this._alreadyOnSelection = false;//set _alreadyOnSelection to false to initialize a focus-blur cycle
WinJS.Utilities.removeClass(this.element, "platformAutoSuggestTextBoxInvalid");
this._isInvalidEntry = false;
if (CommonJS.autoSuggest && CommonJS.autoSuggestID !== this.element.uniqueID) {
CommonJS.autoSuggest._removeEvents();
}
CommonJS.autoSuggest = this;
this._appViewType = Windows.UI.ViewManagement.ApplicationView.value;
this._wireupEvents();
CommonJS.autoSuggestID = this.element.uniqueID;
//set aria Attributes which are for Narrator
this.element.setAttribute("role", "combobox");
this.element.setAttribute("aria-autocomplete", "list");
this.element.setAttribute("aria-owns", "platformAutoSuggestList");
this.element.setAttribute("aria-live", "assertive"); //narrator read when content is changed
this.element.setAttribute("aria-atomic", "true");    //narrator always read the whole content
//always fire a input event to update the autosuggest list for new focus'ed textbox
if (this._updateListWhenFocus) {
var ev = document.createEvent("HTMLEvents");
ev.initEvent("input", false, true);
this.element.dispatchEvent(ev);
}
//handle a case that before tapping to focus on current textbox, soft keyboard is already up (for example, tap another textbox and then tap current textbox)
//check if there's enough vertical space to show autosuggest bar, if not, move window up.
var rectangle = Windows.UI.ViewManagement.InputPane.getForCurrentView().occludedRect;
if (rectangle && rectangle.height && rectangle.height > 0) {
var elementBottomY = this.element.getBoundingClientRect().bottom;
var delta = rectangle.y - elementBottomY;
if (delta < this._staticVars.autosuggestBarHeight) {
//window page may already be shifted up, add the offset (offsetToEnsureInputInView) when calculate
CommonJS.offsetToEnsureInputInView = CommonJS.offsetToEnsureInputInView + (rectangle.y - this._staticVars.autosuggestBarHeight - elementBottomY);
CommonJS.displacement = CommonJS.offsetToEnsureInputInView;
// Appying the transform directly via css style causes the text box to lose focus - a reported Windows/IE bug.
// So as a workaround we are using WinJS.UI.executeTransition API to apply the transform, and setting the focus 
// back to the text box once the promise is fulfilled
WinJS.UI.executeTransition(document.body.parentNode,
{
property: "transform",
delay: 0,
duration: 200,
timing: "linear",
to: "translate(0px," + CommonJS.offsetToEnsureInputInView + "px)"
}).done(function elementTransformCompleteOnFocus() {
if (event.srcElement) {
event.srcElement.focus();
}
});
}
}
},
_showList: function _showList() {
if (!CommonJS.autoSuggest || !CommonJS.autoSuggest.element) {
return;
}
//check if textbox is still in the DOM tree. Do not show autosuggest list without textbox
//This fixed bug 188821: when user type a string and then immediately press enter to submit the string, app might just close searchDialog, 
//but the async updateList call for that string will finish later, which will force autosuggest list to show, even when searchDialog is gone
if (!document.getElementById(CommonJS.autoSuggest.element.id)) {
return;
}
var previousList = document.getElementById("platformAutoSuggestList");
if (previousList) {
WinJS.Utilities.removeClass(previousList, "platformAutoSuggestList");
WinJS.Utilities.removeClass(previousList, "platformAutoSuggestListNonTouch");
WinJS.Utilities.removeClass(previousList, "platformAutoSuggestListAnimation");
WinJS.Utilities.removeClass(previousList, "platformAutoSuggestListShow");
}
//do not show autoSuggest dom object if no item to suggest
if (CommonJS.autoSuggest._platformAutoSuggestListDom.children.length === 0) {
CommonJS.autoSuggest._isShowing = false; //setting 'false' here is a must to force autoSuggest to show up again when swtiching from empty dropdown list to horizontal bar style
return;
}
CommonJS.autoSuggest._isShowing = true;
//the occludedRect_of_inputPane === zero means currently no inputPane on the screen, or see if we force to show dropdown style
if (Windows.UI.ViewManagement.InputPane.getForCurrentView().occludedRect.height === 0 || CommonJS.autoSuggest._forceDropdown) {
CommonJS.autoSuggest._forceDropdown = false;
WinJS.Utilities.removeClass(CommonJS.autoSuggest._platformAutoSuggestListDom, "platformAutoSuggestList");
WinJS.Utilities.addClass(CommonJS.autoSuggest._platformAutoSuggestListDom, "platformAutoSuggestListNonTouch");
WinJS.Utilities.addClass(CommonJS.autoSuggest._platformAutoSuggestListDom, "platformAutoSuggestListShow");
//call _positionlist() after assigning correct style to autosugguest element
CommonJS.autoSuggest._positionList();
} else {
var rectangle = Windows.UI.ViewManagement.InputPane.getForCurrentView().occludedRect;
CommonJS.autoSuggest._platformAutoSuggestListDom.style.left = "";
CommonJS.autoSuggest._platformAutoSuggestListDom.style.right = "";
CommonJS.autoSuggest._platformAutoSuggestListDom.style.top = "";
CommonJS.autoSuggest._platformAutoSuggestListDom.style.width = "";
//calculate position differently depends on if textbox elment is displaced
if (CommonJS.displacement && CommonJS.displacement !== 0) {
CommonJS.autoSuggest._platformAutoSuggestListDom.style.bottom = rectangle.height - 14 + CommonJS.offsetToEnsureInputInView - window.pageYOffset + "px";
//note: window.pageYOffset should not be calculated in offsetToEnsureInputInView, user may swipe screen up/down after the calculation
} else {
CommonJS.autoSuggest._platformAutoSuggestListDom.style.bottom = rectangle.height - 14 - window.pageYOffset + "px";
}
WinJS.Utilities.removeClass(CommonJS.autoSuggest._platformAutoSuggestListDom, "platformAutoSuggestListNonTouch");
WinJS.Utilities.addClass(CommonJS.autoSuggest._platformAutoSuggestListDom, "platformAutoSuggestList");
WinJS.Utilities.addClass(CommonJS.autoSuggest._platformAutoSuggestListDom, "platformAutoSuggestListAnimation");
WinJS.Utilities.addClass(CommonJS.autoSuggest._platformAutoSuggestListDom, "platformAutoSuggestListShow");
}
},
_positionList: function _positionList() {
var rects = this.element.getClientRects()[0];
if (rects) {
var inputBox = WinJS.Utilities.getPosition(this.element, document.body);
var body = WinJS.Utilities.getPosition(document.body, window);
var down = ((document.documentElement.clientHeight - rects.top) > this._staticVars.maxDropdownListHeight);
if (down) {
if (document.dir === "rtl") {
this._platformAutoSuggestListDom.setLeft((document.documentElement.clientWidth - rects.right) + "px");
} else {
this._platformAutoSuggestListDom.setLeft((rects.left) + "px");
}
//2 is the magic number of dropdownlist border, move dropdown list up to overlap with the border of textbox
this._platformAutoSuggestListDom.style.top = (rects.top + rects.height - 2) + "px";
this._platformAutoSuggestListDom.style.bottom = "";
this._platformAutoSuggestListDom.style.width = this.element.clientWidth + "px";
} else {
if (document.dir === "rtl") {
this._platformAutoSuggestListDom.style.left = (document.documentElement.clientWidth - rects.right) + "px";
} else {
this._platformAutoSuggestListDom.setLeft(rects.left + "px");
}
//200 is the magic number of dropdown style autosuggest list height, since current _platformAutoSuggestListDom.style could be in touch style, could not get this in runtime
this._platformAutoSuggestListDom.style.top = "";
this._platformAutoSuggestListDom.style.bottom = (document.documentElement.clientHeight - rects.top) + "px";
this._platformAutoSuggestListDom.style.width = this.element.clientWidth + "px";
}
}
},
_animate: function _animate(element, from, to) {
return WinJS.UI.executeAnimation(element,
{
property: "-ms-transform",
delay: 0,
duration: 733,
timing: "cubic-bezier(0.3, 0.9, 0.2, 1)",
from: from,
to: to
}
);
},
_onItemSelection: function _onItemSelection(index, event) {
if (!this._alreadyOnSelection) { //only call _onItemSelection once in a focus-blur cycle
msWriteProfilerMark("Platform:Autosuggest:ItemSelected");
//update the textbox with selected text, previous only do this in _blur(),
//this is for the case in Travel App that _onItemSelection is called but TextBox is still in focus
if (index < 0) {
this.element.setAttribute("data-value", "");
} else {
var selectedItem = this._platformAutoSuggestListDom.children[index];
if (selectedItem) {
this.element.setAttribute("data-value", selectedItem.getAttribute("data-value"));
this.element.value = selectedItem.getAttribute("data-displayedText");
} else {
console.warn("Could not find a selectedItem with valid index " + index);
}
}
//dismiss autosuggest list. Current mechanism is always show autosuggest list when focus, but this is an exception.
//Note that inpuPane hide event will be after than this _onItemSelection() call, so we still need to handle that case
//in inpuPane hide eventlistener to not show the list if submit(tap/click) selection from touch autosuggest bar.
this._isShowing = false;
this.element.blur();
if (typeof this._userOnItemSelection === "function") {
this._userOnItemSelection(index, event);
}
this._alreadyOnSelection = true;
}
},
_hideList: function _hideList() {
var list = document.getElementById("platformAutoSuggestList");
if (list) {
WinJS.Utilities.removeClass(list, "platformAutoSuggestList");
WinJS.Utilities.removeClass(list, "platformAutoSuggestListNonTouch");
WinJS.Utilities.removeClass(list, "platformAutoSuggestListAnimation");
WinJS.Utilities.removeClass(list, "platformAutoSuggestListShow");
}
if (CommonJS.autoSuggest) {
CommonJS.autoSuggest._forceDropdown = false; //clear to avoid keeping old info in the case of no suggesttion so _showlist() is skipped
CommonJS.autoSuggest._isShowing = false;
}
if (!CommonJS.autoSuggest || !document.getElementById(CommonJS.autoSuggest.element.id)) {
this._windowEventManager.removeEventListener(CommonJS.WindowEventManager.Events.WINDOW_RESIZE, this._onWindowResize);
}
}
},
{
forceSelection: "forceSelection",
freeForm: "freeForm"
})
});
})();
