﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: CloudPano.js */
(function () {
"use strict";
WinJS.Namespace.define("CommonJS", {
CloudPanorama: WinJS.Class.define(function CloudPanorama_ctor(state) {
// Initialize state
this._state = state || {};
this._backgroundList = [];
// clear any existing theme;
CommonJS.setTheme();
if (Windows.UI.ViewManagement.ApplicationView.value === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
Windows.UI.ViewManagement.ApplicationView.tryUnsnap();
}
// Make sure to clear saved state if
// this is a navigation bar event
if (this._state.isNavBarEvent) {
this._state.panoramaState = null;
}
// Get panorama state
this._panoramaState = this._state.panoramaState || {};
this._panoramaElement = document.getElementById("cloudPanoMainlist");
if (this._panoramaElement) {
this._panoramaControl = this._panoramaElement.winControl;
}
// If we have a passed scroll position
if (!this._panoramaState.scrollPosition) {
try {
var scrollPosition = null;
if (CommonJS.Immersive.isPortrait()) {
if (state.pScroll !== null) {
scrollPosition = (parseInt(state.pScroll) * CommonJS.Immersive.calculateScalingRatio()).toFixed();
}
} else {
if (state.lScroll !== null) {
scrollPosition = (parseInt(state.lScroll) * CommonJS.Immersive.calculateScalingRatio()).toFixed();
}
}
this._panoramaState.scrollPosition = scrollPosition;
if (state.minIdx !== null) {
this._panoramaState.count = parseInt(state.minIdx);
}
} catch (e) {
// Couldn't set scroll state
this._panoramaState.scrollPosition = null;
this._panoramaState.count = null;
}
}
// Get pano config settings
this._configSettings = PlatformJS.Services.configuration.getDictionary("CloudPanoSettings");
// Create pano clusters
this._clusters = new CommonJS.CloudPanoBuilder.Clusters();
// Create new instrumentation manager instance
this._instrumentationManager = new CommonJS.CloudPanoInstrumentation.PanoInstrumentationManager(this._configSettings, this._state);
// Add visibility change listener
this._documentVisibilityBinding = this._onVisibilityChange.bind(this);
if (document) {
document.addEventListener("msvisibilitychange", this._documentVisibilityBinding);
}
// Create panel scroll binding (need to listen on scroll
// events to send viewed pages info)
this._panelScrollBinding = this._onScroll.bind(this);
if (this._panoramaControl) {
this._panoramaControl.addScrollListener(this._panelScrollBinding);
}
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
this._errorHandler("no network available");
}
// Get client market
this._clientMarket = Platform.Utilities.Globalization.getCurrentMarket() || "";
// Get store version (default is "v1")
if (!this._state.sver) {
this._state.sver = "v1";
}
if (!this._state.campaign) {
// Can't load pano
this._errorHandler("no campaign specified", true);
}
if (!this._state.market) {
// Market fallback is worldwide
this._state.market = "ww";
}
if (!this._state.index) {
// Index is optional and used
// for nested pano navigation
// within the same campaign
this._state.index = "";
}
this._inRetry = false;
this._snappedDiv = document.getElementById("cloudPanoSnappedMode");
this._snappedImage = document.getElementById("cloudPanoSnappedModeImage");
if (this._snappedImage) {
this._snappedImage.attachEvent("onclick", function snappedCloudPano_onClick() {
Windows.UI.ViewManagement.ApplicationView.tryUnsnap();
});
this._snappedImage.alternateText = PlatformJS.Services.resourceLoader.getString("/platform/advertisement") || "Advertisement";
}
// Initialize pano
this._initialize();
}, {
_state: null,
_continueRendering: true,
_clientMarket: null,
_panoramaState: null,
_configSettings: null,
_instrumentationManager: null,
_documentVisibilityBinding: null,
_panelScrollBinding: null,
_clusters: null,
_panoData: null,
_dataPromise: null,
_backgroundList: [],
_dataCacheID: "CloudPanoDataCache",
_panoramaElement: null,
_moduleElement: null,
_templateStylesElement: null,
_uiStylesElement: null,
_customStylesElement: null,
_scriptLinks: null,
_supportedVersion: {
min: 0,
max: 2.0,
//clusterItemLayout is the minimum version where we started to support layout for cluster items
clusterItemLayout: 2.0
},
_pagesViewed: {},
_lastScrollTimestamp: null,
_lastHiddenTimestamp: null,
_hiddenElapsedTime: 0,
_errorHandled: false,
_safeServerList: null,
_navigatedAway: false,
_inRetry: false,
_cloudPanoLoadTimeout: null,
_snappedDiv: null,
_snappedImage: null,
_snappedImageControl: null,
_initialize: function _initialize() {
var that = this;
this._pagesViewed = {};
this._continueRendering = true;
this._errorHandled = false;
try {
// If we have splash specified
var isCached = false, isCustom = false, loadSplashPromise = WinJS.Promise.wrap(null);
var fragments = document.getElementsByClassName("fragment");
var fragment = fragments ? fragments[0] : null;
if (fragment){
if (this._state.bkgColor) {
fragment.style.background = "#" + this._state.bkgColor;
} else if (this._state.splashBkg) {
fragment.style.background = "#" + this._state.splashBkg;
}
}
if (this._state.splashName && this._state.splashBkg &&
this._state.splashW && this._state.splashH) {
if (this._snappedDiv) {
this._snappedDiv.style.background = "#" + this._state.splashBkg;
}
// Create base splash object
isCustom = true;
loadSplashPromise = this._prepareCustomSplash().then(function afterSplashDownloaded(splash) {
if (splash) {
isCached = splash.isCached;
if (that._snappedImage) {
that._snappedImage.style.backgroundImage = "url('" + splash.imageUri + "')";
}
}
return splash;
});
}
else {
var splashData = PlatformJS.Services.configuration.getDictionary("CloudPanoSplash");
if (splashData.imageUri && splashData.height && splashData.width && splashData.backgroundColor) {
if (fragment && !this._state.bkgColor && !this._state.splashBkg) {
fragment.style.background = splashData.backgroundColor.value;
}
if (that._snappedDiv) {
that._snappedDiv.style.background = splashData.backgroundColor.value;
}
if (that._snappedImage) {
that._snappedImage.style.backgroundImage = "url('" + splashData.imageUri.value + "')";
}
} else {
if (that._snappedDiv) {
that._snappedDiv.style.background = "white";
}
if (that._snappedImage) {
that._snappedImage.style.backgroundImage = "url('/CommonControls/Common/images/snap_placeholder.png')";
}
}
isCached = true;
}
// If no splash screen is already loaded
if (!CommonJS.Progress.playingSplash) {
// Avoid showing splash screen for back stack navigation
if (!this._state.panoramaState) {
loadSplashPromise.then(function (splash) {
CommonJS.Progress.initCloudSplashScreen(splash);
// Log splash screen loaded event
if (that._instrumentationManager) {
that._instrumentationManager.logSplashScreenShown(splash, isCached, isCustom);
}
});
} else {
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
}
}
// Get pano definition
var dataSourceID = this._state.dataSource || "AdPanoCMSDataSource";
var campaign = this._state.campaign;
var market = this._state.market;
var index = this._state.index;
this._dataPromise = this._queryAsync(dataSourceID, campaign, market, index);
// Chain version validation and template/resource fetching
this._dataPromise = this._dataPromise.then(function checkVersion(panoData) {
var dataVersion = that._supportedVersion.min;
if (panoData && panoData.version) {
var versiontype = typeof (panoData.version);
if (versiontype === "string") {
dataVersion = parseFloat(panoData.version);
} else if (versiontype === "number") {
dataVersion = panoData.version;
} else if (versiontype === "object" && panoData.version.major) {
dataVersion = panoData.version.major + ".";
if (panoData.version.minor) {
dataVersion += panoData.version.minor;
}
else {
dataVersion += "0";
}
dataVersion = parseFloat(dataVersion);
} else {
dataVersion = NaN;
}
panoData.version = dataVersion;
}
// We support up to max major version and all minor revs after that
if (dataVersion === NaN || dataVersion >= that._supportedVersion.max + 1 || dataVersion < that._supportedVersion.min) {
that._panoData = null;
var error = "Unsupported pano version: " + dataVersion + ". Client supported max version = " + that._supportedVersion.max + ". Client supported min version = " + that._supportedVersion.min;
that._errorHandler(error, true);
return WinJS.Promise.wrap(null);
} else {
return WinJS.Promise.join([that._initTemplate(panoData), that._downloadResources(panoData)]);
}
}, function (error) {
that._errorHandler(error);
}).then(function () {
that._preprocessCloudData();
that._replaceResources();
return WinJS.UI.processAll();
}, function (error) {
that._errorHandler(error);
}).then(function () {
return that._populatePages();
}, function (error) {
that._errorHandler(error);
});
// If we have timeout settings
if (!this._cloudPanoLoadTimeout &&
this._configSettings &&
this._configSettings.CloudPanoLoadTimeout &&
this._configSettings.CloudPanoLoadTimeout.value > 0) {
this._cloudPanoLoadTimeout = this._configSettings.CloudPanoLoadTimeout.value;
}
if (this._cloudPanoLoadTimeout) {
// Set pano load timeout
this._dataPromise = WinJS.Promise.timeout(this._cloudPanoLoadTimeout, this._dataPromise);
}
this._dataPromise.then(function AfterPagePopulated() {
if (that._inRetry && that._continueRendering) {
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
}
return WinJS.Promise.wrap(null);
},
function CloudPanoLoadTimeOut(error) {
that._errorHandler(error);
});
} catch (e) {
this._errorHandler(e);
}
},
_prepareCustomSplash: function _prepareCustomSplash() {
var splash = {
backgroundColor: "#" + this._state.splashBkg,
imageLocation: {
width: parseInt(this._state.splashW),
height: parseInt(this._state.splashH)
}
},
that = this;
// Create splash screen query service
var splashDataSource = this._state.splashDataSource || "AdPanoSplashCMSDataSource";
var queryService = new PlatformJS.DataService.QueryService(splashDataSource);
var queryParams = PlatformJS.Collections.createStringDictionary();
queryParams.insert("sver", this._state.sver);
queryParams.insert("campaign", this._state.campaign);
queryParams.insert("market", this._state.market);
queryParams.insert("filename", this._state.splashName);
queryParams.insert("cmkt", this._clientMarket);
var options = new Platform.DataServices.QueryServiceOptions();
options.useCompatDataModel = true;
// Get splash screen url
var splashUrl = queryService.getUrl(queryParams);
// Try to find splash image in cache
return PlatformJS.Cache.CacheService.getInstance(this._dataCacheID).findEntry(splashUrl, {
fileNameOnly: true
}).then(function splashCacheFetch_completion(response) {
if (response && !response.isStale() && response.dataValue) {
splash.imageUri = response.dataValue;
splash.isCached = true;
} else {
splash.imageUri = splashUrl;
splash.isCached = false;
PlatformJS.Utilities.fetchImage(that._dataCacheID, splashUrl, null); // don't wait this async function.
}
return splash;
});
},
getPageImpressionContext: function getPageImpressionContext() {
// Used by navigator to build the impression for Platform.Telemetry
return "/Cloud Pano";
},
getPageData: function getPageData() {
var that = this;
// Chain data load to master page load
if (this._dataPromise ) {
return this._dataPromise.then(function (complete) {
if (!that._navigatedAway) {
return {
showInertiaBackground: true,
panoramaState: that._panoramaState,
clusters: that._clusters.dataSource(),
};
} else {
return null;
}
});
} else {
return WinJS.Promise.wrap({
showInertiaBackground: true,
panoramaState: that._panoramaState,
clusters: that._clusters.dataSource(),
});
}
},
onBindingComplete: function onBindingComplete() {
var title = PlatformJS.Services.resourceLoader.getString("/platform/advertisement");
var headerTitle = document.getElementById("header_title");
if (headerTitle) {
headerTitle.setAttribute("aria-label", title);
}
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
// Log pano loaded event
if (this._instrumentationManager) {
this._instrumentationManager.logPanoLoaded();
}
},
_onVisibilityChange: function _onVisibilityChange(evt) {
// Validate
if (document) {
// Get hidden flag
var isHidden = document.msHidden;
// If document is now hidden
if (isHidden) {
// Set last hidden time stamp
this._lastHiddenTimestamp = new Date();
} else if (this._lastHiddenTimestamp) {
// Get current time stamp
var currentTime = new Date();
// Update hidden elapsed time
this._hiddenElapsedTime += currentTime - this._lastHiddenTimestamp;
}
}
},
_onScroll: function _onScroll(evt) {
// Validate
var that = this;
// Initialize last scroll time stamp
// if not already initialized
var firstEvent = false;
if (!this._lastScrollTimestamp) {
this._lastScrollTimestamp = new Date();
firstEvent = true;
}
// Get current time stamp
var currentTimestamp = new Date();
// Only handle this event once
// every 250ms for perf
var elapsedTime = currentTimestamp - this._lastScrollTimestamp;
if (firstEvent || elapsedTime >= 250) {
this._lastScrollTimestamp = currentTimestamp;
msSetImmediate(function cloudPano_onScoll() {
var panoControl = that._panoramaControl;
var instrumentationManager = that._instrumentationManager;
if (that && panoControl && instrumentationManager &&
evt && evt.target) {
// Get current page
var current = panoControl.current;
var bodyWdith = document.body.offsetWidth;
if (current && current.offset && current.width && bodyWdith) {
// Get page index
var currentPageIdx = (current.index || 0) + 1;
// If new page was viewed
if (!that._pagesViewed[currentPageIdx]) {
that._pagesViewed[currentPageIdx] = currentPageIdx;
instrumentationManager.logPageViewed(currentPageIdx);
}
// Get scroll position
var scrollPosition = evt.target.scrollPosition || 0;
// We need to determine how much
// of next page is visible to the user
current = current.next;
var nextPagVisible = true;
while (nextPagVisible && current &&
current.offset && current.width &&
current.index) {
// By default next page isn't visible
nextPagVisible = false;
// Update page index
currentPageIdx = current.index + 1;
if (!that._pagesViewed[currentPageIdx]) {
// Calculate scroll position that
// correspond with right-most next page edge
// in document coordinates
var nextPageEdge = current.offset + current.width;
// Calculate right-most visible edge in
// document coordinates
var visiblePageEdge = scrollPosition + bodyWdith;
// If user is seeing next page fully
if (visiblePageEdge >= nextPageEdge) {
// Update viewed page index
that._pagesViewed[currentPageIdx] = currentPageIdx;
instrumentationManager.logPageViewed(currentPageIdx);
// Move to next page
current = current.next;
nextPagVisible = true;
}
}
}
}
}
});
}
},
getPageState: function getPageState() {
if (this._panoramaControl && this._panoramaControl.getPanoramaState) {
this._state.panoramaState = this._panoramaControl.getPanoramaState();
}
if (this._state.isNavBarEvent) {
this._state.isNavBarEvent = false;
}
return this._state;
},
onNavigateAway: function onNavigateAway(event) {
this._navigatedAway = true;
this._continueRendering = false;
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
if (this._dataPromise) {
this._dataPromise.cancel();
this._dataPromise = null;
}
// Log pano closed event
if (this._instrumentationManager) {
this._instrumentationManager.logPanoClosed(this._pagesViewed, this._hiddenElapsedTime);
}
// Hide panorama (avoids visual artifacts
// since we're removing the dynamically linked styles/templates)
if (this._panoramaElement) {
this._panoramaElement.style.display = "none";
}
},
onLayoutChange: function onLayoutChange(view) {
if (PlatformJS.Services.manifest.snappedChannels.size > 0) {
if (view === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
this._continueRendering = false;
return {
pageInfo: {
page: "CommonJS.SnappedCloudPano",
fragment: "/Common/CloudPano/snappedCloudPano.html"
},
state: this._state
};
} else {
this._updateBackgroundImage();
return this._panoramaControl.onLayoutChange(view);
}
} else {
return this._panoramaControl.onLayoutChange(view);
}
},
onOrientationChange: function onOrientationChange(layout) {
this._updateBackgroundImage();
this._panoramaControl.onOrientationChange(layout);
},
_updateBackgroundImage: function _updateBackgroundImage() {
if (this._backgroundList.length && this._panoramaControl) {
var that = this;
this._backgroundList.forEach(function (backgroundData) {
if (backgroundData.key) {
var imageData = that._createOrientationImageData(backgroundData);
var itemsContainerControl = that._panoramaControl.getClusterContentControl(backgroundData.key);
if (itemsContainerControl) {
itemsContainerControl.imageData = imageData;
} else {
var item = that._clusters.getItem(backgroundData.key);
if (item && item.clusterContent && item.clusterContent.contentOptions) {
item.clusterContent.contentOptions.imageData = imageData;
}
that._clusters.setItem(backgroundData.key, item);
}
}
});
}
},
dispose: function dispose() {
CommonJS.Error.removeError();
this._backgroundList = [];
// Clear document visibility binding
if (document && this._documentVisibilityBinding) {
document.removeEventListener("msvisibilitychange", this._documentVisibilityBinding);
this._documentVisibilityBinding = null;
}
if (this._panoramaControl) {
// Clear panel scroll binding
if (this._panelScrollBinding) {
this._panoramaControl.removeScrollListener(this._panelScrollBinding);
this._panelScrollBinding = null;
}
if (this._panoramaControl.dispose) {
this._panoramaControl.dispose();
} else {
console.error("this._panoramaControl.dispose was undefined.");
}
}
else {
console.error("this._panoramaControl was undefined.");
}
if (this._moduleElement) {
this._panoramaElement.removeChild(this._moduleElement);
}
if (this._customStylesElement) {
document.getElementsByTagName("head")[0].removeChild(this._customStylesElement);
}
if (this._uiStylesElement) {
document.getElementsByTagName("head")[0].removeChild(this._uiStylesElement);
}
if (this._templateStylesElement) {
document.getElementsByTagName("head")[0].removeChild(this._templateStylesElement);
}
if (this._scriptLinks) {
var scriptsCount = this._scriptLinks.length;
for (var scriptIdx = 0; scriptIdx < scriptsCount; scriptIdx++) {
document.getElementsByTagName("head")[0].removeChild(this._scriptLinks[scriptIdx]);
}
}
this._instrumentationManager = null;
},
_errorHandler: function _errorHandler(e, noRefresh) {
console.log("cloud pano exception: " + e);
this._continueRendering = false;
if (!this._errorHandled && !this._navigatedAway) {
CommonJS.Progress.hideProgress(CommonJS.Progress.centerProgressType);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
var that = this;
this._errorHandled = true;
// Log error
if (this._instrumentationManager) {
this._instrumentationManager.logError(e, noRefresh);
}
var altErrorAction = function () {
var errorCode = Platform.Networking.NetworkManager.instance.isNetworkAvailable ? CommonJS.Error.STANDARD_ERROR : CommonJS.Error.NO_INTERNET;
var errorClicked = function () {
that._inRetry = true;
that._initialize();
};
if (noRefresh) {
errorClicked = function () {
if (PlatformJS.Navigation.canGoBack) {
WinJS.Navigation.back();
} else {
PlatformJS.Navigation.navigateToChannel("Home");
}
};
}
CommonJS.Error.showError(errorCode, errorClicked);
};
if (this._state.altUri) {
try {
PlatformJS.Navigation.navigateTo(this._state.altUri);
msSetImmediate(function () {
if (PlatformJS.Navigation.canGoBack) {
WinJS.Navigation.back();
} else {
PlatformJS.Navigation.navigateToChannel("Home");
}
});
} catch (e) {
var error = "failed to navigate to alt uri: " + this._state.altUri;
console.log(error);
if (this._instrumentationManager) {
this._instrumentationManager.logError(error, true);
}
altErrorAction();
}
} else {
altErrorAction();
}
}
},
_queryAsync: function _queryAsync(query, campaign, market, index) {
var that = this;
var queryService = new PlatformJS.DataService.QueryService(query);
var options = new Platform.DataServices.QueryServiceOptions();
options.useCompatDataModel = true;
var queryParams = PlatformJS.Collections.createStringDictionary();
queryParams.insert("sver", this._state.sver);
queryParams.insert("campaign", campaign);
queryParams.insert("market", market);
queryParams.insert("index", index);
queryParams.insert("cmkt", this._clientMarket);
return queryService.downloadDataAsync(queryParams, null, null, options).then(function (data) {
try {
// Ensure response isn't from local stale cache
if (!data.isCachedResponse || !data.isStaleResponse) {
that._panoData = JSON.parse(data.dataString);
if (!that._panoData) {
that._errorHandler("no pano data downloaded");
}
if (that._instrumentationManager) {
that._instrumentationManager.setPanoInfo(that._panoData, that._supportedVersion.max);
}
}
else {
var error = "can't load stale pano definition";
that._errorHandler(error);
}
} catch (parseError) {
that._errorHandler(parseError, true);
}
return that._panoData;
}, function (queryError) {
that._errorHandler(queryError);
});
},
_populatePages: function _populatePages() {
var that = this;
if (this._continueRendering) {
if (this._template && this._panoData && this._panoData.header && this._panoramaControl) {
if (this._panoData.header.image && this._panoData.header.image.url) {
this._panoramaControl.headerImage = this._panoData.header.image;
}
var title = this._panoData.header.title || "";
this._panoramaControl.title = title;
}
if (this._template && this._panoData && this._panoData.pages && this._panoData.pages.length && this._templateStylesElement && this._moduleElement) {
var pageLength = this._panoData.pages.length;
this._panoData.pages.forEach(function addpage(pageData, index) {
var backgroundImageData = null;
var pageClass = that._generatePageClass(pageData);
var pageControlType = that._template.pageControls[pageData.control].name;
var imageData = {};
if (pageData.background) {
if (pageData.background.imageUrl) {
imageData = CommonJS.CloudPanoBuilder.createImageData(pageData.background.imageUrl, pageData.background.imageWidth, pageData.background.imageHeight, that._dataCacheID);
} else if (pageData.background.landscapeImage && pageData.background.landscapeImage.url
&& pageData.background.portraitImage && pageData.background.portraitImage.url) {
backgroundImageData = pageData.background;
imageData = that._createOrientationImageData(backgroundImageData);
}
}
var items = null;
if (pageData.modules && pageData.modules.length) {
items = new CommonJS.CloudPanoBuilder.Modules();
pageData.modules.forEach(function addmodule(moduleData) {
var module = that._template.modules[moduleData.id];
var className = null;
var cssStyle = null;
var layoutCss = [
"width",
"height",
"left",
"right",
"top",
"bottom"
];
if (module) {
var viewMode = moduleData.data && moduleData.data.viewMode;
if (viewMode === "landscape") {
className = "landscapeOnly";
} else if (viewMode === "portrait") {
className = "portraitOnly";
}
if (that._panoData.version >= that._supportedVersion.clusterItemLayout) {
var cssProperties = [];
layoutCss.forEach(function (cssName) {
if (moduleData.data[cssName]) {
cssProperties.push(cssName + ": " + moduleData.data[cssName] + "; ");
}
});
if (cssProperties.length) {
cssStyle = cssProperties.join("");
cssStyle += "position: absolute;";
}
}
items.push(module, moduleData.data, className, cssStyle);
}
});
}
var key = that._clusters.push(pageControlType, pageData.contentOptions, items && items.dataSource(), pageClass, imageData, pageData.title || null /*clusterTitle*/, null /*funcHeaderSelection*/, that._onItemClick.bind(that));
if (backgroundImageData) {
backgroundImageData.key = key;
that._backgroundList.push(backgroundImageData);
}
});
} else {
this._errorHandler("no page in this cloudPano", true);
}
}
},
_createOrientationImageData: function _createOrientationImageData(backgroundData) {
if (this._isLandscape()) {
return CommonJS.CloudPanoBuilder.createImageData(backgroundData.landscapeImage.url, backgroundData.landscapeImage.width, backgroundData.landscapeImage.height, this._dataCacheID);
} else {
return CommonJS.CloudPanoBuilder.createImageData(backgroundData.portraitImage.url, backgroundData.portraitImage.width, backgroundData.portraitImage.height, this._dataCacheID);
}
},
_onItemClick: function _onItemClick(item) {
if (item && item.data) {
if (CommonJS.CloudPano.clickHandler[item.data.moduleId]) {
// Log click handler event
if (this && this._instrumentationManager) {
this._instrumentationManager.logClick(item.data);
}
// Invoke handler
CommonJS.CloudPano.clickHandler[item.data.moduleId](item.data, this._instrumentationManager);
} else {
// Log destination click event
if (this && this._instrumentationManager) {
this._instrumentationManager.logClick(item.data);
}
// Navigate to uri
var uri = item.data && item.data.destinationUri;
if (uri) {
try {
PlatformJS.Navigation.navigateTo(uri);
} catch (e) {
console.warn("failed to navigate to destination. error: " + e);
}
}
}
}
},
_preprocessCloudData: function _preprocessCloudData() {
var that = this;
var scriptsSet = {};
var scripts = [];
var _preprocessContentOptions = function (templateContentOptions, dataContentOptions) {
var contentOptions = {};
for (var i in templateContentOptions) {
if (templateContentOptions[i] !== null) {
contentOptions[i] = templateContentOptions[i];
} else if (dataContentOptions) {
contentOptions[i] = dataContentOptions[i];
}
}
return contentOptions;
};
var _preprocessModules = function (templateModulesCount, templateModules, dataModules) {
var modules = null;
if (dataModules.length > (templateModulesCount || 0)) {
modules = dataModules.slice(0, templateModulesCount);
} else {
modules = dataModules.slice(0, dataModules.length);
}
var len = Math.min(modules.length, templateModules ? templateModules.length : 0);
for (var i = 0; i < len; i++) {
modules[i].id = templateModules[i].id;
var data = {};
templateModules[i].data.forEach(function (moduleData) {
data[moduleData] = modules[i].data[moduleData];
if (moduleData === "altText" && !data[moduleData]) {
data[moduleData] = PlatformJS.Services.resourceLoader.getString("/platform/advertisement");
}
});
modules[i].data = data;
}
modules.forEach(function (module) {
if (!that._template.modules[module.id]) {
module.id = null;
} else {
var scriptPath = that._template.modules[module.id].scriptPath;
if (scriptPath && !scriptsSet[scriptPath]) {
scriptsSet[scriptPath] = true;
scripts.push(scriptPath);
}
module.data.moduleId = module.id;
if (CommonJS.CloudPano.preprocess[module.id]) {
CommonJS.CloudPano.preprocess[module.id](module.data, that._panoData);
}
}
});
return modules;
};
try {
if (this._panoData && this._template) {
// Set ui theme, if specified
var uiTheme = (this._panoData.pano && this._panoData.pano.uiTheme) || "";
if (uiTheme) {
var oldUiTheme = null,
newUiTheme = null,
light = "ms-appx://Microsoft.WinJS.1.0/css/ui-light.css",
dark = "ms-appx://Microsoft.WinJS.1.0/css/ui-dark.css",
searchResult = {
oldTheme: -1,
newTheme: -1
};
if (uiTheme === "light") {
oldUiTheme = dark;
newUiTheme = light;
} else if (uiTheme === "dark") {
oldUiTheme = light;
newUiTheme = dark;
}
if (newUiTheme) {
var createUiTheme = function () {
that._uiStylesElement = document.createElement("link");
if (that._uiStylesElement) {
that._uiStylesElement.setAttribute("rel", "stylesheet");
that._uiStylesElement.setAttribute("type", "text/css");
that._uiStylesElement.setAttribute("href", newUiTheme);
}
return that._uiStylesElement;
};
var headElement = document.getElementsByTagName("head")[0];
var linkElements = headElement.getElementsByTagName("link");
var linkElementsLen = linkElements.length;
var action = searchResult.notFound;
var uiStylesElement = null;
for (var i = 0; i < linkElementsLen; i++) {
var linkElement = linkElements[i];
if (linkElement.rel === "stylesheet") {
var href = linkElement.href.toLowerCase();
if (href === newUiTheme.toLowerCase()) {
searchResult.newTheme = i;
} else if (href === oldUiTheme.toLowerCase()) {
searchResult.oldTheme = i;
}
}
}
if (searchResult.newTheme === searchResult.oldTheme) {
// Not found any target CSS
uiStylesElement = createUiTheme();
if (uiStylesElement) {
if (linkElementsLen > 0) {
headElement.insertBefore(uiStylesElement, linkElements[0]);
} else {
headElement.appendChild(uiStylesElement);
}
}
} else if (searchResult.newTheme < searchResult.oldTheme) {
// Found another CSS
uiStylesElement = createUiTheme();
if (uiStylesElement) {
if (searchResult.oldTheme === linkElementsLen) {
headElement.appendChild(uiStylesElement);
} else {
headElement.insertBefore(uiStylesElement, linkElements[searchResult.oldTheme + 1]);
}
}
} else if (searchResult.newTheme > searchResult.oldTheme) {
// Nothing to do, just ensure we don't create _uiStylesElement
this._uiStylesElement = null;
}
}
}
// Set custom css if specified
if (this._panoData.template && this._panoData.template.customCss) {
if (!this._customStylesElement) {
// Create custom css element
var customCss = this._panoData.template.customCss;
this._customStylesElement = document.createElement("style");
this._customStylesElement.type = "text/css";
// Set css text
if (this._customStylesElement.styleSheet) {
this._customStylesElement.styleSheet.cssText = customCss;
} else {
this._customStylesElement.appendChild(document.createTextNode(customCss));
}
// Add custom css element
document.getElementsByTagName("head")[0].appendChild(this._customStylesElement);
}
}
// Validate panorama control
if (this._panoramaControl && this._panoData.pano) {
// Set compact pano if specified
if (this._panoData.pano.compact) {
this._panoramaControl.compactPanorama = this._panoData.pano.compact;
}
else {
// By default cloud panos are compact
this._panoramaControl.compactPanorama = true;
}
// Set zero offset snap points if specified
if (this._panoData.pano.zeroOffsetSnapPoints) {
this._panoramaControl.zeroOffsetSnapPoints = this._panoData.pano.zeroOffsetSnapPoints;
}
else {
// By default cloud panos have zero offset snap points
this._panoramaControl.zeroOffsetSnapPoints = true;
}
}
// adjust header image dpi before replace it with local cache.
if (this._panoData.header) {
var headerImage = (this._panoData.header.image && this._panoData.header.image.url);
if (headerImage) {
var dpi = screen.deviceYDPI;
if (dpi > 115 && dpi < 153) {
headerImage = headerImage.replace("_100.", "_140.");
} else if (dpi >= 153) {
headerImage = headerImage.replace("_100.", "_180.");
}
this._panoData.header.image.url = headerImage;
}
}
if (this._panoData.pages && this._panoData.pages.length) {
this._panoData.pages.forEach(function preprocesspage(pageData) {
var template = that._template.templates[pageData.template];
if (template) {
if (!that._template.pageControls[template.pageControl]) {
pageData.control = null;
} else {
var controlPath = that._template.pageControls[template.pageControl].path;
if (controlPath && !scriptsSet[controlPath]) {
scriptsSet[controlPath] = true;
scripts.push(controlPath);
}
pageData.control = template.pageControl;
pageData.css = template.pageCSS;
if (template.pageSize) {
pageData.size = template.pageSize;
}
if (template.pageGrid) {
pageData.grid = template.pageGrid;
}
if (template.contentOptions) {
pageData.contentOptions = _preprocessContentOptions(template.contentOptions, pageData.contentOptions);
}
if (pageData.modules && pageData.modules.length) {
pageData.modules = _preprocessModules(template.modulesCount, template.modules, pageData.modules);
}
if (!that._template.pageSize[pageData.size]) {
pageData.control = null;
}
}
} else { // undefined template
pageData.control = null;
}
});
this._panoData.pages = this._panoData.pages.filter(function filterByControl(pageData) {
return !!(pageData.control);
});
if (scripts.length) {
// Inject script dependencies to page
this._scriptLinks = [];
var scriptsCount = scripts.length;
for (var scriptIdx = 0; scriptIdx < scriptsCount; scriptIdx++) {
var scriptElement = document.createElement("script");
scriptElement.type = "text/javascript";
scriptElement.src = scripts[scriptIdx];
if (scriptElement.src.indexOf("ms-appx") === 0 || scriptElement.src.indexOf("/") === 0) {
this._scriptLinks.push(scriptElement);
document.getElementsByTagName("head")[0].appendChild(scriptElement);
}
}
}
}
}
} catch (e) {
that._errorHandler(e);
}
},
_generatePageClass: function _generatePageClass(pageData) {
var classes = [];
var templatesMap = this._template;
if (pageData.css && templatesMap.pageCSS[pageData.css]) {
classes.push(templatesMap.pageCSS[pageData.css].name);
}
if (pageData.size && templatesMap.pageSize[pageData.size]) {
classes.push(templatesMap.pageSize[pageData.size]);
}
if (pageData.grid && templatesMap.pageGrid[pageData.grid]) {
classes.push(templatesMap.pageGrid[pageData.grid]);
}
if (classes.length) {
return classes.join(" ");
} else {
return "";
}
},
_isLandscape: function _isLandscape() {
var value = Windows.UI.ViewManagement.ApplicationView.value;
return !!(value === Windows.UI.ViewManagement.ApplicationViewState.fullScreenLandscape ||
value === Windows.UI.ViewManagement.ApplicationViewState.filled);
},
_verifyServerURL: function _verifyServerURL(url) {
var result = false;
if (url) {
if (!this._safeServerList) {
this._safeServerList = PlatformJS.Services.configuration.getDictionary("CloudPanoTemplatesSafeServer");
}
if (this._safeServerList) {
try {
var urlObj = new Windows.Foundation.Uri(url);
if (urlObj.host) {
result = !!(this._safeServerList[urlObj.host]);
}
}
catch (e) {
// Reporting will be handled by the caller
return false;
}
}
}
return result;
},
_initTemplate: function _initTemplate(panoData) {
var that = this;
if (panoData && panoData.template) {
var query = panoData.template.query,
templateFile = panoData.template.templateFile,
cssFile = panoData.template.cssFile,
htmlFile = panoData.template.htmlFile;
var promises = [];
var queryService = new PlatformJS.DataService.QueryService(query);
var options = new Platform.DataServices.QueryServiceOptions();
options.useCompatDataModel = true;
var queryParams = PlatformJS.Collections.createStringDictionary();
queryParams.insert("filename", templateFile);
var templateUrl = queryService.getUrl(queryParams);
if (!this._verifyServerURL(templateUrl)) {
var error = "Detected invalid/unsafe host: " + templateUrl;
this._errorHandler(error);
return WinJS.Promise.wrap(null);
}
promises.push(queryService.downloadDataAsync(queryParams, null, null, options).then(function (template) {
try {
if (template.dataString && template.dataString.length) {
that._template = JSON.parse(template.dataString);
}
return that._template;
} catch (e) {
that._template = null;
that._errorHandler(e);
}
}, function (downloadError) {
that._errorHandler(downloadError);
}));
promises.push(this._downloadContent(query, cssFile, true).then(function (injectCSS) { // comment this function for debug
if (!that._templateStylesElement) {
that._templateStylesElement = document.createElement("style");
that._templateStylesElement.type = "text/css";
if (that._templateStylesElement.styleSheet) {
that._templateStylesElement.styleSheet.cssText = injectCSS;
} else {
that._templateStylesElement.appendChild(document.createTextNode(injectCSS));
}
document.getElementsByTagName("head")[0].appendChild(that._templateStylesElement);
// Set templateCss in pano data
if (that._panoData && that._panoData.template) {
that._panoData.template.commonCss = injectCSS;
}
}
}, function (downloadCSSError) {
that._errorHandler(downloadCSSError);
}));
promises.push(this._downloadContent(query, htmlFile, true).then(function (fragment) {
if (!that._moduleElement) {
that._moduleElement = document.createElement("div");
that._moduleElement.innerHTML = toStaticHTML(fragment);
that._panoramaElement.appendChild(that._moduleElement);
}
}, function (downloadHtmlError) {
that._errorHandler(downloadHtmlError);
}));
return WinJS.Promise.join(promises);
} else {
return WinJS.Promise.wrap(null);
}
},
_downloadContent: function _downloadContent(query, uri, securityCheck) {
var queryService = new PlatformJS.DataService.QueryService(query);
var options = new Platform.DataServices.QueryServiceOptions();
options.useCompatDataModel = true;
var queryParams = PlatformJS.Collections.createStringDictionary();
queryParams.insert("filename", uri);
var templateUrl = queryService.getUrl(queryParams);
if (securityCheck && !this._verifyServerURL(templateUrl)) {
var error = "Detected invalid/unsafe host: " + templateUrl;
that._errorHandler(error);
return WinJS.Promise.wrap(null);
} else {
return queryService.downloadDataAsync(queryParams, null, null, options).then(function (response) {
return response.dataString;
});
}
},
_downloadFile: function _downloadFile(uri) {
var that = this;
var promise = PlatformJS.Cache.CacheService.getInstance(this._dataCacheID).findEntry(uri, {
fileNameOnly: true
}).then(function cache_completion(response) {
if (response && !response.isStale() && response.dataValue) {
return response.dataValue;
} else {
return PlatformJS.Utilities.fetchImage(that._dataCacheID, uri, null);
}
});
return promise;
},
_downloadResources: function _downloadResources(panoData) {
var that = this;
var resourcesMap = {};
var downloadPromises = [];
var resourceError = false;
var failedUri = "";
if (panoData && panoData.resources && panoData.resources.load && panoData.resources.load.length) {
panoData.resources.load.forEach(function downloadResource(uri) {
var promise = that._downloadFile(uri).then(function (localURL) {
if (localURL && !resourcesMap[uri]) {
resourcesMap[uri] = localURL;
}
return resourcesMap;
}, function (error) {
resourceError = true;
failedUri = uri;
});
downloadPromises.push(promise);
});
}
if (downloadPromises.length) {
return WinJS.Promise.join(downloadPromises).then(function () {
if (resourceError) {
that._resourcesMap = null;
that._errorHandler("failed to download required resource: " + failedUri);
} else {
that._resourcesMap = resourcesMap;
}
return resourcesMap;
});
} else {
return WinJS.Promise.wrap(null);
}
},
_replaceResources: function _replaceResources() {
var that = this;
if (!this._resourcesMap) {
return;
}
var replaceUri = function (data) {
for (var property in data) {
if (data.hasOwnProperty(property)) {
var propertyType = typeof data[property];
if (propertyType === "string" && that._resourcesMap[data[property]]) {
data[property] = that._resourcesMap[data[property]];
} else if (propertyType === "object") {
replaceUri(data[property]);
}
}
}
};
if (this._panoData && this._panoData.header) {
replaceUri(this._panoData.header);
}
if (this._panoData && this._panoData.pages) {
replaceUri(this._panoData.pages);
}
}
})
});
})();
/* File: PanoBuilder.js */
(function cloudPanoBuilderInit() {
"use strict";
WinJS.Namespace.define("CommonJS.CloudPanoBuilder", {
Clusters: WinJS.Class.define(
function clusters_ctor() {
this._clusters = new WinJS.Binding.List();
}, {
dataSource: function dataSource() {
return this._clusters.createSorted(function createdSortedCallback(l, r) {
if (l.clusterIndex < r.clusterIndex) {
return -1;
} else if (l.clusterIndex === r.clusterIndex) {
return 0;
} else {
return 1;
}
}).dataSource;
},
push: function push(contentControl, contentOptions, itemDataSource, clusterClassName, imageData, clusterTitle, funcHeaderSelection, funcItemClick, displayItemCount) {
var order, key, title, onfuncHeaderSelection, itemCount;
if (contentControl && (contentOptions || itemDataSource)) {
order = this._clusters.length;
title = clusterTitle || null;
key = "cloudPanoCluster" + order + title;
itemCount = displayItemCount || (itemDataSource && itemDataSource._list ? itemDataSource._list.length : 1);
onfuncHeaderSelection = funcHeaderSelection || null;
contentOptions = contentOptions || {};
if (itemDataSource) {
contentOptions.itemDataSource = itemDataSource;
}
if (funcItemClick) {
contentOptions.onitemclick = function (value) {
if (value.item && (funcItemClick)) {
funcItemClick(value.item);
}
};
}
if (clusterClassName) {
contentOptions.className = clusterClassName;
}
if (imageData) {
contentOptions.imageData = imageData;
}
contentOptions.displayItemCount = itemCount;
this._clusters.push({
clusterIndex: order,
clusterKey: key,
clusterTitle: title,
hideTitle: !clusterTitle,
onfuncHeaderSelection: onfuncHeaderSelection,
clusterContent: {
contentControl: contentControl,
contentOptions: contentOptions
}
});
this._indexMap[key] = order;
}
return key;
},
getItem: function getItem(key) {
var i = this._indexMap[key];
if (typeof (i) !== "number") {
for (i = 0; i < this._clusters.length; i++) {
var item = this._clusters.getItem(i);
if (item.data.clusterKey === key) {
break;
}
}
}
if (i < this._clusters.length) {
return this._clusters.getAt(i);
} else {
return null;
}
},
setItem: function setItem(key, item) {
var i = this._indexMap[key];
if (typeof (i) !== "number") {
for (i = 0; i < this._clusters.length; i++) {
var cluster = this._clusters.getItem(i);
if (cluster.data.clusterKey === key) {
break;
}
}
}
if (i < this._clusters.length) {
this._clusters.setAt(i, item);
}
},
removeKey: function removeKey(key) {
var i = this._indexMap[key];
if (typeof (i) !== "number") {
for (i = 0; i < this._clusters.length; i++) {
var item = this._clusters.getItem(i);
if (item.data.clusterKey === key) {
break;
}
}
}
if (i < this._clusters.length) {
this._clusters.splice(i, 1);
}
},
_clusterGroup: {},
_clusters: null,
_indexMap: {}
}),
createImageData: function createImageData(imageUrl, width, height, cacheId, anchorPoint, showGradient, attributionText, captionText, fontColor, attributionColor) {
var attributionData = null;
fontColor = fontColor || "headerFontLight";
attributionColor = attributionColor || "headerFontLight";
if (attributionText || captionText) {
attributionData = {
attributionText: attributionText || "",
caption: captionText || ""
};
}
var imageData = WinJS.Binding.as({
fontColor: fontColor,
attributionColor: attributionColor,
imageSize: {
width: width,
height: height
},
anchorPoint: anchorPoint || "anchorMiddle",
backgroundImage: {
url: imageUrl,
cacheId: cacheId || null
},
attributionData: attributionData,
showTitleGradient: !!(showGradient)
});
return imageData;
},
Modules: WinJS.Class.define(function modules_ctor() {
this._modules = new WinJS.Binding.List();
}, {
push: function push(module, data, className, cssStyle) {
if (module) {
var isModuleInteractive = false;
if (data && data.isInteractive) {
isModuleInteractive = data.isInteractive;
}
var moduleInfo = {
fragmentPath: module.path,
isInteractive: isModuleInteractive,
templateId: module.name,
className: className || "",
cssStyle: cssStyle || ""
};
this._modules.push({
moduleInfo: moduleInfo,
data: data
});
}
},
dataSource: function get_dataSource (argument) {
return this._modules.dataSource;
}
})
});
})();
/* File: PanoInstrumentationManager.js */
(function cloudPanoInstrumentationInit() {
"use strict";
WinJS.Namespace.define("CommonJS.CloudPanoInstrumentation", {
PanoInstrumentationManager: WinJS.Class.define(
function panoInstrumentationManager_ctor(configSettings, state) {
// Set manager instance
this._managerInstance = Platform.Telemetry.FlightRecorder;
this._initialize(configSettings, state);
}, {
_dataSetID: Platform.Instrumentation.InstrumentationDataSetId.cloudPanoPing,
_managerInstance: null,
_loggingEnabled: true,
_instrumentationMapping: null,
_isNavBarEvent: null,
_impressionGUID: null,
_newImpression: null,
_campaign: null,
_market: null,
_index: null,
_panoInfo: {},
_panoError: false,
// Methods
_initialize: function _initialize(configSettings, state) {
// Check if instrumentation logging is enabled
if (configSettings &&
configSettings.EnableInstrumentationLogging) {
// Set logging enabled flag
this._loggingEnabled = configSettings.EnableInstrumentationLogging.value;
}
// Get instrumentation mapping
this._instrumentationMapping = Platform.Configuration.ConfigurationManager.custom.getDictionary("CloudPanoInstrumentationMapping");
// Validate state
if (state) {
// Set nav bar event flag
this._isNavBarEvent = state.isNavBarEvent || false;
// If we don't have an impression GUID saved in state yet,
// or if this is a navigation bar event,
// start a new unique impression of the pano
if (!state.impressionGUID || state.isNavBarEvent) {
// Create impression GUID
this._impressionGUID = Platform.Telemetry.FlightRecorder.createStringGuid();
// Save impression GUID in state
state.impressionGUID = this._impressionGUID;
this._newImpression = true;
}
else {
// Restore impression GUID
this._impressionGUID = state.impressionGUID;
this._newImpression = false;
}
// Set campaign
this._campaign = state.campaign || "";
// Set market
this._market = state.market || "";
// Set index
this._index = state.index || "";
}
},
_sendEvent: function _sendEvent(event, dataPointId) {
// Validate
if (this._managerInstance &&
event &&
dataPointId) {
// Check if logging is enabled
if (this._loggingEnabled) {
// Set event
this._managerInstance.logCustom(Platform.Telemetry.LogLevel.normal, dataPointId, event);
}
}
},
setPanoInfo: function setPanoInfo(panoData, supportedVersion) {
this._panoInfo = {};
if (panoData) {
// Get pano ID
if (panoData.pano && panoData.pano.id) {
this._panoInfo.panoId = panoData.pano.id;
}
else {
this._panoInfo.panoId = "";
}
// Get version
if (panoData.version) {
this._panoInfo.panoVersion = panoData.version;
}
else {
this._panoInfo.panoVersion = "";
}
// Get supported version
if (supportedVersion) {
this._panoInfo.supportedVersion = supportedVersion.toString();
} else {
this._panoInfo.supportedVersion = "";
}
// Get page count
if (panoData.pages && panoData.pages.length) {
this._panoInfo.pageCount = panoData.pages.length;
}
else {
this._panoInfo.pageCount = 0;
}
}
},
logSplashScreenShown: function logSplashScreenShown(splash, isCached, isCustom) {
// Get time stamp
this._splashStartTime = new Date();
// Since pano definition isn't loaded at this point
// we don't have pano-specific IDs, just the campaign/market
// pair and splash dimensions and background color
var event = JSON.stringify({
impressionGUID: this._impressionGUID,
newImpression: this._newImpression,
campaign: this._campaign,
market: this._market,
index: this._index,
isNavBar: this._isNavBarEvent,
timeStamp: this._splashStartTime.getTime(),
tzOffset: this._splashStartTime.getTimezoneOffset(),
panoError: this._panoError,
isCustom: isCustom || false,
backgroundColor: splash ? splash.backgroundColor : "",
splashWidth: splash ? splash.imageLocation.width : "",
splashHeight: splash ? splash.imageLocation.height : "",
isCached: isCached || false
});
var dataPointId = "AppCloudPanoSplashScreenShown";
// Set instrumentation event
this._sendEvent(event, dataPointId);
},
logPanoLoaded: function logPanoLoaded() {
// Get time stamp
this._panoViewStartTime = new Date();
// Get splash elapsed time
var splashDelay = -1;
if (this._splashStartTime) {
splashDelay = this._panoViewStartTime - this._splashStartTime;
}
// Create event
var event = JSON.stringify({
impressionGUID: this._impressionGUID,
newImpression: this._newImpression,
campaign: this._campaign,
market: this._market,
index: this._index,
isNavBar: this._isNavBarEvent,
timeStamp: this._panoViewStartTime.getTime(),
tzOffset: this._panoViewStartTime.getTimezoneOffset(),
panoError: this._panoError,
panoId: this._panoInfo.panoId,
panoVersion: this._panoInfo.panoVersion,
supportedVersion: this._panoInfo.supportedVersion,
pageCount: this._panoInfo.pageCount,
splashDelay: splashDelay
});
var dataPointId = "AppCloudPanoLoaded";
// Set instrumentation event
this._sendEvent(event, dataPointId);
},
logClick: function logClick(itemData) {
// Get time stamp
var clickTimeStamp = new Date();
// Extract click info
var clickInfo = {
instrumentationId: "",
clickthrough: ""
};
if (itemData) {
// Get instrumentation ID
if (itemData.instrumentationId) {
clickInfo.instrumentationId = itemData.instrumentationId;
}
// Get clickthrough
if (itemData.videoSource) {
clickInfo.clickthrough = "video: " + itemData.videoSource;
} else if (itemData.destinationUri) {
clickInfo.clickthrough = itemData.destinationUri;
}
}
// Create event
var event = JSON.stringify({
impressionGUID: this._impressionGUID,
newImpression: this._newImpression,
campaign: this._campaign,
market: this._market,
index: this._index,
isNavBar: this._isNavBarEvent,
timeStamp: clickTimeStamp.getTime(),
tzOffset: clickTimeStamp.getTimezoneOffset(),
panoError: this._panoError,
panoId: this._panoInfo.panoId,
panoVersion: this._panoInfo.panoVersion,
supportedVersion: this._panoInfo.supportedVersion,
pageCount: this._panoInfo.pageCount,
instrumentationId: clickInfo.instrumentationId || "",
clickthrough: clickInfo.clickthrough || ""
});
var dataPointId = "AppCloudPanoItemClicked";
// Set instrumentation event
this._sendEvent(event, dataPointId);
},
logPageViewed: function logPageViewed(page) {
// Get time stamp
var pageViewedTimeStamp = new Date();
// Create event
var event = JSON.stringify({
impressionGUID: this._impressionGUID,
newImpression: this._newImpression,
campaign: this._campaign,
market: this._market,
index: this._index,
isNavBar: this._isNavBarEvent,
timeStamp: pageViewedTimeStamp.getTime(),
tzOffset: pageViewedTimeStamp.getTimezoneOffset(),
panoError: this._panoError,
panoId: this._panoInfo.panoId,
panoVersion: this._panoInfo.panoVersion,
supportedVersion: this._panoInfo.supportedVersion,
pageCount: this._panoInfo.pageCount,
page: page || -1
});
var dataPointId = "AppCloudPanoPageViewed";
// Set instrumentation event
this._sendEvent(event, dataPointId);
},
logVideoProgress: function logVideoProgress(status, src) {
// Get time stamp
var videoTimeStamp = new Date();
// Create event
var event = JSON.stringify({
impressionGUID: this._impressionGUID,
newImpression: this._newImpression,
campaign: this._campaign,
market: this._market,
index: this._index,
isNavBar: this._isNavBarEvent,
timeStamp: videoTimeStamp.getTime(),
tzOffset: videoTimeStamp.getTimezoneOffset(),
panoError: this._panoError,
panoId: this._panoInfo.panoId,
panoVersion: this._panoInfo.panoVersion,
supportedVersion: this._panoInfo.supportedVersion,
pageCount: this._panoInfo.pageCount,
videoSource: src || "",
videoStatus: (!status && status !== 0) ? -1 : status
});
var dataPointId = "AppCloudPanoVideoProgress";
// Set instrumentation event
this._sendEvent(event, dataPointId);
},
logPanoClosed: function logPanoClosed(pagesViewed, hiddenElapsedTime) {
// Get time stamp
var panoViewEndTime = new Date();
// Get dwell time
var dwellTime = -1;
if (this._panoViewStartTime) {
dwellTime = panoViewEndTime - this._panoViewStartTime;
}
// Discount time pano was hidden from the report
if (hiddenElapsedTime) {
dwellTime -= hiddenElapsedTime;
// Account for error cases
if (dwellTime <= 0) {
// Essentially that means we shouldn't
// count the impression at all
dwellTime = -1;
}
}
// Get pages viewed array as string
var pagesViewedArray = [];
if (pagesViewed) {
var pageIdx = 0;
for (var page in pagesViewed) {
pagesViewedArray[pageIdx++] = pagesViewed[page];
}
}
// Create event
var event = JSON.stringify({
impressionGUID: this._impressionGUID,
newImpression: this._newImpression,
campaign: this._campaign,
market: this._market,
index: this._index,
isNavBar: this._isNavBarEvent,
timeStamp: panoViewEndTime.getTime(),
tzOffset: panoViewEndTime.getTimezoneOffset(),
panoError: this._panoError,
panoId: this._panoInfo.panoId,
panoVersion: this._panoInfo.panoVersion,
supportedVersion: this._panoInfo.supportedVersion,
pageCount: this._panoInfo.pageCount,
dwellTime: dwellTime || -1,
pagesViewed: pagesViewedArray || ""
});
var dataPointId = "AppCloudPanoClosed";
// Set instrumentation event
this._sendEvent(event, dataPointId);
},
logError: function logError(error, noRefresh) {
// Get time stamp
var errorTimeStamp = new Date();
// Set eror state
this._panoError = true;
// Get error text
var errorText = error;
if (typeof (error) === "object") {
try {
errorText = JSON.stringify(error);
} catch (e) {
errorText = "exception while stringifying error object";
}
}
// Truncate error text to 512 characters + ellipsis
if (errorText.length > 512) {
errorText = errorText.substring(0, 512) + "..";
}
// Create event
var event = JSON.stringify({
impressionGUID: this._impressionGUID,
newImpression: this._newImpression,
campaign: this._campaign,
market: this._market,
index: this._index,
isNavBar: this._isNavBarEvent,
timeStamp: errorTimeStamp.getTime(),
tzOffset: errorTimeStamp.getTimezoneOffset(),
panoError: this._panoError,
error: errorText || "",
noRefresh: noRefresh || false,
});
var dataPointId = "AppCloudPanoError";
// Set instrumentation event
this._sendEvent(event, dataPointId);
}
})
});
})();
/* File: snappedCloudPano.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformDefaultSnappedHomeInit() {
WinJS.Namespace.define("CommonJS", {
SnappedCloudPano: WinJS.Class.define(
function snappedCloudPano_ctor(state) {
this._state = state || {};
var that = this;
if (this._state.splashName && this._state.splashBkg && this._state.splashW && this._state.splashH) {
this._dataPromise = CommonJS.CloudPanorama.prototype._prepareCustomSplash.call(this).then(function (splash) {
return {
imageUri: splash.imageUri,
width: splash.imageLocation.width,
height: splash.imageLocation.height,
backgroundColor: splash.backgroundColor
};
});
} else {
var splashData = PlatformJS.Services.configuration.getDictionary("CloudPanoSplash");
var data = null;
if (splashData.imageUri && splashData.height && splashData.width && splashData.backgroundColor) {
data = {
imageUri: splashData.imageUri.value,
height: splashData.height.value,
width: splashData.width.value,
backgroundColor: splashData.backgroundColor.value
};
}
this._dataPromise = WinJS.Promise.wrap(data);
}
},
{
_state: null,
_splash: null,
_dataCacheID: "CloudPanoDataCache",
_dataPromise: null,
getPageState: function getPageState() {
return this._state;
},
getPageData: function getPageData() {
return this._dataPromise.then(function (data) {
return {
backgroundColor: data.backgroundColor || "white",
imageUri: data.imageUri || "/CommonControls/Common/images/snap_placeholder.png",
alternateText: PlatformJS.Services.resourceLoader.getString("/platform/advertisement") || "Advertisement",
onclick: WinJS.Utilities.markSupportedForProcessing(function defaultUnSnapHandler() {
Windows.UI.ViewManagement.ApplicationView.tryUnsnap();
})
};
});
}
})
});
})();
/* File: cloudPanoTemplatesControls.js */
(function cloudPanoTemplatesControlsInit() {
"use strict";
WinJS.Namespace.define("CommonJS.CloudPano",
{
/// <summary>
/// Safe HTML region
/// </summary>
SafeHtmlRegion: WinJS.Class.define(
/// <summary>
/// Constructor: Creates an instance of the safe HTML region.
/// </summary>
/// <returns>
/// An instance of SafeHtmlRegion.
/// </returns>
function safeHtmlRegion_ctor (element, options) {
element = element || document.createElement("div");
element.winControl = this;
this._init(element);
WinJS.UI.setOptions(this, options);
},
// instance members
{
_domElement: null,
_dataCacheID: "CloudPanoDataCache",
_iframePath: "ms-appx-web:///Common/CloudPano/CloudPanoIFrame.html",
_iframe: null,
_sendMessageToWebContext: function _sendMessageToWebContext(msg) {
// Validate
if (this && this._iframe) {
// Get origin
var origin = "ms-appx-web://" + document.location.host;
// Get target
var target = this._iframe.contentWindow;
try {
// Post message to target
target.postMessage(msg, origin);
} catch (error) {
// Failed to post message to web compartment
// should only happen if we're already disposed
}
}
},
_receiveWebContextMessage: function _receiveWebContextMessage(msg) {
// Verify message and origin
if (!msg ||
!msg.data ||
!msg.isTrusted ||
msg.origin !== "ms-appx-web://" + document.location.host) {
return;
}
// Verify we have an element id
var msgType = msg.data.type;
var elementId = msg.data.id;
if (msgType && elementId) {
// Get iframe element
var iframe = document.getElementById(elementId);
if (!iframe) {
return;
}
// Get element
var element = iframe.parentElement;
if (!element) {
return;
}
// Handle message
if (msgType === "loaded") {
// Show iframe since it's now fully loaded
if (iframe.style) {
iframe.style.visibility = "visible";
}
}
else if (msgType === "interaction" && msg.data.event) {
// Simulate interaction event
var eventData = msg.data.event;
var simulatedEvent = document.createEvent("MouseEvent");
simulatedEvent.initMouseEvent(eventData.type, true, false, window, eventData.detail,
eventData.screenX, eventData.screenY, eventData.clientX, eventData.clientY,
eventData.ctrlKey, eventData.altKey, eventData.shiftKey, eventData.metaKey,
eventData.button, element);
element.dispatchEvent(simulatedEvent);
} else if (msgType === "click") {
// Click element
element.click();
}
}
},
_loadWebContext: function _loadWebContext(element, content, html) {
// Validate
if (element && this) {
// Create web context iframe element
var iframeId = document.uniqueID;
this._iframe = document.createElement("iframe");
// Validate
if (this._iframe && this._iframe.style) {
// Set iframe src and id and add it to the DOM
this._iframe.src = this._iframePath;
this._iframe.id = iframeId;
this._iframe.style.visibility = "hidden";
element.appendChild(this._iframe);
// Add iframe class
WinJS.Utilities.addClass(this._iframe, "cloudPanoIFrame");
var that = this;
msSetImmediate(function safeHtmlRegion_loadWebContextData() {
// Verify iframe
if (that._domElement && that._iframe) {
// Create web context data blob
var iframeData = {
type: "load",
id: iframeId,
html: html,
uiTheme: content.uiTheme,
elementCss: that._domElement.style ? that._domElement.style.cssText : "",
commonCss: content.commonCss,
customCss: content.customCss
};
// Send it to web context
that._sendMessageToWebContext(iframeData);
// Add listener for web context messages
window.addEventListener("message", that._receiveWebContextMessage);
}
});
}
}
},
_init: function _init(element) {
var that = this;
// configure control
var currentContent;
that._content_get = function () {
return currentContent;
};
that._content_set = function (value) {
currentContent = value;
// Validate content
var resourcesMap = {};
var downloadPromises = [];
if (currentContent) {
var contentType = typeof (currentContent);
if (contentType === "string" && currentContent !== "") {
// Simply set safe html string
try {
var safeHtml = toStaticHTML(unescape(currentContent));
that._domElement.innerHTML = safeHtml;
} catch (error) {
// Don't load any html
}
}
else if (contentType === "object" && currentContent.html && currentContent.html.length && currentContent.html.join) {
// If we have preload resources
if (currentContent.preload && currentContent.preload.length) {
currentContent.preload.forEach(function downloadResource(uri) {
var promise = PlatformJS.Utilities.downloadFile(that._dataCacheID, uri).then(function (localURL) {
if (localURL && !resourcesMap[uri]) {
resourcesMap[uri] = localURL;
}
}, function (error) {
// Ignore error and proceed
// this is mostly consistent with
// how trident handles invalid resources
});
downloadPromises.push(promise);
});
}
var loadPromise;
if (downloadPromises.length) {
loadPromise = WinJS.Promise.join(downloadPromises).then(function () {
// Replace html resource uris with local cached ones
for (var field in currentContent.html) {
var fieldValue = currentContent.html[field];
var fieldType = typeof fieldValue;
if (fieldType === "string" && resourcesMap[fieldValue]) {
currentContent.html[field] = resourcesMap[fieldValue];
}
}
});
} else {
loadPromise = WinJS.Promise.wrap({});
}
// Write element html after load promise completes
loadPromise.then(function safeHtmlRegion_loadModule() {
if (that && that._domElement) {
try {
var safeHtml = toStaticHTML(unescape(currentContent.html.join("")));
if (!currentContent.isWebContext) {
// Simply set html string
that._domElement.innerHTML = safeHtml;
}
else {
// Load content in web compartment
that._loadWebContext(element, currentContent, safeHtml);
}
} catch (error) {
// Don't load any html
}
}
}, function (error) {
// Don't load any html
});
}
}
};
// Add to DOM
that._domElement = element;
},
dispose: function dispose() {
// If we were running in web context
if (this && this.isWebContext) {
// Create dispose message
var disposeMessage = {
type: "dispose"
};
// Send it to web context
this._sendMessageToWebContext(disposeMessage);
}
},
content: {
get: function get() {
return this._content_get();
},
set: function set(value) {
this._content_set(value);
}
}
})
}); 
})();
/* File: moduleMethods.js */
(function cloudPanoTemplateJsonInit() {
"use strict"; 
var funcPlayVideo = function (data, instrumentationManager) {
if (data.videoSource) {
if (CommonJS && CommonJS.MediaApp && CommonJS.MediaApp.Controls) {
var videoInstrCallback = function (status, src) {
if (instrumentationManager) {
instrumentationManager.logVideoProgress(status, src);
}
};
var option = {instrumentationCallback: videoInstrCallback };
CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback(data.videoSource, option);
}
} else if (data.destinationUri) {
try{
PlatformJS.Navigation.navigateTo(data.destinationUri);
} catch (e) {
console.warn("failed to navigate to destination. error: " + e);
}
}
};
WinJS.Namespace.define("CommonJS.CloudPano", {
preprocess: {
imageTextModule: function imageTextModule(data){
if(data.title && typeof data.title === "string") {
data.overlayVisible = "visible";
} else {
data.title = "";
data.overlayVisible = "hidden";
}
},
imageVideoModule: function imageVideoModule(data) {
if (data.imageUrl && !data.imageSource) {
if (!data.imageWidth || !data.imageHeight) {
data.imageSource = data.imageUrl;
} else {
data.imageSource = {
url: data.imageUrl,
name: data.altText,
width: data.imageWidth,
height: data.imageHeight,
cacheId: "CloudPanoDataCache"
};
}
}
if (!data.videoSource) {
data.playcontrolVisibility = "hidden";
}
},
imageButtonModule: function imageButtonModule(data) {
if (data.imageUrl && !data.imageSource) {
if (!data.imageWidth || !data.imageHeight) {
data.imageSource = data.imageUrl;
} else {
data.imageSource = {
url: data.imageUrl,
name: data.altText,
width: data.imageWidth,
height: data.imageHeight,
cacheId: "CloudPanoDataCache"
};
data.imageWidth = data.imageWidth + "px";
data.imageHeight = data.imageHeight + "px";
}
} else {
data.hideImage = "none";
}
if (!data.videoSource) {
data.hidePlaybutton = "none";
}
},
safeHTMLModule: function safeHTMLModule(data, panoData) {
// Validate
if (data && panoData) {
if (data.content) {
try {
// Determine if module should run in web context
var contentHtml = toStaticHTML(unescape(data.content));
if (data.fontFamily) {
// Set web context content
data.content = { isWebContext: true, html: [contentHtml] };
// Validate ui theme
if (panoData.pano &&
panoData.pano.uiTheme) {
// Set ui theme for
// module to inherit
data.content.uiTheme = panoData.pano.uiTheme;
}
// Validate template
if (panoData.template) {
// If we have common css
if (panoData.template.commonCss) {
// Set common pano css for 
// module to inherit
data.content.commonCss = panoData.template.commonCss;
}
// If we have custom css
if (panoData.template.customCss) {
// Set custom pano css for 
// module to inherit
data.content.customCss = panoData.template.customCss;
}
}
} else {
// Set local context content
data.content = { isWebContext: false, preload: [], html: [] };
// Create preload html src resources
var htmlSlice = contentHtml;
var nextResource = true;
while (nextResource) {
var currentResourceIdx = htmlSlice.indexOf("http://");
if (currentResourceIdx >= 0) {
// Get html pre-slice
var preSlice = htmlSlice.substring(0, currentResourceIdx);
// Validate and add pre-slice
if (preSlice) {
data.content.html.push(preSlice);
}
// Get resource
var resourceUrl = "";
var resourceEndIdx = htmlSlice.indexOf("\"", currentResourceIdx);
if (resourceEndIdx > currentResourceIdx) {
// Get resource url
resourceUrl = htmlSlice.substring(currentResourceIdx, resourceEndIdx);
// Update html slice
htmlSlice = htmlSlice.substring(resourceEndIdx);
}
else {
// Get resource url
resourceUrl = htmlSlice.substring(currentResourceIdx);
// No remaining html slice
htmlSlice = "";
}
// Add resource url to content html and preload
if (resourceUrl) {
data.content.html.push(resourceUrl);
data.content.preload.push(resourceUrl);
}
} else {
// Done parsing resources
nextResource = false;
break;
}
}
// Add last html slice to content
if (htmlSlice) {
data.content.html.push(htmlSlice);
}
}
} catch (e) {
// Clear content
data.content = "";
}
}
}
}
},
clickHandler: {
imageVideoModule: funcPlayVideo,
imageButtonModule: funcPlayVideo
}
});
})();
