﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: DateSelector.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsDateSelectorInit () {
"use strict";
var G11n = Windows.Globalization,
G11nPref = Windows.System.UserProfile.GlobalizationPreferences,
LocalWeekStartsOn = G11nPref.weekStartsOn,
Languages = G11nPref.languages,
DateTimeFormatting = G11n.DateTimeFormatting,
Util = WinJS.Utilities,
CachedMonthNames = [],
CachedDayNames = [],
DISABLED_TABINDEX_VALUE = -2,
DATA_ATTRIBUTE_NAME = "data-datesel",
SHORT_DAY_NAME_IDEAL_LENGTH = 2,
NUM_DAYS_IN_WEEK = 7,
SCROLL_REFRESH_MS = 150,
EVENTS = { SELECT: "select" },
STRINGS = {
GRID_LABEL: CommonJS.resourceLoader.getString("/Common/Calendar") || "Calendar",
WEEK_NUM: CommonJS.resourceLoader.getString("/Common/WeekNum") || "Week {0}",
},
/// <field>
/// Emunaration of classnames referenced by dateSelector object
/// </field>
CLASSNAMES = {
ROOT: { NAME: "datesel", SELECTOR: ".datesel" },
GRID: { NAME: "datesel-grid", SELECTOR: ".datesel-grid" },
GHEAD: { NAME: "datesel-ghead", SELECTOR: ".datesel-ghead" },
GBODY: { NAME: "datesel-gbody", SELECTOR: ".datesel-gbody" },
HLIVE: { NAME: "datesel-hlive", SELECTOR: ".datesel-hlive" },
HYEAR: { NAME: "datesel-hyear", SELECTOR: ".datesel-hyear" },
HMONTH: { NAME: "datesel-hmonth", SELECTOR: ".datesel-hmonth" },
YEAR: { NAME: "datesel-year", SELECTOR: ".datesel-year" },
MONTH: { NAME: "datesel-month", SELECTOR: ".datesel-month" },
WEEK: { NAME: "datesel-week", SELECTOR: ".datesel-week" },
DAY: { NAME: "datesel-day", SELECTOR: ".datesel-day" },
TODAY: { NAME: "datesel-today", SELECTOR: ".datesel-today" },
COLHEADERS: { NAME: "datesel-colheaders", SELECTOR: ".datesel-colheaders" },
ROW: { NAME: "datesel-row", SELECTOR: ".datesel-row" },
GRIDCELL: { NAME: "datesel-gridcell", SELECTOR: ".datesel-gridcell" },
SELECTED: { NAME: "datesel-selected", SELECTOR: ".datesel-selected" }
};
/// <summary>
/// Date Selector is a widget/flyout that shows selectable calendar.
/// </summary>
WinJS.Namespace.define("CommonJS.UI",
{
DateSelector: WinJS.Class.define(
/// <summary>
/// Constructor: Creates an instance of the DateSelector object.
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
/// <returns>
/// An instance of the DateSelector object.
/// </returns>
function dateSelector_ctor (element, options) {
this._element = element || document.createElement("div");
this._element.winControl = this;
this._todayCal = this._createGregorianCalendar();
this._minDateCal = this._createGregorianCalendar();
this._maxDateCal = this._createGregorianCalendar();
this._currentCal = this._createGregorianCalendar();
this._todayAriaLabel = this._calendarToAriaLabel(this._todayCal);
this._setOptions(options);
this._navigableViewModel = [];
this._navigableViewModelPos = {
col: null,
row: null
};
this._liveHeaderFormatter = this._createLiveHeaderDateFormatter();
this._render();
}
,
{
// The DOM element container
_element: null,
// Calendar that represents today's date
_todayCal: null,
// Calendar that represents the minimum date
_minDateCal: null,
// Calendar that represents the maximum date
_maxDateCal: null,
// Calendar that represents the currently selected date
_currentCal: null,
// GridCell that user has selected
_selectedGridCell: null,
// GridCell that has tabIndex=0
_activeGridCell: null,
// GridCell that is today's date
_todayGridCell: null,
_todayAriaLabel: null,
_scrollInfo: [],
_scrollTimer: null,
_liveHeader: null,
_liveHeaderFormatter: null,
_grid: null,
_gbody: null,
// 2D tilegame-type model for tabindex/focus management
_navigableViewModel: null,
// Current position in model
_navigableViewModelPos: {
col: null,
row: null
},
_numCols: NUM_DAYS_IN_WEEK,
_numRows: 0,
_setOptions: function _setOptions(options) {
if (options) {
if (options.minDate) {
this._minDateCal.setDateTime(new Date(options.minDate));
}
if (options.maxDate) {
this._maxDateCal.setDateTime(new Date(options.maxDate));
}
if (options.current) {
this.current = options.current;
}
}
},
_render: function _render() {
this.destroy();
this._navigableViewModel = [];
Util.addClass(this._element, CLASSNAMES.ROOT.NAME);
/*  gridBuilderCal will be used to iteratively build the UI grid. */
var gridBuilderCal = this._minDateCal.clone();
this._setCalToFirstDayOfMonth(gridBuilderCal);
this._grid = this._createGrid(gridBuilderCal);
this._element.appendChild(this._grid);
var currentEl = this._getCurrentEl();
// If current date is already set, and it is a selectable gridcell
// select that grid cell and set it as the active grid cell
if (currentEl !== null && Util.hasClass(currentEl, CLASSNAMES.DAY.NAME)) {
this._selectGridCell(currentEl, false);
} else { // Otherwise choose another gridcell as the active grid cell and don't select anything
var activeGridCell = this._gbody.querySelector("a[tabindex='{0}']".format(DISABLED_TABINDEX_VALUE));
if (activeGridCell) {
this._setActiveGridCell(activeGridCell);
}
}
// Style gridcell that is today's date
if (this._todayGridCell) {
Util.addClass(this._todayGridCell, CLASSNAMES.TODAY.NAME);
}
// Listen for changes in system date
PlatformJS.Utilities.DayPoller.getInstance().addListener(this._element.id, this._onDayPollerChange.bind(this));
},
_createScrollInfo: function _createScrollInfo(gbody) {
var scrollInfo = [],
gbodyOffsetTop = gbody.offsetTop,
yearEls = gbody.querySelectorAll(CLASSNAMES.YEAR.SELECTOR);
for (var yearIndex = 0, yearElsLength = yearEls.length; yearIndex < yearElsLength; yearIndex++) {
var yearEl = yearEls[yearIndex],
monthEls = (yearEl) ? yearEl.querySelectorAll(CLASSNAMES.MONTH.SELECTOR) : [];
for (var monthIndex = 0, monthElsLength = monthEls.length; monthIndex < monthElsLength; monthIndex++) {
var monthEl = monthEls[monthIndex];
if (monthEl) {
var monthHeadingEl = monthEl.querySelector("h4"),
liveHeaderText = monthEl.getAttribute("data-liveheader"),
scrollStart = monthEl.offsetTop - gbodyOffsetTop,
contentStart = scrollStart + monthHeadingEl.offsetHeight,
scrollEnd = scrollStart + monthEl.offsetHeight;
scrollInfo.push(
{
text: liveHeaderText,
scrollStart: scrollStart,
contentStart: contentStart,
scrollEnd: scrollEnd
});
}
}
}
return scrollInfo;
},
_createGrid: function _createGrid(gridBuilderCal) {
var grid = this._createDivWithClassName(CLASSNAMES.GRID.NAME);
grid.setAttribute("role", "grid");
grid.setAttribute("aria-label", STRINGS.GRID_LABEL);
var ghead = this._createGridHead(gridBuilderCal);
this._gbody = this._createGridBody(gridBuilderCal);
this._gbody.addEventListener("click", this._onGridBodyClick.bind(this), false);
this._gbody.addEventListener("keydown", this._onGridBodyKeyDown.bind(this), false);
this._gbody.addEventListener("scroll", this._onGridBodyScroll.bind(this), false);
this._gbody.addEventListener("wheel", this._onGridBodyWheel.bind(this), false);
this._gbody.setAttribute(DATA_ATTRIBUTE_NAME, JSON.stringify({ numCols: this._numCols, numRows: this._numRows }));
grid.appendChild(ghead);
grid.appendChild(this._gbody);
return grid;
},
_createGridHead: function _createGridHead(gridBuilderCal) {
var ghead = this._createDivWithClassName(CLASSNAMES.GHEAD.NAME);
ghead.setAttribute("role", "rowgroup");
this._liveHeader = this._createLiveHeader(gridBuilderCal.getDateTime());
ghead.appendChild(this._liveHeader);
ghead.appendChild(this._createColHeaderRow());
return ghead;
},
_createLiveHeader: function _createLiveHeader(datetime) {
var liveHeader = document.createElement("h2");
liveHeader.setAttribute("role", "presentation");
liveHeader.setAttribute("aria-live", "ignore");
Util.addClass(liveHeader, CLASSNAMES.HLIVE.NAME);
liveHeader.innerText = this._liveHeaderFormatter.format(datetime);
return liveHeader;
},
_createColHeaderRow: function _createColHeaderRow() {
var colHeaderRow = this._createDivWithClassName(CLASSNAMES.ROW.NAME + " " + CLASSNAMES.COLHEADERS.NAME);
colHeaderRow.setAttribute("role", "row");
var colHeaderRowHeader = document.createElement("div");
colHeaderRow.appendChild(colHeaderRowHeader);
/* to construct the local col header order with localised Calendar day names,
we create a new calendar, position it to the start of the local week
then iterate forward through 7 days */
var cal = this._createGregorianCalendar();
this._setCalToFirstDayOfLocalWeek(cal);
for (var colIndex = 0; colIndex < NUM_DAYS_IN_WEEK; colIndex++) {
if (!CachedDayNames[colIndex]) {
CachedDayNames[colIndex] = {
full: cal.dayOfWeekAsSoloString(),
short: cal.dayOfWeekAsSoloString(SHORT_DAY_NAME_IDEAL_LENGTH)
};
}
colHeaderRow.appendChild(this._createColHeader(colIndex, CachedDayNames[colIndex].full, CachedDayNames[colIndex].short));
cal.addDays(1);
}
return colHeaderRow;
},
_createColHeader: function _createColHeader(colIndex, dayNameFull, dayNameShort) {
var colHeader = document.createElement("div");
colHeader.setAttribute("role", "columnheader");
colHeader.id = this._element.id + "-col-" + (colIndex);
colHeader.innerText = dayNameShort;
colHeader.setAttribute("aria-label", dayNameFull);
return colHeader;
},
_createGridBody: function _createGridBody(gridBuilderCal) {
var gbody = this._createDivWithClassName(CLASSNAMES.GBODY.NAME);
gbody.setAttribute("role", "rowgroup");
gbody.setAttribute("aria-hidden", "false"); // Enable Narrator Touch 2 finger scroll on scrolling container
var monthsCreatedSoFar = 0;
var yearDiff = this._maxDateCal.year - this._minDateCal.year;
var yearDiffInMonths = yearDiff * this._maxDateCal.numberOfMonthsInThisYear;
var totalMonthsNeeded = yearDiffInMonths + this._maxDateCal.month - this._minDateCal.month + 1;
while (monthsCreatedSoFar < totalMonthsNeeded) {
var totalNumMonthsLeftToAdd = totalMonthsNeeded - monthsCreatedSoFar;
var numMonthsLeftInYear = gridBuilderCal.numberOfMonthsInThisYear - gridBuilderCal.month + 1;
var numMonthsToAddForThisYear = Math.min(numMonthsLeftInYear, totalNumMonthsLeftToAdd);
gbody.appendChild(this._createYear(gridBuilderCal, numMonthsToAddForThisYear));
monthsCreatedSoFar += numMonthsToAddForThisYear;
}
return gbody;
},
_createYear: function _createYear(gridBuilderCal, maxMonthsForThisYear) {
var year = this._createDivWithClassName(CLASSNAMES.YEAR.NAME);
year.appendChild(this._createYearHeading(gridBuilderCal.year));
year.setAttribute("aria-label", gridBuilderCal.year);
var numMonthsLeftInYear = gridBuilderCal.numberOfMonthsInThisYear - gridBuilderCal.month + 1;
if (maxMonthsForThisYear) {
numMonthsLeftInYear = maxMonthsForThisYear;
}
for (var i = 0; i < numMonthsLeftInYear; i++) {
year.appendChild(this._createMonth(gridBuilderCal));
gridBuilderCal.addMonths(1);
}
return year;
},
_createMonth: function _createMonth(gridBuilderCal) {
var month = this._createDivWithClassName(CLASSNAMES.MONTH.NAME);
var lastDayInMonth = gridBuilderCal.numberOfDaysInThisMonth,
yearNum = gridBuilderCal.year,
monthNum = gridBuilderCal.month,
dayNum = gridBuilderCal.firstDayInThisMonth;
if (!CachedMonthNames[monthNum]) {
CachedMonthNames[monthNum] = gridBuilderCal.monthAsSoloString();
}
month.appendChild(this._createMonthHeading(CachedMonthNames[monthNum]));
month.setAttribute("aria-label", CachedMonthNames[monthNum]);
month.setAttribute("data-liveheader", this._liveHeaderFormatter.format(gridBuilderCal.getDateTime()));
var weekNum = 1;
var startColIndex = (gridBuilderCal.dayOfWeek >= LocalWeekStartsOn) ? gridBuilderCal.dayOfWeek - LocalWeekStartsOn : NUM_DAYS_IN_WEEK - (LocalWeekStartsOn - gridBuilderCal.dayOfWeek);
while (dayNum <= lastDayInMonth) {
var week = this._createWeek(yearNum, monthNum, weekNum, dayNum, startColIndex, lastDayInMonth, this._numRows);
month.appendChild(week);
dayNum += (NUM_DAYS_IN_WEEK - startColIndex);
startColIndex = 0;
weekNum++;
this._numRows++;
}
return month;
},
_createWeek: function _createWeek(yearNum, monthNum, weekNum, dayNum, startColIndex, lastDayInMonth, rowIndex) {
var week = this._createDivWithClassName(CLASSNAMES.ROW.NAME + " " + CLASSNAMES.WEEK.NAME);
var weekHeader = this._createWeekHeader(this._element.id, yearNum, monthNum, weekNum);
var day = null;
week.appendChild(weekHeader);
week.setAttribute("role", "row");
week.setAttribute("aria-labelledby", weekHeader.id);
for (var colIndex = 0; colIndex < NUM_DAYS_IN_WEEK; colIndex++) {
if (colIndex < startColIndex || dayNum > lastDayInMonth) {
day = this._createGridCell(this._element.id, colIndex, rowIndex, true, weekHeader.id);
} else {
// if current day is outside the start-end range- make it unselectable
if ((this._minDateCal.year >= yearNum && this._minDateCal.month >= monthNum && this._minDateCal.day > dayNum) ||
(this._maxDateCal.year <= yearNum && this._maxDateCal.month <= monthNum && this._maxDateCal.day < dayNum)) {
day = this._createUnselectableDay(colIndex, rowIndex, yearNum, monthNum, dayNum, weekHeader.id);
} else {
day = this._createSelectableDay(colIndex, rowIndex, yearNum, monthNum, dayNum, weekHeader.id);
}
dayNum++;
if (this._todayAriaLabel === day.getAttribute("aria-label")) {
Util.addClass(day, CLASSNAMES.TODAY.NAME);
this._todayGridCell = day;
}
}
week.appendChild(day);
}
return week;
},
_createWeekHeader: function _createWeekHeader(elementId, yearNum, monthNum, weekNum) {
var weekHeader = document.createElement("div");
weekHeader.setAttribute("role", "rowheader");
weekHeader.innerText = STRINGS.WEEK_NUM.format(weekNum);
weekHeader.id = elementId + "-" + yearNum + "-" + monthNum + "-week" + weekNum;
return weekHeader;
},
_createUnselectableDay: function _createUnselectableDay(colIndex, rowIndex, yearNum, monthNum, dayNum, weekId) {
var day = this._createGridCell(this._element.id, colIndex, rowIndex, true, weekId);
day.setAttribute("aria-label", yearNum + "-" + this._padNumber(monthNum) + "-" + this._padNumber(dayNum));
day.innerText = dayNum;
return day;
},
_createSelectableDay: function _createSelectableDay(colIndex, rowIndex, yearNum, monthNum, dayNum, weekId) {
var day = this._createGridCell(this._element.id, colIndex, rowIndex, false, weekId);
var label = yearNum + "-" + this._padNumber(monthNum) + "-" + this._padNumber(dayNum);
day.id = this._element.id + "_cell_" + label;
this._disableTabIndex(day);
day.setAttribute("href", "#");
day.setAttribute("aria-label", label);
day.innerText = dayNum;
Util.addClass(day, CLASSNAMES.DAY.NAME);
day.setAttribute(DATA_ATTRIBUTE_NAME, JSON.stringify({ col: colIndex, row: rowIndex }));
this._addItemToModel(colIndex, rowIndex, day);
return day;
},
_addItemToModel: function _addItemToModel(col, row, item) {
if (!this._navigableViewModel[col]) {
this._navigableViewModel[col] = [];
}
this._navigableViewModel[col][row] = item;
},
_selectGridCell: function _selectGridCell(newSelectedGridCell, raiseEvent) {
// Remove the style from the previously selected grid cell
if (this._selectedGridCell !== null) {
Util.removeClass(this._selectedGridCell, CLASSNAMES.SELECTED.NAME);
}
// Add style to the new selected grid cell
Util.addClass(newSelectedGridCell, CLASSNAMES.SELECTED.NAME);
// When a gridcell is selected, it always becomes the activegridcell (gridcell with tabindex)
this._setActiveGridCell(newSelectedGridCell);
// Update the current calendar with the new date in gridcell
var ariaLabel = newSelectedGridCell.getAttribute("aria-label"),
dateString = this._dashesToSlashes(ariaLabel);
this._currentCal.setDateTime(new Date(dateString));
if (raiseEvent) {
this._raiseCustomEvent(EVENTS.SELECT, this.current);
}
// Finally, cache the new selected grid cell
this._selectedGridCell = newSelectedGridCell;
},
_setActiveGridCell: function _setActiveGridCell(newActiveGridCell) {
// Disable previous activeGridCell
if (this._activeGridCell) {
this._disableTabIndex(this._activeGridCell);
}
// Move the tabindex to the new active grid cell
this._enableTabIndex(newActiveGridCell);
// Update the position of our viewModel row/col
this._updateViewModelPos(newActiveGridCell);
// Finally, cache the new active grid cell
this._activeGridCell = newActiveGridCell;
},
_enableTabIndex: function _enableTabIndex(gridcell) {
gridcell.tabIndex = 0;
},
_disableTabIndex: function _disableTabIndex(gridcell) {
gridcell.tabIndex = DISABLED_TABINDEX_VALUE;
},
_updateViewModelPos: function _updateViewModelPos(gridcell) {
var data = PlatformJS.Utilities.parseDataAttribute(DATA_ATTRIBUTE_NAME, gridcell);
if (data) {
this._navigableViewModelPos.col = data.col;
this._navigableViewModelPos.row = data.row;
}
},
/* event handlers */
_onDayPollerChange: function _onDayPollerChange(event) {
var newDate = event.detail.date;
if (newDate) {
this._todayCal.setDateTime(newDate);
if (this._todayGridCell !== null) {
Util.removeClass(this._todayGridCell, CLASSNAMES.TODAY.NAME);
this._todayGridCell = null;
this._todayAriaLabel = null;
}
var newDateLabel = this._calendarToAriaLabel(this._todayCal),
newDateSelector = "[aria-label='{0}']".format(newDateLabel),
newDateGridCell = this._grid.querySelector(newDateSelector);
if (newDateGridCell) {
this._todayGridCell = newDateGridCell;
this._todayAriaLabel = newDateLabel;
Util.addClass(this._todayGridCell, CLASSNAMES.TODAY.NAME);
}
}
},
_onGridBodyClick: function _onGridBodyClick(event) {
if (Util.hasClass(event.target, CLASSNAMES.DAY.NAME)) {
this._selectGridCell(event.target, true);
}
},
_onGridBodyKeyDown: function _onGridBodyKeyDown(event) {
switch (event.keyCode) {
case Util.Key.space: event.preventDefault(); this._onGridBodyClick(event); break;
case Util.Key.pageUp: this._onPageUpKey(event); break;
case Util.Key.pageDown: this._onPageDownKey(event); break;
case Util.Key.upArrow: this._onUpArrowKey(event); break;
case Util.Key.rightArrow: this._onRightArrowKey(event); break;
case Util.Key.downArrow: this._onDownArrowKey(event); break;
case Util.Key.leftArrow: this._onLeftArrowKey(event); break;
default: break;
}
},
_onPageUpKey: function _onPageUpKey(event) {
event.stopPropagation();
event.preventDefault();
var scrollInfo = this._scrollInfo,
currentTop = this._gbody.scrollTop,
nextTop = currentTop;
for (var scrollIndex = 0, scrollLength = scrollInfo.length; scrollIndex < scrollLength; scrollIndex++) {
var currentContentStart = scrollInfo[scrollIndex].contentStart;
if (currentTop > scrollInfo[scrollIndex].scrollStart && currentTop <= currentContentStart) {
nextTop = scrollInfo[Math.max(scrollIndex - 1, 0)].contentStart;
break;
} else if (currentTop > currentContentStart && currentTop <= scrollInfo[scrollIndex].scrollEnd) {
nextTop = currentContentStart;
break;
}
}
this._gbody.scrollTop = nextTop;
},
_onPageDownKey: function _onPageDownKey(event) {
event.stopPropagation();
event.preventDefault();
var scrollInfo = this._scrollInfo,
currentTop = this._gbody.scrollTop,
nextTop = currentTop;
for (var scrollIndex = 0, scrollLength = scrollInfo.length; scrollIndex < scrollLength; scrollIndex++) {
if (currentTop >= scrollInfo[scrollIndex].scrollStart && currentTop < scrollInfo[scrollIndex].scrollEnd) {
nextTop = scrollInfo[Math.min(scrollIndex + 1, scrollLength - 1)].contentStart;
break;
}
}
this._gbody.scrollTop = nextTop;
},
_onUpArrowKey: function _onUpArrowKey(event) {
event.stopPropagation();
event.preventDefault();
var nextGridCell = null,
rowDecrement = 0;
// Find the first selectable grid cell above
while ((this._navigableViewModelPos.row - rowDecrement) > 0 && !nextGridCell) {
rowDecrement++;
nextGridCell = this._navigableViewModel[this._navigableViewModelPos.col][this._navigableViewModelPos.row - rowDecrement];
}
if (nextGridCell) {
this._navigableViewModelPos.row = this._navigableViewModelPos.row - rowDecrement;
this._disableTabIndex(this._activeGridCell);
this._activeGridCell = nextGridCell;
this._enableTabIndex(this._activeGridCell);
this._activeGridCell.focus();
}
},
_onRightArrowKey: function _onRightArrowKey(event) {
event.stopPropagation();
if (this._navigableViewModelPos.col < this._numCols - 1) {
var nextGridCell = this._navigableViewModel[this._navigableViewModelPos.col + 1][this._navigableViewModelPos.row];
if (nextGridCell) {
this._navigableViewModelPos.col++;
this._disableTabIndex(this._activeGridCell);
this._activeGridCell = nextGridCell;
this._enableTabIndex(this._activeGridCell);
this._activeGridCell.focus();
}
}
},
_onDownArrowKey: function _onDownArrowKey(event) {
event.stopPropagation();
event.preventDefault();
var numRowsInCol = this._navigableViewModel[this._navigableViewModelPos.col].length,
nextGridCell = null,
rowIncrement = 0;
// Find the first selectable grid cell below
while ((this._navigableViewModelPos.row + rowIncrement) < numRowsInCol && !nextGridCell) {
rowIncrement++;
nextGridCell = this._navigableViewModel[this._navigableViewModelPos.col][this._navigableViewModelPos.row + rowIncrement];
}
if (nextGridCell) {
this._navigableViewModelPos.row = this._navigableViewModelPos.row + rowIncrement;
this._disableTabIndex(this._activeGridCell);
this._activeGridCell = nextGridCell;
this._enableTabIndex(this._activeGridCell);
this._activeGridCell.focus();
}
},
_onLeftArrowKey: function _onLeftArrowKey(event) {
event.stopPropagation();
if (this._navigableViewModelPos.col > 0) {
var nextGridCell = this._navigableViewModel[this._navigableViewModelPos.col - 1][this._navigableViewModelPos.row];
if (nextGridCell) {
this._navigableViewModelPos.col--;
this._disableTabIndex(this._activeGridCell);
this._activeGridCell = nextGridCell;
this._enableTabIndex(this._activeGridCell);
this._activeGridCell.focus();
}
}
},
_onGridBodyScroll: function _onGridBodyScroll(event) {
window.clearTimeout(this._scrollTimer);
this._scrollTimer = window.setTimeout(this._doScrollUpdate.bind(this, event), SCROLL_REFRESH_MS);
},
_doScrollUpdate: function _doScrollUpdate(event) {
var scrollTop = event.target.scrollTop;
for (var i = 0, len = this._scrollInfo.length; i < len; i++) {
var currentItem = this._scrollInfo[i];
if (scrollTop >= currentItem.scrollStart && scrollTop < currentItem.scrollEnd) {
this._liveHeader.innerText = currentItem.text;
break;
}
}
},
_onGridBodyWheel: function _onGridBodyWheel(event) {
event.stopPropagation();
},
_raiseCustomEvent: function _raiseCustomEvent(eventName, detailValue) {
var event = document.createEvent("CustomEvent");
event.initCustomEvent(eventName, false, false, detailValue);
this._element.dispatchEvent(event);
},
_getCurrentEl: function _getCurrentEl() {
var currentEl = null;
if (this._gbody !== null) {
var dateString = this._calendarToAriaLabel(this._currentCal);
var selector = "a[aria-label='" + dateString + "']";
currentEl = this._gbody.querySelector(selector) || null;
}
return currentEl;
},
_calendarToAriaLabel: function _calendarToAriaLabel(cal) {
return cal.year + "-" + this._padNumber(cal.month) + "-" + this._padNumber(cal.day);
},
_padNumber: function _padNumber(number) {
return (number < 10) ? "0" + number : number;
},
_dashesToSlashes: function _dashesToSlashes(stringWithDashes) {
var tokens = stringWithDashes.split("-");
return tokens.join("/");
},
_createGregorianCalendar: function _createGregorianCalendar() {
var cal = new G11n.Calendar(Languages);
cal.changeCalendarSystem(G11n.CalendarIdentifiers.gregorian);
return cal;
},
_createDivWithClassName: function _createDivWithClassName(className) {
var element = document.createElement("div");
Util.addClass(element, className);
return element;
},
_createYearHeading: function _createYearHeading(text) {
var element = document.createElement("h3");
element.setAttribute("role", "presentation");
Util.addClass(element, CLASSNAMES.HYEAR.NAME);
element.innerText = text;
return element;
},
_createMonthHeading: function _createMonthHeading(text) {
var element = document.createElement("h4");
element.setAttribute("role", "presentation");
element.innerText = text;
Util.addClass(element, CLASSNAMES.HMONTH.NAME);
return element;
},
_createGridCell: function _createGridCell(id, colNum, rowNum, isDisabled, weekId) {
var gridCell = document.createElement("a");
Util.addClass(gridCell, CLASSNAMES.GRIDCELL.NAME);
gridCell.setAttribute("role", "gridcell");
gridCell.setAttribute("aria-describedby", id + "-col-" + colNum + " " + weekId);
if (isDisabled) {
gridCell.tabIndex = -1;
gridCell.setAttribute("aria-disabled", true);
}
return gridCell;
},
_setCalToFirstDayOfMonth: function setCalToFirstDayOfMonth(cal) {
if (cal.day !== cal.firstDayInThisMonth) {
var dayOffset = cal.day - cal.firstDayInThisMonth;
cal.addDays(-dayOffset);
}
},
_setCalToFirstDayOfLocalWeek: function _setCalToFirstDayOfLocalWeek(cal) {
var localDayOfWeekOffset = cal.dayOfWeek - LocalWeekStartsOn;
cal.addDays(-(localDayOfWeekOffset));
},
_createLiveHeaderDateFormatter: function _createLiveHeaderDateFormatter() {
return new DateTimeFormatting.DateTimeFormatter(
DateTimeFormatting.YearFormat.full,
DateTimeFormatting.MonthFormat.full,
DateTimeFormatting.DayFormat.none,
DateTimeFormatting.DayOfWeekFormat.none,
DateTimeFormatting.HourFormat.none,
DateTimeFormatting.MinuteFormat.none,
DateTimeFormatting.SecondFormat.none,
Languages
);
},
/// <field>
/// The current date time.
/// </field>
current: {
get: function get() {
return { date: this._currentCal.getDateTime() };
},
set: function set(currentDate) {
try {
currentDate = new Date(currentDate);
if (currentDate < this._minDateCal.getDateTime() || currentDate > this._maxDateCal.getDateTime()) {
currentDate = new Date(this._minDateCal.getDateTime());
}
this._currentCal.setDateTime(currentDate);
var currentEl = this._getCurrentEl();
if (currentEl !== null) {
this._selectGridCell(currentEl, false);
}
}
catch (e) {
/* do nothing */
}
}
},
/// <field>
/// The DOM element container.
/// </field>
element: {
get: function get() {
return this._element;
}
},
/// <field>
/// The minimum date time.
/// </field>
minDate: {
set: function set(minDate) {
try {
this._minDateCal.setDateTime(new Date(minDate));
this._render();
}
catch (e) {
/* do nothing. falls back to today's date */
}
},
get: function get() {
return { date: this._minDateCal.getDateTime() };
}
},
/// <field>
/// The maximum date time.
/// </field>
maxDate: {
set: function set(maxDate) {
try {
this._maxDateCal.setDateTime(new Date(maxDate));
this._render();
}
catch (e) {
/* do nothing. falls back to today's date */
}
},
get: function get() {
return { date: this._maxDateCal.getDateTime() };
}
},
/// <field>
/// Set the maximum and minimum date time.
/// </field>
setSelectableDateRange: function setSelectableDateRange(minDate, maxDate) {
try {
minDate = new Date(minDate);
maxDate = new Date(maxDate);
if (minDate <= maxDate) {
this._minDateCal.setDateTime(minDate);
this._maxDateCal.setDateTime(maxDate);
this._render();
}
}
catch (e) {
// do nothing- should already have been default initialized.
}
},
/// <field>
/// Set focus to the active grid cell.
/// </field>
focus: function focus() {
if (this._scrollInfo.length === 0) {
this._scrollInfo = this._createScrollInfo(this._gbody);
}
if (this._activeGridCell !== null) {
this._activeGridCell.focus();
}
},
/// <field>
/// Destroy the dateSelector dom element.
/// </field>
destroy: function destroy() {
PlatformJS.Utilities.DayPoller.getInstance().removeListener(this._element.id, this._onDayPollerChange.bind(this));
// all scrolling information & dependent timers are invalid
// once we decide to rebuild the control.
window.clearTimeout(this._scrollTimer);
this._scrollInfo = [];
this._selectedGridCell = null;
this._activeGridCell = null;
this._todayGridCell = null;
if (this._grid !== null) {
if (this._gbody !== null) {
this._gbody.removeEventListener("click", this._onGridBodyClick.bind(this), false);
this._gbody.removeEventListener("keydown", this._onGridBodyKeyDown.bind(this), false);
this._gbody.removeEventListener("scroll", this._onGridBodyScroll.bind(this), false);
this._gbody.removeEventListener("wheel", this._onGridBodyWheel.bind(this), false);
this._gbody = null;
}
this._liveHeader = null;
this._grid = null;
this._gbody = null;
this._navigableViewModel = null;
this._navigableViewModelPos = {
col: null,
row: null
};
this._numRows = 0;
}
this._element.innerHTML = "";
}
}
)
});
WinJS.Class.mix(CommonJS.UI.DateSelector, WinJS.UI.DOMEventMixin);
})();
