﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: dynamicPanoDataProvider.js */
(function () {
"use strict";
WinJS.Namespace.define("DynamicPanoJS",
{
DynamicPanoProvider: WinJS.Class.define(function (options) {
WinJS.UI.setOptions(this, options);
}, {
_versionChanged: false,
_cmsProgressComplete: false,
cmsData: null,
cmsHash: null,
cmsCss: null,
layoutCacheInfo: null,
algoHash: null,
isNewData: false,
fetchPromise: null,
disableAlgoFetch: false,
dataSourceName: null,
feedName: null,
cssFeedUrl: null,
dataProviderOptions: null,
channelId: null,
callReceiverProgress: function callReceiverProgress() {
if (this._cmsProgressComplete) {
if (this.receiverProgressCallback) {
this.receiverProgressCallback({
offlineData: this.offlineData
});
}
}
},
callReceiverComplete: function callReceiverComplete() {
var str = null;
if (this.cmsData) {
// NULL out field that can't be serialized
this.cmsData.originalResponse = null;
// Convert the CMS articles here so we don't have to do it again on warm start
if (this.cmsData.hero && this.cmsData.hero.content) {
this.cmsData.hero = this.convertCMSArticle(this.cmsData.hero, this.dataProviderOptions);
} else {
this.cmsData.hero = null;
}
for (var category in this.cmsData.categories) {
var categoryData = this.cmsData.categories[category];
if (categoryData && categoryData.entities) {
var newEntities = [];
var count = categoryData.entities.length;
var isVideoCluster = true;
for (var i = 0; i < count ; i++) {
var entity = categoryData.entities[i];
if (entity && entity.content) {
var newEntry = this.convertCMSArticle(entity, this.dataProviderOptions);
newEntities.push(newEntry);
var templateClass = newEntry.templateClass;
if (templateClass !== "V1" && templateClass !== "V2" && templateClass !== "S2") {
isVideoCluster = false;
}
}
}
categoryData.entities = newEntities;
categoryData.isVideoCluster = isVideoCluster;
}
}
// Stash this away so we can recover it quick on warm start
str = {
version: DynamicPanoJS.DynamicPanoProvider._version,
cmsData: this.cmsData,
cmsHash: this.cmsHash,
cmsCss: this.cmsCss,
layoutCacheInfo: this.layoutCacheInfo
};
} 
if (this.receiverCompleteCallback) {
this.receiverCompleteCallback(
{
isNewData: this.isNewData,
offlineData: this.offlineData,
cmsData: this.cmsData,
cmsCss: this.cmsCss,
});
}
if (str) {
// Stash this away so we can recover it quick on warm start
PlatformJS.Cache.CacheService.getInstance("CustomPanoCache").addEntry(this.channelId + "_" + this.feedName, str, { supportsInMemory: true });
}
},
fetchCacheData: function fetchCacheData() {
var that = this;
return PlatformJS.Cache.CacheService.getInstance("CustomPanoCache").findEntry(this.channelId + "_" + this.feedName, { supportsInMemory: true })
.then(function findEntry_Complete(complete) {
if (complete) {
if (complete.dataValue && complete.dataValue.cmsData && complete.dataValue.layoutCacheInfo) {
that.layoutCacheInfo = complete.dataValue.layoutCacheInfo;
}
return complete.dataValue;
}
return null;
});
},
getDynamicCssFeed: function getDynamicCssFeed(bypassCache, url) {
msWriteProfilerMark("NewsApp:getDynamicCssFeed" + ":s");
var dataSource = "CustomPanoCssFeed";
var cssService = new PlatformJS.DataService.QueryService(dataSource);
var urlParams = PlatformJS.Collections.createStringDictionary();
if (url) {
urlParams["url"] = url;
}
var options = new Platform.DataServices.QueryServiceOptions();
options.bypassCache = bypassCache;
options.useCompatDataModel = true;
return new WinJS.Promise(function (complete, error, progress) {
cssService.downloadDataAsync(urlParams, null, null, options).then(function (e) {
msWriteProfilerMark("NewsApp:getDynamicCssFeed" + ":e");
complete(e);
}, function (e) {
msWriteProfilerMark("NewsApp:getDynamicCssFeed" + ":e");
if (PlatformJS.Utilities.isPromiseCanceled(e)) {
return;
}
error(e);
}, function (e) {
if (e.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryFound) {
progress(e);
}
else if (e.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryNotFound) {
progress(null);
}
});
});
},
dynamicServiceAndParams: function dynamicServiceAndParams(dataSourceName, feedName, feedMarket) {
var dataSource = dataSourceName || "CustomPanoFeed";
var cmsHeroService = new PlatformJS.DataService.QueryService(dataSource);
var urlParams = PlatformJS.Collections.createStringDictionary();
var marketParams = feedMarket || this.market || CommonJS.Globalization.getMarketStringForEditorial();
if (marketParams) {
urlParams["market"] = marketParams;
}
if (feedName) {
urlParams["feedName"] = feedName;
}
if (PlatformJS.isDebug && CommonJS.State && CommonJS.State.isPreviewModeEnabled) {
urlParams["previewparams"] = "/" + PlatformJS.Services.appConfig.getString("CMSPreviewParam");
}
else {
urlParams["previewparams"] = "";
}
return { service: cmsHeroService, params: urlParams };
},
getDynamicPanoFeed: function getDynamicPanoFeed(bypassCache, dataSourceName, feedName, feedMarket, /*@optional*/provider) {
msWriteProfilerMark("NewsApp:getDynamicPanoFeed" + ":s");
var serviceAndParams = this.dynamicServiceAndParams(dataSourceName, feedName, feedMarket);
var cmsHeroService = serviceAndParams.service;
var urlParams = serviceAndParams.params;
if (!serviceAndParams.params["market"]) { // market is mandatory, so if there's no market, we don't want to show any data.
// without a market, we can't get a meaningful hash either
return WinJS.Promise.wrap({ data: null, hash: null  });
}
var processResponse = function (response) {
var cmsData = null;
if (response) {
var editorial;
try {
editorial = JSON.parse(response.dataString); 
if (response.isLocalResponse) {
cmsData = {};
cmsData.offlineData = editorial;
cmsData.originalResponse = response;
cmsData.backgroundImages = editorial.backgroundImages;
}
else if (editorial) {
cmsData = {};
PlatformJS.Ads.setAdConfig(editorial.adsData);
if (editorial.lastModified && editorial.lastModified.utctime) {
cmsData.lastModified = editorial.lastModified.utctime;
}
cmsData.version = editorial.version;
cmsData.lastFetchedTime = response.lastUpdateTime ? response.lastUpdateTime.getTime() : null;
cmsData.uniqueResponseId = response.responseId;
cmsData.backgroundImages = editorial.backgroundImages;
var clusterKeyIndex = 0;
for (var i = 0; i < editorial.clusters.length; i++) {
var cluster = editorial.clusters[i];
if (cluster.type === "Hero") {
cmsData.hero = cluster.hero;
}
else if (cluster.type === "EntityList") {
if (!cmsData.categories) {
cmsData.categories = {};
}
if (!cmsData.categoryOrder) {
cmsData.categoryOrder = [];
}
if (cluster.entityList) {
var categoryKey = cluster.entityList.collectionId;
if (!categoryKey) {
categoryKey = "customCluster" + clusterKeyIndex;
clusterKeyIndex++;
}
var categoryData = {};
categoryData.entities = cluster.entityList.entities;
categoryData.template = cluster.entityList.template;
categoryData.categoryName = cluster.entityList.categoryName ? cluster.entityList.categoryName : "";
var maxColumnCount = cluster.entityList.maxColumnCount;
if (maxColumnCount && maxColumnCount > 0) {
categoryData.maxColumnCount = maxColumnCount;
}
cmsData.categories[categoryKey] = categoryData;
cmsData.categoryOrder.push(categoryKey);
}
}
}
cmsData.originalResponse = response;
}
// add the authInfo to cmsData
cmsData.authInfo = editorial.authinfo;
}
catch (ex) {
return null;
}
}
return cmsData;
};
var options = new Platform.DataServices.QueryServiceOptions();
options.bypassCache = bypassCache;
options.useCompatDataModel = true;
var LOCAL_OFFLINE_DATA_FILE = "cannedData\\offline.json";
options.localDataFile = LOCAL_OFFLINE_DATA_FILE;
if (provider) {
provider.layoutCacheInfo = { cacheKey: cmsHeroService.getUrl(urlParams), cacheID: cmsHeroService.getCacheID() };
}
return new WinJS.Promise(function (complete, error, progress) {
cmsHeroService.downloadDataAsync(urlParams, null, null, options).then(function (e) {
if (provider && provider.layoutCacheInfo && e) {
provider.layoutCacheInfo.uniqueResponseID = e.responseId;
}
if (!e.isLocalResponse) {
var cmsData = processResponse(e);
if (!cmsData) {
cmsHeroService.deleteCacheEntryAsync(urlParams, null);
}
msWriteProfilerMark("NewsApp:getDynamicPanoFeed" + ":e");
complete({ data: cmsData, hash: cmsHeroService.entryHash(urlParams, null) });
}
else {
msWriteProfilerMark("NewsApp:getDynamicPanoFeed:e");
complete(null);
}
}, function (e) {
msWriteProfilerMark("NewsApp:getDynamicPanoFeed" + ":e");
if (PlatformJS.Utilities.isPromiseCanceled(e)) {
return;
}
error(e);
}, function (e) {
if (e.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryFound) {
// Cached data was available.
var cmsData = null;
if (e.cachedResponse) {
cmsData = processResponse(e.cachedResponse);
}
progress(cmsData);
}
else if (e.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryNotFound) {
progress(null);
}
});
});
},
fetchNetworkData: function fetchNetworkData(bypassCache) {
var that = this;
var feeds = [];
if (!this._cmsProgressComplete) {
if (this.cssFeedUrl) {
feeds["css"] = this.getDynamicCssFeed(bypassCache, this.cssFeedUrl).then(function (e) {
that.cmsCss = e.dataString;
}, function (err) {
that.cmsCss = null;
});
}
feeds["feed"] = this.getDynamicPanoFeed(bypassCache, this.dataSourceName, this.feedName, this.feedMarket, this).then(
function (val) {
// Check if it's new data and not cached data.
if (val) {
that.cmsData = val.data;
that.cmsHash = val.hash;
if (val.data && val.data.originalResponse && !val.data.originalResponse.isCachedResponse && !val.data.originalResponse.serverResponseNotModified) {
that.isNewData = true;
}
}
},
function (error) {
},
function (progress) {
that._cmsProgressComplete = true;
if (progress && progress.originalResponse && progress.originalResponse.isLocalResponse) {
that.offlineData = progress.offlineData;
}
that.callReceiverProgress();
});
}
var dataPromise = new WinJS.Promise(function (complete, error, progress) {
that.receiverCompleteCallback = complete;
that.receiverErrorCallback = error;
that.receiverProgressCallback = progress;
that.fetchPromise = WinJS.Promise.join(feeds);
that.fetchPromise.then(
function (val) {
that.fetchPromise = null;
that.callReceiverComplete();
},
function (err) {
that.fetchPromise = null;
if (that.receiverErrorCallback) {
that.receiverErrorCallback(err);
}
}
);
});
return dataPromise;
},
convertCMSArticle: function convertCMSArticle(result, options) {
var newsArticle = {};
newsArticle.editorial = true;
newsArticle.lastModified = result.lastModified;
if (result.heroImage) {
newsArticle.anchorpoint = result.heroImage.anchorPoint;
}
var cmsImage = null;
if (result.heroImage && result.heroImage.image) {
cmsImage = result.heroImage.image;
}
else {
cmsImage = result.image;
}
if (cmsImage) {
newsArticle.imageAttribution = cmsImage.attribution ? cmsImage.attribution : "";
newsArticle.altText = cmsImage.altText ? cmsImage.altText : "";
newsArticle.caption = cmsImage.caption ? toStaticHTML(cmsImage.caption) : "";
var originalImage = null;
if (cmsImage.images) {
for (var i = 0; i < cmsImage.images.length; i++) {
var img = cmsImage.images[i];
if (img.url) {
var imgName = img.name && img.name.toLowerCase ? img.name.toLowerCase() : "";
if (imgName === "original") {
newsArticle.thumbnail = {};
newsArticle.thumbnail.url = img.url;
newsArticle.thumbnail.width = img.width;
newsArticle.thumbnail.height = img.height;
newsArticle.thumbnail.name = "original";
}
else if (imgName === "semanticzoom") {
newsArticle.semanticZoomThumbnail = {};
newsArticle.semanticZoomThumbnail.url = img.url;
newsArticle.semanticZoomThumbnail.width = img.width;
newsArticle.semanticZoomThumbnail.height = img.height;
newsArticle.semanticZoomThumbnail.name = "semanticZoom";
}
else if (imgName === "lowres") {
newsArticle.thumbnailLowRes = {};
newsArticle.thumbnailLowRes.url = img.url;
newsArticle.thumbnailLowRes.width = img.width;
newsArticle.thumbnailLowRes.height = img.height;
newsArticle.thumbnailLowRes.name = "lowRes";
}
}
}
}
}
else {
newsArticle.imageAttribution = "";
newsArticle.altText = "";
newsArticle.caption = "";
}
newsArticle.sourceImageUrl = result.content.source && result.content.source.favicon ? result.content.source.favicon : "";
// CMS headline support HTML Fragments
newsArticle.headline = result.content.headline ? toStaticHTML(result.content.headline) : "";
newsArticle.title = newsArticle.headline;
newsArticle.source = result.content.source && result.content.source.name ? result.content.source.name : "";
newsArticle.destination = result.destination ? result.destination : "";
// The following are the different destination URLs we support (newsArticle.destination)
// OLD FORMAT (To be eventually removed)
//      article://33259198
// NEW FORMAT (various urls below, see data contract spec for full details)
//      //application/view?entitytype=article&pageId=33259198
//      bingnews://application/view?entitytype=article&pageId=33259198
//      ///view?entitytype=article&pageId=2123125
//      bingnews:///view?entitytype=article&pageId=2123125
function parseCMSUriString(uriString, /*@optional*/defaultMarket) {
var result = parseUriString(uriString, "bingnews", defaultMarket);
if (result && result.path === "application/view") {
result.entitytype = result.params["entitytype"];
result.pageId = result.params["pageid"];
result.contentId = result.params["contentid"];
} else {
result = null;
}
return result;
}
function parseUri(uri, /*@optional*/defaultMarket) {
var result = null;
if (uri) {
if (!uri.suspicious) {
var query = uri.query;
var path = uri.path;
var host = uri.host;
var actualPath = null;
if (host) {
actualPath = host.toLowerCase();
if (path) {
var pathValue = path.toLowerCase();
if (pathValue !== "/") {
actualPath += path.toLowerCase();
}
}
} else {
if (path) {
actualPath = path.toLowerCase();
}
}
if (actualPath && query) {
var urlParams = PlatformJS.Collections.createStringDictionary();
var queryParser = uri.queryParsed;
var len = queryParser ? queryParser.size : 0;
for (var idx = 0; idx < len; idx++) {
var entry = queryParser.getAt(idx);
if (entry && entry.name && entry.name.length > 0 && entry.value && entry.value.length > 0) {
try {
urlParams[entry.name.toLowerCase()] = entry.value; // the entry value is decoded automatically by the queryParser object
} catch (err) {
;
}
}
}
if (!urlParams["market"] && defaultMarket) {
urlParams["market"] = defaultMarket;
}
result = { protocol: uri.schemeName, params: urlParams, path: actualPath };
}
}
}
return result;
}
function parseUriString(uriString, /*@optional*/defaultProtocol, /*@optional*/defaultMarket) {
var result = null;
if (uriString) {
if (0 >= uriString.indexOf(":") && defaultProtocol) {
// no protocol present, try prepending the default
uriString = defaultProtocol + ":" + uriString;
}
try {
result = parseUri(new Windows.Foundation.Uri(uriString), defaultMarket);
} catch (ex) {
;
}
}
return result;
}
var newsUri = parseCMSUriString(newsArticle.destination);
function stripHTML(string) {
if (string) {
return string.replace(/<(?:.|\n)*?>/gm, "");
} else {
return null;
}
}
if (newsUri) {
if (newsUri.entitytype === "article") {
newsArticle.type = newsUri.entitytype;
newsArticle.articleid = newsUri.pageId && newsUri.pageId.length > 0 ? newsUri.pageId : null;
newsArticle.contentid = newsUri.contentId && newsUri.contentId.length > 0 ? newsUri.contentId : null;
} else if (newsUri.entitytype === "slideshow") {
newsArticle.type = newsUri.entitytype;
newsArticle.contentid = newsUri.pageId;
} else if (newsUri.entitytype === "video") {
newsArticle.type = newsUri.entitytype;
var videoOptions = {};
var videoSource = null;
if (newsUri.contentId && newsUri.contentId.length > 0) {
videoSource = newsUri.contentId;
}
// CMS renders headlines with <p> tags, which the video control doesn't handle. Strip them out
var strippedTitle = stripHTML(newsArticle.headline);
// defend against bad video feed data
// NOTE: as of v2.2.7, CUX video renderer throws out video objects that don't have thumbnails for fullscreen templates.
// snap mode templates will still display. We may want to just toss them here instead.
if (!newsArticle.thumbnail) {
newsArticle.thumbnail = {
width: 0,
height: 0,
url: "",
name: ""
};
}
videoOptions.thumbnail = newsArticle.thumbnail.url;
videoOptions.videoSource = videoSource;
videoOptions.source = newsArticle.source;
videoOptions.sourceImageUrl = newsArticle.sourceImageUrl;
videoOptions.title = strippedTitle;
newsArticle.thumbnail.videoOptions = videoOptions;
}
}
// CMS abstract support HTML Fragments
newsArticle.abstract = result.content.abstract ? toStaticHTML(result.content.abstract) : "";
newsArticle.snippet = newsArticle.abstract;
newsArticle.byline = result.content.byline ? result.content.byline : "";
var publicationDate = result.content.publicationDate;
function getDateString(dateType) {
if (dateType && dateType.utctime) {
var utcTime = dateType.utctime;
// TODO: Let platform know that this may be off an hour, need to investigate
var time = CommonJS.Utils.convertBDITimeToFriendlyTime(utcTime);
return time && time.indexOf("NaN") < 0 ? time : "";
}
else {
return "";
}
}
if (publicationDate) {
newsArticle.updatedTime = getDateString(publicationDate.updated);
newsArticle.publishTime = getDateString(publicationDate.published);
}
newsArticle.template = result.template;
newsArticle.templateClass = result.templateClass;
if (options) {
if (options.useByline) {
newsArticle.source = newsArticle.byline;
}
}
// paywall data (if it exists)
newsArticle.paywall = result.paywallState;
return newsArticle;
}
},
{
_version: 2
}),
});
})();
