﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: Magazine.js */
/********************************************************
* *
* � Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsMagazineInit(WinJS) {
"use strict";
var Magazine = WinJS.Namespace.define("CommonJS.Magazine",
{
getMagazineData: function getMagazineData(dataSource) {
var that = this,
market = "en-us", // TODO, LOCALIZE
qsOptions = new Platform.DataServices.QueryServiceOptions(),
qsUrlParams = PlatformJS.Collections.createStringDictionary(),
dataProvider = new PlatformJS.DataService.QueryService(dataSource);
if (market) {
qsUrlParams["market"] = market;
}
return new WinJS.Promise(function (complete, error, progress) {
var downloadDataAsyncPromise = dataProvider.downloadDataAsync(qsUrlParams, null, null, qsOptions);
downloadDataAsyncPromise.then(function (response) {
var result = JSON.parse(response.dataString);
msWriteProfilerMark("CommonJS:getMagazineData:e");
complete(result);
}, function (e) {
msWriteProfilerMark("CommonJS:getMagazineData:e");
if (PlatformJS.Utilities.isPromiseCanceled(e)) {
return;
}
throw e;
}, function (e) {
if (e.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryFound) {
// Cached data was available.
var result = JSON.parse(e.cachedResponse.dataString);
progress(result);
}
});
});
},
CMSParser: function CMSParser(datamodel) {
var magazines = [],
clusters = datamodel.clusters || [];
if (clusters.length > 0) {
var zinioCluster = clusters.shift(),
zinio = Magazine.parseZinioCluster(zinioCluster);
magazines = Magazine.parseMagazineClusters(clusters);
magazines.unshift(zinio);
}
return magazines;
},
parseZinioCluster: function parseZinioCluster(zinioCluster) {
var zinioEntityList = zinioCluster.entityList,
zinioEntities = zinioEntityList.entities,
zinioCover = zinioEntities.shift(),
articles = Magazine.parseCmsArticles(zinioEntities);
return {
articles: articles,
headline1: articles[0] ? articles[0].headline : "",
headline2: articles[1] ? articles[1].headline : "",
title: zinioCover.content.source.name,
headline: toStaticHTML(zinioCover.content.headline),
abstract: toStaticHTML(zinioCover.content.abstract),
destination: zinioCover.destination
};
},
parseMagazineClusters: function parseMagazineClusters(items) {
var magazines = [];
for (var itemIndex = 0, itemsLength = items.length; itemIndex < itemsLength; itemIndex++) {
var item = items[itemIndex],
entityList = item.entityList,
entities = entityList.entities,
magazineCover = entities.shift(),
contentId = Magazine.parseContentId(magazineCover.destination),
articles = Magazine.parseCmsArticles(entities);
var mag = {
articles: articles,
contentId: contentId,
headline1: articles[0] ? articles[0].headline : "",
headline2: articles[1] ? articles[1].headline : "",
title: magazineCover.content.source.name,
headline: toStaticHTML(magazineCover.content.headline),
abstract: toStaticHTML(magazineCover.content.abstract),
cover: {
contentId: contentId,
image: {
url: magazineCover.image.images[0].url,
altText: magazineCover.image.caption,
width: magazineCover.image.images[0].width,
height: magazineCover.image.images[0].height
},
templateClass: magazineCover.templateClass
}
};
magazines.push(mag);
}
return magazines;
},
parseCmsArticles: function parseCmsArticles(entities) {
var articles = [];
for (var articleIndex = 0, entitiesLength = entities.length; articleIndex < entitiesLength; articleIndex++) {
var cmsArticle = entities[articleIndex],
article = Magazine.parseCmsArticle(cmsArticle);
if (article !== null) {
articles.push(article);
}
}
return articles;
},
parseCmsArticle: function parseCmsArticle(cmsArticle) {
var article = null;
try {
article = {
image: (cmsArticle.image && cmsArticle.image.images) ? {
url: cmsArticle.image.images[0].url,
altText: cmsArticle.image.altText,
width: cmsArticle.image.images[0].width,
height: cmsArticle.image.images[0].height
} : null,
abstract: toStaticHTML(cmsArticle.content.abstract),
headline: toStaticHTML(cmsArticle.content.headline),
byline: cmsArticle.content.byline,
destination: cmsArticle.destination,
contentId: Magazine.parseContentId(cmsArticle.destination),
templateClass: cmsArticle.templateClass
};
}
catch (e) {
console.log(e);
}
return article;
},
parseContentId: function parseContentId(uri) {
// HACK. Brittle. Always assumes contentId is last param
return (uri) ? uri.substring(uri.lastIndexOf("=") + 1, uri.length) : null;
},
articlesToNewsItemsAdapter: function articlesToNewsItemsAdapter(articles) {
var newsItems = [];
articles.forEach(function(article){
newsItems.push(Magazine.articleToNewsItemAdapter(article));
});
return newsItems;
},
articleToNewsItemAdapter: function articleToNewsItemAdapter(article) {
var newsItem = {
contentId: article.contentId,
snippet: article.abstract || "",
source: article.byline || "",
templateClass: article.templateClass || null,
thumbnail: article.image ? {
url: article.image.url,
height: article.image.height,
width: article.image.width
} : null,
title: article.headline || "",
};
return newsItem;
}
});
})(WinJS);
/* File: MagazineCluster.js */
/********************************************************
* *
* � Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsMagazineClusterInit(WinJS) {
"use strict";
var MagazineCluster = WinJS.Namespace.define("CommonJS.Magazine.MagazineCluster",
{
itemRenderer: function itemRenderer(item) {
var data = item.data,
itemsContainerEl = document.createElement("div"),
magazineEl = document.createElement("div"),
imageCard = new CommonJS.ImageCard(magazineEl, {
classification: "large",
altText: data.title,
imageSource: {
url: data.cover.image.url
}
});
WinJS.Utilities.addClass(magazineEl, "magazine fitHeight fitWidth anchorMiddle");
itemsContainerEl.appendChild(magazineEl);
return itemsContainerEl;
},
onHeaderSelection: function onHeaderSelection(event) {
PlatformJS.Navigation.navigateToChannel("MagazineGallery", {
dataSource: "Magazines",
titleResource: "/Resources/MagazineGalleryTitle",
theme: "magazineAppTheme" //TODO get from config
});
},
onItemClick: function onItemClick(element) {
var contentId = element.item.contentId || null;
if (contentId) {
PlatformJS.Navigation.navigateToChannel("MagazineIssue", {
dataSource: "MagazineSpine",
contentId: contentId,
panScanDataSource: "MagazinePanScan"
});
}
}
});
})(WinJS);
/* File: MagazineGallery.js */
/********************************************************
* *
* � Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsMagazineGalleryInit() {
"use strict";
var Magazine = WinJS.Namespace.define("CommonJS.Magazine", {
MagazineGallery: WinJS.Class.define(
function MagazineGallery_ctor(state) {
this._dataSource = state.dataSource;
this._theme = state.theme;
this._titleResource = state.titleResource;
var headerEl = document.getElementById("header");           
if (headerEl && headerEl.winControl) {
headerEl.winControl.headerTitleText = CommonJS.resourceLoader.getString(state.titleResource) || "";
}            
},
{
// BEGIN PRIVATE
_dataSource: null,
_titleResource: null,
_getZinioClusterDefinition: function _getZinioClusterDefinition(magazine) {
var newsItems = Magazine.articlesToNewsItemsAdapter(magazine.articles);
return {
clusterKey: magazine.title,
clusterTitle: magazine.title,
clusterContent: {
contentControl: "CommonJS.News.EntityCluster",
contentOptions: {
dataSet: {
newsItems: newsItems
},
configKey: "EntityClusterDefaultNewsConfig",
onitemclick: CommonJS.Magazine.MagazineGallery.onZinioItemClick,
theme: this._theme
}
}
};
},
_buildMagazineClusters: function _buildMagazineClusters(datamodel) {
var magazines = CommonJS.Magazine.CMSParser(datamodel),
zinioMagazine = magazines.shift(),
clusters = new WinJS.Binding.List();
if (zinioMagazine) {
clusters.push(this._getZinioClusterDefinition(zinioMagazine));
}
for (var magazineIndex = 0, magazinesLength = magazines.length; magazineIndex < magazinesLength; magazineIndex++) {
var magazine = magazines[magazineIndex];
clusters.push({
clusterEntity: magazine,
clusterPosition: magazineIndex,
clusterKey: magazine.title + magazineIndex,
clusterTitle: magazine.title,
clusterContent: {
contentControl: "CommonJS.Magazine.MagazineGalleryCluster",
contentOptions: {
dataSet: magazine,
}
}
});
}
return WinJS.Promise.wrap(clusters);
},
// END PRIVATE
// BEGIN PUBLIC
/// <summary>
/// Set this property to true if you want the back button to appear in the navbar
/// This is used for immersive pages
/// </summary>
isImmersive: true,
// BEGIN IPAGE
getPageState: function getPageState() {
var pageState = {
dataSource: this._dataSource,
titleResource: this._titleResource,
theme: this._theme
};
return pageState;
},
getPageData: function getPageData() {
var that = this,
getMagazineDataPromise = Magazine.getMagazineData(this._dataSource);
return getMagazineDataPromise.then(function (result) {
return that._buildMagazineClusters(result).then(function (clusters) {
return {
clusters: clusters.createSorted(function (a, b) { return 0; }).dataSource,
panoramaState: null,
};
});
});
},
onBindingComplete: function onBindingComplete() {
},
onWindowResize: function onWindowResize() {
},
// END IPAGE
// END PUBLIC
},
{
// BEGIN STATIC
onZinioItemClick: function onZinioItemClick(elt) {
var articleId = elt.detail.item.contentId,
providerConfiguration = PlatformJS.Collections.createStringDictionary();
providerConfiguration.insert("queryServiceId", "MagazineBedrockArticles");
WinJS.Navigation.navigate({
fragment: "/common/ArticleReader/html/ArticleReaderPage.html",
page: "CommonJS.ArticleReaderPage"
}, {
providerType: "AppEx.Common.ArticleReader.BedrockArticleProvider",
providerConfiguration: providerConfiguration,
initialArticleId: articleId
});
},
onItemClick: function onItemClick(element) {
var contentId = element.item.contentId || null;
if (contentId) {
PlatformJS.Navigation.navigateToChannel("MagazineIssue", {
dataSource: "MagazineSpine",
contentId: contentId,
panScanDataSource: "MagazinePanScan"
});
}
}
// END STATIC
})
}
);
})();
/* File: MagazineGalleryCluster.js */
/********************************************************
* *
* � Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsMagazineGalleryClusterInit(WinJS) {
"use strict";
var Magazine = WinJS.Namespace.define("CommonJS.Magazine",
{
/// <summary>
/// A Cluster with modules that represent Magazine items. The items in the cluster conform
/// to AppEx.Common.MagazineCluster.Response.MagazineDescriptor. The module templates for the view assume this interface.
/// </summary>
MagazineGalleryCluster: WinJS.Class.mix(WinJS.Class.define(
function MagazineGalleryCluster_ctor(element, options) {
var that = this;
this._element = element;
this._element.winControl = this;
this._height = this._element.offsetHeight;
this._hostEl = document.createElement("div");
if (options) {
this._dataSet = options.dataSet;
}
WinJS.Utilities.addClass(this._element, options.theme || "");
WinJS.Utilities.addClass(this._hostEl, "magazineGalleryCluster");
this._element.appendChild(this._hostEl);
// TODO: move both predicate and completion handler as utility functions for common controls (northamerica\edesfos)
// [BEGIN]
// register itemclick event
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemclick"));
PlatformJS.Utilities.registerItemClickProxy(this._element,
// predicate
function MagazineCluster_registerItemClickProxyPredicate(element) {
return WinJS.Utilities.hasClass(element, "cluster-item");
},
// completion
function MagazineCluster_registerItemClickProxyCompletion(element) {
var item = element.winControl.data;
that.dispatchEvent("itemclick", {
item: item
});
});
// [END]
Object.defineProperties(this, WinJS.Utilities.createEventProperties("render_complete"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("error"));
this._element.onresize = this._onResizeBinding = this._onResize.bind(this);
},
{
// START PRIVATE
_element: null,
_hostEl: null,
_dataSet: null,
_height: null,
_onResize: function _onResize(evt){
var newHeight = this._element.offsetHeight,
currentHeight = this._height;
if (newHeight !== currentHeight) {
this._height = newHeight;
this.render();
}
},
_renderMagazine: function _renderMagazine(host, magazine) {
var docHeight = document.body.clientHeight,
docWidth = document.body.clientWidth,
templatePath = "/common/Magazine/html/MagazineModuleTemplates.html",
coverContainer = document.createElement("div"),
articleContainer = document.createElement("div"),
articles = magazine.articles,
len = articles.length,
articleMeta = Magazine.MagazineGalleryCluster.getArticleMeta(len),
renderCopyPromise = WinJS.UI.Fragments.renderCopy(templatePath, host);
WinJS.Utilities.addClass(coverContainer, "coverContainer");
WinJS.Utilities.addClass(articleContainer, "articleContainer");
host.appendChild(coverContainer);
host.appendChild(articleContainer);
if (docHeight > Magazine.MagazineGalleryCluster.MINHEIGHT) {
//we need to grow vertically;
WinJS.Utilities.addClass(host, "tall");
len = Math.min(len, Math.floor(host.offsetHeight / (articleMeta.itemHeight + Magazine.MagazineGalleryCluster.MARGIN)));
}
else {
WinJS.Utilities.removeClass(host, "tall");
}
if (docWidth <= Magazine.MagazineGalleryCluster.MINWIDTH) {
WinJS.Utilities.addClass(host, "narrow");
}
else {
WinJS.Utilities.removeClass(host, "narrow");
}
WinJS.Utilities.addClass(articleContainer, articleMeta.articleClass);
renderCopyPromise.then(function list_loadTemplatesRenderCopyComplete() {
return WinJS.UI.processAll(host).then(function () {
var coverTemplate = host.querySelector("#magazine_cover").winControl,
articleTemplate = host.querySelector("#magazine_article").winControl,
descriptionTemplate = host.querySelector("#magazine_description").winControl,                      
coverDiv = document.createElement("div"),
animationSequence = [],
renderPromises = [],
coverPromise = coverTemplate.render(magazine.cover, coverDiv),
height = host.offsetHeight,
numRender = Math.min(len, Math.floor(height / articleMeta.itemHeight));
coverDiv.addEventListener("click", function _onCoverElClick(event) {
var contentId = magazine.cover.contentId;
if (contentId) {
PlatformJS.Navigation.navigateToChannel("MagazineIssue", {
dataSource: "MagazineSpine",
contentId: contentId,
panScanDataSource: "MagazinePanScan"
});
}
});
coverContainer.appendChild(coverDiv);
renderPromises.push(coverPromise);                 
animationSequence.push(coverDiv);
for (var articleIndex = 0; articleIndex < numRender; articleIndex++) {
var div = document.createElement("div"),
article = articles[articleIndex];
if (article.image && article.image.url && article.image.url.length > 1) {
div.className = articleMeta.articleClass;
}
else {
div.className = "small";
numRender = Math.min(len, 4);
}
div.addEventListener("click", function (event) {
var contentId = article.contentId;
if (contentId) {
PlatformJS.Navigation.navigateToChannel("MagazineIssue", {
dataSource: "MagazineSpine",
contentId: contentId,
panScanDataSource: "MagazinePanScan"
});
}
});
renderPromises.push(articleTemplate.render(article, div));
articleContainer.appendChild(div);
animationSequence.push(div);
}
for (var j = 0, l = animationSequence.length; j < l; j++) {
PlatformJS.Utilities.enablePointerUpDownAnimations(animationSequence[j]);
}
return WinJS.Promise.join(renderPromises).then(function () {
return WinJS.Promise.wrap({});
});
});
});
},
// END PRIVATE
onitemclick: null,
onrendercomplete: null,
onerror: null,
/// <summary>
/// Invalidates the Magazine Gallery cluster by querying the service and binding the resulting data to the view.
/// </summary>
render: function render() {
return WinJS.Promise.wrap(this._renderMagazine(this._hostEl, this._dataSet));
}
// END PUBLIC
},
{
// START STATIC
MARGIN: 10,
MINHEIGHT: 768,
MINWIDTH: 768,
getArticleMeta: function getArticleMeta(len) {
var articleMeta = {
articleClass: "medium",
itemHeight: -1
};
switch (len) {
case 0:
case 1:
case 2:
articleMeta.articleClass = "large";
articleMeta.itemHeight = 590;
break;
case 3:
default:
articleMeta.articleClass = "medium";
articleMeta.itemHeight = 280;
break;
}
return articleMeta;
},
itemClickHandler: function itemClickHandler(evt) {
// launch url in browser
var item = evt.item;
var url = item.articleUrl;
if (url) {
var uri = new Windows.Foundation.Uri(url);
Windows.System.Launcher.launchUriAsync(uri).then();
}
}
// END STATIC
}
), WinJS.UI.DOMEventMixin)
});
})(WinJS);
/* File: MagazinePage.js */
/********************************************************
* *
* � Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsMagazineIssuePageInit() {
"use strict";
WinJS.Namespace.define("CommonJS.Magazine", {
MagazineIssuePage: WinJS.Class.define(
// Purpose is to initialize two parameters needed by the data json api.
function MagazineIssuePage_ctor(options) {
//todo, get from options
this.options = options;
this.dataSource = options.dataSource;
this.host = document.querySelector("#magazineIssuePage");
this.host.winControl = this;
this._onSizeChangedHandler = this._onSizeChanged.bind(this);
window.addEventListener("resize", this._onSizeChangedHandler);
this.dataProvider = new PlatformJS.DataService.QueryService(this.dataSource);
var urlParams = PlatformJS.Collections.createStringDictionary();
var providerOptions = new Platform.DataServices.QueryServiceOptions();
urlParams["contentId"] = options.contentId;
var that = this;
this.parserPromise = options.magazine ? WinJS.Promise.wrap(options.magazine) : that.dataProvider.downloadDataAsync(urlParams, null, null, providerOptions).then(function (response) {
var map = JSON.parse(response.dataString);
var parser = that.parser = new CommonJS.Magazine.MagazineParser();
return parser.parse(map);
}, function (error) {
var e = error.message;
console.log(e);
}, function (progress) {
var p2 = progress;
});
CommonJS.setTheme(null, true);
},
{
isImmersive: false,
getPageState: function getPageState() { return this.options; },
_getFileContents: function _getFileContents(file) {
return this._getStorageFile(file).then(function (storageFile) {
return Windows.Storage.FileIO.readTextAsync(storageFile).then(function (fileContent) {
return fileContent;
},
function (error) {
console.log(error);
});
}, function (error) {
console.log(error);
});
},
_getStorageFile: function _getStorageFile(path) {
var folder = Windows.ApplicationModel.Package.current.installedLocation;
return Windows.Storage.StorageFolder.getFolderFromPathAsync(folder.path).then(function (folder) {
return folder.getFileAsync(path).then(function (file) {
return file; //IStorageFile
},
function (error) {
console.log(error);
});
},
function (error) {
console.log(error);
});
},
getPageData: function getPageData() {
var that = this;
return this.parserPromise.then(function (parsedMagazine) {
that.options.magazine = parsedMagazine;
if (parsedMagazine) {
return that._renderMagazine(parsedMagazine);
}
else {
CommonJS.Error.showError(CommonJS.Error.STANDARD_ERROR, function (click) {
console.log("click handler");
});
return WinJS.Promise.wrap({});
}
});
},
_getImageUri: function _getImageUri(uri) {
return this._getStorageFile(uri).then(function (storageFile) {
var thumbnailUrl = URL.createObjectURL(storageFile, { oneTimeOnly: true });
return thumbnailUrl;
});
},
_getImageDimensions: function _getImageDimensions(uri) {
return this._getStorageFile(uri).then(function (storageFile) {
return storageFile.properties.getImagePropertiesAsync().then(function (properties) {
var imageData = { height: properties.height, width: properties.width };
return imageData;
});
});
},
_onSubscribeClick: function _onSubscribeClick() {
},
_renderMagazine: function _renderMagazine(magazine) {
this.host.innerHTML = "";
this.magazine = magazine;
magazine.pages = [magazine.pages[0]];
this.options.magazine = this.magazine;
var pages = magazine.pages;
this.pages = pages;
var that = this;
this.currentPage = 0;
var dimensions = { width: 1181, height: 1600 };
//   this._getImageDimensions(this.folder + "\\" + pages[0].image).then(function (dimensions) {
that._makeContainers();
that.dimensions = dimensions;
that._updateViewport(dimensions, pages.length);
that._renderPages(pages);
var toc = document.createElement("div");
toc.className = "magazineToc";
this.panel.appendChild(toc);
this.toc = toc;
this.toc.style.width = this.scaledSnapWidth + "px";
that._renderToc(that.magazine.articles);
var buyButton = document.createElement("div");
this.buyButton = buyButton;
buyButton.className = "subscribe";
buyButton.textContent = "Subscribe";
buyButton.addEventListener("click", this._onSubscribeClick.bind(this));
toc.appendChild(buyButton);
},
zoomableView: {
get: function get() {
return this;
}
},
_renderToc: function _renderToc(tocs) {
var toc = this.toc;
toc.innerHTML = "";
var tocItems = document.createElement("div");
tocItems.className = "tocItems";
var tocTitle = document.createElement("div");
tocTitle.className = "tocTitle";
tocTitle.textContent = this.magazine.title;
toc.appendChild(tocTitle);
toc.appendChild(tocItems);
tocItems.style.width = toc.style.width;
var that = this;
if (!that.options) {
that.options = {};
}
var onTocClicked = function (event) {
var page = parseInt(event.currentTarget.getAttribute("data-page"));
that.options.pageNumber = page;
PlatformJS.Navigation.navigateToChannel("MagazineReader", { magazine: that.options.magazine, contentId: that.options.contentId, dataSource: that.options.panScanDataSource, pageNumber: page });
};
var tocClickBinding = onTocClicked.bind(this);
for (var i = 0; i < tocs.length; i++) {
var tocItem = tocs[i];
var tocDiv = document.createElement("div");
tocDiv.className = "tocItem " + (tocItem.free ? "isFree" : "isPaid");
tocDiv.innerHTML = toStaticHTML("<img class='tocIcon' src='" + (tocItem.thumbnail ? tocItem.thumbnail.Path : "missing.png") + "' /><div class='tocText'>" + tocItem.title + "</div>");
tocDiv.setAttribute("data-page", tocItem.startPage);
tocDiv.setAttribute("data-article", tocItem.id);
tocDiv.addEventListener("click", tocClickBinding);
tocItems.appendChild(tocDiv);
}
},
_renderPages: function _renderPages(pages) {
var l = pages.length;
var that = this;
var page = 0;
var renderPage = function (pageNum) {
pages[pageNum].pageNumber = pageNum;
that._renderPage(pages[pageNum]).then(function (div) {
that.panel.appendChild(div);
if (++pageNum < l) {
renderPage(pageNum);
}
});
};
renderPage(0);
},
_makeContainers: function _makeContainers() {
var host = this.host;
var spread = document.createElement("div");
spread.className = "spread";
host.appendChild(spread);
this.spread = spread;
var that = this;
that.zoomStd = true; //default
var setZoom = function (zoom) {
that.zoomedOut = false;
that.zoomedIn = false;
that.zoomStd = false;
if (zoom < 1) {
that.zoomedOut = true;
}
else if (zoom > 1) {
that.zoomedIn = true;
}
else {
that.zoomStd = true;
}
};
spread.addEventListener("MSContentZoom", function (event) {
var zoom = spread.msContentZoomFactor;
setZoom(zoom);
if (zoom !== 1) {
WinJS.Utilities.addClass(spread, "zoomed");
if (zoom < 1) {
//    spread.style.top = 100 * (zoom / 2) + "%";
WinJS.Utilities.addClass(spread, "zoomedout");
WinJS.Utilities.removeClass(spread, "zoomedin");
}
else {
//  spread.style.top = 0;
WinJS.Utilities.removeClass(spread, "zoomedout");
WinJS.Utilities.addClass(spread, "zoomedin");
}
}
else {
WinJS.Utilities.removeClass(spread, "zoomed");
WinJS.Utilities.removeClass(spread, "zoomedout");
WinJS.Utilities.removeClass(spread, "zoomedin");
//  spread.style.top = 0;
}
});
var magazinePanel = document.createElement("div");
magazinePanel.className = "magazinePanel";
spread.appendChild(magazinePanel);
this.panel = magazinePanel;
this._boundKeyUp = this._keyUp.bind(this);
window.addEventListener("keydown", this._boundKeyUp);
this._makeFilmstrip(host);
},
_makeFilmstrip: function _makeFilmstrip(host) {
var filmstrip = document.createElement("div");
filmstrip.className = "filmstrip";
this.filmstrip = filmstrip;
host.appendChild(filmstrip);
},
_updateViewport: function _updateViewport(dimensions, numPages) {
var magazinePanel = this.panel;
var host = this.spread;
var height = document.body.clientHeight;
var width = document.body.clientWidth;
var landscape = width > height;
var vRatio = height / dimensions.height;
var hRatio = width / dimensions.width;
var ratio = Math.min(vRatio, hRatio);
var scaledSnapWidth = ratio * dimensions.width;
var xMin = 0;
var snapInterval = scaledSnapWidth * (landscape ? 2 : 1);
var spreadSpacing = 30;
var maxWidth = width + "px";
if (landscape) {
snapInterval += spreadSpacing; //inter-spread spacing;
WinJS.Utilities.addClass(host, "landscape");
maxWidth = scaledSnapWidth + "px";
}
else {
WinJS.Utilities.removeClass(host, "landscape");
xMin = snapInterval;
}
var snapStyle = "snapInterval(" + 0 + "px," + snapInterval + "px)";
host.style.msScrollSnapPointsX = snapStyle;
host.style.width = snapInterval + "px";
var viewportSize = {
width: ((scaledSnapWidth + spreadSpacing / 2) * (numPages)),
height: Math.min(height, ratio * dimensions.height)
};
this.pageWidth = scaledSnapWidth + spreadSpacing / 2;
magazinePanel.style.height = viewportSize.height + "px";
magazinePanel.style.width = viewportSize.width + "px";
this.scaledSnapWidth = scaledSnapWidth;
this.isLandscape = landscape;
//host.style.msScrollLimitXMin = xMin + "px";
var children = magazinePanel.children;
// for (var i = 0; i < children.length; i++) {
//   children[i].style.maxWidth = maxWidth;
//  }
},
_pageNumber: {
get: function get() {
var pageWidth = this.pageWidth;
var left = this.spread.scrollLeft;
var pageNumber = Math.round(left / pageWidth);
console.log("current page is " + pageNumber);
return pageNumber;
},
set: function set(value) {
var pageWidth = this.pageWidth;
if (this.isLandscape) {
value = value - (value % 2);
}
var newLeft = value * pageWidth;
console.log("scrolling to " + newLeft + " for page " + value + " with a pageWidth of " + pageWidth);
this.spread.scrollLeft = newLeft;
}
},
_leftPageClick: function _leftPageClick() {
var page = this._pageNumber;
var indexer = 1;
if (this.isLandscape) {
page = page - (page % 2);
indexer = 2;
}
page = page - indexer;
this._pageNumber = page;
},
_rightPageClick: function _rightPageClick() {
var page = this._pageNumber;
var indexer = 1;
if (this.isLandscape) {
page = page - (page % 2);
indexer = 2;
}
page = page + indexer;
this._pageNumber = page;
},
_keyUp: function _keyUp(event) {
var left = event.keyCode === 37;
var right = event.keyCode === 39;
if (left) {
this._leftPageClick();
}
else if (right) {
this._rightPageClick();
}
event.preventDefault();
},
_isLandscape: {
get: function get() {
var height = document.body.offsetHeight;
var width = document.body.offsetWidth;
return width > height;
}
},
_getOrientationClass: function _getOrientationClass() {
var isLandscape = this._isLandscape;
if (isLandscape) {
return "landscape";
}
else {
return "portrait";
}
},
_renderPage: function _renderPage(page) {
var div = document.createElement("div");
div.className = "magazinePageImageHolder";
var divChild = document.createElement("div");
divChild.className = "magazinePageImage";
div.appendChild(divChild);
var imageCard = new CommonJS.ImageCard(divChild, {
imageSource: {
url: page.image.Path,
cacheId: "MagazinesCache",
}
});
div.style.width = "100%";//this.scaledSnapWidth + "px";
div.setAttribute("data-page", page.pageNumber);
var that = this;
div.addEventListener("click", function (event) {
PlatformJS.Navigation.navigateToChannel("MagazineReader", { magazine: that.options.magazine, contentId: that.options.contentId, dataSource: that.options.panScanDataSource, pageNumber: page.pageNumber });
});
return WinJS.Promise.wrap(div);
},
onBindingComplete: function onBindingComplete() {
},
_onSizeChanged: function _onSizeChanged() {
var currentPageNumber = this._pageNumber;
this._updateViewport(this.dimensions, this.pages.length);
this._pageNumber = currentPageNumber;
},
onLayoutChange: function onLayoutChange(layoutMode) {
},
}
)
});
})();
/* File: MagazineParser.js */
/********************************************************
* *
* � Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsMagazineParserInit() {
"use strict";
WinJS.Namespace.define("CommonJS.Magazine",
{
MagazineParser: WinJS.Class.define(
function MagazineParser_ctor(state) {
},
{
dispose: function dispose() {
},
/// <summary>
/// initializes the page, also used when retrying after connectivity issues
/// </summary>
parse: function parse(magazine) {
if (magazine && magazine.ReplicaMap) {
//ops file
return this._parseOPSFile(magazine);
}
else if (magazine && magazine.gtl) {
return this._parseGTL(magazine);
}
else if (magazine && magazine.Pages) {
return this._parseCMSPages(magazine);
}
else if (magazine && magazine.Sections && magazine.Cover) {
return this._parseMagSpine(magazine);
}
else {
throw "Unsupported magazine object";
}
},
_parseMagSpine: function _parseMagSpine(mag) {
var magazine = {
title: mag.Title,
pubDate: mag.PubDate,
id: mag.Identifier,
pages: [{ pageNum: 0, image: mag.Cover }],
articles: [],
};
var parseArticle = function (art) {
var article = {
id: art.ContentId,
free: art.Free,
startPage: art.PageNum,
title: art.Headline,
byline: art.Byline,
abstract: art.Abstract,
thumbnail: art.Thumbnail ? art.Thumbnail : null
};
return article;
};
var sections = mag.Sections;
for (var s = 0; s < sections.length; s++) {
var section = sections[s];
var articles = section.Articles;
var l = articles.length;
for (var i = 0; i < l; i++) {
magazine.articles.push(parseArticle(articles[i]));
}
}
return magazine;
},
_parseCMSPages: function _parseCMSPages(mag) {
var pages = mag.Pages;
var magazine = {};
var that = this;
var parsePage = function (page) {
var pageResult = {};
pageResult.thumbnail = page.Images[1];
pageResult.image = page.Images[0];
pageResult.textRegions = that._parseTargets(page.Targets);
pageResult.pageNumber = page.PageNumber;
return pageResult;
};
magazine.pages = [];
for (var i = 0; i < pages.length; i++) {
pages[i].index = i;
magazine.pages.push(parsePage(pages[i]));
}
return magazine;
},
_parseTargets: function _parseTargets(regions) {
var textRegions = [];
for (var r = 0, l = regions.length; r < l; r++) {
var reg = regions[r];
var region = {
articleid: reg.ContentId,
blocks: [],
};
for (var b = 0, bLength = reg.Blocks.length; b < bLength; b++) {
var rect = reg.Blocks[b];
var rectArr = [rect.Top, rect.Left, rect.Bottom, rect.Right];
region.blocks.push(rectArr);
}
textRegions.push(region);
}
return textRegions;
},
//_parseGTL: function (mag) {
//    var map = mag.gtl.magazine;
//    var magazine = {
//        title: map.title,
//        creator: map.creator,
//        pubdata: map.pubdate,
//        publisher: map.publisher,
//        id: map.identifier
//    };
//    var processImage = function (img) {
//        var image = {
//            src: img.src,
//            height: img.height,
//            width: img.width,
//            url: img.ImageUrl
//        }
//        return image;
//    }
//    var parsePage = function (p) {
//        var page = {
//            pageNum: p["-pagenum"],
//            thumbnail: {},
//            image: {}
//        };
//        var full = p.images.image[0];
//        var thumb = p.images.image[1];
//        page.thumbnail = processImage(thumb);
//        page.image = processImage(full);
//        page.textRegions = [];
//        page.textRegions = this._parseTargets(p.targets);
//        return page;
//    }
//    magazine.pages = [];
//    var pages = map.pages.page;
//    for (var pg = 0, pl = pages.length; pg < pl; pg++) {
//        magazine.pages.push(parsePage(pages[pg]));
//    }
//    var parseArticle = function (art) {
//        var article = {
//            id: art["-id"],
//            free: art["-free"],
//            startPage: art["-page"],
//            title: art.headline,
//            byline: art.byline,
//            abstract: art.abstract,
//            thumbnail: art.thumbnail ? processImage(art.thumbnail.image) : null
//        }
//        return article;
//    }
//    magazine.articles = [];
//    var articles = map.sections.section.articles.article;
//    for (var a = 0, l = articles.length; a < l; a++) {
//        magazine.articles.push(parseArticle(articles[a], a));
//    }
//    magazine.tocs = magazine.articles;
//    return magazine;
//},
//_parseOPSFile: function (opsObject) {
//    var map = opsObject.ReplicaMap;
//    var fixPath = function (path) {
//        var regex = new RegExp("/", "gi");
//        var newPath = path.replace(regex, "\\");
//        return newPath;
//    }
//    var parseOPSPage = function (page) {
//        var magazinePage = {
//            image: fixPath(page["-file"]),
//            pageNumber: parseInt(page["-pageNum"]),
//            thumbnail: fixPath(page["-thumbFile"])
//        }
//        return magazinePage;
//    };
//    var parseOPSArticle = function (article) {
//        var magazineArticle = {
//            articleContentFile: article["-file"],
//            id: article["-id"],
//            title: article.title["#text"],
//            byline: article.byline,
//        }
//        return magazineArticle;
//    };
//    var parseOPStoc = function (toc) {
//        var tocEntry = {
//            title: toc["-title"],
//            summary: toc["Summary"],
//            startPage: parseInt(toc.Page["-pagenum"]),
//            articleID: toc.Article ? toc.Article["-refid"] : null,
//        }
//        return tocEntry;
//    };
//    var magazine = {};
//    magazine.title = map.Title;
//    magazine.pages = [];
//    var pages = map.Pages.Page;
//    for (var i = 0, l = pages.length; i < l; i++) {
//        magazine.pages.push(parseOPSPage(pages[i]));
//    }
//    magazine.articles = [];
//    var articles = map.Articles.Article;
//    for (var i = 0, l = articles.length; i < l; i++) {
//        magazine.articles.push(parseOPSArticle(articles[i]));
//    }
//    magazine.tocs = [];
//    var tocs = map.TOC.TocEntry;
//    for (var i = 0, l = tocs.length; i < l; i++) {
//        magazine.tocs.push(parseOPStoc(tocs[i]));
//    }
//    return WinJS.Promise.wrap(magazine);
//},
},
{
//static code
})
});
})();
/* File: MagazineReader.js */
/********************************************************
* *
* � Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsMagazineoReaderInit() {
"use strict";
WinJS.Namespace.define("CommonJS.Magazine", {
MagazineReader: WinJS.Class.define(
// Purpose is to initialize two parameters needed by the data json api.
function magazineReader_ctor(options) {
//todo, get from options
this.article = options.contentId;
this.dataSource = options.dataSource;
this.options = options;
this.host = document.querySelector("#magazineReader");
this.host.winControl = this;
var that = this;
this._onSizeChangedHandler = this._onSizeChanged.bind(this);
window.addEventListener("resize", this._onSizeChangedHandler);
this.parserPromise = this.getMagazineData(this.article);
CommonJS.setTheme(null, true);
},
{
getMagazineData: function getMagazineData(id) {
this.dataProvider = new PlatformJS.DataService.QueryService(this.dataSource);
var urlParams = PlatformJS.Collections.createStringDictionary();
var options = new Platform.DataServices.QueryServiceOptions();
urlParams["contentId"] = id;
var that = this;
return that.dataProvider.downloadDataAsync(urlParams, null, null, options).then(function (response) {
var result = JSON.parse(response.dataString);
return (new CommonJS.Magazine.MagazineParser().parse(result));
}, function (error) {
var e = error.message;
console.log(e);
}, function (progress) {
var p2 = progress;
});
},
isImmersive: false,
getPageState: function getPageState() { return this.options; },
_getFileContents: function _getFileContents(file) {
return this._getStorageFile(file).then(function (storageFile) {
return Windows.Storage.FileIO.readTextAsync(storageFile).then(function (fileContent) {
return fileContent;
},
function (error) {
console.log(error);
});
}, function (error) {
console.log(error);
});
},
_getStorageFile: function _getStorageFile(path) {
var folder = Windows.ApplicationModel.Package.current.installedLocation;
return Windows.Storage.StorageFolder.getFolderFromPathAsync(folder.path).then(function (folder) {
return folder.getFileAsync(path).then(function (file) {
return file; //IStorageFile
},
function (error) {
console.log(error);
});
},
function (error) {
console.log(error);
});
},
getPageData: function getPageData() {
var that = this;
return this.parserPromise.then(function (parsedMagazine) {
return that._renderMagazine(parsedMagazine);
});
},
_getImageUri: function _getImageUri(uri) {
return this._getStorageFile(uri).then(function (storageFile) {
var thumbnailUrl = URL.createObjectURL(storageFile, { oneTimeOnly: true });
return thumbnailUrl;
});
},
_getImageDimensions: function _getImageDimensions(uri) {
return this._getStorageFile(uri).then(function (storageFile) {
return storageFile.properties.getImagePropertiesAsync().then(function (properties) {
var imageData = { height: properties.height, width: properties.width };
return imageData;
});
});
},
_getArticles: function _getArticles() {
var articles = this.options.magazine.articles;
return articles;
},
_renderMagazine: function _renderMagazine(magazine) {
this.host.innerHTML = "";
this.magazine = magazine;
magazine.pages.shift(); //remove cover
var pages = magazine.pages;
this.pages = pages;
var that = this;
this.currentPage = 0;
var dimensions = { width: 1181, height: 1600 };
//   this._getImageDimensions(this.folder + "\\" + pages[0].image).then(function (dimensions) {
that._makeContainers();
that.dimensions = dimensions;
that._renderPages(pages);
that._updateViewport(this.dimensions, pages.length);
this.spread.scrollTop = document.body.clientHeight;
// this.drawRegions();
if (this.options.pageNumber) {
this._pageNumber = this.options.pageNumber;
}
},
zoomableView: {
get: function get() {
return this;
}
},
_renderToc: function _renderToc(tocs) {
var toc = this.toc;
toc.innerHTML = "";
var tocItems = document.createElement("div");
tocItems.className = "tocItems";
toc.appendChild(tocItems);
var onTocClicked = function (event) {
var page = parseInt(event.currentTarget.getAttribute("data-page"));
this._pageNumber = page + 1;
};
var tocClickBinding = onTocClicked.bind(this);
for (var i = 1; i < tocs.length; i++) {
var tocItem = tocs[i];
var tocDiv = document.createElement("div");
tocDiv.className = "tocItem";
tocDiv.innerHTML = toStaticHTML("<div class='tocIcon'>" + tocItem.startPage + "</div><div class='tocText'>" + tocItem.title + "</div>");
tocDiv.setAttribute("data-page", tocItem.startPage);
tocDiv.setAttribute("data-article", tocItem.articleID);
tocDiv.addEventListener("click", tocClickBinding);
tocItems.appendChild(tocDiv);
}
},
_renderPages: function _renderPages(pages) {
var l = pages.length;
var that = this;
var page = 0;
var renderPage = function (pageNum) {
pages[pageNum].pageNumber = pageNum;
that._renderPage(pages[pageNum]).then(function (div) {
that.panel.appendChild(div);
if (++pageNum < l) {
renderPage(pageNum);
}
});
};
renderPage(0);
},
_makeContainers: function _makeContainers() {
var host = this.host;
var spread = document.createElement("div");
spread.className = "spread";
host.appendChild(spread);
this.spread = spread;
var that = this;
that.zoomStd = true; //default
document.querySelector("#readingMode").addEventListener("click", this.viewCurrentArticleInReader.bind(this));
var setZoom = function (zoom) {
that.zoomedOut = false;
that.zoomedIn = false;
that.zoomStd = false;
if (zoom < 1) {
that.zoomedOut = true;
}
else if (zoom > 1) {
that.zoomedIn = true;
}
else {
that.zoomStd = true;
}
};
var onTouchEnd = function () {
var zoom = spread.msContentZoomFactor;
if (zoom === 1) {
WinJS.Utilities.removeClass(spread, "zoomed");
// that._pageNumber = that._pageNumber;
spread.style.msContentZoomLimitMax = "300%";
spread.style.msContentZoomLimitMin = "33%";
}
else if (zoom < 1) {
spread.style.msContentZoomLimitMax = "100%";
}
else if (zoom > 1) {
spread.style.msContentZoomLimitMin = "100%";
}
console.log(spread.style.msContentZoomLimitMin + "," + spread.style.msContentZoomLimitMax);
};
spread.addEventListener("MSManipulationStateChanged", function (event) {
var state =event.currentState;
var str = "stopped";
switch (state) {
case 0:
//stopped
onTouchEnd();
break;
case 1:
// str = "touching";
break;
case 2:
// str = "not touching, panning";
break;
}
//  console.log(str);
});
spread.addEventListener("MSLostPointerCapture", function (event) {
that._pointerId = null;
console.log("lost pointer");
});
spread.addEventListener("MSContentZoom", function (event) {
var zoom = spread.msContentZoomFactor;
zoom = Math.round(100 * zoom) / 100;
setZoom(zoom);
var newzoom = Math.round(100 * spread.msContentZoomFactor) / 100;
if (zoom !== newzoom) {
console.log("zoom has not stabilized, waiting");
wait();
return;
}
if (zoom !== 1) {
WinJS.Utilities.addClass(spread, "zoomed");
if (zoom < 1) {
WinJS.Utilities.addClass(spread, "zoomedout");
WinJS.Utilities.removeClass(spread, "zoomedin");
}
else {
//    spread.style.top = 0;
WinJS.Utilities.removeClass(spread, "zoomedout");
WinJS.Utilities.addClass(spread, "zoomedin");
}
}
else {
WinJS.Utilities.removeClass(spread, "zoomedout");
WinJS.Utilities.removeClass(spread, "zoomedin");
// spread.style.top = 0;
}
});
var magazinePanel = document.createElement("div");
magazinePanel.className = "magazinePanel";
spread.appendChild(magazinePanel);
this.panel = magazinePanel;
var toc = document.createElement("div");
toc.className = "magazineToc";
magazinePanel.appendChild(toc);
this.toc = toc;
this._boundKeyUp = this._keyUp.bind(this);
window.addEventListener("keydown", this._boundKeyUp);
this._makeFilmstrip(host);
},
viewCurrentArticleInReader: function viewCurrentArticleInReader() {
var page = this._pageNumber + 1;
var article = this.getArticleForPage(page);
if (article) {
this.viewArticleInArticleReader(article.id);
}
},
_makeFilmstrip: function _makeFilmstrip(host) {
var filmstrip = document.createElement("div");
filmstrip.className = "filmstrip";
this.filmstrip = filmstrip;
host.appendChild(filmstrip);
},
_updateViewport: function _updateViewport(dimensions, numPages) {
var magazinePanel = this.panel;
var host = this.spread;
var height = document.body.offsetHeight;
var width = document.body.offsetWidth;
var landscape = width > height;
var vRatio = height / dimensions.height;
var hRatio = (landscape ? width / 2 : width) / dimensions.width;
var ratio = Math.min(vRatio, hRatio);
var scaledSnapWidth = ratio * dimensions.width;
var snapInterval = scaledSnapWidth * (landscape ? 2 : 1);
var spreadSpacing = 0;
if (landscape) {
spreadSpacing = 30; //inter-spread spacing;
WinJS.Utilities.addClass(host, "landscape");
}
else {
WinJS.Utilities.removeClass(host, "landscape");
spreadSpacing = 0;
}
var snapStyle = "snapInterval(" + 0 + "px," + (snapInterval + spreadSpacing) + "px)";
host.style.msScrollSnapPointsX = snapStyle;
host.style.width = snapInterval + "px";
var viewportSize = {
width: ((scaledSnapWidth + spreadSpacing / 2) * (numPages + 1)),
height: Math.min(height, ratio * dimensions.height)
};
this.pageWidth = scaledSnapWidth + spreadSpacing / 2;
magazinePanel.style.height = viewportSize.height + "px";
magazinePanel.style.width = viewportSize.width + "px";
var children = magazinePanel.children;
for (var i = 0; i < children.length; i++) {
children[i].style.top = height + "px";
children[i].style.width = scaledSnapWidth + "px";
}
// this.toc.style.width = scaledSnapWidth + "px";
this.isLandscape = landscape;
},
_pageNumber: {
get: function get() {
var pageWidth = this.pageWidth;
var left = this.spread.scrollLeft;
var pageNumber = Math.round(left / pageWidth);
console.log("current page is " + pageNumber);
return pageNumber;
},
set: function set(value) {
var pageWidth = this.pageWidth;
if (this.isLandscape) {
value = value - (value % 2);
}
var newLeft = value * pageWidth;
console.log("scrolling to " + newLeft + " for page " + value + " with a pageWidth of " + pageWidth);
this.spread.scrollLeft = newLeft;
}
},
_leftPageClick: function _leftPageClick() {
var page = this._pageNumber;
var indexer = 1;
if (this.isLandscape) {
page = page - (page % 2);
indexer = 2;
}
page = page - indexer;
this._pageNumber = page;
},
_rightPageClick: function _rightPageClick() {
var page = this._pageNumber;
var indexer = 1;
if (this.isLandscape) {
page = page - (page % 2);
indexer = 2;
}
page = page + indexer;
this._pageNumber = page;
},
_keyUp: function _keyUp(event) {
var left = event.keyCode === 37;
var right = event.keyCode === 39;
if (left) {
this._leftPageClick();
}
else if (right) {
this._rightPageClick();
}
event.preventDefault();
},
_isLandscape: {
get: function get() {
var height = document.body.offsetHeight;
var width = document.body.offsetWidth;
return width > height;
}
},
_getOrientationClass: function _getOrientationClass() {
var isLandscape = this._isLandscape;
if (isLandscape) {
return "landscape";
}
else {
return "portrait";
}
},
_drawRegion: function _drawRegion(div, block) {
var dheight = div.offsetHeight;
var dwidth = div.offsetWidth;
var dleft = div.offsetLeft;
var dtop = div.offsetTop;
var region = document.createElement("div");
region.className = "RegionDiv";
region.style.position = "absolute";
var regionLeft = dleft + (dwidth * block[0]);
var regionWidth = (block[2] - block[0]) * dwidth;
var regionTop = dtop + (dheight * block[1]);
var regionHeight = (block[3] - block[1]) * dheight;
region.style.opacity = 0.5;
region.style.backgroundColor = "#666666";
region.style.top = regionTop + "px";
region.style.left = regionLeft + "px";
region.style.width = regionWidth + "px";
region.style.height = regionHeight + "px";
div.parentNode.appendChild(region);
return region;
},
drawRegions: function drawRegions() {
var pages = this.pages;
var divs = this.panel.children;
var infos = [];
for (var p = 0; p < pages.length; p++) {
var page = pages[p];
if (page.textRegions.length > 0) {
for (var q = 0; q < page.textRegions.length; q++) {
infos.push({ div: divs[page.pageNumber + 1], blocks: page.textRegions[q].blocks });
}
}
}
for (var i = 0; i < infos.length; i++) {
var div = infos[i].div;
var blocks = infos[i].blocks;
for (var j = 0; j < blocks.length; j++) {
this._drawRegion(div, blocks[j]); //test only
}
}
},
getRegionInfo: function getRegionInfo(pageNumber, div, coords) {
var height = div.offsetHeight;
var width = div.offsetWidth;
var articles = this.magazine.articles;
var pages = this.magazine.pages;
var currentPage = pages[pageNumber];
if (currentPage.textRegions.length === 0) {
//no article content on this page;
console.log("no articles on page " + pageNumber);
return;
}
var regions = currentPage.textRegions;
var clickScale = { x: coords.x / width, y: coords.y / height };
for (var i = 0, l = regions.length; i < l; i++) {
var region = regions[i];
for (var b = 0, bL = region.blocks.length; b < bL; b++) {
var block = region.blocks[b];
var clickX = clickScale.x;
var fitsX = (clickX > block[0] && clickX < block[2]);
if (fitsX) {
var clickY = clickScale.y;
var fitsY = clickY > block[1] && clickY < block[3];
if (fitsY) {
//inside region box
return region;
}
}
}
}
return null;
},
_getArticleForId: function _getArticleForId(id) {
var articles = this.magazine.articles;
for(var i = 0, l = articles.length; i < l;i++) {
var a = articles[i];
if (a.id === id) {
return a;
}
}
return null;
},
_renderPage: function _renderPage(page) {
var div = document.createElement("div");
div.className = "magazinePageImageHolder";
this.dimensions = { width: page.image.Width, height: page.image.Height }; //todo
div.style.top = document.body.clientHeight + "px";
div.setAttribute("data-page", page.pageNumber);
var divChild = document.createElement("div");
divChild.className = "magazinePageImage";
div.appendChild(divChild);
var imageCard = new CommonJS.ImageCard(divChild, {
imageSource: {
lowResolutionUrl: page.thumbnail.Path,
highResolutionUrl: page.image.Path,
cacheId: "MagazinesCache",
}
});
console.log("low/high: " + imageCard._imageSource.lowResolutionUrl + "/" + imageCard._imageSource.highResolutionUrl);
var that = this;
div.addEventListener("click", function (event) {
var page = parseInt(event.currentTarget.getAttribute("data-page"));
if (that.zoomedOut) {
//var currentZoom = that.spread.msContentZoomFactor;
//var endZoom = 1;
//var steps = 10;
//var height = document.body.clientHeight;
//var stepSize = (endZoom - currentZoom) / steps;
//var step = 0;
//var zoomIn = function () {
//    var newZoom = (++step) * stepSize + currentZoom;
//    that.spread.msContentZoomFactor = newZoom;
//    that.spread.scrollTop = step / steps * height;
//    if (newZoom < endZoom) {
//        setTimeout(zoomIn, 20);
//    }
//    else {
//      //  that._pageNumber = page;
//    }
//}
//zoomIn();
that.spread.msContentZoomFactor = 1;
that.spread.scrollTop = document.body.clientHeight;
that._pageNumber = page;
}
else if (that.zoomStd) {
var ri = that.getRegionInfo(page, event.currentTarget, { x: event.offsetX, y: event.offsetY });
if (ri) {
that.options.pageNumber = that._pageNumber;
that.viewArticleInArticleReader(ri.articleid.toString());
}
}
});
return WinJS.Promise.wrap(div);
//   });
},
getArticleForPage: function getArticleForPage(pageNum) {
var articles = this._getArticles();
var lastArticle = null;
for (var i = 0; i < articles.length; i++) {
var a = articles[i];
if (a.startPage < pageNum) {
lastArticle = articles[i];
}
else if (a.startPage === pageNum) {
return a;
}
if (a.startPage > pageNum && lastArticle !== null) {
return lastArticle;
}
}
return null;
},
getArticleIds: function getArticleIds() {
var ids = [];
var articles = this._getArticles();
for (var i = 0; i < articles.length; i++) {
ids.push({ articleId: articles[i].id.toString() });
}
return ids;
},
viewArticleInArticleReader: function viewArticleInArticleReader(articleId) {
var providerType = "AppEx.Common.ArticleReader.BedrockArticleProvider";
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
providerConfiguration.insert("queryServiceId", "MagazineBedrockArticles");
var otherIds = this.getArticleIds();
var infosStr = JSON.stringify({ articleInfos: otherIds });
providerConfiguration.insert("articleInfos", infosStr);
this._loadArticle(articleId.toString(), providerType, providerConfiguration);
},
_loadArticle: function _loadArticle(articleId, providerType, providerConfiguration) {
WinJS.Navigation.navigate({
fragment: "/common/ArticleReader/html/ArticleReaderPage.html",
page: "CommonJS.ArticleReaderPage"
}, {
providerType: providerType,
providerConfiguration: providerConfiguration,
initialArticleId: articleId,
enableSharing: true
});
},
onBindingComplete: function onBindingComplete() {
},
_onSizeChanged: function _onSizeChanged() {
var currentPageNumber = this._pageNumber;
this._updateViewport(this.dimensions, this.pages.length);
this._pageNumber = currentPageNumber;
},
onLayoutChange: function onLayoutChange(layoutMode) {
},
pageDataSource: {
get: function get() {
return this._pageDataSource;
},
set: function set(pages) {
var value = new WinJS.Binding.List(pages);
var groupedProjection = value.createGrouped(
function (item) {
return "A" + item.pageNumber;
},
function (item) {
return item;
},
function (left, right) {
return 0;
});
this._pageList = value;
this._pageDataSource = groupedProjection;
},
},
// IZoomable begins
beginZoom: function beginZoom() {
},
configureForZoom: function configureForZoom(isZoomedOut, isCurrentView, triggerZoom, prefectchedPages) {
this._isZoomedOut = isZoomedOut;
this._triggerZoom = triggerZoom;
},
endZoom: function endZoom(isCurrentView, setFocus) {
},
getCurrentItem: function getCurrentItem() {
var pos = {
left: 900,
top: 0,
width: 600,
height: 1200
};
var item = this.pageDataSource.groups.dataSource.list.getItem(this._pageNumber);
item.groupIndexHint = 0;
return WinJS.Promise.wrap({
item: item,
position: pos
});
},
getPanAxis: function getPanAxis() { return "horizontal"; },
handlePointer: function handlePointer(pointerId) {
// we don't set pointer capture so nested ListView get the proper event routing
},
positionItem: function positionItem(listItem, position) {
return WinJS.Promise.wrap({
x: 0,
y: 0
});
},
setCurrentItem: function setCurrentItem(x, y) {
},
// IZoomable ends
}
)
});
})();
/* File: ZoomedOutReader.js */
