﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: MediaPlayback.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsMediaPlaybackInit(WinJS) {
// JSCOPBUG: can't use strict because of the use of .calee property in this code
// "use strict";
var Animation = WinJS.UI.Animation;
var Utilities = WinJS.Utilities;
var Promise = WinJS.Promise;
var mouseLeftButton = 0;
var mousePointerType = 4;
var controlTemplateLocation = "/Common/MediaPlayback/html/MediaPlayback.html";
var MediaAppControls = WinJS.Namespace.define("CommonJS.MediaApp.Controls", {
PlaybackCapabilities: {
inlineOnly: "inlineOnly", /* default if unspecified */
fullScreenOnly: "fullScreenOnly",
normal: "normal"    /* Allow toggle in + out of FS */
},
PlaybackMode: {
full: "full",
partial: "partial",
minimal: "minimal"
},
PlaybackType: {
video: "video"
},
ControlState: {
rest: "Rest",
hover: "Hover",
pressed: "Pressed"
},
Control: {
play: "Play",
pause: "Pause",
gripper: "Gripper"
},
MediaPlayback: WinJS.Class.define(
function mediaPlayback_ctor(containerElement, options) {
/// construction
/// <summary>
/// Creates an instance of the MediaPlayback control.
/// </summary>
/// <param name='containerElement'>
/// The element that will host the playback control
/// </param>
/// <param name='options'>
/// </param>
/// <returns>
/// An instance of the mediaplayback control
/// </returns>
if (!containerElement) {
throw new Error("container element cannot be null or undefined");
}
if (this === window || this === MediaAppControls) {
var playback = containerElement.winControl;
if (playback) {
return playback;
} else {
return new MediaAppControls.MediaPlayback(containerElement, options);
}
}
// Attach the JS object to the host DOM element
if (!containerElement.winControl) {
containerElement.winControl = this;
}
this._mediaLibrary = null;
this._currentMediaSrcId = null;
this._currentMediaSrcProvider = null;
this._containerElement = containerElement;
this._type = MediaAppControls.PlaybackType.video;
this._mode = MediaAppControls.PlaybackMode.partial;
this._logFunction = function () { };
this._windowEventManager = CommonJS.WindowEventManager.getInstance();
var adSiteSectionId, pageRandomNumber;
var autoLoad = false;
if (options) {
if (options.type && MediaAppControls.PlaybackType[options.type]) {
this._type = options.type;
}
if (options.mode && MediaAppControls.PlaybackMode[options.mode]) {
this._mode = options.mode;
}
if (options.templateFragmentLocation) {
controlTemplateLocation = options.templateFragmentLocation;
}
if (options.timeFormat) {
this._timeFormat = options.timeFormat;
}
if (options.durationFormatter) {
this._durationFormatter = options.durationFormatter;
}
this._playButtonGlyph = options.playButtonGlyph || "&#xE102";
this._pauseButtonGlyph = options.pauseButtonGlyph || "&#xE103";
if (options.logFunction) {
this._logFunction = options.logFunction;
}
this._stateManager = options.stateManager;
this._setupEtw(options.etwProvider);
if (options.playbackCapabilities && MediaAppControls.PlaybackCapabilities[options.playbackCapabilities]) {
this._playbackCapabilities = options.playbackCapabilities;
} else {
this._playbackCapabilities = MediaAppControls.PlaybackCapabilities.normal;
}
if (options.width && options.height) {
this._containerElement.style.width = options.width;
this._containerElement.style.height = options.height;
}
// Ad configuration
if (options.adSiteSectionId) {
adSiteSectionId = options.adSiteSectionId;
}
if (options.pageRandomNumber) {
pageRandomNumber = options.pageRandomNumber;
}
}
//only supports type === MediaAppControls.PlaybackType.video
this._currentMediaElement = this._createVideoElement();
this._currentMediaElement.isVacant = true;
if (options && options.contentProtectionManager) {
// This has to be called after the elements are created
this._setupContentProtectionMonitor(options.contentProtectionManager);
}
this._instrMgrInstance = Platform.Instrumentation.InstrumentationManager.instance;
return null;
},
{
// Private fields. declaring these explicitly makes sure this is available in intellisense and to  
// make sure we dont endup creating private members accidentally with typos, and slightly different from the one we intended to use.
_playbackCapabilities: null,
_videoRequiresAds: false,   // Videos requiring ad pre-rolls are not supported. 
_overridePosterImageUrl: null,
_thumbSrc: null,
_srcPath: null,
_containerElement: null,
_currentMediaElement: null,
_inputHandlerElement: null,
_controlsElement: null,
_thumbElement: null,
_playbackControlTable: null,
_seekAndPlaybackControlTable: null,
_seekAndPlaybackControlTableSpacerLeft: null,
_seekAndPlaybackControlTableSpacerRight: null,
_seekAndPlaybackControlTableSpacerMiddle: null,
_playButton: null,
_type: null,
_mode: null,
_controlHideTimeout: null,
_restartTrackThreshold: 5,
_controlsAutoHideDuration: 3000,
_controlsAddedHideDuration: null,
_seekBarAddedHidedDuration: null,
_thumbGrabThreshold: 30,
_tapOffsetThreshold: 10,
_seekBarGranularitySteps: [30, 20, 20, 30],
_isControlsVisible: false,
_isThumbUp: false,
_oldThumbState: null,
_onStopping: false,
_timeFormat: null,
_timeIndicator: null,
_playToConnectionEventSubscriptions: [],
_playToDeviceFriendlyName: "",
_currentContentProtectionMonitor: null,
_currentMediaItem: null,
_miniHeroItemCounter: 0,
_mediaControl: Windows.Media.MediaControl,
_lastTimeDisplayAriaUpdate: Math.min(),
_msnVideosFormatCodes: [1004, 103, 102, 101, 1002],
_inFullScreenMode: false,
_canScrubVideo: true,
_instrumentationCallback: null,
_instrMgrInstance: null,
_instrDataSetID: 6,
_isNewInstr: true,
_instrIndex: -1,
_currentVideoAdsInterval: 0,
_videoAdsInterval: 5,
_adsConfig: null,
_userPreferenceVideoScreenSize: "auto", // auto - by default i.e. depending on resolution we will take care; The other TWO (50%/100%) will be set through UI by user
// [Video DCR]: Support Continuous video playback
_enableContinuousVideoPlayback: {
get: function get() {
return PlatformJS.Services.configuration.getBool("EnableContinuousVideoPlayback");
}
}, 
_shouldContinueNextVideo: false,
_countdownForNextVideo: 3,
_isCountdownUIShowing: false,
_nextContentSrc: null, 
_nextVideoOptions: null,
_nextVideoPlaybackButton: null,
_videoList: [],
_errorLog: function _errorLog(msg) {
var src = "Src_Not_set";
if (this._currentMediaSrcId) {
src = this._currentMediaSrcId;
}
if (this._isNewInstr) { //in some error situations, _instrVideoStart() is not called so we need to call addContent() here
this._instrIndex = PlatformJS.Navigation.mainNavigator.getCurrentImpression().addContent(
this._currentMediaSrcProvider,    // sourceName
null,                             // partnerCode
this._currentMediaSrcId,          // contentId
"video",                          // type
new Date(0),                      // date
null,                             // uri
this._topBarTitle.innerText,      // slug
false,                            // isSummary
null,                             // worth
false,                            // isAd
null                              // adCampaign
);
}
PlatformJS.Navigation.mainNavigator.getCurrentImpression().logContentError(
Platform.Telemetry.LogLevel.normal, //level
this._instrIndex,                   //k
"",                                 //uri
src + " " + msg,                      //message
0,                                  //httpCode
"",                                 //exception
null                                //timespan
);
console.error("Video Control: " + src + " " + msg);
},
_instrLog: function _instrLog(percentage, isNew) {
var src = "Src_Not_set";
if (this._currentMediaSrcId) {
src = this._currentMediaSrcId;
}
var jsonStr = "";
if (this._adsConfig) {
jsonStr = JSON.stringify({
adProvider: this._adsConfig.adProvider,
adServer: this._adsConfig.adServer
});
}
var progress = new Platform.Telemetry.ContentViewProgress();
progress.percent = percentage;
PlatformJS.Navigation.mainNavigator.getCurrentImpression().logContentViewWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,                 //level,
this._instrIndex,                                   //k,
Platform.Telemetry.ContentViewMechanism.unknown,    //viewMechanism,
!isNew,                                              //logOnlyIfNotViewed,
progress,                                           //progress
jsonStr                                             //jsonAttributes
);
console.log("Video Control: sending Appex Platform perf ping... :" + this._currentMediaSrcId + " percentage:" + percentage + " New_Instrumentation:" + isNew);
},
_instrVideoStart: function _instrVideoStart() {
if (this._isNewInstr) {
this._instrIndex = PlatformJS.Navigation.mainNavigator.getCurrentImpression().addContent(
this._currentMediaSrcProvider,    // sourceName
null,                             // partnerCode
this._currentMediaSrcId,          // contentId
"video",                          // type
new Date(0),                      // date
null,                             // uri
this._topBarTitle.innerText,      // slug
false,                            // isSummary
null,                             // worth
false,                            // isAd
null                              // adCampaign
);
}
this._instrLog(0, this._isNewInstr);
if (this._isNewInstr) {
this._isNewInstr = false; //only set is new for one video play one time at video start
}
if (this._instrumentationCallback) {
this._instrumentationCallback(0, this._currentMediaSrcId);
}
},
_instrVideo25: function _instrVideo25() {
this._instrLog(25, this._isNewInstr);
if (this._instrumentationCallback) {
this._instrumentationCallback(1, this._currentMediaSrcId);
}
},
_instrVideo50: function _instrVideo50() {
this._instrLog(50, this._isNewInstr);
if (this._instrumentationCallback) {
this._instrumentationCallback(2, this._currentMediaSrcId);
}
},
_instrVideo75: function _instrVideo75() {
this._instrLog(75, this._isNewInstr);
if (this._instrumentationCallback) {
this._instrumentationCallback(3, this._currentMediaSrcId);
}
},
_instrVideoComplete: function _instrVideoComplete() {
this._instrLog(100, this._isNewInstr);
if (this._instrumentationCallback) {
this._instrumentationCallback(4, this._currentMediaSrcId);
}
},
// Set up Content Protection Monitor
_setupContentProtectionMonitor: function _setupContentProtectionMonitor(ContentProtectionManager) {
this._currentContentProtectionMonitor = new ContentProtectionManager.ContentProtectionMonitor();
// There is no need to set the CPM again if the src changes.
// Just set it once here.
this._currentContentProtectionMonitor.setMediaElement(this._currentMediaElement);
},
// START SEQUENCER FUNCTIONS
_assignContentVideo: function _assignContentVideo(contentVideoObject) {
var that = this;
this._currentMediaElement.src = contentVideoObject.SourceUrl;
this._currentMediaElement.isVacant = false;
//if this is exeternal video, contentVideoObject might not contain poster image, 
if (contentVideoObject.PosterImageUrl) {
this._posterImage.src = contentVideoObject.PosterImageUrl;
}
},
_getEventSourceElement: function _getEventSourceElement() {
return this._containerElement;
},
_disableScrubbing: function _disableScrubbing() {
this._canScrubVideo = false;
Utilities.addClass(this._thumbElement, "platformHide");
},
_enableScrubbing: function _enableScrubbing() {
this._canScrubVideo = true;
Utilities.removeClass(this._thumbElement, "platformHide");
},
_assignAdClickthroughUrl: function _assignAdClickthroughUrl(url) {
var that = this;
Utilities.removeEventListener(this._mediaPlaybackAdClickthroughButton, "click");
if (url && url.length > 0) {
Utilities.addRemovableEventListener(this._mediaPlaybackAdClickthroughButton, "click", function mediaPlayback_onClickAd() { window.open(url, "_blank"); });
this._mediaPlaybackAdClickthroughButton.url = url;
} else {
this._mediaPlaybackAdClickthroughButton.url = null;
}
},
_hideAdClickthroughButton: function _hideAdClickthroughButton() {
Utilities.addClass(this._mediaPlaybackAdClickthroughContainer, "platformHide");
},
_showAdClickthroughButton: function _showAdClickthroughButton() {
Utilities.removeClass(this._mediaPlaybackAdClickthroughContainer, "platformHide");
},
_showContentVideoElement: function _showContentVideoElement() {
Utilities.removeClass(this._currentMediaElement, "platformHide");
this._hideAdClickthroughButton();
this._setupMediaElement(this._currentMediaElement);
},
// END SEQUENCER FUNCTIONS
_createMockMsnVideoLibrary: function _createMockMsnVideoLibrary() {
var that = this;
var library = {};
library.assignSrcUrlFromContentId = function (catalogUrlPrefix, videoId, overridePosterImageUrl) {
var xhr = WinJS.xhr({ url: catalogUrlPrefix + videoId, });
return xhr.then(function mediaPlayback_xhrComplete(xmlHttpRequest) {
var msnVidsResponse = xmlHttpRequest.responseXML;
if (!msnVidsResponse) {
return Promise.wrap(null);
}
var currentMsnVideosFormatCode = null;
that._videoSrcList = [];
for (var i = 0; i < that._msnVideosFormatCodes.length ; i++) {
var videoNode = msnVidsResponse.querySelector("video > videoFiles > videoFile[formatCode='" + that._msnVideosFormatCodes[i] + "'] > uri");
if (videoNode) {
if (!currentMsnVideosFormatCode) { //only keep the video format code for the 0th element of the list
currentMsnVideosFormatCode = that._msnVideosFormatCodes[i];
}
var sourceUrl = null;
if (videoNode.textContent) {
if (that._msnVideosFormatCodes[i] === 1004) {
sourceUrl = videoNode.textContent + "/manifest";
} else {
sourceUrl = videoNode.textContent;
}
that._videoSrcList.push(sourceUrl);
}
}
}
console.log("Video Control: the avaliable video source list is: " + that._videoSrcList);
var geoFenceNode = msnVidsResponse.querySelector("video > tags > tag[namespace='Geofence']");
if (geoFenceNode && geoFenceNode.textContent) {
that._geoFence = geoFenceNode.textContent;
}
else {
that._geoFence = "";
}
console.log("Video Control: current geofence string is " + that._geoFence);
console.log("Video Control: current format code is " + currentMsnVideosFormatCode);
if (that._userPreferenceVideoScreenSize === "auto") {
var hasHighResVideoFormat = false;
if (msnVidsResponse.querySelector("video > videoFiles > videoFile[formatCode='103'] > uri")) {
hasHighResVideoFormat = true;
console.log("Video Control: current video has a high resolution format, will play in fullscreen");
}
that._adjustVideoSizeToScreen(hasHighResVideoFormat);
}
else {
that._adjustVideoSizeToScreen(that._userPreferenceVideoScreenSize === "100%" ? true : false);
}
if (!that._videoSrcList || that._videoSrcList.length === 0) {
return Promise.wrap(null);
}
var posterNode = msnVidsResponse.querySelector("video > files > file[formatCode='2001'] >uri");
var pageGroupNode = msnVidsResponse.querySelector("video > pageGroup");
var MSNCatalogShowAd = pageGroupNode && !pageGroupNode.textContent.endsWith("NAX");
var providerIdNode = msnVidsResponse.querySelector("video > csId");
var partnerContentIdNode = msnVidsResponse.querySelector("video > providerId");
var sourceIdNode = msnVidsResponse.querySelector("video > source");
var providerId = providerIdNode ? providerIdNode.textContent : "";
var partnerContentId = partnerContentIdNode ? partnerContentIdNode.textContent : "";
var sourceId = sourceIdNode ? sourceIdNode.textContent : "";
var sourceFriendlyName = sourceIdNode && sourceIdNode.getAttribute ? sourceIdNode.getAttribute("friendlyName") : "";
var appEnableVideoAds = PlatformJS.Services.configuration.getBool("EnableVideoAds");
if (!appEnableVideoAds) {
console.warn("Video Control: App configuration disables video ads");
}
//adsConfig object definition
//  {
//    adProvider:               "ProviderName",
//    adStartFrequency:          2,
//    adRepeatFrequency:         3,
//    adServer:                  "mmppf", // "mmppf"|"legacy"|"mdialog"|"noads"
//    appID:                     <application id for the ad>, // could be null
//    adUnitId:                  "12345",
//  }
var adsConfig = null;
//Note: the name of provider could be in either providerId, or sourceFriendlyName, or sourceId
adsConfig = PlatformJS.Ads.getVideoAdsConfigByProvider(providerId);
if (!adsConfig) {
adsConfig = PlatformJS.Ads.getVideoAdsConfigByProvider(sourceFriendlyName);
}
if (!adsConfig) {
adsConfig = PlatformJS.Ads.getVideoAdsConfigByProvider(sourceId);
}
var controlShowVideoAds = false;
//count to see if we want to show video this time
if (appEnableVideoAds && adsConfig && adsConfig.adStartFrequency && adsConfig.adRepeatFrequency && adsConfig.adServer && adsConfig.appID && adsConfig.adUnitId) {
that._videoAdsInterval = adsConfig.adRepeatFrequency;
var videoAdsInterval = that._videoAdsInterval;
if (videoAdsInterval < 1) {
videoAdsInterval = 5; //default interval
}
var videoAdsPlayTimeAtInterval = adsConfig.adStartFrequency;
if (videoAdsPlayTimeAtInterval < 0 || videoAdsPlayTimeAtInterval >= videoAdsInterval) {
videoAdsPlayTimeAtInterval = 0;
}
if (that._currentVideoAdsInterval === videoAdsPlayTimeAtInterval) {
controlShowVideoAds = true;
console.log("Video Control: Show video ad, currentVideoAdsInterval=" + that._currentVideoAdsInterval);
} else {
console.log("Video Control: No video ad, currentVideoAdsInterval=" + that._currentVideoAdsInterval);
that._currentVideoAdsInterval = (that._currentVideoAdsInterval + 1) % videoAdsInterval;
}
}
else if (!adsConfig) { //fails to get adsConfig object
console.warn("invalid adsConfig object, will skip video ad playing");
}
// Prohibit videos from playing unless they are marked as not requiring ads
if (controlShowVideoAds && MSNCatalogShowAd && adsConfig && adsConfig.adUnitId && adsConfig.appID) {
that._videoRequiresAds = true;
} else {
that._videoRequiresAds = false;
}
var videoContentObject = {
ContentType: CommonJS.MediaApp.Controls.MediaContentType.VideoContent,
PingHandler: new CommonJS.MediaApp.Controls.MsnVideosInstrumentationHandler(videoId, currentMsnVideosFormatCode, "http://appex.msn.com"), //TODO: use app specific url definition for param 3
VideoUuid: videoId,
CanScrubVideo: true,
RequiresAds: that._videoRequiresAds,
partnerContentId: partnerContentId,
adsConfig: adsConfig ? adsConfig : null
};
if (that._videoSrcList && that._videoSrcList.length > 0) {
if (that._currentMediaElement) {
videoContentObject.SourceUrl = that._videoSrcList[0];
that._videoSrcListIndex = 0;
}
} else {
that._logFunction("MediaPlayback._createMockMsnVideoLibrary", "Failed while attempting to load content id {0}. Response was {1}".format(videoId, msnVidsResponse.xml));
videoContentObject = null;
}
// Don't assign the poster image from the MSN Vids catalog if explicitly specified
if (posterNode && !overridePosterImageUrl) {
if (that._posterImage && videoContentObject) {
videoContentObject.PosterImageUrl = posterNode.textContent;
}
}
else if (overridePosterImageUrl) {
videoContentObject.PosterImageUrl = overridePosterImageUrl;
}
if (videoContentObject) {
that._sequencer.AddContent(videoContentObject);
}
return Promise.wrap(videoContentObject);
}, function (error) {
Promise.wrap(null);
that._errorLog("MSNcatalog video fetching error: " + error.responseText + " URL: " + catalogUrlPrefix + videoId);
});
};
return library;
},
_adjustVideoSizeToScreen: function _adjustVideoSizeToScreen(shouldFitToScreen) {
if (!shouldFitToScreen) {
console.log("Video Control: use 50% screen size");
this._currentMediaElement.visual.style.width = "50%";
this._currentMediaElement.visual.style.height = "50%";
this._mediaStrip.style.paddingTop = "25%";
this._mediaStrip.style.textAlign = "center";
Utilities.addClass(this._screenSizeToggleButton, "fullScreenSizeToggleButton");
Utilities.removeClass(this._screenSizeToggleButton, "halfScreenSizeToggleButton");
} else {
this._currentMediaElement.visual.style.width = "100%";
this._currentMediaElement.visual.style.height = "100%";
this._mediaStrip.style.paddingTop = "";
this._mediaStrip.style.textAlign = "";
Utilities.addClass(this._screenSizeToggleButton, "halfScreenSizeToggleButton");
Utilities.removeClass(this._screenSizeToggleButton, "fullScreenSizeToggleButton");
}
},
// Creates a new video element
_createVideoElement: function _createVideoElement() {
var videoElementContainer = document.createElement("div");
Utilities.setInnerHTMLUnsafe(videoElementContainer, "<video preload='auto' />");
var videoElement = videoElementContainer.firstChild.cloneNode(true);
Utilities.addClass(videoElement, "mediaPlaybackVideo");
videoElement.visual = videoElement;
videoElement._mediaEventSubscriptions = [];
videoElement._posterEventSubscriptions = [];
return videoElement;
},
// setup etw with default noop handlers if no real provider is passed
_setupEtw: function _setupEtw(etwProvider) {
if (etwProvider) {
this._etw = etwProvider;
} else {
var defaultEtw = {};
defaultEtw["mediaPlaybackSliderUpdateStart"] = function mediaPlayback_defaultMediaPlaybackSliderUpdateStart() { };
defaultEtw["mediaPlaybackSliderUpdateEnd"] = function mediaPlayback_defaultMediaPlaybackSliderUpdateEnd() { };
defaultEtw["mediaPlaybackSeekStart"] = function mediaPlayback_defaultMediaPlaybackSeekStart() { };
defaultEtw["mediaPlaybackSeekEnd"] = function mediaPlayback_defaultMediaPlaybackSeekEnd() { };
defaultEtw["mediaPlaybackPlayStart"] = function mediaPlayback_defaultMediaPlaybackPlayStart() { msWriteProfilerMark("Common:MediaPlayback:mediaPlaybackPlayStart"); };
defaultEtw["mediaPlaybackPlayEnd"] = function mediaPlayback_defaultMediaPlaybackPlayEnd() { msWriteProfilerMark("Common:MediaPlayback:mediaPlaybackPlayEnd"); };
defaultEtw["mediaPlaybackPauseStart"] = function mediaPlayback_defaultMediaPlaybackPauseStart() { msWriteProfilerMark("Common:MediaPlayback:mediaPlaybackPauseStart"); };
defaultEtw["mediaPlaybackPauseEnd"] = function mediaPlayback_defaultMediaPlaybackPauseEnd() { msWriteProfilerMark("Common:MediaPlayback:mediaPlaybackPauseEnd"); };
defaultEtw["mediaPlaybackShowControlsStart"] = function mediaPlayback_defaultMediaPlaybackShowControlsStart() { };
defaultEtw["mediaPlaybackShowControlsEnd"] = function mediaPlayback_defaultMediaPlaybackShowControlsEnd() { };
defaultEtw["mediaPlaybackHideControlsStart"] = function mediaPlayback_defaultMediaPlaybackHideControlsStart() { };
defaultEtw["mediaPlaybackHideControlsEnd"] = function mediaPlayback_defaultMediaPlaybackHideControlsEnd() { };
defaultEtw["mediaPlaybackSrcSet"] = function mediaPlayback_defaultMediaPlaybackSrcSet() { msWriteProfilerMark("Common:MediaPlayback:mediaPlaybackSrcSet"); };
defaultEtw["mediaPlaybackMediaEnd"] = function mediaPlayback_defaultMediaPlaybackMediaEnd() { msWriteProfilerMark("Common:MediaPlayback:mediaPlaybackMediaEnd"); };
this._etw = defaultEtw;
}
},
// Remove the auto hide timer for hiding controls
_removeControlsTimer: function _removeControlsTimer() {
if (this._controlHideTimeout) {
clearTimeout(this._controlHideTimeout);
this._controlHideTimeout = null;
}
},
// Start an auto hide timer for hiding controls
_setControlsTimer: function _setControlsTimer() {
var that = this;
this._removeControlsTimer();
this._controlHideTimeout = setTimeout(function mediaPlayback_afterSetControlsTimer() {
// Reset the timer with an added duration if we've moved the mouse since the last reset, otherwise hide controls
if (that._lastPointerMoveTimeStamp && that._lastPointerMoveTimeStamp > that._lastControlsResetTimeStamp) {
var currentHideDuration = that._controlsAddedHideDuration || that._controlsAutoHideDuration;
that._controlsAddedHideDuration = that._controlsAutoHideDuration - (that._lastControlsResetTimeStamp + currentHideDuration - that._lastPointerMoveTimeStamp);
that._removeControlsTimer();
that._setControlsTimer();
that._controlsAddedHideDuration = null;
} else {
that._hideControls();
}
}, this._controlsAddedHideDuration || this._controlsAutoHideDuration);
this._lastControlsResetTimeStamp = Date.now();
},
// Hide the controls bar, this will initiate an slideout + fadeout animation immediately.
_hideControls: function _hideControls(isInternal) {
/// <param name="isInternal" optional="true"/>
this._etw.mediaPlaybackHideControlsStart();
this._removeControlsTimer();
var that = this;
if (!Utilities.hasClass(this._topBar, "platformHide") && Utilities.hasClass(this._errorScreen, "platformHide")) {
Animation.fadeOut(this._fullScreenToggleButton);
Animation.fadeOut(this._topBar).done(function topBarFadeOutComplete() {
Utilities.addClass(that._topBar, "platformHide");
that._topBarShowed = false;
});
}
Animation.fadeOut(this._controlsElement)
.then(function mediaPlayback_fadeOutComplete() {
that._isControlsVisible = false;
Utilities.addClass(that._controlsElement, "hiddenObject");
});
if (!isInternal) {
this._dispatchEvent("hidecontrols");
}
this._lastPointerPosition = null;
this._etw.mediaPlaybackHideControlsEnd();
},
// Show the controls bar, controls bar will slide/fade in, but will auto hide automatically.
_showControls: function _showControls(forceShow, skipAutoHide) {
/// <param name="forceShow" optional="true"/>
/// <param name="skipAutoHide" optional="true"/>
if (!this._inFullScreenMode && this._isAdPlaying()) {
return; //do not show control during video Ad playing in inlineplayback mode
}
if (this._isControlsVisible && !forceShow) {
return;
}
this._removeControlsTimer();
if (Utilities.hasClass(this._topBar, "platformHide")) {
var that = this;
Utilities.removeClass(that._topBar, "platformHide");//if this removeClass called in done() block, there will be no animation when using touch (mouse will still work)
Animation.fadeIn(this._fullScreenToggleButton);//now backbutton is a seperate component from topbar
Animation.fadeIn(this._topBar).done(function showTopBarComplete() {
that._topBarShowed = true;
});
}
if (!skipAutoHide) {
this._setControlsTimer();
}
if (!WinJS.Utilities.hasClass(this._waitingSpinnerContainer, "platformHide")) {
return; //do not show playButton during loading
}
if (this._videoRequiresAds && this._adStatus !== CommonJS.MediaApp.Controls.AdStates.AdCompleted) {
return; //do not show playButton during video Ad playing
}
if (!Utilities.hasClass(this._errorScreen, "platformHide")) {
return; //do not show playButton during errorScreen
}
this._etw.mediaPlaybackShowControlsStart();
Utilities.removeClass(this._controlsElement, "hiddenObject");
if (this._isThumbUp) {
this._rotateThumbDown();
}
this._isControlsVisible = true;
this._controlsElement.style.opacity = 1;
this._dispatchEvent("showcontrols");
this._etw.mediaPlaybackShowControlsEnd();
},
// Returns true if the seek bar is curently visible
_isSeekBarVisible: {
get: function get() {
if (this._canScrubVideo) {
if (this._thumbElement.currentStyle) {
return this._thumbElement.currentStyle.opacity !== "0";
} else {
//TODO: in the case of exiting in Article Reader, this function will be called when currentStyle is null, find better fix than if/else
return false;
}
} else {
if (this._seekBar.currentStyle) {
return this._seekBar.currentStyle.opacity !== "0";
} else {
return false;
}
}
}
},
// Removes the current auto hide timer for hiding the seek bar
_removeSeekBarTimer: function _removeSeekBarTimer() {
if (this._seekBarHideTimeout) {
clearTimeout(this._seekBarHideTimeout);
this._seekBarHideTimeout = null;
}
},
// Start an auto hide timer for hiding seekbar
_setSeekBarTimer: function _setSeekBarTimer() {
var that = this;
this._removeSeekBarTimer();
this._seekBarHideTimeout = setTimeout(function mediaPlayback_afterSeekBarHideTimeout() {
// Reset the timer with an added duration if we've moved the mouse since the last reset, otherwise hide seekbar
if (that._lastPointerMoveTimeStamp && that._lastPointerMoveTimeStamp > that._lastSeekBarResetTimeStamp) {
var currentHideDuration = that._seekBarAddedHideDuration || that._controlsAutoHideDuration;
that._seekBarAddedHideDuration = that._controlsAutoHideDuration - (that._lastSeekBarResetTimeStamp + currentHideDuration - that._lastPointerMoveTimeStamp);
that._removeSeekBarTimer();
that._setSeekBarTimer();
that._seekBarAddedHideDuration = null;
} else {
that._hideSeekBar();
}
}, this._seekBarAddedHideDuration || this._controlsAutoHideDuration);
this._lastSeekBarResetTimeStamp = Date.now();
},
// Hide the seek bar, this will initiate an fadeout animation immediately.
_hideSeekBar: function _hideSeekBar() {
var that = this;
if (this._isSeekBarVisible) {
this._removeSeekBarTimer();
Animation.fadeOut(this._thumbElement);
var promise = Animation.fadeOut(this._seekBar);
promise.then(function mediaPlayback_fadeOutSeekBarComplete() {
that._rotateThumbDown();
});
Animation.fadeOut(this._timeIndicator);
Animation.fadeOut(this._currentTimeIndicator);
Animation.fadeOut(this._totalTimeIndicator);
this._backdrop.style.opacity = 0;
}
},
// Show the seek bar, seek bar will fade in, but will auto hide automatically.
_showSeekBar: function _showSeekBar(forceShow, skipAutoHide) {
/// <param name="forceShow" optional="true"/>
/// <param name="skipAutoHide" optional="true"/>
if (this._videoRequiresAds && this._adStatus !== CommonJS.MediaApp.Controls.AdStates.AdCompleted) {
return; //do not show seekbar during video Ad playing in inlineplayback mode
}
if (this._isSeekBarVisible && !forceShow) {
return;
}
if (!WinJS.Utilities.hasClass(this._errorScreen, "platformHide")) {
return;
}
//need to hide seekbar when loading, bug 338083 shows that if user seek before onplaying event being fired,
//we will NOT get onplaying event, which will trigger more problems since _currentState is not correctly set
if (!WinJS.Utilities.hasClass(this._waitingSpinnerContainer, "platformHide")) {
return;
}
if (this._seekBarHideTimeout) {
this._removeSeekBarTimer();
}
if (!this._isSeekBarVisible) {
this._syncTimeAndProgress(true);
if (this._canScrubVideo) {
Animation.fadeIn(this._thumbElement);
}
Animation.fadeIn(this._seekBar);
Animation.fadeIn(this._timeIndicator);
Animation.fadeIn(this._currentTimeIndicator);
Animation.fadeIn(this._totalTimeIndicator);
this._backdrop.style.opacity = 1;
}
if (!skipAutoHide) {
this._setSeekBarTimer();
}
},
// This is a helper to get an element by id, within the container element
_getChildElementById: function _getChildElementById(childId) {
var theParent = this._containerElement;
var children = theParent.getElementsByTagName("*");
for (var index = children.length - 1; index >= 0; index--) {
if (children[index].id === childId) {
return children[index];
}
}
return null;
},
// Get offsetX of an element relative to immediate or non-immediate parent element.
_getRelativeOffsetX: function _getRelativeOffsetX(parentElement, element) {
var offSetX = 0;
while (element && parentElement !== element) {
offSetX += element.offsetLeft;
element = element.offsetParent;
}
return offSetX;
},
_dispatchEvent: function _dispatchEvent(eventName, payload) {
// TODO: completely remove _dispatchEvent if no further issue is found
},
// This initializes the control's parts after the fragment containing the controls template is loaded
_init: function _init() {
console.log("Video Control: entered init");
this._thumbSrc = this._getChildElementById("mediaPlaybackGripperHarvest").src;
this._srcPath = this._thumbSrc.substring(0, this._thumbSrc.lastIndexOf("/"));
this._contentElement = this._getChildElementById("mediaPlaybackContent");
this._controlsElement = this._getChildElementById("mediaPlaybackControls");
this._controlsLayerElement = this._getChildElementById("mediaPlaybackControlsLayer");
this._thumbElement = this._getChildElementById("mediaPlaybackSeekBarThumb");
this._thumbElementToolTip = this._getChildElementById("mediaPlaybackSeekBarThumbToolTip").firstChild;
this._seekBar = this._getChildElementById("mediaPlaybackSeekBar");
this._seekBarRail = this._getChildElementById("mediaPlaybackSeekBarRail");
this._progress = this._getChildElementById("mediaPlaybackSeekBarProgress");
this._playbackControlTable = this._getChildElementById("playbackControlTable");
this._seekAndPlaybackControlTable = this._getChildElementById("seekAndPlaybackControlTable");
this._seekAndPlaybackControlTableSpacerLeft = this._getChildElementById("seekAndPlaybackControlTableSpacerLeft");
this._seekAndPlaybackControlTableSpacerRight = this._getChildElementById("seekAndPlaybackControlTableSpacerRight");
this._seekAndPlaybackControlTableSpacerMiddle = this._getChildElementById("seekAndPlaybackControlTableSpacerMiddle");
this._playButton = this._getChildElementById("mediaPlaybackPlayButton");
this._screenSizeToggleButton = this._getChildElementById("screenSizeToggleButton");
this._inputHandlerElement = this._getChildElementById("mediaPlaybackInputHandler");
this._onPlayingEventFired = false;
this._timeIndicator = this._getChildElementById("mediaPlaybackTimeIndicator");
this._currentTimeIndicator = this._getChildElementById("mediaPlaybackCurrentTimeIndicator");
this._totalTimeIndicator = this._getChildElementById("mediaPlaybackTotalTimeIndicator");
this._fullScreenToggleButton = this._getChildElementById("mediaPlaybackFullScreenToggleButton");
this._fullScreenControl = CommonJS.FullScreen;
this._navigationControl = WinJS.Navigation;
this._posterImageElement = this._getChildElementById("mediaPlaybackPosterImageContainer");
this._posterImage = this._getChildElementById("mediaPlaybackPosterImage");
this._loadingScreen = this._getChildElementById("mediaPlaybackPosterImagePlaceholder");
this._waitingSpinnerContainer = this._getChildElementById("mediaPlaybackSpinnerContainer");
this._waitingSpinnerVisible = false;
this._message = this._getChildElementById("mediaPlaybackMessage");
this._errorScreen = this._getChildElementById("mediaPlaybackErrorScreen");
this._topBar = this._getChildElementById("mediaPlaybackTopBar");
this._topBarCurrentTime = this._getChildElementById("topBarCurrentTime");
this._topBarTotalTime = this._getChildElementById("topBarTotalTime");
this._topBarTitle = this._getChildElementById("topBarTitle");
this._topBarAttribution = this._getChildElementById("topBarAttribution");
this._topBarLogo = this._getChildElementById("topBarLogo");
this._mediaPlaybackAdClickthroughContainer = this._getChildElementById("mediaPlaybackAdClickthroughContainer");
this._mediaPlaybackAdClickthroughButton = this._getChildElementById("mediaPlaybackAdClickthroughButton");
this._adControl = null;
this._legacyAdControl = false;
this._videoAd = this._getChildElementById("mediaPlaybackVideoAd");
this._geoFence = "";
this._disableVideoForGeoFence = false;
// Reset media source provider
this._currentMediaSrcProvider = null;
this._resetVidSrc = "/Common/MediaPlayback/Resources/black.mp4";
this._videoAdsInterval = 5;
this._adsConfig = null;
this._isNewInstr = true;
this._instrIndex = -1;
//following intructions from http://blogs.iis.net/cenkd/archive/2012/03/28/How-to-build-your-first-html5-metro-style-smooth-streaming-player.aspx
//                           http://blogs.iis.net/cenkd/archive/2012/08/09/how-to-build-a-smooth-streaming-windows-8-javascript-application-with-advanced-features.aspx
if (!CommonJS.MediaApp.Controls.ssPlugin) {
console.warn("Video Control: creating new SmoothStreaming object, only expect to see this once");
var ssPlugin = new Windows.Media.MediaExtensionManager();
var mgr = Microsoft.Media.AdaptiveStreaming.AdaptiveSourceManager.getDefault();
var property = new Windows.Foundation.Collections.PropertySet();
mgr.addEventListener("manifestreadyevent", function onManifestReadey(e) {
console.log("Video Control: in Smooth Streaming Manifestreadyevent");
try {
var manifest = e.adaptiveSource.manifest;
var availableStreams = manifest.availableStreams;
for (var i = 0; i < availableStreams.length; i++) {
var stream = availableStreams[i];
var tracks = stream.availableTracks;
var currentSelectedTracks = [];
var k = 0;
var networkStatus = Platform.Networking.NetworkManager.instance.networkCostSuggestion;
if (stream.type === Microsoft.Media.AdaptiveStreaming.MediaStreamType.video) {
if (Platform.Networking.NetworkCostSuggestion.standard === networkStatus) {
//select 1st and 2nd high resolution video for high speed internet 
var highestResIndex = 0;
var secondaryResIndex = -1;
var currentMaxWidth = 0;
for (k = 0; k < tracks.length; k++) {
if (tracks[k].maxWidth > currentMaxWidth) {
currentMaxWidth = tracks[k].maxWidth;
secondaryResIndex = highestResIndex;
highestResIndex = k;
}
}
if (secondaryResIndex !== -1 && secondaryResIndex !== highestResIndex) {
currentSelectedTracks.push(tracks[secondaryResIndex]);
}
currentSelectedTracks.push(tracks[highestResIndex]);
stream.selectTracks(currentSelectedTracks);
} else { //remove the lowest track for all other situations
var currentMinWidth = Number.MAX_VALUE; //just a very large number;
var minIndex = 0;
for (k = 0; k < tracks.length; k++) {
if (tracks[k].maxWidth < currentMinWidth) {
currentMinWidth = tracks[k].maxWidth;
minIndex = k;
}
}
for (k = 0; k < tracks.length; k++) { //select all tracks except for the lowest res track
if (k !== minIndex) {
currentSelectedTracks.push(tracks[k]);
}
}
if (currentSelectedTracks.length > 0) { //will hit length === 0 when there's only one track in the stream
stream.selectTracks(currentSelectedTracks);
}
}
} 
}
}
catch (error) {
console.warn("Video Control: onManifestReadey(): " + error.message.toString());
}
}, false);
property["{A5CE1DE8-1D00-427B-ACEF-FB9A3C93DE2D}"] = mgr;
ssPlugin.registerByteStreamHandler("Microsoft.Media.AdaptiveStreaming.SmoothByteStreamHandler", ".ism", "text/xml", property);
ssPlugin.registerByteStreamHandler("Microsoft.Media.AdaptiveStreaming.SmoothByteStreamHandler", ".ism", "application/vnd.ms-ss", property);
CommonJS.MediaApp.Controls.ssPlugin = ssPlugin;
}
if (!this._posterImageElement) {
console.log("Video Control: _init():: PosterImageElement not found. Cannot initialize the video control. Throwing...");
throw "Unable to initialize video playback.";
}
if ((this._playbackCapabilities !== MediaAppControls.PlaybackCapabilities.inlineOnly) && (this._fullScreenControl)) {
this._fullScreenToggleButton.style.display = "inline-block";
}
this._backdrop = this._getChildElementById("mediaPlaybackControlsBackdrop");
// Create the sequencer prior to setting up event listeners for 
// the video controls, as it listens to these events
this._sequencer = new CommonJS.MediaApp.Controls.MediaSequencer(this);
this._mediaStrip = this._getChildElementById("mediaPlaybackMediaStrip");
this._setupMediaElement(this._currentMediaElement);
this._mediaStrip.appendChild(this._currentMediaElement.visual);
this._setupMediaStrip();
this._updateControls();
this._showPlayButton();
var that = this;
// Handle focus for accessibility
Utilities.addRemovableEventListener(this._playButton, "blur", function mediaPlayback_onBlurPlayButton(event) { that.resetAutoHideTimers(true); }, false);
Utilities.addRemovableEventListener(this._fullScreenToggleButton, "blur", function mediaPlayback_onBlurFullScreenButton(event) { that.resetAutoHideTimers(true); }, false);
Utilities.addRemovableEventListener(this._thumbElement, "focus", function mediaPlayback_onFocusThumb(event) { that._showControls(true, false); that._showSeekBar(true, false); }, false);
Utilities.addRemovableEventListener(this._thumbElement, "blur", function mediaPlayback_onBlurThumb(event) { that.resetAutoHideTimers(true); }, false);
// Handle keystrokes for accessibility
Utilities.addRemovableEventListener(this._playButton, "keydown", function mediaPlayback_onKeyDownPlayButton(event) { that._stopEventPropagation(event); that._onInputHandlerKeyDown(event); }, false);
Utilities.addRemovableEventListener(this._playButton, "keyup", function mediaPlayback_onKeyUpPlayButton(event) { that._stopEventPropagation(event);that._onInputHandlerKeyUp(event); }, false);
Utilities.addRemovableEventListener(this._fullScreenToggleButton, "keydown", function mediaPlayback_onKeyDownFullScreenButton(event) { that._stopEventPropagation(event); that._onInputHandlerKeyDown(event); }, false);
Utilities.addRemovableEventListener(this._fullScreenToggleButton, "keyup", function mediaPlayback_onKeyUpFullScreenButton(event) { that._stopEventPropagation(event);that._onInputHandlerKeyUp(event); }, false);
Utilities.addRemovableEventListener(this._thumbElement, "keydown", function mediaPlayback_onKeyDownThumb(event) { that._onInputHandlerKeyDown(event); }, false);
Utilities.addRemovableEventListener(this._thumbElement, "keyup", function mediaPlayback_onKeyUpThumb(event) { that._onInputHandlerKeyUp(event); }, false);
Utilities.addRemovableEventListener(this._playButton, "click", function mediaPlayback_onClickPlayButton(event) {
if (event.offsetX < 0 || event.offsetY < 0) {
return;
}
that.resetAutoHideControlsTimer();
that._onPlayPauseToggleCommand();
that._stopEventPropagation(event);
that._thumbElement.focus();
}, false);
Utilities.addRemovableEventListener(this._screenSizeToggleButton, "click", function mediaPlayback_onClickScreenSizeToggleButton(event) {
that._toggleScreenSize(event);
that._thumbElement.focus();
}, false);
Utilities.addRemovableEventListener(this._screenSizeToggleButton, "keydown", function mediaPlayback_onKeyDownScreenSizeToggleButton(event) { that._stopEventPropagation(event); that._onInputHandlerKeyDown(event); }, false);
Utilities.addRemovableEventListener(this._screenSizeToggleButton, "keyup", function mediaPlayback_onKeyUpScreenSizeToggleButton(event) { that._stopEventPropagation(event); that._onInputHandlerKeyUp(event); }, false);
var mediaPlayback_onClickFullScreenButton = function mediaPlayback_onClickFullScreenButton(event) {
that.resetAutoHideControlsTimer();
that._toggleFullScreen();
that._stopEventPropagation(event);
that._thumbElement.focus();
};
if (document) {
Utilities.addRemovableEventListener(document, "msvisibilitychange", function mediaPlayback_onVisibilityChange(event) {
if (document) {
// Get hidden flag
var isHidden = document.msHidden;
// If document is now hidden
if (isHidden) {
if ( that._sequencer && (!that._sequencer.IsPaused())) {
that._resumePlay = true;
that.pause();
console.log("Video Control: pause video on suspending");
} 
}
else if (that._resumePlay) {
that._resumePlay = false;
that.play();
console.log("Video Control: play video on resuming");
}
}
}, false);
}
var navControl = document.getElementById("platformNavigationBar").winControl;
if (navControl) {
Utilities.addRemovableEventListener(navControl,"beforeshow", function beforeShowNav() {
if (that._sequencer && (!that._sequencer.IsPaused())) {
that._resumePlay = true;
that.pause();
console.log("Video Control: pause video on Appbar show");
}
}, false);
Utilities.addRemovableEventListener(navControl, "beforehide", function beforeHideNav() {
if (that._resumePlay && (!document.msHidden)) {
that._resumePlay = false;
that.play();
console.log("Video Control: play video on Appbar hide");
}
}, false);
}
Utilities.addRemovableEventListener(this._fullScreenToggleButton, "click", mediaPlayback_onClickFullScreenButton, false);
Utilities.addRemovableEventListener(this._inputHandlerElement, "click", function mediaPlayback_onInputHandlerClick(event) {
if (that._message && that._message.innerText && that._message.innerText !== "") {
that._message.innerText = "";
that.load().then(function reloadComplete(event) {
if (that._currentMediaSrcId && that._currentMediaSrcId.indexOf("http://") >= 0) {
that.src = that._currentMediaSrcId;
} else {
return that.loadContentId(that._currentMediaSrcId);
}
})
.done(function reloadPlayerPlayVideo() {
that._resetControlsElementState();
that.play();
});
}
}, false);
this._onSamePageNavBinding = this._onSamePageNav.bind(this);
if (this._windowEventManager) {
Utilities.addRemovableEventListener(this._windowEventManager, "samePageNav", this._onSamePageNavBinding);
}
Utilities.addRemovableEventListener(this._navigationControl, "navigating", this._onSamePageNavBinding);
Utilities.addRemovableEventListener (this._controlsElement, "mousemove", function mediaPlayback_onMouseMove(event) {
that._onControlsElementMouseMove();
that._stopEventPropagation(event);
}, true);
Utilities.addRemovableEventListener(this._thumbElement, "resize", function mediaPlayback_onResizeThumb(event) { that._syncTimeAndProgress(); }, false);
//_fullScreenControl is shared by all video instances and we do not want duplicated listeners
Utilities.addRemovableEventListener(this._fullScreenControl, "FullScreenEnter", function mediaPlayback_onFullScreenEnter(event) { that._sequencer._onEnteredFullScreen(event); });
Utilities.addRemovableEventListener(this._fullScreenControl, "FullScreenExit", function mediaPlayback_onFullScreenExit(event) { that._sequencer._onExitedFullScreen(event); });
this._startOffsetX = 0;
this._lastGranularity = 1;
this._hasBeenInitialized = true;
this._isPointerDown = false;
// [Video DCR]: Support Continuous video playback
// Init Countdown UI
this._shouldContinueNextVideo = this._shouldContinueNextVideo && window.navigator.onLine; // Connection may lost while during video play. Then, we should not show the countdown popup
if (this._shouldContinueNextVideo) {
this._mediaPlaybackCountdownUI = this._getChildElementById("mediaPlaybackCountdownUI");
var mediaPlaybackCountdownUI = this._mediaPlaybackCountdownUI;
if (mediaPlaybackCountdownUI) {
this._countdownText = mediaPlaybackCountdownUI.querySelector("#countdownText");
this._nextVideoTitle = mediaPlaybackCountdownUI.querySelector("#nextVideoTitle");
this._nextVideoFavicon = mediaPlaybackCountdownUI.querySelector("#nextVideoFavicon");
this._nextVideoSource = mediaPlaybackCountdownUI.querySelector("#nextVideoSource");
this._nextVideoThumbnail = mediaPlaybackCountdownUI.querySelector("#nextVideoThumbnail");
this._nextVideoPlaybackButton = mediaPlaybackCountdownUI.querySelector("#nextVideoPlaybackButton");
Utilities.addRemovableEventListener(this._nextVideoPlaybackButton, "click", function nextVedioPlayback_onClickPlayButton(event) {
that._stopEventPropagation(event);
that._clearCountdownTimer();
CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback(that._nextContentSrc, that._nextVideoOptions, that._videoList);
});
}
}
// This call will ensure any one time work is run such as the splash screen dismissal
// only if this control was loaded as the initial startup page.
PlatformJS.mainProcessManager.afterFirstView();
console.log("Video Control: leaving init");
},
_resetControlsElementState: function _resetControlsElementState() {
if (this._controlHideTimeout) {
this._removeControlsTimer();
this._isControlsVisible = false;
}
},
_onSamePageNav: function _onSamePageNav(event) {
this._leaveFullScreenMode();
},
//Get offsetTop to the root element in DOM tree
_getRelativeTop: function _getRelativeTop(obj) {
var offsetTop = 0;
while (obj) {
offsetTop += obj.offsetTop;
obj = obj.offsetParent;
}
return offsetTop;
},
// Helper function to stop event propagation
_stopEventPropagation: function _stopEventPropagation(event) {
event.stopPropagation();
event.preventDefault();
},
// Setup media strip visuals in right order
_setupMediaStrip: function _setupMediaStrip() {
var inputHandlerWidth = this._inputHandlerElement.clientWidth;
this._mediaStrip.style.msTransform = "";
},
_isAdPlaying: function _isAdPlaying() {
if (this._adStatus !== CommonJS.MediaApp.Controls.AdStates.AdCompleted && this._videoRequiresAds) {
return true;
}
else {
return false;
}
},
// Handles the gesture start event on the input Handler element
_onInputHandlerPointerDown: function _onInputHandlerPointerDown(event) {
if (!this._canScrubVideo) {
return;
}
if (event.pointerType === mousePointerType && event.button !== mouseLeftButton) {
return;
} else {
this._inputHandlerElement.msSetPointerCapture(event.pointerId);
event.preventDefault();
}
this._startTouchOffsetX = event.offsetX;
this._startTouchOffsetY = event.offsetY;
var thumbOffsetX = (this._thumbElement.offsetLeft - this._thumbElement.parentElement.offsetLeft) + this._thumbElement.clientWidth / 2;
var thumbOffsetY = (this._thumbElement.offsetTop - this._thumbElement.parentElement.offsetTop) + this._thumbElement.clientHeight / 2;
if (this._isSeekBarVisible && Math.abs(thumbOffsetX - this._startTouchOffsetX) < this._thumbGrabThreshold && Math.abs(thumbOffsetY - this._startTouchOffsetY) < this._thumbGrabThreshold) {
this._onThumbStartDrag(event);
this._stopEventPropagation(event);
return true;
}
this._isPointerDown = true;
},
// Handles the pointer move event on the input Handler element
_onInputHandlerPointerMove: function _onInputHandlerPointerMove(event) {
var currentPointerPosition = event.pageX + "," + event.pageY;
this._testThumbState(event.pageX, event.pageY);
var lastPointerPosition = this._lastPointerPosition;
this._lastPointerPosition = currentPointerPosition;
event.preventDefault();
if (!lastPointerPosition || lastPointerPosition === currentPointerPosition) {
return;
}
if (this._isThumbGrabbed) {
this._etw.mediaPlaybackSliderUpdateStart();
this._onThumbDrag(event);
return true;
}
if (this._isPointerDown) {
var newOffset = event.offsetX - this._startTouchOffsetX;
var inputHandlerWidth = this._inputHandlerElement.clientWidth;
if (newOffset > inputHandlerWidth) {
newOffset = inputHandlerWidth;
} else if (newOffset < -inputHandlerWidth) {
newOffset = -inputHandlerWidth;
}
// Disable swipe in minimal mode
if (this._mode !== MediaAppControls.PlaybackMode.minimal && this._type !== MediaAppControls.PlaybackType.video) {
this._currentMediaStripTranslation = newOffset;
this._mediaStrip.style.msTransform = "translate(" + newOffset + "px,0px)";
if (!this._isSeekBarVisible) {
this._hideControls();
this._hideSeekBar();
}
}
event.preventDefault();
return true;
}
// Save the timeStamp to see if we need to add time to the controls and seekbar hide timeout
this._lastPointerMoveTimeStamp = event.timeStamp;
if (!this._isAdPlaying()) {
this.showControls(true, false);
}
if (this._mode === MediaAppControls.PlaybackMode.partial) {
// Fire mousemove to trigger the Now Playing details to reappear
this._dispatchEvent("mousemove");
}
return false;
},
// Handles the gesture end event on the input Handler element
_onInputHandlerPointerUp: function _onInputHandlerPointerUp(event) {
var mini = event.currentTarget.getAttribute("data-mini") === "root";
if (!mini) {
this._inputHandlerElement.msReleasePointerCapture(event.pointerId);
if (this._isThumbGrabbed) {
if (this._isThumbUp) {
this._rotateThumbDown();
}
this._onThumbStopDrag(event);
//_onThumbStopDrag will set the state of _thumbElement as MediaAppControls.ControlState.hover
//here we test its real state based on the pointer's position
this._testThumbState(event.pageX, event.pageY);
return true;
}
}
if (this._isPointerDown) {
var offsetX = event.offsetX + event.target.offsetLeft - event.currentTarget.offsetLeft;
// If we're dragging the thumb
if (Math.abs(event.offsetY + this._getRelativeTop(event.currentTarget) - (this._getRelativeTop(this._seekBar) + this._seekBar.clientHeight / 2)) < this._thumbGrabThreshold && this._isSeekBarVisible) {
this._startOffsetX = event.offsetX;
if (this._startOffsetX > this._getSeekbarStart() && this._startOffsetX < (parseFloat(this._seekBar.clientWidth) + this._getSeekbarStart())) {
this._onThumbDrag(event);
}
this._showControls();
this._showSeekBar();
} else {
this._onInputHandlerTap(event);
}
}
this._isPointerDown = false;
return false;
},
// For tap on mediaplayback, to bring up controls bar
_onInputHandlerTap: function _onInputHandlerTap(event) {
if (this._controlHideTimeout) {
this._hideControls();
this._hideSeekBar();
} else {
this._showControls();
if (this._mode !== MediaAppControls.PlaybackMode.minimal) {
this._showSeekBar();
}
}
},
// Handles the gesture end event on the input Handler element
// This is required because touch down events dont work without a registration for gesture events.
_onInputHandlerGestureEnd: function _onInputHandlerGestureEnd(event) {
},
// Handle key presses when the video control has the focus accessibility
// Support play/pause
_onInputHandlerKeyUp: function _onInputHandlerKeyUp(event) {
this.showControls(true, false);
var keyPressed = event.key;
if ((keyPressed === "Spacebar") || (keyPressed === "Enter")) {
if (document.activeElement === this._fullScreenToggleButton || (document.activeElement && document.activeElement.children &&
document.activeElement.children[0] && document.activeElement.children[0] === this._fullScreenToggleButton)) {
this._toggleFullScreen();
} else if (!this._isAdPlaying() && (document.activeElement === this._screenSizeToggleButton || (document.activeElement && document.activeElement.children &&
document.activeElement.children[0] && document.activeElement.children[0] === this._screenSizeToggleButton))) {
this._toggleScreenSize();
}
else {
this._onPlayPauseToggleCommand();
}
return;
}
if (keyPressed === "Esc" || keyPressed === "Backspace") {
this._leaveFullScreenMode();
}
if (keyPressed === "Tab") {
if (!this._isAdPlaying() && (document.activeElement === this._playButton || (document.activeElement && document.activeElement.children &&
document.activeElement.children[0] && document.activeElement.children[0] === this._playButton))) {
this._fullScreenToggleButton.focus();
} else if (!this._isAdPlaying() && (document.activeElement === this._fullScreenToggleButton || (document.activeElement && document.activeElement.children &&
document.activeElement.children[0] && document.activeElement.children[0] === this._fullScreenToggleButton))) {
this._screenSizeToggleButton.focus();
} else {
this._playButton.focus();
}
}
},
_onInputHandlerKeyDown: function _onInputHandlerKeyDown(event) {
//_onPlayingEventFired to avoid using left/right arrows to drag seekbar before video starts playing
//i.e., when ads is playing or video is loading
if (!this._canScrubVideo || !this._onPlayingEventFired) {
return;
}
var keyPressed = event.key;
switch (keyPressed) {
case "Left":
this.showControls();
this.currentTime = this.currentTime - 10;
this._stopEventPropagation(event);
return;
case "Right":
this.showControls();
this.currentTime = this.currentTime + 10;
this._stopEventPropagation(event);
return;
}
},
// Marks a mediaElement as vacant and makes it available to be recycled
_vacateMediaElement: function _vacateMediaElement(mediaElement) {
mediaElement.isVacant = true;
},
// Handles the mouse move element within the controls bar
_onControlsElementMouseMove: function _onControlsElementMouseMove() {
this._showControls();
this._showSeekBar();
},
_getTotalSeekbarWidth: function _getTotalSeekbarWidth() {
return parseFloat(this._seekBar.clientWidth) - this._thumbElement.clientWidth;
},
_getSeekbarStart: function _getSeekbarStart() {
return this._seekAndPlaybackControlTableSpacerLeft.clientWidth;
},
// Handles the pointer down event on the seek bar thumb 
_onThumbStartDrag: function _onThumbStartDrag(event) {
if (this._isSeekBarVisible && this._canScrubVideo) {
this._changeControlSource(this._thumbElement, MediaAppControls.Control.gripper, MediaAppControls.ControlState.pressed);
var thumbOffset = (this._thumbElement.offsetLeft - this._thumbElement.parentElement.offsetLeft) + this._thumbElement.clientWidth / 2;
this._thumbGrabOffsetX = event.clientX - thumbOffset;
//videoElement.pause cannot distinguish end state and pause state
this._wasPausedBeforeScrubbing = this._sequencer._currentState === CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused;
// Only do this for video as this will stop audio playback
if (this._type === MediaAppControls.PlaybackType.video) {
this._currentMediaElement.playbackRate = 0;
}
this._startOffsetX = event.offsetX;
this._isThumbGrabbed = true;
this._removeControlsTimer();
this._removeSeekBarTimer();
this._lastSeekBarLocation = this._seekBar.currentStyle.bottom;
this._lastThumbLocation = this._thumbElement.currentStyle.bottom;
this._rotateThumbUp();
this._dispatchEvent("seeking");
return true;
} else {
this._onInputHandlerPointerDown(event);
event.preventDefault();
return true;
}
},
// Handles the mouse up event on the seek bar thumb
_onThumbStopDrag: function _onThumbStopDrag(event) {
if (this._isThumbGrabbed) {
if (!this._wasPausedBeforeScrubbing) {
this._currentMediaElement.playbackRate = 1;
// Only resume playback if we are not at the end of the queues last song, as this can occur in the middle of a song (hence the .ended being a clause).
// the reason of not using videoElement.ended as the condition of if clause below is that 
// videoElement.endedwill be immediately set false once we drag the seekbar, which will trigger 
//the update of current time of suqencer and turn off videoElement.ended, 
//even though it doesn't start playing. So, _sequencer._currentState is more reliable here.
if (this._sequencer._currentState !== CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing &&
this._sequencer._currentState !== CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused) {
// [Video DCR]: Support Continuous video playback
// Hide CountdownUI if opened
if (this._isCountdownUIShowing) {
this._hideCountdownUI();
}
// TODO: Fix this. Not working even before. Since, we were closing fullscreenmode immediately when video ends, nobody noticed it.
this._currentMediaElement.play();
}
}
this._isThumbGrabbed = false;
this._showSeekBar(true);
this._thumbElementToolTip.style.opacity = 0;
this._changeControlSource(this._thumbElement, MediaAppControls.Control.gripper, MediaAppControls.ControlState.hover);
this.resetAutoHideTimers(true);
return true;
}
return false;
},
_rotateThumbUp: function _rotateThumbUp() {
this._seekBar.style.msTransitionProperty = "margin-bottom";
this._backdrop.style.msTransitionProperty = "height";
Utilities.addClass(this._seekBar, "seekBarRaised");
Utilities.addClass(this._backdrop, "backDropExpanded");
var totalSeekBarWidth = this._getTotalSeekbarWidth();
var seekStart = this._getSeekbarStart();
var progress = this._sequencer.CurrentTime * totalSeekBarWidth / this._sequencer.Duration;
// Handle the case of scrubbing before the video is loaded
if (isNaN(this._sequencer.Duration)) {
progress = 0;
}
var angle = -90 + progress / totalSeekBarWidth * 90;
// Rotate counterclockwise after halfway point
if (angle >= -45 && !this._isThumbUp) {
angle -= 360;
}
this._thumbElement.style.msTransitionProperty = "all";
this._thumbElement.style.msTransitionDuration = "200ms";
this._thumbElement.style.msTransitionTimingFunction = "ease-in-out";
this._thumbElement.style.msTransform = "rotate(" + angle + "deg)";
this._thumbElement.style.msTransitionProperty = "";
this._thumbElement.style.msTransitionDuration = "";
this._thumbElement.style.msTransitionTimingFunction = "";
this._isThumbUp = true;
},
_rotateThumbDown: function _rotateThumbDown() {
this._isThumbUp = false;
this._syncTimeAndProgress(true);
Utilities.removeClass(this._backdrop, "backDropExpanded");
Utilities.removeClass(this._seekBar, "seekBarRaised");
},
// Calculates the seek bar scrub granularity based on the time over position.
_getSeekGranularity: function _getSeekGranularity(event) {
var granularity = 1;
return granularity;
},
// Handles the mouse move event on the input handler element when thumb bar is held down
_onThumbDrag: function _onThumbDrag(event) {
if (this._currentMediaElement.isVacant) {
return;
}
var difference = event.offsetX - this._startOffsetX;
var totalSeekBarWidth = this._getTotalSeekbarWidth();
var seekStart = this._getSeekbarStart();
// Calculate new bar position
var granularity = 1;
var progress = this._startOffsetX + difference * granularity - seekStart;
progress = progress > 0 ? progress : 0;
progress = progress < totalSeekBarWidth ? progress : totalSeekBarWidth;
var angle = 0;
if (this._isThumbGrabbed || this._isThumbUp) {
angle = -90 + progress / totalSeekBarWidth * 90;
} else {
angle = -180 - progress / totalSeekBarWidth * 90;
}
this._thumbElement.style.msTransform = "rotate(" + angle + "deg)";
// If the video isn't loaded yet, don't update the time...
if (isNaN(this._sequencer.Duration)) {
return;
}
// Update current time to match the new position
var newTime = progress / totalSeekBarWidth * this._sequencer.Duration;
this._thumbElementToolTip.innerText = this._formatTime(newTime);
// Cache the position we want to seek to
//disable dragging to the very end of video, in a case that video already ends then user keep drgging to the end, video control will behave incorrectly with continue play feature.
//also, with smooth streaming format, when seeking to the end, video might be "freezing"/ multiple "onEnded" events will be fired/ video will be playback again automatically
if (newTime >= (this._sequencer.Duration - 4) && this._sequencer.Duration > 4) {
newTime = this._sequencer.Duration - 4;
}
this._newTime = newTime;
this.currentTime = newTime;
if (this.currentTime !== 0) {
this._posterImage.style.visibility = "hidden";
}
// Update the visual position of the seek bar
this._thumbElement.style.marginLeft = (seekStart + progress) /* - this._thumbElement.clientWidth / 2) */ + "px";
var tooltipBottomMargin = (this._mode === MediaAppControls.PlaybackMode.full) ? 9 : 0;
if (this._isThumbGrabbed) {
this._thumbElementToolTip.style.opacity = 1;
this._thumbElementToolTip.style.marginLeft =
Math.min((this._thumbElement.offsetLeft - this._thumbElement.parentElement.offsetLeft) - ((this._thumbElementToolTip.clientWidth - this._thumbElement.clientWidth) / 2),
(this._thumbElementToolTip.parentElement.clientWidth - this._thumbElementToolTip.clientWidth) - 5) + "px";
this._thumbElementToolTip.style.offsetTop = (this._thumbElement.offsetTop) - (this._thumbElementToolTip.clientHeight - tooltipBottomMargin) /* - this._thumbElementToolTip.clientHeight - tooltipBottomMargin) */ + "px";
}
this._progress.style.width = progress / totalSeekBarWidth * 100 + "%";
this._etw.mediaPlaybackSliderUpdateEnd();
return true;
},
// Updates the seek bar position to match the current media play time
_syncTimeAndProgress: function _syncTimeAndProgress(force) {
/// <param name="force" optional="true"/>
if (force || this._isSeekBarVisible || this.playbackMode === MediaAppControls.PlaybackMode.partial) {
var totalSeekBarWidth = this._getTotalSeekbarWidth();
var seekStart = this._getSeekbarStart();
var currentTime = this._sequencer.CurrentTime;
var duration = this._sequencer.Duration;
if (currentTime > duration) {
currentTime = duration;
}
var progress = ((this._newTime) ? this._newTime : currentTime) * totalSeekBarWidth / duration;
if (force || this._isSeekBarVisible) {
var angle = 0;
if (this._isThumbUp) {
angle = -90 + progress / totalSeekBarWidth * 90;
} else {
angle = -180 - progress / totalSeekBarWidth * 90;
}
this._thumbElement.style.msTransform = "rotate(" + angle + "deg)";
this._thumbElement.style.marginLeft = seekStart + progress /*- this._thumbElement.clientWidth / 2 */ + "px";
}
this._progress.style.width = progress / totalSeekBarWidth * 100 + "%";
}
},
// Places the seekbar thumb at the beginning for moving to next or previous track
_moveThumbToBeginning: function _moveThumbToBeginning() {
var angle = this._isThumbUp ? -90 : -180;
this._thumbElement.style.msTransform = "rotate(" + angle + "deg)";
this._thumbElement.style.marginLeft = "0px";
this._progress.style.width = 0 + "px";
},
// Pads second or minutes with leading zeros and returns string format. seconds or Minutes should be less than 60
_getTimeString: function _getTimeString(secondsOrMinutes) {
var stringForm = secondsOrMinutes.toString();
if (secondsOrMinutes < 10) {
stringForm = "0" + stringForm;
}
return stringForm;
},
// Formats time in seconds to hh:mm:dd format.
_formatTime: function _formatTime(seconds) {
if (isNaN(seconds)) {
return "";
}
if (this._durationFormatter) {
return this._durationFormatter.formatLong(seconds * 1000);
}
var minutes = Math.floor(seconds / 60);
seconds = Math.floor(seconds % 60);
var hours = Math.floor(minutes / 60);
minutes = minutes % 60;
var timeString = minutes.toString() + ":" + this._getTimeString(seconds);
if (hours > 0) {
timeString = hours.toString() + ":" + this._getTimeString(timeString);
}
return timeString;
},
_getTimeIndicatorText: function _getTimeIndicatorText() {
return (this._timeFormat || "{0} / {1}").format(
this._formatTime(this._sequencer.CurrentTime),
this._formatTime(this._sequencer.Duration)
);
},
_getElapsedTimeText: function _getElapsedTimeText() {
return this._formatTime(Math.ceil(this._sequencer.CurrentTime));
},
_getRemainingTimeText: function _getRemainingTimeText() {
return "-" + this._formatTime(Math.floor(this._sequencer.Duration) - Math.floor(this._sequencer.CurrentTime));
},
// Updates the time display div
_updateTimeDisplay: function _updateTimeDisplay() {
this._clearTimeDisplay();
this._timeIndicator.innerText = this.timeIndicatorText;
this._currentTimeIndicator.innerText = this._formatTime(this._sequencer.CurrentTime);
this._totalTimeIndicator.innerText = this._formatTime(this._sequencer.Duration);
if (this._currentMediaElement && this._currentMediaElement.src.indexOf(this._resetVidSrc) >= 0) {
this._topBarCurrentTime.innerText = "0:00";
this._topBarTotalTime.innerText = "/ 0:00";
} else {
this._topBarCurrentTime.innerText = this._currentTimeIndicator.innerText;
this._topBarTotalTime.innerText = "/ " + this._totalTimeIndicator.innerText;
}
// Only update the narrator every 5 secs
if (Math.abs(this._currentMediaElement.currentTime - this._lastTimeDisplayAriaUpdate) > 5) {
this._thumbElement.setAttribute("aria-valuenow", this._sequencer.CurrentTime);
this._thumbElement.setAttribute("aria-valuemin", 0);
this._thumbElement.setAttribute("aria-valuemax", this._sequencer.Duration);
this._lastTimeDisplayAriaUpdate = this._sequencer.CurrentTime;
}
var payload = {
elapsedTime: this.elapsedTimeText,
remainingTime: this.remainingTimeText
};
this._dispatchEvent("timechange", payload);
},
// Clears the time display divs
_clearTimeDisplay: function _clearTimeDisplay() {
Utilities.empty(this._timeIndicator);
Utilities.empty(this._currentTimeIndicator);
Utilities.empty(this._totalTimeIndicator);
},
// Handles the timeupdate event of the current media element
_onTimeUpdate: function _onTimeUpdate() {
if (this._hasBeenInitialized) {
if (!this._isThumbGrabbed) {
this._syncTimeAndProgress();
}
this._updateTimeDisplay();
}
},
// This updates the controls with property visibility based on the mode (full/partial/minimal) using the 
// declarative data-supportedModes and data-supportedMediaTypes attributes on the control buttons.
_updateControls: function _updateControls() {
var controls = this._controlsElement.getElementsByTagName("button");
if (controls && controls.length > 0) {
for (var index = controls.length - 1; index >= 0; index--) {
var control = controls[index];
control.style.defaultDisplay = control.style.display;
control.style.display = "none";
control.setAttribute("data-mode", this._mode);
var supportedModes = control.getAttribute("data-supportedModes");
var supportedTypes = control.getAttribute("data-supportedMediaTypes");
if (supportedModes && supportedModes.indexOf(this._mode) > -1) {
if (supportedTypes && supportedTypes.indexOf(this._type) > -1) {
control.style.display = control.style.defaultDisplay;
}
}
}
}
this._timeIndicator.setAttribute("data-mode", this._mode);
this._currentTimeIndicator.setAttribute("data-mode", this._mode);
this._totalTimeIndicator.setAttribute("data-mode", this._mode);
this._thumbElement.setAttribute("data-mode", this._mode);
this._seekBar.setAttribute("data-mode", this._mode);
this._backdrop.setAttribute("data-mode", this._mode);
this._playButton.setAttribute("data-mode", this._mode);
this._fullScreenToggleButton.setAttribute("data-mode", this._mode);
this._thumbElementToolTip.setAttribute("data-mode", this._mode);
this._playbackControlTable.setAttribute("data-mode", this._mode);
this._seekAndPlaybackControlTable.setAttribute("data-mode", this._mode);
this._seekAndPlaybackControlTableSpacerLeft.setAttribute("data-mode", this._mode);
this._seekAndPlaybackControlTableSpacerRight.setAttribute("data-mode", this._mode);
this._seekAndPlaybackControlTableSpacerMiddle.setAttribute("data-mode", this._mode);
},
// Adds an poster image for an mediaElement
_setPoster: function _setPoster(imageSrc) {
try {
this._overridePosterImageUrl = imageSrc;
this._posterImage.src = imageSrc;
} catch (e) {
// Invalid image source
}
},
//Adds title for an mediaElement
_setTitle: function _setTitle(title) {
this._topBarTitle.innerText = title;
},
//Adds attribution for an mediaElement
_setAttribution: function _setAttribution(attr) {
this._topBarAttribution.innerText = attr;
this._currentMediaSrcProvider = attr;
},
//Adds logo for an mediaElement
_setLogo: function _setLogo(src) {
try {
this._topBarLogo.src = src;
} catch (e) {
// Invalid image source
}
},
// Turns the play button into a pause button
_showPauseButton: function _showPauseButton() {
this._playButton.isPause = true;
this._playButton.setAttribute("aria-labelledby", "mediaPlaybackPauseButtonAriaLabel");
Utilities.addClass(this._playButton, "playing");
Utilities.removeClass(this._playButton, "grayButton");
Utilities.removeClass(this._fullScreenToggleButton, "grayButton");
},
// Turns the pause button into a play button
_showPlayButton: function _showPlayButton() {
this._playButton.isPause = false;
this._playButton.setAttribute("aria-labelledby", "mediaPlaybackPlayButtonAriaLabel");
Utilities.removeClass(this._playButton, "playing");
},
// Add a media event listener, this method tracks the handlers in a list so that they can be used to unsubscribe later
_addMediaEventListener: function _addMediaEventListener(element, eventName, handler) {
element.addEventListener(eventName, handler, false);
element._mediaEventSubscriptions.push({ eventName: eventName, handler: handler });
},
// Unsubscribe from previously subscribed media events
_unsubscribeFromMediaEvents: function _unsubscribeFromMediaEvents(element) {
var that = this;
if (element._mediaEventSubscriptions) {
element._mediaEventSubscriptions.forEach(function mediaPlayback_foreachMediaEventSubscription(subscription) {
element.removeEventListener(subscription.eventName, subscription.handler);
});
}
element._mediaEventSubscriptions = [];
},
// Unsubscribe a specific media event
_unsubscribeFromMediaEvent: function _unsubscribeFromMediaEvent(element, eventName) {
if (element._mediaEventSubscriptions) {
for (var i = 0, len = element._mediaEventSubscriptions.length; i < len; i++) {
var otherEventName = element._mediaEventSubscriptions[i].eventName;
if (eventName === otherEventName) {
element.removeEventListener(otherEventName, element._mediaEventSubscriptions[i].handler);
element._mediaEventSubscriptions.splice(i, 1);
break;
}
}
}
},
// Setup event handlers on the currently active media element
_setupMediaElement: function _setupMediaElement(mediaElement) {
if (!mediaElement) {
return;
}
this._unsubscribeFromMediaEvents(mediaElement);
var that = this._sequencer;
this._addMediaEventListener(mediaElement, "playing", function mediaPlayback_mediaElementOnPlaying(event) { that._onPlaying(event); });
this._addMediaEventListener(mediaElement, "loadedmetadata", function mediaPlayback_mediaElementOnLoadedMetadata(event) { that._onMetaDataLoaded(event); });
this._addMediaEventListener(mediaElement, "pause", function mediaPlayback_mediaElementOnPause(event) { that._onPause(event); });
this._addMediaEventListener(mediaElement, "ended", function mediaPlayback_mediaElementOnEnded(event) { that._onEnded(event); });
this._addMediaEventListener(mediaElement, "ratechange", function mediaPlayback_mediaElementOnRateChange(event) { that._onRateChange(event); });
this._addMediaEventListener(mediaElement, "error", function mediaPlayback_mediaElementOnError(event) { that._onError(event); });
this._addMediaEventListener(mediaElement, "timeupdate", function mediaPlayback_mediaElementOnTimeUpdate(event) { that._onTimeUpdate(event); });
this._addMediaEventListener(mediaElement, "seeked", function mediaPlayback_mediaElementOnSeeked(event) {
Utilities.addClass(that._mediaPlayer._waitingSpinnerContainer, "platformHide");
Utilities.removeClass(that._mediaPlayer._waitingSpinnerContainer, "mediaPlaybackSpinnerTransBG");
that._onSeeked(event);
});
this._addMediaEventListener(mediaElement, "seeking", function mediaPlayback_mediaElementOnSeeked(event) {
Utilities.removeClass(that._mediaPlayer._waitingSpinnerContainer, "platformHide");
Utilities.addClass(that._mediaPlayer._waitingSpinnerContainer, "mediaPlaybackSpinnerTransBG");
});
},
_changeControlSource: function _changeControlSource(control, controlName, state) {
control.src = "{0}/{1}{2}_{3}.svg".format(this._srcPath, controlName, state, (this._inFullScreenMode) ? "fullscreen" : "inline");
},
_testThumbState: function _testThumbState(mouseX, mouseY) {
var position = Utilities.getPosition(this._thumbElement);
var state;
if (this._isThumbGrabbed) {
state = MediaAppControls.ControlState.pressed;
} else if (mouseX >= position.left && mouseX <= position.left + position.width && mouseY >= position.top && mouseY <= position.top + position.height) {
state = MediaAppControls.ControlState.hover;
} else {
state = MediaAppControls.ControlState.rest;
}
if (state !== this._oldThumbState) {
this._oldThumbState = state;
this._changeControlSource(this._thumbElement, MediaAppControls.Control.gripper, state);
}
},
_onReadyForInteraction: function _onReadyForInteraction() {
var that = this;
this._loadPromise.then(function mediaPlayback_onReadyForInteractionPromiseComplete() {
that._enableInteraction();
});
},
_enableInteraction: function _enableInteraction() {
var that = this;
Utilities.addRemovableEventListener(this._inputHandlerElement, "MSPointerDown", function mediaPlayback_inputHandlerElementOnMSPointerDown(event) { that._onInputHandlerPointerDown(event); }, true);
Utilities.addRemovableEventListener(this._inputHandlerElement, "MSPointerMove", function mediaPlayback_inputHandlerElementOnMSPointerMove(event) { that._onInputHandlerPointerMove(event); }, false);
Utilities.addRemovableEventListener(this._inputHandlerElement, "MSGestureEnd", function mediaPlayback_inputHandlerElementOnMSGestureEnd(event) { that._onInputHandlerGestureEnd(event); }, false);
Utilities.addRemovableEventListener(this._inputHandlerElement, "MSPointerUp", function mediaPlayback_inputHandlerElementOnMSPointerUp(event) { that._onInputHandlerPointerUp(event); }, false);
},
_disableInteraction: function _disableInteraction() {
Utilities.removeEventListener(this._inputHandlerElement, "MSPointerDown", true);
Utilities.removeEventListener(this._inputHandlerElement, "MSPointerMove", false);
Utilities.removeEventListener(this._inputHandlerElement, "MSGestureEnd", false);
Utilities.removeEventListener(this._inputHandlerElement, "MSPointerUp", false);
this.hideControls();
},
_onMetaDataLoaded: function _onMetaDataLoaded() {
this._dispatchEvent("loadedmetadata");
},
// Handler for media elements onplaying event, used to raise event on outbound subscribers
_onPlaying: function _onPlaying() {
var that = this;
//force video to show... work round bug 388898 on Nvidia machine that sometimes the first video after app restart will show black screen only
if (that && that._currentMediaElement && that._currentMediaElement.visual && that._currentMediaElement.visual.style) {
that._currentMediaElement.visual.style.opacity = "0.99";
}
//set opacity to other value than 1 will hide top bar background and drabar background, looks like a system bug
setTimeout(function mediaPlaybackResetVideoOpacity() {
if (that && that._currentMediaElement && that._currentMediaElement.visual && that._currentMediaElement.visual.style) {
that._currentMediaElement.visual.style.opacity = "1";
console.log("Video Control: set video element opacity back to 1");
}
}, 500);
this._posterImage.style.visibility = "hidden";
Utilities.addClass(this._waitingSpinnerContainer, "platformHide");
if (this._hasBeenInitialized) {
if (this._currentMediaElement.playbackRate === this._currentMediaElement.defaultPlaybackRate) {
this._showPauseButton();
}
Utilities.addClass(this._playButton, "playbackActiveButton");
Utilities.addClass(this._fullScreenToggleButton, "playbackActiveButton");
}
this._dispatchEvent("playing");
},
// Handler for media elements pause event, used to raise event on outbound subscribers
_onPause: function _onPause() {
this._etw.mediaPlaybackPauseEnd();
if (this._hasBeenInitialized) {
this._showPlayButton();
}
if (!this._onStopping) {
this._dispatchEvent("pause");
} else {
this._onStopping = false;
}
Utilities.removeClass(this._playButton, "playbackActiveButton");
Utilities.removeClass(this._fullScreenToggleButton, "playbackActiveButton");
var that = this;
},
_onPosterLoad: function _onPosterLoad() {
this._dispatchEvent("posterloaded");
},
// Handler for media elements ended event, used to raise event on outbound subscribers
_onEnded: function _onEnded(event) {
if (this._isThumbGrabbed) {
this._onThumbStopDrag(null);
}
Utilities.removeClass(this._playButton, "playbackActiveButton");
Utilities.removeClass(this._fullScreenToggleButton, "playbackActiveButton");
//we decided not to show poster image in the end
//this._posterImage.style.visibility = "visible";
this._dispatchEvent("ended");
this._dispatchEvent("stopped");
// [Video DCR]: Support Continuous video playback
// Setup NEXT video
if (this._shouldContinueNextVideo) {
// We will not show countdown UI for snapmode. just play videos continuously
// TODO: But what if the app is snapped while countdown UI is showing. Need to fix this.
var state = Windows.UI.ViewManagement.ApplicationView.value;
if (state === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback(this._nextContentSrc, this._nextVideoOptions, this._videoList);
}
else if(this._mediaPlaybackCountdownUI) {
var sec = this._countdownForNextVideo;
var countdownMessage = CommonJS.resourceLoader.getString("/platform/videoCountdownText");
this._showCountdownUI();
var that = this;
this._countdownTimerId = setInterval(function () {
if (sec === 0) {
that._hideCountdownUI();
CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback(that._nextContentSrc, that._nextVideoOptions, that._videoList);
return;
}
that._countdownText.innerHTML = countdownMessage.format("<strong>" + sec + "</strong>");
sec--;
}, 1000);
}
}
else {
if (Utilities.hasClass(this._errorScreen, "platformHide")) {
//_leaveFullScreenMode will check if it it in fullScreen mode
this._leaveFullScreenMode();
}
}
this._etw.mediaPlaybackMediaEnd();
},
_replaceHTMLEncodedChars: function _replaceHTMLEncodedChars(str) {
if (typeof (str) !== "string") {
return "";
}
var ret = str.replace(/<(?:.|\n)*?>/gm, "");
ret = ret.replace(/&quot;|&#34;/g, "\"");
ret = ret.replace(/&lt;|&#60;/g, "<");
ret = ret.replace(/&gt;|&#62;/g, ">");
ret = ret.replace(/&nbsp;|&#160;/g, " ");
ret = ret.replace(/&amp;|&#38;/g, "&");
return ret;
},
_showCountdownUI: function _showCountdownUI() {
this._populateCountdownUI();
this._screenSizeToggleButton.disabled = true;
Animation.fadeOut(this._currentMediaElement);
var that = this;
Utilities.removeClass(this._mediaPlaybackCountdownUI, "platformHide");
Animation.fadeIn(this._mediaPlaybackCountdownUI).done(function showCountdownUIComplete() {
CommonJS.Utils.truncateMultilineText(that._nextVideoTitle);
Utilities.removeClass(that._nextVideoTitle, "mask");
that._isCountdownUIShowing = true;
});
},
_populateCountdownUI: function _populateCountdownUI() {
var sec = this._countdownForNextVideo;
var countdownMessage = CommonJS.resourceLoader.getString("/platform/videoCountdownText");
this._countdownText.innerHTML = countdownMessage.format("<strong>" + sec + "</strong>");
this._nextVideoTitle.innerText = this._replaceHTMLEncodedChars(this._nextVideoOptions.title || "");
if (this._nextVideoOptions.sourceImageUrl) {
this._nextVideoFavicon.src = this._nextVideoOptions.sourceImageUrl;
this._nextVideoFavicon.alt = this._nextVideoOptions.source || "";
}
else {
this._nextVideoFavicon.style.display = "none";
}
this._nextVideoSource.innerText = this._nextVideoOptions.source || "";
if (this._nextVideoOptions.thumbnail) {
this._nextVideoThumbnail.src = this._nextVideoOptions.thumbnail;
}
else {
this._nextVideoThumbnail.style.display = "none";
}
},
_hideCountdownUI: function _hideCountdownUI() {
this._clearCountdownTimer();
var that = this;
Animation.fadeOut(this._mediaPlaybackCountdownUI).done(function hideCountdownUIComplete() {
Utilities.addClass(that._mediaPlaybackCountdownUI, "platformHide");
that._isCountdownUIShowing = false;
});
Animation.fadeIn(this._currentMediaElement);
this._screenSizeToggleButton.disabled = false;
},
_clearCountdownTimer: function _clearCountdownTimer() {
if (this._countdownTimerId) {
clearInterval(this._countdownTimerId);
}
},
// Handler for media elements seek has ended event
_onSeeked: function _onSeeked() {
this._etw.mediaPlaybackSeekEnd(this._currentMediaElement.currentTime);
// We need cache the seeking position (the latest seeking position if the video /audio tag is in seeking already)
// and wait for the "seeked" event before we send in another seeking request (since otherwise it will be abandoned by the MF).
if (this._currentMediaElement && this._newTime) {
var newTime = Math.round(this._newTime);
var currentTime = Math.round(this._currentMediaElement.currentTime);
if (newTime !== currentTime) {
this.currentTime = this._newTime;
}
this._newTime = null;
}
},
// Handler for media elements ratechangee event, this helps control the show play pause button during ff/rev
_onRateChange: function _onRateChange() {
/// <disable>JS2045.ReviewEmptyBlocks</disable>
if (this._currentMediaElement.playbackRate === this._currentMediaElement.defaultPlaybackRate) {
this._showPauseButton();
} else if (this._currentMediaElement.playbackRate === 0) {
// do nothing
} else {
this._showPlayButton();
}
/// <enable>JS2045.ReviewEmptyBlocks</enable>
},
// Error event handler for the media element's error event
_onError: function _onError(event) {
var errorCode = event.srcElement && event.srcElement.error ? event.srcElement.error.code : "n/a";
this._errorLog("Video element error code:" + errorCode);
this._videoSrcListIndex = this._videoSrcListIndex + 1;
nextVideoSrc = this._videoSrcList && this._videoSrcList.length > 0 && this._videoSrcListIndex < this._videoSrcList.length ? this._videoSrcList[this._videoSrcListIndex] : null;
if (window.navigator.onLine && nextVideoSrc) { //play next video in the list if this is NOT an offline error
console.warn("Video Control: failed to play " + this._currentMediaElement.src);
console.warn("Video Control: fall back to play list[" + this._videoSrcListIndex  + "]: " + nextVideoSrc);
this._currentMediaElement.src = nextVideoSrc;
this.play();
}
else if (this._message) {
Utilities.removeClass(this._errorScreen, "platformHide");
Utilities.addClass(this._waitingSpinnerContainer, "platformHide");
this._message.innerText = window.navigator.onLine ? CommonJS.resourceLoader.getString("/platform/videoLoadErrorText") : CommonJS.resourceLoader.getString("/platform/videoConnectErrorText");
//enable exit controls for "try again"
this._showControls(true, false);
this._fullScreenToggleButton.focus();
}
},
_hideAdDivCleanupAdRenderer: function _hideAdDivCleanupAdRenderer(suspended) {
this._adStatus = CommonJS.MediaApp.Controls.AdStates.AdCompleted;
if (!suspended && this._adControl && this._legacyAdControl) {
this._adControl.suspend(); //call suspend to tell adRenderer to destroy itself
console.log("Video Control: ask ad Control to terminate and release memory...");
}
//for MMPPF and mDialog
if (this._adControl && !this._legacyAdControl) {
this._adControl.dispose();
this._adControl = null;
console.log("Video Control: _hideAdDivCleanupAdRenderer(): dispose ad Control ...");
}
this._videoAd.style.display = "none";
if (this._videoAdTimeOutFinish) {
clearTimeout(this._videoAdTimeOutFinish);
this._videoAdTimeOutFinish = null;
}
Utilities.removeClass(this._waitingSpinnerContainer, "platformHide");
},
_removeAdAndPlayVideo: function _removeAdAndPlayVideo(suspended) {
this._hideAdDivCleanupAdRenderer(suspended);
if (document && (!document.msHidden)) {
this._sequencer.Play();
console.log("Video Control: play video content after Ads");
} else {
this._resumePlay = true;
}
Utilities.addClass(this._waitingSpinnerContainer, "platformHide");
},
_resetToBlack: function _resetToBlack() {
try {
this._currentMediaElement.src = this._resetVidSrc;
this._currentMediaElement.load();
}
catch (e) {
console.warn("Video Control: failed to load black.mp4: " + this._resetVidSrc + ", error message: " + e.message);
}
},
_validateLocationForVideo: function _validateLocationForVideo(loc) {
if (loc === "IPLookupError") {
console.warn("Video Control: location Lookup service error, will ignore and play video");
return true;
}
if (this._geoFence === "") {//no geoFence for current video
return true;
}
if (typeof this._geoFence !== "string") {
console.error("video Control: incorrect MSN Catalog geoFence type");
return false;
}
if (typeof loc !== "string") {
console.error("video Control: location lookup value" + loc + " is not a string");
return false;
}
var bitmasks = PlatformJS.Services.configuration.getDictionary("MSNVideoLocBitmask");
var maskPos = bitmasks ? bitmasks.getInt32(loc.toLowerCase(), -1) : -1;
if (bitmasks && maskPos >= 0) {
console.log("video Control: validating location " + loc);
var bitPosInChar = maskPos % 4;
var bitPosInStr = this._geoFence.length - 1 - Math.floor(maskPos / 4);
if (bitPosInStr < 0 || bitPosInStr >= this._geoFence.length) {
console.error("video Control: geoFence: incorrect bit position: " + bitPosInStr + "; _geoFence=" + this._geoFence);
return false;
}
var num = parseInt(this._geoFence.charAt(bitPosInStr), 16);
if (!(num >= 0 && num < 16)) { //if NaN
console.error("video Control: error during validating location: " + "geoFence: " + this._geoFence + "; bitmask=" + maskPos);
return false;
}
var bitAtNum = (num >>> bitPosInChar) & 1;
if (bitAtNum) {
return false;
}
else {
return true; //0 on mask bit means location is valid
}
}
else {
console.error("video Control: error during validating location: " + "geoFence: " + this._geoFence + "; bitmask=" + maskPos);
return false;
}
},
_onAdStart: function _onAdStart() {
console.log("Video Control: Video Ad start");
Utilities.addClass(this._waitingSpinnerContainer, "platformHide");
this._adStatus = CommonJS.MediaApp.Controls.AdStates.AdStarted;
this._showControls(true, false);
this._fullScreenToggleButton.focus();
},
_onAdComplete: function _onAdComplete() {
var videoAdsInterval = this._videoAdsInterval;
if (!videoAdsInterval || videoAdsInterval < 1) {
videoAdsInterval = 5; //default interval
}
this._currentVideoAdsInterval = (this._currentVideoAdsInterval + 1) % videoAdsInterval;
this._removeAdAndPlayVideo(true);
console.log("Video Control: Ad is complete, about to play video content");
},
_createLegacyAd: function _createLegacyAd(results) {
var that = this;
that._legacyAdControl = true;
that._adControl = new MicrosoftNSJS.Advertising.AdControl(that._videoAd, {
isAutoRefreshEnabled: false, onErrorOccurred: function onAdError(sender, error) {
that._removeAdAndPlayVideo(true);
var errorMessage = error ? error.errorMessage : "";
that._errorLog("Ad Control threw error when playing video Ad, skip and play video content:" + errorMessage);
},
onEngagedChanged: function onAdEngaged(sender) {
if (sender.isEngaged) {
that._onAdStart.bind(that)();
} else { //adRenderer fires isUserEngaged === false to notify it has detroyed itself
if (that._delayNavigateBack) {
console.log("Video Control: disposing legacy AdControl after adRenderer disposed...");
that._adControl.dispose();
that._adControl = null;
//exit fullscreen
var fsControl = that._fullScreenControl;
if (fsControl) {
if (!fsControl.tryLeaveFullScreen()) {
console.log("Video Control: Failed to leave fullscreen mode");
}
}
} else {
//we only go here when
//1) ad length is longer than maxAdTime
//2) ad finish completely
//in both case, we consider ad is complete, should update the interval
that._onAdComplete.bind(that)();
console.log("Video Control: disposing legacy AdControl after adRenderer disposed...");
that._adControl.dispose();
that._adControl = null;
console.log("Video Control: Video Ad is complete, about to play video content");
}
}
}
});
//this is for Video adRenderer to work around onOrientationChange Bug #333972
var createIFrame = that._adControl._createIFrame;
that._adControl._createIFrame = function () {
var iframe = createIFrame.apply(that._adControl, arguments);
iframe.width = "100%";
iframe.height = "100%";
return iframe;
};
try {
that._adControl.applicationId = results[0].adsConfig.appID;
that._adControl.adUnitId = results[0].adsConfig.adUnitId;
that._adControl.addAdTag("caid", that._currentMediaSrcId);
that._adControl.addAdTag("csid", results[0].adsConfig.adUnitId);
that._adControl.addAdTag("partnerContentId", results[0].partnerContentId);
//http://msdn.microsoft.com/en-us/library/windows/desktop/windows.system.processorarchitecture.aspx
var arch = Windows.ApplicationModel.Package.current.id.architecture;
var archString = "undefinedEnum";
if (arch === 0) {
archString = "x86";
} else if (arch === 5) {
archString = "arm";
} else if (arch === 9) {
archString = "x64";
} else if (arch === 11) {
archString = "neutral";
} else if (arch === 65535) {
archString = "unknown";
}
that._adControl.addAdTag("processorArc", archString);  //processorArc = x86| arm| x64 |neutral | unknown
}
catch (e) {
console.warn("Video Control: failed to assign video IDs to adControl, appId: " + results[0].adsConfig.appID + ",caid " + that._currentMediaSrcId + ",csid" + results[0].adsConfig.adUnitId + ",error message: " + e.message);
}
},
_createMMPPFAd: function _createMMPPFAd(results) {
var that = this;
console.log("Video Control: Creating MMPPF control.....");
that._adControl = new PlayerFramework.MediaPlayer(that._videoAd, {
width: "100%",
height: "100%",
autoplay: true,
isFullScreenVisible: false,
isTimelineVisible: false,
isPlayPauseVisible: false,
isVolumeMuteVisible: false,
isElapsedTimeVisible: false,
autohide: false,
adPlayerFactoryPlugin: {
supportedVideoMimeTypes: ['video/mp4']
},
adSDKPlugin: {
onadrefreshed: that._onAdStart.bind(that),
onerroroccurred: function errorOccurredLogger(e) {
var msg = e && e.detail && e.detail.errorMessage ? e.detail.errorMessage : "";
that._errorLog("Video Control: MMPPF: errorOccurredLogger: error in adControl: " + msg);
that._removeAdAndPlayVideo(true);
},
adUnitId: results[0].adsConfig.adUnitId,
applicationId: results[0].adsConfig.appID,
adTags: {
contentid: that._currentMediaSrcId,
playerid:  results[0].adsConfig.adUnitId,
partnerContentId: results[0].partnerContentId
}
},
});
that._adControl.adSDKPlugin.playAd().done(
that._onAdComplete.bind(that),
function onAdError(e) {
that._errorLog("Video Control: MMPPF Ad plugin throws a Promise error");
if (that._adControl) {
that._removeAdAndPlayVideo(true);
}
}
);
},
_getFakeGUID: function _getFakeGUID() {
var s4 = function s4() {
return Math.floor((1 + Math.random()) * 0x10000)
.toString(16)
.substring(1);
};
return s4() + s4() + '-' + s4() + '-' + s4() + '-' +  s4() + '-' + s4() + s4() + s4();
},
_createMDialogAd: function _createMDialogAd(results) {
var that = this;
var adsConfig = results[0] ? results[0].adsConfig : null;
var partnerContentId = results[0] ? results[0].partnerContentId : "";
partnerContentId = partnerContentId.toLowerCase().replace(/{|}/g, "");          //remove "{" and "}" from a partner content id, for exmpale, WSJ's
var providerStr = adsConfig.adProvider ? adsConfig.adProvider : "";
var appName = (PlatformJS.Services.manifest && PlatformJS.Services.manifest.appId) ? PlatformJS.Services.manifest.appId : "AppEX";
if (!this._sectionID) {
this._sectionID = this._getFakeGUID();
}
var videoAdDicts = PlatformJS.Services.configuration.getDictionary("VideoAdParams");
//CMS requires case-insensitive but getDictionary is case-sensitive so use the lowercase of providerStr to compare
var moreParams = videoAdDicts ? videoAdDicts.getDictionary(providerStr.toLowerCase()) : null; 
if (!moreParams || !moreParams.size) {
that._errorLog("Video Control: mDialog Ad plugin failed to get necessary parameters from configuration");
that._removeAdAndPlayVideo(true);
return;
}
var account = moreParams.getString("account");
var removeStr = moreParams.getString("toRemove");
if (removeStr) {
partnerContentId = partnerContentId.replace(removeStr.toLowerCase(), "");
}
//Creating the sdk decisioning parameters
var sdk_decisioning_data = [];
sdk_decisioning_data.push({ key: "video_asset_id", value: partnerContentId });
sdk_decisioning_data.push({ key: "application_version", value: appName });
sdk_decisioning_data.push({ key: "model", value: moreParams.getString("model") });
sdk_decisioning_data.push({ key: "device_unique_identifier", value: this._sectionID });
//Creating parameters provided by mDialog
//  for FOX
//  api_key = "b68dfe81f017b133bed6ea37972417d2";
//  application_key = "7a488239ecfb97d0805106292359fcc7";
//  for WSJ
//  api_key = "37e0927939ad3e4337f56150b3a891ac";
//  application_key = "0747deacc0faa7033164617ac9d8f2c0";
var api_key = moreParams.getString("api_key");
var application_key = moreParams.getString("application_key");
var client_decisioning_data = [];
//Creating the client specific decisioning parameters
client_decisioning_data.push({ key: "g", value: moreParams.getString("g") });
client_decisioning_data.push({ key: "z", value: moreParams.getString("z") });
client_decisioning_data.push({ key: "app", value: moreParams.getString("app")});
client_decisioning_data.push({ key: "device", value: moreParams.getString("device") });
//Creating the client specific tokens + values for third party tracking
var client_tracking_data = [];
client_tracking_data.push({ key: "BUSINESS_NAME", value: account });
client_tracking_data.push({ key: "APPLICATION_NAME", value: appName });
client_tracking_data.push({ key: "DEVICE", value: moreParams.getString("model") });
//The mDialogAd control is created with a set of parameters, one of which is 
//the mediaplayer container element. 
console.log("Video Control: Creating mDialog ad control.....");
this._adControl = new MDialog.mDialogAd(this._videoAd, {
account: account,
apiKey: api_key,
applicationKey: application_key,
playerWidth: "100%",
playerHeight: "100%",
sdkDecisioningData: sdk_decisioning_data,
clientDecisioningData: client_decisioning_data,
clientTrackingData: client_tracking_data,
controls: false,
onStart: that._onAdStart.bind(that),
onTimeUpdate: function mDialogOnTimeUpdate() {
//do nothing for now
},
onError: function onMDialogAdError(e) {
that._errorLog("Video Control: mDialog Ad plugin throws an error");
that._removeAdAndPlayVideo(true);
},
onComplete: that._onAdComplete.bind(that)
});
},
// Resets the media element to position 0
_resetMediaElement: function _resetMediaElement(mediaElement, src, useMediaLibrary) {
console.log("Video Control: entered resetMediaElement");
var returnPromise;
if (src) {
var source = src;
this._currentMediaSrcId = src;
mediaElement.objId = null;
if (src.objId && src.src) {
source = src.src;
mediaElement.objId = src.objId;
}
this._resetToBlack();
this._currentMediaElement.isMediaLoadedFromLibrary = useMediaLibrary;
if (this._currentMediaElement.isMediaLoadedFromLibrary) {
var that = this;
returnPromise = Promise.join([
this._loadContentFromMediaLibrary(source),
Platform.Globalization.Marketization.getLocationFromServerAsync().
then(function ipLookupComplete(p) {
return WinJS.Promise.wrap(p);
}, function ipLookupError() {
return WinJS.Promise.wrap("IPLookupError");
})
]).then(function mediaPlayback_loadAdsForVideoComplete(results) {
if (results) {
if (!results[0]) {
Utilities.addClass(that._waitingSpinnerContainer, "platformHide");
Utilities.removeClass(that._errorScreen, "platformHide");
that._message.innerText = window.navigator.onLine ? CommonJS.resourceLoader.getString("/platform/videoLoadErrorText") : CommonJS.resourceLoader.getString("/platform/videoConnectErrorText");
that._errorLog("failed to load video source from MSNCatalog");
//enable UIs to allow user to exit
that._showControls(true, false);
that._fullScreenToggleButton.focus();
}
else if (!that._validateLocationForVideo(results[1])) {
that._resetToBlack();
that._disableVideoForGeoFence = true; //disable video play action later
Utilities.addClass(that._waitingSpinnerContainer, "platformHide");
Utilities.removeClass(that._errorScreen, "platformHide");
that._message.innerText = CommonJS.resourceLoader.getString("/platform/videoGeoErrorText");
//enable UIs to allow user to exit
that._showControls(true, false);
that._fullScreenToggleButton.focus();                                        
}									
else {
that._sequencer.AddContent(results[0]);
if (that._videoRequiresAds && results[0].adsConfig && results[0].adsConfig.adServer && results[0].adsConfig.adServer.toLowerCase() !== "noad") {
if (that._videoAd) {
that._delayNavigateBack = false;
that._adStatus = CommonJS.MediaApp.Controls.AdStates.AdNotStart;
console.log("Video Control: creating new AdControl...");
var adControlName = results[0].adsConfig.adServer.toLowerCase();
that._adsConfig = results[0].adsConfig; //record adsConfig information for intrumentation
if (adControlName === "legacy") {
that._createLegacyAd.bind(that)(results);
} else if (adControlName === "mmppf") {
that._createMMPPFAd.bind(that)(results);
} else if (adControlName === "mdialog") {
that._createMDialogAd.bind(that)(results);
}
//clear previous timeout
if (that._videoAdTimeOutFinish) {
clearTimeout(that._videoAdTimeOutFinish);
that._videoAdTimeOutFinish = null;
}
var maxAdTime = PlatformJS.Services.appConfig.getInt32("MaxAdTimeMS");
if (!maxAdTime || maxAdTime < 45000) {
maxAdTime = 45000; //give at least 45seconds for ads to finish (consider buffering time)
}
that._videoAdTimeOutFinish = setTimeout(function videoAdTimeOutFinish() {
if (that && that._adStatus !== CommonJS.MediaApp.Controls.AdStates.AdCompleted && that._adControl) {
that._removeAdAndPlayVideo(false);
that._errorLog("Video Ad timed out to finish, force to close and about to play video content");
}
},
maxAdTime //this is the total time that user wait on Ad, including Ad loading time
);
}
}
else { //no ad need to be shown, clean previous ad status
that._delayNavigateBack = false;
that._hideAdDivCleanupAdRenderer(true); //call to hide ad div without any cleanup request
that._adStatus = CommonJS.MediaApp.Controls.AdStates.AdNotStart;
}
}
}
});
} else {
mediaElement.src = source;
var extVid = {
ContentType: CommonJS.MediaApp.Controls.MediaContentType.VideoContent,
PingHandler: null,
VideoUuid: null,
CanScrubVideo: true,
RequiresAds: false,
SourceUrl: source,
};
this._videoRequiresAds = false;
this._delayNavigateBack = false;
this._sequencer.AddContent(extVid);
this._adjustVideoSizeToScreen(this._userPreferenceVideoScreenSize === "50%" ? false : true);
}
mediaElement.initialTime = 0;
this._newTime = null;
} else {
mediaElement.isVacant = true;
mediaElement.src = "";
Utilities.removeClass(this._loadingScreen, "hiddenObject");
}
if (!returnPromise) {
returnPromise = Promise.wrap(true);
}
return returnPromise;
},
_loadContentFromMediaLibrary: function _loadContentFromMediaLibrary(contentId) {
if (!window.navigator.onLine) {
if (this._message) {
Utilities.removeClass(this._errorScreen, "platformHide");
this._message.innerText = window.navigator.onLine ? CommonJS.resourceLoader.getString("/platform/videoLoadErrorText") : CommonJS.resourceLoader.getString("/platform/videoConnectErrorText");
console.log("Video Control: Network error when loading MSN video");
//enable exit controls for "try again"
this._showControls(true, false);
this._fullScreenToggleButton.focus();
}
return Promise.wrap(null);
}
if (this._mediaLibrary === null) {
this._mediaLibrary = this._createMockMsnVideoLibrary();
}
var that = this;
var overridePosterImageUrl = this._overridePosterImageUrl;
var videoEndpointProvider = new CommonJS.MediaApp.Controls.VideoEndpointProvider();
return this._mediaLibrary.assignSrcUrlFromContentId(videoEndpointProvider.getNextCatalogPrefix(), contentId, overridePosterImageUrl).then(function mediaPlayback_assignSrcUrlComplete(resultVideoObject) {
if (!resultVideoObject) {
return Promise.wrap(null);
} else {
return Promise.wrap(resultVideoObject);
}
});
},
_onPlayCommand: function _onPlayCommand() {
/// <summary>
/// Received Play Command
/// </summary>
/// <param name='event'>
/// MediaControl object on which the app registered this handler with
/// </param>
this.showControls();
this._sequencer.Play();
},
_onStopCommand: function _onStopCommand() {
/// <summary>
/// Received Stop Command
/// </summary>
/// <param name='event'>
/// MediaControl object on which the app registered this handler with
/// </param>
this.showControls();
this._sequencer.Pause();
},
_onPauseCommand: function _onPauseCommand() {
/// <summary>
/// Received Pause Command
/// </summary>
/// <param name='event'>
/// MediaControl object on which the app registered this handler with
/// </param>
this.showControls();
this._sequencer.Pause();
},
_onPlayPauseToggleCommand: function _onPlayPauseToggleCommand() {
/// <summary>
/// Received Play/Pause Toggle Command
/// </summary>
/// <param name='event'>
/// MediaControl object on which the app registered this handler with
/// </param>
this.showControls(true, false);
if (this._sequencer.IsPaused() || this._sequencer.HasEnded()) {
this.play();
} else {
this.pause();
}
},
_systemTransportControlsEnabled: {
set: function set(value) {
/// <summary>
/// Register or deregister for multimedia key events
/// </summary>
var thisFunc = arguments.callee;
if (!thisFunc.playFuncRef) {
var that = this;
thisFunc.playFuncRef = function () { that._onPlayCommand(); };
thisFunc.stopFuncRef = function () { that._onStopCommand(); };
thisFunc.pauseFuncRef = function () { that._onPauseCommand(); };
thisFunc.playPauseToggleFuncRef = function () { that._onPlayPauseToggleCommand(); };
}
if (thisFunc.registered === value) {
return;
}
try {
var eventFunc = value ? this._mediaControl.addEventListener : this._mediaControl.removeEventListener;
// Hookup or unhook transport control handlers
eventFunc("playpressed", thisFunc.playFuncRef, false);
eventFunc("stoppressed", thisFunc.stopFuncRef, false);
eventFunc("pausepressed", thisFunc.pauseFuncRef, false);
eventFunc("playpausetogglepressed", thisFunc.playPauseToggleFuncRef, false);
thisFunc.registered = value;
}
catch (error) {
this._logFunction("registerForMultimediaEvents failed with '{0}'".format(error.toString()));
}
}
},
load: function load() {
var that = this;
console.log("Video Control: entered load");
//always reload html fragment on load(), avoiding invisible video issue when load() is called after DOM structure has been destroyed (for example, page manually reloaded)
// TODO: Load template only once i.e. for the first video.
this._loadPromise = WinJS.UI.Fragments.renderCopy(controlTemplateLocation, null)
.then(function mediaPlayback_loadRenderCopyComplete(documentFragment) {
console.log("Video Control: entered loadpromise.then");
that._containerElement.innerHTML = "";
that._containerElement.appendChild(documentFragment);
that._init();
that._controlsElement.setAttribute("data-mediatype", that._type);
console.log("Video Control: leavingt loadpromise.then");
return WinJS.Resources.processAll(that._containerElement);
},
function mediaPlayback_loadRenderCopyError(errorDetails) { console.log("An error while rendering the mediaplayback control template" + errorDetails); });
console.log("Video Control: leaving load");
return this._loadPromise;
},
loadContentId: function loadContentId(contentId) {
console.log("Video Control: entered loadContentId");
// We have to order the system transport control enabling/disabling as there is a change coming our way
// that will prevent the apps from playing in the background if this is not so ordered.
if (contentId) {
this._systemTransportControlsEnabled = true;
}
Utilities.removeClass(this._loadingScreen, "hiddenObject");
this._etw.mediaPlaybackSrcSet(contentId ? contentId.objId : "invalid source");
this._sequencer.Reset();
var resetPromise = this._resetMediaElement(this._currentMediaElement, contentId, true);
var that = this;
var resetPromiseNext = resetPromise.then(function mediaPlayback_resetPromiseComplete() {
Utilities.addClass(that._loadingScreen, "hiddenObject");
});
if (!contentId) {
this._systemTransportControlsEnabled = false;
}
console.log("Video Control: leavingt loadContentId");
return resetPromiseNext;
},
/// <summary>
/// Plays the current source ( if set)
/// </summary>
/// <returns>
/// None
/// </returns>
play: function play() {
console.log("Video Control: entered play");
this._dispatchEvent("abouttoplay");
if (this._playbackCapabilities === MediaAppControls.PlaybackCapabilities.fullScreenOnly && !this._inFullScreenMode) {
this._enterFullScreenMode();
}
// Hide CountdownUI if opened
if(this._isCountdownUIShowing) {
this._hideCountdownUI();
}
if (this._isAdPlaying()) {
if (!this._inFullScreenMode) {
this.hideControls(); //if in inlinePlayback mode, do not allow user to go fullscreen during video ad play, saw issue when enlarging Ad div,TODO: check if this is a bug on Ad Renderer side
}
return;
}
this._etw.mediaPlaybackPlayStart();
this._sequencer.Play();
},
_toggleFullScreen: function _toggleFullScreen() {
if (!this._inFullScreenMode) {
this._enterFullScreenMode();
} else {
this._leaveFullScreenMode();
}
},
_toggleScreenSize: function _toggleScreenSize(event) {
var currentMediaElmStyle = this._currentMediaElement.visual.style;
var isFullSize = (currentMediaElmStyle.width === "100%") && (currentMediaElmStyle.height === "100%");
this._userPreferenceVideoScreenSize = isFullSize ? "50%" : "100%";
this._adjustVideoSizeToScreen(!isFullSize);
if (event) {
this._stopEventPropagation(event);
};
this._screenSizeToggleButton.focus();
},
_enterFullScreenMode: function _enterFullScreenMode() {
if (this._inFullScreenMode) {
return;
}
var fsControl = this._fullScreenControl;
this._canResumePlayback = !this._sequencer.IsPaused();
if (fsControl) {
// Save style before going FS
var currentStyle = this._containerElement.style;
this._containerElement.oldStyle = {};
var oldStyle = this._containerElement.oldStyle;
oldStyle.width = currentStyle.width;
oldStyle.height = currentStyle.height;
if (!fsControl.tryEnterFullScreen(this._containerElement)) {
console.log("Video Control: Failed to enter fullscreen mode");
this._containerElement.oldStyle = null;
}
}
},
_onEnteredFullScreen: function _onEnteredFullScreen() {
this._inFullScreenMode = true;
this._fullScreenToggleButton.focus();
var currentStyle = this._containerElement.style;
currentStyle.width = "";
currentStyle.height = "";
// This hides the timeline bar to enable dp flip
this.playbackMode = MediaAppControls.PlaybackMode.full;
Utilities.addClass(this._containerElement, "fullScreen");
this._fullScreenToggleButton.setAttribute("aria-labelledby", "mediaPlaybackExitFullScreenAriaLabel");
this._changeControlSource(this._thumbElement, MediaAppControls.Control.gripper, MediaAppControls.ControlState.rest);
if (this._canResumePlayback) {
this._sequencer.Play();
}
this._canResumePlayback = null;
this._syncTimeAndProgress(true);
},
_onExitedFullScreen: function _onExitedFullScreen() {
// restore old style
var currentStyle = this._containerElement.style;
var oldStyle = this._containerElement.oldStyle;
if (oldStyle) {
currentStyle.width = oldStyle.width;
currentStyle.height = oldStyle.height;
this._containerElement.oldStyle = null;
}
this.playbackMode = MediaAppControls.PlaybackMode.partial;
Utilities.removeClass(this._containerElement, "fullScreen");
this._fullScreenToggleButton.setAttribute("aria-labelledby", "mediaPlaybackEnterFullScreenAriaLabel");
this._inFullScreenMode = false;
this._changeControlSource(this._thumbElement, MediaAppControls.Control.gripper, MediaAppControls.ControlState.rest);
this._fullScreenToggleButton.focus();
if (this._canResumePlayback) {
this._sequencer.Play();
}
this._canResumePlayback = null;
this._syncTimeAndProgress(true);
this.showControls(true);
},
_leaveFullScreenMode: function _leaveFullScreenMode() {
if (!this._inFullScreenMode) {
return;
}
var fsControl = this._fullScreenControl;
this.hideControls();
this._canResumePlayback = !this._sequencer.IsPaused() && (this._playbackCapabilities !== MediaAppControls.PlaybackCapabilities.fullScreenOnly);
//clean up ad stuff on leaving fullscreen mode
if (this._playbackCapabilities === MediaAppControls.PlaybackCapabilities.fullScreenOnly) {
if (this._videoRequiresAds && this._adStatus !== CommonJS.MediaApp.Controls.AdStates.AdCompleted) {
this._delayNavigateBack = true;
this._hideAdDivCleanupAdRenderer(false);
var that = this;
setTimeout(function () {
if (that) { //current video control object could be destroyed already
that._delayNavigateBack = false; //in case adControl's userEngaged===false is not fired,this timeout makes sure user can exit fullscreen 
}
}, 1000);
}
}
if (!this._legacyAdControl || (!this._delayNavigateBack && fsControl)) {
if (!fsControl.tryLeaveFullScreen()) {
console.log("Video Control: Failed to leave fullscreen mode");
}
}
// [Video DCR]: Support Continuous video playback
// Clear the countdown timer interval upon leaving fullscreen mode
if (this._shouldContinueNextVideo) {
this._clearCountdownTimer();
}
},
/// <summary>
/// Pauses the current media stream (if playing)
/// </summary>
/// <returns>
/// None
/// </returns>
pause: function pause() {
this._etw.mediaPlaybackPauseStart();
if (this._sequencer && !this._sequencer.IsPaused()) {
try {
this._sequencer.Pause();
} catch (error) { }
} else {
this._onStopping = false;
}
},
/// <summary>
/// Plays the current source if paused; pauses the current stream if playing
/// </summary>
/// <returns>
/// None
/// </returns>
togglePlayPause: function togglePlayPause() {
this._onPlayPauseToggleCommand();
},
/// <summary>
/// Stops the current media stream
/// </summary>
/// <returns>
/// None
/// </returns>
stop: function stop() {
this._onStopping = true;
this._sequencer.Pause();
this._dispatchEvent("stopped");
},
/// <summary>
/// Triggers the controls to be visible
/// </summary>
/// <returns>
/// None
/// </returns>
showControls: function showControls(forceShow, skipAutoHide) {
/// <param name="forceShow" optional="true"/>
/// <param name="skipAutoHide" optional="true"/>
this._showControls(forceShow, skipAutoHide);
this._showSeekBar(forceShow, skipAutoHide);
},
/// <summary>
/// Triggers the controls to hide
/// </summary>
/// <returns>
/// None
/// </returns>
hideControls: function hideControls() {
this._hideControls();
this._hideSeekBar();
},
/// <summary>
/// Subscribes to timeupdate events from MPT
/// </summary>
/// <returns>
/// None
/// </returns>
subscribeTimeUpdates: function subscribeTimeUpdates() {
// Make sure there is only one timeupdate event ever, so we remove it first everytime
var that = this._sequencer;
this.unsubscribeTimeUpdates();
this._addMediaEventListener(this._currentMediaElement, "timeupdate", function mediaElement_currentMediaElementTimeUpdate() { that._onTimeUpdate(); });
},
/// <summary>
/// Unsubscribes to timeupdate events from MPT
/// </summary>
/// <returns>
/// None
/// </returns>
unsubscribeTimeUpdates: function unsubscribeTimeUpdates() {
this._unsubscribeFromMediaEvent(this._currentMediaElement, "timeupdate");
},
/// <summary>
/// Resets both the controls timer and the seekbar timer if available
/// </summary>
/// <returns>
/// None
/// </returns>
resetAutoHideTimers: function resetAutoHideTimers(forceReset) {
this.resetAutoHideControlsTimer(forceReset);
this.resetAutoHideSeekbarTimer(forceReset);
},
/// <summary>
/// Reset the auto hide timer of the controls if available
/// </summary>
/// <returns>
/// None
/// </returns>
resetAutoHideControlsTimer: function resetAutoHideControlsTimer(forceReset) {
/// <param name="forceReset" optional="true"/>
if (this._controlHideTimeout || forceReset) {
var that = this;
this._removeControlsTimer();
this._controlHideTimeout = setTimeout(function mediaPlayback_controlHideTimeout() {
that._hideControls();
}, this._controlsAutoHideDuration);
}
},
/// <summary>
/// Reset the auto hide timer of the seekbar if available
/// </summary>
/// <returns>
/// None
/// </returns>
resetAutoHideSeekbarTimer: function resetAutoHideSeekbarTimer(forceReset) {
if (this._seekBarHideTimeout || forceReset) {
var that = this;
if (this._seekBarHideTimeout) {
clearTimeout(this._seekBarHideTimeout);
}
this._seekBarHideTimeout = setTimeout(function mediaElement_seekBarHideTimeout() {
that._hideSeekBar();
}, this._controlsAutoHideDuration);
}
},
/// <summary>
/// Subscribes to events on the control
/// Supported events:
/// 1. playing                      - when playing starts on the current media src
/// 2. pause                        - when currently playing media is paused
/// 3. ended                        - when the current track ends.
/// 4. stopped                      - when the current track ends and next track is not available
/// 5. nexttrack                    - when the users moves to next track (through swipe gesture, or next track button)
/// 7. previoustrack                - when the user moves to preious track (through swipe gesture, or next track button)
/// 8. error                        - when the underlying media element raises and error event
/// 9. showcontrols                 - when the playback controls are shown
/// 10. hidecontrols                - when the playback controls are hidden
/// </summary>
/// <returns>
/// None
/// </returns>
addEventListener: function addEventListener(eventName, handler) {
this._containerElement.addEventListener(eventName, handler, false);
},
/// <summary>
/// Unsubscribes from events
/// </summary>
/// <returns>
/// None
/// </returns>
removeEventListener: function removeEventListener(eventName, handler) {
this._containerElement.removeEventListener(eventName, handler, false);
},
/// <summary>
/// Sets gets a media stream as the current source
/// </summary>
src: {
get: function get() {
if (this._currentMediaElement.isVacant) {
return null;
} else {
return this._currentMediaElement.src;
}
},
set: function set(value) {
// We have to order the system transport control enabling/disabling as there is a change coming our way
// that will prevent the apps from playing in the background if this is not so ordered.
if (value) {
this._systemTransportControlsEnabled = true;
}
this._etw.mediaPlaybackSrcSet(value ? value.objId : "invalid source");
this._resetMediaElement(this._currentMediaElement, value, false);
if (!value) {
this._systemTransportControlsEnabled = false;
}
}
},
/// <summary>
/// Sets gets a media id as the current source - this uses the MediaLibrary component
/// 
/// </summary>
srcId: {
get: function get() {
if (this._currentMediaElement.isVacant) {
return null;
} else {
if (!this._currentMediaElement.isMediaLoadedFromLibrary) {
return this._currentMediaElement.src;
} else {
return this._sequencer.GetCurrentVideoId();
}
}
},
set: function set(value) {
this.loadContentId(value);
}
},
/// <summary>
/// Sets the media element's logo
/// </summary>
logoSrc: {
set: function set(value) {
this._setLogo(value);
}
},
/// <summary>
/// Sets the media element's attribution
/// </summary>
attributionSrc: {
set: function set(value) {
this._setAttribution(value);
}
},
/// <summary>
/// Sets the media element's title
/// </summary>
titleSrc: {
set: function set(value) {
this._setTitle(value);
}
},
/// <summary>
/// Sets the media element's poster
/// </summary>
posterSrc: {
set: function set(value) {
this._setPoster(value);
}
},
/// <summary>
/// Gets the media element's poster
/// </summary>
poster: {
get: function get() {
return this._posterImage.src;
}
},
/// <summary>
/// Retrieves the error from the underlying media element
/// </summary>
error: {
get: function get() {
return this._currentMediaElement.error;
}
},
/// <summary>
/// Gets the duration in seconds of the current media source
/// </summary>
duration: {
get: function get() {
return this._sequencer.Duration;
}
},
/// <summary>
/// Gets sets the current time (in seconds) of the currently played media
/// </summary>
currentTime: {
get: function get() {
return this._sequencer.CurrentTime;
},
set: function set(value) {
if (this._currentMediaElement.isVacant) {
console.log("Video Control: Attempted to seek an empty media element");
return;
}
this._etw.mediaPlaybackSeekStart(value);
var newTime = value;
if (newTime > this._sequencer.Duration) {
this._sequencer.CurrentTime = this._sequencer.Duration;
} else if (newTime < 0) {
this._sequencer.CurrentTime = 0;
} else {
this._sequencer.CurrentTime = newTime;
}
}
},
timeIndicatorText: {
get: function get() {
return this._getTimeIndicatorText();
}
},
elapsedTimeText: {
get: function get() {
return this._getElapsedTimeText();
}
},
remainingTimeText: {
get: function get() {
return this._getRemainingTimeText();
}
},
/// <summary>
/// Gets sets the playback rate of the currently played media
/// 1.0 is normal speed.
/// </summary>
rate: {
get: function get() {
return this._currentMediaElement.playbackRate;
},
set: function set(value) {
this._currentMediaElement.playbackRate = value;
}
},
/// <summary>
/// Orgiginal video width.
/// </summary>
videoWidth: {
get: function get() {
return this._currentMediaElement.videoWidth;
}
},
/// <summary>
/// Orgiginal video height.
/// </summary>
videoHeight: {
get: function get() {
return this._currentMediaElement.videoHeight;
}
},
/// <summary>
/// Seeks to the beginning of the current source (if set)
/// </summary>
/// <returns>
/// None
/// </returns>
seekToBeginning: function seekToBeginning() {
if (this._currentMediaElement.ended) {
this._currentMediaElement.load();
}
if (this.currentTime !== 0) {
this.currentTime = 0;
}
this._newTime = null;
},
addTextTrack: function addTextTrack(kind, label, language) {
/// <summary>
/// adds a TextTrack, see HTML5 spec for detauls
/// </summary>
/// <returns>
/// a MutableTextTrack object
/// </returns>
return this._currentMediaElement.addTrack(kind, label, language);
},
/// <summary>
/// Gets / sets the zoom factor of the currently played media
/// </summary>
zoom: {
get: function get() {
return this._currentMediaElement.msZoom;
},
set: function set(value) {
this._currentMediaElement.msZoom = value;
}
},
/// <summary>
/// Gets / sets muted state of the currently played media
/// "Full" or "Partial" or "Minimal"
/// </summary>
playbackMode: {
get: function get() {
return this._mode;
},
set: function set(value) {
this._mode = value;
this._updateControls();
this._contentElement.setAttribute("data-mode", this._mode);
var that = this._sequencer;
// Make sure we alway subscribe to media position change events again if not in full screen mode
this.subscribeTimeUpdates();
}
},
/// <summary>
/// Refreshes the media strip.
/// </summary>
refresh: function refresh() {
this._setupMediaStrip();
},
},
{
// Provide a singleton instance for apps that want to show video content full screen but don't
// need an inline element to show the control.
//TODO: investigate the possiblity of using one singleton for both FullScreen and InlinePlayback
instance: {
get: function get() {
if (!CommonJS.MediaApp.Controls.MediaPlayback_instance) {
// Create a hidden container for the video control to live
var hiddenVideoContainer = document.createElement("div");
WinJS.Utilities.addClass(hiddenVideoContainer, "hiddenObject");
var hiddenVideoContainerChild = document.createElement("div");
hiddenVideoContainer.appendChild(hiddenVideoContainerChild);
document.body.appendChild(hiddenVideoContainer);
CommonJS.MediaApp.Controls.MediaPlayback_instance = new CommonJS.MediaApp.Controls.MediaPlayback(
hiddenVideoContainerChild,
{
type: CommonJS.MediaApp.Controls.PlaybackType.video,
mode: CommonJS.MediaApp.Controls.PlaybackMode.partial,
playbackCapabilities: CommonJS.MediaApp.Controls.PlaybackCapabilities.fullScreenOnly
});
}
return CommonJS.MediaApp.Controls.MediaPlayback_instance;
}
},
isFullscreenVideoPlaying: function isFullscreenVideoPlaying() {
var contentElement = document.getElementById("mediaPlaybackContent");
return contentElement && contentElement.getAttribute("data-mode") === "full";
},
// Provide a singleton instance for inlineplayback, to avoid multiple HTML5 Video element memeory issue, we should always use inlineInstance to show video in clusters or Article Reader
inlineInstance: {
get: function get() {
if (!CommonJS.MediaApp.Controls.MediaPlayback_inlineInstance) {
var hiddenVideoContainer = document.createElement("div");
WinJS.Utilities.addClass(hiddenVideoContainer, "mediaPlaybackContainer");
CommonJS.MediaApp.Controls.MediaPlayback_inlineInstance = new CommonJS.MediaApp.Controls.MediaPlayback(
hiddenVideoContainer,
{
type: CommonJS.MediaApp.Controls.PlaybackType.video,
mode: CommonJS.MediaApp.Controls.PlaybackMode.partial
});
}
return CommonJS.MediaApp.Controls.MediaPlayback_inlineInstance;
}
},
fullscreenPlayback: function fullscreenPlayback(contentSrc, videoOptions, videoList) {
if (!contentSrc) {
return;
}
var player = CommonJS.MediaApp.Controls.MediaPlayback.instance;
if (player) {
// [Video DCR]: Support Continuous video playback
// Set VideoOptions of next video
player._shouldContinueNextVideo = player._enableContinuousVideoPlayback && window.navigator.onLine;
if (player._shouldContinueNextVideo) {
if (videoList && videoList.length > 1) {
var currentIndex = videoOptions.index;
var nextIndex = currentIndex + 1;
nextIndex = nextIndex === videoList.length ? 0 : nextIndex;
player._nextContentSrc = videoList[nextIndex].videoSource;
player._nextVideoOptions = videoList[nextIndex];
player._videoList = videoList;
}
else {
player._shouldContinueNextVideo = false;
}
}
else {
if (player._fullScreenControl && player._fullScreenControl._element) {
var videoDivs = player._fullScreenControl._element.getElementsByClassName("mediaPlaybackInputHandler");
if (videoDivs && videoDivs.length > 0) {
return; //already have a video displayed, so skip
}
}
}
player.load()
.then(function fullscreenPlaybackLoadComplete() {
var inlinePlayer = CommonJS.MediaApp.Controls.MediaPlayback_inlineInstance;
if (inlinePlayer) {
if (inlinePlayer._videoRequiresAds) {
inlinePlayer._hideAdDivCleanupAdRenderer();
//reset ad status
inlinePlayer._adStatus = CommonJS.MediaApp.Controls.AdStates.AdNotStart;
}
inlinePlayer.pause(); //pause possible inlinePlayback singleton
try {
var parent = inlinePlayer._containerElement.parentNode;
if (parent) {
parent.removeChild(inlinePlayer._containerElement);
//restore original placeholder content in the div
if (parent.children && parent.children[0]) {
Utilities.removeClass(parent.children[0], "platformHide");
}
}
}
catch (e) {
player._errorLog("unexpected error when trying to remove inlineInstance from its current parent, error message: " + e.message);
}
}
if (videoOptions) {
if (videoOptions.thumbnail) {
player.posterSrc = videoOptions.thumbnail;
}
else {
player.posterSrc = "";
console.warn("Video Control: video poster image is not specified for video " + contentSrc);
}
if (videoOptions.source) {
player.attributionSrc = videoOptions.source;
}
else {
player.attributionSrc = "";
console.warn("Video Control: video attribution is not specified for video " + contentSrc);
}
if (videoOptions.sourceImageUrl) {
player.logoSrc = videoOptions.sourceImageUrl;
Utilities.removeClass(player._topBarLogo, "platformHide");
}
else {
player.logoSrc = "";
console.warn("Video Control: video logo is not specified for video " + contentSrc);
}
if (videoOptions.title) {
player.titleSrc = player._replaceHTMLEncodedChars(videoOptions.title);
}
else {
player.titleSrc = "";
console.warn("Video Control: video title is not specified for video " + contentSrc);
}
if (videoOptions.instrumentationCallback) {
player._instrumentationCallback = videoOptions.instrumentationCallback;
} else {
player._instrumentationCallback = "";
}
//instrumentationEntryPoint could be value 0
if (videoOptions.instrumentationEntryPoint || videoOptions.instrumentationEntryPoint === 0) {
player._instrumentationEntryPoint = videoOptions.instrumentationEntryPoint;
} else {
player._instrumentationEntryPoint = "";
}
}
if (contentSrc.indexOf("http://") >= 0) {
player.src = contentSrc;
} else {
player._resetToBlack();
player._enterFullScreenMode(); //response to user click first! since loadContentId() may take a few seconds
return player.loadContentId(contentSrc);
}
})
.done(function fullScreenPlayerPlayVideo() {
if (!player._disableVideoForGeoFence) {
if (document && (!document.msHidden)) {
player.play();
} else {
player._resumePlay = true;
}
}
});
}
},
//TODO: rename to capturePlayback afte all app switch to EntityCluster.js
inlinePlayback: function inlinePlayback(event, contentSrc, wrapperElement, videoOptions) {
if (!contentSrc) {
return;
}
var currentItem = wrapperElement;
//legacy code path, for current NewsCluster.js compatibility
if (event) {
var clusterItems = event.srcElement.getElementsByClassName("cluster-item");
}
var len = clusterItems ? clusterItems.length : 0;
if (!currentItem) {
var findItem = false;
if (!(event.detail && event.detail.item && event.detail.item.title)) {
return;
}
for (var i = 0; i < len; i++) {
currentItem = clusterItems[i];
var headlineElements = currentItem.getElementsByClassName("headline");
var videoTitleElements = currentItem.getElementsByClassName("video-title");
var videoTitleWithSubTitleElements = currentItem.getElementsByClassName("video-title-with-subtitle");
if (headlineElements && headlineElements[0] && event.detail.item.title.indexOf(headlineElements[0].innerText) >= 0) {
findItem = true;
break;
} else if (videoTitleElements && videoTitleElements[0] && event.detail.item.title.indexOf(videoTitleElements[0].innerText) >= 0) {
findItem = true;
break;
} else if (videoTitleWithSubTitleElements && videoTitleWithSubTitleElements[0] && event.detail.item.title.indexOf(videoTitleWithSubTitleElements[0].innerText) >= 0) {
findItem = true;
break;
}
}
}
//legacy code path end
var player = CommonJS.MediaApp.Controls.MediaPlayback.inlineInstance;
if (!player) {
player._errorLog("failed to get video control inlineInstance singleton, something goes wrong");
return;
}
if (currentItem || findItem) {
var videoDivs = currentItem.getElementsByClassName("mediaPlaybackInputHandler");
if (videoDivs && videoDivs.length > 0) {
return; //already have a video displayed, so skip
}
var containerEle = document.createElement("div");
containerEle.style.width = "100%";
containerEle.style.height = "100%";
WinJS.Utilities.addClass(containerEle, "mediaPlaybackContainer");
//replace original child DOM in cluster item div with Video DOM
if (currentItem.children && currentItem.children[0]) {
Utilities.addClass(currentItem.children[0], "platformHide");
}
player._containerElement.style.width = "100%";
player._containerElement.style.height = "100%";
player._resetToBlack();
//remove it from previous parent
var parent = player._containerElement.parentNode;
if (parent) {
parent.removeChild(player._containerElement);
//restore original placeholder content in the div
if (parent.children && parent.children[0]) {
Utilities.removeClass(parent.children[0], "platformHide");
}
}
currentItem.appendChild(player._containerElement);
player.load()
.then(function inlinePlaybackLoadComplete(event) {
if (videoOptions) {
if (videoOptions.thumbnail) {
player.posterSrc = videoOptions.thumbnail;
}
else {
console.warn("Video Control: video poster image is not specified for video " + contentSrc);
player.posterSrc = "";
}
if (videoOptions.source) {
player.attributionSrc = videoOptions.source;
}
else {
console.warn("Video Control: video attribution is not specified for video " + contentSrc);
player.attributionSrc = "";
}
if (videoOptions.title) {
player.titleSrc = player._replaceHTMLEncodedChars(videoOptions.title);
}
else {
console.warn("Video Control: video title is not specified for video " + contentSrc);
player.titleSrc = "";
}
if (videoOptions.sourceImageUrl) {
player.logoSrc = videoOptions.sourceImageUrl;
Utilities.removeClass(player._topBarLogo, "platformHide");
}
else {
console.warn("Video Control: video logo is not specified for video " + contentSrc);
player.logoSrc = "";
}
if (videoOptions.instrumentationCallback) {
player._instrumentationCallback = videoOptions.instrumentationCallback;
} else {
player._instrumentationCallback = "";
}
//instrumentationEntryPoint could be value 0
if (videoOptions.instrumentationEntryPoint || videoOptions.instrumentationEntryPoint === 0) {
player._instrumentationEntryPoint = videoOptions.instrumentationEntryPoint;
} else {
player._instrumentationEntryPoint = "";
}
}
if (contentSrc.indexOf("http://") >= 0) {
//usage: specify external source directly with http prefix
//e.g.  player.src = "http://video1.nytimes.com/video/2012/04/18/16471_1_metro-120418-intrepid_wg_16x9_xxxl_hb_mm.mp4";
player.src = contentSrc;
} else {
//Note: directly assign "player.srcId = contentSrc" will miss "resetElment.then" promise in LoadContentId(), this is causing problem when playing video with Ads enabled.
return player.loadContentId(contentSrc);
}
})
.done(function playerPlayVideo() {
if (!player._disableVideoForGeoFence) {
if (document && (!document.msHidden)) {
player.play();
} else {
player._resumePlay = true;
}
}	
});
}
}
})
});
// Helpers extending animations.js's animations to suit our needs (intervals, and static key frame cssrule)
Animation.snapHorizontal = function (target, posX, callback) {
var transitionEnd = function () {
target.removeEventListener("MSTransitionEnd", transitionEnd, false);
target.style.msTransitionProperty = "";
target.style.msTransitionDelay = "";
target.style.msTransitionDuration = "";
target.style.msTransitionTimingFunction = "";
if (callback) {
callback();
}
};
target.addEventListener("MSTransitionEnd", transitionEnd, false);
target.style.msTransitionProperty = "-ms-transform";
target.style.msTransitionDelay = "0ms";
target.style.msTransitionDuration = "300ms";
target.style.msTransitionTimingFunction = "ease-in-out";
target.style.msTransform = "translate(" + posX + "px,0px)";
};
Utilities.waitOnce = function (target, eventName) {
return new WinJS.Promise(function (complete) {
var eventHandler = function () {
target.removeEventListener(eventName, eventHandler, false);
complete();
};
target.addEventListener(eventName, eventHandler, false);
});
};
Utilities.addRemovableEventListener = function (object, eventName, eventHandler, capture) {
/// <param name="capture" optional="true"/>
if (!object) {
return;
}
if (!object.events) {
object.events = {};
}
if (object.events[eventName]) {
Utilities.removeEventListener(object, eventName, capture);
}
object.events[eventName] = eventHandler;
if (object.addEventListener) {
object.addEventListener(eventName, eventHandler, capture);
} else {
if (object.addListener) {
object.addListener(eventName, eventHandler);
}
}
};
Utilities.removeEventListener = function (object, eventName, capture) {
/// <param name="capture" optional="true"/>
if ((!object) || (!object.events)) {
return;
}
var eventHandler = object.events[eventName];
if (eventHandler) {
if (object.removeEventListener) {
object.removeEventListener(eventName, eventHandler, capture);
} else {
if (object.removeListener) {
object.removeListener(eventName, eventHandler);
}
}
object.events[eventName] = null;
}
};
})(WinJS);
(function appexCommonControlsMediaAppControlsInit(WinJS) {
"use strict";
var Animation = WinJS.UI.Animation;
var Utilities = WinJS.Utilities;
var Promise = WinJS.Promise;
var MediaAppControls = WinJS.Namespace.define("CommonJS.MediaApp.Controls", {
MediaContentType: {
VideoContent: "videoContent",
VideoPreRollAd: "videoPreRollAd"
},
//after video complete, Video Control will set state back to InitialState
MediaSequencerStates: {
InitialState: "InitialState",
ContentPlayback_Playing: "ContentPlayback_Playing",
ContentPlayback_Paused: "ContentPlayback_Paused",
},
AdStates: {
AdNotStart: "AdNotStart",
AdStarted: "AdStarted",
AdCompleted: "AdCompleted"
},
// TODO: Refactor this to use the State pattern
MediaSequencer: WinJS.Class.define(function (mediaPlayer) {
this._mediaPlayer = mediaPlayer;
this._contentVideoElement = mediaPlayer._currentMediaElement;
this._currentState = CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState;
},
{
_contentVideoObject: null,
_isAdElementVisible: false,
// Private methods
_assignContentVideo: function _assignContentVideo(contentObject) {
this._contentVideoObject = contentObject;
this._currentState = CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState;
this._mediaPlayer._assignContentVideo(contentObject);
},
_sendPerfPing: function _sendPerfPing(pingType) {
var currentVideoObject, currentVideoTime;
switch (this._currentState) {
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused:
currentVideoObject = this._contentVideoObject;
currentVideoTime = this._contentVideoElement.currentTime;
break;
case CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState:
// Nothing to do but not an error case
break;
default:
console.log("Video Control: Bad state in _sendPerfPing(): " + this._currentState);
break;
}
//Do Apppex Platform instrumentation Pingback here
switch (pingType) {
case CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayStart:
//reset all AppexPlatform instumentaion states
if (this._mediaPlayer._currentMediaElement.src.indexOf(this._mediaPlayer._resetVidSrc) >= 0) {
this._mediaPlayer._instrVideoStartDone = true;
this._mediaPlayer._instrVideo25Done = true;
this._mediaPlayer._instrVideo50Done = true;
this._mediaPlayer._instrVideo75Done = true;
this._mediaPlayer._instrVideoCompleteDone = true;
}
else {
this._mediaPlayer._instrVideoStartDone = false;
this._mediaPlayer._instrVideo25Done = false;
this._mediaPlayer._instrVideo50Done = false;
this._mediaPlayer._instrVideo75Done = false;
this._mediaPlayer._instrVideoCompleteDone = false;
}
this._mediaPlayer._instrVideo25Time = this.Duration * 0.25;
this._mediaPlayer._instrVideo50Time = this.Duration * 0.50;
this._mediaPlayer._instrVideo75Time = this.Duration * 0.75;
if (!this._mediaPlayer._instrVideoStartDone) {
this._mediaPlayer._instrVideoStart();
this._mediaPlayer._instrVideoStartDone = true;
}
break;
case CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayEnd:
if (!this._mediaPlayer._instrVideoCompleteDone) {
this._mediaPlayer._instrVideoComplete();
this._mediaPlayer._instrVideoCompleteDone = true;
}
break;
case CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayContinue:
if (this._mediaPlayer._instrVideo25Time && (this.CurrentTime > this._mediaPlayer._instrVideo25Time) && !this._mediaPlayer._instrVideo25Done) {
this._mediaPlayer._instrVideo25Done = true;
this._mediaPlayer._instrVideo25();
}
if (this._mediaPlayer._instrVideo50Time && (this.CurrentTime > this._mediaPlayer._instrVideo50Time) && !this._mediaPlayer._instrVideo50Done) {
this._mediaPlayer._instrVideo50Done = true;
this._mediaPlayer._instrVideo50();
}
if (this._mediaPlayer._instrVideo75Time && (this.CurrentTime > this._mediaPlayer._instrVideo75Time) && !this._mediaPlayer._instrVideo75Done) {
this._mediaPlayer._instrVideo75Done = true;
this._mediaPlayer._instrVideo75();
}
break;
}
//this is MSNCatalog ping
if (currentVideoObject && currentVideoObject.PingHandler && currentVideoTime) {
currentVideoObject.PingHandler.SendPing(pingType, currentVideoTime);
}
},
_updateScrubber: function _updateScrubber(videoObject) {
if (videoObject) {
if (videoObject.CanScrubVideo) {
this._mediaPlayer._enableScrubbing();
} else {
this._mediaPlayer._disableScrubbing();
}
}
},
// Event handlers - update the sequencer state and call the media playback
// object's event handler. 
_onEnded: function _onEnded(event) {
this._sendPerfPing(CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayEnd);
switch (this._currentState) {
// Content completed: reset state to not require another preroll
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
//in order to trigger onplaying event(which will set the _currentState correctly) when playing the video again after ending, 
//we have to explicitly call the system pause() before we call system play().
//this.Pause() only works when _currentState === playing, so we need first call it and then reset _currentState.
this.Pause();
this._currentState = CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState;
this._mediaPlayer._onEnded(event);
break;
default:
console.log("Video Control: Bad state in _onEnded(): " + this._currentState);
break;
}
},
_onError: function _onError(event) {
this._mediaPlayer._onError(event);
},
_setPausedState: function _setPausedState() {
switch (this._currentState) {
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
this._currentState = CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused;
break;
}
},
_onEnteredFullScreen: function _onEnteredFullScreen(event) {
if (event.detail && (event.detail.winControl !== this._mediaPlayer)) {
return;
}
this._setPausedState();
this._mediaPlayer._onEnteredFullScreen();
},
_onExitedFullScreen: function _onExitedFullScreen(event) {
if (event.detail && (event.detail.winControl !== this._mediaPlayer)) {
return;
}
this._setPausedState();
this._mediaPlayer._onExitedFullScreen();
},
_onMetaDataLoaded: function _onMetaDataLoaded(event) {
// The ad may have loaded when the content errored out - we're not
// ready to play yet.
if (!this._contentVideoObject) {
return;
}
this._updateScrubber(this._contentVideoObject);
this._mediaPlayer._onReadyForInteraction();
this._mediaPlayer._onMetaDataLoaded(event);
},
_onPause: function _onPause(event) {
switch (this._currentState) {
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
this._currentState = CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused;
break;
default:
console.log("Video Control: Bad state in _onPause(): " + this._currentState);
break;
}
this._mediaPlayer._onPause(event);
},
_onPlaying: function _onPlaying(event) {
var videoObject;
var videoDuration;
var isInitialState = false;
var mediaSequencerStates = CommonJS.MediaApp.Controls.MediaSequencerStates;
switch (this._currentState) {
case mediaSequencerStates.InitialState:
case mediaSequencerStates.ContentPlayback_Paused:
if (this._currentState === mediaSequencerStates.InitialState) {
isInitialState = true;
}
this._currentState = CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing;
var contentElement = this._contentVideoElement;
videoObject = this._contentVideoObject;
videoDuration = contentElement.duration;
//TODO: when fast switching between cluster items, sometimes videoobject will be null, need correct solution than this workaround
if (videoObject && videoObject.PingHandler) {
videoObject.PingHandler.SetDimensions(contentElement.clientWidth, contentElement.clientHeight, videoDuration);
}
break;
default:
console.log("Video Control: Bad state in _onPlaying(): " + this._currentState);
break;
}
if (isInitialState) {
this._sendPerfPing(CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayStart);
}
this._updateScrubber(videoObject);
this._mediaPlayer._onPlaying(event);
this._mediaPlayer._showPauseButton();
this._mediaPlayer._showControls();
//See bug 337079, enable interaction once video starts playing, instead of metadata loaded
if (!this._mediaPlayer._onPlayingEventFired) {
this._mediaPlayer._onPlayingEventFired = true;
this._mediaPlayer._thumbElement.focus(); //focus set to thumbElement (invisible) by default
} else {
//keyboard access for setting focus to backbutton/playbutton
if (document.activeElement === this._mediaPlayer._fullScreenToggleButton || (document.activeElement && document.activeElement.children &&
document.activeElement.children[0] && document.activeElement.children[0] === this._mediaPlayer._fullScreenToggleButton)) {
this._mediaPlayer._fullScreenToggleButton.focus();
} else {
this._mediaPlayer._thumbElement.focus();
}
}
},
_onPosterLoad: function _onPosterLoad(event) {
this._mediaPlayer._onPosterLoad(event);
},
_onRateChange: function _onRateChange(event) {
this._mediaPlayer._onRateChange(event);
},
_onSeeked: function _onSeeked(event) {
this._mediaPlayer._onSeeked(event);
},
_onTimeUpdate: function _onTimeUpdate(event) {
this._mediaPlayer._onTimeUpdate(event);
this._sendPerfPing(CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayContinue);
},
// Public methods
AddContent: function AddContent(contentObject) {
switch (contentObject.ContentType) {
case CommonJS.MediaApp.Controls.MediaContentType.VideoContent:
this._assignContentVideo(contentObject);
break;
default:
console.log("Unknown ContentType in sequencer: " + contentObject.ContentType);
break;
}
},
CurrentTime: {
get: function get() {
var videoElement;
switch (this._currentState) {
case CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState:
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused:
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
videoElement = this._contentVideoElement;
break;
}
if (videoElement) {
return videoElement.currentTime;
} else {
return 0;
}
},
set: function set(newTime) {
var videoElement;
switch (this._currentState) {
case CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState:
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused:
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
videoElement = this._contentVideoElement;
break;
}
if (videoElement) {
videoElement.currentTime = newTime;
}
},
},
Duration: {
get: function get() {
var videoElement;
switch (this._currentState) {
case CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState:
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused:
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
videoElement = this._contentVideoElement;
break;
}
if (videoElement) {
return videoElement.duration;
} else {
return 0;
}
},
},
GetCurrentVideoId: function GetCurrentVideoId() {
if (this._contentVideoObject) {
return this._contentVideoObject.VideoUuid;
} else {
return null;
}
},
HasEnded: function HasEnded() {
return ((this._currentState === CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState) && this._contentVideoElement.ended);
},
IsPaused: function IsPaused() {
switch (this._currentState) {
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
return false;
}
return true;
},
Pause: function Pause() {
switch (this._currentState) {
// Play Content
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Playing:
this._contentVideoElement.pause();
break;
default:
break;
}
},
Play: function Play() {
var videoObject, videoDuration;
var mediaPlayer = this._mediaPlayer;
/// <disable>JS2051.DoNotFallThroughInSwitchStatements</disable>
switch (this._currentState) {
// Play Ads
case CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState:
if (this._contentVideoElement.ended) {
this._contentVideoElement.pause();
this._contentVideoElement.currentTime = 0;
}
mediaPlayer._showContentVideoElement();
// Fall through - start playing the content
case CommonJS.MediaApp.Controls.MediaSequencerStates.ContentPlayback_Paused:
this._contentVideoElement.play();
break;
default:
console.log("Video Control: Bad state in Play(): " + this._currentState);
break;
}
/// <enable>JS2051.DoNotFallThroughInSwitchStatements</enable>
},
Reset: function Reset() {
this._currentState = CommonJS.MediaApp.Controls.MediaSequencerStates.InitialState;
this._contentVideoObject = null;
},
})
});
})(WinJS);
(function appexCommonControlsMediaAppControlsInit() {
"use strict";
var MediaAppControls = WinJS.Namespace.define("CommonJS.MediaApp.Controls", {
VideoPlaybackEvents: {
PlayStart: "PLAY_START",
PlayEnd: "PLAY_END",
PlayContinue: "PLAY_CONTINUE"
},
MsnVideosInstrumentationHandler: WinJS.Class.define(function msnVideosInstrumentationHandler_ctor(videoUuid, msnVideosFormatCode, appStoreUrl) {
var ppName = "";
var appVersion = encodeURIComponent(PlatformJS.Services.process.getAppVersionString());
var formatCode = msnVideosFormatCode;
var installationId = encodeURIComponent(Platform.Instrumentation.InstrumentationManager.instance.installationId);
var locale = Windows.Globalization.ApplicationLanguages.languages.toString().replace("-", "").toLowerCase();
var market = Platform.Globalization.Marketization.getCurrentMarket().toLowerCase();
//TODO: add this to app config
/*
pl= values would be associated with these apps:
http://appex.Sports.msn.com
http://appex.weather.msn.com
http://appex.travel.msn.com
http://appex.nyt.msn.com
http://appex.news.msn.com
http://appex.map.msn.com
http://appex.finance.msn.com
http://appex.bingsearch.msn.com
fr= values would be associated with these apps:
en-us_appex8_sports
en-us_appex8_weather
en-us_appex8_travel
en-us_appex8_nyt
en-us_appex8_news
en-us_appex8_map
en-us_appex8_finance
en-us_appex8_bingsearch
*/
//pl value
var playerLocation = PlatformJS.Services.appConfig.getString("VideoMSNPingPL");
if (!playerLocation) {
playerLocation = encodeURIComponent("http://appex.msn.com");
} else {
playerLocation = encodeURIComponent(playerLocation);
}
//fr value
var videoLocationInsideApp = PlatformJS.Services.appConfig.getString("VideoMSNPingFR");
if (!videoLocationInsideApp) {
videoLocationInsideApp = encodeURIComponent("en-us_appex8");
} else {
videoLocationInsideApp = encodeURIComponent(videoLocationInsideApp);
}
var playerTechnology = "AppEx";
var playerType = "appex_mediaplayer";
var urlBase = "http://catalog.video.msn.com/FraudDetect.aspx?";
var videoId = encodeURIComponent(videoUuid);
this._pingUrl = "{0}md={1}&av={2}&c8={3}&c9={4}_v{5}&d={6}&fc={7}&fr={8}&mkt={9}&pbStatus={10}&pl={11}&plt={12}&pt=pb&size={13}x{14}&t={15}&u={16}".format(urlBase, installationId, appVersion, playerTechnology, playerTechnology, appVersion, "{0}", formatCode, videoLocationInsideApp, market, "{1}", playerLocation, playerType, "{2}", "{3}", "{4}", videoId);
this._lastPingInterval = 0;
this._continuationPingIntervalSeconds = 20;
},
{
_pingUrl: "",
_lastPingInterval: 0,
_continuationPingIntervalSeconds: 20,
SetDimensions: function SetDimensions(videoWidth, videoHeight, videoDuration) {
this.videoWidth = videoWidth;
this.videoHeight = videoHeight;
this.videoDuration = videoDuration;
},
SendPing: function SendPing(eventType, currentTimeinSeconds) {
var pbStatus;
var tCode;
switch (eventType) {
case CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayStart:
pbStatus = "VideoPlaying";
tCode = 1;
break;
case CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayEnd:
pbStatus = "VideoPlayCompleted";
tCode = 11;
this._lastPingInterval = 0;
break;
case CommonJS.MediaApp.Controls.VideoPlaybackEvents.PlayContinue:
if (this._continuationPingIntervalSeconds === 0) {
console.log("Video Control: Video Playback instrumation ping interval was not set. Not sending continuation pings");
return;
}
var currentPingInterval = Math.floor(currentTimeinSeconds / this._continuationPingIntervalSeconds);
if (currentPingInterval > this._lastPingInterval) {
pbStatus = "VideoPlaying";
tCode = 8;
this._lastPingInterval = currentPingInterval;
} else {
return;
}
break;
}
var pingUrl = this._pingUrl.format(Math.round(this.videoDuration), pbStatus, this.videoWidth, this.videoHeight, tCode);
var xhrPing = WinJS.xhr({ url: pingUrl });
var that = this;
xhrPing.then(function msnVideosInstrumentationHandler_xhrComplete(xmlHttpRequestPing) {
console.log("Video Control: Playback ping sent to MSNCatalog: " + " Response status:" + xmlHttpRequestPing.status + " ;Ping IntervalSeconds: " + that._continuationPingIntervalSeconds + " ;pingUrl: " + pingUrl);
},
function msnVideosInstrumentationHandler_xhrError(xmlHttpRequestPing) {
if (xmlHttpRequestPing.status !== 399) {
console.log("Video Control: Error sending playback ping: " + " Response status:" + xmlHttpRequestPing.status + " ;Ping IntervalSeconds: " + that._continuationPingIntervalSeconds + " ;pingUrl: " + pingUrl);
}
else {
console.log("Video Control: Playback ping sent to MSNCatalog: " + " Response status:" + xmlHttpRequestPing.status + " ;Ping IntervalSeconds: " + that._continuationPingIntervalSeconds + " ;pingUrl: " + pingUrl);
}
});
}
}),
});
})();
(function appexCommonControlsMediaAppVideoEndpointProviderInit() {
"use strict";
WinJS.Namespace.define("CommonJS.MediaApp.Controls", {
VideoEndpointProvider: WinJS.Class.define(function () {
this._videoCatalogEndpoints = PlatformJS.Services.configuration.getList("VideoCatalogEndpoints");
},
{
_defaultCatalogPrefix: "http://edge1.catalog.video.msn.com/videobyuuid.aspx?uuid=",
getNextCatalogPrefix: function getNextCatalogPrefix() {
if (this._videoCatalogEndpoints.length > 0) {
// pick a random endpoint 
var randomNumber = Math.floor(Math.random() * 100000) + 10000;
var endpointIndex = randomNumber % this._videoCatalogEndpoints.length;
var catalogEntry = this._videoCatalogEndpoints[endpointIndex];
if (catalogEntry && catalogEntry.value) {
return catalogEntry.value;
}
}
return this._defaultCatalogPrefix;
}
}),
});
})();
/* File: VideoWrapper.js */
/********************************************************
* *
* � Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsVideoWrapperInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// videoWrapper control
/// when user click, replace with a global inlineplayback video element
/// </summary>
VideoWrapper: WinJS.Class.define(function VideoWrapper_ctor(element, options) {
this._element = element;
if (options) {
this._videoOptions = options.videoOptions;
if (options.subElement) {
element.appendChild(options.subElement);
}
this._videoList = options.videoList;
}
this._clickBinding = this._onClick.bind(this);
this._keyUpBinding = this._onKeyUp.bind(this);
this._element.addEventListener("click", this._clickBinding);
this._element.addEventListener("keyup", this._keyUpBinding);
}, {
// PRIVATE
_element: null,
_videoOptions: null,
_clickBinding: null,
_keyUpBinding: null,
_enableContinuousVideoPlayback: false,
_videoList: [],
_onClick: function _onClick(e) {
//currently, if we do not mannually setAttribute("data-disableclick", true) in EntityCluster,
//the default click handler will still be triggerred
var evt = e ? e : window.event;
if (evt.stopPropagation) {
evt.stopPropagation();
}
if (evt.cancelBubble !== null) {
evt.cancelBubble = true;
}
this._launchVideo();
},
_onKeyUp: function _onKeyUp(event) {
var keyCode = event.keyCode;
if (WinJS.Utilities.Key.enter === keyCode || WinJS.Utilities.Key.space === keyCode) {
this._launchVideo();
}
},
_launchVideo: function _launchVideo() {
if (this._videoOptions && this._videoOptions.fullscreen === "true") {
if (this._videoOptions.videoSource) {
CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback(this._videoOptions.videoSource, this._videoOptions, this._videoList);
}
}
else {
if (this._videoOptions && this._videoOptions.videoSource) {
CommonJS.MediaApp.Controls.MediaPlayback.inlinePlayback(null, this._videoOptions.videoSource, this._element, this._videoOptions);
}
}
},
// PUBLIC
dispose: function dispose() {
this._element.removeEventListener("click", this._clickBinding);
this._element.removeEventListener("keyup", this._keyUpBinding);
}
})
});
})();
