﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: SquareButton.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsSquareButtonInit() {
"use strict";
var STRINGS = {
L2OPENED: CommonJS.resourceLoader.getString("/Common/L2Opened") || "SubChannels Shown",
CLICKTOOPENL2: CommonJS.resourceLoader.getString("/Common/ClickToOpenL2") || "Click to show subChannel"
};
var NS = WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// A Special Button in top edgy for channel
/// </summary>
SquareButton: WinJS.Class.define(function squareButton_ctor (element, options) {
// Create the button
this.element = element || document.createElement("div");
element = this.element;
element.winControl = this;
CommonJS.setAutomationId(element);
WinJS.Utilities.addClass(element, "platformChannelButtonDiv win-focus-hide");
var buttonElement = this.buttonElement = document.createElement("button");
WinJS.Utilities.addClass(this.buttonElement, "platformChannelButton win-focus-hide");
var toggleElement = this.toggleElement = document.createElement("button");
WinJS.Utilities.addClass(this.toggleElement, "platformChannelToggle platformHidden win-focus-hide");
element.setAttribute("tabIndex", -1);
buttonElement.setAttribute("role", "link");
toggleElement.setAttribute("role", "button");
CommonJS.setAutomationId(buttonElement, element, "link");
CommonJS.setAutomationId(toggleElement, element, "toggle");
var blurHandler = this._blurHandler = this._onpointerblur.bind(this);
var focusHandler = this._focusHandler = this._onpointerfocus.bind(this);
var keyDownHandler = this._keyDownHandler = this._onKeyDown.bind(this);
buttonElement.addEventListener("blur", blurHandler);
buttonElement.addEventListener("focus", focusHandler);
buttonElement.addEventListener("keydown", keyDownHandler);
toggleElement.addEventListener("blur", blurHandler);
toggleElement.addEventListener("focus", focusHandler);
toggleElement.addEventListener("keydown", keyDownHandler);
this._initButton();
WinJS.UI.setOptions(this, options);
this.setImage(CommonJS.SquareButton.ImageType.SINGLE);
this._setAriaLabel(false);
this.buttonElement.setAttribute("aria-label", this._title || this.channelId);
element.appendChild(this.buttonElement);
element.appendChild(this.toggleElement);
},
{
/// <field>
/// The DOM element container
/// </field>
element: null,
/// <field>
/// The button element
/// </field>
buttonElement: null,
/// <field>
/// The toggle button element
/// </field>
toggleElement: null,
/// <field>
/// The parent Id of channel
/// </field>
parentId: null,
/// <field>
/// The channel Id
/// </field>
channelId: null,
/// <field>
/// Either the channel is a subchannel or not: true or false
/// </field>
isSubChannel: false,
/// <field>
/// Button container type, including L1, LF, LV or L2
/// </field>
buttonContainerType: null,
/// <field>
/// The images for the button.
/// </field>
images: null,
/// <field>
/// The image to use when the button is pressed
/// </field>
pressedIcon: null,
_title: null,
_label: null,
_icon: null,
_iconDiv: null,
_buttonDiv: null,
_active: false,
_expanded: false,
_hasSubChannels: false,
_blurHandler: null,
_focusHandler: null,
_keyDownHandler: null,
_toggleButtonColor: null,
_iconRest: null,
_doubleWide: null,
_onClick: null,
_onkeyDown: null,
_onKeyDown: function _onKeyDown(event) {
//In order to allow key accessibility to L2 buttons,
//enable "downArrow" key to set focus to the first button in the expanded subChannel
//and "upArrow" key to set focus back to its parent togglebutton in L1.
if (event.keyCode === WinJS.Utilities.Key.downArrow && this.expanded) {
event.stopPropagation();
event.preventDefault();
var subChannelsDiv = document.querySelector(".platformChannelBar.subChannels .platformChannelBarButtonGroup.show");
if (subChannelsDiv && subChannelsDiv.childNodes.length > 0) {
//in case that no "tab" key event ever happens, .focus stylesheet takes effect only when platformNavigationBar has class .tab
var navigationBar = document.querySelector(".platformNavigationBar");
if (navigationBar) {
WinJS.Utilities.addClass(navigationBar, "tab");
}
var firstButton = subChannelsDiv.firstChild;
if (firstButton && firstButton.childNodes.length > 0) {
firstButton.firstChild.focus();
}
}
} else if (event.keyCode === WinJS.Utilities.Key.upArrow && this.isSubChannel) {
event.stopPropagation();
event.preventDefault();
var parentButton = document.getElementById(this.parentId);
if (parentButton && parentButton.childNodes.length > 0) {
parentButton.lastChild.focus();
}
}
},
_onpointerblur: function _onpointerblur(event) {
WinJS.Utilities.removeClass(this.element, "focus");
},
_onpointerfocus: function _onpointerfocus(event) {
WinJS.Utilities.addClass(this.element, "focus");
},
_invokeButton: function _invokeButton(event, handler) {
if (handler) {
handler(event);
}
},
_initButton: function _initButton() {
var buttonHolder = this._buttonDiv = document.createElement("div");
buttonHolder.setAttribute("aria-hidden", "true");
WinJS.Utilities.addClass(buttonHolder, "platformEdgyIconHolder");
// Create the icon
var iconDiv = this._iconDiv = document.createElement("div");
WinJS.Utilities.addClass(iconDiv, "platformEdgyIconDiv");
var icon = this._icon = document.createElement("div");
WinJS.Utilities.addClass(icon, "platformEdgyIcon ");
iconDiv.appendChild(icon);
buttonHolder.appendChild(iconDiv);
// Create the label
var labelDiv = document.createElement("div");
WinJS.Utilities.addClass(labelDiv, "platformEdgyLabelDiv");
var label = this._label = document.createElement("div");
WinJS.Utilities.addClass(label, "platformEdgyLabel");
labelDiv.appendChild(label);
buttonHolder.appendChild(labelDiv);
this.buttonElement.appendChild(buttonHolder);
},
_setImage: function _setImage(imageUrl) {
if (imageUrl) {
var buttonElement = this.buttonElement;
var element = this.element;
WinJS.Utilities.addClass(element, "platformSquareButtonImage");
if (this.hasSubChannels) {
WinJS.Utilities.addClass(buttonElement, "platformSquareButtonImage image fitBoth");
} else {
WinJS.Utilities.addClass(buttonElement, "platformSquareButtonImage image fitHeight");
}
buttonElement.innerHTML = "";
var imageCard = new CommonJS.ImageCard(buttonElement, {
imageSource: {
url: imageUrl,
cacheId: "PlatformTopEdgyImageCache",
}
});
}
},
// Arial label for button:
// Navigation only: title
// Has sub Channel(L2 not shown): title + " Click to show subChannel"
// Has sub Channel(L2 shown): title + "SubChannels Shown"
_setAriaLabel: function _setAriaLabel(subChannelsShown) {
var element = this.toggleElement;
if (subChannelsShown) {
element.setAttribute("aria-label", STRINGS.L2OPENED);
}
else {
var ariaLabel = this.title || "";
if (this.hasSubChannels) {
ariaLabel += STRINGS.CLICKTOOPENL2;
}
element.setAttribute("aria-label", ariaLabel);
}
},
_setIcon: function _setIcon(value) {
var icon = this._icon;
icon.textContent = "";
if (value) {
if (value.substring(0, 4) === "url(") {
WinJS.Utilities.addClass(icon, "platformSquareButtonImage");
icon.style.backgroundImage = value;
}
else {
WinJS.Utilities.addClass(icon, value);
}
}
else {
WinJS.Utilities.addClass(this._iconDiv, "platformHidden");
}
},
/// <field>
/// Disabled
/// </field>
disabled: {
set: function set(value) {
var element = this.element;
if (value) {
WinJS.Utilities.addClass(element, "disabled");
} else {
WinJS.Utilities.removeClass(element, "disabled");
}
},
get: function get() {
var element = this.element;
return WinJS.Utilities.hasClass(element, "disabled");
}
},
/// <field>
/// IsSelected
/// </field>
isSelected: {
get: function get() {
return this.active || this.disabled;
}
},
/// <field>
/// either channel is selected, or button is direct parent of selected channel
/// </field>
active: {
set: function set(isButtonActive) {
var element = this.element;
this._active = isButtonActive;
if (isButtonActive) {
WinJS.Utilities.addClass(element, "currentActive");
}
else {
WinJS.Utilities.removeClass(element, "currentActive");
}
},
get: function get() {
return this._active;
}
},
/// <field>
/// either channel has subChannels or not
/// </field>
hasSubChannels: {
get: function get() {
return this._hasSubChannels;
},
set: function set(v) {
this._hasSubChannels = v;
if (v) {
this.element.setAttribute("aria-owns", "platformChannelBarButtonGroup_" + this.channelId);
WinJS.Utilities.removeClass(this.toggleElement, "platformHidden");
}
}
},
/// <field>
/// Set and get the backgroundColor of toggleButton
/// </field>
toggleButtonColor: {
get: function get() {
return this._toggleButtonColor;
},
set: function set(v) {
this._toggleButtonColor = v;
if (v) {
this.toggleElement.style.backgroundColor = v;
//For the pano without L2 view and toggleButton, 
//we want their logo to be left aligned (not stretch all over the button) 
//and fill the rest of the space on the right with the background color of the logo
if (!this._hasSubChannels) {
var buttonElement = this.buttonElement;
if (buttonElement) {
buttonElement.style.backgroundColor = v;
}
}
}
}
},
/// <field>
/// either channel can be expanded or not
/// </field>
canExpand: {
get: function get() {
var buttonContainerType = PlatformJS.Navigation.NavigationBar.ButtonContainerType;
return (this.buttonContainerType === buttonContainerType.L1
|| this.buttonContainerType === buttonContainerType.LV);
}
},
/// <field>
/// Set and get expanded
/// </field>
expanded: {
get: function get() {
return this._expanded;
},
set: function set(expand) {
this._expanded = expand;
var element = this.element;
if (expand) {
WinJS.Utilities.addClass(element, "expanded");
}
else {
WinJS.Utilities.removeClass(element, "expanded");
}
this._setAriaLabel(expand);
}
},
/// <field>
/// Set and get platformHidden
/// </field>
platformHidden: {
get: function get() {
return WinJS.Utilities.hasClass(this.element, "platformHidden");
},
set: function set(v) {
if (v) {
WinJS.Utilities.addClass(this.element, "platformHidden");
}
else {
WinJS.Utilities.removeClass(this.element, "platformHidden");
}
}
},
/// <field>
/// L2 specific
/// </field>
showIcon: {
get: function get() {
return WinJS.Utilities.hasClass(this.element, "platformShowIcon");
},
set: function set(v) {
if (v) {
WinJS.Utilities.addClass(this.element, "platformShowIcon");
}
else {
WinJS.Utilities.removeClass(this.element, "platformShowIcon");
}
}
},
/// <field>
/// The icon for the button. May be a CSS class or a URL
/// </field>
icon: {
set: function set(value) {
this._setIcon(value);
this._iconRest = value;
},
get: function get() {
return this._iconRest;
}
},
/// <field>
/// The doubleWide for the button.
/// </field>
doubleWide: {
set: function set(v) {
this._doubleWide = v;
var element = this.element;
if (v) {
WinJS.Utilities.addClass(element, "doubleWide");
}
else {
WinJS.Utilities.removeClass(element, "doubleWide");
}
},
get: function get() {
return this._doubleWide;
}
},
/// <field>
/// The title for the button
/// </field>
title: {
get: function get() {
return this._title;
},
set: function set(value) {
if (value) {
this._title = value;
this._label.textContent = value;
}
}
},
/// <field>
/// Set up images
/// </field>
setImage: function setImage(type) {
if (this.images) {
var imageUrl;
switch (type) {
case CommonJS.SquareButton.ImageType.SNAP:
imageUrl = this.images.snap;
break;
case CommonJS.SquareButton.ImageType.HOIST:
imageUrl = this.images.hoist;
break;
case CommonJS.SquareButton.ImageType.DOUBLE:
imageUrl = this.images.double;
break;
default:
imageUrl = this.images.single;
break;
}
this._setImage(imageUrl);
}
},
/// <field>
/// The onclick handler
/// </field>
onclick: {
set: function set(value) {
var that = this;
var element = this.element;
var onclick = this._onClick = function squareButton_onClick(event) {
that._invokeButton(event, value);
};
element.addEventListener("click", onclick, false);
var onkeydown = this._onkeyDown = function squareButton_onKeyDown(event) {
switch (event.keyCode) {
case WinJS.Utilities.Key.enter:
case WinJS.Utilities.Key.space:
that._invokeButton(event, value);
break;
default:
break;
}
};
//element.addEventListener("keydown", onkeydown, false);
}
},
/// <field>
/// Dispose function
/// </field>
dispose: function dispose() {
var element = this.element;
var buttonElement = this.buttonElement;
var toggleElement = this.toggleElement;
buttonElement.removeEventListener("blur", this._blurHandler);
buttonElement.removeEventListener("focus", this._focusHandler);
buttonElement.removeEventListener("keydown", this._keyDownHandler);
toggleElement.removeEventListener("blur", this._blurHandler);
toggleElement.removeEventListener("focus", this._focusHandler);
toggleElement.removeEventListener("keydown", this._keyDownHandler);
element.removeEventListener("click", this._onClick);
//element.removeEventListener("keydown", this._onkeyDown);
}
},
{
ImageType: {
SINGLE: 0,
DOUBLE: 2,
SNAP: 3,
HOIST: 4
}
}),
});
})();
/* File: SquareButtonContainer.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsSquareButtonContainerInit() {
"use strict";
/// <summary>
/// Button container that contains v2 buttons and an more button
/// </summary>
var NS = WinJS.Namespace.define("CommonJS", {
SquareButtonContainer: WinJS.Class.define(function squareButtonContainer_ctor(element, options) {
/// <summary>
/// constructor
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
this.element = element || document.createElement("div");
WinJS.Utilities.addClass(this.element, "platformChannelBar");
if (options && options.id) {
CommonJS.setAutomationId(this.element, null, null, "platformChannelBar_" + options.id);
}
else {
CommonJS.setAutomationId(this.element);
}
this.element.winControl = this;
WinJS.UI.setOptions(this, options);
this._init();
},
{
_scrollAnimation: WinJS.Promise.wrap(null),
_buttons: null,
_maxWidth: 0,
_maxHeight: 0,
_scrollLeft: 0,
_scrollTop: 0,
_pageWidth: 0,
_pageHeight: 0,
_itemPerRow: null,
_columnNum: null,
_fullWidth: true,
_map: {},
_total: 0,
// map to remember already rendered channel button groups.
_channelButtonGroups: {},
_pageCountArray: {},
_titleContainer: null,
_pageContainer: null,
_leftArrowContainer: null,
_rightArrowContainer: null,
_currentGroup: null,
_inAutoScroll: false,  // ignore scroll events when this is true, since we are using a timer to update the scroll position smoothly
/// <field>
/// The DOM element container
/// </field>
element: null,
/// <field>
/// Mode
/// </field>
mode: null,
/// <field>
/// Wide Scroll Mode: true or false
/// </field>
wideScrollMode: true,
/// <field>
/// title
/// </field>
title: null,
/// <field>
/// Active groupId
/// </field>
activeGroupId: null,
/// <field>
/// title
/// </field>
peak: null,
/// <field>
/// pageControl
/// </field>
pageControl: null,
/// <field>
/// Is content empty
/// </field>
isEmpty: false,
/// <field>
/// Title resource
/// </field>
titleResource: null,
/// <field>
/// button container
/// </field>
buttonContainer: null,
/// <field>
/// channelIdToSubGroupMap
/// </field>
channelIdToSubGroupMap: {},
// public functions
/// <field>
/// Get current active button
/// </field>
active: {
get: function get() {
var button = this.element.querySelector(".platformChannelButtonDiv.currentActive");
return button ? button.winControl : null;
}
},
/// <field>
/// Set and get fullWidth
/// </field>
fullWidth: {
set: function set(v) {
this._fullWidth = v;
if (!v) {
WinJS.Utilities.addClass(this.element, "showLV");
}
else {
WinJS.Utilities.removeClass(this.element, "showLV");
}
},
get: function get() {
return this._fullWidth;
}
},
/// <field>
/// Set and get active channelBar
/// </field>
channelBarActive: {
set: function set(v) {
if (v) {
WinJS.Utilities.addClass(this.element, "channelBarActive");
}
else {
WinJS.Utilities.removeClass(this.element, "channelBarActive");
}
},
get: function get() {
return WinJS.Utilities.hasClass(this.element, "channelBarActive");
}
},
/// <field>
/// Set and get platformHidden
/// </field>
platformHidden: {
get: function get() {
return WinJS.Utilities.hasClass(this.element, "platformHidden");
},
set: function set(v) {
if (v) {
WinJS.Utilities.addClass(this.element, "platformHidden");
}
else {
WinJS.Utilities.removeClass(this.element, "platformHidden");
}
}
},
/// <field>
/// Get disabled button
/// </field>
disabled: {
get: function get() {
var button = this.element.querySelector(".platformChannelButtonDiv.disabled");
return button ? button.winControl : null;
}
},
// public methods
/// <field>
/// Update title text
/// </field>
updateTitleText: function updateTitleText() {
if (this.titleResource) {
var titleButton = this.element.querySelector("button.platformChannelBarTitleButton");
var titleLabel = this.element.querySelector(".platformChannelBarTitleLabel");
var titleText = PlatformJS.Services.resourceLoader.getString(this.titleResource);
if (titleText) {
titleButton.innerText = titleText;
titleLabel.innerText = titleText;
}
}
},
/// <field>
/// Buttons command list to populate the container with
/// </field>
commands: {
set: function set(value) {
msWriteProfilerMark("Platform:ButtonContainer:commands:s");
var currentLayoutState = Windows.UI.ViewManagement.ApplicationView.value;
this.mode = currentLayoutState !== 2 ? PlatformJS.Navigation.StandardPresentationMode : PlatformJS.Navigation.SnappedPresentationMode;
this._buttons = value;
this._channelButtonGroups = {};
this._pageCountArray = {};
var groupContainer = this.buttonContainer;
// clear container, remove all buttons event listener before redraw
this.dispose();
this._buildChannelIdToDataMap(value);
//Reset the container
groupContainer.innerHTML = "";
if (this._buttons.length > 0 && this._buttons[0] && this._buttons[0].length > 0) {
this.selectGroup(this._buttons[0][0].parentId, true); // default is the first group
this.isEmpty = false;
}
msWriteProfilerMark("Platform:ButtonContainer:commands:e");
}
},
/// <field>
/// remove all contents
/// </field>
clearChannelBar: function clearChannelBar() {
this.platformHidden = true;
this._buttons = null;
this._total = 0;
this.isEmpty = true;
this.pageControl = null;
this._channelButtonGroups = {};
this._pageCountArray = {};
this.buttonContainer.innerHTML = "";
this._pageContainer.innerHTML = "";
this._leftArrowContainer.innerHTML = "";
this._rightArrowContainer.innerHTML = "";
},
/// <field>
/// return button data and how many buttons in this group
/// </field>
containerOf: function containerOf(channelId) {
return channelId ? this._map[channelId] : null;
},
/// <field>
/// Dispose
/// </field>
dispose: function dispose() {
this._scrollAnimation.cancel();
var buttons = this.buttonContainer.querySelectorAll(".platformChannelButtonDiv");
var length = buttons.length;
var button = null;
for (var i = 0; i < length; i++) {
button = buttons[i].winControl;
button.dispose();
}
},
/// <field>
/// Select group
/// </field>
selectGroup: function selectGroup(channelId, force) {
var layoutStale = false;
var current = null;
if (this._currentGroup) {
if (channelId !== this.activeGroupId) {
WinJS.Utilities.removeClass(this._currentGroup, "show");
this.activeGroupId = null;
}
else {
current = this._currentGroup;
}
}
if (force) {
current = this._renderOnDemand(channelId);
}
else if (!current) {
current = this._channelButtonGroups[channelId];
if (!current) {
current = this._renderOnDemand(channelId);
}
else {
layoutStale = true;
}
}
if (current) {
this._currentGroup = current;
WinJS.Utilities.addClass(current, "show");
this.activeGroupId = channelId;
}
// if button group is previously rendered, needs to recalculate page info.
if (layoutStale) {
if (this.mode !== PlatformJS.Navigation.SnappedPresentationMode) {
this._layoutStandard(channelId);
}
else {
this._layoutSnap(channelId);
}
}
},
/// <field>
/// Update paging
/// </field>
updatePaging: function updatePaging() {
if (this.pageControl) {
var pageNo = this.pageControl.pageNo;
this._scrollTo(pageNo);
}
},
/// <field>
/// Get first visible button
/// </field>
getFirstVisibleButton: function getFirstVisibleButton() {
var firstVisibleIndex = 0;
var button = null;
if (this.pageControl && this._itemPerRow) {
var pageNo = this.pageControl.pageNo;
firstVisibleIndex = (pageNo - 1) * this._itemPerRow;
}
var buttons = this.buttonContainer.querySelectorAll(".platformChannelButton");
var length = buttons.length;
if (length && length > firstVisibleIndex) {
button = buttons[firstVisibleIndex];
}
return button;
},
/// <field>
/// when width changed, caculate page width and button space and pageBar
/// </field>
updateContainerLayout: function updateContainerLayout() {
var groupId = this.activeGroupId;
this.selectGroup(groupId);
if (this.mode !== PlatformJS.Navigation.SnappedPresentationMode) {
this._layoutStandard(groupId);
}
else {
this._layoutSnap(groupId);
}
},
/// <field>
/// if button group is empty or the this group has only one button and is hoisted, hide the bar
/// </field>
updateVisibility: function updateVisibility(channelIdToHoist) {
// nothing to show, hide the bar
if (this._total === 0) {
this.platformHidden = true;
this.isEmpty = true;
return;
}
// clear old hidden button
this.platformHidden = false;
// Show/Hide hoisted LF channel
selector = ".platformChannelButtonDiv.platformHidden";
button = this.element.querySelector(selector);
buttonControl = button ? button.winControl : null;
if (buttonControl) {
buttonControl.platformHidden = false;
// put button back.
this.isEmpty = false;
}
// find new button to hide
var buttonMap = this.containerOf(channelIdToHoist);
var selector,
button,
buttonControl;
if (buttonMap) {
selector = '.platformChannelButtonDiv[id="' + channelIdToHoist + '"]';
button = this.element.querySelector(selector);
buttonControl = button ? button.winControl : null;
if (buttonControl) {
buttonControl.platformHidden = true;
if (buttonMap.count === 1) {
//Featured channel has only one button, hoist it and hide the bar.
this.isEmpty = true;
this.platformHidden = true;
}
else {
this.platformHidden = false;
this.isEmpty = false;
//Featured channel has more than one button, hoist one button and update the bar.
this._updatePageBar(this.activeGroupId, Math.ceil((buttonMap.count - 1) / this._itemPerRow));
}
}
} else {
//No button to hoist, update featured bar with all buttons
this._updatePageBar(this.activeGroupId, Math.ceil(this._total / this._itemPerRow));
}
},
// private methods
_init: function _init() {
var title = this._titleContainer = document.createElement("div");
WinJS.Utilities.addClass(title, "platformChannelBarTitle win-focus-hide");
title.setAttribute("tabIndex", -1);
var label = document.createElement("div");
WinJS.Utilities.addClass(label, "platformChannelBarTitleLabel win-focus-hide");
title.appendChild(label);
var titleButton = document.createElement("button");
WinJS.Utilities.addClass(titleButton, "platformChannelBarTitleButton win-focus-hide");
title.appendChild(titleButton);
titleButton.addEventListener("click", this._onClickHeader.bind(this));
titleButton.setAttribute("tabIndex", 0);
if (this.title) {
label.innerText = this.title;
titleButton.innerText = this.title;
}
var peak = document.createElement("div");
WinJS.Utilities.addClass(peak, "platformChannelBarPeak");
this.peak = peak;
var paging = this._pageContainer = document.createElement("div");
WinJS.Utilities.addClass(paging, "platformChannelBarPaging");
var left = this._leftArrowContainer = document.createElement("div");
WinJS.Utilities.addClass(left, "platformChannelBarArrowContainer platformChannelBarLeft");
var right = this._rightArrowContainer = document.createElement("div");
WinJS.Utilities.addClass(right, "platformChannelBarArrowContainer platformChannelBarRight");
this._updatePagingArrow(1);
var buttonGroupContainer = this.buttonContainer = document.createElement("div");
WinJS.Utilities.addClass(buttonGroupContainer, "platformChannelGroups");
this.element.appendChild(title);
this.element.appendChild(peak);
this.element.appendChild(paging);
this.element.appendChild(left);
this.element.appendChild(right);
this.element.appendChild(buttonGroupContainer);
buttonGroupContainer.addEventListener("scroll", this._onScroll.bind(this));
},
// flatten button structure
_buildChannelIdToDataMap: function _buildChannelIdToDataMap(value) {
var total = 0,
map = {},
button = null,
buttonGroup = null,
groupIndex = 0,
channelIndex = 0,
channelCount = 0,
groupCount = this._buttons ? this._buttons.length : 0;
for (groupIndex = 0; groupIndex < groupCount; groupIndex++) {
buttonGroup = this._buttons[groupIndex];
channelCount = buttonGroup ? buttonGroup.length : 0;
for (channelIndex = 0; channelIndex < channelCount; channelIndex++) {
button = buttonGroup[channelIndex];
if (button) {
total++;
// channelId -> { buttonData, groupChannelCount }
map[button.channelId] = { button: button, count: channelCount };
}
}
}
this._total = total;
this._map = map;
},
// if groupid invalid, then render first group
_renderOnDemand: function _renderOnDemand(groupId) {
var buttons = groupId ? this.channelIdToSubGroupMap[groupId] : null;
buttons = buttons || this._buttons[0];
var buttonGroupContainer = this._renderButtonGroup(buttons);
if (buttonGroupContainer) {
// insert group in dom tree.
var groupContainer = this.buttonContainer;
groupContainer.appendChild(buttonGroupContainer);
}
var mainButton = buttonGroupContainer.querySelector('[id=\"' + groupId + '\"]');
var location = WinJS.Navigation.location;
var channelId = location.channelId;
if (mainButton && groupId === channelId) {
mainButton.winControl.disabled = true;
}
return buttonGroupContainer;
},
// page indicator and arrows
_renderPageBar: function _renderPageBar(groupId) {
this._pageContainer.innerHTML = "";
this._leftArrowContainer.innerHTML = "";
this._rightArrowContainer.innerHTML = "";
var buttonContainer = this.buttonContainer;
var pageCount = groupId ? this._pageCountArray[groupId] : 1;
var pageNo = this.pageControl ? this.pageControl.pageNo : 1;
if (pageCount && pageCount > 1) {
// pageBar
//layout and pageNo change when viewing channel becomes invisible or visible
if (this._activeChannel && this._activeChannel.index) {
pageNo = Math.ceil(this._activeChannel.index / this._itemPerRow);
}
this.pageControl = new CommonJS.PagingBar(null, { pageCount: pageCount, pageNo: pageNo });
this._pageContainer.appendChild(this.pageControl.element);
//left arrow
var left = this._leftArrowContainer;
var leftButton = document.createElement("button");
left.appendChild(leftButton);
CommonJS.setAutomationId(leftButton, left.parentElement, "left");
leftButton.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/platform/scrollLeftAriaLabel"));
leftButton.addEventListener("click", this._onClickLeft.bind(this));
//right arrow
var right = this._rightArrowContainer;
var rightButton = document.createElement("button");
right.appendChild(rightButton);
CommonJS.setAutomationId(rightButton, right.parentElement, "right");
rightButton.addEventListener("click", this._onClickRight.bind(this));
rightButton.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/platform/scrollRightAriaLabel"));
this._updatePagingArrow(this.pageControl.pageNo);
this._goto(this.pageControl.pageNo, false);
}
else {
this.pageControl = null;
}
if (pageCount && pageCount > 1) {
if (this.mode !== PlatformJS.Navigation.SnappedPresentationMode) {
buttonContainer.style.overflowX = "auto";
buttonContainer.style.overflowY = "hidden";
}
else {
buttonContainer.style.overflowX = "hidden";
buttonContainer.style.overflowY = "auto";
}
}
else {
buttonContainer.style.overflowX = "hidden";
buttonContainer.style.overflowY = "hidden";
}
},
_onScroll: function _onScroll(evt) {
if (!this.pageControl) {
return;
}
if (this._inAutoScroll) {
return;
}
var newPage;
if (this.mode !== PlatformJS.Navigation.SnappedPresentationMode) {
var scrollLeft = this.buttonContainer.scrollLeft;
var deltaHoriz = scrollLeft - this._scrollLeft;
var currentPage = this.pageControl.pageNo;
if (deltaHoriz !== 0) {
this._scrollLeft = scrollLeft;
newPage = Math.round(this._scrollLeft / this._pageWidth) + 1;
if (newPage !== this.pageControl.pageNo) {
this._goto(newPage, false);
} else {
var subChannelsDiv = document.querySelector(".platformChannelBar.subChannels");
if (subChannelsDiv && !WinJS.Utilities.hasClass(subChannelsDiv, "platformHidden")) {
this._updateSubChannels(subChannelsDiv);
}
}
}
}
else {
var scrollTop = this.buttonContainer.scrollTop;
var deltaVert = scrollTop - this._scrollTop;
if (deltaVert !== 0) {
this._scrollTop = scrollTop;
newPage = Math.ceil(this._scrollTop / this._pageHeight) + 1;
if (newPage !== currentPage) {
this.pageControl.goto(newPage);
this._updatePagingArrow(this.pageControl.pageNo);
}
}
}
},
_updateSubChannels: function _updateSubChannels(subChannelsDiv) {
var subChannels = subChannelsDiv.winControl;
var isViewing = false;
var viewingDiv = document.querySelector(".platformChannelBar.viewing");
if (viewingDiv && !WinJS.Utilities.hasClass(viewingDiv, "platformHidden")) {
var viewing = viewingDiv.winControl;
if (viewing.activeGroupId === subChannels.activeGroupId) {
isViewing = true;
}
}
if (!isViewing) {
var buttonL1 = document.getElementById(subChannels.activeGroupId);
if (buttonL1) {
var buttonControl = buttonL1.winControl;
if (subChannelsDiv.parentNode && subChannelsDiv.parentNode.winControl.positionPeak) {
subChannelsDiv.parentNode.winControl.positionPeak(buttonControl);
}
}
}
},
_scrollTo: function _scrollTo(pageNo) {
var that = this;
if (this._inAutoScroll) {
return;
} 
pageNo = Math.max(1, pageNo);  // make sure dont try scrolling before 0 left or top
if (this.mode !== PlatformJS.Navigation.SnappedPresentationMode) {
this._scrollLeft = (pageNo - 1) * (this._pageWidth);
this._inAutoScroll = true;
var toValue = this._scrollLeft;
this._scrollAnimation = PlatformJS.Utilities.Transitions.applyTransitions(this.buttonContainer,
{
property: "scrollLeft", 
duration: 120,  
to: this._scrollLeft 
}); 
this._scrollAnimation.done(
function scroll_finished(e) {
that._inAutoScroll = false;
var subChannelsDiv = document.querySelector(".platformChannelBar.subChannels");
if (subChannelsDiv && !WinJS.Utilities.hasClass(subChannelsDiv, "platformHidden")) {
that._updateSubChannels(subChannelsDiv, true);
}
},
function error_handler(e)   { that._inAutoScroll = false; }
);
}
else {
this.buttonContainer.scrollTop = (pageNo - 1) * (this._pageHeight);
this._scrollTop = this.buttonContainer.scrollTop;
}
},
_goto: function _goto(newPage, autoPosition) {
this.pageControl.goto(newPage);
var newPageNo = this.pageControl.pageNo;
if (!autoPosition) {
this._scrollTo(newPageNo);
}
this._updatePagingArrow(newPageNo);
},
_updatePagingArrow: function _updatePagingArrow(newPageNo) {
if (newPageNo === 1) {
WinJS.Utilities.addClass(this._leftArrowContainer, "platformInvisible");
}
else {
WinJS.Utilities.removeClass(this._leftArrowContainer, "platformInvisible");
}
var pageCount = this.pageControl ? this.pageControl.pageCount : 1;
if (newPageNo === pageCount) {
WinJS.Utilities.addClass(this._rightArrowContainer, "platformInvisible");
}
else {
WinJS.Utilities.removeClass(this._rightArrowContainer, "platformInvisible");
}
},
_onClickLeft: function _onClickLeft(evt) {
var newPage = this.pageControl.pageNo;
this._goto(newPage - 1);
},
_onClickRight: function _onClickRight(evt) {
var newPage = this.pageControl.pageNo;
this._goto(newPage + 1);
},
_onClickHeader: function _onClickHeader(evt) {
var navBar = this.element.parentNode;
if (navBar) {
navBar.winControl.clearChannelBarState();
this.channelBarActive = true;
}
},
// factor out layout part.
// create button first, and then do layout.
// if no data change, only size change, can just call layout to update position
_renderButtonGroup: function _renderButtonGroup(buttons) {
var i = 0,
button = null,
buttonData = null;
var length = buttons.length;
if (length === 0) {
return null;
}
var element = this.element;
buttonData = buttons[0];
if(!buttonData) {
console.log("button contains invalid data");
return null;
}
var container = document.createElement("div");
var containerId = buttonData.parentId;
container.setAttribute("parentId", containerId);
WinJS.Utilities.addClass(container, "platformChannelBarButtonGroup");
container.id = "platformChannelBarButtonGroup_" + containerId;
//container.setAttribute("tabIndex", 0);
container.setAttribute("aria-controls", containerId);
this._channelButtonGroups[containerId] = container;
for (i = 0; i < length; i++) {
buttonData = buttons[i];
if (!buttonData) {
continue;
}
button = new CommonJS.SquareButton(CommonJS.createElement("div", element, "button" + i, buttonData.channelId), buttonData);
container.appendChild(button.element);
}
if (this.mode !== PlatformJS.Navigation.SnappedPresentationMode) {
this._layoutStandard(containerId);
}
else {
this._layoutSnap(containerId);
}
return container;
},
_calculateButtonGap: function _calculateButtonGap() {
var pageWidth = WinJS.Utilities.getTotalWidth(document.getElementById("platformPageArea")) - 40;
var buttonWidth = 158;
var itemPerRow = Math.floor(pageWidth / buttonWidth);
var gap = Math.floor((pageWidth - (itemPerRow * buttonWidth)) / (itemPerRow - 1));
if (gap < 6) {
itemPerRow -= 1;
gap = Math.floor((pageWidth - (itemPerRow * buttonWidth)) / (itemPerRow - 1));
}
this._itemPerRow = itemPerRow;
return gap;
},
// What this function does:
// 1. set scroll container width = pageCount & pageWidth
// 2. position each button to the right grid column
// 3. add last big div to fill out the last page.
// 4. update snap interval to the exact value.
// values calcuated:
// itemPerRow, pageCount, gap, acuratePageWidth
// if full page, use pageAreaWidth, 
// otherwise, Viewing Item is shown, itemPerRow -= 1 
_layoutStandard: function _layoutStandard(containerId) {
var i,
element = this.element;
var container = this._channelButtonGroups[containerId];
// only do layout when button is already rendered
if (!container) {
return;
}
var gap = this._calculateButtonGap();
// use page area with always. Sometime ChannelBar is hidden and width is unpredictable.
var maxWidth = WinJS.Utilities.getTotalWidth(document.getElementById("platformPageArea")) - 40;
var pageWidth = this._pageWidth = this._maxWidth = maxWidth;
var buttonWidth = 158;
var itemPerRow = this._itemPerRow = Math.max(Math.floor(pageWidth / buttonWidth), 1);
if (!this.fullWidth) {
itemPerRow -= 1;
}
this._itemPerRow = itemPerRow;
// set page width to exact value after gap estination: will be smaller
pageWidth = this._pageWidth = (buttonWidth + gap) * itemPerRow;
var columnNum = 1;
var buttons = container.querySelectorAll(".platformChannelButtonDiv");
var length = buttons.length;
var totalSingleButtonSpace = length;
var button = null;
for (i = 0; i < length; i++) {
button = buttons[i].winControl;
button.element.style.msGridColumn = columnNum++;
var rtl = (document.dir === "rtl");
if (!rtl) {
button.element.style.marginRight = (gap + "px");
button.element.style.marginLeft = ("0px");
}
else {
button.element.style.marginLeft = (gap + "px");
button.element.style.marginRight = ("0px");
}
// decide whether we can fit a wide button in this row.
// if can not, then use regular size
if (button.doubleWide && columnNum % itemPerRow !== 0) {
button.setImage(CommonJS.SquareButton.ImageType.DOUBLE);
button.element.style.width = 158 * 2 + gap + "px";
totalSingleButtonSpace += 1; // if double size, take one more space
}
var buttonType = PlatformJS.Navigation.NavigationBar.ButtonContainerType;
if (button.buttonContainerType === buttonType.LV) {
button.setImage(CommonJS.SquareButton.ImageType.HOIST);
}
}
var pageCount = Math.ceil(totalSingleButtonSpace / itemPerRow);
container.setAttribute("pageCount", pageCount);
this._pageCountArray[containerId] = pageCount;
this._columnNum = columnNum - 1;
var msGridColumns = "(auto)[" + this._columnNum + "]";
if (this.wideScrollMode) {
container.style.width = pageCount * pageWidth + "px";
this.buttonContainer.style.msScrollSnapPointsX = "snapInterval(0px," + pageWidth + "px)";
var placeHolderDiv = container.querySelector(".platformChannelSpaceHolder") || document.createElement("div");
WinJS.Utilities.addClass(placeHolderDiv, "platformChannelSpaceHolder");
msGridColumns += "1fr";
}
container.style.msGridRows = "1fr";
container.style.msGridColumns = msGridColumns;
this.buttonContainer.scrollLeft = 0;
this._scrollLeft = this.buttonContainer.scrollLeft;
this._renderPageBar(containerId);
},
_layoutSnap: function _layoutSnap(containerId) {
var i;
var container = this._channelButtonGroups[containerId];
// only do layout when button is already rendered
if (!container) {
return;
}
this._maxWidth = 0;
this._pageHeight = 55 * 8;
var buttons = container.querySelectorAll(".platformChannelButtonDiv");
var length = buttons.length;
var itemPerColumn = Math.min(8, length);
var pageCount = Math.ceil(length / itemPerColumn);
container.setAttribute("pageCount", pageCount);
this._pageCountArray[containerId] = pageCount;
this._columnNum = 1;
var button = null;
for (i = 0; i < length; i++) {
button = buttons[i].winControl;
button.element.style.msGridRow = i + 1;
button.setImage(CommonJS.SquareButton.ImageType.SNAP);
}
var msGridRows = "(auto)[" + length + "]";
if (this.wideScrollMode && pageCount > 1) {
msGridRows += " 1fr";
container.style.height = pageCount * this._pageHeight + "px";
var placeHolderDiv = this.platHolderDiv = document.createElement("div");
placeHolderDiv.style.msGridRow = i + 1;
container.appendChild(placeHolderDiv);
this.buttonContainer.style.msScrollSnapPointsY = "snapInterval(0px," + this._pageHeight + "px)";
}
container.style.msGridColumns = "1fr";
container.style.msGridRows = msGridRows;
this._renderPageBar(containerId);
},
//when hoisting a channel and hide that button in featured bar,
//the page count may be changed (e.g. initially 5 buttons in 2 pages, 
//after hoisting, only 4 buttons left in 1 page) and we need update the paging bar
_updatePageBar: function _updatePageBar(groupId, pageCount) {
if (!this.pageControl || pageCount === this.pageControl.pageCount) {
return;
}
var pageNo = this.pageControl.pageNo;
var newPageNo = pageNo > pageCount ? pageCount : pageNo;
if (pageCount && pageCount > 1) {
//if there are at least 2 pages both now and before,
//i.e. pagination is already there, just update the pagebar
if (this._pageContainer.innerHTML !== "") {
this._pageContainer.innerHTML = "";
this.pageControl = new CommonJS.PagingBar(null, { pageCount: pageCount });
this._pageContainer.appendChild(this.pageControl.element);
this._channelButtonGroups[groupId].style.width = pageCount * this._pageWidth + "px";
if (pageNo > pageCount) {
this._goto(newPageNo, false);
} else {
this.pageControl.goto(newPageNo);
}
this._updatePagingArrow(newPageNo);
} else {
//if before there is only 1 page and no pagination,
//render a new pageBar
this._renderPageBar(groupId);
}
} else {
//if 2 pages shrink to 1 page, hide pagination.
pageNo = this.pageControl.pageNo;
if (pageNo > pageCount) {
this._goto(1, false);
}
this.pageControl.pageCount = this.pageControl.pageNo = 1;
this._pageContainer.innerHTML = "";
this._leftArrowContainer.innerHTML = "";
this._rightArrowContainer.innerHTML = "";
this.buttonContainer.style.overflowX = "hidden";
this.buttonContainer.style.overflowY = "hidden";
}
},
})
});
WinJS.Class.mix(CommonJS.SquareButtonContainer,
WinJS.UI.DOMEventMixin,
WinJS.Utilities.createEventProperties("pageUpdated")
);
})();
/* File: PageBar.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsPageBarInit() {
"use strict";
/// <summary>
/// Button container that contains v2 buttons and an more button
/// </summary>
var NS = WinJS.Namespace.define("CommonJS", {       
// page start from 1
PagingBar: WinJS.Class.define(function pagingBar_ctor(element, options) {
this.element = element || document.createElement("div");
WinJS.Utilities.addClass(this.element, "platformPagination");
CommonJS.setAutomationId(this.element);
this.element.winControl = this;
WinJS.UI.setOptions(this, options);
this._init();
},
{
_pageNo: null,
_on: null,
/// <field>
/// The count of pages
/// </field>
pageCount: null,
/// <field>
/// Set and get the current page number.
/// </field>
pageNo: {
get: function get() {
return this._pageNo;
},
set: function set(v) {
this._pageNo = v < 1 ? 1 : v > this.pageCount ? this.pageCount : v;
}
},
/// <field>
/// Set and get the pageOn property
/// </field>
on: {
get: function get() {
return this._on;
},
set: function set(v) {
this._on = v;
if (v) {
WinJS.Utilities.addClass(bar, "pageOn");
}
else {
WinJS.Utilities.removeClass(bar, "pageOn");
}
}
},
// private methods
_init: function _init() {
var i = 0;
var bar = null;
var count = this.pageCount;
this.element.style.msGridColumns = "(auto)[" + count + "]";
for (i = 0; i < count; i++) {
bar = document.createElement("div");
bar.style.msGridColumn = i + 1;
this.element.appendChild(bar);
}
this.goto(this._pageNo);
},
// public methods
/// <field>
/// Reset pageBar and go back to the first page.
/// </field>
reset: function reset() {
this.goto(1);
},
/// <field>
/// Go to the specified page.
/// </field>
goto: function goto(pageNo) {
this.pageNo = pageNo;
var currentBar = this.element.querySelector(".pageOn");
if (currentBar) {
WinJS.Utilities.removeClass(currentBar, "pageOn");
}
var children = this.element.childNodes;
var newPage = this.pageNo;
var curr = children[newPage - 1];
if (curr) {
WinJS.Utilities.addClass(curr, "pageOn");
}
}
})
});
})();
/* File: NavigationBar.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
/// <dictionary target="comment">backstack,nav,onbefore,Stevenfe</dictionary>
/// <dictionary target="variable">arg,Config,obj</dictionary>
/// <dictionary target="member">Config,nav</dictionary>
(function appexPlatformNavigationBarInit() {
"use strict";
var PEAKWIDTH = 30;
var CHANNELBARBUTTONWIDTH = 158;
var LEFTARROWCONTAINERWIDTH = 16;
/// <summary>
/// Navigation Bar, i.e. top edgy, will show and help navigate to other channels
/// </summary>
var NS = WinJS.Namespace.define("PlatformJS.Navigation", {
NavigationBar: WinJS.Class.define(function navigationBar_ctor(element, options) {
/// <summary>
/// constructor
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
var that = this;
this.element = element ? element : document.createElement("div");
this.element.winControl = this;
WinJS.Utilities.addClass(this.element, "platformNavigationBar");
var otherButtonsContainer = document.createElement("div");
WinJS.Utilities.addClass(otherButtonsContainer, "platformNavOtherButtons");
this.element.appendChild(otherButtonsContainer);
window.addEventListener("resize", this._onLayoutUpdated.bind(this));
if (PlatformJS.isDebug) {
this._setupButtonsInDebugMode(otherButtonsContainer);
}
var channelBar = this._channelBar = new PlatformJS.Navigation.NavBarContainer(null,
{
title: CommonJS.getAppName(),
id: CommonJS.getAppName()
});
channelBar.channelBarActive = true;
channelBar.addEventListener("pageUpdated", function navBarContainer_pageUpdated(event) {
that._expandedActive = null;
that._hideL2(null, true);
});
var viewingChannelBar = this._viewingChannelBar = new PlatformJS.Navigation.NavBarContainer(null,
{
titleResource: "/platform/viewing",
id: "platform_viewing",
});
WinJS.Utilities.addClass(viewingChannelBar.element, "viewing");
viewingChannelBar.platformHidden = true;
var featuredChannelBar = this._featuredChannelBar = new PlatformJS.Navigation.NavBarContainer(null, {
titleResource: "/platform/featured",
id: "platform_featured",
});
WinJS.Utilities.addClass(featuredChannelBar.element, "featured");
featuredChannelBar.platformHidden = true;
var subChannelBar = this._subChannelBar = new PlatformJS.Navigation.NavBarContainer(null, { id: "subChannels" });
WinJS.Utilities.addClass(subChannelBar.element, "subChannels");
subChannelBar.platformHidden = true;
var element1 = this.element;
element1.appendChild(viewingChannelBar.element);
element1.appendChild(channelBar.element);
element1.appendChild(featuredChannelBar.element);
element1.appendChild(subChannelBar.element);
var bottomBar = this.bottomBar = document.createElement("div");
WinJS.Utilities.addClass(bottomBar, "platformBottomBar");
element1.appendChild(bottomBar);
WinJS.UI.setOptions(this, options);
},
{
_channelBar: null,
_viewingChannelBar: null,
_featuredChannelBar: null,
_subChannelBar: null,
_buttons: null,
_subChannelButtons: null,
_featuredButtons: null,
_expandedChannel: null,
_expandedActive: null,
_hoistable: {},
_animating: null,
_tabKeypressHandler: null,
/// <field>
/// The DOM element container
/// </field>
element: null,
/// <field>
/// List of channels
/// </field>
channels: {
get: function get() {
return this._buttons;
},
set: function set(value) {
if (this._animating) {
this._animating.cancel();
this._animating = null; //reset if channel updated
}
var buttons = this._buildCommands(value, buttonType.L1);
if (buttons && buttons.length > 0) {
this._channelBar.commands = buttons;
}
this._buttons = buttons;
}
},
/// <field>
/// List of featuredChannels
/// </field>
featuredChannels: {
get: function get() {
return this._featuredButtons;
},
set: function set(value) {
this._featuredChannelBar.platformHidden = true;
if (value && value.length > 0) {
var buttons = this._buildCommands(value, buttonType.LF);
if (buttons && buttons.length > 0) {
this._featuredChannelBar.commands = buttons;
this._featuredChannelBar.platformHidden = false;
}
this._featuredButtons = buttons;
}
if (this._featuredChannelBar.platformHidden) {
this._featuredChannelBar.clearChannelBar();
}
}
},
/// <field>
/// List of subChannels
/// </field>
subChannels: {
get: function get() {
return this._subChannelButtons;
},
set: function set(value) {
var channelIdToSubGroupMap = {};
this._hoistable = {};
var subChannelButtonsL1 = this._buildSubLevelCommands(value.L1, null, channelIdToSubGroupMap, buttonType.L1);
var subChannelButtonsLF = this._buildSubLevelCommands(value.LF, null, channelIdToSubGroupMap, buttonType.LF);
this._subChannelButtons = subChannelButtonsL1.concat(subChannelButtonsLF);
if (this._subChannelButtons && this._subChannelButtons.length > 0) {
this._subChannelBar.channelIdToSubGroupMap = channelIdToSubGroupMap;
this._subChannelBar.commands = this._subChannelButtons;
}
else {
this._subChannelBar.clearChannelBar();
}
}
},
/// <field>
/// List of expandedChannel
/// </field>
expandedChannel: {
get: function get() {
return this._expandedChannel;
},
set: function set(buttonControl) {
var old = this._expandedChannel;
if (old) {
old.expanded = false;
}
this._expandedChannel = buttonControl;
if (buttonControl) {
buttonControl.expanded = true;
}
}
},
//private functions
_setupGridTool: function _setupGridTool(otherButtonsContainer) {
///// Grid Tool
var button = document.createElement("button");
var gridButton = new WinJS.UI.AppBarCommand(button, { icon: "\uE063", extraClass: "appexSymbol" });
gridButton.label = "Grid Overlay";
gridButton.currentState = "gridOFF";
gridButton.onclick = function gridButton_onClick(event) {
var currentState = event.srcElement.winControl.currentState;
if (currentState === "gridOFF") {
currentState = event.srcElement.winControl.currentState = "gridALL";
} else if (currentState === "gridALL") {
currentState = event.srcElement.winControl.currentState = "grid10";
}
else if (currentState === "grid10") {
currentState = event.srcElement.winControl.currentState = "grid20";
}
else if (currentState === "grid20") {
currentState = event.srcElement.winControl.currentState = "grid100";
}
else if (currentState === "grid100") {
currentState = event.srcElement.winControl.currentState = "gridOFF";
}
var previousGrid = document.querySelector(".platformGridOverlay");
if (previousGrid) {
previousGrid.outerHTML = "";
}
if (currentState !== "gridOFF") {
var grid = document.createElement("div");
WinJS.Utilities.addClass(grid, "platformGridOverlay");
document.body.appendChild(grid);
if (currentState === "grid10" || currentState === "gridALL") {
var grid10 = document.createElement("div");
WinJS.Utilities.addClass(grid10, "grid10");
grid.appendChild(grid10);
}
if (currentState === "grid20" || currentState === "gridALL") {
var grid20 = document.createElement("div");
WinJS.Utilities.addClass(grid20, "grid20");
grid.appendChild(grid20);
}
if (currentState === "grid100" || currentState === "gridALL") {
var grid100 = document.createElement("div");
WinJS.Utilities.addClass(grid100, "grid100");
grid.appendChild(grid100);
}
}
};
otherButtonsContainer.appendChild(button);
///// End of Grid Tool
},
_setupCSSViewTool: function _setupCSSViewTool(otherButtonsContainer) {
//// Start CSS View Tool
var buttonCSSView = document.createElement("button");
var buttonCSSViewControl = new WinJS.UI.AppBarCommand(buttonCSSView, { icon: "\uE002", extraClass: "appexSymbol" });
buttonCSSViewControl.label = "CSS View";
buttonCSSViewControl.currentState = "viewOFF";
buttonCSSView.onclick = function buttonCSSView_onClick(event) {
var currentState = event.srcElement.winControl.currentState;
if (currentState === "viewOFF") {
currentState = event.srcElement.winControl.currentState = "viewON";
PlatformJS.Tools.init();
document.getElementById("cssview").style.display = "block";
}
else {
currentState = event.srcElement.winControl.currentState = "viewOFF";
document.getElementById("cssview").style.display = "none";
PlatformJS.Tools.resetToDefault();
}
var topAppBar = PlatformJS.Utilities.getControl(document.getElementById("platformNavigationBar")).hide();
if (topAppBar) {
topAppBar.hide();
}
};
otherButtonsContainer.appendChild(buttonCSSView);
//// End CSS View Tool
},
_setupGarbageCollectorTool: function _setupGarbageCollectorTool(otherButtonsContainer) {
///// Garbage Collector Tool
var buttonGCJS = document.createElement("button");
var buttonGCJSControl = new WinJS.UI.AppBarCommand(buttonGCJS, { icon: "\uE066", extraClass: "appexSymbol" });
buttonGCJSControl.label = "Garbage Collector JS";
buttonGCJS.onclick = function buttonGCJS_onClick(event) {
/// <disable> JS2064 </disable>
CollectGarbage();
/// <enable> JS2064 </enable>
};
otherButtonsContainer.appendChild(buttonGCJS);
var ButtonGCCSharp = document.createElement("button");
var ButtonGCCSharpControl = new WinJS.UI.AppBarCommand(ButtonGCCSharp, { icon: "\uE066", extraClass: "appexSymbol" });
ButtonGCCSharpControl.label = "Garbage Collector C#";
ButtonGCCSharp.onclick = function ButtonGCCSharp_onClick(event) {
Platform.Process.collect();
};
otherButtonsContainer.appendChild(ButtonGCCSharp);
///// End of Garbage Collector Tool
},
_setupBlankPageButton: function _setupBlankPageButton(otherButtonsContainer) {
///// Navigate to a blank page
var buttonNavigateToBlank = document.createElement("button");
var buttonNavigateToBlankControl = new WinJS.UI.AppBarCommand(buttonNavigateToBlank, { icon: "\uE054", extraClass: "appexSymbol" });
buttonNavigateToBlankControl.label = "Navigate to blank page";
buttonNavigateToBlank.onclick = function buttonNavigateToBlank_onClick(event) {
PlatformJS.Navigation.navigateToChannel("EmptyPage");
};
otherButtonsContainer.appendChild(buttonNavigateToBlank);
///// End of Navigate to a blank page
},
_setupDebugTool: function _setupDebugTool(otherButtonsContainer) {
///// Debug Tools
var buttonDebugTools = document.createElement("button");
var buttonDebugToolsControl = new WinJS.UI.AppBarCommand(buttonDebugTools, { icon: "\uE063", extraClass: "appexSymbol" });
buttonDebugToolsControl.label = "Debug Tools";
buttonDebugTools.onclick = function buttonDebugTools_onClick(event) {
var debugToolOptions = new Windows.UI.Popups.MessageDialog("Debug Tools Options");
debugToolOptions.commands.append(new Windows.UI.Popups.UICommand("Load String Resource Json"));
debugToolOptions.commands.append(new Windows.UI.Popups.UICommand("Load Configuration File"));
debugToolOptions.commands.append(new Windows.UI.Popups.UICommand("Load Versions File"));
debugToolOptions.showAsync().then(function debugTool_afterShow(command) {
var commandLabel = command.label;
switch (commandLabel) {
case "Load String Resource Json":
Platform.Resources.PlatformResourceManager.instance.loadResourcesFromFileAsync();
break;
case "Load Configuration File":
/// <disable> JS2064 </disable>
Platform.Configuration.ConfigurationManager.instance.loadConfigurationFromFileAsync();
/// <enable> JS2064 </enable>
break;
case "Load Versions File":
Platform.Utilities.VersionManagerUtil.instance.loadVersionFromDiskAsync();
break;
default:
//DO Nothing
break;
}
});
};
otherButtonsContainer.appendChild(buttonDebugTools);
///// End of Debug Tools 
},
_setupPreviewNavBar: function _setupPreviewNavBar(otherButtonsContainer) {
///// Preview navbar
var buttonDebugTools = document.createElement("button");
var buttonDebugToolsControl = new WinJS.UI.AppBarCommand(buttonDebugTools, { icon: "\uE063", extraClass: "appexSymbol previewFeaturedDataSourceButton" });
var previewFeaturedDataSource = Platform.Configuration.ConfigurationManager.instance.previewFeaturedDataSource;
var updatePreviewDataSourceButton = function updatePreviewDataSourceButton(previewFeaturedDataSource) {
if (previewFeaturedDataSource) {
buttonDebugToolsControl.label = "Disable Preview Mode";
} else {
buttonDebugToolsControl.label = "Enable Preview Mode";
}
};
updatePreviewDataSourceButton(previewFeaturedDataSource);
buttonDebugTools.onclick = function buttonDebugTools_onlick(event) {
previewFeaturedDataSource = !previewFeaturedDataSource;
if (CommonJS && CommonJS.State) {
CommonJS.State.isPreviewModeEnabled = previewFeaturedDataSource;
}
if (typeof NewsJS !== "undefined" && NewsJS.State && NewsJS.State.temp) {
NewsJS.State.temp.isPreviewModeEnabled = previewFeaturedDataSource;
}
Platform.Configuration.ConfigurationManager.instance.previewFeaturedDataSource = previewFeaturedDataSource;
updatePreviewDataSourceButton(previewFeaturedDataSource);
var currentMarket = Platform.Configuration.ConfigurationManager.instance.currentConfigurationFolderName;
PlatformJS.Cache.CacheService.getInstance("ConfigurationCache").removeEntry(currentMarket + "-Features");
var reloadFunction = function previewNavBar_reload() {
var channelManager = PlatformJS.Navigation.mainNavigator.channelManager;
channelManager.downloadFeaturesConfigAsync().then(function downloadFeaturesConfigAsync_complete() {
// load the features config even if it is blank (the downloadFeaturesConfigAsync will not overwrite the config if is blank)
Platform.Configuration.ConfigurationManager.instance.hasPendingFeaturesConfigUpdate = true;
channelManager.loadFeaturesConfigAsync(true).then(function loadFeaturesConfigAsync_complete() {
PlatformJS.Navigation.mainNavigator.onPreviewToggle();
channelManager.channelConfigChanged = true;
var md = Windows.UI.Popups.MessageDialog("Preview Mode Toggled!");
md.showAsync();
});
});
};
// Force reload of regional feeds.
CommonJS.LocalPano.LocalPanoManager.instance.getRegionsFeed(true).then(function getRegionsFeed_complete() {
reloadFunction();
},
function getRegionsFeed_error() {
reloadFunction();
}
);
};
otherButtonsContainer.appendChild(buttonDebugTools);
///// Flush Cache button
var buttonFlushCache = document.createElement("button");
var buttonFlushCacheControl = new WinJS.UI.AppBarCommand(buttonFlushCache, { icon: "clear", label: "Flush Caches" });
buttonFlushCache.onclick = function buttonFlushCache_onClick(event) {
var flushCacheOptions = new Windows.UI.Popups.MessageDialog("This operation will terminate the app. Do you wish to continue?");
flushCacheOptions.commands.append(new Windows.UI.Popups.UICommand("Ok"));
flushCacheOptions.commands.append(new Windows.UI.Popups.UICommand("Cancel"));
flushCacheOptions.showAsync().then(function flushCache_afterShow(command) {
if (command.label === "Ok") {
Platform.Process.signalFlushCachesAndShutdown();
}
});
};
otherButtonsContainer.appendChild(buttonFlushCache);
///// EC Template Selection button
var buttonECTemplate = document.createElement("button");
var buttonECTemplateControl = new WinJS.UI.AppBarCommand(buttonECTemplate, { icon: "\uE245", label: "EC Templates" });
buttonECTemplate.onclick = function (event) {
CommonJS.News.EntityCluster.TemplateSelectorDebuggingEnabled = !CommonJS.News.EntityCluster.TemplateSelectorDebuggingEnabled;
};
otherButtonsContainer.appendChild(buttonECTemplate);
///// End Preview navbar
},
_setupButtonsInDebugMode: function _setupButtonsInDebugMode(otherButtonsContainer) {
this._setupGridTool(otherButtonsContainer);
this._setupCSSViewTool(otherButtonsContainer);
this._setupGarbageCollectorTool(otherButtonsContainer);
this._setupBlankPageButton(otherButtonsContainer);
this._setupDebugTool(otherButtonsContainer);
this._setupPreviewNavBar(otherButtonsContainer);
},
_buildCommands: function _buildCommands(channels, buttonContainerType) {
var buttons = [],
i = 0,
channel = null,
edgyEnabled = false;
for (i = 0; i < channels.length; i++) {
channel = channels[i];
if (typeof channel.visible === "undefined" || channel.visible) {
buttons.push(this._createNavBarButton(channel, channel.id, buttonContainerType, i, buttons.length + 1));
edgyEnabled = true;
}
}
// Disable the top edgy when there are no visible channels
var edgy = PlatformJS.Utilities.getControl("platformNavigationBar");
edgy.disabled = !edgyEnabled;
return [buttons];
},
_buildSubLevelCommands: function _buildSubLevelCommands(channels, parentId, channelIdToSubGroupMap, buttonContainerType) {
if (!channels) {
return [];
}
var buttons = [],
i = 0,
channel = null;
for (i = 0; i < channels.length; i++) {
channel = channels[i];
var buttonGroup = this._buildSubChannel(channel, parentId, buttonContainerType);
if (buttonGroup.length > 0) {
channelIdToSubGroupMap[channel.id] = buttonGroup;
buttons.push(buttonGroup);
var nextLevel = this._buildSubLevelCommands(channel.subChannels, channel.id, channelIdToSubGroupMap, buttonType.L2);
if (nextLevel.length > 0) {
buttons = buttons.concat(nextLevel);
}
}
}
return buttons;
},
// add sub channel to a group
_buildSubChannel: function _buildSubChannel(channel, parentId, buttonContainerType) {
var subChannels = channel.subChannels;
var len = subChannels.length;
if (channel.visible === false) {
return [];
}
// mark all channels that might appear in LV
var hoistable = (parentId || buttonContainerType === buttonType.LF);
if (hoistable) {
this._hoistable[channel.id] = (this._createNavBarButton(channel, parentId ? parentId : channel.id, buttonType.LV, 0));
}
var buttons = [];
var i = 0;
for (i = 0; i < len; i++) {
var subChannel = subChannels[i];
if (subChannel.visible === undefined || subChannel.visible) {
buttons.push(this._createNavBarButton(subChannel, channel.id, buttonType.L2, i, buttons.length + 1));
}
}
return buttons;
},
_updateChannelSelection: function _updateChannelSelection() {
this._channelBar.updateChannels();
this._featuredChannelBar.updateChannels();
},
_initialFocus: function _initialFocus() {
var activeChannel = null;
if (!this._viewingChannelBar.platformHidden) {
activeChannel = this._viewingChannelBar.getFirstVisibleButton();
}
else {
activeChannel = this._channelBar.getFirstVisibleButton();
}
return activeChannel;
},
_tabKeypress: function _tabKeypress(event) {
if (event.keyCode === WinJS.Utilities.Key.tab) {
WinJS.Utilities.addClass(this.element, "tab");
this.element.removeEventListener("keydown", this._tabKeypressHandler, true);
}
},
_handleTabKeyPress: function _handleTabKeyPress(add) {
var tabKeypressHandler = this._tabKeypressHandler = this._tabKeypress.bind(this);
if (add) {
this.element.addEventListener("keydown", tabKeypressHandler, true);
}
else {
WinJS.Utilities.removeClass(this.element, "tab");
this.element.removeEventListener("keydown", tabKeypressHandler, true);
}
},
_updateTitleText: function _updateTitleText() {
this._viewingChannelBar.updateTitleText();
this._featuredChannelBar.updateTitleText();
},
_renderChannelBar: function _renderChannelBar(channelConfigChanged, standard, featured) {
this._updateTitleText();
if (this._animating) {
this._animating.cancel();
this._animating = null;
}
// update channel data
if (channelConfigChanged) {
this.expandedChannel = null;
this.channels = standard;
this.subChannels = { L1: standard, LF: featured };
this.featuredChannels = featured;
this._updateChannelSelection();
}
this._showLV();
this._updatePaging();
this._updateL2();
this._initChannelBarActive();
featured.forEach(this._featuredDisplayInstrumentation);
},
_featuredDisplayInstrumentation: function _featuredDisplayInstrumentation(featuredChannel) {
if (featuredChannel.displayInstrumentation) {
WinJS.xhr({ url: featuredChannel.displayInstrumentation }).done(null, function featuredDisplayInstrumentation_complete() { }); // Ignore success & failure
}
},
// update layout when state changed.
_onLayoutUpdated: function _onLayoutUpdated() {
var state = Windows.UI.ViewManagement.ApplicationView.value;
var snap = (state === Windows.UI.ViewManagement.ApplicationViewState.snapped);
if (snap) {
WinJS.Utilities.addClass(this.element, "platformNavigationBarSnap");
this._expandedActive = null;
}
else {
WinJS.Utilities.removeClass(this.element, "platformNavigationBarSnap");
}
this._hideL2(null, false);
},
// L2 drawer open condition
_updateL2: function _updateL2() {
var currentChannel = WinJS.Navigation.location.channelId;
var channelData = this._subChannelBar.containerOf(currentChannel);
if (channelData) {
currentChannel = channelData.button.parentId;
}
if (currentChannel === this._expandedActive) {
var selector = '[id=\"' + currentChannel + '\"]';
var expandedButton = this._viewingChannelBar.element.querySelector(selector) || this._channelBar.element.querySelector(selector);
var expandedChannel = expandedButton ? expandedButton.winControl : null;
if (expandedChannel && expandedChannel.hasSubChannels) {
this._showL2(expandedChannel, false);
}
else {
this._expandedActive = null;
this._hideL2(null, false);
}
}
else {
this._hideL2(null, false);
}
},
_updatePaging: function _updatePaging() {
this._channelBar.updatePaging();
this._featuredChannelBar.updatePaging();
this._subChannelBar.updatePaging();
},
// toggle L2 visibility and content
_toggleL2: function _toggleL2(buttonControl) {
var channelId = buttonControl.channelId;
// close L2 if already opened
if (!this._subChannelBar.platformHidden && this._subChannelBar.activeGroupId === channelId) {
this._hideL2(buttonControl, true);
this._expandedActive = null;
}
else {
this._showL2(buttonControl, true);
if (WinJS.Navigation.location.channelId === buttonControl.channelId ||
this._subChannelBar.activeGroupId === WinJS.Navigation.location.channelId) {
this._expandedActive = buttonControl.channelId;
}
}
},
_hoistChannel: function _hoistChannel(channelId, activeOnly) {
var viewingChannelBar = this._viewingChannelBar;
var channelBar = this._channelBar;
var featuredBar = this._featuredChannelBar;
var subChannelBar = this._subChannelBar;
var hoistBarVisible = viewingChannelBar.platformHidden;
if (!channelId) {
viewingChannelBar.platformHidden = true;
channelBar.fullWidth = true;
// Featured channels might change because of hoisted button
this._featuredChannelBar.updateVisibility(null);
if (hoistBarVisible !== viewingChannelBar.platformHidden) {
channelBar.updateContainerLayout();
}
return;
}
var hoistChannel = this._hoistable[channelId];
if (hoistChannel) {
viewingChannelBar.wideScrollMode = false;
viewingChannelBar.commands = [[hoistChannel]];
channelBar.fullWidth = false;
viewingChannelBar.platformHidden = false;
var button = viewingChannelBar.element.querySelector(".platformChannelButtonDiv");
if (button) {
var buttonControl = button.winControl;
buttonControl.expanded = false;
if (!activeOnly) {
buttonControl.disabled = true;
}
}
}
else {
viewingChannelBar.platformHidden = true;
channelBar.fullWidth = true;
}
// Featured channels might lose on hoisted button
this._featuredChannelBar.updateVisibility(hoistChannel ? channelId : null);
// if LV shown, then need to update L1 scroll information
if (hoistBarVisible !== viewingChannelBar.platformHidden) {
channelBar.updateContainerLayout();
}
},
_initChannelBarActive: function _initChannelBarActive() {
var channelId = WinJS.Navigation.location.channelId;
var buttonData = this._featuredChannelBar.containerOf(channelId);
this._featuredChannelBar.channelBarActive = buttonData ? true : false;
this._channelBar.channelBarActive = buttonData ? false : true;
},
// skip snap mode
_showLV: function _showLV() {
var state = Windows.UI.ViewManagement.ApplicationView.value;
var snap = (state === Windows.UI.ViewManagement.ApplicationViewState.snapped);
if (snap) {
return;
}
var channelId = WinJS.Navigation.location.channelId;
var buttonData = this._channelBar.containerOf(channelId)
|| this._featuredChannelBar.containerOf(channelId)
|| this._subChannelBar.containerOf(channelId);
var hoistChannelId = null;
if (buttonData) {
var buttonControl = buttonData.button;
channelId = buttonControl.channelId;
switch (buttonControl.buttonContainerType) {
case buttonType.LF:
hoistChannelId = channelId;
break;
case buttonType.L2:
if (buttonControl.hasSubChannels) {
hoistChannelId = channelId;
}
else {
hoistChannelId = buttonControl.parentId;
}
break;
default:
break;
}
}
this._hoistChannel(hoistChannelId, hoistChannelId !== channelId);
},
_hideL2: function _hideL2(buttonControl, animate) {
var that = this;
if (!this._animating) {
this._animating = this._hideL2Animation(animate);
this._animating.done(function hideL2Animation_complete() {
if (that.expandedChannel) {
that.expandedChannel.expanded = false;
}
that.expandedChannel = null;
that._animating = null;
});
}
},
_showL2: function _showL2(buttonControl, animate) {
if (buttonControl) {
var that = this;
if (!this._animating) {
var channelId = buttonControl.channelId;
this._subChannelBar.selectGroup(channelId);
this.positionPeak(buttonControl);
this._animating = this._showL2Animation(animate);
this._animating.done(function showL2Animation_complete() {
that.expandedChannel = buttonControl;
buttonControl.expanded = true;
that._animating = null;
});
}
}
},
_showL2Animation: function _showL2Animation(animate) {
var that = this;
var featured = this._featuredChannelBar.element;
var l2 = this._subChannelBar.element;
var animation;
var l2Height = "99px";
// no LF
if (this._featuredChannelBar.isEmpty) {
if (this._subChannelBar.platformHidden === true) {
l2.style.height = "0px";
this._subChannelBar.platformHidden = false;
this._subChannelBar.updatePaging();
if (animate) {
animation = WinJS.UI.executeTransition(
l2,
{
property: "height",
delay: 0,
duration: 120,
timing: "linear",
to: l2Height
});
}
else {
l2.style.height = l2Height;
animation = WinJS.Promise.wrap(null);
}
return animation.then(function showL2EmptyFeatured_complete() {
WinJS.Utilities.addClass(that._channelBar.element, "showL2");
return WinJS.Promise.wrap(null);
}, function showL2EmptyFeaturedNoAnimate_complete() {
l2.style.height = l2Height;
WinJS.Utilities.addClass(that._channelBar.element, "showL2");
return WinJS.Promise.wrap(null);
});
}
return WinJS.Promise.wrap(null);
}
else if (this._subChannelBar.platformHidden === true) {
var height = featured.offsetHeight;
var translate = "translateY(" + height + "px)";
l2.style.height = height - 1 + "px";
this._subChannelBar.platformHidden = false;
this._subChannelBar.updatePaging();
WinJS.Utilities.addClass(this._channelBar.element, "showL2");
if (animate) {
animation = WinJS.UI.executeTransition(
featured,
{
property: "-ms-transform",
delay: 0,
duration: 120,
timing: "linear",
from: "",
to: translate
});
}
else {
featured.style.transform = translate;
animation = WinJS.Promise.wrap(null);
}
return animation.then(function showL2WithFeatured_complete() {
that._featuredChannelBar.platformHidden = true;
that._subChannelBar.platformHidden = false;
return WinJS.Promise.wrap(null);
});
} else {
this._featuredChannelBar.platformHidden = true;
this._subChannelBar.platformHidden = false;
return WinJS.Promise.wrap(null);
}
},
_hideL2Animation: function _hideL2Animation(animate) {
var that = this;
var featured = this._featuredChannelBar.element;
var l2 = this._subChannelBar.element;
var animation;
// no LF
if (this._featuredChannelBar.isEmpty) {
if (animate) {
animation = WinJS.UI.executeTransition(
l2,
{
property: "height",
delay: 0,
duration: 120,
timing: "linear",
to: "0px"
});
}
else {
l2.style.height = "0px";
animation = WinJS.Promise.wrap(null);
}
return animation.then(function hideL2EmptyFeatured_complete() {
that._subChannelBar.platformHidden = true;
WinJS.Utilities.removeClass(that._channelBar.element, "showL2");
return WinJS.Promise.wrap(null);
});
} else {
var height = featured.offsetHeight;
this._featuredChannelBar.platformHidden = false;
if (animate) {
var animation1 = WinJS.UI.executeTransition(
this._subChannelBar.peak,
{
property: "opacity",
delay: 60,
duration: 40,
timing: "linear",
from: 1,
to: 0
});
var animation2 = WinJS.UI.executeTransition(
featured,
{
property: "-ms-transform",
delay: 0,
duration: 120,
timing: "linear",
to: ""
});
animation = WinJS.Promise.join([animation2, animation1]);
}
else {
featured.style.transform = "";
animation = WinJS.Promise.wrap(null);
//p1 = WinJS.Promise.wrap(null);
}
return animation.then(function hideL2WithFeatured_complete() {
that._subChannelBar.platformHidden = true;
that._subChannelBar.peak.style.opacity = 1;
WinJS.Utilities.removeClass(that._channelBar.element, "showL2");
return WinJS.Promise.wrap(null);
});
}
},           
_createNavBarButton: function _createNavBarButton(channel, parentId, buttonContainerType, index, sequence) {
/// <summary>
/// Creates a nav bar button
/// </summary>
/// <param name="channel">
/// The channel data. Example:
/// {
///     id: "Page0",
///     title: "Page0",
///     icon: "/TestApp/images/smalllogo.png",
///     pageInfo: { fragment: "/TestApp/html/page0.html", page: "TestApp.Page0" }
/// }
/// </param>
/// <returns>
/// Nothing
/// </returns>
var that = this;
var button = {};
var title;
try {
title = channel.isDisplayValue ? channel.title : (channel.title ? PlatformJS.Services.resourceLoader.getString(channel.title) : "");
title = title || channel.title;
} catch (e) {
if (channel.visible === "true") {
PlatformJS.Utilities.onError("You need to provide a localized string for the channel title");
}
}
var subTitle;
try {
subTitle = channel.isDisplayValue ? channel.subTitle : (channel.subTitle ? PlatformJS.Services.resourceLoader.getString(channel.subTitle) : "");
subTitle = subTitle || channel.subTitle;
} catch (e) {
if (channel.visible === "true") {
PlatformJS.Utilities.onError("You need to provide a localized string for the channel subtitle");
}
}
// first button in L2 is the home button. show subTitle as title
if (buttonContainerType === buttonType.L2 && channel.id === parentId) {
title = subTitle || title;
}
button.title = title;
button.icon = channel.icon;
button.pressedIcon = channel.pressedIcon;
button.channelId = channel.pageInfo.channelId;
button.isSubChannel = parentId !== button.channelId;
button.hasSubChannels = channel.subChannels.length > 0;
button.parentId = parentId;
button.index = sequence; //index of the button in its parent Container.
button.toggleButtonColor = channel.toggleButtonColor;
button.images = channel.images;
button.doubleWide = channel.doubleWide;
button.subTitle = subTitle;
button.showIcon = channel.showIcon;
button.buttonContainerType = buttonContainerType ? buttonContainerType : buttonType.L1;
button.onclick = function navBarButton_onClick(elt) {
msWriteProfilerMark("Platform:Navigation:NavBarClick:s");
// add instrumentation for nav bar button tapped here
var buttonContainerTypeReverseMap = {
1: 'L1',
2: 'LF',
3: 'LV',
4: 'L2'
};
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
Platform.Telemetry.ActionContext.navBar,
"",
Platform.Telemetry.UserActionOperation.click,
Platform.Telemetry.UserActionMethod.unknown,
0,
JSON.stringify({ ButtonType: buttonContainerTypeReverseMap[button.buttonContainerType], index: button.index }));
// this is used to set the entry point of next page like partner pano
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.navBar);
if (channel.hitInstrumentation) {
WinJS.xhr({ url: channel.hitInstrumentation }).done(null, function navBarButtonHitInstrumentation_complete() { });
}
var buttonElt = elt.currentTarget;
var buttonControl = buttonElt.winControl;
var channelIDInView = PlatformJS.Navigation.mainNavigator.channelIDInView;
// toggle only
var clickTarget = elt.target;
if (clickTarget && WinJS.Utilities.hasClass(clickTarget, "platformChannelToggle")) {
that._toggleL2(buttonControl);
}
// to navigate
else if (WinJS.Navigation.location.channelId !== channel.pageInfo.channelId ||
WinJS.Navigation.location.page !== channel.pageInfo.page ||
(channelIDInView && channelIDInView !== channel.pageInfo.channelId)) {
// click sub
if (buttonControl.buttonContainerType === buttonType.L2) {
that._expandedActive = buttonControl.parentId;
}
// click Lv or L1
else if (buttonControl.expanded) {
that._expandedActive = buttonControl.channelId;
}
else {
that._expandedActive = null;
}
if (!buttonControl.expanded && that._channelBar.containerOf(channel.pageInfo.channelId)) {
that._hideL2();
}
if (channel.state) {
channel.state.isNavBarEvent = true;
}
if (channel.isDeepLink) {
PlatformJS.Navigation.navigateTo(channel.pageInfo.fragment);
}
else {
WinJS.Navigation.navigate(channel.pageInfo, channel.state);
}
} else {
CommonJS.dismissAllEdgies();
that._panningToFront(); 
CommonJS.WindowEventManager.getInstance().dispatchEvent("samePageNav", event);
}
msWriteProfilerMark("Platform:Navigation:NavBarClick:e");
};
return button;
},
_panningToFront: function _panningToFront() {
if (document.querySelector(".platformPanorama")) {
//For normal panorama panel
if (Windows.UI.ViewManagement.ApplicationView.value !== Windows.UI.ViewManagement.ApplicationViewState.snapped) {
//non-snap mode
var containerPano = document.querySelector(".platformPanoramaViewport");
if (containerPano) {
var contentPano = containerPano.querySelector(".platformPanoramaSurface");
if (contentPano) {
this._horizontalScrollToLeft(containerPano, contentPano);
}
}
} else {
//snap mode
containerPano = document.querySelector(".win-viewport.win-vertical");
if (containerPano) {
contentPano = containerPano.querySelector(".win-surface");
if (contentPano) {
this._verticalScrollToTop(containerPano, contentPano);
}
}
}
} else {
if (document.querySelector(".ecv")) {
//For ecv view in destination page in Travel
containerPano = document.querySelector(".win-viewport.win-horizontal");
if (containerPano) {
contentPano = containerPano.querySelector(".win-surface");
if (contentPano) {
this._horizontalScrollToLeft(containerPano, contentPano);
}
}
}
}
},
_horizontalScrollToLeft: function _horizontalScrollToLeft(containerDiv, childDiv) {
if (!containerDiv || !childDiv) {
return;
}
var scrollLeft = containerDiv.scrollLeft;
var scrollMin = parseInt(containerDiv.currentStyle.msScrollLimitXMin);
if (scrollLeft === scrollMin) {
return;
}
var activeScrollAnimation = WinJS.UI.executeAnimation(childDiv,
{
property: "-ms-transform",
delay: 0,
duration: 500,
timing: "ease",
from: "translateX(-" + (scrollLeft - scrollMin) + "px)",
to: "none"
});
containerDiv.scrollLeft = scrollMin;
},
_verticalScrollToTop: function _verticalScrollToTop(containerDiv, childDiv) {
if (!containerDiv || !childDiv) {
return;
}
var scrollTop = containerDiv.scrollTop;
var scrollMin = parseInt(containerDiv.currentStyle.msScrollLimitYMin);
if (scrollTop === scrollMin) {
return;
}
var activeScrollAnimation = WinJS.UI.executeAnimation(childDiv,
{
property: "-ms-transform",
delay: 0,
duration: 500,
timing: "ease",
from: "translateY(-" + (scrollTop - scrollMin) + "px)",
to: "none"
});
containerDiv.scrollTop = scrollMin;
},
// public methods
/// <summary>
/// Switch the channel set. 
/// </summary>
/// <param name="channels">Array of channels</param>
switchChannelSet: function switchChannelSet(channels) {
this._channelBar.commands = this._buildCommands(channels, buttonType.L1);
this._channelBar.updateChannels();
},
/// <field>
/// used in snap mode. to toggle visible channel group
/// </field>
clearChannelBarState: function clearChannelBarState() {
this._channelBar.channelBarActive = false;
this._featuredChannelBar.channelBarActive = false;
},
/// <field>
/// Set the position of peak for subChannels to point to the center of its parent button. 
/// </field>
positionPeak: function positionPeak(buttonControl) {
var middlePoint = 0;
var channelBarOffsetLeft = Math.max(this._channelBar.element.getOffsetLeft(), LEFTARROWCONTAINERWIDTH);
var channelBarWidth = this._channelBar.buttonContainer.clientWidth;
var buttonContainerType = buttonControl.buttonContainerType;
if (buttonContainerType === buttonType.L1) {
buttonControl = this._channelBar.element.querySelector('[id=\"' + buttonControl.channelId + '\"]');
middlePoint += buttonControl.getOffsetLeft();
middlePoint -= this._channelBar.buttonContainer.scrollLeft;
buttonControl = buttonControl.winControl;
}
middlePoint = middlePoint >= 0 && middlePoint <= channelBarWidth - CHANNELBARBUTTONWIDTH ?
middlePoint + CHANNELBARBUTTONWIDTH / 2 : (middlePoint < 0 ? (CHANNELBARBUTTONWIDTH + middlePoint) / 2 : (channelBarWidth + middlePoint) / 2);
if (buttonContainerType === buttonType.L1) {
middlePoint += channelBarOffsetLeft;
}
//if the expanded channel is not partner pano and beak is moving out of the view, collapse L2
if ((WinJS.Utilities.hasClass(this._viewingChannelBar.element, "platformHidden") || buttonControl.channelId !== this._viewingChannelBar.activeGroupId)
&& (middlePoint < channelBarOffsetLeft || middlePoint > channelBarOffsetLeft + channelBarWidth)) {
this._channelBar.dispatchEvent("pageUpdated");
} else {
//otherwise, move the beak together with channelBar
this._subChannelBar.peak.setLeft(middlePoint - PEAKWIDTH / 2 + "px");
}
},
}, 
{
ButtonContainerType: {
L1: 1,
LF: 2,
LV: 3,
L2: 4
},
NavBarControl: null,
//static function
buildNavBar: function buildNavBar() {
/// <summary>
/// Builds the navigation bar
/// </summary>
/// <returns>
/// Nothing
/// </returns>
var that = this;
var navBar = document.getElementById("platformNavigationBar");
WinJS.UI.processAll(navBar).then(function navigator_buildNavBarProcessAllComplete() {
that.NavBarControl = navBar.children[0].winControl;
// add instrumentation for the app bar
var control = document.getElementById("platformNavigationBar").winControl;
control.addEventListener("aftershow", function navigator_onAfterShow() {
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
Platform.Telemetry.ActionContext.navBar,
"",
Platform.Telemetry.UserActionOperation.open,
Platform.Telemetry.UserActionMethod.unknown,
0,
"");
});
var firstTime = true;
// Delay building the navbar UX
var beforeShow = function navigator_onBeforeShow() {
var channelManager = PlatformJS.Navigation.mainNavigator.channelManager;
// if app hasn't set data till now, use data from appConfiguration.xml
if (!channelManager.channelConfigChanged && firstTime) {
channelManager.setChannelConfigData();
}
firstTime = false;
var featured = channelManager.featuredChannels;
var standard = channelManager.channels;
that.NavBarControl._handleTabKeyPress(true);
that.NavBarControl._renderChannelBar(channelManager.channelConfigChanged, standard, featured);
channelManager.channelConfigChanged = false;
};
var beforeHide = function navigator_onBeforeHide () {
if (that.NavBarControl) {
that.NavBarControl._handleTabKeyPress(false);
}
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
Platform.Telemetry.ActionContext.navBar,
"",
Platform.Telemetry.UserActionOperation.close,
Platform.Telemetry.UserActionMethod.unknown,
0,
"");
};
control.addEventListener("beforeshow", beforeShow);
control.addEventListener("beforehide", beforeHide);
// We are makeing sure bottom edgy does not steal focus from top edgy.
var activeChannel;
var afterShow = function navigator_onAfterShow() {
activeChannel = that.NavBarControl._initialFocus();
if (activeChannel) {
activeChannel.focus();
activeChannel.addEventListener("blur", onBlur);
control.addEventListener("keydown", dismissActiveChannel);
control.addEventListener("MSPointerDown", dismissActiveChannel);
}
};
var onBlur = function navigator_onBlur(event) {
control.removeEventListener("keydown", dismissActiveChannel);
control.removeEventListener("MSPointerDown", dismissActiveChannel);
if (activeChannel) {
activeChannel.removeEventListener("blur", onBlur);
activeChannel.focus();
}
};
var dismissActiveChannel = function navigator_dismissActiveChannel(event) {
control.removeEventListener("keydown", dismissActiveChannel);
control.removeEventListener("MSPointerDown", dismissActiveChannel);
if (activeChannel) {
activeChannel.removeEventListener("blur", onBlur);
}
};
control.addEventListener("aftershow", afterShow);
});
},
}),
});
var buttonType = PlatformJS.Navigation.NavigationBar.ButtonContainerType;
}
)();
/* File: NavBarContainer.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsNavBarContainerInit() {
"use strict";
WinJS.Namespace.define("PlatformJS.Navigation",
{
/// <summary>
/// Navigation bar container control
/// </summary>
NavBarContainer: WinJS.Class.derive(CommonJS.SquareButtonContainer, function navBarContainer_ctor(domElement, options) {
CommonJS.SquareButtonContainer.call(this, domElement, options);
var that = this;
window.addEventListener("resize", function navBarContainer_onResize() {
if (!that._buttons || !that._buttons[0] || that._buttons[0].length === 0) {
return;
}
var currentLayoutState = Windows.UI.ViewManagement.ApplicationView.value;
var mode = currentLayoutState !== 2 ? PlatformJS.Navigation.StandardPresentationMode : PlatformJS.Navigation.SnappedPresentationMode;
var modeSwitched = mode !== that.mode;
if (modeSwitched) {
that.commands = that._buttons;
}
that.updateChannels();
that.updateContainerLayout();
});
WinJS.Navigation.addEventListener("navigated", function navBarContainer_onNavigated() {
that.updateChannels();
});
},
{
_activeChannel: null,
activeChannel: {
get: function get() {
return this._activeChannel;
}
},
updateChannels: function updateChannels() {
var button = null,
i = 0;
var location = WinJS.Navigation.location;
var channelId = location.channelId; //location ? location.channelId : null;
var buttons = this.element.querySelectorAll(".platformChannelButtonDiv");
var len = buttons.length;
for (i = 0; i < len; i++) {
button = buttons[i].winControl;
if (channelId === button.channelId) {
this._activeChannel = button;
button.disabled = true;
}
else {
button.disabled = false;
}
}
},
})
});
})();
/* File: Navigator.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformNavigationInit() {
"use strict";
WinJS.Namespace.define("PlatformJS.Navigation",
{
mainNavigator: null,
enablingDiskPaging: true,
namespace: "PlatformJS.Navigation.",
/// <field>Enumeration indicating that the presentation mode is standard</field>
StandardPresentationMode: "standard",
/// <field>Enumeration indicating that the presentation mode is snapped</field>
SnappedPresentationMode: "snapped",
/// <field>Enumeration indicating that the navigation was a back navigation</field>
BackNavigation: "backNavigation",
/// <field>Enumeration indicating that the navigation was a new navigation</field>
NewNavigation: "newNavigation", 
/// 
/// <summary>
/// Main object to handle page navigation.
/// </summary>
Navigator: WinJS.Class.define(function navigator_ctor() {
/// <summary>
/// Constructor: Creates an instance of the navigator object.
/// It initializes the navigation framework
/// </summary>
/// <returns>
/// An instance of the navigator object.
/// </returns>
msWriteProfilerMark("Platform:Navigation:Constructor:s");
this._displayData = new PlatformJS.Utilities.DisplayData();
this._displayData.compute();
Platform.Telemetry.FlightRecorder.initializeDisplay(this._displayData.width, this._displayData.height, this._displayData.scale);
var that = this;
this.weightedNavigationCounter = new PlatformJS.WeightedNavigationCounter();
this._searchOnTypeAllowed = true;
try {
Windows.ApplicationModel.Search.SearchPane.getForCurrentView().showOnKeyboardInput = true;
} catch (e) {
// app doesnt support search contract 
this._searchOnTypeAllowed = false;
}
this.channelManager = new PlatformJS.Navigation.ChannelManager();
// Initial layout is always full-screen
if (this.channelManager.snappedChannels.length > 0) {
this.currentLayoutState = Windows.UI.ViewManagement.ApplicationView.value;
} else {
this.currentLayoutState = Windows.UI.ViewManagement.ApplicationViewState.fullScreenLandscape;
}
this.mode = this.currentLayoutState !== 2 ? PlatformJS.Navigation.StandardPresentationMode : PlatformJS.Navigation.SnappedPresentationMode;
this.lastIPage = null;
this.currentIPage = null;
CommonJS.LocalPano.LocalPanoManager.instance.addEventListener("regionChanged", function navigator_onRegionChanged() {
that._onRegionChange();
});
// Add the onbefore navigate event
WinJS.Navigation.addEventListener("beforenavigate", this._beforeNavigating.bind(this));
WinJS.Navigation.addEventListener("navigating", this._navigating.bind(this));
// Add the navigated event listener
WinJS.Navigation.addEventListener("navigated", function navigator_onNavigated(event) {
PlatformJS.Events.dispose();
PlatformJS.Events.init();
PlatformJS.Images.dispose();
PlatformJS.Images.init();
that.loadPage(event.detail);
// update local reference to the last known channel identifier
if (event.detail.location.channelId) {
that._lastChannelIdentifier = event.detail.location.channelId;
} else {
WinJS.Navigation.location.channelId = that._lastChannelIdentifier;
}
});
// Get the main DOM elements
this._pageArea = document.getElementById("platformPageArea");
// Read the configuration file to get the channels
this.channelManager.channels = this.mode === "standard" ? this.channelManager.standardChannels : this.channelManager.snappedChannels;
this.canNavigate = true;
// Build the navigation bar
PlatformJS.Navigation.NavigationBar.buildNavBar();
msWriteProfilerMark("Platform:Navigation:Constructor:e");
},
{
/// <field>
/// Can navigate or not: true or false
/// </field>
canNavigate: true,
/// <field>
/// Current mode: standard or snapped
/// </field>
mode: null,
/// <field>
/// Current layout: fullscreenlandscape, fullscreenportrait, filled or snapped
/// </field>
currentLayoutState: null,
/// <field>
/// Current IPage
/// </field>
currentIPage: null,
/// <field>
/// Last IPage
/// </field>
lastIPage: null,
/// <field>
/// Channel manager: configure, parse and maintain channel information
/// </field>
channelManager: null,
/// <field>
/// Id of the channel in view
/// </field>
channelIDInView: null,
/// <field>
/// Weighted navigation counter
/// </field>
weightedNavigationCounter: null,
/// <field>
/// Promise of loading feature channels
/// </field>
_loadFeaturesPromise: null,
// PRIVATE VARIABLES
_pageArea: null,
_currentStackSize: 0,
_currentPageTimer: null,
_lastChannelIdentifier: null,
_searchOnTypeAllowed: true,
_lastSuspendedTime: null,
_loadPagePromise: null,
_appBarDiv: null,
_backstackHistory: {},
_forwardstackHistory: {},
_lastContext: null,
_currentLocation: {},
//private methods
_setSearchOnType: function _setSearchOnType(value) {
if (this._searchOnTypeAllowed) {
try {
Windows.ApplicationModel.Search.SearchPane.getForCurrentView().showOnKeyboardInput = value;
} catch (e) {
// app doesnt support search contract 
this._searchOnTypeAllowed = false;
}
}
},
_beforeNavigating: function _beforeNavigating(event) {
this.canNavigate = false;
this.storeCurrentPage();
if (this.currentIPage) {
this.lastIPage = this.currentIPage;
}
this.currentIPage = null;
// remember the page to navigate
if (event.detail.location.channelId) {
this.channelIDInView = event.detail.location.channelId;
} else {
this.channelIDInView = null;
}
},
_navigating: function _navigating(event) {
this.enableSearchOnType();
if (this.lastIPage) {
if (this.lastIPage.onNavigateAway) {
// Check to see if we can go forward. If we can, then the navigation was a back navigation.
// If we cannot go forward, then we are at the top of the stack and the navigation was a new navigation.
// This assumes that we can never do a forward navigation in the history.
this.lastIPage.onNavigateAway(WinJS.Navigation.canGoForward ? PlatformJS.Navigation.BackNavigation : PlatformJS.Navigation.NewNavigation);
}
if (this.lastIPage.dispose) {
this.lastIPage.dispose();
}
this.lastIPage = null;
}
},
_logNavigationStart: function _logNavigationStart(context, impressionType, navMethod, partnerCode, attributes) {
// pass "" for impressionType, navMethod and partner code and null for attributes if not used
var logLevel = Platform.Telemetry.LogLevel.normal;
if (attributes) {
var jsonAttributes = JSON.stringify(attributes);
return Platform.Telemetry.FlightRecorder.logImpressionStartWithJsonAttributes(logLevel, context, impressionType, navMethod, partnerCode, jsonAttributes);
}
return Platform.Telemetry.FlightRecorder.logImpressionStart(logLevel, context, impressionType, navMethod, partnerCode);
},
_logNavigationEnd: function _logNavigationEnd() {
var logLevel = Platform.Telemetry.LogLevel.normal;
Platform.Telemetry.FlightRecorder.logImpressionExit(logLevel);
},
_logNavigationResume: function _logNavigationResume(impression) {
var logLevel = Platform.Telemetry.LogLevel.normal;
Platform.Telemetry.FlightRecorder.logImpressionResume(logLevel, impression);
},
_computeStackSize: function _computeStackSize() {
/// <summary>
/// Compute stack size
/// </summary>
/// <returns>
/// stack size = back + forward + current
/// </returns>
var stackSize = WinJS.Navigation.history.backStack.length + WinJS.Navigation.history.forwardStack.length;
if (!WinJS.Navigation.history.current.initialPlaceholder) {
stackSize += 1;
}
return stackSize;
},
_readEntryStateFromDiskAsync: function _readEntryStateFromDiskAsync(entry, mode, position) {
/// <summary>
/// Read entry from disk
/// </summary>
/// <param name="entry">
/// History stack object {page, state}
/// </param>
/// <param name="mode">
/// Presentation mode
/// </param>
/// <param name="position">
/// Position in the history
/// </param>
/// <returns>
/// Promise
/// </returns>
var that = this;
return new WinJS.Promise(function navigator_readEntryStateFromDiskAsyncPromiseInit(complete, error) {
var state = entry.state;
complete({ state: state, readFromDisk: false });
});
},
_clearPage: function _clearPage() {
if (!this._appBarDiv) {
this._appBarDiv = document.getElementById("ms-appbardiv");
}
if (this._appBarDiv) {
var appbars = WinJS.Utilities.query(".win-appbar", this._appBarDiv);
for (var i = 0; i < appbars.length; i++) {
if (appbars[i].id !== "platformNavigationBar") {
this._appBarDiv.removeChild(appbars[i]);
}
}
}
WinJS.Utilities.empty(this._pageArea);
var appBarControl = PlatformJS.Utilities.getControl(document.getElementById("platformNavigationBar"));
if (appBarControl) {
appBarControl.hide();
}
},
_clearHistory: function _clearHistory() {
/// <summary>
/// Clears the navigation history and returns the app to the Home channel
/// </summary>
/// <returns>
/// Nothing
/// </returns>
this._resetMemoryStructures();
this._clearPage();
},
// add a length limitation to history back stack. The limitation is read from configuration.xml
_limitHistoryLength: function _limitHistoryLength() {
var history = WinJS.Navigation.history;
var backStack = history.backStack;
if (!this._historyRestrictionConfig) {
var historyConfig = PlatformJS.Services.configuration.getDictionary("HistoryBackStackSizeLimitation");
var enableLimitation = (historyConfig.Enable && historyConfig.Enable.value) || false;
var triggerSize = (historyConfig.TriggerSize && historyConfig.TriggerSize.value) || 0;
var restrictionSize = (historyConfig.RestrictionSize && historyConfig.RestrictionSize.value) || 0;
triggerSize = Math.max(triggerSize, restrictionSize + 1);
this._historyRestrictionConfig = {
enable: enableLimitation,
triggerSize: triggerSize,
restrictionSize: restrictionSize
};
}
if (this._historyRestrictionConfig.enable && this._historyRestrictionConfig.restrictionSize > 2 && backStack.length >= this._historyRestrictionConfig.triggerSize) {
// The restiction on theory is 1, but it needs more code, and doesn't make sense...
var firstItem = backStack[0];
var numberToRemove = backStack.length - this._historyRestrictionConfig.restrictionSize;
var nextItem = backStack[numberToRemove + 1];
if (firstItem.location && nextItem.location && firstItem.location.fragment === nextItem.location.fragment) {
// it is possible that same fragment with different state can be totally different page, (articleReader, cloudPano). Anyway, to remove 1 more page will not be a critical issue here.
numberToRemove++;
}
backStack.splice(1, numberToRemove);
}
},
_switchHistory: function _switchHistory() {
/// <summary>
/// Restores the state of the app
/// </summary>
/// <returns>
/// Nothing
/// </returns>
var currentMode = this.mode === "snapped" ? "snapped" : "standard";
if (this._backstackHistory[currentMode] &&
this._forwardstackHistory[currentMode] &&
this._currentLocation[currentMode]) {
WinJS.Navigation.history.backStack = this._backstackHistory[currentMode];
WinJS.Navigation.history.forwardStack = this._forwardstackHistory[currentMode];
WinJS.Navigation.history.current = this._currentLocation[currentMode];
this._currentStackSize = this._computeStackSize();
// Navigate
if (WinJS.Navigation.history.current.initialPlaceholder || this.mode === "snapped") {
return this.channelManager.returnHomeChannel();
} else {
var current = WinJS.Navigation.history.current;
return WinJS.Promise.wrap({
pageInfo: current.location,
state: current.state,
mode: "print",
});
}
} else {
return this.channelManager.returnHomeChannel();
}
},
_resetMemoryStructures: function _resetMemoryStructures() {
/// <summary>
/// Resets the WinJS navigation data structures
/// </summary>
/// <returns>
/// Nothing
/// </returns>
WinJS.Navigation.history.backStack = [];
WinJS.Navigation.history.forwardStack = [];
WinJS.Navigation.history.current = {
location: "",
initialPlaceholder: true
};
this.canNavigate = true;
},
_switchMode: function _switchMode(mode, newLocationInfo) {
/// <summary>
/// Switch presentation modes
/// </summary>
/// <param name="mode">
/// PlatformJS.Navigation.StandardPresentationMode or PlatformJS.Navigation.SnappedPresentationMode
/// </param>
/// <param name="newLocationInfo" optional="true">
/// Location to navigate to after restoring the state
/// </param>
/// <returns>
/// Nothing
/// </returns>
// if newLoacationInfo is set, or there are snappedChannels, this is a seperate mode pages
var modeSwitched = newLocationInfo || false;
// Swap the channels only if there are available channels for the mode
if (this.channelManager.snappedChannels &&
this.channelManager.snappedChannels.length > 0) {
if (this.mode !== mode) {
if (mode === PlatformJS.Navigation.StandardPresentationMode) {
this.channelManager.channels = this.channelManager.standardChannels;
modeSwitched = true;
} else {
this.channelManager.channels = this.channelManager.snappedChannels;
modeSwitched = true;
}
}
} else if (!newLocationInfo) {
// this page is a dual mode page
if (this.mode !== PlatformJS.Navigation.StandardPresentationMode) {
// switch back from a seperate mode page, need to restore standard mode history.
this.mode = PlatformJS.Navigation.StandardPresentationMode;
this.channelManager.channels = this.channelManager.standardChannels;
}
}
if (modeSwitched) {
WinJS.Utilities.addClass(this._pageArea, "modeSwitching");
// Save the current page state
this.storeCurrentPage();
// Reset history data structures will clear out current IPage, but we need this to tell the previous page it needs to clean itself up
var lastIPage = this.currentIPage;
// Reset the history data structures
this._resetMemoryStructures();
this.currentIPage = lastIPage;
this.lastIPage = this.currentIPage;
// Switch the mode
this.mode = mode;
var that = this;
// Reset error messages
CommonJS.Error.removeError();
// We are pushing the restore method to msSetImmediate in order to avoid refreshing while the
// document.body.offsetWidth has not been resized yet.  This causes the panorama (amongst other
// controls) to only render a subset of the clusters since the body is still the width of the
// previous view.  Deferring the refresh allows the body to resize, enabling the panorama to
// render more than one cluster
msSetImmediate(function navigator_afterSwitchMode() {
// Restore the history of the mode
var restorePromise = null;
if (newLocationInfo && newLocationInfo.resetHistory) {
restorePromise = WinJS.Promise.wrap({});
} else {
restorePromise = that._switchHistory();
}
restorePromise.then(function navigator_switchModeRestoreComplete(target) {
// Change the nav bar
PlatformJS.Navigation.NavigationBar.NavBarControl.switchChannelSet(that.channelManager.channels);
// winJS.Navigation history has been switched, but currentIPage is still pointing to old-mode page.
that.currentIPage = null;
that._lastChannelIdentifier = null;
if (newLocationInfo) {
if (newLocationInfo.pageInfo) {
WinJS.Navigation.navigate(newLocationInfo.pageInfo, newLocationInfo.state);
}
} else if (target.pageInfo) {
that._navigating();
that.loadPage({
location: target.pageInfo,
state: target.state
});
} else {
that._navigateHome();
}
});
});
}
},
_navigateHome: function _navigateHome(state) {
/// <summary>
/// Navigates to home channel
/// </summary>
/// <returns>
/// Nothing
/// </returns>
this.navigateToChannel("Home", state);
},
// public methods
/// <field>
/// Return current impression
/// </field>
getCurrentImpression: function getCurrentImpression() {
var current = WinJS.Navigation.history.current; // WinJS.Navigation.history.current is always validate
var impression = current.impression || null;
return impression;
},
replaceCurrentImpression: function replaceCurrentImpression(context) {
/// <summary>
/// Replaces current impression and returns
/// the new impression
/// </summary>
var current = WinJS.Navigation.history.current; // WinJS.Navigation.history.current is always validate
if (!current.impression) {
// We maintain replace semantics so 
// this should be a no-op
console.log("replaceCurrentImpression() called on empty impression stack");
return null;
}
// End last impression
this._logNavigationEnd();
// Replace new impression at top of the stack
var newImpression = this._logNavigationStart(context,"","","");
current.impression = newImpression;
this._lastContext = context;
// Return new impression
return newImpression;
},
/// <field>
/// Disable searchOnType
/// </field>
disableSearchOnType: function disableSearchOnType() {
this._setSearchOnType(false);
},
/// <field>
/// Enable searchOnType
/// </field>
enableSearchOnType: function enableSearchOnType() {
this._setSearchOnType(true);
},
/// <field>
/// Navigate away
/// </field>
navigateAway: function navigateAway() {
if (this.currentIPage) {
this.lastIPage = this.currentIPage;
}
this._navigating();
},
/// <field>
/// Initialize mainNavigator
/// </field>
initialize: function initialize(clearState, isVersionUpdate) {
var that = this;
msWriteProfilerMark("Platform:Navigation:initialize:s");
var getChannelPromise = null; 
getChannelPromise = this.channelManager.returnHomeChannel();
this._clearHistory();
// delay loading the Features config until after the inital page has been loaded.
// unless loadFeatureConfigImmediately which is when launched from deeplink or pinnedApp 
// because they need features load sooner (is still async but the load promise is started now as opposed to post page load)
this._loadFeatureConfigAsyncBind = function() {
CommonJS.WindowEventManager.getInstance().removeEventListener("pagePopulated", that._loadFeatureConfigAsyncBind);
that._loadFeatureConfigAsyncBind = null; 
// doing msSetImmediate so that will create launch promise after the actual rendering now that the page is populated
msSetImmediate(that.loadFeaturesConfigAsync.bind(that));
};
CommonJS.WindowEventManager.getInstance().addEventListener("pagePopulated", this._loadFeatureConfigAsyncBind);
return getChannelPromise.then(function navigator_initComplete() {
return getChannelPromise;
});
},
loadFeaturesConfigAsync: function loadFeaturesConfigAsync() {
if (!this._loadFeaturesPromise) {
this._loadFeaturesPromise = this.channelManager.loadFeaturesConfigAsync();
}
return this._loadFeaturesPromise;
},
storeCurrentPage: function storeCurrentPage() {
/// <summary>
/// Store current page
/// </summary>
/// <returns>
/// Nothing
/// </returns>
if (this.currentIPage && WinJS.Navigation.history.current) {
// Store the state of the page
WinJS.Navigation.state = this.currentIPage.getPageState();
var currentMode = this.mode === "snapped" ? "snapped" : "standard";
this._backstackHistory[currentMode] = WinJS.Navigation.history.backStack;
this._forwardstackHistory[currentMode] = WinJS.Navigation.history.forwardStack;
this._currentLocation[currentMode] = WinJS.Navigation.history.current;
}
},
/// <field>
/// Reset App, clear history and navigate back to home.
/// </field>
resetApp: function resetApp(e) {
if (WinJS.Navigation.history.current.location.channelId === "Home") {
throw e;
} else {
this.returnHomeAndClearHistoryIfNecessary(true);              
}
},
/// <field>
/// Return to home channel
/// Clear history if clearHistory is true.
/// </field>
returnHomeAndClearHistoryIfNecessary: function returnHomeAndClearHistoryIfNecessary(clearHistory, state) {                
if (clearHistory) {
this._clearHistory();
this._navigateHome(state);                   
}
else {
var that = this;
this.channelManager.returnHomeChannel().then(function navigator_returnHomeChannelComplete(channelInfo) {
if (WinJS.Navigation.history.current.location.page !== channelInfo.pageInfo.page) {
that._navigateHome(state);
}
});
}
},
navigateToChannel: function navigateToChannel(channelId, state) {
/// <summary>
/// Navigates to a specific channel.
/// </summary>
/// <param name="channelId">
/// The Id of the channel to navigate to
/// </param>
/// <param name="state">
/// State object to pass
/// </param>
/// <returns>
/// Nothing
/// </returns>
var channel;
var i;
if (this.canNavigate) {
for (i = 0; i < this.channelManager.channels.length; i++) {
channel = this.channelManager.channels[i];
if (channel.id === channelId) {
WinJS.Navigation.navigate(channel.pageInfo, state);
return;
}
}
}
// if not found in standard L1, search Featured
for (i = 0; i < this.channelManager.featuredChannels.length; i++) {
channel = this.channelManager.featuredChannels[i];
if (channel.id === channelId) {
WinJS.Navigation.navigate(channel.pageInfo, state);
return;
}
}
},
/// <field>
/// Read entry state from disk asynchronously
/// </field>
readEntryStateFromDiskAsync: function readEntryStateFromDiskAsync(detail) {
return this._readEntryStateFromDiskAsync(detail, this.mode, WinJS.Navigation.history.backStack.length);
},
/// <summary>
/// The event handler for the onNavigated event
/// </summary>
/// <param name="detail">
/// An object that contains information about the event. The detail property of this object contains the following sub-properties:
/// location: the location being navigated to.
/// state: the state associated with the location being navigated to.
/// </param>
/// <returns>
/// Nothing
/// </returns>
loadPage: function loadPage(detail) {
msWriteProfilerMark("Platform:Navigation:LoadPage:s");
var that = this,
state = null,
history = WinJS.Navigation.history;
WinJS.Utilities.removeClass(this._pageArea, "modeSwitching");
// Nothing can be done
if (!detail.location ||
!detail.location.fragment) {
return;
}
if (this._loadPagePromise) {
this._loadPagePromise.cancel();
}
this._loadPagePromise = this._readEntryStateFromDiskAsync(detail, this.mode, WinJS.Navigation.history.backStack.length)
.then(function navigator_readEntryStateFromDiskAsyncComplete(stateInfo) {
state = stateInfo.state;
// Clear out page area
that._clearPage();
that.canNavigate = true;
// Load the fragment
return new WinJS.Promise(function (c, e) {
// here we need to wrap the renderCopy operation into a larger promise set
// to prevent cancellation from reaching the internal WinJS promise.  the reason
// for this lies in the WinJS internal implementation of in-memory cache, which will
// keep the errored-out promise in cache for the length of the session, preventing us
// from ever successfully rendering this fragment again.
WinJS.UI.Fragments
.renderCopy(detail.location.fragment, that._pageArea)
.then(function () { c(); }, function (exc) { e(exc); });
});
})
.then(function navigator_pageFragmentRenderCopyComplete() {
var fragments = WinJS.Utilities.query(".fragment", that._pageArea);
var elementToBind = fragments.length > 0 ? fragments[0] : that._pageArea;
WinJS.Utilities.addClass(elementToBind, "platformPage");
// Activate the controls in the fragment
return WinJS.UI.processAll(that._pageArea);
})
.then(function navigator_pageFragmentProcessAllComplete() {
return WinJS.Resources.processAll(that._pageArea);
})
.then(function navigator_pageFragmentResourcesProcessAllComplete() {
// Initialize the IPage object for the page                        
if (detail.location.page) {
try {
that.currentIPage = PlatformJS.Utilities.createObject(detail.location.page, state);
that.weightedNavigationCounter.countWeightedNode(that.currentIPage);
} catch (e) {
// There was some bad state that page could not handle, so reset the state of the app
that.resetApp(e);
return;
}
msWriteProfilerMark("Platform:Navigation:LoadPage:e");
// Mark the end of any previous page impression
if (that._lastContext) {
that._logNavigationEnd();
that._lastContext = null;
}
// Start impression for this new page
var current = history.current;
if (WinJS.Navigation.canGoForward) {
if (current.impression) {
var lastImpression = current.impression;
that._logNavigationResume(lastImpression);
that._lastContext = "resume";
}
} else {
var context = null;
// Try to get the context dynamically from the current page
if (that.currentIPage && that.currentIPage.getPageImpressionContext) {
context = that.currentIPage.getPageImpressionContext();
} else if (detail && detail.location && detail.location.page && detail.location.fragment) {
if (!detail.location.channelId) {
context = detail.location.fragment + "_" + detail.location.page; //Default context for snapped pages
} else {
var channelId = detail.location.channelId;
var found = false;
context = channelId + "_" + detail.location.fragment + "_" + detail.location.page; //Default context
for (var i = 0; !found && i < that.channelManager.channels.length; i++) {
var channel = that.channelManager.channels[i];
if (channel.id === channelId && channel.context) {
context = channel.context;
found = true;
}
if (channel.subChannels) {
for (var j = 0; !found && j < channel.subChannels.length; j++) {
var subChannel = channel.subChannels[j];
if (subChannel.id === channelId && subChannel.context) {
context = subChannel.context;
found = true;
}
}
}
}
}
}
if (!context) {
context = "unknown";
}
var partnerCode = "";
var attributes = null;
if (that.currentIPage && that.currentIPage.getPageImpressionPartnerCodeAndAttributes) {
var results = that.currentIPage.getPageImpressionPartnerCodeAndAttributes();
partnerCode = results.partnerCode;
attributes = results.attributes;
}
current.impression = that._logNavigationStart(context, "", "", partnerCode, attributes);
that._lastContext = context;
}
return that.populatePage();
} else {
PlatformJS.Utilities.onError("Invalid page");
}
});
this._loadPagePromise.done(null, function loadingPage_error(err) { PlatformJS.Utilities.onError(err); });
},
/// <summary>Populate page</summary>
/// <returns>Nothing</returns>
populatePage: function populatePage(isRefreshed) {
msWriteProfilerMark("Platform:Navigation:PopulatePage:s");
var that = this;
var currentPage = this.currentIPage;
// Get the page data
if (currentPage &&
currentPage.getPageData) {
return currentPage.getPageData()
.then(function navigator_currentPageGetDataComplete(data) {
if (data) {
// Bind the data to the UX
var fragments = WinJS.Utilities.query(".fragment", that._pageArea);
var elementToBind = fragments.length > 0 ? fragments[0] : that._pageArea;
return WinJS.Binding.processAll(elementToBind, data);
} else {
return WinJS.Promise.wrap(true);
}
})
.then(function navigator_currentPageBindingProcessAllComplete(stopProcessing) {
// there is no data to 
if (stopProcessing === true) {
CommonJS.WindowEventManager.getInstance().dispatchEvent("pagePopulated");
return;
}
// Notify the page that binding is complete
if (currentPage.onBindingComplete) {
currentPage.onBindingComplete();
msWriteProfilerMark("Platform:Navigation:PopulatePage:e");
if (!isRefreshed) {
// add instrumentation for page counter here per page
var pageBeingViewed = WinJS.Navigation.location;
var dataPointId = null;
var key = pageBeingViewed.fragment + "-" + pageBeingViewed.page;
if (PlatformJS.Services.manifest.instrumentationMapping.hasKey(key)) {
dataPointId = PlatformJS.Services.manifest.instrumentationMapping.lookup(key);
PlatformJS.Services.instrumentation.incrementInt32(
Platform.Instrumentation.InstrumentationDataSetId.platform,
dataPointId,
1);
}
// add instrumentation for total page view counter after 3 seconds
// cancel the previous timer first
if (that._currentPageTimer !== null) {
clearTimeout(that._currentPageTimer);
}
that._currentPageTimer = setTimeout(function navigator_currentPageTimeout() {
PlatformJS.Services.instrumentation.incrementInt32BasedOnConnectivity(
Platform.Instrumentation.InstrumentationDataSetId.platform,
Platform.Instrumentation.InstrumentationDataPointId.totalPageViewCount,
1);
if (!that._historyRestrictionConfig || that._historyRestrictionConfig.enable) {
that._limitHistoryLength();
}
}, 3000);
}
CommonJS.WindowEventManager.getInstance().dispatchEvent("pagePopulated");
}
}, PlatformJS.Utilities.onError);
} else {
PlatformJS.Utilities.onError("getPageData not defined");
}
},           
///// ------ IProcessListener interface
onLayoutChange: function onLayoutChange(event) {
/// <summary>
/// Handles layout change event
/// </summary>
/// <param name="event">
/// Layout change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
var layoutStates = Windows.UI.ViewManagement.ApplicationViewState,
newLocation = null;
// Inform the current page about the change in layout
if (this.currentIPage &&
this.currentIPage.onLayoutChange) {
newLocation = this.currentIPage.onLayoutChange(event.viewState);
}
if (this.pendingNavigation) {
newLocation = this.pendingNavigation;
this.pendingNavigation = null;
}
// Switch presentation modes if necessary
if (this.currentLayoutState === layoutStates.filled ||
this.currentLayoutState === layoutStates.fullScreenLandscape ||
this.currentLayoutState === layoutStates.fullScreenPortrait) {
if (event.viewState === layoutStates.snapped) {
this._switchMode(PlatformJS.Navigation.SnappedPresentationMode, newLocation);
}
} else {
if (event.viewState === layoutStates.fullScreenLandscape ||
event.viewState === layoutStates.fullScreenPortrait ||
event.viewState === layoutStates.filled) {
this._switchMode(PlatformJS.Navigation.StandardPresentationMode, newLocation);
}
}
this.currentLayoutState = event.viewState;
this._displayData.compute();
Platform.Telemetry.FlightRecorder.logDisplayChange(Platform.Telemetry.LogLevel.normal, this._displayData.width, this._displayData.height, this._displayData.scale);
},
onOrientationChange: function onOrientationChange(event) {
/// <summary>
/// Handles orientation change event
/// </summary>
/// <param name="event">
/// Orientation change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
if (this.currentIPage &&
this.currentIPage.onOrientationChange) {
this.currentIPage.onOrientationChange(event.layout);
}
this._displayData.compute();
Platform.Telemetry.FlightRecorder.logDisplayChange(Platform.Telemetry.LogLevel.normal, this._displayData.width, this._displayData.height, this._displayData.scale);
},
onPreviewToggle: function onPreviewToggle() {
if (this.currentIPage &&
this.currentIPage.onPreviewToggle) {
this.currentIPage.onPreviewToggle();
}
},
onDpiChange: function onDpiChange(event) {
/// <summary>
/// Handles dpi change event ("Make everything on your screen bigger" setting)
/// </summary>
/// <param name="event">
/// Dpi change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
if (this.currentIPage &&
this.currentIPage.onDpiChange) {
this.currentIPage.onDpiChange(event);
}
},
onWindowResize: function onWindowResize(event) {
/// <summary>
/// Handles window resize event. Occurs when:
///  - changing resolution in control panel
///  - WINDOWS+ARROW when using multiple monitors
/// </summary>
/// <param name="event">
/// resize event
/// </param>
/// <returns>
/// Nothing
/// </returns>
if (this.currentIPage &&
this.currentIPage.onWindowResize) {
this.currentIPage.onWindowResize(event);
}
},
onSuspending: function onSuspending(event) {
/// <summary>
/// Handles Suspend change event
/// </summary>
/// <param name="event">
/// Suspend change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
// disable history trimming
this._historyRestrictionConfig = {
enable: false,
triggerSize: 0,
restrictionSize: 0
};
if (this.currentIPage &&
this.currentIPage.onSuspending) {
this.currentIPage.onSuspending(event);
}
this.storeCurrentPage();
this._lastSuspendedTime = Date.now();
},
onResuming: function onResuming(event) {
/// <summary>
/// Handles Resume change event
/// </summary>
/// <param name="event">
/// Resume change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
if (this.currentIPage &&
this.currentIPage.onResuming) {
this.currentIPage.onResuming(event);
}
this.channelManager.downloadFeaturesConfigAsync();
if (PlatformJS.Services.isConfigurationInitialized) {
// check for terminate threshold if configuration is initialized
var resetHistoryMillisecondThreshold = Platform.Configuration.ConfigurationManager.custom.getInt32("ResetHistoryMillisecondThreshold");
var millisecondsSinceLastSuspend = Date.now() - this._lastSuspendedTime;
if (millisecondsSinceLastSuspend > resetHistoryMillisecondThreshold) {
// clear history, navigate to home.
this.returnHomeAndClearHistoryIfNecessary(true);     
}
}
// reset the history trimming config to force it refresh
this._historyRestrictionConfig = null;
},
_onRegionChange: function _onRegionChange() {
/// <summary>
/// Handles Region change event by attempting to redownload Features Config
/// </summary> 
/// <returns>
/// Nothing
/// </returns>
Platform.Configuration.ConfigurationManager.instance.hasPendingFeaturesConfigUpdate = true;
this.channelManager.downloadFeaturesConfigAsync();
}
})
});
})();
(function appexPlatformWeightedNavigationCounterInit(WinJS) {
"use strict";
var NS = WinJS.Namespace.define("PlatformJS", {
WeightedNavigationCounter: WinJS.Class.define(function weightedNavigationCounter_ctor() {
this._count = 0;
}, {
// private variables
_count: null,
// public methods
countWeightedNode: function countWeightedNode(node) {
var weight = node.countWeight || 1;
this._count += weight;
},
getCount: function getCount(event) {
return this._count;
},
}),
});
})(WinJS);
/* File: ChannelManager.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformChannelManagerInit() {
WinJS.Namespace.define("PlatformJS.Navigation",
{
/// <summary>
/// Channel management
/// </summary>
ChannelManager: WinJS.Class.define(function ChannelManager_ctor() {
/// <summary>
/// Constructor: Creates an instance of channel manager object.
/// </summary>
/// <returns>
/// An instance of channel manager object.
/// </returns>
var that = this;
// Read the configuration file to get the channels
this._readConfig(PlatformJS.Services.manifest.channels, this.standardChannels);
this._readConfig(PlatformJS.Services.manifest.snappedChannels, this.snappedChannels);
this._readConfig(PlatformJS.Services.manifest.featuredChannels, this.featuredChannels);
Platform.Networking.NetworkManager.addEventListener("networkstatuschanged", function navigator_onNetworkStatusChanged() {
if (Platform.Networking.NetworkManager.instance.isNetworkAvailable &&
Platform.Configuration.ConfigurationManager.locationChangedOffline) {
that.downloadFeaturesConfigAsync();
};
});
msWriteProfilerMark("Platform:Navigation:ChannelManager:Constructor:e");
},
{
/// <field>
/// Channels, either standardChannels or snappedChannels based on current mode
/// </field>
channels: null,
/// <field>
/// Channels in standard mode
/// </field>
standardChannels: [],
/// <field>
/// Channels in snapped mode
/// </field>
snappedChannels: [],
/// <field>
/// Featured channels
/// </field>
featuredChannels: [],
/// <field>
/// If channel config is changed, true or false
/// </field>
channelConfigChanged: false,
_cloudFeaturedChannelsLoaded: false,
/// <field>
/// Reload channels
/// </field>
reloadChannels: function reloadChannels() {
this.standardChannels = [];
this.snappedChannels = [];
// Read the configuration file to get the channels
this._readConfig(PlatformJS.Services.manifest.channels, this.standardChannels);
this._readConfig(PlatformJS.Services.manifest.snappedChannels, this.snappedChannels);
PlatformJS.Navigation.mainNavigator.lastIPage = null;
PlatformJS.Navigation.mainNavigator.currentIPage = null;
// Read the configuration file to get the channels
this.channels = PlatformJS.Navigation.mainNavigator.mode === "standard" ? this.standardChannels : this.snappedChannels;
PlatformJS.Navigation.mainNavigator.canNavigate = true;
},
/// <field>
/// Resolve channels
/// </field>
resolveChannel: function resolveChannel(channelId, channelSet) {
var channels = this.channels;
if (channelSet === "standard") {
channels = this.standardChannels;
} else if (channelSet === "snapped") {
channels = this.snappedChannels;
} else if (channelSet === "featured") {
channels = this.featuredChannels;
}
for (var i = 0, ilen = channels.length; i < ilen; i++) {
channel = channels[i];
if (channel.id === channelId) {
return channel;
}
}
},
/// <field>
/// Set channel config data
/// </field>
setChannelConfigData: function setChannelConfigData(channelConfig) {
var mainNavigator = PlatformJS.Navigation.mainNavigator;
this.channelConfigChanged = true;
if (channelConfig) {
this.standardChannels = channelConfig.standardChannels;
this.snappedChannels = channelConfig.snappedChannels;
this.featuredChannels = channelConfig.featuredChannels;
}
if (this.snappedChannels.length > 0) {
mainNavigator.currentLayoutState = Windows.UI.ViewManagement.ApplicationView.value;
} else {
mainNavigator.currentLayoutState = Windows.UI.ViewManagement.ApplicationViewState.fullScreenLandscape;
}
mainNavigator.mode = mainNavigator.currentLayoutState !== 2 ? PlatformJS.Navigation.StandardPresentationMode : PlatformJS.Navigation.SnappedPresentationMode;
// Read the configuration file to get the channels
this.channels = mainNavigator.mode === "standard" ? this.standardChannels : this.snappedChannels;
},
/// <field>
/// Download feature config asynchronously
/// </field>
downloadFeaturesConfigAsync: function downloadFeaturesConfigAsync() {
var that = this; 
return Platform.Configuration.ConfigurationManager.instance.downloadFeaturesConfigurationAsync().then(
function features_downloadComplete() {
return that.loadFeaturesConfigAsync(true);
},
function features_downloadError(error) {
console.log("Unable to load feature partner.js");
// do nothing here.
});
},
/// <field>
/// Load feature config asynchronously
/// </field>
loadFeaturesConfigAsync: function loadFeaturesConfigAsync(bypassCheckForServerVersion) {
var that = this;
msWriteProfilerMark("LoadFeaturedConfigAsync:s");
if (!this._cloudFeaturedChannelsLoaded || Platform.Configuration.ConfigurationManager.instance.hasPendingFeaturesConfigUpdate) {
var currentMarket = Platform.Configuration.ConfigurationManager.instance.currentConfigurationFolderName; 
var region = CommonJS.LocalPano.LocalPanoManager.instance.activeRegion;
var getFeaturesConfigPromise = null;
if (!region) {
getFeaturesConfigPromise = PlatformJS.Cache.CacheService.getInstance("ConfigurationCache").findEntry(currentMarket + "-Features", { supportsInMemory: true, bypassInMemory: true })
.then(function getFeaturesConfig_Complete(response) {
if (response) {
msWriteProfilerMark("ParseFeaturedConfig:s");
that.parseFeaturesConfig(response.dataValue); 
that.channelConfigChanged = true;
Platform.Configuration.ConfigurationManager.instance.hasPendingFeaturesConfigUpdate = false;
}
if (!bypassCheckForServerVersion) {
// Kick off the download if required
that.downloadFeaturesConfigAsync();
}
msWriteProfilerMark("LoadFeaturedConfigAsync:e");
});
}
else {
var sourcesSearchEntry = currentMarket + "-Features";
var navItemsSearchEntry = currentMarket + "-NavBarItems-" + region;
var results = { navbaritems: null, sources: null };
// since sources comes from partner.js but navitems can come from parnter-seattle.js...
var promiseArray = [];
promiseArray[0] = PlatformJS.Cache.CacheService.getInstance("ConfigurationCache").findEntry(navItemsSearchEntry, { supportsInMemory: true, bypassInMemory: true })
.then(function getFeaturesNavBarItemsConfig_Complete(response) {
if (response) {
results.navbaritems = response.dataValue.navbaritems;
}
});
promiseArray[1] = PlatformJS.Cache.CacheService.getInstance("ConfigurationCache").findEntry(sourcesSearchEntry, { supportsInMemory: true, bypassInMemory: true })
.then(function getFeaturesSourcesConfig_Complete(response) {
if (response) {
results.sources = response.dataValue.sources;
}
});
getFeaturesConfigPromise = WinJS.Promise.join(promiseArray)
.then(function getFeaturesConfig_Complete() {
if (results.navbaritems && results.sources) {
msWriteProfilerMark("ParseFeaturedConfig:s");
that.parseFeaturesConfig(results);
that.channelConfigChanged = true;
Platform.Configuration.ConfigurationManager.instance.hasPendingFeaturesConfigUpdate = false;
}
if (!bypassCheckForServerVersion) {
// Kick off the download if required
that.downloadFeaturesConfigAsync();
}
msWriteProfilerMark("LoadFeaturedConfigAsync:e");
});
}
this._cloudFeaturedChannelsLoaded = true;
return getFeaturesConfigPromise;
}
msWriteProfilerMark("LoadFeaturedConfigAsync:e");
return WinJS.Promise.wrap(null);
},
/// <field>
/// Parse feature config
/// </field>
parseFeaturesConfig: function parseFeaturesConfig(response){
var featuredChannels = [];
var that = this;
response.navbaritems.forEach(function parseFeaturedChannels(navBarItem) {
var channelInfo = that.parseChannel(navBarItem, response.sources);
if (channelInfo) {
featuredChannels.push(channelInfo);
}
});
this.featuredChannels = featuredChannels;
},
/// <field>
/// Parse channel
/// </field>
parseChannel: function parseChannel(navBarItem, sources) {
var channel = null;
console.log("parserChannel " + navBarItem.destination);
if (navBarItem.destination.startsWith("//sources/")) {
var id = navBarItem.destination.substring(10);
for (var i = 0; i < sources.length; i++) {
if (sources[i].id && sources[i].id.toLowerCase() === id.toLowerCase()) {
channel = {};
channel.id = id;
channel.visible = true;
// If the featured Channels read from the config contains this channel Id then simply use it as is.
if (this.featuredChannels && this.featuredChannels.length > 0) {
for (var j = 0; j < this.featuredChannels.length; j++) {
if (this.featuredChannels[j].id === sources[i].channelid) {
return this.featuredChannels[j];
}
}
}
for (var k = 0; k < this.standardChannels.length; k++) {
if (this.standardChannels[k].id === sources[i].channelid) {
channel.pageInfo = { channelId: id, fragment: this.standardChannels[k].pageInfo.fragment, page: this.standardChannels[k].pageInfo.page };
break;
}
}
if (navBarItem.title) {
channel.isDisplayValue = true;
channel.title = navBarItem.title;
}
channel.displayInstrumentation = navBarItem.displayinstrumentation;
channel.hitInstrumentation = navBarItem.hitinstrumentation;
channel.toggleButtonColor = navBarItem.dropdowncolor;
channel.state = sources[i].state ? JSON.parse(sources[i].state) : { dynamicInfo: { isFeaturedSource: true, dataProviderOptions: { useByline: true } } };
channel.state.dynamicInfo = this._readDynamicInfo(channel, sources[i]);
channel.images = {};
if (navBarItem.partnerlogo) {
channel.images.single = navBarItem.partnerlogo;
}
if (navBarItem.partnerlogosnap) {
channel.images.snap = navBarItem.partnerlogosnap;
}
if (navBarItem.partnerlogohoisted) {
channel.images.hoist = navBarItem.partnerlogohoisted;
}
channel.subChannels = [];
var that = this;
if (navBarItem.subchannels && navBarItem.subchannels.length > 0) {
navBarItem.subchannels.forEach(function parseSubChannels(subChannel) {
var subchannelInfo = that.parseChannel(subChannel, sources);
if (subchannelInfo) {
channel.subChannels.push(subchannelInfo);
}
});
}
break;
}
}
}
else {
var parseResult = Platform.Utilities.AppExUri.tryCreate(navBarItem.destination);
var uri = parseResult && parseResult.value ? parseResult.appExUri : null;
// Only allow deep links to cloud pano at this time.
if (uri && uri.queryParameters && uri.queryParameters.hasKey("entitytype") && uri.queryParameters["entitytype"] === "cloudPano") {
var channelId = uri.queryParameters["entitytype"] + "_" + uri.queryParameters["campaign"] + "_" + uri.queryParameters["market"];
if (uri.queryParameters.hasKey("index")) {
channelId = channelId + "_" + uri.queryParameters["index"];
}
channel = {
id: channelId,
displayInstrumentation: navBarItem.displayinstrumentation,
hitInstrumentation: navBarItem.hitinstrumentation,
isDeepLink: true,
images: {
single: navBarItem.partnerlogo,
snap: navBarItem.partnerlogosnap,
hoist: navBarItem.partnerlogohoisted
},
pageInfo: { fragment: navBarItem.destination, channelId: channelId },
subChannels: []
};
if (navBarItem.title) {
channel.title = navBarItem.title;
channel.isDisplayValue = true;
}
}
}
// do some validation of the channel
if (channel && !channel.pageInfo) {
console.error("missing pageInfo for navBarItem" + navBarItem.destination);
channel = null;
}
if (channel && channel.images) {
if (channel.images.single) {
PlatformJS.Utilities.downloadFile("PlatformTopEdgyImageCache", channel.images.single).done(null, function singleImageDownloadFile_error(error) { });
}
if (channel.images.hoist) {
PlatformJS.Utilities.downloadFile("PlatformTopEdgyImageCache", channel.images.hoist).done(null, function hoistImageDownloadFile_error(error) { });
}
if (channel.images.snap) {
PlatformJS.Utilities.downloadFile("PlatformTopEdgyImageCache", channel.images.snap).done(null, function snapImageDownloadFile_error(error) { });
}
}
return channel;
},
/// <field>
/// Return to home Channel
/// </field>
returnHomeChannel: function returnHomeChannel() {
var key = "Home";
var channels = this.channels;
var channel = null;
for (var i = 0, ilen = channels.length; i < ilen; i++) {
channel = channels[i];
if (channel.id === key) {
break;
}
}
return WinJS.Promise.wrap({
pageInfo: channel.pageInfo,
state: channel.state,
});
},
//PRIVATE FUNCTIONS
_readDynamicInfo: function _readDynamicInfo(channel, source) {
var dynamicInfo = channel.state.dynamicInfo;
if (dynamicInfo) {
dynamicInfo.entrypoint = "navbar";
dynamicInfo.channelID = channel.id;
dynamicInfo.adUnitId = source.adunitid;
dynamicInfo.id = channel.id;
dynamicInfo.instrumentationId = source.instrumentationid;
// TODO: Keeping these for now for back-compat but need to remove later
if (source.icon) {
dynamicInfo.logoUrl = source.icon;
}
if (source.searchIcon) {
dynamicInfo.snapLogoUrl = source.searchIcon;
}
// support for data contract changes
// set additional state values from sources data when present
if (source.displayname) {
dynamicInfo.panoTitle = source.displayname;
}
if (source.featuredurl) {
dynamicInfo.feedName = source.featuredurl;
}
if (source.css) {
dynamicInfo.css = source.css;
}
if (source.backbuttontype) {
dynamicInfo.backbuttontype = source.backbuttontype;
}
if (source.themetype === 0) {
dynamicInfo.theme = "dynamicPanoBoxedTheme";
}
if (source.headerlogo) {
dynamicInfo.logoUrl = source.headerlogo;
}
if (source.headerlogosnapped) {
dynamicInfo.snapLogoUrl = source.headerlogosnapped;
}
//set pinning logo url values of partner pano across different resolution scales(1x,1.4x,1.8x)
dynamicInfo.pinLogoUrls = {};
dynamicInfo.pinLogoUrls[Windows.Graphics.Display.ResolutionScale.scale100Percent] = source.pinnedlogo1 ? source.pinnedlogo1 : null;
dynamicInfo.pinLogoUrls[Windows.Graphics.Display.ResolutionScale.scale140Percent] = source.pinnedlogo14 ? source.pinnedlogo14 : null;
dynamicInfo.pinLogoUrls[Windows.Graphics.Display.ResolutionScale.scale180Percent] = source.pinnedlogo18 ? source.pinnedlogo18 : null;
//set periodic live tile urls of pinned partner pano.
var feedName = dynamicInfo.feedName;
if (feedName) {
dynamicInfo.liveTileUrls = this._setUpLiveTileUrls(feedName.replace(".js", ""));
dynamicInfo.liveTileRecurrence = PlatformJS.Services.configuration.getDictionary("LiveTile").getInt32("secondaryTileUpdateFrequency");
}
}
return dynamicInfo;
},
_setUpLiveTileUrls: function _setUpLiveTileUrls(feedName) {
var pollUrls = [];
var liveTileDic = PlatformJS.Services.configuration.getDictionary("LiveTile");
var urlTemplate = liveTileDic.getString("secondaryTileURL");
var urlCount = liveTileDic.getInt32("secondaryTileCount");
var currentMarket = Platform.Utilities.Globalization.getCurrentMarket();
var appName = CommonJS.getAppName();
//Partner pano has no hero and its live tiles cycle the first three entities 
//in the first entity list in the cluster group page, i.e., use 2.xml through 4.xml 
for (var i = 2; i <= urlCount; i++) {
var url = urlTemplate;
while (url.indexOf("{mkt}") >= 0) {
url = url.replace("{mkt}", currentMarket);
}
url = url.replace("{app}", appName);
url = url.replace("{feedName}", feedName);
url = url.replace("{id}", i);
pollUrls.push(url);
}
return pollUrls;
},
_readConfig: function _readConfig(channelsList, channelArray) {
/// <summary>
/// Reads the config to get channel info.
/// </summary>
/// <param name="channelsList">
/// List of channels
/// </param>
/// <param name="channelArray">
/// array to populate
/// </param>
/// <returns>
/// Nothing
/// </returns>
var i = 0;
var channelConfig = null;
for (i = 0; i < channelsList.size; i++) {
channelConfig = channelsList[i];
var channel = this._readChannel(channelConfig);
channelArray.push(channel);
}
},
_readChannel: function _readChannel(channelConfig) {
var i = 0,
state = null,
title = null,
images = null;
if (!channelConfig.id) {
PlatformJS.Utilities.onError("Invalid channel Id");
}
if (!channelConfig.title) {
PlatformJS.Utilities.onError("Invalid channel title");
}
if (!channelConfig.fragment) {
PlatformJS.Utilities.onError("Invalid channel fragment");
}
if (!channelConfig.page) {
PlatformJS.Utilities.onError("Invalid channel page object name");
}
var subChannels = [];
for (i = 0; i < channelConfig.subChannels.size; i++) {
var subChannelConfig = channelConfig.subChannels[i];
var subChannel = this._readChannel(subChannelConfig);
subChannels.push(subChannel);
}
try {
state = channelConfig.state.length > 0 ? JSON.parse(channelConfig.state) : null;
} catch (e) {
PlatformJS.Utilities.onError(e);
}
try {
images = channelConfig.images && channelConfig.images.length > 0 ? JSON.parse(channelConfig.images) : null;
} catch (e) {
PlatformJS.Utilities.onError(e);
}
title = channelConfig.title;
var channel = ({
id: channelConfig.id,
visible: channelConfig.visible === "true",
title: title,
icon: channelConfig.icon,
context: channelConfig.context,
pressedIcon: channelConfig.pressedIcon,
pageInfo: {
fragment: channelConfig.fragment,
page: channelConfig.page,
channelId: channelConfig.id
},
subChannels: subChannels,
doubleWide: channelConfig.doubleWide,
images: images,
subTitle: channelConfig.subTitle,
state: state
});
return channel;
}
})
});
})();
/* File: navigation.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
/// <dictionary target="comment">backstack,nav,onbefore,Stevenfe</dictionary>
/// <dictionary target="variable">arg,Config,obj</dictionary>
/// <dictionary target="member">Config,nav</dictionary>
(function appexPlatformNavigationInit() {
"use strict";
WinJS.Namespace.define("PlatformJS.Navigation",
{
uriNavigationControllers: {},
entityViewMap: {},
/// <summary>
/// Verify that the Controler is a no-op
/// </summary>
/// <param name="url">
/// The Applink url
/// </param>
/// <returns>
/// A bool indicating if the Controler is no-op
/// </returns>
isNoop: function isNoop(url) {
if (!url || url.length === 0) {
return true;
} else {
var parseResult = Platform.Utilities.AppExUri.tryCreate(url);
return parseResult.value && parseResult.appExUri.controllerId === "noop";
}
},
/// <summary>
/// Navigates to a specific applink.
/// </summary>
/// <param name="uri">
/// The Applink uri to navigate to
/// </param>
/// <param name="context">
/// The context where the navigation is coming from
/// </param>
/// <returns>
/// Nothing
/// </returns>
navigateTo: function navigateTo(uri, context) {
var command = this.createCommandFromUri(uri, context);
if (WinJS.Promise.is(command)) {
command.then(function navigation_createCommand(target) {
if (target && target.pageInfo) {
WinJS.Navigation.navigate(
target.pageInfo,
target.state);
}
});
} else if (command.pageInfo) {
WinJS.Navigation.navigate(
command.pageInfo,
command.state);
} else {
command();
}
},
/// <summary>
/// Create an applink Command from a Uri
/// </summary>
/// <param name="uri">
/// The Applink uri formated like thi: scheme : / / [ controllerId ] [ / commandId ] [ # entityId ] [ ? parameters ] 
/// </param>
/// <param name="context">
/// The context where the navigation is coming from
/// </param>
/// <returns>
/// An applink Command
/// </returns>
createCommandFromUri: function createCommandFromUri(uri, context) {
var command = null;
if (uri && uri.length !== 0) {
var parseResult = Platform.Utilities.AppExUri.tryCreate(uri);
if (parseResult.value) {
var appExUri = parseResult.appExUri;
var parameters = appExUri.queryParameters;
if (parameters) {
if (parseResult.appExUri.entityId !== "") {
parameters["id"] = parseResult.appExUri.entityId;
}
console.log("AppEx Navigation to: " + appExUri);
command = this.createCommand(appExUri.controllerId, appExUri.commandId, parameters, context);
} else {
command = this.createNoopAction();
}
} else {
// fall back to external navigation
console.log("External navigation to: " + uri);
var externalUri = new Windows.Foundation.Uri(uri);
command = this.createLaunchUriAction(externalUri);
}
}
else {
command = this.createNoopAction();
}
if (command && command.options) {
command.options.__uri = uri;
}
return command;
},
/// <summary>
/// Create an applink Command
/// </summary>
/// <param name="controllerId">
/// The Applink Controler Id
/// </param>
/// <param name="commandId">
/// The Applink Command Id
/// </param>
/// <param name="parameters">
/// An array (key/value) of Applink parameters
/// </param>
/// <param name="context">
/// The context where the navigation is coming from
/// </param>
/// <returns>
/// An applink Command
/// </returns>
createCommand: function createCommand(controllerId, commandId, parameters, context) {
console.log("Creating command: " + controllerId + ", " + commandId + ", " + parameters + ", " + context);
var command = null;
try {
command = PlatformJS.Navigation.uriNavigationControllers[controllerId][commandId](parameters, context);
} catch (e) {
command = this.createNoopAction();
}
// command must *not* be null.. if it is, it's a bug in the controller, not this code
var uriBuilder = new Platform.Utilities.AppExUriBuilder();
uriBuilder.controllerId = controllerId;
uriBuilder.commandId = commandId;
var isValidElement;
// should return true iff:
// value is a string, number, boolean, or date
// OR value is an array of valid elements
isValidElement = function navigation_isValidParameterElement(element) {
var result =
typeof (element) === "string"
|| typeof (element) === "number"
|| typeof (element) === "boolean"
|| element instanceof Date
|| element instanceof Array;
if (element instanceof Array) {
for (var i = 0; i < value.length; i++) {
var arrayValue = value[i];
result = result && isValidElement(arrayValue);
}
}
return result;
};
for (var key in parameters) {
var value = parameters[key];
if (!parameters.hasOwnProperty(key) || !isValidElement(value)) {
continue;
}
uriBuilder.queryParameters.insert(key, value);
}
command.uri = uriBuilder.toString();
return command;
},
/// <summary>
/// Create an applink navigation action
/// </summary>
/// <param name="channelId">
/// The App channel Id
/// </param>
/// <param name="options">
/// An json object
/// </param>
/// <returns>
/// An applink action
/// </returns>
createNavigationAction: function createNavigationAction(channelId, options) {
var action;
options = options || {};
action = function navigation_createAction() {
options.__uri = action.uri;
PlatformJS.Navigation.navigateToChannel(channelId, options);
};
var channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel(channelId);
action["channelInfo"] = channelInfo;
action["options"] = options;
return action;
},
/// <summary>
/// Create an applink launch action
/// </summary>
/// <param name="uri">
/// The Applink uri formated like thi: scheme : / / [ controllerId ] [ / commandId ] [ # entityId ] [ ? parameters ] 
/// </param>
/// <returns>
/// A function launching the given uri
/// </returns>
createLaunchUriAction: function createLaunchUriAction(uri) {
return function navigation_launchUriAction() {
Windows.System.Launcher.launchUriAsync(uri);
};
},
/// <summary>
/// Create a no-op action
/// </summary>
/// <returns>
/// An empty function
/// </returns>
createNoopAction: function createNoopAction() {
return function navigation_noopAction() { };
},
/// <summary>
/// Execute an applink Command
/// </summary>
/// <param name="controllerId">
/// The Applink Controler Id
/// </param>
/// <param name="commandId">
/// The Applink Command Id
/// </param>
/// <param name="parameters">
/// An array (key/value) of Applink parameters
/// </param>
/// <param name="context">
/// The context where the navigation is coming from
/// </param>
/// <returns>
/// Nothing
/// </returns>
executeCommand: function executeCommand(controllerId, commandId, parameters, context) {
console.log("Executing command: " + controllerId + ", " + commandId + ", " + parameters + ", " + context);
var command = this.createCommand(controllerId, commandId, parameters, context);
if (command) {
command();
}
},
addController: function addController(id, controller) {
console.log("Adding controller: " + id);
PlatformJS.Navigation.uriNavigationControllers[id] = controller;
},
addEntityView: function addEntityView(id, command) {
console.log("Adding entity view: " + id);
PlatformJS.Navigation.entityViewMap[id] = command;
},
addCloudPanoEntityView: function addCloudPanoEntityView(id) {
PlatformJS.Navigation.addEntityView("cloudPano",
function navigation_cloudPanoEntityViewCommand(args) {
var prop = [
"dataSource",
"splashDataSource",
"sver",
"campaign",
"market",
"index",
"bkgColor",
"splashName",
"splashW",
"splashH",
"splashBkg",
"altUri",
"lScroll", //"landscapeScrollPosition",
"pScroll", //"portraitScrollPosition",
"minIdx"
];
var state = {};
prop.forEach(function (p) {
state[p] = args[p] || args[p.toLowerCase()] || null;
});
state.sver = state.sver || "";
state.campaign = state.campaign || "";
state.market = state.market || "";
state.index = state.index || "";
return PlatformJS.Navigation.createNavigationAction(id || "CloudPano", state);
});
},
addFeedbackEntityView: function addFeedbackEntityView(){
PlatformJS.Navigation.addEntityView("feedback",
function navigation_FeedbackEntityViewCommand() {
return CommonJS.Feedback.onFeedbackCmd;
});
},
addCustomizationEntityView: function addCustomizationEntityView() {
var that = this;
PlatformJS.Navigation.addEntityView("customize",
function navigation_CustomizationEntityViewCommand() {
return function execution_customization(args) {
var page = that.currentIPage;
if (page && page.onCustomize) {
page.onCustomize();
} else {
var panoElements = WinJS.Utilities.query(".platformMainContent");
if (panoElements && panoElements.length === 1) {
if (panoElements[0]) {
var panoControl = panoElements[0].winControl;
if (panoControl && panoControl.showCustomizationView) {
panoControl.showCustomizationView();
}
}
}
}
};
});
},
addHelpEntityView: function addHelpEntityView() {
PlatformJS.Navigation.addEntityView("help",
function navigation_HelpEntityViewCommand() {
return CommonJS.Settings.onHelpCmd;
});
},
addTourEntityView: function addTourEntityView() {
PlatformJS.Navigation.addEntityView("video",
function navigation_TourEntityViewCommand(args) {
return function execution_presentTour() {
var playVideoFunction = null;
try {
playVideoFunction = CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback;
} catch(error) {}
if (playVideoFunction) {
var contentId = args.contentId || args.contentid;
if (contentId) {
playVideoFunction(contentId);
}   
}
};
});
},
addCommonEntityViews: function addCommonEntityViews() {
PlatformJS.Navigation.addFeedbackEntityView();
PlatformJS.Navigation.addCustomizationEntityView();
PlatformJS.Navigation.addHelpEntityView();
PlatformJS.Navigation.addTourEntityView();
},
navigateToChannel: function navigateToChannel(channelId, state) {
/// <summary>
/// Navigates to a specific channel.
/// </summary>
/// <param name="channelId">
/// The Id of the channel to navigate to
/// </param>
/// <param name="state">
/// State object to pass
/// </param>
/// <returns>
/// Nothing
/// </returns>
PlatformJS.Navigation.mainNavigator.navigateToChannel(channelId, state);
},
/// <field>
/// Returns current IPage object.
/// </field>
currentIPage: {
get: function get() {
if (PlatformJS.Navigation.mainNavigator) {
return PlatformJS.Navigation.mainNavigator.currentIPage;
} else {
return null;
}
}
},
saveCurrentPageState: function saveCurrentPageState() {
/// <summary>
/// Saves the current page state.
/// </summary>
/// <returns>
/// Nothing
/// </returns>
if (PlatformJS.Navigation.mainNavigator) {
PlatformJS.Navigation.mainNavigator.storeCurrentPage();
}
},
/// <field>
/// Returns a flag indicating if navigation can occur.
/// </field>
canNavigate: {
get: function get() {
if (PlatformJS.Navigation.mainNavigator) {
return PlatformJS.Navigation.mainNavigator.canNavigate;
} else {
return true;
}
}
},
/// <field>
/// Returns a flag indicating if back navigation can occur.
/// </field>
canGoBack: {
get: function get() {
return PlatformJS.Navigation.canNavigate && WinJS.Navigation.canGoBack;
}
},
});
})();
(function appexPlatformNoopControllerInit() {
"use strict";
var noopController = {
default: function navigation_noopController(args) {
// do nothing
return PlatformJS.Navigation.createNoopAction();
}
};
PlatformJS.Navigation.addController("noop", noopController);
})();
