﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: utils.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsUtilsInit (WinJS) {
"use strict";
function _getEstimatedStringLength(strLength, currentHeight, desiredHeight) {
/// <summary>
/// For the input string length, given that its height is currentHeight, returns an estimated string length that would fit in desiredHeight.
/// </summary>
return Math.floor(strLength * desiredHeight / currentHeight);
};
var Utils = WinJS.Namespace.define("CommonJS.Utils", {
itemClickProxy: function itemClickProxy(control) {
return function utils_itemClickProxy (evt) {
var item = null;
var currentElt = evt.srcElement;
// walk up the DOM until the module control is found, and get the item from it.
while (currentElt) {
if (WinJS.Utilities.hasClass(currentElt, "cluster-item")) {
item = currentElt.winControl.data;
break;
}
currentElt = currentElt.parentElement;
}
if (item) {
control.dispatchEvent("itemclick", { item: item });
}
};
},
convertRSSTimeToFriendlyTime: function convertRSSTimeToFriendlyTime(rssTime) {
/// <summary>
/// Returns a friendly name for the bdi time such as "21 minutes ago"
/// </summary>
/// <param name="rssTime">Time string from an RSS feed</param>
var now = new Date();
var nowEpoch = now.getTime();
var date = new Date(rssTime);
var rssTimeEpoch = date.getTime();
if (!isNaN(rssTimeEpoch)) {
var delta = Math.max(0, nowEpoch - rssTimeEpoch);
return CommonJS.Utils.convertDeltaToFriendlyTime(delta);
}
return "";
},
convertBDITimeToFriendlyTime: function convertBDITimeToFriendlyTime(bdiTime) {
/// <summary>
/// Returns a friendly name for the bdi time such as "21 minutes ago"
/// </summary>
/// <param name="bdiTime">UTC Time in 100 nanosecond intervals since midnight of 1601/1/1</param>
var now = new Date();
var nowEpoch = Date.UTC(now.getUTCFullYear(), now.getUTCMonth(), now.getUTCDate(), now.getUTCHours(), now.getUTCMinutes(), now.getUTCSeconds());
var bdiTimeEpoch = (bdiTime / 10000) + Utils._bdiTimeOriginEpoch;
var delta = Math.max(0, nowEpoch - bdiTimeEpoch);
return CommonJS.Utils.convertDeltaToFriendlyTime(delta);
},
convertDeltaToFriendlyTime: function convertDeltaToFriendlyTime(delta) {
/// <summary>
/// Returns a friendly name for the bdi time such as "21 minutes ago"
/// </summary>
/// <param name="delta">Time delta from now, in ms</param>
var friendlyTime = "";
var resourceLoader = CommonJS.resourceLoader;
if (0 <= delta && delta < 120000 /* 2mins */) {
friendlyTime = resourceLoader.getString("/Common/OneMinAgo");
} else if (120000 <= delta && delta < 3600000 /* 1hr */) {
friendlyTime = resourceLoader.getString("/Common/MinsAgo").format(Math.floor(delta / 60000));
} else if (3600000 <= delta && delta < 7200000 /* 2hr */) {
friendlyTime = resourceLoader.getString("/Common/OneHourAgo");
} else if (7200000 <= delta && delta < 86400000 /* 1 day */) {
friendlyTime = resourceLoader.getString("/Common/HoursAgo").format(Math.floor(delta / 3600000));
} else if (86400000 <= delta && delta < 172800000 /* 2 days */) {
friendlyTime = resourceLoader.getString("/Common/OneDayAgo");
} else {
friendlyTime = resourceLoader.getString("/Common/DaysAgo").format(Math.floor(delta / 86400000));
}
return friendlyTime;
},
truncateString: function truncateString(str, len) {
/// <summary>
/// Returns the truncation of str to the given length, and adds "..." if any truncation happens.
/// </summary>
return str.substr(0, len) + (len >= str.length ? "" : "...");
},
truncateMultilineText: function truncateMultilineText(multilineElt) {
msWriteProfilerMark("CommonControls:Utils:truncateMultilineText:s");
var parentElt = multilineElt.parentElement;
// measure the height of the element and its container
var parentHeight = parentElt.clientHeight;
var height = multilineElt.clientHeight;
// if the element doesn't fit within its container
if (height > parentHeight) {
var originalString = multilineElt.innerText;
var originalLength = originalString.length;
// estimate the string length based on the height ratio of the element and its parent
var initialLength = _getEstimatedStringLength(originalLength, height, parentHeight);
// find the next word boundary
var nextWordBoundaryIndex = originalString.indexOf(" ", initialLength);
initialLength = (nextWordBoundaryIndex === -1 ? initialLength : nextWordBoundaryIndex);
// truncate at the initial estimate
var tryLength = initialLength;
var tryString = CommonJS.Utils.truncateString(originalString, tryLength);
multilineElt.innerText = tryString;
// measure the height again
height = multilineElt.clientHeight;
// while element is still too high to fit its container
while (height > parentHeight) {
// find the previous word boundary
var oneLessWordLength = originalString.lastIndexOf(" ", tryLength - 1);
tryLength = oneLessWordLength === -1 ? tryLength - 1 : oneLessWordLength;
// if reached the beginning of the string, quit
if (tryLength <= 0) {
break;
}
// truncate, measure, and repeat
tryString = CommonJS.Utils.truncateString(originalString, tryLength);
multilineElt.innerText = tryString;
height = multilineElt.clientHeight;
}
}
msWriteProfilerMark("CommonControls:Utils:truncateMultilineText:e");
},
_bdiTimeOriginEpoch: Date.UTC(1601, 0, 1, 0, 0, 0),
convertCMSArticle: function convertCMSArticle(result, createURIFunction) {
var newsArticle = {};
newsArticle.editorial = true;
if (result.heroImage) {
newsArticle.anchorpoint = result.heroImage.anchorPoint;
}
var cmsImage = null;
if (result.heroImage && result.heroImage.image) {
cmsImage = result.heroImage.image;
}
else {
cmsImage = result.image;
}
if (cmsImage) {
newsArticle.imageAttribution = cmsImage.attribution ? cmsImage.attribution : "";
newsArticle.altText = cmsImage.altText ? cmsImage.altText : "";
newsArticle.caption = cmsImage.caption ? toStaticHTML(cmsImage.caption) : "";
var originalImage = null;
if (cmsImage.images) {
for (var i = 0; i < cmsImage.images.length; i++) {
var img = cmsImage.images[i];
var imgName = img.name ? img.name.toLowerCase() : "";
if (imgName === "original") {
newsArticle.thumbnail = {};
newsArticle.thumbnail.url = img.url;
newsArticle.thumbnail.width = img.width;
newsArticle.thumbnail.height = img.height;
newsArticle.thumbnail.name = "original";
}
else if (imgName === "semanticzoom") {
newsArticle.semanticZoomThumbnail = {};
newsArticle.semanticZoomThumbnail.url = img.url;
newsArticle.semanticZoomThumbnail.width = img.width;
newsArticle.semanticZoomThumbnail.height = img.height;
newsArticle.semanticZoomThumbnail.name = "semanticZoom";
}
else if (imgName === "lowres") {
newsArticle.thumbnailLowRes = {};
newsArticle.thumbnailLowRes.url = img.url;
newsArticle.thumbnailLowRes.width = img.width;
newsArticle.thumbnailLowRes.height = img.height;
newsArticle.thumbnailLowRes.name = "lowRes";
}
}
}
}
else {
newsArticle.imageAttribution = "";
newsArticle.altText = "";
newsArticle.caption = "";
}
newsArticle.sourceImageUrl = result.content.source && result.content.source.favicon ? result.content.source.favicon : "";
// CMS headline support HTML Fragments
newsArticle.headline = result.content.headline ? toStaticHTML(result.content.headline) : "";
newsArticle.title = newsArticle.headline;
newsArticle.source = result.content.source && result.content.source.name ? result.content.source.name : "";
newsArticle.destination = result.destination;
// The following are the different destination URLs we support (newsArticle.destination)
// OLD FORMAT (To be eventually removed)
//      article://33259198
// NEW FORMAT (various urls below, see data contract spec for full details)
//      //application/view?entitytype=article&pageId=33259198
//      bingnews://application/view?entitytype=article&pageId=33259198
//      ///view?entitytype=article&pageId=2123125
//      bingnews:///view?entitytype=article&pageId=2123125
var newsUri = createURIFunction(newsArticle.destination);
if (newsUri) {
if (newsUri.entitytype === "article") {
newsArticle.type = newsUri.entitytype;
newsArticle.articleid = newsUri.pageId && newsUri.pageId.length > 0 ? newsUri.pageId : null;
newsArticle.contentid = newsUri.contentId && newsUri.contentId.length > 0 ? newsUri.contentId : null;
}
else if (newsUri.entitytype === "slideshow") {
newsArticle.type = newsUri.entitytype;
newsArticle.contentid = newsUri.pageId;
}
else if (newsUri.entitytype === "video") {
if (!newsArticle.thumbnail) {
newsArticle.thumbnail = {
width: 0,
height: 0,
url: "",
name: ""
};
}
newsArticle.type = newsUri.entitytype;
newsArticle.thumbnail.videoOptions = {
videoSource: newsUri.contentId,
source: newsArticle.source,
title: newsArticle.title,
sourceImageUrl: newsArticle.sourceImageUrl,
thumbnail: newsArticle.thumbnail.url
};
}
}
// CMS abstract support HTML Fragments
newsArticle.abstract = result.content.abstract ? toStaticHTML(result.content.abstract) : "";
newsArticle.snippet = newsArticle.abstract;
newsArticle.byline = result.byline ? result.byline : "";
var publicationDate = result.content.publicationDate;
function getDateString(dateType) {
if (dateType && dateType.utctime) {
var utcTime = dateType.utctime;
// TODO: Let platform know that this may be off an hour, need to investigate
var time = CommonJS.Utils.convertBDITimeToFriendlyTime(utcTime);
return time && time.indexOf("NaN") < 0 ? time : "";
}
else {
return "";
}
}
if (publicationDate) {
newsArticle.updatedTime = getDateString(publicationDate.updated);
newsArticle.publishTime = getDateString(publicationDate.published);
}
newsArticle.template = result.template;
newsArticle.templateClass = result.templateClass;
return newsArticle;
}
});
})(WinJS);
