﻿/* base\newsPanoBasePage.js */
(function () {
"use strict";
function createWrapperFunction(object) {
return function () { return object; };
}
WinJS.Namespace.define("NewsJS", {
NewsBasePage: WinJS.Class.define(
function (state) {
var that = this;
that.isOnline = PlatformJS.Utilities.hasInternetConnection();
NewsJS.Partners.Theme.onNavigated(state);
this._isSnapped = Windows.UI.ViewManagement.ApplicationView.value === Windows.UI.ViewManagement.ApplicationViewState.snapped;
this._lastViewState = Windows.UI.ViewManagement.ApplicationView.value;
this._bdiReporter = (Platform.Utilities.BdiResultReport);
this._clusterClickHandlers = [];
this._lastRenderTime = null;
this.semanticZoomRenderer = function (itemPromise) {
return {
element: itemPromise.then(function (group) {
var div = document.createElement("div");
WinJS.Utilities.addClass(div, "platformSemanticZoomItem");
var binding = NewsJS.Bindings.semanticZoomTile(that._clusterTitle(group.data));
CommonJS.setModuleSizeAndClass(binding.moduleInfo, div);
CommonJS.loadModule(binding.moduleInfo, binding, div).then();
return div;
})
};
};
this.newsDailySemanticZoomRenderer = function (itemPromise) {
return {
element: itemPromise.then(function (group) {
var div = document.createElement("div");
if (group.data.clusterEntity) {
WinJS.Utilities.addClass(div, "platformSemanticZoomItem");
var binding = group.data.clusterEntity;
CommonJS.setModuleSizeAndClass(binding.moduleInfo, div);
CommonJS.loadModule(binding.moduleInfo, binding, div).then();
}
return div;
})
};
};
WinJS.Utilities.markSupportedForProcessing(this.newsDailySemanticZoomRenderer);
WinJS.Utilities.markSupportedForProcessing(this.semanticZoomRenderer);
this.commonHeaderOptions = {
};
this._fullContainer = document.getElementById("news_Panorama");
this.progressType = CommonJS.Progress.headerProgressType;
if (this._bdiReporter) {
// wire up the reporting button if the reporter object exists and the button dom element is on the page
var reportButtonControl = PlatformJS.Utilities.getControl("reportBadResults");
if (reportButtonControl) {
var reportButtonElem = document.getElementById("reportBadResults");
if (reportButtonElem) {
WinJS.Utilities.removeClass(reportButtonElem, "hiddenButton");
}
reportButtonControl.label = "Report BDI Result";
// sad face icon
reportButtonControl.icon = "\u2639";
reportButtonControl.onclick = function (e) {
that.reportButtonClicked();
};
}
}
if (state && state.videoContentId) {
var videoOptions = {
videoSource: state.videoContentId,
};
CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback(videoOptions.videoSource, that.videoOptions);
state.videoContentId = null;
}
this.connectivityChange = this.connectivityChanged.bind(this);
Platform.Networking.NetworkManager.addEventListener("networkstatuschanged", this.connectivityChange);
var panoRefreshThreshold = PlatformJS.Services.appConfig.getInt32("PanoRefreshThreshold");
if (panoRefreshThreshold) {
this._refreshThreshold = panoRefreshThreshold;
}
var helpButton = PlatformJS.Utilities.getControl("helpButton");
if (helpButton) {
helpButton.label = CommonJS.resourceLoader.getString("/platform/HelpLabel");
helpButton.onclick = function () {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.appBar);
CommonJS.Settings.onHelpCmd();
NewsJS.Telemetry.Utilities.recordButtonClick("Help Button", Platform.Telemetry.ActionContext.appBar);
};
}
},
{
_fullContainer: null,
_snappedContainer: null,
_snappedHeaderList: null,
_offlineData: null,
_openedSnapped: false,
_pageDataPromise: null,
_semanticZoomForceLayout: false,
_refreshThreshold: null,
getPanoramaControl: function getPanoramaControl() {
return (this.getPanoramaControl = createWrapperFunction(PlatformJS.Utilities.getControl("news_Panorama")))();
},
dispose: function dispose() {
Platform.Networking.NetworkManager.removeEventListener("networkstatuschanged", this.connectivityChange);
var panoControl = PlatformJS.Utilities.getControl("news_Panorama");
if (panoControl) {
panoControl.dispose();
}
NewsJS.Partners.Theme.onNavigateAway();
},
toggleReportButtonText: function toggleReportButtonText(enableReporting) {
var reportButtonControl = PlatformJS.Utilities.getControl("reportBadResults");
if (reportButtonControl) {
if (enableReporting) {
reportButtonControl.label = "Cancel BDI Report";
} else {
reportButtonControl.label = "Report BDI Result";
}
}
},
reportItemAndRestoreClickHandlers: function reportItemAndRestoreClickHandlers(e) {
var that = this;
var item = e.item;
// only allow reporting of BDI result items
if (that._bdiReporter && item && item.transformer && item.bdiRequestUrl) {
var imageUrl = item.thumbnail ? item.thumbnail.url : null;
var rawResponse = item.transformer ? item.transformer.getRawResponse() : null;
CommonJS.Feedback.showBdiReportForm({
bdiRequestUrl: item.bdiRequestUrl,
rawResponse: rawResponse,
articleHeadline: item.title,
articleSnippet: item.snippet,
articleUrl: item.articleUrl,
imageUrl: imageUrl
});
} else {
NewsJS.Utilities.showToast("Only BDI results may be reported");
}
that.switchClickHandlersForReporting(false);
},
switchClickHandlersForReporting: function switchClickHandlersForReporting(enableReporting) {
// Persists all cluster click handlers and toggles the reporting click handler.
// <param name="enableReporting" type="Boolean>If TRUE, saves all the existing click handlers and replaces them with the reporting handler.
// If FALSE, restores the original click handlers from the saved list.</param>
var panorama = PlatformJS.Utilities.getControl("news_Panorama");
var clusters = this._clusters ? this._clusters : this._visibleClusters;
if (panorama && clusters && this._bdiReporter) {
var that = this;
var currentCluster = null;
var currentClusterInstance = null;
if (enableReporting) {
// protect against double-saving!
if (that._clusterClickHandlers.length === 0) {
var clustersLength = clusters.length;
for (var i = 0; i < clustersLength; i++) {
currentCluster = clusters.getAt(i);
currentClusterInstance = panorama.getClusterContentControl(currentCluster.clusterKey);
if (currentClusterInstance) {
that._clusterClickHandlers.push(currentCluster.clusterContent.contentOptions.onitemclick);
currentClusterInstance.removeEventListener("itemclick", currentCluster.clusterContent.contentOptions.onitemclick);
currentClusterInstance.onitemclick = function (e) { that.reportItemAndRestoreClickHandlers(e); };
}
}
}
} else {
var clickHandlersLength = that._clusterClickHandlers.length;
if (clickHandlersLength > 0) {
for (var j = clickHandlersLength - 1; j >= 0; j--) {
currentCluster = clusters.getAt(j);
currentClusterInstance = panorama.getClusterContentControl(currentCluster.clusterKey);
if (currentClusterInstance) {
var currentClickHandler = that._clusterClickHandlers.pop();
currentClusterInstance.removeEventListener("itemclick", currentClusterInstance.onitemclick);
currentClusterInstance.onitemclick = currentClickHandler;
}
}
}
}
that.toggleReportButtonText(enableReporting);
}
},
reportButtonClicked: function reportButtonClicked() {
var that = this;
if (that._clusterClickHandlers.length === 0) {
that.switchClickHandlersForReporting(true);
} else {
that.switchClickHandlersForReporting(false);
}
CommonJS.dismissAllEdgies();
//NewsJS.Utilities.showToast("Please select an article to report.");
},
onNavigateAway: function onNavigateAway() {
NewsJS.Utilities.cancelPromise(this._pageDataPromise);
},
onBindingComplete: function onBindingComplete() {
if (this._isSnapped) {
// If we were launched in snapped mode, show the snapped container
this._openedSnapped = true;
this.showSnappedContainer();
}
},
connectivityChanged: function connectivityChanged() {
var connection = PlatformJS.Utilities.hasInternetConnection();
if (this.isOnline !== connection) {
if (connection && this.appWentOnline) {
this.appWentOnline();
} else if (!connection && this.appWentOffline) {
this.appWentOffline();
}
}
this.isOnline = connection;
},
appWentOffline: function appWentOffline() {
},
appWentOnline: function appWentOnline() {
},
_hideFREBackground: function _hideFREBackground() {
if (this.FREVisible) {
CommonJS.disableAllEdgies(false);
this.FREVisible = false;
var panoControl = PlatformJS.Utilities.getControl("news_Panorama");
if (panoControl) {
panoControl.zoomOptions = {
locked: false
};
}
var offlineEl = document.getElementById("offlineMessageContainer");
if (offlineEl) {
WinJS.Utilities.addClass(offlineEl, "hidden");
}
// If we're currently in snapped mode, unhide the snap container when going back online
if (this._snappedContainer && this._isSnapped) {
WinJS.Utilities.removeClass(this._snappedContainer, "hidden");
}
this._semanticZoomForceLayout = true;
}
},
_showFREBackground: function _showFREBackground() {
if (!this.FREVisible) {
CommonJS.disableAllEdgies(true);
this.FREVisible = true;
}
},
FREVisible: null,
FREShown: {
get: function get() {
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
return localSettings.values["FREShown"];
},
set: function set(shown) {
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
localSettings.values["FREShown"] = shown;
}
},
showFRE: function showFRE(title, subtitle) {
var that = this;
var moduleInfo = {
fragmentPath: "/html/templates.html",
templateId: "offlineMessage",
};
var data = {
title: title,
snippet: subtitle
};
this._showFREBackground();
var panoControl = PlatformJS.Utilities.getControl("news_Panorama");
if (panoControl) {
panoControl.zoomOptions = {
locked: true
};
}
// Modify the existing the immersive header created by the pano control since its Z-order is on top of everything else.
// The fontColor will change once the pano actually renders real cluster content
var headerControl = PlatformJS.Utilities.getControl("news_Panorama_header");
if (headerControl) {
headerControl.fontColor = "headerFontLight";
}
var offlineContainerContentElement = document.getElementById("offlineContainerContent");
offlineContainerContentElement.innerHTML = "";
CommonJS.loadModule(moduleInfo, data, offlineContainerContentElement);
// Reveal the overlaid full-screen offline message
var offlineEl = document.getElementById("offlineMessageContainer");
if (offlineEl) {
WinJS.Utilities.removeClass(offlineEl, "hidden");
}
},
hideFRE: function hideFRE() {
this._hideFREBackground();
},
showCannedFRE: function showCannedFRE() {
var title = PlatformJS.Services.resourceLoader.getString("WelcomeToBingNews");
var subtitle = PlatformJS.Services.resourceLoader.getString("SettingUp");
this.showFRE(title, subtitle);
this.progressType = CommonJS.Progress.headerProgressType;
CommonJS.Progress.showProgress(this.progressType);
},
showOfflineFRE: function showOfflineFRE() {
var that = this;
if (this._offlineData &&
this._offlineData.OfflineTitle &&
this._offlineData.OfflineConnect) {
var title = PlatformJS.Services.resourceLoader.getString(this._offlineData.OfflineTitle);
var subtitle = PlatformJS.Services.resourceLoader.getString(this._offlineData.OfflineConnect);
that.showFRE(title, subtitle);
}
},
// Creates a div to be used in snap mode.  Defaults to creating a SnappedHeaderList control
// Derived class must implement getSnappedHeaderListOptions() and optionally getSnappedHeaderListDataSource()
// Derived class can override this function to create a different DOM element
createSnappedContainer: function createSnappedContainer() {
var snappedContainer = document.createElement("div");
this._snappedHeaderList = PlatformJS.Utilities.createObject("NewsJS.Controls.SnappedHeaderList", snappedContainer, this.getSnappedHeaderListOptions());
return snappedContainer;
},
getSnappedHeaderListDataSource: function getSnappedHeaderListDataSource() {
return this._newsProviderAdapter ? new CommonJS.VirtualizedDataSource(this._newsProviderAdapter) : null;
},
snappedFragmentCreated: function snappedFragmentCreated() {
// overload if necessary
},
showSnappedContainer: function showSnappedContainer() {
var that = this;
if (!this._useSnappedContainer) {
CommonJS.Snapped.loadDefaultSnappedPage();
return;
}
// Hide full container
if (this._fullContainer) {
WinJS.Utilities.addClass(this._fullContainer, "hidden");
}
// Show snapped container
if (this._snappedContainer) {
if (this._snappedHeaderList && !this._snappedHeaderList.itemDataSource) {
this._snappedHeaderList.itemDataSource = this.getSnappedHeaderListDataSource();
}
WinJS.Utilities.removeClass(this._snappedContainer, "hidden");
} else {
// Create it if it doesn't exist
var fragment = document.querySelector(".fragment");
if (fragment) {
this._snappedContainer = this.createSnappedContainer();
fragment.appendChild(this._snappedContainer);
this.snappedFragmentCreated();
}
}
if (this._snappedContainer && this.FREVisible) {
WinJS.Utilities.addClass(this._snappedContainer, "hidden");
}
var listView = that.getSnappedListView();
if (listView) {
listView.forceLayout();
listView.scrollPosition = 0;
}
},
refreshPanoramaControl: function refreshPanoramaControl() {
var panoramaControl = this._fullContainer ? this._fullContainer.winControl : null;
if (panoramaControl) {
panoramaControl.cancelPendingOperations();
panoramaControl.refresh();
}
},
hideSnappedContainer: function hideSnappedContainer() {
if (!this._useSnappedContainer) {
CommonJS.Snapped.unloadDefaultSnappedPage();
return;
}
// Hide snapped container
if (this._snappedContainer) {
WinJS.Utilities.addClass(this._snappedContainer, "hidden");
}
// Show pano
if (this._fullContainer) {
WinJS.Utilities.removeClass(this._fullContainer, "hidden");
// TODO: remove this workaround for semantic zoom not resizing correctly when WinJS fix is available
if (this._fullContainer.winControl && this._fullContainer.winControl.semanticZoom) {
var panoramaControl = this._fullContainer.winControl;
if (this._openedSnapped) {
// This works around a separate issue where certain panorama controls don't get properly 
// realized when the page is initially opened snapped (and hence panorama control is hidden).
this.refreshPanoramaControl();
}
// This is the initial workaround for a resizing concerns
panoramaControl.semanticZoom.forceLayout();
if (this._openedSnapped || this._semanticZoomForceLayout) {
this._semanticZoomForceLayout = false;
// This is the second part of the workaround above which is necessary to force the layout
// of the zoomed out listview to be initialized to avoid a crash
// (also only necessary when the page is opened intially snapped).
var listView = PlatformJS.Utilities.getControl("news_Panorama_zoomOut");
if (listView) {
listView.forceLayout();
}
}
// once we've fixed this once on the page, we nolonger need to repeat it.
this._openedSnapped = false;
}
}
},
// Returns the list view used in the default snapped container
getSnappedListView: function getSnappedListView() {
if (this._snappedHeaderList) {
return this._snappedHeaderList.listview;
}
return null;
},
changeSnapState: function changeSnapState(viewState) {
var needLayout = false;
var isSnapped = viewState === Windows.UI.ViewManagement.ApplicationViewState.snapped;
if (isSnapped !== this._isSnapped) {
needLayout = true;
}
this._isSnapped = isSnapped;
if (needLayout) {
if (this._isSnapped) {
this.showSnappedContainer();
} else {
this.hideSnappedContainer();
}
if (this.onEnterExitSnap) {
this.onEnterExitSnap();
}
}
},
onViewStateChange: function onViewStateChange(viewState, /*@optional*/e) {
var lastViewState = this._lastViewState;
this._lastViewState = viewState;
if (viewState === this.applicationViewState.snapped || lastViewState === this.applicationViewState.snapped) {
this.changeSnapState(viewState);
}
var panoControl = PlatformJS.Utilities.getControl("news_Panorama");
if (panoControl) {
if (viewState === this.applicationViewState.fullScreenPortrait || lastViewState === this.applicationViewState.fullScreenPortrait) {
panoControl.onOrientationChange(e);
}
if (!(viewState === this.applicationViewState.fullScreenLandScape && lastViewState === this.applicationViewState.fullScreenPortrait) && !(viewState === this.applicationViewState.fullScreenPortrait && lastViewState === this.applicationViewState.fullScreenLandScape)) {
panoControl.onLayoutChange(viewState);
}
}
},
onOrientationChange: function onOrientationChange(e) {
this.onViewStateChange(Windows.UI.ViewManagement.ApplicationView.value, e);
},
onLayoutChange: function onLayoutChange(viewState) {
this.onViewStateChange(viewState);
},
/// <summary>
/// This function is invoked when dpi changes ("Make everything on your screen bigger" setting)
/// </summary>
/// <param name="e">
/// Dpi change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onDpiChange: function onDpiChange(e) {
var panoControl = PlatformJS.Utilities.getControl("news_Panorama");
if (panoControl && panoControl.onDpiChange) {
panoControl.onDpiChange(e);
}
},
/// <summary>
/// Handles window resize event. Occurs when:
///  - changing resolution in control panel
///  - WINDOWS+ARROW when using multiple monitors
/// </summary>
/// <param name="e">
/// resize event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onWindowResize: function onWindowResize(e) {
var panoControl = PlatformJS.Utilities.getControl("news_Panorama");
if (panoControl && panoControl.onWindowResize) {
panoControl.onWindowResize(e);
}
},
onSuspending: function onSuspending(e) { },
applicationViewState: { fullScreenLandScape: 0, filled: 1, snapped: 2, fullScreenPortrait: 3 },
isOnline: true,
semanticZoomRenderer: null,
commonHeaderOptions: null,
addRemovePanoramaFlush: function addRemovePanoramaFlush(shouldAdd) {
var panoEl = document.getElementById("news_Panorama");
if (panoEl) {
if (shouldAdd) {
WinJS.Utilities.addClass(panoEl, "platformPanoramaFlushLeft");
}
else {
WinJS.Utilities.removeClass(panoEl, "platformPanoramaFlushLeft");
}
}
},
checkOffline: function checkOffline(dontShowToast) {
this.isOnline = PlatformJS.Utilities.hasInternetConnection();
if (this.isOnline) {
this.toggleOfflineArticles(true);
return false;
}
else {
if (!dontShowToast) {
var msg = PlatformJS.Services.resourceLoader.getString("ArticleUnavailableOffline");
NewsJS.Utilities.showToast(msg);
}
return true;
}
},
toggleOfflineArticles: function toggleOfflineArticles(enabled) {
// subclass should overload
},
raiseEdgyTemporarily: function raiseEdgyTemporarily() {
var that = this;
var edgy = document.getElementById("actionEdgy");
if (edgy) {
edgy.addEventListener("afterhide", function (event) {
that.cancelEdgyHide();
});
edgy.addEventListener("MSPointerDown", function (event) {
that.cancelEdgyHide();
});
}
var actionEdgyCtl = PlatformJS.Utilities.getControl("actionEdgy");
if (actionEdgyCtl) {
actionEdgyCtl.show();
}
that._hideTimeout = setTimeout(function () { that.hideEdgy(); }, 5000);
},
cancelEdgyHide: function cancelEdgyHide() {
var that = this;
if (that._hideTimeout) {
console.log("Cancelling hide of app bar edgy");
clearTimeout(that._hideTimeout);
that._hideTimeout = null;
}
},
hideEdgy: function hideEdgy() {
var edgy = document.getElementById("actionEdgy");
if (edgy) {
if (edgy.style.visibility !== "hidden") {
WinJS.UI.Animation.fadeOut(edgy).then(function () {
edgy.style.visibility = "hidden";
});
}
}
this._hideTimeout = null;
},
onRoamingStateChanged: function onRoamingStateChanged() {
// override where needed
},
onPreviewToggle: function onPreviewToggle() {
var that = this;
that._refreshPage();
},
_shouldRefreshedRenderedContent: function _shouldRefreshedRenderedContent() {
var shouldRefresh = false;
if (this._lastRenderTime && this._refreshThreshold) {
var currentTime = Date.now();
var elapsedSecondsSinceLastRender = (currentTime - this._lastRenderTime);
if (elapsedSecondsSinceLastRender > this._refreshThreshold) {
shouldRefresh = true;
this._lastRenderTime = Date.now();
}
}
return shouldRefresh;
},
_pageLoadSucceeded: function _pageLoadSucceeded() {
CommonJS.Error.removeError();
NewsJS.Utilities.disableButton("refreshButton", false);
CommonJS.Progress.hideProgress(this.progressType);
var platformPage = document.querySelector(".pageLoading");
if (platformPage) {
WinJS.Utilities.removeClass(platformPage, "pageLoading");
}
},
_pageLoadFailed: function _pageLoadFailed(refreshFunction) {
var that = this;
CommonJS.Error.showError(PlatformJS.Utilities.hasInternetConnection() ? CommonJS.Error.STANDARD_ERROR : CommonJS.Error.NO_INTERNET,
refreshFunction ? refreshFunction : function () { if (that && that.loadPageData) { that.loadPageData(false); } });
NewsJS.Utilities.disableButton("refreshButton", false);
CommonJS.Progress.hideProgress(this.progressType);
},
_refreshPage: function _refreshPage(/*@optional*/showToast) {
if (!this.checkOffline(!showToast)) {
this._offlineData = null;
var panoControl = PlatformJS.Utilities.getControl("news_Panorama");
if (panoControl) {
panoControl.panoramaState = null;
}
this.loadPageData(true);
if (this._isSnapped) {
this._openedSnapped = true;
}
}
CommonJS.dismissAllEdgies();
},
_clusterTitle: function _clusterTitle(groupData) {
return groupData.clusterTitle;
},
}
)
});
})();
/* base\baseBuiltinPartnerPano.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS", {
BasePartnerPano: WinJS.Class.derive(NewsJS.NewsBasePage, function BasePartnerPano_ctor(state) {
NewsJS.NewsBasePage.call(this, state);
var that = this;
this._clusters = [];
if (!state) {
state = {};
} else {
this._panoramaState = state.panoramaState;
}
this._state = state;
this._providers = [];
this.addRemovePanoramaFlush(false);
this._useSnappedContainer = true;
this._timestamp = null;
this._autoRefresh = null;
this._panoControl = PlatformJS.Utilities.getControl("news_Panorama");
var refreshButton = PlatformJS.Utilities.getControl("refreshButton");
refreshButton.label = PlatformJS.Services.resourceLoader.getString("/partners/Refresh");
refreshButton.onclick = function () {
that._refreshPage();
that.onManualRefreshButtonClick();
};
WinJS.Utilities.removeClass(refreshButton.element, "hidden");
if (this._setupSectionPin) {
this._handleSectionPin = function () {
that._setupSectionPin(state);
};
var edgy = PlatformJS.Utilities.getControl("actionEdgy");
edgy.addEventListener("beforeshow", this._handleSectionPin);
}
this._autoRefreshInterval = NewsJS.Partners.Config.getConfig(that._partner, "AutoRefreshInterval", 15 * 60, true);
this._dataChangeHandler = function (event) {
var newRefreshValue = NewsJS.Partners.Config.getConfig(that._partner, "AutoRefreshInterval", 15 * 60, true);
if (that._autoRefreshInterval !== newRefreshValue) {
// If we already have a refresh timer or the timer was turned off
if (that._autoRefresh || that._autoRefreshInterval === 0) {
that._autoRefreshInterval = newRefreshValue;
that._createTimer();
}
}
};
Windows.Storage.ApplicationData.current.addEventListener("datachanged", this._dataChangeHandler);
this._isActivePage = true;
this._panoBounded = false;
this.semanticZoomRenderer = function (itemPromise) {
return {
element: itemPromise.then(function (group) {
var div = document.createElement("div"),
semanticZoomTileContainer = document.createElement("div"),
semanticZoomTextContainer = document.createElement("div"),
semanticZoomText = document.createElement("div"),
semanticZoomIcon = document.createElement("div");
WinJS.Utilities.addClass(div, "platformSemanticZoomItem");
WinJS.Utilities.addClass(semanticZoomTileContainer, "semanticZoomTileContainer");
WinJS.Utilities.addClass(semanticZoomTextContainer, "semanticZoomTextContainer");
WinJS.Utilities.addClass(semanticZoomText, "semanticZoomText");
WinJS.Utilities.addClass(semanticZoomIcon, "semanticZoomIcon");
semanticZoomText.textContent = that._clusterTitle(group.data);
var sectionName = (group.data.clusterEntity && group.data.clusterEntity.sectionName) ? group.data.clusterEntity.sectionName : null;
WinJS.Utilities.addClass(semanticZoomIcon, NewsJS.Partners.Config.getSectionImage(that._partner, sectionName));
semanticZoomTextContainer.appendChild(semanticZoomText);
semanticZoomTextContainer.appendChild(semanticZoomIcon);
semanticZoomTileContainer.appendChild(semanticZoomTextContainer);
div.appendChild(semanticZoomTileContainer);
return div;
})
};
};
WinJS.Utilities.markSupportedForProcessing(this.semanticZoomRenderer);
},
{
_panoControl: null,
_timestamp: null,
_partner: null,
_autoRefresh: null,
_dataChangeHandler: null,
_autoRefreshInterval: null,
countWeight: 2,
_isActivePage: false,
_setupSectionPin: function _setupSectionPin(state, pinSection, unpinSection) {
/// <summary>Set up pinning button </summary>
/// <param name="buttonElementId">Id of pin button</param>
/// <param name="state">State of pinned category</param>
var that = this;
var pin = document.getElementById("pinSection");
if (!pin) {
return;
}
if (!NewsJS.Partners.Config.isPartnerApp) {
var newsActionEdgyRight = document.querySelector(".newsActionEdgyRight");
if (newsActionEdgyRight) {
if (newsActionEdgyRight.firstChild) {
newsActionEdgyRight.insertBefore(pin, newsActionEdgyRight.firstChild);
} else {
newsActionEdgyRight.appendChild(pin);
}
}
}
WinJS.Utilities.addClass(pin, "platformHide");
return this._articleManager.getFeedInfoAsync(state.feedType, state.feedIdentifierValue, false).then(function (feedInfo) {
WinJS.Utilities.removeClass(pin, "platformHide");
var feedInfoTitle = feedInfo.pinTitle ? feedInfo.pinTitle : feedInfo.title;
var title = !NewsJS.Partners.Config.isPartnerApp ? that._title + " - " + feedInfoTitle : feedInfoTitle;
var logo = PlatformJS.Services.appConfig.getString("PinLogo");
if (!logo) {
logo = NewsJS.Partners.Config.isPartnerApp ? "images/pinlogo.png" : "images/logo.png";
}
return NewsJS.Utilities.Pinning._setupPinButton(
"pinSection",
PlatformJS.Services.resourceLoader.getString("/platform/pinToStart"),
PlatformJS.Services.resourceLoader.getString("/platform/unpinFromStart"),
"",
"",
"",  // no tileNotAddedToastMessage message
"",  // no tileNotRemovedToastMessage message
state.feedType + "-" + state.feedIdentifierValue.replace(" ", "_"), // make the a ID guaranteed unique tile (someone could pin "U.S." as category and topic)
title,
title,
that._categoryPano.page,
{ feedType: feedInfo.feedType, feedIdentifierValue: state.feedIdentifierValue, theme: that._partner, pinLogoUrls: state.pinLogoUrls, channelID: WinJS.Navigation.location.channelId },
null,
"ms-appx:///" + logo,
NewsJS.Partners.Config.isPartnerApp ? null : state.pinLogoUrls,
NewsJS.Partners.Config.isPartnerApp && feedInfo.sectionInfo ? feedInfo.sectionInfo.liveTileUrls : null,
NewsJS.Partners.Config.getConfig(that._partner, "LiveTileUpdateFrequency", Windows.UI.Notifications.PeriodicUpdateRecurrence.hour),
that.onPinSection.bind(that));
},
function () {
WinJS.Utilities.addClass(pin, "platformHide");
});
},
onPinSection: function onPinSection() {
},
onManualRefreshButtonClick: function onManualRefreshButtonClick() {
},
onLayoutChange: function onLayoutChange(e) {
NewsJS.NewsBasePage.prototype.onLayoutChange.call(this, e);
},
onBindingComplete: function onBindingComplete() {
NewsJS.NewsBasePage.prototype.onBindingComplete.call(this);
this.loadPageData(false);
},
dispose: function dispose() {
NewsJS.NewsBasePage.prototype.dispose.call(this);
for (var i = 0; i < this._providers.length; i++) {
if (this._providers[i].dispose) {
this._providers[i].dispose();
}
}
this._providers = [];
this._resetTimer();
Windows.Storage.ApplicationData.current.removeEventListener("datachanged", this._dataChangeHandler);
this._dataChangeHandler = null;
this._isActivePage = false;
if (this._subscribeButton) {
this._subscribeButton.onclick = null;
this._subscribeButton = null;
}
if (this._handleSectionPin) {
var edgy = PlatformJS.Utilities.getControl("actionEdgy");
if (edgy) {
edgy.removeEventListener("beforeshow", this._handleSectionPin);
}
this._handleSectionPin = null;
}
WinJS.Utilities.removeClass(document.body, "inErrorMode");
NewsJS.BasePartnerPano.disposeLoginButton();
},
appWentOnline: function appWentOnline() {
var that = this;
NewsJS.NewsBasePage.prototype.appWentOnline.call(this);
if (that._newsProviderAdapter) {
// if we've already previously entered snapped mode and were previously offline with no
// cached data, then the newsProviderAdapter needs to kick its listview listener so that
// it knows to request for additional data now that it's back online
that._newsProviderAdapter.reloadContentIfNecessary();
}
},
getSnappedHeaderListOptions: function getSnappedHeaderListOptions() {
/// <summary>Get snap header options</summary>
var that = this;
return {
headerOptions: that.commonHeaderOptions,
listViewOptions: {
itemDataSource: that.getSnappedHeaderListDataSource(),
oniteminvoked: function oniteminvoked(e) {
that.snappedItemInvoked(e);
}
}
};
},
snappedItemInvoked: function snappedItemInvoked(e) {
/// <summary> Snapped item invoked</summary>
var that = this;
var listView = this.getSnappedListView();
if (listView) {
listView.itemDataSource.itemFromIndex(e.detail.itemIndex).then(function (f) {
if (f.data.article) {
that.itemInvoked(f.data.article, null);
}
});
}
},
getPageState: function getPageState() {
/// <summary> Get the page state </summary>
/// <returns> Returns the page state</returns>
var panoControl = PlatformJS.Utilities.getControl("news_Panorama");
if (panoControl) {
this._state.panoramaState = panoControl.getPanoramaState();
}
return this._state;
},
getPageData: function getPageData() {
/// <summary> Get the page data to bind to the page</summary>
/// <returns> Returns the promise containing the page data</returns>
var that = this;
return WinJS.Promise.wrap({
semanticZoomRenderer: that.semanticZoomRenderer,
headerOptions: that.commonHeaderOptions,
panoramaState: that._panoramaState
});
},
loadPageData: function loadPageData(bypassCache) {
this.progressType = !bypassCache ? CommonJS.Progress.centerProgressType : CommonJS.Progress.headerProgressType;
NewsJS.Utilities.disableButton("refreshButton", true);
CommonJS.Progress.showProgress(this.progressType);
},
_resetTimer: function _resetTimer() {
if (this._autoRefresh) {
clearTimeout(this._autoRefresh);
this._autoRefresh = null;
}
},
_createTimer: function _createTimer() {
var that = this;
this._resetTimer();
var refreshInterval = this._autoRefreshInterval;
if (refreshInterval !== 0) {
this._autoRefresh = setTimeout(function () {
that.loadPageData(true);
NewsJS.Telemetry.Utilities.recordRefresh(Platform.Telemetry.ContentClear.autoRefresh);
}, refreshInterval * 1000);
}
},
_pageLoadSucceeded: function _pageLoadSucceeded() {
if (this._isActivePage) {
NewsJS.NewsBasePage.prototype._pageLoadSucceeded.call(this);
WinJS.Utilities.removeClass(document.body, "inErrorMode");
}
},
_pageLoadFailed: function _pageLoadFailed() {
if (this._isActivePage) {
var that = this;
NewsJS.NewsBasePage.prototype._pageLoadFailed.call(this, function () {
that.loadPageData(true);
});
WinJS.Utilities.addClass(document.body, "inErrorMode");
}
this._resetTimer();
},
_panoBounded: false,
hideSnappedContainer: function hideSnappedContainer() {
var that = this;
if (!this._panoBounded && that._panoControl) {
// Populate the pano
msSetImmediate(function () {
that._panoBounded = true;
that._panoControl.clusterDataSource = new WinJS.Binding.List(that._clusters).dataSource;
});
}
NewsJS.NewsBasePage.prototype.hideSnappedContainer.call(this);
},
_populatePano: function _populatePano() {
var that = this;
if (that._providers && that._providers.length > 0) {
that._newsProviderAdapter = new NewsJS.NewsProviderAdapter({
providers: that._providers,
articleFilter: function articleFilter(item) {
// filter the heading tile a no-results tile from the snapped view
if (item.template === "clusterHeadingTopicTile") {
return true;
}
if (item.sentinel) {
return true;
}
return false;
},
noResultsTile: function noResultsTile() {
return NewsJS.Bindings.noResultsTile(PlatformJS.Services.resourceLoader.getString("/partners/NoArticles"),
PlatformJS.Services.resourceLoader.getString("NoTopicResultsMessage2"),
"\uE171");
},
errorTile: function errorTile() {
return NewsJS.Bindings.noResultsTile(PlatformJS.Services.resourceLoader.getString("/platform/standardErrorDescription"),
"",
"\uE069");
},
offlineTile: function offlineTile() {
return NewsJS.Bindings.noResultsTile(PlatformJS.Services.resourceLoader.getString("/platform/offline_noContent"),
"",
"\uE069");
},
offline: !that.isOnline
});
} else {
that._newsProviderAdapter = null;
}
// Render the snap view if needed
if (Windows.UI.ViewManagement.ApplicationView.value === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
that._panoBounded = false;
var listView = that.getSnappedListView();
if (listView) {
listView.itemDataSource = that.getSnappedHeaderListOptions().listViewOptions.itemDataSource;
} else {
that.showSnappedContainer();
}
} else {
that._panoBounded = true;
// Populate the pano
that._panoControl.clusterDataSource = new WinJS.Binding.List(that._clusters).dataSource;
}
return WinJS.Promise.wrap();
},
itemInvoked: function itemInvoked(item, event) {
/// <summary>Cluster tile click handler</summary>
/// <param name="item">Item invoked</param>
/// <param name="event">Click event</param>
var that = this,
promise = null;
if (!item.access || item.access === "anonymous") {
promise = WinJS.Promise.wrap(true);
} else {
promise = this._authManager.getEntitlementsAsync(true, false, false, true);
}
promise.then(function (isEntitled) {
if (!isEntitled) {
return;
}
that._itemInvoked(item, event);
}, function (error) {
if (error !== CommonJS.Partners.Auth.BaseAuth.notLoggedInError) {
NewsJS.Utilities.showToast(PlatformJS.Services.resourceLoader.getString("/partners/unableToVerifyEntitlements"));
}
});
},
_itemInvoked: function _itemInvoked(item, event) {
var that = this;
if (item.type === "pano") {
item.state.theme = this._partner;
WinJS.Navigation.navigate(this._categoryPano, item.state);
} else if (item.type === "article" || (item.type === "blog" && !this._openBlogInFrame)) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.cluster);
WinJS.Navigation.navigate({
fragment: "/common/ArticleReader/html/ArticleReaderPage.html",
page: this._articleReaderPage
},
{
providerType: this._articleReaderProvider,
providerConfiguration: {
feedType: item.feedType,
feedIdentifierValue: item.feedIdentifierValue,
articleList: JSON.stringify(item.articleList)
},
initialArticleId: item.articleId,
enableSharing: true,
enableSnap: true,
snappedHeaderTitle: " ",
snappedHeaderFontColor: "headerFontDark",
theme: this._partner,
validActions: this.getValidActions ? this.getValidActions() : null,
actionsHandlerType: this.getActionsHandlerType ? this.getActionsHandlerType() : null,
actionKeys: this.getActionKeys ? this.getActionKeys() : null,
originatingSection: that._state.feedIdentifierValue,
originatingFeed: item.feedIdentifierValue,
entryPoint: item.instrumentationEntryPoint
});
} else if (item.type === "blog" && this._openBlogInFrame) {
WinJS.Navigation.navigate({
fragment: "/html/storyPage.html",
page: "NewsJS.StoryPage"
},
{
article: {
articleUrl: item.articleURL
},
theme: this._partner
});
} else if (item.type === "slideshow" && this._slideShowPage) {
WinJS.Navigation.navigate({
fragment: "/html/newsSlideshow.html",
page: this._slideShowPage
},
{
providerType: this._slideshowProvider,
providerConfiguration: {
articleId: item.slideshowId ? item.slideshowId : item.articleId,
feedType: item.feedType,
feedIdentifierValue: item.feedIdentifierValue,
articleGuid: item.articleGuid
},
theme: this._partner,
access: item.access
});
} else if (item.type === "video") {
CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback(item.video.url,
that.createVideoOptions(item),
that.createVideoList(item));
that.onPlayVideo(item.articleURL, item.feedIdentifierValue, item.feedType);
}
},
createVideoOptions: function createVideoOptions(item) {
return {
title: item.title ? NewsJS.Utilities.stripHTML(item.title) : item.snippet,
thumbnail: (item.thumbnail && item.thumbnail.url) || item.thumbnailUrl,
source: PlatformJS.Services.resourceLoader.getString("/" + this._partner.toLowerCase() + "/the" + this._partner.toLowerCase()),
index: item.articleIndex - 1,
instrumentationEntryPoint: item.instrumentationEntryPoint
};
},
createVideoList: function createVideoList(item) {
var videoList = [];
var videos = item.videoList;
if (videos) {
for (var k = 0; k < videos.length; k++) {
var video = this.createVideoOptions(videos[k]);
video.videoSource = videos[k].url;
videoList.push(video);
}
}
return videoList;
},
onPlayVideo: function onPlayVideo(articleURL, feedIdentifierValue, feedType) {
// Implemented in derived classes.
},
createNewsCluster: function createNewsCluster(section, provider, title, onheaderselection, maxColumnCount, isPrimaryCluster, configKey, configObject, adInfo, marginDistance, marginWidth, instrumentationEntryPoint) {
/// <summary>Create a news cluster</summary>
/// <param name="section">Section info</param>
/// <param name="provider">Cluster provider</param>
/// <param name="title" optional="true">Title of the cluster. Otherwise, use the title in the section info</param>
/// <param name="onheaderselection" optional="true">Use header selection handler for cluster</param>
/// <param name="maxColumnCount" optional="true">Maximum column count</param>
/// <param name="isPrimaryCluster" optional="true">Is a primary cluster</param>
/// <param name="configKey" optional="true">Another cluster config key</param>
/// <param name="configObject" optional="true">Cluster configuration object</param>
/// <param name="adInfo" optional="true">Ad information</param>
/// <param name="marginDistance" optional="true">Margin distance</param>
/// <param name="marginWidth" optional="true">Margin width</param>
/// <param name="instrumentationEntryPoint" optional="true">Instrumentation data point for video</param>
var that = this;
marginDistance = marginDistance ? marginDistance : 6;
marginWidth = marginWidth ? marginWidth : 8;
return {
clusterEntity: section,
clusterKey: section.sectionName || section.blogName || title || section.title,
clusterTitle: title || section.title,
onHeaderSelection: onheaderselection ? function (clusterEntity, index, pano) {
that._onHeaderSelection(clusterEntity, index, pano, null);
} : null,
clusterContent: {
contentControl: "CommonJS.News.EntityCluster",
contentOptions: {
mode: CommonJS.News.ClusterMode.dynamic,
onitemclick: function onitemclick(e) {
if (instrumentationEntryPoint) {
e.detail.item.instrumentationEntryPoint = instrumentationEntryPoint;
}
that.itemInvoked(e.detail.item, e);
},
configKey: configKey ? configKey : "EntityClusterDefaultNewsConfig",
configObject: configObject ? configObject : CommonJS.News.EntityClusterConfig.DefaultNews,
provider: provider,
alignBottom: true,
maxColumnCount: maxColumnCount,
marginDistance: maxColumnCount ? Math.min(maxColumnCount, marginDistance) : marginDistance,
marginWidth: marginWidth,
theme: "newsAppTheme" + (isPrimaryCluster ? " primaryCluster" : ""),
adUnitId: adInfo ? adInfo.adUnitId : null,
adsList: adInfo ? adInfo.adsList : null,
subclusterAdFrequency: adInfo ? adInfo.subclusterAdFrequency : null,
heightOffset: 158,
}
}
};
},
_onHeaderSelection: function _onHeaderSelection(clusterKey, index, pano, event) {
/// <summary>Header selection handler</summary>
/// <param name="clusterKey">Key of cluster clicked</param>
/// <param name="index">Index of cluster </param>
/// <param name="pano">Pano clicked</param>
/// <param name="event">Event of click</param>
var pageInfo = {
fragment: this._categoryPano.fragment,
page: this._categoryPano.page,
channelId: this._partner + "_" + clusterKey
};
WinJS.Navigation.navigate(pageInfo, {
feedType: "section",
feedIdentifierValue: clusterKey,
theme: this._partner
});
},
},
{
disposeLoginButton: function disposeLoginButton() {
var loginCmd = PlatformJS.Utilities.getControl("signInButton");
if (loginCmd) {
loginCmd.onclick = null;
}
var logoutCmd = PlatformJS.Utilities.getControl("logoutButton");
if (logoutCmd) {
logoutCmd.onclick = null;
}
},
launchPartnerApp: function launchPartnerApp(partner) {
var protocol = NewsJS.Partners.Config.getConfig(partner, "Protocol");
var options = new Windows.System.LauncherOptions();
options.displayApplicationPicker = false;
options.preferredApplicationDisplayName = NewsJS.Partners.Config.getConfig(partner, "PackageName");
options.preferredApplicationPackageFamilyName = NewsJS.Partners.Config.getConfig(partner, "PackageFamilyName");
var url = protocol + "://home?referrer=" + Windows.ApplicationModel.Package.current.id.name;
Windows.System.Launcher.launchUriAsync(new Windows.Foundation.Uri(url), options);
}
})
});
})();
