﻿(function () {
"use strict";
WinJS.Namespace.define("NewsJS",
{
NewsProviderAdapter: WinJS.Class.define(function (options) {
this._totalCount = options.providers.length === 0 ? 0 : Number.MAX_VALUE;
this._providerDisplayLimit = options.providerDisplayLimit ? options.providerDisplayLimit : -1;
this._providerDatas = [];
this._articleFilter = options.articleFilter && typeof options.articleFilter === "function" ? options.articleFilter : null;
this._noResultsTile = options.noResultsTile;
this._errorTile = options.errorTile;
this._offlineTile = options.offlineTile;
this._runningCount = 0;
this._pendingCountCheck = null;
var providers = options.providers;
if (providers) {
// providers should be either an array or an associative array (map)
if (!Array.isArray(providers)) {
for (var key in providers) {
this._providerDatas.push({ bindings: null, provider: providers[key] });
}
} else {
for (var i = 0; i < providers.length; i++) {
this._providerDatas.push({ bindings: null, provider: providers[i] });
}
}
}
if (options.offline) {
var that = this;
// This check essentially determines whether there is cached data available for snapped mode 
// which is needed information so that it can be determined whether to show the offline error or not.  
this._pendingCountCheck = this.providerIndexContainingIndex(0).then(function (e) {
that._pendingCountCheck = null;
if (e.providerIndex < 0) {
// Setting the total count to 0 signifies to snapped mode's listview that it need not
// query for data up front.  This removes the progress ring when offline and no cached data exists.
that._totalCount = 0;
}
return that._totalCount;
});
}
}, {
removeProvider: function removeProvider(provider) {
var found = false;
for (var i = 0; i < this._providerDatas.length; i++) {
if (!found && this._providerDatas[i].provider === provider) {
this._providerDatas.splice(i, 1);
found = true;
break;
}
}
if (found) {
this._runningCount = 0;
for (var k = 0; k < i; k++) {
if (this._providerDatas[k].bindings) {
// need to find the last valid binding set to update the running count
this._runningCount = this._providerDatas[k].endIndex + 1;
}
}
for (var j = i; j < this._providerDatas.length; j++) {
if (this._providerDatas[j].bindings) {
// we need to reset the start and end index for all providers after the one removed
this._providerDatas[j].bindings = null;
this._providerDatas[j].startIndex = this._providerDatas[j].endIndex = -1;
}
}
}
this._totalCount = this._providerDatas.length > 0 ? Number.MAX_VALUE : 0;
if (this._providerDatas.length === 0) {
if (this.handler) {
this.handler.reload();
}
}
},
insertProvider: function insertProvider(provider) {
this._providerDatas.splice(0, 0, { bindings: null, provider: provider });
// we need to reset the start and end index all providers
for (var i = 0; i < this._providerDatas.length; i++) {
if (this._providerDatas[i].bindings) {
this._providerDatas[i].bindings = null;
this._providerDatas[i].startIndex = this._providerDatas[i].endIndex = -1;
}
}
this._totalCount = Number.MAX_VALUE;
this._runningCount = 0;
},
wentOnline: function wentOnline() {
var foundOffline = false;
for (var i = 0; i < this._providerDatas.length; i++) {
if (foundOffline || this._providerDatas[i].isOffline) {
if (i > 0 && !foundOffline) {
this._runningCount = this._providerDatas[i - 1].endIndex + 1;
}
foundOffline = true;
this._providerDatas[i].bindings = null;
this._providerDatas[i].startIndex = this._providerDatas[i].endIndex = -1;
this._providerDatas[i].isOffline = false;
}
}
if (foundOffline) {
this._totalCount = Number.MAX_VALUE;
if (this.handler) {
this.handler.reload();
}
}
},
// IListDataAdapter
getCount: function getCount() {
console.log("getCount: " + this._totalCount);
// return a huge number for now
return this._pendingCountCheck ? this._pendingCountCheck : WinJS.Promise.wrap(this._totalCount);
},
providerIndexContainingIndex: function providerIndexContainingIndex(index) {
var that = this;
var providerIndex = 0;
var startIndex = 0;
var endIndex = -1;
// Find which provider has the requested item
// Since we don't know how much data each provider will return ahead of time, we need to iterate sequentially
// until we find where we need to be
var limit = this._providerDisplayLimit >= 0 ? Math.min(this._providerDatas.length, this._providerDisplayLimit) : this._providerDatas.length;
for (providerIndex = 0; providerIndex < limit && (endIndex < index || (endIndex === startIndex && endIndex === index)); providerIndex++) {
var providerData = this._providerDatas[providerIndex];
// Does the provider have data we need to convert to bindings?
if (!providerData.bindings && (providerData.isMissing || providerData.hasError || providerData.isOffline || providerData.provider.articles)) {
var articles = providerData.provider.articles || [];
providerData.bindings = [];
var extraBindingEntries = 0;
var binding = null;
if (!providerData.isMissing) {
if (providerData.provider.getSnappedTitleTile) {
binding = providerData.provider.getSnappedTitleTile();
} else if (providerData.provider.title) {
binding = NewsJS.Bindings.snappedTitleTile(providerData.provider.title);
}
}
if (binding) {
providerData.bindings.push({ key: providerIndex + "_title", groupKey: providerIndex + "_", data: binding });
extraBindingEntries++;
}
var origBindingsLength = providerData.bindings.length;
for (var i = 0; i < articles.length; i++) {
if (that._articleFilter) {
if (that._articleFilter.call(that, articles[i])) {
continue;
}
}
binding = NewsJS.Bindings.snappedStoryTile(articles[i]);
providerData.bindings.push({ key: providerIndex + "_" + i, groupKey: providerIndex + "_", data: binding });
}
var messageTile = providerData.isOffline ? this._offlineTile : (providerData.hasError ? this._errorTile : this._noResultsTile);
if (providerData.bindings.length === origBindingsLength && messageTile) {
var tile = null;
if (typeof messageTile === "object") {
tile = messageTile;
} else if (typeof messageTile === "function") {
tile = messageTile();
}
if (tile) {
providerData.bindings.push({ key: providerIndex + "_0", groupKey: providerIndex + "_", data: tile });
}
}
providerData.startIndex = startIndex;
providerData.endIndex = startIndex + providerData.bindings.length - extraBindingEntries;
providerData.extraBindingEntries = extraBindingEntries;
that._runningCount = providerData.endIndex + extraBindingEntries;
}
// Do we already have snapped bindings?
if (providerData.bindings) {
startIndex = providerData.endIndex + (providerData.isMissing ? 0 : 1);
endIndex = providerData.endIndex;
} else {
// Otherwise, query for the data and call this again when data is back
return providerData.provider.queryAsync().then(function (results) {
if (!results) {
// There is simply no data (e.g., unprogrammed in the feed)
// Set the state as missing and skip it in the view
providerData.isMissing = true;
} else {
if (that._totalCount === 0) {
// We now know there is a nonzero dataset.  If the count was previously 0, 
// update it and notify listeners to reload
that._totalCount = Number.MAX_VALUE;
if (that.handler) {
that.handler.reload();
}
}
}
return that.providerIndexContainingIndex(index);
}, function (err) {
// Couldn't retrieve data for this provider.  This must stop the chain across providers
// since the data access model is necessarilly sequential (unless we know the error is not retryable,
// in which case we can continue with the chain in order to render the remaining providers' data
if (NewsJS.Utilities.isBadHttpStatusCode(err)) {
providerData.hasError = true;
return that.providerIndexContainingIndex(index);
} else if (!PlatformJS.Utilities.hasInternetConnection()) {
providerData.isOffline = true;
return that.providerIndexContainingIndex(index);
}
return WinJS.Promise.as({ providerIndex: -1});
});
}
}
// Index was auto-incremented at end of for-loop.  Need to decrement.
return WinJS.Promise.as({ providerIndex: providerIndex - 1 });
},
clusterItemsContainingIndex: function clusterItemsContainingIndex(index) {
var that = this;
return this.providerIndexContainingIndex(index).then(function (e) {
var providerIndex = e.providerIndex;
var providerData = that._providerDatas[providerIndex];
return { title: providerData.provider.title, articles: providerData.provider.articles, groupData: providerData.provider.groupData, clusterArticles: providerData.provider.clusterArticles };
});
},
setNotificationHandler: function setNotificationHandler(handler) {
this.handler = handler;
},
// called when a connection becomes available
reloadContentIfNecessary: function reloadContentIfNecessary() {
var that = this;
if (that._totalCount === 0) {
that._totalCount = Number.MAX_VALUE;
if (that.handler) {
that.handler.reload();
}
} else {
that.wentOnline();
}
},
itemsFromIndex: function itemsFromIndex(index, countBefore, countAfter) {
console.log("itemsFromIndex: " + index + ", " + countBefore + ", " + countAfter);
var that = this;
return this.providerIndexContainingIndex(index).then(function (e) {
var providerIndex = e.providerIndex;
if (providerIndex < 0) {
var count = that._runningCount;
that._totalCount = (that._totalCount < Number.MAX_VALUE) ? that._totalCount : count;
console.log("itemsFromIndex returning: " +
"providerIndex: " + providerIndex +
", absoluteIndex: " + index +
", data.length: 0" +
", offset: " + index +
", atStart: true" +
", atEnd: true" +
", totalCount: " + that._totalCount);
return {
absoluteIndex: index,
items: new WinJS.Binding.List(),
offset: index,
atStart: true,
atEnd: true,
totalCount: that._totalCount
};
}
var providerData = that._providerDatas[providerIndex];
var startIndex = providerData.startIndex;
var endIndex = providerData.endIndex;
var start = Math.max(startIndex, index - countBefore);
var end = Math.min(index + countAfter, endIndex);
var realCount = end - start + 1;
var offset = index - start;
var atStart = start === 0;
var limit = that._providerDisplayLimit >= 0 ? Math.min(that._providerDatas.length, that._providerDisplayLimit) : that._providerDatas.length;
var atEnd = providerIndex >= limit - 1 && index + countAfter >= endIndex;
that._totalCount = (that._totalCount < Number.MAX_VALUE) ? that._totalCount : (atEnd ? endIndex + providerData.extraBindingEntries : Number.MAX_VALUE);
var data = providerData.bindings.slice(start - startIndex, end - startIndex + 1);
console.log("itemsFromIndex returning: " +
"providerIndex: " + providerIndex +
", absoluteIndex: " + index +
", data.length: " + data.length +
", offset: " + offset +
", atStart: " + atStart +
", atEnd: " + atEnd +
", totalCount: " + that._totalCount);
return {
absoluteIndex: index,
items: data,
offset: offset,
atStart: atStart,
atEnd: atEnd,
totalCount: that._totalCount
};
});
}
}),
});
})();
