﻿/* utils\newsUtilitiesArticle.js */
(function () {
"use strict";
function getArticleInfo(editorialArticle) {
var articleInfo = null;
var articleIdPath = NewsJS.Utilities.getEditorialArticleIdPath(editorialArticle);
if (articleIdPath) {
articleInfo = {
articleId: articleIdPath
};
}
return articleInfo;
}
function getArticleReaderNavigationInfo(article, articleIdPath, marketString, articleInfos, themeOverride, /*@optional*/indexBased, /*@optional*/articleIndex) {
var navigation = { state: {}, pageInfo: {} };
var providerType = "AppEx.Common.ArticleReader.BedrockArticleProvider";
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
var queryServiceId = "";
var fallbackQueryServiceId = "";
if (PlatformJS.isDebug && NewsJS.State.temp.isPreviewModeEnabled) {
queryServiceId = "RenderProxy_MarketPreviewBedrockArticles";
fallbackQueryServiceId = "MarketPreviewBedrockArticles";
}
else {
queryServiceId = "RenderProxy_MarketBedrockArticles";
fallbackQueryServiceId = "MarketBedrockArticles2";
}
providerConfiguration.insert("queryServiceId", queryServiceId);
providerConfiguration.insert("fallbackQueryServiceId", fallbackQueryServiceId);
providerConfiguration.insert("imageCacheId", "PlatformImageCache");
providerConfiguration.insert("market", marketString);
if (articleInfos) {
providerConfiguration.insert("articleInfos", articleInfos);
if (indexBased) {
providerConfiguration.insert("indexBased", true);
}
}
var initialArticleId = indexBased ? (articleIndex + "") : articleIdPath;
navigation.state = {
providerType: providerType,
providerConfiguration: providerConfiguration,
initialArticleId: initialArticleId,
enableSnap: true,
enableSharing: true,
adPartnerName: article ? (article.telemetry ? article.telemetry.partnerCode : null) : null,
instrumentationId: article ? (article.telemetry ? article.telemetry.partnerCode : null) : null, // TODO: move to article.telemetry
snappedHeaderTitle: PlatformJS.Services.resourceLoader.getString("News"),
snappedHeaderFontColor: "headerFontLight",
theme: themeOverride || "BingDailyArticleReaderTheme",
market: marketString,
telemetry: article ? article.telemetry : null
};
if (article && article.telemetry) {
if (typeof article.telemetry.source === "number") {
navigation.state.source = article.telemetry.source; // TODO: Get rid of source and use article.telemetry.source
} else {
navigation.state.source = Platform.Instrumentation.InstrumentationEditorialSourceId.protocol; // instrument article as coming from URI (2)
}
navigation.state.entryPoint = article.telemetry.entryPoint;
}
navigation.pageInfo.channelId = null;
navigation.pageInfo.fragment = "/common/ArticleReader/html/ArticleReaderPage.html";
navigation.pageInfo.page = "NewsJS.NewsArticleReaderPage";
if (!PlatformJS.Utilities.hasInternetConnection()) {
var queryService = new PlatformJS.DataService.QueryService(queryServiceId);
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("market", marketString);
urlParams.insert("articleId", articleIdPath);
var fallbackQueryService = new PlatformJS.DataService.QueryService(fallbackQueryServiceId);
var fallbackUrlParams = PlatformJS.Collections.createStringDictionary();
fallbackUrlParams.insert("market", marketString);
fallbackUrlParams.insert("articleId", articleIdPath);
var promises = [];
promises.push(queryService.hasEntryAsync(urlParams, null));
promises.push(fallbackQueryService.hasEntryAsync(fallbackUrlParams, null));
navigation.dataAvailable = new WinJS.Promise(function (complete) {
WinJS.Promise.join(promises).then(function (results) {
if (results[0] || results[1]) {
complete(true);
}
else {
complete(false);
}
}, function (err) {
complete(false);
});
});
} else {
navigation.dataAvailable = WinJS.Promise.wrap(true);
}
return navigation;
}
function getEditorialArticleIdPath(article) {
var articleIdPath = null;
if (article.editorial) {
articleIdPath = article.contentid ? article.articleid + "/" + article.contentid : article.articleid + "";
}
return articleIdPath;
}
function launchArticle(article, listInfo, themeOverride) {
var market = article.market || NewsJS.Globalization.getMarketStringForEditorial();
if (article.editorial) {
if (article.type === "slideshow") {
//TODO: get market for current article
var sourceMarket = market.toLowerCase();
if (article.contentid && sourceMarket) {
var targetState = { slideshowId: article.contentid, market: sourceMarket };
if (article.telemetry && (article.telemetry.entryPoint || article.telemetry.entryPoint === 0)) {
targetState.instrumentationEntryPoint = article.telemetry.entryPoint;
}
WinJS.Navigation.navigate({ fragment: "/html/newsSlideshow.html", page: "NewsJS.SlideShowPage" }, targetState);
}
}
else if (article.type === "article") {
var articleIdPath = getEditorialArticleIdPath(article);
var articleInfos;
NewsJS.Utilities.populateArticleInfoAds(article.clusterArticles);
var indexBased = false;
var articleIndex;
if (article.clusterArticles) {
if (article.clusterArticles.indexBased) {
indexBased = true;
articleIndex = article.clusterArticles.articleIndex;
}
articleInfos = JSON.stringify(article.clusterArticles);
article.clusterArticles = null;
}
var navigation = getArticleReaderNavigationInfo(
article,
articleIdPath,
market.toLowerCase(),
articleInfos,
themeOverride,
indexBased,
articleIndex);
if (navigation && navigation.dataAvailable) {
navigation.dataAvailable.then(function (navigationOk) {
if (navigationOk) {
WinJS.Navigation.navigate({
fragment: navigation.pageInfo.fragment,
page: navigation.pageInfo.page
}, navigation.state);
} else {
NewsJS.Utilities.showToast(PlatformJS.Services.resourceLoader.getString("ArticleUnavailableOffline") || PlatformJS.Services.resourceLoader.getString("/platform/ArticleUnavailableOffline"));
}
});
}
} else if (article.type === "video") {
var videoOptions = article.thumbnail.videoOptions;
CommonJS.MediaApp.Controls.MediaPlayback.fullscreenPlayback(videoOptions.videoSource, videoOptions);
}
} else {
WinJS.Navigation.navigate(
{
fragment: "/html/storyPage.html",
page: "NewsJS.StoryPage"
},
{
article: article,
listInfo: listInfo
});
}
}
function populateArticleInfoAds(articleInfos) {
if (!NewsJS.Utilities._articleAdConfig) {
var articleAdConfigString = NewsJS.Partners.Config.isPartnerApp ? PlatformJS.Services.configuration.getDictionary("PartnerAppInfo").getString("ArticleReaderAdConfig") : PlatformJS.Services.configuration.getDictionary("Ads2").getString("ArticleReaderAdConfig");
if (articleAdConfigString) {
try {
NewsJS.Utilities._articleAdConfig = JSON.parse(articleAdConfigString);
} catch (e) {
NewsJS.Utilities._articleAdConfig = null;
}
}
}
var applicationId = PlatformJS.Ads.Config.instance.applicationId;
if (articleInfos && articleInfos.articleInfos) {
PlatformJS.Utilities.annotateArticleInfosWithAdInfo(articleInfos.articleInfos, NewsJS.Utilities._articleAdConfig, applicationId);
}
}
function createConfig(url, panoId) {
if (url && url.length > 0) {
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
providerConfiguration.insert("url", url);
var prefetchConfig = {
articleIdPath: null,
priority: Platform.DataServices.QueryServicePriority.medium,
providerType: "EntityClusterImageProvider",
providerConfiguration: providerConfiguration,
token: panoId + ":" + AppEx.Common.ArticleReader.GuidProvider.getGuid()
};
return prefetchConfig;
}
}
function getClusterPrefetchConfigs(cluster, panoId) {
var configs = [];
if (cluster) {
if (cluster.sourceImageUrl) {
var imageConfig = createConfig(cluster.sourceImageUrl, panoId);
if (imageConfig) {
configs.push(imageConfig);
}
}
if (cluster.thumbnail) {
var thumbnailConfig = createConfig(cluster.thumbnail.url, panoId);
if (thumbnailConfig) {
configs.push(thumbnailConfig);
}
}
}
return configs;
}
function getPrefetchConfig(article, priority, panoId) {
var prefetchConfig = null;
if (article) {
var articleIdPath = getEditorialArticleIdPath(article);
var nav = getArticleReaderNavigationInfo(article, articleIdPath, NewsJS.Globalization.getMarketStringForEditorial().toLowerCase(), null, null);
var articleReaderState = nav.state;
// create a copy of the provider configuration
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
for (var key in articleReaderState.providerConfiguration) {
providerConfiguration.insert(key, articleReaderState.providerConfiguration[key]);
}
// create the provider
var providerType = articleReaderState.providerType;
var token = articleIdPath;
// create the grouping as group:subgroup where the group is the pano identifier and the subgroup is the article and all its related assets.
if (panoId) {
token = panoId + ":" + articleIdPath + "_" + AppEx.Common.ArticleReader.GuidProvider.getGuid();
}
prefetchConfig = {
articleIdPath: articleIdPath,
priority: priority,
providerType: providerType,
providerConfiguration: providerConfiguration,
token: token
};
}
return prefetchConfig;
}
function prefetchArticle(article, priority, panoId) {
var prefetchConfig = NewsJS.Utilities.getPrefetchConfig(article, priority, panoId);
if (prefetchConfig) {
AppEx.Common.PrefetchUtils.prefetchArticle(
prefetchConfig.articleIdPath,
prefetchConfig.priority,
prefetchConfig.providerType,
prefetchConfig.providerConfiguration,
prefetchConfig.token);
}
}
var prefetchImageService = null;
function prefetchArticlesByPioritizedCategory(newsClusterList, sortedClusterList) {
// Prefetch the hero first.
if (newsClusterList.hero) {
var heroArticle = newsClusterList.hero;
if (heroArticle) {
NewsJS.Utilities.prefetchArticle(heroArticle, Platform.DataServices.QueryServicePriority.high);
}
}
// Prefetch the articles in the order they're displayed.
if (sortedClusterList && newsClusterList.categories) {
for (var i = 0; i < sortedClusterList.length; i++) {
var cluster = sortedClusterList.getItem(i);
if (cluster.data && cluster.data.clusterKey) {
var categoryData = newsClusterList.categories[cluster.data.clusterKey];
if (categoryData && categoryData.entities) {
// use a separate loop for fetching the entity list images since these need to be given
// higher priority than the articles. We are limited by the fact that there are only 3 priority levels
// available today. To mitigate priority collision, we use a separate loop since the prefetch engine
// has inbuilt prioritization by timestamp
var article = null;
for (var k = 0; k < categoryData.entities.length; k++) {
article = categoryData.entities[k];
if (article) {
if (!prefetchImageService) {
prefetchImageService = new Platform.ImageService("PlatformImageCache");
}
var options = new Platform.DataServices.PrefetchQueryServiceOptions();
options.priority = Platform.DataServices.QueryServicePriority.high;
// fetch the low res image if there is one, otherwise get the high res
if (article.thumbnailLowRes && article.thumbnailLowRes.url) {
prefetchImageService.prefetchImage(article.thumbnailLowRes.url, options);
}
else if (article.thumbnail && article.thumbnail.url) {
prefetchImageService.prefetchImage(article.thumbnail.url, options);
}
}
}
// now prefetch all the articles.
for (var j = 0; j < categoryData.entities.length; j++) {
article = categoryData.entities[j];
if (article) {
if (article.type && article.type === "article") {
var priority = Platform.DataServices.QueryServicePriority.low;
if (j < 10) {
priority = Platform.DataServices.QueryServicePriority.high;
}
else if (j >= 10 && j < 20) {
priority = Platform.DataServices.QueryServicePriority.medium;
}
NewsJS.Utilities.prefetchArticle(article, priority);
}
}
}
}
}
}
}
}
WinJS.Namespace.define("NewsJS.Utilities", {
// ARTICLE
_articleAdConfig: null,
getArticleInfo: getArticleInfo,
getArticleReaderNavigationInfo: getArticleReaderNavigationInfo,
getEditorialArticleIdPath: getEditorialArticleIdPath,
launchArticle: launchArticle,
populateArticleInfoAds: populateArticleInfoAds,
prefetchArticle: prefetchArticle,
prefetchArticlesByPioritizedCategory: prefetchArticlesByPioritizedCategory,
getPrefetchConfig: getPrefetchConfig,
getClusterPrefetchConfigs: getClusterPrefetchConfigs,
directionAttribute: WinJS.Binding.converter(function (text) {
return PlatformJS.Utilities.getTextDirection(text);
})
});
})();
/* utils\newsUtilitiesBIDI.js */
(function () {
"use strict";
function fixupBidiPunctuation(text) {
var directionText = PlatformJS.Utilities.getTextDirection(text);
var result = text + ((directionText === "ltr") ? "\u200E" : "\u200F");
return result;
}
function fixupBidiTextNodesWorker(node) {
var that = this;
var nodeText = "";
if (node) {
if (node.nodeType === 3) {
node.data = fixupBidiPunctuation(node.data);
} else if ((node.nodeType === 1 || node.nodeType === 9) && node.childNodes) {
var nodeTagName = node.tagName.toUpperCase();
if (!(nodeTagName === "SCRIPT") && !(nodeTagName === "STYLE")) {
for (var i = 0; i < node.childNodes.length; i++) {
fixupBidiTextNodesWorker(node.childNodes[i]);
}
}
}
}
}
function fixupBidiTextNodes(htmlText, parser) {
var result = htmlText;
if (parser) {
var descDom = parser.parseFromString(htmlText, "text/html");
if (descDom.body) {
fixupBidiTextNodesWorker(descDom.body);
result = descDom.body.innerHTML;
}
}
return result;
}
WinJS.Namespace.define("NewsJS.Utilities", {
// BIDI
fixupBidiPunctuation: fixupBidiPunctuation,
fixupBidiTextNodes: fixupBidiTextNodes
});
})();
/* utils\newsUtilitiesDate.js */
(function () {
"use strict";
// lastUpdateString is optional
function setLastUpdatedTime(lastUpdatedArray, lastUpdatedString) {
if (Array.isArray(lastUpdatedArray)) {
var watermark = document.querySelector(".platformWatermark");
if (watermark) {
var mostRecentDateTime = Math.max.apply(Math, lastUpdatedArray);
if (mostRecentDateTime > 0) {
var dateTime = new Date(mostRecentDateTime);
var market = Platform.Utilities.Globalization.getCurrentMarket();
var dateFormatting = Windows.Globalization.DateTimeFormatting;
var now = new Date();
var anchor = new Date(now.getFullYear(), now.getMonth(), now.getDate());
var dateTimeFormat;
if (anchor > dateTime) {
dateTimeFormat = new dateFormatting.DateTimeFormatter(dateFormatting.YearFormat.none, dateFormatting.MonthFormat.abbreviated, dateFormatting.DayFormat.default, dateFormatting.DayOfWeekFormat.default, dateFormatting.HourFormat.none, dateFormatting.MinuteFormat.none, dateFormatting.SecondFormat.none, [market]);
}
else {
dateTimeFormat = new dateFormatting.DateTimeFormatter(dateFormatting.YearFormat.none, dateFormatting.MonthFormat.none, dateFormatting.DayFormat.none, dateFormatting.DayOfWeekFormat.none, dateFormatting.HourFormat.default, dateFormatting.MinuteFormat.default, dateFormatting.SecondFormat.none, [market]);
}
lastUpdatedString = lastUpdatedString || PlatformJS.Services.resourceLoader.getString("/platform/offline_lastUpdated");
var innerHTML = ["<div id=\"platformLastUpdatedTime\"><span class=\"platformLastUpdatedLabel\">",
lastUpdatedString.format(dateTimeFormat.format(dateTime)),
"</span></div>"
].join("");
watermark.innerHTML = toStaticHTML(innerHTML);
watermark.tabIndex = 0;
}
else {
watermark.innerHTML = "";
watermark.tabIndex = -1;
}
}
}
}
function removeLastUpdatedTime() {
var lastUpdatedTime = document.getElementById("platformLastUpdatedTime");
if (lastUpdatedTime) {
lastUpdatedTime.parentNode.removeChild(lastUpdatedTime);
}
}
WinJS.Namespace.define("NewsJS.Utilities", {
// DATE
setLastUpdatedTime: setLastUpdatedTime,
removeLastUpdatedTime: removeLastUpdatedTime
});
})();
/* utils\newsUtilitiesDOM.js */
(function () {
"use strict";
function addAboutPageFragment() {
var aboutElement = document.getElementById("AboutPageDiv");
if (!aboutElement) {
aboutElement = document.createElement("div");
aboutElement.id = "AboutPageDiv";
document.body.appendChild(aboutElement);
}
var promise = new WinJS.Promise(function (complete) {
WinJS.UI.Fragments.renderCopy("/html/aboutSettings.html", aboutElement).then(function (doc) {
var version = Windows.ApplicationModel.Package.current.id.version;
var aboutLabelText = PlatformJS.Services.resourceLoader.getString("About");
// App Name and PublisherDisplayName should be retrieved from Platform Configuration Manifest, but they're not populated
var appName = PlatformJS.Services.resourceLoader.getString("News");
var publisherName = PlatformJS.Services.resourceLoader.getString("MicrosoftCorp");
var poweredBy = PlatformJS.Services.resourceLoader.getString("PoweredBy");
var buildVersion = version.major + "." + version.minor + "." + version.build + "." + version.revision;
var dataContext = {
versionText: PlatformJS.Services.resourceLoader.getString("/platform/appVersion").format(buildVersion),
aboutLabelText: aboutLabelText,
appName: appName,
publisherName: publisherName,
poweredBy: poweredBy,
};
WinJS.UI.processAll(aboutElement).then();
WinJS.Binding.processAll(aboutElement, dataContext);
complete(PlatformJS.Utilities.getControl("AboutPage"));
}, function (error) {
complete(null);
});
});
return promise;
}
function removeAboutPageFragment() {
var aboutElement = document.getElementById("AboutPageDiv");
if (aboutElement) {
document.body.removeChild(aboutElement);
}
}
function stripHTML(string) {
if (string) {
return string.replace(/<(?:.|\n)*?>/gm, "");
} else {
return null;
}
}
WinJS.Namespace.define("NewsJS.Utilities", {
// DOM
addAboutPageFragment: addAboutPageFragment,
removeAboutPageFragment: removeAboutPageFragment,
stripHTML: stripHTML
});
})();
/* utils\newsUtilitiesError.js */
(function () {
"use strict";
function isBadHttpStatusCode(error) {
// 103 = NetworkFailureBadHttpStatusCode
return PlatformJS.Utilities.getPlatformErrorCode(error) === 103;
}
function cancelPromise(promise) {
if (promise && promise.cancel && typeof promise.cancel === "function") {
promise.cancel();
}
}
function isFREOffline(error) {
if (Array.isArray(error)) {
for (var i = 0; i < error.length; i++) {
if (isFREOffline(error[i])) {
return true;
}
}
return false;
} else {
return (error && error.message.indexOf("FREOffline") === 0);
}
}
function isDataMissing(error) {
return (error && error.message.indexOf("DataError:ExpectedDataMissing") === 0);
}
function isCancellationError(error) {
return (error && (error.name === "Error" || error.name === "Canceled") &&
error.message && error.message === "Canceled");
}
function isRetryableNetworkError(error) {
// 101 = NetworkTimeout, 102 = NetworkFailureWithNoCachedData : retryable
// 103 = NetworkFailureBadHttpStatusCode : not retryable
var errorCode = PlatformJS.Utilities.getPlatformErrorCode(error);
return errorCode === 101 || errorCode === 102;
}
function isSyntaxError(error) {
return (error && (error.name === "SyntaxError") &&
error.message && error.message === "SyntaxError");
}
function showErrorMessage(message, targetChannel) {
var sNavButtonText = "";
switch (targetChannel) {
case "MySources":
sNavButtonText = PlatformJS.Services.resourceLoader.getString("AllSources");
break;
case "MyTopics":
sNavButtonText = "";
break;
default:
sNavButtonText = PlatformJS.Services.resourceLoader.getString("Unfollow");
break;
}
var messageBar = new CommonJS.MessageBar(message);
if (!sNavButtonText === "") {
messageBar.addButton(sNavButtonText, function () { PlatformJS.Navigation.navigateToChannel(targetChannel, { showActionEdgy: true }); });
}
messageBar.addButton(PlatformJS.Services.resourceLoader.getString("Dismiss"), function () { messageBar.hide(); });
messageBar.show();
}
WinJS.Namespace.define("NewsJS.Utilities", {
// ERROR
isBadHttpStatusCode: isBadHttpStatusCode,
cancelPromise: cancelPromise,
isDataMissing: isDataMissing,
isFREOffline: isFREOffline,
isCancellationError: isCancellationError,
isRetryableNetworkError: isRetryableNetworkError,
isSyntaxError: isSyntaxError,
showErrorMessage: showErrorMessage
});
})();
/* utils\newsUtilitiesMisc.js */
(function () {
"use strict";
// TODO: Move to config
function getAnchorMapping(mappingText) {
var fallBackAnchor = "anchorLeft";
if (mappingText === "Top") {
return "anchorTop";
}
else if (mappingText === "Left") {
return "anchorLeft";
}
else if (mappingText === "Bottom") {
return "anchorBottom";
}
else if (mappingText === "Right") {
return "anchorRight";
}
else if (mappingText === "Middle") {
return "anchorMiddle";
}
else {
return fallBackAnchor;
}
}
function appServerDomain() {
var domain = PlatformJS.Services.appConfig.getString("AppServerDomain");
return domain || "appex.bing.com";
}
function getRawArticleUrl(url, path, paramName) {
var rawUrl = url;
if (url && url.length > 0) {
var uri = Windows.Foundation.Uri(url);
if (uri.path && uri.path.toLowerCase() === path && paramName && paramName.length > 0) {
var query = uri.query;
if (query && query.length > 0 && query[0] === "?") {
var queryTextValue = query.substring(1);
var vars = queryTextValue.split("&");
for (var i = 0; i < vars.length; i++) {
var pair = vars[i].split("=");
if (pair && pair.length === 2 && pair[0].toLowerCase() === paramName) {
rawUrl = pair[1];
break;
}
}
}
}
}
return decodeURIComponent(rawUrl);
}
function isPrefetchEnabled() {
return NewsJS && NewsJS.PrefetchManager && NewsJS.PrefetchManager.isPrefetchEnabled;
}
function _prefetchToggled(evt) {
if (NewsJS && NewsJS.PrefetchManager) {
NewsJS.PrefetchManager.prefetchToggled(evt);
}
}
WinJS.Namespace.define("NewsJS.Utilities", {
// MISC        
getAnchorMapping: getAnchorMapping,
appServerDomain: appServerDomain,
getRawArticleUrl: getRawArticleUrl,
isPrefetchEnabled: isPrefetchEnabled,
prefetchToggled: {
get: function get() {
_prefetchToggled.supportedForProcessing = true;
return _prefetchToggled;
}
},
isPrefetchUserEnabled: {
get: function get() {
return NewsJS && NewsJS.PrefetchManager && NewsJS.PrefetchManager.isPrefetchUserEnabled;
}
},
isPrefetchCloudEnabled: {
get: function get() {
return PlatformJS.Services.manifest.prefetching.enablePrefetching && NewsJS && NewsJS.PrefetchManager && NewsJS.PrefetchManager.isPrefetchCloudEnabled;
}
},
offlineSyncConverter: WinJS.Binding.converter(function (val) {
return val ? "block" : "none";
})
});
})();
/* utils\newsUtilitiesNavigation.js */
(function () {
"use strict";
function navigateToSource(source, updateStateFunction, entrypoint, /*@optional*/ market) {
var navigation = navigationInfoForSource(source, entrypoint, market);
if (navigation) {
updateStateFunction(source);
if (navigation.state && navigation.state.dynamicInfo) {
navigation.state.dynamicInfo.entrypoint = entrypoint;
}
WinJS.Navigation.navigate(navigation.pageInfo, navigation.state);
}
}
function buildStateForPartnerSource(source) {
var state = { dynamicInfo: { isFeaturedSource: true, dataProviderOptions: { useByline: true } } };
state.dynamicInfo.channelID = source.id;
state.dynamicInfo.adUnitId = source.adunitid;
state.dynamicInfo.id = source.id;
state.dynamicInfo.instrumentationId = source.instrumentationid;
if (source.displayname) {
state.dynamicInfo.panoTitle = source.displayname;
}
if (source.feedmarket) {
state.dynamicInfo.feedMarket = source.feedmarket;
}
if (source.featuredurl) {
state.dynamicInfo.feedName = source.featuredurl;
}
if (source.css) {
state.dynamicInfo.css = source.css;
}
if (source.backbuttontype) {
state.dynamicInfo.backbuttontype = source.backbuttontype;
}
if (source.themetype === 0) {
state.dynamicInfo.theme = "dynamicPanoBoxedTheme";
}
if (source.headerlogo) {
state.dynamicInfo.logoUrl = source.headerlogo;
}
if (source.headerlogosnapped) {
state.dynamicInfo.snapLogoUrl = source.headerlogosnapped;
}
return state;
}
function navigationInfoForPartnerSource(source, entrypoint, sourceMarket) {
if (source.channelid && !source.featuredurl) {
return PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel(source.channelid, "featured");
}
var state = buildStateForPartnerSource(source);
state.dynamicInfo.entrypoint = entrypoint;
state.dynamicInfo.market = sourceMarket;
return { pageInfo: { channelId: state.dynamicInfo.id, fragment: "/dynamicPano/dynamicPano.html", page: "DynamicPano.DynamicPanoPage" }, state: state };
}
function navigationInfoForSource(source, entrypoint, /*@optional*/ market) {
var isFeatured = false;
var isRss = false;
var navigation = null;
var channelInfo = null;
if (source && source.featured) {
isFeatured = true;
} else {
if (source && source.rss_urls && source.rss_names) {
source.feedNamesArray = source.rss_names.split(",");
source.feedUrlsArray = source.rss_urls.split(",");
if (source.feedNamesArray.length === source.feedUrlsArray.length) {
isRss = true;
} else {
// treat this source as an algo Source and remove corrupt feed info 
source.rss_urls = source.rss_names = source.feedNamesArray = source.feedUrlsArray = null;
}
}
}
if (isFeatured) {
if (source.channel_id) {
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel(source.channel_id, "featured");
if (!channelInfo) {
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel(source.channel_id);
}
if (channelInfo) {
navigation = channelInfo;
}
} else if (source.displayname && source.featured_url) {
var dynamicInfo = { panoTitle: source.displayname, feedName: source.featured_url, dataProviderOptions: { useByline: true }, isFeaturedSource: true, id: source.id };
if (source.feedmarket) {
dynamicInfo.feedMarket = source.feedmarket;
}
if (entrypoint) {
dynamicInfo.entrypoint = entrypoint;
}
if (source.partner_header_logo) {
dynamicInfo.logoUrl = source.partner_header_logo;
if (source.partner_header_logo_snap) {
dynamicInfo.snapLogoUrl = source.partner_header_logo_snap;
}
}
if (source.theme) {
dynamicInfo.theme = source.theme;
}
if (source.css) {
dynamicInfo.css = source.css;
}
if (source.ad_unit_id) {
dynamicInfo.adUnitId = source.ad_unit_id;
}
if (source.instrumentation_id) {
dynamicInfo.instrumentationId = source.instrumentation_id;
}
if (source.backbuttontype) {
dynamicInfo.backbuttontype = source.backbuttontype;
}
if (market) {
dynamicInfo.market = market;
}
navigation = { pageInfo: { channelId: dynamicInfo.id, fragment: "/dynamicPano/dynamicPano.html", page: "DynamicPano.DynamicPanoPage" }, state: { dynamicInfo: dynamicInfo } };
}
} else if (isRss) {
// Go to RSS source pano
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("RSSSourcePage");
if (channelInfo) {
navigation = { pageInfo: channelInfo.pageInfo, state: { entity: source } };
}
} else {
// Go to the regular individual Source page
var targetState = { entity: source };
if (market) {
targetState.market = market;
}
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("SourceNavPage");
if (channelInfo) {
navigation = { pageInfo: channelInfo.pageInfo, state: targetState };
}
}
return navigation;
}
WinJS.Namespace.define("NewsJS.Utilities", {
// NAVIGATION
buildStateForPartnerSource: buildStateForPartnerSource,
navigateToSource: navigateToSource,
navigationInfoForPartnerSource: navigationInfoForPartnerSource,
navigationInfoForSource: navigationInfoForSource
});
})();
/* utils\newsUtilitiesSources.js */
(function () {
"use strict";
function fetchFeaturedSourcesData(market) {
if (!market) {
return WinJS.Promise.wrap({});
}
var infoQueryService = new PlatformJS.DataService.QueryService("FeaturedSourcesDataSource");
var infoParams = PlatformJS.Collections.createStringDictionary();
infoParams.insert("market", market.toLowerCase());
// overridden partner data url?
if (Platform.Configuration.ConfigurationManager.partnerDataFeedUrl.length) {
infoQueryService = new PlatformJS.DataService.QueryService("FeaturesConfigurationFullUrlDataSource");
infoParams.insert("url", Platform.Configuration.ConfigurationManager.partnerDataFeedUrl);
}
if (PlatformJS.isDebug && NewsJS.State.temp.isPreviewModeEnabled) {
infoParams["previewparams"] = "/" + PlatformJS.Services.appConfig.getString("CMSPreviewParam");
}
else {
infoParams["previewparams"] = "";
}
var options = new Platform.DataServices.QueryServiceOptions();
options.bypassCache = false;
return new WinJS.Promise(function (complete, error, progress) {
infoQueryService.downloadDataAsync(infoParams, null, null, options).done(
function (data) {
var errorResult = null;
try {
var response = data && data.dataString ? JSON.parse(data.dataString) : null;
if (response && response.sources && Array.isArray(response.sources)) {
var array = response.sources;
var featuredSources = {};
for (var idx = 0; idx < array.length; idx++) {
featuredSources[array[idx].id] = array[idx];
}
complete(featuredSources);
} else {
errorResult = new SyntaxError("SyntaxError");
}
} catch (err) {
errorResult = err;
}
if (errorResult) {
error(errorResult);
}
},
function (e) {
if (PlatformJS.Utilities.isPromiseCanceled(e)) {
return;
}
error(e);
});
});
}
function readSources(/*@optional*/market) {
var editorialMarketString = NewsJS.Globalization.getMarketStringForEditorial();
var localMarket = { market: market || NewsJS.Globalization.getMarketStringForSources() };
var isLocalMarketValid = localMarket.market && localMarket.market.length > 0;
var worldWideMarket = { market: PlatformJS.Services.appConfig.getString("WorldWideMarket") };
var sourcesProviders = [];
var mergeFeaturedSources = isLocalMarketValid && localMarket.market === editorialMarketString && PlatformJS.Services.appConfig.getDictionary("SourcesConfig").getBool("MergeFeaturedSourcesDataSource");
if (!market && Platform.Utilities.Globalization.isFeatureEnabled("ROWTopEdgy")) {
if (isLocalMarketValid) {
sourcesProviders.push(new NewsJS.SourcesGalleryProvider(localMarket));
}
sourcesProviders.push(new NewsJS.SourcesGalleryProvider(worldWideMarket));
} else if (!market && Platform.Utilities.Globalization.isFeatureEnabled("ROW_ENWW_TopEdgy")) {
sourcesProviders.push(new NewsJS.SourcesGalleryProvider(worldWideMarket));
} else {
if (isLocalMarketValid) {
sourcesProviders.push(new NewsJS.SourcesGalleryProvider(localMarket));
}
}
var promises = [];
if (mergeFeaturedSources) {
promises.push(fetchFeaturedSourcesData(editorialMarketString).then(function (results) {
return WinJS.Promise.wrap(results);
}, function (error) {
return WinJS.Promise.wrap({});
}));
}
for (var sourceIdx = 0; sourceIdx < sourcesProviders.length; sourceIdx++) {
promises.push(sourcesProviders[sourceIdx].fetchData());
}
var featuredSources = null;
return WinJS.Promise.join(promises).then(
function (e) {
var allSources = [];
var sourcesResultsIndex = 0;
if (mergeFeaturedSources) {
featuredSources = e[0];
sourcesResultsIndex = 1;
}
for (var i = sourcesResultsIndex; i < e.length; i++) {
var sourceProvider = e[i];
if (sourceProvider && sourceProvider.dataResponse && sourceProvider.dataResponse.Topics) {
for (var j = 0; j < sourceProvider.dataResponse.Topics.length; j++) {
allSources.push(NewsJS.Utilities.createSourceData(sourceProvider, sourceProvider.dataResponse.Topics[j], null, null, featuredSources));
}
}
}
return WinJS.Promise.wrap(allSources);
}, function (err) {
return WinJS.Promise.wrap(null);
}
);
}
function _getFollowedSources(options) {
var sourceKey = "followedSources";
if (options && options.market && options.guid) {
sourceKey += "_" + options.market.toUpperCase() + "_" + options.guid;
}
if (!NewsJS.State.roaming[sourceKey]) {
NewsJS.State.roaming[sourceKey] = [];
}
var followedSources = NewsJS.State.roaming[sourceKey];
return followedSources;
}
function _setFollowedSources(newSources, options) {
var sourceKey = "followedSources";
if (options && options.market && options.guid) {
sourceKey += "_" + options.market.toUpperCase() + "_" + options.guid;
}
if (!NewsJS.State.roaming[sourceKey]) {
NewsJS.State.roaming[sourceKey] = [];
}
NewsJS.State.roaming[sourceKey] = newSources;
}
function readUserSources(options) {
/// <summary>
/// Reads the list of sources from the roaming data. Assumes state has been loaded and sanity verified.
/// </summary>
/// <returns>list of sources saved in roaming</returns>
var followedSources = _getFollowedSources(options);
// return a copy as we don't want anyone to reference the roaming store directly
var fSources = JSON.parse(JSON.stringify(followedSources));
return WinJS.Promise.wrap(fSources);
}
function showFollowSourcesExceededError() {
var message = PlatformJS.Services.resourceLoader.getString("SourcesExceeded");
NewsJS.Utilities.showErrorMessage(message, "MySources");
}
function isSourceFollowed(source, options) {
var index = -1;
var followedSources = _getFollowedSources(options);
for (var i = 0; i < followedSources.length; i++) {
var followedSource = followedSources[i];
if (followedSource.id === source.id) {
index = i;
break;
}
}
return index >= 0;
}
function saveSourcesAsync(sources, options) {
/// <summary>
/// saves the sources list again to preserve order
/// </summary>
/// <returns>TRUE</returns>
_setFollowedSources(sources, options);
return WinJS.Promise.wrap({});
}
function setShowFeaturedFlag(key) {
NewsJS.State.roaming[key] = false;       
return NewsJS.State.saveAll();
}
function getShowFeaturedFlag(key) {
return NewsJS.State.roaming[key] === false ? false : true;
}
function _addSource(source, options) {
/// <summary>
/// Adds the given source to the list of followed sources and pushes it to the front. Assumes state has been loaded and sanity verified.
/// </summary>
/// <returns>TRUE if the source was actually added (ie, it was a new source), FALSE if the source is already being followed.</returns>
// Remove writer if previously followed so we can move the writer to the front
var existingIndex = -1;
var followedSources = _getFollowedSources(options);
for (var i = 0; i < followedSources.length; i++) {
var followedSource = followedSources[i];
if (followedSource.id === source.id) {
existingIndex = i;
break;
}
}
if (existingIndex >= 0) {
followedSources.splice(existingIndex, 1);
}
followedSources.splice(0, 0, source);
var promise = NewsJS.State.saveAll();
return promise.then(function () {
return (existingIndex === -1);
});
}
function _removeSource(source, options) {
// Remove writer if previously followed so we can move the writer to the front
var index = -1;
var followedSources = _getFollowedSources(options);
for (var i = 0; i < followedSources.length; i++) {
var followedSource = followedSources[i];
if (followedSource.id === source.id) {
index = i;
break;
}
}
if (index >= 0) {
followedSources.splice(index, 1);
}
return NewsJS.State.saveAll();
}
function updateSources(addSources, removeSources, options, ignoreToast) {
/// <summary>
/// Adds an array of sources to the user's list of followed sources. Examines each source to determine whether it's already followed - if so, move it to the front.
/// Blocks user from adding if the configurable limit is reached.
/// </summary>
var success;
var msg;
var fileIOPromise = WinJS.Promise.wrap(null);
addSources = addSources || [];
removeSources = removeSources || [];
var numberOfNewsSources = addSources.length - removeSources.length;
if (addSources.length > 0 || removeSources.length > 0) {
var followedSources = _getFollowedSources(options);
var numNewSources = 0;
var newSourceIndex = 0;
var src = null;
// remove items
var followedSourcesCopy = JSON.parse(JSON.stringify(followedSources));
for (k = 0; k < removeSources.length; k++) {
src = removeSources[k];
existingIndex = -1;
for (i = 0; i < followedSourcesCopy.length; i++) {
followedSource = followedSourcesCopy[i];
if (followedSource.id === src.id) {
existingIndex = i;
break;
}
}
if (existingIndex > -1) {
followedSourcesCopy.splice(existingIndex, 1);
}
}
// add items
for (var k = 0; k < addSources.length; k++) {
src = addSources[k];
var existingIndex = -1;
for (var i = 0; i < followedSourcesCopy.length; i++) {
var followedSource = followedSourcesCopy[i];
if (followedSource.id === src.id) {
existingIndex = i;
break;
}
}
if (existingIndex === -1) {
followedSourcesCopy.splice(0, 0, src);
}
}
_setFollowedSources(followedSourcesCopy, options);
// persist state
fileIOPromise = NewsJS.State.saveAll();
}
return fileIOPromise;
}
// expected behavior example:
// adding [1, 2, 3] to [x, y] should result in [1, 2, 3, x, y]
function followSources(sources, options, ignoreToast) {
/// <summary>
/// Adds an array of sources to the user's list of followed sources. Examines each source to determine whether it's already followed - if so, move it to the front.
/// Blocks user from adding if the configurable limit is reached.
/// </summary>
var success;
var msg;
var fileIOPromise = WinJS.Promise.wrap(null);
if (sources && sources.length > 0) {
var followedSources = _getFollowedSources(options);
// reverse orignal sources to make sure once added to roaming array, it's still in the same order
var reverseSources = JSON.parse(JSON.stringify(sources));
reverseSources.reverse();
// If we've hit the limit, alert the user and don't add any from the current selection set. Otherwise, add the new sources.
if (followedSources && followedSources.length + sources.length > PlatformJS.Services.appConfig.getDictionary("SourcesConfig").getInt32("maxFollows")) {
msg = PlatformJS.Services.resourceLoader.getString("MaxSourcesReached").format(followedSources.length, PlatformJS.Services.appConfig.getDictionary("SourcesConfig").getInt32("maxFollows"));
fileIOPromise.then(function () {
success = false;
showFollowSourcesExceededError();
return success;
});
} else {
var numNewSources = 0;
var newSourceIndex = 0;
var i = 0;
var addSource = function (index) {
return _addSource(reverseSources[index], options).then(function (result) {
i++;
if (result === true) {
numNewSources = i;
}
else {
return false;
}
if (i < sources.length) {
return addSource(i);
}
else {
return true;
}
});                    
};
//adding a source is an ascynchronous operation, we need a promise chain.
fileIOPromise = addSource(0);
fileIOPromise.then(function () {
success = true;
if (numNewSources > 1) {
msg = PlatformJS.Services.resourceLoader.getString("NowFollowingSources").format(numNewSources);
} else if (numNewSources === 1) {
msg = PlatformJS.Services.resourceLoader.getString("NowFollowingSource").format(sources[newSourceIndex].displayname);
}
else {
success = false;
}
if (msg && !ignoreToast) {
NewsJS.Utilities.showToast(msg);
}
return success;
});
}
}
return fileIOPromise;
}
function unfollowSources(sources, options, ignoreToast) {
var success;
var promises = [];
if (sources && sources.length > 0) {
for (var i = 0; i < sources.length; i++) {
promises.push(_removeSource(sources[i], options));
}
var msg;
if (sources.length > 1) {
msg = PlatformJS.Services.resourceLoader.getString("NowUnfollowingSources").format(sources.length);
} else {
msg = PlatformJS.Services.resourceLoader.getString("NowUnfollowingSource").format(sources[0].displayname);
}
success = true;
if (!ignoreToast) {
NewsJS.Utilities.showToast(msg);
}
}
return WinJS.Promise.join(promises);
}
function updateStateOnSourceVisited(sourceId, market) {
//This used to maintain the map of 'recent' sources, which is no longer used by the app.
//TODO: clean up by removing this function from everywhere it's called
}
function readFeaturedSources(market) {
return readSources(market).then(
function (results) {
var featuredSources = [];
var allSources = results;
for (var s in allSources) {
var source = allSources[s];
if (source.featured && !source.is_not_featured) {
featuredSources.push(source);
}
}
return WinJS.Promise.wrap(featuredSources);
}, function (err) {
return WinJS.Promise.wrap(null);
}
);
}
function createSourceData(provider, entity, recentSources, /*@optional*/createFeaturedSourcesCluster, /*@optional*/featuredSources) {
var fullImageUrl = "";
var win8Image = entity[provider.columnMap["win8_image"]];
if (provider && provider.dataResponse && provider.dataResponse.Image && provider.dataResponse.Image.Path && win8Image) {
fullImageUrl = provider.dataResponse.Image.Path.replace("{0}", win8Image);
}
/*note: is_not_featured is "yes"/true if the source should be a partner pano, but should not show up in the featured sources lists.  Ex: The Guardian in EN-US*/
var result = {
freq: recentSources ? recentSources[entity[provider.columnMap["rb_hash"]]] : null,
alpha: entity[provider.columnMap["alphaorder"]],
displayname: entity[provider.columnMap["displayname"]],
win8_image: fullImageUrl,
id: entity[provider.columnMap["rb_hash"]],
title: entity[provider.columnMap["topicname"]],
sourcedescription: entity[provider.columnMap["sourcedescription"]],
rss_names: entity[provider.columnMap["rss_names"]],
rss_urls: entity[provider.columnMap["rss_urls"]],
newscategory: entity[provider.columnMap["newscategory"]],
is_not_featured: entity[provider.columnMap["is_not_featured"]] === "yes" ? true : false,
};
if (featuredSources && featuredSources[result.id]) {
var featuredSource = featuredSources[result.id];
result.featured_category = featuredSource.featuredcategory;
result.featured_url = featuredSource.featuredurl;
if (featuredSource.feedmarket) {
result.feedmarket = featuredSource.feedmarket;
}
result.channel_id = featuredSource.featuredurl ? null : featuredSource.channelid;
result.ad_unit_id = featuredSource.adunitid;
result.instrumentation_id = featuredSource.instrumentationid;
result.partner_header_logo = featuredSource.headerlogo;
result.partner_header_logo_snap = featuredSource.headerlogosnapped;
if (featuredSource.themetype === 0) {
result.theme = "dynamicPanoBoxedTheme";
} else {
result.theme = "dynamicPanoUnboxedTheme";
}
result.backbuttontype = featuredSource.backbuttontype;
result.css = featuredSource.css;
} else {
result.featured_category = entity[provider.columnMap["featured_category"]];
result.featured_url = entity[provider.columnMap["featured_url"]];
result.channel_id = entity[provider.columnMap["channel_id"]];
result.ad_unit_id = entity[provider.columnMap["ad_unit_id"]];
result.instrumentation_id = entity[provider.columnMap["instrumentation_id"]];
result.partner_header_logo = entity[provider.columnMap["partner_header_logo"]];
result.partner_header_logo_snap = entity[provider.columnMap["partner_header_logo_snap"]];
}
var featured = result.featured_url || result.channel_id;
if (featured) {
// This is a featured source.  Record this fact
result.featured = true;
if (!featuredSources) {
result.theme = "dynamicPanoUnboxedTheme";
if (provider && provider.dataResponse && provider.dataResponse.Image && provider.dataResponse.Image.Path && result.partner_header_logo) {
var fullImageHeaderUrl = provider.dataResponse.Image.Path.replace("{0}", result.partner_header_logo);
result.partner_header_logo = fullImageHeaderUrl;
if (result.partner_header_logo_snap) {
var fullImageSnapHeaderUrl = provider.dataResponse.Image.Path.replace("{0}", result.partner_header_logo_snap);
result.partner_header_logo_snap = fullImageSnapHeaderUrl;
}
}
}
// update the newscategory so it appears under the "Featured Sources" cluster in Sources pano
if (createFeaturedSourcesCluster && result.newscategory !== null && !result.is_not_featured) {
result.newscategory = result.newscategory + "," + PlatformJS.Services.resourceLoader.getString("Featured");
}
// record a map of the BingDaily category ids to which this featured source belongs
var categoryKeys = result.featured_category ? result.featured_category.split(",") : [];
result.categoryKeys = {};
if (categoryKeys) {
categoryKeys.forEach(function (key) {
result.categoryKeys[key] = 1;
});
}
}
return result;
}
WinJS.Namespace.define("NewsJS.Utilities", {
// SOURCES
createSourceData: createSourceData,
fetchFeaturedSourcesData: fetchFeaturedSourcesData,
isSourceFollowed: isSourceFollowed,
followSources: followSources,
unfollowSources: unfollowSources,
readSources: readSources,
readUserSources: readUserSources,
updateStateOnSourceVisited: updateStateOnSourceVisited,
saveSourcesAsync: saveSourcesAsync,
readFeaturedSources: readFeaturedSources,
updateSources: updateSources,
setShowFeaturedFlag: setShowFeaturedFlag,
getShowFeaturedFlag: getShowFeaturedFlag
});
})();
/* utils\newsUtilitiesString.js */
(function () {
"use strict";
// Use locale culture to compare strings
function culturedStringCompare(str1, str2) {
return str1.toLocaleLowerCase().localeCompare(str2.toLocaleLowerCase());
}
// Use locale culture to compare strings
function culturedStringsEqual(str1, str2) {
return culturedStringCompare(str1, str2) === 0;
}
//Create a hash code from a string
function GetStringHashCode(str) {
return News.NewsUtil.instance.getStringHashCode(str);
}
function trimString(s) {
if (!s) {
return s;
}
var re_ltrim = /^\s+/;
var re_rtrim = /\s+$/;
var re_mtrim = /\s+/;
return ((s.replace(re_ltrim, "")).replace(re_rtrim, "")).replace(re_mtrim, " ");
}
WinJS.Namespace.define("NewsJS.Utilities", {
// STRING
culturedStringCompare: culturedStringCompare,
culturedStringsEqual: culturedStringsEqual,
GetStringHashCode: GetStringHashCode,
trimString: trimString
});
})();
/* utils\newsUtilitiesTile.js */
(function () {
"use strict";
function disableTileNotifications() {
// Turn off notification cycling
var Notifications = Windows.UI.Notifications;
var isDesign = Windows.ApplicationModel.DesignMode.designModeEnabled;
if (Notifications && !isDesign) {
var tileUpdater = Notifications.TileUpdateManager.createTileUpdaterForApplication();
if (tileUpdater) {
tileUpdater.enableNotificationQueue(false);
return tileUpdater;
}
}
return null;
}
function enableTileNotifications() {
// Turn on notification cycling
var Notifications = Windows.UI.Notifications;
var isDesign = Windows.ApplicationModel.DesignMode.designModeEnabled;
if (Notifications && !isDesign) {
var tileUpdater = Notifications.TileUpdateManager.createTileUpdaterForApplication();
if (tileUpdater && tileUpdater.setting === Notifications.NotificationSetting.enabled) {
tileUpdater.enableNotificationQueue(true);
return tileUpdater;
}
}
return null;
}
function addItemToTileNotification(title) {
var Notifications = Windows.UI.Notifications;
if (Notifications) {
// Make wide template
var tileXml = Notifications.TileUpdateManager.getTemplateContent(Notifications.TileTemplateType.tileWideText03);
var tileTextAttributes = tileXml.getElementsByTagName("text");
tileTextAttributes[0].appendChild(tileXml.createTextNode(title));
var binding = tileXml.getElementsByTagName("binding");
binding[0].setAttribute("branding", "name");
// Make a square template
var squareTileXml = Notifications.TileUpdateManager.getTemplateContent(Notifications.TileTemplateType.tileSquareText04);
tileTextAttributes = squareTileXml.getElementsByTagName("text");
tileTextAttributes[0].appendChild(squareTileXml.createTextNode(title));
binding = squareTileXml.getElementsByTagName("binding");
binding[0].setAttribute("branding", "name");
var node = tileXml.importNode(squareTileXml.getElementsByTagName("binding").item(0), true);
tileXml.getElementsByTagName("visual").item(0).appendChild(node);
var tileNotification = new Notifications.TileNotification(tileXml);
Notifications.TileUpdateManager.createTileUpdaterForApplication().update(tileNotification);
}
}
function registerLiveTileNotifications(forceClear) {
var tileUpdater = null;
// Check to see if we need to unregister the OOBE live tile registration
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
var oobeLiveTileState = localSettings.values["OOBELiveTileState"];
if (oobeLiveTileState !== "2") {
if (oobeLiveTileState === "1") {
tileUpdater = disableTileNotifications();
if (tileUpdater) {
tileUpdater.clear();
tileUpdater.stopPeriodicUpdate();
}
}
// Change state to indicate unregistered
localSettings.values["OOBELiveTileState"] = "2";
}
// Check to see if this is the first time we're doing this or if the user's market has changed.
var currLiveTileMarket = NewsJS.State.local.liveTileMarket;
var newLiveTileMarket = NewsJS.Globalization.getMarketStringForLiveTile();
if (!newLiveTileMarket) {
// Disable notifications if we don't support live tiles for this market
tileUpdater = disableTileNotifications();
if (tileUpdater) {
// Stop and clear the previous periodic update to get rid of data from previous market
tileUpdater.clear();
tileUpdater.stopPeriodicUpdate();
}
} else {
// Otherwise enable
tileUpdater = enableTileNotifications();
if (tileUpdater) {
if (currLiveTileMarket !== newLiveTileMarket || forceClear) {
// Stop and clear the previous periodic update to get rid of data from previous market
tileUpdater.clear();
tileUpdater.stopPeriodicUpdate();
}
var urlList = PlatformJS.Services.appConfig.getList("MarketLiveTileURLs");
if (urlList) {
var urls = [];
var updateFrequency = PlatformJS.Services.appConfig.getInt32("LiveTileUpdateFrequency");
for (var i = 0, l = urlList.size; i < l; i++) {
var url = urlList[i].value;
if (url) {
while (url.indexOf("{mkt}") >= 0) {
url = url.replace("{mkt}", newLiveTileMarket);
}
var uri = Windows.Foundation.Uri(url);
if (uri) {
urls.push(uri);
}
}
}
tileUpdater.startPeriodicUpdateBatch(urls, updateFrequency);
}
}
}
// Stash way the new live tile market
NewsJS.State.local.liveTileMarket = newLiveTileMarket;
NewsJS.State.saveLocal();
}
WinJS.Namespace.define("NewsJS.Utilities", {
// TILE
addItemToTileNotification: addItemToTileNotification,
disableTileNotifications: disableTileNotifications,
enableTileNotifications: enableTileNotifications,
registerLiveTileNotifications: registerLiveTileNotifications
});
})();
/* utils\newsUtilitiesTopics.js */
(function () {
"use strict";
/// <summary>
/// sort function used when sorting a group of clusters, when the sort criteria are based on type, frequency, and recency
/// NOTE: the entities being sorted are panorama clusters.  
/// The top level sort criteria is based on the ordinal property "clusterPosition".  
/// Clusters which are grouped together share the same clusterPosition.  Hence these are sorted based on criteria in the underlying entity
/// The clusterEntity property on the cluster holds the entity state which is assumed to include: type, ageCount, and searchCount.
/// Entity types discriminate highest in the sort hierarchy, with type === "favorite" being above any other type, all other types treated equivalently.
/// A "search" or "default" type entity uses the searchCount property as the next highest sorting criteria. Favorites ignore searchCount.
/// The final sort criteria is ageCount which is a measure of how recently an entity has changed, the higher the value, the older it is.
/// </summary>
function sortByTypeAndFrequency(l, r) {
var result;
if (l.clusterPosition === r.clusterPosition) {
var lEntity = l.clusterEntity;
var rEntity = r.clusterEntity;
if (!lEntity || !rEntity) {
// This shouldn't occur, but protect against an exception in case it does
return 0;
}
// collapse default topics into search topics for comparison
var lEntityType = lEntity.type === "default" ? "search" : lEntity.type;
var rEntityType = rEntity.type === "default" ? "search" : rEntity.type;
if (lEntityType === rEntityType) {
// comparing same-type topics
if (lEntityType === "search" && lEntity.searchCount !== rEntity.searchCount) {
// prefer a search topic with a higher search count (frequency)
result = lEntity.searchCount > rEntity.searchCount ? -1 : 1;
} else {
// default to a recency score
var lCriterion, rCriterion;
if (lEntity.lastVisitDate && rEntity.lastVisitDate) {
if (lEntity.ageCount === rEntity.ageCount) {
lCriterion = lEntity.lastVisitDate;
rCriterion = rEntity.lastVisitDate;
} else {
// give more recently visited items a greater chance to bubble up
var now = new Date().getTime();
// one hour = 1000 * 60 * 60;
var oneHour = 3600000;
var leftDiff = Math.ceil((now - lEntity.lastVisitDate) / oneHour);
var rightDiff = Math.ceil((now - rEntity.lastVisitDate) / oneHour);
if (leftDiff === 0 || rightDiff === 0) {
leftDiff = 1;
rightDiff = 1;
}
lCriterion = lEntity.ageCount + (1 / leftDiff);
rCriterion = rEntity.ageCount + (1 / rightDiff);
}
} else {
lCriterion = lEntity.ageCount;
rCriterion = rEntity.ageCount;
}
result = lCriterion > rCriterion ? -1 : (lCriterion === rCriterion ? 0 : 1);
}
} else {
// comparing different-type topics, prefer a favorite
result = lEntity.type === "favorite" ? -1 : (rEntity.type === "favorite" ? 1 : 0);
}
} else {
// different logical clusters, so use the cluster position
result = l.clusterPosition < r.clusterPosition ? -1 : 1;
}
return result;
}
function sameTopics(topic1, topic2) {
return topic1 && topic2 && (topic1 === topic2 || topic1.title.toLocaleLowerCase() === topic2.title.toLocaleLowerCase());
}
function topicIndexOf(topic) {
var result = -1;
var len = NewsJS.State.roaming.followedSearchTopics.length;
for (var index = 0; index < len; index++) {
if (topic.toLocaleLowerCase() === NewsJS.State.roaming.followedSearchTopics[index].title.toLocaleLowerCase()) {
result = index;
break;
}
}
return result;
}
function isTopicFollowed(topic) {
return (0 <= topicIndexOf(topic));
}
function followTopic(topic) {
var success = addToFollowedSearchTopics(topic, true, true);
if (success) {
NewsJS.TopEdgy.instance.setupNewsMyTopics();  // Refresh top edgy MyNews content
}
return success;
}
function registerSearchTopic(topic, addIfNew) {
return addToFollowedSearchTopics(topic, false, addIfNew);
}
function showFollowTopicsExceededError() {
var message = PlatformJS.Services.resourceLoader.getString("TopicsExceeded");
NewsJS.Utilities.showErrorMessage(message, "MyTopics");
}
function addToFollowedSearchTopics(topic, markAsFavorite, addIfNew) {
var maxDisplay = PlatformJS.Services.appConfig.getDictionary("TopicsConfig").getInt32("maxFollowsDisplay");
var maxStore = PlatformJS.Services.appConfig.getDictionary("TopicsConfig").getInt32("maxFollowsStore");
var favoriteCount = NewsJS.State.roaming.followedSearchTopicsState.state.favoriteCount;
var storedCount = NewsJS.State.roaming.followedSearchTopics.length;
var topicExists = false;
var topicIsFavorite = false;
var ageCount = ++NewsJS.State.local.followedSearchTopicsState.state.ageCount;
var index = topicIndexOf(topic);
if (index >= 0) {
topicExists = true;
topicIsFavorite = NewsJS.State.roaming.followedSearchTopics[index].type === "favorite";
}
// Don't take any action if we shouldn't be adding this topic if it doesn't already exist
if (!addIfNew && !topicExists) {
return;
}
// Check if user exceeded limit
if (markAsFavorite && !topicExists && favoriteCount >= maxDisplay) {
showFollowTopicsExceededError();
return false;
}
var searchCount = markAsFavorite ? 0 : 1;
if (!topicExists && storedCount >= maxStore) {
// need to eject a previous search to make room for this one
NewsJS.State.roaming.followedSearchTopics.splice(storedCount - 1, 1);
storedCount--;
}
var wasDefault = false;
var addRequired = true;
if (topicExists) {
wasDefault = NewsJS.State.roaming.followedSearchTopics[index].type === "default";
searchCount += NewsJS.State.roaming.followedSearchTopics[index].searchCount;
if (topicIsFavorite && !markAsFavorite) {
// update the topic in place
// NOTE: when updating in place, don't modify the ageCount
NewsJS.State.roaming.followedSearchTopics[index].searchCount = searchCount;
addRequired = false;
}
// Remove topic if previously followed so we can properly insert it in its new location
if (addRequired) {
NewsJS.State.roaming.followedSearchTopics.splice(index, 1);
storedCount--;
}
}
var insertAt = 0;
if (addRequired) {
insertAt = topicIsFavorite || markAsFavorite ? 0 : favoriteCount;
if (insertAt >= favoriteCount && !topicIsFavorite && !markAsFavorite) {
// inserting a searched topic.  Find the spot by scanning for the first topic with a searchCount <= this one.
// this should leave the state of the array in an invariant state orderred by searchCount (outside the favorite entries)
// further, it gives preference to recently made searches to the degree that they go ahead of equally-searched terms
while (insertAt < storedCount && NewsJS.State.roaming.followedSearchTopics[insertAt].searchCount > searchCount) {
insertAt++;
}
}
NewsJS.State.roaming.followedSearchTopics.splice(insertAt, 0, { title: topic, query: topic, searchCount: searchCount, ageCount: ageCount, type: topicIsFavorite || markAsFavorite ? "favorite" : (wasDefault ? "default" : "search") });
}
if (!topicIsFavorite && markAsFavorite) {
++NewsJS.State.roaming.followedSearchTopicsState.state.favoriteCount;
}
NewsJS.State.saveAllUpdatingAgeCount();
return true;
}
function unfollowTopics(topicTitles) {
var len = topicTitles.length;
if (len > 1) {
for (var idx = 0; idx < len; idx++) {
unfollowTopicWorker(topicTitles[idx], false);
}
// TODO: showToast for all topics removed.
NewsJS.State.saveAllUpdatingAgeCount();
}
else if (len === 1) {
unfollowTopicWorker(topicTitles[0], true);
}
NewsJS.TopEdgy.instance.setupNewsMyTopics();  // Refresh top edgy MyNews content
}
function unfollowTopic(topic) {
unfollowTopicWorker(topic, true);
NewsJS.TopEdgy.instance.setupNewsMyTopics();  // Refresh top edgy MyNews content
return true;
}
function unfollowTopicWorker(topic, atomic) {
// Remove topic if previously followed so we can move the topic to the front
var index = topicIndexOf(topic);
if (index >= 0) {
// Here we bump up ageCount so it can be used as a means for establishing whether
// a change on the topic pano has occurred or not.  ageCount increases now
// whenever an a topic is added or removed.
++NewsJS.State.local.followedSearchTopicsState.state.ageCount;
if (NewsJS.State.roaming.followedSearchTopics[index].type === "favorite") {
NewsJS.State.roaming.followedSearchTopicsState.state.favoriteCount--;
}
NewsJS.State.roaming.followedSearchTopics.splice(index, 1);
if (atomic) {
NewsJS.State.saveAllUpdatingAgeCount();
}
}
return index;
}
function getFollowedTopics() {
var followedTopics = NewsJS.State.roaming.followedSearchTopics;
// return a copy as we don't want anyone to reference the roaming store directly
var fTopics = JSON.parse(JSON.stringify(followedTopics));
return fTopics;
}
var searchDialogVisible = false;
function updateSuggestions(event, sources) {
var that = this;
var text = event.srcElement.value;
var srcElement = document.getElementById(event.srcElement.id);
var autoSuggest = PlatformJS.Utilities.getControl(srcElement);
if (autoSuggest && searchDialogVisible) {
getNewsSuggestions(text, sources).then(function (suggestionList) {
if (searchDialogVisible) {
autoSuggest.itemDataSource = suggestionList;
} else {
console.log("Suggestion response received when autosuggest is inactive.");
}
});
} else {
console.log("Suggestion request received when autosuggest is inactive.");
}
}
function getNewsSuggestions(query, sources) {
return new WinJS.Promise(function (complete, error) {
if (!sources) {
var marketString = NewsJS.Globalization.getMarketStringForAutosuggest();
var dataPromise = marketString.length > 0 ? NewsJS.Data.Bing.getQuerySuggestions(query, marketString) : WinJS.Promise.wrap({ dataObjectList: { size: 0 } });
dataPromise.then(function (results) {
// Convert the results to a JS array
var resultsArray = [];
for (var i = 0; i < results.dataObjectList.size; i++) {
var result = results.dataObjectList.getAt(i);
resultsArray.push({ Symbol: result, displayedText: result, value: result });
}
complete(resultsArray);
}, function (e) {
complete([]);
//error(e);
});
}
else {
var convert = function (entity) {
return { Symbol: entity.displayname, value: entity.id };
};
var resultsArray2 = NewsJS.Autosuggest.findMatchingEntitiesUsingPartialMatch(sources, query, true, convert);
complete(resultsArray2);
}
});
}
// used as part of a temporary workaround for bug 129094 to determine when 
// invalid characters are present in a topic string.
function hasInvalidCharacters(s) {
var invalid = false;
try {
encodeURIComponent(s);
} catch (err) {
console.log("** Invalid characters encountered in input string: " + s + ". err=" + err);
invalid = true;
}
return invalid;
}
function presentTopicModalPrompt(autoSuggestMode, sources, title) {
var promise = new WinJS.Promise(function (complete) {
CommonJS.dismissAllEdgies();
var dialog = CommonJS.SearchDialog.show();
searchDialogVisible = true;
var dismissed = false;
dialog.searchPlaceholderText = PlatformJS.Services.resourceLoader.getString("TopicFlyoutPlaceholderText");
dialog.promptText = title ? title : PlatformJS.Services.resourceLoader.getString("AddTopic");
var textEntered = function (value) {
var topicText = NewsJS.Utilities.trimString(value.queryText);
if (topicText.length > 0) {
// Don't accept a topic with invalid characters (bug 129094)
if (hasInvalidCharacters(topicText)) {
dialog.searchBoxText = "";
} else {
dismissed = true;
searchDialogVisible = false;
dialog.searchVisible = false;
return complete(topicText);
}
}
};
var onKeyUp = function (event) {
NewsJS.Utilities.updateSuggestions(event, sources);
};
dialog.autoSuggestOptions = {
itemDataSource: [],
onItemSelection: textEntered,
keyup: onKeyUp,
selectionMode: autoSuggestMode ? autoSuggestMode : CommonJS.AutoSuggest.freeForm
};
dialog.onHide = function () {
if (!dismissed) {
return complete();
}
};
dialog.onSearchTextEntered = textEntered;
dialog.onCloseButtonClick = function () {
dismissed = true;
searchDialogVisible = false;
dialog.searchVisible = false;
return complete();
};
});
return promise;
}
function imageCardTopics(url) {
return {
url: url,
cacheId: "TopicsData"
};
}
WinJS.Namespace.define("NewsJS.Utilities", {
// TOPICS        
sortByTypeAndFrequency: sortByTypeAndFrequency,
followTopic: followTopic,
isTopicFollowed: isTopicFollowed,
sameTopics: sameTopics,
topicIndexOf: topicIndexOf,
unfollowTopic: unfollowTopic,
unfollowTopics: unfollowTopics,
registerSearchTopic: registerSearchTopic,
getNewsSuggestions: getNewsSuggestions,
updateSuggestions: updateSuggestions,
presentTopicModalPrompt: presentTopicModalPrompt,
getFollowedTopics: getFollowedTopics,
imageCardTopics: WinJS.Binding.converter(imageCardTopics)
});
})();
/* utils\newsUtilitiesUI.js */
(function () {
"use strict";
function _setupLocationButtons() {
var _changeLocationMenuItems = [];
var localPanoManager = CommonJS.LocalPano.LocalPanoManager.instance;
if (localPanoManager.isLocalPanoEnabled) {
return localPanoManager.getRegionsFeed().then(function (regionData) {
var bottomEdgyMinimumRegionCount = Platform.Configuration.ConfigurationManager.custom.getDictionary("LocalPano").getInt32("BottomEdgyMinimumRegionCount", -1);
var result = false;
if (regionData && regionData.regions && regionData.regions.length > 0 && regionData.regions.length >= bottomEdgyMinimumRegionCount) {
result = true;
for (var i = 0; i < regionData.regions.length; i++) {
var region = regionData.regions[i];
var item = {
label: region.name,
selected: localPanoManager.isActiveRegion(region.id),
value: region.id
};
_changeLocationMenuItems.push(item);
}
var changeLocationFlyout = null;
var changeLocationFlyoutOptions = {
buttonText: PlatformJS.Services.resourceLoader.getString("ChangeLocationButtonText"),
icon: "\uE185",
extraClass: "appexSymbol",
selectionChanged: function selectionChanged(selectedItem) {
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
_showOfflineErrorForLocation();
// Change the selection back to what it was in the first place.
for (var j = 0; j < _changeLocationMenuItems.length; j++) {
var menuItem = _changeLocationMenuItems[j];
menuItem.selected = localPanoManager.isActiveRegion(menuItem.value);
}
}
else {
for (var k = 0; k < _changeLocationMenuItems.length; k++) {
_changeLocationMenuItems[k].selected = (k === selectedItem.index);
}
changeLocationFlyout.setItems(_changeLocationMenuItems);
var oldRegion = localPanoManager.activeRegion;
localPanoManager.activeRegion = selectedItem.value;
CommonJS.dismissAllEdgies();
NewsJS.Telemetry.LocalNews.recordLocationSelection(localPanoManager.activeRegion, oldRegion);
}
}
};
changeLocationFlyout = new CommonJS.SortFlyout(null, changeLocationFlyoutOptions);
changeLocationFlyout.setItems(_changeLocationMenuItems);
var changeLocationCmd = changeLocationFlyout.appbarbtn;
changeLocationCmd.element.id = "changeLocationButton";
changeLocationCmd.onclick = function (event) {
for (var j = 0; j < _changeLocationMenuItems.length; j++) {
var menuItem = _changeLocationMenuItems[j];
menuItem.selected = localPanoManager.isActiveRegion(menuItem.value);
}
changeLocationFlyout.setItems(_changeLocationMenuItems);
NewsJS.Telemetry.LocalNews.recordChangeLocationClick();
};
// Current Location button setup.
var currentLocationCmd = new WinJS.UI.AppBarCommand(document.createElement("button"), {
label: PlatformJS.Services.resourceLoader.getString("CurrentLocationButtonText"),
extraClass: "appexSymbol",
icon: "\uE013",
onclick: function onclick(event) {
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
_showOfflineErrorForLocation();
}
else {
var oldRegion = localPanoManager.activeRegion;
localPanoManager.refreshRegionByGeoposition().then(function refreshRegionSuccess() {
if (!localPanoManager.hasRegionData) {
_deselectLocation(_changeLocationMenuItems);
_showNoContentErrorForLocation();
}
NewsJS.Telemetry.LocalNews.recordCurrentLocationClick(localPanoManager.activeRegion, oldRegion);
NewsJS.Telemetry.LocalNews.recordLocationSelection(localPanoManager.activeRegion, oldRegion);
},
function refreshRegionError() {
if (Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
_deselectLocation(_changeLocationMenuItems);
_showNoLocationError();
}
else {
_showOfflineErrorForLocation();
}
NewsJS.Telemetry.LocalNews.recordCurrentLocationClick(localPanoManager.activeRegion, oldRegion);
NewsJS.Telemetry.LocalNews.recordLocationSelection(localPanoManager.activeRegion, oldRegion);
});
}
CommonJS.dismissAllEdgies();
}
});
currentLocationCmd.element.id = "currentLocationButton";
// Add buttons to bottom edgy.
var edgy = WinJS.Utilities.query(".newsActionEdgyRight");
if (edgy && edgy.length > 0) {
edgy = edgy[0];
} else {
edgy = document.getElementById("bottomEdgy");
}
if (edgy) {
var refreshButton = PlatformJS.Utilities.getControl("refreshButton");
edgy.insertBefore(changeLocationCmd.element, refreshButton.element);
edgy.insertBefore(currentLocationCmd.element, refreshButton.element);
}
}
return WinJS.Promise.wrap(result);
},
function (error) {
return WinJS.Promise.wrap(false);
// Ignore error.
});
} else {
return WinJS.Promise.wrap(false);
}
}
function _deselectLocation(_changeLocationMenuItems) {
var localPanoManager = CommonJS.LocalPano.LocalPanoManager.instance;
var oldRegion = localPanoManager.activeRegion;
for (var j = 0; j < _changeLocationMenuItems.length; j++) {
_changeLocationMenuItems[j].selected = false;
}
localPanoManager.activeRegion = null;
NewsJS.Telemetry.LocalNews.recordLocationSelection(localPanoManager.activeRegion, oldRegion);
}
function _showOfflineErrorForLocation() {
var messageString = PlatformJS.Services.resourceLoader.getString("/platform/feedbackOfflineMessage");
var buttonString = PlatformJS.Services.resourceLoader.getString("/platform/close");
var messageBar = new CommonJS.MessageBar(messageString, { autoHide: false });
messageBar.addButton(buttonString, function () { messageBar.hide(); });
messageBar.show();
}
function _showNoContentErrorForLocation() {
var messageString = PlatformJS.Services.resourceLoader.getString("/platform/offline_noContent");
var messageBar = new CommonJS.MessageBar(messageString, { autoHide: true });
messageBar.show();
}
function _showNoLocationError() {
var messageString = PlatformJS.Services.resourceLoader.getString("/Common/LocationCannotBeFound");
var buttonString = PlatformJS.Services.resourceLoader.getString("/platform/close");
var messageBar = new CommonJS.MessageBar(messageString, { autoHide: false });
messageBar.addButton(buttonString, function () { messageBar.hide(); });
messageBar.show();
}
function showToast(message) {
var messageBar = new CommonJS.MessageBar(message, { autoHide: true });
messageBar.addButton(PlatformJS.Services.resourceLoader.getString("Dismiss") ||
PlatformJS.Services.resourceLoader.getString("/partners/Dismiss"), function () { messageBar.hide(); });
messageBar.show();
}
function disableButton(buttonId, disabled) {
var button = PlatformJS.Utilities.getControl(buttonId);
if (button) {
button.disabled = disabled;
}
}
WinJS.Namespace.define("NewsJS.Utilities", {
// UI
disableButton: disableButton,
showToast: showToast,
setupLocationButtons: _setupLocationButtons
});
})();
/* utils\newsUtilitiesURI.js */
(function () {
"use strict";
function getAppProtocol() {
return PlatformJS.Services.appConfig.getString("AppProtocol", "bingnews");
}
function parseCMSUriString(uriString, /*@optional*/defaultMarket) {
var result = parseUriString(uriString, getAppProtocol(), defaultMarket);
if (result && result.path === "application/view") {
result.entitytype = result.params["entitytype"];
result.pageId = result.params["pageid"];
result.contentId = result.params["contentid"];
} else {
result = null;
}
return result;
}
function parseUriString(uriString, /*@optional*/defaultProtocol, /*@optional*/defaultMarket) {
var result = null;
if (uriString) {
if (0 >= uriString.indexOf(":") && defaultProtocol) {
// no protocol present, try prepending the default
uriString = defaultProtocol + ":" + uriString;
}
try {
result = parseUri(new Windows.Foundation.Uri(uriString), defaultMarket);
} catch (ex) {
;
}
}
return result;
}
function parseUri(uri, /*@optional*/defaultMarket) {
var result = null;
if (uri) {
if (!uri.suspicious) {
var query = uri.query;
var path = uri.path;
var host = uri.host;
var actualPath = null;
if (host) {
actualPath = host.toLowerCase();
if (path) {
var pathValue = path.toLowerCase();
if (pathValue !== "/") {
actualPath += path.toLowerCase();
}
}
} else {
if (path) {
actualPath = path.toLowerCase();
}
}
if (actualPath) {
var urlParams = PlatformJS.Collections.createStringDictionary();
var queryParser = uri.queryParsed;
var len = queryParser ? queryParser.size : 0;
for (var idx = 0; idx < len; idx++) {
var entry = queryParser.getAt(idx);
if (entry && entry.name && entry.name.length > 0 && entry.value && entry.value.length > 0) {
try {
urlParams[entry.name.toLowerCase()] = entry.value; // the entry value is decoded automatically by the queryParser object
} catch (err) {
;
}
}
}
if (!urlParams["market"] && defaultMarket) {
urlParams["market"] = defaultMarket;
}
result = { protocol: uri.schemeName, params: urlParams, path: actualPath, uri: uri };
}
}
}
return result;
}
function mapUriToNavigation(result) {
var navigation = null;
var urlParams = result.params;
var targetState;
var channelInfo = null;
var entityType = urlParams["entitytype"];
var pageId, sourceMarket, entryPoint;
var contentId = null;
if (entityType === "article") {
// deep link to article
var articleIdPath = urlParams["articleid"];
contentId = urlParams["contentid"];
pageId = urlParams["pageid"];
sourceMarket = urlParams["market"];
if (pageId) {
articleIdPath = pageId;
if (contentId) {
articleIdPath += "/" + contentId;
}
}
if (articleIdPath && sourceMarket) {
navigation = WinJS.Promise.wrap(NewsJS.Utilities.getArticleReaderNavigationInfo(null, articleIdPath, sourceMarket));
}
} else if (entityType === "slideshow") {
// deep link to slideshow
sourceMarket = urlParams["market"];
pageId = urlParams["pageid"];
var image = urlParams["image"];
if (pageId && sourceMarket) {
targetState = { slideshowId: pageId, market: sourceMarket };
if (image) {
var imageValue = parseInt(image);
if (imageValue > 0) {
targetState.imageIndex = imageValue - 1;
}
}
navigation = WinJS.Promise.wrap({ state: targetState, pageInfo: { channelId: null, fragment: "/html/newsSlideshow.html", page: "NewsJS.SlideShowPage" } });
}
} else if (entityType === "video") {
contentId = urlParams["contentid"];
if (contentId) {
targetState = { videoContentId: contentId };
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("Home");
navigation = WinJS.Promise.wrap({ state: targetState, pageInfo: channelInfo.pageInfo });
}
} else if (entityType === "allvideos") {
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("Video");
targetState = channelInfo.state;
navigation = WinJS.Promise.wrap({ state: targetState, pageInfo: channelInfo.pageInfo });
} else if (entityType === "alltopics") {
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("MyTopics");
targetState = channelInfo.state;
navigation = WinJS.Promise.wrap({ state: targetState, pageInfo: channelInfo.pageInfo });
} else if (entityType === "allsources") {
targetState = {
mode: NewsJS.Personalization.Utilities.selectionPageFormat.FORMAT_SOURCE,
features: [NewsJS.Personalization.Utilities.selectionPageFeature.TYPE_SOURCES,
NewsJS.Personalization.Utilities.selectionPageFeature.TYPE_INTERNATIONAL],
isBrowse: true,
title: PlatformJS.Services.resourceLoader.getString("BrowseSourceTile")
};
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("SelectionPage");
navigation = WinJS.Promise.wrap({ state: targetState, pageInfo: channelInfo.pageInfo });
} else if (entityType === "addsource") {
targetState = {
mode: NewsJS.Personalization.Utilities.selectionPageFormat.FORMAT_SOURCE,
features: [NewsJS.Personalization.Utilities.selectionPageFeature.TYPE_SOURCES,
NewsJS.Personalization.Utilities.selectionPageFeature.TYPE_INTERNATIONAL],
isBrowse: false,
title: PlatformJS.Services.resourceLoader.getString("AddSourceTile")
};
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("SelectionPage");
navigation = WinJS.Promise.wrap({ state: targetState, pageInfo: channelInfo.pageInfo });
} else if (entityType === "browseinternational") {
targetState = {
mode: NewsJS.Personalization.Utilities.selectionPageFormat.FORMAT_SOURCE,
features: [NewsJS.Personalization.Utilities.selectionPageFeature.TYPE_SOURCES,
NewsJS.Personalization.Utilities.selectionPageFeature.TYPE_INTERNATIONAL],
isBrowse: true,
title: PlatformJS.Services.resourceLoader.getString("BrowseSourceTile"),
selectedCategory: "internationalBingDailyTab",
};
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("SelectionPage");
navigation = WinJS.Promise.wrap({ state: targetState, pageInfo: channelInfo.pageInfo });
} else if (entityType === "pano") {
var panoId = urlParams["panoid"];
var categoryId = urlParams["categoryid"];
if (panoId) {
// deep link to BingDaily category pano
if (panoId === "bingdaily" && categoryId) {
if (!Platform.Utilities.Globalization.isFeatureEnabled("ROWTopEdgy") &&
!Platform.Utilities.Globalization.isFeatureEnabled("ROW_ENWW_TopEdgy")) {
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel("CategoryPage", "standard");
if (channelInfo && channelInfo.pageInfo) {
targetState = { categoryKey: categoryId };
navigation = WinJS.Promise.wrap({ state: targetState, pageInfo: channelInfo.pageInfo });
}
}
} else if (!categoryId) {
entryPoint = urlParams["entrypoint"] || "deeplink";
if (entryPoint === "endofarticleblock") {
var method = CommonJS.ArticleReader.ArticleReaderUtils.impressionNavMethod.endOfArticleBlock;
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(method);
NewsJS.Telemetry.ArticleReader.recordActionClick(panoId);
}
// deep link to featured partner pano
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel(panoId, "featured");
if (!channelInfo) {
channelInfo = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel(panoId, "standard");
}
if (channelInfo && channelInfo.pageInfo) {
navigation = WinJS.Promise.wrap({ state: channelInfo.state, pageInfo: channelInfo.pageInfo });
}
}
}
} else if (entityType === "partnerPano") {
var sourceId = urlParams["sourceid"];
entryPoint = urlParams["entrypoint"] || "deeplink";
sourceMarket = urlParams["market"];
var featuredSourceMarket = sourceMarket || NewsJS.Globalization.getMarketStringForEditorial();
if (sourceId) {
navigation = new WinJS.Promise(function (complete) {
NewsJS.Utilities.fetchFeaturedSourcesData(featuredSourceMarket).then(function (partnerSources) {
var partnerSource = partnerSources[sourceId];
if (partnerSource) {
complete(NewsJS.Utilities.navigationInfoForPartnerSource(partnerSource, entryPoint, sourceMarket));
}
complete(null);
}, function (error) {
if (entryPoint === "bingdailycluster" && !PlatformJS.Utilities.hasInternetConnection() && !NewsJS.Utilities.isCancellationError(error)) {
NewsJS.Utilities.showToast(PlatformJS.Services.resourceLoader.getString("/platform/offline_noContent"));
complete(null);
}
});
});
}
} else {
// Get entity view
var entityView = PlatformJS.Navigation.entityViewMap[entityType];
// Validate entity view
if (entityView && result && result.params) {
// Create action
var actionProtocol = entityView(result.params);
// Validate channel info
if (WinJS.Promise.is(actionProtocol)) {
actionProtocol.then(function (target) {
if (target && target.pageInfo) {
WinJS.Navigation.navigate(
target.pageInfo,
target.state);
}
});
} else if (actionProtocol.pageInfo) {
WinJS.Navigation.navigate(
actionProtocol.pageInfo,
actionProtocol.state);
} else if (actionProtocol.channelInfo) {
navigation = WinJS.Promise.wrap({ pageInfo: actionProtocol.channelInfo.pageInfo, state: actionProtocol.options });
} else {
actionProtocol();
}
}
}
return navigation;
}
function stringAsUri(value) {
/// <signature helpKeyword="NewsJS.Utilities.stringAsUri">
/// <summary locid="NewsJS.Utilities.stringAsUri">
/// Helper function for wrapping try/catch when creating a uri
/// </summary>
/// <param name="value" type="String" locid="NewsJS.Utilities.stringAsUri:value">
/// The string to be used in Uri constructor
/// </param>
/// <returns type="Object" locid="NewsJS.Utilities.stringAsUri_returnValue">
/// A Windows.Foundation.Uri representation of the string
/// </returns>
/// </signature>
try {
return new Windows.Foundation.Uri(value);
}
catch (e) {
return null;
}
}
WinJS.Namespace.define("NewsJS.Utilities", {
// URI
mapUriToNavigation: mapUriToNavigation,
getAppProtocol: getAppProtocol,
parseCMSUriString: parseCMSUriString,
parseUri: parseUri,
parseUriString: parseUriString,
stringAsUri: stringAsUri
});
})();
