﻿(function () {
"use strict";
// setup a category pin button
// buttonElementId: the html id of the button on the page                
// category: context data
function setupPinCategoryButton(buttonElementId, telemetryId, category, categoryName) {
if (typeof (buttonElementId) === "undefined" ||
typeof (category) === "undefined" ||
typeof (categoryName) === "undefined") {
return;
}
if (typeof (category.categoryKey) === "undefined") {
//this is the headline category so there was no catkey
category.categoryKey = "";
}
return NewsJS.Utilities.Pinning._setupPinButton(
buttonElementId,
PlatformJS.Services.resourceLoader.getString("/platform/pinToStart"),
PlatformJS.Services.resourceLoader.getString("/platform/unpinFromStart"),
PlatformJS.Services.resourceLoader.getString("NowPinningCategory"),
PlatformJS.Services.resourceLoader.getString("NowUnpinningCategory"),
"",  // no tileNotAddedToastMessage message
"",  // no tileNotRemovedToastMessage message
categoryName + "NewsJS.CategoryPage", // make the a ID guaranteed unique tile (someone could pin "U.S." as category and topic)
categoryName,
categoryName,
"NewsJS.CategoryPage",
{ category: categoryName, categoryKey: category.categoryKey, categoryName: category.categoryName, market: category.market },
null, // no existingArticles
"ms-appx:///images/logoCategory.png",
null, // no tilePinLogos
null, // no liveTileUrls
null, // no liveTileRecurrence
null, // no onPinSection
telemetryId);
}
// setup a search (topic) pin button
// buttonElementId: the html id of the button on the page
// category: context data
function setupPinTopicButton(buttonElementId, telemetryId, topic) {
if (typeof (buttonElementId) === "undefined" ||
typeof (topic) === "undefined" ||
typeof (topic.queryText) === "undefined" ||
!topic.queryText) {
return;
}
return NewsJS.Utilities.Pinning._setupPinButton(
buttonElementId,
PlatformJS.Services.resourceLoader.getString("/platform/pinToStart"),
PlatformJS.Services.resourceLoader.getString("/platform/unpinFromStart"),
PlatformJS.Services.resourceLoader.getString("NowPinningTopic"),
PlatformJS.Services.resourceLoader.getString("NowUnpinningTopic"),
"",  // no tileNotAddedToastMessage message
"",  // no tileNotRemovedToastMessage message
topic.queryText + "NewsJS.Search", // make the a ID guaranteed unique tile (someone could pin "U.S." as category and topic)
topic.queryText,
topic.queryText,
"NewsJS.Search",
topic,
null, // no existingArticles
"ms-appx:///images/logoTopic.png",
null, // no tilePinLogos
null, // no liveTileUrls
null, // no liveTileRecurrence
null, // no onPinSection
telemetryId);
}
// setup a regular source and RSSsource pin button
// buttonElementId: the html id of the button on the page
// category: context data
// target: optional; the target page the app should launch into when the pin button on start menu is clicked
function setupPinSourceButton(buttonElementId, telemetryId, sourceEntity, target) {
if (typeof (buttonElementId) === "undefined" ||
typeof (sourceEntity) === "undefined" ||
typeof (sourceEntity.id) === "undefined" ||
typeof (sourceEntity.displayname) === "undefined") {
return;
}
return NewsJS.Utilities.Pinning._setupPinButton(
buttonElementId,
PlatformJS.Services.resourceLoader.getString("/platform/pinToStart"),
PlatformJS.Services.resourceLoader.getString("/platform/unpinFromStart"),
PlatformJS.Services.resourceLoader.getString("NowPinningSource"),
PlatformJS.Services.resourceLoader.getString("NowUnpinningSource"),
"",  //no tileNotAddedToastMessage message
"",  //no tileNotRemovedToastMessage message
sourceEntity.id + (target ? target : "NewsJS.SourcePage"),  // differientiate RSS source tiles from source tiles in case they have the same name
sourceEntity.displayname,
sourceEntity.displayname,
target ? target : "NewsJS.SourcePage",
sourceEntity,
null, // no existingArticles
"ms-appx:///images/logoSource.png",
null, // no tilePinLogos
null, // no liveTileUrls
null, // no liveTileRecurrence
null, // no onPinSection
telemetryId);
}
// generic functionality for setting up a pinButton
function _setupPinButton(
buttonElementId,
buttonTextForAddTile,
buttonTextForRemoveTile,
tileAddSuccessToastMessage,
tileRemoveSuccessToastMessage,
tileNotAddedToastMessage,
tileNotRemovedToastMessage,
tileId,
tileShortName,
tileDisplayName,
tileLaunchTargetPage,
tileLaunchArgument,
existingArticles,
tileLogo,
tilePinLogos,
liveTileUrls,
liveTileRecurrence,
onPinSection,
telemetryId) {
var pinIcon = "\uE015";
var unpinIcon = "\uE016";
var pinElement = document.getElementById(buttonElementId);
var pinButton = pinElement.winControl;
var tileExceedsMaxMessage = PlatformJS.Services.resourceLoader.getString("PinExceeded");
var config = PlatformJS.Services.appConfig.getDictionary("Pinning");
var maxSecondaryTileCount = config ? config.getInt32("maxSecondaryTileCount") : 100;
var cleanedTileId = NewsJS.Tile.newsSecondaryTile.cleanIdName(tileId); //clean the ID name
if (pinButton && pinButton.element) {
pinButton.extraClass = "appexSymbol";
// find out if this tile already exists
NewsJS.Utilities.Pinning.findSecondaryTilesAsync(cleanedTileId).then(
function (setupResult) {
if (setupResult.found) {
pinButton.label = buttonTextForRemoveTile;
pinButton.icon = unpinIcon;
}
else {
pinButton.label = buttonTextForAddTile;
pinButton.icon = pinIcon;
}
pinButton.tooltip = pinButton.label;
pinButton.onclick = function () {
stickEdgy(true);
NewsJS.Telemetry.Utilities.recordButtonClick(telemetryId, Platform.Telemetry.ActionContext.appBar);
var boundingClientRect = pinElement.getBoundingClientRect();
var tile = new NewsJS.Tile.newsSecondaryTile(
cleanedTileId,
tileShortName,
tileDisplayName,
tileLaunchTargetPage,
tileLaunchArgument,
tileLogo);
NewsJS.Utilities.Pinning.findSecondaryTilesAsync(cleanedTileId).then(
function (result) {
if (!result.found && result.totalTileCount >= maxSecondaryTileCount) {
// the 2ndary tile count has exceed the configured value
// don't pin the tile
if (tileExceedsMaxMessage) {
NewsJS.Utilities.showToast(tileExceedsMaxMessage);
}
stickEdgy(false);
}
else {
if (result.found) {
tile.requestDeleteAsync(boundingClientRect.left, boundingClientRect.top + NewsJS.Utilities.Pinning.yAxisSpacer).then(function (isDeleted) {
if (isDeleted) {
NewsJS.Telemetry.Utilities.recordPinnedTileRemoved(tileShortName, tileDisplayName, tileLaunchTargetPage, tileLaunchArgument);
// successfully removed the pin
pinButton.label = buttonTextForAddTile;
pinButton.tooltip = pinButton.label;
pinButton.icon = pinIcon;
if (tileRemoveSuccessToastMessage) {
NewsJS.Utilities.showToast(tileRemoveSuccessToastMessage);
}
}
else {
if (tileNotRemovedToastMessage) {
NewsJS.Utilities.showToast(tileNotRemovedToastMessage);
}
}
stickEdgy(false);
},
function (error) {
stickEdgy(false);
});
}
else {
tile.requestCreateAsync(boundingClientRect.left, boundingClientRect.top + NewsJS.Utilities.Pinning.yAxisSpacer, tilePinLogos, liveTileUrls, liveTileRecurrence).then(function (isAdded) {
if (isAdded) {
NewsJS.Telemetry.Utilities.recordPinnedTileAdded(tileShortName, tileDisplayName, tileLaunchTargetPage, tileLaunchArgument);
// successfully added the pin
pinButton.label = buttonTextForRemoveTile;
pinButton.tooltip = pinButton.label;
pinButton.icon = unpinIcon;
if (tileAddSuccessToastMessage) {
NewsJS.Utilities.showToast(tileAddSuccessToastMessage);
}
if (onPinSection) {
onPinSection();
}
}
else {
if (tileNotAddedToastMessage) {
NewsJS.Utilities.showToast(tileNotAddedToastMessage);
}
}
stickEdgy(false);
},
function (error) {
stickEdgy(false);
});
}
}
},
function (error) {
// find tile failed during onClick
stickEdgy(false);
});
};
},
function (error) {
// find tile failed during button setup - no action
});
}
}
function stickEdgy(sticky) {
var actionEdgyCtl = PlatformJS.Utilities.getControl("actionEdgy");
if (actionEdgyCtl) {
actionEdgyCtl.sticky = sticky; // Allow/prevent edgy from disappearing automatically
if (!sticky) {
actionEdgyCtl.hide();
}
}
}
// search through the app's secondary tiles and see if this tileId exists, return the 2ndary count tile as well
function findSecondaryTilesAsync (myTileId) {
return new WinJS.Promise(function (complete, error, progress) {
Windows.UI.StartScreen.SecondaryTile.findAllAsync().then(
function (tiles) {
var found = false;
tiles.forEach(function (tile) {
if (myTileId === tile.tileId) {
found = true;
}
});
complete({ found: found, totalTileCount: tiles.length });
},
function (e) {
error(e);
});
});
}
WinJS.Namespace.define("NewsJS.Utilities.Pinning", {
setupPinCategoryButton: setupPinCategoryButton,
setupPinTopicButton: setupPinTopicButton,
setupPinSourceButton: setupPinSourceButton,
_setupPinButton: _setupPinButton,
findSecondaryTilesAsync: findSecondaryTilesAsync,
yAxisSpacer: -5, // Add padding for the position of the add/remove pin box
});
})();
