﻿(function () {
"use strict";
function syndicationFeedAsNewsSource(url, feed) {
/// <signature helpKeyword="NewsJS.Utilities.RSS.syndicationFeedAsNewsSource">
/// <summary locid="NewsJS.Utilities.RSS.syndicationFeedAsNewsSource">
/// Converts a synidcationFeed to a news source
/// </summary>
/// <param name="syndicationFeed" type="Object" locid="NewsJS.Utilities.RSS.syndicationFeedAsNewsSourcec_p:syndicationFeed">
/// The syndication feed to be converted
/// </param>
/// <returns type="Object" locid="NewsJS.Utilities.RSS.syndicationFeedAsNewsSource_returnValue">
/// News source data object
/// </returns>
/// </signature>     
return {
categoryKey: "sourcesContent",
data: {
ad_unit_id: "",
alpha: feed && feed.title ? feed.title.text : "",
channel_id: "",
displayname: feed && feed.title ? feed.title.text : "",
featured_category: "",
featured_url: "",
freq: null,
id: url,
instrumentation_id: "",
newscategory: "RSS",
partner_header_logo: "",
partner_header_logo_snap: "",
rss_names: feed && feed.title ? feed.title.text : "",
rss_urls: url,
sourcedescription: "",
tileType: "Customize",
title: feed && feed.title ? feed.title.text : "",
win8_image: "/images/rss_icon_1.png"
},
moduleInfo: { renderer: null },
template: "sourceItemTemplate"
};
}
function uriAsSyndicationFeedAsync(uri) {
/// <signature helpKeyword="NewsJS.Utilities.RSS.uriAsSyndicationFeedAsync">
/// <summary locid="NewsJS.Utilities.RSS.uriAsSyndicationFeedAsync">
/// Attempts to create a Windows.Web.Syndication.SyndicationFeed from the given uri.
/// </summary>
/// <param name="uri" type="Object" locid="NewsJS.Utilities.RSS.uriAsSyndicationFeedAsync_p:uri">
/// The uri to be validated
/// </param>
/// <returns type="Object" locid="NewsJS.Utilities.RSS.uriAsSyndicationFeedAsync_returnValue">
/// A WinJS.Promise that contains a Windows.Web.Syndication.SyndicationFeed
/// </returns>
/// </signature>
return new WinJS.Promise(function (completeCallback, errorCallback) {
var url = (uri) ? uri.absoluteUri : null;
if (url) {
var qs = new Platform.QueryService("RSSFeed");
var options = new Platform.DataServices.QueryServiceOptions();
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("url", url);
qs.downloadDataAsync(urlParams, null, null, options).then(
function xhrComplete(result) {
var syndicationFeed = new Windows.Web.Syndication.SyndicationFeed();
try {
syndicationFeed.load(result.dataString);
} catch (e) {
errorCallback(e.message);
}
completeCallback(syndicationFeed);
},
function xhrError(error) {
errorCallback(error.message || error.statusText || "XHR ERROR");
}
);
}
else {
errorCallback("malformed uri");
}
});
}
function urlAsSyndicationFeedAsync(url) {
/// <signature helpKeyword="NewsJS.Utilities.RSS.urlAsSyndicationFeedAsync">
/// <summary locid="NewsJS.Utilities.RSS.uriAsSyndicationFeedAsync">
/// Attempts to create a Windows.Web.Syndication.SyndicationFeed from the given url.
/// </summary>
/// <param name="url" type="Object" locid="NewsJS.Utilities.RSS.urlAsSyndicationFeedAsync_p:uri">
/// The url to be validated
/// </param>
/// <returns type="Object" locid="NewsJS.Utilities.RSS.urlAsSyndicationFeedAsync_returnValue">
/// A WinJS.Promise that contains a Windows.Web.Syndication.SyndicationFeed
/// </returns>
/// </signature>
return new WinJS.Promise(function (completeCallback, errorCallback) {
var uri = NewsJS.Utilities.stringAsUri(url);
if (uriLooksValid(uri)) {
uriAsSyndicationFeedAsync(uri).done(
function uriAsSyndicationFeedAsyncComplete(syndicationFeed) {
completeCallback(syndicationFeed);
},
function uriAsSyndicationFeedAsyncError(error) {
errorCallback(error);
}
);
}
else {
errorCallback("Invalid URI");
}
});
}
function uriLooksValid(uri) {
/// <signature helpKeyword="NewsJS.Utilities.RSS.uriLooksValid">
/// <summary locid="NewsJS.Utilities.RSS.uriLooksValid">
/// Returns true if the given uri looks like a potenital rss feed
/// </summary>
/// <param name="uri" type="Object" locid="NewsJS.Utilities.RSS.uriLooksValid_p:uri">
/// The uri to be validated
/// </param>
/// <returns type="Object" locid="NewsJS.Utilities.RSS.uriLooksValid_returnValue">
/// True if uri looks like an RSS feed, false if not
/// </returns>
/// </signature>
return (uri && uri.domain && uri.schemeName === "http" && !uri.suspicious) ? true : false;
}
WinJS.Namespace.define("NewsJS.Utilities.RSS", {
syndicationFeedAsNewsSource: syndicationFeedAsNewsSource,
uriAsSyndicationFeedAsync: uriAsSyndicationFeedAsync,
urlAsSyndicationFeedAsync: urlAsSyndicationFeedAsync,
uriLooksValid: uriLooksValid
});
})();