﻿/* builtinPartners\helpersAds.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners.Ads", {
DisplayController: WinJS.Class.derive(PlatformJS.Ads.DisplayController, function (maxCount) {
PlatformJS.Ads.DisplayController.call(this, maxCount);
},
{
isVisible: function isVisible() {
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
return false;
}
// never visible
if (this._maxCount === 0) {
return false;
}
// always visible
if (this._maxCount === 1 || this._maxCount === -1) {
return true;
}
if (this._counter > this._maxCount) {
return false;
}
return true;
},
getMaxAdDisplayCount: function getMaxAdDisplayCount() {
var max = Number.POSITIVE_INFINITY;
if (this._maxCount !== -1) {
max = this._maxCount - this._counter;
}
return max;
}
}),
AdFormatter: WinJS.Class.define(function () {
},
{
getKeywordString: function getKeywordString(options) {
return [];
},
formatAd: function formatAd(ad, adType, options) {
var newAd = ad;
var newMetadata = ad.adMetadata;
if (options) {
if (adType === "interstitial") {
// push keywords
options.orientation = "landscape";
newMetadata.controlOptions.adOptions.landscapeOptions.otherAdOptions.adTags = this.getKeywordString(options);
options.orientation = "portrait";
newMetadata.controlOptions.adOptions.portraitOptions.otherAdOptions.adTags = this.getKeywordString(options);
newAd = {
articleId: ad.articleId,
adMetadata: JSON.stringify(newMetadata)
};
} else if (adType === "cluster") {
newAd = {
subclusterAdFrequency: ad.subclusterAdFrequency,
adsList: []
};
for (var i = 0; i < options.numberOfAds; i++) {
options.index = i;
newAd.adsList.push({
adTags: this.getKeywordString(options),
applicationId: ad.applicationId
});
}
} else {
newMetadata.controlOptions.otherAdOptions.adTags = this.getKeywordString(options);
newAd = JSON.stringify(newMetadata);
}
}
return newAd;
}
}),
AdsManager: WinJS.Class.define(function () {
this._partnerAds = {};
this._formattedAds = {};
},
{
_partnerAds: {},
_formattedAds: {},
getFormattedAds: function getFormattedAds(partner) {
var adsNode = null,
appId = null,
config = null;
if (!this._partnerAds[partner]) {
adsNode = NewsJS.Partners.Config.getConfig(partner, "Ads", {});
appId = adsNode.getString("ApplicationID");
config = adsNode.getDictionary("PartnerAdsConfig");
this._partnerAds[partner] = {
appId: appId,
interstitialConfig: config.getDictionary("Interstitial"),
endConfig: config.getDictionary("End"),
clusterConfig: config.getDictionary("Cluster")
};
}
if (!this._formattedAds[partner]) {
var configuredAds = {};
configuredAds = {
interstitial: {
adsList: [],
adDisplayController: null
},
end: {
adsList: [],
adDisplayController: null
},
cluster: {
adsList: [],
adDisplayController: null
}
};
var p = this._partnerAds[partner];
appId = this._partnerAds[partner].appId;
configuredAds.interstitial.adDisplayController = new NewsJS.Partners.Ads.DisplayController(p.interstitialConfig.getInt32("MaxCount"));
// This is the number of articles to skip before an interstitial ad is shown 
configuredAds.interstitial.articleGap = p.interstitialConfig.getInt32("ArticleGap");
configuredAds.interstitial.startAfterArticle = p.interstitialConfig.getInt32("StartAfterArticleNum");
configuredAds.end.adDisplayController = new NewsJS.Partners.Ads.DisplayController(p.endConfig.getInt32("MaxCount"));
configuredAds.cluster.adDisplayController = new NewsJS.Partners.Ads.DisplayController(1);
// Set up End ads
var ad = p.endConfig;
var adUnitId = ad.getString("AdUnitID");
var adMetadata = {
controlType: "PlatformJS.Ads.AdsWrapperControl",
controlOptions: {
otherAdOptions: {
applicationId: appId,
adUnitId: adUnitId
},
className: ad.getString("Classname"),
adUnitId: adUnitId
},
height: ad.getInt32("AdHeight"),
width: ad.getInt32("AdWidth")
};
configuredAds.end.adsList.push({ adMetadata: adMetadata });
// Set up Intersitial ads
ad = p.interstitialConfig;
var adOrientations = ad.getDictionary("AdOrientations");
var landscape = adOrientations.getDictionary("Landscape");
var portrait = adOrientations.getDictionary("Portrait");
var landscapeAdUnitId = landscape.getString("AdUnitID");
var portraitAdUnitId = portrait.getString("AdUnitID");
var landscapeOptions = {
otherAdOptions: {
applicationId: appId,
adUnitId: landscapeAdUnitId
},
className: landscape.getString("Classname"),
adUnitId: landscapeAdUnitId
};
var portraitOptions = {
otherAdOptions: {
applicationId: appId,
adUnitId: portraitAdUnitId
},
className: portrait.getString("Classname"),
adUnitId: portraitAdUnitId
};
adMetadata = {
controlType: "PlatformJS.Ads.RotatingAdsWrapperControl",
controlOptions: {
adOptions: {
landscapeOptions: landscapeOptions,
portraitOptions: portraitOptions
}
}
};
var interAd = {
adMetadata: adMetadata
};
configuredAds.interstitial.adsList.push(interAd);
// Set up cluster ads
ad = p.clusterConfig;
configuredAds.cluster.adsList.push({
subclusterAdFrequency: p.clusterConfig.getInt32("SubClusterAdFrequency", 1),
applicationId: appId
});
var sectionAdUnitIds = ad.getDictionary("SectionAdUnitIds") || {};
this.sectionAdUnitIds[partner] = {};
var sections = Object.keys(sectionAdUnitIds);
for (var i = 0; i < sections.length; i++) {
var section = sections[i];
var sectionAdUnitId = sectionAdUnitIds[section];
if (sectionAdUnitId.value) {
this.sectionAdUnitIds[partner][section] = sectionAdUnitId.value;
}
}
this._formattedAds[partner] = configuredAds;
}
return this._formattedAds[partner];
},
getSectionAdUnitId: function getSectionAdUnitId(partner, section) {
if (!this.sectionAdUnitIds[partner]) {
this.getFormattedAds(partner);
}
return this.sectionAdUnitIds[partner][section];
},
sectionAdUnitIds: {},
// returns the complete ad with metadata
getFormattedAdsMetadataForPartnerByType: function getFormattedAdsMetadataForPartnerByType(partnerName, adType, adOptions) {
var ads = this.getFormattedAds(partnerName);
var finalAds = null;
if (ads) {
finalAds = ads[adType];
var newAdList = [];
var adFormatter = NewsJS.Partners.Ads.AdsManager.adFormatters[partnerName];
if (adFormatter) {
for (var ad in finalAds.adsList) {
var p = adFormatter.formatAd(finalAds.adsList[ad], adType, adOptions);
newAdList.push(p);
}
}
finalAds.formattedAdsList = newAdList;
}
return finalAds;
},
// returns just the list of ads already counted
getCountedAds: function getCountedAds(partnerName, type, adOptions) {
var ads = this.getFormattedAdsMetadataForPartnerByType(partnerName, type, adOptions);
var adsList = [];
for (var i = 0; i < ads.formattedAdsList.length; i++) {
if (ads.adDisplayController.isVisible()) {
adsList.push(ads.formattedAdsList[i]);
// as they are fetched for display, increment the ad counter
ads.adDisplayController.increment();
}
}
return adsList;
},
getDisplayControllerForAdType: function getDisplayControllerForAdType(partnerName, adType) {
var ads = this.getFormattedAdsMetadataForPartnerByType(partnerName, adType, null);
return ads.adDisplayController;
}
},
{
adFormatters: {},
_instance: null,
instance: {
get: function get() {
if (!NewsJS.Partners.Ads.AdsManager._instance) {
NewsJS.Partners.Ads.AdsManager._instance = new NewsJS.Partners.Ads.AdsManager();
}
return NewsJS.Partners.Ads.AdsManager._instance;
}
}
}),
});
})();
/* builtinPartners\helpersArticleReader.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners", {
ArticleReaderProvider: WinJS.Class.define(function partner_articlereaderprovider_ctor(state) {
},
{
_sprinkleContent: function _sprinkleContent(article, contentBlocks, assetBlocks, state) {
var i = 0;
var blocks = article.blocks;
if (contentBlocks.length < 7 && (assetBlocks.length > 0 || (article.title && article.title.titleImage))) {
article.metadata.renderAll = true;
article.title.publisher = null;
for (i = 0; i < assetBlocks.length; i++) {
blocks.append(assetBlocks[i]);
}
for (i = 0; i < contentBlocks.length; i++) {
blocks.append(contentBlocks[i]);
}
return;
}
if (state.initialContentBlocks) {
for (i = 0; i < state.initialContentBlocks && contentBlocks.length > 0; i++) {
blocks.append(contentBlocks.shift());
}
}
var sprinkleSpacing = Math.floor(assetBlocks.length > 0 ? contentBlocks.length / (assetBlocks.length + 1) : 0),
blockCount = 0;
while (assetBlocks.length > 0 || contentBlocks.length > 0) {
if ((state.haveMultimediaSpan || blockCount > 0) && sprinkleSpacing > 0 && blockCount % sprinkleSpacing === 0 && assetBlocks.length > 0) {
blocks.append(assetBlocks.shift());
} else {
if (contentBlocks.length > 0) {
blocks.append(contentBlocks.shift());
} else if (assetBlocks.length > 0) {
blocks.append(assetBlocks.shift());
}
}
blockCount++;
}
},
_validateTitleImage: function _validateTitleImage(imageResource) {
var minAspectRatio = 1.3;
var maxAspectRatio = 1.89;
// title image exists
var width = imageResource.width;
var height = imageResource.height;
var aspectRatio = width / height;
if (aspectRatio < minAspectRatio || aspectRatio > maxAspectRatio) {
// title image is not valid, demote it and go to 1
return false;
}
else {
return true;
}
},
sprinkleAds: function sprinkleAds(feed) {
var options = {
section: this.feedIdentifierValue,
location: "article",
position: "interstitial"
};
var partnerInterstitials = NewsJS.Partners.Ads.AdsManager.instance.getFormattedAdsMetadataForPartnerByType(this.partner, "interstitial", options);
// If there are no interstitials, return the original feed.
if (!partnerInterstitials || partnerInterstitials.length === 0 || !partnerInterstitials.adDisplayController.isVisible()) {
return feed;
}
// Create a copy
var newList = [];
for (var i = 0; i < feed.length; i++) {
newList.push(feed[i]);
}
var interstitialAds = partnerInterstitials.formattedAdsList;
var controller = partnerInterstitials.adDisplayController;
// This is the number of articles to skip before an interstitial ad is shown 
var articleGap = partnerInterstitials.articleGap;
var maxInterstiatialsToShow = controller.getMaxAdDisplayCount();
var k = 0;
var adStartPos = partnerInterstitials.startAfterArticle;
var insertPos = adStartPos;
// Go through the list of ads
for (var l = adStartPos; l < newList.length; l++) {
// If we run out of ads to show, break.
if (maxInterstiatialsToShow <= 0) {
break;
}
// Create the ad
var adData = interstitialAds[k];
adData.articleId = "ad" + this.adCount;
var ad = {
type: "ad",
partner: this.partner,
url: adData.articleId,
metadata: adData,
articleId: adData.articleId
};
this.adCount++;
// Insert the ad at gaps of "articleGap"
if (insertPos < newList.length) {
newList.splice(insertPos, 0, ad);
insertPos += articleGap + 1; // add 1 because we just added a new element
}
else {
// Overflow
break;
}
k++;
maxInterstiatialsToShow--;
// Wrap the ads and repeat
if (k === interstitialAds.length) {
k = 0;
}
}
return newList;
},
createArticleAds: function createArticleAds() {
var model = AppEx.Common.ArticleReader.Model,
adArray = [],
adsToAppend = [],
k = 0;
var options = {
section: this.feedIdentifierValue,
location: "article",
position: "MiddleRight"
};
var endAds = NewsJS.Partners.Ads.AdsManager.instance.getFormattedAdsMetadataForPartnerByType(this.partner, "end", options);
if (endAds && endAds.formattedAdsList && endAds.formattedAdsList.length > 0 && endAds.adDisplayController.isVisible()) {
var endAd = new model.AdGroup();
endAd.type = "end";
adsToAppend = [];
for (k = 0; k < endAds.formattedAdsList.length; k++) {
adsToAppend.push(endAds.formattedAdsList[k]);
}
endAd.adMetadatas = [adsToAppend];
adArray.push(endAd);
}
return adArray;
},
adCount: 0,
articleManager: null,
getAdInfo: function getAdInfo(articleId) {
},
getArticleMetadataAsync: function getArticleMetadataAsync(articleId) {
var fullfeed = null;
if (this.fullfeed) {
fullfeed = this.fullfeed;
for (var i = 0; i < fullfeed.length; i++) {
if (fullfeed[i].newsClusterItem && fullfeed[i].newsClusterItem.articleId === articleId) {
return WinJS.Promise.wrap(fullfeed[i].newsClusterItem);
}
}
}
return this.articleManager.getFullFeedAsync(this.feedType, this.feedIdentifierValue).then(function (fullFeedItem) {
fullfeed = fullFeedItem.fullfeed;
for (var j = 0; j < fullfeed.length; j++) {
if (fullfeed[j].newsClusterItem && fullfeed[j].newsClusterItem.articleId === articleId) {
return WinJS.Promise.wrap(fullfeed[j].newsClusterItem);
}
}
return WinJS.Promise.wrap({});
}, function (error) {
return WinJS.Promise.wrap({});
});
},
}),
ArticleReaderPage: WinJS.Class.derive(NewsJS.NewsArticleReaderPage, function partner_articlereaderpage_ctor(state) {
NewsJS.NewsArticleReaderPage.call(this, state);
this.feedType = state.feedType;
this.feedIdentifierValue = state.feedIdentiferValue;
this.disableSaveArticle = state.disableSaveArticle;
this.originatingSection = state.originatingSection;
this.originatingFeed = state.originatingFeed;
this.authManager = CommonJS.Partners.Auth.BaseAuth.authenticators[this.partner];
this._isCurrentArticleAd = null;
var that = this;
this._dataChangeHandler = function (event) {
if (that.handleLoginStateChange) {
that.handleLoginStateChange();
}
that.checkPermissionForCurrentArticle();
};
this._articleChanged = function (event) {
var isAd = that._isCurrentArticleAd = that.incrementAdCount();
if (that.updateUX) {
that.updateUX(isAd);
}
};
if (this._articleManager.addEventListener) {
this._articleManager.addEventListener("articlechanged", this._articleChanged);
}
Windows.Storage.ApplicationData.current.addEventListener("datachanged", this._dataChangeHandler);
},
{
dispose: function dispose() {
NewsJS.NewsArticleReaderPage.prototype.dispose.call(this);
Windows.Storage.ApplicationData.current.removeEventListener("datachanged", this._dataChangeHandler);
if (this._articleManager.removeEventListener) {
this._articleManager.removeEventListener("articlechanged", this._articleChanged);
}
this._dataChangeHandler = null;
this._articleChanged = null;
},
partner: null,
_saveButton: null,
_isCurrentArticleAd: null,
isCurrentArticleLocked: {
get: function get() {
if (this._provider.isArticleLocked) {
return this._provider.isArticleLocked(this._articleManager.currentArticleId);
} else {
return false;
}
}
},
goBack: function goBack() {
if (Windows.UI.ViewManagement.ApplicationView.value !== Windows.UI.ViewManagement.ApplicationViewState.snapped &&
WinJS.Navigation.history.current.location.fragment === "/common/ArticleReader/html/ArticleReaderPage.html") {
if (WinJS.Navigation.canGoBack) {
WinJS.Navigation.back();
} else {
CommonJS.Navigation.mainNavigator.resetApp(new Error("ArticleReaderPage cant go back"));
}
}
},
incrementAdCount: function incrementAdCount() {
var that = this;
var articleId = this._articleManager.currentArticleId;
var articleData = this._provider.getAdInfo(articleId);
if (articleData && articleData.type && articleData.type === "ad") {
if (that._saveButton) {
that._saveButton.disabled = true;
}
var partnerName = articleData.partner;
var controller = NewsJS.Partners.Ads.AdsManager.instance.getDisplayControllerForAdType(partnerName, "interstitial");
controller.increment();
return true;
}
return false;
},
onLayoutChange: function onLayoutChange(event) {
NewsJS.NewsArticleReaderPage.prototype.onLayoutChange.call(this, event);
this.checkPermissionForCurrentArticle(this._isCurrentArticleAd);
},
checkPermissionForCurrentArticle: function checkPermissionForCurrentArticle(isAd) {
var that = this;
if (!isAd && this.isCurrentArticleLocked) {
if (that._saveButton) {
that._saveButton.disabled = true;
}
if (Windows.UI.ViewManagement.ApplicationView.value !== Windows.UI.ViewManagement.ApplicationViewState.snapped) {
this.authManager.getEntitlementsAsync(true, false, true).then(function (isEntitled) {
if (!isEntitled) {
that.goBack();
} else {
that.recordArticleView(that._articleManager.currentArticleId, that._articleManager.previousArticleId);
if (that._saveButton) {
that._saveButton.disabled = false;
}
}
}, function () {
that.goBack();
});
}
} else if (!isAd) {
that.recordArticleView(that._articleManager.currentArticleId, that._articleManager.previousArticleId);
if (that._saveButton) {
that._saveButton.disabled = false;
}
}
},
recordArticleView: function recordArticleView(currentArticleId, previousArticleId) {
// overridden in partners 
},
_handleShareRequestImpl: function _handleShareRequestImpl(request, shareData) {
var adInfo = this._provider.getAdInfo(this._articleManager.currentArticleId);
if (adInfo) {
var errorStringResourceId = "/partners/ShareInstr";
request.failWithDisplayText(PlatformJS.Services.resourceLoader.getString(errorStringResourceId));
return;
}
var deferral = request.getDeferral();
var that = this;
var articleId = this._articleManager.currentArticleId;
this._provider.getArticleMetadataAsync(articleId).then(function (articleData) {
that.onShareRequest(articleData);
var uriText = articleData.shareURL || articleData.articleURL || "";
request.data.setText(that.formatShareText(shareData, request.data.properties.description, uriText));
if (uriText) {
request.data.setUri(new Windows.Foundation.Uri(uriText));
}
request.data.properties.title = that.formatShareTitle(shareData);
request.data.properties.description = that.formatShareDescription(shareData);
var htmlText = that.formatShareHTML(shareData, uriText, articleData);
var htmlFormat = Windows.ApplicationModel.DataTransfer.HtmlFormatHelper.createHtmlFormat(htmlText);
request.data.setHtmlFormat(htmlFormat);
deferral.complete();
});
},
onShareRequest: function onShareRequest(articleData) {
},
formatShareTitle: function formatShareTitle(shareData) {
return shareData.articleHeader.headline;
},
getThumbnail: function getThumbnail(articleData) {
// Look for a thumbnail
if (articleData.thumbnail) {
return "<img src='" + articleData.thumbnail.url + "' align='left' style='margin-right: 10px'/>";
}
return "";
},
getSnippet: function getSnippet(articleData) {
if (articleData.snippet) {
return articleData.snippet;
} else {
return "";
}
},
formatShareHTML: function formatShareHTML(shareData, uriText, articleData) {
var link = "<a href=\"' + uriText + '\">" + uriText + "</a>";
var img = this.getThumbnail(articleData);
var snippet = this.getSnippet(articleData);
return "<h2><b>" + shareData.articleHeader.headline +
"</b></h2><p>" + shareData.articleHeader.author +
"</p><p>" + shareData.articleHeader.date +
"</p>" + img + snippet +
"</p><p>" + link + "</p>";
},
}),
ArticleManager: WinJS.Class.define(function article_mgr_ctor() {
},
{
_imageService: null,
_prefetchImage: function _prefetchImage(imageData) {
if (!this._imageService) {
this._imageService = new Platform.ImageService("PlatformImageCache");
}
var prefetchOptions = new Platform.DataServices.PrefetchQueryServiceOptions();
prefetchOptions.priority = NewsJS.Partners.Config.isPartnerApp ? Platform.DataServices.QueryServicePriority.medium : Platform.DataServices.QueryServicePriority.low;
if (imageData) {
this._imageService.prefetchImage(imageData.url, prefetchOptions);
}
},
})
});
})();
/* builtinPartners\helpersAuth.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners", {
BaseAuth: WinJS.Class.derive(CommonJS.Partners.Auth.BaseAuth, function newsjs_baseauthctrl_ctor(options) {
CommonJS.Partners.Auth.BaseAuth.call(this, options);
},
{
getRefreshInterval: function getRefreshInterval() {
return CommonJS.Partners.Config.getConfig(this._partner, "EntitlementsRefreshInterval", 6 * 60 * 60) * 1000;
},
}),
BaseNativeAuthControl: WinJS.Class.derive(CommonJS.Partners.Auth.BaseNativeAuthControl, function newsjs_basenativeauthctrol_ctor(options) {
this._partnerStringPrefix = "/" + this._partner.toLowerCase() + "/";
CommonJS.Partners.Auth.BaseNativeAuthControl.call(this, options);
},
{
getString: function getString(stringId) {
var string = PlatformJS.Services.resourceLoader.getString(this._partnerStringPrefix + stringId);
if (!string) {
string = CommonJS.Partners.Auth.BaseNativeAuthControl.prototype.getString.call(this, stringId);
}
return string;
}
}),
BaseWebAuthControl: WinJS.Class.derive(CommonJS.Partners.Auth.BaseWebAuthControl, function newsjs_basewebauthctrl_ctor(options) {
CommonJS.Partners.Auth.BaseWebAuthControl.call(this, options);
},
{
})
});
})();
/* builtinPartners\helpersBaseProcessListener.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners", {
BaseProcessListener: WinJS.Class.define(function Partner_BaseProcessListener_ctor() {
this._pageMapping = {};
this._nextShareIsAppShare = false;
this._postActivated = false;
this._delayTopEdgy = PlatformJS.Services.configuration.getBool("DelayTopEdgyPopulation", true);
},
{
_partner: null,
_nextShareIsAppShare: false,
_pageMapping: {},
_postActivated: false,
populateEdgy: function populateEdgy() {
},
startPostActivateWork: function startPostActivateWork() {
if (!this._postActivated) {
this._postActivated = true;
if (NewsJS.Partners.Config.isPartnerApp) {
this._setupLiveTile();
}
if (NewsJS.Partners.Config.isPartnerApp ||
Platform.Utilities.Globalization.isFeatureEnabled("featuredChannel" + this._partner)) {
// Defer populating top edgy for performance reasons if possible.
if (NewsJS.Partners.Config.isPartnerApp && this._delayTopEdgy) {
this.populateEdgy();
}
}
}
},
onSuspending: function onSuspending(event) {
/// <summary>
/// Handles Suspend event
/// </summary>
/// <param name="event">
/// Suspend event
/// </param>
/// <returns>
/// Nothing
/// </returns>
if (this._articleManager && this._articleManager.serializeAsync) {
this._articleManager.serializeAsync();
}
},
onResuming: function onResuming(event) {
/// <summary>
/// Handles Resume event
/// </summary>
/// <param name="event">
/// resume event
/// </param>
/// <returns>
/// Nothing
/// </returns>
},
onCheckpoint: function onCheckpoint(event) {
/// <summary>
/// Handles Checkpoint event
/// </summary>
/// <param name="event">
/// Suspend suspend event
/// </param>
/// <returns>
/// Nothing
/// </returns>
if (this._articleManager && this._articleManager.serializeAsync) {
this._articleManager.serializeAsync();
}
},
onActivated: function onActivated(event, isRecoveryMode) {
/// <summary>
/// Handles activation event
/// </summary>
/// <param name="event">
/// activation event
/// </param>
/// <returns>
/// Nothing
/// </returns>
var promise = WinJS.Promise.wrap(),
that = this;
if (isRecoveryMode && this._articleManager && this._articleManager.resetAsync) {
promise = this._articleManager.resetAsync();
}
if (NewsJS.Partners.Config.isPartnerApp) {
return promise.then(function () {
return that._onActivatedStandalone(event, isRecoveryMode);
});
} else {
return null;
}
},
onVisibilityChange: function onVisibilityChange(msHidden) {
/// <summary>
/// Handles visibility change event
/// </summary>
/// <param name="event">
/// Visibility change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
//if (document.msHidden) {
//    //localyticsSession.open();
//    //localyticsSession.upload();
//}
//else {
//    //localyticsSession.close();
//    //localyticsSession.upload();
//}
},
onLayoutChange: function onLayoutChange(event) {
if (event.detail[0].viewState === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
if (NewsJS.Partners.Config.isPartnerPage(this._partner)) {
var auth = CommonJS.Partners.Auth.BaseAuth.authenticators[this._partner];
if (auth) {
auth.closeExistingPromptControl();
}
}
}
},
onShareRequest: function onShareRequest() {
// Implemented in derived classes.
},
_setupLiveTile: function _setupLiveTile() {
var urls = [];
var market = Platform.Globalization.Marketization.getCurrentMarket();
for (var i = 0; i < 5; i++) {
var polledUrl = NewsJS.Partners.Config.getConfig(this._partner, "liveTileUrls" + i, "");
polledUrl = polledUrl.replace("{market}", market);
if (polledUrl) {
try {
var url = new Windows.Foundation.Uri(polledUrl);
urls.push(url);
} catch (e) { }
}
}
if (urls.length > 0) {
var notifications = Windows.UI.Notifications;
var periodicRecurrence = notifications.PeriodicUpdateRecurrence;
var recurrence = parseInt(NewsJS.Partners.Config.getConfig(this._partner, "LiveTileUpdateFrequency", notifications.PeriodicUpdateRecurrence.hour));
// Check recurrence values
if (recurrence === periodicRecurrence.daily ||
recurrence === periodicRecurrence.halfHour ||
recurrence === periodicRecurrence.hour ||
recurrence === periodicRecurrence.sixHours ||
recurrence === periodicRecurrence.twelveHours) {
var updater = notifications.TileUpdateManager.createTileUpdaterForApplication();
updater.enableNotificationQueue(true);
updater.startPeriodicUpdateBatch(urls, recurrence);
}
}
},
_addSettingsPageFragment: function _addSettingsPageFragment(divId, fragment, dataContext) {
var that = this;
var aboutElement = document.getElementById(divId);
if (!aboutElement) {
aboutElement = document.createElement("div");
aboutElement.id = divId;
document.body.appendChild(aboutElement);
}
return WinJS.UI.Fragments.renderCopy(fragment, aboutElement).then(function (doc) {
return WinJS.UI.processAll(aboutElement);
}).then(function () {
return WinJS.Binding.processAll(aboutElement, dataContext);
}).done(function () {
that._onSettingsPageCreate(PlatformJS.Utilities.getControl(divId + "Ctrl"), divId);
});
},
_onSettingsPageCreate: function _onSettingsPageCreate(flyout, flyoutId) {
/// <summary>Set up flyout after creating it </summary>
/// <param name="flyout">Flyout object</param>
/// <param name="flyoutId">Id of flyout div</param>
var that = this;
if (flyout) {
flyout.onafterhide = function () {
that._removeSettingsPage(flyoutId);
flyout.onafterhide = null;
};
flyout.show();
}
},
_removeSettingsPage: function _removeSettingsPage(pageId) {
/// <summary>Remove settings page</summary>
var page = document.getElementById(pageId);
if (page) {
document.body.removeChild(page);
}
},
_shareApp: function _shareApp(request) {
var appName = PlatformJS.Services.resourceLoader.getString("appName");
var publisherName = PlatformJS.Services.resourceLoader.getString("publisherName");
request.data.setText(appName);
var storeUrl = "ms-windows-store:PDP?PFN=" + Windows.ApplicationModel.Package.current.id.familyName;
var url = NewsJS.Partners.Config.getConfig(this._partner, "StoreURL", storeUrl);
var desc = PlatformJS.Services.resourceLoader.getString("/partners/ShareppAppDesc").format(appName, publisherName);
var htmlText = "<a href='{0}'>{1}</a>".format(url, PlatformJS.Services.resourceLoader.getString("/partners/DownloadApp")) + " " + desc;
var uri = new Windows.Foundation.Uri(url);
request.data.setHtmlFormat(Windows.ApplicationModel.DataTransfer.HtmlFormatHelper.createHtmlFormat(htmlText));
request.data.setUri(uri);
request.data.properties.title = appName;
request.data.properties.description = desc;
this.onShareRequest();
},
_setupEventHandling: function _setupEventHandling() {
var that = this;
var dataTransferManager = Windows.ApplicationModel.DataTransfer.DataTransferManager.getForCurrentView();
try {
dataTransferManager.addEventListener("datarequested", function (e) {
var request = e.request;
var page = WinJS.Navigation.history.current.location.page;
var currentIPage = PlatformJS.Navigation.currentIPage;
var errorStringResourceId = "/partners/ShareInstr";
if (that._nextShareIsAppShare) {
that._shareApp(request);
that._nextShareIsAppShare = false;
} else if (currentIPage && currentIPage.handleShareRequest) {
if (!currentIPage.canShareRequest || currentIPage.canShareRequest()) {
currentIPage.handleShareRequest(request);
}
} else {
that._shareApp(request);
}
});
} catch (e) { }
var settingsPane = Windows.UI.ApplicationSettings.SettingsPane.getForCurrentView();
settingsPane.oncommandsrequested = function (e) {
var appSettings = Windows.UI.ApplicationSettings;
var vector = e.request.applicationCommands;
var onAboutCmd = function (ev) {
that._addSettingsPageFragment("AboutPageDiv", "/html/aboutSettings.html", that.getAboutPageData());
};
var onSettingsCmd = function () {
that._addSettingsPageFragment("SettingsPageDiv", "/html/settingSettings.html", that.getSettingsPageData());
};
var onFeedbackCmd = function () {
that._addSettingsPageFragment("FeedbackPageDiv", "/html/feedbackSettings.html", that.getFeedbackPageData());
};
if (PlatformJS.Services.configuration.getBool("OptionsEnabled", true)) {
var settingsCmd = new appSettings.SettingsCommand("SettingsPage", CommonJS.resourceLoader.getString("/platform/optionsTitle"), onSettingsCmd);
vector.append(settingsCmd);
}
// Add app settings command to the settings charm
var aboutCmd = new appSettings.SettingsCommand("AboutPage", PlatformJS.Services.resourceLoader.getString("/platform/aboutLabel"), onAboutCmd);
vector.append(aboutCmd);
var feedbackCmd = new appSettings.SettingsCommand("FeedbackPage", CommonJS.resourceLoader.getString("/partners/Support"), onFeedbackCmd);
vector.append(feedbackCmd);
// Add TOU cmd
var tocCmd = new appSettings.SettingsCommand("tocPage", CommonJS.resourceLoader.getString("/platform/lca_serviceAgreement"), CommonJS.Settings.onTocCmd);
vector.append(tocCmd);
// Add privacy settings command to the settings charm
var privacyCmd = new appSettings.SettingsCommand("privacyPage", CommonJS.resourceLoader.getString("/platform/lca_privacy"), CommonJS.Settings.onPrivacyCmd);
vector.append(privacyCmd);
var shareCmd = new appSettings.SettingsCommand("shareApp",
PlatformJS.Services.resourceLoader.getString("/partners/shareApp").format(PlatformJS.Services.resourceLoader.getString("appName")), function () {
that._nextShareIsAppShare = true;
Windows.ApplicationModel.DataTransfer.DataTransferManager.showShareUI();
});
vector.append(shareCmd);
if (PlatformJS.isDebug) {
var debugCmd = new appSettings.SettingsCommand("debugPage", "Debug", function () {
that._addSettingsPageFragment("DebugPageDiv", "/html/debugSettings.html", {
output: CommonJS.Partners.Debug.instance.output
});
});
vector.append(debugCmd);
}
};
},
_onActivatedStandalone: function _onActivatedStandalone(event, isRecoveryMode) {
var that = this;
var edgyPromise = null;
if (!this._delayTopEdgy) {
edgyPromise = this.populateEdgy();
}
var navigation = null; // Default navigate to home
// Setup the navigation
if (event.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.launch) {
// If this app was launched from a custom tile, lets navigate to that location within the app
if (event && event.detail && event.detail.arguments) {
var tileData = JSON.parse(event.detail.arguments);
navigation = this.getPinTileMapping(tileData);
}
} else if (event.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.protocol) {
navigation = this.handleProtocolLaunch(event);
}
if (!that._hasInitialized) {
this._setupEventHandling();
that._hasInitialized = true;
}
if (navigation) {
return navigation;
} else if (edgyPromise) {
return edgyPromise.then(function () {
return null;
}, function () {
return null;
});
} else {
return null;
}
},
handleProtocolLaunch: function handleProtocolLaunch(event) {
return null;
},
getPinTileMapping: function getPinTileMapping(tileData) {
var navigation = null;
if (typeof (tileData) !== "undefined" && typeof (tileData.target) !== "undefined" && typeof (tileData.data) !== "undefined") {
var fragment = this._pageMapping[tileData.target];
if (fragment) {
navigation = WinJS.Promise.wrap({ pageInfo: { fragment: fragment, page: tileData.target, channelId: tileData.data.channelID }, state: tileData.data });
}
}
return navigation;
},
getAboutPageData: function getAboutPageData() {
var version = Windows.ApplicationModel.Package.current.id.version;
var aboutLabelText = PlatformJS.Services.resourceLoader.getString("/platform/aboutLabel");
// App Name and PublisherDisplayName should be retrieved from Platform Configuration Manifest, but they're not populated
var appName = PlatformJS.Services.resourceLoader.getString("appName");
var publisherName = PlatformJS.Services.resourceLoader.getString("publisherName");
var buildVersion = version.major + "." + version.minor + "." + version.build + "." + version.revision;
var dataContext = {
versionText: PlatformJS.Services.resourceLoader.getString("/platform/appVersion").format(buildVersion),
aboutLabelText: aboutLabelText,
appName: appName,
publisherName: publisherName
};
return dataContext;
},
getFeedbackPageData: function getFeedbackPageData() {
var emailLabel = PlatformJS.Services.resourceLoader.getString("/partners/emailLabel"),
phoneLabel = PlatformJS.Services.resourceLoader.getString("/partners/phoneLabel"),
email = NewsJS.Partners.Config.getConfig(this._partner, "Email"),
phone = NewsJS.Partners.Config.getConfig(this._partner, "Phone"),
feedbackLabelText = PlatformJS.Services.resourceLoader.getString("/partners/Support"),
dataContext = {
feedbackLabelText: feedbackLabelText,
emailLabel: emailLabel,
mailToLink: "mailto:" + email,
email: email,
phoneLabel: phoneLabel,
phone: phone
};
return dataContext;
}
},
{
saveChannelData: function saveChannelData() {
PlatformJS.Navigation.mainNavigator.channelManager.setChannelConfigData();
},
getChannelConfigData: function getChannelConfigData(channelID) {
var channelManager = PlatformJS.Navigation.mainNavigator.channelManager;
var channelConfigData = null,
j = 0;
var channelSet = NewsJS.Partners.Config.isPartnerApp ? channelManager.standardChannels : channelManager.featuredChannels;
for (j = 0; j < channelSet.length; j++) {
if (channelSet[j].id === channelID) {
channelConfigData = channelSet[j];
break;
}
}
return channelConfigData;
},
getChannelArray: function getChannelArray(partner) {
var channelManager = PlatformJS.Navigation.mainNavigator.channelManager;
var channelArray = null,
j = 0;
if (NewsJS.Partners.Config.isPartnerApp) {
channelArray = channelManager.standardChannels;
} else {
var allFeaturedChannels = channelManager.featuredChannels;
for (j = 0; j < allFeaturedChannels.length; j++) {
if (allFeaturedChannels[j].id === partner) {
channelArray = allFeaturedChannels[j].subChannels;
break;
}
}
}
return channelArray;
},
})
});
})();
/* builtinPartners\helpersConfig.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners.Config", {
_availableMarkets: null,
isInAvailableMarket: {
get: function get() {
if (NewsJS.Partners.Config._availableMarkets === null) {
NewsJS.Partners.Config._availableMarkets = PlatformJS.Services.configuration.getString("AvailableMarkets").toLowerCase();
}
var market = Platform.Utilities.Globalization.getMarketString().toLowerCase();
return NewsJS.Partners.Config._availableMarkets === "all" || NewsJS.Partners.Config._availableMarkets.indexOf(market) >= 0;
}
},
_isPartnerApp: null,
isPartnerApp: {
get: function get() {
if (NewsJS.Partners.Config._isPartnerApp === null) {
var packageName = Windows.ApplicationModel.Package.current.id.name;
NewsJS.Partners.Config._isPartnerApp = packageName.indexOf("Microsoft.") === -1;
}
return NewsJS.Partners.Config._isPartnerApp;
}
},
isPartnerPage: function isPartnerPage(partner) {
return NewsJS.Partners.Theme.currentPartner === partner;
},
_sectionImages: {},
getSectionImage: function getSectionImage(partner, sectionName) {
var defaultLookUp = "default";
var lookUpValue = sectionName ? sectionName.toLowerCase() : defaultLookUp;  // If no section is passed, use the default loop up value
var dictionary = this._sectionImages[partner];
if (!dictionary) {
this._sectionImages[partner] = dictionary = NewsJS.Partners.Config.getConfig(partner, "SectionImages", {});
}
var image = dictionary.getString(lookUpValue);
if (!image) {
image = dictionary.getString(defaultLookUp);
}
return image; //"url('/" + partner.toLowerCase() + "shared/images/" + image + "')";
},
getConfig: CommonJS.Partners.Config.getConfig,
setConfig: CommonJS.Partners.Config.setConfig
});
})();
/* builtinPartners\helpersNewsClusterProvider.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners", {
NewsClusterProvider: WinJS.Class.define(function (categoryInfo, title, maxArticles, onsuccess, onerror, bypassCache) {
this.categoryInfo = categoryInfo;
this.sectionName = this.categoryInfo.sectionName;
this.articles = null;
this.maxArticles = maxArticles;
this.title = title;
this.onsuccess = onsuccess;
this.onerror = onerror;
this.bypassCache = bypassCache;
this._innerPromise = null;
},
{
_articleManager: null,
categoryInfo: null,
bypassCache: null,
articles: null,
maxArticles: null,
title: null,
dispose: function dispose() {
this.categoryInfo = null;
this.articles = null;
this.onsuccess = null;
this.onerror = null;
if (this._innerPromise) {
this._innerPromise.cancel();
this._innerPromise = null;
}
},
getSnappedTitleTile: function getSnappedTitleTile() {
var binding = {
title: this.title,
icon: "snappedTitleTileIcon " + NewsJS.Partners.Config.getSectionImage(this._partner, this.sectionName),
moduleInfo: {
fragmentPath: "/html/templates.html",
templateId: "snappedTitleTileWithIcon"
}
};
return binding;
},
initializeAsync: function initializeAsync() {
/// <summary>Initializer</summary>
/// <returns>Completion promise</returns>
return WinJS.Promise.wrap({});
},
selectTemplateClass: function selectTemplateClass(index, item) {
return item.newsClusterItem;
},
getFeedIdentifierValue: function getFeedIdentifierValue() {
return this.categoryInfo[this.categoryInfo.feedType + "Name"];
},
_innerPromise: null,
processFeed: function processFeed(feed) {
var that = this;
if (that.onsuccess) {
that.onsuccess();
}
that.articles = [];
that.categoryInfo = feed.categoryInfo;
var fullfeed = feed.fullfeed;
var max = that.maxArticles !== undefined && that.maxArticles !== null ? Math.min(that.maxArticles, fullfeed.length) : fullfeed.length;
for (var i = 0; i < max; i++) {
that.articles.push(that.selectTemplateClass(i, fullfeed[i]));
}
var dataResults = {
native: true,
newsItems: that.articles,
isCachedResponse: feed.isCachedResponse
};
if (that.layoutCacheInfo) {
dataResults.clusterID = that.title + "_" + that.isPrimaryCluster;
dataResults.dataCacheID = that.layoutCacheInfo.cacheKey;
dataResults.cacheID = that.layoutCacheInfo.cacheID;
dataResults.uniqueResponseID = that.layoutCacheInfo.uniqueResponseID;
}
return dataResults;
},
handleProgress: function handleProgress(feed) {
if (this.progressHandler) {
this.progressHandler(this.processFeed(feed));
}
},
queryAsync: function queryAsync() {
/// <summary>Query function</summary>
/// <returns>Promise containing newsItems</returns>
if (!this.categoryInfo) {
return WinJS.Promise.wrapError("NotFound");
}
var that = this;
return new WinJS.Promise(function (completeHandler, errorHandler, progressHandler) {
that.progressHandler = progressHandler;
// Get the full feed and get the newsClusterItems
that._innerPromise = that._articleManager.getFullFeedAsync(that.categoryInfo.feedType, that.getFeedIdentifierValue(), that._categoryInfo, that.bypassCache, that).then(function (feed) {
return WinJS.Promise.wrap(that.processFeed(feed));
}, function (error) {
if (that.onerror) {
that.onerror();
}
return WinJS.Promise.wrapError(error);
});
that._innerPromise.then(function (completeData) {
completeHandler(completeData);
}, function (errorData) {
errorHandler(errorData);
});
}, function () {
that._innerPromise.cancel();
that._innerPromise = null;
});
}
})
});
})();
/* builtinPartners\helpersSlideShowPage.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners", {
SlideShowPage: WinJS.Class.derive(NewsJS.SlideShowPage, function partner_slideshowpage_ctor(state) {
NewsJS.SlideShowPage.call(this, state);
this.access = state.access;
var that = this;
this._dataChangeHandler = function (event) {
that.checkPermissionForCurrentArticle();
};
this.authManager = CommonJS.Partners.Auth.BaseAuth.authenticators[this.partner];
Windows.Storage.ApplicationData.current.addEventListener("datachanged", this._dataChangeHandler);
},
{
dispose: function dispose() {
NewsJS.SlideShowPage.prototype.dispose.call(this);
Windows.Storage.ApplicationData.current.removeEventListener("datachanged", this._dataChangeHandler);
},
goBack: function goBack() {
if (WinJS.Navigation.canGoBack) {
WinJS.Navigation.back();
} else {
PlatformJS.Navigation.mainNavigator.resetApp(new Error("SlideShowPage cant go back"));
}
},
checkPermissionForCurrentArticle: function checkPermissionForCurrentArticle() {
var that = this;
if (this.access === "subscribed") {
this.authManager.getEntitlementsAsync(true, false, true).then(function (isEntitled) {
if (!isEntitled) {
that.goBack();
}
}, function () {
that.goBack();
});
}
}
})
});
})();
/* builtinPartners\helpersTheme.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners.Theme", {
onNavigated: function onNavigated(state) {
if (NewsJS.Partners.Config.isPartnerApp && !NewsJS.Partners.Theme.navigated && !(state && state.dynamicInfo)) {
if (PlatformJS.Services.configuration.getBool("PopulateL1EdgyOnFirstNavigate", false)) {
NewsJS.Partners.WhiteLabelProcessListener.populateL1Edgy();
}
}
NewsJS.Partners.Theme.navigated = true;
var theme = state && state.theme ? state.theme : NewsJS.Partners.Theme.defaultTheme;
CommonJS.setTheme(theme);
if (theme) {
NewsJS.Partners.Theme.currentPartner = theme.split(" ")[0];
}
NewsJS.Telemetry.Utilities.closePartnerSession();
},
onNavigateAway: function onNavigateAway() {
CommonJS.setTheme(NewsJS.Partners.Theme.defaultTheme);
NewsJS.Partners.Theme.currentPartner = null;
NewsJS.Utilities.cancelPromise(NewsJS.Partners.Theme.edgyL1Promise);
},
navigated: false,
currentPartner: null,
defaultTheme: "bing",
edgyL1Promise: null
});
})();
/* builtinPartners\helpersWLProcessListener.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Partners", {
WhiteLabelProcessListener: WinJS.Class.derive(NewsJS.Partners.BaseProcessListener, function () {
this._partner = "WhiteLabel";
NewsJS.Partners.BaseProcessListener.call(this);
},
{
populateEdgy: function populateEdgy() {
if (!NewsJS.Partners.WhiteLabelProcessListener.edgyPopulated) {
NewsJS.Partners.WhiteLabelProcessListener.enableTopEdgy(true);
return NewsJS.Partners.WhiteLabelProcessListener.populateEdgy();
} else {
return WinJS.Promise.wrap();
}
},
getPinTileMapping: function getPinTileMapping(tileData) {
switch (tileData.target) {
case "DynamicPano.DynamicCategoryPano"://stevenfe: backcompat
case "DynamicPano.DynamicPanoPage": 
NewsJS.Telemetry.Utilities.recordPinnedTileLaunch("Partner Panorama");
var dynamicInfo = tileData.data;
dynamicInfo.entrypoint = "secondarytile";
return WinJS.Promise.wrap({ state: { dynamicInfo: dynamicInfo }, pageInfo: { channelId: dynamicInfo.id, fragment: "/dynamicPano/dynamicPano.html", page: tileData.target } });
default:
return null;
}
},
handleProtocolLaunch: function handleProtocolLaunch(event) {
var arg = event.detail.uri;
var result = NewsJS.Utilities.parseUri(arg);
if (result) {
var urlParams = result.params;
if (urlParams && result.path === "application/view") {
return NewsJS.Utilities.mapUriToNavigation(result);
}
}
return null;
}
},
{
populateEdgyPromise: null,
state: null,
populateEdgy: function populateEdgy() {
var market = NewsJS.Globalization.getMarketStringForEditorial();
if (!market || !NewsJS.Partners.Config.isInAvailableMarket) {
NewsJS.Partners.WhiteLabelProcessListener.populateEdgyPromise = WinJS.Promise.wrapError("InvalidMarket");
} else {
NewsJS.Partners.WhiteLabelProcessListener.populateEdgyPromise = NewsJS.Utilities.fetchFeaturedSourcesData(market).then(function (sources) {
var appSourceId = PlatformJS.Services.appConfig.getDictionary("PartnerAppInfo").getString("PartnerSourceId");
var source = sources[appSourceId];
if (source) {
var state = NewsJS.Partners.WhiteLabelProcessListener.state = NewsJS.Utilities.buildStateForPartnerSource(source);
state.dynamicInfo.isPartnerApp = true;
var home = NewsJS.Partners.WhiteLabelProcessListener._createHomeChannel(state);
state.dynamicInfo.clusterTitle = home.title;
var channelArray = NewsJS.Partners.BaseProcessListener.getChannelArray();
channelArray.splice(0, channelArray.length);
channelArray.push(home);
NewsJS.Partners.BaseProcessListener.saveChannelData();
return WinJS.Promise.wrap(home.state);
} else {
return WinJS.Promise.wrapError("FeatureSourcesDataFailure");
}
});
}
return NewsJS.Partners.WhiteLabelProcessListener.populateEdgyPromise;
},
populateL1Edgy: function populateL1Edgy() {
var promise = NewsJS.Partners.WhiteLabelProcessListener.populateEdgyPromise;
if (promise) {
promise.then(function () {
var partnerState = NewsJS.Partners.WhiteLabelProcessListener.state;
if (partnerState && partnerState.dynamicInfo) {
var options = {
dataSourceName: partnerState.dynamicInfo.dataSourceName,
feedName: partnerState.dynamicInfo.feedName,
dataProviderOptions: partnerState.dynamicInfo.dataProviderOptions,
cssFeedUrl: partnerState.dynamicInfo.css,
channelId: partnerState.channelId
};
var dataProvider = new DynamicPanoJS.DynamicPanoProvider(options);
NewsJS.Partners.Theme.edgyL1Promise = dataProvider.fetchCacheData().then(function (complete) {
NewsJS.Partners.Theme.edgyL1Promise = null;
if (complete && complete.cmsData) {
var categoryOrder = complete.cmsData.categoryOrder;
var categoryList = [];
var changed = false;
if (categoryOrder && categoryOrder.length > 1) {
var channelArray = NewsJS.Partners.BaseProcessListener.getChannelArray();
if (channelArray) {
if (channelArray.length > 1) {
// clear out all channels after Home
channelArray.splice(1, channelArray.length - 1);
}
var channelConfigData = NewsJS.Partners.BaseProcessListener.getChannelConfigData("Home");
if (channelConfigData) {
changed = true;
for (var idx = 0; idx < categoryOrder.length; idx++) {
var item = { clusterKey: categoryOrder[idx], clusterTitle: complete.cmsData.categories[categoryOrder[idx]].categoryName };
if (item && item.clusterTitle && item.clusterKey) {
var title = item.clusterTitle;
var key = item.clusterKey;
var id = channelConfigData.id + "_" + title;
var state = JSON.parse(JSON.stringify(channelConfigData.state));
state.dynamicInfo.clusterID = key;
state.dynamicInfo.clusterToRender = key;
var newChannel = {
id: id,
title: title,
icon: "",
subChannels: [],
pageInfo: {
fragment: channelConfigData.pageInfo.fragment,
page: channelConfigData.pageInfo.page,
channelId: id,
},
state: state,
isDisplayValue: true
};
channelArray.push(newChannel);
}
}
}
}
}
if (changed) {
NewsJS.Partners.WhiteLabelProcessListener.edgyPopulated = true;
NewsJS.Partners.BaseProcessListener.saveChannelData();
NewsJS.Partners.WhiteLabelProcessListener.enableTopEdgy(false);
}
}
}, function (error) {
NewsJS.Partners.Theme.edgyL1Promise = null;
});
}
}, function () { });
}
},
edgyPopulated: false,
enableTopEdgy: function enableTopEdgy(value) {
var platformNavigationBar = PlatformJS.Utilities.getControl("platformNavigationBar");
if (platformNavigationBar) {
platformNavigationBar.disabled = value;
}
},
_createHomeChannel: function _createHomeChannel(state) {
NewsJS.Partners.WhiteLabelProcessListener.defaultAppState = state;
return {
icon: "",
id: "Home",
pageInfo: {
channelId: "Home",
fragment: "/dynamicPano/dynamicPano.html",
page: "DynamicPano.DynamicPanoPage",
},
state: state,
subChannels: [],
title: PlatformJS.Services.resourceLoader.getString("/partners/Home"),
visible: true,
isDisplayValue: true
};
},
})
});
})();
