﻿(function () {
"use strict";
WinJS.Namespace.define("NewsJS", {
StoryPage: WinJS.Class.derive(NewsJS.NewsBasePage,
function (state) {
/// <summary>
/// Constructor for a simple IPage class that controls the page
/// </summary>
/// <param name="state">
/// The state object passed to WinJS.Navigation.navigate or the state object
/// preserved when the page has been navigated away from
/// </param>
PlatformJS.Navigation.mainNavigator.disableSearchOnType();
NewsJS.NewsBasePage.call(this, state);
this._useSnappedContainer = true;
this._state = state ? state : {};
var that = this;
var storyRefreshButton = PlatformJS.Utilities.getControl("storyRefreshButton");
storyRefreshButton.label = PlatformJS.Services.resourceLoader.getString("Refresh");
storyRefreshButton.onclick = function () { that._refreshPage(true); };
// Quickly show and hide the progress indicator to get the secondary splash screen to go away when resuming to the story page
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
this.loadPageData();
},
{
countWeight: 10,
_storyFrame: null,
_parentContainer: null,
_onLoadHandler: null,
loadPageData: function loadPageData() {
var that = this;
var openInBrowserButton = PlatformJS.Utilities.getControl("openInBrowserButton");
openInBrowserButton.label = PlatformJS.Services.resourceLoader.getString("OpenInBrowser");
if (this._state.article && this._state.article.articleUrl) {
if (this._state.listInfo && this._state.listInfo.articles) {
if (typeof this._state.listInfo.currentArticleIndex === "undefined") {
// Try to find article index if not provided
this._state.listInfo.currentArticleIndex = this._state.listInfo.articles.indexOf(this._state.article);
}
}
var url = this._state.article.articleUrl;
if (PlatformJS.Utilities.hasInternetConnection()) {
this._loadiframe(url);
}
else {
this._loadiframe("");
this._pageLoadFailed();
}
openInBrowserButton.onclick = function () { that._openInBrowser(url); };
}
},
canShareRequest: function canShareRequest() {
return this._state.article && this._state.article.articleUrl;
},
handleShareRequest: function handleShareRequest(request) {
request.data.properties.title = this._state.article.title;
request.data.properties.description = this._state.article.articleUrl;
request.data.setUri(new Windows.Foundation.Uri(this._state.article.articleUrl));
NewsJS.Telemetry.Utilities.recordShare(
this._state.article.articleUrl, 
this._state.article.telemetry ? this._state.article.telemetry.k : 0, 
"IFrame Article");
},
getPageImpressionContext: function getPageImpressionContext() {
return NewsJS.Telemetry.String.ImpressionContext.iframeArticle;
},
getPageState: function getPageState() {
return this._state;
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away and when the app is suspending.
/// Return any data that you need to resurrect your UX when the page is recreated
/// Keep this state object as small as possible
/// </summary>
/// <returns>
/// This function must return a JSON object that can be serializable. Do not pass a Managed object
/// </returns>
},
getPageData: function getPageData() {
/// <summary>
/// This function is invoked by the UX Framework when the page is being created and when
/// it is necessary to bind data to the UX.
/// </summary>
/// <returns>
/// Returns a promise that contains data to bind the UX to
/// </returns>
var that = this;
var k = NewsJS.Telemetry.StoryPage.recordContent(this._state.article);
NewsJS.Telemetry.StoryPage.recordContentView(k);
return WinJS.Promise.wrap({
title: "",
subTitle: ""
});
},
onNavigateAway: function onNavigateAway(reason) {
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away
/// </summary>
/// <param name="reason">
/// The reason why the page is being navigated away: "backNavigation" or "newNavigation"
/// </param>
/// <returns>
/// Nothing
/// </returns>
if (this._storyFrame && this._onLoadHandler) {
this._storyFrame.removeEventListener("load", this._onLoadHandler);
}
// Destroy the iframe
CommonJS.Progress.resetAll();
if (this._parentContainer) {
this._parentContainer.innerHTML = "";
}
},
createSnappedContainer: function createSnappedContainer() {
var articleReaderPage = document.createElement("div");
articleReaderPage.className = "articleReaderPage";
var header = document.createElement("div");
var headerControl = PlatformJS.Utilities.createObject("CommonJS.Immersive.Header", header, {
fontColor: "headerFontLight",
headerTitleText: PlatformJS.Services.resourceLoader.getString("News")
});
var dateString = "";
if (this._state.article.publishTimeRaw) {
dateString = News.NewsUtil.instance.bdiTimeToString(this._state.article.publishTimeRaw);
}
var snappedContainer = document.createElement("div");
var snappedArticleReader = new CommonJS.ArticleReader.SnappedArticleReader(snappedContainer);
snappedArticleReader.render(
null,
{
title: {
headline: this._state.article.title || "",
byline: this._state.article.source || "",
date: dateString
}
});
articleReaderPage.appendChild(header);
articleReaderPage.appendChild(snappedContainer);
return articleReaderPage;
},
onEnterExitSnap: function onEnterExitSnap() {
},
snappedItemInvoked: function snappedItemInvoked(e) {
if (this._state.listInfo) {
var articles = this._state.listInfo.articles;
if (articles) {
var article = articles[e.detail.itemIndex];
if (article) {
NewsJS.Telemetry.StoryPage.recordSnappedItemPress(article);
NewsJS.Utilities.launchArticle(
article,
{
title: this._state.listInfo.title,
articles: this._state.listInfo.articles,
currentArticleIndex: e.detail.itemIndex
});
}
}
}
},
_loadiframe: function _loadiframe(url) {
if (PlatformJS.mainProcessManager.afterFirstView) {
PlatformJS.mainProcessManager.afterFirstView();
}
if (!this._storyFrame) {
this._initializeiFrame();
}
// Need to first try to hide the progress indicator in case this is a refresh and the previous load
// didn't finish, which means that it didn't hit the onload callback to hide the progress indicator
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
this._storyFrame.src = url;
},
_initializeiFrame: function _initializeiFrame() {
var storyFrame = document.createElement("iframe");
storyFrame.className = "storyFrame";
storyFrame.contentEditable = true;
if (Debug.debuggerEnabled) {
// Turn off scripts when in debugger for ease of debugging
storyFrame.sandbox = "";
}
this._onLoadHandler = function _onLoadHandler() {
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
};
storyFrame.addEventListener("load", this._onLoadHandler);
this._storyFrame = storyFrame;
this._parentContainer = document.getElementById("mainContent");
this._parentContainer.appendChild(storyFrame);
},
_openInBrowser: function _openInBrowser(url) {
try {
var uri = new Windows.Foundation.Uri(url);
Windows.System.Launcher.launchUriAsync(uri);
} catch (e) {
}
}
})
});
})();
