﻿/* telemetry\articleReaderTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.ArticleReader", {
recordActionClick: function recordActionClick(partnerChannelId) {
if (partnerChannelId) {
if (partnerChannelId === "NYT") {
NYTJS.Instrumentation.instance.instrumentBingDailyToPanorama();
}
var event = {
PartnerChannelID: partnerChannelId
};
NewsJS.Telemetry.Utilities.logUserAction(
"Block 1",
NewsJS.Telemetry.String.ActionContext.endOfArticleBlock,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
});
WinJS.Namespace.define("NewsJS.Telemetry.ArticleReader.EntryPoint", {
/*
Values 0, 1 and others are defined in Platform.Instrumentation.InstrumentationArticleEntryPoint.
App specific entry points should start at PlatformReserved + 1 and go up.
*/
categoryPage: Platform.Instrumentation.InstrumentationArticleEntryPoint.platformReserved + 1,
});
})();
/* telemetry\bingDailyTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.BingDaily", {
recordHeroContent: function recordHeroContent(content) {
var k;
if (content && content.itemDataSource) {
var impression = PlatformJS.Navigation.mainNavigator.getCurrentImpression();
if (impression) {
content.itemDataSource.itemFromIndex(0).then(function (item) {
// Do not perform this telemetry during offline, FRE scenario as the 'destination' property in the article
// is unavailable.
if (item && item.data && item.data.article && !item.data.article.isFREOffline) {
var article = item.data.article,
date = NewsJS.Telemetry.Utilities.convertUtcTime(article.published ? article.published.utctime : 0);
k = impression.addContent(
article.source,                             // sourceName
null,                                       // partnerCode
article.destination,                        // contentId
Platform.Telemetry.ContentType.article,     // type
date,                                       // date
null,                                       // uri
NewsJS.Utilities.stripHTML(article.title),  // slug
true,                                       // isSummary
Platform.Telemetry.ContentWorth.free,       // worth
false,                                      // isAd
null                                        // adCampaign
);
impression.logContent(Platform.Telemetry.LogLevel.normal);
if (article.telemetry) {
article.telemetry.k = k;
}
else {
article.telemetry = {
k: k
};
}
}
});
}
}
return k;
},
recordHeroContentView: function recordHeroContentView(content) {
if (content && content.itemDataSource) {
var impression = PlatformJS.Navigation.mainNavigator.getCurrentImpression();
if (impression) {
content.itemDataSource.itemFromIndex(0).then(function (item) {
// Do not perform this telemetry during offline, FRE scenario as the 'destination' property in the article
// is unavailable.
if (item && item.data && item.data.article && !item.data.article.isFREOffline) {
var article = item.data.article;
if (article && article.telemetry && article.telemetry.k > 0) {
impression.logContentView(
Platform.Telemetry.LogLevel.normal,                 // level
article.telemetry.k,                                // k
Platform.Telemetry.ContentViewMechanism.unknown,    // viewMechanism
true,                                               // logOnlyIfNotViewed
null                                                // progress
);
}
}
});
}
}
},
recordHeroClick: function recordHeroClick(item) {
if (item) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.hero);
NewsJS.Telemetry.Utilities.logUserAction(
"Hero Title",
NewsJS.Telemetry.String.ActionContext.hero,
Platform.Telemetry.UserActionOperation.click,
item.telemetry ? item.telemetry.k : 0,
null);
}
},
recordClusterLayout: function recordClusterLayout(clustersDefinition) {
if (clustersDefinition && clustersDefinition.length > 0) {
var clustersPreferences = {};
var processTitle = function (title) {
var result = title,
len = title.length;
if (title.indexOf("_") === 0 && title.lastIndexOf("_") === len - 1) {
result = PlatformJS.Services.resourceLoader.getString(title.substring(1, len - 1));
}
return result;
};
// Start at one so that the reports look nicer.
for (var i = 1; i <= clustersDefinition.length; i++) {
var clusterDefinition = clustersDefinition[i - 1];
var providerConfiguration = clusterDefinition.providerConfiguration;
var clusterId;
if (providerConfiguration) {
// Don't use "TempCluster" as the ID.
if (providerConfiguration.providerType === "NewsApp.DataProviders.TempClusterProvider") {
clusterId = providerConfiguration.strategy;
}
else {
clusterId = providerConfiguration.categoryKey;
}
}
var clusterPreference = {
ClusterID: clusterId || "None",
ClusterGUID: clusterDefinition.guid,
ClusterTitle: clusterDefinition.title ? processTitle(clusterDefinition.title) : "None"
};
// Zero pad the index so that the alphabetical sort of the preferences name correctly shows the
// cluster order.
var index = i < 10 ? "0" + i : i;
clustersPreferences["BingDaily/Cluster" + index] = JSON.stringify(clusterPreference); // Whole objects are not supported, only primitives
}
var jsonString = JSON.stringify(clustersPreferences);
Platform.Telemetry.FlightRecorder.logPreferencesAsJson(Platform.Telemetry.LogLevel.normal, jsonString);
}
},
recordClusterHeaderPress: function recordClusterHeaderPress(categoryKey, categoryTitle, position) {
NewsJS.Telemetry.Cluster.recordClusterHeaderPress(categoryKey, categoryTitle, position);
},
recordRefreshButtonClick: function recordRefreshButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Refresh Button", Platform.Telemetry.ActionContext.appBar);
NewsJS.Telemetry.Utilities.recordRefresh(Platform.Telemetry.ContentClear.refresh);
},
recordEditButtonClick: function recordEditButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Edit Button", Platform.Telemetry.ActionContext.appBar);
},
recordSeeMorePress: function recordSeeMorePress(clusterKey, clusterName) {
NewsJS.Telemetry.Cluster.recordSeeMorePress(clusterKey, clusterName);
},
recordPopularNowPress: function recordPopularNowPress(item) {
if (item) {
var event = {
Title: item.title,
TileSize: item.tileSize
};
NewsJS.Telemetry.Utilities.logUserAction(
"Popular Now Tile",
NewsJS.Telemetry.String.ActionContext.clusterPopularNow,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
recordSourcesItemPress: function recordSourcesItemPress(item) {
if (item && item.data && item.data.data) {
var event = {
Index: item.index,
TileType: item.data.tileType,
DisplayName: item.data.data.displayname,
SourceID: item.data.data.id,
PartnerCode: item.data.data.instrumentation_id,
NewsCategory: item.data.data.newscategory,
};
NewsJS.Telemetry.Utilities.logUserAction(
"Source Tile",
NewsJS.Telemetry.String.ActionContext.clusterSources,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
recordSourcesAddSourcePress: function recordSourcesAddSourcePress() {
NewsJS.Telemetry.Utilities.logUserAction(
"Add Source",
NewsJS.Telemetry.String.ActionContext.clusterSources,
Platform.Telemetry.UserActionOperation.click,
0,
null);
},
});
})();
/* telemetry\categoryTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.Category", {
recordRefreshButtonClick: function recordRefreshButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Refresh Button", Platform.Telemetry.ActionContext.appBar);
NewsJS.Telemetry.Utilities.recordRefresh(Platform.Telemetry.ContentClear.refresh);
},
});
})();
/* telemetry\clusterTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.Cluster", {
recordClusterItemPress: function recordClusterItemPress(clusterKey, clusterItem) {
if (!clusterItem || !clusterItem.isHero) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.cluster);
}
else {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.hero);
}
if (clusterItem) {
var clusterId = NewsJS.Telemetry.Utilities.convertClusterGuidToClusterId(clusterKey);
var event = {
Editorial: clusterItem.editorial ? clusterItem.editorial : false,
ItemType: clusterItem.type,
TemplateClass: clusterItem.templateClass,
ClusterID: clusterId,
ClusterName: clusterItem.telemetry ? clusterItem.telemetry.clusterName : null
};
NewsJS.Telemetry.Utilities.logUserAction(
"Entity Tile",
NewsJS.Telemetry.String.ActionContext.cluster,
Platform.Telemetry.UserActionOperation.click,
clusterItem.telemetry ? clusterItem.telemetry.k : 0,
event);
}
},
recordClusterHeaderPress: function recordClusterHeaderPress(clusterKey, clusterTitle, clusterPosition) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.cluster);
var clusterId = NewsJS.Telemetry.Utilities.convertClusterGuidToClusterId(clusterKey);
var event = {
ClusterPosition: clusterPosition,
ClusterID: clusterId,
ClusterName: clusterTitle
};
NewsJS.Telemetry.Utilities.logUserAction(
"Entity Cluster Header",
NewsJS.Telemetry.String.ActionContext.clusterHeader,
Platform.Telemetry.UserActionOperation.click,
0,
event);
},
recordSeeMorePress: function recordSeeMorePress(clusterKey, clusterTitle) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.cluster);
var clusterId = NewsJS.Telemetry.Utilities.convertClusterGuidToClusterId(clusterKey);
var event = {
ClusterID: clusterId,
ClusterName: clusterTitle
};
NewsJS.Telemetry.Utilities.logUserAction(
"Entity Cluster See More",
NewsJS.Telemetry.String.ActionContext.cluster,
Platform.Telemetry.UserActionOperation.click,
0,
event);
},
});
})();
/* telemetry\customizationTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.Customization", {
recordCustomFeedAdded: function recordCustomFeedAdded(title, url) {
/// <summary>
/// Records the title and URL of a custom RSS feed.
/// </summary>
/// <param name="title" type="String">The RSS feed title.</param>
/// <param name="url" type="String">The RSS feed URL.</param>
if (title && url) {
var event = {
Title: title,
URL: url
};
NewsJS.Telemetry.Utilities.logUserAction(
"Add Feed",
NewsJS.Telemetry.String.ActionContext.sourceManagement,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
recordAddRemove: function recordAddRemove(market, item, type, category) {
if (item) {
var event = {
SourceMarket: NewsJS.Telemetry.Utilities.normalizeMarketString(market),
SourceName: item.displayname,
SourceID: item.id,
Category: category
};
var prefix = item.selected ? "Add" : "Remove";
var action = prefix + " " + type;
var context = type === "Source" ? NewsJS.Telemetry.String.ActionContext.sourceManagement : NewsJS.Telemetry.String.ActionContext.categoryManagement;
NewsJS.Telemetry.Utilities.logUserAction(
action,
context,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
recordCodeError: function recordCodeError(url, message, src) {
if (message) {
var attributes = {
URL: url,
Location: src
};
Platform.Telemetry.FlightRecorder.logCodeErrorWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
Platform.Telemetry.RuntimeEnvironment.javascript,
message,
null,
JSON.stringify(attributes));
}
},
recordSourceSearch: function recordSourceSearch(query, market, resultCount, experience) {
if (query) {                
var details = {
SourceMarket: NewsJS.Telemetry.Utilities.normalizeMarketString(market),
Experience: experience
};
var jsonString = JSON.stringify(details);
Platform.Telemetry.FlightRecorder.logSearch(
Platform.Telemetry.LogLevel.normal,     // level
query,                                  // query
"Sources",                              // index
Platform.Telemetry.SearchMethod.form,   // method
true,                                   // inResults
resultCount,                            // resultCount
true,                                   // isExplicit
false,                                  // isCOMSCORE
false,                                  // autoRefresh
null,                                   // categoryCode
jsonString                              // details
);               
}
},
recordMarketSelection: function recordMarketSelection(market, context) {
var event = {
SourceMarket: NewsJS.Telemetry.Utilities.normalizeMarketString(market)
};
NewsJS.Telemetry.Utilities.logUserAction(
"Market Selection Menu",
context,
Platform.Telemetry.UserActionOperation.click,
0,
event);
},
recordCategorySelection: function recordCategorySelection(market, category, context) {
var event = {
SourceMarket: NewsJS.Telemetry.Utilities.normalizeMarketString(market),
Category: category || "None"
};
NewsJS.Telemetry.Utilities.logUserAction(
"Category Selection List",
context,
Platform.Telemetry.UserActionOperation.click,
0,
event);
},
recordSourceClick: function recordSourceClick(sourceMarket, item) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.browseSources);
if (item) {
var sourceType = NewsJS.Telemetry.Customization.getSourceType(item);
var event = {
PartnerCode: item.instrumentation_id || "None",
SourceName: item.displayname,
SourceID: item.id || "None",
SourceType: sourceType,
SourceMarket: NewsJS.Telemetry.Utilities.normalizeMarketString(sourceMarket)
};
NewsJS.Telemetry.Utilities.logUserAction(
"Browse Source",
NewsJS.Telemetry.String.ActionContext.browseSources,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
// Telemetry: We only need to record the sources once for the lifetime of the process unless there are changes
// to the sources at which point will record the sources at that time.
sourcesReported: false,
recordSources: function recordSources(sources, keyMap) {
if (sources && sources.length > 0) {
var sourcesPreferences = {};
// Start at one so that the reports look nicer.
for (var i = 1; i <= sources.length; i++) {
var source = sources[i - 1].data;
var sourceType = NewsJS.Telemetry.Customization.getSourceType(source);
var sourcePreference = {
PartnerCode: source.instrumentation_id || "None",
SourceName: source.displayname,
SourceID: source.id || "None",
SourceType: sourceType,
SourceMarket: NewsJS.Telemetry.Utilities.normalizeMarketString(source.market)
};
// Zero pad the index so that the alphabetical sort of the preferences name correctly shows the
// cluster order.
var index = i < 10 ? "0" + i : i;
sourcesPreferences["Sources/Source" + index] = JSON.stringify(sourcePreference); // Whole objects are not supported, only primitives
}
var jsonString = JSON.stringify(sourcesPreferences);
Platform.Telemetry.FlightRecorder.logPreferencesAsJson(Platform.Telemetry.LogLevel.normal, jsonString);
}
},
getActionContext: function getActionContext(state) {
// Set the context to the fallback value.
var context = NewsJS.Telemetry.String.ActionContext.selectionPage;
if (state && state.mode) {
switch (state.mode) {
case "Source":
if (state.isBrowse) {
context = NewsJS.Telemetry.String.ActionContext.browseSources;
}
else {
context = NewsJS.Telemetry.String.ActionContext.sourceManagement;
}
break;
case "Cluster":
context = NewsJS.Telemetry.String.ActionContext.categoryManagement;
break;
}
}
return context;
},
getSourceType: function getSourceType(source) {
var type;
if (source.featured) {
type = "Featured";
}
else if (source.type === "InternationalBingDaily") {
type = "International";
}
else if (source.isCustomRSS) {
type = "Custom RSS";
}
else {
type = "RSS";
}
return type;
}
});
})();
/* telemetry\localNewsTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.LocalNews", {
recordCurrentLocationClick: function recordCurrentLocationClick(newLocation, oldLocation) {
/// <summary>
/// Records that the Current Location button was clicked if the location actually changed.
/// </summary>
/// <param name="newLocation" type="String">The new location.</param>
/// <param name="oldLocation" type="String">The old location.</param>
if (newLocation !== oldLocation) {
var event = {
NewLocation: newLocation ? newLocation : "Unknown",
OldLocation: oldLocation ? oldLocation : "Unknown"
};
NewsJS.Telemetry.Utilities.logUserAction(
"Current Location Button",
Platform.Telemetry.ActionContext.appBar,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
recordChangeLocationClick: function recordChangeLocationClick() {
/// <summary>
/// Records that the Change Location button was clicked. This brings up the menu of locations.
/// </summary>
NewsJS.Telemetry.Utilities.recordButtonClick("Change Location Button", Platform.Telemetry.ActionContext.appBar);
},
recordLocationSelection: function recordLocationSelection(newLocation, oldLocation) {
/// <summary>
/// Records that a location was selected if the location actually changed.
/// </summary>
/// <param name="newLocation" type="String">The new location.</param>
/// <param name="oldLocation" type="String">The old location.</param>
if (newLocation) {
if (newLocation !== oldLocation) {
var event = {
NewLocation: newLocation ? newLocation : "Unknown",
OldLocation: oldLocation ? oldLocation : "Unknown"
};
NewsJS.Telemetry.Utilities.logUserAction(
"Location Selection Menu",
Platform.Telemetry.ActionContext.appBar,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
}
},
});
})();
/* telemetry\rssSourcePageTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.RSSSourcePage", {
recordRSSSourceItemPress: function recordRSSSourceItemPress(item) {
if (item) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.rssSource);
NewsJS.Telemetry.Utilities.logUserAction(
"RSS Source Tile",
NewsJS.Telemetry.String.ActionContext.clusterRSSSource,
Platform.Telemetry.UserActionOperation.click,
item.telemetry ? item.telemetry.k : 0,
null);
}
},
});
})();
/* telemetry\searchTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.Search", {
recordSearchResultPress: function recordSearchResultPress(result) {
if (result) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.searchResult);
NewsJS.Telemetry.Utilities.logUserAction(
"Result",
NewsJS.Telemetry.String.ActionContext.results,
Platform.Telemetry.UserActionOperation.click,
result.telemetry ? result.telemetry.k : 0,
null);
}
},
recordAddSectionButtonClick: function recordAddSectionButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Add Section Button", Platform.Telemetry.ActionContext.appBar);
},
recordRefreshButtonClick: function recordRefreshButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Refresh Button", Platform.Telemetry.ActionContext.appBar);
NewsJS.Telemetry.Utilities.recordRefresh(Platform.Telemetry.ContentClear.refresh);
},
recordSearch: function recordSearch(query, resultCount, searchOrigin, /*@optional*/fromCache) {
if (query) {
var searchClassification = NewsJS.Telemetry.Search.getSearchClassification(searchOrigin);
Platform.Telemetry.FlightRecorder.logSearch(
Platform.Telemetry.LogLevel.normal,                     // level
query,                                                  // query
"Bing",                                                 // index
searchClassification.method,                            // method
true,                                                   // inResults
resultCount,                                            // resultCount
fromCache ? false : searchClassification.isExplicit,    // isExplicit
fromCache ? false : searchClassification.isCOMSCORE,    // isCOMSCORE
false,                                                  // autoRefresh
searchClassification.formCode,                          // categoryCode
null);                                                  // details
}
},
getSearchClassification: function getSearchClassification(searchOrigin) {
var formCode;
var method;
var isExplicit;
var isCOMSCORE;
switch (searchOrigin) {
case NewsJS.Telemetry.Search.Origin.pin:
formCode = NewsJS.Telemetry.Search.FormCode.pin;
method = Platform.Telemetry.SearchMethod.view;
isExplicit = false;
isCOMSCORE = true;
break;
case NewsJS.Telemetry.Search.Origin.internalSearchCharm:
case NewsJS.Telemetry.Search.Origin.externalSearchCharm:
formCode = NewsJS.Telemetry.Search.FormCode.charm;
method = Platform.Telemetry.SearchMethod.charm;
isExplicit = true;                                      // Only true when the user explicitly types something in.
isCOMSCORE = true;
break;
case NewsJS.Telemetry.Search.Origin.popularNow:
formCode = NewsJS.Telemetry.Search.FormCode.popularNow;
method = Platform.Telemetry.SearchMethod.view;
isExplicit = false;
isCOMSCORE = true;
break;
case NewsJS.Telemetry.Search.Origin.topics:
formCode = NewsJS.Telemetry.Search.FormCode.topics;
method = Platform.Telemetry.SearchMethod.view;
isExplicit = false;
isCOMSCORE = false;
break;
case NewsJS.Telemetry.Search.Origin.topicHeader:
formCode = NewsJS.Telemetry.Search.FormCode.topics;
method = Platform.Telemetry.SearchMethod.view;
isExplicit = false;
isCOMSCORE = true;
break;
case NewsJS.Telemetry.Search.Origin.navBar:
formCode = NewsJS.Telemetry.Search.FormCode.topics;
method = Platform.Telemetry.SearchMethod.view;
isExplicit = false;
isCOMSCORE = true;
break;
case NewsJS.Telemetry.Search.Origin.newsFromTheWeb:
formCode = NewsJS.Telemetry.Search.FormCode.category;
method = Platform.Telemetry.SearchMethod.view;
isExplicit = false;
isCOMSCORE = false;
break;
default:
formCode = NewsJS.Telemetry.Search.FormCode.unknown;
method = Platform.Telemetry.SearchMethod.unknown;
isExplicit = false;
isCOMSCORE = false;
break;
}
return {
formCode: formCode,
method: method,
isExplicit: isExplicit,
isCOMSCORE: isCOMSCORE
};
},
});
WinJS.Namespace.define("NewsJS.Telemetry.Search.Origin", {
internalSearchCharm: "Internal Search Charm",
externalSearchCharm: "External Search Charm",
popularNow: "Popular Now",
pin: "Pin",
topics: "Topics",
topicHeader: "Topic Header",
navBar: "Nav Bar",
newsFromTheWeb: "News from the Web"
});
WinJS.Namespace.define("NewsJS.Telemetry.Search.FormCode", {
charm: "APXN01",
pin: "APXN02",
topics: "APXN03",
popularNow: "APXN04",
category: "APXN05",
unknown: "APXN99"                                       // This is not an official code. If this ever shows up in the logs we know there's a scenario that is not covered.
});
})();
/* telemetry\slideShowTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.SlideShow", {
recordSlideShowToArticle: function recordSlideShowToArticle(slideShowId, market, pageId, contentId) {
if (slideShowId && pageId && contentId) {
var event = {
SlideshowID: slideShowId,
ContentMarket: NewsJS.Telemetry.Utilities.normalizeMarketString(market),
PageID: pageId,
ContentID: contentId
};
NewsJS.Telemetry.Utilities.logUserAction(
"Article",
NewsJS.Telemetry.String.ActionContext.slideShow,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
});
})();
/* telemetry\storyPageTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.StoryPage", {
recordContent: function recordContent(content) {
var k;
var impression = PlatformJS.Navigation.mainNavigator.getCurrentImpression();
if (impression) {
var date = NewsJS.Telemetry.Utilities.convertUtcTime(content.publishTimeRaw);
k = impression.addContent(
content.source,                             // sourceName
null,                                       // partnerCode
null,                                       // contentId
Platform.Telemetry.ContentType.article,     // type
date,                                       // date
content.articleUrl,                         // uri
NewsJS.Utilities.stripHTML(content.title),  // slug
false,                                      // isSummary
Platform.Telemetry.ContentWorth.free,       // worth
false,                                      // isAd
null                                        // adCampaign
);
impression.logContent(Platform.Telemetry.LogLevel.normal);
if (content.telemetry) {
content.telemetry.k = k;
}
}
return k;
},
recordContentView: function recordContentView(k) {
if (k) {
var impression = PlatformJS.Navigation.mainNavigator.getCurrentImpression();
if (impression) {
impression.logContentView(
Platform.Telemetry.LogLevel.normal,                 // level
k,                                                  // k
Platform.Telemetry.ContentViewMechanism.unknown,    // viewMechanism
true,                                               // logOnlyIfNotViewed
null                                                // progress
);
}
}
},
recordSnappedItemPress: function recordSnappedItemPress(item) {
if (item) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.iframeArticle);
//NewsJS.Telemetry.Utilities.logUserAction(
//    "RSS Source Tile",
//    "Cluster: RSS Source",
//    Platform.Telemetry.UserActionOperation.click,
//    item.telemetry ? item.telemetry.k : 0,
//    null);
}
},
});
})();
/* telemetry\sourcePageTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.SourcePage", {
recordSourceItemPress: function recordSourceItemPress(item) {
if (item) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.sourceTile);
NewsJS.Telemetry.Utilities.logUserAction(
"Source Tile",
NewsJS.Telemetry.String.ActionContext.clusterSource,
Platform.Telemetry.UserActionOperation.click,
item.telemetry ? item.telemetry.k : 0,
null);
}
},
recordRefreshButtonClick: function recordRefreshButtonClick(item) {
if (item) {
var event = {
SourceID: item.id,
SourceName: item.displayname,
Category: item.newscategory
};
NewsJS.Telemetry.Utilities.logUserAction(
"Refresh Button",
Platform.Telemetry.ActionContext.appBar,
Platform.Telemetry.UserActionOperation.click,
0,
event);
NewsJS.Telemetry.Utilities.recordRefresh(Platform.Telemetry.ContentClear.refresh);
}
},
});
})();
/* telemetry\telemetryStrings.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.String.ImpressionContext", {
bingDaily: "/Bing Daily",
category: "/Category",
source: "/Source",
rssSource: "/RSS Source",
topics: "/Topics",
search: "/Search Results",
slideShow: "/Slideshow",
iframeArticle: "/IFrame Article",
selectionPage: "/Selection Page",               // Fallback
browseSources: "/Browse Sources",
addSource: "/Add Source",
addCategory: "/Add Category",
partnerPano: "/Partner Pano",
subPartnerPano: "/Partner Pano/Sub Pano",
emptyPage: "/Empty Page"                        // Used for memory leak testing
});
WinJS.Namespace.define("NewsJS.Telemetry.String.ImpressionNavMethod", {
iframeArticle: "IFrame Article",
customizationPage: "Customization Page",
browseSources: "Browse Sources",
topic: "Topic",
topicHeader: "Topic Header",
slideShow: "Slideshow",
pinnedTile: "Pinned Tile",
mySources: "My Sources",
popularNow: "Popular Now",
rssSource: "RSS Source",
searchResult: "Search Result",
sourceTile: "Source Tile"
});
WinJS.Namespace.define("NewsJS.Telemetry.String.ActionContext", {
endOfArticleBlock: "End of Article Block",
hero: "Hero",
cluster: "Cluster",
clusterHeader: "Cluster Header",
clusterPopularNow: "Cluster: Popular Now",
clusterSources: "Cluster: Sources",
clusterRSSSource: "Cluster: RSS Source",
clusterSource: "Cluster: Source",
results: "Results",
browseSources: "Browse Sources",
sourceManagement: "Source Management",
categoryManagement: "Category Management",
selectionPage: "Selection Page",               // Fallback
topics: "Topics",
slideShow: "Slideshow"
});
WinJS.Namespace.define("NewsJS.Telemetry.String.ErrorContext", {
topics: "Topics",
source: "Source",
prefetch: "Prefetch",
utilities: "Utilities",
pano: "Pano",
builtInPartnerPano: "Built In Partner Pano"
});
})();
/* telemetry\telemetryUtilities.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.Utilities", {
logCustom: function logCustom(logLevel, eventType, jsonObject) {
/// <summary>
/// Logs a custom telemetry event.
/// </summary>
/// <param name="logLevel">The Platform.Telemetry.LogLevel to use.</param>
/// <param name="evenType" type="String">The name of the event type. Must start with the prefix 'App'.</param>.
/// <param name="jsonObject">The JSON object to stringify.</param>
if (eventType && eventType.indexOf("App") === 0 && jsonObject) {
var jsonText = JSON.stringify(jsonObject);
Platform.Telemetry.FlightRecorder.logCustom(logLevel, eventType, jsonText);
}
},
logUserAction: function logUserAction(element, context, operation, k, jsonObject) {
if (element && operation) {
if (jsonObject) {
var jsonText = JSON.stringify(jsonObject);
Platform.Telemetry.FlightRecorder.logUserActionWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
context,
element,
operation,
Platform.Telemetry.UserActionMethod.unknown,
k,
jsonText);
}
else {
Platform.Telemetry.FlightRecorder.logUserAction(
Platform.Telemetry.LogLevel.normal,
context,
element,
operation,
Platform.Telemetry.UserActionMethod.unknown,
k);
}
}
},
recordButtonClick: function recordButtonClick(element, context) {
if (element) {
Platform.Telemetry.FlightRecorder.logUserAction(
Platform.Telemetry.LogLevel.normal,
context,
element,
Platform.Telemetry.UserActionOperation.click,
Platform.Telemetry.UserActionMethod.unknown,
0);
}
},
closePartnerSession: function closePartnerSession() {
if (!NewsJS.Partners.Config.isPartnerApp) {
if (!NewsJS.Partners.Config.isPartnerPage("NYT") && (typeof NYTJS !== "undefined")) {
NYTJS.Instrumentation.instance.closeSession();
}
if (!NewsJS.Partners.Config.isPartnerPage("WSJ") && (typeof WSJJS !== "undefined")) {
WSJJS.Instrumentation.instance.closeSession();
}
}
},
recordPinnedTileLaunch: function recordPinnedTileLaunch(tileType) {
if (tileType) {
var event = {
TileType: tileType
};
// TODO: We must get CI approval for custom events.
NewsJS.Telemetry.Utilities.logCustom(Platform.Telemetry.LogLevel.normal, "AppPinnedTileLaunch", event);
}
},
recordPinnedTileAdded: function recordPinnedTileAdded(tileShortName, tileDisplayName, tileLaunchTargetPage, tileLaunchArgument) {
if (tileShortName && tileDisplayName && tileLaunchTargetPage && tileLaunchArgument) {
var event = NewsJS.Telemetry.Utilities.createPinnedTileEvent(tileShortName, tileDisplayName, tileLaunchTargetPage, tileLaunchArgument);
NewsJS.Telemetry.Utilities.logUserAction(
"Pinned Tile Added",
Platform.Telemetry.ActionContext.appBar,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
recordPinnedTileRemoved: function recordPinnedTileRemoved(tileShortName, tileDisplayName, tileLaunchTargetPage, tileLaunchArgument) {
if (tileShortName && tileDisplayName && tileLaunchTargetPage && tileLaunchArgument) {
var event = NewsJS.Telemetry.Utilities.createPinnedTileEvent(tileShortName, tileDisplayName, tileLaunchTargetPage, tileLaunchArgument);
NewsJS.Telemetry.Utilities.logUserAction(
"Pinned Tile Removed",
Platform.Telemetry.ActionContext.appBar,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
createPinnedTileEvent: function createPinnedTileEvent(tileShortName, tileDisplayName, tileLaunchTargetPage, tileLaunchArgument) {
var event;
if (tileLaunchTargetPage === "NewsJS.CategoryPage") {
event = {
ShortName: tileShortName,
DisplayName: tileDisplayName,
LaunchTargetPage: "Category Page",
CategoryName: tileLaunchArgument.categoryName,
CategoryKey: tileLaunchArgument.categoryKey,
Market: NewsJS.Telemetry.Utilities.normalizeMarketString(tileLaunchArgument.market)
};
}
else if (tileLaunchTargetPage === "NewsJS.Search") {
event = {
ShortName: tileShortName,
DisplayName: tileDisplayName,
LaunchTargetPage: "Search Results Page",
QueryText: tileLaunchArgument.queryText,
SearchOrigin: tileLaunchArgument.searchOrigin
};
}
else if (tileLaunchTargetPage === "NewsJS.RSSSourcePage") {
event = {
ShortName: tileShortName,
DisplayName: tileDisplayName,
LaunchTargetPage: "RSS Source Page",
URL: tileLaunchArgument.id
};
}
else {
// Fall-back instrumentation in case there are changes in the pages but the above list is not updated.
event = {
ShortName: tileShortName,
DisplayName: tileDisplayName,
LaunchTargetPage: tileLaunchTargetPage
};
}
return event;
},
recordAutoRefresh: function recordAutoRefresh() {
NewsJS.Telemetry.Utilities.recordRefresh(Platform.Telemetry.ContentClear.autoRefresh);
},
recordRefresh: function recordRefresh(contentClearKind) {
var impression = PlatformJS.Navigation.mainNavigator.getCurrentImpression();
if (impression) {
impression.logContentClear(Platform.Telemetry.LogLevel.normal, contentClearKind);
}
},
recordToastActivation: function recordToastActivation(uri) {
var event = {
Uri: uri
};
NewsJS.Telemetry.Utilities.logUserAction(
"Toast Notification Click",
Platform.Telemetry.ActionContext.toast,
Platform.Telemetry.UserActionOperation.click,
0,
event);
},
recordShare: function recordShare(uri, k, shareSource) {
if (uri) {
var impression = PlatformJS.Navigation.mainNavigator.getCurrentImpression();
if (impression) {
var event = {
ShareSource: shareSource
};
var jsonText = JSON.stringify(event);
impression.logContentShareWithJsonAttributes(
Platform.Telemetry.LogLevel.normal,
null,
uri,
k ? k : 0,
jsonText);
}
}
},
recordPrefetchPreferences: function recordPrefetchPreferences() {
var prefetchPreferences = {};
prefetchPreferences["Prefetch/IsEnabled"] = NewsJS.PrefetchManager.isPrefetchEnabled;
var jsonString = JSON.stringify(prefetchPreferences);
Platform.Telemetry.FlightRecorder.logPreferencesAsJson(Platform.Telemetry.LogLevel.normal, jsonString);
},
convertUtcTime: function convertUtcTime(utctime, epoch) {
var dateValue = 0;
if (typeof (utctime) === "number") {
epoch = epoch || Date.UTC(1601, 0, 1, 0, 0, 0);
dateValue = utctime / 10000 + epoch;
}
return new Date(dateValue);
},
convertClusterGuidToClusterId: function convertClusterGuidToClusterId(clusterGuid) {
var clusterId = clusterGuid;
if (clusterGuid) {
var index = clusterGuid.lastIndexOf(".");
if (index !== -1) {
clusterId = clusterGuid.substring(0, index);
}
}
return clusterId;
},
normalizeMarketString: function normalizeMarketString(market) {
return market ? market.toLowerCase() : "None";
}
});
})();
/* telemetry\topicsTelemetry.js */
(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Telemetry.Topics", {
recordPreferences: function recordPreferences() {
var preferences = {};
var followedSearchTopics = NewsJS.State.roaming.followedSearchTopics;
if (followedSearchTopics) {
for (var i = 1; i <= followedSearchTopics.length; i++) {
var index = i < 10 ? "0" + i : i;
preferences["Topics/Topic" + index] = followedSearchTopics[i - 1].query;
}
}
var jsonString = JSON.stringify(preferences);
Platform.Telemetry.FlightRecorder.logPreferencesAsJson(Platform.Telemetry.LogLevel.normal, jsonString);
},
recordSectionHeaderPress: function recordSectionHeaderPress(sectionName, sectionIndex) {
if (sectionName) {
NewsJS.Telemetry.Cluster.recordClusterHeaderPress("topicCluster" + sectionIndex, sectionName, sectionIndex);
}
},
recordTopicItemPress: function recordTopicItemPress(item) {
if (item) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.topic);
var event = {
CategoryKey: item.categoryKey
};
NewsJS.Telemetry.Utilities.logUserAction(
"Article",
NewsJS.Telemetry.String.ActionContext.topics,
Platform.Telemetry.UserActionOperation.click,
item.telemetry ? item.telemetry.k : 0,
event);
}
},
recordAddTopic: function recordAddTopic(section) {
if (section) {
var event = {
Topic: section
};
NewsJS.Telemetry.Utilities.logUserAction(
"Add Topic",
NewsJS.Telemetry.String.ActionContext.topics,
Platform.Telemetry.UserActionOperation.click,
0,
event);
}
},
recordAddButtonClick: function recordAddButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Add Button", Platform.Telemetry.ActionContext.appBar);
},
recordRemoveButtonClick: function recordRemoveButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Remove Button", Platform.Telemetry.ActionContext.appBar);
},
recordPromoteButtonClick: function recordPromoteButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Promote Button", Platform.Telemetry.ActionContext.appBar);
},
recordClearButtonClick: function recordClearButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Clear Button", Platform.Telemetry.ActionContext.appBar);
},
recordEditButtonClick: function recordEditButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Edit Button", Platform.Telemetry.ActionContext.appBar);
},
recordRefreshButtonClick: function recordRefreshButtonClick() {
NewsJS.Telemetry.Utilities.recordButtonClick("Refresh Button", Platform.Telemetry.ActionContext.appBar);
NewsJS.Telemetry.Utilities.recordRefresh(Platform.Telemetry.ContentClear.refresh);
},
recordSeeMorePress: function recordSeeMorePress(sectionName) {
NewsJS.Telemetry.Cluster.recordSeeMorePress("topicCluster", sectionName);
},
recordAddTopicTilePress: function recordAddTopicTilePress() {
NewsJS.Telemetry.Utilities.logUserAction(
"Add Topic Tile",
NewsJS.Telemetry.String.ActionContext.topics,
Platform.Telemetry.UserActionOperation.click,
0,
null);
},
});
})();
