﻿(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
BondsDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
}, {
queryServiceId: "bonds",
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("localizeFor", FinanceAppJS.Globalization.getMarket());
return urlParams; 
},
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.Bonds,
dataBuilder: function dataBuilder(response) {
var dataSet = {};
var dt = response;
if (dt && dt.length > 0 && dt[0].Data.length > 0) {
for (var i = 0; i < dt[0].Data.length; i++) {
dt[0].Data[i].Dn = dt[0].Data[i].Dn; 
}
}
var data = [];
var tableHeaders = ['Maturity', 'YieldPercent', 'Yesterday', 'Last Week Header', 'Last Month', 'Last Year'];
var localizedTableHeaders = [];
//Yield% is a special case as in RTL it becomes %Yield adding localization escape character to avoid this
for (var k = 0; k < tableHeaders.length; k++){
if (tableHeaders[i] === 'YieldPercent'){
localizedTableHeaders.push(utils.addLtrMark(utils.getResourceString(tableHeaders[k])));
}
else {
localizedTableHeaders.push(utils.getResourceString(tableHeaders[k]));
}
}
data.push(localizedTableHeaders);
if (dt && dt[0] && dt[0].Data && dt[0].Data.length > 0) {
for (var j = 0; j < dt[0].Data.length; j++) {
var item = [];
var dataItem = dt[0].Data[j];
item.push(dataItem.Dn); 
item.push(FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Y, 2));
item.push(FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Yes, 2));
item.push(FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Lw, 2));
item.push(FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Lm, 2));
item.push(FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Ly, 2));
data.push(item);
}
}
var bonds = dt;
//Data source for bonds chart
if (bonds && bonds.length > 0 && bonds[0].Data && bonds[0].Data.length >= 10) {
var noOfYears = 2; //bonds data has 2 series, one for current year and one for last year.                    
var chartData =  [];                        
var xAxisIndices = [1, 3, 5, 7, 8, 9, 11, 13, 14, 15];
var xAxisLabels = ['1m', '3m', '6m', '1y', '2y', '3y', '5y', '10y', '20y', '30y'];
var activeSeriesObjects = [{seriesId: 'year', displayName: ''}, {seriesId: 'lastYear', displayName: ''}];
for (var i = 0; i < noOfYears; i++) {
chartData[i] = { series: [], seriesId: activeSeriesObjects[i].seriesId, chartType: "bonds", Ycp: 0};                        
}
for (var i = 0; i < bonds[0].Data.length; i++) {
chartData[0].series[i] = [];
chartData[0].series[i].push(xAxisIndices[i], bonds[0].Data[i].Y, utils.getResourceString("Bonds" + xAxisLabels[i]));
chartData[1].series[i] = [];
chartData[1].series[i].push(xAxisIndices[i], bonds[0].Data[i].Ly, utils.getResourceString("Bonds" + xAxisLabels[i]));
}
dataSet.table = data;
dataSet.chart = chartData;
dataSet.activeSeries = activeSeriesObjects;             
return dataSet;
}
} 
})
});
})();
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS", {
BondsPage: WinJS.Class.derive(FinanceAppJS.FinanceBasePage, function (state) {
/// <summary>
/// Bonds Page Controller
/// </summary>
FinanceAppJS.FinanceBasePage.call(this, state);
var clusterDefinition = {
chart: {
clusterTitle: utils.getResourceString("curve"),
dataProviders: "BondsDataProvider",
clusterControl: "FinanceAppJS.Controls.ModuleClusterControl",
clusterOptions: {
moduleInfo: { templateId: "bondChart" }
}
},
bonds: {
clusterTitle: utils.getResourceString("BondsUS Treasury Bonds"),
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: "BondsDataProvider",
clusterOptions: {
dataKey: ["table"],
tapBehavior: 'none',
summary: utils.getResourceString("BondsUS Treasury Bonds"),
}
}
};
this.setClusterDefinitions(clusterDefinition);
this.pageTitle = utils.getResourceString("Bonds");
},
{
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Bonds";
},
onNavigateAway: function onNavigateAway(reason) {
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away
/// </summary>
/// <param name="reason">
/// The reason why the page is being navigated away: "backNavigation" or "newNavigation"
/// </param>
/// <returns>
/// Nothing
/// </returns>
FinanceAppJS.FinanceBasePage.prototype.onNavigateAway.call(this);
var chartControlDiv = document.querySelector(".bondschart");
if (chartControlDiv) {
var chartControl = chartControlDiv.winControl;
if (chartControl) {
chartControl.dispose();
}
}
}
})
});
})();