﻿(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.Calculator", {
FinancialCalcPage: WinJS.Class.derive(FinanceAppJS.Calculator.BasePage, function (state) {
FinanceAppJS.Calculator.BasePage.call(this);
var commonChartOptions = { isSplitBar: true };
var dataObject = {
pageTitle: "FinancialCalculator",
data: {
modes: [
{
title: "TimeValueOfMoney",
subModeListLabel: utils.getResourceString("Calculate"),
navInstrumentId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Nav_Click,
subModes: [
{
title: "TimeValueOfMoney",
subtitle: "FutureValue",
inputTemplate: "FutureValueInputTemplate",
outputTemplate: "timeValueOutputTemplate",
inputFields: ["monthlyInvestment", "currentSavings", "annualInterest", "compounding", "noOfYrs"],
chartOptions: commonChartOptions,
firstTabDisplayString: "Chart",
secondTabDisplayString: "CashFlow",
calculateFunction: this.savingsEstimatorCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Calculate_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_PageView_Click,
precision: { annualInterest: 4 },
},
{
title: "TimeValueOfMoney",
subtitle: "PresentValue",
inputTemplate: "PresentValueInputTemplate",
outputTemplate: "timeValueOutputTemplate",
inputFields: ["savingsGoal", "monthlyInvestment", "annualInterest", "compounding", "noOfYrs"],
chartOptions: commonChartOptions,
firstTabDisplayString: "Chart",
secondTabDisplayString: "CashFlow",
calculateFunction: this.presentValueCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Calculate_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_PageView_Click,
precision: { annualInterest: 4 },
},
{
title: "TimeValueOfMoney",
subtitle: "NoOfYrs",
inputTemplate: "NoOfYrsInputTemplate",
outputTemplate: "TimeForGoalOutputTemplate",
inputFields: ["currentSavings", "monthlyInvestment", "annualInterest", "compounding", "savingsGoal"],
chartOptions: commonChartOptions,
firstTabDisplayString: "Chart",
secondTabDisplayString: "CashFlow",
calculateFunction: this.timeForGoalCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Calculate_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_PageView_Click,
precision: { annualInterest: 4 },
},
{
title: "TimeValueOfMoney",
subtitle: "MonthlyPayment",
inputTemplate: "MonthlyPaymentInputTemplate",
outputTemplate: "timeValueOutputTemplate",
inputFields: ["savingsGoal", "currentSavings", "annualInterest", "compounding", "noOfYrs"],
chartOptions: commonChartOptions,
firstTabDisplayString: "Chart",
secondTabDisplayString: "CashFlow",
calculateFunction: this.savingsForGoalCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Calculate_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_PageView_Click,
precision: { annualInterest: 4 },
}
]
},
{
title: "ROICalculator",
inputTemplate: "ROIInputTemplate",
outputTemplate: "ROIOutputTemplate",
inputFields: ["originalInvestment", "investmentReturn", "noOfYrs", "noOfMonths"],
chartOptions: commonChartOptions,
firstTabDisplayString: "Simple",
secondTabDisplayString: "Compound",
calculateFunction: this.ROICalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialROI_Calculate_Click,
navInstrumentId: FinanceAppJS.Constants.Instrumentation.FinancialROI_Nav_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.FinancialROI_PageView_Click,
},
]
}
};
this.setDataObject(dataObject);
}, {
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Tools/Financial Calculator";
},
savingsForGoalCalculateFunction: function savingsForGoalCalculateFunction(inputFields, minValues) {
try {
var output = {};
output.opfields = {};
if (!inputFields.savingsGoal.length) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueEmptyMsg") };
}
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestEmptyMsg") };
}
if (!inputFields.currentSavings.length) {
inputFields.currentSavings = 0;
}
var noOfYears = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYrs);
var savingsGoal = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.savingsGoal);
if (savingsGoal < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.savingsGoal)) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueLessThanMinimumMsg").format(minValues.savingsGoal) };
}
var currentSavings = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentSavings);
if (savingsGoal < currentSavings) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueLessThanPresentValueMsg") };
}
var compoundingNMonths = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.compounding);
var interestPercent = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest);
if (interestPercent === 0) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestZeroMsg") };
}
var annualInterest = interestPercent / 100;
var periodsPerYear = 12 / compoundingNMonths;
var totalPeriods = noOfYears * periodsPerYear;
var interestPerPeriod = annualInterest / periodsPerYear;
var FVCurrentSaving = currentSavings * Math.pow((1 + interestPerPeriod), totalPeriods);
if (savingsGoal < FVCurrentSaving) {
return { type: 'error', errorMsg: utils.getResourceString("PresentValueSufficientMsg") };
}
var savingGoalByPeriodicPayment = savingsGoal - FVCurrentSaving;
var savingPerPeriod = savingGoalByPeriodicPayment / ((Math.pow((1 + interestPerPeriod), totalPeriods) - 1) / interestPerPeriod);
var monthlyInvestment = savingPerPeriod / compoundingNMonths;
var savingsPerYear = savingPerPeriod * periodsPerYear;
var cumulativeInvestment = savingPerPeriod * totalPeriods + currentSavings;
var interestEarned = savingsGoal - cumulativeInvestment;
var formatActualValue = null;
//result fields
output.opfields.LabelLine1 = utils.getResourceString("FMILabelLine1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(currentSavings),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(savingsGoal),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYears, 0),
CollectionProvider.Formatters.formatPercentWithoutRounding(interestPercent));
output.opfields.LabelLine2 = utils.getResourceString("LoanStatement_2");
output.opfields.opfield1 = FinanceAppJS.Utils.Formatters.numberFormatter(monthlyInvestment,0);
output.opfields.cumulativeInvestment = FinanceAppJS.Utils.Formatters.numberFormatter(cumulativeInvestment,0);
output.opfields.interest = FinanceAppJS.Utils.Formatters.numberFormatter(interestEarned,0);
var data = this.calculateChartAndTable(currentSavings, totalPeriods, compoundingNMonths, savingPerPeriod, interestPerPeriod);
output.firstTab = { data: this.scaleChart(data.chartData, compoundingNMonths, totalPeriods), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Chart_Click };
output.secondTab = { data: data.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.AutoLoan_Chart_Click };
return output;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
savingsEstimatorCalculateFunction: function savingsEstimatorCalculateFunction(inputFields, minValues) {
try {
var output = {};
output.opfields = {};
if ((!inputFields.monthlyInvestment.length) && (!inputFields.currentSavings.length)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentAndPresentValueEmptyMsg") };
}
else if (!inputFields.monthlyInvestment.length) {
inputFields.monthlyInvestment = 0;
}
else if (!inputFields.currentSavings.length) {
inputFields.currentSavings = 0;
}
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestEmptyMsg") };
}
var noOfYears = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYrs);
var currentSavings = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentSavings);
var monthlyInvestment = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.monthlyInvestment);
if ((monthlyInvestment === 0) && (currentSavings === 0)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentAndPresentValueZeroMsg") };
}
if (currentSavings === 0 && monthlyInvestment < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.monthlyInvestment)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentLessThanMinimumMsg").format(minValues.monthlyInvestment) };
}
var interestPercent = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest);
if (interestPercent === 0) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestZeroMsg") };
}
var annualInterest = interestPercent / 100;
var compoundingNMonths = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.compounding);
var periodsPerYear = 12 / compoundingNMonths;
var totalPeriods = noOfYears * periodsPerYear;
var savingPerPeriod = monthlyInvestment * compoundingNMonths;
var savingsPerYear = savingPerPeriod * periodsPerYear;
var interestPerPeriod = annualInterest / periodsPerYear;
var savingGoalByPeriodicPayment = savingPerPeriod * ((Math.pow(1 + interestPerPeriod, totalPeriods) - 1) / interestPerPeriod);
var FVCurrentSaving = currentSavings * Math.pow((1 + interestPerPeriod), totalPeriods);
var savingsGoal = savingGoalByPeriodicPayment + FVCurrentSaving;
var cumulativeInvestment = savingPerPeriod * totalPeriods + currentSavings;
var interestEarned = savingsGoal - cumulativeInvestment;
var formatActualValue = null;
//result fields
output.opfields.LabelLine1 = utils.getResourceString("FVLabelLine1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(currentSavings),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(monthlyInvestment),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYears, 0),
CollectionProvider.Formatters.formatPercentWithoutRounding(interestPercent));
output.opfields.LabelLine2 = utils.getResourceString("FVLabelLine2");
// Format Output Number precision.
formatActualValue = FinanceAppJS.Utils.Formatters.formatOutputNumber(savingsGoal);
if (formatActualValue && formatActualValue !== savingsGoal) {
output.opfields.opfield1 = formatActualValue;
}
else {
output.opfields.opfield1 = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(savingsGoal, 0);
}
var formatCI = FinanceAppJS.Utils.Formatters.formatOutputNumber(cumulativeInvestment);
if (formatCI && formatCI !== cumulativeInvestment) {
output.opfields.cumulativeInvestment = formatCI;
}
else {
output.opfields.cumulativeInvestment = FinanceAppJS.Utils.Formatters.numberFormatter(cumulativeInvestment, 0);
}
var formatIE = FinanceAppJS.Utils.Formatters.formatOutputNumber(interestEarned);
if (formatIE && formatIE !== interestEarned) {
output.opfields.interest = formatIE;
}
else {
output.opfields.interest = FinanceAppJS.Utils.Formatters.numberFormatter(interestEarned, 0);
}
if (savingsGoal > 999999999999999) {
return {
type: 'error', errorMsg: utils.getResourceString("FutureValueMsg").format(FinanceAppJS.Utils.Formatters.toolsNumberFormatter(savingsGoal, 0)) + "</br>" +
utils.getResourceString("CumulativeInvestmentMsg").format(output.opfields.cumulativeInvestment) +"</br>" +
utils.getResourceString("CumulativeInterestMsg").format(output.opfields.interest)
};
}
var data = this.calculateChartAndTable(currentSavings, totalPeriods, compoundingNMonths, savingPerPeriod, interestPerPeriod);
output.firstTab = { data: this.scaleChart(data.chartData, compoundingNMonths, totalPeriods), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Chart_Click };
output.secondTab = { data: data.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.AutoLoan_Chart_Click };
return output;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
timeForGoalCalculateFunction: function timeForGoalCalculateFunction(inputFields, minValues) {
try {
var output = {};
output.opfields = {};
if (!inputFields.savingsGoal.length) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueEmptyMsg") };
}
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestEmptyMsg") };
}
if ((!inputFields.monthlyInvestment.length) && (!inputFields.currentSavings.length)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentAndPresentValueEmptyMsg") };
}
else if (!inputFields.monthlyInvestment.length) {
inputFields.monthlyInvestment = 0;
}
else if (!inputFields.currentSavings.length) {
inputFields.currentSavings = 0;
}
var currentSavings = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentSavings);
var savingsGoal = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.savingsGoal);
if (savingsGoal < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.savingsGoal)) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueLessThanMinimumMsg").format(minValues.savingsGoal) };
}
var monthlyInvestment = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.monthlyInvestment);
if (savingsGoal < currentSavings) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueLessThanPresentValueMsg") };
}
if (savingsGoal === currentSavings) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueEqualsPresentValueMsg") };
}
if (savingsGoal < monthlyInvestment) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueLessThanMonthlyPaymentMsg") };
}
if ((monthlyInvestment === 0) && (currentSavings === 0)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentAndPresentValueZeroMsg") };
}
if (currentSavings === 0 && monthlyInvestment < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.monthlyInvestment)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentLessThanMinimumMsg").format(minValues.monthlyInvestment) };
}
var interestPercent = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest);
if (interestPercent === 0) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestZeroMsg") };
}
var annualInterest = interestPercent / 100;
var compoundingNMonths = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.compounding);
var periodsPerYear = 12 / compoundingNMonths;
var interestPerPeriod = annualInterest / periodsPerYear;
var savingPerPeriod = monthlyInvestment * compoundingNMonths;
var totalPeriods = Math.log(((savingsGoal * interestPerPeriod) + savingPerPeriod) / ((currentSavings * interestPerPeriod) + savingPerPeriod)) / Math.log(1 + interestPerPeriod);
var noOfYears = totalPeriods / periodsPerYear;
var totalMonths = noOfYears * 12;
var yearComponent = Math.floor(totalMonths / 12);
var monthComponent = Math.ceil(totalMonths % 12);
var ceilNoOfYears = Math.ceil(noOfYears);
var savingGoalByPeriodicPayment = savingPerPeriod * ((Math.pow(1 + interestPerPeriod, totalPeriods) - 1) / interestPerPeriod);
var FVCurrentSaving = currentSavings * Math.pow((1 + interestPerPeriod), totalPeriods);
var cumulativeInvestment = savingPerPeriod * totalPeriods + currentSavings;
var interestEarned = savingsGoal - cumulativeInvestment;
output.opfields.LabelLine1 = utils.getResourceString("FNYLabelLine1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(currentSavings),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(savingsGoal),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(monthlyInvestment),
CollectionProvider.Formatters.formatPercentWithoutRounding(interestPercent));
output.opfields.LabelLine2 = utils.getResourceString("NYLabelLine2");
output.opfields.noOfYrs = "";
output.opfields.noOfMonths = "";
output.opfields.yearLabel = "";
output.opfields.monthLabel = "";
if (yearComponent > 0) {
output.opfields.noOfYrs = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(yearComponent, 0);
output.opfields.yearLabel = (yearComponent === 1) ? utils.getResourceString("yearLabel") : utils.getResourceString("yearsLabel");
}
if (monthComponent > 0) {
output.opfields.noOfMonths = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(monthComponent, 0);
output.opfields.monthLabel = (monthComponent === 1) ? utils.getResourceString("monthLabel") : utils.getResourceString("monthsLabel");
}
if (yearComponent > 99) {
return {
type: 'error', errorMsg: utils.getResourceString("FinancialCalculatorYearsExceededMsg").format(output.opfields.noOfYrs + " " +
output.opfields.yearLabel + " " +
output.opfields.noOfMonths + " " +
output.opfields.monthLabel)
};
}
var data = this.calculateChartAndTable(currentSavings, Math.ceil(totalPeriods), compoundingNMonths, savingPerPeriod, interestPerPeriod, totalPeriods);
output.firstTab = { data: this.scaleChart(data.chartData, compoundingNMonths, Math.ceil(totalPeriods)), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Chart_Click };
output.secondTab = { data: data.tableData, type: 'table' };
output.opfields.cumulativeInvestment = data.finalCumulativeInvestment;
output.opfields.interest = data.finalCumulativeInvestment;
return output;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
presentValueCalculateFunction: function presentValueCalculateFunction(inputFields, minValues) {
try {
var output = {};
output.opfields = {};
if (!inputFields.savingsGoal.length) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueEmptyMsg") };
}
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestEmptyMsg") };
}
if (!inputFields.monthlyInvestment.length) {
inputFields.monthlyInvestment = 0;
}
var noOfYears = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYrs);
var savingsGoal = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.savingsGoal);
if (savingsGoal < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.savingsGoal)) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueLessThanMinimumMsg").format(minValues.savingsGoal) };
}
var monthlyInvestment = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.monthlyInvestment);
if (savingsGoal < monthlyInvestment) {
return { type: 'error', errorMsg: utils.getResourceString("FutureValueLessThanMonthlyPaymentMsg") };
}
var interestPercent = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest);
if (interestPercent === 0) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestZeroMsg") };
}
var annualInterest = interestPercent / 100;
var compoundingNMonths = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.compounding);
var periodsPerYear = 12 / compoundingNMonths;
var totalPeriods = noOfYears * periodsPerYear;
var savingPerPeriod = monthlyInvestment * compoundingNMonths;
var savingsPerYear = savingPerPeriod * periodsPerYear;
var interestPerPeriod = annualInterest / periodsPerYear;
var savingGoalByPeriodicPayment = savingPerPeriod * ((Math.pow(1 + interestPerPeriod, totalPeriods) - 1) / interestPerPeriod);
var FVCurrentSaving = savingsGoal - savingGoalByPeriodicPayment;
var currentSavings = FVCurrentSaving / Math.pow((1 + interestPerPeriod), totalPeriods);
if (savingGoalByPeriodicPayment > savingsGoal) {
return { type: 'error', errorMsg: utils.getResourceString("PresentValueNegativeMsg").format(FinanceAppJS.Utils.Formatters.toolsNumberFormatter(currentSavings, 0)) };
}
var cumulativeInvestment = savingPerPeriod * totalPeriods + currentSavings;
var interestEarned = savingsGoal - cumulativeInvestment;
//result fields
output.opfields.LabelLine1 = utils.getResourceString("PVLabelLine1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(savingsGoal),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(monthlyInvestment),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYears, 0),
CollectionProvider.Formatters.formatPercentWithoutRounding(interestPercent)
);
output.opfields.LabelLine2 = utils.getResourceString("PVLabelLine2");
output.opfields.opfield1 = FinanceAppJS.Utils.Formatters.numberFormatter(currentSavings, 0);
output.opfields.fontTimeValue = currentSavings/ 1000;
output.opfields.cumulativeInvestment = FinanceAppJS.Utils.Formatters.numberFormatter(cumulativeInvestment, 0);;
output.opfields.interest = FinanceAppJS.Utils.Formatters.numberFormatter(interestEarned, 0);
var data = this.calculateChartAndTable(currentSavings, totalPeriods, compoundingNMonths, savingPerPeriod, interestPerPeriod);
output.firstTab = { data: this.scaleChart(data.chartData, compoundingNMonths, totalPeriods), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Chart_Click };
output.secondTab = { data: data.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialTVM_Schedule_Click };
return output;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
calculateChartAndTable: function calculateChartAndTable(currentSavings, noOfPeriods, compoundingNMonths, savingPerPeriod, interestPerPeriod, totalPeriodsForTable) {
//chart
var output = {};
var xlabels = [], currentSavingsPoints = [], totalSavingsPoints = [], IEPoints = [], CIPoints = [];
var month = 0, CI = 0, IE = 0, totalSavings = currentSavings;
for (var i = 1; i <= noOfPeriods; i++) {
var month = (i * compoundingNMonths);
var CI = currentSavings + (savingPerPeriod * i);
var IE = interestPerPeriod * totalSavings;
totalSavings = ((1 + interestPerPeriod) * totalSavings) + savingPerPeriod;
xlabels.push({ value: month, label: month });
currentSavingsPoints.push({ x: i, y: parseFloat(currentSavings.toFixed(2)) });
IEPoints.push({ x: i, y: parseFloat((totalSavings - CI).toFixed(2)) });
CIPoints.push({ x: i, y: parseFloat((CI - currentSavings).toFixed(2)) });
totalSavingsPoints.push({ x: i, y: totalSavings });
}
output.chartData = {
xaxis: {
name: utils.getResourceString("Calc_months"),
labels: xlabels
},
yaxis: {
name: "Amount",
},
data: [
{ color: FinanceAppJS.Calculator.BasePage.colorBar0, label: utils.getResourceString("PresentValue"), points: currentSavingsPoints },
{ color: FinanceAppJS.Calculator.BasePage.colorBar2, label: utils.getResourceString("CumulativeInvestment"), points: CIPoints },
{ color: FinanceAppJS.Calculator.BasePage.colorBar1, label: utils.getResourceString("CumulativeInterest"), points: IEPoints }
]
};
//table
output.tableData = [];
var unit, multiplier;
if (compoundingNMonths === 12) {
unit = "YearNumber";
multiplier = 1;
}
else {
unit = "MonthNumber";
multiplier = compoundingNMonths;
}
var headings = [unit, "CumulativeInvestment", "CumulativeInterest", "TotalSavings"];
output.tableData.push(headings.map(function (item) { return utils.getResourceString(item); }));
var max = totalPeriodsForTable ? totalPeriodsForTable : totalSavingsPoints.length;
for (var i = 0; i < max; i++) {
output.tableData.push([(i + 1) * multiplier, CIPoints[i].y + currentSavings, IEPoints[i].y, totalSavingsPoints[i].y]
.map(function (item, index) {
//var precision = (index === 0) ? 0 : 2;
return FinanceAppJS.Utils.Formatters.toolsNumberFormatter(item, 0);
}));
}
max = Math.ceil(max);
output.finalCumulativeInterest = IEPoints[max - 1].y;
output.finalCumulativeInvestment = CIPoints[max - 1].y + currentSavings;
return output;
},
ROICalculateFunction: function ROICalculateFunction(inputFields, minvalues) {
if (!inputFields.originalInvestment.length) {
return { type: 'error', errorMsg: utils.getResourceString("OriginalInvestmentEmptyMsg") };
}
if (!inputFields.investmentReturn.length) {
return { type: 'error', errorMsg: utils.getResourceString("InvestmentReturnEmptyMsg") };
}
var originalInvestment = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.originalInvestment);
if (originalInvestment === 0) {
return { type: 'error', errorMsg: utils.getResourceString("OriginalInvestmentZeroMsg") };
}
var investmentReturn = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.investmentReturn);
var changeColor, investmentChange, colors, investment;
if (originalInvestment < investmentReturn) {
changeColor = utils.getColorWithOpacity(utils.JsLessVariables(true).colorProfit, 50);
investmentChange = "InvestmentGain";
investment = "gain";
}
else if (originalInvestment > investmentReturn) {
changeColor = utils.getColorWithOpacity(utils.JsLessVariables(true).colorLoss, 50);
investmentChange = "InvestmentLoss";
investment = "lose";
}
else {
investmentChange = "";
}
var periodsPerYear = 12;
var noOfYrs = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYrs);
var noOfMonths = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfMonths);
if (noOfYrs === 0 && noOfMonths === 0) {
return { type: 'error', errorMsg: utils.getResourceString("YearsMonthsZeroMsg") };
}
var totalPeriods = (noOfYrs * periodsPerYear) + noOfMonths;
var tenure = noOfYrs + (noOfMonths / 12);
var SI = (investmentReturn - originalInvestment) / (originalInvestment * tenure);
var CI = Math.pow((investmentReturn / originalInvestment), 1 / totalPeriods) - 1;
var YCI = Math.pow((investmentReturn / originalInvestment), 1 / tenure) - 1;
var opfields = {}, output = {}, time = "", resourceKey;
if (noOfYrs > 0) {
if (noOfYrs === 1) {
resourceKey = "YearConcat";
}
else {
resourceKey = "YearsConcat";
}
time =  utils.getResourceString(resourceKey)
.format(FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYrs, 0));
}
if(noOfMonths > 0)
{
if (noOfMonths === 1) {
resourceKey = "MonthConcat";
}
else {
resourceKey = "MonthsConcat";
}
time += " " + utils.getResourceString(resourceKey)
.format(FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfMonths, 0));
}
opfields.LabelLine1 = utils.getResourceString("ROILabelLine1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(originalInvestment),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(investmentReturn),
time);
opfields.LabelLine2 = utils.getResourceString("ROILabelLine2");
opfields.LabelLine3 = utils.getResourceString("ROILabelLine3");
opfields.simpleROI = FinanceAppJS.Utils.Formatters.percentFormatter(SI * 100, 2);
opfields.compoundROI = FinanceAppJS.Utils.Formatters.percentFormatter(YCI * 100, 2);
opfields.fontsimpleROI = SI;
opfields.fontcompoundROI = YCI;
if (YCI >= (1e+19)) {
return {
type: 'error', errorMsg: utils.getResourceString("ROILabelLine2") + "<br/>" + opfields.simpleROI + "<br/><br/>" + utils.getResourceString("ROILabelLine3") + "<br/>" + opfields.compoundROI
};
}
output.opfields = opfields;
var monthlySI = SI / periodsPerYear;
var interestPerMonth = originalInvestment * monthlySI;
var CIPoints = [], IEPoints = [], xlabels = [];
for (var i = 1; i <= totalPeriods; i++) {
xlabels.push({ value: i, label: i});
CIPoints.push({x: i, y: originalInvestment});
IEPoints.push({x: i, y: interestPerMonth * i });
}
var value = originalInvestment;
var interestPM;
var compoundCIPoints = [], compoundIEPoints = [];
for (var i = 1; i <= totalPeriods; i++) {
interestPM = value * CI;
compoundIEPoints.push({ x: i, y: value - originalInvestment + interestPM });
value = value + interestPM;
}
var firstTabData = {
xaxis: {
name: utils.getResourceString("Calc_months"),
labels: xlabels
},
yaxis: {
name: "Amount",
},
data: [
{ color: FinanceAppJS.Calculator.BasePage.colorBar2, label: utils.getResourceString("OriginalInvestment"), points: CIPoints },
{ color: changeColor, label: utils.getResourceString(investmentChange), points: IEPoints, investment: investment }
],
};
output.firstTab = { data: this.scaleChart(firstTabData, 1, totalPeriods, 0), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialROI_Simple_Click};
var secondTabData = {
xaxis: {
name: utils.getResourceString("Calc_months"),
labels: xlabels
},
yaxis: {
name: "Amount",
},
data: [
{ color: FinanceAppJS.Calculator.BasePage.colorBar2, label: utils.getResourceString("OriginalInvestment"), points: CIPoints },
{ color: changeColor, label: utils.getResourceString(investmentChange), points: compoundIEPoints, investment: investment }
],                   
};
output.secondTab = { data: this.scaleChart(secondTabData, 1, totalPeriods, 1), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.FinancialROI_Compound_Click};
if (investmentReturn === 0) {
output.secondTab.data.type = 'error';
output.secondTab.data.errorMsg = utils.getResourceString('ROICompoundChartErrorMsg');
}
return output;
}
}, {
})
});
})();