﻿(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.Calculator", {
SavingsCalcPage: WinJS.Class.derive(FinanceAppJS.Calculator.BasePage, function (state) {
FinanceAppJS.Calculator.BasePage.call(this);
var commonChartOptions = { isSplitBar: true };
var dataObject = {
pageTitle: "Savings",
data: {
modes: [
{
title: "SavingsForGoal",
inputTemplate: "savingsGoalInputTemplate",
outputTemplate: "savingsOutputTemplate",
inputFields: ["savingsGoal", "currentSavings", "annualInterest", "compounding", "noOfYrs"],
chartOptions: commonChartOptions,
firstTabDisplayString: "Chart",
secondTabDisplayString: "Savings",
calculateFunction: this.savingsForGoalCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingsCalc_Calculate_Click,
navInstrumentId: FinanceAppJS.Constants.Instrumentation.SavingsCalc_Nav_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.SavingsCalc_PageView_Click,
precision: { annualInterest: 4 },
},
{
title: "SavingsEstimator",
inputTemplate: "savingsEstimatorInputTemplate",
outputTemplate: "savingsOutputTemplate",
inputFields: ["currentSavings", "monthlyInvestment", "annualInterest", "compounding", "noOfYrs"],
chartOptions: commonChartOptions,
firstTabDisplayString: "Chart",
secondTabDisplayString: "Savings",
calculateFunction: this.savingsEstimatorCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingEstimator_Calculate_Click,
navInstrumentId: FinanceAppJS.Constants.Instrumentation.SavingEstimator_Nav_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.SavingEstimator_PageView_Click,
precision: { annualInterest: 4 },
},
{
title: "TimeToAchieveGoal",
inputTemplate: "timeForGoalInputTemplate",
outputTemplate: "TimeForGoalOutputTemplate",
inputFields: ["currentSavings", "monthlyInvestment", "annualInterest", "compounding", "savingsGoal"],
chartOptions: commonChartOptions,
firstTabDisplayString: "Chart",
secondTabDisplayString: "Savings",
calculateFunction: this.timeForGoalCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingsTAG_Calculate_Click,
navInstrumentId: FinanceAppJS.Constants.Instrumentation.SavingsTAG_Nav_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.SavingsTAG_PageView_Click,
precision: { annualInterest: 4 },
},
]
}
};
this.setDataObject(dataObject);
}, {
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Tools/Savings Calculator";
},
//TODO: Commonise some part of the code and put it in a function
savingsForGoalCalculateFunction: function savingsForGoalCalculateFunction(inputFields, minValues) {
try {
var output = {};
output.opfields = {};
if (!inputFields.savingsGoal.length) {
return { type: 'error', errorMsg: utils.getResourceString("SavingsGoalEmptyMsg") };
}
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestEmptyMsg") };
}
if (!inputFields.currentSavings.length) {
inputFields.currentSavings = 0;
}
var noOfYears = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYrs);
var savingsGoal = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.savingsGoal);
if (savingsGoal < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.savingsGoal)) {
return { type: 'error', errorMsg: utils.getResourceString("SavingsGoalLessThanMinimumMsg").format(minValues.savingsGoal) };
}
var currentSavings = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentSavings);
if (savingsGoal < currentSavings) {
return { type: 'error', errorMsg: utils.getResourceString("SavingsGoalLessThanCurrentSavingsMsg") };
}
var compoundingNMonths = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.compounding);
var interestPercent = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest);
if (interestPercent === 0) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestZeroMsg") };
}
var annualInterest = interestPercent / 100;
var periodsPerYear = 12 / compoundingNMonths;
var totalPeriods = noOfYears * periodsPerYear;
var interestPerPeriod = annualInterest / periodsPerYear;
var FVCurrentSaving = currentSavings * Math.pow((1 + interestPerPeriod), totalPeriods);
if (savingsGoal < FVCurrentSaving) {
return { type: 'error', errorMsg: utils.getResourceString("CurrentSavingsSufficientMsg") };
}
var savingGoalByPeriodicPayment = savingsGoal - FVCurrentSaving;
var savingPerPeriod = savingGoalByPeriodicPayment / ((Math.pow((1 + interestPerPeriod), totalPeriods) - 1) / interestPerPeriod);
var monthlyInvestment = savingPerPeriod / compoundingNMonths;
var savingsPerYear = savingPerPeriod * periodsPerYear;
var cumulativeInvestment = savingPerPeriod * totalPeriods + currentSavings;
var interestEarned = savingsGoal - cumulativeInvestment;
//result fields
output.opfields.LabelLine1 = utils.getResourceString("MILabelLine1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(savingsGoal),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYears, 0),
CollectionProvider.Formatters.formatPercentWithoutRounding(interestPercent));
output.opfields.LabelLine2 = utils.getResourceString("MILabelLine2");
output.opfields.opfield1 = FinanceAppJS.Utils.Formatters.numberFormatter(monthlyInvestment,0);
output.opfields.cumulativeInvestment = FinanceAppJS.Utils.Formatters.numberFormatter(cumulativeInvestment,0);
output.opfields.interest = FinanceAppJS.Utils.Formatters.numberFormatter(interestEarned,0);
var data = this.calculateChartAndTable(currentSavings, totalPeriods, compoundingNMonths, savingPerPeriod, interestPerPeriod );
output.firstTab = { data: this.scaleChart(data.chartData, compoundingNMonths, totalPeriods), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingsCalc_Chart_Click };
output.secondTab = { data: data.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingsCalc_Schedule_Click };
return output;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
savingsEstimatorCalculateFunction: function savingsEstimatorCalculateFunction(inputFields, minValues) {
try {
var output = {};
output.opfields = {};
if ((!inputFields.monthlyInvestment.length) && (!inputFields.currentSavings.length)){
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentAndCurrentSavingsEmptyMsg") };
}
else if (!inputFields.monthlyInvestment.length) {
inputFields.monthlyInvestment = 0;
}
else if (!inputFields.currentSavings.length) {
inputFields.currentSavings = 0;
}
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestEmptyMsg") };
}
var noOfYears = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYrs);
var currentSavings = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentSavings);
var monthlyInvestment = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.monthlyInvestment);
if ((monthlyInvestment === 0) && (currentSavings === 0)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentAndCurrentSavingsZeroMsg") };
}
if (currentSavings === 0 && monthlyInvestment < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.monthlyInvestment)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentLessThanMinimumMsg").format(minValues.monthlyInvestment) };
}
var interestPercent = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest);
if (interestPercent === 0) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestZeroMsg") };
}
var annualInterest = interestPercent / 100;
var compoundingNMonths = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.compounding);
var periodsPerYear = 12 / compoundingNMonths;
var totalPeriods = noOfYears * periodsPerYear;
var savingPerPeriod = monthlyInvestment * compoundingNMonths;
var savingsPerYear = savingPerPeriod * periodsPerYear;
var interestPerPeriod = annualInterest / periodsPerYear;
var savingGoalByPeriodicPayment = savingPerPeriod * ((Math.pow(1 + interestPerPeriod, totalPeriods) - 1) / interestPerPeriod);
var FVCurrentSaving = currentSavings * Math.pow((1 + interestPerPeriod), totalPeriods);
var savingsGoal = savingGoalByPeriodicPayment + FVCurrentSaving;
var cumulativeInvestment = savingPerPeriod * totalPeriods + currentSavings;
var interestEarned = savingsGoal - cumulativeInvestment;
var formatSavingGoalValue = null;
//result fields
output.opfields.LabelLine1 = utils.getResourceString("TSLabelLine1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(monthlyInvestment),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYears, 0),
CollectionProvider.Formatters.formatPercentWithoutRounding(interestPercent));
output.opfields.LabelLine2 = utils.getResourceString("TSLabelLine2");
// Format Output Number precision.
formatSavingGoalValue = FinanceAppJS.Utils.Formatters.formatOutputNumber(savingsGoal);
if (formatSavingGoalValue && formatSavingGoalValue !== savingsGoal) {
output.opfields.opfield1 = formatSavingGoalValue;
}
else {
output.opfields.opfield1 = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(savingsGoal, 0);
}
var formatCI = FinanceAppJS.Utils.Formatters.formatOutputNumber(cumulativeInvestment);
if (formatCI && formatCI !== cumulativeInvestment) {
output.opfields.cumulativeInvestment = formatCI;
}
else {
output.opfields.cumulativeInvestment = FinanceAppJS.Utils.Formatters.numberFormatter(cumulativeInvestment, 0);
}
var formatIE = FinanceAppJS.Utils.Formatters.formatOutputNumber(interestEarned);
if (formatIE && formatIE !== interestEarned) {
output.opfields.interest = formatIE;
}
else {
output.opfields.interest = FinanceAppJS.Utils.Formatters.numberFormatter(interestEarned, 0);
}
if (savingsGoal > 999999999999999) {
return {
type: 'error', errorMsg: utils.getResourceString("TotalSavingsMsg").format(FinanceAppJS.Utils.Formatters.toolsNumberFormatter(savingsGoal, 0)) + "</br>" +
utils.getResourceString("CumulativeInvestmentMsg").format(output.opfields.cumulativeInvestment) +"</br>" +
utils.getResourceString("CumulativeInterestMsg").format(output.opfields.interest)
};
}
var data = this.calculateChartAndTable(currentSavings, totalPeriods, compoundingNMonths, savingPerPeriod, interestPerPeriod);
output.firstTab = { data: this.scaleChart(data.chartData, compoundingNMonths, totalPeriods), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingEstimator_Chart_Click };
output.secondTab = { data: data.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingEstimator_Schedule_Click };
return output;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
timeForGoalCalculateFunction: function timeForGoalCalculateFunction(inputFields, minValues) {
try {
var output = {};
output.opfields = {};
if (!inputFields.savingsGoal.length) {
return { type: 'error', errorMsg: utils.getResourceString("SavingsGoalEmptyMsg") };
}
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestEmptyMsg") };
}
if ((!inputFields.monthlyInvestment.length) && (!inputFields.currentSavings.length)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentAndCurrentSavingsEmptyMsg") };
}
else if (!inputFields.monthlyInvestment.length) {
inputFields.monthlyInvestment = 0;
}
else if (!inputFields.currentSavings.length) {
inputFields.currentSavings = 0;
}
var currentSavings = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentSavings);
var savingsGoal = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.savingsGoal);
if (savingsGoal < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.savingsGoal)) {
return { type: 'error', errorMsg: utils.getResourceString("SavingsGoalLessThanMinimumMsg").format(minValues.savingsGoal) };
}
var monthlyInvestment = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.monthlyInvestment);
if (savingsGoal < currentSavings) {
return { type: 'error', errorMsg: utils.getResourceString("SavingsGoalLessThanCurrentSavingsMsg") };
}
if (savingsGoal === currentSavings) {
return { type: 'error', errorMsg: utils.getResourceString("SavingsGoalEqualsCurrentSavingsMsg") };
}
if (savingsGoal < monthlyInvestment) {
return { type: 'error', errorMsg: utils.getResourceString("SavingsGoalLessThanMonthlyPaymentMsg") };
}
if ((monthlyInvestment === 0) && (currentSavings === 0)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentAndCurrentSavingsZeroMsg") };
}
if (currentSavings === 0 && monthlyInvestment < CollectionProvider.Formatters.parseNumberWithSeparator(minValues.monthlyInvestment)) {
return { type: 'error', errorMsg: utils.getResourceString("MonthlyPaymentLessThanMinimumMsg").format(minValues.monthlyInvestment) };
}
var interestPercent = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest);
if (interestPercent === 0) {
return { type: 'error', errorMsg: utils.getResourceString("AnnualInterestZeroMsg") };
}
var annualInterest = interestPercent / 100;
var compoundingNMonths = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.compounding);
var periodsPerYear = 12 / compoundingNMonths;
var interestPerPeriod = annualInterest / periodsPerYear;
var savingPerPeriod = monthlyInvestment * compoundingNMonths;
var totalPeriods = Math.log(((savingsGoal * interestPerPeriod) + savingPerPeriod) / ((currentSavings * interestPerPeriod) + savingPerPeriod)) / Math.log(1 + interestPerPeriod);
var noOfYears = totalPeriods / periodsPerYear;
var totalMonths = noOfYears * 12;
var yearComponent = Math.floor(totalMonths / 12);
var monthComponent = Math.ceil(totalMonths % 12);
if (monthComponent === 12) {
yearComponent += 1;
monthComponent = 0;
}
var ceilNoOfYears = Math.ceil(noOfYears);
var savingGoalByPeriodicPayment = savingPerPeriod * ((Math.pow(1 + interestPerPeriod, totalPeriods) - 1) / interestPerPeriod);
var FVCurrentSaving = currentSavings * Math.pow((1 + interestPerPeriod), totalPeriods);
var cumulativeInvestment = savingPerPeriod * totalPeriods + currentSavings;
var interestEarned = savingsGoal - cumulativeInvestment;
//result fields
output.opfields.LabelLine1 = utils.getResourceString("NYLabelLine1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(savingsGoal),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(monthlyInvestment),
CollectionProvider.Formatters.formatPercentWithoutRounding(interestPercent));
output.opfields.LabelLine2 = utils.getResourceString("NYLabelLine2");
output.opfields.noOfYrs = "";
output.opfields.noOfMonths = "";
output.opfields.yearLabel = "";
output.opfields.monthLabel = "";
if (yearComponent > 0) {
output.opfields.noOfYrs = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(yearComponent, 0);
output.opfields.yearLabel = (yearComponent === 1) ? utils.getResourceString("yearLabel") : utils.getResourceString("yearsLabel");
}
if (monthComponent > 0) {
output.opfields.noOfMonths = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(monthComponent, 0);
output.opfields.monthLabel = (monthComponent === 1) ? utils.getResourceString("monthLabel") : utils.getResourceString("monthsLabel");
}
if (yearComponent > 99) {
return { type: 'error', errorMsg: utils.getResourceString("YearsExceededMsg").format(output.opfields.noOfYrs + " " +
output.opfields.yearLabel + " " +
output.opfields.noOfMonths + " " +
output.opfields.monthLabel) };
}
var data = this.calculateChartAndTable(currentSavings, Math.ceil(totalPeriods), compoundingNMonths, savingPerPeriod, interestPerPeriod, totalPeriods);
output.firstTab = { data: this.scaleChart(data.chartData, compoundingNMonths, Math.ceil(totalPeriods)), type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingsTAG_Chart_Click};
output.secondTab = { data: data.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.SavingsTAG_Schedule_Click };
var endingData = data.tableData[data.tableData.length - 1];
output.opfields.cumulativeInvestment = data.finalCumulativeInvestment;
output.opfields.interest = data.finalCumulativeInterest;
return output;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
calculateChartAndTable: function calculateChartAndTable(currentSavings, noOfPeriods, compoundingNMonths, savingPerPeriod, interestPerPeriod, totalPeriodsForTable ) {
//chart
var output = {};
var xlabels = [], currentSavingsPoints = [], totalSavingsPoints = [], IEPoints = [], CIPoints = [];
var month = 0, CI = 0, IE = 0, totalSavings = currentSavings;
for (var i = 1; i <= noOfPeriods; i++) {
var month = (i * compoundingNMonths);
var CI = currentSavings + (savingPerPeriod * i);
var IE = interestPerPeriod * totalSavings;
totalSavings = ((1 + interestPerPeriod) * totalSavings) + savingPerPeriod;
xlabels.push({ value: month, label: month });
currentSavingsPoints.push({ x: i, y: parseFloat(currentSavings.toFixed(2)) });
IEPoints.push({ x: i, y: parseFloat((totalSavings - CI).toFixed(2)) });
CIPoints.push({ x: i, y: parseFloat((CI - currentSavings).toFixed(2)) });
totalSavingsPoints.push({ x: i, y: totalSavings });
}
output.chartData = {
xaxis: {
name: utils.getResourceString("Calc_months"),
labels: xlabels
},
yaxis: {
name: "Amount",
},
data: [
{ color: FinanceAppJS.Calculator.BasePage.colorBar0, label: utils.getResourceString("CurrentSavings"), points: currentSavingsPoints },
{ color: FinanceAppJS.Calculator.BasePage.colorBar2, label: utils.getResourceString("CumulativeInvestment"), points: CIPoints },
{ color: FinanceAppJS.Calculator.BasePage.colorBar1, label: utils.getResourceString("CumulativeInterest"), points: IEPoints }
]
};
//table
output.tableData = [];
var unit, multiplier;
if (compoundingNMonths === 12) {
unit = "YearNumber";
multiplier = 1;
}
else {
unit = "MonthNumber";
multiplier = compoundingNMonths;
}
var headings = [unit, "CumulativeInvestment", "CumulativeInterest", "TotalSavings"];
output.tableData.push(headings.map(function (item) { return utils.getResourceString(item); }));
var max = totalPeriodsForTable ? totalPeriodsForTable : totalSavingsPoints.length;
for (var i = 0; i < max; i++) {
output.tableData.push([(i + 1) * multiplier, CIPoints[i].y + currentSavings, IEPoints[i].y, totalSavingsPoints[i].y]
.map(function (item, index) {
//var precision = (index === 0) ? 0 : 2;
return FinanceAppJS.Utils.Formatters.toolsNumberFormatter(item, 0);
}));
}
max = Math.ceil(max);
output.finalCumulativeInterest = IEPoints[max -1].y;
output.finalCumulativeInvestment = CIPoints[max - 1].y + currentSavings;
return output;
},
}, {
})
});
})();