﻿(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.Calculator", {
AutoLoanPage: WinJS.Class.derive(FinanceAppJS.Calculator.BasePage, function (state) {
FinanceAppJS.Calculator.BasePage.call(this);            
var market = FinanceAppJS.Globalization.getRegion().codeThreeLetter;
if ( this.isTradeInSupported(market)) {
this.tradeInAmountDisplay = true;
}
var dataObject = {
pageTitle: "Loan Payments",
data: {
modes: [
{
title: "AutoLoan",
inputTemplate: "autoLoanInputTemplate",
extraInputBindingInfo: {tradeInAmountDisplay: this.tradeInAmountDisplay},
outputTemplate: "autoLoanOutputTemplate",
inputFields: ["carPrice", "downPayment", "loanFees", "tradeInAmount", "annualInterest", "noOfYears"],
firstTabDisplayString: utils.getResourceString("Chart"),
secondTabDisplayString: utils.getResourceString("Schedule"),
calculateFunction: this.autoLoanCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.AutoLoan_Calculate_Click,
navInstrumentId: FinanceAppJS.Constants.Instrumentation.AutoLoan_Nav_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.AutoLoan_PageView_Click,
precision: { annualInterest: 4 },
},                      
{
title: "AutoLease",
inputTemplate: "autoLeaseInputTemplate",
outputTemplate: "autoLeaseOutputTemplate",
inputFields: ["carPrice", "downPayment", "residualAmount", "annualInterest", "noOfYears"],
chartOptions: { isSplitBar: true },
firstTabDisplayString: utils.getResourceString("Chart"),
secondTabDisplayString: utils.getResourceString("Schedule"),
calculateFunction: this.autoLeaseCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.AutoLoan_Calculate_Click,
navInstrumentId: FinanceAppJS.Constants.Instrumentation.AutoLease_Nav_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.AutoLease_PageView_Click,
precision: { annualInterest: 4 },
},
/*     {
title: "EffectiveInterestRate",
inputTemplate: "effectiveInterestRateInputTemplate",
outputTemplate: "effectiveInterestRateOutputTemplate",
inputFields: ["loanAmount", "loanFees", "annualInterest", "noOfYears","compounding"],
chartOptions: { is3d: false, forceDraw: true, isSplitBar: false, colors: ["#0077b3", "#00283c"], legendParentProvided: true },
firstTabDisplayString: utils.getResourceString("Chart"),
secondTabDisplayString: utils.getResourceString("Schedule"),
calculateFunction: this.effectiveInterestRateCalculate.bind(this),
},*/
]
}            
};
this.setDataObject(dataObject);
}, {
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Tools/Auto Payments Calculator";
},
//TODO: Commonise some part of the code and put it in a function
tradeInAmountDisplay: false,
isTradeInSupported: function isTradeInSupported(market) {
//Todo: Confirm if this is required only for EN-US, else move it to AppConfig
if (market === "USA") {
return true;                
}
else {
return false;
}
},
computeChartTableData: function computeChartTableData( monthlyPayment, periodicInterest, actualLoanAmount, noOfPeriods ) {                                     
var xlabels = [];
var payments = [];
var endBalance = [];
var cumulativePayments = [];
var totalInterest = 0;
var startingBalance = 0;
var payment = 0;
var endingBalance = actualLoanAmount;
var interest = 0;
var result = {};
result.tableData = [];
var headings = ["MonthNumber", "StartingBalance", "Calc_interest", "Calc_payment", "Calc_endingbalance"];
result.tableData.push(headings.map(function (item) { return utils.getResourceString(item); }));
for (var i = 1; i <= noOfPeriods; i++) {
startingBalance = endingBalance;
interest = ( periodicInterest * startingBalance );
endingBalance = startingBalance + interest - monthlyPayment;
payment = payment + monthlyPayment;
totalInterest = totalInterest + interest;
result.tableData.push([
i,
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(startingBalance,0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(interest,0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(monthlyPayment,0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(endingBalance, 0)
]);                      
payments.push({
x:i,
y:payment
});
endBalance.push({
x:i,
y:endingBalance
});
xlabels.push({
value: i,
label:i
});
}
result.totalInterest = totalInterest;                    
result.chartData = {
xaxis: {
name: utils.getResourceString("Calc_months"),
labels: xlabels
},
data: [
{ color: FinanceAppJS.Calculator.BasePage.colorBar1, label: utils.getResourceString("Calc_endingbalance"), points: endBalance },
{ color: FinanceAppJS.Calculator.BasePage.colorBar2, label: utils.getResourceString("calc_labelCumulativePayment"), points: payments }                        
]
}
return result;
},
autoLoanCalculateFunction: function autoLoanCalculateFunction(inputFields, minFields) {
try {
var output = {};
output.opfields = {};
var carPriceMinValue = 0;
var annualInterestMinValue = 0;
if (!inputFields.carPrice.length) {
return { type: 'error', errorMsg: utils.getResourceString("carPriceReqd") };
}
var carPrice = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.carPrice);
if (carPrice <= carPriceMinValue ) {
return { type: 'error', errorMsg: utils.getResourceString("carPriceZero") + " " + utils.getResourceString("pleaseTryAgain") };
}
var downPayment = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.downPayment) ;
if(!downPayment){
downPayment = 0;
}
var loanFees = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.loanFees) || 0;
var tradeInAmount = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.tradeInAmount) || 0;
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("annualInterestRequired") };
}
var annualInterest = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest) / 100;
if (annualInterest <= annualInterestMinValue ) {
return { type: 'error', errorMsg: utils.getResourceString("annualInterestZero") + " " + utils.getResourceString("pleaseTryAgain") };
}
var noOfYears = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYears);
var actualLoanAmount = carPrice - downPayment - tradeInAmount + loanFees;
var noOfPeriods = 12 * noOfYears;
var periodicInterest = annualInterest / 12;
if (carPrice < tradeInAmount + downPayment) {
var errorMsgString = this.tradeInAmountDisplay ? utils.getResourceString("tradeInAmountMore") : utils.getResourceString("downPaymentMore")
return { type: 'error', errorMsg: errorMsgString };                        
}
if (carPrice === tradeInAmount + downPayment) {
var errorMsgString = this.tradeInAmountDisplay ? utils.getResourceString("tradeInAmountEqual") : utils.getResourceString("downPaymentEqual")
return { type: 'error', errorMsg: errorMsgString };
}
var monthlyPayment = (actualLoanAmount * periodicInterest)/(1 - Math.pow((1 + periodicInterest), (-1 * noOfPeriods)));                    
var overallPayment = monthlyPayment * noOfPeriods;
var interestPaid = overallPayment - actualLoanAmount;            
var chartTableData = this.computeChartTableData(monthlyPayment, periodicInterest, actualLoanAmount, noOfPeriods);
output.tableData = chartTableData.tableData;
output.chartData = chartTableData.chartData;
output.opfields.statement = utils.getResourceString("LoanStatement_1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(actualLoanAmount),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYears, 0),
CollectionProvider.Formatters.formatPercentWithoutRounding(annualInterest * 100));
output.opfields.monthlyPaymentText = utils.getResourceString("LoanStatement_2");
output.opfields.monthlyPaymentValue = monthlyPayment;
output.opfields.overallPaymentText = utils.getResourceString("LoanStatement_3");
output.opfields.overallPaymentValue = overallPayment;
output.opfields.interestPaidText = utils.getResourceString("LoanStatement_4");
output.opfields.interestPaidValue =(chartTableData.totalInterest);
output.chartData = this.scaleChart(output.chartData, 1, noOfPeriods);
output.firstTab = { data: output.chartData, type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.AutoLoan_Chart_Click };
output.secondTab = { data: output.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.AutoLoan_Schedule_Click };
return output;
}                   
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
autoLeaseCalculateFunction: function autoLeaseCalculateFunction(inputFields, minFields) {
try {
var output = {};
output.opfields = {};
output.tableData = [];
var carPriceMinValue = 0;
if (!inputFields.carPrice.length) {
return { type: 'error', errorMsg: utils.getResourceString("carPriceReqd") };
}
var carPrice = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.carPrice);
if (carPrice <= carPriceMinValue) {
return { type: 'error',  errorMsg: utils.getResourceString("carPriceZero") + " " + utils.getResourceString("pleaseTryAgain") };
}
var downPayment = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.downPayment) ? CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.downPayment):0;
var residualAmount = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.residualAmount) ? CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.residualAmount): 0;
var annualInterestMinValue = 0;
if (!inputFields.annualInterest.length) {
return { type: 'error', errorMsg: utils.getResourceString("annualInterestRequired") };
}
var annualInterest = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest) / 100;
if (annualInterest <= annualInterestMinValue) {
return { type: 'error', errorMsg: utils.getResourceString("annualInterestZero") + " " + utils.getResourceString("pleaseTryAgain") };
}
if (downPayment > carPrice) {
return { type: 'error', errorMsg: utils.getResourceString("downPaymentGreater") + " " + utils.getResourceString("pleaseTryAgain") };
}
if (downPayment === carPrice) {
return { type: 'error', errorMsg: utils.getResourceString("downPaymentCantBeEqual") + " " + utils.getResourceString("pleaseTryAgain") };
}
if (residualAmount > (carPrice - downPayment)) {
return { type: 'error', errorMsg: utils.getResourceString("residualAmountGreater") + " " + utils.getResourceString("pleaseTryAgain") }; 
}
if (residualAmount === (carPrice - downPayment)) {
return { type: 'error', errorMsg: utils.getResourceString("residualAmountEqual") + " " + utils.getResourceString("pleaseTryAgain") };
}
var noOfYears = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYears);
var actualLoanAmount = carPrice - downPayment;
var noOfPeriods = 12 * noOfYears;
var moneyFactor = annualInterest / 24; // formula - as per spec
var monthlyDepreciationFee = (actualLoanAmount - residualAmount) / noOfPeriods;
var monthlyLeaseFee = (actualLoanAmount + residualAmount) * moneyFactor;
var cumulativeDepreciationFee = 0;
var cumulativeLeaseFees = 0;
var totalMonthlyLeasePayment = 0;
var cumulativeLeaseFeesPoints = [];
var cumulativeDepreciationFeePoints = [];
var xlabels = [];
var headings = ["MonthNumber", "CumulativeDepreciationFee", "CumulativeLeaseFees", "CumulativePayments"];
output.tableData.push(headings.map(function (item) { return utils.getResourceString(item); }));
for (var i = 1; i <= noOfPeriods; i++) {
cumulativeLeaseFees = cumulativeLeaseFees + monthlyLeaseFee;
cumulativeDepreciationFee = cumulativeDepreciationFee + monthlyDepreciationFee;
totalMonthlyLeasePayment = cumulativeDepreciationFee + cumulativeLeaseFees;
output.tableData.push([
i,
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(cumulativeDepreciationFee, 0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(cumulativeLeaseFees, 0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(totalMonthlyLeasePayment, 0),
]);
cumulativeLeaseFeesPoints.push({
x: i,
y: cumulativeLeaseFees
});
cumulativeDepreciationFeePoints.push({
x: i,
y: cumulativeDepreciationFee
});
xlabels.push({
value: i,
label: i
});
}
var totalLeasePayment = totalMonthlyLeasePayment;
output.chartData = {
xaxis: {
name: utils.getResourceString("Calc_months"),
labels: xlabels
},
data: [
{ color: FinanceAppJS.Calculator.BasePage.colorBar1, label: utils.getResourceString("CumulativeLeaseFees"), points: cumulativeLeaseFeesPoints },
{ color: FinanceAppJS.Calculator.BasePage.colorBar2, label: utils.getResourceString("CumulativeDepreciation"), points: cumulativeDepreciationFeePoints }
]
}
output.opfields.statement1 = utils.getResourceString("LeaseStatement_1")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(carPrice),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(downPayment),
CollectionProvider.Formatters.formatPercentWithoutRounding(annualInterest * 100),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYears,0)
);
output.opfields.statement2 = utils.getResourceString("LeaseStatement_2");
output.opfields.totalMonthlyLeasePayment =(monthlyDepreciationFee + monthlyLeaseFee); 
output.opfields.totalPaymentText = utils.getResourceString("totalLeasePayments");
output.opfields.totalPaymentValue =(totalLeasePayment);
output.opfields.monthlyDepreciationText = utils.getResourceString("monthlyDepreciationText");
output.opfields.monthlyDepreciationFee =(monthlyDepreciationFee);
output.opfields.monthlyLeaseText = utils.getResourceString("monthlyLeaseText");
output.opfields.monthlyLeaseFee =(monthlyLeaseFee);
output.chartData = this.scaleChart(output.chartData, 1, noOfPeriods);
output.firstTab = { data: output.chartData, type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.AutoLease_Chart_Click };
output.secondTab = { data: output.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.AutoLease_Schedule_Click };
return output;
}                   
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
/*   effectiveInterestRateCalculate: function effectiveInterestRateCalculate(inputFields) {
var output = {};
output.opfields = {};
var loanAmount = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.loanAmount);
var loanFees = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.loanFees);
var annualInterest = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.annualInterest)/100;
var noOfYears = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.noOfYears);
var compoundingInterval = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.compounding);
var periodsPerYear = 12 / compoundingInterval;
var totalPeriods = noOfYears * periodsPerYear;
var interestPerPeriod = annualInterest / periodsPerYear;
var monthlyPayment = interestPerPeriod * loanAmount / (1 - Math.pow((1+interestPerPeriod),(-1*totalPeriods)));
var totalPayment = monthlyPayment * totalPeriods;
var interestPaid = totalPayment - loanAmount;
var effectiveInterestRate = 5.0; //hardcoding till IRR comes.
output.tableData = [];
var headings = ["MonthNumber", "StartingBalance", "Calc_interest", "Calc_payment", "Calc_endingbalance"];
output.tableData.push(headings.map(function (item) { return utils.getResourceString(item); }));
var xlabels = [];
var payments = [];
var endBalance = [];
var cumulativePayments = [];
var totalInterest = 0;
var startingBalance = 0;
var payment = 0;
var endingBalance = loanAmount;
var interest = 0;
for (var i = 1; i <= totalPeriods; i++) {
startingBalance = endingBalance;
interest = (interestPerPeriod * startingBalance);
endingBalance = startingBalance + interest - monthlyPayment;
payment = payment + monthlyPayment;
totalInterest = totalInterest + interest;
output.tableData.push([
i,
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(startingBalance, 2),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(interest, 2),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(monthlyPayment, 2),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(endingBalance, 2)
]);
payments.push({
x: i,
y: payment
});
endBalance.push({
x: i,
y: endingBalance
});
xlabels.push({
value: i ,
label: i
});
}
output.chartData = {
xaxis: {
name: "Months",
labels: xlabels
},
yaxis: {
name: "Amount"
},
data: [
{ color: "#D32020", label: "Payments", points: payments },
{ color: "#74B005", label: "EndingBalance", points: endBalance }
]
}
output.chartData = this.scaleChart(output.chartData, 1, totalPeriods);
output.opfields.statement = utils.getResourceString("EffectiveInterestStatement_1")
.format(FinanceAppJS.Utils.Formatters.toolsNumberFormatter( loanAmount, 2),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter( loanFees , 2),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(noOfYears, 0),
FinanceAppJS.Utils.Formatters.percentFormatter(annualInterest)
);
output.opfields.effectiveInterestRate = FinanceAppJS.Utils.Formatters.percentFormatter(effectiveInterestRate, 2);
output.opfields.monthlyPaymentText = utils.getResourceString("LoanStatement_2");
output.opfields.monthlyPaymentValue = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(monthlyPayment, 2);
output.opfields.interestPaidText = utils.getResourceString("LoanStatement_4");
output.opfields.interestPaidValue = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(totalInterest, 2);
output.firstTab = { data: output.chartData, type: 'chart'};
output.secondTab = { data: output.tableData, type: 'table' };
return output;
},*/
}, {
})
});
})();