﻿(function () {
"use strict";   
WinJS.Namespace.define("FinanceAppJS.Calculator", {
MortgagePage: WinJS.Class.derive(FinanceAppJS.Calculator.BasePage, function (state) {
var selector = document.getElementById("noOfYears");            
FinanceAppJS.Calculator.BasePage.call(this);
var dataObject = {
pageTitle: "Mortgage",
data: {
modes: [
{
title: "MortgageCalculator",
inputTemplate: "inputTemplate",
outputTemplate: "outputTemplate",
inputFields: ["loanAmount", "loanFees", "annualInterest","noOfYears"],
firstTabDisplayString: utils.getResourceString("Chart"),
secondTabDisplayString: utils.getResourceString("Schedule"),
calculateFunction: this.mortgageLoanCalculate.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.MortgageCalc_Calculate_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.MortgageCaluclatorPage,
precision: { annualInterest: 4 }
}
]
}
};
this.setDataObject(dataObject);
}, {
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Tools/Mortgage Calculator";
},
getTableData: function getTableData(payments) {
var tableData = [];
var headings = ["MonthNumber", "StartingBalance", "Calc_interest", "Calc_payment", "Calc_endingbalance"];
tableData.push(headings.map(function (item) { return utils.getResourceString(item); }));        
for (var index = 0; index < payments.length; index++) {
var payment = payments[index];
var precision =  0 ;
tableData.push([
index + 1,
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(payment.BeginBalance, precision),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(payment.Interest, precision),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(payment.Payment, precision),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(payment.EndBalance, precision)
]);
}
return tableData;
},
mortgageLoanCalculate: function mortgageLoanCalculate(ipfields, minFields) {
try {
var obj = {};
obj.opfields = {};
var loanAmountMinValue = 0;
if (!ipfields.loanAmount.length ) {
return { type: 'error', errorMsg: utils.getResourceString("loanAmountRequired")};
}
var amount = CollectionProvider.Formatters.parseNumberWithSeparator(ipfields.loanAmount);
if (amount <= loanAmountMinValue) {
return { type: 'error', errorMsg: utils.getResourceString("loanAmountZero") + " " + utils.getResourceString("pleaseTryAgain") };
}
var fees = ipfields.loanFees ? CollectionProvider.Formatters.parseNumberWithSeparator(ipfields.loanFees) : 0;
amount = amount + fees;
if ( !ipfields.annualInterest.length ) {
return { type: 'error', errorMsg: utils.getResourceString("annualInterestRequired") };
}
var minRate = 0;
var rate = CollectionProvider.Formatters.parseNumberWithSeparator(ipfields.annualInterest);
if (rate <= minRate) {
return { type: 'error', errorMsg: utils.getResourceString("annualInterestZero") + " " + utils.getResourceString("pleaseTryAgain") };
}
var months = CollectionProvider.Formatters.parseNumberWithSeparator(noOfYears.value) * 12;
var ratepm = rate / 1200.0;
var monthlyPayment = (ratepm * amount) / (1 - Math.pow(1 + ratepm, -1 * months));
var overallPayment = monthlyPayment * months;
var cumulative = 0;
var beginBalance = amount;
var endingBalancePoints = [];
var cumulativePaymentPoints = [];
var payments = [];
var totalInterest = 0;
for (var i = 0; i < months && beginBalance > 0; i++) {
var payment = Math.min(beginBalance * (1 + ratepm), monthlyPayment);
var interest = ratepm * beginBalance;
var principle = payment - interest;
var endBalance = beginBalance - principle;
totalInterest += interest;
cumulative += payment;
payments.push({
BeginBalance: beginBalance,
Payment: payment,
Interest: interest,
Principle: principle,
EndBalance: endBalance,
Cumulative: cumulative
});
beginBalance = endBalance;
}
for (i = 0 ; i < payments.length; i++) {
endingBalancePoints.push({
x: Math.floor(i),
y: payments[i].EndBalance
});
cumulativePaymentPoints.push({
x: Math.floor(i),
y: payments[i].Cumulative
});
}
var xlabels = [];
for (var k = 1; k <= payments.length; k++) {
xlabels.push({ value: k, label: k });
}
var temp = [];
obj.chartData = {
xaxis: {
name: utils.getResourceString("Calc_months"),
labels: xlabels
},
data: [
{ color: FinanceAppJS.Calculator.BasePage.colorBar1, label: utils.getResourceString("Calc_endingbalance"), points: endingBalancePoints },
{ color: FinanceAppJS.Calculator.BasePage.colorBar2, label: utils.getResourceString("calc_labelCumulativePayment"), points: cumulativePaymentPoints }
]
};
obj.chartData = this.scaleChart(obj.chartData, 1, months);
obj.tableData = this.getTableData(payments);
var formatMonthlyPaymentValue = null;
obj.opfields.statement = utils.getResourceString("MortgageLoanStatement")
.format(FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(amount - fees),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(months / 12, 0),
CollectionProvider.Formatters.formatPercentWithoutRounding(rate));
obj.opfields.monthlyPaymentText = utils.getResourceString("LoanStatement_2");
// Format Output Number precision.
formatMonthlyPaymentValue = FinanceAppJS.Utils.Formatters.formatOutputNumber(monthlyPayment);
if (formatMonthlyPaymentValue && formatMonthlyPaymentValue !== monthlyPayment) {
obj.opfields.monthlyPaymentValue = formatMonthlyPaymentValue;
}
else {
obj.opfields.monthlyPaymentValue = FinanceAppJS.Utils.Formatters.toolsNumberFormatter(monthlyPayment, 0);
}
obj.opfields.overallPaymentText = utils.getResourceString("LoanStatement_3");
obj.opfields.overallPaymentValue = overallPayment;
obj.opfields.interestPaidText = utils.getResourceString("LoanStatement_4");
obj.opfields.interestPaidValue = totalInterest;
obj.firstTab = { data: obj.chartData, type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.MortgageCalc_Chart_Click };
obj.secondTab = { data: obj.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.MortgageCalc_Schedule_Click };
return obj;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
}
},
{
})
});
})();