﻿(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.Calculator", {
RetirementCalcPage: WinJS.Class.derive(FinanceAppJS.Calculator.BasePage, function (state) {
FinanceAppJS.Calculator.BasePage.call(this);
var dataObject = {
pageTitle: "RetirementPlanner",
data: {
modes: [
{
title: "RetirementPlanner",
inputTemplate: "retirementInputTemplate",
outputTemplate: "retirementOutputTemplate",
inputFields: ["currentIncome", "currentSavings", "retirementIncome", "currentAge", "retirementAge", "lifeExpectancy","investReturnPre","investReturnPost"],
firstTabDisplayString: utils.getResourceString("Chart"),
secondTabDisplayString: utils.getResourceString("Schedule"),
calculateFunction: this.retirementCalculateFunction.bind(this),
instrumentationId: FinanceAppJS.Constants.Instrumentation.RetirementCalc_Calculate_Click,
pageViewId: FinanceAppJS.Constants.Instrumentation.RetirementCalc_PageView_Click,
precision: { investReturnPre: 4, investReturnPost: 4, currentAge: 0, retirementAge: 0, lifeExpectancy: 0 }
}
]
}
};
this.setDataObject(dataObject);
}, {
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Tools/Retirement Calculator";
},
//TODO: Commonise some part of the code and put it in a function
retirementCalculateFunction: function retirementCalculateFunction(inputFields, minFields) {
try {
var output = {};
output.opfields = {};
if (!inputFields.currentIncome.length && !inputFields.currentSavings.length) {
return { type: 'error', errorMsg: utils.getResourceString("incomeSavingsBlank") };
}
var currentIncome = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentIncome) ? CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentIncome) : 0;
var currentSavings = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentSavings) ? CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentSavings) : 0;
if (currentIncome == 0 && currentSavings == 0) {
return { type: 'error', errorMsg: utils.getResourceString("incomeSavingsZero") + " " + utils.getResourceString("pleaseTryAgain") };
}
if (!inputFields.retirementIncome.length ) {
return { type: 'error', errorMsg: utils.getResourceString("annualIncomeRequired")};
}
var retirementAnnualIncome = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.retirementIncome);
var retirementAnnualIncomeMinValue = 0; //TODO: get these from json
if (retirementAnnualIncome <= retirementAnnualIncomeMinValue) {
return { type: 'error', errorMsg: utils.getResourceString("annualIncomeZero") + " " + utils.getResourceString("pleaseTryAgain") };
}
if (!inputFields.currentAge.length) {
return { type: 'error', errorMsg: utils.getResourceString("currentAgeRequired")}; 
}
var currentAge = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.currentAge);
var minCurrentAge = 0;
if (currentAge <= minCurrentAge) {
return { type: 'error', errorMsg: utils.getResourceString("currentAgeZero") + utils.getResourceString("pleaseTryAgain")}; 
}
if (!inputFields.retirementAge.length) {
return { type: 'error', errorMsg: utils.getResourceString("retirementAgeRequired") }; 
}
var retirementAge = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.retirementAge);
var minRetirementAge = 0;
if (retirementAge <= minRetirementAge) {
return { type: 'error', errorMsg: utils.getResourceString("retirementAgeZero") + utils.getResourceString("pleaseTryAgain") }; 
}
if (retirementAge <= currentAge) {
return { type: 'error', errorMsg: utils.getResourceString("retirementAgeLess") }; 
}
if (!inputFields.lifeExpectancy.length) {
return { type: 'error', errorMsg: utils.getResourceString("lifeExpectancyRequired") };  
}
var lifeExpectancy = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.lifeExpectancy);
if (lifeExpectancy <= retirementAge) {
return { type: 'error', errorMsg: utils.getResourceString("lifeExpectancyGreater") }; 
}
var yearsToRetire = retirementAge - currentAge;
var yearsInRetirement = lifeExpectancy - retirementAge;
if (!inputFields.investReturnPre.length) {
return { type: 'error', errorMsg: utils.getResourceString("preRetirementRequired")};
}
var investReturnPre = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.investReturnPre) / 100;
var minInvestReturnPre = 0;
if (investReturnPre <= minInvestReturnPre) {
return { type: 'error', errorMsg: utils.getResourceString("preRetirementZero") + " " + utils.getResourceString("pleaseTryAgain")}; 
}
if (!inputFields.investReturnPost.length) {
return { type: 'error', errorMsg: utils.getResourceString("postRetirementRequired")};
}
var investReturnPost = CollectionProvider.Formatters.parseNumberWithSeparator(inputFields.investReturnPost) / 100;
var minInvestReturnPost = 0;
if (investReturnPost <= minInvestReturnPost) {
return { type: 'error', errorMsg: utils.getResourceString("postRetirementZero") + " " + utils.getResourceString("pleaseTryAgain")};  
}
var futureValueOfAnnuity = (retirementAnnualIncome * (Math.pow(1 + investReturnPost, yearsInRetirement) - 1)) / investReturnPost;
var presentValueOfRetirementAmount = futureValueOfAnnuity / (Math.pow(1 + investReturnPost, yearsInRetirement));
var futureValueOfCurrentSavings = currentSavings * Math.pow(1 + investReturnPre, yearsToRetire);
var amountToBeSaved = presentValueOfRetirementAmount - futureValueOfCurrentSavings;
var regularYearlyPayment = amountToBeSaved /((Math.pow(1+investReturnPre,yearsToRetire)-1) / investReturnPre);
if (amountToBeSaved <= 0) {
return { type: 'error', errorMsg: utils.getResourceString("currentSavingsEnough")};  
}
if (regularYearlyPayment > currentIncome) {
return { type: 'error', errorMsg: utils.getResourceString("currentIncomeInsufficient").format(FinanceAppJS.Utils.Formatters.toolsNumberFormatter(regularYearlyPayment, 0))}; 
}
var savings = [];
var retirementIncome = [];
var xlabels = [];
var cumulativeInvestment = currentSavings, interestEarned;
var totalValue = currentSavings;
output.tableData = [];
var headings = ["ChartYear", "cumulativeSavings", "Calc_interest", "retirementIncome", "Calc_endingbalance"].map(utils.getResourceString);
output.tableData.push(headings.map(function (item) { return item; }));
output.tableData.push([
currentAge,
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(cumulativeInvestment, 0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(interestEarned, 0),
"-",
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(totalValue, 0)
]);
for (var i = 1; i <= yearsToRetire; i++) {
interestEarned = investReturnPre * totalValue;
cumulativeInvestment = cumulativeInvestment + regularYearlyPayment;
totalValue = totalValue + interestEarned + regularYearlyPayment;
output.tableData.push([
currentAge + i,
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(cumulativeInvestment, 0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(interestEarned, 0),
"-",
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(totalValue, 0)
]);
savings.push({
x: currentAge + i,
y: totalValue
});
retirementIncome.push({
x: currentAge + i,
y: 0
});
xlabels.push({
value: i,
label: currentAge + i,
});
}
var totalSavings = totalValue;
cumulativeInvestment = totalValue;
interestEarned = cumulativeInvestment * investReturnPost;
totalValue = cumulativeInvestment + interestEarned - retirementAnnualIncome;
var limit = yearsToRetire + yearsInRetirement;
for (i = yearsToRetire + 1 ; i <= limit ; i++) {
output.tableData.push([
currentAge + i,
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(cumulativeInvestment, 0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(interestEarned, 0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(retirementAnnualIncome, 0),
FinanceAppJS.Utils.Formatters.toolsNumberFormatter(totalValue, 0)
]);
savings.push({
x: currentAge + i,
y: totalValue
});
retirementIncome.push({
x: currentAge + i,
y: retirementAnnualIncome
});
xlabels.push({
value: i,
label: currentAge + i,
});
cumulativeInvestment = totalValue;
interestEarned = cumulativeInvestment * investReturnPost;
totalValue = cumulativeInvestment + interestEarned - retirementAnnualIncome;
}
output.chartData = {
xaxis: {
name: utils.getResourceString("YrsLabel"),
labels: xlabels
},
data: [
{ color: FinanceAppJS.Calculator.BasePage.colorBar1, label: utils.getResourceString("Savings"), points: savings },
{ color: FinanceAppJS.Calculator.BasePage.colorBar2, label: utils.getResourceString("RetirementIncome"), points: retirementIncome }
]
}
output.chartData = this.scaleChart(output.chartData, 12, limit);
if (currentIncome !== 0) {
output.opfields.statement1 = utils.getResourceString("retirementStatement1")
.format(FinanceAppJS.Utils.Formatters.numberFormatter(retirementAge, 0),
FinanceAppJS.ConverterFunc.formatValueAndAddCcySymbol(retirementAnnualIncome));
output.opfields.statement2 = utils.getResourceString("retirementStatement2")
.format(FinanceAppJS.Utils.Formatters.percentFormatter((regularYearlyPayment / currentIncome) * 100, 2));
output.opfields.statement2_value = regularYearlyPayment;
}
else {
output.opfields.statement1 = "";
output.opfields.statement2 = "";
output.opfields.statement2_value = "";
}
output.opfields.statement3 = utils.getResourceString("retirementStatement3");
output.opfields.retirementSavingAmount =totalSavings;                    
output.firstTab = { data: output.chartData, type: 'chart', instrumentationId: FinanceAppJS.Constants.Instrumentation.RetirementCalc_Chart_Click };
output.secondTab = { data: output.tableData, type: 'table', instrumentationId: FinanceAppJS.Constants.Instrumentation.RetirementCalc_Schedule_Click };
return output;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return { type: 'error' };
}
},
}, {
})
});
})();