﻿(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS", {
ETF: WinJS.Class.derive(FinanceAppJS.FinanceBasePage, function (state) {
/// <summary>
/// Constructor for a simple IPage class that controls the FundFilmStrip
/// </summary>
/// <param name="state">
/// The state object passed to WinJS.Navigation.navigate or the state object
/// preserved when the page has been navigated away from
/// </param>
this.showDisclaimer = false;
FinanceAppJS.FinanceBasePage.call(this, state);
this.pageState = { Sym: state.entity.Sym, DisplaySymbol: utils.getAutoSuggestDisplaySymbol(state.entity), Title: state.entity.Cmp, E1: state.entity.E1, E2: state.entity.E2, St: state.entity.St };
this.param = utils.getGlobalInstrumentSymbol(this.pageState);
this.isEOD = utils.isEODInstrument(this.param);
var exchangeConfig = FinanceAppJS.Globalization.getExchangeConfig(state.entity.E2);
if (exchangeConfig && exchangeConfig.countryCode) {
this.showDisclaimer = utils.showBATSDisclaimerForCountry(exchangeConfig.countryCode);
}
this.quotesDelay = exchangeConfig ? exchangeConfig.delay : null;
PlatformJS.Utilities.getControl('ETFEntityMainList').title = this.pageState.Title;
var defaultNoDataDisplayValue = FinanceAppJS.Constants.Funds.DefaultNoDataDisplayValue;
var clusterDefinitions = {
overview: {
clusterTitle: this.pageState.DisplaySymbol,
clusterControl: "FinanceAppJS.EtfDetailsPage.EtfOverviewCluster",
dataProviders: {
ETFDataProvider: { entities: this.param,  refreshInterval: this.isEOD ? 0 : FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData },
ETFChartDataProvider: { instrumentType: "etf", stockSymbol: this.param, isEOD: this.isEOD, duration: this.isEOD ? "1m" : "1d", refreshInterval: this.isEOD ? 0 : FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData },
},
primaryProviders: "ETFDataProvider",
clusterOptions: {
isEOD: this.isEOD,
instrumentType: "etf",
moduleInfo: {
height: "100%",
width: "auto",
fragmentPath: "/pages/ETF/html/ETFTemplates.html",
templateId: "overview",
}
}
},
keyStats: {
clusterTitle: utils.getResourceString("KeyStats"),
clusterControl: "FinanceAppJS.Controls.ModuleClusterControl",
dataProviders: { ETFDataProvider: { entities: this.param } },
dataProviderOptions: { entities: this.param },
postDataValidateFunc: function postDataValidateFunc(data) {
return data && data.peers &&
(["Fyr", "Rfyc", "Erj", "Erc", "Rj", "Rc", "Nc", "Nj", "Ej", "Ec", "Dj", "Dc", "Cj", "Cc", "Terr", "Cter"].some(function (k) { return data.peers[k] && data.peers[k] !== defaultNoDataDisplayValue; }));
},
clusterOptions: {
dataKey: "peers",
moduleInfo: {
fragmentPath: "/pages/ETF/html/ETFTemplates.html",
templateId: "peercluster",
}
}
},
fundDetails: {
clusterTitle: utils.getResourceString("ETFFundDet"),
dataProviders: { ETFDataProvider: { entities: this.param } },
clusterControl: "FinanceAppJS.Controls.ModuleClusterControl",
postDataValidateFunc: function postDataValidateFunc(data) {
return data && data.profile &&
(["Sn", "Ff", "Is", "Fm", "In", "Incd"].some(function (k) { return data.profile[k] && data.profile[k] !== defaultNoDataDisplayValue; }));
},
clusterOptions: {
dataKey: "profile",
moduleInfo: {
fragmentPath: "/pages/ETF/html/ETFTemplates.html",
templateId: "profilecluster",
}
}
},
mrating: {
clusterTitle: utils.getResourceString("ProviderRating"),
clusterControl: "FinanceAppJS.Controls.ModuleClusterControl",
dataProviders: { ETFDataProvider: { entities: this.param } },
postDataValidateFunc: function postDataValidateFunc(data) {
return data && data.rating &&
(["Og", "Tyg", "Fyg", "Nyg"].some(function (k) { return data.rating[k]; }) &&
["Rtyr", "Rfyr", "Rnyr", "Or", "Tyk", "Fyk"].some(function (k) { return data.rating[k] !== defaultNoDataDisplayValue; }));
},
clusterOptions: {
dataKey: "rating",
moduleInfo: {
fragmentPath: "/pages/FundDetails/html/FundDetailsTemplates.html",
templateId: "ratingcluster",
}
}
},
topHoldings: {
clusterTitle: utils.getResourceString("ETFTop10"),
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: { ETFDataProvider: { entities: this.param } },
postDataValidateFunc: function postDataValidateFunc(data) { return data && data.holdings; },
clusterOptions: {
className: "holdingsTable",
footerKey: "portfolioDate",
dataKey: "holdings",
onrendercomplete: function onrendercomplete() {
// Attach cluster with Arrow key navigation - scroll event.
FinanceAppJS.Utils.Navigator.enableArrowKeyAccessibilty();
}
}
}
};
this.setClusterDefinitions(clusterDefinitions);
this.pageTitle = this.pageState.Title;
this.roaming = new FinanceAppJS.Roaming;
}, {
isEOD: null,
pinUnpinManager: null,
onBindingComplete: function onBindingComplete() {
/// <summary>
/// This function is invoked by the UX Framework when the data binding is complete
/// </summary>
/// <returns>
/// Nothing
/// </returns>
FinanceAppJS.FinanceBasePage.prototype.onBindingComplete.call(this);
// Setup actions for bottom edgy
var btmEdgy = document.querySelector("#platformActionEdgy");
this.btnManage = btmEdgy.querySelector("#btnETF");
var isRoamingEntryExists = this.roaming.isWatchListEntryExists(this.param, FinanceAppJS.Constants.IDs.Etf);
WatchListManage.toggleEdgyLabels(isRoamingEntryExists, this.btnManage);
var options = {
symbol: this.param,
type: FinanceAppJS.Constants.IDs.Etf,
isEntryExists: isRoamingEntryExists,
elm: this.btnManage,
actionBar: btmEdgy,
name: this.pageState.Title,
roamingObj: this.roaming
};
this.edgyBtnClickHandler = WatchListManage.addOrRemoveFromEdgy.bind(null, options);
this.btnManage.addEventListener("click", this.edgyBtnClickHandler);
btmEdgy.style.display = 'block';
this.pinUnpinManager = new FinanceAppJS.PinUnpinManager(btmEdgy, this.getPageState(), FinanceAppJS.Constants.IDs.Etf, this.param, this.pageState.Title, this.pageState.DisplaySymbol, null, null, FinanceAppJS.Constants.Instrumentation.PinStart_ETF_Click);
},
onNavigateAway: function onNavigateAway(reason) {
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away
/// </summary>
/// <param name="reason">
/// The reason why the page is being navigated away: "backNavigation" or "newNavigation"
/// </param>
/// <returns>
/// Nothing
/// </returns>
FinanceAppJS.FinanceBasePage.prototype.onNavigateAway.call(this);
var chartControlDiv = document.querySelector(".fundChart");
if (chartControlDiv) {
var chartControl = chartControlDiv.winControl;
if (chartControl) {
chartControl.dispose();
}
}
if (this.pinUnpinManager) {
this.pinUnpinManager.clean();
}
if (this.btnManage && this.edgyBtnClickHandler) {
this.btnManage.removeEventListener("click", this.edgyBtnClickHandler);
this.btnManage = null;
}
},
onVisibilityChanged: function onVisibilityChanged() {
if (document.msVisibilityState === "visible" && this.pinUnpinManager) {
this.pinUnpinManager.updateBottomSwipyPinButtons(this.param, FinanceAppJS.Constants.IDs.Etf);
}
},
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/ETF details page";
},
getCurrentContent: function getCurrentContent() {
var content = {
Symbol: this.param,
Page: FinanceAppJS.Telemetry.Constants.Page.ETF,
};
return content;
},
onPageLoadComplete: function onPageLoadComplete() {
FinanceAppJS.Telemetry.FlightRecorder.LogSearchEventFromResultPage();
FinanceAppJS.FinanceBasePage.prototype.onPageLoadComplete.call(this);
var content = this.getCurrentContent();
FinanceAppJS.Telemetry.FlightRecorder.LogContentView(content);
},
/// <summary>
/// This function is invoked when dpi changes ("Make everything on your screen bigger" setting)
/// </summary>
/// <param name="event">
/// Dpi change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onDpiChange: function onDpiChange() {
FinanceAppJS.FinanceBasePage.prototype.onDpiChange.call(this);
},
getSharedContractData: function getSharedContractData() {
var that = this;
var dataProvider = this.getDataProvider("ETFDataProvider");
var data = dataProvider._lastFetchData;
var frmtLastUpdatedTime = utils.formatLastUpdatedTime(this._lastUpdatedTime);
var sharedContractDataPromise;
if (data) {
var etfEntityData = {
timeAttribution: utils.getResourceString('ShareContractTimeAttribution').format(frmtLastUpdatedTime),
dataAttribution: utils.getResourceString('ShareContractDataAttribution'),
source: FinanceAppJS.Constants.IDs.NonIndices,
entities: [{
type: FinanceAppJS.Constants.IDs.Etf,
header: this.pageState.Title,
cmp: this.pageState.Title,
sym: this.pageState.DisplaySymbol,
lp: data.overview.Lp,
changeStatus: data.overview.changeStatus,
ch: data.overview.Ch,
chp: data.overview.Chp
}]
};
var uriBuilder = new Platform.Utilities.AppExUriBuilder();
uriBuilder.controllerId = "application";
uriBuilder.commandId = "view";
uriBuilder.queryParameters.insert("entitytype", "etf");
uriBuilder.queryParameters.insert("sym", this.pageState.Sym);
uriBuilder.queryParameters.insert("eqsm", this.pageState.Eqsm);
uriBuilder.queryParameters.insert("e1", this.pageState.E1);
uriBuilder.queryParameters.insert("e2", this.pageState.E2);
uriBuilder.queryParameters.insert("st", this.pageState.St);
uriBuilder.queryParameters.insert("cmp", this.pageState.Title);
var uri = uriBuilder.toString();
etfEntityData.deepLink = uri;
sharedContractDataPromise = new WinJS.Promise(function (complete, error) {
utils.getShareContractHtmlFormat(etfEntityData).then(function (htmlOutput) {
complete({
title: utils.getShareContractSubject(that.pageState.Title),
htmlContent: htmlOutput,
textContent: utils.getShareContractTextFormat(etfEntityData),
uri: uri
});
});
});
}
else {
sharedContractDataPromise = new WinJS.Promise.wrapError({});
}
return sharedContractDataPromise;
}
},
{
areStateEqual: function areStateEqual(firstState, secondState) {
var firstItem = firstState.entity;
var secondItem = secondState.entity;
if (firstItem && secondItem) {
return utils.getGlobalInstrumentSymbol(firstItem) === utils.getGlobalInstrumentSymbol(secondItem);
}
return false;
}
})
});
})();
(function () {
'use strict';
var FundDetailsPage = WinJS.Namespace.define("FinanceAppJS.EtfDetailsPage",
{
/// <summary>
/// Cluster with Chart as view for stock quotes
/// </summary>
EtfOverviewCluster: WinJS.Class.derive(FinanceAppJS.Controls.BaseOverviewCluster, function (domElement, options) {
this._rangeSelectors = options.isEOD ? ['1m', '1y', '5y', 'max'] : ['1d', '5d', '1m', '1y', '5y', 'max'];
FinanceAppJS.Controls.BaseOverviewCluster.call(this, domElement, options);
// set options
WinJS.UI.setOptions(this, options);
},
{
update: function update(data, providerName) {
var updatePromise = WinJS.Promise.as({});
switch (providerName) {
case 'ETFDataProvider':
updatePromise = this.updateETFData(data);
var quoteSearchButton = document.querySelectorAll(".quoteSearchglyphParent");
if (quoteSearchButton.length > 0) {
for (var index = 0; index < quoteSearchButton.length; index++) {
quoteSearchButton[index].addEventListener("MSLostPointerCapture", PlatformJS.Utilities.pointerEventListenersMap["MSLostPointerCapture"], false);
quoteSearchButton[index].addEventListener("MSPointerCancel", PlatformJS.Utilities.pointerEventListenersMap["MSPointerCancel"], false);
quoteSearchButton[index].addEventListener("MSPointerUp", PlatformJS.Utilities.pointerEventListenersMap["MSPointerUp"], false);
quoteSearchButton[index].addEventListener("MSPointerDown", PlatformJS.Utilities.pointerEventListenersMap["MSPointerDown"], false);
quoteSearchButton[index].addEventListener("MSPointerOut", PlatformJS.Utilities.pointerEventListenersMap["MSPointerOut"], false);
}
}
this.realtimeQuotesLastUpdatedTimeString = utils.getResourceString("ChartRTLastTraded").format(data.overview.Dt);
break;
case 'ETFChartDataProvider':
updatePromise = this.updateChart(data);
break;
default:
debugger;
break;
}
return updatePromise;
},
updateETFData: function updateETFData(data) {
this.etfData = data;
this.etfRtData = this.etfRtData || data;
if (data) {
this.equityUpdateTime = data.lastUpdateTime;
}
if (this.etfRtData && this.etfRtData.overview && this.etfRtData.overview.displayName) {
this.displayName = this.etfRtData.overview.displayName;
}
else {
this.displayName = "-";
}
if (this.etfRtData) {
return this.updateEtfTile();
}
else {
return WinJS.Promise.as({});
}
},
updateEtfTile: function updateEtfTile() {
var that = this;
var collatedData = utils.deepCopy(this.etfData, this.etfRtData);
return FinanceAppJS.Controls.ModuleClusterControl.prototype.update.call(this, collatedData).then(function () {
return that.performPendingUpdate();
});
},
})
});
})();
