﻿(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
ETFDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this.entities = options && options.entities;
this.refreshInterval = options.refreshInterval;
}, {
queryServiceId: "ETF",
entities: null,
ipage: null,
refreshInterval: 0,
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams["entities"] = encodeURIComponent(this.entities);
urlParams.insert("lang", FinanceAppJS.Globalization.getLanguage());
urlParams.insert("localizeFor", FinanceAppJS.Globalization.getMarket());
return urlParams;
},
adsControlInvoked: false,
dataBuilder: function dataBuilder(data, response) {
var dataSet = {};
var that = this;
var dt = data[0].Eqdta;
var rtdata = data[0].Rtdta;
dataSet.overview = this._getOverviewData(dt, rtdata);
dataSet.peers = this._getPeersData(dt);
dataSet.profile = this._getProfileData(dt);
dataSet.rating = this._getRatingData(dt);
var columns = this._getHoldingsData(dt, -1);
dataSet.holdings = dt && dt.Hlds && dt.Hlds.length && [columns].concat(dt.Hlds.map(this._getHoldingsData));
//Adding Space between keywords and timestamp
dataSet.portfolioDate = dt && dt.Pdt ? utils.getResourceString("ETFPortfolioDate") + " " + utils.dateFormat(dt.Pdt) : "-";
// Can't bind the ads control twice because of message pump with the iFrame
if (!that.adsControlInvoked) {
var adConfig = PlatformJS.Ads.getAdsConfig("sponsoredBy");
if (adConfig) {
dataSet.overview["adUnitId"] = adConfig.adUnitId;
that.adsControlInvoked = true;
}
}
if (data && response) {
dataSet.lastUpdateTime = response.lastUpdateTime;
}
return dataSet;
},
_getOverviewData: function _getOverviewData(data, rtdata) {
var item = this._getRealTimeData(rtdata);
item.Nast = data ? utils.formatVolume(data.Nast, 2) : "-";
item.Yld = data ? FinanceAppJS.Utils.Formatters.percentFormatter(data.Yld, 2) : "-";
item.Cat = data && data.Cat ? data.Cat : "-"; 
item.Ex = data ? FinanceAppJS.Utils.Formatters.percentFormatter(data.Erj, 2) : "-";
item.Og = data && data.Og ? data.Og : "";
var fundDetails = this._getFundDetailsTemplateData(data);
for (var prop in fundDetails) {
item[prop] = fundDetails[prop];
}
// add narator text.
item.naratorText =  item.displayName;
return item;
},
_getPeersData: function _getPeersData(data) {
var peerItem = {};
if (data) {
peerItem.name = data.Tkr || utils.getResourceString("Fund");
peerItem.Fyr = FinanceAppJS.Utils.Formatters.percentFormatter(data.Fyr, 2);
peerItem.Rfyc = FinanceAppJS.Utils.Formatters.percentFormatter(data.Rfyc, 2);
peerItem.Erj = FinanceAppJS.Utils.Formatters.percentFormatter(data.Erj, 2);
peerItem.Erc = FinanceAppJS.Utils.Formatters.percentFormatter(data.Erc, 2);
peerItem.Rj = FinanceAppJS.Utils.Formatters.numberFormatter(data.Rj, 2);
peerItem.Rc = FinanceAppJS.Utils.Formatters.numberFormatter(data.Rc, 2);
//TODO: do these (M, B formatting)
peerItem.Nc = data.Nc ? utils.formatVolume(data.Nc, 2) : "-";
peerItem.Nj = data.Nast ? utils.formatVolume(data.Nast, 2) : "-";
peerItem.Ej = data.Ahld ? FinanceAppJS.Utils.Formatters.percentFormatter(data.Ahld.Ust, 2) : "-";
peerItem.Ec = FinanceAppJS.Utils.Formatters.percentFormatter(data.Ec, 2);
peerItem.Dj = FinanceAppJS.Utils.Formatters.percentFormatter(data.Dj, 2);
peerItem.Dc = FinanceAppJS.Utils.Formatters.percentFormatter(data.Dc, 2);
peerItem.Cj = data.Ahld ? FinanceAppJS.Utils.Formatters.percentFormatter(data.Ahld.Csh, 2) : "-";
//TODO: do these (M, B formatting)
peerItem.Cc = utils.formatValue("", utils.formatVolume(parseFloat(data.Cc, 2)), "%");
peerItem.Terr = FinanceAppJS.Utils.Formatters.numberFormatter(data.Terr, 2);
peerItem.Cter = FinanceAppJS.Utils.Formatters.numberFormatter(data.Cter, 2);
peerItem.Og = data.Og ? data.Og : "-";
peerItem.Cat = data.Cat ? data.Cat : "-";
// add Narator text
peerItem.naratorText = utils.getResourceString("KeyStats") + ". " + peerItem.name;
}
return peerItem;
},
_getProfileData: function _getProfileData(dataItem) {
var profileItem = {};
if (dataItem) {
var map = FinanceAppJS.Constants.Funds.FundStyleMap();
profileItem.Ff = dataItem.Ff ? utils.addLtrMark(dataItem.Ff) : "-";
profileItem.Is = !dataItem.Fs ? "-" : utils.addLtrMark(map[dataItem.Fs - 1]);
//Removing tenure for now. Will bring it back with more clarity
//profileItem.Fm = (dataItem.Fm ? dataItem.Fm : "-") + " / " + (dataItem.T ? utils.getResourceString("yrs").format(dataItem.T) : "-");
profileItem.Fm = (dataItem.Fm ? dataItem.Fm : "-");
profileItem.Fm = utils.addLtrMark(profileItem.Fm);
profileItem.Mp = FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Mp, 0);
profileItem.Mpr = FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Mpr, 0);
profileItem.Mpp = FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Mpp, 0);
profileItem.Sn = dataItem.Sn ? utils.addLtrMark(dataItem.Sn) : "-";
profileItem.In = dataItem.In ? dataItem.In : "-";
profileItem.Incd = utils.dateFormat(dataItem.Incd);
var fundWebsite = dataItem.Fw;
if (!fundWebsite || fundWebsite === "") {
profileItem.wd = 'none';
}
else {
if (fundWebsite.indexOf("http") !== 0) {
profileItem.fw = "http://" + fundWebsite;
}
else {
profileItem.fw = fundWebsite;
}
}
}
return profileItem;
},
_getRatingData: function _getRatingData(dataItem) {
var ratingItem = {};
if (dataItem) {
ratingItem.Org = FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Og, 0);
ratingItem.Og = dataItem.Og;
ratingItem.Rtyr = dataItem.Rtyr ? dataItem.Rtyr : "-"; 
ratingItem.Rfyr = dataItem.Rfyr ? dataItem.Rfyr : "-"; 
ratingItem.Rnyr = dataItem.Rnyr ? dataItem.Rnyr : "-"; 
ratingItem.Or = dataItem.Or ? dataItem.Or : "-"; 
ratingItem.Tyk = dataItem.Tyk ? dataItem.Tyk : "-"; 
ratingItem.Fyk = dataItem.Fyk ? dataItem.Fyk : "-"; 
ratingItem.Nyk = dataItem.Nyk ? dataItem.Nyk : "-"; 
ratingItem.Ok = dataItem.Ok ? dataItem.Ok : "-"; 
ratingItem.Tyg = dataItem.Tyg;
ratingItem.Fyg = dataItem.Fyg;
ratingItem.Nyg = dataItem.Nyg;
ratingItem.ticker = dataItem.Fw;
}
return ratingItem;
},
_getHoldingsData: function _getHoldingsData(dataItem, x) {
if (dataItem) {
var holdingsItem = [];
if (x === -1) {
holdingsItem[holdingsItem.length] = utils.getResourceString("ETFName");
holdingsItem[holdingsItem.length] = utils.getResourceString("PercentPortfolioWt");
holdingsItem[holdingsItem.length] = utils.getResourceString("ETFSharesOwned");
}
else {
holdingsItem[holdingsItem.length] = utils.addLtrMark(dataItem.Hn);
holdingsItem[holdingsItem.length] = utils.addLtrMark(FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Hp, 2));
holdingsItem[holdingsItem.length] = utils.addLtrMark(FinanceAppJS.Utils.Formatters.numberFormatter(dataItem.Tsh, 0));
}
}
return holdingsItem;
},
_getFundDetailsTemplateData: function _getFundDetailsTemplateData(data) {
var fundItem = {};
if (!data || (!(data.Fs || data.Ytr || data.Omr || data.Tyr || data.Fyr || data.Nyr))) {
fundItem.fundPerfClass = 'hide';
}
else {
fundItem.fundPerfClass = 'fsparent';
}
if (data) {
if (!data.Fs) {
fundItem.index = 0;
} else {
fundItem.index = data.Fs;
}
var returns = [];
returns.push(data.Ytr ? data.Ytr : 0, data.Omr ? data.Omr : 0, data.Tyr ? data.Tyr : 0, data.Fyr ? data.Fyr : 0, data.Nyr ? data.Nyr : 0);
fundItem.returns = returns;
fundItem.Ytr = FinanceAppJS.Utils.Formatters.numberFormatter(data.Ytr, 2);
fundItem.Omr = FinanceAppJS.Utils.Formatters.numberFormatter(data.Omr, 2);
fundItem.Tyr = FinanceAppJS.Utils.Formatters.numberFormatter(data.Tyr, 2);
fundItem.Fyr = FinanceAppJS.Utils.Formatters.numberFormatter(data.Fyr, 2);
fundItem.Nyr = FinanceAppJS.Utils.Formatters.numberFormatter(data.Nyr, 2);
fundItem.Ytc = data.Ytc ? FinanceAppJS.Utils.Formatters.numberFormatter(data.Ytc.R, 0) : "-";
fundItem.Omc = data.Omc ? FinanceAppJS.Utils.Formatters.numberFormatter(data.Omc.R, 0) : "-";
fundItem.Tyc = data.Tyc ? FinanceAppJS.Utils.Formatters.numberFormatter(data.Tyc.R, 0) : "-";
fundItem.Fyc = data.Fyc ? FinanceAppJS.Utils.Formatters.numberFormatter(data.Fyc.R, 0) : "-";
fundItem.Nyc = data.Nyc ? FinanceAppJS.Utils.Formatters.numberFormatter(data.Nyc.R, 0) : "-";
}
return fundItem;
},
_getRealTimeData: function _getRealTimeData(rtdata) {
var item = {};
var chartStartDuration = "1d";
item.col1 = "row2 col3 fontMB pttc";
item.col2 = "row3 col3 fontMB pttc";
item.img = "row3 col1 t3 ptp";
item.Lp = FinanceAppJS.Utils.Formatters.numberFormatter(rtdata.Lp, 2);
item.Ch = FinanceAppJS.Utils.Formatters.numberFormatter(rtdata.Ch, 2, true);
item.Chp = FinanceAppJS.Utils.Formatters.percentFormatter(rtdata.Chp, 2, true);
if (rtdata.Ch > 0) {
item.changeStatus = 1;
item.img += " largeGlyph financeGlyphUp";
item.col1 += " upcolor";
item.col2 += " upcolor";
}
else if (rtdata.Ch < 0) {
item.changeStatus = -1;
item.img += " largeGlyph financeGlyphDn";
item.col1 += " downcolor";
item.col2 += " downcolor";
}
else if (rtdata.Ch === 0) {
item.changeStatus = 0;
item.img += " mediumGlyph financeGlyphEq";
item.col1 += " equalcolor";
item.col2 += " equalcolor";
}
item.Dl = FinanceAppJS.Utils.Formatters.numberFormatter(rtdata.Dl, 2);
item.Dh = FinanceAppJS.Utils.Formatters.numberFormatter(rtdata.Dh, 2);
item.Yl = FinanceAppJS.Utils.Formatters.numberFormatter(rtdata.Yl, 2);
item.Yh = FinanceAppJS.Utils.Formatters.numberFormatter(rtdata.Yh, 2);
var dayRange = rtdata.Dh - rtdata.Dl;
var lpDay = rtdata.Lp - rtdata.Dl;
item.dpos = ((lpDay / dayRange) * 50) - 5 + "px";
var yearRange = rtdata.Yh - rtdata.Yl;
var lpYear = rtdata.Lp - rtdata.Yl;
item.ypos = ((lpYear / yearRange) * 50) - 5 + "px";
item.Op = FinanceAppJS.Utils.Formatters.numberFormatter(rtdata.Op, 2);
item.V = rtdata.V ? utils.formatVolume(rtdata.V, 2) : "-";
item.Dt = FinanceAppJS.Utils.Formatters.shortDateFormat(rtdata.Ld) + " " + FinanceAppJS.Utils.Formatters.shortTimeFormat(rtdata.Lt) + " " + rtdata.tz;
item.timeZone = utils.timeZoneFormat(rtdata.tz, rtdata.utcOff);
item.displayName = rtdata.Cmp;
return item;
}
}, {
})
});
})();