﻿(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
GlobalCurrencyDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this.localCurrency = (options && options.localCurrency);
var currencyList = FinanceAppJS.CurrencyInfo.getCurrencyList();
this.crossRateList = currencyList.CrossRates;
this.crossRatesConfig = currencyList.CrossRatesConfig;
this.flatRates = currencyList.FlatRates;
}, {
queryServiceId: "GlobalCurrency",
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData,
localCurrency: null,
crossRateList: null,
flatRates: null,
getQueryServiceParams: function getQueryServiceParams() {
var param = PlatformJS.Collections.createStringDictionary();
param.insert("localCurrency", encodeURIComponent(this.localCurrency));
return param;
},
dataBuilder: function dataBuilder(data) {
FinanceAppJS.GlobalCurrency.currencyData = data;
var currencyInfo = FinanceAppJS.CurrencyInfo.getCurrencyInfo();
for (var cur in currencyInfo) {
currencyInfo[cur].currencyCode = cur;
}
var currencyMap = {}, flatRates = {};
data.forEach(function (item) {
currencyMap[item.An] = item;
});
var crossRateList = this.crossRateList;
if (!currencyMap.hasOwnProperty(this.localCurrency)) {
this.localCurrency = FinanceAppJS.CurrencyFlatRates.DefaultCurrency();
}
var localCurrency = this.localCurrency;
var index = this.crossRateList.indexOf(this.localCurrency);
if (index === -1) {
index = this.crossRateList.indexOf(FinanceAppJS.CurrencyFlatRates.RemovedCurrency());
}
this.crossRateList.splice(index, 1);
this.crossRateList.unshift(this.localCurrency);
//The config for cross rates is read based on the local currency, and reverseRatio is set to true or false based on it.
//This is generally true for currencies like JPY where the Yen is smaller than most other currencies. 
var reverseRatio = this.crossRatesConfig && this.crossRatesConfig[localCurrency] && this.crossRatesConfig[localCurrency].reverseRatio ? this.crossRatesConfig[localCurrency].reverseRatio : 0;
var headerRow = [""].concat(crossRateList.map(function (item) {
if (currencyInfo[item].currencyCode === currencyInfo[item].symbol) {
return utils.addLtrMark(currencyInfo[item].currencyCode);
}
else {
return utils.addLtrMark(currencyInfo[item].currencyCode + " " + currencyInfo[item].symbol);
}
}));
var crossRates;
// If reverseRatio is false column headers should be the unit currency (ie, 1 USD, 1 GBP, etc). So prepending "1" here
if (!reverseRatio) {
headerRow = headerRow.map(function (header) {
if (header !== "") {
return utils.addLtrMark("1 " + header);
}
else {
return header;
}
});
crossRates = [headerRow].concat(crossRateList.map(function (curr1) {
//data required by first column (flag - country code, currency ticker and currency symbol) is populated here
//ISO codes are the same for all languages. Should not be localized
var bindingObject = {};
bindingObject.countryCode = currencyInfo[curr1].countryCode;
if (currencyInfo[curr1].currencyCode === currencyInfo[curr1].symbol) {
bindingObject.currencyCode = currencyInfo[curr1].currencyCode;
}
else {
bindingObject.currencyCode = utils.addLtrMark(currencyInfo[curr1].currencyCode + " " + currencyInfo[curr1].symbol);
}
return [bindingObject].concat(crossRateList.map(function (curr2) {
if (curr1 === curr2 || !currencyMap[curr1] || !currencyMap[curr1].Rt || !currencyMap[curr2] || !currencyMap[curr2].Rt) {
return { rate: "-", change: "", changep: "" };
}
if (curr2 === localCurrency) {
return { rate: currencyMap[curr1].Rt, change: currencyMap[curr1].Ch, changep: currencyMap[curr1].Pc };
};
return currencyMap[curr1].Rt / currencyMap[curr2].Rt;
}));
}));
}
// If reverseRatio is true row headers should be the unit currency. So prepending "1" here. Also reversing the ratio of the rate and calculating change and change % based on the reverse ratio.
// Using a different code fragment for reverse ratio currencies. If we have it in the same block, we have to introduce few 'if' statements inside the loop, which are redundant.
else {
crossRates = [headerRow].concat(crossRateList.map(function (curr1) {
var bindingObject = {};
bindingObject.countryCode = currencyInfo[curr1].countryCode;
if (currencyInfo[curr1].currencyCode === currencyInfo[curr1].symbol) {
bindingObject.currencyCode = currencyInfo[curr1].currencyCode;
}
else {
bindingObject.currencyCode = utils.addLtrMark(currencyInfo[curr1].currencyCode + " " + currencyInfo[curr1].symbol);
}
bindingObject.currencyCode = utils.addLtrMark("1 " + bindingObject.currencyCode);
return [bindingObject].concat(crossRateList.map(function (curr2) {
if (curr1 === curr2 || !currencyMap[curr1] || !currencyMap[curr1].Rt || !currencyMap[curr2] || !currencyMap[curr2].Rt) {
return { rate: "-", change: "", changep: "" };
}
if (curr2 === localCurrency) {
var reverseChange = utils.getCurrencyChangeForReverseRatio(currencyMap[curr1].Rt, currencyMap[curr1].Ch);
return { rate: 1 / currencyMap[curr1].Rt, change: reverseChange.Ch, changep: reverseChange.Pc };
};
return currencyMap[curr2].Rt / currencyMap[curr1].Rt;
}));
}));
}
for (var category in this.flatRates) {
var flatRateList = this.flatRates[category].filter(function (item) { return (currencyMap.hasOwnProperty(item) && currencyMap[item].Rt); });
flatRates[category] = flatRateList.map(function (item) {
var rtInfo = currencyMap[item];
var curInfo = currencyInfo[item];
return [{ code: curInfo.countryCode, name: curInfo.country }, curInfo, rtInfo.Rt, rtInfo.Ch, rtInfo.Pc];
});
}
return { crossRates: crossRates, flatRates: flatRates };
}
})
});
})();
(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS", {
GlobalCurrency: WinJS.Class.derive(FinanceAppJS.FinanceBasePage, function (state) {
FinanceAppJS.FinanceBasePage.call(this, state);
utils.setWinLabelText(document.getElementById('converter'), "Converter");
var currencyParams = FinanceAppJS.CurrencyInfo.getLocalCurrencyParameters();
this.localCurrency = currencyParams.localCurrency;
this.convertTo = currencyParams.convertTo;
this.convertFrom = currencyParams.convertFrom;
var isEventHandlerAdded = false;
var that = this;
var clusterDefinitions = {
majorCurrencies: {
clusterTitle: utils.getResourceString("CMajor Currencies"),
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: { GlobalCurrencyDataProvider: { localCurrency: this.localCurrency } },
clusterOptions: {
dataKey: ["crossRates"],
columnTemplates: ["tmpl_flagcurr", "tmpl_localrate", "tmpl_rate", "tmpl_rate", "tmpl_rate", "tmpl_rate", "tmpl_rate", "tmpl_rate"].map(PlatformJS.Utilities.getControl),
footerTemplate: PlatformJS.Utilities.getControl("ccbutton"),
footerData: {
ccclick: WinJS.Utilities.markSupportedForProcessing(function openConverter(event) {
var Page_Id = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.history.current.location.channelId);
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(Page_Id,
"",
FinanceAppJS.Telemetry.Constants.Location.CCYCONVERTER,
"");
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.CurrencyConverter_Button_Click, 1);
navigationHelper.navigateToCurrencyCalculator();
})
},
onrendercomplete: function onrendercomplete(event) {
if (!isEventHandlerAdded) {
isEventHandlerAdded = true;
this.btnConverter = document.getElementById("converter");
if (this.btnConverter) {
this.btnConverter.addEventListener('click', that.launchConverterFromEdgy);
}
}
},
className: "majorCurr"
}
},
worldCurrencies: {
clusterTitle: "",
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: { GlobalCurrencyDataProvider: { localCurrency: this.localCurrency } },
cloneFunc: function cloneFunc(data) {
var clusterDef = {};
for (var cat in data.flatRates) {
clusterDef[cat] = {
clusterTitle: utils.getResourceString(cat),
clusterData: data.flatRates[cat]
};
}
return clusterDef;
},
clusterOptions: {
hideHeader: true,
className: "worldCurr",
columnTemplates: ["tmpl_flag", "tmpl_curr", "tmpl_flatrate", "tmpl_change", "tmpl_changep"].map(PlatformJS.Utilities.getControl),
}
}
};
this.setClusterDefinitions(clusterDefinitions);
this.pageTitle = utils.getResourceString("Currencies");
}, {
localCurrency: null,
convertTo: null,
convertFrom: null,
getSharedContractData: function getSharedContractData() {
var uriBuilder = new Platform.Utilities.AppExUriBuilder();
uriBuilder.controllerId = "currencies";
var uri = uriBuilder.toString();
var textToDisplay = utils.getResourceString("ShareContractCurrenciesMessage");
var headline = utils.getResourceString("Currencies");
var currenciesData = {
source: FinanceAppJS.Constants.IDs.Currencies,
headline: headline,
text: textToDisplay,
deepLink: uri
};
return new WinJS.Promise(function (complete, error) {
utils.getShareContractHtmlFormat(currenciesData).then(function (htmlOutput) {
complete({
title: utils.getShareContractSubject(headline),
htmlContent: htmlOutput,
textContent: textToDisplay,
uri: uri
});
});
});
},
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Currencies";
},
launchConverterFromEdgy: function launchConverterFromEdgy(event)
{
var Page_Id = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.history.current.location.channelId);
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(Page_Id,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.Location.CCYCONVERTER,
"");
navigationHelper.navigateToCurrencyCalculator();
},
onNavigateAway: function onNavigateAway(reason)
{
FinanceAppJS.FinanceBasePage.prototype.onNavigateAway.call(this);
FinanceAppJS.GlobalCurrency.currencyData = null;
if (this.btnConverter) {
this.btnConverter.removeEventListener('click', this.launchConverterFromEdgy);
}
}
}, {
currencyData: null,
showFlyout: function showFlyout(currencyData, currencyParams, event, launchedFromEdgy) {
if (currencyData) {
var options = {};
options.data = currencyData;
options.from = currencyParams.convertFrom;
options.to = currencyParams.convertTo;
var converter = new FinanceAppJS.GlobalCurrencyConverter(options);
var dataObj = converter.getDataAndBindings();
var CCFlyout = document.getElementById('flyout');
var flyoutControl = CCFlyout.winControl;
CCFlyout.innerHTML = "";
var moduleInfo = { templateId: "cctemplate", fragmentPath: "/pages/GlobalCurrency/html/GlobalCurrencyConverter.html" };
var data = { ccyData: currencyData, ccyParams: currencyParams };
CommonJS.loadModule(moduleInfo, dataObj, null, null, null).then(function (ccTemplate) {
CCFlyout.appendChild(ccTemplate);
FinanceAppJS.Telemetry.FlightRecorder.LogAppEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.AppUtility.FLYOUT,
data);
flyoutControl.show(event.target, "top", "center");
FinanceAppJS.Utils.Logger.LogPerformance("currencyConverter", "load", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
});
if (launchedFromEdgy) {
FinanceAppJS.GlobalCurrency.handleEdgyOnFlyoutShow();
}
flyoutControl.onafterhide = FinanceAppJS.GlobalCurrency.handleEdgyOnFlyoutHide.bind(FinanceAppJS.GlobalCurrency, launchedFromEdgy);
}
},
handleEdgyOnFlyoutHide: function handleEdgyOnFlyoutHide(launchedFromEdgy)
{
if (launchedFromEdgy) {
var bottomEdgy = document.querySelector(".financeAppBar");
if (bottomEdgy && bottomEdgy.winControl) {
var topEdgy = document.getElementById("platformNavigationBar");
if (topEdgy && topEdgy.winControl) {
topEdgy.winControl.disabled = false;
}
bottomEdgy.winControl.hide();
}
}
PlatformJS.Navigation.mainNavigator.enableSearchOnType();
},
handleEdgyOnFlyoutShow: function handleEdgyOnFlyoutShow()
{
var bottomEdgy = document.querySelector(".financeAppBar");
if (bottomEdgy && bottomEdgy.winControl) {
var topEdgy = document.getElementById("platformNavigationBar");
if (topEdgy && topEdgy.winControl) {
topEdgy.winControl.hide();
topEdgy.winControl.disabled = true;
bottomEdgy.winControl.onafterhide = function () {
topEdgy.winControl.disabled = false;
};
}
}
},
launchConverter: function launchConverter(event, launchedFromEdgy) {
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.CurrencyConverter, 1);
FinanceAppJS.Utils.Logger.LogPerformance("currencyConverter", "load", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);
var currencyData = FinanceAppJS.GlobalCurrency.currencyData;
var currencyParams = FinanceAppJS.CurrencyInfo.getLocalCurrencyParameters();
if (!currencyData) {
var queryService = new PlatformJS.DataService.QueryService("GlobalCurrency");
var param = PlatformJS.Collections.createStringDictionary();
param.insert("localCurrency", currencyParams.localCurrency);
var options = new Platform.DataServices.QueryServiceOptions();
options.useCompatDataModel = true;
var dataReqObj = FinanceAppJS.Telemetry.FlightRecorder.LogDataRequest("request", "GlobalCurrency" );
return queryService.downloadDataAsync(param, null, null, options).then(
function (data) {
FinanceAppJS.Telemetry.FlightRecorder.LogDataRequest("service", "GlobalCurrency", dataReqObj);
var converterData = JSON.parse(data.dataString);
FinanceAppJS.GlobalCurrency.showFlyout(converterData, currencyParams, event, launchedFromEdgy);
});
}
else {
FinanceAppJS.GlobalCurrency.showFlyout(currencyData, currencyParams, event, launchedFromEdgy);
}
PlatformJS.Navigation.mainNavigator.disableSearchOnType();
}
})
});
})();
