﻿(function () {
"use strict";
//Data Provider for Overview cluster
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
IndexEntityOverviewDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this.symbolsCsv = options && options.symbolsCsv;
this.cmpName = options && options.cmpName;
this.isUSExchange = options.isUSExchange;
this.refreshInterval = options.refreshInterval;
}, {
queryServiceId: "indexRealTime",
symbolsCsv: null,
cmpName: null,
isUSExchange: true,
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("Sym", encodeURIComponent(this.symbolsCsv));
return urlParams;
},
dataBuilder: function dataBuilder(data, response) {
var overviewResponseObj = data[0];
var entitySymbol = this.symbolsCsv;
var overviewData = {};
if (overviewResponseObj) {
overviewData = overviewResponseObj;
var lastPrice = overviewResponseObj.Lp;
overviewData["Lp"] = utils.formatNumber(lastPrice, "FLOAT2", false, "");
if (overviewResponseObj.Ch > 0) {
overviewData["changeStatus"] = 1;
overviewData["statusIndicator"] = "largeGlyph financeGlyphUp";
overviewData["col1"] = "upcolor";
overviewData["col2"] = "upcolor";
} else if (overviewResponseObj.Ch < 0) {
overviewData["changeStatus"] = -1;
overviewData["statusIndicator"] = "largeGlyph financeGlyphDn";
overviewData["col1"] = "downcolor";
overviewData["col2"] = "downcolor";
} else {
overviewData["changeStatus"] = 0;
overviewData["statusIndicator"] = "mediumGlyph financeGlyphEq";
overviewData["col1"] = "equalcolor";
overviewData["col2"] = "equalcolor";
}
overviewData["Ch"] = utils.formatNumber(overviewResponseObj.Ch, "FLOAT2", true, "");
overviewData["Pc"] = utils.formatNumber(overviewResponseObj.Chp, "PERCENT", true);
overviewData["Prevc"] = utils.roundFloat(overviewResponseObj.Pp, 2);
overviewData["Openc"] = utils.roundFloat(overviewResponseObj.Op, 2);
overviewData["Pe"] = utils.formatNumber(overviewResponseObj.Pe, "P/E");
overviewData["DrStart"] = utils.roundFloat(overviewResponseObj.Dl, 2);
overviewData["DrEnd"] = utils.roundFloat(overviewResponseObj.Dh, 2);
overviewData["DrRangePointerPos"] = utils.getRangeIndicatorPos(overviewResponseObj.Dl, overviewResponseObj.Dh, lastPrice);
overviewData["Dy"] = utils.formatNumber(parseFloat(overviewResponseObj.Dy), "PERCENT", false, "-");
overviewData["Mc"] = overviewResponseObj.Mc ? utils.formatVolume(parseFloat(overviewResponseObj.Mc)) : "-";
overviewData["V"] = utils.formatNumber(overviewResponseObj.V, "VOLUME");
overviewData["avgV"] = utils.formatNumber(overviewResponseObj.avgV, "VOLUME");
overviewData["FwStart"] = utils.roundFloat(overviewResponseObj.Yl, 2);
overviewData["FwEnd"] = utils.roundFloat(overviewResponseObj.Yh, 2);
overviewData["yrr"] = utils.roundFloat(overviewResponseObj.yrr, 2); // if not avl use '-'
overviewData["Cts"] = FinanceAppJS.Utils.Formatters.numberFormatter(overviewResponseObj.Cts, 0, false); // if not avl use '-'
var fwh = overviewResponseObj.Yh;
var fwl = overviewResponseObj.Yl;
var wkLp = lastPrice < fwl ? fwl : lastPrice > fwh ? fwh : lastPrice;
overviewData["FwRangePointerPos"] = utils.getRangeIndicatorPos(overviewResponseObj.Yl, overviewResponseObj.Yh, wkLp);
overviewData["dateTime"] = FinanceAppJS.Utils.Formatters.shortDateFormat(overviewResponseObj.Ld) + " " + FinanceAppJS.Utils.Formatters.shortTimeFormat(overviewResponseObj.Lt) + " " + overviewData.tz;
overviewData["priceUpdateInfo"] = utils.getResourceString("SEPriceUpdateInfo").format(FinanceAppJS.Constants.RefreshIntervals.StockDetailsRT / 60);
// overviewData["timeZone"] = ;
overviewData["showPriceCls"] = "fontSR";
// After Hours
if (overviewData["trSts"]) {
var trCh = null, trChp = null;
if (overviewData["trSts"] === 1) {
overviewData["trLabel"] = utils.getResourceString("SEPreMarket");
overviewData["trLp"] = utils.formatNumber(overviewResponseObj.pre, "FLOAT2");
trCh = overviewResponseObj.preCh;
trChp = overviewResponseObj.preChp;
}
else if (overviewData["trSts"] === 2) {
overviewData["trLabel"] = utils.getResourceString("SEAfterHours");
overviewData["trLp"] = utils.formatNumber(overviewResponseObj.post, "FLOAT2");
trCh = overviewResponseObj.postCh;
trChp = overviewResponseObj.postChp;
}
if (trCh > 0) {
overviewData["trCh"] = utils.formatNumber(trCh, "FLOAT2", true);
overviewData["trChp"] = utils.formatNumber(trChp, "PERCENT", true);
overviewData["trChangeCls"] = "up";
}
else {
overviewData["trCh"] = utils.formatNumber(trCh, "FLOAT2");
overviewData["trChp"] = utils.formatNumber(trChp, "PERCENT");
if (trCh === 0 || !trCh) {
overviewData["trChangeCls"] = "equalcolor";
}
else {
overviewData["trChangeCls"] = "dn";
}
}
overviewData["trChangeCls"] += " minSpace";
overviewData["trClass"] = "tradingStatus";
overviewData["trCh"] = utils.addLtrMark(overviewData["trCh"]);
overviewData["trChp"] = utils.addLtrMark(overviewData["trChp"]);
}
else {
overviewData["trClass"] = "hide";
overviewData["trLabel"] = overviewData["trLp"] = overviewData["trCh"] = overviewData["trChp"] = overviewData["trChangeCls"] = null;
}
}
else {
overviewData = { Lp: "", Ch: "", Pc: "", Prevc: "-", Openc: "-", Pe: "-", DrStart: "", DrEnd: "", FwStart: "", FwEnd: "", Dy: "-", Mc: "-", V: "-", dateTime: "", priceUpdateInfo: "", showPriceCls: "hide", trClass: "hide" };
}
overviewData["stock"] = entitySymbol + "1d";
overviewData["sym"] = overviewData["stock"].displayName;
overviewData["tkr"] = overviewData["stock"].symbol;
overviewData["seriesObject"] = { seriesId: overviewData.stock, displayName: overviewData.Sym };
var adConfig = PlatformJS.Ads.getAdsConfig("sponsoredBy");
if (adConfig) {
overviewData["adUnitId"] = adConfig.adUnitId;
}
if (overviewData && response) {
overviewData.lastUpdateTime = response.lastUpdateTime;
}
// Add Narator text
overviewData["naratorText"] = overviewData.Cmp + ". " + utils.getResourceString("TickerName_Al") + overviewData.Sym;
return overviewData;
}
})
});
})();
(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
IndexConstituentsDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this.marketSymbol = options && options.marketSymbol;
}, {
marketSymbol: null,
queryServiceId: "IndexDetails",
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.IndexDetailsConstituent,
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("Sym", encodeURIComponent(this.marketSymbol));
urlParams.insert("NumConstituents", FinanceAppJS.Constants.Global.MaxIndexConstituentsToBeQueried);
urlParams.insert("lang", FinanceAppJS.Globalization.getLanguage());
return urlParams;
},
processIndexConstituents: function processIndexConstituents(data) {
var mrkConstituents = [];
mrkConstituents[0] = [
utils.getResourceString("IndexSymbol"),
utils.getResourceString("IndexName"),
utils.getResourceString("IndexPrice"),
utils.getResourceString("IndexChPct")];
var j = 1;
for (var i = 1; i <= data[0].Cl.length; i++) {
var constituent = data[0].Cl[i - 1];
if (!constituent) {
continue;
}
var thisdata = [];
thisdata[0] = constituent.Sym;
thisdata[1] = constituent.Cmp;
thisdata[2] = constituent.Lp;
thisdata[3] = constituent.Chp;
thisdata[4] = constituent;
mrkConstituents[j++] = thisdata;
}
return mrkConstituents;
},
processIndexMovers: function processIndexMovers(data) {
var marketMoversData = {};
var moversData = data[0].Mvs;
marketMoversData["gainers"] = utils.populateMoversData(moversData, "gainers");
marketMoversData["losers"] = utils.populateMoversData(moversData, "losers");
marketMoversData["active"] = utils.populateMoversData(moversData, "active");
return marketMoversData;
},
dataBuilder: function dataBuilder(data, response) {
var mrkConstituents = [];
if (!data || data.length === 0) {
return null;
}
mrkConstituents.Cts = this.processIndexConstituents(data);
mrkConstituents.Mov = this.processIndexMovers(data);
return mrkConstituents;
}
}, {
})
});
})();