﻿(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
MarketDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
}, {
queryServiceId: "TodayCCBWRData",
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.Bonds,
isCurrencyChangeRequired: true,
getQueryServiceParams: function getQueryServiceParams() {
var currency = FinanceAppJS.Globalization.getDefaultCurrency();
var market_data = FinanceAppJS.Globalization.getMarket();
var urlParams = PlatformJS.Collections.createStringDictionary();
//Using canned data of the AcrossMarket cluster of the home page. (the data is the same. The providers are different)
if (FinanceAppJS.Globalization.isUSMarket()) {
//This indicates it is US market. Backend expects no params for US market
urlParams["params"] = "localizeFor=" + market_data;
this.cannedDataFile = "FREData\\market_{market}\\AcrossMarketDataProvider.json";
}
else {
urlParams["params"] = "localcurrency=" + encodeURIComponent(currency) + "&localizeFor=" + market_data;
this.cannedDataFile = "FREData\\market_{market}\\AcrossMarketCCDataProvider.json";
}
return urlParams;
},
dataBuilder: function dataBuilder(response) {
var dataSet = {};
var homeIndices = FinanceAppJS.Globalization.getHomePageIndices().map(function (indx) {
// This is a small function to extract only the realtime symbol from a ticker which has both realtime and equity symbols. 
// Extracts only the first three components of the ticker which is the realtime symbol
var symbolParts = indx.symbol.split(".");
if (symbolParts.length > FinanceAppJS.Constants.WorldMarket.RealTimeSymbolComponents) {
var symbol = "";
for (var i = 0; i < FinanceAppJS.Constants.WorldMarket.RealTimeSymbolComponents; i++) {
symbol = symbol + symbolParts[i] + ".";
}
return symbol.substring(0, symbol.length - 1);
}
else {
return indx.symbol;
}
});
var wmData = response && response.Wmd ? response.Wmd : [];
dataSet.wm = [["WMIndices", "IndexLevel", "ChPercent"]
.map(utils.getResourceString)]
.concat(wmData.filter(function (index) {
if (index && index.idx) {
return homeIndices.indexOf(index.idx) < 0;
}
else {
return false;
}
})
.map(function (item) {
if (item) {
return [{ desc: item.cntry === "BRA" ? utils.getResourceString("Brazil") : utils.getResourceString(item.cntry), name: item.nm, country: item.cntry }, item.lvl, item.chp, item]; // This is a temporary change until we get the Brazil String. Once we get the brazil string, it has to be reverted.
}
}));
dataSet.bonds = [];
if (response.Bnd && response.Bnd.length > 0) {
for (var i in response.Bnd[0].Data) {
var item = response.Bnd[0].Data[i];
if (item && item.Dn) {
var key = item.EnDn ? item.EnDn.replace("Bond_", "") : "";
var splitNameParts = key.replace("-", " ").split(" ");
var shortNames = { month: "Mo", year: "Yr" };
var subIcon = splitNameParts.length > 2 ? shortNames[splitNameParts[1].toLowerCase()] : null;
dataSet.bonds.push([{
name: item.Dn,
iconText: subIcon ? FinanceAppJS.Utils.Formatters.numberFormatter(splitNameParts[0], 0) : "",
subIconText: subIcon ? utils.getResourceString(subIcon) : "",
},
item.Y,
{ title: item.Y - item.Yes, subtitle: item.YPct }
]);
}
};
}
dataSet.currencies = [];
if (response.Cur) {
var currencyInfo = FinanceAppJS.CurrencyInfo.getCurrencyInfo();
var dataObj = FinanceAppJS.DataProvider.MarketDataProvider.getCurrencies(response.Cur, null, FinanceAppJS.Globalization.isUSMarket());
for (var curr in dataObj) {
var currData = dataObj[curr];
var item = currencyInfo[currData.ticker];
var desc = utils.getResourceString(item.currency) + " " + item.symbol;
var ratio = (currData.num === currData.ticker) ? desc + "/" + currData.den : currData.num + "/" + desc;
if (item) {
FinanceAppJS.Globalization.isUSMarket() ?
dataSet.currencies.push([{ name: item.country, country: item.countryCode, desc: ratio }, currData.rate, { title: currData.ch, subtitle: currData.pc }]) :
dataSet.currencies.push([{ name: item.country, country: item.countryCode, desc: ratio }, currData.rate]);
}
}
}
dataSet.commodities = [];
if (response.Cmdt) {
for (var cmdt in response.Cmdt) {
var item = response.Cmdt[cmdt];
if (item && item.Dn) {
var iconClass = item.EnDn ? item.EnDn.replace("Com_", "") : "";
dataSet.commodities.push([{ name: item.Dn, iconClass: iconClass.replace(/[\W]/g, "").toLowerCase() + "Glyph", desc: utils.contractFormat(item.Cm, item.Cy) }, item.Lp, { title: item.C, subtitle: item.Cp }]);
}
}
}
return dataSet;
}
}, {
getCurrencies: function getCurrencies(rawData, maxlength, isChangeReqd) {
//This will get all the currencies required for the pano.
//This will read CurrencyList.json config file and finds if local currency is a top Currency
//If it is a top currency, based on the ratio specified in the config, it calculates the rate, change and change %
//If it is not a top currency, it will take the default list, and invert the ratio such that the rata is > 1
//Returns the rate, change, change% and the numerator and denominator of the ratio
var dataSet = [];
var localCurrency = FinanceAppJS.Globalization.getDefaultCurrency();
var currencyList = FinanceAppJS.CurrencyInfo.getCurrencyList();
var currencyObject;
var isTopCurrency = currencyList.TopCurrencies.hasOwnProperty(localCurrency);
if (isTopCurrency) {
currencyObject = currencyList.TopCurrencies[localCurrency];
}
else {
currencyObject = currencyList.MarketCurrencies;
}
var count = 0;
for (var curr in currencyObject) {
var currency = currencyObject[curr];
var curData = [];
for (var i in rawData) {
if (rawData[i]) {
curData.push(rawData[i].An);
}
}
rawData = rawData.filter(function (item) {
if (item) {
return true;
}
else {
return false;
}
});
var ticker = isTopCurrency ? curr : currency;
var index = curData.indexOf(ticker);
if (index >= 0) {
var dataObject = rawData[index];
if (dataObject) {
var rate, change, reverseCondition, num, den;
if (isTopCurrency) {
reverseCondition = currency.reverseRatio;
}
else {
reverseCondition = (dataObject.Rt < 1);
}
if (reverseCondition) {
rate = (1 / dataObject.Rt);
change = isChangeReqd ? utils.getCurrencyChangeForReverseRatio(dataObject.Rt, dataObject.Ch) : null;
num = dataObject.An;
den = localCurrency;
}
else {
rate = dataObject.Rt;
change = isChangeReqd ? { Ch: dataObject.Ch, Pc: dataObject.Pc } : null;
den = dataObject.An;
num = localCurrency;
}
count++;
if (maxlength && count > maxlength) {
break;
}
dataSet.push({ rate: rate, ch: change ? change.Ch : "", pc: change ? change.Pc : "", num: num, den: den, ticker: dataObject.An });
}
}
}
return dataSet;
}
})
});
})();
(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS", {
MarketPage: WinJS.Class.derive(FinanceAppJS.FinanceBasePage, function (state) {
/// <summary>
/// Constructor for a simple IPage class that controls the FundFilmStrip
/// </summary>
/// <param name="state">
/// The state object passed to WinJS.Navigation.navigate or the state object
/// preserved when the page has been navigated away from
/// </param>
FinanceAppJS.FinanceBasePage.call(this, state);
var HeaderSelectionCallbacks = {
marketMoversCluster: function marketMoversCluster() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.MOVERS,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERHEADER,
"");
navigationHelper.navigateToMarketMoversPage(null, "Market_MarketMoversHeader_Click");
},
worldMarket: function worldMarket() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.WORLDMARKET,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERHEADER,
"");
navigationHelper.navigateToWorldMarketPage(null, "Market_WorldMarketClusterHeader_Click");
},
currencies: function currencies() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.CURRENCIES,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERHEADER,
"");
navigationHelper.navigateToCurrenciesPage(null, "Market_CurrenciesClusterHeader_Click");
},
commodities: function commodities() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.COMMODITIES,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERHEADER,
"");
navigationHelper.navigateToCommoditiesPage(null, "Market_CommoditiesClusterHeader_Click");
},
bonds: function bonds() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.BONDS,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERHEADER,
"");
navigationHelper.navigateToBondsPage(null, "Market_BondsClusterHeader_Click");
},
fundPicksCluster: function fundPicksCluster() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.FUNDPICKS,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERHEADER,
"");
navigationHelper.navigateToFundPicksPage(null, "Market_FundPicksCluster_Click");
}
};
var TableClickCallbacks = {
currencies: function currencies() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.CURRENCIES,
FinanceAppJS.Telemetry.Constants.Location.TABLE,
"");
navigationHelper.navigateToCurrenciesPage(null, "Market_CurrenciesCluster_Click");
},
commodities: function commodities() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.COMMODITIES,
FinanceAppJS.Telemetry.Constants.Location.TABLE,
"");
navigationHelper.navigateToCommoditiesPage(null, "Market_CommoditiesCluster_Click");
},
bonds: function bonds() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.BONDS,
FinanceAppJS.Telemetry.Constants.Location.TABLE,
"");
navigationHelper.navigateToBondsPage(null, "Market_BondsCluster_Click");
},
};
var TableRowClickCallbacks = {
worldMarketRow: function worldMarketRow(data) {
var instrumentID = FinanceAppJS.Constants.Instrumentation.Index_MarketWMCluster_Click;
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.WORLDMARKET,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
FinanceAppJS.Constants.IDs.RoamingIndexPrefix + data[3].nm);
var state = utils.getIndexDetails(data[3], "WorldMarketData");
navigationHelper.navigateToIndexDetailsPage(state, instrumentID);
}
};
var ItemClickCallbacks = {
fundPicksCluster: function fundPicksCluster(event) {
var instrumentNameKey;
var item = event.detail.item;
if (item.SId) {
instrumentNameKey = "Market_FundPicksCluster_" + item.SId + "_Click";
}
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKET,
FinanceAppJS.Telemetry.Constants.Cluster.FUNDPICKS,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
item.SId);
navigationHelper.navigateToFundBrowsePage({ title: item.Nm, sid: item.SId, category: item.Cat }, instrumentNameKey); 
}                
};
var that = this;
this.roaming = new FinanceAppJS.Roaming;
var indicesConfig = utils.getDefaultConfigForIndices();
var equityDataProviderOptions = FinanceAppJS.DataProvider.EquityDataProvider.getDefaultProviderOptions(this.roaming);
equityDataProviderOptions.refreshInterval = indicesConfig.shouldRefresh ? FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData : 0;
var clusterDefinitions = {
indexChartCluster: {
clusterTitle: utils.getResourceString("Indices"),
onHeaderSelection: HeaderSelectionCallbacks.indexChartCluster,
clusterControl: "FinanceAppJS.HomePage.IndexChartCluster",
dataProviders: {
StockChartDataProvider: { instrumentType: "stock", stockSymbol: indicesConfig.chartDefaultSymbol, duration: indicesConfig.chartDefaultDuration, isEOD: indicesConfig.isEOD, refreshInterval: indicesConfig.isEOD ? 0 : FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData },
EquityDataProvider: equityDataProviderOptions
},
primaryProviders: "EquityDataProvider",
clusterOptions: {
instrumentType: "index",
isEOD: indicesConfig.isEOD,
className: "indexCluster",
moduleInfo: { renderer: FinanceAppJS.Renderers.TodayIndexChart },
}
},
worldMarket: {
clusterTitle: utils.getResourceString("WorldMarket"),
onHeaderSelection: HeaderSelectionCallbacks.worldMarket,
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: "MarketDataProvider",
// 1st row is always the header. So checking if length > 1
postDataValidateFunc: function postDataValidateFunc(data) { return data && data.wm && data.wm.length > 1; },
clusterOptions: {
dataKey: ["wm"],
stickyHeader: true,
columnTemplates: ["tmpl_countryname", "tmpl_lvl", "tmpl_change1"].map(PlatformJS.Utilities.getControl),
summary: utils.getResourceString("WorldMarket"),
className: "WorldMarket",
useDataObjectForBinding: true,
rowClickCallback: TableRowClickCallbacks.worldMarketRow,
onrendercomplete: function onrendercomplete(event) {
msSetImmediate(function () {
var worldMarketTableBody = document.querySelector(".WorldMarket .tablebody");
var worldMarketTableRows = document.querySelectorAll(".WorldMarket .tablerow");
for (var index = 0; index < worldMarketTableRows.length; index++) {
worldMarketTableRows[index].addEventListener("blur", function (e) {
WinJS.Utilities.removeClass(worldMarketTableBody, "onRowFocus");
}, true);
worldMarketTableRows[index].addEventListener("focus", function (e) {
WinJS.Utilities.addClass(worldMarketTableBody, "onRowFocus");
}, true);
}
});
},
}
},
marketMoversCluster: {
clusterTitle: utils.getResourceString("MarketMovers"),
onHeaderSelection: HeaderSelectionCallbacks.marketMoversCluster,
postDataValidateFunc: function postDataValidateFunc(data) {
return data && data.Mov && (data.Mov.active || data.Mov.gainers || data.Mov.losers);
},
clusterControl: "FinanceAppJS.HomePage.MarketMoversCluster",
dataProviders: {
EquityDataProvider: equityDataProviderOptions
},
clusterOptions: {
width: "auto",
// TODO: Looks like onrendercomplete is being called for every item container. Should be fixed in base controller/provider.
onrendercomplete: function onrendercomplete() {
msSetImmediate(function () {
//Bind aria-labels to respective win-item in watchlist.
var markWatchListObj = document.querySelector(".MarketMoversCluster");
if (markWatchListObj) {
var watchListItems = markWatchListObj.querySelectorAll(".win-template");
if (watchListItems && watchListItems.length > 0) {
utils.addCustomAriaLabelToTiles(watchListItems, "naratorText");
}
if (!that._isMMClusterRenderedOnce) {
var marketMoverHeaders = markWatchListObj.querySelectorAll(".moversHeaderTemplate");
/*Bug Fix - 206045, we don't need headers to have tabIndex and and list should have aria-label*/
if (marketMoverHeaders) {
var setMoverListItem = true;
for (var index = 0; index < marketMoverHeaders.length; index++) {
if (marketMoverHeaders[index].parentElement) {
marketMoverHeaders[index].parentElement.tabIndex = -1;
var moverListElements = marketMoverHeaders[index].parentElement.querySelector(".financeMoversInnerList");
if (moverListElements.childElementCount > 0) {
moverListElements.children[0].tabIndex = 0;
}
}
}
}
that._isMMClusterRenderedOnce = true;
}
}
});
},
}
},
currencies: {
clusterTitle: utils.getResourceString("Currencies"),
onHeaderSelection: HeaderSelectionCallbacks.currencies,
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: "MarketDataProvider",
postDataValidateFunc: function postDataValidateFunc(data) { return (data && data.currencies && data.currencies.length > 0); },
clusterOptions: {
dataKey: ["currencies"],
hideHeader: true,
tapBehavior: 'none',
columnTemplates: ["tmpl_countryname", "tmpl_rate", "tmpl_change2"].map(PlatformJS.Utilities.getControl),
summary: utils.getResourceString("Currencies"),
className: "table",
tableClickHandler: TableClickCallbacks.currencies
}
},
currenciesNonUS: {
clusterTitle: utils.getResourceString("Currencies"),
onHeaderSelection: HeaderSelectionCallbacks.currencies,
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: "MarketDataProvider",
postDataValidateFunc: function postDataValidateFunc(data) { return (data && data.currencies && data.currencies.length > 0); },
clusterOptions: {
dataKey: ["currencies"],
hideHeader: true,
tapBehavior: 'none',
columnTemplates: ["tmpl_countryname", "tmpl_rate"].map(PlatformJS.Utilities.getControl),
summary: utils.getResourceString("Currencies"),
className: "currencyNonUS",
tableClickHandler: TableClickCallbacks.currencies
}
},
commodities: {
clusterTitle: utils.getResourceString("Commodities"),
onHeaderSelection: HeaderSelectionCallbacks.commodities,
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: "MarketDataProvider",
postDataValidateFunc: function postDataValidateFunc(data) {
return (data && data.commodities && data.commodities.length > 0);
},
clusterOptions: {
dataKey: ["commodities"],
tapBehavior: 'none',
hideHeader: true,
columnTemplates: ["tmpl_name", "tmpl_lvl", "tmpl_change2"].map(PlatformJS.Utilities.getControl),
summary: utils.getResourceString("Commodities"),
className: "table",
tableClickHandler: TableClickCallbacks.commodities
}
},
bonds: {
clusterTitle: utils.getResourceString("Bonds"),
onHeaderSelection: HeaderSelectionCallbacks.bonds,
clusterControl: "FinanceAppJS.Controls.TableCluster",
dataProviders: "MarketDataProvider",
postDataValidateFunc: function postDataValidateFunc(data) { return (data && data.bonds && data.bonds.length > 0); },
clusterOptions: {
dataKey: ["bonds"],
tapBehavior: 'none',
hideHeader: true,
columnTemplates: ["tmpl_label", "tmpl_lvl", "tmpl_change2"].map(PlatformJS.Utilities.getControl),
summary: utils.getResourceString("Bonds"),
className: "table bondsTable",
tableClickHandler: TableClickCallbacks.bonds
}
},
fundPicksCluster: {
clusterTitle: utils.getResourceString("ChFund Picks"),
onHeaderSelection: HeaderSelectionCallbacks.fundPicksCluster,
clusterControl: "FinanceAppJS.Controls.ItemsCluster",
dataProviders: "TodayFundPicksDataProvider",
postDataValidateFunc: function postDataValidateFunc(data) { return data && data.length > 0; },
clusterOptions: {
useDataObjectForBinding: true,
className: "platformListLayout",
itemTemplate: {
fragmentPath: "/pages/MarketToday/html/MarketTodayTemplates.html",
templateId: "FundPicksTemplate"
},
onitemclick: ItemClickCallbacks.fundPicksCluster,
}
},
};
this.setClusterDefinitions(clusterDefinitions);
this.setPageTimeUpdateDataProvider("EquityDataProvider");
this.pageTitle = utils.getResourceString("Market_Pano");
}, {
onNavigateAway: function onNavigateAway(reason) {
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away
/// </summary>
/// <param name="reason">
/// The reason why the page is being navigated away: "backNavigation" or "newNavigation"
/// </param>
/// <returns>
/// Nothing
/// </returns>
FinanceAppJS.FinanceBasePage.prototype.onNavigateAway.call(this);
},
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Market";
},
onPageLoadComplete: function onPageLoadComplete() {
FinanceAppJS.FinanceBasePage.prototype.onPageLoadComplete.call(this);
// Prefetching currency panorama data in home page as the cache gets populated and if the user goes offline in b/w, the page will load
utils.prefetchCurrencies();
},
onBindingComplete: function onBindingComplete() {
FinanceAppJS.FinanceBasePage.prototype.onBindingComplete.call(this);
}
})
});
})();
