﻿(function () {
'use strict';
WinJS.Namespace.define('FinanceAppJS.MarketMovers', {
Collection: WinJS.Class.derive(CommonJS.EntityCollection.Collection, function (options) {
CommonJS.EntityCollection.Collection.call(this, options);
this.options = options || {};
this._data = [];
this.impressionContext = "/Market Movers";
}, {
/*CommonJS.EntityCollection.Collection Members*/
Initialize: function Initialize(config) {
FinanceAppJS.Utils.disableBottomEdgy();
config = config || {};
this.UIConfig = config.UIConfig || new FinanceAppJS.MarketMovers.UIConfig();
this.DataConfig = config.DataConfig || new CommonJS.EntityCollection.DataConfig();
this.Provider = new CollectionProvider.MarketMoversProvider();
var providerConfig = PlatformJS.Collections.createStringDictionary();
providerConfig.insert("Exchanges", JSON.stringify(FinanceAppJS.Globalization.getMoversExchangeList()));
var idConstants = FinanceAppJS.Constants.IDs;
providerConfig.insert("MappingConstantStock", idConstants.Stock);
providerConfig.insert("MappingConstantEtf", idConstants.Etf);
this.invalidataView_binding = this.invalidateView.bind(this);
FinanceAppJS.Utils.ResumeHandler.setRefreshCallback(this.invalidataView_binding);
FinanceAppJS.Globals.networkChangeCallback = this.invalidataView_binding;
return this.Provider.initializeAsync(providerConfig);
},
Query: function Query(sort, filters) {
if (filters.length > 0) {
var data = {
Type: "Exchange",
Name: filters[0].value[0],
};
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKETMOVERS,
FinanceAppJS.Telemetry.Constants.Cluster.MOVERS,
FinanceAppJS.Telemetry.Constants.Location.EXCHSELECTOR,
data);
}
return CommonJS.EntityCollection.Collection.prototype.Query.call(this, sort, filters);
},
}),
UIConfig: WinJS.Class.derive(CommonJS.EntityCollection.UIConfig, function (options) {
CommonJS.EntityCollection.UIConfig.call(this, options);
this.wlRoaming = new FinanceAppJS.Roaming();
this.visibilityHandler_binding = this.visibilityHandler.bind(this);
document.addEventListener("msvisibilitychange", this.visibilityHandler_binding);
this.addButtonClickedPromise_binding = this.addButtonClickedPromise.bind(this);
}, {
OnItemClicked: function OnItemClicked(item) {
// TODO: Find out a better way to set the tile type
item = this.setTileType(item);
var idConsts = FinanceAppJS.Constants.IDs;
// Instrument MarketMover Tile Click
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.MarketMovers_Cluster_Click, 1);
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKETMOVERS,
FinanceAppJS.Telemetry.Constants.Cluster.MOVERS,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
item.Cmp);
switch (item.type) {
case idConsts.Etf:
navigationHelper.navigateToETFDetailsPage(WatchListManage.getETFDetailsPageInfo(item));
break;
case idConsts.Stock:
navigationHelper.navigateToEntityPage(WatchListManage.getEntityPageInfo(item));
break;
default:
break;
};
return true;
},
GetHeaderTemplate: function GetHeaderTemplate(view) {
return document.querySelector('.marketMoversHeaderTemplate');
},
GetEntityTemplate: function GetEntityTemplate() {
return document.querySelector('.marketMoversItemTemplate');
},
HideAppBar: function HideAppBar() {
return false;
},
SetFilterType: function SetFilterType(filter) {
filter.type = 'Toggle';
filter.selectionMode = 'SingleToggle';
},
GetViewOptions: function GetViewOptions(view) {
var options = {};
/* this sample shows how to enable multiselect for the grid view of ECV */
switch (view) {
case CommonJS.EntityCollection.Views.Grid:
options.itemSelectionMode = 'multi'; // Tile selection
options.disableClearFilters = true;
options.enableKeydownHandler = true;
break;
}
// Bug#158094: disabling semanticZoom for all ECVs
options.disableSemanticZoom = true;
return options;
},
getSleepDuration: function getSleepDuration() {
/*any value zero or lower means don't sleep ever*/
/*value is in milliseconds*/
var exchange = FinanceAppJS.Globalization.getDefaultExchange();
if (exchange && exchange.code) {
var isEOD = utils.isEODInstrument(exchange.code);
if (isEOD) {
return 0;
}
}
return FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData * 1000;
},
GetAppBarItems: function GetAppBarItems(controller) {
/*must return an array of WinJS.UI.AppBarCommand options objects controller is passed in so consumer can grab handles to entities, sort, fitler, etc in their callbacks*/
var buttons = [
{
id: "btnClear",
icon: "\uE009",
extraClass: 'appexSymbol left hide',
label: utils.getResourceString("MyStocksClearSelection"),
onclick: this.clearButtonClicked.bind(this, controller)
},
{
id: "btnAdd",
icon: "\uE017",
extraClass: 'appexSymbol left hide',
label: utils.getResourceString("BTN_LABEL_ADD_TO_WATCHLIST"),
onclick: this.addButtonClicked.bind(this, controller)
},
{
id: "helpButton",
icon: "\uFF60",
extraClass: 'appexSymbol right',
label: utils.getResourceString("/Platform/HelpLabel"),
instrumentId: FinanceAppJS.Constants.Instrumentation.Help_Movers_Click,
onclick: navigationHelper.navigateToHelpPage
},
{
id: "pinTileButton",
icon: "\uE015",
extraClass: 'appexSymbol right hide',
label: utils.getResourceString("/Platform/pinToStart"),
},
{
id: "unpinTileButton",
icon: "\uE016",
extraClass: 'appexSymbol right hide',
label: utils.getResourceString("/Platform/unpinFromStart"),
}
];
return buttons;
},
wlRoaming: null,
pinUnpinManager: null,
_selectedItem: null,
btnInitialized: false,
addButton: null,
clearButton: null,
pinButton: null,
unpinButton: null,
bottomEdgy: null,
topEdgyWinControl: null,
pinButtonCallback: function pinButtonCallback(controller) {
controller.appbar.hide();
controller.currentView.clearSelection();
},
addButtonClicked: function addButtonClicked(controller) {
var items = controller.currentView.getSelectedItems();
var allItems = items.getItems();
var allItemDesc = ""; 
for (var i = 0; i < allItems._value.length; i++) {
if (i > 0) {
allItemDesc += "|";
}
allItemDesc += allItems._value[i].data.Cmp;
}
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKETMOVERS,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.Location.ADD,
allItemDesc);
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.Add_Movers_Click, 1);
items.getItems().then(this.addButtonClickedPromise_binding);
controller.appbar.hide();
},
addButtonClickedPromise: function addButtonClickedPromise(items) {
if (items.length) {
var symbols = [];
var itemsUnique = [];
for (var i = 0; i < items.length; i++) {
var symbol = utils.getGlobalInstrumentSymbol(items[i].data);
if (itemsUnique.indexOf(symbol) === -1) {
symbols.push({ symbol: symbol, type: items[i].type });
itemsUnique.push(symbol);
}
}
if (this.wlRoaming.addToWatchlist(symbols, true)) {
FinanceAppJS.Telemetry.FlightRecorder.LogAppEvent(FinanceAppJS.Telemetry.Constants.Page.MARKETMOVERS,
FinanceAppJS.Telemetry.Constants.AppUtility.TOPBARMSG,
"MSG_ADD_MULTIPLE_STOCKS");
var msgbar = new CommonJS.MessageBar(utils.getResourceString("MSG_ADD_MULTIPLE_STOCKS"), { autoHide: true });
this.topEdgyWinControl.hide();
msgbar.show();
}
}
},
clearButtonClicked: function clearButtonClicked(controller) {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKETMOVERS,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.Location.CLEARBTN,
"Clear Selection");
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.Clear_Movers_Click, 1);
controller.currentView.clearSelection();
controller.appbar.hide();
},
onSelectionChanging: function onSelectionChanging(items, controller) { },
onSelectionChanged: function onSelectionChanged(items, controller) {
if (this.btnInitialized === false) {
this.topEdgyWinControl = document.querySelector("#platformNavigationBar").winControl;
this.bottomEdgy = document.querySelector(".entityappbar");
this.addButton = this.bottomEdgy.querySelector("#btnAdd");
this.clearButton = this.bottomEdgy.querySelector("#btnClear");
this.pinButton = this.bottomEdgy.querySelector("#pinTileButton");
this.unpinButton = this.bottomEdgy.querySelector("#unpinTileButton");
if (this.bottomEdgy && controller) {
this.clearSelectionHandler_binding = controller.currentView.clearSelection.bind(controller.currentView);
this.bottomEdgy.addEventListener("beforehide", this.clearSelectionHandler_binding);
}
this.btnInitialized = true;
}
if (items && items.length > 0) {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKETMOVERS,
FinanceAppJS.Telemetry.Constants.Cluster.RIGHTCLICKSELECT,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
"Items Selected - " + items.length);
// TODO: Find out a better way to set the tile type
for (var k = 0; k < items.length; k++) {
items[k] = this.setTileType(items[k]);
}
WinJS.Utilities.removeClass(this.addButton, "hide");
WinJS.Utilities.removeClass(this.clearButton, "hide");
if (items.length > 1) {
this.pinButton.style.display = "none";
this.unpinButton.style.display = "none";
this._selectedItem = null;
}
if (controller.appbar.hidden) {
//FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKETMOVERS,
//      FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
//      FinanceAppJS.Telemetry.Constants.Location.APPBARIVK,
//      "");
controller.appbar.sticky = true;
controller.appbar.show();
}
}
else {
WinJS.Utilities.addClass(this.addButton, "hide");
WinJS.Utilities.addClass(this.clearButton, "hide");
this.pinButton.style.display = "none";
this.unpinButton.style.display = "none";
this._selectedItem = null;
//FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.MARKETMOVERS,
//      FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
//      FinanceAppJS.Telemetry.Constants.Location.APPBARRVK,
//      "");
controller.appbar.sticky = false;
controller.appbar.hide();
}
var item;
if (items && items.length === 1 && (item = items[0])) {
var tileData;
var idConstants = FinanceAppJS.Constants.IDs;
this._selectedItem = item;
if (WatchListManage.isFundTile(item)) {
tileData = {
entityObj: WatchListManage.getFundDetailsPageInfo(item),
type: idConstants.Fund,
symbol: item.Tkr || item.Scid,
displayName: item.Fn,
shortName: item.Tkr || item.Scid
};
}
else if (WatchListManage.isStockTile(item)) {
if (item.type === idConstants.Stock) {
tileData = {
entityObj: WatchListManage.getEntityPageInfo(item),
type: idConstants.Stock,
symbol: utils.getGlobalInstrumentSymbol(item),
displayName: item.Cmp,
shortName: item.Sym
};
}
else {
tileData = {
entityObj: WatchListManage.getETFDetailsPageInfo(item),
type: idConstants.Etf,
symbol: utils.getGlobalInstrumentSymbol(item),
displayName: item.Cmp,
shortName: item.Sym
};
}
}
// Create instances
if (this.pinUnpinManager === null) {
this.pinUnpinManager = new FinanceAppJS.PinUnpinManager(this.bottomEdgy, null, null, null, null, null, null, null, FinanceAppJS.Constants.Instrumentation.PinStart_Movers_Click);
}
var pinButtonCallback_binding = this.pinButtonCallback.bind(null, controller);
this.pinUnpinManager.initializeBottomSwipyPinButtons(tileData.entityObj,
tileData.type,
tileData.symbol,
tileData.displayName,
tileData.shortName,
pinButtonCallback_binding,
pinButtonCallback_binding);
}
},
setTileType: function setTileType(item) {
if (!item) {
return;
}
item.type = parseInt(item.type);
return item;
},
visibilityHandler: function visibilityHandler() {
if (document.msVisibilityState === "visible") {
if (this._selectedItem) {
this.pinUnpinManager.updateBottomSwipyPinButtons(utils.getGlobalInstrumentSymbol(this._selectedItem), this._selectedItem.type);
}
}
},
_shouldProceedTabNavigation: function _shouldProceedTabNavigation(controller) {
var groups = (controller && controller.Collection && controller.Collection.Groups) || [];
// If controller.collection doesn't exist OR If there is only one group/cluster, TAB navigation is not required.
if (groups.length < 2) {
return false;
}
return true;
},
_getGroupStartIndices: function _getGroupStartIndices(controller) {
var groups = controller.Collection.Groups;
var startIndex = 1;
var groupStartIndices = [];
for (var i = 0; i < groups.length; i++) {
if (groups[i].count === 0) {
continue;
}
groupStartIndices.push(startIndex - 1);
startIndex += groups[i].count;
}
return groupStartIndices;
},
_getGroupEndIndices: function _getGroupEndIndices(controller) {
var groups = controller.Collection.Groups;
var endIndex = 0;
var groupEndIndices = [];
for (var i = 0; i < groups.length; i++) {
if (groups[i].count === 0) {
continue;
}
endIndex += groups[i].count;
groupEndIndices.push(endIndex - 1);
}
return groupEndIndices;
},
_setTab: function _setTab(node, nodeParent, event) {
if (node && nodeParent) {
event.preventDefault();
event.cancelBubble = true;
node.focus();
this._createFocusContainer(nodeParent);
}
},
_createFocusContainer: function _createFocusContainer(node) {
var div = document.createElement("div");
div.setAttribute("class", "win-focusedoutline");
node.appendChild(div);
WinJS.Utilities.addClass(node, "win-focused");
},
onKeydown: function onKeydown(controller, event) {
if (event.key === "Tab" && !event.shiftKey) {
if (!this._shouldProceedTabNavigation(controller)) {
return;
}
var groupEndIndices = this._getGroupEndIndices(controller);
var grid = controller.currentView.grid;
var currentIndex = grid.currentItem.index;
var nextIndex = 0;
for (var j = 0; j < groupEndIndices.length; j++) {
if (currentIndex <= groupEndIndices[j] && groupEndIndices[j + 1]) {
nextIndex = groupEndIndices[j] + 1;
break;
}
}
if (nextIndex !== 0) {
var nextElm = grid.elementFromIndex(nextIndex);
this._setTab(nextElm, nextElm.parentNode, event);
}
else {
var nextElm = grid.elementFromIndex(nextIndex);
if (nextElm) {
nextElm.focus();
}
}
}
if (event.key === "Tab" && event.shiftKey) {
if (!this._shouldProceedTabNavigation(controller)) {
return;
}
var groupStartIndices = this._getGroupStartIndices(controller);
var grid = controller.currentView.grid;
var currentIndex = grid.currentItem.index;
var nextIndex = -1;
for (var j = groupStartIndices.length - 1; j > 0; j--) {
if (currentIndex >= groupStartIndices[j] && typeof groupStartIndices[j - 1] !== "undefined") {
nextIndex = groupStartIndices[j - 1];
break;
}
}
if (nextIndex !== -1) {
var nextElm = grid.elementFromIndex(nextIndex);
this._setTab(nextElm, nextElm.parentNode, event);
}
}
},
onNavigateAway: function onNavigateAway() {
document.removeEventListener("msvisibilitychange", this.visibilityHandler_binding);
if (this.bottomEdgy && this.clearSelectionHandler_binding) {
this.bottomEdgy.removeEventListener("beforehide", this.clearSelectionHandler_binding);
}
this.wlRoaming = null;
if (this.pinUnpinManager) {
this.pinUnpinManager.clean();
}
this.pinUnpinManager = null;
},
})
});
})();
