﻿(function () {
"use strict";
//Combined Data Provider for MyStocks, Movers, Index realtime data
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
EquityDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this._financeRoaming = new FinanceAppJS.Roaming();
}, {
queryServiceId: "HomepageEquityData",
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData,
_financeRoaming: null,
_wlSymbols: null,
isSnapped: false,
wlSymbols: null,
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
// always get data from roaming.
this.wlSymbols = this._financeRoaming.getWatchListSymbols();
urlParams.insert("stockSymbol", encodeURIComponent(this.wlSymbols.es));
urlParams.insert("fundSymbol", encodeURIComponent(this.wlSymbols.mft));
urlParams.insert("fundSecId", encodeURIComponent(this.wlSymbols.mfs));
urlParams.insert("rtSymbol", encodeURIComponent(this.wlSymbols.rt));
urlParams.insert("rtexchangeIds", encodeURIComponent(FinanceAppJS.Globalization.getDefaultExchange().code));
urlParams.insert("mvrCnt", encodeURIComponent(FinanceAppJS.Constants.MarketPage.MoversMaxCount));
urlParams.insert("lang", FinanceAppJS.Globalization.getLanguage());
urlParams.insert("locale", FinanceAppJS.Globalization.getMarket().toLowerCase());
return urlParams;
},
processMarketMoversData: function processMarketMoversData(moversData) {
var marketMoversData = {};
if (moversData && moversData.length) {
marketMoversData["gainers"] = utils.populateMoversData(moversData[0], "gainers");
marketMoversData["losers"] = utils.populateMoversData(moversData[0], "losers");
marketMoversData["active"] = utils.populateMoversData(moversData[0], "active");
}
return marketMoversData;
},
getQueryServiceOptions: function getQueryServiceOptions() {
var options = FinanceAppJS.DataProvider.DataProviderBase.prototype.getQueryServiceOptions.call(this);
options.customCacheKey = WatchListDataProvider.getWatchlistCustomCacheKey();
return options;
},
_populateMissing: function _populateMissing(wldata) {
if (!utils.isNonEmptyString(this.wlSymbols.all)) {
return;
}
var allSymbols = this.wlSymbols.all.split(",");
for (var i = 0; i < allSymbols.length; i++) {
/* Prefix: E1.st.Sym.E2 or T/M:tkr/scid*/
if (!wldata[i] && (allSymbols[i].indexOf(":") !== -1)) {
var tileData = { isBlankTile: true };
var currentSymbol = allSymbols[i];
var prefix = currentSymbol.split(":")[0];
prefix = prefix + ":";
var symbol = currentSymbol.split(":")[1];
var tickerSymbol = null;
if (prefix === 'M:' || prefix === 'T:') { //Fund
if (prefix === 'T:') {
tileData.Tkr = symbol;
tileData.type = FinanceAppJS.Constants.IDs.Fund;
} else if (prefix === 'M:') {
tileData.Scid = symbol;
tileData.type = FinanceAppJS.Constants.IDs.Fund;
}
}
else {
var split = symbol.split(".");
tileData.E1 = split[0];
tileData.St = split[1];
tileData.Sym = split[2];
tileData.E2 = split[3];
tileData.Eqsm = split[4];
if (prefix === FinanceAppJS.Constants.IDs.RoamingIndexPrefix) {
tileData.type = FinanceAppJS.Constants.IDs.Index;
} else if (prefix === FinanceAppJS.Constants.IDs.RoamingStockPrefix) {
tileData.type = FinanceAppJS.Constants.IDs.Stock;
} else if (prefix === FinanceAppJS.Constants.IDs.RoamingETFPrefix) {
tileData.type = FinanceAppJS.Constants.IDs.Etf;
}
}
wldata[i] = tileData;
} else if (wldata[i]) {
wldata[i].isBlankTile = false;
}
}
},
dataBuilder: function dataBuilder(data, response) {
data.Rtd = data.Rtd || [{}, {}, {}];
var considerResponse = response.isLocalResponse ?  ( this.wlSymbols.isDefault || FinanceAppJS.Globals.FRE ) : true ;  // Respect FRE data response only if it is FRE or in case of Default Watchlist.
if (data.Wl && considerResponse) {
data.Wl = this._financeRoaming.getWatchListDataSorted(this.wlSymbols, data.Wl);
}
else {
data.Wl = [];
}
this._populateMissing(data.Wl);
data.Wl = WatchListDataProvider.processWatchListData(data.Wl, false, this.isSnapped);
data.Mov = this.processMarketMoversData(data.Mov);
var wlSymbols = this.wlSymbols;
var tickers = data.Rtd.map(function (item) {
return utils.getIndicesSymbol(item);
});
var indicesData = [];
var emptyData = { Lp: null, Ch: null, Chp: null, Op: null, Pp: null, Dl: null, Dh: null, Yl: null, Yh: null, Ld: null, Lt: null };
// Adding a check for returning only requested items, and also making sure that the displayname of the index matches with the actual data,
// since the localisation key is from the json file, and not in the dataObject. (Bugs 381025 and 380762)
wlSymbols.rt.forEach(function (item, index) {
var itemIndex = tickers.indexOf(item);
var dataObject = {};
var indexName = utils.getResourceString(wlSymbols.rtNames[index]);
dataObject.displayName = indexName;
dataObject.indexSymbol = item;
if (itemIndex >= 0) {
utils.deepCopy(data.Rtd[itemIndex], dataObject);
indicesData.push(dataObject);
}
else {
utils.deepCopy(emptyData, dataObject);
indicesData.push(dataObject);
}                       
});
if (data && response) {
data.lastUpdateTime = response.lastUpdateTime;
data.isLocalResponse = response.isLocalResponse;
}
data.Rtd = indicesData;
return data;
}
}, {
getDefaultProviderOptions: function getDefaultProviderOptions(roaming) {
return {
isSnapped: false,
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData,
wlSymbols: roaming.getWatchListSymbols()
};
}
})
});
})();
(function () {
"use strict";
//Combined Data Provider for MyStocks, Movers, Index realtime data
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
TopEdgyWatchlistDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.EquityDataProvider, function (options) {
FinanceAppJS.DataProvider.EquityDataProvider.call(this, options);
}, {
refreshInterval: 0,
_updateDateTime: false,
})
});
})();
(function () {
'use strict';
//Rates Data Provider
var FinanceAppJS2 = WinJS.Namespace.define("FinanceAppJS.DataProvider", {
HomeRatesDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
}, {
queryServiceId: "HomeRatesData",
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("localizeFor", FinanceAppJS.Globalization.getMarket());
return urlParams; 
},
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.HomePageICCBEWM,
dataBuilder: function dataBuilder(data) {
var dataObject = {};
var defaultData = { label: "-", value: "-", change: "-", pchange: "-" };
dataObject.list = [];
dataObject.list.push({});
dataObject.list[0].titleClass = "logo ratesGlyph";                
dataObject.list[0].ratesData = [];
dataObject.list[0].title = utils.getResourceString("avgTitle");
dataObject.list[0].classToAppend = "averageRates";
dataObject.list[0].templateClass = "ratesItems";
dataObject.list[0].separator = "hidden";
dataObject.list[0].itemModuleInfo = {
fragmentPath: "/pages/MarketToday/html/MarketTodayTemplates.html",
templateId: "RatesItemTemplate"
};
dataObject.list[0].moduleInfo = {
fragmentPath: "/pages/MarketToday/html/MarketTodayTemplates.html",
templateId: "AcrossMarketRatesTemplate"
};
for (var j = 0; j < FinanceAppJS.Constants.HomePage.USAcrossMarketRatesMaxCount; j++) {
dataObject.list[0].ratesData[j] = data && data.length > j && data[j] ? {
label: data[j].ProductName,
value: data[j].Rate,
change: utils.getResourceString("Last Week Header"),
pchange: FinanceAppJS.Utils.Formatters.percentFormatter(data[j].PrevRate, 3),
} : defaultData;
}
return dataObject.list;
}
})
});
})();
(function () {
'use strict';
//Today Fund Picks Data Provider
var FinanceAppJS2 = WinJS.Namespace.define("FinanceAppJS.DataProvider", {
TodayFundPicksDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
}, {
queryServiceId: "TodayFundPicksData",
refreshInterval: 0,
dataBuilder: function dataBuilder(data) {
var dataObject = {};
dataObject.list = [];
var fundPicksData = {};
var groupedData = {};
for (var i = 0 ; i < data.length; i++) {
var f = data[i];
fundPicksData[f.SId] = f;
f.Fnds = f.Fnds || [];
f.data1 = f.Fnds[0] || {};
f.data2 = f.Fnds[1] || {};
f.data3 = f.Fnds[2] || {};
f.data1.Fn = utils.formatString(f.data1.Fn);
f.data2.Fn = utils.formatString(f.data2.Fn);
f.data3.Fn = utils.formatString(f.data3.Fn);
f.logo = "/pages/FundBrowse/images/" + f.Cat.replace(/[\W]/g, "") + ".png";
f.ReturnTitle = utils.getResourceString("FundBrowse_Col_Return" + (f.Cat === "Performance" ? "_" + f.SId : ""));
}
for (var j = 0; j < data.length; j++) {
var d = data[j];
if (!groupedData.hasOwnProperty(d.Cat)) { groupedData[d.Cat] = []; }
groupedData[d.Cat].push(d);
}
dataObject.list.push([fundPicksData["topperformerytd"]]);
dataObject.list.push([fundPicksData["topperformer5y"]]);
dataObject.list.push(groupedData["Highest Rated"]);
dataObject.list.push(groupedData["Pre-Screened"]);
return dataObject.list.map(function (item) { return item[0]; });
}
})
});
})();
(function () {
'use strict';
//Across Market Data Provider
var FinanceAppJS2 = WinJS.Namespace.define("FinanceAppJS.DataProvider", {
AcrossMarketDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
}, {
queryServiceId: "TodayCCBWRData",
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.HomePageICCBEWM,
getQueryServiceParams: function getQueryServiceParams() {
var currency = FinanceAppJS.Globalization.getDefaultCurrency();
var market_data = FinanceAppJS.Globalization.getMarket();
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams["params"] = "localizeFor=" + market_data;
return urlParams;
},
dataBuilder: function dataBuilder(data) {                
var dataObject = {};
var defaultData = { label: "-", value: "-", change: "-", pchange: "-" };
var valueconverter = function (obj) { return obj ? parseFloat(obj) : obj; };
var changeconverter = valueconverter;
var pchangeconverter = changeconverter;
dataObject.list = [];
for (var i = 0; i < FinanceAppJS.Constants.HomePage.ccClusterLength; i++) {
dataObject.list[i] = {};
}
dataObject.list[0].titleClass = "logo logoAlign currencyGlyph";
dataObject.list[0].templateClass = "currencyItems";
dataObject.list[0].title = utils.getResourceString("Currencies");
dataObject.list[0].classToAppend = "currencies";
dataObject.list[0].separator = "hidden";
dataObject.list[0].ccItemArray = [];
dataObject.list[0].itemModuleInfo = {
fragmentPath: "/pages/MarketToday/html/MarketTodayTemplates.html",
templateId: "currencyCommodityItemTemplate"
};
var dataObj = FinanceAppJS.DataProvider.MarketDataProvider.getCurrencies(data.Cur, FinanceAppJS.Constants.HomePage.USAcrossMarketCurrencyMaxCount, true);
var currencyInfo = FinanceAppJS.CurrencyInfo.getCurrencyInfo();
if (dataObj.length && currencyInfo) {
for (var i = 0; i < FinanceAppJS.Constants.HomePage.USAcrossMarketCurrencyMaxCount; i++) {                            
try {
var currData = dataObj[i];
var item = currencyInfo[currData.ticker];
var desc = utils.getResourceString(item.country) + " " + utils.getResourceString(item.currency);
var ratio = (currData.num === currData.ticker) ? desc + "/" + currData.den : currData.num + "/" + desc;
dataObject.list[0].ccItemArray[i] = {
label: ratio,
value: currData.rate,
change: currData.ch,
pchange: currData.pc,
};
}
catch (e) {
dataObject.list[0].ccItemArray[i] = defaultData;
}                            
}                    
}
else {
dataObject.list[0].ccItemArray = null;                                        
dataObject.list[0].title = null;
}
dataObject.list[1].titleClass = "logo logoAlign commoditiesGlyph";
dataObject.list[1].templateClass = "commodityItems";
dataObject.list[1].title = utils.getResourceString("Commodities");
dataObject.list[1].classToAppend = "commodities";
dataObject.list[1].separator = "hidden";
dataObject.list[1].ccItemArray = [];
dataObject.list[1].itemModuleInfo = {
fragmentPath: "/pages/MarketToday/html/MarketTodayTemplates.html",
templateId: "currencyCommodityItemTemplate"
};
for (var j = 0; j < FinanceAppJS.Constants.HomePage.USAcrossMarketCommoditiesMaxCount; j++) {
dataObject.list[1].ccItemArray[j] = data.Cmdt && data.Cmdt.length > j && data.Cmdt[j] ? {
label: data.Cmdt[j].Dn,
value: valueconverter(data.Cmdt[j].Lp),
change: changeconverter(data.Cmdt[j].C),
pchange: pchangeconverter(data.Cmdt[j].Cp),
} : defaultData;
}
return dataObject.list;
}
})
});
})();
(function () {
'use strict';
//Data Provider for Alternate Across Market with just Currency and Commodities
var FinanceAppJS2 = WinJS.Namespace.define("FinanceAppJS.DataProvider", {
AcrossMarketCCDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
}, {
queryServiceId: "TodayCCBWRData",
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.HomePageICCBEWM,
_locale: null,
_currency: null,
getQueryServiceParams: function getQueryServiceParams() {
var market_data = FinanceAppJS.Globalization.getMarket();
this._currency = FinanceAppJS.Globalization.getDefaultCurrency();
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams["params"] = "localcurrency=" + encodeURIComponent(this._currency) + "&localizeFor=" + market_data;
return urlParams;
},
dataBuilder: function dataBuilder(data) {
var dataObject = {};
var defaultData = { label: "-", value: "-" };
var valueconverter = function (obj) { return obj ? parseFloat(obj) : obj; };
var changeconverter = valueconverter;
var pchangeconverter = changeconverter;
var currency = this._currency;
var headerStr = utils.getResourceString("PER_CURRENCY");
data = data || {};
data.Cur = data.Cur || [];
data.Cmdt = data.Cmdt || [];
dataObject.list = [{}, {}];
dataObject.list[0].title = utils.getResourceString("Currencies");
dataObject.list[0].classToAppend = "currencies";
dataObject.list[0].className = "currencyItems items";
dataObject.list[0].itemTemplate = {
fragmentPath: "/pages/MarketToday/html/MarketTodayTemplates.html",
templateId: "AcrossMarket_CC_CurrencyItem"
};
var itemArray = [];
var currencyInfo = FinanceAppJS.CurrencyInfo.getCurrencyInfo();
var dataObj = FinanceAppJS.DataProvider.MarketDataProvider.getCurrencies(data.Cur, FinanceAppJS.Constants.HomePage.NonUSCurrenciesCount, false);
if (dataObj.length && currencyInfo) {
for (var curr in dataObj) {
var currData = dataObj[curr];
var item = currencyInfo[currData.ticker];
var desc = utils.getResourceString(item.currency) + " " + item.symbol;
var ratio = (currData.num === currData.ticker) ? desc + "/" + currData.den : currData.num + "/" + desc;
itemArray.push({
flagClass: "flag " + FinanceAppJS.Utils.getCountryFlagClass(item.countryCode, true),
country: utils.getResourceString(item.country),
conversion: ratio,
rate: currData.rate
});
}
}
else {
itemArray = null;
dataObject.list[0].title = null;
}
dataObject.list[0].data = itemArray;
headerStr = utils.getResourceString("IN_CURRENCY");
dataObject.list[1].title = utils.getResourceString("Commodities") + headerStr.format(utils.getResourceString("USD"));
dataObject.list[1].classToAppend = "commodities";
dataObject.list[1].className = "items";
dataObject.list[1].itemTemplate = {
fragmentPath: "/pages/MarketToday/html/MarketTodayTemplates.html",
templateId: "AcrossMarket_CC_CommodityItem"
};
data.Cmdt = data.Cmdt.filter(function (value, n) { return n < FinanceAppJS.Constants.HomePage.NonUSCommoditiesCount; });
if (data.Cmdt && data.Cmdt.length) {
dataObject.list[1].data = data.Cmdt.map(function (value) {
if (!value.Dn) {
return { logo: "", label: "-" };
}
return {
logo: value.EnDn.replace(/[\W]/g, "").replace("Com_", "").toLowerCase() + "Glyph",
label: value.Dn,
value: valueconverter(value.Lp),
change: changeconverter(value.C),
pchange: pchangeconverter(value.Cp),
alt: value.Dn,
};
});
}
else {
dataObject.list[1].data = null;
dataObject.list[1].title = null;
}
return dataObject.list;
}
})
});
})();
