﻿(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
EditorialNewsDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this.isPreview = FinanceAppJS.Settings.cmsPreviewEnabled;
this.market = options.market || FinanceAppJS.Globalization.getMarket();
var collectionGroupMap = PlatformJS.Services.appConfig.getDictionary("EditorialCollectionIdMap");
if (collectionGroupMap) {
this._newsItemsCollectionGroup = collectionGroupMap.getString("EditorialNews");
this._magazineItemsCollectionGroup = collectionGroupMap.getString("MagazineNews");
this._videoItemsCollectionGroup = collectionGroupMap.getString("VideoNews");
this._moneyItemsCollectionGroup = collectionGroupMap.getString("MoneyNews");
}
this._imageService = this._imageService || new Platform.ImageService(this.imageCacheId);
this._imageCache = this._imageCache || new Platform.DataServices.CacheService(this.imageCacheId);
this._qsOptions = new Platform.DataServices.QueryServiceOptions();
this._qsOptions.useCompatDataModel = true;
this._qsOptions.enableAutoRefresh = false;
this._qsOptions.queueResponseInMemory = true;
this._postLowRes_binding = this._postLowRes.bind(this);
this._postHighRes_binding = this._postHighRes.bind(this);
}, {
_qsOption: null,
_imageService: null,
_prev_response_promise: null,
/* Override ProcessResponseData of Provider to pre-fetch hero image data */
processResponseData: function processResponseData(data, response) {
if (response.isLocalResponse) {
utils.setDeepObject(data, this.fetchImages.highres, null);
utils.setDeepObject(data, this.fetchImages.lowres, null);
return WinJS.Promise.as(data);
}
// some markets dont have hero data like ar-SA. no need to fetch images for those markets.
if (data.heroCluster) {
if (this._prev_response_promise) {
this._prev_response_promise.cancel();
}
return (this._prev_response_promise = this._processImages(data, response));
} else {
/* for markets like ar-SA, there is no heroCluster. so save today.js explicitly in that case. */
Platform.QueryServiceResponseQueue.instance.commitResponseToCache([response.responseId]);
}
return FinanceAppJS.DataProvider.DataProviderBase.prototype.processResponseData.call(this, data, response);
},
getQueryServiceOptions: function getQueryServiceOptions() {
Platform.QueryServiceResponseQueue.instance.clearQueue();
var options = FinanceAppJS.DataProvider.DataProviderBase.prototype.getQueryServiceOptions.call(this);
options.queueResponseInMemory = true;
this._cleanUpPromises();
return options;
},
_cleanUpPromises: function _cleanUpPromises() {
var inProcImages = this._inProcessingImages;
if (inProcImages) {
inProcImages.lowRes.promise && inProcImages.lowRes.promise.cancel();
inProcImages.highRes.promise && inProcImages.highRes.promise.cancel();
inProcImages.joinPromise && inProcImages.joinPromise.cancel();
this._inProcessingImages = null;
}
if (this._prev_response_promise) {
this._prev_response_promise.cancel();
this._prev_response_promise = null;
}
},
_inProcessingImages: null,
/*
{
responseID: ""
lowRes: {url: "", promise: },
highRes: {url: "", promise: }
joinPromise:
lastProc: data
}
*/
_postLowRes: function _postLowRes(data, newsResponse, key, url, lowResResponse) {
var inProcImages = this._inProcessingImages;
var that = this;
return Platform.QueryServiceResponseQueue.instance.commitResponseToCache([lowResResponse.responseId, newsResponse.responseId]).
then(function lowres_done() {
if (!inProcImages.highRes.rendered && !inProcImages.lowRes.rendered) {
data.heroCluster.imageData.imageSize = {
width: data.heroCluster.lowResImage.width,
height: data.heroCluster.lowResImage.height
};
var entry = that._imageCache.findEntry(url, true);
utils.setDeepObject(data, that.fetchImages.orig, (entry && entry.fileName) || url);
utils.setDeepObject(data, that.fetchImages.highres, null);
utils.setDeepObject(data, that.fetchImages.lowres, null);
that._scheduleEvent("dataavailable", data);
inProcImages.lowRes.rendered = true;
inProcImages.lastProc = data;
// Set lastfetchData of the provider as well.
that._lastFetchData = data;
}
return "lowres";
}, function lowres_error(e) {
return "lowreserror";
});
},
imageCacheId: "FinanceHeroData",
_postHighRes: function _postHighRes(data, newsResponse, key, url, highResResponse) {
var that = this;
var inProcImages = this._inProcessingImages;
// return WinJS.Promise.as({});
return Platform.QueryServiceResponseQueue.instance.commitResponseToCache([highResResponse.responseId, newsResponse.responseId])
.then(function highres_done() {
// Set the Low res URL and send new data.
if (!inProcImages.highRes.rendered) {
data.heroCluster.imageData.imageSize = {
width: data.heroCluster.image.width,
height: data.heroCluster.image.height
};
var entry = that._imageCache.findEntry(url, true);
utils.setDeepObject(data, that.fetchImages.orig, (entry && entry.fileName) || url);
utils.setDeepObject(data, that.fetchImages.highres, null);
utils.setDeepObject(data, that.fetchImages.lowres, null);
inProcImages.highRes.rendered = true;
inProcImages.lastProc = data;
}
return "highres";
}, function highres_erro(e) {
if (!inProcImages.lowRes.rendered) {
return "highreserror";
}
});
},
fetchImages:
{
orig: ["heroCluster", "imageData", "backgroundImage", "url"],
lowres: ["heroCluster", "imageData", "backgroundImage", "lowResolutionUrl"],
highres: ["heroCluster", "imageData", "backgroundImage", "highResolutionUrl"]
},
_processImages: function _processImages(data, newsResponse) {
var imgKey = this.fetchImages;
/* For FRE or cached data, no need to follow the imagePromise path and remove lowres image for cachedResponse */
/* set the lowres if high res is not in memory */
var that = this;
this._imagesPromises = [];
this._origImagesToDownload = [];
this._imgInfo = [];
var imgInfo = {
origUrl: utils.getDeepObject(data, imgKey.orig),
lowResUrl: utils.getDeepObject(data, imgKey.lowres),
};
var qsOptions = this._qsOptions;
var that = this;
var imagePromise = this._imageService.hasEntryAsync(imgInfo.origUrl).then(
function (isAvailable) {
if (isAvailable) {
var entry = that._imageCache.findEntry(imgInfo.origUrl, true);
if (entry && entry.fileName) {
utils.setDeepObject(data, imgKey.orig, entry.fileName);
}
utils.setDeepObject(data, imgKey.lowres, null);
utils.setDeepObject(data, imgKey.highres, null);
// Save the newsResponse to disk now. 
// If original URL is in cache, it might be the same response or a 
// different response (new news items) with the same hero image.
var prom = WinJS.Promise.as({});
if (!newsResponse.isCachedResponse) {
prom = Platform.QueryServiceResponseQueue.instance.commitResponseToCache([newsResponse.responseId]);
}
return prom.then(function () {
return data;
});
} else {
// Check if there is a lowRes res download in progress for the responseId.
// If available just wait on it. Else create one.
var inProcImages = that._inProcessingImages;
if (inProcImages && inProcImages.responseId === newsResponse.responseId) {
return inProcImages.joinPromise;
} else {
/* clean up old promises if any */
if (inProcImages) {
inProcImages.lowRes.promise && inProcImages.lowRes.promise.cancel();
inProcImages.highRes.promise && inProcImages.highRes.promise.cancel();
inProcImages.joinPromise.cancel();
}
that._inProcessingImages = {
responseId: newsResponse.responseId,
lowRes: {
rendered: false,
promise: WinJS.Promise.as({}) 
/* disabling lowres loading 
that._imageService.downloadImageAsync(imgInfo.lowResUrl, qsOptions).then(
function (lowResResponse) { return that._postLowRes_binding(data, newsResponse, imgKey.lowres, imgInfo.lowResUrl, lowResResponse); },
function (error) {
// Continue loading high res *
return WinJS.Promise.wrap("lowreserror");
})*/
},
highRes: {
rendered: false,
promise: that._imageService.downloadImageAsync(imgInfo.origUrl, qsOptions).then(
function (highResResponse) { return that._postHighRes_binding(data, newsResponse, imgKey.orig, imgInfo.origUrl, highResResponse); },
function (error) {
return WinJS.Promise.wrap("highreserror");
})
},
lastProc: null
};
that._inProcessingImages.joinPromise = WinJS.Promise.join([that._inProcessingImages.lowRes.promise, that._inProcessingImages.highRes.promise]);
return that._inProcessingImages.joinPromise;
}
}
});
return imagePromise.then(function (values) {
if (values && values.length === 2) {
if (values[0] === "lowreserror" && values[1] === "highreserror") {
/* clear queryserviceresponsequeue and cancel current promise */
Platform.QueryServiceResponseQueue.instance.clearQueue();
that._prev_response_promise.cancel();
} else {
return that._inProcessingImages.lastProc;
}
} else if (values) {
return values;
} else {
// This should never happen. values should always be present.
debugger;
}
}, function (values) {
});
},
_articleInfos: [],
_isPreview: false,
enableFRE: true,
bypassCache: false,
_articleIndexCounter: -1,
refreshInterval: FinanceAppJS.Constants.RefreshIntervals.EditorialNewsDataProvider, // In MilliSeconds.
_newsItemsCollectionGroup: "",
_magazineItemsCollectionGroup: "",
_videoItemsCollectionGroup: "",
_moneyItemsCollectionGroup: "",
queryServiceId: "HomepageNewsData",
entities: null,
isPreview: {
get: function get(value) {
return this._isPreview;
},
set: function set(value) {
this._isPreview = value;
if (this._isPreview) {
this.bypassCache = true;
this.enableFRE = false;
}
else {
this.bypassCache = false;
this.enableFRE = true;
}
}
},
forceRefresh: function forceRefresh() {
FinanceAppJS.DataProvider.DataProviderBase.prototype.getData.call(this, { forceRefresh: true });
},
market: null,
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams['market'] = this.market;
urlParams['previewString'] = this.isPreview ? encodeURIComponent("/preview") : "";
if (this.isPreview) {
urlParams['previewparams'] = PlatformJS.Services.appConfig.getDictionary("CMSParams").getString("previewMode");
}
return urlParams;
},
//destination field will be of format //application/view?entitytype=article&pageid=33265626
getEditorialArticleId: function getEditorialArticleId(destination) {
var destinationURL = destination.toLowerCase();
var articleId = null;
var queryParams = utils.getUrlParameters(destinationURL);
//check if target entitytype is article
if (queryParams.entitytype === "article") {
if (queryParams.contentid) {
articleId = queryParams.pageid + '/' + queryParams.contentid;
}
else {
articleId = queryParams.pageid;
}
}
return articleId;
},
_createHeroData: function _createHeroData(jsonResponse, isLocalResponse ) {
var heroImageData = null;
if (jsonResponse && utils.isNonEmptyArray(jsonResponse.clusters) && jsonResponse.clusters[0].hero) {
heroImageData = jsonResponse.clusters[0].hero;
//set the isPriview flag
heroImageData.isPreview = this.isPreview;
var images = utils.getDeepObject(heroImageData, ["heroImage", "image", "images"]);
if (!images) {
return null;
}
var image = images.filter(function (item) { return item.name === 'original'; })[0] || {};
var lowResImage = images.filter(function (item) { return item.name === 'lowRes'; })[0] || {};
if (image && utils.isNonEmptyString(image.url)) {
heroImageData.image = image;
}
else {
return null;
}
if (lowResImage) {
heroImageData.lowResImage = lowResImage;
}
heroImageData.content = heroImageData.content || {};
if (heroImageData.content.headline && isLocalResponse) {
heroImageData.content.headline = utils.getResourceString("heroHeadline");
if (heroImageData.content.source) {
heroImageData.content.source.name = "";
}
heroImageData.content.isFRE = true;
}
//trim the news headline 
if (heroImageData.content.headline) {
heroImageData.content.headline = toStaticHTML(heroImageData.content.headline);
}
//Check the destination url format also start article fetching if article id is provided.
if (heroImageData.destination) {
heroImageData.content.isAbsoluteURL = utils.isAbsoluteURL(heroImageData.destination);
// check if given url is not absolute
if (heroImageData.content.isAbsoluteURL === false) {
// find the article id
heroImageData.content.articleid = this.getEditorialArticleId(heroImageData.destination);
// fetch the config variable UseArticleReader
var useArticleReader = FinanceAppJS.Utils.UseArticleReader();
//Prefetch the article if it is not preview data and we have an article id(there is an associated artilce in Bedrock and we have enabled article reader)
if (heroImageData.content.articleid && useArticleReader) {
this._articleIndexCounter++;
heroImageData.content.articleIndex = this._articleIndexCounter;
heroImageData.content.articleUrl = null;
// Populate ArticleInfos array.
// This is required for previous/Next in article reader and actionKeys is required for end of article.
this._articleInfos.push({ articleId: heroImageData.content.articleid, actionKeys: [] });
}
}
else {
// absolute url
heroImageData.content.articleUrl = heroImageData.destination;
heroImageData.content.articleid = null;
}
}
var heroImage = heroImageData.heroImage;
if (isLocalResponse) {
heroImage.image.caption = utils.getResourceString("heroAbstract");
heroImage.image.attribution = PlatformJS.Services.configuration.getString("FREImageCopyRight");                       
}
heroImageData.imageData = {
fontColor: "headerFont" + heroImage.headerFont,
attributionColor: "headerFont" + heroImage.headerFont,
imageSize: { width: image.width, height: image.height },
anchorPoint: "anchor" + heroImage.anchorPoint,
backgroundImage: { lowResolutionUrl: lowResImage.url, highResolutionUrl: image.url, url: image.url },
attributionData: {
attributionText: utils.isNonEmptyString(heroImage.image.attribution) ? utils.addLtrMark(toStaticHTML(heroImage.image.attribution)) : "",
caption: utils.isNonEmptyString(heroImage.image.caption) ? utils.addLtrMark(toStaticHTML(heroImage.image.caption)) : "",
},
showGradient: true,
};
}
return heroImageData;
},
_createEditorialNewsData: function _createEditorialNewsData(jsonResponse, entityListName, type) {
var data = utils.parseMagazineResponse(jsonResponse, entityListName, type);
if (data && data.newsItems) {
var newsItems = data.newsItems;
// Also set the sourceImageUrl to "";
for (var i = 0, len = newsItems.length; i < len; i++) {
var newsArticle = newsItems[i];
if (newsArticle.type === "article") {
this._articleIndexCounter++;
newsArticle.articleIndex = this._articleIndexCounter;
}
if (!newsArticle.sourceImageUrl) {
newsArticle.sourceImageUrl = "";
}
if (newsArticle.editorial && newsArticle.type === "article" && newsArticle.articleid) {
this._articleInfos.push({ articleId: utils.getArticleReaderId(newsArticle) });
}
}
return newsItems;
} else {
return [];
}
},
dataBuilder: function dataBuilder(jsonResponse, response) {
this._articleIndexCounter = -1;
var editorialData = {};
this._articleInfos = [];
PlatformJS.Ads.setAdConfig(jsonResponse.adsData);
editorialData.heroCluster = this._createHeroData(jsonResponse, response.isLocalResponse);
editorialData.newsItems = this._createEditorialNewsData(jsonResponse, [this._newsItemsCollectionGroup]);
editorialData.magazineItems = this._createEditorialNewsData(jsonResponse, [this._magazineItemsCollectionGroup]);
editorialData.videoItems = this._createEditorialNewsData(jsonResponse, [this._videoItemsCollectionGroup]);
editorialData.moneyItems = this._createEditorialNewsData(jsonResponse, [this._moneyItemsCollectionGroup], "money");
editorialData.articleInfos = this._articleInfos;
editorialData.cacheID = "Editorial"; // cacheID may not be available because of queryservice response queue.
editorialData.dataCacheID = this.getQueryServiceURL();
editorialData.isLocalResponse = response.isLocalResponse;
editorialData.uniqueResponseID = response.isLocalResponse ? null : response.responseId;
return editorialData;
},
}, {
})
});
})();
(function () {
"use strict";
var FinanceAppJS2 = WinJS.Namespace.define("FinanceAppJS.DataProvider", {
BingStockNewsDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.BingNewsDataProvider, function (options) {
// The difference between this and the base class is of the Options
// only one Instance of Data provider has only one set of options working
// changing options is not a good idea so created a derived class for a new set of options.
// DO NOT REVERT THIS - used by stock details page news cluster.
FinanceAppJS.DataProvider.BingNewsDataProvider.call(this, options);
}, {
}, {
})
});
})();
(function () {
"use strict";
var FinanceAppJS2 = WinJS.Namespace.define("FinanceAppJS.DataProvider", {
BingNewsDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
var newsDict = PlatformJS.Services.appConfig.getDictionary("BdiNewsVersions");
var currentVersion = newsDict.getString("CurrentVersion");
this.queryServiceId = newsDict.getDictionary(currentVersion).getString(options.queryType);
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
if (!options || !options.queryType) {
throw new Exception("options or query Type is a required parameter.");
}
this.dataTransformer = new CollectionProvider.NewsDataTransformer(currentVersion);
this.query = "";
this.market = "";
if (options.query) {
this.query = options.query;
}
if (options.market) {
this.market = options.market;
}
else {
this.market = FinanceAppJS.Globalization.getMarket();
}
}, {
entities: null,
refreshInterval: 0,
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("query", encodeURIComponent(this.query));
urlParams.insert("offset", encodeURIComponent(this.market));
urlParams.insert("count", encodeURIComponent(FinanceAppJS.Constants.NewsCluster.NewsResultsCount));
urlParams.insert("issortbydate", encodeURIComponent(FinanceAppJS.Constants.NewsCluster.NewsSortByDate));
return urlParams;
},
dataBuilder: function dataBuilder(data, response) {
var jsData = null;
if (data) {
jsData = PlatformJS.Utilities.convertManageToJSON(data);
}
for (var j = 0; j < jsData.newsResults.length; j++) {
jsData.newsResults[j].publishTime = CommonJS.Utils.convertBDITimeToFriendlyTime(jsData.newsResults[j].publishTimeRaw);
}
return {
native: true,
newsItems: jsData ? jsData.newsResults : null,
cacheID: this.getCacheId(),
dataCacheID: this.getQueryServiceURL(),
isLocalResponse: response.isLocalResponse,
uniqueResponseID: response.isLocalResponse ? null : response.responseId
};
}
}, {
})
});
})();