﻿(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS", {
NewsRSSSources: WinJS.Class.define(
/// <summary>
/// Constructor for a simple IPage class that controls the multisizelistview
/// </summary>
/// <param name="state">
/// The state object passed to WinJS.Navigation.navigate or the state object
/// preserved when the page has been navigated away from
/// </param>
function (state) {
var that = this;
FinanceAppJS.Utils.Logger.LogPerformance("NewsRSSSources.js", "page_load", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);
that._clusters = new WinJS.Binding.List();
that.loadPageData();
// Attach action to Help button
utils.tryAttachHelpButtonAction();
},
{
_sourcesList: null,
_sourcesQuery: null,
_sourcesProvider: null,
_sourcesProviderConfig: null,
createSourcesProviderAndConfig: function createSourcesProviderAndConfig() {
var that = this;
that._sourcesProvider = new AppEx.Common.RichBrowse.EntityCollection.RichBrowseProvider();
that._sourcesProviderConfig = PlatformJS.Collections.createStringDictionary();
// Set the gallery to rss_ <locale>
var gallery = that._getRSSMarketCode();
gallery = gallery.toLowerCase();
that._sourcesProviderConfig.insert('appName', 'Finance');
that._sourcesProviderConfig.insert('pageName', 'Sources');
that._sourcesProviderConfig.insert('market', gallery);
return that._sourcesProvider.initializeAsync(that._sourcesProviderConfig);
},
_getRSSMarketCode: function _getRSSMarketCode() {
var currentMarket = FinanceAppJS.Globalization.getMarket();
var dictString = null;
/* Override below markets for worldwide news in the local language */
dictString = PlatformJS.Services.configuration.getDictionary("WorldWideNewsOverrideMap").getString(currentMarket.toLowerCase());
if (dictString && dictString !== "") {
currentMarket = dictString;
}
return currentMarket;
},
createSourcesQuery: function createSourcesQuery() {
var that = this;
that._sourcesQuery = new AppEx.Common.EntityCollection.Query.EntityQuery();
that._sourcesQuery.attributes.push('displayname');
that._sourcesQuery.attributes.push('win8_image');
that._sourcesQuery.attributes.push('rss_names');
that._sourcesQuery.attributes.push('rss_urls');
that._sourcesQuery.attributes.push('alphaorder');
that._sourcesQuery.attributes.push('featured_url');
that._sourcesQuery.attributes.push('channel_id');
that._sourcesQuery.attributes.push('featured_category');
},
createSourceContent: function createSourceContent(entity) {
return {
categoryKey: "sourcesContent",
template: "sourceItemTemplate",
data: {
alpha: entity.attributes.alphaorder,
displayname: entity.attributes.displayname,
win8_image: entity.attributes.win8_image,
id: entity.id,
title: entity.title,
rss_names: entity.attributes.rss_names,
rss_urls: entity.attributes.rss_urls
},
moduleInfo: {
isInteractive: false,
fragmentPath: "/shared/html/templates.html",
templateId: "sourceItemTemplate"
}
};
},
/// <summary>
/// Creates a List of RSS 
/// </summary>
createSourcesList: function createSourcesList(feedEntities) {
FinanceAppJS.Utils.Logger.LogPerformance("NewsRSSSources.js", "SourcesListPopulate", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);
var that = this;
if (feedEntities) {
that._sourcesList = [];
for (var i = 0; i < feedEntities.entities.length; i++) {
var entity = feedEntities.entities[i];
var content = that.createSourceContent(entity);
that._sourcesList.push(content);
}
that._sourcesList.sort(function (a, b) {
var alphaA = a.data.alpha;
var alphaB = b.data.alpha;
if (alphaA < alphaB) {
return -1;
} else if (alphaA > alphaB) {
return 1;
} else {
return 0;
}
});
that._clusters.push(that.createSourceCluster(0, " ", that._sourcesList));
}
FinanceAppJS.Utils.Logger.LogPerformance("NewsRSSSources.js", "SourcesListPopulate", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
},
sourceNavigate: function sourceNavigate(source) {
var isRss = false;
if (source && source.rss_urls && source.rss_names) {
source.feedNamesArray = source.rss_names.split(',');
source.feedUrlsArray = source.rss_urls.split(',');
if (source.feedNamesArray.length === source.feedUrlsArray.length) {
isRss = true;
}
}
if (isRss) {
// Go to RSS source pano
PlatformJS.Navigation.navigateToChannel("NewsRSSReader", { entity: source });
} 
},
itemInvoked: function itemInvoked(e) {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.Cluster.SOURCES,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
e.item.data.displayname);
this.sourceNavigate(e.item.data);
},
createSourceCluster: function createSourceCluster(position, title, items) {
var that = this;
return {
clusterPosition: position,
clusterKey: title,
clusterTitle: title,
clusterContent: {
contentControl: "CommonJS.Immersive.ItemsContainer",
contentOptions: {
onitemclick: function onitemclick(e) {
that.itemInvoked(e);
},
itemDataSource: new WinJS.Binding.List(items).dataSource,
className: "platformListLayout"
}
}
};
},
loadPageData: function loadPageData() {
var that = this;
CommonJS.Progress.showProgress(CommonJS.Progress.headerProgressType);
that.createSourcesProviderAndConfig().then(
function () {
that.createSourcesQuery();
that._sourcesProvider.queryAsync(that._sourcesQuery).then(
function (val) {
that.sourcesFetchCompleted(val);
},
function (val) {
// handle failure
var errType = FinanceAppJS.Globals.internetAvailable ? CommonJS.Error.STANDARD_ERROR : CommonJS.Error.NO_INTERNET;
CommonJS.Error.showError(errType, that.loadPageData.bind(that));
});
},
function (err) {
// handle failure
var errType = FinanceAppJS.Globals.internetAvailable ? CommonJS.Error.STANDARD_ERROR : CommonJS.Error.NO_INTERNET;
CommonJS.Error.showError(errType, that.loadPageData.bind(that));
});
},
sourcesFetchCompleted: function sourcesFetchCompleted(val) {
var that = this;
that.createSourcesList(val);
var panoControl = PlatformJS.Utilities.getControl("newsRSSPanorama");
if (panoControl) {
panoControl.clusterDataSource = this._clusters.dataSource;
FinanceAppJS.Utils.Logger.LogPerformance("NewsRSSSources.js", "page_load", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
}
CommonJS.Progress.hideProgress(CommonJS.Progress.headerProgressType);
},
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away and when the app is suspending.
/// Return any data that you need to resurrect your UX when the page is recreated
/// Keep this state object as small as possible
/// </summary>
/// <returns>
/// This function must return a JSON object that can be serializable. Do not pass a Managed object
/// </returns>
getPageState: function getPageState() { },
/// <summary>
/// This function is invoked by the UX Framework when the page is being created and when
/// it is necessary to bind data to the UX.
/// </summary>
/// <returns>
/// Returns a promise that contains data to bind the UX to
/// </returns>
getPageData: function getPageData() {
var that = this;
return WinJS.Promise.wrap({
title: PlatformJS.Services.resourceLoader.getString("rssSources"),
});
},
/// <summary>
/// This function is invoked by the UX Framework when the data binding is complete
/// </summary>
/// <returns>
/// Nothing
/// </returns>
onBindingComplete: function onBindingComplete() { },
/// <summary>
/// This function is invoked by the UX Framework when the layout has changed
/// </summary>
/// <param name="layoutState">
/// The current layout state
/// </param>
/// <returns>
/// Nothing
/// </returns>
onLayoutChange: function onLayoutChange(layoutState) { },
/// <summary>
/// This function is invoked by the device has been roated
/// </summary>
/// <param name="event">
/// orientation change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onOrientationChange: function onOrientationChange(event) { },
/// <summary>
/// This function is invoked when dpi changes ("Make everything on your screen bigger" setting)
/// </summary>
/// <param name="event">
/// Dpi change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onDpiChange: function onDpiChange(event) {
var newsRSSPanoramaControl = PlatformJS.Utilities.getControl("newsRSSPanorama");
if (newsRSSPanoramaControl) {
newsRSSPanoramaControl.onDpiChange(event);
}
},
getPageImpressionContext: function getPageImpressionContext() {
// DO NOT LOCALIZE THESE STRING HERE
return "/Sources";
},
/// <summary>
/// This function is invoked by the UX Framework when the page is being navigated away
/// </summary>
/// <param name="reason">
/// The reason why the page is being navigated away: "backNavigation" or "newNavigation"
/// </param>
/// <returns>
/// Nothing
/// </returns>
onNavigateAway: function onNavigateAway(reason) { },
/// <summary>
/// Set this property to true if you want the back button to appear in the navbar
/// This is used for immersive pages
/// </summary>
isImmersive: true,
})
});
})();
