﻿(function () {
'use strict';
var donotlaunchurl = false;
WinJS.Namespace.define('FinanceAppJS.Rates', {
Collection: WinJS.Class.derive(CommonJS.EntityCollection.Collection, function (options) {
/// <summary>
/// ECV Controller for Rates
/// </summary>
CommonJS.EntityCollection.Collection.call(this, options);
this.options = options || {};
this._data = [];
}, {
Initialize: function Initialize(config) {
/// <summary>
/// Initialize ECV with a given config
/// </summary>
/// <param name="config" type="Object">Configuration Parameters for Initialization</param>
config = config || {};
var that = this;
var storedLocation = Windows.Storage.ApplicationData.current.roamingSettings.values["StateLocation"];
var location = config.location || storedLocation;
/// No location was passed and cannot find Stored Location. Use Windows Locator
var promises = [];
this.providerConfig = PlatformJS.Collections.createStringDictionary();
var category = this.providerConfig["Category"] = config.category || "Mortgage";
WinJS.Utilities.addClass(document.querySelector(".ImersivePage"), category);
this.UIConfig = config.UIConfig || new FinanceAppJS.Rates.UIConfig({ Category: this.providerConfig["Category"] });
this.DataConfig = config.DataConfig || new CommonJS.EntityCollection.DataConfig();
this.Provider = new CollectionProvider.RatesCollection();
if (config.type) {
this.providerConfig["type"] = config.type;
}
if (config.purpose) {
this.providerConfig["purpose"] = config.purpose;
}
if (config.amount) {
this.providerConfig["amount"] = config.amount;
}
if (config.period) {
this.providerConfig["period"] = config.period;
}
if (config.location) {
this.providerConfig["Location"] = config.location;
}
this.providerConfig["ValidLocations"] = JSON.stringify(FinanceAppJS.ConfigManager.getConfigData("RatesLocations"));
this.providerConfig["CategoryFilters"] = JSON.stringify(FinanceAppJS.ConfigManager.getConfigData("RatesCategoryFilters"));
/// Update subtitle text when valid locations are found
/// Initializing the Provider (C#) with the config
this.invalidataView_binding = this.invalidateView.bind(this);
FinanceAppJS.Utils.ResumeHandler.setRefreshCallback(this.invalidataView_binding);
FinanceAppJS.Globals.networkChangeCallback = this.invalidataView_binding;
return that.Provider.initializeAsync(that.providerConfig);
},
getPageImpressionContext: function getPageImpressionContext(state) {
return "/Rates/" + FinanceAppJS.Telemetry.FlightRecorder.GetRatesContextFromId(state.category, true);
},
}),
/// <summary>UI Config for Entity Collection </summary>
UIConfig: WinJS.Class.derive(CommonJS.EntityCollection.UIConfig, function (options) {
CommonJS.EntityCollection.UIConfig.call(this, options);
WinJS.UI.setOptions(this, options);
this.Category = options && options.Category ? options.Category : "Mortgage";
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.PlatformInstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation["Rates_" + options.Category + "_PageView"], 1);
}, {
Category: null,
GetSupportedViews: function GetSupportedViews() {
return {
list: [CommonJS.EntityCollection.Views.List],
defaultView: CommonJS.EntityCollection.Views.List
};
},
GetViewOptions: function GetViewOptions(view) {
var options = {};
/* this sample shows how to enable multiselect for the grid view of ECV */
switch (view) {
case CommonJS.EntityCollection.Views.Grid:
options.selectionMode = 'multi';
options.disableClearFilters = true;
break;
case CommonJS.EntityCollection.Views.List:
options.selectionMode = 'none';
//options.disableTap = true;
break;
}
// Bug#158094: disabling semanticZoom for all ECVs
options.disableSemanticZoom = true;
return options;
},
OnItemClicked: function OnItemClicked(item) {
if (!item) {
return;
}
var url = item.Insturl;
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(item.Category,
FinanceAppJS.Telemetry.Constants.Location.ECV,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
item.title);
if (url && url.substring(0, 7) === "http://") {
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation["Rates_" + this.Category + "_LenderClick"], 1);
var uri = new Windows.Foundation.Uri(url);
var options = new Windows.System.LauncherOptions;
options.displayApplicationPicker = false;
Windows.System.Launcher.launchUriAsync(uri, options).then(function () {
}, function (err) {
var erorr = err;
});
}
},
GetEntityTemplate: function GetEntityTemplate(view) {
switch (view) {
case CommonJS.EntityCollection.Views.Grid:
return document.querySelector('#MortgageInstRates');
break;
case CommonJS.EntityCollection.Views.List:
return document.querySelector('#MortgageInstRatesListTemplate');
break;
}
},
SetFilterType: function SetFilterType(filter) {
switch (filter.attribute) {
case "purpose":
case "amount":
case "Sort":
case "state":
// FALLTHROUGH
filter.selectionMode = 'SingleToggleFilter';
break;
case "type":
if (this.Category === "creditcard") {
filter.selectionMode = 'SingleToggleFilter';
}
break;
}
},
GetAppBarItems: function GetAppBarItems(controller) {
/*must return an array of WinJS.UI.AppBarCommand options objects controller is passed in so consumer can grab handles to entities, sort, fitler, etc in their callbacks*/
var buttons = [];
var calculator = {
icon: "\uE079",
id: 'mortagecalculator',
extraClass: 'appexSymbol',
label: utils.getResourceString("Rates_Calculator"),
onclick: function onclick(event) {
CommonJS.dismissAllEdgies();
PlatformJS.Navigation.navigateToChannel("MortgageCalc", {});
}
};
if (this.Category === "mortgage") {
buttons.push(calculator);
}
buttons.push({
id: "helpButton",
icon: "\uFF60",
extraClass: 'appexSymbol right',
label: utils.getResourceString("/Platform/HelpLabel"),
instrumentId: FinanceAppJS.Constants.Instrumentation.Help_RatesEcv_Click,
onclick: navigationHelper.navigateToHelpPage
});
return buttons;
},
GetTableDefinition: function GetTableDefinition(entities) {
var td = new CommonJS.Table.TableDefinition();
td.cssWhiteList.push("rates.css");
td.cssWhiteList.push("averages.css");
var allFields = {
mortgage: {
Insturl: {name:"",sortId:""},
InstName:{name: "Name",sortId:"instname"},
ProductFriendlyName: {name:"Product Type",sortId:"productid"},
Apr: { name: "APR", sortId: "apr" },
Rate: { name: "Rate", sortId: "rate"},
Points: {name:"Points",sortId:"points"},
Fees: {name:"Fees",sortId:"fees"},
Amount: { name: "Monthly", sortId: "payment" },
//Phone: ""
},
homequity: {
Insturl: {name:"",sortId:""},
InstName: { name: "Name", sortId: "instname" },
ProductFriendlyName: { name: "Product Type", sortId: "productid" },
Rate: { name: "Rate", sortId: "rate" },
Discount: {name:"Discount",sortId:"discounts", priority: 10},
Monthly: { name: "Monthly", sortId: "payment" },
ClosingCost: { name: "Closing Cost", sortId: "closing" },
//Phone: ""
},
auto: {
Insturl: { name: "", sortId: "" },
InstName: { name: "Name", sortId: "instname" },
ProductFriendlyName: { name: "Product Type", sortId: "productid" },
Rate: { name: "Rate", sortId: "rate" },
Discount: { name: "Discount", sortId: "discounts", priority: 10 },
Monthly: { name: "Monthly", sortId: "payment" },
//Phone: ""
},
deposit: {
Insturl: { name: "", sortId: "" },
InstName: { name: "Name", sortId: "instname" },
ProductFriendlyName: { name: "Product Type", sortId: "productdesc" },
Rate: {name:"APY",sortId:"apy", sortDesc:true},
MinAmount: { name: "Min Amount", sortId: "mintier" }
},
creditcard: {
Insturl: { name: "", sortId: "" },
CardName: { name: "Name", sortId: "CardName"},
PurchaseRate: {name:"Purchase Rate",sortId:"purratemin"},
BalanceTransferRate: { name: "Balance Transfer Rate", sortId: "btratemin" },
AnnualFee: {name:"Annual Fees",sortId:"Annualfee"},
LateFee: { name: "Late Fee", sortId: "LateFee"}
},
};
var platform = {};
for (var cat in allFields) {
var a = allFields[cat];
for (var f in a) {
platform["Rates_Column_" + a[f].name.split(" ").join("_")] = a[f].name;
}
}
var fields = allFields[this.Category];
var i = 0;
for (f in fields) {
var colDefinition = new CommonJS.Table.ColumnDefinition();
colDefinition.headerText = fields[f].name !== "" ? utils.getResourceString("Rates_Column_" + fields[f].name.split(" ").join("_")) : "";
colDefinition.priority = fields[f].priority || i++;
if (fields[f].sortId !== "") {
colDefinition.sortIdentifier = fields[f].sortId;
}
if (fields[f].sortDesc) {
colDefinition.initialSortDirDesc = true;
}
colDefinition.boundField = [f, "PurchaseIntroRate", "BalanceTransferIntroRate"];
colDefinition.ID = f;
colDefinition.headerTemplate = document.querySelector("#columnHeaderTemplate");
colDefinition.fieldTemplate = document.querySelector("#textFieldTemplate" + "_" + this.Category + "_" + f) || document.querySelector("#textFieldTemplate" + "_" + f) || document.querySelector("#textFieldTemplate");
td.columns.push(colDefinition);
}
td.itemSpacing = { min: 0, max: 0 };
td.GroupPosition = 'top'; //can also be 'top'
td.showColumnHeaders = true;
return td;
},
}),
ResourceMapper: function ResourceMapper(data, fields) {
var str = "";
var alt = "";
switch (fields[0]) {
case "InstName":
alt = str = data[fields[0]];
break;
case "Product":
alt = data.ProductFriendlyName;
str = data.ProductId; 
break;
}
try {
return utils.getResourceString(str);
}
catch (err) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(err.message, err.stack);
return alt;
}
},
PhoneClickCallback: function PhoneClickCallback() {
console.log("Phone clicked");
return;
donotlaunchurl = true;
var phone = window.event.srcElement.nextElementSibling.innerText;
var url = window.event.srcElement.nextElementSibling.nextElementSibling.innerText;
window.event.srcElement.style.backgroundImage = "";
window.event.srcElement.innerText = phone;
if (!url || url === "") {
return;
}
WinJS.xhr({
url: url
});
return false;
//window.event.stopPropagation();
},
TileClickCallback: function TileClickCallback(e) {
if (donotlaunchurl) {
donotlaunchurl = false;
return;
}
var url = window.event.srcElement.parentElement.parentElement.querySelector(".insturl").url;
if (url && url.substring(0, 7) === "http://") {
var uri = new Windows.Foundation.Uri(url);
var options = new Windows.System.LauncherOptions;
options.displayApplicationPicker = false;
Windows.System.Launcher.launchUriAsync(uri, options).then(function () {
var complete = "";
}, function (err) {
var erorr = err;
});
}
}
});
})();
