﻿(function () {
"use strict";
//Data Provider for Overview cluster
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
EntityOverviewDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this.symbolsCsv = options && options.symbolsCsv;
this.cmpName = options && options.cmpName;
this.isUSExchange = options.isUSExchange;
this.refreshInterval = options.refreshInterval;
}, {
queryServiceId: "entityRT",
symbolsCsv: null,
cmpName: null,
isUSExchange: true,
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("entities", encodeURIComponent(this.symbolsCsv));
urlParams.insert("lang", FinanceAppJS.Globalization.getLanguage());
return urlParams;
},
dataBuilder: function dataBuilder(data, response) {
var overviewResponseObj = data[0];
var entitySymbol = this.symbolsCsv;
var overviewData = {};
if (overviewResponseObj) {
overviewData = overviewResponseObj;
var lastPrice = overviewResponseObj.Lp;
var market = Platform.Utilities.Globalization.getCurrentMarket() || "";
var isCurrencyDecimalIgnored = FinanceAppJS.Utils.Formatters.isCurrencyDecimalIgnored(market, overviewData.E2);
var currMarketPrecision, currMarketPrecisionString;
if ( isCurrencyDecimalIgnored ) {
currMarketPrecisionString = "FLOAT0";
currMarketPrecision = 0;
}
else if (lastPrice < 1) {
currMarketPrecisionString = "FLOAT4";
currMarketPrecision = 4;
}
else {
currMarketPrecisionString = "FLOAT2";
currMarketPrecision = 2;
}              
if (overviewResponseObj.Ch > 0) {
overviewData["changeStatus"] = 1;
overviewData["statusIndicator"] = "largeGlyph financeGlyphUp";
overviewData["col1"] = "upcolor";
overviewData["col2"] = "upcolor";
}
else {
if (overviewResponseObj.Ch !== 0) {
overviewData["changeStatus"] = -1;
overviewData["statusIndicator"] = "largeGlyph financeGlyphDn";
overviewData["col1"] = "downcolor";
overviewData["col2"] = "downcolor";
}
else {
overviewData["changeStatus"] = 0;
overviewData["statusIndicator"] = "mediumGlyph financeGlyphEq";
overviewData["col1"] = "equalcolor";
overviewData["col2"] = "equalcolor";
}
}
overviewData["Lp"] = utils.formatNumber(lastPrice, currMarketPrecisionString, false, "");
overviewData["Ch"] = utils.formatNumber(overviewResponseObj.Ch, currMarketPrecisionString, true, "");
overviewData["Pc"] = utils.formatNumber(overviewResponseObj.Chp, "PERCENT", true);
overviewData["Prevc"] = utils.roundFloat(overviewResponseObj.Pp, currMarketPrecision);
overviewData["Openc"] = utils.roundFloat(overviewResponseObj.Op, currMarketPrecision);
overviewData["Pe"] = utils.formatNumber(overviewResponseObj.Pe, "P/E");
overviewData["DrStart"] = utils.roundFloat(overviewResponseObj.Dl, currMarketPrecision);
overviewData["DrEnd"] = utils.roundFloat(overviewResponseObj.Dh, currMarketPrecision);
overviewData["DrRangePointerPos"] = getRangeIndicatorPos(overviewResponseObj.Dl, overviewResponseObj.Dh, lastPrice);
overviewData["Dy"] = utils.formatNumber(parseFloat(overviewResponseObj.Dy), "PERCENT", false, "-");
overviewData["Mc"] = overviewResponseObj.Mc ? utils.formatVolume(parseFloat(overviewResponseObj.Mc)) : "-";
overviewData["V"] = utils.formatNumber(overviewResponseObj.V, "VOLUME");
overviewData["avgV"] = utils.formatNumber(overviewResponseObj.avgV, "VOLUME");
overviewData["FwStart"] = utils.roundFloat(overviewResponseObj.Yl, currMarketPrecision);
overviewData["FwEnd"] = utils.roundFloat(overviewResponseObj.Yh, currMarketPrecision);
var fwh = overviewResponseObj.Yh;
var fwl = overviewResponseObj.Yl;
var wkLp = lastPrice < fwl ? fwl : lastPrice > fwh ? fwh : lastPrice;
overviewData["FwRangePointerPos"] = getRangeIndicatorPos(overviewResponseObj.Yl, overviewResponseObj.Yh, wkLp);
overviewData["dateTime"] = FinanceAppJS.Utils.Formatters.shortDateFormat(overviewResponseObj.Ld) + " " + FinanceAppJS.Utils.Formatters.shortTimeFormat(overviewResponseObj.Lt) + " " + overviewResponseObj.tz;
overviewData["priceUpdateInfo"] = utils.getResourceString("SEPriceUpdateInfo").format(FinanceAppJS.Constants.RefreshIntervals.StockDetailsRT / 60);
overviewData["timeZone"] = utils.timeZoneFormat(overviewResponseObj.tz,overviewResponseObj.utcOff);
overviewData["showPriceCls"] = "fontSR";
// After Hours
if (overviewData["trSts"]) {
var trCh = null, trChp = null;
if (overviewData["trSts"] === 1) {
overviewData["trLabel"] = utils.getResourceString("SEPreMarket");
overviewData["trLp"] = utils.formatNumber(overviewResponseObj.pre, currMarketPrecisionString);
trCh = overviewResponseObj.preCh;
trChp = overviewResponseObj.preChp;
}
else if (overviewData["trSts"] === 2) {
overviewData["trLabel"] = utils.getResourceString("SEAfterHours");
overviewData["trLp"] = utils.formatNumber(overviewResponseObj.post, currMarketPrecisionString);
trCh = overviewResponseObj.postCh;
trChp = overviewResponseObj.postChp;
}
if (trCh > 0) {
overviewData["trCh"] = utils.formatNumber(trCh, currMarketPrecisionString, true);
overviewData["trChp"] = utils.formatNumber(trChp, "PERCENT", true);
overviewData["trChangeCls"] = "up";
}
else {
overviewData["trCh"] = utils.formatNumber(trCh, currMarketPrecisionString);
overviewData["trChp"] = utils.formatNumber(trChp, "PERCENT");
if (trCh === 0 || !trCh) {
overviewData["trChangeCls"] = "equalcolor";
}
else {
overviewData["trChangeCls"] = "dn";
}
}
overviewData["trChangeCls"] += " minSpace";
overviewData["trClass"] = "tradingStatus";
overviewData["trCh"] = utils.addLtrMark(overviewData["trCh"]);
overviewData["trChp"] = utils.addLtrMark(overviewData["trChp"]);
}
else {
overviewData["trClass"] = "hide";
overviewData["trLabel"] = overviewData["trLp"] = overviewData["trCh"] = overviewData["trChp"] = overviewData["trChangeCls"] = null;
}
}
else {
overviewData = { Lp: "", Ch: "", Pc: "", Prevc: "-", Openc: "-", Pe: "-", DrStart: "", DrEnd: "", FwStart: "", FwEnd: "", Dy: "-", Mc: "-", V: "-", dateTime: "", priceUpdateInfo: "", showPriceCls: "hide", trClass: "hide" };
}
overviewData["stock"] = entitySymbol + "1d";
overviewData["sym"] = overviewData["stock"].displayName;
overviewData["tkr"] = overviewData["stock"].symbol;
overviewData["seriesObject"] = { seriesId: overviewData.stock, displayName: overviewData.Sym };
// Add Narator text
overviewData["naratorText"] = this.cmpName + " " + utils.getResourceString("TickerName_Al") + overviewData.Sym;
var adConfig = PlatformJS.Ads.getAdsConfig("sponsoredBy");
if (adConfig) {
overviewData["adUnitId"] = adConfig.adUnitId;
}
if (overviewData && response) {
overviewData.lastUpdateTime = response.lastUpdateTime;
}
return overviewData;
}
})
});
function getRangeIndicatorPos(start, end, currentPrice) {
return (parseInt((50 / (end - start)) * (currentPrice - start)) - 5) + "px";
}
})();
(function () {
"use strict";
//Data Provider for Key Stats cluster
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
EntityDetailDataProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this.symbolsCsv = options && options.symbolsCsv;
this.cmpName = options && options.cmpName;
this.isUSExchange = options.isUSExchange;
this.isEOD = options.isEOD || false;
this.isOTC = options.isOTC || false;
}, {
queryServiceId: "StockEntityDetail",
refreshInterval: 0,
isEOD: false,
isOTC: false,
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("entities", encodeURIComponent(this.symbolsCsv));
urlParams.insert("lang", FinanceAppJS.Globalization.getLanguage());
urlParams.insert("isEOD", this.isEOD);
urlParams.insert("isOTC", this.isOTC);
urlParams.insert("localizeFor", FinanceAppJS.Globalization.getMarket());                return urlParams;
},
dataBuilder: function dataBuilder(responseObj) {
responseObj = responseObj[0];
var that = this;
var processedData = {};
responseObj = (responseObj) ? responseObj : {};
responseObj.Stat = (responseObj.Stat) ? responseObj.Stat : [];
responseObj.Info = (responseObj.Info) ? responseObj.Info : {};
responseObj.Rt = (responseObj.Rt) ? responseObj.Rt : {};
responseObj.Rap = (responseObj.Rap) ? responseObj.Rap : {};
responseObj.Cr = (responseObj.Cr) ? responseObj.Cr : {};
responseObj.Fot = (responseObj.Fot) ? responseObj.Fot : [];
// Key Stats cluster UI is different for US and Non-US markets
if (that.isUSExchange) {
processedData.keyStats = that.processKeyStatsData(responseObj.Stat, 5);
processedData.cmpProfile = that.processCmpProfileData(responseObj.Info, that.cmpName);
processedData.cmpProfile.Ind = (processedData.cmpProfile.Ind);
processedData.cmpProfile.Sec = (processedData.cmpProfile.Sec);
processedData.cmpProfile.Stl = (processedData.cmpProfile.Stl);
processedData.cmpProfile.Typ = (processedData.cmpProfile.Typ);
processedData.proRating = that.processProRatingData(responseObj.Rt);
}
else {
processedData.keyStats = that.processKeyStatsDataNonUs(responseObj.Stat, responseObj.Info, that.cmpName);
}
processedData.revProfits = that.processRevProfitData(responseObj.Rap);
processedData.cashReserves = that.processCashReservesData(responseObj.Cr);
processedData.trends = that.processTrendsData(responseObj.Fot);
return processedData;
},
processKeyStatsData: function processKeyStatsData(keyStatsResponseObj, noOfRecs) {
var cnt;
var keyStatsTemplateData = {};
keyStatsTemplateData.naratorText = utils.getResourceString("KeyStats") + " " + this.cmpName;
var pValues = [];
var nValues = [];
for (cnt = 0; cnt < noOfRecs; cnt++) {
if (!keyStatsResponseObj[cnt]) {
keyStatsResponseObj[cnt] = {};
keyStatsResponseObj[cnt].isDataAvailable = false;
keyStatsResponseObj[cnt].Nm = "-";
}
else {
keyStatsResponseObj[cnt].isDataAvailable = true;
}
if (keyStatsResponseObj[cnt].Nm === "Industry") {
var strIn = utils.formatString((utils.getResourceString("SEIndustry")).split(":"));
keyStatsResponseObj[cnt].Nm = strIn[0];
}
else if (keyStatsResponseObj[cnt].Nm === "S&P 500") {
keyStatsResponseObj[cnt].Nm = utils.getResourceString("Indx_SPX");
}
else {
keyStatsResponseObj[cnt].Nm = utils.formatString(keyStatsResponseObj[cnt].Nm);
}
keyStatsResponseObj[cnt].Sp = utils.formatNumber(keyStatsResponseObj[cnt].Sp, "FLOAT2");
keyStatsResponseObj[cnt].Mc = utils.formatNumber(keyStatsResponseObj[cnt].Mc, "VOLUME");
keyStatsResponseObj[cnt].Pe = utils.formatNumber(keyStatsResponseObj[cnt].Pe, "P/E");
keyStatsResponseObj[cnt].Dy = utils.formatNumber(keyStatsResponseObj[cnt].Dy, "FLOAT2");
keyStatsResponseObj[cnt].Ar = utils.formatString(keyStatsResponseObj[cnt].Ar, "-");
keyStatsResponseObj[cnt].De = utils.formatNumber(keyStatsResponseObj[cnt].De, "FLOAT2");
keyStatsResponseObj[cnt].Nmp = utils.formatNumber(keyStatsResponseObj[cnt].Nmp, "FLOAT2");
keyStatsResponseObj[cnt].Roe = utils.formatNumber(keyStatsResponseObj[cnt].Roe, "FLOAT2");
keyStatsResponseObj[cnt].Eps = utils.formatNumber(keyStatsResponseObj[cnt].Eps, "FLOAT2");
keyStatsResponseObj[cnt].Roa = utils.formatNumber(keyStatsResponseObj[cnt].Roa, "FLOAT2");
keyStatsResponseObj[cnt].FpEPS = utils.formatNumber(keyStatsResponseObj[cnt].FpEPS, "P/E");
keyStatsResponseObj[cnt].PtS = utils.formatNumber(keyStatsResponseObj[cnt].PtS, "FLOAT2");
keyStatsResponseObj[cnt].PtB = utils.formatNumber(keyStatsResponseObj[cnt].PtB, "FLOAT2");
keyStatsResponseObj[cnt].Rv5YrGrth = utils.formatNumber(keyStatsResponseObj[cnt].Rv5YrGrth, "FLOAT2");
keyStatsResponseObj[cnt].Beta = utils.formatNumber(keyStatsResponseObj[cnt].Beta, "FLOAT2");
if (keyStatsResponseObj[cnt].Rm && keyStatsResponseObj[cnt].Rm >= 0) {
pValues.push(keyStatsResponseObj[cnt].Rm);
}
else if (keyStatsResponseObj[cnt].Rm) {
nValues.push(keyStatsResponseObj[cnt].Rm);
}
if (keyStatsResponseObj[cnt].Ry && keyStatsResponseObj[cnt].Ry >= 0) {
pValues.push(keyStatsResponseObj[cnt].Ry);
}
else if (keyStatsResponseObj[cnt].Ry) {
nValues.push(keyStatsResponseObj[cnt].Ry);
}
}
// Compute graph values
var pMax = 0;
var pMin = 0;
var nMax = 0;
var nMin = 0;
var nMinAbs = 0;
pMax = Math.max.apply(Math, pValues);
pMin = Math.min.apply(Math, pValues);
nMax = Math.max.apply(Math, nValues);
nMin = Math.min.apply(Math, nValues);
nMinAbs = Math.abs(nMin);
var maxHeight = 90; // Height of the Return-% graph container
var avgHtP = 0;
var avgHtN = 0;
if (pValues.length > 0 && nValues.length === 0) {
avgHtP = maxHeight / pMax;
avgHtN = avgHtP;
}
else if (pValues.length === 0 && nValues.length > 0) {
avgHtN = maxHeight / nMinAbs;
avgHtP = avgHtN;
}
else {
avgHtP = maxHeight / (pMax + nMinAbs);
avgHtN = avgHtP;
}
var tmpPHeight = parseInt(avgHtP * pMax);
var tmpNHeight = parseInt(avgHtN * nMinAbs);
keyStatsTemplateData.pHeight = isNaN(tmpPHeight) ? 0 : tmpPHeight + "px";
keyStatsTemplateData.nHeight = isNaN(tmpNHeight) ? 0 : tmpNHeight + "px";
for (cnt = 0; cnt < noOfRecs; cnt++) {
var mpHt = 0;
var mnHt = 0;
var ypHt = 0;
var ynHt = 0;
if (keyStatsResponseObj[cnt].Rm >= 0) {
mpHt = parseInt(avgHtP * keyStatsResponseObj[cnt].Rm);
keyStatsResponseObj[cnt].MpLabel = utils.formatNumber(keyStatsResponseObj[cnt].Rm, "FLOAT1", false, "");
if (keyStatsResponseObj[cnt].MpLabel) {
keyStatsResponseObj[cnt].MpLabel = (keyStatsResponseObj[cnt].Rm > 0) ? "+" + keyStatsResponseObj[cnt].MpLabel : keyStatsResponseObj[cnt].MpLabel;
}
keyStatsResponseObj[cnt].MnLabel = "";
}
else {
mnHt = parseInt(avgHtN * (keyStatsResponseObj[cnt].Rm * -1));
keyStatsResponseObj[cnt].MpLabel = "";
keyStatsResponseObj[cnt].MnLabel = utils.formatNumber(keyStatsResponseObj[cnt].Rm, "FLOAT1", false, "");
}
if (keyStatsResponseObj[cnt].Ry >= 0) {
ypHt = parseInt(avgHtP * keyStatsResponseObj[cnt].Ry);
keyStatsResponseObj[cnt].YpLabel = utils.formatNumber(keyStatsResponseObj[cnt].Ry, "FLOAT1", false, "");
if (keyStatsResponseObj[cnt].YpLabel) {
keyStatsResponseObj[cnt].YpLabel = (keyStatsResponseObj[cnt].Ry > 0) ? "+" + keyStatsResponseObj[cnt].YpLabel : keyStatsResponseObj[cnt].YpLabel;
}
keyStatsResponseObj[cnt].YnLabel = "";
}
else {
ynHt = parseInt(avgHtN * (keyStatsResponseObj[cnt].Ry * -1));
keyStatsResponseObj[cnt].YpLabel = "";
keyStatsResponseObj[cnt].YnLabel = utils.formatNumber(keyStatsResponseObj[cnt].Ry, "FLOAT1", false, "");
}
keyStatsResponseObj[cnt].MpHeight = (mpHt ? mpHt : 0) + "px";
keyStatsResponseObj[cnt].YpHeight = (ypHt ? ypHt : 0) + "px";
keyStatsResponseObj[cnt].MnHeight = (mnHt ? mnHt : 0) + "px";
keyStatsResponseObj[cnt].YnHeight = (ynHt ? ynHt : 0) + "px";
keyStatsResponseObj[cnt].MpBot = keyStatsResponseObj[cnt].MpHeight;
keyStatsResponseObj[cnt].YpBot = keyStatsResponseObj[cnt].YpHeight;
keyStatsTemplateData["Rec" + cnt] = keyStatsResponseObj[cnt];
}
if ( (!keyStatsTemplateData["Rec0"]) || (!keyStatsTemplateData["Rec0"].isDataAvailable)) {
keyStatsTemplateData = null;
}
return keyStatsTemplateData;
},
processKeyStatsDataNonUs: function processKeyStatsDataNonUs(keyStatsResponseObj, cmpProfileResponseObj, cmpName) {
var keyStatsTemplateData = this.processKeyStatsData(keyStatsResponseObj, 1);
if (keyStatsTemplateData && keyStatsTemplateData["Rec0"]) {
keyStatsTemplateData = keyStatsTemplateData["Rec0"];
keyStatsTemplateData.returnP = [keyStatsTemplateData.Rm, keyStatsTemplateData.Ry];
keyStatsTemplateData.cmpProfile = this.processCmpProfileData(cmpProfileResponseObj, cmpName, true);
keyStatsTemplateData.naratorText = utils.getResourceString("KeyStats") + " " + cmpName;
}
return keyStatsTemplateData;
},
processCmpProfileData: function processCmpProfileData(cmpProfileResponseObj, cmpName, isNonUs) {
var cmpProfileTemplateData = {};
if (utils.isNonEmptyObject(cmpProfileResponseObj)) {
cmpProfileResponseObj.Ind = utils.formatString((cmpProfileResponseObj.Ind));
cmpProfileResponseObj.Sec = utils.formatString((cmpProfileResponseObj.Sec));
cmpProfileResponseObj.Stl = utils.formatString((cmpProfileResponseObj.Stl));
cmpProfileResponseObj.Typ = utils.formatString((cmpProfileResponseObj.Typ));
cmpProfileResponseObj.cmpWebsiteClass = cmpProfileResponseObj.Ws ? "cmpWs" : "hide";
cmpProfileResponseObj.naratorText = this.cmpName + utils.getResourceString("CmpProfile");
cmpProfileResponseObj.companyLinkNaratorText = this.cmpName + utils.getResourceString("SECompany Website");
cmpProfileResponseObj.fullDes = cmpProfileResponseObj.Des;
cmpProfileResponseObj.currentResDes = utils.getCompanyProfileDescForCurrentRes(cmpProfileResponseObj.Des);
if (cmpProfileResponseObj.Des) {
//The container can hold upto 300 chars
if (isNonUs || cmpProfileResponseObj.Des.length > 300) {
var desLong = utils.addLtrMark(cmpProfileResponseObj.Des);
cmpProfileResponseObj.readMoreClickHandler = WinJS.Utilities.markSupportedForProcessing(function (event) {
// Platform Flyout
if (event.keyCode) {
if (event.keyCode !== WinJS.Utilities.Key.enter && event.keyCode !== WinJS.Utilities.Key.space) {
return;
}
}
FinanceAppJS.Utils.Logger.LogPerformance("readMoreFlyout", "load", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);
var flyoutData = {
cmpProfileReadMore: {
title: utils.getResourceString("CmpProfile"),
Des: desLong,
Ind: cmpProfileResponseObj.Ind,
Sec: cmpProfileResponseObj.Sec,
Stl: cmpProfileResponseObj.Stl,
Typ: cmpProfileResponseObj.Typ,
Ws: cmpProfileResponseObj.Ws,
WsClass: cmpProfileResponseObj.Ws ? "" : "hide"
}
};
var readMoreFlyout = document.querySelector('#readMoreFlyout');
var readMoreFlyoutRelativeContainer = document.querySelector('#stockEntityMainList');
if (document.querySelector('.cmpProfileFlyoutContainer')) {
WinJS.Binding.processAll(readMoreFlyout, flyoutData);
readMoreFlyout.winControl.show(readMoreFlyoutRelativeContainer, "top", "center");
FinanceAppJS.Utils.Logger.LogPerformance("readMoreFlyout", "load", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
utils.vAlignContainerInMiddle(readMoreFlyout);
}
else {
var moduleInfo = { templateId: "tmplCmpProfileReadMore", fragmentPath: "/pages/StockDetails/html/StockDetailsTemplates.html" };
CommonJS.loadModule(moduleInfo, flyoutData, null, null, null).then(function (readMoreTemplate) {
readMoreFlyout.appendChild(readMoreTemplate);
readMoreFlyout.winControl.show(readMoreFlyoutRelativeContainer, "top", "center");
FinanceAppJS.Utils.Logger.LogPerformance("readMoreFlyout", "load", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
utils.vAlignContainerInMiddle(readMoreFlyout);
});
}
//Instrumentation for readmore buttonclick.
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(FinanceAppJS.Telemetry.Constants.Page.STOCKDETAILS,
FinanceAppJS.Telemetry.Constants.Cluster.COMPANYPROFILE,
FinanceAppJS.Telemetry.Constants.Location.READMOREBUTTON,
"Read More for " + cmpName);
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.QuoteDetails_ReadMoreBtn_Click, 1);
});
cmpProfileResponseObj.readMoreClass = "readMore";
}
else {
cmpProfileResponseObj.readMoreClass = "hide";
}
}
else {
cmpProfileResponseObj.fullDes = "";
cmpProfileResponseObj.currentResDes = "";
cmpProfileResponseObj.readMoreClass = "hide";
}
cmpProfileResponseObj.isDataAvailable = true;
}
else {
cmpProfileResponseObj = { Sec: "-", Ind: "-", Typ: "-", Stl: "-", currentResDes: "", fullDes: "", isDataAvailable: false };
}
cmpProfileTemplateData = cmpProfileResponseObj;
return cmpProfileTemplateData;
},
processRevProfitData: function processRevProfitData(revProfitResponseObj) {
var revenue = [];
var ntIncome = [];
var xlabels = [];
var ylabels = [];
//The expected format = "Mar 2011"
var formatter = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter('month.abbreviated year');
var revenueDate;
for (var i = 0; i < revProfitResponseObj.length; i++) {
if (revProfitResponseObj[i] && revProfitResponseObj[i].Rv && revProfitResponseObj[i].Ni) {
revenue.push({
x: i,
y: revProfitResponseObj[i].Rv
});
ntIncome.push({
x: i,
y: revProfitResponseObj[i].Ni
});
//The javascript date constructor takes zero indexed month numbers, substracting one from month index.
revenueDate = new Date(revProfitResponseObj[i].Yr, revProfitResponseObj[i].MtNum - 1);
xlabels.push({
value: i,
label: formatter.format(revenueDate)
});
}
}
var revenueChartdata = {
isDataAvailable: revenue.length ? true : false,
clstProfitContainerClass: revenue.length ? "clstProfitContainer" : "hide",
xaxis: {
labels: xlabels
},
yaxis: {
labels: ylabels
},
data: [
{ label: utils.getResourceString("SE Revenue"), points: revenue },
{ label: utils.getResourceString("SENet Income"), points: ntIncome }
]
};
return revenueChartdata;
},
processCashReservesData: function processCashReservesData(cashReservesResponseObj) {
var cashReserves = [];
var xlabels = [];
var ylabels = [];
for (var i = 0; i < cashReservesResponseObj.length; i++) {
if (cashReservesResponseObj[i] && cashReservesResponseObj[i].Tc && cashReservesResponseObj[i].Yr) {
cashReserves.push({
x: i,
y: cashReservesResponseObj[i].Tc
});
xlabels.push({
value: i,
label: cashReservesResponseObj[i].Yr
});
}
}
var cashReservesChartdata = {
isDataAvailable: cashReserves.length ? true : false,
clstCashReservesContainerClass: cashReserves.length ? "clstCashReservesContainer" : "hide",
xaxis: {
labels: xlabels
},
yaxis: {
labels: ylabels
},
data: [
{ label: utils.getResourceString("SETotal Cash"), points: cashReserves }
]
};
return cashReservesChartdata;
},
//Making sure truncation takes at correct place in company profile data and morning star information
getStringEnd: function getStringEnd(description, iter) {
var descriptionChar = description[iter];
while (iter > 0 && ((descriptionChar >= 'a' && descriptionChar <= 'z') || (descriptionChar >= 'A' && descriptionChar <= 'Z') || descriptionChar === '-')) {
descriptionChar = description[--iter];
}
return iter;
},
processProRatingData: function processProRatingData(proRatingResponseObj) {
var proRatingTemplateData = {};
if (utils.isNonEmptyObject(proRatingResponseObj)) {
proRatingResponseObj.Gr = utils.formatString(proRatingResponseObj.Gr);
proRatingResponseObj.Pr = utils.formatString(proRatingResponseObj.Pr);
proRatingResponseObj.Cr = utils.formatString(proRatingResponseObj.Cr);
proRatingResponseObj.naratorText = utils.getResourceString("MorningStarRatingFor_Al") + this.cmpName;
if (proRatingResponseObj.Rp) {
//The container can hold upto 300 chars
proRatingResponseObj.Rp = (proRatingResponseObj.Rp && proRatingResponseObj.Rp !== "" && proRatingResponseObj.Rp.length > 300) ? proRatingResponseObj.Rp.substring(0, this.getStringEnd(proRatingResponseObj.Rp,300)) + "&hellip;" : (proRatingResponseObj.Rp && proRatingResponseObj.Rp !== "" ? proRatingResponseObj.Rp : "");
proRatingResponseObj.Rp = utils.addLtrMark(proRatingResponseObj.Rp);
proRatingResponseObj.PaClass = (proRatingResponseObj.Lnk) ? "prmAccess" : "hide";
}
else {
proRatingResponseObj.Rp = "";
proRatingResponseObj.PaClass = "hide";
}
proRatingResponseObj.clstProviderRatingClass = "clstProviderRating";
proRatingResponseObj.Title = utils.getResourceString("ProviderRating");
proRatingResponseObj.isDataAvailable = true;
proRatingTemplateData = proRatingResponseObj;
}
else {
proRatingTemplateData = { clstProviderRatingClass: "hide", isDataAvailable: false };
}
return proRatingTemplateData;
},
processTrendsData: function processTrendsData(trendResponseObj) {
var trendData = [];
trendData.push(['SETop Funds', 'SEFund Rating', 'SEBuying/Selling Trend', 'SE Change'].map(utils.getResourceString));
if (trendResponseObj) {
for (var cnt = 0; cnt < trendResponseObj.length; cnt++) {
var trendTemplateData = [trendResponseObj[cnt].Tf, trendResponseObj[cnt].Rt, trendResponseObj[cnt].ChP, utils.addLtrMark(FinanceAppJS.Utils.Formatters.percentFormatter(trendResponseObj[cnt].ChP))];
trendData.push(trendTemplateData);
}
}
return trendData;
}
})
});
})();
