﻿(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Personalization",
{
MySourcesControl: WinJS.Class.derive(NewsApp.Controls.ControlBase, function (element, options, orchestrator) {
NewsApp.Controls.ControlBase.call(this, element, options, orchestrator);
options = options ||{};
WinJS.Utilities.addClass(this.element, "mySourcesItemsContainer");
this.element.addEventListener("itemdeleted", this.handleItemDeleted.bind(this));
this.element.addEventListener("listreordered", this.handleItemReordered.bind(this));
this.element.addEventListener("additeminvoked", this._onAddItemInvoked.bind(this));
this._market = NewsJS.Globalization.getMarketString();
this._guid = options.guid;
},
{
_provider: null,
createProviderInstance: function createProviderInstance() {
this._provider = new NewsJS.Personalization.MySourcesDataProvider({ guid: this._guid, market: this._market });
return this._provider;
},
_dataSource: null,
_listBinding: null,
_addIndex: 0,
clearControl: function clearControl() {
if (this.control) {
if (this.control._dispose) {
this.control._dispose();
this.control = null;
}
if (this.element) {
this.element.innerHTML = "";
}
}
},
renderFirstTime: function renderFirstTime(response) {
var that = this;
return new WinJS.Promise(function (completeHandler, errorHandler, progressHandler) {
that.clearControl();
var itemsToShow = response;
that._listBinding = new WinJS.Binding.List(itemsToShow);
that._dataSource = that._listBinding.dataSource;
var clusterBinding = {
groupKey: "",
groupTitle: "",
contentOptions: {
friendlyName: "mySourcesList",
selectionMode: "multi",
itemDataSource: that._dataSource,
itemTemplate: that._renderItem.bind(that),
oniteminvoked: that.onItemInvoke.bind(that),
addTileItemTemplate: that._renderAddItem.bind(that),
enabledFeatures: CommonJS.UI.ListView2.FEATURES.ALL
}
}
// We only need to instrument this here once during the lifetime of the process.
if (!NewsJS.Telemetry.Customization.sourcesReported) {
NewsJS.Telemetry.Customization.sourcesReported = true;
NewsJS.Telemetry.Customization.recordSources(itemsToShow);
}
var control = that.control = new CommonJS.UI.ListView2(that.element, clusterBinding.contentOptions);
control.addEventListener("loadingstatechanged", function (e) {
var loadingState = control.loadingState;
if (loadingState === "complete") {
completeHandler();
}
});
});
},
// after this fix on 03/25 we are reusing the renderFirstTime to create new control 
// instead of resuing the control and // change data binding only
// Reason is, for some reason(unknown yet) the ListView "loadingstatechanged" event only return "itemLoading"
// it never reach the sate of "complete" which we are expecting to complete the promise.
renderNextTime: function renderNextTime(response) {
//return this.renderFirstTime(response);
//Do nothing since data will never change.
return;
},
_renderItem: function _renderItem(itemPromise) {
return {
element: itemPromise.then(function _cnpRenderer(item) {
var div = document.createElement("div");
WinJS.Utilities.addClass(div, "mySourcesItem");
var binding = item.data;
return CommonJS.loadModule(binding.moduleInfo, binding, div).then(
function () {
return div;
});
})
};
},
_renderAddItem: function _renderAddItem(itemPromise) {
var that = this;
return {
element: itemPromise.then(function _cnpRenderer(item) {
var div = document.createElement("div");
WinJS.Utilities.addClass(div, "mySourcesItem");
var templateId = NewsJS.Personalization.Utilities.myItemsAddTile;
var item = that._provider.createAddMoreItem(templateId);
CommonJS.loadModule(item.moduleInfo, item, div).then();
return div;
})
};
},
onWindowResize: function onWindowResize() {
var control = this.control;
if (control && control.forceLayout) {
control.forceLayout();
}
},
onConnectionChanged: function onConnectionChanged(evt) {
if (evt.connection) {
this.render();
}
},
onResumed: function onResumed(evt) {
this.render();
},
onRefreshed: function onRefreshed(evt) {
//TODO: force refresh
this.render();
},
handleItemDeleted: function handleItemDeleted(e) {
var that = this;
var item = e.detail;
if (item) {
that._provider.handleRemoveItemsAsync([item]).then(function () {
that._addIndex--;
item.selected = false;
if (item.data) {
item.data.selected = false;
var category = null;
if (item.data.type === "InternationalBingDaily" ||
item.data.type === "marketCluster") {
category = item.data.type;
}
else if (item.data.featured) {
category = "Featured";
}
else if (item.data.newscategory) {
category = item.data.newscategory;
}
NewsJS.Telemetry.Customization.recordAddRemove(that._market, item.data, "Source", category);
}
that._provider.request().then(function (sources) {
NewsJS.Telemetry.Customization.recordSources(sources);
});
});
}
},
handleItemReordered: function handleItemReordered(e) {
var that = this;
var order = this.control.getOrder();
that._provider.handleItemReorderedAsync(order).then(function (done) {
that._provider.request().then(function (sources) {
NewsJS.Telemetry.Customization.recordSources(sources);
});
});
},
onItemInvoke: function onItemInvoke(e) {
var that = this;
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.mySources);
if (!that.control || that.control.isHandlePrevented()) {
return;
}
var ds = this._dataSource;
ds.itemFromIndex(e.detail.itemIndex).then(function (item) {
if (!that.isAddTile(item)) {
that._myItemsItemInvoked(item);
}
});
},
isAddTile: function isAddTile(tileItem) {
return (tileItem && tileItem.data && tileItem.data.isCommonJSAddTile);
},
// Item click action for tiles:
// MySources: click go to source page
// MyTopics: top: click go to "Topic Page"
//          bottom: trending: do nothing
_myItemsItemInvoked: function _myItemsItemInvoked(item) {
var that = this;
NewsJS.Telemetry.BingDaily.recordSourcesItemPress(item);
var data = item.data;
if (data.data.type === NewsJS.Personalization.Utilities.sourceType.InternationalBingDaily) {
PlatformJS.Navigation.navigateToChannel("HomePage", { market: data.data.market, title: data.data.displayname });
}
else if (data.tileType === NewsJS.Personalization.Utilities.tileType.Featured) {
this._partnerPanoItemInvoked(data.data);
}
// Tile data getting from "Sources Page" 
else if (data.tileType === NewsJS.Personalization.Utilities.tileType.Customize) {
this._sourceNavigate(data.data, "sourcegallery");
}
//More sources Tile Clicked to go to "Sources Page"
else if (data.isCommonJSAddTile) {
PlatformJS.Navigation.navigateToChannel("Sources");
}
//Tile data getting from "FeaturedChannels"
else {
if (data.isDeepLink) {
PlatformJS.Navigation.navigateTo(data.pageInfo.fragment);
}
else {
WinJS.Navigation.navigate(data.pageInfo, data.state);
}
}
},
_partnerPanoItemInvoked: function _partnerPanoItemInvoked(item) {
if (item.destination) {
var newsUri = NewsJS.Utilities.parseCMSUriString(item.destination);
if (newsUri && newsUri.entitytype === "partnerPano" && newsUri.params && !newsUri.params["entrypoint"]) {
newsUri = NewsJS.Utilities.parseCMSUriString(item.destination + "&entrypoint=" + Windows.Foundation.Uri.escapeComponent("bingdailycluster"));
}
if (newsUri && newsUri.uri) {
Windows.System.Launcher.launchUriAsync(newsUri.uri);
}
}
},
// navigate to selected source page
_sourceNavigate: function _sourceNavigate(source, entrypoint, /*@optional*/ market) {
var that = this;
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.mySources);
market = market || source.market;
if (source.channel_id === "NYT") {
NYTJS.Instrumentation.instance.instrumentBingDailyToPanorama();
}
NewsJS.Utilities.navigateToSource(source,
function (src) {
NewsJS.Utilities.updateStateOnSourceVisited(src.id, market);
}, entrypoint, market);
},
_onAddItemInvoked: function _onAddItemInvoked(e) {
NewsJS.Telemetry.BingDaily.recordSourcesAddSourcePress();
var state = {
mode: NewsJS.Personalization.Utilities.selectionPageFormat.FORMAT_SOURCE,
features: [NewsJS.Personalization.Utilities.selectionPageFeature.TYPE_SOURCES,
NewsJS.Personalization.Utilities.selectionPageFeature.TYPE_INTERNATIONAL],
isBrowse: false,
title: PlatformJS.Services.resourceLoader.getString("AddSourceTile"),
guid: this._guid,
};
PlatformJS.Navigation.navigateToChannel("SelectionPage", state);
},
},
{
}),
});
})();
