﻿(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Personalization",
{
MySourcesDataProvider: WinJS.Class.define(function (options) {
this._init(options);
}, {
_cnpDataManager: null,
_options: null,
_init: function _init(options) {
this._options = options || {};
this._cnpDataManager = NewsJS.Customization.getSourcesDataManager(options);
},
request: function request() {
var that = this;
var market = this._options.market || Platform.Globalization.Marketization.getCurrentMarket();
return this._cnpDataManager.getAllAsync(market).then(function (sources) {
return that._transformToMyItems(sources);
});
},
_transformToMyItems: function _transformToMyItems(sources) {
var that = this;
var items = [];
if (sources) {
var newOrder = sources.sort(this.compareItems.bind(this));
for (var id in newOrder) {
var item = that.createNewItem(sources[id]);
if (item) {
items.push(item);
}
}
}
return items;
},
_transformToSources: function _transformToSources(myItems) {
var that = this;
var sources = [];
if (myItems) {
for (var id in myItems) {
var source = myItems[id];
sources.push(source.data);
}
}
return sources;
},
// normalize all kinds of data to have basic expect fields for a tile:
// deep copy original data if any, and return new item
_uniformItem: function _uniformItem(originalData, templateId, tileType) {
var item = {};
item.data = originalData ? JSON.parse(JSON.stringify(originalData)) : {};
var market = this._options.market || NewsJS.Globalization.getMarketStringForSources();
item.data.market = item.data.market || market;
item.itemKey = "key",
item.groupKey = "";
item.groupTitle = "";
item.moduleInfo = {
fragmentPath: "/html/templates.html",
templateId: templateId
};
item.tileType = tileType;
item.displayName = "";
item.thumbnail = { url: "" };
item.subTitle = "";
return item;
},
_addOrderBy: function _addOrderBy(sources, order) {
if (sources && order) {
sources.forEach(function (source) {
if (source && source.id )
{
source.orderBy = order.indexOf(source.id) + 1;
}
});
}
},
compareItems: function compareItems(a, b) {
if (a.orderBy && b.orderBy) {
return a.orderBy - b.orderBy;
}
else {
return 0;
}
},
// create item from sources
createNewItem: function createNewItem(selectedSource) {
var that = this;
var templateId = NewsJS.Personalization.Utilities.todayMyItemsTemplate;
var item = that._uniformItem(selectedSource, templateId, NewsJS.Personalization.Utilities.tileType.Customize);
item.displayName = selectedSource.displayname;
item.sourceDesc = selectedSource.sourcedescription;
item.thumbnail = { url: selectedSource.win8_image };
item.key = selectedSource.id;
item.itemKey = selectedSource.id;
item.id = selectedSource.id;
return item;
},
// create item from sources
createAddMoreItem: function createAddMoreItem(templateId) {
var that = this;
var item = that._uniformItem(null, templateId, NewsJS.Personalization.Utilities.tileType.AddMore);
item.displayName = "More Sources";
item.isCommonJSAddTile = true;
item.id = "addMore";
item.key = "addMore";
item.itemKey = "addMore";
return item;
},
// Action handler: Add/Remove
handleRemoveItemsAsync: function handleRemoveItemsAsync(removedItems) {
var that = this;
var sources = that._transformToSources(removedItems);
return that._cnpDataManager.removeManyAsync(sources);
},
handleSaveItemsAsync: function handleSaveItemsAsync(items) {
var that = this;
var sources = that._transformToSources(items);
return that._cnpDataManager.saveAsync(sources);
},
handleItemReorderedAsync: function handleItemReorderedAsync(order) {
var that = this;
return this._cnpDataManager.getAllAsync().then(function (sources) {
that._addOrderBy(sources, order);
return that._cnpDataManager.saveAsync(sources);
});
},
}), 
});
})();
