﻿(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Personalization",
{
MyTopicsControl: WinJS.Class.mix(WinJS.Class.define(function (element, options) {
var that = this;
this.element = element || document.createElement("div");
element.winControl = this;
WinJS.Utilities.addClass(this.element, "myTopicsItemsContainer");
WinJS.UI.setOptions(this, options);
this._provider = new NewsJS.Personalization.MyTopicsDataProvider();
this.element.addEventListener("listreordered", this.handleItemReordered.bind(this));
this.element.addEventListener("itemdeleted", this.handleItemDeleted.bind(this));
},
{
_dataSource: null,
_listBinding: null,
_listView: null,
_addIndex: 0,
_control: null,
_provider: null,
render: function render() {
var that = this;
return that.renderMyTopics();
},
renderMyTopics: function renderMyTopics(){
var that = this; 
return new WinJS.Promise(function (completeHandler, errorHandler, progressHandler) {
that._provider.getAsync().then( function (items) {
if (items) {
that._addIndex = items.length;
var itemsToShow = items;
if (that._addIndex > NewsJS.Personalization.Utilities.maxCnPItems) {
itemsToShow = items.slice(0, NewsJS.Personalization.Utilities.maxCnPItems);
that._addIndex = itemsToShow.length;
}
that._listBinding = new WinJS.Binding.List(itemsToShow);
that._dataSource = that._listBinding.dataSource;
that._handleItemCountChanged(that._addIndex);
var clusterBinding = {
contentOptions: {
friendlyName: "myTopicsList",
itemDataSource: that._dataSource,
itemTemplate: that._renderItem.bind(that),
oniteminvoked: that.onItemInvoke.bind(that),
onadditeminvoked: that._onAddItemInvoked.bind(that),
addTileItemTemplate: that._renderAddItem.bind(that),
enabledFeatures: CommonJS.UI.ListView2.FEATURES.ALL
}
}
that._renderOuter(clusterBinding, completeHandler);
};
});
});
},
_renderOuter: function _renderOuter(binding, completeHandler) {
var div = this._myItemsContainer = document.createElement("div");
WinJS.Utilities.addClass(div, "myTopicsListViewContainer");
var that = this;
this._listView = new CommonJS.UI.ListView2(div, binding.contentOptions);
this._listView.addEventListener("selectionchanging", this._onSelectionChangingHandler.bind(this));
this._listView.addEventListener("loadingstatechanged", function (e) {
var loadingState = that._listView.loadingState;
if (loadingState === "complete") {
completeHandler();
}
});
this.element.appendChild(div);
},
_renderItem: function _renderItem(itemPromise) {
return {
element: itemPromise.then(function _cnpRenderer(item) {
var div = document.createElement("div");
WinJS.Utilities.addClass(div, "myTopicsItem");
var binding = item.data;
CommonJS.loadModule(binding.moduleInfo, binding, div).then();
return div;
})
};
},
renderMyTrends: function renderMyTrends() {
var that = this;
var trendsBinding = {
contentControl: "NewsJS.Personalization.MyTrendsControl",
moduleInfo: {
renderer: that._renderTrendsOuter.bind(that),
isInteractive: true,
},
}
return that._renderTrendsOuter(trendsBinding);
},
_renderTrendsOuter: function _renderTrendsOuter(binding) {
var div = this._myTrendsItemsContainer = document.createElement("div");
this.element.appendChild(div);
WinJS.Utilities.addClass(div, "myTrendsItemsContainer");
this._trendsView = PlatformJS.Utilities.createObject(binding.contentControl, div, binding.contentOptions);
return this._trendsView.render();
},
_onSelectionChangingHandler: function _onSelectionChangingHandler(e) {
var that = this;
if (e) {
e.preventDefault();
return;
}
},
handleItemDeleted: function handleItemDeleted(e) {
var that = this;
var item = e.detail;
if (item) {
that._provider.handleRemoveItemsAsync([item]).then(function () {
that._addIndex--;
that._handleItemCountChanged(that._addIndex);
});
}
},
handleItemReordered: function handleItemReordered(e) {
var order = this._listView.getOrder();
this._provider.handleItemReorderedAsync(order).then();
},
dispose: function dispose() {
},
onItemInvoke: function onItemInvoke(e) {
var that = this;
var ds = this._dataSource;
ds.itemFromIndex(e.detail.itemIndex).then(function (item) {
if (!that.isAddTile(item) || that._isListFull) {
that._myItemsItemInvoked(item);
}
else {
that._onAddItemInvoked(item)
}
});
},
isAddTile: function isAddTile(tileItem) {
return (tileItem && tileItem.data && tileItem.data.isCommonJSAddTile);
},
// Item click action for tiles:
// MySources: click go to source page
// MyTopics: top: click go to "Topic Page"
//          bottom: trending: do nothing
_myItemsItemInvoked: function _myItemsItemInvoked(item) {
var that = this;
// Tiles in MyTopics Cluster
var data = item ? item.data : null;
//More sources Tile Clicked to go to "Sources Page"
if (data.isCommonJSAddTile) {
PlatformJS.Navigation.navigateToChannel("MyTopics");
}
else if (data && data.data) {
that._openTopicPage(data.data);//todo
}
},
_openTopicPage: function _openTopicPage(topic) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(NewsJS.Telemetry.String.ImpressionNavMethod.topic);
WinJS.Navigation.navigate(
{
fragment: "/html/search.html",
page: "NewsJS.Search"
},
{
topicEntry: topic,
queryText: topic.query,
searchOrigin: NewsJS.Telemetry.Search.Origin.topics
});
},
_onAddItemInvoked: function _onAddItemInvoked(e) {
var state = {
dataProvider: "NewsJS.TopicsGalleryProvider",
type: "TOPICS",
allowAdd: true,
allowBrowse: true,
};
PlatformJS.Navigation.navigateToChannel("SelectionPage", state);
},
_handleItemCountChanged: function _handleItemCountChanged(itemCount) {
var that = this;
var isFull = itemCount >= NewsJS.Personalization.Utilities.maxCnPItems;
var isEmpty = itemCount <= 1;
if (isFull) {
WinJS.Utilities.addClass(this.element, "myListFull");
}
else {
WinJS.Utilities.removeClass(this.element, "myListFull");
}
if (isEmpty) {
WinJS.Utilities.addClass(this.element, "myListEmpty");
}
else {
WinJS.Utilities.removeClass(this.element, "myListEmpty");
}
this._isListFull = isFull;
},
_renderAddItem: function _renderAddItem(itemPromise) {
var that = this;
return {
element: itemPromise.then(function _cnpRenderer(item) {
var div = document.createElement("div");
WinJS.Utilities.addClass(div, "myTopicsItem");
var templateId = NewsJS.Personalization.Utilities.myItemsAddTile;
var item = that._provider.createAddMoreItem(templateId);
CommonJS.loadModule(item.moduleInfo, item, div).then();
return div;
})
};
},
},
{
}), WinJS.Utilities.eventMixin),
});
})();
