﻿(function () {
"use strict";
WinJS.Namespace.define("NewsJS.Personalization",
{
MyTopicsDataProvider: WinJS.Class.define(function (options) {
this._init();
}, {
_cnpDataManager: null,
_init: function _init() {
this._cnpDataManager = NewsJS.Customization.getTopicsDataManager();
},
getAsync: function getAsync() {
var that = this;
var market = Platform.Globalization.Marketization.getCurrentMarket();
return this._cnpDataManager.getAllAsync(market).then(function (topics) {
return that._transformToMyItems(topics);
});
},
_transformToMyItems: function _transformToMyItems(topics) {
var that = this;
var items = [];
if (topics) {
var newOrder = topics.sort(this.compareItems.bind(this));
for (var id in newOrder) {
var item = that.createNewItem(topics[id]);
if (item) {
items.push(item);
}
}
}
return items;
},
_transformToTopics: function _transformToTopics(myItems) {
var that = this;
var topics = [];
if (myItems) {
for (var id in myItems) {
var topic = myItems[id];
topics.push(topic.displayName);
}
}
return topics;
},
// normalize all kinds of data to have basic expect fields for a tile:
// deep copy original data if any, and return new item
_uniformItem: function _uniformItem(originalData, templateId, tileType) {
var item = {};
item.data = originalData ? JSON.parse(JSON.stringify(originalData)) : {};
item.itemKey = "key",
item.groupKey = "";
item.groupTitle = "";
item.moduleInfo = {
fragmentPath: "/html/templates.html",
templateId: templateId
};
item.tileType = tileType;
item.displayName = "";
item.thumbnail = { url: "" };
item.subTitle = "";
return item;
},
_addOrderBy: function _addOrderBy(topics, order) {
if (topics && order) {
topics.forEach(function (item) {
if (item && item.query) {
item.orderBy = order.indexOf(item.query) + 1;
}
});
}
},
compareItems: function compareItems(a, b) {
if (a.orderBy && b.orderBy) {
return a.orderBy - b.orderBy;
}
else {
return 0;
}
},
createNewItem: function createNewItem(topic) {
var that = this;
var item = that._uniformItem(topic, NewsJS.Personalization.Utilities.todayMyItemsTemplate, NewsJS.Personalization.Utilities.tileType.Customize);
item.displayName = topic.query;
item.query = topic.query;
item.subTitle = "";
item.thumbnail = { url: "" };
item.itemKey = topic.query;
item.id = topic.query;
item.key = topic.query;
return item;
},
// create item from sources
createAddMoreItem: function createAddMoreItem(templateId) {
var that = this;
var item = that._uniformItem(null, templateId, NewsJS.Personalization.Utilities.tileType.AddMore);
item.displayName = "More Topics";
item.isCommonJSAddTile = true;
item.id = "addMore";
item.key = "addMore";
item.itemKey = "addMore";
return item;
},
// Action handler: Add/Remove
handleRemoveItemsAsync: function handleRemoveItemsAsync(removedItems) {
var that = this;
var topics = that._transformToTopics(removedItems);
return that._cnpDataManager.removeManyAsync(topics);
},
handleSaveItemsAsync: function handleSaveItemsAsync(items) {
var that = this;
var topics = that._transformToTopics(items);
return that._cnpDataManager.saveAsync(topics);
},
handleItemReorderedAsync: function handleItemReorderedAsync(order) {
var that = this;
return this._cnpDataManager.getAllAsync().then(function (topics) {
that._addOrderBy(topics, order);
return that._cnpDataManager.saveAsync(topics);
});
},
}),
});
})();
