﻿/* shared\js\FinanceDataProvider.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
DataProviderBase: WinJS.Class.derive(CommonJS.PanoEx.BaseDataProvider, function (options) {
CommonJS.PanoEx.BaseDataProvider.call(this, options);
}, {
_lastUpdateTime: null,
_updateDateTime: true,
lastUpdateTime: {
get: function get() {
return this._lastUpdateTime;
},
set: function set(dateTime) {
if (this._updateDateTime && dateTime) {
msSetImmediate(function () {
var ipage = PlatformJS.Utilities.getDeepObject(PlatformJS, "Navigation.currentIPage");
ipage && ipage.updatePageTimeStamp && ipage.updatePageTimeStamp(dateTime);
});
this._lastUpdateTime = dateTime;
}
},
},
_processResponseData: function _processResponseData(response) {
var promise = CommonJS.PanoEx.BaseDataProvider.prototype._processResponseData.call(this, response);
if (!response.isStaleResponse && this._recordForEarlyFetchEnabled) {
promise = promise.then(this._recordDataForEarlyFetch.bind(this));
}
return promise;
},
_recordForEarlyFetchEnabled: false,
_recordDataForEarlyFetch: function _recordDataForEarlyFetch(data) {
FinanceAppJS.LocalSettingsManager.instance.setData(this.providerName, { data: data, params: JSON.stringify(this.getOptions()) });
return data;
},
prefetchData: function prefetchData() {
if (this._recordDataForEarlyFetch && !Platform.Process.isRecoverMode) {
var data = FinanceAppJS.LocalSettingsManager.instance.getData(this.providerName, data);
if (data && typeof data === 'object' && data.data && data.params === JSON.stringify(this.getOptions())) {
this._lastFetchData = data.data;
} else {
debugger;
}
}
if (!this.queryService) {
return;
}
var urlParams = this.getQueryServiceParams();
var qsOptions = this.getQueryServiceOptions();
this.queryService.downloadDataAsync(urlParams, null, null, qsOptions);
},
getOptions: function getOptions() {
var options = {};
for (var param in this) {
if (typeof this[param] === 'function' || param[0] === '_' || CommonJS.PanoEx.BaseDataProvider.prototype.hasOwnProperty(param)) {
continue;
}
options[param] = this[param];
}
return options;
},
/*getCannedDataFile functin overriden, to get the row and secondary market FRE files*/
getCannedDataFile: function getCannedDataFile() {
if (!this._cannedDataFile) {
if (this.enableFRE) {
var market = Platform.Globalization.Marketization.getCurrentMarket();
// We check if the market is row by checking for the default exchange which is none in this case
if (FinanceAppJS.Globalization.getDefaultExchange() === 'none') {
market = "row";
}
if (market !== "row" && FinanceAppJS.Globalization.isSecondaryMarket(market)) {
if (FinanceAppJS.Globalization.isProviderSupported(market, this.providerName)) {
market = FinanceAppJS.Globalization.getPrimaryMarket(market);
}
}
this._cannedDataFile = this.cannedDataFile.replace("{market}", market);
this._cannedDataFile = this._cannedDataFile.replace(/\{\w+\}/, function (params, value) {
value = value.substr(1, value.length - 2);
return params[value] || value;
}.bind(null, this));
}
}
return this._cannedDataFile;
},
}),
_startupData: null,
loadStartupDataProviders: function loadStartupDataProviders() {
var clusters = FinanceAppJS.Globalization.getClusters("Home");
var InitialProviders = {};
/* clusters("Home") is null for row markets */
if (clusters && clusters[0] === 'newsCluster') {
InitialProviders = {
EquityDataProvider: { isSnapped: false, wlSymbols: (new FinanceAppJS.Roaming).getWatchListSymbols() },
EditorialNewsDataProvider: {}
};
}
if (clusters && clusters[1] === 'gettingStartedCluster') {
if (PlatformJS.Services.configuration.getBool("EnableGettingStarted") && CommonJS.TempCluster && CommonJS.TempCluster.shouldShowTempCluster()) {
InitialProviders.TempClusterProvider = { isPreview: FinanceAppJS.Settings.cmsPreviewEnabled };
}
}
var startupDataProviders = Windows.Storage.ApplicationData.current.localSettings.values["StartupDataProviders"];
Windows.Storage.ApplicationData.current.localSettings.values["StartupDataProviders"] = null;
startupDataProviders = startupDataProviders ? JSON.parse(startupDataProviders) : InitialProviders;
for (var p in startupDataProviders) {
var dp = CommonJS.PanoEx.BaseDataProvider.findOrCreateProvider(p, startupDataProviders[p]);
dp._recordForEarlyFetchEnabled = true;
dp.prefetchData();
}
},
saveStartupDataProviders: function saveStartupDataProviders(providers) {
var startupDataProviders = {};
var count = 0;
for (var p in providers) {
var data = providers[p]._lastFetchData;
if (data) {
startupDataProviders[p] = providers[p].getOptions();
count++;
if (count === 2) {
break;
}
}
}
Windows.Storage.ApplicationData.current.localSettings.values["StartupDataProviders"] = JSON.stringify(startupDataProviders);
}
});
var findOrCreateProvider = CommonJS.PanoEx.BaseDataProvider.findOrCreateProvider;
CommonJS.PanoEx.BaseDataProvider.findOrCreateProvider = function (providerName, options) {
var provider = FinanceAppJS.DataProvider[providerName];
if (provider) {
CommonJS.PanoEx.BaseDataProvider.registerProvider(providerName, provider);
}
return findOrCreateProvider(providerName, options);
};
})();
(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
TempClusterProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this._tempClusterProvider = new CommonJS.TempClusterDataProvider();
this.market = encodeURIComponent(FinanceAppJS.Globalization.getMarket().toLowerCase());
}, {
queryServiceId: "GettingStarted",
refreshInterval: 0,
bypassCache: false,
_isPreview: false,
enableFRE: false,
market: null,
isPreview: {
get: function get(value) {
return this._isPreview;
},
set: function set(value) {
this._isPreview = value;
if (this._isPreview) {
this.bypassCache = true;
}
else {
this.bypassCache = false;
}
}
},
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams["previewString"] = this._isPreview ? encodeURIComponent("/preview") : "";
urlParams.insert("market", this.market);
return urlParams;
},
forceRefresh: function forceRefresh() {
FinanceAppJS.DataProvider.DataProviderBase.prototype.getData.call(this, { forceRefresh: true });
},
dataBuilder: function dataBuilder(data) {
var parsedData = this._tempClusterProvider.parseData(data);
Windows.Storage.ApplicationData.current.localSettings.values[FinanceAppJS.DataProvider.TempClusterProvider.clusterValidationLocalKey] = !utils.isNonEmptyObject(parsedData);
return parsedData;
}
}, {
_localKey: null,
clusterValidationLocalKey: {
get: function get() {
if (!FinanceAppJS.DataProvider.TempClusterProvider._localKey) {
FinanceAppJS.DataProvider.TempClusterProvider._localKey = FinanceAppJS.Globalization.getMarket().toLowerCase() + "_IsTempClusterInvalid";
}
return FinanceAppJS.DataProvider.TempClusterProvider._localKey;
}
}
})
});
})();
(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.DataProvider", {
NextStepsClusterProvider: WinJS.Class.derive(FinanceAppJS.DataProvider.DataProviderBase, function (options) {
FinanceAppJS.DataProvider.DataProviderBase.call(this, options);
this._nextStepsClusterProvider = new CommonJS.TempClusterDataProvider();
this._isPreview = FinanceAppJS.Settings.cmsPreviewEnabled;
this.market = encodeURIComponent(FinanceAppJS.Globalization.getMarket().toLowerCase());
}, {
market: null,
queryServiceId: "NextSteps",
refreshInterval: 0,
bypassCache: false,
_isPreview: false,
isPreview: {
get: function get(value) {
return this._isPreview;
},
set: function set(value) {
this._isPreview = value;
if (this._isPreview) {
this.bypassCache = true;
}
else {
this.bypassCache = false;
}
}
},
getQueryServiceParams: function getQueryServiceParams() {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams["previewString"] = this._isPreview ? encodeURIComponent("/preview") : "";
urlParams.insert("market", this.market);
return urlParams;
},
forceRefresh: function forceRefresh() {
FinanceAppJS.DataProvider.DataProviderBase.prototype.getData.call(this, { forceRefresh: true });
},
dataBuilder: function dataBuilder(data) {
var parsedData = this._nextStepsClusterProvider.parseNextStepsData(data);
return parsedData;
}
}, {
})
});
})();
/* shared\js\FinanceBasePage.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS", {
FinanceBasePage: WinJS.Class.derive(CommonJS.PanoEx.BasePage, function (state) {
/// <summary>
/// The Base Finance Page to be derived by each of the Page in the Finance App
/// Automatically handles dataprovider initialization and clusters for the page given the definitions of clusters.
/// </summary>
CommonJS.PanoEx.BasePage.call(this, state);
if (FinanceAppJS.Globals.FRE) {
utils.disableEdgies();
}
/* Count Weight for App Termination on suspend Heuristic */
var weight, pageCountWeightDict = Platform.Configuration.ConfigurationManager.custom.getDictionary("PageCountWeight");
if (pageCountWeightDict && (weight = pageCountWeightDict.getInt32(WinJS.Navigation.location.channelId))) {
this.countWeight = weight;
}
// Attach action to Help button
msSetImmediate(utils.tryAttachHelpButtonAction());
/* */
}, {
_freClickMessageBar: null,
enableQuickSearchDefault: true,
_lastSuspendedTime: null,
progressType: "headerProgressType",
_watermarkTemplate: "{0}",
showDisclaimer: false,
updatePageTimeStamp: function updatePageTimeStamp(value) {
if (!value) { return; }
value = value.constructor !== Date ? new Date(value) : value;
if (this._lastUpdatedTime && this._lastUpdatedTime > value && this._lastUpdatedTime < new Date()) { return; }
this._lastUpdatedTime = value;
var waterMarkElement = this._panoControl._watermarkElement;  //accessing _watermarkElement which is a private variable of Panorama control, as cux has not provided with template
if (waterMarkElement.children.length !== 0) {
waterMarkElement.innerHTML = "";
}
var dateStrWaterMark = this._getWaterMarkDateString(value);
if (dateStrWaterMark) {
var start = document.createElement("div");
start.innerText = dateStrWaterMark + " ";
waterMarkElement.appendChild(start);
}
var quotesDisclaimer = this._getWaterMarkQuotesDisclaimer();
if (quotesDisclaimer) {
var mid = document.createElement("div");
mid.innerText = " " + quotesDisclaimer + " ";
waterMarkElement.appendChild(mid);
if (this.showDisclaimer) {
mid.addEventListener("click", function (event) {
var productDetailsURL = FinanceAppJS.Globalization.getAboutLinks();
Windows.System.Launcher.launchUriAsync(new Windows.Foundation.Uri(productDetailsURL + "#Disclaimer"));
});
}
}
var MStarAttribute = this._getWaterMarkMStarAttribute();
if (MStarAttribute) {
var last = document.createElement("div");
last.innerText = MStarAttribute;
waterMarkElement.appendChild(last);
}
},
_getWaterMarkDateString: function _getWaterMarkDateString(value) {
return utils.getResourceString("LastUpdate").format(utils.formatLastUpdatedTime(value, this._isSnapped));
},
_getWaterMarkQuotesDisclaimer: function _getWaterMarkQuotesDisclaimer() {
var quotesDelayStr;
if (this.showDisclaimer) {  //(US mkt and stock/ETF/Home) or (non-us mkt and US-Stock/US-ETF)
var disclaimer = utils.getResourceString("BATS_RT_Disclaimer") + ".";
quotesDelayStr = disclaimer;
}
else if (!(this.showDisclaimer) && this.quotesDelay) { //non US and quotes delay
quotesDelayStr = FinanceAppJS.Utils.Formatters.stringFormatter(utils.getResourceString("Quotes_DelayedByTime"), [this.quotesDelay]);
}
else { //all others
quotesDelayStr = utils.getResourceString("Quotes_Delayed");
}
if (!this._hideQuotesDelayText) {
return quotesDelayStr;
}
return null;
},
_getWaterMarkMStarAttribute: function _getWaterMarkMStarAttribute() {
var quotesDelayStr;
if (!this.isSnapped && this._showMarketDataAttribution) {
quotesDelayStr = " " + utils.getResourceString("MARKETDATA_MSTAR_ATTRIB");
}
if (!this._hideQuotesDelayText) {
return quotesDelayStr;
}
return null;
},
timeStampFormatter: function timeStampFormatter(value) {
// TODO: this function to be removed if we are not using it anymore
// to be verified against the calls from the base class.
var dateStr = utils.getResourceString("LastUpdate").format(utils.formatLastUpdatedTime(value, this._isSnapped));
var quotesDelayStr;
if (this.showDisclaimer) {  //(US mkt and stock/ETF/Home) or (non-us mkt and US-Stock/US-ETF)
var disclaimer = utils.getResourceString("BATS_RT_Disclaimer") + ". ";
quotesDelayStr = disclaimer;
}
else if (!(this.showDisclaimer) && this.quotesDelay) { //non US and quotes delay
quotesDelayStr = FinanceAppJS.Utils.Formatters.stringFormatter(utils.getResourceString("Quotes_DelayedByTime"), [this.quotesDelay]);
}
else { //all others
quotesDelayStr = utils.getResourceString("Quotes_Delayed");
}
if (quotesDelayStr && !this.isSnapped && this._showMarketDataAttribution) {
quotesDelayStr += " " + utils.getResourceString("MARKETDATA_MSTAR_ATTRIB");
}
return this._hideQuotesDelayText ? dateStr : dateStr + " " + quotesDelayStr;
},
quotesDelay: null,
setPageTimeUpdateDataProvider: function setPageTimeUpdateDataProvider(providerName) {
var dataProvider = this._dataProviders && this._dataProviders[providerName];
if (dataProvider) {
for (var p in this._dataProviders) {
this._dataProviders[p]._updateDateTime = false;
}
dataProvider._updateDateTime = true;
}
},
onSuspending: function onSuspending(event) {
/// <summary>
/// Handles Suspend change event
/// </summary>
/// <param name="event">
/// Suspend change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
this._lastSuspendedTime = Date.now();
},
onResuming: function onResuming(event) {
/* Dont refresh Providers if we are resuming after ResetHistoryMillisecondThreshold, as we will be navigating away from the page anyway. */
var resetHistoryMillisecondThreshold = Platform.Configuration.ConfigurationManager.custom.getInt32("ResetHistoryMillisecondThreshold");
var millisecondsSinceLastSuspend = Date.now() - this._lastSuspendedTime;
if (millisecondsSinceLastSuspend <= resetHistoryMillisecondThreshold) {
CommonJS.PanoEx.BasePage.prototype.onResuming.call(this, event);
}
},
_createClusterFromDef: function _createClusterFromDef(key, clusterDef) {
var cluster = CommonJS.PanoEx.BasePage.prototype._createClusterFromDef.call(this, key, clusterDef);
cluster.pinned = clusterDef.pinned;
cluster.isLastCluster = clusterDef.isLastCluster;
cluster.isTempCluster = clusterDef.isTempCluster;
cluster.isImmersive = clusterDef.isImmersive;
cluster.platformInvisibleCluster = clusterDef.platformInvisibleCluster;
cluster.platformInvisiblePermanentCluster = clusterDef.platformInvisiblePermanentCluster;
return cluster;
},
setClusterDefinitions: function setClusterDefinitions(clusterDefs, clusterList) {
var clusters = CommonJS.PanoEx.BasePage.prototype.setClusterDefinitions.call(this, clusterDefs, clusterList);
// If the last cluster in the pano is meant to be immersive, disable scrolling beyond the cluster, by setting compactPanorama to true
if (clusters && clusters.length > 0) {
if (clusters.getAt(clusters.length - 1).isImmersive) {
if (this._panoControl) {
this._panoControl.compactPanorama = true;
}
}
}
return clusters;
},
onNavigateAway: function onNavigateAway() {
CommonJS.PanoEx.BasePage.prototype.onNavigateAway.call(this);
FinanceAppJS.AppManager.processIdleTasks();
},
onPageLoadComplete: function onPageLoadComplete() {
CommonJS.PanoEx.BasePage.prototype.onPageLoadComplete.call(this);
//FinanceAppJS.DataProvider.saveStartupDataProviders(this._dataProviders);
FinanceAppJS.AppManager.appIsIdle = true;
//Make CMS call if GS cluster data is invalid and reset the variable if reqd
if (WinJS.Navigation.location.channelId === "Home") {
if (Windows.Storage.ApplicationData.current.localSettings.values[FinanceAppJS.DataProvider.TempClusterProvider.clusterValidationLocalKey] === true) {
var provider = CommonJS.PanoEx.BaseDataProvider.findOrCreateProvider("TempClusterProvider", {});
provider.getData({ forceRefresh: true });
}
}
},
}, {
saveStartupDataProviders: function saveStartupDataProviders() {
var ipage = PlatformJS.Navigation.mainNavigator.currentIPage;
if (ipage instanceof FinanceAppJS.FinanceBasePage && ipage._dataProviders) {
FinanceAppJS.DataProvider.saveStartupDataProviders(ipage._dataProviders);
}
},
freNavigationSuppressor: function freNavigationSuppressor(event) {
var loc = WinJS.Navigation.location;
var BasePageManager = CommonJS.PanoEx.BasePageManager;
if (loc && FinanceAppJS.Constants.HomePage.availableOnFREChannelIdList.indexOf(event.detail.location.channelId) === -1) {
BasePageManager.showFREClickMessage();
event.preventDefault();
event.stopImmediatePropagation();
PlatformJS.Navigation.mainNavigator.canNavigate = true;
PlatformJS.Navigation.mainNavigator.currentIPage = BasePageManager.currentIPage;
}
},
showFREClickMessage: function showFREClickMessage() {
var createElements = PlatformJS.Utilities.createElements;
var anchor = document.body;
if (!this._freClickMessageBar) {
var flyoutDiv = document.createElement("div");
var freFlyout = this._freClickMessageBar = new WinJS.UI.Flyout(flyoutDiv);
WinJS.Utilities.addClass(flyoutDiv, 'networkUnavailableFlyoutContainer');
var container = createElements(
{
tag: "div", attrs: null, children: [
{
tag: "div", attrs: { className: 'noInternetIcon' }, children: [
{ tag: "div", attrs: { className: 'platformErrorIconGlyph1' } },
{ tag: "div", attrs: { className: 'platformErrorIconGlyph2' } },
]
},
{ tag: "br", attrs: null },
{ tag: "div", attrs: { className: 'errortextLine1', textContent: utils.getResourceString('OfflineNoNetConnectionLine1') } },
{ tag: "div", attrs: { className: 'errortextLine2', textContent: utils.getResourceString('OfflineNoNetConnectionLine2') } }
]
}, flyoutDiv);
anchor.appendChild(flyoutDiv);
utils.vAlignContainerInMiddle(flyoutDiv);
}
this._freClickMessageBar.show(anchor, "top", "center");
},
})
});
WinJS.Namespace.define("FinanceAppJS.DataProvider", {});
var providerMap = {
EquityDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
HomeRatesDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
chartDataProvider: "/shared/js/chartDataProviders.js",
StockChartDataProvider: "/shared/js/chartDataProviders.js",
FundChartDataProvider: "/shared/js/chartDataProviders.js",
ETFChartDataProvider: "/shared/js/chartDataProviders.js",
IndexChartDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
MyStocksDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
TodayRatesDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
TodayFundPicksDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
AcrossMarketDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
AcrossMarketCCDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
EditorialNewsDataProvider: "/pages/News/js/NewsDataProviders.js",
BingNewsDataProvider: "/pages/News/js/NewsDataProviders.js",
BingStockNewsDataProvider: "/pages/News/js/NewsDataProviders.js",
WorldMarketDataProvider: "/pages/WorldMarket/js/WorldMarket.js",
GlobalCurrencyDataProvider: "/pages/GlobalCurrency/js/GlobalCurrency.js",
AverageRatesDataProvider: "/pages/Rates/js/averages.js",
BondsDataProvider: "/pages/Bonds/js/BondsPage.js",
CommoditiesDataProvider: "/pages/Commodities/js/Commodities.js",
ETFDataProvider: "/pages/ETF/js/ETFDataProvider.js",
MarketDataProvider: "/pages/Market/js/Market.js",
EntityPanoProvider: "/pages/EntityCluster/js/EntityPanoProviders.js",
SourcesDataProvider: "/pages/EntityCluster/js/EntityPanoProviders.js",
FundPicksDataProvider: "/pages/Picks/js/FundPicks.js",
TopEdgyWatchlistDataProvider: "/pages/MarketToday/js/MarketTodayDataProviders.js",
TodaySourcesDataProvider: "/pages/EntityCluster/js/EntityPanoProviders.js",
MoneyEntityPanoProvider: "/pages/EntityCluster/js/EntityPanoProviders.js",
CurrencyConverterDataProvider: "/pages/CurrencyConverter/js/CurrencyConverterDataProvider.js",
};
for (var p in providerMap) {
CommonJS.PanoEx.BaseDataProvider.registerProvider(p, "FinanceAppJS.DataProvider." + p, providerMap[p]);
}
CommonJS.PanoEx.BasePageManager.getClusterList = function (channelId) {
return FinanceAppJS.Globalization.getClusters(channelId);
};
CommonJS.PanoEx.BasePageManager._freNavigationSuppressor = FinanceAppJS.FinanceBasePage.freNavigationSuppressor.bind(FinanceAppJS.FinanceBasePage);
CommonJS.PanoEx.BasePageManager.showFREClickMessage = FinanceAppJS.FinanceBasePage.showFREClickMessage.bind(FinanceAppJS.FinanceBasePage);
CommonJS.PanoEx.Clusters.EntityCluster.defaultOptions = {
className: "financeNewsClusterHeight",
mode: "static",
maxColumnCount: null,
theme: "financeAppTheme",
unitWidth: null,
configKey: "EntityClusterDefaultFinanceSportsConfig",
configObject: "CommonJS.News.EntityClusterConfig.DefaultFinanceSports",
};
})();
/* shared\js\FinanceControls.js */
(function () {
'use strict';
function showHeaderAfterRender(domElement, event) {
if (this._binding_showHeaderAfterRender) {
this.removeEventListener("rendercomplete", this._binding_showHeaderAfterRender);
this._binding_showHeaderAfterRender = null;
}
try {
if (typeof event.detail === 'string' && event.detail !== 'success') {
return;
}
var headerElem = domElement.parentElement.children[0];
if (WinJS.Utilities.hasClass(headerElem, "platformClusterHeader")) {
if (getComputedStyle(headerElem).visibility === 'hidden') {
headerElem.style.visibility = 'visible';
}
}
}
catch (ex) {
debugger;
}
}
WinJS.Namespace.define("FinanceAppJS.Controls", {
ModuleClusterControl: WinJS.Class.derive(CommonJS.PanoEx.Clusters.ModuleCluster, function (domElement, options) {
CommonJS.PanoEx.Clusters.ModuleCluster.call(this, domElement, options);
this._binding_showHeaderAfterRender = showHeaderAfterRender.bind(this, domElement);
this.addEventListener("rendercomplete", this._binding_showHeaderAfterRender);
}, {
dispose: function dispose() {
CommonJS.PanoEx.Clusters.ModuleCluster.prototype.dispose.call(this);
if (this._binding_showHeaderAfterRender) {
this.removeEventListener("rendercomplete", this._binding_showHeaderAfterRender);
this._binding_showHeaderAfterRender = null;
}
},
}),
BaseCluster: WinJS.Class.derive(CommonJS.PanoEx.BaseCluster, function (domElement, options) {
CommonJS.PanoEx.BaseCluster.call(this, domElement, options);
this._binding_showHeaderAfterRender = showHeaderAfterRender.bind(this, domElement);
this.addEventListener("rendercomplete", this._binding_showHeaderAfterRender);
}, {
dispose: function dispose() {
CommonJS.PanoEx.BaseCluster.prototype.dispose.call(this);
if (this._binding_showHeaderAfterRender) {
this.removeEventListener("rendercomplete", this._binding_showHeaderAfterRender);
this._binding_showHeaderAfterRender = null;
}
},
}),
ItemsCluster: WinJS.Class.derive(CommonJS.PanoEx.Clusters.ItemsCluster, function (domElement, options) {
CommonJS.PanoEx.Clusters.ItemsCluster.call(this, domElement, options);
this._binding_showHeaderAfterRender = showHeaderAfterRender.bind(this, domElement);
this.addEventListener("rendercomplete", this._binding_showHeaderAfterRender);
}, {
dispose: function dispose() {
CommonJS.PanoEx.Clusters.ItemsCluster.prototype.dispose.call(this);
if (this._binding_showHeaderAfterRender) {
this.removeEventListener("rendercomplete", this._binding_showHeaderAfterRender);
this._binding_showHeaderAfterRender = null;
}
},
}),
EntityCluster: WinJS.Class.derive(CommonJS.PanoEx.Clusters.EntityCluster, function (domElement, options) {
CommonJS.PanoEx.Clusters.EntityCluster.call(this, domElement, options);
this._binding_showHeaderAfterRender = showHeaderAfterRender.bind(this, domElement);
this.addEventListener("rendercomplete", this._binding_showHeaderAfterRender);
}, {
dispose: function dispose() {
CommonJS.PanoEx.Clusters.EntityCluster.prototype.dispose.call(this);
if (this._binding_showHeaderAfterRender) {
this.removeEventListener("rendercomplete", this._binding_showHeaderAfterRender);
this._binding_showHeaderAfterRender = null;
}
},
}),
TableCluster: WinJS.Class.derive(CommonJS.PanoEx.Clusters.TableCluster, function (domElement, options) {
CommonJS.PanoEx.Clusters.TableCluster.call(this, domElement, options);
this._binding_showHeaderAfterRender = showHeaderAfterRender.bind(this, domElement);
this.addEventListener("rendercomplete", this._binding_showHeaderAfterRender);
}, {
dispose: function dispose() {
CommonJS.PanoEx.Clusters.TableCluster.prototype.dispose.call(this);
if (this._binding_showHeaderAfterRender) {
this.removeEventListener("rendercomplete", this._binding_showHeaderAfterRender);
this._binding_showHeaderAfterRender = null;
}
},
}),
TempCluster: WinJS.Class.derive(CommonJS.PanoEx.BaseCluster, function (domElement, options) {
//During FRE launch, set display of cluster to none so that there is no jerkiness
this._clusterElement = domElement.parentElement;
if (typeof Windows.Storage.ApplicationData.current.localSettings.values[FinanceAppJS.DataProvider.TempClusterProvider.clusterValidationLocalKey] === 'undefined') {
if (this._clusterElement) {
this._clusterElement.style.display = 'none';
}
}
if (!FinanceAppJS.Utilities.NavigationController.defaultControllerAdded) {
FinanceAppJS.Utilities.NavigationController.addDefaultController();
}
if (!FinanceAppJS.Utilities.NavigationController.startClusterEntityViewsAdded) {
FinanceAppJS.Utilities.NavigationController.addStartClusterEntityViews();
}
this._domElement = domElement;
CommonJS.PanoEx.BaseCluster.call(this, domElement, options);
WinJS.UI.setOptions(this, options);
var options = {
templateName: "gsTemplate",
semanticZoomEnabled: false
};
this._tempClusterControl = new CommonJS.TempCluster(domElement, options);
this._binding_showHeaderAfterRender = showHeaderAfterRender.bind(this, domElement);
this.addEventListener("rendercomplete", this._binding_showHeaderAfterRender);
},
{
_clusterElement: null,
update: function update(data) {
this._domElement.innerHTML = "";
if (this._clusterElement) {
if (this._clusterElement.style.display === 'none') {
this._clusterElement.style.display = 'inline-block';
}
}
if (!this._tempClusterControl.removeClusterHandler) {
this._tempClusterControl.removeClusterHandler = this.ipage.removeCluster.bind(this.ipage, this.clusterDef.clusterKey, true);
}
this._tempClusterControl.messageData = data.data;
var headerElem = this._domElement.parentElement.children[0];
if (WinJS.Utilities.hasClass(headerElem, "platformClusterHeader")) {
headerElem.textContent = data.clusterTitle;
}
return this._tempClusterControl.render();
},
dispose: function dispose() {
CommonJS.PanoEx.BaseCluster.prototype.dispose.call(this);
if (this._binding_showHeaderAfterRender) {
this.removeEventListener("rendercomplete", this._binding_showHeaderAfterRender);
this._binding_showHeaderAfterRender = null;
}
}
}),
NextStepsCluster: WinJS.Class.derive(CommonJS.PanoEx.BaseCluster, function (domElement, options) {
if (!FinanceAppJS.Utilities.NavigationController.defaultControllerAdded) {
FinanceAppJS.Utilities.NavigationController.addDefaultController();
}
if (!FinanceAppJS.Utilities.NavigationController.pageEntityViewsAdded) {
FinanceAppJS.Utilities.NavigationController.addPageEntityViews();
}
this._domElement = domElement;
CommonJS.PanoEx.BaseCluster.call(this, domElement, options);
WinJS.UI.setOptions(this, options);
var options = {
templateName: "nsTemplate",
clusterClass: "nextStepsCluster"
};
this._tempClusterControl = new CommonJS.TempCluster(domElement, options);
this._binding_showHeaderAfterRender = showHeaderAfterRender.bind(this, domElement);
this.addEventListener("rendercomplete", this._binding_showHeaderAfterRender);
},
{
update: function update(data) {
this._domElement.innerHTML = "";
this._tempClusterControl.messageData = data.data;
var headerElem = this._domElement.parentElement.children[0];
if (WinJS.Utilities.hasClass(headerElem, "platformClusterHeader")) {
headerElem.textContent = data.clusterTitle;
}
return this._tempClusterControl.render();
},
dispose: function dispose() {
CommonJS.PanoEx.BaseCluster.prototype.dispose.call(this);
if (this._binding_showHeaderAfterRender) {
this.removeEventListener("rendercomplete", this._binding_showHeaderAfterRender);
this._binding_showHeaderAfterRender = null;
}
}
}
),
ToolsCluster: WinJS.Class.derive(CommonJS.PanoEx.BaseCluster, function (domElement, options) {
this.domElement = domElement;
CommonJS.PanoEx.BaseCluster.call(this, domElement, options);
WinJS.UI.setOptions(this, options);
options.itemDataSource = FinanceAppJS.Tools.getToolsList("/pages/MarketToday/html/MarketTodayTemplates.html", "toolsItemTemplate").dataSource,
options.onitemclick = FinanceAppJS.Tools.itemInvokedHandler;
this._view = new CommonJS.Immersive.ItemsContainer(domElement, options);
},
{
_view: null,
dispose: function dispose() {
if (this._view) {
this._view.dispose();
}
},
render: function render() {
var renderPromise = this._view.render();
var domElement = this.domElement;
var that = this;
return renderPromise.then(function () {
var tilesTitlesEleList = document.getElementsByClassName("toolsTileTitle");
var tilesDescsEleList = document.getElementsByClassName("toolsTileTextDescription");
var maxWidth = 0;
for (var i = 0; i < tilesTitlesEleList.length; i++) {
if (tilesTitlesEleList[i].offsetWidth > maxWidth) {
maxWidth = tilesTitlesEleList[i].offsetWidth;
}
}
for (var i = 0; i < tilesTitlesEleList.length; i++) {
tilesDescsEleList[i].style.width = maxWidth + "px";
tilesDescsEleList[i].style.maxWidth = maxWidth + "px";
}
var headerElem = domElement.parentElement.children[0];
if (WinJS.Utilities.hasClass(headerElem, "platformClusterHeader")) {
if (getComputedStyle(headerElem).visibility === 'hidden') {
headerElem.style.visibility = 'visible';
}
}
that.dispatchEvent("rendercomplete");
});
}
})
});
})();
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Controls", {
SingleClusterPanorama: WinJS.Class.define(function (domElement, options) {
this._domElement = domElement;
this._watermarkElement = utils.createElement(this._domElement, "div", { className: "platformWatermark" });
}, {
_clusterDataSource: null,
_childElement: null,
_domElement: null,
_watermark: null,
_watermarkElement: null,
getPanoramaState: function getPanoramaState() { },
beginNotifications: function beginNotifications() { },
inserted: function inserted() { },
onLayoutChange: function onLayoutChange() { },
onOrientationChange: function onOrientationChange() { },
dispose: function dispose() { },
endNotifications: function endNotifications() {
var list = this.clusterDataSource.list;
if (list.length === 0) {
this._domElement.innerHTML = "";
} else {
var item = list.getAt(0);
var container = utils.createElement(this._domElement, "div");
container.style.width = "100%";
container.style.height = "100%";
container.className = "financeAppSnappedContainer";
var obj = PlatformJS.Utilities.createObject(
item.clusterContent.contentControl,
container,
item.clusterContent.contentOptions);
}
},
watermark: {
get: function get() { return this._watermark; },
set: function set(value) {
this._watermarkElement.textContent = value;
}
},
clusterDataSource: {
get: function get() { return this._clusterDataSource; },
set: function set(value) {
this._clusterDataSource = value;
value.createListBinding(this);
this.endNotifications();
}
}
})
});
})();
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Controls", {
GridBox: WinJS.Class.define(function (element, options) {
element.winControl = this;
this._domElement = element;
WinJS.UI.setOptions(this, options);
}, {
_domElement: null,
_index: null,
_vlabels: null,
_hlabels: null,
index: {
get: function get() { return this._index; },
set: function set(value) {
this._index = value;
this.render();
}
},
vlabels: {
get: function get() { return this._index; },
set: function set(value) {
this._vlabels = value;
//this.render();
}
},
hlabels: {
get: function get() { return this._index; },
set: function set(value) {
this._hlabels = value;
//this.render();
}
},
render: function render() {
this._domElement.innerHTML = "";
if (this._hlabels && this._vlabels && this._index >= 0) {
var container = document.createElement('div');
container.className = "boxes";
var count = 0;
for (var i = 0; i < 3; i++) {
for (var j = 0; j < 3; j++) {
var cell = document.createElement('div');
cell.style.msGridRow = i + 1;
cell.style.msGridColumn = j + 1;
count++;
if (count === this._index) {
cell.className = "colored cell";
}
else {
cell.className = "uncolored cell";
}
container.appendChild(cell);
}
var label = document.createElement('div');
label.textContent = utils.getResourceString(this._hlabels[i]);
label.style.msGridRow = i + 1;
label.style.msGridColumn = j + 1;
label.className = "hlabels";
container.appendChild(label);
label = document.createElement('div');
label.textContent = utils.getResourceString(this._vlabels[i]);
label.style.msGridRow = j + 1;
label.style.msGridColumn = i + 1;
label.className = "vlabels";
container.appendChild(label);
}
this._domElement.appendChild(container);
}
}
}, {
})
});
})();
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Controls", {
BarChart: WinJS.Class.define(function (element, options) {
element.winControl = this;
this._domElement = element;
WinJS.UI.setOptions(this, options);
}, {
_domElement: null,
_data: null,
_barMaxWidth: 150,
_barClasses: null,
_showLabels: false,
data: {
get: function get() {
return this._data;
},
set: function set(value) {
this._data = value;
this.render();
}
},
barMaxWidth: {
get: function get() {
return this._barMaxWidth;
},
set: function set(value) {
this._barMaxWidth = value;
}
},
barClasses: {
get: function get() {
return this._barClasses;
},
set: function set(value) {
this._barClasses = value;
}
},
showLabels: {
get: function get() {
return this._showLabels;
},
set: function set(value) {
this._showLabels = value;
}
},
//TODO decide the width of the labels based on the actual pixels occupied by the maximum text and remove hardcoding of 45px
render: function render() {
this._domElement.innerHTML = "";
if (this._data) {
var max = this._data[0];
var min = this._data[0];
for (var k = 0; k < this._data.length; k++) {
if (this._data[k] > max) {
max = this._data[k];
}
if (this._data[k] < min) {
min = this._data[k];
}
}
var scale2, col1, col2, barwidth;
var maxLabelWidth = 45;
var extraLabelWidth = 5;
if ((max >= 0) && (min >= 0)) {
scale2 = max;
col1 = 0;
col2 = (max * this._barMaxWidth) / scale2;
barwidth = this._barMaxWidth - maxLabelWidth;
}
if (max > 0 && min < 0) {
scale2 = max + Math.abs(min);
col1 = (Math.abs(min) * this._barMaxWidth) / scale2;
col2 = (max * this._barMaxWidth) / scale2;
barwidth = this._barMaxWidth - (maxLabelWidth * 2);
}
if (max <= 0 && min <= 0) {
scale2 = Math.abs(min);
col1 = Math.abs(min) * this._barMaxWidth / scale2;
col2 = 0;
barwidth = this._barMaxWidth - maxLabelWidth;
}
var leftMaxWidth = min < 0 ? (Math.abs(min) / scale2 * barwidth + maxLabelWidth) + "px" : 0;
if (!this._showLabels) { leftMaxWidth = col1; barwidth = this._barMaxWidth; }
var container = document.createElement('div');
container.style.display = "-ms-grid";
for (var i = 0; i < this._data.length; i++) {
var leftOuterDiv = document.createElement('div');
var leftInnerDiv = document.createElement('div');
var rightOuterDiv = document.createElement('div');
var rightInnerDiv = document.createElement('div');
var leftLabel = document.createElement('div');
var rightLabel = document.createElement('div');
var rightWidth = 0;
if (this._data[i] < 0) {
leftInnerDiv.style.width = (Math.abs(this._data[i] / scale2) * (barwidth)) + "px";
if (this._showLabels) {
leftLabel.textContent = utils.addLtrMark(FinanceAppJS.Utils.Formatters.numberFormatter(this._data[i], 1, true, ""));
}
leftLabel.appendChild(leftInnerDiv);
}
else if (this._data[i] > 0) {
rightWidth = ((this._data[i] / scale2) * (barwidth));
rightInnerDiv.style.width = rightWidth + "px";
if (this._showLabels) {
rightLabel.textContent = utils.addLtrMark(FinanceAppJS.Utils.Formatters.numberFormatter(this._data[i], 1, true, ""));
}
rightOuterDiv.appendChild(rightInnerDiv);
}
leftLabel.className = "leftLabel";
rightLabel.className = "rightLabel";
leftOuterDiv.appendChild(leftLabel);
rightOuterDiv.appendChild(rightLabel);
leftOuterDiv.style.msGridColumn = 1;
rightOuterDiv.style.msGridColumn = 3;
leftOuterDiv.style.width = leftMaxWidth;
rightOuterDiv.style.width = this._showLabels ? rightWidth + maxLabelWidth + extraLabelWidth + "px" : rightWidth + "px";
leftOuterDiv.style.msGridRow = rightOuterDiv.style.msGridRow = i + 1;
var extraClass = (this.barClasses && this.barClasses[i] ? (" " + this.barClasses[i]) : "");
leftInnerDiv.className = "loseBar" + extraClass;
rightInnerDiv.className = "gainBar" + extraClass;
leftOuterDiv.className = rightOuterDiv.className = "outerbar";
container.appendChild(leftOuterDiv);
container.appendChild(rightOuterDiv);
}
var line = document.createElement('div');
line.style.msGridColumn = 2;
line.style.msGridRow = 1;
line.style.msGridRowSpan = this._data.length;
line.className = "line";
container.appendChild(line);
this._domElement.appendChild(container);
}
}
}, {
})
});
})();
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Controls", {
CurrencyPopulator: WinJS.Class.define(function (element, options) {
element.winControl = this;
this._domElement = element;
WinJS.UI.setOptions(this, options);
}, {
_domElement: null,
_ticker: null,
_map: null,
_data: null,
ticker: {
get: function get() { return this._ticker; },
set: function set(value) {
this._ticker = value;
this.render();
}
},
data: {
get: function get() { return this._data; },
set: function set(value) {
this._data = value;
this.render();
}
},
map: {
get: function get() { return this._map; },
set: function set(value) {
this._map = value;
this.render();
}
},
render: function render() {
this._domElement.innerHTML = "";
if (this._ticker && this._data && this._map) {
var currencyInfo = FinanceAppJS.CurrencyInfo.getCurrencyInfo();
var sortedCurrencies = FinanceAppJS.Controls.CurrencyPopulator.sortCurrencyList(currencyInfo);
var map = this._map;
var dt = this._data;
for (var curr in sortedCurrencies) {
var key = sortedCurrencies[curr];
if ((typeof map[key] !== 'undefined') && (map[key] !== null) && (dt[map[key]].Rt)) {
var option = document.createElement('option');
option.value = key;
if (currencyInfo[key].displayOrder && currencyInfo[key].displayOrder === "reverse") {
option.textContent = utils.getResourceString(currencyInfo[key].currency) + " " + utils.getResourceString(currencyInfo[key].symbol) + " " + currencyInfo[key].country;
}
else {
option.textContent = utils.getResourceString(currencyInfo[key].country) + " " + utils.getResourceString(currencyInfo[key].currency) + " " + currencyInfo[key].symbol;
}
if (this._ticker === key) {
option.selected = true;
}
this._domElement.appendChild(option);
}
}
}
}
}, {
sortedCurrencyList: null,
sortCurrencyList: function sortCurrencyList(currencyInfo) {
if (!FinanceAppJS.Controls.CurrencyPopulator.sortedCurrencyList) {
var sortedCurrencies = [];
for (var prop in currencyInfo) {
sortedCurrencies.push({
key: prop,
value: currencyInfo[prop].country
});
}
sortedCurrencies.sort(function (a, b) {
var currency1 = a.value.toLowerCase();
var currency2 = b.value.toLowerCase();
if (currency1 < currency2) { //sort string ascending
return -1;
}
if (currency1 > currency2) { //sort string descending
return 1;
}
return 0; //default return value (no sorting)
});
FinanceAppJS.Controls.CurrencyPopulator.sortedCurrencyList = sortedCurrencies.map(function (item) {
return item.key;
});
}
return FinanceAppJS.Controls.CurrencyPopulator.sortedCurrencyList;
}
})
});
})();
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Controls", {
ArrayItemsContainer: WinJS.Class.define(function (domElement, options) {
/// A simple Control that creates Tiles from an array instead of list bindings
/// Should be used for static content.
this._domElement = domElement;
WinJS.UI.setOptions(this, options);
if (this.autoRender) {
this.render();
}
this.binding_renderImpl = this._renderImpl.bind(this);
this.binding_attachElements = this._attachElements.bind(this);
this.binding_itemClickPredicate = this._itemClickPredicate.bind(this);
this.binding_itemClickHandler = this._itemClickHandlerImpl.bind(this);
PlatformJS.Utilities.registerItemClickProxy(domElement, this.binding_itemClickPredicate, this.binding_itemClickHandler);
}, {
_domElement: null,
_itemArray: null,
_className: null,
autoRender: false,
enableClickAnimation: false,
className: {
get: function get() { return this._className; },
set: function set(value) { return this._className = value; }
},
_moduleInfo: null,
moduleInfo: {
get: function get() { return this._moduleInfo; },
set: function set(value) { return this._moduleInfo = value; }
},
itemArray: {
get: function get() { return this._itemArray; },
set: function set(value) {
this._itemArray = value;
if (this.autoRender) {
this.render();
}
}
},
_renderPromise: null,
_itemElements: null,
_itemClickPredicate: function _itemClickPredicate(element) {
return element && element.parentElement === this._domElement;
},
_itemClickHandler: null,
itemClickHandler: {
set: function set(value) {
this._itemClickHandler = value;
},
get: function get() {
return this._itemClickHandler;
}
},
_itemClickHandlerImpl: function _itemClickHandlerImpl(element) {
if (!element || !this.itemClickHandler) { return; }
var index = Array.prototype.indexOf.call(this._domElement.children, element);
this.itemClickHandler({ index: index, item: this._itemArray[index], domItem: element });
},
render: function render() {
if (!this._domElement) { return; }
if (!this.moduleInfo) { return; }
if (!this.itemArray) { return; }
if (this._renderPromise) {
var that = this;
return this._renderPromise.then(this.binding_renderImpl);
} else {
return this._renderImpl();
}
},
_attachElements: function _attachElements(elements) {
for (var i = 0; i < elements.length; i++) {
this._itemElements.push(elements[i]);
if (this.enableClickAnimation) { PlatformJS.Utilities.enablePointerUpDownAnimations(elements[i]); }
this._domElement.appendChild(elements[i]);
}
return elements;
},
_renderImpl: function _renderImpl() {
this._domElement.innerHTML = "";
this._itemElements = null;
// WinJS.Utilities.addClass throws error when className is null or undefined in Win8321, adding a check
if (this.className) {
WinJS.Utilities.addClass(this._domElement, this.className);
}
var numElems = (this._itemElements && this._itemElements.length) || 0;
var numItems = this._itemArray.length;
var deleteElems = Math.max(numElems - numItems, 0);
var addElems = Math.max(numItems - numElems, 0);
var rebindElems = Math.min(numElems, numItems);
var i = 0;
for (i = 0; i < deleteElems; i++) {
var elem = this._itemElements.pop();
elem.parentElement.removeChild(elem);
}
var addPromise = null, addPromises = [];
for (i = numElems; i < numElems + addElems; i++) {
var moduleInfo = {
templateId: this.moduleInfo.templateId,
fragmentPath: this.moduleInfo.fragmentPath
};
addPromises.push(CommonJS.loadModule(moduleInfo, this._itemArray[i], null));
}
this._itemElements = this._itemElements || [];
if (addPromises.length) {
addPromise = WinJS.Promise.join(addPromises).then(this.binding_attachElements);
}
var promises = [];
for (i = 0; i < rebindElems; i++) {
promises.push(WinJS.Binding.processAll(this._itemElements[i], this._itemArray[i]));
}
if (addPromise) {
promises.push(addPromise);
}
return this._renderPromise = WinJS.Promise.join(promises);
}
}, {
})
});
})();
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Controls",
{
/// <summary>
/// Cluster with Chart as view for stock quotes
/// </summary>
BaseOverviewCluster: WinJS.Class.derive(FinanceAppJS.Controls.ModuleClusterControl, function (domElement, options) {
FinanceAppJS.Controls.ModuleClusterControl.call(this, domElement, options);
// ctor
this.isCurrentInstrumentEOD = options.isEOD;
this.chartData = null;
this._chartDataCache = null;
this.chartUpdatePending = false;
this.currentRangeIndex = 0;
this.rangeSelectorConrol = null;
this.dateRangeControlInitialized = false;
this.displayName = null;
this.chartInitialized = false;
this._chartControl = null;
this.equityUpdateTime = null;
this._realtimeQuotesLastUpdatedTimeString = null;
options = options || {};
var originalRenderComplete = options.onrendercomplete;
var that = this;
options.onrendercomplete = function () {
if (originalRenderComplete) {
originalRenderComplete();
}
that.realtimeQuotesLastUpdatedTimeString = that.realtimeQuotesLastUpdatedTimeString; // to exec setter
};
// set options
WinJS.UI.setOptions(this, options);
this.type = options.instrumentType;
switch (options.instrumentType) {
case 'index':
case 'stock':
this._chartDataProvider = options.dataProviders.StockChartDataProvider;
break;
case 'etf':
this._chartDataProvider = options.dataProviders.ETFChartDataProvider;
break;
case 'fund':
this._chartDataProvider = options.dataProviders.FundChartDataProvider;
break;
default:
debugger;
break;
}
if (!this._chartDataProvider) {
debugger;
}
this._chartDataProvider.addListeners({ error: this._chartDataErrorHandler.bind(this) });
},
{
_chartDataProvider: null,
_chartDataErrorHandler: function _chartDataErrorHandler() {
this.updateChart(null);
},
chartControl: {
get: function get() {
if (!this._chartControl) {
this._chartControl = PlatformJS.Utilities.getControl('chartContainer');
}
return this._chartControl;
}
},
realtimeQuotesLastUpdatedTimeString: {
get: function get() {
return this._realtimeQuotesLastUpdatedTimeString;
},
set: function set(value) {
this._realtimeQuotesLastUpdatedTimeString = value;
if (this.chartControl) {
this.chartControl.realtimeQuotesLastUpdatedTimeString = value;
}
}
},
dateRangedChanged: function dateRangedChanged(item) {
if (item.index !== this.currentRangeIndex) {
var selectors = document.getElementsByClassName("rectangularButton");
for (var i = 0; i < selectors.length; i++) {
if (i === item.index) {
utils.setHighlight(selectors[i]);
}
else {
utils.removeHighlight(selectors[i]);
}
}
this.currentRangeIndex = item.index;
if (this.chartControl) {
this.chartControl.activeSeries = { seriesId: this._chartDataProvider.stockSymbol + this._rangeSelectors[this.currentRangeIndex], displayName: this.displayName };
this._chartDataProvider.updateOptions({ duration: this._rangeSelectors[this.currentRangeIndex], isEOD: this.isCurrentInstrumentEOD, refreshInterval: (this._rangeSelectors[this.currentRangeIndex] === '1d') ? FinanceAppJS.Constants.RefreshIntervals.HomePageEquityData : 0 });
var pageId = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(pageId,
FinanceAppJS.Telemetry.Constants.Cluster.CHART,
FinanceAppJS.Telemetry.Constants.Location.CHARTRANGESELECTOR,
this._rangeSelectors[this.currentRangeIndex]);
}
}
},
initializeChart: function initializeChart() {
if (!this.chartInitialized && this.chartControl) {
this.chartControl.activeSeries = { seriesId: this._chartDataProvider.stockSymbol + this._chartDataProvider.duration, displayName: this.displayName };
this.chartInitialized = true;
}
},
performPendingUpdate: function performPendingUpdate() {
this.initializeChart();
// check if recent chart data is available.
if (this.chartControl && this._chartDataCache) {
// recent chart data is available which was not used because it was not in sync with equity data.
this.chartControl.data = this._chartDataCache;
this._chartDataCache = null;
return WinJS.Promise.as({});
}
if (this.chartUpdatePending) {
return this.updateChart(this.chartData);
}
else {
return WinJS.Promise.as({});
}
},
updateChart: function updateChart(data) {
this._chartDataCache = null;
if (!FinanceAppJS.Globals.FRE) {
// compare the update time of chart data and equity data
if (!this.isEquityLocalResponse && this.equityUpdateTime && data && data.length > 0 && data[0] && data[0].lastUpdateTime) {
if ((new Date(data[0].lastUpdateTime)).getTime() - (new Date(this.equityUpdateTime).getTime()) > FinanceAppJS.Constants.Global.ChartAndEquitydata_Allowed_TimeDifference) {
// save the chart data for later use
this._chartDataCache = data;
// chart data is more fresh. Set it to null.
data = null;
}
}
}
if (this.dateRangeControlInitialized === false) {
this.updateDateSelectors();
}
this.initializeChart();
if (this.chartControl) {
this.chartControl.data = data;
this.chartUpdatePending = false;
this.chartData = null;
}
else {
// chart div is not rendered yet cache the data and set the pending flag
this.chartUpdatePending = true;
this.chartData = data;
}
return WinJS.Promise.as({});
},
updateDateSelectors: function updateDateSelectors() {
var rangeSelectors = [];
var className = '';
for (var i = 0; i < this._rangeSelectors.length; i++) {
className = "rectangularButton";
if (i === 0) {
className += " start";
}
if (i === this.currentRangeIndex) {
className += " active";
}
else if (i === this._rangeSelectors.length - 1) {
className += " end";
}
rangeSelectors.push({
range: utils.getDateRangeButtonName(this._rangeSelectors[i]),
selectorClass: className,
});
}
var rangeSelectorList = {
data: rangeSelectors,
moduleInfo: {
fragmentPath: "/pages/MarketToday/html/MarketTodayTemplates.html",
templateId: "chartRangeSelectorTemplate",
isInteractive: true,
}
};
this.rangeSelectorConrol = PlatformJS.Utilities.getControl('chartRangeSelectorContainer');
if (this.rangeSelectorConrol) {
this.dateRangeControlInitialized = true;
this.rangeSelectorConrol.moduleInfo = rangeSelectorList.moduleInfo;
this.rangeSelectorConrol.itemArray = rangeSelectorList.data;
this.rangeSelectorConrol.itemClickHandler = this.dateRangedChanged.bind(this);
}
},
dispose: function dispose() {
if (this.rangeSelectorConrol) {
this.rangeSelectorConrol.itemClickHandler = null;
}
if (this.chartControl) {
WinJS.UI.setOptions(this.chartControl, { refreshEnabled: false });
this.chartControl.dispose();
}
},
}, {
})
});
})();
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Controls", {
Slider: WinJS.Class.define(function (domElement, options) {
domElement.winControl = this;
this._domElement = domElement;
WinJS.Utilities.addClass(this._domElement, "SliderControl");
WinJS.UI.setOptions(this, options);
}, {
_domElement: null,
_values: null,
_isRendered: false,
values: {
set: function set(values) {
this._values = values;
if (values) {
this.render(values);
}
},
get: function get() {
return this._values;
}
},
render: function render(values) {
if (this._isRendered) {
// just update the values
this.minElement.textContent = values.min;
this.maxElement.textContent = values.max;
} else {
// create slider structure
this.sliderBar = utils.createElement(this._domElement, "div", { className: "SliderBar" });
this.sliderTip = utils.createElement(this.sliderBar, "div", { className: "SliderTip" });
this.minElement = utils.createElement(this._domElement, "div", { className: "MinElement", textContent: values.min });
this.maxElement = utils.createElement(this._domElement, "div", { className: "MaxElement", textContent: values.max });
this._isRendered = true;
}
if (values.max - values.min !== 0) {
if ((document.dir === "rtl")) {
this.sliderTip.style.marginRight = (-7 + 120 * (values.value - values.min) / (values.max - values.min)) + "px";
} else {
this.sliderTip.style.marginLeft = (-7 + 120 * (values.value - values.min) / (values.max - values.min)) + "px";
}
}
}
})
});
})();
(function () {
// This overrides featuredItem Click of the platform FeaturedCluster to  
// set additional click instrumentation required by Finance.
'use strict';
var NS = WinJS.Namespace.define("FinanceAppJS.Controls", {
FinanceFeaturedCluster: WinJS.Class.derive(CommonJS.FeaturedCluster, function (domElement, options) {
CommonJS.FeaturedCluster.call(this, domElement, options);
},
{
clickInstrumentationID: null,
featuredItemClick: function featuredItemClick(event) {
var item = event.detail.item.data;
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.Cluster.FEATUREDCLUSTER,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
item.id);
if (item.pageInfo && item.state) {
if (item.state.dynamicInfo) {
if (item.state.dynamicInfo.instrumentationId) {
PlatformJS.Services.instrumentation.incrementInt32Dictionary(
FinanceAppJS.Constants.Instrumentation.PanoInstrumentationDataSetId,
this.clickInstrumentationID,
item.state.dynamicInfo.instrumentationId);
}
}
}
CommonJS.FeaturedCluster.prototype.featuredItemClick.call(this, event);
}
})
});
})();
