﻿var PieChartHelper = function () {
'use strict';
var chartData;
var canvasId;
var ctx;
var canvasWidth;
var canvasHeight;
var radius;
var centerX;
var centerY;
var legendContainer;
/* Whenever these colors are updated, should be changed in appropriate CSS class also in chart.css file. */
var colors = ['#5697CB', '#C877BA', '#86AB41', '#E0931F', '#B0CADB', 'dimgray', 'yellow', 'pink', 'orange'];
this.LoadChart = function (containerDiv, data, landScape) {
try {
if (data) {
if (!ctx) {
var innerContainer = document.createElement("div");
var canvasElement = document.createElement("canvas");
canvasId = 'C' + Math.ceil((100000 * Math.random())).toString() + '_pieChart';
canvasElement.setAttribute("id", canvasId);
var maxRadiusX, maxRadiusY;
if (landScape) {
maxRadiusX = (containerDiv.clientWidth / 4) - 10;
maxRadiusY = (containerDiv.clientHeight / 2) - 10;
radius = Math.min(maxRadiusX, maxRadiusY);
centerX = radius + 10;
centerY = radius + 10;
canvasWidth = radius * 2 + 20;
canvasHeight = radius * 2 + 20;
canvasElement.setAttribute("width", canvasWidth);
canvasElement.setAttribute("height", canvasHeight);
WinJS.Utilities.addClass(innerContainer, 'pieChartInnerContainerLandscape');
WinJS.Utilities.addClass(canvasElement, 'pieChartCanvasLandscape');
legendContainer = document.createElement("div");
WinJS.Utilities.addClass(legendContainer, 'pieChartLegendContainerLandscape');
legendContainer.style.height = (containerDiv.clientHeight - 20) + "px";
}
else {
maxRadiusX = (containerDiv.clientWidth / 2) - 10;
maxRadiusY = (containerDiv.clientHeight / 4) - 10;
radius = Math.min(maxRadiusX, maxRadiusY);
centerX = radius + 10;
centerY = radius + 10;
canvasWidth = radius * 2 + 20;
canvasHeight = radius * 2 + 20;
canvasElement.setAttribute("width", canvasWidth);
canvasElement.setAttribute("height", canvasHeight);
WinJS.Utilities.addClass(innerContainer, 'pieChartInnerContainerPortrait');
WinJS.Utilities.addClass(canvasElement, 'pieChartCanvasPortrait');
legendContainer = document.createElement("div");
WinJS.Utilities.addClass(legendContainer, 'pieChartLegendContainerPortrait');
legendContainer.style.height = (canvasHeight - 20) + "px";
}
ctx = canvasElement.getContext("2d");
innerContainer.appendChild(canvasElement);
innerContainer.appendChild(legendContainer);
containerDiv.appendChild(innerContainer);
var start = 0, end;
var angle = 360;
var i = 0;
drawShadow(centerX, centerY, radius);
for (var asset in data) {
end = data[asset] / 100;
end = start - (angle * (Math.PI / 180) * end);
drawPie(centerX, centerY, radius, start, end, colors[i]);
start = end;
var legend = document.createElement("div");
WinJS.Utilities.addClass(legend, 'pieChartLegend');
legend.style.msGridRow = i + 1;
var assetColorRect = document.createElement("div");
WinJS.Utilities.addClass(assetColorRect, 'pieChartRect');
var legendColorClass = "pieLegendColor" + i;
WinJS.Utilities.addClass(assetColorRect, legendColorClass);
assetColorRect.style.backgroundColor = colors[i];
var assetName = document.createElement("span");
WinJS.Utilities.addClass(assetName, 'fontMB');
assetName.textContent = utils.getResourceString(asset) + ": ";
var assetValue = document.createElement("span");
assetValue.setAttribute("dir", "ltr");
WinJS.Utilities.addClass(assetValue, 'fontMR');
//loc-glob issue
//TODO: Shouldn't hard-code number of digits after decimal (should be formatted using globalization rules)
assetValue.textContent = Number(data[asset]).toFixed(2) + "%";
legend.appendChild(assetColorRect);
legend.appendChild(assetName);
legend.appendChild(assetValue);
legendContainer.appendChild(legend);
i++;
}
}
return ctx;
}
}
catch (err) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(err.message, err.stack);
}
};
function drawShadow(cx, cy, r) {
ctx.beginPath();
ctx.arc(cx, cy, r - 1, 0, Math.PI * 2, true);
ctx.closePath();
ctx.fillStyle = '#ffffff';
ctx.shadowColor = "#bbbbbb";
ctx.shadowBlur = 10;
ctx.shadowOffsetX = 3;
ctx.shadowOffsetY = 3;
ctx.fill();
ctx.shadowColor = "transparent";
ctx.shadowBlur = 0;
ctx.shadowOffsetX = 0;
ctx.shadowOffsetY = 0;
}
function drawPie(cx, cy, r, preAngle, postAngle, color) {
ctx.beginPath();
ctx.moveTo(cx, cy);
ctx.arc(cx, cy, r, preAngle, postAngle, true);
ctx.lineTo(cx, cy);
ctx.closePath();
ctx.fillStyle = color;
ctx.fill();
}
function clearCanvas() {
ctx.clearRect(0, 0, canvasWidth, canvasHeight);
}
};
var BarChartHelper = function () {
'use strict';
var chartOptions;
var chartData;
var itemWidth;
var canvasId;
var ctx;
var canvasWidth;
var canvasHeight;
var chartWidth;
var chartHeight;
var interSeriesGap = 3;
var projX = 0;
var projY = 0;
var maxY = 0.0;
var minY = 0.0;
var seriesCount;
var seriesLength;
var chunkItemWidth;
var scaleY = 0.0;
var extraY = 5; //find its use
var JsLessVars = utils.JsLessVariables(true);
var isLtr = (document.dir === "ltr");
this.LoadChart = function (containerDiv, data, barChartOptions) {
chartOptions = barChartOptions;
if (chartOptions.is3d) {
projX = 8;
projY = 8;
}
if (chartOptions.populateColorsArray) {
try {
var i = 0;
var colors = data.data.map(function (item) {
return item.color ? item.color : chartOptions.colors[i++];
});
chartOptions.colors = colors;
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
}
}
if (!ctx || chartOptions.forceDraw) {
if (!isLtr) {
chartOptions.paddingRight = chartOptions.paddingLeft;
chartOptions.paddingLeft = 0;
}
containerDiv.innerHTML = "";
var canvasElement = document.createElement("canvas");
containerDiv.appendChild(canvasElement);
canvasId = 'B' + Math.ceil((100000 * Math.random())).toString() + '_barchart';
canvasElement.setAttribute("id", canvasId);
canvasElement.setAttribute("dir", "ltr");
canvasWidth = containerDiv.clientWidth;
canvasHeight = containerDiv.clientHeight;
canvasElement.setAttribute("width", containerDiv.clientWidth);
canvasElement.setAttribute("height", containerDiv.clientHeight - chartOptions.legendContainerHeight);
chartWidth = canvasWidth - chartOptions.paddingLeft - chartOptions.paddingRight - projX;
chartHeight = canvasHeight - (2 * chartOptions.paddingY) - projY;
ctx = canvasElement.getContext("2d");
if (data.xaxis.name) {
addXAxisName(data.xaxis.name, containerDiv);
}
addLegends(data, containerDiv);
draw(data, true);
}
if (ctx && chartOptions.forceDraw) {
draw(data, true);
}
};
function addXAxisName(name, containerDiv) {
var element = document.createElement("div");
element.textContent = name;
element.id = containerDiv.id + "_" + name;
element.setAttribute("aria-describedby", name);
WinJS.Utilities.addClass(element, 'XAxisName');
containerDiv.appendChild(element);
}
function addLegends(data, containerDiv) {
var legendContainer = document.createElement("div");
WinJS.Utilities.addClass(legendContainer, 'barChartLegendContainer');
if (!chartOptions.legendParent) {
legendContainer.style.width = (containerDiv.clientWidth - chartOptions.paddingLeft - chartOptions.paddingRight) + "px";
legendContainer.style.height = chartOptions.legendContainerHeight + "px";
containerDiv.appendChild(legendContainer);
}
else {
chartOptions.legendParent.appendChild(legendContainer);
}
for (var i = 0; i < data.data.length; i++) {
var legend = document.createElement("div");
WinJS.Utilities.addClass(legend, 'barChartLegend');
var legendColorRect = document.createElement("div");
legendColorRect.style.backgroundColor = data.data[i].color;
WinJS.Utilities.addClass(legendColorRect, 'barChartRect ');
var legendColor = "barChartRect" + i;
WinJS.Utilities.addClass(legendColorRect, legendColor);
if (data.data[i].investment) {
if (data.data[i].investment === "gain") {
WinJS.Utilities.addClass(legendColorRect, "barChartRectGain");
}
if (data.data[i].investment === "lose") {
WinJS.Utilities.addClass(legendColorRect, "barChartRectLose");
}
}
var legendName = document.createElement("span");
legendName.textContent = data.data[i].label;
var legendAutomationId = legendName.textContent.toString().replace(/\s/g, "");
legendName.id = containerDiv.id + "_" + legendAutomationId;
legendName.setAttribute("aria-describedby", legendAutomationId);
legend.appendChild(legendColorRect);
legend.appendChild(legendName);
legendContainer.appendChild(legend);
}
}
function clearCanvas() {
ctx.clearRect(0, 0, canvasWidth, canvasHeight);
}
function getItemWidth() {
interSeriesGap = 0.5;
if (chartOptions.isSplitBar) {
chunkItemWidth = chartWidth / seriesLength ;
itemWidth = chartWidth / (2 * seriesLength );
}
else {
chunkItemWidth = chartWidth / seriesLength;
itemWidth = (chunkItemWidth - (interSeriesGap * seriesCount)) / (seriesCount + 1);
}
return itemWidth;
}
function getMaxY() {
var maxY = 0.0;
if (chartOptions.isSplitBar) {
for (var i = 0; i < seriesLength; i++) {
var sum = 0;
for (var j = 0; j < seriesCount; j++) {
sum = sum + chartData.data[j].points[i].y; //check if parseFloat is necessary here
}
if (maxY < sum) {
maxY = sum;
}
}
}
else {
for (var i = 0; i < seriesLength; i++) {
for (var j = 0; j < seriesCount; j++) {
if (maxY < parseFloat(chartData.data[j].points[i].y)) {
maxY = parseFloat(chartData.data[j].points[i].y);
}
}
}
}        
maxY = roundNumber(maxY); //find a better way to do this
return maxY;
}
function getMinY() {
var minY = 0.0;
if (!chartOptions.isSplitBar) {
for (var i = 0; i < seriesLength; i++) {
for (var j = 0; j < seriesCount; j++) {
if (minY > parseFloat(chartData.data[j].points[i].y)) {
minY = parseFloat(chartData.data[j].points[i].y);
}
}
}
if (minY >= 0) {
minY = 0;
}
else {
minY = -1 * roundNumber(Math.abs(minY));
}
}        
return minY;
}
function draw(data, is3d) {
try {
chartData = data;
clearCanvas();
ctx.beginPath();
ctx.rect(chartOptions.paddingLeft + projX, chartOptions.paddingY, chartWidth + extraY, chartHeight);
ctx.lineWidth = 1;
ctx.fillStyle = chartOptions.backgroundColor;
ctx.fill();
var topY = canvasHeight - (1 + chartOptions.paddingY) + minY * scaleY; //todo cleanup
plotBar(0, chartWidth + extraY, 1, topY,"silver", false, chartOptions.paddingLeft, chartOptions.paddingY);
seriesCount = chartData.data.length;
seriesLength = chartData.data[0].points.length;
itemWidth = getItemWidth();
maxY = getMaxY();
minY = getMinY();
scaleY = chartHeight / (maxY - minY);
var yAxisLabelCount = 5; //temporarily hardcoded. Needs to be passed as an option.
drawLines(scaleY,yAxisLabelCount);
drawLines(scaleY);
if (chartOptions.animationTimer) {
clearInterval(chartOptions.animationTimer);
}
if (chartOptions.playAnimation) {
chartOptions.animationScale = 0.0;
chartOptions.animationTimer = setInterval(function () { drawBars(); }, 50);
} else {
chartOptions.animationScale = 50.0;
drawBars();
}
//x-axis labels - Refactor as separate function
var x = 0;
for (var i = 0; i < chartData.xaxis.labels.length; i++) {
ctx.beginPath();
var textWd = ctx.measureText(chartData.xaxis.labels[i].label);
ctx.font = "10pt"; //chartOption
ctx.beginPath();
ctx.fillStyle = chartOptions.horizontalLineColor;
ctx.fillText(chartData.xaxis.labels[i].label, (x + chartOptions.paddingLeft + (chunkItemWidth / 2) - (textWd.width / 2)), (chartHeight + chartOptions.paddingY + extraY + projY + 10));
ctx.fill();
x = x + chunkItemWidth;
}
ctx.beginPath();
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
}
}
function roundNumber(n) {
var tmp = n;
var tens = 0;
for (var tens = 0; tens < 10; tens++) {
if (tmp < 10) {
break;
}
tmp = Math.ceil(tmp / 10);
}
return tmp * Math.pow(10, tens);
}
function drawBars() {
try {
var x = itemWidth / 2;
var coveredHeight;
if (chartOptions.isSplitBar) {
for (var i = 0; i < seriesLength; i++) {
coveredHeight = 0;
for (var j = 0; j < seriesCount; j++) {
var height = chartData.data[j].points[i].y * scaleY * chartOptions.animationScale / 50;
var topY = canvasHeight - (height + coveredHeight + chartOptions.paddingY) + minY * scaleY;
coveredHeight = coveredHeight + height;
plotBar(x, itemWidth, height, topY, chartOptions.colors[j], false, chartOptions.paddingLeft, chartOptions.paddingY);
}
x = x + 2 * itemWidth;
}
}
else {
for (var i = 0; i < seriesLength; i++) {
for (var j = 0; j < seriesCount; j++) {
var height = chartData.data[j].points[i].y * scaleY * chartOptions.animationScale / 50;
var topY = canvasHeight - (height + chartOptions.paddingY) + minY * scaleY;
plotBar(x, itemWidth, height, topY, chartOptions.colors[j], false, chartOptions.paddingLeft, chartOptions.paddingY);
x = x + interSeriesGap + itemWidth;
}
x = x + itemWidth;
}
}
ctx.beginPath();
if (chartOptions.animationScale >= 50) {
clearInterval(chartOptions.animationTimer);
chartOptions.animationScale = 0;
drawLineGraph();
}
else {
chartOptions.animationScale = chartOptions.animationScale + 2;
}
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
clearInterval(chartOptions.animationTimer);
}
}
function drawLineGraph() {
//todo: code for drawing line graph for profit margin comes here
}
function drawLines(scaleY) {
var ht = chartHeight;
var yOffset = (ht / (chartOptions.yAxisLabelCount - 1)); 
for (var i = 0; i < chartOptions.yAxisLabelCount; i++) {
ctx.beginPath();
ctx.moveTo(chartOptions.paddingLeft + projX, (yOffset * i) + chartOptions.paddingY);
ctx.lineTo(chartOptions.paddingLeft + projX + chartWidth + extraY, (yOffset * i) + chartOptions.paddingY);
ctx.strokeStyle = chartOptions.horizontalLineColor;
ctx.lineWidth = 0.2;
ctx.stroke();
ctx.beginPath();
ctx.fillStyle = chartOptions.horizontalLineColor;
var yLabel = Math.round(maxY - (yOffset * i) / scaleY); //Y axis Label to be in diff function
yLabel = formatNumber(yLabel, 1);
var tag = yLabel;
var textWd = ctx.measureText(tag);
if (isLtr) {
ctx.fillText(tag, chartOptions.paddingLeft + projX - textWd.width - 5, (yOffset * i) + chartOptions.paddingY + 4);
}
else {
ctx.fillText(tag, chartWidth + chartOptions.paddingLeft + projX + 5, (yOffset * i) + chartOptions.paddingY + 4);
}
ctx.fill();
}
if (minY < 0) {
ctx.beginPath();
var y = (maxY * scaleY) + chartOptions.paddingY;
ctx.moveTo(chartOptions.paddingLeft + projX, y);
ctx.lineTo(chartOptions.paddingLeft + projX + chartWidth + extraY, y);
ctx.strokeStyle = 'skyblue';
ctx.lineWidth = 0.5;
ctx.stroke();
}
}
function formatNumber(n, dec) {
var text = '';
if (n < 0) {
text = '-';
n = -n;
}
if (n < 1000) {
//loc-glob issue
//TODO: Shouldn't hard-code number of digits after decimal (should be formatted using globalization rules)
text = text + n.toFixed(dec);
}
else if (n < 1000000) {
n = n / 1000;
//loc-glob issue
//TODO: Shouldn't hard-code number of digits after decimal (should be formatted using globalization rules)
text = text + n.toFixed(dec) + utils.getResourceString("K");
}
else if (n < 1000000000) {
n = n / 1000000;
//loc-glob issue
//TODO: Shouldn't hard-code number of digits after decimal (should be formatted using globalization rules)
text = text + n.toFixed(dec) + utils.getResourceString("M");
}
else if (n < 1000000000000) {
n = n / 1000000000;
//loc-glob issue
//TODO: Shouldn't hard-code number of digits after decimal (should be formatted using globalization rules)
text = text + n.toFixed(dec) + utils.getResourceString("B");
}
else {
n = n / 1000000000000;
//loc-glob issue
//TODO: Shouldn't hard-code number of digits after decimal (should be formatted using globalization rules)
text = text + n.toFixed(dec) + utils.getResourceString("T");
}
return text;
}
function plotBar(x, wd, ht,topY, color, isfaceGradient, padingX, padingY) {
ctx.beginPath();
var topX = x + padingX;
//var topY = canvasHeight - (ht + padingY) + minY * scaleY;
var width = wd;
var height = ht;
var gradientClr = "#000000";
var sidegradient = ctx.createLinearGradient(topX, topY, topX, topY + 4.5 * ht);
sidegradient.addColorStop(0, color);
if (isfaceGradient) {
sidegradient.addColorStop(1, gradientClr);
}
else {
sidegradient.addColorStop(1, color);
}
var topgradient = ctx.createLinearGradient(topX - 2 * projX, topY + 6 * projY, topX - 2 * projX + (0.25 * projX), topY - 1 * projY);
topgradient.addColorStop(0, gradientClr);
topgradient.addColorStop(1, color);
var frontgradient = ctx.createLinearGradient(topX, topY - projY, topX + projX, topY + 1.1 * ht);
frontgradient.addColorStop(0, color);
frontgradient.addColorStop(1, gradientClr);
//side
//Nullify Previous color
ctx.beginPath();
ctx.rect(topX, topY, width, height);
ctx.lineWidth = 1;
ctx.fillStyle = "#FFFFFF";
ctx.fill();
ctx.beginPath();
ctx.rect(topX, topY, width, height);
ctx.lineWidth = 1;
ctx.fillStyle = color;
ctx.fill();
//top
ctx.beginPath();
ctx.moveTo(topX, topY);
ctx.lineTo(topX + projX, topY - projY);
ctx.lineTo(topX + projX + width, topY - projY);
ctx.lineTo(topX + width, topY);
ctx.lineWidth = 1;
ctx.fillStyle = topgradient;
ctx.fill();
//front
ctx.beginPath();
ctx.moveTo(topX + projX + width, topY - projY);
ctx.lineTo(topX + projX + width, topY + height - projY);
ctx.lineTo(topX + width, topY + height);
ctx.lineTo(topX + width, topY);
ctx.lineWidth = 1;
ctx.fillStyle = frontgradient;
ctx.fill();
}
};
