﻿(function () {
"use strict";
var partner = "WSJ";
var dateFormatting = Windows.Globalization.DateTimeFormatting;
WinJS.Namespace.define("WSJJS", {
MarketData: WinJS.Class.define(function (element, options) {
this.element = element || document.createElement("div");
this.element.winControl = this;
WinJS.Utilities.addClass(this.element, "marketData");
WinJS.UI.setOptions(this, options);
this._quoteUrl = NewsJS.Partners.Config.getConfig(partner, "QuotesUrl", "");
this._refreshInterval = NewsJS.Partners.Config.getConfig(partner, "QuotesRefreshInterval", 60);
this._token = NewsJS.Partners.Config.getConfig(partner, "QuotesToken", "");
var symbols = NewsJS.Partners.Config.getConfig(partner, "QuotesSymbols", []);
this.symbols = [];
for (var i = 0; i < symbols.size; i++) {
this.symbols.push(symbols[i].value);
}
this._timer = null;
this.container = document.createElement("div");
WinJS.Utilities.addClass(this.container, "container");
this.element.appendChild(this.container);
var getQuote = document.createElement("button");
getQuote.textContent = PlatformJS.Services.resourceLoader.getString("/wsj/GetQuote");
WinJS.Utilities.addClass(getQuote, "quoteButton");
var that = this;
getQuote.onclick = function () {
window.open(that._quoteUrl);
};
this.element.appendChild(getQuote);
this.fetch();
},
{
symbols: [],
update: function update(dataString) {
var that = this;
this.container.innerHTML = "";
var xml = new Windows.Data.Xml.Dom.XmlDocument();
xml.loadXml(dataString);
var nodes = xml.selectNodes("//InstrumentMatch");
for (var i = 0; i < nodes.size; i++) {
var node = nodes.getAt(i);
var ticker = node.selectSingleNode(".//Ticker");
var price = node.selectSingleNode(".//Last/Price/Value");
if (ticker && price) {
var tickerSpan = document.createElement("span");
var priceSpan = document.createElement("span");
WinJS.Utilities.addClass(tickerSpan, "ticker");
WinJS.Utilities.addClass(priceSpan, "price");
tickerSpan.innerText = ticker.innerText;
priceSpan.innerText = price.innerText;
this.container.appendChild(tickerSpan);
this.container.appendChild(priceSpan);
}
}
},
_timer: null,
resetTimer: function resetTimer() {
if (this._timer) {
clearTimeout(this._timer);
this._timer = null;
}
},
scheduleNextRefresh: function scheduleNextRefresh() {
var that = this;
this._timer = setTimeout(function () {
that.resetTimer();
that.fetch();
}, this._refreshInterval);
},
dispose: function dispose() {
this.resetTimer();
},
fetch: function fetch() {
//MSFT|us||Stock,DELL|us||Stock
var queryService = new PlatformJS.DataService.QueryService("WSJ.Quotes"),
params = PlatformJS.Collections.createStringDictionary(),
options = new Platform.DataServices.QueryServiceOptions(),
symbols = this.symbols.join(",");
options.requestHeaders.insert("Dylan2010.EntitlementToken", this._token);
options.bypassCache = false;
params.insert("symbols", symbols);
params.insert("key", this._token.substring(0, 10));
var quotes = null,
that = this;
return queryService.downloadDataAsync(params, null, null, options).then(function (response) {
if (!quotes || !response.isCachedResponse) {
quotes = response.dataString;
that.update(quotes);
}
that.scheduleNextRefresh();
}, function (error) {
that.scheduleNextRefresh();
}, function (progress) {
if (progress.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryFound &&
progress.cachedResponse) {
quotes = progress.cachedResponse.dataString;
that.update(quotes);
}
});
}
}),
Formatter: WinJS.Class.define(function () {
this._dateFormat = new dateFormatting.DateTimeFormatter(dateFormatting.YearFormat.full, dateFormatting.MonthFormat.abbreviated, dateFormatting.DayFormat.default, dateFormatting.DayOfWeekFormat.none);
this._timeFormat = new dateFormatting.DateTimeFormatter(dateFormatting.HourFormat.default, dateFormatting.MinuteFormat.default, dateFormatting.SecondFormat.none);
},
{
_dateFormat: null,
_timeFormat: null,
getDateString: function getDateString(timestamp) {
try {
return this._dateFormat.format(new Date(timestamp));
} catch (e) {
return "";
}
},
getTimeString: function getTimeString(timestamp) {
try {
return this._timeFormat.format(new Date(timestamp));
} catch (e) {
return "";
}
},
parseTime: function parseTime(dateString) {
var timestamp = Date.parse(dateString) / 1000,
year = null,
month = null,
day = null,
hour = null,
minute = null;
if (isNaN(timestamp) && (typeof dateString === "string")) {
var dateTime = dateString.split(" ");
if (dateTime[0]) {
var date = dateTime[0].split("-");
if (date.length >= 3) {
year = parseInt(date[0]);
month = parseInt(date[1]);
day = parseInt(date[2]);
}
}
if (dateTime[1]) {
var time = dateTime[1].split(":");
if (time.length >= 2) {
hour = parseInt(time[0]);
minute = parseInt(time[1]);
}
}
if (year !== null &&
month !== null &&
day !== null &&
hour !== null &&
minute !== null) {
var newDateString = [month, "/", day, "/", year, " ", hour, ":", minute, " EDT"].join("");
var dateObj = new Date(newDateString);
timestamp = dateObj.getTime() / 1000;
}
}
return timestamp;
},
formatTime: function formatTime(dateString, formatType) {
var now = Date.now(),
timestamp = this.parseTime(dateString);
if (isNaN(timestamp)) {
return "";
}
var delta = now / 1000 - timestamp,
time = new Date(timestamp * 1000);
if (formatType === WSJJS.Formatter.panoWatermark) {
return this._dateFormat.format(time) + " " + this._timeFormat.format(time);
} else if (formatType === WSJJS.Formatter.articleFormat) {
var formattedDate = this._dateFormat.format(time);
if (this._dateFormat.format(new Date()) === formattedDate) {
return this._timeFormat.format(time);
} else if (delta < 6 * 60 * 60) {
return this._timeFormat.format(time);
} else {
return formattedDate;
}
} else {
return this._dateFormat.format(time) + " " + this._timeFormat.format(time);
}
},
},
{
other: 0,
tileFormat: 1,
panoWatermark: 2,
articleFormat: 3,
_instance: null,
instance: {
get: function get() {
if (!WSJJS.Formatter._instance) {
WSJJS.Formatter._instance = new WSJJS.Formatter();
}
return WSJJS.Formatter._instance;
}
}
}),
AuthControl: WinJS.Class.derive(NewsJS.Partners.BaseNativeAuthControl, function wsj_auth_ctrl_ctor(options) {
this._partner = partner;
NewsJS.Partners.BaseNativeAuthControl.call(this, options);
var subscribeInstructions = document.createElement("div");
subscribeInstructions.innerText = PlatformJS.Services.resourceLoader.getString("/wsj/SubscribeNow");
this.customContainer.appendChild(subscribeInstructions);
WinJS.Utilities.addClass(subscribeInstructions, "subscribeInstructions");
var subscribeInfo = document.createElement("div");
subscribeInfo.innerHTML = PlatformJS.Services.resourceLoader.getString("/wsj/subscribeInfo");
this.customContainer.appendChild(subscribeInfo);
WinJS.Utilities.addClass(subscribeInfo, "subscribeInfo");
this._subscribeButton = document.createElement("button");
this._subscribeButton.tabIndex = 11;
this._subscribeButton.innerText = "Subscribe";
WinJS.Utilities.addClass(this._subscribeButton, "subscribeButton");
this.customContainer.appendChild(this._subscribeButton);
this._subscribeButton.onclick = this._onSubscribeClick.bind(this);
},
{
_onCreateAccountClick: function _onCreateAccountClick() {
var url = NewsJS.Partners.Config.getConfig(partner, "CreateAccountUrl");
window.open(url);
},
_onForgotPasswordClick: function _onForgotPasswordClick() {
var url = NewsJS.Partners.Config.getConfig(partner, "ForgotPasswordUrl");
window.open(url);
},
_onPrivacyPolicyClick: function _onPrivacyPolicyClick() {
var url = NewsJS.Partners.Config.getConfig(partner, "PrivacyStatementLink");
window.open(url);
},
_onTOUClick: function _onTOUClick() {
var url = NewsJS.Partners.Config.getConfig(partner, "TermsOfUseLink");
window.open(url);
},
validateAsync: function validateAsync(userName, password) {
if (!userName || !password) {
return WinJS.Promise.wrapError(PlatformJS.Services.resourceLoader.getString("/wsj/credentialsInvalid"));
}
//return WSJ.Auth.instance.loginAsync(encodeURIComponent(userName), encodeURIComponent(password));
var url = NewsJS.Partners.Config.getConfig(partner, "AuthUrl", "https://iwap.wsj.com/iphone/role?username={0}&password={1}");
url = url.format(encodeURIComponent(userName), encodeURIComponent(password));
return WinJS.xhr({
url: url,
type: "GET"
}).then(function (response) {
if (response.status === 200) {
var xml = response.responseXML;
if (xml) {
var description = xml.querySelector("description");
if (description && description.textContent === "Precondition Failure") {
var error = xml.querySelector("value");
if (error) {
WSJJS.Instrumentation.instance.instrumentError("Login failed");
return WinJS.Promise.wrapError(error.textContent);
} else {
WSJJS.Instrumentation.instance.instrumentError("Login failed");
return WinJS.Promise.wrapError(PlatformJS.Services.resourceLoader.getString("/wsj/credentialsInvalid"));
}
}                            
}
WSJJS.Instrumentation.instance.instrumentSignIn();
return WinJS.Promise.wrap(xml);
} else {
WSJJS.Instrumentation.instance.instrumentError("Login failed");
return WinJS.Promise.wrapError(PlatformJS.Services.resourceLoader.getString("/wsj/credentialsInvalid"));
}
});
},
_onSubscribeClick: function _onSubscribeClick() {
WSJJS.Auth.instance.showSubscribePromptAsync().then(function () { }, function () { });
},
_replaceText: function _replaceText(className, newText) {
newText = PlatformJS.Services.resourceLoader.getString("/wsj/" + newText);
if (className && this.container) {
var elements = WinJS.Utilities.query(className, this.container);
if (elements && elements[0] && elements[0].tagName) {
switch (elements[0].tagName.toUpperCase()) {
case "DIV":
case "BUTTON":
elements[0].innerText = newText;
break;
case "INPUT":
elements[0].placeholder = newText;
break;
}
}
}
},
}),
Auth: WinJS.Class.derive(NewsJS.Partners.BaseAuth, function wsj_auth_ctor() {
this._partner = partner;
this._lastGatewayShowing = null;
this._entitlements = null;
this.authUrlAppId = NewsJS.Partners.Config.getConfig(partner, "AuthUrlAppId", "microsoft.wsj");
NewsJS.Partners.BaseAuth.call(this);
},
{
_serializeEntitlements: function _serializeEntitlements() {
if (this._entitlements) {
var serializer = new XMLSerializer();
var serialized = serializer.serializeToString(this._entitlements);
if (this._serializedEntitlements !== serialized) {
NewsJS.Partners.Config.setConfig(partner, "Entitlements", serialized);
}
} else {
NewsJS.Partners.Config.setConfig(partner, "Entitlements", null);
}
},
_deserializeEntitlements: function _deserializeEntitlements() {
var serialized = NewsJS.Partners.Config.getConfig(partner, "Entitlements", "");
this._serializedEntitlements = serialized;
if (serialized) {
return this._parseEntitlements(serialized);
} else {
return null;
}
},
showSignInPromptAsync: function showSignInPromptAsync(promptText) {
NewsJS.Partners.BaseAuth.prototype.showSignInPromptAsync.call(this);                
var that = this;
promptText = promptText || PlatformJS.Services.resourceLoader.getString("/wsj/loginPrompt");
CommonJS.Partners.Auth.BaseAuthControl.promptControl = this.promptControl = new WSJJS.AuthControl({
prompt: promptText
});
return this.promptControl.showAsync().then(function (xml) {
if (xml) {
var uuid = xml.querySelector("uuid");
if (uuid) {
that.authToken = uuid.textContent;
that.entitlements = xml;
that._scheduleNextEntitlementsRefresh(new Date());
return WinJS.Promise.wrap(that._checkEntitlement(xml));
}
}
return WinJS.Promise.wrapError("AuthError");
});
},
showSubscribePromptAsync: function showSubscribePromptAsync() {
WSJJS.Instrumentation.instance.instrumentSubscriptionClicked();
var url = NewsJS.Partners.Config.getConfig(partner, "SubscribeUrl");
window.open(url);
return WinJS.Promise.wrapError(false);
},
showToast: function showToast() {
var that = this,
message = PlatformJS.Services.resourceLoader.getString("/wsj/InvalidSubscriptionMessage"),
buttonLabel = PlatformJS.Services.resourceLoader.getString("/wsj/SubscribeButton"),
messageBar = new CommonJS.MessageBar(message);
messageBar.addButton(buttonLabel, function () {
messageBar.hide();
that.showSubscribePromptAsync().then(function () { }, function () { });
});
messageBar.addButton(PlatformJS.Services.resourceLoader.getString("/partners/Dismiss"), function () {
messageBar.hide();
});
messageBar.show();
},
_checkEntitlement: function _checkEntitlement(xml) {
if (xml) {
var roles = xml.querySelectorAll("role");
if (roles.length) {
for (var i = 0; i < roles.length; i++) {
if (roles[i].textContent === "WSJ-IPAD") {
this.isEntitled = true;
return this.isEntitled;
}
}
this.isEntitled = false;
} else {
this.isEntitled = false;
}
} else {
this.isEntitled = false;
}
return this.isEntitled;
},
_parseEntitlements: function _parseEntitlements(dataString) {
var parser = new DOMParser();
return parser.parseFromString(dataString, "text/xml");
},
_getEntitlementsAsync: function _getEntitlementsAsync(authToken, bypassCache) {
var that = this;
// Otherwise, ask the entitlements server
var queryService = new PlatformJS.DataService.QueryService("WSJ.Entitlements"),
params = PlatformJS.Collections.createStringDictionary(),
options = new Platform.DataServices.QueryServiceOptions();
options.bypassCache = bypassCache;
params.insert("token", encodeURIComponent(authToken));
var entitlements = null,
lastUpdateTime = null,
isStaleResponse = null;
return queryService.downloadDataAsync(params, null, null, options).then(function (response) {
var promise = null;
try {
entitlements = that._parseEntitlements(response.dataString);
lastUpdateTime = response.lastUpdateTime;
isStaleResponse = response.isStaleResponse;
} catch (e) { }
that._onlineEntitlementsChecked = true;
if (entitlements && !isStaleResponse) {
that.entitlements = entitlements;
if (!response.isCachedResponse) {
that._scheduleNextEntitlementsRefresh(lastUpdateTime);
}
} else {
that.entitlements = null;
promise = queryService.deleteCacheEntryAsync(params, null);
}
if (!promise) {
promise = WinJS.Promise.wrap();
}
return promise.then(function () {
return WinJS.Promise.wrap(that._checkEntitlement(that.entitlements));
}, function () {
return WinJS.Promise.wrap(that._checkEntitlement(that.entitlements));
});
}, function (error) {
that.entitlements = null;
return WinJS.Promise.wrap(that._checkEntitlement(that.entitlements));
});
},
getAuthTokenAsync: function getAuthTokenAsync() {
return WinJS.Promise.wrap(this.authToken);
},
getEntitlementsAsync: function getEntitlementsAsync(showPrompt, bypassCache, isBlockingPrompt) {
var that = this;
var isOnline = PlatformJS.Utilities.hasInternetConnection();
return this.getAuthTokenAsync().then(function (authToken) {
if (authToken) {
var promise = null;
// See if the in-memory version of the entitlements is still good
if (that.entitlements &&
((Date.now() < that.entitlementsCheckExpireTime || !that._onlineEntitlementsChecked) && !bypassCache ||
!isOnline)) {
promise = WinJS.Promise.wrap(that._checkEntitlement(that.entitlements));
}
if (!promise) {
promise = that._getEntitlementsAsync(authToken, bypassCache);
}
return promise.then(function (isEntitled) {
if (!isEntitled) {
return that.handleEntitlementsErrorAsync(showPrompt, isBlockingPrompt);
} else {
return WinJS.Promise.wrap(isEntitled);
}
});
} else {
return that.handleEntitlementsErrorAsync(showPrompt, isBlockingPrompt);
}
}, function () {
return that.handleEntitlementsErrorAsync(showPrompt, isBlockingPrompt);
});
},
handleEntitlementsErrorAsync: function handleEntitlementsErrorAsync(showPrompt, isBlockingPrompt) {
var that = this;
if (showPrompt) {
if (this.isSignedIn && !isBlockingPrompt) {
this.showToast();
return WinJS.Promise.wrap(false);
} else {
return this.showSignInPromptAsync(PlatformJS.Services.resourceLoader.getString("/wsj/loginPromptContentLocked")).then(function (isEntitled) {
if (!isEntitled && showPrompt) {
that.showToast();
}
return WinJS.Promise.wrap(isEntitled);
});
}
} else {
return WinJS.Promise.wrap(false);
}
}
},
{
_instance: null,
instance: {
get: function get() {
if (!WSJJS.Auth._instance) {
WSJJS.Auth._instance = new WSJJS.Auth();
}
return WSJJS.Auth._instance;
}
},
}),
ProcessListener: WinJS.Class.derive(NewsJS.Partners.BaseProcessListener, function wsj_processListener_ctor() {
/// <summary>
/// Base process listener for WSJ. To be included in the integrated app.
/// To be derived from in the standalone
/// </summary>
this._partner = partner;
NewsJS.Partners.BaseProcessListener.call(this);
this._pageMapping["WSJJS.CategoryPano"] = "/wsjshared/wsjCategoryPano.html";
},
{
startPostActivateWork: function startPostActivateWork() {
this._articleManager = WSJJS.ArticleManager.instance;
NewsJS.Partners.BaseProcessListener.prototype.startPostActivateWork.call(this);
},
onActivated: function onActivated(event, isRecoveryMode) {
if (NewsJS.Partners.Config.isPartnerApp) {
WSJJS.Instrumentation.instance.openSession();
if (event.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.protocol) {
var launchUrl = NewsJS.Utilities.parseUri(event.detail.uri);
if (launchUrl && launchUrl.params && launchUrl.params.referrer) {
WSJJS.Instrumentation.instance.instrumentAppLaunchFromProtocol(launchUrl.params.referrer);
} else {
WSJJS.Instrumentation.instance.instrumentAppLaunchFromProtocol(null);
}
}
}
if (isRecoveryMode) {
this._articleManager = WSJJS.ArticleManager.instance;
}
return NewsJS.Partners.BaseProcessListener.prototype.onActivated.call(this, event, isRecoveryMode);
},
onResuming: function onResuming(event) {
/// <summary>
/// Handles Resume event
/// </summary>
/// <param name="event">
/// resume event
/// </param>
/// <returns>
/// Nothing
/// </returns>
NewsJS.Partners.BaseProcessListener.prototype.onResuming.call(this, event);
this.populateEdgy();
},
onVisibilityChange: function onVisibilityChange(msHidden) {
NewsJS.Partners.BaseProcessListener.prototype.onVisibilityChange.call(this, msHidden);
if (NewsJS.Partners.Config.isPartnerApp) {
if (document.msHidden) {
WSJJS.Instrumentation.instance.openSession();
}
else {
WSJJS.Instrumentation.instance.closeSession();
}
}
},
createWSJLive: function createWSJLive() {
var wsjLive = this.createChannel("WSJ LIVE", true);
wsjLive.state.feedType = "WSJ.GetVideoSection";
wsjLive.pageInfo.fragment = "/wsjshared/wsjZoomCategoryPano.html";
return wsjLive;
},
createChannel: function createChannel(section, visible) {
var title = PlatformJS.Services.resourceLoader.getString("/wsj/" + WSJJS.ArticleManager.getSectionTitle(section));
var id = section === "FRONT SECTION" && NewsJS.Partners.Config.isPartnerApp ? "Home" : partner + "_" + section;
return {
icon: NewsJS.Partners.Config.getSectionImage(partner, section),
id: id,
pageInfo: {
channelId: id,
fragment: "/wsjshared/wsjCategoryPano.html",
page: "WSJJS.CategoryPano",
},
state: {
feedIdentifierValue: section,
feedType: "section",
theme: partner,
pinLogoUrls: { "100": "/images/wsj_pin.scale-100.png", "140": "/images/wsj_pin.scale-140.png", "180": "/images/wsj_pin.scale-180.png" }
},
subChannels: [],
title: title,
visible: visible,
isDisplayValue: true
};
},
populateEdgy: function populateEdgy() {
var channelArray = NewsJS.Partners.BaseProcessListener.getChannelArray(partner);
if (!channelArray) {
return;
}
channelArray.splice(0, channelArray.length);
var feeds = WSJJS.ArticleManager.instance.getFeeds();
for (var i = 0; i < feeds.size; i++) {
var feed = feeds[i];
if (feed.IsPano && feed.IsPano.value) {
var section = feed.Identifier.value;
if (section !== "FRONT SECTION" || NewsJS.Partners.Config.isPartnerApp) {
channelArray.push(this.createChannel(section, true));
}
}
}
channelArray.push(this.createWSJLive());
NewsJS.Partners.BaseProcessListener.saveChannelData();
},
getSettingsPageData: function getSettingsPageData() {
/// <summary>Add settings page</summary>
/// <returns>Promise containing flyout</returns>
// editions are being temporarily taken out
// uncomment the below code to enable editions again
//var currentEdition = WSJJS.ArticleManager.instance.edition;
var headerText = PlatformJS.Services.resourceLoader.getString("/platform/optionsTitle");
var dataContext = {
headerText: headerText,
onSignIn: WinJS.Utilities.markSupportedForProcessing(function (event) {
PlatformJS.Utilities.getControl("SettingsPageDivCtrl").hide();
WSJJS.Auth.instance.signIn();
}),
onSignOut: WinJS.Utilities.markSupportedForProcessing(function (event) {
WSJJS.Auth.instance.signOut();
WSJJS.Instrumentation.instance.instrumentSignOut();
}),
signInLabel: PlatformJS.Services.resourceLoader.getString("/wsj/signInLabel"),
signOutLabel: PlatformJS.Services.resourceLoader.getString("/wsj/signOutLabel"),
subscribeLabel: PlatformJS.Services.resourceLoader.getString("/wsj/subscribeLabel"),
onSubscribe: WinJS.Utilities.markSupportedForProcessing(function (event) {
PlatformJS.Utilities.getControl("SettingsPageDivCtrl").hide();
WSJJS.Auth.instance.signIn();
}),
// editions are being temporarily taken out
// uncomment the below code to enable editions again
//onEditionSelection: WinJS.Utilities.markSupportedForProcessing(function (event) {
//    document.getElementById("applyEditionButton").disabled = event.currentTarget.value === currentEdition;
//}),
//selectEdition: PlatformJS.Services.resourceLoader.getString("/wsj/selectEdition"),
//onApplyEdition: WinJS.Utilities.markSupportedForProcessing(function (event) {
//    PlatformJS.Utilities.getControl("SettingsPageDivCtrl").hide();
//    WSJJS.ArticleManager.instance.edition = document.getElementById("editionList").value;
//}),
//applyLabel: PlatformJS.Services.resourceLoader.getString("/wsj/apply"),
disabled: CommonJS.Partners.Auth.BaseAuthControl.promptControl !== null
};
// editions are being temporarily taken out
// uncomment the below code to enable editions again
//msSetImmediate(function () {
//    var editions = NewsJS.Partners.Config.getConfig(partner, "Editions", []);
//    if (!editions) {
//        return;
//    }
//    var editionList = document.getElementById("editionList");
//    editions.forEach(function (edition) {
//        var option = document.createElement("option");
//        option.value = edition.value;
//        option.innerText = PlatformJS.Services.resourceLoader.getString("/wsj/" + edition.value);
//        editionList.appendChild(option);
//    });
//    editionList.value = currentEdition;
//});
return dataContext;
},
onLayoutChange: function onLayoutChange(event) {
NewsJS.Partners.BaseProcessListener.prototype.onLayoutChange.call(this, event);
if (event.detail[0].viewState === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
if (NewsJS.Partners.Config.isPartnerPage(partner)) {
WSJJS.Instrumentation.instance.instrumentSnap();
}
}
},
},
{
channelListAvailable: false,
processNewChannels: function processNewChannels(latestFeed, isCachedResponse) {
if (!WSJJS.ProcessListener.channelListAvailable || !isCachedResponse) {
WSJJS.ProcessListener.channelListAvailable = true;
}
}
}),
ArticleManager: WinJS.Class.derive(NewsJS.Partners.ArticleManager, function articleManager_ctor() {
/// <summary> 
/// Article manager for WSJ 
/// </summary>
this._latest = null;
this._partner = partner;
this._articlePrefix = NewsJS.Partners.Config.getConfig(partner, "ArticlePrefix", "http://online.wsj.com/article/");
this._initialized = false;
this._sectionsMap = {};
},
{
_latest: null,
_initialized: false,
getFeeds: function getFeeds() {
var edition = this.edition;
var feedsRoot = NewsJS.Partners.Config.getConfig(partner, "FeedsRoot", {});
var feeds = feedsRoot.getList(edition + "_Feeds");
return feeds;
},
_listContains: function _listContains(list, value) {
for (var i = 0; i < list.length; i++) {
if (list[i].value === value) {
return true;
}
}
return false;
},
edition: {
get: function get() {
return "edition_us";
//if (!this._edition) {
//    this._edition = NewsJS.Partners.Config.getConfig(partner, WSJJS.ArticleManager.currentEdition, "");
//    if (!this._edition || !this._listContains(NewsJS.Partners.Config.getConfig(partner, "Editions", []), this._edition)) {
//        this._edition = "edition_us";
//    }
//}
//return this._edition;
},
set: function set(value) {
if (value !== this._edition) {
this._edition = value;
this._latest = null;
NewsJS.Partners.Config.setConfig(partner, WSJJS.ArticleManager.currentEdition, value);
PlatformJS.Navigation.mainNavigator.returnHomeAndClearHistoryIfNecessary(true);
}
}
},
initializeAsync: function initializeAsync(config) {
/// <summary>
/// Initialize the list of the list of downloaded sections
/// </summary>
/// <returns>
/// Completion promise
/// </returns>
return WinJS.Promise.wrap();
},
serializeAsync: function serializeAsync() {
/// <summary>
/// Serialize the downloaded hashes if it has changed
/// </summary>
/// <returns>
/// Completion promise
/// </returns>
return WinJS.Promise.wrap();
},
resetAsync: function resetAsync() {
/// <summary>
/// Destroy the downloaded hashes file
/// </summary>
/// <returns>
/// Completion promise
/// </returns>
return WinJS.Promise.wrap();
},
prefetch: function prefetch() {
/// <summary>
/// Kick off prefetching of sections that have changed
/// </summary>
var that = this;
that.getLatestAsync(true).then(function (latest) {
for (var i = 0; i < latest.sections.length; i++) {
var categoryInfo = latest.sections[i];
(function (catInfo) {
var fullfeed = catInfo.fullfeed,
queryServiceName = "WSJ.GetSection";
if (catInfo.feedType !== "section" && catInfo.feedType) {
queryServiceName = catInfo.feedType;
}
var queryService = new PlatformJS.DataService.QueryService(queryServiceName),
prefetchOptions = new Platform.DataServices.PrefetchQueryServiceOptions(),
urlParams = PlatformJS.Collections.createStringDictionary();
prefetchOptions.bypassCache = true;
prefetchOptions.priority = NewsJS.Partners.Config.isPartnerApp ? Platform.DataServices.QueryServicePriority.medium : Platform.DataServices.QueryServicePriority.low;
urlParams.insert("sectionFeed", fullfeed);
queryService.prefetchData(urlParams, null, null, prefetchOptions, function (response) {
if (response) {
try {
var feed = that._parseFullFeed(response.dataString, catInfo.sectionName, catInfo.feedType);
if (!feed || catInfo.feedType !== "section") {
return;
}
for (var j = 0; j < feed.feedItems.length; j++) {
var feedItem = feed.feedItems[j];
if (feedItem.newsClusterItem && feedItem.newsClusterItem.thumbnail && feedItem.newsClusterItem.thumbnail.url) {
that._prefetchImage({ url: feedItem.newsClusterItem.thumbnail.url });
}
if (!NewsJS.Partners.Config.isPartnerApp && feedItem.access === "subscribed") {
continue;
}
var getArticleService = new PlatformJS.DataService.QueryService("WSJ.GetArticle"),
getArticleOptions = new Platform.DataServices.PrefetchQueryServiceOptions(),
getArticleUrlParams = PlatformJS.Collections.createStringDictionary();
getArticleOptions.priority = NewsJS.Partners.Config.isPartnerApp ? Platform.DataServices.QueryServicePriority.medium : Platform.DataServices.QueryServicePriority.low;
getArticleUrlParams.insert("source", feedItem.guid.indexOf("http") === 0 ? "online" : "mobilefeeds");
getArticleUrlParams.insert("guid", feedItem.guid);
getArticleUrlParams.insert("timestamp", new Date(feedItem.pubDate).getTime());
getArticleService.prefetchData(getArticleUrlParams, null, null, getArticleOptions, function (articleResponse) {
if (articleResponse) {
that._prefetchArticleImages(articleResponse.dataString);
}
});
}
} catch (e) {
// do nothing
}
}
});
})(categoryInfo);
}
});
},
_prefetchArticleImages: function _prefetchArticleImages(articleString) {
var articleData = new Windows.Data.Xml.Dom.XmlDocument();
try {
articleData.loadXml(articleString);
} catch (e) {
return;
}
articleData.normalize();
articleData = articleData.selectSingleNode("/article-doc");
var articleBody = articleData.selectSingleNode("./article/article-body");
var blockNodes = articleBody.selectNodes("./*");
var that = this;
for (var i = 0; i < blockNodes.length; i++) {
var blockNode = blockNodes.getAt(i);
if (blockNode.tagName === "media") {
var mediaType = blockNode.attributes.getNamedItem("type").nodeValue;
if (mediaType === "ILLUSTRATION" || mediaType === "FLASH") {
that._prefetchImageBlock(blockNode, mediaType);
}
}
}
},
_imagePrefix: null,
_prefetchImageBlock: function _prefetchImageBlock(media, type) {
var imageNode = type === "ILLUSTRATION" ? media.selectNodes("./alt-image[last()]|./image|./link/image") : media;
if (imageNode.length) {
imageNode = imageNode[imageNode.length - 1];
}
var thumbnailSrc = type === "ILLUSTRATION" ? imageNode.attributes.getNamedItem("src-id") : imageNode.attributes.getNamedItem("thumbnail-src");
if (!this._imagePrefix) {
this._imagePrefix = NewsJS.Partners.Config.getConfig(partner, "ImagePrefix", "http://s.wsj.net/public/resources/images/");
}
if (thumbnailSrc) {
this._prefetchImage({ url: this._imagePrefix + thumbnailSrc.nodeValue });
}
},
getLatestAsync: function getLatestAsync(bypassCache) {
/// <summary>
/// Make the article manager is initialized and fetch the lastest feed
/// </summary>
/// <param name="bypassCache">
/// Bypass the cache and hit the network
/// </param>
/// <returns>
/// Promise to the parsed latest feed
/// </returns>
var that = this;
if (this._initialized) {
return that._getLatestAsync(bypassCache);
} else {
return this.initializeAsync(null).then(function () {
return that._getLatestAsync(bypassCache);
});
}
},
_getLatestAsync: function _getLatestAsync(bypassCache) {
/// <summary>
/// Fetch the latest feed
/// </summary>
/// <param name="bypassCache"> 
/// Bypass the cache
/// </param>
/// <returns>
/// Promise to the parsed latest feed
/// </returns>
var config = this.getFeeds(), i = 0;
if (!this._latest) {
this._latest = {
sections: []
};
for (i = 0; i < config.size; i++) {
var title = PlatformJS.Services.resourceLoader.getString("/wsj/" + WSJJS.ArticleManager.getSectionTitle(config[i].Identifier.value));
this._latest.sections.push({
sectionName: config[i].Identifier.value,
fullfeed: config[i].Feed.value,
title: title,
feedType: config[i].Type ? config[i].Type.value : "section",
pinTitle: config[i].Type && config[i].Type.value === "WSJ.GetVideoSection" ? PlatformJS.Services.resourceLoader.getString("/wsj/VideoTitleFormat").format(title) : null,
});
}
for (i = 0; i < this._latest.sections.length; i++) {
this._sectionsMap[this._latest.sections[i].sectionName] = this._latest.sections[i];
}
}
return WinJS.Promise.wrap(this._latest);
},
_getFullFeedAsync: function _getFullFeedAsync(categoryInfo, bypassCache, feedType, feedIdentifierValue, /*@optional*/provider) {
/// <summary>
/// Get a the full feed using the query service
/// </summary>
/// <param name="categoryInfo">
/// Category info object
/// </param>
/// <param name="bypassCache">
/// By pass the cache
/// </param>
/// <param name="feedType">
/// The feed type: blog or section
/// </param>
/// <param name="feedIdentifierValue">
/// sectionName or blogName
/// </param>
/// <param name="provider" optional="true">
/// provider to receive EntityCluster cache info
/// </param>
/// <returns>
/// Promise containing the response from the query service
/// </returns>
var that = this,
fullfeed = categoryInfo.fullfeed,
queryServiceName = "WSJ.GetSection";
if (categoryInfo.queryServiceName) {
queryServiceName = categoryInfo.queryServiceName;
}
else if (feedType !== "section" && feedType) {
queryServiceName = feedType;
}
var queryService = new PlatformJS.DataService.QueryService(queryServiceName),
options = new Platform.DataServices.QueryServiceOptions(),
urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("sectionFeed", fullfeed);
options.bypassCache = bypassCache;
if (provider) {
provider.layoutCacheInfo = { cacheKey: queryService.getUrl(urlParams), cacheID: queryService.getCacheID() };
}
var result = queryService.downloadDataAsync(urlParams, null, null, options);
return result;
},
getFeedInfo: function getFeedInfo(feedType, feedIdentifierValue) {
var category = this._sectionsMap[feedIdentifierValue];
return category;
},
getFeedInfoAsync: function getFeedInfoAsync(feedType, feedIdentifierValue, bypassCache) {
var that = this;
// Get the lastest feed 
return that.getLatestAsync(bypassCache).then(function (latest) {
if (feedIdentifierValue === "WSJ LIVE") {
return WinJS.Promise.wrap({
feedType: feedType,
sectionName: feedIdentifierValue,
title: PlatformJS.Services.resourceLoader.getString("/wsj/WSJ LIVE")
});
}
var category = that.getFeedInfo(feedType, feedIdentifierValue);
//HACK leadership is the only video section that has a different url
if (feedIdentifierValue === "LEADERSHIP VIDEO") {
category.queryServiceName = "WSJ.GetVideoSubSection";
}
if (category) {
return WinJS.Promise.wrap(category);
} else {
return WinJS.Promise.wrapError("Unable to find feed info");
}
});
},
getFullFeedAsync: function getFullFeedAsync(feedType, feedIdentifierValue, feedInfo, bypassCache, /*@optional*/provider) {
/// <summary>
/// Get the full feed
/// </summary>
/// <param name="feedType">
/// blog or section
/// </param>
/// <param name="feedIdentifierValue">
/// blogName or sectionName
/// </param>
/// <param name="feedInfo" optional="true">
/// Feed info object
/// </param>
/// <param name="bypassCache" optional="true">
/// Bypass the cache
/// </param>
/// <param name="provider" optional="true">
/// provider to receive EntityCluster cache info
/// </param>
/// <returns>
/// Promise containing the parsed full feed
/// </returns>
var that = this,
identifierKey = "sectionName",
feedInfoPromise = null;
if (feedInfo) {
feedInfoPromise = WinJS.Promise.wrap(feedInfo);
} else {
feedInfoPromise = this.getFeedInfoAsync(feedType, feedIdentifierValue, bypassCache);
}
return feedInfoPromise.then(function (category) {
var fullfeed = null,
isCachedResponse = false;
return that._getFullFeedAsync(category, bypassCache, feedType, feedIdentifierValue, provider).then(function (response) {
if (response && provider && provider.layoutCacheInfo) {
provider.layoutCacheInfo.uniqueResponseID = response.responseId;
}
var feed = null;
// We don't need to parse again if this is a cached response because we already did it in the progress handler
if (!response.isCachedResponse || !fullfeed) {
feed = that._parseFullFeed(response.dataString, feedIdentifierValue, feedType);
if (feed) {
fullfeed = feed;
isCachedResponse = response.isCachedResponse;
}
}
// If we have a full feed, return it
if (fullfeed) {
category.lastBuildDate = fullfeed.categoryInfo.lastBuildDate;
return WinJS.Promise.wrap({
categoryInfo: category,
fullfeed: fullfeed.feedItems,
isCachedResponse: isCachedResponse
});
} else {
return WinJS.Promise.wrapError("Unable to fetch fullfeed");
}
}, function (error) {
// If we have a full feed return it
if (fullfeed) {
return WinJS.Promise.wrap({
categoryInfo: fullfeed.categoryInfo,
fullfeed: fullfeed.feedItems,
isCachedResponse: isCachedResponse
});
} else {
return WinJS.Promise.wrapError(error);
}
}, function (progress) {
// If have a cached response, store it for later
if (progress.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryFound &&
progress.cachedResponse) {
if (provider && provider.layoutCacheInfo) {
provider.layoutCacheInfo.uniqueResponseID = progress.cachedResponse.responseId;
}
fullfeed = that._parseFullFeed(progress.cachedResponse.dataString, feedIdentifierValue, feedType);
isCachedResponse = true;
if (provider && provider.handleProgress) {
provider.handleProgress({
categoryInfo: category,
fullfeed: fullfeed.feedItems,
isCachedResponse: isCachedResponse
});
}
}
});
});
},
normalizeGuid: function normalizeGuid(url) {
var articlePrefix = "http://.*/article/";
if (url.search(articlePrefix) === 0) {
var urlObj = new Windows.Foundation.Uri(url);
var path = urlObj.path;
path = path.replace("/article/", "");
path = path.replace(".html", "");
return path;
} else {
return encodeURI(url);
}
},
_removeArticleFromList: function _removeArticleFromList(guidList, guid) {
for (var i = 0; i < guidList.length; i++) {
if (guidList[i].articleId === guid) {
guidList.splice(i, 1);
return;
}
}
},
_parseFullFeed: function _parseFullFeed(dataString, feedIdentifierValue, feedType) {
/// <summary>
/// Parse the full feed
/// </summary>
/// <param name="dataString">
/// JSON string
/// </param>
/// <param name="feedIdentifierValue">
/// blogName or sectionName
/// </param>
/// <param name="feedType">
/// blog or section
/// </param>
/// <returns>
/// The parsed full feed
/// </returns>
var data = null,
that = this;
try {
var jsonString = Platform.Utilities.XmlToJsonTransformer.convertXmlToJsonString(dataString);
data = JSON.parse(jsonString);
} catch (e) { }
if (!data) {
return null;
}
var feedItems = data.rss.channel.item;
var articles = feedItems || [];
var articleMap = {};
var guidList = [],
i = 0;
for (i = 0; i < articles.length; i++) {
var article = articles[i];
article.access = feedType === "WSJ.GetVideoSection" ? "anonymous" : "subscribed";
if (typeof (article.category) === "string") {
article.access = article.category === "FREE" ? "anonymous" : "subscribed";
} else if (article.category && article.category.domain === "AccessClassName") {
article.access = article.category.value === "PAID" ? "subscribed" : "anonymous";
} else if (feedType === "WSJ.GetVideoSection") {
article.access = "anonymous";
}
if (article.guid && article.guid.value) {
article.guid = article.guid.value;
} else {
article.guid = this.normalizeGuid(article.link);
}
guidList.push({
articleId: article.guid,
access: article.access,
timestamp: new Date(article.pubDate).getTime()
});
if (i === 0 && !data.rss.channel.lastBuildDate) {
data.rss.channel.lastBuildDate = article.pubDate;
}
}
var finalList = [];
var videoList = [];
for (i = 0; i < articles.length; i++) {
var feedItem = articles[i];
var thumbnail = null;
var video = null;
if (feedItem.content) {
thumbnail = feedItem.content;
}
if (feedItem.group) {
if (feedItem.group.thumbnail && feedItem.group.thumbnail.length > 0
&& feedItem.group.thumbnail[0].url && feedItem.group.thumbnail[0].height && feedItem.group.thumbnail[0].width) {
thumbnail = feedItem.group.thumbnail[0];
}
if (feedItem.group.content && feedItem.group.content.length > 0 && feedItem.group.content[0].url) {
video = feedItem.group.content[0];
}
}
var scaling = 0;
if (thumbnail) {
if (thumbnail.height && thumbnail.width) {
thumbnail.height = parseInt(thumbnail.height);
thumbnail.width = parseInt(thumbnail.width);
} else {
thumbnail.height = 369;
thumbnail.width = 553;
}
}
if (thumbnail && thumbnail.width >= 480 && thumbnail.width < 610) {
scaling = 610 / thumbnail.width;
thumbnail.width *= scaling;
thumbnail.height *= scaling;
}
if (thumbnail && thumbnail.height >= 340 && thumbnail.height < 350) {
scaling = 350 / thumbnail.height;
thumbnail.width *= scaling;
thumbnail.height *= scaling;
}
var description = "";
if (feedItem.description && Array.isArray(feedItem.description.value)) {
if (!feedItem.description.value[0]) {
this._removeArticleFromList(guidList, feedItem.guid);
continue;
}
description = feedItem.description.value.join("&nbsp;");
} else if ((typeof (feedItem.description) === "string")) {
description = feedItem.description;
}
// Must have a video and a thumbnail
if (feedType === "WSJ.GetVideoSection" && (!thumbnail || !video)) {
this._removeArticleFromList(guidList, feedItem.guid);
continue;
}
var byline = feedItem.byline ? PlatformJS.Services.resourceLoader.getString("/wsj/ByLineFormat").format(feedItem.byline) : "";
var kicker = feedItem.displayName ? feedItem.displayName : "";
if (i === 0 && feedItem.encoded && (!thumbnail || !thumbnail.width || !thumbnail.height || !thumbnail.url)) {
var desc = feedItem.encoded;
desc = desc.split(" ");
var len = desc.length;
var maxCount = Math.min(100, len);
desc = desc.splice(0, maxCount);
desc = desc.join(" ");
if (desc) {
description = desc;
if (maxCount < len) {
description += "&nbsp;&#187;";
}
}
}
// Store the news cluster item as an property to the feed item
feedItem.newsClusterItem = {
articleUrl: feedItem.link,
articleURL: feedItem.link,
publishTime: WSJJS.Formatter.instance.formatTime(feedItem.pubDate, WSJJS.Formatter.tileFormat),
publishTimeRaw: feedItem.pubDate,
snippet: description,
source: byline,
sourceImageUrl: "",
kicker: kicker,
title: feedItem.title + (feedItem.access === "subscribed" ? "<div class=\"lockIcon\"></div>" : ""),
headline: feedItem.title,
byline: byline,
thumbnail: thumbnail,
video: video,
imageAttribution: thumbnail && thumbnail.credit ? thumbnail.credit : "",
articleId: feedItem.guid,
type: feedType === "WSJ.GetVideoSection" ? "video" : "article",
access: feedItem.access,
articleIndex: i + 1,
feedIdentifierValue: feedIdentifierValue,
feedType: feedType,
articleList: guidList,
articleid: feedItem.guid,
contentid: feedItem.guid
};
if (video) {
var newsClusterItem = feedItem.newsClusterItem;
video.title = newsClusterItem.title;
video.snippet = newsClusterItem.snippet;
video.thumbnailUrl = newsClusterItem.thumbnail.url;
video.articleIndex = newsClusterItem.articleIndex;
videoList.push(video);
}
finalList.push(feedItem);
}
for (var k = 0; k < finalList.length; k++) {
finalList[k].newsClusterItem.videoList = videoList;
}
return {
categoryInfo: data.rss.channel,
feedItems: finalList
};
},
getArticleAsync: function getArticleAsync(feedType, feedIdentifierValue, articleId, pubDate) {
/// <summary>
/// Get a parsed article from the cache
/// </summary>
/// <param name="feedType">
/// blog or section
/// </param>
/// <param name="feedIdentifierValue">
/// blogName or sectionName
/// </param>
/// <param name="articleId">
/// WSJ article ID
/// </param>
/// <returns>
/// Promise containing the article
/// </returns>
var that = this,
queryService = new PlatformJS.DataService.QueryService("WSJ.GetArticle"),
options = new Platform.DataServices.QueryServiceOptions(),
urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("source", "mobilefeeds");
urlParams.insert("guid", articleId);
urlParams.insert("timestamp", encodeURIComponent(pubDate));
var article = null;
return queryService.downloadDataAsync(urlParams, null, null, options).then(function (response) {
if (!response.isCachedResponse || !article) {
try {
article = response.dataString;
} catch (e) { }
}
if (article) {
return WinJS.Promise.wrap(article);
} else {
return WinJS.Promise.wrapError("ArticleNotFound");
}
}, function (error) {
if (article) {
return WinJS.Promise.wrap(article);
} else {
return WinJS.Promise.wrapError(error);
}
}, function (progress) {
if (progress.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryFound && progress.cachedResponse) {
try {
article = progress.cachedResponse.dataString;
} catch (e) { }
}
});
},
getVideoInfoAsync: function getVideoInfoAsync(guidList) {
var queryService = new PlatformJS.DataService.QueryService("WSJ.GetVideoInfo"),
options = new Platform.DataServices.QueryServiceOptions(),
urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("guid", guidList.join(","));
return queryService.downloadDataAsync(urlParams, null, null, options).then(function (response) {
var json = JSON.parse(response.dataString);
return WinJS.Promise.wrap(json);
});
}
},
{
_instance: null,
currentEdition: "CurrentEdition",
instance: {
/// <summary>Singleton instance</summary>
get: function get() {
if (!WSJJS.ArticleManager._instance) {
WSJJS.ArticleManager._instance = new WSJJS.ArticleManager();
}
return WSJJS.ArticleManager._instance;
}
},
getSectionTitle: function getSectionTitle(section) {
var title = section;
if (section === "PERSONAL JOURNAL") {
var today = new Date();
var offset = today.getTimezoneOffset() * 60 * 1000;
var utc = today.getTime() + offset;
var eastOffset = 3600000 * -5;
var eastTime = new Date(utc + eastOffset);
switch (eastTime.getDay()) {
case 5:
title = "FRIDAY JOURNAL";
break;
case 6:
case 0:
title = "WEEKEND JOURNAL";
break;
default:
break;
}
}
return title;
},
})
});
WinJS.Namespace.define("WSJJS.NewsCluster",
{
Provider: WinJS.Class.derive(NewsJS.Partners.NewsClusterProvider,
function WSJJS_cluster_provider(categoryInfo, title, maxArticles, onsuccess, onerror, bypassCache, isPrimaryCluster) {
/// <summary>News cluster provider for WSJ</summary>
/// <param name="categoryInfo">Feed info</param>
/// <param name="title" optional="true">Title of cluster</param>
/// <param name="maxArticles" optional="true">Maximum number of articles to show</param>
/// <param name="onsuccess" optional="true">Success callback</param>
/// <param name="onerror" optional="true">error callback</param>
/// <param name="bypassCache" optional="true">Bypass the cache</param>
/// <param name="isPrimaryCluster" optional="true">Is primary cluster</param>
this.isPrimaryCluster = isPrimaryCluster;
this._articleManager = WSJJS.ArticleManager.instance;
this._partner = partner;
this._imagePrefix = NewsJS.Partners.Config.getConfig(partner, "ImagePrefix", "http://s.wsj.net/public/resources/images/");
NewsJS.Partners.NewsClusterProvider.call(this, categoryInfo, title, maxArticles, onsuccess, onerror, bypassCache);
},
{
selectTemplateClass: function selectTemplateClass(index, item) {
if (index < 1 && item.newsClusterItem.type !== "video") {
item.newsClusterItem.templateClass = "WSJLead";
}
else {
if (item.newsClusterItem.type === "video") {
item.newsClusterItem.templateClass = "WSJVideo";
} else {
item.newsClusterItem.templateClass = "WSJRegular";
}
}
return item.newsClusterItem;
},
getFeedIdentifierValue: function getFeedIdentifierValue() {
return this.categoryInfo.sectionName;
},
}),
});
WinJS.Namespace.define("WSJJS",
{
BasePanoPage: WinJS.Class.derive(NewsJS.BasePartnerPano,
function (state) {
if (!NewsJS.Partners.Ads.AdsManager.adFormatters[partner]) {
NewsJS.Partners.Ads.AdsManager.adFormatters[partner] = new WSJJS.Ads.AdFormatter();
}
this._partner = "WSJ";
this._authManager = WSJJS.Auth.instance;
NewsJS.BasePartnerPano.call(this, state);
WSJJS.BasePanoPage.setupPage();
this.addSubscribeButton();
},
{
addSubscribeButton: function addSubscribeButton() {
if (!this._subscribeButton) {
var subscribeButton = this._subscribeButton = document.createElement("button");
WinJS.Utilities.addClass(subscribeButton, "subscribeButton");
subscribeButton.textContent = PlatformJS.Services.resourceLoader.getString("/wsj/SubscribeButton");
subscribeButton.onclick = function () {
WSJJS.Auth.instance.signIn();
};
var page = document.querySelector(".platformPage");
page.appendChild(subscribeButton);
}
},
itemInvoked: function itemInvoked(item, event) {
if (item.type !== "article") {
NewsJS.BasePartnerPano.prototype.itemInvoked.call(this, item, event);
} else {
this._itemInvoked(item, event);
}
},
onManualRefreshButtonClick: function onManualRefreshButtonClick() {
WSJJS.Instrumentation.instance.instrumentManualRefresh();
},
getValidActions: function getValidActions() {
var defaultAction = {
icon: {
type: "image",
url: "/wsjshared/images/WSJLogoBlack.png",
},
text: PlatformJS.Services.resourceLoader.getString("/wsj/thewsj"),
textSize: "small",
};
var subscribeAction = {
icon: {
type: "image",
url: "/wsjshared/images/gray_sub.svg",
},
text: PlatformJS.Services.resourceLoader.getString("/wsj/ArticleSubscriptionMessage"),
textSize: "small",
};
var downloadAction = {
icon: {
type: "image",
url: "/wsjshared/images/WSJLogoBlack.png",
},
text: PlatformJS.Services.resourceLoader.getString("/wsj/ArticleDownloadMessage"),
textSize: "small",
};
if (NewsJS.Partners.Config.isPartnerApp) {
return {
defaultAction: defaultAction,
subscribeAction: subscribeAction
};
} else {
return {
subscribeAction: subscribeAction,
downloadAction: downloadAction
};
}
},
getActionsHandlerType: function getActionsHandlerType() {
return "WSJJS.ArticleReader.WSJArticleActionsHandler";
},
getActionKeys: function getActionKeys() {
if (NewsJS.Partners.Config.isPartnerApp) {
return ["defaultAction", "subscribeAction"];
} else {
return ["subscribeAction", "downloadAction"];
}
},
},
{
setupPage: function setupPage() {
WSJJS.Auth.instance;
WSJJS.BasePanoPage.setupLoginButton();
},
setupLoginButton: function setupLoginButton() {
var separator = new WinJS.UI.AppBarCommand(document.createElement("hr"), {
type: "separator"
});
separator.element.id = "separator";
var loginCmd = new WinJS.UI.AppBarCommand(document.createElement("button"), {
label: PlatformJS.Services.resourceLoader.getString("/partners/Login"),
icon: "\uE181",
onclick: function onclick(event) {
CommonJS.dismissAllEdgies();
WSJJS.Auth.instance.signIn();
}
});
loginCmd.element.id = "loginButton";
WinJS.Utilities.addClass(loginCmd.element, "signInButton");
var logoutCmd = new WinJS.UI.AppBarCommand(document.createElement("button"), {
label: PlatformJS.Services.resourceLoader.getString("/wsj/signOutLabel"),
icon: "\uE181",
onclick: function onclick(event) {
CommonJS.dismissAllEdgies();
WSJJS.Auth.instance.signOut();
WSJJS.Instrumentation.instance.instrumentSignOut();
}
});
logoutCmd.element.id = "logoutButton";
WinJS.Utilities.addClass(logoutCmd.element, "signOutButton");
var edgy = WinJS.Utilities.query(".newsActionEdgyRight");
if (edgy.length > 0) {
edgy = edgy[0];
} else {
edgy = document.getElementById("bottomEdgy");
}
edgy.appendChild(separator.element);
edgy.appendChild(loginCmd.element);
edgy.appendChild(logoutCmd.element);
},
})
});
WinJS.Namespace.define("WSJJS.ArticleReader",
{
Page: WinJS.Class.derive(NewsJS.Partners.ArticleReaderPage, function WSJJS_articlereader_page(state) {
this.partner = partner;
this.articleManager = WSJJS.ArticleManager.instance;
if (!NewsJS.Partners.Ads.AdsManager.adFormatters[partner]) {
NewsJS.Partners.Ads.AdsManager.adFormatters[partner] = new WSJJS.Ads.AdFormatter();
}
NewsJS.Partners.ArticleReaderPage.call(this, state);
this._wsjArticleChangedListener = this._wsjArticleChanged.bind(this);
this._articleManager.addEventListener("articlechanged", this._wsjArticleChangedListener);
},
{
_setupBottomEdgy: function _setupBottomEdgy() {
NewsJS.NewsArticleReaderPage.prototype._setupBottomEdgy.call(this);
WSJJS.BasePanoPage.setupPage();
},
_wsjArticleChanged: function _wsjArticleChanged(event) {
var instrumentationData = event.detail.instrumentationData;
if (instrumentationData) {
var currentArticleId = instrumentationData.content.contentId;
var that = this;
if (!instrumentationData.content.isAd) {
this._provider.getArticleMetadataAsync(currentArticleId).then(function (articleData) {
WSJJS.Instrumentation.instance.instrumentArticleView(
articleData.articleUrl,
currentArticleId,
that.originatingFeed,
that.originatingSection,
that._provider.articleTypeMap[articleData.articleId],
articleData.headline);
});
}
}
},
onShareRequest: function onShareRequest(articleData) {
WSJJS.Instrumentation.instance.instrumentArticleShared("Shared Article", articleData.articleURL, articleData.feedIdentifierValue);
},
dispose: function dispose() {
NewsJS.Partners.ArticleReaderPage.prototype.dispose.call(this);
this._articleManager.removeEventListener("articlechanged", this._wsjArticleChangedListener);
},
}),
Provider: WinJS.Class.derive(NewsJS.Partners.ArticleReaderProvider, function WSJJS_articlereader_provider_ctor(state) {
this.partner = partner;
this._imagePrefix = NewsJS.Partners.Config.getConfig(partner, "ImagePrefix", "http://s.wsj.net/public/resources/images/");
this.guidMap = {};
this.articleTypeMap = {};
this.articleManager = WSJJS.ArticleManager.instance;
NewsJS.Partners.ArticleReaderProvider.call(this, state);
},
{
_imagePrefix: null,
_createImageDescriptor: function _createImageDescriptor(media, type) {
var imageNodes = type === "ILLUSTRATION" ? media.selectNodes("(./alt-image[last()])|(./image)|(./link/image)") : media;
var imageNode = null;
for (var i = 0; i < imageNodes.length; i++) {
if (imageNodes.length) {
imageNode = imageNodes[imageNodes.length - 1 - i];
}
var mediaCredit = media.selectSingleNode("./media-credit"),
mediaCaption = media.selectSingleNode("./media-caption"),
thumbnailSrc = type === "ILLUSTRATION" ? imageNode.attributes.getNamedItem("src-id") : imageNode.attributes.getNamedItem("thumbnail-src"),
height = imageNode.attributes.getNamedItem("height"),
width = imageNode.attributes.getNamedItem("width");
if (thumbnailSrc && height && width &&
thumbnailSrc.nodeValue && height.nodeValue && width.nodeValue) {
break;
}
}
if (!thumbnailSrc || !height || !width ||
!thumbnailSrc.nodeValue || !height.nodeValue || !width.nodeValue) {
return null;
}
var imageData = {
url: this._imagePrefix + thumbnailSrc.nodeValue,
height: parseInt(height.nodeValue),
width: parseInt(width.nodeValue)
},
asset = {
credit: mediaCredit ? mediaCredit.innerText : "",
caption: mediaCaption ? mediaCaption.innerText : ""
};
var model = AppEx.Common.ArticleReader.Model;
var image = new model.ImageDescriptor();
image.attribution = asset.credit ? asset.credit : "";
image.caption = asset.caption ? asset.caption : "";
image.altText = image.attribution;
var resource = new model.ImageResourceDescriptor();
resource.height = imageData.height;
resource.width = imageData.width;
resource.name = "original";
resource.url = imageData.url;
image.images = [resource];
return image;
},
_processContentBlock: function _processContentBlock(p) {
var model = AppEx.Common.ArticleReader.Model,
contentBlock = new model.Block.ContentBlock(),
attributes = new model.BlockAttributes.ContentAttributes();
contentBlock.attributes = attributes;
var result = WSJJS.ArticleReader.Provider.transform.transformToString(p);
contentBlock.attributes.content = result.replace("<?xml version=\"1.0\"?>", "");
return contentBlock;
},
_processTitleImageBlock: function _processTitleImageBlock(media, type) {
var model = AppEx.Common.ArticleReader.Model;
var titleImage = new model.Block.TitleImageBlock();
titleImage.image = this._createImageDescriptor(media, type);
titleImage.locationHint = "";
titleImage.sizeHint = 2;
if (titleImage.image) {
var resource = titleImage.image.images[0];
// Scale up the image to make sure the article reader takes the title image
var minWidth = 1070;
if (resource.width > 740 && resource.height > 490 && resource.width > resource.height) {
var scaleFactor = minWidth / resource.width;
resource.width *= scaleFactor;
resource.height *= scaleFactor;
}
if (this._validateTitleImage(resource)) {
return titleImage;
} else {
return null;
}
} else {
return null;
}
},
_processImageBlock: function _processImageBlock(media, type) {
var model = AppEx.Common.ArticleReader.Model,
imageBlock = new model.Block.InlineImageBlock(),
attributes = imageBlock.attributes = new model.BlockAttributes.InlineImageAttributes();
attributes.image = this._createImageDescriptor(media, type);
if (attributes.image) {
return imageBlock;
} else {
return null;
}
},
_processInset: function _processInset(inset, articleInfo) {
var model = AppEx.Common.ArticleReader.Model,
media = inset.selectSingleNode("./media"),
that = this;
if (!media) {
return null;
}
var param1 = inset.attributes.getNamedItem("param1");
if (param1 && param1.nodeValue === "TABLETEXCLUDE") {
return null;
}
var type = media.attributes.getNamedItem("type");
type = type ? type.value : null;
if (type === "FLASH") {
var caption = inset.selectSingleNode("./small-hed");
caption = caption ? caption.innerText : "";
return this._processFlash(media, articleInfo, caption);
}
return null;
},
_processFlash: function _processFlash(media, articleInfo, caption) {
var image = media.selectSingleNode("./image");
if (image) {
var slug = image.attributes.getNamedItem("slug");
slug = slug ? slug.value : null;
if (slug === "slideshow") {
return this._processSlideShowBlock(media, articleInfo, image, caption);
}
}
},
_processSlideShowBlock: function _processSlideShowBlock(media, articleInfo, image, caption) {
var model = AppEx.Common.ArticleReader.Model,
that = this,
externalBlock = new model.Block.ExternalBlock(),
attributes = externalBlock.attributes = new model.BlockAttributes.ExternalAttributes();
attributes.controlType = "NewsJS.SlideShowEntryBlock";
var slideshowId = image.attributes.getNamedItem("alternate-text");
slideshowId = slideshowId ? slideshowId.value : null;
var mediaCredit = media.selectSingleNode("./media-credit");
mediaCredit = mediaCredit ? mediaCredit.innerText : "";
var mediaCaption = media.selectSingleNode("./media-caption");
mediaCaption = mediaCaption ? mediaCaption.innerText : "";
mediaCaption = caption || mediaCaption;
var thumbnail = media.attributes.getNamedItem("thumbnail-src");
var height = media.attributes.getNamedItem("height");
var width = media.attributes.getNamedItem("width");
if (!thumbnail ||
!height ||
!width ||
!slideshowId) {
return;
}
height = height.value;
width = width.value;
thumbnail = thumbnail.value;
var options = {
articleData: {
articleId: slideshowId
},
providerType: "WSJJS.SlideShow.Provider",
theme: partner,
parentArticle: {
id: articleInfo.articleId,
type: articleInfo.articleType,
feedIdentifierValue: articleInfo.feedIdentifierValue,
caption: caption
},
slideShowPage: "WSJJS.SlideShow.Page"
};
attributes.placement = "inline";
thumbnail = options.thumbnail = {
url: this._imagePrefix + thumbnail,
height: height,
width: width
};
thumbnail.altText = caption || mediaCaption + " " + mediaCredit;
thumbnail.attribution = mediaCaption + " " + mediaCredit;
attributes.maxHeight = options.thumbnail.height;
attributes.maxWidth = options.thumbnail.width;
attributes.controlOptionsSerialized = JSON.stringify(options);
return externalBlock;
},
feedType: null,
feedIdentifierValue: null,
guidMap: null,
articleTypeMap: null,
initializeAsync: function initializeAsync(config) {
this.feedType = config.feedType;
this.feedIdentifierValue = config.feedIdentifierValue;
var guidList = this.sprinkleAds(JSON.parse(config.articleList));
for (var i = 0; i < guidList.length; i++) {
var current = this.guidMap[guidList[i].articleId] = guidList[i];
if (i > 0) {
current.previousArticleId = guidList[i - 1].articleId;
}
if (i < guidList.length - 1) {
current.nextArticleId = guidList[i + 1].articleId;
}
}
if (!WSJJS.ArticleReader.Provider.transform) {
var uri = new Windows.Foundation.Uri("ms-appx:///wsjshared/transform.xml");
return Windows.Storage.StorageFile.getFileFromApplicationUriAsync(uri).then(function (file) {
return Windows.Data.Xml.Dom.XmlDocument.loadFromFileAsync(file);
}).then(function (xml) {
WSJJS.ArticleReader.Provider.transform = new Windows.Data.Xml.Xsl.XsltProcessor(xml);
return WinJS.Promise.wrap();
}, function () { });
} else {
return WinJS.Promise.wrap({});
}
},
isAd: function isAd(article) {
if (typeof article === "string") {
return this.getAdInfo(article) !== null;
} else if (article.type === "ad") {
return true;
}
return false;
},
getAdInfo: function getAdInfo(articleId) {
var article = this.guidMap[articleId];
if (article && article.type === "ad") {
return article;
} else {
return null;
}
},
_processTitleBlock: function _processTitleBlock(articleBody, date) {
var model = AppEx.Common.ArticleReader.Model,
that = this,
title = new model.Title(),
publisher = new model.SourceDescriptor(),
favicon = new model.ImageResourceDescriptor(),
bylineNode = articleBody.selectSingleNode("./byline"),
mainHedNode = articleBody.selectSingleNode("./headline/main-hed");
if (bylineNode) {
title.byline = bylineNode.innerText.replace(/\s\s+/g, " ").replace(/^\s+|\s+$/g, "");
} else {
title.byline = "";
}
title.date = date ? WSJJS.Formatter.instance.formatTime(date.innerText, WSJJS.Formatter.articleFormat) : "";
title.headline = mainHedNode ? mainHedNode.innerText.replace(/\s\s+/g, " ").replace(/^\s+|\s+$/g, "") : "";
title.allowAds = true;
title.pgCode = "";
title.author = title.byline || "";
title.publisher = publisher;
title.style = 100;
publisher.name = NewsJS.Partners.Config.isPartnerApp ? "" : PlatformJS.Services.resourceLoader.getString("/wsj/thewsj");
publisher.favicon = favicon;
favicon.height = 40;
favicon.width = 360;
favicon.url = NewsJS.Partners.Config.isPartnerApp ? "" : "/wsjshared/images/WSJ_Logo_black.svg";
favicon.name = publisher.name;
return title;
},
_processVideoBlock: function _processVideoBlock(media) {
var model = AppEx.Common.ArticleReader.Model,
videoBlock = new model.Block.InlineVideoBlock(),
attributes = new model.BlockAttributes.InlineVideoAttributes(),
video = new model.VideoDescriptor(),
imageNode = media.selectSingleNode("./image"),
mediaCredit = media.selectSingleNode("./media-credit"),
mediaCaption = media.selectSingleNode("./media-caption");
if (!imageNode) {
return null;
}
var slug = imageNode.attributes.getNamedItem("slug");
if (!slug) {
return null;
}
slug = slug.value;
if (slug.indexOf("video") !== 0) {
return null;
}
var videoId = imageNode.attributes.getNamedItem("src-id");
if (!videoId) {
return null;
}
videoId = videoId.nodeValue;
mediaCaption = mediaCaption ? mediaCaption.innerText : "";
mediaCredit = mediaCredit ? mediaCredit.innerText : "";
videoBlock.attributes = attributes;
attributes.video = video;
attributes.sizeHint = 2;
video.attribution = mediaCaption ? mediaCaption + " " + mediaCredit : mediaCredit;
video.caption = video.attribution;
video.videoSource = videoId;
return videoBlock;
},
_removeBlock: function _removeBlock(block, assetList) {
var index = assetList.indexOf(block);
assetList.splice(index, 1);
},
processVideosAsync: function processVideosAsync(videoBlocks, assetList) {
var guidList = [],
that = this;
for (var i = 0; i < videoBlocks.length; i++) {
guidList.push(videoBlocks[i].attributes.video.videoSource);
}
return WSJJS.ArticleManager.instance.getVideoInfoAsync(guidList).then(function (videoInfos) {
var videoMap = {},
j = 0;
videoInfos = videoInfos.items;
for (j = 0; j < videoInfos.length; j++) {
if (videoInfos[j].id) {
videoMap[videoInfos[j].id.replace(/[{}]/g, "")] = videoInfos[j];
}
}
for (j = 0; j < videoBlocks.length; j++) {
var videoBlock = videoBlocks[j],
videoObj = videoBlock.attributes.video;
var videoInfo = videoMap[videoObj.videoSource];
if (videoInfo) {
var thumbnailList = videoInfo.thumbnailList;
var poster = thumbnailList && thumbnailList.length > 0 ? thumbnailList[thumbnailList.length - 1] : null;
var videoList = videoInfo.videoMP4List;
var video = videoList && videoList.length > 0 ? videoList[0] : null;
if (!video) {
that._removeBlock(videoBlock, assetList);
}
videoObj.height = poster.height;
videoObj.width = poster.width;
videoObj.videoSource = video.url;
videoObj.posterUrl = poster.url;
} else {
that._removeBlock(videoBlock, assetList);
}
}
return WinJS.Promise.wrap();
}, function () {
// If there is an error, remove all the video blocks
for (var j = 0; j < videoBlocks.length; j++) {
that._removeBlock(videoBlocks[j], assetList);
}
return WinJS.Promise.wrap();
});
},
isArticleLocked: function isArticleLocked(articleId) {
var articleInfo = this.guidMap[articleId];
if (articleInfo) {
return articleInfo.access !== "anonymous";
}
return false;
},
getArticleAsync: function getArticleAsync(articleId) {
var that = this;
var nextArticleId = null;
var previousArticleId = null;
var articleInfo = this.guidMap[articleId];
if (articleInfo) {
nextArticleId = articleInfo.nextArticleId;
previousArticleId = articleInfo.previousArticleId;
}
if (articleInfo.type === "ad") {
articleInfo.nextArticle = {
articleId: nextArticleId
};
articleInfo.previousArticle = {
articleId: previousArticleId
};
return WinJS.Promise.wrap(articleInfo);
}
articleInfo.feedIdentifierValue = this.feedIdentifierValue;
var dataPromise = WSJJS.ArticleManager.instance.getArticleAsync(this.feedType, this.feedIdentifierValue, articleId, articleInfo.timestamp);
return dataPromise.then(function (articleString) {
var articleData = new Windows.Data.Xml.Dom.XmlDocument();
var model = AppEx.Common.ArticleReader.Model,
article = new model.Article(),
title = null;
articleData.loadXml(articleString);
articleData.normalize();
articleData = articleData.selectSingleNode("/article-doc");
var articleType = articleData.selectSingleNode("@type");
articleType = articleType ? articleType.innerText : "";
articleInfo.articleType = articleType;
var articleBody = articleData.selectSingleNode("./article/article-body");
// Process the title block
article.title = title = that._processTitleBlock(articleBody, articleData.selectSingleNode("@last-pub-date"));
that.articleTypeMap[articleId] = articleType;
// Try article body first
var processedArticleBody = that.processArticleBody(articleBody, articleInfo, title);
// If that fails, try the alt-summary-body
if (!processedArticleBody ||
(processedArticleBody.assetBlocks.length === 0 &&
processedArticleBody.contentBlocks.length === 0 &&
processedArticleBody.videoBlocks.length === 0)) {
processedArticleBody = that.processArticleBody(articleData.selectSingleNode("./summary/summary-body"), articleInfo, title);
}
if (!processedArticleBody) {
return WinJS.Promise.wrapError("Unable to process article");
}
var haveTitleImage = processedArticleBody.haveTitleImage,
contentBlocks = processedArticleBody.contentBlocks,
assetBlocks = processedArticleBody.assetBlocks,
videoBlocks = processedArticleBody.videoBlocks;
article.metadata = new model.ArticleInfo();
article.metadata.articleId = articleId;
article.metadata.adGroups = that.createArticleAds();
article.metadata.instrumentationId = NewsJS.Partners.Config.getConfig(partner, "InstrumentationId", "");
// Assign next article 
if (nextArticleId) {
var nextArticle = null;
article.nextArticle = nextArticle = new model.ArticleInfo();
nextArticle.articleId = nextArticleId;
}
// Assign previous article
if (previousArticleId) {
var previousArticle = null;
article.previousArticle = previousArticle = new model.ArticleInfo();
previousArticle.articleId = previousArticleId;
}
var postProcess = null;
if (videoBlocks.length > 0) {
postProcess = that.processVideosAsync(videoBlocks, assetBlocks);
} else {
postProcess = new WinJS.Promise.wrap();
}
return postProcess.then(function () {
// Return combined results
that._sprinkleContent(article, contentBlocks, assetBlocks, {
haveMultimediaSpan: false,
initialContentBlocks: haveTitleImage ? 5 : 0
});
return WinJS.Promise.wrap(article);
}, function (error) {
// Return combined results
that._sprinkleContent(article, contentBlocks, assetBlocks, {
haveMultimediaSpan: false,
initialContentBlocks: haveTitleImage ? 5 : 0
});
return WinJS.Promise.wrap(article);
});
});
},
getCacheId: function getCacheId() {
return "PlatformImageCache";
},
processArticleBody: function processArticleBody(articleBody, articleInfo, title) {
var haveTitleImage = false,
contentBlocks = [],
assetBlocks = [],
videoBlocks = [],
blockNodes = null,
that = this;
if (articleBody) {
blockNodes = articleBody.selectNodes("./*");
} else {
return null;
}
for (var i = 0; i < blockNodes.length; i++) {
var blockNode = blockNodes.getAt(i);
var block = null;
if (blockNode.tagName === "p" ||
blockNode.tagName === "tagline") {
block = that._processContentBlock(blockNode);
contentBlocks.push(block);
} else if (blockNode.tagName === "media") {
var mediaType = blockNode.attributes.getNamedItem("type").nodeValue;
if (mediaType === "ILLUSTRATION") {
if (!haveTitleImage) {
title.titleImage = that._processTitleImageBlock(blockNode, mediaType);
if (title.titleImage) {
title.style = 2;
haveTitleImage = true;
}
}
if (!title.titleImage) {
block = that._processImageBlock(blockNode, mediaType);
}
} else if (mediaType === "VIDEO") {
block = that._processVideoBlock(blockNode);
if (block) {
videoBlocks.push(block);
}
} else if (mediaType === "FLASH") {
block = that._processFlash(blockNode, articleInfo, null);
}
if (block) {
assetBlocks.push(block);
}
} else if (blockNode.tagName === "inset") {
block = that._processInset(blockNode, articleInfo);
if (block) {
assetBlocks.push(block);
}
}
}
return {
haveTitleImage: haveTitleImage,
contentBlocks: contentBlocks,
assetBlocks: assetBlocks,
videoBlocks: videoBlocks
};
}
},
{
transform: null
}),
});
WinJS.Namespace.define("WSJJS.ArticleReader", {
WSJArticleActionsHandler: WinJS.Class.define(
function () {
},
{
onActionInvoked: function onActionInvoked(event) {
var data = event.data;
var actionKey = data.actionKey;
switch (actionKey) {
case "defaultAction":
// do nothing
break;
case "subscribeAction":
WSJJS.Auth.instance.signIn();
break;
case "downloadAction":
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(data.impressionNavMethod);
WSJJS.Instrumentation.instance.instrumentAppLaunchFromArticle(data.articleMetadata.articleId);
NewsJS.BasePartnerPano.launchPartnerApp(partner);
break;
}
},
}),
});
WinJS.Namespace.define("WSJJS.SlideShow", {
Page: WinJS.Class.derive(NewsJS.Partners.SlideShowPage, function wsj_slideshowpage_ctor(state) {
this.partner = partner;
NewsJS.Partners.SlideShowPage.call(this, state);
var articleId = state.providerConfiguration.articleId;
var articleType = state.parentArticle.type;
var sectionName = state.parentArticle.feedIdentifierValue;
var headline = state.parentArticle.caption;
WSJJS.Instrumentation.instance.instrumentSlideShowView(articleId, articleType, sectionName, headline);
},
{
}),
Provider: WinJS.Class.define(function (articleData) {
this.articleId = articleData.articleId;
this._imagePrefix = NewsJS.Partners.Config.getConfig(partner, "ImagePrefix", "http://s.wsj.net/public/resources/images/");
},
{
_imagePrefix: null,
articleId: null,
initializeAsync: function initializeAsync(config) {
return WinJS.Promise.wrap({});
},
queryAsync: function queryAsync() {
var that = this,
promise = WSJJS.ArticleManager.instance.getArticleAsync("section", "section", that.articleId);
return promise.then(function (articleString) {
var articleData = new Windows.Data.Xml.Dom.XmlDocument();
articleData.loadXml(articleString);
var summary = articleData.selectSingleNode("/article-doc/summary/summary-body");
articleData = articleData.selectSingleNode("/article-doc/article/article-body");
var headline = summary.selectSingleNode("./headline/main-hed");
var articleSummary = summary.selectSingleNode("./p");
articleSummary = articleSummary ? articleSummary.innerText : "";
headline = headline ? headline.innerText : "";
var insets = articleData.selectNodes("./inset");
var slides = [];
for (var i = 0; i < insets.length; i++) {
var inset = insets.getAt(i);
var media = inset.selectSingleNode("./media");
if (!media) {
continue;
}
var image = media.selectNodes("./alt-image[last()]|./image");
if (image.length) {
image = image[image.length - 1];
}
var credit = media.selectSingleNode("./media-credit");
credit = credit ? credit.innerText : "";
var caption = inset.selectSingleNode("./p");
caption = caption ? caption.innerText : "";
var src = image.attributes.getNamedItem("src-id");
src = src ? that._imagePrefix + src.nodeValue : null;
var height = image.attributes.getNamedItem("height");
height = height ? height.nodeValue : null;
var width = image.attributes.getNamedItem("width");
width = width ? width.nodeValue : null;
if (!src) {
continue;
}
slides.push({
thumbnail: {
url: src,
altText: caption
},
altText: caption,
attribution: credit,
caption: caption,
desc: caption,
image: {
url: src,
height: height,
width: width
},
title: ""
});
}
return WinJS.Promise.wrap({
title: headline,
desc: articleSummary,
byline: "",
version: 1,
slides: slides
});
});
}
}),
});
WinJS.Namespace.define("WSJJS", {
Instrumentation: WinJS.Class.define(function () {
var config = Platform.Configuration.ConfigurationManager.custom.getDictionary("WSJ").getDictionary("Instrumentation").getDictionary("Localytics");
// This will get the correct key depending on the context the code is running. The configuration build process will cause this key to have
// different value depending on whether it's the Bing News app or the NYT app.
var wsjKey = config.getString("APIKey");
var prod = Platform.Process.isProd();
if (!prod) {
wsjKey = config.getString("APIKey_appex");
}
if (wsjKey && wsjKey.length > 0) {
this._wsjLocalyticsSession = LocalyticsSession("WSJ");
var version = Windows.ApplicationModel.Package.current.id.version;
var buildVersion = version.major + "." + version.minor + "." + version.build + "." + version.revision;
this._wsjLocalyticsSession.init(wsjKey,
{
appVersion: buildVersion
});
}
},
{
_wsjLocalyticsSession: null,
_isSessionOpen: false,
openSession: function openSession() {
if (!this._isSessionOpen) {
this._wsjLocalyticsSession.open();
this._wsjLocalyticsSession.upload();
this._isSessionOpen = true;
}
},
closeSession: function closeSession() {
if (this._isSessionOpen) {
this._wsjLocalyticsSession.close();
this._wsjLocalyticsSession.upload();
this._isSessionOpen = false;
}
},
instrumentSectionView: function instrumentSectionView(appSection) {
var attributes = this._getCommonEventAttributes();
appSection = appSection ? appSection : "(Unknown)";
if (appSection.toUpperCase() === "FRONT SECTION") {
this._tagEvent("Front Page", attributes);
}
else {
attributes["Content Section"] = appSection;
attributes["Page Name"] = "summaries_" + appSection; // WSJ request both Content Section and Page Name to be present, even if the same value.
this._tagEvent("Summaries", attributes);
}
},
instrumentArticleView: function instrumentArticleView(url, articleId, originatingFeed, originatingSection, articleType, headline) {
var attributes = this._getCommonEventAttributes();
attributes["URL"] = url;
attributes["Page Name"] = "article_" + (articleType ? articleType : "Unknown") + "_" + articleId;
attributes["Content Section"] = originatingSection;
attributes["Referrer"] = originatingFeed;
attributes["Article Type"] = articleType ? articleType : "(Unknown)";
attributes["Headline"] = "article_" + (articleType ? articleType : "Unknown") + "_" + headline;
this._tagEvent("Article View", attributes);
},
instrumentSlideShowView: function instrumentSlideShowView(articleId, articleType, sectionName, headline) {
var attributes = this._getCommonEventAttributes();
attributes["URL"] = "http://online.wsj.com/article/" + articleId + ".html";
attributes["Content Section"] = sectionName;
attributes["Content Type"] = "slideshow";
attributes["Headline"] = "article_" + (articleType ? articleType : "Unknown") + "_" + headline;
attributes["Page Name"] = "slideshow_" + headline;
this._tagEvent("Multimedia Event", attributes);
},
instrumentSectionPinned: function instrumentSectionPinned(section) {
var attributes = this._getCommonEventAttributes();
attributes["Content Section"] = section;
this._tagEvent("Section Pinned", attributes);
},
instrumentArticleShared: function instrumentArticleShared(eventType, url, appSection) {
var attributes = this._getCommonEventAttributes();
attributes["URL"] = url;
attributes["Content Section"] = appSection;
this._tagEvent(eventType, attributes);
},
instrumentAppLaunchFromProtocol: function instrumentAppLaunchFromProtocol(referrer) {
var attributes = this._getCommonEventAttributes();
attributes["Referrer"] = referrer ? referrer : "(None)";
this._tagEvent("App Launch from URL", attributes);
},
instrumentAppLaunchFromArticle: function instrumentAppLaunchFromArticle(articleId) {
var attributes = this._getCommonEventAttributes();
attributes["ID"] = articleId;
this._tagEvent("App Launch from Article", attributes);
},
instrumentSubscriptionClicked: function instrumentSubscriptionClicked() {
var attributes = this._getCommonEventAttributes();
this._tagEvent("Subscription Tapped", attributes);
},
instrumentSignIn: function instrumentSignIn() {
var attributes = this._getCommonEventAttributes();
this._tagEvent("Login", attributes);
},
instrumentSignOut: function instrumentSignOut() {
var attributes = this._getCommonEventAttributes();
this._tagEvent("Logout", attributes);
},
instrumentManualRefresh: function instrumentManualRefresh() {
var attributes = this._getCommonEventAttributes();
this._tagEvent("Manual Refresh", attributes);
},
instrumentSnap: function instrumentSnap() {
var attributes = this._getCommonEventAttributes();
this._tagEvent("Snap", attributes);
},
instrumentError: function instrumentError(errorType) {
var attributes = {
"Error Type": errorType
};
this._tagEvent("Error", attributes);
},
_tagEvent: function _tagEvent(event, attributes) {
this._wsjLocalyticsSession.tagEvent(event, attributes);
},
_getReferringSource: function _getReferringSource() {
var backStack = WinJS.Navigation.history.backStack;
var forwardStack = WinJS.Navigation.history.forwardStack;
var referringPage = null;
if (forwardStack && forwardStack.length > 0) {
// This is the WSJ instrumentation. Since the user can hit the back button from any page, including other partners, we
// report this simply as 'Back Button'.
referringPage = "Back Button";
}
else if (backStack && backStack.length > 0) {
var backStackState = backStack[backStack.length - 1].state;
// Only report WSJ pages.
if (backStackState.theme === "WSJ") {
referringPage = backStackState.feedIdentifierValue;
}
}
return referringPage ? referringPage : "App";
},
_getCommonEventAttributes: function _getCommonEventAttributes() {
var view;
switch (Windows.UI.ViewManagement.ApplicationView.value) {
case Windows.UI.ViewManagement.ApplicationViewState.fullScreenLandscape:
view = "Full screen (landscape)";
break;
case Windows.UI.ViewManagement.ApplicationViewState.filled:
view = "Filled";
break;
case Windows.UI.ViewManagement.ApplicationViewState.snapped:
view = "Snapped";
break;
case Windows.UI.ViewManagement.ApplicationViewState.fullScreenPortrait:
view = "Full screen (portrait)";
break;
}
var attributes = {
"Subscriber Type": WSJJS.Auth.instance.isEntitled ? "Subscribed" : WSJJS.Auth.instance.isSignedIn ? "Registered" : "Anonymous",
"Network Status": Platform.Networking.NetworkManager.instance.isNetworkAvailable ? "Online" : "Offline",
"Network Cost": this._getNetworkCostString(),
"Network Connectivity Level": this._getNetworkConnectivityLevelString(),
"Resolution": screen.width + "x" + screen.height,
"View": view,
"Orientation": Windows.Graphics.Display.DisplayProperties.currentOrientation === Windows.Graphics.Display.DisplayOrientations.landscape ? "Landscape" : "Portrait",
"Edition": "US",
"UUID": WSJJS.Auth.instance.authToken ? WSJJS.Auth.instance.authToken : "Anonymous"
};
return attributes;
},
_getNetworkCostString: function _getNetworkCostString() {
switch (Platform.Networking.NetworkManager.instance.networkCostType) {
case 1:
return "Unrestricted";
case 2:
return "Fixed";
case 3:
return "Variable";
default:
return "Unknown";
}
},
_getNetworkConnectivityLevelString: function _getNetworkConnectivityLevelString() {
switch (Platform.Networking.NetworkManager.instance.networkConnectivityLevel) {
case 1:
return "Local Access";
case 2:
return "Constrained Internet Access";
case 3:
return "Internet Access";
default:
return "None";
}
},
},
{
_instance: null,
instance: {
get: function get() {
if (!WSJJS.Instrumentation._instance) {
WSJJS.Instrumentation._instance = new WSJJS.Instrumentation();
}
return WSJJS.Instrumentation._instance;
}
}
})
});
WinJS.Namespace.define("WSJJS.Ads", {
AdFormatter: WinJS.Class.derive(NewsJS.Partners.Ads.AdFormatter, function () {
var adsNode = NewsJS.Partners.Config.getConfig(partner, "Ads", {});
var sectionIds = adsNode.getDictionary("SiteIds");
var sections = Object.keys(sectionIds);
for (var i = 0; i < sections.length; i++) {
var section = sections[i];
var siteId = sectionIds[section];
if (siteId.value) {
this._siteIds[section] = siteId.value;
}
}
this._partnerId = adsNode.getString("PartnerId");
},
{
_siteIds: {},
_partnerId: null,
getKeywordString: function getKeywordString(options) {
var adTags = [];
if (options.location === "cluster") {
adTags.push({ key: "WSJsId", value: this._siteIds[options.section] });
} else if (options.location === "article") {
if (options.position === "interstitial") {
adTags.push({ key: "WSJsId", value: this._siteIds["int_" + options.orientation] });
} else if (options.position === "MiddleRight") {
adTags.push({ key: "WSJsId", value: this._siteIds["ArticleTower"] });
}
}
adTags.push({ key: "WSJpId", value: this._partnerId });
return adTags;
}
})
});
})();