﻿WinJS.Namespace.define('CommonJS', { Version: 'latest' });
/* File: ButtonContainer.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsButtonContainerInit() {
"use strict";
/// <summary>
/// Button container that contains simple buttons and an over-flow menu
/// </summary>
var NS = WinJS.Namespace.define("CommonJS", {
ButtonContainer: WinJS.Class.define(function buttonContainer_ctor(element, options) {
/// <summary>
/// constructor
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
this.element = element || document.createElement("div");
CommonJS.setAutomationId(this.element);
WinJS.Utilities.addClass(this.element, "platformActionBarItem");
this.element.winControl = this;
WinJS.UI.setOptions(this, options);
}, {
/// <field>
/// The DOM element container
/// </field>
element: null,
_buttons: null,
_menu: null,
_maxWidth: 0,
_addMenu: function _addMenu() {
/// <summary>
/// Adds the menu to the DOM
/// </summary>
/// <returns>
/// Nothing
/// </returns>
var container = document.getElementById("platformPageArea");
container.appendChild(this._menu.element);
},
_removeMenu: function _removeMenu() {
/// <summary>
/// Removes the menu to the DOM
/// </summary>
/// <returns>
/// Nothing
/// </returns>
if (this._menu) {
var container = document.getElementById("platformPageArea");
container.removeChild(this._menu.element);
this._menu = null;
}
},
_initButtonHandler: function _initButtonHandler(index) {
var that = this;
var button = new CommonJS.Button(CommonJS.createElement("button", this.element, "button" + index), this._buttons[index]);
button.element.className = "";
this._menu.element.appendChild(button.element);
button.element.addEventListener("onclick", function buttonContainer_onClick() {
that.menu.hide();
});
},
/// <field>
/// Buttons command list to populate the container with
/// </field>
commands: {
set: function set(value) {
msWriteProfilerMark("Platform:ButtonContainer:commands:s");
var i = 0,
j = 0,
button = null,
buttonWidth = 0,
currentWidth = 0,
lastButton = null,
that = this;
this._maxWidth = WinJS.Utilities.getTotalWidth(this.element);
this._buttons = value;
// Reset the container
WinJS.Utilities.empty(this.element);
this._removeMenu();
for (i = 0; i < this._buttons.length; i++) {
button = new CommonJS.Button(CommonJS.createElement("button", this.element, "button" + i, this._buttons[i].channelId), this._buttons[i]);
// Create the regular button
this.element.appendChild(button.element);
buttonWidth = WinJS.Utilities.getTotalWidth(button.element);
// Create the more button
if (currentWidth + buttonWidth > this._maxWidth) {
this.element.removeChild(button.element);
// Remove the last button to make room for the more button
if (lastButton) {
this.element.removeChild(lastButton.element);
}
this._menu = new WinJS.UI.Flyout(CommonJS.createElement("div", this.element, "moreFlyout"));
WinJS.Utilities.addClass(this._menu.element, "platformButtonOverflow");
for (j = i - 1; j < this._buttons.length; j++) {
this._initButtonHandler(j);
}
button = new CommonJS.Button(CommonJS.createElement("button", this.element, "moreButton"), {
title: CommonJS.resourceLoader.getString("/platform/More"), icon: "iconMore", onclick: function buttonContainer_onClickIconMore() {
that._menu.show(button.element);
button.selected = true;
}
});
this._menu.addEventListener("afterhide", function buttonContainer_onAfterHide() {
button.selected = false;
});
button.element.setAttribute("aria-haspopup", true);
this.element.appendChild(button.element);
this._addMenu();
break;
} else {
lastButton = button;
currentWidth += buttonWidth;
}
}
msWriteProfilerMark("Platform:ButtonContainer:commands:e");
}
}
})
});
})();
/* File: Button.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsButtonInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// A Button that can toggle, have mutliple states, show flyouts, show other action bars
/// </summary>
Button: WinJS.Class.define(function button_ctor(element, options) {
/// <summary>
/// constructor
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
// Create the button
this.element = element || document.createElement("button");
this.element.winControl = this;
CommonJS.setAutomationId(this.element);
WinJS.Utilities.addClass(this.element, "win-commandring win-commandicon roundUIBtnLight");
this.element.setAttribute("tabIndex", 0);
var that = this;
this.element.addEventListener("MSPointerDown", function button_onMSPointerDown(event) {
that._onpointerdown(event);
that._pointerDown = true;
});
this.element.addEventListener("MSPointerUp", function button_onMSPointerUp(event) {
that._onpointerup(event);
that._pointerDown = false;
if (WinJS.Utilities.eventWithinElement(that.element, event)) {
console.log("MSPointerUP event registered");
}
});
this.element.addEventListener("MSPointerCancel", function button_onMSPointerCancel(event) {
that._onpointerup(event);
});
this.element.addEventListener("blur", function button_onBlur(event) {
that._onpointerup(event);
});
this.element.addEventListener("deactivate", function button_onDeactivate(event) {
that._onpointerup(event);
});
this.element.addEventListener("MSLostPointerCapture", function button_onMSLostPointerCapture(event) {
that._onpointerup(event);
});
this.element.addEventListener("MSPointerOut", function button_onMSPointerOut(event) {
if (that._pointerDown) {
try {
that.element.msSetPointerCapture(event.pointerId);
}
catch (e) {
//someone else got capture before us. if this element is in semantic zoom, that's most likely what happened.
//see ui.js in winjs sdk for similar patterns of try/catching msSetPointerCapture
};
}
});
this.isToggle = false;
this.onclick = null;
this.mode = null;
this.selected = false;
this._currentState = null;
this._flyout = null;
this._actionBar = null;
this._flyoutShown = false;
WinJS.UI.setOptions(this, options);
},
{
/// <field>
/// The selected state: true or false
/// </field>
selected: false,
/// <field>
/// The placement of flyout to show when the button is clicked
/// </field>
flyoutPlacement: null,
/// <field>
/// The DOM element container
/// </field>
element: null,
/// <field>
/// Selection mode. Possible values are : CommonJS.Button.Toggle, CommonJS.Button.Dynamic, CommonJS.Button.Radio, CommonJS.Button.Simple
/// </field>
mode: null,
/// <field>
/// The image to use when the button is pressed
/// </field>
pressedIcon: null,
_states: {},
_currentState: null,
_flyout: null,
_actionBar: null,
_flyoutShown: false,
_iconRest: null,
_onpointerdown: function _onpointerdown(event) {
if (this.pressedIcon && this.pressedIcon.length > 0) {
this._setIcon(this.pressedIcon);
}
},
_onpointerup: function _onpointerup(event) {
this._setIcon(this.icon);
},
_getParentActionBar: function _getParentActionBar() {
/// <summary>
/// Returns the parent action bar
/// <returns>
/// The parent action bar if the button is contained within an appbar
/// </returns>
/// </summary>
var appBarDom = this.element.parentNode;
if (appBarDom) {
var appBar = appBarDom.winControl;
if (appBar &&
appBar.hide) {
return appBar;
}
}
},
/// <field>
/// Disabled
/// </field>
disabled: {
set: function set(value) {
if (value) {
this.element.disabled = "disabled";
} else {
this.element.removeAttribute("disabled");
}
}
},
/// <field>
/// change the css style that platform button is using
/// </field>
theme: {
set: function set(value) {
var ele = this.element;
if (value === "dark") {
WinJS.Utilities.removeClass(ele, "roundUIBtnLight");
WinJS.Utilities.addClass(ele, "roundUIBtnDark");
} else if (value === "light") {
WinJS.Utilities.removeClass(ele, "roundUIBtnDark");
WinJS.Utilities.addClass(ele, "roundUIBtnLight");
} else {
console.log("Incorrect platform button theme, possible values are: light|dark");
}
}
},
/// <field>
/// The icon for the button. May be a CSS class or a URL
/// </field>
icon: {
set: function set(value) {
this._iconRest = value;
this._setIcon(value);
},
get: function get() {
return this._iconRest;
}
},
_setIcon: function _setIcon(value) {
if (value &&
value.substring(0, 4) === "url(") {
console.error("image icon button is not supported, please use appexFont: " + value);
} else {
// Check to see if there is an icon available to use
if (WinJS.UI.AppBarIcon[value]) {
this.element.textContent = WinJS.UI.AppBarIcon[value];
} else {
//add css content to the element, which contains font symbols
WinJS.Utilities.addClass(this.element, value);
}
}
},
/// <field>
/// The title for the button
/// </field>
title: {
set: function set(value) {
this.element.setAttribute("aria-label", value);
}
},
_invokeButton: function _invokeButton(event, handler) {
var parentActionBar = null;
if (this.mode === CommonJS.Button.Toggle) {
this.selected = !this.selected;
}
if (this._flyout && !this._flyoutShown && this._flyout.hidden) {
this._flyoutShown = true;
this._flyout.show(this.element, this.flyoutPlacement || "top");
}
if (this._actionBar) {
parentActionBar = this._getParentActionBar();
if (parentActionBar) {
parentActionBar.hide();
}
this._actionBar.disabled = false;
this._actionBar.show();
}
if (handler) {
handler(event);
}
},
/// <field>
/// The onclick handler
/// </field>
onclick: {
set: function set(value) {
var that = this;
this.element.onclick = function (event) {
if (event.offsetX > that.element.offsetWidth ||
event.offsetY > that.element.offsetHeight ||
event.offsetX < 0 ||
event.offsetY < 0) {
return;
}
that._invokeButton(event, value);
};
this.element.onkeydown = function (event) {
switch (event.keyCode) {
case WinJS.Utilities.Key.enter:
case WinJS.Utilities.Key.space:
that._invokeButton(event, value);
break;
default:
break;
}
};
}
},
/// <field>
/// The possible states for the button. A state should be:
/// {
///    id: string,
///    title: string,
///    icon: string
/// }
/// </field>
states: {
set: function set(value) {
var i = 0;
this._states = {};
for (i = 0; i < value.length; i++) {
this._states[value[i].id] = value[i];
}
}
},
/// <field>
/// The current state id
/// </field>
currentState: {
set: function set(value) {
var state = this._states[value];
if (state) {
this._currentState = value;
if (state.icon) {
this.icon = state.icon;
}
if (state.iconRemove) {
WinJS.Utilities.removeClass(this.element, state.iconRemove);
}
if (state.iconAdd) {
WinJS.Utilities.addClass(this.element, state.iconAdd);
}
this.title = state.title;
if (!(typeof state.pressedIcon === "undefined")) {
this.pressedIcon = state.pressedIcon;
}
}
else {
PlatformJS.Utilities.onError("Invalid state id");
}
},
get: function get() {
return this._currentState;
}
},
/// <field>
/// The DOM id or DOM element for the flyout to show when the button is clicked
/// </field>
flyout: {
set: function set(value) {
var that = this;
this.element.setAttribute("aria-haspopup", true);
this._flyout = PlatformJS.Utilities.getControl(value);
if (this._flyout) {
this._flyout.addEventListener("afterhide", function button_onAfterHideFlyout() {
that._flyoutShown = false;
});
}
}
},
/// <field>
/// The DOM id or DOM element for the appbar to show when the button is clicked
/// </field>
actionBar: {
set: function set(value) {
var that = this;
this._actionBar = PlatformJS.Utilities.getControl(value);
if (this._actionBar) {
this._actionBar.addEventListener("afterhide", function button_onAfterHideActionBar() {
that._actionBar.disabled = true;
});
}
}
}
},
{
/// <field>
/// The selection mode where the button behaves as a toggle button
/// </field>
Toggle: "Toggle",
/// <field>
/// The selection mode where the button behaves as a radio button
/// </field>
Radio: "Radio",
/// <field>
/// The selection mode where the button can have multiple states
/// </field>
Dynamic: "Dynamic",
/// <field>
/// Just a regular button
/// </field>
Simple: "Simple"
}),
});
})();
/* File: MenuButton.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsMenuButtonInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS", {
/// <summary>
/// Button that shows a menu when clicked on
/// </summary>
MenuButton: WinJS.Class.define(function menuButton_ctor(element, options) {
/// <summary>
/// constructor
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
var that = this;
this.element = element || document.createElement("div");
CommonJS.setAutomationId(element);
this.element.setAttribute("aria-haspopup", true);
WinJS.Utilities.addClass(this.element, "platformActionBarItem");
this._button = new CommonJS.Button(CommonJS.createElement("button", this.element, "button"), {
onclick: function menuButton_onClick() {
that._button.selected = true;
if (!that._flyoutShown && that._flyout.hidden) {
that._flyoutShown = true;
that._flyout.show(that._button.element, "top");
}
},
mode: CommonJS.Button.Toggle
});
this.element.appendChild(this._button.element);
this._flyout = new WinJS.UI.Flyout(null, {});
this._flyout.addEventListener("afterhide", function menuButton_onAfterHide() {
that._flyoutShown = false;
that._button.selected = false;
});
CommonJS.setAutomationId(this._flyout.element, this.element, "flyout");
// Need to define the flyout outside of the edgeui. To make sure we do, just add it to the platformPageArea
WinJS.Utilities.addClass(this._flyout.element, "platformMenu win-menu");
this._flyout.element.setAttribute("role", "menu");
document.getElementById("platformPageArea").appendChild(this._flyout.element);
this.element.winControl = this;
this._flyoutShown = false;
this._currentState = null;
this._states = null;
this.selectionMode = CommonJS.MenuButton.Single;
WinJS.UI.setOptions(this, options);
},
{
/// <field>
/// The DOM element container
/// </field>
element: null,
/// <field>
/// Function to invoke when there is a state change
/// </field>
onStateChange: null,
/// <field>
/// Selection mode: "Multi" or "Single"
/// </field>
selectionMode: null,
_flyoutShown: false,
_currentState: null,
_button: null,
_flyout: null,
_states: null,
/// <field>
/// Disabled mode
/// </field>
disabled: {
set: function set(value) {
this._button.disabled = value;
}
},
/// <field>
/// The available states for the button. A state should be:
/// {
///    id: string,
///    title: string
/// }
/// </field>
states: {
set: function set(value) {
msWriteProfilerMark("Platform:MenuButton:states:s");
var i = 0,
state = null,
that = this;
this._states = {};
WinJS.Utilities.empty(that._flyout.element);
for (i = 0; i < value.length; i++) {
state = value[i];
if (typeof state.id === "undefined" || state.id === null) {
PlatformJS.Utilities.onError("ID must be included in a button state");
}
if (typeof state.title === "undefined" || state.title === null) {
PlatformJS.Utilities.onError("Title must be included in a button state");
}
this._states[state.id] = state;
this._initElements(state);
}
// update the check states if we haven't already
if (this._currentState) {
this.currentState = this._currentState;
}
msWriteProfilerMark("Platform:MenuButton:states:e");
}
},
_initElements: function _initElements(state) {
var s = null,
otherState = null,
that = this;
// Create the menu item
state.button = document.createElement("button");
CommonJS.setAutomationId(state.button, this.element, state.id);
WinJS.Utilities.addClass(state.button, "platformMenuItem");
state.button.setAttribute("role", "menuitem");
state.label = document.createElement("div");
state.label.textContent = state.title;
WinJS.Utilities.addClass(state.label, "platformMenuLabel");
state.button.appendChild(state.label);
state.check = document.createElement("div");
WinJS.Utilities.addClass(state.check, "platformMenuCheck");
state.button.appendChild(state.check);
state.button.onclick = function menuButton_stateButton_onclick(event) {
if (that.selectionMode === CommonJS.MenuButton.Menu) {
that._flyout.hide();
that._currentState = [state.id];
} else {
if (that.selectionMode === CommonJS.MenuButton.Single) {
// Uncheck the other button if single select mode
for (s in that._states) {
otherState = that._states[s];
if (otherState.isChecked) {
if (state !== otherState) {
otherState.isChecked = !otherState.isChecked;
WinJS.Utilities.toggleClass(otherState.check, "platformMenuLabelCheck");
}
break;
}
}
}
// Toggle the current button
state.isChecked = !state.isChecked;
WinJS.Utilities.toggleClass(state.check, "platformMenuLabelCheck");
that._currentState = [];
// Build up the state array
for (s in that._states) {
otherState = that._states[s];
if (otherState.isChecked) {
that._currentState.push(otherState.id);
}
}
}
if (that.onStateChange) {
that.onStateChange(that._currentState);
}
};
that._flyout.element.appendChild(state.button);
},
/// <field>
/// The array of currently selected state id's
/// </field>
currentState: {
set: function set(value) {
msWriteProfilerMark("Platform:MenuButton:currentState:s");
var state = null,
i = 0,
s = null;
this._currentState = value;
if (this.selectionMode !== CommonJS.MenuButton.Menu && this._states) {
for (s in this._states) {
state = this._states[s];
state.isChecked = false;
WinJS.Utilities.removeClass(state.check, "platformMenuLabelCheck");
}
for (i = 0; i < this._currentState.length; i++) {
state = this._states[this._currentState[i]];
if (state) {
state.isChecked = true;
WinJS.Utilities.addClass(state.check, "platformMenuLabelCheck");
} else {
PlatformJS.Utilities.onError("Invalid button state id");
}
}
}
msWriteProfilerMark("Platform:MenuButton:currentState:e");
},
get: function get() {
return this._currentState;
}
},
/// <field>
/// The title of the button
/// </field>
title: {
set: function set(value) {
if (value) {
this._button.title = value;
} else {
PlatformJS.Utilities.onError("Button title must be non-null");
}
}
},
/// <field>
/// The icon for the button. It may be a CSS class or a URL (e.g., url(‘http://my.domain.com/img.png’))
/// </field>
icon: {
set: function set(value) {
this._button.icon = value;
}
}
},
{
/// <field>
/// Multi select
/// </field>
Multi: "Multi",
/// <field>
/// Single select
/// </field>
Single: "Single",
/// <field>
/// Menu select
/// </field>
Menu: "Menu"
})
});
})();
/* File: SelectorButton.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsSelectorButtonInit() {
"use strict";
/// <summary>
/// Selector button is an AppBarCommand that toggles through an ordered list of items. Each item defines the button tooltip and icon.
/// </summary>
WinJS.Namespace.define("CommonJS", {
SelectorButton: WinJS.Class.mix(WinJS.Class.derive(WinJS.UI.AppBarCommand,
function selectorButton_ctor(elt, options) {
/// <summary>
/// Creates an instance of the SelectorButton control.
/// </summary>
/// <param name="elt">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
// call super constructor
WinJS.UI.AppBarCommand.call(this, elt, options);
var element = this._element;
// construct objects
var items = this._items = options.items;
for (var i = 0, len = items.length; i < len; i++) {
var item = items[i];
if (item.selected) {
this.selectedIndex = i;
break;
}
}
// attach to events
var clickListener = this._clickListener = this._onClick.bind(this);
element.addEventListener("click", clickListener);
// define events
Object.defineProperties(this, WinJS.Utilities.createEventProperties("selectionchanged"));
},
{
_items: null,
_selectedIndex: null,
_clickListener: null,
/// <field>
/// dispose function: remove onclick event listener
/// </field>
dispose: function dispose() {
this._element.removeEventListener("click", this._clickListener);
},
/// <field>
/// The index of selected item
/// </field>
selectedIndex: {
get: function get() {
return this._selectedIndex;
},
set: function set(value) {
var items = this._items;
var item = items[value];
if (item) {
this._selectedIndex = value;
var element = this._element;
element.querySelector(".win-commandimage").innerText = item.icon;
this.tooltip = item.label;
}
},
},
_onClick: function _onClick(event) {
var selectedIndex = this._selectedIndex;
var items = this._items;
var len = items.length;
var newSelectedIndex = this.selectedIndex = (selectedIndex + 1) % len;
this.dispatchEvent("selectionchanged", newSelectedIndex);
}
},
{
}), WinJS.Utilities.eventMixin),
});
})();
/* File: defaultSnappedHome.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformDefaultSnappedHomeInit () {
"use strict";
WinJS.Namespace.define("CommonJS", {
DefaultSnappedHome: WinJS.Class.define(
function defaultSnappedHome_ctor (state) {
},
{
getPageState: function getPageState() {
return null;
},
getPageData: function getPageData()
{
var that = this;
return WinJS.Promise.wrap({
title: CommonJS.resourceLoader.getString("SnappedAppTitle"),
message: CommonJS.resourceLoader.getString("/platform/SnappedMessage"),
actionMessage: CommonJS.resourceLoader.getString("/platform/Unsnap"),
image: "url('/CommonControls/Common/images/snap_placeholder.png')",
onclick: WinJS.Utilities.markSupportedForProcessing(function defaultUnSnapHandler() {
Windows.UI.ViewManagement.ApplicationView.tryUnsnap();
})
});
},
}
)
});
})();
/* File: PaginatedViewManager.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function () {
"use strict";
var leftArrowGlyph = "&#57570;";
var rightArrowGlyph = "&#57571;";
// PaginatedViewManager is a common helper class that owns the flip view style left and right buttons.
// It creates the UI for next/previous buttons, handle their mouse behavior(clicks and move) and animate them in and out.
// It exposes the following interface to its subclass: _getContainerElement(), _hasNextPage(), _hasPreviousPage(), _goNextPage() and _goPreviousPage().
WinJS.Namespace.define("CommonJS", {
PaginatedViewManager: WinJS.Class.define(
function (options) {
var eventListeners = this._eventListeners = [];
// page buttons
var elt = this._elt = this._getContainerElement();
var rtl = window.getComputedStyle(elt, null).direction === "rtl";
var nextPageButton = this._nextPageButton = document.createElement("button");
nextPageButton.style.visibility = "hidden";
nextPageButton.setAttribute("tabIndex", -1);
nextPageButton.setAttribute("aria-hidden", "true");
nextPageButton.innerHTML = rtl ? leftArrowGlyph : rightArrowGlyph;
WinJS.Utilities.addClass(nextPageButton, "win-navbutton");
WinJS.Utilities.addClass(nextPageButton, rtl ? "win-navleft" : "win-navright");
elt.appendChild(nextPageButton);
var previousPageButton = this._previousPageButton = document.createElement("button");
previousPageButton.style.visibility = "hidden";
previousPageButton.setAttribute("tabIndex", -1);
previousPageButton.setAttribute("aria-hidden", "true");
previousPageButton.innerHTML = rtl ? rightArrowGlyph : leftArrowGlyph;
WinJS.Utilities.addClass(previousPageButton, "win-navbutton");
WinJS.Utilities.addClass(previousPageButton, rtl ? "win-navright" : "win-navleft");
elt.appendChild(previousPageButton);
// events
this._attach(nextPageButton, "click", this._onNextPageClick);
this._attach(nextPageButton, "mouseover", this._onPageButtonMouseOver);
this._attach(nextPageButton, "mouseout", this._onPageButtonMouseOut);
this._attach(previousPageButton, "click", this._onPreviousPageClick);
this._attach(previousPageButton, "mouseover", this._onPageButtonMouseOver);
this._attach(previousPageButton, "mouseout", this._onPageButtonMouseOut);
this._buttonFadePromise = null;
this._nextPageButtonPromise = null;
this._previousPageButtonPromise = null;
this._buttonsVisible = false;
this._touch = false;
},
{
_elt: null,
_eventListeners: null,
_nextPageButton: null,
_previousPageButton: null,
_buttonFadePromise: null,
_buttonsVisible: null,
_nextPageButtonPromise: null,
_previousPageButtonPromise: null,
_pageButtonOnHover: null,
_touch: null,
dispose: function dispose() {
try {
var eventListeners = this._eventListeners;
for (var i = 0, len = eventListeners.length; i < len; i++) {
var eventListener = eventListeners[i];
var elt = eventListener.elt;
var name = eventListener.name;
var listener = eventListener.listener;
elt.removeEventListener(name, listener);
}
} catch (ex) {
//HACK: silence exceptions
}
var buttonFadePromise = this._buttonFadePromise;
if (buttonFadePromise) {
buttonFadePromise.cancel();
}
},
_attach: function _attach(elt, name, listener) {
var eventListeners = this._eventListeners;
var l = listener.bind(this);
elt.addEventListener(name, l);
eventListeners.push({
elt: elt,
name: name,
listener: l,
});
},
_pointerHoverListener: function _pointerHoverListener(event) {
this._detectTouch(event);
this._maybeShowButtons(event);
},
_pointerMoveListener: function _pointerMoveListener(event) {
this._detectTouch(event);
this._maybeShowButtons(event);
},
_pointerDownListener: function _pointerDownListener(event) {
this._detectTouch(event);
// block middle mouse button click
if ((event.buttons & 4) !== 0) {
event.stopPropagation();
event.preventDefault();
}
},
_detectTouch: function _detectTouch(event) {
if (event.pointerType === event.MSPOINTER_TYPE_TOUCH) {
this._fadeOutButtons(true);
this._touch = true;
}
else {
this._touch = false;
}
},
_maybeShowButtons: function _maybeShowButtons(event) {
if (event.pointerType !== event.MSPOINTER_TYPE_TOUCH) {
this._showButtons();
}
},
_maybeUpdateButtons: function _maybeUpdateButtons(event) {
var buttonsVisible = this._buttonsVisible;
if (buttonsVisible) {
this._showButtons();
}
},
_showButtons: function _showButtons() {
this._fadeInButtons();
this._fadeOutButtons(false);
},
_fadeInButtons: function _fadeInButtons() {
var hasNextPage = this._hasNext();
this._animateButton("next", hasNextPage);
var hasPreviousPage = this._hasPrevious();
this._animateButton("previous", hasPreviousPage);
this._buttonsVisible = true;
},
_fadeOutButtons: function _fadeOutButtons(immediate) {
var that = this;
var buttonFadePromise = this._buttonFadePromise;
if (buttonFadePromise) {
buttonFadePromise.cancel();
this._buttonFadePromise = null;
}
var pageButtonOnHover = this._pageButtonOnHover;
if (!pageButtonOnHover || (pageButtonOnHover === "next" && !this._hasNext())
|| (pageButtonOnHover === "prev" && !this._hasPrevious())) {
this._buttonFadePromise = WinJS.Promise.timeout(immediate ? 0 : 3000)
.then(function () {
that._animateButton("next", false);
that._animateButton("previous", false);
that._buttonFadePromise = null;
that._buttonsVisible = false;
});
}
},
_animateButton: function _animateButton(button, visible) {
var elt;
var promise;
switch (button) {
case "next":
elt = this._nextPageButton;
promise = this._nextPageButtonPromise;
break;
case "previous":
elt = this._previousPageButton;
promise = this._previousPageButtonPromise;
break;
}
if (elt) {
if (promise) {
promise.cancel();
}
if (visible) {
elt.style.visibility = "visible";
promise = WinJS.UI.executeTransition(
elt,
{
property: "opacity",
delay: 0,
duration: 167,
timing: "linear",
to: 1
});
}
else {
promise = WinJS.UI.Animation.fadeOut(elt)
.then(function () {
elt.style.visibility = "hidden";
});
}
}
switch (button) {
case "next":
this._nextPageButtonPromise = promise;
break;
case "previous":
this._previousPageButtonPromise = promise;
break;
}
},
_stopPropagation: function _stopPropagation(event) {
event.stopPropagation();
},
_onPageButtonMouseOver: function _onPageButtonMouseOver(event) {
switch (event.target) {
case this._nextPageButton:
this._pageButtonOnHover = "next";
break;
case this._previousPageButton:
this._pageButtonOnHover = "prev";
break;
}
},
_onPageButtonMouseOut: function _onPageButtonMouseOut(event) {
this._pageButtonOnHover = null;
},
_manipulationListener: function _manipulationListener(event) {
this._touch = true;
},
_getContainerElement: function _getContainerElement() {
throw "_getContainerElement() needs to be implemented by the subclass";
},
_hasNext: function _hasNext() {
throw "_hasNext() needs to be implemented by the subclass";
},
_hasPrevious: function _hasPrevious() {
throw "_hasPrevious() needs to be implemented by the subclass";
},
_onNextPageClick: function _onNextPageClick(event) {
throw "_onNextPageClick() needs to be implemented by the subclass";
},
_onPreviousPageClick: function _onPreviousPageClick(event) {
throw "_onPreviousPageClick() needs to be implemented by the subclass";
},
}),
});
})();
/* File: ui.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
/// <reference path="services.js" />
/// <dictionary target='parameter'>frag,obj</dictionary>
(function appexPlatformUIInit() {
"use strict";
WinJS.Namespace.define("CommonJS",
{
getAppName: function getAppName() {
/// <summary>Get the app name</summary>
var appName = "";
try {
appName = CommonJS.resourceLoader.getString("AppTitle");
} catch (e) {
appName = CommonJS.resourceLoader.getString("/platform/AppTitle");
}
return appName;
},
_elementIds: 0,
_previousTheme: "",
setTheme: function setTheme(className, hasDarkBackground) {
/// <summary>Adds the className to document.body for theming. Removes previous theme</summary>
/// <param name="className">CSS class to apply</param>
/// <param name="hasDarkBackground">Theme has a dark background</param>
msWriteProfilerMark("Platform:SetTheme:s");
WinJS.Utilities.removeClass(document.body, CommonJS._previousTheme);
if (className) {
WinJS.Utilities.addClass(document.body, className);
CommonJS._previousTheme = className;
}
if (hasDarkBackground) {
WinJS.Utilities.addClass(document.body, "platformDarkBackground");
} else {
WinJS.Utilities.removeClass(document.body, "platformDarkBackground");
}
msWriteProfilerMark("Platform:SetTheme:e");
},
setAutomationId: function setAutomationId(domElement, parent, suffix, id) {
/// <summary> Set the automation id of an element </summary>
/// <param name="domElement">The dom element to the id for</param>
/// <param name="parent">The parent element whose id to use for the prefix</param>
/// <param name="suffix">The suffix to use for the id</param>
/// <param name="id">The raw id to use (if present, overrides the parent and suffix parameters)</param>
if (id && id !== "") {
domElement.id = id;
}
else if (!domElement.id && !parent && !suffix) {
CommonJS._elementIds++;
domElement.id = "platformUAId" + CommonJS._elementIds;
} else if (parent && suffix) {
domElement.id = parent.id + "_" + suffix;
}
},
createElement: function createElement(elementType, parent, suffix, id) {
/// <summary> Creates an element and sets the automation id of an element </summary>
/// <param name="elementType">The element type</param>
/// <param name="parent">The parent element whose id to use for the prefix</param>
/// <param name="suffix">The suffix to use for the id</param>
/// <param name="id">The raw id to use (if present, overrides the parent and suffix parameters)</param>
var e = document.createElement(elementType);
CommonJS.setAutomationId(e, parent, suffix, id);
return e;
},
/// <field>Cached data placeholder</field>
cacheDataPlaceHolder: "cacheDataPlaceholder",
disableAllEdgies: function disableAllEdgies(disabled) {
/// <summary>
/// Disables all edgies
/// </summary>
/// <returns>
/// Nothing
/// </returns>
this._forEachEdgy(function platformUI_foreachEdgyDisable(edgy) {
if (edgy) {
edgy.disabled = disabled;
}
});
},
dismissAllEdgies: function dismissAllEdgies() {
/// <summary>
/// Dismiss all edgies
/// </summary>
/// <returns>
/// Nothing
/// </returns>
this._forEachEdgy(function platformUI_foreachEdgyHide(edgy) {
if (edgy && edgy.hide) {
edgy.hide();
}
});
},
_forEachEdgy: function _forEachEdgy(f) {
/// <summary>
/// Call a function on each edgy
/// </summary>
/// <returns>
/// Nothing
/// </returns>
var appbarDoms = WinJS.Utilities.query(".win-appbar");
for (var i = 0; i < appbarDoms.length; i++) {
f(appbarDoms[i].winControl);
}
},
/// <field>
/// Fragment cache
/// </field>
_fragmentCache: new PlatformJS.Utilities.MemoryCache(2),
/// <field>
/// Template cache
/// </field>
_templateCache: new PlatformJS.Utilities.MemoryCache(20),
cellSizeInPixels: 100,
setModuleSizeAndClass: function setModuleSizeAndClass(moduleInfo, div, gridLayout) {
/// <summary>
/// Function to load size a module and apply class name
/// </summary>
/// <param name="moduleInfo">
/// An object that contains information about the module.
/// {
///   fragmenthPath: path of fragment,
///   templateId: DOM id of template,
///   width: # of cells wide,
///   height: # of cells high,
///   className: css class name to applied to module
/// }
/// </param>
/// <param name="div">
/// Container DOM element.
/// </param>
/// <param name="gridLayout">
/// Grid Layout structure that describes the layout of the grid (optional)
/// {
///     cellWidth: width of cells in pixels
///     cellHeight: height of cells in pixels
///     horizontalPadding: horizontal padding between cells in pixels
///     verticalPadding: vertical padding between cells in pixels
/// }
/// </param>
/// <returns>
/// Nothing
/// </returns>
if (moduleInfo.width &&
moduleInfo.height) {
var cellHeight = CommonJS.cellSizeInPixels,
cellWidth = CommonJS.cellSizeInPixels,
verticalSpacing = 0,
horizontalSpacing = 0,
width = 0,
height = 0;
if (gridLayout) {
if (gridLayout.horizontalSpacing < 0) {
PlatformJS.Utilities.onError("Horizontal spacing must be greater than or equal to 0");
}
if (gridLayout.horizontalSpacing) {
horizontalSpacing = gridLayout.horizontalSpacing;
}
if (gridLayout.horizontalSpacing < 0) {
PlatformJS.Utilities.onError("Vertical spacing must be greater than or equal to 0");
}
if (gridLayout.verticalSpacing) {
verticalSpacing = gridLayout.verticalSpacing;
}
if (gridLayout.cellHeight <= 0) {
PlatformJS.Utilities.onError("Cell height must be greater than 0");
}
if (gridLayout.cellHeight) {
cellHeight = gridLayout.cellHeight;
}
if (gridLayout.cellWidth <= 0) {
PlatformJS.Utilities.onError("Cell width must be greater than 0");
}
if (gridLayout.cellWidth) {
cellWidth = gridLayout.cellWidth;
}
}
if (isNaN(moduleInfo.width)) {
// Allow freeform values
width = moduleInfo.width;
} else {
if (moduleInfo.width <= 0) {
PlatformJS.Utilities.onError("ModuleInfo width must be greater than 0");
}
width = ((moduleInfo.width * cellWidth) + (moduleInfo.width - 1) * horizontalSpacing) + "px";
}
if (isNaN(moduleInfo.height)) {
// Allow freeform values
height = moduleInfo.height;
} else {
if (moduleInfo.height <= 0) {
PlatformJS.Utilities.onError("ModuleInfo height must be greater than 0");
}
height = ((moduleInfo.height * cellHeight) + (moduleInfo.height - 1) * verticalSpacing) + "px";
}
div.style.width = width;
div.style.height = height;
}
if (moduleInfo.className) {
WinJS.Utilities.addClass(div, moduleInfo.className);
}
},
_createPlaceholder: function _createPlaceholder(data, div) {
var placeHolder = document.createElement("div");
var placeHolderContainer = document.createElement("div");
WinJS.Utilities.addClass(placeHolderContainer, "platformModuleContainer platformPlaceHolderContainer " + data.placeHolder);
div.appendChild(placeHolderContainer);
CommonJS.classifyPlaceholder(div);
return placeHolderContainer;
},
classifyPlaceholder: function classifyPlaceholder(div, classification) {
if (!classification) {
var height = div.offsetHeight;
if (height > 250) {
classification = "large";
} else if (height > 150 && height <= 250) {
classification = "medium";
} else if (height <= 150 && height > 49) {
classification = "small";
} else {
classification = "tiny";
}
}
switch (classification) {
case "hidden":
WinJS.Utilities.addClass(div, "platformPlaceHolderHidden");
break;
case "tiny":
WinJS.Utilities.addClass(div, "platformPlaceHolderTiny");
break;
case "small":
WinJS.Utilities.addClass(div, "platformPlaceHolderSmall");
break;
case "large":
WinJS.Utilities.addClass(div, "platformPlaceHolderLarge");
break;
case "medium":
default:
WinJS.Utilities.addClass(div, "platformPlaceHolderMedium");
break;
}
},
_renderFragment: function _renderFragment(fragmentPath, data, div, item) {
/// <summary>
/// Function to render a fragment
/// </summary>
/// <param name="fragmentPath">
/// The path of the fragment to load
/// </param>
/// <param name="data">
/// Data context
/// </param>
/// <param name="div">
/// Container
/// </param>
/// <param name="item">
/// Item object returned by the ListView
/// </param>
/// <returns>
/// Returns a promise for the rendered item
/// </returns>
return new WinJS.Promise(function platformUI_renderFragmentPromiseInit(complete, error) {
var cover = null,
placeHolder = null;
// Check to see if we have a placeholder
if (data.placeHolder && data.placeHolder !== CommonJS.cacheDataPlaceHolder) {
placeHolder = CommonJS.document._createPlaceholder(data, div);
if (data.dataPromise) {
complete(div);
// Wait on the actual data in the meantime
data.dataPromise.then(function platformUI_dataPromiseComplete(actualData) {
var container = document.createElement(container);
WinJS.UI.Fragments.renderCopy(fragmentPath, container).then(function platformUI_fragmentWithPlaceholderRenderCopyComplete() {
var fragments = WinJS.Utilities.query(".fragment", container);
var elementToBind = fragments.length > 0 ? fragments[0] : container;
WinJS.UI.processAll(elementToBind).then(function platformUI_fragmentWithPlaceholderProcessAllComplete() {
WinJS.Resources.processAll(elementToBind).then(function platformUI_fragmentWithPlaceholderResourcesProcessAllComplete() {
WinJS.Binding.processAll(elementToBind, actualData).then(function platformUI_fragmentWithPlaceholderBindingProcessAll() {
WinJS.Utilities.addClass(container, "platformModuleContainer");
div.appendChild(container);
if (typeof item === "undefined" || item.isOnScreen()) {
//TODO: WinJS.UI.Animation has been deprecated by the Windows team
//      and we now need to figure out the plans.
//[BEGIN]
//var revealArgs = { top: container.clientHeight + "px" };
//if (window.getComputedStyle(placeHolder).direction === "rtl") {
//    revealArgs.left = "0px";
//} else {
//    revealArgs.right = "0px";
//}
div.removeChild(placeHolder);
//WinJS.UI.Animation.reveal(placeHolder, container, revealArgs)
//    .then(function () {
//        div.removeChild(placeHolder);
//    }, function () {
//        div.removeChild(placeHolder);
//    });
//[END]
}
else {
div.removeChild(placeHolder);
}
}, PlatformJS.Utilities.onError);
}, PlatformJS.Utilities.onError);
}, PlatformJS.Utilities.onError);
}, PlatformJS.Utilities.onError);
});
} else {
PlatformJS.Utilities.onError("No data promise provided");
}
// Check to see if we have cached data to show
} else if (data.placeHolder && data.placeHolder === CommonJS.cacheDataPlaceHolder) {
placeHolder = document.createElement("div");
cover = document.createElement("div");
WinJS.Utilities.addClass(cover, "platformCachedCover platformModuleContainer");
WinJS.Utilities.addClass(placeHolder, "platformModuleContainer");
div.appendChild(placeHolder);
placeHolder.appendChild(cover);
WinJS.UI.Fragments.renderCopy(fragmentPath, placeHolder).then(function platformUI_fragmentWithCacheRenderCopyComplete() {
var fragments = WinJS.Utilities.query(".fragment", placeHolder);
var elementToBind = fragments.length > 0 ? fragments[0] : placeHolder;
WinJS.UI.processAll(elementToBind).then(function platformUI_fragmentWithCacheProcessAllComplete() {
WinJS.Resources.processAll(elementToBind).then(function platformUI_fragmentWithCacheResourcesProcessAllComplete() {
WinJS.Binding.processAll(elementToBind, data).then(function platformUI_fragmentWithCacheBindingProcessAllComplete() {
complete(div);
if (data.dataPromise) {
// Wait on the actual data in the meantime
data.dataPromise.then(function platformUI_dataPromiseComplete(actualData) {
var container = document.createElement("div");
WinJS.UI.Fragments.renderCopy(fragmentPath, container).then(function platformUI_fragmentWithCacheRenderCopyToContainerComplete() {
var fragments = WinJS.Utilities.query(".fragment", container);
var elementToBind = fragments.length > 0 ? fragments[0] : container;
WinJS.Binding.processAll(elementToBind, actualData).then(function platformUI_fragmentWithCacheBindDataComplete() {
WinJS.Utilities.addClass(container, "platformModuleContainer");
container.style.zIndex = -1;
div.appendChild(container);
if (typeof item === "undefined" || item.isOnScreen()) {
WinJS.UI.Animation.crossFade(container, placeHolder).then(function platformUI_crossFadeComplete() {
div.removeChild(placeHolder);
});
} else {
div.removeChild(placeHolder);
}
}, PlatformJS.Utilities.onError);
}, PlatformJS.Utilities.onError);
});
} else {
PlatformJS.Utilities.onError("No data promise provided");
}
}, PlatformJS.Utilities.onError);
}, PlatformJS.Utilities.onError);
}, PlatformJS.Utilities.onError);
}, PlatformJS.Utilities.onError);
// Standard case where all data is available
} else {
WinJS.UI.Fragments.renderCopy(fragmentPath, div).then(function platformUI_normalFragmentRenderCopyComplete() {
var fragments = WinJS.Utilities.query(".fragment", div);
var elementToBind = fragments.length > 0 ? fragments[0] : div;
WinJS.UI.processAll(elementToBind).then(function platformUI_normalFragmentBindComplete() {
WinJS.Resources.processAll(elementToBind).then(function platformUI_normalFragmentResourcesProcessAllComplete() {
WinJS.Binding.processAll(elementToBind, data).then(function platformUI_normalFragmentBindDataComplete() {
complete(div);
},
PlatformJS.Utilities.onError);
},
PlatformJS.Utilities.onError);
},
PlatformJS.Utilities.onError);
}, PlatformJS.Utilities.onError);
}
});
},
_renderTemplate: function _renderTemplate(template, data, div, item) {
/// <summary>
/// Function to render a template
/// </summary>
/// <param name="template">
/// WinJS.Binding.Template
/// </param>
/// <param name="data">
/// Data context
/// </param>
/// <param name="div">
/// Container
/// </param>
/// <param name="item">
/// Item object returned by the ListView
/// </param>
/// <returns>
/// Returns a promise for the rendered item
/// </returns>
return new WinJS.Promise(function platformUI_renderTemplatePromiseInit(complete, error) {
var placeHolder = null,
cover = null;
// Check to see if we have a placeholder
if (data.placeHolder && data.placeHolder !== CommonJS.cacheDataPlaceHolder) {
placeHolder = CommonJS._createPlaceholder(data, div);
if (data.dataPromise) {
complete(div);
// Wait on the actual data in the meantime
data.dataPromise.then(function platformUI_actualDataPromiseComplete(actualData) {
template.render(actualData, null).then(function platformUI_renderActualDataComplete(container) {
WinJS.Utilities.addClass(container, "platformModuleContainer");
div.appendChild(container);
if (typeof item === "undefined" || item.isOnScreen()) {
//TODO: WinJS.UI.Animation has been deprecated by the Windows team
//      and we now need to figure out the plans.
//[BEGIN]
div.removeChild(placeHolder);
//WinJS.UI.Animation.reveal(placeHolder, container, { left: "0px", top: container.clientHeight + "px" }).then(function () {
//    div.removeChild(placeHolder);
//});
//[END]
}
else {
div.removeChild(placeHolder);
}
}, PlatformJS.Utilities.onError);
});
} else {
PlatformJS.Utilities.onError("No data promise provided");
}
// Check to see if we have cached data to show
} else if (data.placeHolder && data.placeHolder === CommonJS.cacheDataPlaceHolder) {
placeHolder = document.createElement("div");
cover = document.createElement("div");
WinJS.Utilities.addClass(cover, "platformCachedCover platformModuleContainer");
WinJS.Utilities.addClass(placeHolder, "platformModuleContainer");
div.appendChild(placeHolder);
placeHolder.appendChild(cover);
template.render(data, placeHolder).then(function platformUI_renderDataToPlaceholderComplete() {
complete(div);
if (data.dataPromise) {
// Wait on the actual data in the meantime
data.dataPromise.then(function platformUI_actualDataPromiseComplete(actualData) {
template.render(actualData, null).then(function platformUI_renderActualDataComplete(container) {
WinJS.Utilities.addClass(container, "platformModuleContainer");
container.style.zIndex = -1;
div.appendChild(container);
if (typeof item === "undefined" || item.isOnScreen()) {
WinJS.UI.Animation.crossFade(container, placeHolder).then(function platformUI_crossFadeComplete() {
div.removeChild(placeHolder);
});
} else {
div.removeChild(placeHolder);
}
}, PlatformJS.Utilities.onError);
});
} else {
PlatformJS.Utilities.onError("No data promise provided");
}
},
PlatformJS.Utilities.onError);
// Standard case where all data is available
} else {
template.render(data, div).then(function platformUI_normalRenderTemplateComplete(container) {
complete(div);
}, PlatformJS.Utilities.onError);
}
});
},
_getTemplateKey: function _getTemplateKey(moduleInfo) {
/// <summary>
/// Function to load size a module and apply class name
/// </summary>
/// <param name="moduleInfo">
/// An object that contains information about the module.
/// {
///   fragmenthPath: path of fragment,
///   templateId: DOM id of template,
///   width: # of cells wide,
///   height: # of cells high,
///   className: css class name to applied to module
/// }
/// </param>
/// <returns>
/// Returns a unique identifier for the module template
/// </returns>
if (moduleInfo.fragmentPath &&
moduleInfo.templateId) {
return moduleInfo.fragmentPath + "/" + moduleInfo.templateId;
} else if (moduleInfo.templateId) {
return WinJS.Navigation.location.fragment + "/" + moduleInfo.templateId;
} else {
return moduleInfo.fragmentPath;
}
},
loadModule: function loadModule(moduleInfo, data, div, gridLayout, item) {
/// <summary>
/// Function to load a module
/// </summary>
/// <param name="moduleInfo">
/// An object that contains information about the module.
/// {
///   fragmenthPath: path of fragment,
///   templateId: DOM id of template,
///   width: # of cells wide,
///   height: # of cells high,
///   className: css class name to applied to module
/// }
/// </param>
/// <param name="data">
/// Data to bind to template
/// </param>
/// <param name="div">
/// Container DOM element. Optional. If omitted, one will be created and
/// moduleInfo sizing and class will be applied to the created one.
/// </param>
/// <param name="gridLayout">
/// Grid Layout structure that describes the layout of the grid (optional)
/// {
///     cellWidth: width of cells in pixels
///     cellHeight: height of cells in pixels
///     horizontalPadding: horizontal padding between cells in pixels
///     verticalPadding: vertical padding between cells in pixels
/// }
/// </param>
/// <param name="item">
/// Item object returned by the ListView
/// </param>
/// <returns>
/// Promise containing the complete DOM element
/// </returns>
var uniqueMarker = moduleInfo.templateId;
msWriteProfilerMark("Platform:loadModule:" + uniqueMarker + ":s");
return new WinJS.Promise(function platformUI_loadModulePromiseInit(complete, error) {
// If the template is in external fragment
if (moduleInfo.fragmentPath && moduleInfo.templateId) {
// Create a container div if one has not been provided
if (!div) {
div = document.createElement("div");
CommonJS.setModuleSizeAndClass(moduleInfo, div, gridLayout);
}
var templatePromise = CommonJS._fetchModule(moduleInfo);
if (WinJS.Promise.is(templatePromise)) {
templatePromise.then(function platformUI_templatePromiseComplete(template) {
// Render the template
CommonJS._renderTemplate(template, data, div, item).then(function platformUI_renderTemplateFromPromiseComplete() {
msWriteProfilerMark("Platform:loadModule:" + uniqueMarker + ":e");
complete(div);
});
});
} else {
// Render the template
CommonJS._renderTemplate(templatePromise, data, div, item).then(function platformUI_renderTemplateComplete() {
msWriteProfilerMark("Platform:loadModule:" + uniqueMarker + ":e");
complete(div);
});
}
// If the template is an entire fragment
} else if (moduleInfo.fragmentPath) {
CommonJS.setModuleSizeAndClass(moduleInfo, div, gridLayout);
CommonJS._renderFragment(moduleInfo.fragmentPath, data, div, item).then(function platformUI_renderModuleFragmentPathComplete(div) {
complete(div);
msWriteProfilerMark("Platform:loadModule:" + uniqueMarker + ":e");
});
// If the template is already on the page
} else if (moduleInfo.templateId) {
var template = document.getElementById(moduleInfo.templateId);
if (template && template.winControl) {
CommonJS._renderTemplate(template.winControl, data, div, item).then(function platformUI_renderTemplateFromIdComplete() {
CommonJS._templateCache.addItem(CommonJS._getTemplateKey(moduleInfo), template);
msWriteProfilerMark("Platform:loadModule:" + uniqueMarker + ":e");
complete(div);
},
PlatformJS.Utilities.onError);
} else {
if (PlatformJS.isDebug) {
PlatformJS.Utilities.onError("Unable to find template. Check templateId");
} else {
complete(div);
}
}
} else {
PlatformJS.Utilities.onError("Invalid id and fragment path");
}
});
},
_lastFragmentPath: null,
_lastFragment: null,
_fetchModule: function _fetchModule(moduleInfo) {
/// <summary>
/// Function to fetch a module
/// </summary>
/// <param name="moduleInfo">
/// An object that contains information about the module.
/// {
///   fragmenthPath: path of fragment,
///   templateId: DOM id of template,
///   width: # of cells wide,
///   height: # of cells high,
///   className: css class name to applied to module
/// }
/// </param>
/// <returns>
/// Promise containing the template
/// </returns>
var key = CommonJS._getTemplateKey(moduleInfo),
template = CommonJS._templateCache.getItem(key),
promise = null;
if (template) {
return template;
} else {
promise = new WinJS.Promise(function platformUI_fetchModulePromiseInit(complete, error) {
var fragPromise = null;
// Avoid making another copy of the fragment if the last fragment is available
if (moduleInfo.fragmentPath === CommonJS._lastFragmentPath) {
fragPromise = WinJS.Promise.wrap(CommonJS._lastFragment);
} else {
// We don't need to explicitly call cache to cache the fragment. RenderCopy will do
// that automatically
fragPromise = WinJS.UI.Fragments.renderCopy(moduleInfo.fragmentPath);
}
fragPromise.then(function platformUI_moduleFragmentComplete(frag) {
CommonJS._lastFragment = frag;
CommonJS._lastFragmentPath = moduleInfo.fragmentPath;
var collection = WinJS.Utilities.query("#" + moduleInfo.templateId, frag);
if (collection.length > 0) {
// Find the template
var template = collection[0];
WinJS.UI.processAll(template).then(function platformUI_templateProcessAllComplete() {
WinJS.Resources.processAll(template).then(function platformUI_templateResourcesProcessAllComplete() {
complete(template.winControl);
},
PlatformJS.Utilities.onError);
},
PlatformJS.Utilities.onError);
} else {
PlatformJS.Utilities.onError("Unable to find template");
}
},
PlatformJS.Utilities.onError);
});
// Add the promise to the cache
CommonJS._templateCache.addItem(key, promise);
promise.then(function platformUI_fetchModuleComplete(template) {
// Replace the promise with the real thing
CommonJS._templateCache.addItem(key, template);
},
PlatformJS.Utilities.onError);
return promise;
}
},
/// <summary>
/// A WinJS control to load a module for its UX
/// </summary>
/// <property name="moduleInfo">
/// An object that contains information about the module.
/// {
///   fragmenthPath: path of fragment,
///   templateId: DOM id of template,
///   width: # of cells wide,
///   height: # of cells high,
///   className: css class name to applied to module
/// }
/// </property>
/// <property name="data">
/// Data to bind to template
/// </property>
ModuleControl: WinJS.Class.define(function moduleControl_ctor(domElement, options) {
// Constructor
this._domElement = domElement ? domElement : document.createElement("div");
this._moduleInfo = null;
this._data = null;
this._loadMode = CommonJS.ModuleControl.LoadOnData;
this._domElement.winControl = this;
this.oneTimeBinding = false;
// Set the options
WinJS.UI.setOptions(this, options);
},
{
// PRIVATE MEMBERS
_domElement: null,
_moduleInfo: null,
_data: null,
_loadMode: null,
oneTimeBinding: false,
// PUBLIC PROPERTIES
/// <summary>The render promise</summary>
renderPromise: null,
/// <summary>Module Info property</summary>
moduleInfo: {
get: function get() {
return this._moduleInfo;
},
set: function set(value) {
this._moduleInfo = value;
// Apply the size and css class
if (this._moduleInfo) {
CommonJS.setModuleSizeAndClass(this._moduleInfo, this._domElement);
// By default, the load mode should be to wait on data
this._loadMode = this._moduleInfo.loadMode ? this._moduleInfo.loadMode : CommonJS.ModuleControl.LoadOnData;
}
// Load the module if we have both the data and the module info
this._loadModule();
}
},
// Data for template property
data: {
get: function get() {
return this._data;
},
set: function set(value) {
this._data = value;
this._loadModule();
}
},
_loadModule: function _loadModule() {
/// <summary>
/// Only loads the module when all the necessary components are available. ModuleInfo must be available.
/// Either data or fragmentOptions should be present depending on the load behavior
/// </summary>
/// <returns>
/// nothing
/// </returns>
// Load the module  depending on
if (this._moduleInfo) {
var that = this;
if ((this._loadMode === CommonJS.ModuleControl.LoadOnData && this._data) ||
(this._loadMode === CommonJS.ModuleControl.LoadImmediate)) {
this.renderPromise = CommonJS.loadModule(this._moduleInfo, this._data, this._domElement, null, this._fragmentOptions);
this.renderPromise.then(function moduleControl_loadModuleRenderPromiseComplete() {
if (that.oneTimeBinding) {
var elements = WinJS.Utilities.query("[data-win-bind]", that._domElement),
element = null,
i = 0,
len = elements.length;
for (i = 0; i < len; i++) {
element = elements[i];
element.removeAttribute("data-win-bind");
}
}
}, PlatformJS.Utilities.onError);
}
}
}
},
{
/// <summary>
/// Load the module when data is available
/// </summary>
LoadOnData: "loadOnData",
/// <summary>
/// Load the module when moduleInfo is ready
/// </summary>
LoadImmediate: "loadImmediate"
}),
forceRefresh: function forceRefresh() {
/// <summary>Forces refresh of the current page</summary>
/// <returns>Nothing</returns>
PlatformJS.Navigation.mainNavigator.populatePage(true);
},
VirtualizedDataSource: WinJS.Class.derive(WinJS.UI.VirtualizedDataSource, function virtualizedDataSource_ctor(listDataAdapter) {
/// <summary>Virtualized data source implementation</summary>
/// <param name="listDataAdapter">The List data adapter</param>
this._baseDataSourceConstructor(listDataAdapter);
})
});
// Network Indicator icon
WinJS.Namespace.define("CommonJS.NetworkIndicator", {
_indicator: null,
statusChange: function statusChange(event) {
/// <summary> Network status change event handler </summary>
/// <param name="event">The event</param>
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
CommonJS.NetworkIndicator.show();
} else {
CommonJS.NetworkIndicator.hide();
}
},
show: function show() {
/// <summary> Show the no internet icon </summary>
var platformNetworkIndicator = CommonJS.NetworkIndicator;
if (!platformNetworkIndicator._indicator) {
var networkIndicator = document.createElement("div");
var offlineText = document.createElement("div");
WinJS.Utilities.addClass(offlineText, "platformNetworkIndicatorText");
offlineText.innerText = CommonJS.resourceLoader.getString("/Platform/offline_label");
networkIndicator.appendChild(offlineText);
var offlineIcon = document.createElement("div");
WinJS.Utilities.addClass(offlineIcon, "platformNetworkIndicatorIcon");
networkIndicator.appendChild(offlineIcon);
var offlineIconCross = document.createElement("div");
WinJS.Utilities.addClass(offlineIconCross, "platformNetworkIndicatorIconCross");
networkIndicator.appendChild(offlineIconCross);
document.body.appendChild(networkIndicator);
WinJS.Utilities.addClass(networkIndicator, "platformNetworkIndicator platformNetworkIndicatorHide");
CommonJS.NetworkIndicator._indicator = networkIndicator;
// Need to wait for the DOM to be initialized
setTimeout(CommonJS.NetworkIndicator.statusChange, 500);
} else {
WinJS.Utilities.removeClass(platformNetworkIndicator._indicator, "platformNetworkIndicatorHide");
}
},
hide: function hide() {
/// <summary> Hide the no internet icon </summary>
if (CommonJS.NetworkIndicator._indicator) {
WinJS.Utilities.addClass(CommonJS.NetworkIndicator._indicator, "platformNetworkIndicatorHide");
}
},
showNetworkMessage: function showNetworkMessage(resourceString) {
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
var appName = CommonJS.getAppName();
var networkMessage = PlatformJS.Services.resourceLoader.getString(resourceString).format(appName);
var messageBar = new CommonJS.MessageBar(networkMessage);
messageBar.addButton(PlatformJS.Services.resourceLoader.getString("/platform/okButton"),
function platformNetworkIndicator_onClose() {
messageBar.hide();
localSettings.values["networkStateDismissed"] = true;
});
messageBar.show();
localSettings.values["networkStateDismissed"] = false;
localSettings.values["lastNetworkStateNotificationString"] = resourceString;
},
navigated: function navigated() {
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
var dismissed = localSettings.values["networkStateDismissed"];
if (dismissed === false) {
CommonJS.NetworkIndicator.showNetworkMessage(localSettings.values.lookup("lastNetworkStateNotificationString"));
}
},
checkNetworkState: function checkNetworkState() {
/// <summary>Check and see if we should show the network state message</summary>
var currentTime = (new Date()).getTime(),
overLimit = false,
roaming = false,
shouldCheck = true,
localSettings = null,
lastCheck = null;
var networkAvailable = Platform.Networking.NetworkManager.instance.isNetworkAvailable;
if (networkAvailable) {
overLimit = Platform.Networking.NetworkManager.instance.isOverDataLimit;
roaming = Platform.Networking.NetworkManager.instance.isRoaming;
localSettings = Windows.Storage.ApplicationData.current.localSettings;
if (overLimit || roaming) {
lastCheck = localSettings.values.lookup("lastNetworkStateNotification");
if (lastCheck !== null) {
shouldCheck = currentTime - lastCheck > PlatformJS.Services.configuration.getInt32("NetworkStateNotificationInterval", 30 * 60 * 1000);
}
if (shouldCheck) {
if (overLimit) {
CommonJS.NetworkIndicator.showNetworkMessage("/platform/overLimitMessage");
} else {
CommonJS.NetworkIndicator.showNetworkMessage("/platform/roamingMessage");
}
localSettings.values["lastNetworkStateNotification"] = currentTime;
}
}
else {
localSettings.values["networkStateDismissed"] = true;
}
}
}
});
// Subscribe the network status changed event
Platform.Networking.NetworkManager.addEventListener("networkstatuschanged", CommonJS.NetworkIndicator.statusChange);
// Subscribe the navigated event
WinJS.Navigation.addEventListener("navigated", CommonJS.NetworkIndicator.navigated);
WinJS.Namespace.define("CommonJS.Update", {
showForceUpdate: function showForceUpdate(storeUrl) {
/// <summary> Show force update message</summary>
/// <param name="storeUrl">store URL</param>
var appName = CommonJS.getAppName();
var updateButtonText = PlatformJS.Services.resourceLoader.getString("/platform/update_2");
var forceUpdate = new Windows.UI.Popups.MessageDialog(CommonJS.resourceLoader.getString("/platform/updateMessage").format(appName, updateButtonText));
forceUpdate.commands.append(new Windows.UI.Popups.UICommand(updateButtonText,
function platformUpdate_onForceUpdate(event) {
if (storeUrl) {
Windows.System.Launcher.launchUriAsync(new Windows.Foundation.Uri(storeUrl)).then();
}
}));
var showUpdate = function () {
// The dialog completes and dismisses when the user clicks any command
// To keep the dialog visible, we show it again when the dialog completes
// This forces the user to update the app if they want to continue using it
forceUpdate.showAsync().then(showUpdate, function platformUpdate_showUpdateError(error) {
// If the user cancels the dialog some how (by activating the search charm, for example)
// show the dialog again as well
if (error.name === "Canceled") {
showUpdate();
}
});
};
try {
showUpdate();
} catch (e) {
// "Access is denied." error may occur if a message dialog is already up 
};
}
});
WinJS.Namespace.define("CommonJS.Snapped", {
loadDefaultSnappedPage: function loadDefaultSnappedPage() {
/// <summary>Load the default snapped page content into the page area. Hide existing content.</summary>
var platformPageArea = document.getElementById("platformPageArea"),
child = null,
div = document.createElement("div"),
i = 0;
// Hide the existing content
for (i = 0; i < platformPageArea.children.length; i++) {
child = platformPageArea.children[i];
WinJS.Utilities.addClass(child, "platformFullContent");
WinJS.Utilities.addClass(child, "platformHide");
}
// Load the default snapped page
WinJS.UI.Fragments.render("/common/html/defaultSnappedHome.html", platformPageArea).then(function platformSnap_defaultSnappedHomeRenderComplete(frag) {
var snappedObj = new CommonJS.DefaultSnappedHome();
snappedObj.getPageData().then(function platformSnap_defaultSnappedHomeGetPageDataComplete(dataContext) {
var snappedContainer = WinJS.Utilities.query(".snappedContainer")[0];
WinJS.Binding.processAll(snappedContainer, dataContext).then();
});
});
},
unloadDefaultSnappedPage: function unloadDefaultSnappedPage() {
/// <summary>Unload the default snapped page from the page area. Unhide original content.</summary>
var platformPageArea = document.getElementById("platformPageArea"),
snappedContainers = WinJS.Utilities.query(".snappedContainer"),
snappedContainer = snappedContainers.length > 0 ? snappedContainers[0] : null,
platformFullContent = WinJS.Utilities.query(".platformFullContent"),
i = 0;
if (snappedContainer) {
// Remove the snapped page
platformPageArea.removeChild(snappedContainer);
// Unhide the original content
for (i = 0; i < platformFullContent.length; i++) {
WinJS.Utilities.removeClass(platformFullContent[i], "platformHide");
WinJS.Utilities.removeClass(platformFullContent[i], "platformFullContent");
}
}
}
});
WinJS.Namespace.define("CommonJS.Settings", {
showMarketNotification: function showMarketNotification() {
/// <summary>Show the market notification</summary>
var marketPickerEnabled = PlatformJS.Services.configuration.getBool("MarketPickerEnabled", true);
if (!marketPickerEnabled) {
return;
}
var isSet = Platform.Globalization.Marketization.isPersistedMarketValid(),
isValid = Platform.Globalization.Marketization.isCurrentLanguageLocationValid();
if (!isValid && !isSet) {
// Use the appropriate message
var settingsMessage = "";
var currentMarketRegion = Platform.Globalization.Marketization.getCurrentMarketInfo().geographicRegion;
if (currentMarketRegion) {
var currentRegionName = Platform.Globalization.MarketResources.getLocalizedRegionDisplayName(currentMarketRegion);
settingsMessage = PlatformJS.Services.resourceLoader.getString("/platform/marketSel_contentFrom").format(currentRegionName) +
" ";
}
var messageBar = new CommonJS.MessageBar(settingsMessage);
messageBar.addButton(PlatformJS.Services.resourceLoader.getString("/platform/changeSettings"),
function platformSettings_changeSettingsHandler() {
// set it anyway to make sure it only showup once.
Platform.Globalization.Marketization.setCurrentMarket(Platform.Globalization.Marketization.getCurrentMarket());
messageBar.hide();
PlatformJS.mainProcessManager.showSettings();
});
messageBar.addButton(PlatformJS.Services.resourceLoader.getString("/platform/noThanks"), function platformSettings_noThanks() {
Platform.Globalization.Marketization.setCurrentMarket(Platform.Globalization.Marketization.getCurrentMarket());
messageBar.hide();
});
messageBar.hideOnSnap = true;
messageBar.show();
}
},
getDefaultSettingsCommand: function getDefaultSettingsCommand() {
/// <summary> Get the default settings command to add to the search charm</summary>
return new Windows.UI.ApplicationSettings.SettingsCommand("SettingsPage", CommonJS.resourceLoader.getString("/platform/optionsTitle"), function platformSettings_defaultSettings() {
CommonJS.Settings.showDefaultSettingsFlyout();
});
},
addCommonButtons: function addCommonButtons(vector) {
var appSettings = Windows.UI.ApplicationSettings;
// Add app Feedback settings command to the settings charm
var feedbackCmd = new appSettings.SettingsCommand("FeedbackPage", CommonJS.resourceLoader.getString("/platform/feedbackButtonLabel"), CommonJS.Feedback.onFeedbackCmd);
vector.append(feedbackCmd);
// Add terms of use command settings command to the settings charm
var tocCmd = new appSettings.SettingsCommand("tocPage", CommonJS.resourceLoader.getString("/platform/lca_serviceAgreement"), CommonJS.Settings.onTocCmd);
vector.append(tocCmd);
// Add privacy settings command to the settings charm
var privacyCmd = new appSettings.SettingsCommand("privacyPage", CommonJS.resourceLoader.getString("/platform/lca_privacy"), CommonJS.Settings.onPrivacyCmd);
vector.append(privacyCmd);
// Add privacy settings command to the settings charm
if (PlatformJS.isDebug) {
var commonUXVersionCmd = new appSettings.SettingsCommand("CommonUXVersion", "Common UX Version: " + (CommonJS.Version || "latest"), function (ev) { });
vector.append(commonUXVersionCmd);
}
// Add help page command to the settings charm
var helpCmd = new appSettings.SettingsCommand("HelpPage", CommonJS.resourceLoader.getString("/platform/HelpLabel"), CommonJS.Settings.onHelpCmd);
vector.append(helpCmd);
// Add Impressum settings command to the settings charm
if (PlatformJS.Services.isConfigurationInitialized) {
var impLink = PlatformJS.Services.configuration.getDictionary("Setting").getString("ImpressumLink");
if (impLink) {
var impCmd = new appSettings.SettingsCommand("impPage", CommonJS.resourceLoader.getString("/platform/impressum"), CommonJS.Settings.onImpCmd);
vector.append(impCmd);
}
// Add the icpstring settings command to the settings charm
var icpString = PlatformJS.Services.configuration.getDictionary("Setting").getString("ICPNumber");
if (icpString) {
var ICPCmd = new appSettings.SettingsCommand("ICPPage", icpString, CommonJS.Settings.onICPCmd);
vector.append(ICPCmd);
}
}
},
onTocCmd: function onTocCmd(ev) {
var link = PlatformJS.Services.configuration.getDictionary("Setting").getString("TermsOfUseLink");
if (link) {
window.open(link);
}
},
onPrivacyCmd: function onPrivacyCmd(ev) {
var link = PlatformJS.Services.configuration.getDictionary("Setting").getString("PrivacyStatementLink");
if (link) {
window.open(link);
}
},
onHelpCmd: function onHelpCmd(evt) {
// Try to get help url link
var link = "";
if (PlatformJS.Services.appConfig) {
link = PlatformJS.Services.appConfig.getString("HelpURL");
}
// If we couldn't get help url link, 
// try to get help page link
if (!link) {
link = PlatformJS.Services.configuration.getDictionary("Setting").getString("HelpPageLink");
}
// Open link
if (link) {
try {
PlatformJS.Navigation.navigateTo(link);
} catch (e) {
console.warn("Failed to navigate to " + link + ", the exception: " + e);
}
} else {
console.warn("This app does not provide a help website URL");
}
},
getHelpButton: function getHelpButton() {
//help button
var helpButton = document.createElement("button");
var helpButtonCommand = new WinJS.UI.AppBarCommand(helpButton, {
icon: "\uFF60",
extraClass: "appexSymbol",
label: PlatformJS.Services.resourceLoader.getString("/platform/HelpLabel")
});
helpButton.id = "helpButton";
helpButtonCommand.onclick = WinJS.Utilities.markSupportedForProcessing(function () {
CommonJS.Settings.onHelpCmd();
});
return helpButton;
},
onImpCmd: function onImpCmd(ev) {
var link = PlatformJS.Services.configuration.getDictionary("Setting").getString("ImpressumLink");
if (link) {
window.open(link);
}
},
onICPCmd: function onICPCmd(ev) {
//The current plan is that we should not have this link so it would be an empty function
/*var link = PlatformJS.Services.configuration.getDictionary("Setting").getString("ICPNumberLink");
if (link) {
window.open(link);
}*/
},
getTermsOfUseUrl: function getTermsOfUseUrl() {
/// <summary> Get terms of use url</summary>
return PlatformJS.Services.configuration.getDictionary("Urls").getString("TermsOfUse");
},
getPrivacyUrl: function getPrivacyUrl() {
/// <summary> Get privacy url</summary>
return PlatformJS.Services.configuration.getDictionary("Urls").getString("PrivacyStatement");
},
showDefaultSettingsFlyout: function showDefaultSettingsFlyout() {
/// <summary> Show the default settings flyout</summary>
var page = PlatformJS.Utilities.getControl("platformSettingsPage");
if (page) {
page.show();
} else {
var container = null;
WinJS.UI.Fragments.renderCopy("/common/html/configurationFragment.html", document.body).then(function platformSettings_configurationFragmentRenderComplete(doc) {
container = document.getElementById("platformSettingsPage");
var title = PlatformJS.Services.resourceLoader.getString("/platform/optionsTitle");
var optionsBackbuttonclick = WinJS.Utilities.markSupportedForProcessing(WinJS.UI.SettingsFlyout.show);
var dataContext = {
pageTitle: title,
optionsBackbuttonclick: optionsBackbuttonclick
};
return WinJS.Binding.processAll(container, dataContext);
}).then(function platformSettings_configurationFragmentBindingComplete() {
return WinJS.UI.processAll(container);
}).done(function platformSettings_configurationFragmentProcessAllComplete() {
var page = container.winControl;
page.addEventListener("afterhide", function () {
PlatformJS.Navigation.mainNavigator.enableSearchOnType();
});
page.addEventListener("aftershow", function () {
PlatformJS.Navigation.mainNavigator.disableSearchOnType();
});
page.show();
});
}
}
});
})();
/* File: Miscellaneous.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
/// <dictionary target="comment">doms,edgeui,flyout,listview,multisize</dictionary>
/// <dictionary target="parameter">frag,obj</dictionary>
/// <dictionary target="variable">arg,args,Bool,Config,flyout,gids,obj,src</dictionary>
/// <dictionary target="member">Buton,Config,Flyout,Multisize,nav,onclick,onclosed,onselectionchanged,Renderer,wireup</dictionary>
(function appexPlatformControlsResourceInit() {
"use strict";
WinJS.Namespace.define("CommonJS",
{
_resourceLoader: null,
/// <summary>
/// Resource loader for controls
/// </summary>
resourceLoader: {
get: function get() {
if (!CommonJS._resourceLoader) {
CommonJS._resourceLoader = (PlatformJS.Services && PlatformJS.Services.resourceLoader) ? PlatformJS.Services.resourceLoader : new Platform.Resources.StringResourceLoader();
}
return CommonJS._resourceLoader;
}
},
reloadResourceLoader: function reloadResourceLoader() {
CommonJS._resourceLoader = (PlatformJS.Services && PlatformJS.Services.resourceLoader) ? PlatformJS.Services.resourceLoader : new Platform.Resources.StringResourceLoader();
},
resourceTester: {
tryGetString: function tryGetString(string) {
var result = { hasValue: false, value: null };
try {
var resourceString = string.indexOf("/") > -1 ? string : ("resources/" + string);
//best way to test existance of resourceString across languages according to win8 Modern Resource Team
result.value = Windows.ApplicationModel.Resources.Core.ResourceManager.current.mainResourceMap.lookup(resourceString).resolve().valueAsString;
result.hasValue = true;
}
catch (e) { }
return result;
},
}
});
})();
(function appexPlatformControlsFeedbackInit() {
"use strict";
WinJS.Namespace.define("CommonJS.Feedback",
{
_showSearchCharmOnKeyboardInput: true,
// "data" format:
// bdiRequestUrl: the GET request URL made to AppEx IPE servers
// rawResponse: the raw BDI response from AppEx IPE servers
// articleHeadline: the headline text of the article to be reported
// articleSnippet: the snippet text of the article to be reported
// articleUrl: the Bing URL of the article to be reported
// imageUrl: the URL of the image associated with the article to be reported (if available, otherwise null or empty)
showBdiReportForm: function showBdiReportForm(data) {
CommonJS.Feedback.addBdiReportFragment(data).then(function feedback_addBdiReportComplete(flyout) {
if (flyout) {
flyout.addEventListener("afterhide", function feedback_onAfterHideBdiReport(event) {
CommonJS.Feedback.removeBdiReportFragment();
});
flyout.show();
}
});
},
addBdiReportFragment: function addBdiReportFragment(data) {
var bdiReportDivId = "BdiReportFragment";
var bdiReportDiv = document.getElementById(bdiReportDivId);
if (!bdiReportDiv) {
bdiReportDiv = document.createElement("div");
bdiReportDiv.id = bdiReportDivId;
document.body.appendChild(bdiReportDiv);
}
var promise = new WinJS.Promise(function feedback_addBdiReportPagePromiseInit (complete) {
WinJS.UI.Fragments.renderCopy("/common/html/bdiReportFragment.html", bdiReportDiv).then(function feedback_renderCopyBdiReportFragmentComplete (doc) {
var categoryText = CommonJS.resourceLoader.getString("/platform/feedbackButtonLabel");
var submitLabel = CommonJS.resourceLoader.getString("/platform/feedbackSubmitButtonLabel");
var dataContext = {
categoryText: "What's wrong with this article?",
submitLabel: submitLabel,
data: data
};
WinJS.Binding.processAll(bdiReportDiv, dataContext);
WinJS.UI.processAll(bdiReportDiv);
WinJS.Resources.processAll(bdiReportDiv);
complete(PlatformJS.Utilities.getControl("SharedBdiReportPage"));
}, function feedback_fragmentRenderCopyError (error) {
complete(null);
});
});
return promise;
},
removeBdiReportFragment: function removeBdiReportFragment() {
var bdiReportDiv = document.getElementById("BdiReportFragment");
if (bdiReportDiv) {
document.body.removeChild(bdiReportDiv);
}
},
onFeedbackCmd: function onFeedbackCmd(ev) {
console.log("onFeedbackCmd");
CommonJS.Feedback.addFeedbackPageFragment().then(function feedback_addFeedbackPageComplete(feedbackFlyout) {
if (feedbackFlyout) {
feedbackFlyout.addEventListener("afterhide", function feedbackFlyout_onAfterHide(event) {
CommonJS.Feedback.removeFeedbackPageFragment();
});
feedbackFlyout.addEventListener("aftershow", function feedbackFlyout_onAfterShow(event) {
var userSuggest = document.getElementById("userSuggest");
if (userSuggest) {
userSuggest.focus();
}
});
feedbackFlyout.addEventListener("keydown", CommonJS.Feedback.onKeyDown);
var question1 = document.getElementById("question1");
var question2 = document.getElementById("question2");
if (question1) {
question1.selectedIndex = -1;
}
if (question2) {
question2.selectedIndex = -1;
}
feedbackFlyout.show();
}
});
},
removeFeedbackPageFragment: function removeFeedbackPageFragment() {
PlatformJS.Navigation.mainNavigator.enableSearchOnType();
var feedbackElement = document.getElementById("FeedbackPageDiv");
if (feedbackElement) {
document.body.removeChild(feedbackElement);
}
},
addFeedbackPageFragment: function addFeedbackPageFragment() {
PlatformJS.Navigation.mainNavigator.disableSearchOnType();
var feedbackElement = document.getElementById("FeedbackPageDiv");
if (!feedbackElement) {
feedbackElement = document.createElement("div");
feedbackElement.id = "FeedbackPageDiv";
document.body.appendChild(feedbackElement);
}
var promise = new WinJS.Promise(function feedback_addFeedbackPagePromiseInit(complete) {
WinJS.UI.Fragments.renderCopy("/common/html/feedbackFragment.html", feedbackElement).then(function feedback_renderCopyFragmentComplete(doc) {
var categoryText = CommonJS.resourceLoader.getString("/platform/feedbackButtonLabel");
var questionTitle = CommonJS.resourceLoader.getString("/platform/feedbackLikeCommentQuestion");
var autosavedComment = "";
var currentTime = new Date();
//restore autosaved comments not older than 30 minutes
if (CommonJS.feedbackAutosaveTime && (currentTime.getTime() - CommonJS.feedbackAutosaveTime) < 1000 * 60 * 30) {
if (CommonJS.feedbackAutosave) {
autosavedComment = CommonJS.feedbackAutosave;
CommonJS.feedbackAutosaveTime = currentTime.getTime(); //refresh timestamp to current when loading autosaved comments into feedback control
}
}
var privacyStatementText = CommonJS.resourceLoader.getString("/platform/feedbackPrivacyStatementLabel");
var privacyStatementUrl = "http://go.microsoft.com/fwlink/?LinkId=74170";
var submitLabel = CommonJS.resourceLoader.getString("/platform/feedbackSubmitButtonLabel");
var likeIcon = CommonJS.resourceLoader.getString("/platform/feedbackLikeIcon");
var likeLabel = CommonJS.resourceLoader.getString("/platform/feedbackLikeLabel");
var dislikeIcon = CommonJS.resourceLoader.getString("/platform/feedbackDislikeIcon");
var dislikeLabel = CommonJS.resourceLoader.getString("/platform/feedbackDislikeLabel");
var feedbackBackbuttonclick = WinJS.Utilities.markSupportedForProcessing(WinJS.UI.SettingsFlyout.show);
var buttonLikeclick = WinJS.Utilities.markSupportedForProcessing(CommonJS.Feedback.switchStatus.bind(null, 'like'));
var buttonDislikeclick = WinJS.Utilities.markSupportedForProcessing(CommonJS.Feedback.switchStatus.bind(null, 'dislike'));
var submitButtonclick = WinJS.Utilities.markSupportedForProcessing(CommonJS.Feedback.sendFeedback);
var dataContext = {
submitLabel: submitLabel,
likeIcon: likeIcon,
likeLabel: likeLabel,
dislikeIcon: dislikeIcon,
dislikeLabel: dislikeLabel,
categoryText: categoryText,
questionTitle: questionTitle,
autosavedComment: autosavedComment,
privacyStatementUrl: privacyStatementUrl,
privacyStatementText: privacyStatementText,
feedbackBackbuttonclick: feedbackBackbuttonclick,
buttonLikeclick: buttonLikeclick,
buttonDislikeclick: buttonDislikeclick,
submitButtonclick: submitButtonclick
};
//loop and add feedback questions and all 5 options of each question in dataContext
for (var qIndex = 1; qIndex <= 2; qIndex++) {
dataContext["question" + qIndex] = CommonJS.resourceLoader.getString("/platform/feedbackQuestion" + qIndex);
for (var i = 1; i <= 5; i++) {
dataContext["question" + qIndex + "option" + i] = CommonJS.resourceLoader.getString("/platform/feedbackQ" + qIndex + "option" + i);
}
}
WinJS.Binding.processAll(feedbackElement, dataContext);
WinJS.UI.processAll(feedbackElement);
WinJS.Resources.processAll(feedbackElement);
complete(PlatformJS.Utilities.getControl("SharedFeedbackPage"));
}, function feedback_fragmentRenderCopyError(error) {
complete(null);
});
});
return promise;
},
});
})();
(function appexPlatformControlsToggleGroupInit() {
"use strict";
/// <summary>
/// A toggle button group
/// </summary>
WinJS.Namespace.define("CommonJS",
{
ToggleGroup: WinJS.Class.define(
function toggleGroup_ctor(element, options) {
/// <summary>
/// constructor
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
this.element = element || document.createElement("div");
this.element.setAttribute("role", "radiogroup");
CommonJS.setAutomationId(this.element);
WinJS.Utilities.addClass(this.element, "platformActionBarItem");
this.element.winControl = this;
WinJS.UI.setOptions(this, options);
},
{
_states: {},
_currentState: null,
/// <field>
/// The DOM element container
/// </field>
element: null,
/// <field>
/// Function to invoke when there is a state change
/// </field>
onStateChange: null,
/// <field>
/// The current state id of the toggle group
/// </field>
currentState: {
set: function set(value) {
// Unset the previous button
if (this._currentState) {
this._states[this._currentState].button.selected = false;
}
// Set the current button
this._currentState = value;
if (this._states[this._currentState]) {
this._states[this._currentState].button.selected = true;
} else {
PlatformJS.Utilities.onError("Invalid button state id");
}
},
get: function get() {
return this._currentState;
}
},
/// <field>
/// The available states for the button group. A state should be:
/// {
///    id: string,
///    title: string,
///    icon: string
/// }
/// </field>
states: {
set: function set(value) {
msWriteProfilerMark("Platform:ToggleGroup:states:s");
var i = 0,
state = null,
that = this;
this._states = {};
for (i = 0; i < value.length; i++) {
state = value[i];
this._states[state.id] = state;
if (typeof state.id === "undefined" || state.id === null) {
PlatformJS.Utilities.onError("ID must be included in a button state");
}
(function toggleGroup_createStateChangeButton(state) {
state.button = new CommonJS.Button(CommonJS.createElement("button", that.element, state.id), {
title: state.title,
onclick: function onclick(event) {
if (that.currentState !== state.id) {
that.currentState = state.id;
if (that.onStateChange) {
that.onStateChange(that.currentState);
}
}
},
mode: CommonJS.Button.Radio
});
state.button.element.setAttribute("role", "radio");
that.element.appendChild(state.button.element);
})(state);
}
msWriteProfilerMark("Platform:ToggleGroup:states:e");
}
}
}),
});
})();
(function appexPlatformControlsFlyoutManagerInit(WinJS) {
"use strict";
var FlyoutManagerImpl = WinJS.Class.define(
function flyoutManagerImpl_ctor() {
var container = document.getElementById("platformFlyoutContainer");
if (!container) {
container = this._createContainer();
}
this._container = container;
WinJS.Navigation.addEventListener("beforenavigate", this._onBeforeNavigate.bind(this));
},
{
_container: null,
getFlyoutElement: function getFlyoutElement() {
var element = document.createElement("div");
this._container.appendChild(element);
return element;
},
_createContainer: function _createContainer() {
var container = document.createElement("div");
container.id = "platformFlyoutContainer";
var parent = document.getElementById("platformPageArea").parentElement;
parent.appendChild(container);
return container;
},
_onBeforeNavigate: function _onBeforeNavigate() {
var container = this._container;
WinJS.Utilities.empty(container);
},
});
var _instance = null;
/// <summary>
/// Singleton to host the DOM element associated with WinJS.UI.Flyout controls. Any WinJS.UI.Flyout instantiation
/// must call CommonJS.FlyoutManager.getFlyoutElement() and use it as its DOM element.
/// </summary>
WinJS.Namespace.define("CommonJS", {
FlyoutManager: {
get: function get() {
if (!_instance) {
_instance = new FlyoutManagerImpl();
}
return _instance;
}
}
});
})(WinJS);
(function appexPlatformControlsFullScreenInit() {
"use strict";
var FullScreenImpl = WinJS.Class.mix(WinJS.Class.define(
function fullScreenImpl_ctor() {
// Ensure there is only one full screen element per page. 
this._element = document.getElementById("fullScreenContainer");
if (!this._element) {
this._element = document.createElement("div");
WinJS.Utilities.addClass(this._element, "platformFullScreenContainer");
this._element.id = "fullScreenContainer";
var fsContainerElement = document.getElementById("platformPageArea");
if (!fsContainerElement) {
return null;
}
fsContainerElement = fsContainerElement.parentElement;
fsContainerElement.appendChild(this._element);
this._element.winControl = this;
var that = this;
//do not directly leave fullscreen, wait for user to cleanup (e.g. adControl cleanup) and manually exit
//WinJS.Navigation.addEventListener("navigating", function fullScreenImpl_onNavigating() { that.onNavigating(); });
}
},
{
_fullScreenContentElement: null,
canEnterFullScreen: function canEnterFullScreen() {
return !this._fullScreenContentElement;
},
tryEnterFullScreen: function tryEnterFullScreen(fullScreenContentElement) {
if (this._fullScreenContentElement) {
return false;
}
this._fullScreenContentElement = fullScreenContentElement;
fullScreenContentElement.inlineParentElement = fullScreenContentElement.parentElement;
fullScreenContentElement.parentElement.removeChild(fullScreenContentElement);
this._element.appendChild(fullScreenContentElement);
this._element.style.display = "block";
this.dispatchEvent("FullScreenEnter", this._fullScreenContentElement);
WinJS.UI.Animation.fadeIn(this._fullScreenContentElement);
return true;
},
tryLeaveFullScreen: function tryLeaveFullScreen() {
if (!this._fullScreenContentElement) {
return false;
}
var that = this;
WinJS.UI.Animation.fadeOut(this._fullScreenContentElement).done(function afterFullScreenFadeOut() {
//When navigating back at the same time as video ends, we call tryLeaveFullScreen for twice
//the first done promise will set _fullScreenContentElement = null,
//thus, we need check null here for the following done promise
if (that._fullScreenContentElement) {
that._element.removeChild(that._fullScreenContentElement);
that._fullScreenContentElement.inlineParentElement.appendChild(that._fullScreenContentElement);
that._fullScreenContentElement.inlineParentElement = null;
var eventProperties = that._fullScreenContentElement;
that._element.style.display = "none";
that._fullScreenContentElement = null;
that.dispatchEvent("FullScreenExit", eventProperties);
}
});
return true;
},
}
),
WinJS.Utilities.eventMixin);
var _instance = null;
WinJS.Namespace.define("CommonJS", {
FullScreen: {
get: function get() {
if (!_instance) {
_instance = new FullScreenImpl();
}
return _instance;
}
}
});
})();
(function appexPlatformControlsHTMLElementInit() {
"use strict";
HTMLElement.prototype.getOffset = function () {
return this.getOffsetLeft();
};
HTMLElement.prototype.getOffsetLeft = function () {
if (window.getComputedStyle(this).direction === "rtl") {
var element = this;
var parent = element.offsetParent;
return parent.offsetWidth - element.offsetLeft - element.offsetWidth;
} else {
return this.offsetLeft;
}
};
HTMLElement.prototype.setOffsetLeft = function (value) {
if (window.getComputedStyle(this).direction === "rtl") {
var element = this;
var parent = element.offsetParent;
this.offsetLeft = parent.offsetWidth - value - element.offsetWidth;
} else {
this.offsetLeft = value;
}
};
HTMLElement.prototype.getMarginLeft = function () {
var margin = null;
if (window.getComputedStyle(this).direction === "rtl") {
margin = window.getComputedStyle(this).marginRight;
} else {
margin = window.getComputedStyle(this).marginLeft;
}
return parseInt(margin.replace("px", ""));
};
HTMLElement.prototype.setMarginLeft = function (value) {
if (window.getComputedStyle(this).direction === "rtl") {
this.style.marginRight = value;
} else {
this.style.marginLeft = value;
}
};
HTMLElement.prototype.getMarginRight = function () {
var margin = null;
if (window.getComputedStyle(this).direction === "rtl") {
margin = window.getComputedStyle(this).marginLeft;
} else {
margin = window.getComputedStyle(this).marginRight;
}
return parseInt(margin.replace("px", ""));
};
HTMLElement.prototype.setMarginRight = function (value) {
if (window.getComputedStyle(this).direction === "rtl") {
this.style.marginLeft = value;
} else {
this.style.marginRight = value;
}
};
HTMLElement.prototype.getPaddingLeft = function () {
if (window.getComputedStyle(this).direction === "rtl") {
return this.style.paddingRight;
} else {
return this.style.paddingLeft;
}
};
HTMLElement.prototype.setPaddingLeft = function (value) {
if (window.getComputedStyle(this).direction === "rtl") {
this.style.paddingRight = value;
} else {
this.style.paddingLeft = value;
}
};
HTMLElement.prototype.getLeft = function () {
if (window.getComputedStyle(this).direction === "rtl") {
return this.style.right;
} else {
return this.style.left;
}
};
HTMLElement.prototype.setLeft = function (value) {
if (window.getComputedStyle(this).direction === "rtl") {
this.style.right = value;
} else {
this.style.left = value;
}
};
})();
/* File: Error.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformErrorInit() {
"use strict";
WinJS.Namespace.define("CommonJS.Error", {
_errorControl: null,
_callback: null,
_type: null,
NO_INTERNET: "noInternet",
STANDARD_ERROR: "standardError",
errorExists: function errorExists() {
/// <summary> Determine if a platform error exists </summary>
return this._errorControl !== null;
},
removeError: function removeError(restoreProgress) {
/// <summary> Remove the error control and restore the page </summary>
if (CommonJS.Error._errorControl) {
var element = CommonJS.Error._errorControl.element;
if (element && element.parentNode) {
element.parentNode.removeChild(element);
}
CommonJS.Error._errorControl = null;
var elementsToShow = WinJS.Utilities.query(".platformHideableContent", document.body);
var i = 0;
for (i = 0; i < elementsToShow.length; i++) {
WinJS.Utilities.removeClass(elementsToShow[i], "platformHide");
}
if (restoreProgress) {
for (var type in this._previousProgressCounts) {
for (i = 0; i < this._previousProgressCounts[type]; i++) {
CommonJS.Progress.showProgress(type);
}
}
}
}
},
_previousProgressCounts: [],
showError: function showError(type, callback, customErrorControl, suspend, host) {
/// <summary> Remove the error control and hide parts of the page </summary>
/// <param name="type">The error type: CommonJS.Error.NO_INTERNET or STANDARD_ERROR</param>
/// <param name="callback">The callback function to invoke when the button is pressed</param>
if (suspend) {
if (CommonJS.Error._errorControl) {
return;
}
this._previousProgressCounts = {};
for (var progressType in CommonJS.Progress._counts) {
this._previousProgressCounts[progressType] = CommonJS.Progress._counts[progressType];
}
}
// Hide all progress indicators
CommonJS.Progress.resetAll();
// Remove the old error
CommonJS.Error.removeError();
if (customErrorControl) {
CommonJS.Error._errorControl = customErrorControl;
}
else {
CommonJS.Error._errorControl = new CommonJS.ErrorControl();
CommonJS.Error._errorControl.type = type;
}
CommonJS.Error._errorControl.buttonHide = suspend;
CommonJS.Error._callback = callback;
CommonJS.Error._type = type;
// Create the error control
if (!suspend) {
CommonJS.Error._errorControl.callback = function () {
CommonJS.Error.removeError();
if (callback) {
callback();
}
};
}
// Hide all major elements
var elementsToHide = WinJS.Utilities.query(".platformHideableContent");
for (var i = 0; i < elementsToHide.length; i++) {
WinJS.Utilities.addClass(elementsToHide[i], "platformHide");
}
var parent = host || document.body;
parent.appendChild(CommonJS.Error._errorControl.element);
// Ensure that afterFirstView event is raised if this was an error during startup
PlatformJS.mainProcessManager.afterFirstView();
},
showSystemMessage: function showSystemMessage(errorText, callback, isSticky, closeButtonHidden, additionalClass) {
/// <summary> Shows the system error message</summary>
/// <param name="errorText">Error text.</param>
/// <param name="callback">Callback function to invoke when button is pressed.</param>
/// <param name="isSticky">Don't dismiss the flyout</param>
/// <param name="closeButtonHidden">Hide the close button</param>
CommonJS.dismissAllEdgies();
var div = document.createElement("div"),
flyout = null;
WinJS.Utilities.addClass(div, "platformSystemFlyout");
if (additionalClass) {
WinJS.Utilities.addClass(div, additionalClass);
}
flyout = new CommonJS.Flyout(div, {
buttonText: "",
contentFragment: {
path: "/common/html/platform.html",
id: "platformSystemMessage",
data: {
message: errorText
}
},
isSticky: isSticky,
closeButtonHidden: closeButtonHidden
});
flyout.onHide = callback;
// Ensure that afterFirstView event is raised if this was an error during startup
PlatformJS.mainProcessManager.afterFirstView();
},
showMarketError: function showMarketError() {
/// <summary> Shows the market unavailability error</summary>
var marketError = new Windows.UI.Popups.MessageDialog(PlatformJS.Services.resourceLoader.getString("/platform/marketError"));
marketError.commands.append(new Windows.UI.Popups.UICommand(
PlatformJS.Services.resourceLoader.getString("/platform/okButton"),
function platformError_showMarketError() {
}
));
// Ensure that afterFirstView event is raised if this was an error during startup
PlatformJS.mainProcessManager.afterFirstView();
try {
marketError.showAsync();
} catch (e) {
// "Access is denied." error may occur if a message dialog is already up 
};
},
showCatastrophicError: function showCatastrophicError(e) {
throw e;
},
getErrorModuleItem: function getErrorModuleItem(retryCallback, width, height, messageResource, errorContainerClassName) {
/// <summary>Returns an error module item for use in a MultisizeHorizontalGrid</summary>
/// <param name="retryCallback">Function to invoke when retry button is pressed</param>
/// <param name="width">Optional width of module</param>
/// <param name="height">Optional height of module</param>
/// <param name="messageResource"> Optional resource path to message content</param>
/// <param name="divClassName"> Optional html class of the error message</param>
/// <returns> Returns an item for use in a Binding.List</returns>
var messageLocation = messageResource || "/platform/standardErrorDescription";
return {
message: PlatformJS.Services.resourceLoader.getString(messageLocation),
retryCallback: retryCallback,
moduleInfo: {
width: width ? width : "250px",
height: height ? height : "250px",
isInteractive: true,
disableHover: true,
fragmentPath: "/common/html/platform.html",
templateId: "platformErrorModule"
},
errorContainerClassName: errorContainerClassName ? "platformErrorModule " + errorContainerClassName : "platformErrorModule"
};
},
getProgressModuleItem: function getProgressModuleItem(width, height) {
/// <summary>Returns a progress module with the progress ring.</summary>
/// <param name="width">Optional width of module</param>
/// <param name="height">Optional height of module</param>
return {
moduleInfo: {
width: width ? width : "450px",
height: height ? height : "100%",
isInteractive: true,
disableHover: true,
fragmentPath: "/common/html/platform.html",
templateId: "platformProgressModule"
}
};
}
});
// Remove the error control whenever navigating
WinJS.Navigation.addEventListener("navigating", CommonJS.Error.removeError);
Platform.Networking.NetworkManager.addEventListener("networkstatuschanged", function platform_onNetworkStatusChanged() {
try {
if (PlatformJS.Utilities.hasInternetConnection()) {
if (CommonJS.Error.errorExists() &&
CommonJS.Error._type === CommonJS.Error.NO_INTERNET &&
CommonJS.Error._callback) {
CommonJS.Error.removeError();
CommonJS.Error._callback();
}
var errorModules = WinJS.Utilities.query(".platformRetryButton", document.body);
for (var i = 0; i < errorModules.length; i++) {
var errorModule = errorModules.get(i);
if (errorModule.onclick) {
errorModule.onclick();
}
}
}
}
catch (e) {
}
});
})();
/* File: ErrorControl.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsErrorControlInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS", {
/// <summary>Error Control</summary>
ErrorControl: WinJS.Class.define(function errorControl_ctor(element, options) {
/// <summary>
/// constructor
/// </summary>
/// <param name="element">
/// The DOM element
/// </param>
/// <param name="options">
/// Options structure
/// </param>
this.element = element || document.createElement("div");
this.element.winControl = this;
WinJS.Utilities.addClass(this.element, "platformErrorHost");
this.element.setAttribute("role", "alert");
var host = document.createElement("div");
WinJS.Utilities.addClass(host, "platformError");
this.element.appendChild(host);
this._icon = document.createElement("div");
WinJS.Utilities.addClass(this._icon, "platformErrorIcon");
host.appendChild(this._icon);
var icon1 = document.createElement("div");
WinJS.Utilities.addClass(icon1, "platformErrorIconGlyph1");
this._icon.appendChild(icon1);
var icon2 = document.createElement("div");
WinJS.Utilities.addClass(icon2, "platformErrorIconGlyph2");
this._icon.appendChild(icon2);
this._title = document.createElement("div");
WinJS.Utilities.addClass(this._title, "platformErrorTitle");
host.appendChild(this._title);
this._description = document.createElement("div");
WinJS.Utilities.addClass(this._description, "platformErrorDescription");
host.appendChild(this._description);
this._buttonContainer = document.createElement("div");
WinJS.Utilities.addClass(this._buttonContainer, "platformErrorButtonContainer");
host.appendChild(this._buttonContainer);
this._button = document.createElement("button");
WinJS.Utilities.addClass(this._button, "platformErrorButton");
this._buttonContainer.appendChild(this._button);
CommonJS.setAutomationId(this.element);
CommonJS.setAutomationId(this._button, this.element, "button");
WinJS.UI.setOptions(this, options);
},
{
_icon: null,
_title: null,
_description: null,
_button: null,
_buttonContainer: null,
elenent: null,
/// <field>The callback function to invoke when the button is pressed</field>
callback: {
set: function set(value) {
this._button.onclick = value;
}
},
/// <field>The error type: CommonJS.Error.NO_INTERNET or STANDARD_ERROR</field>
type: {
set: function set(value) {
if (value !== CommonJS.Error.STANDARD_ERROR &&
value !== CommonJS.Error.NO_INTERNET) {
PlatformJS.Utilities.onError("Invalid error type");
}
this._description.innerText = CommonJS.resourceLoader.getString("/platform/" + value + "Description");
this._title.innerText = CommonJS.resourceLoader.getString("/platform/" + value + "Title");
this.element.setAttribute("aria-label", this._title.innerText);
if (value === CommonJS.Error.NO_INTERNET) {
WinJS.Utilities.addClass(this._icon, "iconNoInternet");
WinJS.Utilities.addClass(this._buttonContainer, "platformHide");
} else {
WinJS.Utilities.addClass(this._icon, "iconError");
}
this._button.innerText = CommonJS.resourceLoader.getString("/platform/" + value + "Button");
this._button.setAttribute("aria-label", this._button.innerText);
}
}
},
{
})
});
})();
/* File: Progress.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformProgressInit() {
"use strict";
WinJS.Namespace.define("CommonJS.Progress", {
_domElements: {},
_counts: {},
_splashDiv: null,
_splashTimeout: null,
_splashComplete: [],
// Fix for bug 527426 (change list # 155565):
// flag to indicate if splash screen can be dismissed (initialized as true to avoid regression)
_canDestroySplash: true,
// End of fix for bug 527426 (change list # 155565):
/// <summary> Header progress type </summary>
headerProgressType: "headerProgressType",
/// <summary> Center progress type </summary>
centerProgressType: "centerProgressType",
_createElementIfNeeded: function _createElementIfNeeded(type) {
/// <summary> Returns the DOM element for the progress type. Creates it if it has not been created yet. </summary>
/// <param name="type">The type of the progress: headerProgressType or centerProgressType</param>
/// <returns>Nothing</returns>
var element = CommonJS.Progress._domElements[type];
if (!element) {
if (type === CommonJS.Progress.headerProgressType) {
element = document.createElement("progress");
WinJS.Utilities.addClass(element, "platformProgressHeader");
element.setAttribute("role", "presentation");
} else if (type === CommonJS.Progress.centerProgressType) {
element = document.createElement("div");
WinJS.Utilities.addClass(element, "platformProgressCenterWrapper");
var progressElement = document.createElement("div");
WinJS.Utilities.addClass(progressElement, "platformProgressCenter");
var progress = document.createElement("progress");
WinJS.Utilities.addClass(progress, "win-large win-ring");
progress.setAttribute("role", "presentation");
progressElement.appendChild(progress);
var loading = document.createElement("div");
WinJS.Utilities.addClass(loading, "platformProgressAlert");
loading.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/platform/progressAlertAriaLabel"));
loading.setAttribute("role", "alert"); // implicit aria-live=assertive region
loading.setAttribute("aria-atomic", true);
loading.innerText = PlatformJS.Services.resourceLoader.getString("/platform/Loading");
progressElement.appendChild(loading);
element.appendChild(progressElement);
} else {
PlatformJS.Utilities.onError("Invalid progress type");
}
WinJS.Utilities.addClass(element, "platformProgress");
CommonJS.Progress._domElements[type] = element;
document.body.appendChild(element);
CommonJS.Progress._counts[type] = 0;
}
return element;
},
_deleteElement: function _deleteElement(type) {
var element = CommonJS.Progress._domElements[type];
if (element) {
if (element.parentElement) {
element.parentElement.removeChild(element);
}
CommonJS.Progress._domElements[type] = null;
}
},
_updateAriaLiveRegion: function _updateAriaLiveRegion(type) {
var element = CommonJS.Progress._domElements[type];
if (element) {
var alertEl = element.querySelector(".platformProgressAlert");
if (alertEl) {
alertEl.innerText = PlatformJS.Services.resourceLoader.getString("/platform/Loading");
}
}
},
// Fix for bug 527426 (change list # 155565):
canDestroySplash: function canDestroySplash(loaded) {
/// <summary> set _canDestroySplash flag to indicate if splash screen can be dismissed </summary>
/// <param name="loaded">true or false</param>
/// <returns>Nothing</returns>
this._canDestroySplash = loaded;
},
// End of fix for bug 527426 (change list # 155565):
showProgress: function showProgress(type) {
/// <summary> Increment the ref count for progress indicator. Activates indicator if it hasn't already. </summary>
/// <param name="type">The type of the progress: headerProgressType or centerProgressType</param>
/// <returns>Nothing</returns>
CommonJS.Progress._createElementIfNeeded(type);
CommonJS.Progress._updateAriaLiveRegion(type);
CommonJS.Progress._counts[type]++;
},
hideProgress: function hideProgress(type) {
/// <summary> Decrements the ref count for progress indicator. Deactivates indicator if the ref count reaches 0. </summary>
/// <param name="type">The type of the progress: headerProgressType or centerProgressType</param>
/// <returns>Nothing</returns>
CommonJS.Progress._counts[type] = Math.max(0, CommonJS.Progress._counts[type] - 1);
if (CommonJS.Progress._counts[type] === 0) {
this._deleteElement(type);
CommonJS.Progress.destroySplash();
}
},
resetProgress: function resetProgress(type) {
/// <summary> Resets the ref count for progress indicator to 0 and turns off indicator</summary>
/// <param name="type">The type of the progress: headerProgressType or centerProgressType</param>
/// <returns>Nothing</returns>
this._deleteElement(type);
CommonJS.Progress._counts[type] = 0;
},
resetAll: function resetAll() {
/// <summary> Resets all progress indicators</summary>
/// <param name="type">The type of the progress: headerProgressType or centerProgressType</param>
/// <returns>Nothing</returns>
CommonJS.Progress.resetProgress(CommonJS.Progress.headerProgressType);
CommonJS.Progress.resetProgress(CommonJS.Progress.centerProgressType);
},
initializeExtendedSplash: function initializeExtendedSplash(splash, centerSplash) {
/// <summary>Initialize the extended splash</summary>
/// <returns>Nothing</returns>
var that = this;
this.playingExtendedSplash = true;
msWriteProfilerMark("Platform:InitializeSplash:s");
this.onResize = function () {
that.positionSplash(splash, centerSplash);
};
if (!this._splashDiv) {
this._splashDiv = document.getElementById("splashContainer");
this._splashImageDiv = document.getElementById("splashImage");
this._splasyProgress = document.getElementById("splashProgress");
}
if (this._splashDiv && !this._headerBackButton) {
var backButton = this._headerBackButton = document.createElement("button");
backButton.setAttribute("aria-label", CommonJS.resourceLoader.getString("/platform/GoBack"));
CommonJS.setAutomationId(backButton, this._domElement, "backButton");
WinJS.Utilities.addClass(this._headerBackButton, "win-backbutton immersiveBackButton splashBackButton platformHide");
this._headerBackButton.addEventListener("click", this._splashScreenBackButtonClick.bind(this));
this._splashDiv.appendChild(this._headerBackButton);
}
if (this._headerBackButton) {
if (WinJS.Navigation.canGoBack) {
this._headerBackButton.addEventListener("click", this._splashScreenBackButtonClick);
this._headerBackButton.disabled = false;
WinJS.Utilities.removeClass(this._headerBackButton, "platformHide");
} else {
this._headerBackButton.disabled = true;
WinJS.Utilities.addClass(this._headerBackButton, "platformHide");
}
}
if (this._splashImageDiv) {
if (!this._splashImageSrc) {
this._splashImageSrc = this._splashImageDiv.src;
}
if (splash.imageUri) {
this._splashImageDiv.src = splash.imageUri;
}
}
if (this._splashDiv) {
if (!this._splashBackgroundColor) {
this._splashBackgroundColor = this._splashDiv.style.background;
}
if (splash.backgroundColor) {
this._splashDiv.style.background = splash.backgroundColor;
}
WinJS.Utilities.removeClass(CommonJS.Progress._splashDiv, "platformHide");
}
this.positionSplash(splash, centerSplash);
window.addEventListener("resize", that.onResize, false);
msWriteProfilerMark("Platform:InitializeSplash:e");
},
_splashScreenBackButtonClick: function _splashScreenBackButtonClick(event) {
if (!PlatformJS.Navigation || PlatformJS.Navigation.canGoBack) {
WinJS.Navigation.back();
} else {
PlatformJS.Navigation.navigateToChannel("Home");
}
},
positionSplash: function positionSplash(splash, centerSplash) {
if (splash && splash.imageLocation) {
var currentCoordinates = splash.imageLocation;
if (centerSplash) {
currentCoordinates.x = (document.body.offsetWidth - currentCoordinates.width) / 2;
currentCoordinates.y = (document.body.offsetHeight - currentCoordinates.height) / 2;
}
if (this._splashImageDiv) {
this._splashImageDiv.style.top = currentCoordinates.y + "px";
this._splashImageDiv.setLeft(currentCoordinates.x + "px");
this._splashImageDiv.style.width = currentCoordinates.width + "px";
this._splashImageDiv.style.height = currentCoordinates.height + "px";
}
if (this._splasyProgress) {
this._splasyProgress.style.top = currentCoordinates.y + currentCoordinates.height + "px";
}
}
},
destroySplash: function destroySplash(forceDestroy) {
/// <summary>Destroys the extended splash</summary>
/// <param name="forceDestroy">Force the destroy</param>
/// <returns>Nothing</returns>
if (this.playingExtendedSplash || this._systemSplash) {
var splashDestroyed = false;
var headerProgress = CommonJS.Progress._counts[CommonJS.Progress.headerProgressType] || 0;
var centerProgress = CommonJS.Progress._counts[CommonJS.Progress.centerProgressType] || 0;
if ((this._splashDiv || this._systemSplash)  &&
(forceDestroy || (centerProgress === 0 && headerProgress === 0))
// Fix for bug 527426 (change list # 155565):
// check to see if splash screen can be dismissed or not
&& this._canDestroySplash) {
// End of fix for bug 527426 (change list # 155565):
msWriteProfilerMark("Platform:DestroySplash:s");
if (this._splashDiv) {
WinJS.Utilities.addClass(this._splashDiv, "platformHide");
if (this._headerBackButton) {
this._headerBackButton.removeEventListener("click", this._splashScreenBackButtonClick);
}
this._splashImageDiv.src = this._splashImageSrc;
this._splashDiv.style.background = this._splashBackgroundColor;
this.playingExtendedSplash = false;
if (CommonJS.Progress._splashTimeout) {
clearTimeout(CommonJS.Progress._splashTimeout);
CommonJS.Progress._splashTimeout = null;
}
window.removeEventListener("resize", this.onResize);
if (CommonJS.Progress._fadeinSplash) {
document.body.removeChild(CommonJS.Progress._fadeinSplash);
CommonJS.Progress._fadeinSplash = null;
}
if (CommonJS.Progress._splashImageDiv) {
CommonJS.Progress._splashImageDiv.onload = null;
}
splashDestroyed = true;
}
else {
this.removeExtendedSplashElement();
}
if (this._systemSplash) {
this.destroySystemSplash();
splashDestroyed = true;
}
if (splashDestroyed) {
Platform.Process.instance.perfTrackWriteStopEventWithMetadata(Platform.PerfTrackScenarioId.launch, "App Launch", null, false);
}
msWriteProfilerMark("Platform:DestroySplash:e");
if (CommonJS.Progress._splashComplete.length) {
CommonJS.Progress._splashComplete.forEach(function (splashComplete) {
splashComplete();
});
}
CommonJS.Progress._splashComplete = [];
}
}
},
setSystemSplash: function setSystemSplash(systemSplash, splashComplete) {
this._systemSplash = systemSplash;
if (splashComplete) {
CommonJS.Progress._splashComplete.push(splashComplete);
}
},
isShowingSystemSplash: function isShowingSystemSplash() {
return this._systemSplash !== null;
},
destroySystemSplash: function destroySystemSplash() {
if (this._systemSplash) {
this._systemSplash.complete();
this._systemSplash = null;
}
},
removeExtendedSplashElement: function removeExtendedSplashElement() {
if (!this._splashDiv) {
this._splashDiv = document.getElementById("splashContainer");
}
if (this._splashDiv) {
this._splashDiv.parentNode.removeChild(this._splashDiv);
this._splashDiv = null;
}
},
initCloudSplashScreen: function initCloudSplashScreen(splash) {
if (!splash) {
var splashData = PlatformJS.Services.configuration.getDictionary("CloudPanoSplash");
if (splashData.imageUri && splashData.height && splashData.width && splashData.backgroundColor) {
splash = {
imageUri: splashData.imageUri.value,
imageLocation: {
height: splashData.height.value,
width: splashData.width.value
},
backgroundColor: splashData.backgroundColor.value
};
}
}
if (splash && splash.imageUri && splash.imageLocation) {
CommonJS.Progress.initializeExtendedSplash(splash, true);
var showAnimation = true;
// Hide old splash image
if (Windows.UI.ViewManagement.ApplicationView.value === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
showAnimation = false;
}
if (this._splashImageDiv) {
this._splashImageDiv.style.opacity = 0;
}
if (this._splashDiv && this._splashImageDiv) {
var that = this;
if (showAnimation) {
// Create fade in foreground for image
var fadeinSplash = document.createElement("div");
fadeinSplash.style.position = "absolute";
fadeinSplash.style.top = this._splashImageDiv.style.top;
fadeinSplash.style.left = this._splashImageDiv.style.left;
fadeinSplash.style.width = "100%";
fadeinSplash.style.height = "100%";
fadeinSplash.style.zIndex = window.getComputedStyle(this._splashDiv).zIndex;
document.body.appendChild(fadeinSplash);
this._fadeinSplash = fadeinSplash;
// Set handler on splash image load
that._splashImageDiv.onload = function () {
// Validate
if (that._fadeinSplash && that._splashImageDiv) {
that._splashImageDiv.style.opacity = 0;
// Remove attached event
that._splashImageDiv.onload = null;
// Clone loaded image into overlay div
var cloneSplash = that._splashImageDiv.cloneNode(true);
cloneSplash.style.top = "0px";
cloneSplash.style.left = "0px";
that._fadeinSplash.appendChild(cloneSplash);
// Fade loaded image in
WinJS.UI.Animation.fadeIn(cloneSplash).then(function () {
// Ensure splash image is now fully visible
if (that._splashImageDiv) {
that._splashImageDiv.style.opacity = 1;
}
// Remove fade in overlay
if (that._fadeinSplash) {
document.body.removeChild(that._fadeinSplash);
that._fadeinSplash = null;
}
});
}
};
} else {
that._splashImageDiv.onload = function () {
if (that._splashImageDiv) {
that._splashImageDiv.style.opacity = 1;
}
};
}
}
}  else {
CommonJS.Progress.showProgress(CommonJS.Progress.centerProgressType);
}
}
});
// Upon page navigation, reset the progress indicators
WinJS.Navigation.addEventListener("navigating", CommonJS.Progress.resetAll);
})();
/* File: MessageBar.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsMessageBarInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS", {
/// <field>
/// Global refrence for the MessageBar
/// </field>
messageBar: null,
/// <field>
/// Enumeration indicating that the MessageBar is showing an error message
/// </field>
MessageBarLevelError: "MessageBarLevelError",
/// <field>
/// Enumeration indicating that the MessageBar is showing a warning message
/// </field>
MessageBarLevelWarning: "MessageBarLevelWarning",
/// <summary>
/// MessageBar control.
/// Renders a warning/error messagebar at the top of the screen
/// </summary>
MessageBar: WinJS.Class.define(
function messageBar_ctor(message, options) {
/// <summary>
/// Constructor: Creates an instance of the MessageBar Control object.
/// </summary>
/// <param name="message">
/// The error message to be shown
/// </param>
/// <param name="options">
/// Options structure
/// </param>
/// <returns>
/// An instance of the MessageBar control object.
/// </returns>
if (CommonJS.messageBar) {
CommonJS.messageBar._destroy();
}
CommonJS.messageBar = this;
this._domElement = document.createElement("div");
this._domElement.setAttribute("aria-hidden", true);
this._message = message;
this._buttons = [];
this._autoHide = false;
this._autoHideTimeout = 3000;
this._level = CommonJS.MessageBarLevelWarning;
this._isSticky = false;
this._destroyBinding = this._destroy.bind(this);
if (options) {
this._setOptions(options);
}
this._init(this._domElement);
},
// Instance members
{
/// <field>
/// If hide messageBar on snap mode: true or false.
/// </field>
hideOnSnap: false,
_domElement: null,
_message: null,
_buttons: [],
_autoHide: false,
_autoHideTimeout: null,
_level: null,
_isSticky: false,
_destroyBinding: null,
/// <field>
/// Message to show up in messageBar.
/// </field>
message: {
get: function get() {
return this._message;
},
set: function set(value) {
this._message = value;
this._init(this._domElement);
}
},
/// <field>
/// Show the MessageBar
/// </field>
show: function show() {
var that = this;
if (this._autoHide) {
var timout = setTimeout(function messageBar_autoHideTimeout() { that.hide(); }, that._autoHideTimeout);
}
this._domElement.setAttribute("role", "alert");
this._domElement.setAttribute("aria-hidden", false);
if (this.hideOnSnap) {
WinJS.Utilities.addClass(this._domElement, "platformHideOnSnap");
}
msSetImmediate(function messageBar_setVisibleClass() { WinJS.Utilities.addClass(that._domElement, "platformMessageBarVisible"); });
},
/// <field>
/// Hide the MessageBar
/// </field>
hide: function hide() {
this._domElement.addEventListener("MSTransitionEnd", this._destroyBinding);
WinJS.Utilities.removeClass(this._domElement, "platformMessageBarVisible");
this._domElement.setAttribute("aria-hidden", true);
},
/// <field>
/// Adding a button to the MessageBar
/// </field>
/// <param name="label">
/// The text inside the button
/// </param>
/// <param name="callback">
/// Callback function to call on click
/// </param>
addButton: function addButton(label, callback) {
if (this._buttons.length < 2) {
var button = document.createElement("button");
button.innerText = label;
button.addEventListener("click", callback);
this._buttons.push(button);
this._domElement.appendChild(button);
CommonJS.setAutomationId(button, this._domElement, label);
}
else {
PlatformJS.Utilities.onError("The MessageBar can only contain up to 2 buttons");
}
},
_init: function _init(element) {
var messageDiv = document.createElement("div");
messageDiv.innerText = this.message;
WinJS.Utilities.addClass(this._domElement, "platformMessageBar");
WinJS.Utilities.addClass(messageDiv, "platformMessageBarText");
if (this._level === CommonJS.MessageBarLevelError) {
WinJS.Utilities.addClass(this._domElement, "platformMessageBarError");
}
this._domElement.appendChild(messageDiv);
document.body.appendChild(this._domElement);
CommonJS.setAutomationId(this._domElement);
if (!this._isSticky) {
WinJS.Navigation.addEventListener("beforenavigate", this._destroyBinding);
}
},
_setOptions: function _setOptions(options) {
if (options.autoHide) {
this._autoHide = options.autoHide;
}
if (options.level) {
this._level = options.level;
}
if (options.isSticky) {
this._isSticky = options.isSticky;
}
if (options.autoHideTimeout) {
this._autoHideTimeout = options.autoHideTimeout;
}
},
_destroy: function _destroy() {
if (this._domElement) {
this._domElement.removeEventListener("MSTransitionEnd", this._destroyBinding);
WinJS.Navigation.removeEventListener("beforenavigate", this._destroyBinding);
this._domElement.outerHTML = "";
}
if (CommonJS.messageBar === this) {
CommonJS.messageBar = null;
}
}
})
});
})();
/* File: WindowEventManager.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsWindowEventManagerInit() {
"use strict";
/// <summary>
/// WindowEventManager handles all document and window events related to keyboard navigation and mouse/keyboard shortcuts
/// WindowEventManager uses a singleton pattern
/// </summary>
var instance = null;
var WindowEventManager = WinJS.Namespace.define("CommonJS.WindowEventManager", {
/// <field>
/// Enumeration of window events
/// </field>
Events: {
PAGEUP: "pageup",
PAGEDOWN: "pagedown",
CURSORKEY: "cursorkey",
TAB: "tab",
MOUSE_BACK: "mouse_back",
MOUSE_FORWARD: "mouse_forward",
WINDOW_RESIZE: "window_resize"
},
getInstance: function getInstance() {
if (!instance) {
instance = new WindowEventManager._WindowEventManager();
}
return instance;
},
destroyInstance: function destroyInstance() {
if (instance) {
instance.destroy();
instance = null;
}
},
// Private Constructor
_WindowEventManager: WinJS.Class.mix(WinJS.Class.define(function windowEventManager_ctor() {
this._element = document.createElement("div");
this._pointerUpBinding = this._onWindowPointerUp.bind(this);
this._keyDownBinding = this._onWindowKeyDown.bind(this);
this._resizeBinding = this._resize.bind(this);
window.addEventListener("MSPointerUp", this._pointerUpBinding);
window.addEventListener("keydown", this._keyDownBinding);
window.addEventListener("resize", this._resizeBinding);
},
{
// Private Fields
_element: null,
_pointerUpBinding: null,
_keyDownBinding: null,
_resizeBinding: null,
// Private Functions
_resize: function _resize(event) {
this.dispatchEvent(WindowEventManager.Events.WINDOW_RESIZE, event);
},
_onWindowPointerUp: function _onWindowPointerUp(event) {
switch (event.button) {
// Mouse back button
case 3:
if (WinJS.Navigation.canGoBack) {
WinJS.Navigation.back();
this.dispatchEvent(WindowEventManager.Events.MOUSE_BACK, event);
}
break;
// Mouse forward button
case 4:
if (WinJS.Navigation.canGoForward) {
WinJS.Navigation.forward();
this.dispatchEvent(WindowEventManager.Events.MOUSE_FORWARD, event);
}
break;
}
},
_onWindowKeyDown: function _onWindowKeyDown(event) {
switch (event.keyCode) {
case WinJS.Utilities.Key.pageUp:
this.dispatchEvent(WindowEventManager.Events.PAGEUP, event);
break;
case WinJS.Utilities.Key.pageDown:
this.dispatchEvent(WindowEventManager.Events.PAGEDOWN, event);
break;
case WinJS.Utilities.Key.browserBack:
if (event.target.isContentEditable !== true) {
this._onShortcutKeyBack(event);
}
break;
case WinJS.Utilities.Key.leftArrow:
if (event.altKey) {
this._onShortcutKeyBack(event);
}
else {
this.dispatchEvent(WindowEventManager.Events.CURSORKEY, event);
}
break;
case WinJS.Utilities.Key.rightArrow:
if (!event.altKey) {
this.dispatchEvent(WindowEventManager.Events.CURSORKEY, event);
}
break;
case WinJS.Utilities.Key.upArrow:
case WinJS.Utilities.Key.downArrow:
this.dispatchEvent(WindowEventManager.Events.CURSORKEY, event);
break;
case WinJS.Utilities.Key.tab:
this.dispatchEvent(WindowEventManager.Events.TAB, event);
break;
default:
break;
}
},
_onShortcutKeyBack: function _onShortcutKeyBack(event) {
if (WinJS.Navigation.canGoBack) {
WinJS.Navigation.back();
event.preventDefault();
event.stopImmediatePropagation();
}
},
// Public Methods
/// <field>
/// Dispose windowEventManager and remove eventListeners.
/// </field>
destroy: function destroy() {
window.removeEventListener("MSPointerUp", this._pointerUpBinding);
window.removeEventListener("keydown", this._keyDownBinding);
window.removeEventListener("resize", this._resizeBinding);
}
}), WinJS.Utilities.eventMixin)
});
})();
/* File: utils.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsUtilsInit (WinJS) {
"use strict";
function _getEstimatedStringLength(strLength, currentHeight, desiredHeight) {
/// <summary>
/// For the input string length, given that its height is currentHeight, returns an estimated string length that would fit in desiredHeight.
/// </summary>
return Math.floor(strLength * desiredHeight / currentHeight);
};
var Utils = WinJS.Namespace.define("CommonJS.Utils", {
itemClickProxy: function itemClickProxy(control) {
return function utils_itemClickProxy (evt) {
var item = null;
var currentElt = evt.srcElement;
// walk up the DOM until the module control is found, and get the item from it.
while (currentElt) {
if (WinJS.Utilities.hasClass(currentElt, "cluster-item")) {
item = currentElt.winControl.data;
break;
}
currentElt = currentElt.parentElement;
}
if (item) {
control.dispatchEvent("itemclick", { item: item });
}
};
},
convertRSSTimeToFriendlyTime: function convertRSSTimeToFriendlyTime(rssTime) {
/// <summary>
/// Returns a friendly name for the bdi time such as "21 minutes ago"
/// </summary>
/// <param name="rssTime">Time string from an RSS feed</param>
var now = new Date();
var nowEpoch = now.getTime();
var date = new Date(rssTime);
var rssTimeEpoch = date.getTime();
if (!isNaN(rssTimeEpoch)) {
var delta = Math.max(0, nowEpoch - rssTimeEpoch);
return CommonJS.Utils.convertDeltaToFriendlyTime(delta);
}
return "";
},
convertBDITimeToFriendlyTime: function convertBDITimeToFriendlyTime(bdiTime) {
/// <summary>
/// Returns a friendly name for the bdi time such as "21 minutes ago"
/// </summary>
/// <param name="bdiTime">UTC Time in 100 nanosecond intervals since midnight of 1601/1/1</param>
var now = new Date();
var nowEpoch = Date.UTC(now.getUTCFullYear(), now.getUTCMonth(), now.getUTCDate(), now.getUTCHours(), now.getUTCMinutes(), now.getUTCSeconds());
var bdiTimeEpoch = (bdiTime / 10000) + Utils._bdiTimeOriginEpoch;
var delta = Math.max(0, nowEpoch - bdiTimeEpoch);
return CommonJS.Utils.convertDeltaToFriendlyTime(delta);
},
convertDeltaToFriendlyTime: function convertDeltaToFriendlyTime(delta) {
/// <summary>
/// Returns a friendly name for the bdi time such as "21 minutes ago"
/// </summary>
/// <param name="delta">Time delta from now, in ms</param>
var friendlyTime = "";
var resourceLoader = CommonJS.resourceLoader;
if (0 <= delta && delta < 120000 /* 2mins */) {
friendlyTime = resourceLoader.getString("/Common/OneMinAgo");
} else if (120000 <= delta && delta < 3600000 /* 1hr */) {
friendlyTime = resourceLoader.getString("/Common/MinsAgo").format(Math.floor(delta / 60000));
} else if (3600000 <= delta && delta < 7200000 /* 2hr */) {
friendlyTime = resourceLoader.getString("/Common/OneHourAgo");
} else if (7200000 <= delta && delta < 86400000 /* 1 day */) {
friendlyTime = resourceLoader.getString("/Common/HoursAgo").format(Math.floor(delta / 3600000));
} else if (86400000 <= delta && delta < 172800000 /* 2 days */) {
friendlyTime = resourceLoader.getString("/Common/OneDayAgo");
} else {
friendlyTime = resourceLoader.getString("/Common/DaysAgo").format(Math.floor(delta / 86400000));
}
return friendlyTime;
},
truncateString: function truncateString(str, len) {
/// <summary>
/// Returns the truncation of str to the given length, and adds "..." if any truncation happens.
/// </summary>
return str.substr(0, len) + (len >= str.length ? "" : "...");
},
truncateMultilineText: function truncateMultilineText(multilineElt) {
msWriteProfilerMark("CommonControls:Utils:truncateMultilineText:s");
var parentElt = multilineElt.parentElement;
// measure the height of the element and its container
var parentHeight = parentElt.clientHeight;
var height = multilineElt.clientHeight;
// if the element doesn't fit within its container
if (height > parentHeight) {
var originalString = multilineElt.innerText;
var originalLength = originalString.length;
// estimate the string length based on the height ratio of the element and its parent
var initialLength = _getEstimatedStringLength(originalLength, height, parentHeight);
// find the next word boundary
var nextWordBoundaryIndex = originalString.indexOf(" ", initialLength);
initialLength = (nextWordBoundaryIndex === -1 ? initialLength : nextWordBoundaryIndex);
// truncate at the initial estimate
var tryLength = initialLength;
var tryString = CommonJS.Utils.truncateString(originalString, tryLength);
multilineElt.innerText = tryString;
// measure the height again
height = multilineElt.clientHeight;
// while element is still too high to fit its container
while (height > parentHeight) {
// find the previous word boundary
var oneLessWordLength = originalString.lastIndexOf(" ", tryLength - 1);
tryLength = oneLessWordLength === -1 ? tryLength - 1 : oneLessWordLength;
// if reached the beginning of the string, quit
if (tryLength <= 0) {
break;
}
// truncate, measure, and repeat
tryString = CommonJS.Utils.truncateString(originalString, tryLength);
multilineElt.innerText = tryString;
height = multilineElt.clientHeight;
}
}
msWriteProfilerMark("CommonControls:Utils:truncateMultilineText:e");
},
_bdiTimeOriginEpoch: Date.UTC(1601, 0, 1, 0, 0, 0),
convertCMSArticle: function convertCMSArticle(result, createURIFunction) {
var newsArticle = {};
newsArticle.editorial = true;
if (result.heroImage) {
newsArticle.anchorpoint = result.heroImage.anchorPoint;
}
var cmsImage = null;
if (result.heroImage && result.heroImage.image) {
cmsImage = result.heroImage.image;
}
else {
cmsImage = result.image;
}
if (cmsImage) {
newsArticle.imageAttribution = cmsImage.attribution ? cmsImage.attribution : "";
newsArticle.altText = cmsImage.altText ? cmsImage.altText : "";
newsArticle.caption = cmsImage.caption ? toStaticHTML(cmsImage.caption) : "";
var originalImage = null;
if (cmsImage.images) {
for (var i = 0; i < cmsImage.images.length; i++) {
var img = cmsImage.images[i];
var imgName = img.name ? img.name.toLowerCase() : "";
if (imgName === "original") {
newsArticle.thumbnail = {};
newsArticle.thumbnail.url = img.url;
newsArticle.thumbnail.width = img.width;
newsArticle.thumbnail.height = img.height;
newsArticle.thumbnail.name = "original";
}
else if (imgName === "semanticzoom") {
newsArticle.semanticZoomThumbnail = {};
newsArticle.semanticZoomThumbnail.url = img.url;
newsArticle.semanticZoomThumbnail.width = img.width;
newsArticle.semanticZoomThumbnail.height = img.height;
newsArticle.semanticZoomThumbnail.name = "semanticZoom";
}
else if (imgName === "lowres") {
newsArticle.thumbnailLowRes = {};
newsArticle.thumbnailLowRes.url = img.url;
newsArticle.thumbnailLowRes.width = img.width;
newsArticle.thumbnailLowRes.height = img.height;
newsArticle.thumbnailLowRes.name = "lowRes";
}
}
}
}
else {
newsArticle.imageAttribution = "";
newsArticle.altText = "";
newsArticle.caption = "";
}
newsArticle.sourceImageUrl = result.content.source && result.content.source.favicon ? result.content.source.favicon : "";
// CMS headline support HTML Fragments
newsArticle.headline = result.content.headline ? toStaticHTML(result.content.headline) : "";
newsArticle.title = newsArticle.headline;
newsArticle.source = result.content.source && result.content.source.name ? result.content.source.name : "";
newsArticle.destination = result.destination;
// The following are the different destination URLs we support (newsArticle.destination)
// OLD FORMAT (To be eventually removed)
//      article://33259198
// NEW FORMAT (various urls below, see data contract spec for full details)
//      //application/view?entitytype=article&pageId=33259198
//      bingnews://application/view?entitytype=article&pageId=33259198
//      ///view?entitytype=article&pageId=2123125
//      bingnews:///view?entitytype=article&pageId=2123125
var newsUri = createURIFunction(newsArticle.destination);
if (newsUri) {
if (newsUri.entitytype === "article") {
newsArticle.type = newsUri.entitytype;
newsArticle.articleid = newsUri.pageId && newsUri.pageId.length > 0 ? newsUri.pageId : null;
newsArticle.contentid = newsUri.contentId && newsUri.contentId.length > 0 ? newsUri.contentId : null;
}
else if (newsUri.entitytype === "slideshow") {
newsArticle.type = newsUri.entitytype;
newsArticle.contentid = newsUri.pageId;
}
else if (newsUri.entitytype === "video") {
if (!newsArticle.thumbnail) {
newsArticle.thumbnail = {
width: 0,
height: 0,
url: "",
name: ""
};
}
newsArticle.type = newsUri.entitytype;
newsArticle.thumbnail.videoOptions = {
videoSource: newsUri.contentId,
source: newsArticle.source,
title: newsArticle.title,
sourceImageUrl: newsArticle.sourceImageUrl,
thumbnail: newsArticle.thumbnail.url
};
}
}
// CMS abstract support HTML Fragments
newsArticle.abstract = result.content.abstract ? toStaticHTML(result.content.abstract) : "";
newsArticle.snippet = newsArticle.abstract;
newsArticle.byline = result.byline ? result.byline : "";
var publicationDate = result.content.publicationDate;
function getDateString(dateType) {
if (dateType && dateType.utctime) {
var utcTime = dateType.utctime;
// TODO: Let platform know that this may be off an hour, need to investigate
var time = CommonJS.Utils.convertBDITimeToFriendlyTime(utcTime);
return time && time.indexOf("NaN") < 0 ? time : "";
}
else {
return "";
}
}
if (publicationDate) {
newsArticle.updatedTime = getDateString(publicationDate.updated);
newsArticle.publishTime = getDateString(publicationDate.published);
}
newsArticle.template = result.template;
newsArticle.templateClass = result.templateClass;
return newsArticle;
}
});
})(WinJS);
/* File: FilmStrip.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsFilmStripInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// A filmstrip is used to show multiple entities on the same page. An entity is a
/// particular and discrete unit, such as a Microsoft, Apple, San Francisco, CA. Information
/// about a multiple entities may be presented on a single page with a ListView and Filmstrip control.
/// The diagram below shows how the filmstrip control works. . When the view for the current entity
/// reaches its end, the filmstrip is updated to reflect the next entity. The filmstrip control is
/// connected to a ListView control and Header control.  The Filmstrip is informed by the ListView of its
/// current position and what entity it is currently showing. The header control is updated by
/// the filmstrip control to reflect the current entity showing.
/// The film strip control
/// Handles multisize tiles and grouping by default
/// </summary>
FilmStrip: WinJS.Class.define(function filmStrip_ctor(domElement, options) {
/// <summary>
/// Constructor
/// </summary>
var stripDom = document.createElement("div"),
that = this;
this._leftButton = document.createElement("button");
this._rightButton = document.createElement("button");
WinJS.Utilities.addClass(stripDom, "platformFilmstripList");
stripDom.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/platform/filmstrip"));
this._leftButton.onclick = function () {
that._leftButtonClick();
};
WinJS.Utilities.addClass(this._leftButton, "platformFilmstripButton platformFilmstripLeftButton");
this._leftButton.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/platform/previousButton"));
this._rightButton.onclick = function () {
that._rightButtonClick();
};
this._rightButton.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/platform/nextButton"));
WinJS.Utilities.addClass(this._rightButton, "platformFilmstripButton platformFilmstripRightButton");
this._filmStripList = new WinJS.UI.ListView(stripDom, { layout: { type: WinJS.UI.GridLayout }, swipeBehavior: "none" });
this._filmStripList.layout.maxRows = 1;
this._filmStripList.itemTemplate = function (itemPromise) {
return {
element: itemPromise.then(function filmStrip_listItemPromiseComplete(item) {
var div = document.createElement("div");
WinJS.Utilities.addClass(div, "platformFilmStripItem");
div.innerText = item.data.itemTitle ? item.data.itemTitle : item.data.headerTitle;
return div;
})
};
};
this._filmStripList.addEventListener("iteminvoked", function filmStrip_onItemInvoked(event) {
that._needToScroll = true;
that.currentSelectedItem = event.detail.itemIndex;
// Collapse the filmstrip if necessary
that.isCollapsed = true;
});
this._domElement = domElement ? domElement : document.createElement("div");
this._filmstripBar = document.createElement("div");
WinJS.Utilities.addClass(this._domElement, "platformFilmStrip");
WinJS.Utilities.addClass(this._filmstripBar, "platformFilmStripBar");
CommonJS.setAutomationId(this._domElement);
CommonJS.setAutomationId(stripDom, this._domElement, "list");
CommonJS.setAutomationId(this._leftButton, this._domElement, "left");
CommonJS.setAutomationId(this._rightButton, this._domElement, "right");
this._filmstripBar.appendChild(stripDom);
this._domElement.appendChild(this._filmstripBar);
this._buttonsShown = false;
this.showButtons = true;
this._initialized = false;
this._scrolling = false;
this._listViewInitialized = false;
this._filmstripInitialized = false;
this._currentSelectedItem = 0;
this._domElement.winControl = this;
this._isCollapsed = false;
this._isCollapsible = false;
WinJS.UI.setOptions(this, options);
},
{
_filmstripBar: null,
_leftButton: null,
_rightButton: null,
_filmstripInitialized: false,
_listViewInitialized: false,
_filmStripList: null,
_domElement: null,
_headerControl: null,
_listView: null,
_filmstripData: null,
_currentSelectedItem: 0,
_ignoreScrolling: false,
_needToScroll: false,
_buttonsShown: false,
_launchingPoint: null,
_isCollapsed: false,
_isCollapsible: false,
/// <field> Flag indicating if the filmstrip is collapsed </field>
isCollapsed: {
set: function set(value) {
if (this.isCollapsible) {
this._isCollapsed = value;
if (this._isCollapsed) {
WinJS.Utilities.addClass(this._domElement, "platformFilmstripCollapsed");
} else {
WinJS.Utilities.removeClass(this._domElement, "platformFilmstripCollapsed");
}
}
},
get: function get(value) {
return this._isCollapsed;
}
},
/// <field> Flag indicating if the filmstrip is collapsible </field>
isCollapsible: {
set: function set(value) {
this._isCollapsible = value;
var that = this;
if (!this._launchingPoint) {
this._launchingPoint = document.createElement("div");
WinJS.Utilities.addClass(this._launchingPoint, "platformFilmstripLaunchingPoint");
this._launchingPoint.onclick = function () {
that.isCollapsed = false;
};
this._domElement.appendChild(this._launchingPoint);
}
},
get: function get() {
return this._isCollapsible;
}
},
/// <field>
/// Show navigation buttons
/// </field>
showButtons: {
set: function set(value) {
if (value && !this._buttonsShown) {
this._filmstripBar.appendChild(this._leftButton);
this._filmstripBar.appendChild(this._rightButton);
} else if (!value && this._buttonsShown) {
this._filmstripBar.removeChild(this._leftButton);
this._filmstripBar.removeChild(this._rightButton);
}
this._buttonsShown = value;
}
},
_leftButtonClick: function _leftButtonClick() {
/// <summary>
/// Left click handler
/// </summary>
this._needToScroll = true;
this.currentSelectedItem = this.currentSelectedItem - 1;
},
_rightButtonClick: function _rightButtonClick() {
/// <summary>
/// Right click handler
/// </summary>
this._needToScroll = true;
this.currentSelectedItem = this.currentSelectedItem + 1;
},
/// <field>
/// Item renderer for the film strip list
/// </field>
itemRenderer: {
set: function set(value) {
this._filmStripList.itemTemplate = value;
}
},
/// <field>
/// The header control
/// </field>
headerControl: {
set: function set(value) {
this._headerControl = PlatformJS.Utilities.getControl(value);
if (!this._headerControl) {
this._headerControl = value;
}
}
},
/// <field>
/// The currently selected entity index
/// </field>
currentSelectedItem: {
get: function get() {
return this._currentSelectedItem;
},
set: function set(value) {
this._currentSelectedItem = value;
if (this._listViewInitialized &&
this._filmstripInitialized) {
if (this._filmstripData &&
value >= 0 && value < this._filmstripData.length) {
var currentItem = this._filmstripData[value];
this._filmStripList.selection.set(value);
this._headerControl.headerTitleText = currentItem.headerTitle;
this._filmStripList.ensureVisible(value);
if (this._needToScroll) {
this._ignoreScrolling = true;
this._needToScroll = false;
this._listView.indexOfFirstVisible = currentItem.itemOffset;
}
this._leftButton.disabled = (value === 0);
this._rightButton.disabled = (value === this._filmstripData.length - 1);
} else {
PlatformJS.Utilities.onError("Invalid current selected item");
}
}
}
},
findEntityIndexForItemOffset: function findEntityIndexForItemOffset(itemOffset) {
var that = this,
firstVisible = itemOffset,
index = that.currentSelectedItem,
item = that._filmstripData[that.currentSelectedItem],
lowerBound = 0,
upperBound = 0,
nextItem = null;
if (item.itemOffset < 0) {
PlatformJS.Utilities.onError("The itemOffset of a Filmstrip datasource must be zero or greater");
}
if (firstVisible > item.itemOffset) {
lowerBound = item.itemOffset;
if (index + 1 < that._filmstripData.length) {
nextItem = that._filmstripData[index + 1];
upperBound = nextItem.itemOffset;
if (nextItem.itemOffset < 0) {
PlatformJS.Utilities.onError("The itemOffset of a Filmstrip datasource must be zero or greater");
}
}
while (nextItem &&
!(lowerBound <= firstVisible && upperBound > firstVisible) &&
index + 1 < that._filmstripData.length) {
index++;
item = nextItem;
lowerBound = item.itemOffset;
nextItem = that._filmstripData[index + 1];
if (nextItem) {
upperBound = nextItem.itemOffset;
if (nextItem.itemOffset < 0) {
PlatformJS.Utilities.onError("The itemOffset of a Filmstrip datasource must be zero or greater");
}
}
}
} else if (firstVisible < item.itemOffset) {
upperBound = item.itemOffset;
if (index - 1 >= 0) {
nextItem = that._filmstripData[index - 1];
lowerBound = nextItem.itemOffset;
if (nextItem.itemOffset < 0) {
PlatformJS.Utilities.onError("The itemOffset of a Filmstrip datasource must be zero or greater");
}
index--;
}
while (nextItem &&
!(lowerBound <= firstVisible && upperBound > firstVisible) &&
index - 1 >= 0) {
index--;
item = nextItem;
upperBound = item.itemOffset;
nextItem = that._filmstripData[index - 1];
if (nextItem) {
lowerBound = nextItem.itemOffset;
if (nextItem.itemOffset < 0) {
PlatformJS.Utilities.onError("The itemOffset of a Filmstrip datasource must be zero or greater");
}
}
}
}
return index;
},
/// <field>
/// The listview to hook into
/// </field>
listView: {
set: function set(value) {
var that = this;
this._listView = PlatformJS.Utilities.getControl(value);
if (!this._listView) {
PlatformJS.Utilities.onError("Invalid listview object");
}
this._listView.addEventListener("loadingstatechanged", function filmStrip_onLoadingStateChanged(event) {
if (!that._filmstripData) {
return;
}
if (that._listView.loadingState && !(that._listView.loadingState === "itemsLoaded" || that._listView.loadingState === "completed")) {
return;
}
if (!that._listViewInitialized) {
that._listViewInitialized = true;
that._needToScroll = true;
that.currentSelectedItem = that._currentSelectedItem;
return;
}
if (that._ignoreScrolling) {
that._ignoreScrolling = false;
return;
}
// Check if this is defined, if it is check if it is true
var index = that.findEntityIndexForItemOffset(that._listView.indexOfFirstVisible);
if (index !== that.currentSelectedItem &&
index >= 0 &&
index < that._filmstripData.length) {
var item = that._filmstripData[index];
that._needToScroll = false;
that.currentSelectedItem = index;
}
});
}
},
/// <field>
/// An array of the following items:
/// FilmStripItem:
/// {
///    headerTitle: string,
///    itemTitle: string (optional),
///    itemOffset: number
/// }
/// </field>
dataSource: {
set: function set(value) {
this._filmstripData = value;
var that = this;
var fx = function () {
if (that._filmStripList.loadingState === "complete") {
that._filmStripList.removeEventListener("loadingstatechanged", fx);
that._filmstripInitialized = true;
that._changeHandled = false;
that.currentSelectedItem = that._currentSelectedItem;
}
};
this._filmStripList.addEventListener("loadingstatechanged", fx);
this._filmstripInitialized = false;
this._filmStripList.itemDataSource = new WinJS.Binding.List(value).dataSource;
}
}
})
});
})();
/* File: ImmersiveFilmStrip.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsRealizedListInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
/// <summary>
/// A filmstrip is used to show multiple entities on the same page. An entity is a
/// particular and discrete unit, such as a Microsoft, Apple, San Francisco, CA.
/// </summary>
FilmStrip: WinJS.Class.derive(CommonJS.FilmStrip, function filmStrip_ctor(domElement, options) {
/// <summary>
/// Constructor
/// </summary>
this._changeHandled = false;
CommonJS.FilmStrip.call(this, domElement, options);
},
{
_entityControl: null,
_changeHandled: false,
/// <field>
/// The currently selected entity index
/// </field>
currentSelectedItem: {
get: function get() {
return this._currentSelectedItem;
},
set: function set(value) {
this._currentSelectedItem = value;
if (this._filmstripInitialized) {
if (this._filmstripData &&
value >= 0 && value < this._filmstripData.length) {
var currentItem = this._filmstripData[value];
this._filmStripList.selection.set(value);
this._headerControl.headerTitleText = currentItem.headerTitle;
if (!this._changeHandled) {
this._entityControl.showEntityForIndex(this._filmstripData[value].itemOffset);
}
this._changeHandled = false;
this._filmStripList.ensureVisible(value);
this._leftButton.disabled = (value === 0);
this._rightButton.disabled = (value === this._filmstripData.length - 1);
} else {
PlatformJS.Utilities.onError("Invalid current selected item");
}
}
}
},
/// <field>
/// The entity control to hook into
/// </field>
entityControl: {
set: function set(value) {
this._entityControl = PlatformJS.Utilities.getControl(value);
if (!this._entityControl) {
this._entityControl = value;
}
var that = this;
this._entityControl.onItemChanged = function (event) {
that._onItemChanged(event);
};
}
},
_onItemChanged: function _onItemChanged(event) {
/// <summary>Handles item changed event</summary>
/// <param name="event">Event object</param>
this._changeHandled = true;
this.currentSelectedItem = this.findEntityIndexForItemOffset(event.detail.index);
},
})
});
})();
/* File: FeaturedCluster.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// Base class for the featured sources cluster
/// </summary>
FeaturedCluster: WinJS.Class.define(function (domElement, options) {
this._domElement = domElement;
this._createContainerControl();
this._currentMarket = Platform.Utilities.Globalization.getCurrentMarket().toLocaleLowerCase();
// set options
WinJS.UI.setOptions(this, options);
},
{
_currentMarket: "en-us",
_view: null,
_clusterItems: null,
featuredCategory: "",
featuredSources: null,
// The user can provide a predefined data set to be used for rendering. 
dataSet: null,
render: function render() {
var that = this;
if (this.dataSet && this.dataSet.length > 0) {
return this._update(this.dataSet);
}
else {
return this._getQueryServicePromise().then(function renderSuccess(response) {
var parsedData = that._parseData(response);
return that._update(parsedData);
},
function renderFailure(error) {
return WinJS.Promise.wrapError({});
});
}
},
currentMarket: {
set: function set(value) {
this._currentMarket = value;
}
},
// Default click handler. Users can override this with their own handlers.
featuredItemClick: function featuredItemClick(event) {
var item = event.detail.item.data;
if (item.pageInfo && item.state) {
if (item.state.dynamicInfo) {
item.state.dynamicInfo.entrypoint = "featuredsourcescluster";
}
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.featuredSourcesCluster);
WinJS.Navigation.navigate(item.pageInfo, item.state);
}
},
moduleInfo: {
isInteractive: false,
fragmentPath: "/common/FeaturedCluster/html/featuredClusterTemplates.html",
templateId: "moreSourceItemTemplate"
},
dispose: function dispose() {
if (this._view) {
this._view.dispose();
}
},
_getQueryServicePromise: function _getQueryServicePromise(urlParams, payloadParams, transformer, options) {
var that = this;
return new WinJS.Promise(function (complete, error, progress) {
var cacheInstance = PlatformJS.Cache.CacheService.getInstance("ConfigurationCache");
var searchEntry = that._currentMarket + "-Features";
PlatformJS.Navigation.mainNavigator.channelManager.downloadFeaturesConfigAsync().then(function downloadComplete() {
cacheInstance.findEntry(searchEntry, { supportsInMemory: true })
.then(function getFeaturesConfig_Complete(response) {
var returnValue = null;
if (response && response.dataValue) {
returnValue = response.dataValue;
}
complete(returnValue);
}, function () {
complete(null);
});
}, function downloadError() {
complete(null);
});
});
},
_parseData: function _parseData(jsonResponse) {
var that = this;
var featuredChannels = [];
var fc = this.featuredCategory;
var mn = PlatformJS.Navigation.mainNavigator.channelManager;
if (jsonResponse && jsonResponse.navbaritems && jsonResponse.sources) {
var featuredSources = {};
var sources = jsonResponse.sources;
for (var idx = 0, len = sources.length; idx < len; idx++) {
if (sources[idx] && sources[idx].id) {
var sourceId = sources[idx].id.toLowerCase();
featuredSources[sourceId] = sources[idx];
}
}
this.featuredSources = featuredSources;
jsonResponse.navbaritems.forEach(function (navBarItem) {
if (navBarItem.destination.startsWith("//sources/")) {
var channelInfo = mn.parseChannel(navBarItem, jsonResponse.sources);
if (channelInfo) {
/* Id is the string after "//sources/"  hence substring(10). get the panotitle and 
displayname of the id from sources array. */
var sourceId = navBarItem.destination.substring(10).toLowerCase();
var sourceItem = that.featuredSources[sourceId];
if (sourceItem
&& sourceItem.clusterlogo
&& sourceItem.displayname) {
channelInfo.panoTitle = sourceItem.displayname;
channelInfo.panoDescription = sourceItem.description ? sourceItem.description : "";
channelInfo.clusterlogo = sourceItem.clusterlogo;
channelInfo.featured_url = sourceItem.featuredurl;
channelInfo.ad_unit_id = sourceItem.adunitid;
channelInfo.instrumentation_id = sourceItem.instrumentationid;
featuredChannels.push(channelInfo);
}
}
}
});
}
return featuredChannels.length > 0 ? featuredChannels : null;
},
_createContainerControl: function _createContainerControl() {
var parent = document.createElement("div");
var child = document.createElement("div");
this._domElement.appendChild(parent);
var className = "platformListLayout moreSourceList";
this._view = new CommonJS.Immersive.ItemsContainer(child,
{
className: className,
onitemclick: this.featuredItemClick.bind(this)
});
parent.appendChild(child);
},
_update: function _update(data) {
var that = this;
var view = this._view;
if (data && data.length > 0) {
this.featuredSources = data;
this._clusterItems = this._prepareFeaturedSources(data);
WinJS.UI.setOptions(view, {
itemDataSource: this._clusterItems.dataSource
});
return view.render();
}
return WinJS.Promise.wrap({});
},
_createSourceContent: function _createSourceContent(entity) {
// News uses its own renderer function, instead of this.moduleInfo
if (entity.moduleInfo) {
return {
data: entity.data,
moduleInfo: entity.moduleInfo
};
} else {
return {
data: entity,
moduleInfo: this.moduleInfo
};
}
},
_prepareFeaturedSources: function _prepareFeaturedSources(entities) {
var clusterItems = new WinJS.Binding.List();
for (var i = 0; i < entities.length; i++) {
clusterItems.push(this._createSourceContent(entities[i]));
}
return clusterItems;
}
})
});
})();
/* File: EdgyFlyout.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsECEdgyFlyoutInit () {
"use strict";
WinJS.Namespace.define("CommonJS", {
EdgyFlyout: WinJS.Class.define(function edgyFlyout_ctor (element, options) {
this.element = element || document.createElement("button");
var t = this;
options.onclick = function () { t._onClick.call(t); };
var appbarOptions = {
type: "flyout",
icon: options.icon,
label: options.buttonText
};
if (options.extraClass) {
appbarOptions.extraClass = options.extraClass;
}
this.appbarbtn = new WinJS.UI.AppBarCommand(this.element, appbarOptions);
WinJS.UI.setOptions(this, options);
}, {
_onClick: function _onClick() {
this.show();
},
_buildControl: function _buildControl() {
this._buildFlyout();
},
_buildFlyout: function _buildFlyout() {
if (this.flyoutContainer) {
return;
}
var div = document.createElement("div");
div.className = "edgyFlyout " + (this.flyoutClassName ? this.flyoutClassName : "");
document.body.appendChild(div);
var flyout = this.flyout = new WinJS.UI.Flyout(div);
// must set after building the flyout, as this.flyout's setter tries to grab the flyout winControl
this.flyoutContainer = div;
this.appbarbtn.flyout = flyout;
},
flyoutClassName: {
get: function get() {
return this._flyoutClassName;
},
set: function set(value) {
this._flyoutClassName = value;
}
},
flyoutContent: {
get: function get() {
return this._flyoutContent;
},
set: function set(value) {
this._flyoutContent = value;
}
},
buttonText: {
get: function get() {
return this._buttonText;
},
set: function set(value) {
this._buttonText = value;
}
},
icon: {
get: function get() {
return this._icon;
},
set: function set(value) {
this._icon = value;
}
},
show: function show() {
//  this._flyout.show(this.element);
},
hide: function hide() {
this.flyout.hide();
},
dispose: function dispose() {
if (this.flyout) {
this.flyoutContainer.innerHTML = "";
if (this.flyoutContainer.parentNode) {
this.flyoutContainer.parentNode.removeChild(this.flyoutContainer);
}
this.appbarbtn = null;
this.flyout = null;
}
}
// properties go here
})
});
})();
/* File: Flyout.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsFlyoutInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// Flyout control.
/// Renders a full screen flyout dialog
/// </summary>
Flyout: WinJS.Class.define(
function flyout_ctor(element, options) {
/// <summary>
/// Constructor: Creates an instance of the Flyout Control object.
/// </summary>
/// <returns>
/// An instance of the Flyout control object.
/// </returns>
element = element || document.createElement("div");
element.winControl = this;
this._isSticky = false;
this._init(element);
WinJS.UI.setOptions(this, options);
},
// Instance members
{
_domElement: null,
_contentFragment: null,
_flyoutContent: null,
//_flyoutCloseButton: null,
_isSticky: false,
onCloseButtonClick: null,
onReady: null,
isSticky: {
set: function set(value) {
this._isSticky = value;
this._domElement.style.zIndex = 2500;
},
get: function get() {
return this._isSticky;
}
},
contentFragment: {
get: function get() {
return this._contentFragmentPath;
},
set: function set(value) {
var that = this;
this._contentFragmentPath = value;
if (!value.path) {
PlatformJS.Utilities.onError("Fragment path must not be null");
}
CommonJS.loadModule({ fragmentPath: value.path, templateId: value.id }, value.data, this._flyoutContent).then(function flyout_loadModuleComplete() {
that.show();
});
}
},
buttonText: {
set: function set(value) {
//this._flyoutCloseButton.textContent = value;
}
},
onHide: null,
hide: function hide() {
if (this.navEvent) {
WinJS.Navigation.removeEventListener("navigating", this.navEvent);
}
if (this.layoutChangeEvent) {
window.removeEventListener("resize", this.layoutChangeEvent);
}
if (CommonJS._currentFlyout && this._domElement) {
WinJS.Utilities.removeClass(this._domElement, "platformFlyoutVisible");
this._domElement.outerHTML = "";
}
if (this.onHide) {
this.onHide();
}
CommonJS._currentFlyout = null;
},
_init: function _init(element) {
this._domElement = element;
element.id = "platformFlyout";
element.setAttribute("role", "alertdialog");
//TODO: register to all dismissing event (touch, ...)
//[BEGIN]
var that = this;
element.addEventListener("keydown", function flyout_onKeyDown(event) {
if (event.keyCode === 27) {
that.hide();
}
});
//[END]
var flyoutContainer = document.createElement("div");
this._flyoutContent = document.createElement("div");
//this._flyoutCloseButton = document.createElement("button");
//this._flyoutCloseButton.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/platform/close"));
//this._flyoutCloseButton.addEventListener("click", function (e) { that._onCloseButtonClick(e); }, false);
//CommonJS.setAutomationId(this._flyoutCloseButton, element, "close");
//WinJS.Utilities.addClass(this._flyoutCloseButton, "win-commandicon win-large platformFlyoutCloseButton iconCancel ");
WinJS.Utilities.addClass(element, "platformFlyout");
WinJS.Utilities.addClass(flyoutContainer, "platformFlyoutContainer");
WinJS.Utilities.addClass(this._flyoutContent, "platformFlyoutContent");
flyoutContainer.appendChild(this._flyoutContent);
//flyoutContainer.appendChild(this._flyoutCloseButton);
element.appendChild(flyoutContainer);
},
_loadContent: function _loadContent(path, destination) {
msWriteProfilerMark("Platform:Flyout:loadContent:s");
WinJS.UI.Fragments.cloneTo(path, null, destination).then(function flyout_cloneToComplete(frag) {
WinJS.UI.processAll(frag).then(function flyout_fragmentProcessAllComplete() { msWriteProfilerMark("Platform:Flyout:loadContent:e"); });
},
function flyout_cloneToError() { });
},
navEvent: null,
layoutChangeEvent: null,
show: function show() {
if (!CommonJS._currentFlyout) {
CommonJS._currentFlyout = this;
document.body.appendChild(this._domElement);
WinJS.Utilities.addClass(this._domElement, "platformFlyoutVisible");
this._onReady();
if (!this.isSticky) {
var that = this;
this.navEvent = function () {
that.hide();
};
WinJS.Navigation.addEventListener("navigating", this.navEvent);
this.layoutChangeEvent = function () {
var viewState = Windows.UI.ViewManagement.ApplicationView.value;
if (viewState === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
that.hide();
}
};
//TODO://Should this be listening to the platform, instead of registering on its own
window.addEventListener("resize", this.layoutChangeEvent);
}
}
},
_onCloseButtonClick: function _onCloseButtonClick(event) {
if (!this.isSticky) {
this.hide();
}
if (this.onCloseButtonClick) {
this.onCloseButtonClick();
}
},
_onReady: function _onReady() {
if (typeof this.onReady === "function") {
this.onReady();
}
},
closeButtonHidden: {
set: function set(value) {
//if (value) {
//    WinJS.Utilities.addClass(this._flyoutCloseButton, "platformHide");
//} else {
//    WinJS.Utilities.removeClass(this._flyoutCloseButton, "platformHide");
//}
}
}
})
});
})();
/* File: SelectionEdgyFlyout.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexCommonControlsECSortFlyoutInit () {
"use strict";
WinJS.Namespace.define("CommonJS", {
SortFlyout: WinJS.Class.derive(CommonJS.EdgyFlyout, function sortFlyout_ctor (element, options) {
CommonJS.EdgyFlyout.apply(this, [element, options]);
WinJS.UI.setOptions(this, options);
this._buildSortControl();
},
{
flyoutClassName: {
get: function get() {
return this._flyoutClassName;
},
set: function set(value) {
this._flyoutClassName = value;
}
},
onSortChanged: {
get: function get() {
return this._onSortChanged || function sortFlyout_onSortChangedNoop () { };
},
set: function set(value) {
this._onSortChanged = value;
}
},
Items: {
get: function get() {
return this.getItems();
},
set: function set(value) {
this.setItems(value);
}
},
selectionChanged: {
get: function get() {
return this._selectionChanged;
},
set: function set(value) {
this._selectionChanged = value;
}
},
flyoutHeader: {
get: function get() {
return this._flyoutHeader;
},
set: function set(value) {
var div = document.createElement("div");
div.textContent = value;
this._flyoutHeader = div;
},
},
setItems: function setItems(items) {
this._items = items;
this._setItems(items);
},
_buildSortControl: function _buildSortControl() {
var header = this.flyoutHeader;
this._buildControl(); // build flyout
this._buildScrollList();
},
_selectionChange: function _selectionChange(event) {
console.log("edgy sort changed");
var indexStr = event.currentTarget.getAttribute("index");
var index = parseInt(indexStr);
var item = this._items[index];
item.selectedIndex = index; // compat with existing codepath for dropdown filter
var that = this;
// give time for the selection UI to appear before hiding the menu
setTimeout(function sortFlyout_afterSelectionChange () {
that.hide();
var appbars = document.querySelectorAll("[data-win-control='WinJS.UI.AppBar']");
if (appbars && appbars.length > 0) {
for (var i = 0, l = appbars.length; i < l; i++) {
appbars[i].winControl.hide();
}
}
}, 500);
if (this.selectionChanged) {
this.selectionChanged(item);
}
},
_setItems: function _setItems(items) {
var itemsDiv = this.itemsDiv;
itemsDiv.innerHTML = "";
this._items = items;
for (var i = 0, l = items.length; i < l; i++) {
var item = items[i];
item.index = i;
this._addItem(itemsDiv, "edgySort", item);
}
return itemsDiv;
},
_buildScrollList: function _buildScrollList() {
var itemsDiv = document.createElement("div");
var that = this;
itemsDiv.className = "edgySelectableFilterItems";
itemsDiv.setAttribute("aria-role", "menu");
itemsDiv.addEventListener("change", function sortFlyout_onItemChange (event) { that._selectionChange.call(that, event); });
this._selectionContainer = itemsDiv;
this.flyout.element.appendChild(itemsDiv);
this.itemsDiv = itemsDiv;
},
_deselectAll: function _deselectAll(index) {
var items = document.querySelectorAll("sort-item");
for (var i = 0; i < items.length; i++) {
WinJS.Utilities.removeClass(items[i], "sort-checked");
}
},
_addItem: function _addItem(itemsContainer, filterName, item) {
var itemText = item.label;
var itemValue = item.value;
var checked = item.selected;
var index = item.index;
var that = this;
var row = document.createElement("div");
var validKeys = { 13: true, /*enter*/ 32: true /*space*/ };
var onSelection = function (evt) {
if (evt.keyCode) {
if (!validKeys[evt.keyCode]) {
return;
}
}
that._selectionChange.call(that, evt);
that._deselectAll(index);
WinJS.Utilities.addClass(evt.currentTarget, "sort-checked");
};
row.addEventListener("click", onSelection);
row.addEventListener("keyup", onSelection);
row.className = "sort-item";
row.setAttribute("tabindex", 0);
row.setAttribute("role", "menuitem");
row.setAttribute("value", itemValue);
row.setAttribute("index", index);
var check = document.createElement("div");
check.className = "sort-item-check";
row.appendChild(check);
if (checked) {
WinJS.Utilities.addClass(check, "sort-checked");
}
var text = document.createElement("div");
text.className = "sort-text";
text.textContent = itemText;
row.appendChild(text);
itemsContainer.appendChild(row);
},
})
});
})();
/* File: Header.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsHeaderInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
/// <summary>
/// Header control.
/// Renders Title, sub-title, back button, filterBar and header image
/// </summary>
Header: WinJS.Class.define(function header_ctor(element, options) {
/// <summary>
/// Constructor: Creates an instance of the Header Control object.
/// </summary>
/// <returns>
/// An instance of the Header control object.
/// </returns>
var that = this;
element = element || document.createElement("div");
element.winControl = this;
this._headerBackButton = null;
this._init(element);
WinJS.UI.setOptions(this, options);
// Only make the header small if we have a subtitle or a filter bar
// if (that._headerSubTitleText.length > 0 || that._filterBar) {
//     WinJS.Utilities.removeClass(that._platformHeaderTitle, "immersiveHeaderTitleBig");
// }
that._headerRenderComplete = true;
},
// Instance members
{
_domElement: null,
_dataSource: null,
_headerTitleText: "",
_headerSubTitleText: "",
_headerBackButton: null,
_platformHeaderSubTitle: null,
_platformHeaderTitle: null,
_fontColor: null,
_headerTitleSentenceCasing: false,
_filterBar: null,
_filterBarData: null,
_filterControls: [],
_hasFilterRendered: false,
_offsetWidth: null,
_offsetHeight: null,
_direction: null,
_clippedElement: null,
_onSubTitleClick: null,
dispose: function dispose() {
if (this._onBackButtonBlurredHandler) {
this._headerBackButton.removeEventListener("blur", this._onBackButtonBlurredHandler);
this._onBackButtonBlurredHandler = null;
}
if (this._onBackButtonFocusedHandler) {
this._headerBackButton.removeEventListener("focus", this._onBackButtonFocusedHandler);
this._onBackButtonFocusedHandler = null;
}
if (this._onBackButtonClickHandler) {
this._headerBackButton.removeEventListener("click", this._onBackButtonClickHandler);
document.removeEventListener("keydown", this._onBackButtonClickHandler);
this._onBackButtonClickHandler = null;
}
if (this._subtitleClickHandlerHandler) {
this._platformHeaderSubTitle.removeEventListener("click", this._subtitleClickHandlerHandler);
this._subtitleClickHandlerHandler = null;
}
if (this._subtitleKeyUpHandlerHandler) {
this._platformHeaderSubTitle.removeEventListener("keyup", this._subtitleKeyUpHandlerHandler);
this._subtitleKeyUpHandlerHandler = null;
}
if (this._filterBar) {
this._filterBar.dispose();
this._filterBar = null;
}
// break the connection between js and html, for the garbage collector to recycle.
if (this._domElement) {
this._domElement.winControl = null;
this._domElement = null;
}
},
initFilterBar: function initFilterBar(filterDisplayUI) {
var filterBarDiv = document.createElement("div");
var options = {};
filterBarDiv.setAttribute("role", "menubar");
filterBarDiv.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/platform/filterMenu"));
filterBarDiv.setAttribute("id", "immersiveFilterBar");
WinJS.Utilities.addClass(filterBarDiv, "immersiveFilterBar");
CommonJS.setAutomationId(filterBarDiv);
this._domElement.appendChild(filterBarDiv);
this._filterBar = new CommonJS.Filters.FilterMenu(filterBarDiv, options);
if (filterDisplayUI) {
this._filterBar.onFlyoutShowing = function (event) {
WinJS.Utilities.removeClass(filterDisplayUI, "ecvHide");
};
this._filterBar.onFlyoutHidden = function (event) {
WinJS.Utilities.addClass(filterDisplayUI, "ecvHide");
};
}
},
onSubTitleClick: {
get: function get() {
return this._onSubTitleClick;
},
set: function set(onSubTitleClick) {
this._onSubTitleClick = onSubTitleClick;
}
},
filterControls: {
get: function get() {
return this._filterBarData;
},
set: function set(value) {
if (value && value.length > 0) {
// WinJS.Utilities.removeClass(this._platformHeaderTitle, "immersiveHeaderTitleBig");
this._filterBarData = value;
this._updateFilterBar();
}
},
},
headerTitleText: {
get: function get() {
return this._headerTitleText;
},
set: function set(value) {
this._headerTitleText = value;
this._platformHeaderTitle.innerText = value;
if (value !== "") {
this._platformHeaderTitle.setAttribute("role", "heading");
this.headerImage = null;
}
}
},
headerSubTitleText: {
get: function get() {
return this._headerSubTitleText;
},
set: function set(value) {
value = value.trim() || "";
var element = this._platformHeaderSubTitle;
element.innerHTML = value;
this._headerSubTitleText = value;
if (value.length === 0) {
// WinJS.Utilities.addClass(this._platformHeaderTitle, "immersiveHeaderTitleBig");
element = this._platformHeaderTitle;
}
else {
this._resetHeader(this._platformHeaderTitle);
// WinJS.Utilities.removeClass(this._platformHeaderTitle, "immersiveHeaderTitleBig");
this._platformHeaderSubTitle.setAttribute("role", "heading");
this.headerImage = null;
}
this._offsetWidth = element.offsetWidth;
this._offsetHeight = element.offsetHeight;
this._clippedElement = element;
}
},
headerImage: {
get: function get() {
return this._headerImage;
},
set: function set(value) {
this._headerImage = value;
if (value && !this.headerTitleText && !this.headerSubTitleText) {
var valueType = typeof value;
if (valueType === "string") {
this._platformHeaderImage.src = value;
WinJS.Utilities.removeClass(this._platformHeaderImage, "platformVisibilityNone");
} else if (valueType === "object" && value.url && typeof value.url === "string") {
this._platformHeaderImage.src = value.url;
if (value.width && value.height) {
this._platformHeaderImage.style.width = value.width + "px";
this._platformHeaderImage.style.height = value.height + "px";
}
WinJS.Utilities.removeClass(this._platformHeaderImage, "platformVisibilityNone");
} else {
WinJS.Utilities.addClass(this._platformHeaderImage, "platformVisibilityNone");
}
} else {
WinJS.Utilities.addClass(this._platformHeaderImage, "platformVisibilityNone");
}
}
},
headerTitleSentenceCasing: {
set: function set(value) {
this._headerTitleSentenceCasing = value;
if (value && this._platformHeaderTitle) {
this._platformHeaderTitle.style.textTransform = "none";
}
}
},
fontColor: {
get: function get() {
return this._fontColor;
},
set: function set(value) {
var oldFontColor = this._fontColor;
this._fontColor = value;
if (this._fontColor && this._fontColor !== "") {
if (oldFontColor) {
WinJS.Utilities.removeClass(this._domElement, oldFontColor);
}
WinJS.Utilities.addClass(this._domElement, this._fontColor);
} else {
this._domElement.className = "";
WinJS.Utilities.addClass(this._domElement, "immersiveHeader");
}
},
},
setTitleState: function setTitleState(hideTitle) {
if (hideTitle !== this._platformHeaderTitle.disabled) {
this._platformHeaderTitle.disabled = hideTitle;
if (hideTitle) {
this._platformHeaderTitle.style.visibility = "hidden";
}
else {
this._platformHeaderTitle.style.visibility = "visible";
}
}
},                
getFilterControl: function getFilterControl(filterName) {
if (this._filterControls[filterName]) {
return this._filterControls[filterName];
} else {
return null;
}
},
getSubTitleWidth: function getSubTitleWidth() {
if (this._platformHeaderSubTitle.clientWidth > 0) {
return this._platformHeaderSubTitle.clientWidth;
} else {
// In the case where there is no subtitle
// Need to scale the header title because the animation may not have finished.
return CommonJS.Immersive.Header.largeHeaderHeight / this._platformHeaderTitle.clientHeight * this._platformHeaderTitle.clientWidth;
}
},
getTitleWidth: function getTitleWidth() {
return this._platformHeaderTitle.clientWidth;
},
subTitleMaxWidth: {
set: function set(value) {
if (value < CommonJS.Immersive.PanoramaPanel.minClusterWidth) {
value = CommonJS.Immersive.PanoramaPanel.minClusterWidth;
}
this._platformHeaderSubTitle.style.maxWidth = value + "px";
}
},
transitionHeader: function transitionHeader(currentPosition, nextClusterPosition) {
var offsetWidth = this._offsetWidth;
var divToClip = this._clippedElement;
var clipWidth = nextClusterPosition - currentPosition /*- CommonJS.Immersive.PanoramaPanel.leftMargin*/ - CommonJS.Immersive.PanoramaPanel.clusterSpacing;
if (nextClusterPosition === Number.POSITIVE_INFINITY) {
this._resetHeader(divToClip);
} else {
if (clipWidth >= 0) {
if (!divToClip.isVisible) {
WinJS.Utilities.removeClass(divToClip, "platformVisibilityNone");
divToClip.isVisible = true;
}
if (offsetWidth > clipWidth) {
//HACK: change values for RTL
if (this._direction) {
var width = offsetWidth;
divToClip.style.clip = "rect(0px," + width + "px, " + this._offsetHeight + "px, " + (width - clipWidth) + "px)";
} else {
divToClip.style.clip = "rect(0px," + clipWidth + "px, " + this._offsetHeight + "px, 0px)";
}
divToClip.style.opacity = clipWidth / offsetWidth;
divToClip.isClipped = true;
} else if (divToClip.isClipped) {
this._resetHeader(divToClip);
}
} else {
if (divToClip.isVisible) {
WinJS.Utilities.addClass(divToClip, "platformVisibilityNone");
divToClip.isVisible = false;
}
}
}
},
_init: function _init(element) {
this._domElement = element;
CommonJS.setAutomationId(this._domElement);
element.setAttribute("role", "banner");
var backButton = this._headerBackButton = document.createElement("button");
backButton.setAttribute("aria-label", CommonJS.resourceLoader.getString("/platform/GoBack"));
CommonJS.setAutomationId(backButton, this._domElement, "backButton");
var hgroup = document.createElement("hgroup");
this._platformHeaderTitle = document.createElement("h1");
this._platformHeaderSubTitle = document.createElement("h2");
this._platformHeaderImage = document.createElement("img");
hgroup.setAttribute("role", "group");
this._platformHeaderTitle.setAttribute("role", "heading");
this._platformHeaderSubTitle.setAttribute("role", "heading");
this._platformHeaderImage.setAttribute("role", "heading");
WinJS.Utilities.addClass(this._headerBackButton, "win-backbutton immersiveBackButton");
WinJS.Utilities.addClass(this._platformHeaderTitle, "immersiveHeaderTitle"); // immersiveHeaderTitleBig");
WinJS.Utilities.addClass(this._platformHeaderSubTitle, "immersiveHeaderSubTitle");
WinJS.Utilities.addClass(this._platformHeaderImage, "immersiveHeaderImage platformVisibilityNone");
WinJS.Utilities.addClass(this._domElement, "immersiveHeader");
CommonJS.setAutomationId(hgroup, this._domElement, "hgroup");
CommonJS.setAutomationId(this._platformHeaderTitle, this._domElement, "title");
CommonJS.setAutomationId(this._platformHeaderSubTitle, this._domElement, "subTitle");
CommonJS.setAutomationId(this._platformHeaderImage, this._domElement, "subTitle");
hgroup.setAttribute("aria-labelledby", this._platformHeaderTitle.id + " " + this._platformHeaderSubTitle.id);
this._domElement.setAttribute("aria-labelledby", this._platformHeaderTitle.id);
this._platformHeaderImage.onerror = function (e) {
WinJS.Utilities.addClass(e.target, "platformVisibilityNone");
};
hgroup.appendChild(this._platformHeaderTitle);
hgroup.appendChild(this._platformHeaderSubTitle);
hgroup.appendChild(this._platformHeaderImage);
this._domElement.appendChild(this._headerBackButton);
this._domElement.appendChild(hgroup);
if (WinJS.Navigation.canGoBack) {
this._headerBackButton.isfocused = false;
this._onBackButtonBlurredHandler = this._onBackButtonBlurred.bind(this);
this._onBackButtonFocusedHandler = this._onBackButtonFocused.bind(this);
this._onBackButtonClickHandler = this._onBackButtonClick.bind(this);
this._headerBackButton.addEventListener("blur", this._onBackButtonBlurredHandler);
this._headerBackButton.addEventListener("focus", this._onBackButtonFocusedHandler);
this._headerBackButton.addEventListener("click", this._onBackButtonClickHandler);
document.addEventListener("keydown", this._onBackButtonClickHandler, false);
} else {
this._headerBackButton.disabled = "disabled";
WinJS.Utilities.addClass(this._headerBackButton, "platformHide");
}
this._subtitleClickHandlerHandler = this._subtitleClickHandler.bind(this);
this._subtitleKeyUpHandlerHandler = this._subtitleKeyUpHandler.bind(this);
this._platformHeaderSubTitle.addEventListener("click", this._subtitleClickHandlerHandler, false);
this._platformHeaderSubTitle.addEventListener("keyup", this._subtitleKeyUpHandlerHandler, false);
this._direction = (window.getComputedStyle(document.body).direction === "rtl");
},
_updateFilterBar: function _updateFilterBar() {
msWriteProfilerMark("Platform:FilterBar:updateLayout:s");
if (!this._filterBar) {
this._initFilterBar();
}
//  this._filterBar.setTitle(CommonJS.resourceLoader.getString("/platform/FilterBy"));
if (this._filterBarData.length > 0) {
var onDismiss = this._filterBarData[0].onclosed;
this._filterBar.onDismiss = onDismiss;
this._filterBar.onSelectionChanged = this._filterBarData[0].onselectionchanged;
}
this._filterBar.filterGroupModels = this._translateFilterModels(this._filterBarData);
msWriteProfilerMark("Platform:FilterBar:updateLayout:e");
},
_translateFilterModels: function _translateFilterModels(oldFilters) {
var filterGroups = [];
for (var i = 0, l = oldFilters.length; i < l; i++) {
filterGroups.push(this._translateFilterModel(oldFilters[i]));
}
return filterGroups;
},
_translateFilterModel: function _translateFilterModel(oldFilter) {
var items = oldFilter.filterItems;
var filterType = CommonJS.Filters.FilterType.DropDownSingleSelect;
if (oldFilter.filterType === "Toggle") {
filterType = CommonJS.Filters.FilterType.Toggle;
}
else if (oldFilter.selectionMode === "Multiple") {
filterType = CommonJS.Filters.FilterType.DropDownMultipleSelect;
}
else if (oldFilter.selectionMode === "Slider") {
filterType = CommonJS.Filters.FilterType.Slider;
}
else if (oldFilter.selectionMode === "Custom") {
filterType = CommonJS.Filters.FilterType.Custom;
}
var fg = {}; //filterGroup
var f = {}; //filter        
fg.filterModels = [f];
fg.summaryData = {
label: oldFilter.filterLabel
};
fg.key = oldFilter.filterValue; //group ID
fg.isRemovable = typeof oldFilter.isRemovable !== "undefined" ? oldFilter.isRemovable : true; //all removable if not defined
fg.hidden = oldFilter.hidden;
fg.displayText = oldFilter.filterName;
f.type = filterType;
f.key = oldFilter.filterValue;
f.displayText = oldFilter.filterName;
f.filterData = {};
f.minValue = oldFilter.minValue;
f.maxValue = oldFilter.maxValue;
f.ordinalToValue = oldFilter.ordinalToValue;
f.getValueText = oldFilter.getValueText;
f.hintFunction = oldFilter.hintFunction;
f.useWideTile = oldFilter.useWideTile;
f.customView = (oldFilter.selectionMode === "Custom" && oldFilter.customView) || null;
f.immediateRefresh = (oldFilter.selectionMode === "Custom" && oldFilter.immediateRefresh) || false;
for (var i = 0, l = items.length; i < l; i++) {
var item = items[i];
f.filterData[i] = {
isSelected: item.checked,
key: item.value,
index: i,
displayText: item.text
};
}
return fg;
},
_subtitleClickHandler: function _subtitleClickHandler(event) {
if (typeof this.onSubTitleClick === "function") {
this.onSubTitleClick(event);
}
},
_subtitleKeyUpHandler: function _subtitleKeyUpHandler(event) {
if (event.keyCode === WinJS.Utilities.Key.enter || event.keyCode === WinJS.Utilities.Key.space) {
this._subtitleClickHandler(event);
}
},
_onBackButtonBlurred: function _onBackButtonBlurred(event) {
this._headerBackButton.isfocused = false;
},
_onBackButtonFocused: function _onBackButtonFocused(event) {
this._headerBackButton.isfocused = true;
},
_onBackButtonClick: function _onBackButtonClick(event) {
if (!event.channel) {
event.channel = WinJS.Navigation.location.channelId;
event.page = WinJS.Navigation.location.page;
}
if (this._ifNavigateBack(event)) {
var T = Platform.Telemetry;
T.FlightRecorder.logUserAction(
T.LogLevel.normal,
"",
"Back Button",
T.UserActionOperation.click,
T.UserActionMethod.unknown,
0);
WinJS.Navigation.back();
}
},
_ifNavigateBack: function _ifNavigateBack(event){
var isBackButtonClicked = event.type === "click" && this._headerBackButton.isfocused;
var isBackSpacePressed =  event.keyCode === WinJS.Utilities.Key.backspace && event.channel === WinJS.Navigation.location.channelId && event.page === WinJS.Navigation.location.page;
var isOnTextBox = event.target.tagName === "INPUT";
var isFullscreenVideoPlaying = CommonJS.MediaApp && CommonJS.MediaApp.Controls && CommonJS.MediaApp.Controls.MediaPlayback.isFullscreenVideoPlaying();
return this._headerBackButton && (isBackButtonClicked || (isBackSpacePressed && !isOnTextBox && !isFullscreenVideoPlaying))
&& (!PlatformJS.Navigation || PlatformJS.Navigation.canGoBack);
},
_resetHeader: function _resetHeader(div) {
div.style.clip = "auto";
div.style.opacity = 1;
div.isClipped = false;
if (!div.isVisible) {
WinJS.Utilities.removeClass(div, "platformVisibilityNone");
div.isVisible = true;
}
},
},
{
/// <field>Local Protocol</field>
localProtocol: "ms-appx:///",
largeHeaderHeight: 32,
})
});
})();
/* File: ImageAttribution.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsImageAttributionInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS", {
/// <summary>
/// Control used for attribution of images like hero, slideshow, and article reader focus images. Has a CommonJS.Button instance.
/// </summary>
ImageAttribution: WinJS.Class.define(
function imageAttribution_ctor(element, options) {
if (!element) {
throw "ImageAttribution must be associated with an element that is parented in the DOM before construction.";
}
this.element = element;
element.winControl = this;
// create button
var title = PlatformJS.Services.resourceLoader.getString("/platform/imageAttribution");
var button = new CommonJS.Button(null, {
icon: "platformInfoButton",
title: title,
onclick: function onclick() {
button.currentState = "on";
},
states: [
{
id: "on",
title: title,
iconRemove: "platformInfoButtonOff",
iconAdd: "platformInfoButtonOn",
},
{
id: "off",
title: title,
iconRemove: "platformInfoButtonOn",
iconAdd: "platformInfoButtonOff",
}
]
});
button.currentState = "off";
element.appendChild(button.element);
// create flyout if needed
var flyout = options.flyout;
if (options.flyout) {
var label = flyout.label;
var placement = flyout.placement;
var alignment = flyout.alignment;
var labelElt = this._labelElt = document.createElement("div");
WinJS.Utilities.addClass(labelElt, "platformAttributionLabel");
this.label = label;
var flyoutElt = CommonJS.FlyoutManager.getFlyoutElement();
flyoutElt.appendChild(labelElt);
WinJS.Utilities.addClass(flyoutElt, "platformAttributionFlyout");
var flyoutControl = new WinJS.UI.Flyout(flyoutElt, {
alignment: alignment
});
flyoutControl.addEventListener("afterhide", function imageAttribution_onAfterHide() {
button.currentState = "off";
});
button.flyout = flyoutControl.element;
button.flyoutPlacement = placement;
}
},
{
_labelElt: null,
label: {
set: function set(value) {
PlatformJS.Utilities.setInnerHtml(this._labelElt, value);
}
}
})
});
})();
/* File: ImageCard.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsImageCardInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// ImageCard control.
/// Renders Title, sub-title and back button
/// </summary>
ImageCard: WinJS.Class.define(function imageCard_ctor(element, options) {
/// <summary>
/// Constructor: Creates an instance of the ImageCard Control object.
/// </summary>
/// <returns>
/// An instance of the ImageCard control object.
/// </returns>
element = element || document.createElement("div");
element.winControl = this;
this._element = element;
options = options || {};
this.noIdentifier = options.noIdentifier || 0;
if (options.classification) {
this._classification = options.classification;
}
if (options.defaultImageErrorHandler) {
this._defaultImageErrorHandler = options.defaultImageErrorHandler;
}
this._init();
WinJS.UI.setOptions(this, options);
}, {
// private variables
_domElement: null,
_dataSource: null,
_imageCardUrl: null,
_imgElement: null,
_classification: null,
_defaultImageErrorHandler: null,
noIdentifier: 0,
// private variables
_cacheId: null,
_alternateText: null,
// controls
_element: null,
_imageElement: null,
_placeholderElement: null,
_iconElement: null,
_imageSource: null,
_uniqueId: null,
// properties
alternateText: {
set: function set(value) {
this._alternateText = value;
this._element.setAttribute("aria-label", value);
}
},
imageSource: {
set: function set(value) {
this._imageSource = value;
if (!this._uniqueId) {
this._uniqueId = PlatformJS.Utilities.generateUniqueId();
}
this._loadFromImageSource();
}
},
imageErrorHandler: {
set: function set(value) {
this._imageErrorHandler = value;
},
get: function get() {
switch (this._defaultImageErrorHandler) {
case "hide":
return this._imageErrorHandlerHideImage;
default:
return this._imageErrorHandler;
}
}
},
_imageErrorHandlerHideImage: function _imageErrorHandlerHideImage(e) {
/// <summary>
/// Hide all elements of the image card
/// </summary>
this._toggle(false);
},
_reloadImage: function _reloadImage() {
this._loadFromImageSource();
},
_loadFromImageSource: function _loadFromImageSource() {
if (!this._imageSource) {
return;
} else if (typeof this._imageSource === "string") {
this._loadFromUrl(this._imageSource);
} else if (this._imageSource.lowResolutionUrl && this._imageSource.highResolutionUrl) {
this._loadDualResolution(this._imageSource);
} else {
this._loadSingleResolution(this._imageSource);
}
},
// methods
_toggleElement: function _toggleElement(element, show) {
/// <summary>
/// Shows or hides the given element.
/// </summary>
if (element) {
if (show) {
WinJS.Utilities.removeClass(element, "platformHide");
}
else {
WinJS.Utilities.addClass(element, "platformHide");
}
}
},
_toggle: function _toggle(show) {
/// <summary>
/// Shows or hides imageCard element
/// </summary>
this._toggleElement(this._element, show);
},
_toggleIcon: function _toggleIcon(show) {
/// <summary>
/// Shows or hides the camera icon.
/// </summary>
this._toggleElement(this._iconElement, show);
},
_togglePlaceholder: function _togglePlaceholder(show) {
/// <summary>
/// Shows or hides the entire placeholder.
/// </summary>
if (show) {
WinJS.Utilities.addClass(this._element, "platformNavbarPlaceholderShown");
}
else {
WinJS.Utilities.removeClass(this._element, "platformNavbarPlaceholderShown");
}
this._toggleElement(this._placeholderElement, show);
},
_init: function _init() {
var element = this._element;
WinJS.Utilities.addClass(element, "platformImageCard");
element.setAttribute("role", "img");
if (!this.noIdentifier) {
CommonJS.setAutomationId(element);
}
// placeholder element
var placeholderElement = this._placeholderElement = document.createElement("div");
WinJS.Utilities.addClass(placeholderElement, "platformImageCardPlaceholder");
element.appendChild(placeholderElement);
// hide the placeholder initially
this._togglePlaceholder(false);
// icon element
var iconElement = document.createElement("div");
WinJS.Utilities.addClass(iconElement, "platformImageCardPlaceholderIcon platformCameraIcon");
placeholderElement.appendChild(iconElement);
this._iconElement = iconElement;
// image element
var imageElement = this._imageElement = document.createElement("div");
WinJS.Utilities.addClass(imageElement, "platformImageCardImage");
element.appendChild(imageElement);
CommonJS.classifyPlaceholder(element, this._classification);
},
_loadFromUrl: function _loadFromUrl(url) {
this._loadSingleResolution({
cacheId: null,
url: url
});
},
_loadSingleResolution: function _loadSingleResolution(imageSource, bypassCache) {
var lowResolution = imageSource.url;
this._cacheId = imageSource.cacheId;
var that = this;
if (!lowResolution) {
return WinJS.Promise.wrap(null);
}
var promise = WinJS.Promise.wrap(null);
if (!bypassCache && this._cacheId) {
promise = PlatformJS.Cache.CacheService.getInstance(this._cacheId).findEntry(lowResolution, { fileNameOnly: true })
.then(function image_cache_completion(response) {
if (response) {
that._setImage(response.dataValue, lowResolution);
}
return response;
});
}
return promise
.then(function image_refresh(response) {
if (!response || response.isStale()) {
return that._loadImage(lowResolution, that._showPlaceholder.bind(that));
}
return WinJS.Promise.wrap(null);
}).then(function imageCard_loadImageComplete(localUrl) {
if (localUrl) {
that._setImage(localUrl, lowResolution);
if (that._highResolutionImageElement && that._highResolutionImageElement.parent === that._element) {
that._element.removeChild(that._highResolutionImageElement);
that._highResolutionImageElement = null;
}
}
return WinJS.Promise.wrap(null);
}, function imageCard_loadImageError(err) {
that._handleImageError(imageSource);
return WinJS.Promise.wrap(null);
});
},
_loadDualResolution: function _loadDualResolution(imageSource) {
var that = this;
var lowResolution = imageSource.lowResolutionUrl;
var highResolution = imageSource.highResolutionUrl;
var cacheId = imageSource.cacheId;
var imageService = new Platform.ImageService(cacheId);
var promise = WinJS.Promise.wrap(null);
if (cacheId) {
promise = PlatformJS.Cache.CacheService.getInstance(cacheId).findEntry(highResolution, { fileNameOnly: true })
.then(function image_cache_completion(response) {
if (response) {
that._setImage(response.dataValue, highResolution);
}
return response;
});
}
promise.then(function image_refresh(response) {
if (!response || response.isStale()) {
//Otherwise load low res then high res
var lowResolutionMetadata = {
url: lowResolution,
cacheId: cacheId
};
that._loadSingleResolution(lowResolutionMetadata, true)
.then(function imageCard_loadSingleResolutionComplete() {
if (highResolution) {
that._loadImage(highResolution, null)
.then(function imageCard_loadImageComplete(localUrl) {
if (that._highResolutionImageElement && that._highResolutionImageElement.parent === that._element) {
that._element.removeChild(that._highResolutionImageElement);
}
var imageElement = that._highResolutionImageElement = document.createElement("div");
WinJS.Utilities.addClass(imageElement, "platformImageCardImage");
imageElement.style.backgroundImage = "url('" + localUrl + "')";
that._element.appendChild(imageElement);
//HACK: IE renderer does not always draw background images on screen (Windows 8 bug: #685585). Setting the opacity seems to force it to draw. (naeims)
//[BEGIN]
var imgStyle = window.getComputedStyle(imageElement);
imageElement.style.opacity = imgStyle ? imgStyle.opacity : "1";
//[END]
}, function imageCard_loadImageError(err) {
that._handleImageError(imageSource);
});
}
});
}
});
},
_loadImage: function _loadImage(imageUrl, networkCallRequired) {
// Check to see if this is a local image
if (typeof imageUrl !== "string") {
return WinJS.Promise.wrap("");
}
else if (imageUrl.indexOf("/") === 0 || imageUrl.indexOf("ms-appdata://") === 0 || imageUrl.indexOf("ms-appx://") === 0) {
return WinJS.Promise.wrap(imageUrl);
} else {
return PlatformJS.Utilities.fetchImage(this._cacheId, imageUrl, networkCallRequired);
}
},
_showPlaceholder: function _showPlaceholder() {
this._toggleIcon(true);
this._togglePlaceholder(true);
},
_setImage: function _setImage(url, imageSource) {
if (!this._alternateText && imageSource) {
this._element.setAttribute("aria-label", imageSource);
}
var that = this;
var imageElement = this._imageElement;
imageElement.style.backgroundImage = "url('" + url + "')";
//HACK: IE renderer does not always draw background images on screen (Windows 8 bug: #685585). Setting the opacity seems to force it to draw. (naeims)
//[BEGIN]
var imgStyle = window.getComputedStyle(imageElement);
imageElement.style.opacity = imgStyle ? imgStyle.opacity : "1";
//[END]
this._transitionOutPlaceholder()
.then(function imageCard_transitionOutPlaceholderComplete() {
that._removePlaceholderImage();
}).then(null, function imageCard_removePlaceholderImageError(err) {
// here, we need a fake error handler in order to swallow exceptions when trying
// to manipulate DOM elements that have been since virtualized by the listview
});
},
_transitionOutPlaceholder: function _transitionOutPlaceholder() {
var imageElement = this._imageElement;
var placeholderElement = this._placeholderElement;
if (placeholderElement && !WinJS.Utilities.hasClass(placeholderElement, "platformHide")) {
return WinJS.UI.Animation.crossFade(imageElement, placeholderElement);
} else {
return WinJS.Promise.wrap({});
}
},
_handleImageError: function _handleImageError(imageSource) {
var that = this;
// turn on the camera icon
this._toggleIcon(true);
// turn on the placeholder
this._togglePlaceholder(true);
// turn on listening for notifications in case of an error
PlatformJS.Events.addEventListener(this._uniqueId, "networkstatuschanged", function imageCard_onNetworkStatusChanged() {
if (PlatformJS.Utilities.hasInternetConnection()) {
that._reloadImage();
}
});
if (this.imageErrorHandler) {
this.imageErrorHandler(this._element);
}
},
_removePlaceholderImage: function _removePlaceholderImage() {
if (this._placeholderElement) {
// if image card was disconnected from document then parent of placeholder element may not longer have it as a child.
// if life is good then parent should be element.
var parent = this._placeholderElement.parentElement;
if (parent) {
parent.removeChild(this._placeholderElement);
}
if (this._element) {
WinJS.Utilities.removeClass(this._element, "platformNavbarPlaceholderShown");
}
this._placeholderElement = null;
}
}
})
});
})();
/* File: TwoStepBindingImageCard.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsTwoStepBindingImageCardInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS",
{
/// <summary>
/// ImageCard sublass that supports setting lowres and highres urls sequentially.
/// The loading of the image is triggered on set of the highres url.
/// </summary>
TwoStepBindingImageCard: WinJS.Class.derive(CommonJS.ImageCard,
function twoStepBindingImageCard_ctor (element, options) {
CommonJS.ImageCard.call(this, element, options);
this._lowResolutionUrl = null;
this._highResolutionUrl = null;
},
{
_lowResolutionUrl: null,
_highResolutionUrl: null,
lowResolutionUrl: {
set: function set(value) {
this._lowResolutionUrl = value;
}
},
highResolutionUrl: {
set: function set(value) {
var cacheId;
var url;
if (typeof value === "string") {
url = value;
}
else {
url = value.url;
cacheId = value.cacheId;
}
var highResolutionUrl = this._highResolutionUrl = url;
var lowResolutionUrl = this._lowResolutionUrl;
if (lowResolutionUrl) {
this.imageSource = {
lowResolutionUrl: lowResolutionUrl,
highResolutionUrl: highResolutionUrl,
cacheId: cacheId
};
} else {
this.imageSource = {
url: highResolutionUrl,
cacheId: cacheId
};
}
}
}
})
});
})();
/* File: ItemsContainer.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsItemsContainerInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive",
{
ItemsContainer: WinJS.Class.mix(WinJS.Class.define(function itemsContainer_ctor(element, options) {
/// <summary>A simple DIV container</summary>
/// <param name="element">DOM element</param>
/// <param name="options">Options struct</param>
var that = this;
element = this.element = element ? element : document.createElement("div");
element.winControl = this;
var cluster = PlatformJS.Utilities.getAncestorByClassName(element, "platformCluster");
if (cluster) {
var clusterLabelledBy = cluster.getAttribute("aria-labelledby");
if (clusterLabelledBy !== "") {
element.setAttribute("role", "listbox");
element.setAttribute("aria-labelledby", clusterLabelledBy);
}
}
WinJS.Utilities.addClass(this.element, "platformItemsContainer");
CommonJS.setAutomationId(element);
// register itemclick event
PlatformJS.Utilities.registerItemClickProxy(element,
function itemsContainer_registerItemClickProxyPredicate(domElement) {
return WinJS.Utilities.hasClass(domElement, "platformClusterItem");
}, function itemsContainer_registerItemClickProxyCompletion(domElement) {
that._onItemClicked(domElement);
});
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemclick"));
this._bindingPromises = [];
this._elements = {};
WinJS.UI.setOptions(this, options);         
},
{
_renderPromise: null,
_bindingPromises: null,
_pointerDown: false,
_dataSource: null,
_elements: null,
_navigableViewOrchestrator: null,
onitemclick: null,
_onItemClicked: function _onItemClicked(domElement) {
if (domElement) {
var that = this;
var index = Array.prototype.indexOf.call(this.element.children, domElement);
this._listBinding
.fromIndex(index)
.then(function itemsContainer_onItemClickedFromIndexComplete(model) {
that.dispatchEvent("itemclick", {
item: model.data
});
});
}
},
/// <field>Sets the css class for the container</field>
className: {
set: function set(value) {
WinJS.Utilities.addClass(this.container, value);
}
},
/// <field>Item data source</field>
itemDataSource: {
set: function set(value) {
this._dataSource = value;
if (this.container) {
this.container.innerHTML = "";
}
if (this._listBinding) {
this._listBinding.release();
}
this._listBinding = this._dataSource.createListBinding(this);
}
},
dispose: function dispose() {
var currentRenderPromise = this._renderPromise;
this._renderPromise = null;
if (currentRenderPromise) {
currentRenderPromise.cancel();
}
this._bindingPromises = null;
msClearImmediate(this._immediateIdentifier);
if (this._listBinding) {
this._listBinding.release();
}
if (this.container) {
this.container.innerHTML = "";
}
if (this._navigableViewOrchestrator) {
this._navigableViewOrchestrator.dispose();
this._navigableViewOrchestrator = null;
}
},
render: function render() {
/// <summary>Refreshes the container</summary>
/// <returns>Rendering promise</returns>
msWriteProfilerMark("Platform:ItemsContainer:render:s");
this.container.innerHTML = "";
this._bindingPromises = [];
var that = this;
var listBinding = this._listBinding;
var dataSource = this._dataSource;
if (!dataSource || !listBinding) {
return;
}
return new WinJS.Promise(function itemsContainer_renderPromiseInit(c, e) {
that.completionHandler = c;
that.errorHandler = e;
that.renderImpl();
});
},
renderImpl: function renderImpl() {
var that = this;
var fragment = document.createDocumentFragment();
var listBinding = this._listBinding;
var dataSource = this._dataSource;
this._renderPromise = dataSource.getCount()
.then(function itemsContainer_dataSourceGetCountComplete(itemCount) {
var promise = WinJS.Promise.wrap(null);
for (var i = 0; i < itemCount; i++) {
(function itemsContainer_renderNextItem(i) {
promise = promise.then(function itemsContainer_nextRenderPromiseComplete() {
return that._renderItem(i, listBinding, fragment);
});
})(i);
}
return promise;
}).then(function itemsContainer_renderNextItemComplete() {
return WinJS.Promise.join(that._bindingPromises);
}).then(function itemsContainer_bindingPromisesJoinComplete() {
that.container.appendChild(fragment);
that._createNavigableView();
msWriteProfilerMark("Platform:ItemsContainer:render:e");
that.completionHandler();
}, function itemsContainer_bindingPromisesJoinError(err) {
if (err.message !== "Canceled") {
that.errorHandler(err);
}
});
},
_renderItem: function _renderItem(i, listBinding, fragment) {
var that = this;
return listBinding
.fromIndex(i)
.retain()
.then(function itemsContainer_renderItemRetainComplete(listItem) {
that._createItem(fragment, listItem);
});
},
_createNavigableView: function _createNavigableView() {
this._navigableViewOrchestrator = new CommonJS.NavigableView.NavigableViewOrchestrator(this.element, this._elements);
},
_createItem: function _createItem(fragment, item, insertBefore) {
/// <summary>Creates the tile</summary>
/// <param name="item">Item to create tile for </param>
/// <param name="insertBefore">DOM element to insert before</param>
var container = null;
var that = this;
if (item && item.data && item.data.moduleInfo) {
if (item.data.moduleInfo.renderer) {
// Use the custom renderer supplied
container = item.data.moduleInfo.renderer(item);
WinJS.Utilities.addClass(container, "platformClusterItem");
if (!insertBefore) {
fragment.appendChild(container);
} else {
fragment.insertBefore(container, insertBefore);
}
if (this._bindingPromises) {
this._bindingPromises.push(WinJS.Promise.wrap(null));
}
} else {
container = document.createElement("div");
WinJS.Utilities.addClass(container, "platformClusterItem");
if (!insertBefore) {
fragment.appendChild(container);
} else {
fragment.insertBefore(container, insertBefore);
}
CommonJS.setAutomationId(container, this.container, item.handle);
if (item.data.moduleInfo.className) {
WinJS.Utilities.addClass(container, item.data.moduleInfo.className);
}
if (item.data.moduleInfo.cssStyle) {
container.setAttribute("style", item.data.moduleInfo.cssStyle);
}
if (item.data.moduleInfo.height) {
container.style.height = item.data.moduleInfo.height;
}
if (item.data.moduleInfo.width) {
container.style.width = item.data.moduleInfo.width;
}
var renderPromise = WinJS.Promise
.wrap(null)
.then(function itemsContainer_renderPromiseTimeoutComplete() {
return CommonJS.loadModule(item.data.moduleInfo, item.data, container);
});
if (this._bindingPromises) {
this._bindingPromises.push(renderPromise);
}
}
var isInteractive = item.data.moduleInfo.isInteractive;
if (!isInteractive) {
PlatformJS.Utilities.enablePointerUpDownAnimations(container);
}
this._elements[item.handle] = container;
container.tabIndex = 0;
container.setAttribute("role", "option");
}
return container;
},
// DEPRECATED
_initItemInfos: function _initItemInfos(container, item) {
// NO-OP for backwards compatibility
},
container: {
get: function get() {
return this.element;
}
},
getNavigableViewOrchestrator: function getNavigableViewOrchestrator() {
return this._navigableViewOrchestrator;
},
// IListNotificationHandler interface
beginNotifications: function beginNotifications() { },
endNotifications: function endNotifications() { },
changed: function changed() { },
itemAvailable: function itemAvailable() { },
moved: function moved() { },
countChanged: function countChanged(newCount, oldCount) { },
indexChanged: function indexChanged(handle, newIndex, oldIndex) { },
inserted: function inserted(listItem, previousHandle, nextHandle) {
var that = this;
listItem
.retain()
.then(function itemsContainer_insertedListItemRetainComplete(item) {
that._createItem(that.container, item, nextHandle && that._elements[nextHandle] ? that._elements[nextHandle] : null);
});
},
removed: function removed(handle, mirage) {
if (this._elements[handle]) {
this.container.removeChild(this._elements[handle]);
delete this._elements[handle];
}
},
onOrientationChange: function onOrientationChange(event) {
if (this._navigableViewOrchestrator) {
this._navigableViewOrchestrator.onOrientationChange(event);
}
},
onWindowResize: function onWindowResize(event) {            
if (this._navigableViewOrchestrator) {
this._navigableViewOrchestrator.onWindowResize(event);
}
}
},
{
reActivate: function reActivate(item) {
var itemsContainer = item.parentNode;
var itemsContainerControl = itemsContainer && itemsContainer.winControl;
if (itemsContainerControl) {
var navigableViewOrchestrator = itemsContainerControl.getNavigableViewOrchestrator();
var navigableView = navigableViewOrchestrator && navigableViewOrchestrator.navigableView;
if (navigableView) {
navigableView.isActivated = false;
}
}
}
}), WinJS.UI.DOMEventMixin)
});
})();
/* File: ParallaxItemsContainer.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsParallaxItemsContainerInit() {
"use strict";
var allAnchorPointTypes = "anchorTop anchorMiddle anchorBottom anchorLeft anchorRight";
function promise(response) {
return WinJS.Promise.wrap(response);
}
function animateImageIn(imageElement) {
imageElement.style.opacity = 0.25;
//this is replacing WinJS.UI.Animation.fadeIn(imageElement);
return WinJS.UI.executeTransition(
imageElement,
{
property: "opacity",
delay: 0,
duration: 200,   // default was 250
timing: "linear",
from: 0.25,
to: 1
}); 
}
function animateElementIn(itemsContainerElement) {
return WinJS.UI.Animation.enterContent(itemsContainerElement);
}
function loadImageIntoElement(imageElement, imageUrl) {
imageElement.style.backgroundImage = "url('" + imageUrl + "');";
}
function getCacheServiceInstance(cacheIdentifier) {
return cacheIdentifier ? PlatformJS.Cache.CacheService.getInstance(cacheIdentifier) : { findEntry: function () { return promise(null); } };
}
function storeImageToCache(imageUrl, cacheIdentifier, networkCallRequired) {
networkCallRequired = networkCallRequired || function () { };
// Check to see if this is a local image
if (imageUrl) {
if (imageUrl.indexOf("/") === 0 || imageUrl.indexOf("ms-appdata://") === 0) {
return promise({
url: imageUrl,
isLocalResponse: true
});
} else {
return getCacheServiceInstance(cacheIdentifier)
.findEntry(imageUrl, { fileNameOnly: true })
.then(function (response) {
if (!response || response.isStale()) {
return PlatformJS.Utilities.fetchImage(cacheIdentifier, imageUrl, networkCallRequired)
.then(function (fetchResponse) {
return promise({
url: fetchResponse,
isLocalResponse: false
});
});
} else {
return promise({
url: response.dataValue,
isLocalResponse: true
});
}
});
}
}
return promise(null);
}
function resetImageAndStagingImage(imageElement, stagingElement) {
//imageElement.src = stagingElement.src;
imageElement.style.backgroundImage = stagingElement.style.backgroundImage;
stagingElement.style.opacity = 0;
}
function getImageSource(imageSource) {
if (typeof imageSource === "string") {
return { url: imageSource };
} else if (imageSource.highResolutionUrl) {
return {
url: imageSource.highResolutionUrl,
cacheId: imageSource.cacheId
};
} else {
return imageSource;
}
}
function logTelemetryHeroImageDownload(loadElapsedTime) {
// log hero image download information to flight recorder
// Use page location and channel info as the perf context
var perfContext;
if (WinJS.Navigation.location) {
perfContext = WinJS.Navigation.location.page + "*" + WinJS.Navigation.location.channelId;
}
// this event represents time elapsed to download a hero image from the network
Platform.Telemetry.FlightRecorder.logPerf(Platform.Telemetry.LogLevel.normal, "Platform:ParallaxItemsContainer:loadFirstAvailableImage", "HeroImageDownload", perfContext, loadElapsedTime);
}
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
ParallaxItemsContainer: WinJS.Class.derive(CommonJS.Immersive.ItemsContainer, function parallaxItemsContainer_ctor(element, options) {
var that = this;
this._backgroundImageSetPromise = new WinJS.Promise(function(complete) {
// calling _backgroundImageSet will immediatly release the _backgroundImageSetPromise and execute its then statement.
that._backgroundImageSet = complete;
});
this.element = element ? element : document.createElement("div");
WinJS.Utilities.addClass(this.element, "platformHeroCluster");
this._initItemsContainer(element);
this._initBackgroundImage(options);
this._initPanoramaReference();
this._initItemClickedHandler();
this._onLayoutChangeEventBinding = this._layoutChange.bind(this);
window.addEventListener("resize", this._onLayoutChangeEventBinding);
WinJS.UI.setOptions(this, options);
},
{
_itemsContainer: null,
_backgroundImage: null,
_panorama: null,
_imageSize: null,
_gradient: null,
_titleGradient: null,
_imageAttributionElt: null,
_imageAttribution: null,
_onLayoutChangeEvent: null,
_currentImageElement: null,
_stagingImageElement: null,
_setBackgroundImageProperty: false,
_backgroundImageSetPromise: null,
_backgroundImageSet: null,
_loadFromImageSourcePromise: null,
_parallaxRenderPromise: null,
_refreshPromise:null,
// prevent enter content animation on parallax items container
preventAnimation: true,
preventPlatformAnimation: false,
getLoadFromImageSourcePromise: function getLoadFromImageSourcePromise() {
return this._loadFromImageSourcePromise;
},
_releaseBackgroundImageSetPromise: function _releaseBackgroundImageSetPromise() {
// avoid possibility of executing _backgroundImageSet multiple times.
// calling _backgroundImageSet will immediately release the _backgroundImageSetPromise and execute its then statement.
if (this._backgroundImageSet) {
var temp = this._backgroundImageSet;
this._backgroundImageSet = null;
temp.call(this);
}
},
_initItemsContainer: function _initItemsContainer(element) {
// create _itemsContainer
var itemsContainerElement = this._itemsContainer = document.createElement("div");
itemsContainerElement.className = "platformItemsContainer";
//itemsContainerElement.style.opacity = 0;
this._elements = {};
this._itemInfos = [];
this._bindings = [];
CommonJS.setAutomationId(this.element);
var cluster = PlatformJS.Utilities.getAncestorByClassName(element, "platformCluster");
if (cluster) {
var clusterLabelledBy = cluster.getAttribute("aria-labelledby");
if (clusterLabelledBy !== "") {
this._itemsContainer.setAttribute("role", "listbox");
this._itemsContainer.setAttribute("aria-labelledby", clusterLabelledBy);
}
}
},
_initBackgroundImage: function _initBackgroundImage(options) {
var viewport = this.element;
WinJS.Utilities.addClass(viewport, "platformParallaxImageViewport");
var imageParent = document.createElement("div");
imageParent.className = "platformImageCard platformPlaceHolderLarge";
viewport.appendChild(imageParent);
var imageElement = this._currentImageElement = document.createElement("div");
imageElement.id = "platformHeroImage";
//imageElement.style.opacity = 0;
imageElement.className = "platformImageCardImage";
imageParent.appendChild(imageElement);
var stagingImageElement = this._stagingImageElement = document.createElement("div");
stagingImageElement.id = "platformHeroStagingImage";
stagingImageElement.style.opacity = 0;
stagingImageElement.className = "platformImageCardImage";
imageParent.appendChild(stagingImageElement);
// gradient
var gradient = this._gradient = document.createElement("div");
gradient.className = "platformImageGradientContainer platformImageGradient platformHide";
viewport.appendChild(gradient);
// title gradient
var titleGradient = this._titleGradient = document.createElement("div");
titleGradient.className = "platformTitleGradient";
viewport.appendChild(titleGradient);
// show title gradient by default
var showTitleGradient = options.showTitleGradient;
showTitleGradient = (typeof showTitleGradient === "undefined") || showTitleGradient || false;
this.showTitleGradient = showTitleGradient;
// image attribution
var imageAttributionElt = this._imageAttributionElt = document.createElement("div");
imageAttributionElt.className = "platformHeroImageAttribution platformHide";
// create image attribution control
this._imageAttribution = new CommonJS.ImageAttribution(imageAttributionElt, {
flyout: {
label: "",
placement: "top",
alignment: (window.getComputedStyle(document.body).direction === "rtl") ? "left" : "right"
}
});
},
_initPanoramaReference: function _initPanoramaReference() {
var panorama = WinJS.Utilities.query(".platformPanorama");
if (panorama.length) {
this._panorama = panorama[0].winControl;
} else {
PlatformJS.Utilities.onError("This can only be used in a panorama.");
}
this.element.appendChild(this._itemsContainer);
if (this._imageAttributionElt) {
this.element.appendChild(this._imageAttributionElt);
}
},
_initItemClickedHandler: function _initItemClickedHandler() {
// register itemclick event
var that = this;
PlatformJS.Utilities.registerItemClickProxy(this.element,
function itemsContainer_registerClusterItemClickProxyPredicate(domElement) {
// stop propagating when top level dom element is hit for hero.
return (WinJS.Utilities.hasClass(domElement, "platformClusterContent") || WinJS.Utilities.hasClass(domElement, "platformClusterItem"));
}, function itemsContainer_registerClusterItemClickProxyCompletion(domElement) {
that._onItemClicked(domElement);
});
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemclick"));
},
_handleImageError: function _handleImageError(err) {
console.log("ParallaxItemsContainer - HandleImageError " + err);
this._releaseBackgroundImageSetPromise();
if (this._panorama) {
this._panorama.showInertiaBackground = false;
}
},
_onItemClicked: function _onItemClicked(domElement) {               
var that = this;
var clusterItemElement = WinJS.Utilities.hasClass(domElement, "platformClusterItem") ? domElement : null;
if (clusterItemElement) {
var index = Array.prototype.indexOf.call(this._itemsContainer.children, clusterItemElement);
this._listBinding.fromIndex(index)
.then(function itemsContainer_itemsContainerOnItemClickedFromIndexComplete(model) {
if (model) {
that.dispatchEvent("itemclick", {
item: model.data
});
}
});
}
},        
_layoutChange: function _layoutChange(event) {
this._layoutHeroImage();
},
/// <field>Attribution Data</field>
attributionData: {
set: function set(value) {
if (value) {
if (value.attributionText.length > 0) {
WinJS.Utilities.removeClass(this._imageAttribution.element, "platformHide");
} else {
WinJS.Utilities.addClass(this._imageAttribution.element, "platformHide");
}
this._imageAttribution.label = value.caption + " " + value.attributionText;
this._currentImageElement.alt = value.attributionText;
} else {
this._currentImageElement.alt = "";
this._imageAttribution.label = "";
WinJS.Utilities.addClass(this._imageAttribution.element, "platformHide");
}
}
},
_bindings: null,
/// <field>An all-in-one property for all the image data related to the hero image</field>
imageData: {
set: function set(value) {
var that = this;
if (value.bind) {
var bindableProperties = ["fontColor", "imageSize", "anchorPoint", "backgroundImage", "showGradient", "attributionData"];
// Listen to each of these properties
for (var i = 0; i < bindableProperties.length; i++) {
(function itemsContainer_imageDataBindPropertyAndAction(propertyName) {
var bindingAction = function (value) {
that[propertyName] = value;
};
that._bindings.push({
observable: value,
action: bindingAction
});
value.bind(propertyName, bindingAction);
})(bindableProperties[i]);
}
} else {
WinJS.UI.setOptions(this, value);
}
}
},
/// <field>Size of the image</field>
imageSize: {
set: function set(value) {
this._imageSize = value;
this._layoutHeroImage(this._currentImageElement, this._stagingImageElement);
}
},
_layoutHeroImage: function _layoutHeroImage() {
var imageElement = this.element;
var imageSize = this._imageSize;
if (imageSize) {
// image dimensions
var imageWidth = imageSize.width;
var imageHeight = imageSize.height;
var imageRatio = (imageWidth / imageHeight);
// container dimensions
var containerWidth = imageElement.clientWidth;
var containerHeight = imageElement.clientHeight;
var classname = "fitHeroImageCover";
var view = Windows.UI.ViewManagement.ApplicationView.value;
var viewStates = Windows.UI.ViewManagement.ApplicationViewState;
if (view === viewStates.filled || view === viewStates.fullScreenLandscape) {
// Just stretch the image instead of cropping when image ratio is within the range of container ratio.
var deltaError = 0.05;
var containerRatio = (containerWidth / containerHeight);
if (((imageRatio - deltaError) < containerRatio) && ((imageRatio + deltaError) > containerRatio)) {
classname = "fitHeroImageStretch";
}
}
WinJS.Utilities.removeClass(imageElement, "fitHeroImageStretch fitHeroImageCover");
WinJS.Utilities.addClass(imageElement, classname);
}
},
/// <field>Anchor point of the image</field>
anchorPoint: {
set: function set(value) {
if (value) {
WinJS.Utilities.removeClass(this._currentImageElement, allAnchorPointTypes);
WinJS.Utilities.removeClass(this._stagingImageElement, allAnchorPointTypes);
WinJS.Utilities.addClass(this._currentImageElement, value);
WinJS.Utilities.addClass(this._stagingImageElement, value);
}
}
},
/// <field>The background image url</field>
backgroundImage: {
set: function set(value) {
if (value) {
var imageSource = value;
if (typeof value === "string") { 
imageSource = { url: value, cacheId: "PlatformImageCache" };
}
else if (!imageSource.cacheId) { 
imageSource.cacheId = "PlatformImageCache";
}
this._loadFromImageSource(imageSource);
// set this variable to track correct usage of the interface. 
// backgroundImage or imageData property must be used to load hero image when hero image is not already loaded.
this._setBackgroundImageProperty = true;
}
}
},
refresh: function refresh(model) {
// refresh() should be used when hero image is already displayed using imageData or backgroundImage properties
if (!this._setBackgroundImageProperty) {
debugger;
}
var that = this;
var itemsContainerElement = this._itemsContainer;
var currentImageElement = this._currentImageElement;
var stagingImageElement = this._stagingImageElement;
var imageSource = getImageSource(model.backgroundImage); 
if (!imageSource || !imageSource.url) {
this._refreshPromise = WinJS.Promise.wrap(null);
return this._refreshPromise;
}
var imageUrl;
//TODO: refresh attribution
//TODO: refresh anchor point
//TODO: introduce timeout between render and refresh to avoid jitter
// log to flight recorder when hero image not found in cache
var loadStartTime = new Date();
var previousRefreshPromise = this._refreshPromise;
var refreshPromise = storeImageToCache(imageSource.url, imageSource.cacheId)
.then(function (localUrl) {
if (!localUrl) {
throw "refreshing the hero with invalid image";
}
imageUrl = localUrl.url;
loadImageIntoElement(stagingImageElement, imageUrl);
if (!localUrl.isLocalResponse) {
var loadEndTime = new Date();
var loadElapsedTime = loadEndTime - loadStartTime;
logTelemetryHeroImageDownload(loadElapsedTime);
}
// wait for any current image loads or rendering to finish.
var promises = [];
promises.push(WinJS.Promise.wrap({}));
if (that._parallaxRenderPromise) {
promises.push(that._parallaxRenderPromise);
}
if (that._loadFromImageSourcePromise) {
promises.push(that._loadFromImageSourcePromise);
}
if (previousRefreshPromise) {
promises.push(previousRefreshPromise);
}
return WinJS.Promise.join(promises);
})
.then(function () {
itemsContainerElement.style.opacity = 0;  // replaces WinJS.UI.Animation.exitContent(itemsContainerElement)
that.itemDataSource = model.itemDataSource;
return CommonJS.Immersive.ItemsContainer.prototype.render.call(that, false);})
.then(function () {
// FUSED Animation 
var animateInPromises = [];
animateInPromises.push(animateImageIn(stagingImageElement));
that.attributionData = model.attributionData;
if (that.preventPlatformAnimation) {
itemsContainerElement.style.opacity = 1; 
} else {
animateInPromises.push(animateElementIn(itemsContainerElement));
}
return WinJS.Promise.join(animateInPromises); 
})
.then(function () {
resetImageAndStagingImage(currentImageElement, stagingImageElement);
})
.then(null, function onRefreshPromiseError(err) { that._handleImageError(err); });
this._refreshPromise = refreshPromise;
return this._refreshPromise;
},
_loadFromImageSource: function _loadFromImageSource(source) {
if (source) {
var p = null;
if (source.lowResolutionUrl && source.highResolutionUrl) {
p = this._loadDualResolution(source);
} else if (source.url) {
p = this._loadSingleResolution(source);
} else {
this._releaseBackgroundImageSetPromise(); // release the backgroundimagesetpromise even if we don't set any backgroundimage.
return;
}
if (this._loadFromImageSourcePromise) {
this._loadFromImageSourcePromise.then(function() { return p; });
} else {
this._loadFromImageSourcePromise = p;
}
}
},
_loadSingleResolution: function _loadSingleResolution(imageSource, bypassCache, imageElement) {
imageElement = imageElement || this._currentImageElement; 
var lowResolution = imageSource.url;
var cacheId = imageSource.cacheId;
var that = this;
// log to flight recorder when hero image not found in cache
var loadStartTime = new Date();
// Do not block on image load
that._releaseBackgroundImageSetPromise(); // release initial render
return storeImageToCache(lowResolution, cacheId)
.then(function (localUrl) {
if (!localUrl) {
throw "loading single resolution image with invalid image url";
}
if (!localUrl.isLocalResponse) {
var loadEndTime = new Date();
var loadElapsedTime = loadEndTime - loadStartTime;
logTelemetryHeroImageDownload(loadElapsedTime);
}
loadImageIntoElement(imageElement, localUrl.url);
})
.then(null, function onLoadSingleResolutionError(err) { that._handleImageError(err); });
},
_loadDualResolution: function _loadDualResolution(imageSource) {
var that = this;
var buffer = null;
var currentImageElement = this._currentImageElement,
stagingImageElement = this._stagingImageElement,
itemsContainerElement = this._itemsContainer;
var lowResolution = imageSource.lowResolutionUrl,
highResolution = imageSource.highResolutionUrl,
cacheIdentifier = imageSource.cacheId;
var cacheService = getCacheServiceInstance(cacheIdentifier);
// log to flight recorder when hero image not found in cache
var loadStartTime = new Date();
msWriteProfilerMark("Platform:ParallaxItemsContainer:loadFirstAvailableImage:s");
var isHighResolutionRendered = false;
var isLowResolutionRendered = false;
// If high resolution image is in cache, show immediately 
return cacheService
.findEntry(highResolution, { fileNameOnly: true })
.then(function (response) {
if (response && !response.isStale()) {
loadImageIntoElement(currentImageElement, response.dataValue);
that._releaseBackgroundImageSetPromise(); // release initial render and execute it immediately
isHighResolutionRendered = true;
return WinJS.Promise.wrap({});
}
else {
// Download low and high resolution in parallel.
// Do not block on image load
that._releaseBackgroundImageSetPromise(); // release initial render and execute it immediately
var highResLoadPromise = storeImageToCache(highResolution, cacheIdentifier).then(function highResolution_downloadComplete(response) {
if (!response) {
throw "loading high resolution image with invalid image url";
}
if (isLowResolutionRendered) {
loadImageIntoElement(stagingImageElement, response.url);
animateImageIn(stagingImageElement)
.then(function () {
resetImageAndStagingImage(currentImageElement, stagingImageElement);
});
}
else {
loadImageIntoElement(currentImageElement, response.url);
}
isHighResolutionRendered = true;
msWriteProfilerMark("Platform:ParallaxItemsContainer:loadFirstAvailableImage:e");
var loadEndTime = new Date();
// Only log flight recorder event if the hero image was not pulled from cache
var loadElapsedTime = loadEndTime - loadStartTime;
logTelemetryHeroImageDownload(loadElapsedTime);
}).then(null, function onLoadHighResolutionError(err) { that._handleImageError(err); });
var lowResLoadPromise = storeImageToCache(lowResolution, cacheIdentifier).then(function lowResolutionImage_downloadComplete(response) {
WinJS.Promise.timeout(100).then(function () {
if (!isHighResolutionRendered) {
if (!response) {
throw "loading low resolution image with invalid image url";
}
loadImageIntoElement(currentImageElement, response.url);
isLowResolutionRendered = true;
}
});
}).then(null, function onLoadLowResolutionError(err) { that._handleImageError(err); });
// this promise is used to block refresh so need to wait for both high and low res
// the _backgroundImageSet allows the render to process and is unlocked by either
return WinJS.Promise.join([highResLoadPromise, lowResLoadPromise]);
}
}).then(null, function (err) { that._handleImageError(err); });
},
/// <field>Show gradient in the hero image</field>
showGradient: {
set: function set(value) {
if (value && this._gradient) {
WinJS.Utilities.removeClass(this._gradient, "platformHide");
} else if (this._gradient) {
WinJS.Utilities.addClass(this._gradient, "platformHide");
}
}
},
showTitleGradient: {
set: function set(value) {
if (value && this._gradient) {
WinJS.Utilities.removeClass(this._titleGradient, "platformHide");
}
else if (this._titleGradient) {
WinJS.Utilities.addClass(this._titleGradient, "platformHide");
}
}
},
render: function render() {
var implementation = CommonJS.Immersive.ItemsContainer.prototype.render.call(this, false);
this._parallaxRenderPromise = WinJS.Promise.join([this._backgroundImageSetPromise, implementation]);
return this._parallaxRenderPromise;
},
dispose: function dispose() {
var bindings = this._bindings;
for (var i = 0, ilen = bindings.length; i < ilen; i++) {
var binding = bindings[i];
binding.observable.unbind(binding.action);
}
this._backgroundImage = null;
this._panorama = null;
this._attributionLabel = null;
this._gradient = null;
if (this._onLayoutChangeEventBinding) {
window.removeEventListener("resize", this._onLayoutChangeEventBinding);
this._onLayoutChangeEventBinding = null;
}
CommonJS.Immersive.ItemsContainer.prototype.dispose.call(this);
},
container: {
get: function get() {
return this._itemsContainer;
}
}
}),
});
})();
/* File: ScalingItemsContainer.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsScalingItemsContainerInit() {
"use strict";
function _isPortrait (viewState) {
var viewStates = Windows.UI.ViewManagement.ApplicationViewState;
return (viewState === viewStates.fullScreenPortrait);
}
function _getScreenSize () {
var bodyWidth = document.body.offsetWidth,
bodyHeight = document.body.offsetHeight,
result = {
width: 0,
height: 0
};
if (bodyWidth && bodyHeight) {
var viewStates = Windows.UI.ViewManagement.ApplicationViewState;
if (_isPortrait( Windows.UI.ViewManagement.ApplicationView.value)) {
result.width = bodyHeight;
result.height = bodyWidth;
} else {
result.width = bodyWidth;
result.height = bodyHeight;
}
}
return result;
}
var _screenWidth = 0;
var _screenHeight = 0;
var _screenRatioLimits = {
"5:4": { value: 5 / 4, limit: 1.29, width: 1024, height: 819 }, //1.25
"4:3": { value: 4 / 3, limit: 1.47, width: 1024, height: 768 }, // 1.33
"16:10": { value: 16 / 10, limit: 1.69, width: 1366, height: 853 }, // 1.6
"16:9": { value: 16 / 9, width: 1366, height: 768 }// 1.78
};
var _screenRatio = "16:9";
function _checkScreenSize() {
var size = _getScreenSize();
var screenW = size.width;
var screenH = size.height;
if (screenW !== _screenWidth || screenH !== _screenHeight) {
_screenWidth = screenW;
_screenHeight = screenH;
var thisScreenRatio = _screenWidth / _screenHeight;
if (thisScreenRatio < _screenRatioLimits["5:4"].limit) {
_screenRatio = "5:4";
} else if (thisScreenRatio < _screenRatioLimits["4:3"].limit) {
_screenRatio = "4:3";
} else if (thisScreenRatio < _screenRatioLimits["16:10"].limit) {
_screenRatio = "16:10";
} else {
_screenRatio = "16:9";
}
return true;
} else {
return false;
}
}
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
isPortrait: function isPortrait() {
return _isPortrait(Windows.UI.ViewManagement.ApplicationView.value);
},
calculateScalingRatio: function calculateScalingRatio() {
_checkScreenSize();
return (_screenHeight / _screenRatioLimits[_screenRatio].height).toFixed(2);
},
getScreenRatio: function getScreenRatio() {
_checkScreenSize();
return _screenRatio;
},
ScalingItemsContainer: WinJS.Class.derive(CommonJS.Immersive.ParallaxItemsContainer, function parallaxItemsContainer_ctor(element, options) {
this.preventPlatformAnimation = true;
CommonJS.Immersive.ParallaxItemsContainer.call(this, element, options);
this._initScaling();
this._updateLayout();
},
{
_sizer: null,
_box: null,
_updateLayoutEventBinding: null,
_sizerW: 0,
_sizerH: 0,
_viewState: null,
_imageWidth: 0,
_imageHeight: 0,
attributionData: {
set: function set(value) {
// ScalingItemsContainer does not support attribution data
},
},
_needUpdate: function _needUpdate() {
var sizer = this._sizer;
var w = (sizer && sizer.clientWidth) || 0;
var h = (sizer && sizer.clientHeight) || 0;
var view = Windows.UI.ViewManagement.ApplicationView.value;
var imageSize = this._imageSize;
var imageWidth = (imageSize && imageSize.width) || 0;
var imageHeight = (imageSize && imageSize.height) || 0;
if (w !== this._sizerW || h !== this._sizerH
|| view !== this._viewState || _checkScreenSize()
|| imageWidth !== this._imageWidth || imageHeight !== this._imageHeight) {
this._sizerW = w;
this._sizerH = h;
this._viewState = view;
this._imageWidth = imageWidth;
this._imageHeight = imageHeight;
return true;
} else {
return false;
}
},
_calculateScalingRatio: function _calculateScalingRatio() {
// calculate scaling
this._itemsContainerScale = (_screenHeight / _screenRatioLimits[_screenRatio].height).toFixed(2);
},
_updateLayout: function _updateLayout() {
if (this._needUpdate()) {
var sizer = this._sizer;
var w = sizer && sizer.clientWidth;
var h = sizer && sizer.clientHeight;
if (w && h) {
var offsiteX = 0;
var offsiteY = 0;
this._calculateScalingRatio();
var sh = document.body.offsetHeight;
var bw = (w * this._itemsContainerScale).toFixed();
// calcuate offsite
if (_isPortrait(this._viewState)) {
bw = _screenHeight;
offsiteX = (Math.max(_screenHeight, bw) - w * this._itemsContainerScale) / 2;
if (_screenRatioLimits[_screenRatio].value > _screenRatioLimits["4:3"].limit) {
// wide screen
offsiteY = (sh - h * this._itemsContainerScale) / 2;
} else {
// normal screen
offsiteY = (sh - h * this._itemsContainerScale) / 3;
}
} else {
offsiteY = (sh - h * this._itemsContainerScale) / 2;
}
this._sizer.style["transform"] = "scale(" + this._itemsContainerScale + ")";
this._sizer.style["transform-origin"] = this._rtl ? "top right" : "top left";
this._sizer.style.marginTop = offsiteY + "px";
this._sizer.style.marginLeft = offsiteX + "px";
// layout container
this._box.style["width"] = [bw, "px"].join("");
this._box.style["height"] = [sh, "px"].join("");
}
// layout image
if (w && h && this._currentImageElement) {
var imageSize = this._imageSize;
if (imageSize) {
// image dimensions
var imageWidth = imageSize.width;
var imageHeight = imageSize.height;
var imageRatio = imageWidth / imageHeight;
var imageScale = 0;
var imageOffsiteX = 0;
var imageOffsiteY = 0;
var itemRatio = w / h;
if (imageRatio > itemRatio) {
imageScale = h * this._itemsContainerScale / imageHeight;
} else {
imageScale = w * this._itemsContainerScale / imageWidth;
}
if (_isPortrait(this._viewState)) {
imageOffsiteX = (screen.width - imageWidth * imageScale) / 2;
} else {
imageOffsiteX = (bw - imageWidth * imageScale) / 2;
}
if (_isPortrait(this._viewState) && _screenRatioLimits[_screenRatio].value < _screenRatioLimits["4:3"].limit) {
imageOffsiteY = (sh - imageHeight * imageScale) / 3;
} else {
imageOffsiteY = (sh - imageHeight * imageScale) / 2;
}
this._currentImageElement.style["marginLeft"] = imageOffsiteX + "px";
this._currentImageElement.style["marginTop"] = imageOffsiteY + "px";
this._currentImageElement.style["width"] = (imageWidth * imageScale).toFixed() + "px";
this._currentImageElement.style["height"] = (imageHeight * imageScale).toFixed() + "px";
}
}
if (w && h) {
this._sizer.style.opacity = 1;
}
}
},
// overwrite ParallexItemsContainer function
_initBackgroundImage: function _initBackgroundImage(options) {
var that = this;
var viewport = this.element;
WinJS.Utilities.addClass(viewport, "platformPlaceHolderLarge");
var imageElement = this._currentImageElement = document.createElement("div");
imageElement.id = "platformHeroImage";
//imageElement.style.opacity = 0;
WinJS.Utilities.addClass(imageElement, "platformImageCardImage");
viewport.appendChild(imageElement);
var stagingImageElement = this._stagingImageElement = document.createElement("img");
stagingImageElement.id = "platformHeroStagingImage";
stagingImageElement.style.opacity = 0;
WinJS.Utilities.addClass(stagingImageElement, "platformImageCardImage");
viewport.appendChild(stagingImageElement);
this._imagePosition = 0;
},
_initScaling: function _initScaling(){
this._box = this.element;
this._sizer = this._itemsContainer;
CommonJS.setAutomationId(this._itemsContainer, this.element, "itemsContainer");
_checkScreenSize();
this._calculateScalingRatio();
this._updateLayoutEventBinding = this._updateLayout.bind(this);
window.addEventListener("resize", this._updateLayoutEventBinding);
},
// overwrite ParallexItemsContainer function
_layoutHeroImage: function _layoutHeroImage() {
this._updateLayout();
},
_rtl: {
get: function get() {
return window.getComputedStyle(this.element).direction === "rtl";
}
},
_disposeTree: function _disposeTree(element) {
if (element) {
// Check if element has a disposable win control
if (element.winControl && element.winControl.dispose) {
element.winControl.dispose();
}
// Recursively dispose element children
if (element.children && element.children.length) {
for (var childIdx = 0; childIdx < element.children.length; childIdx++) {
this._disposeTree(element.children[childIdx]);
}
}
}
},
dispose: function dispose() {
if (this._updateLayoutEventBinding) {
window.removeEventListener("resize", this._updateLayoutEventBinding);
}
CommonJS.Immersive.ParallaxItemsContainer.prototype.dispose.call(this);
// Since we can be loading custom win controls
// we need to call dispose on them recursively
if (this._elements) {
for (var handle in this._elements) {
// Get element
var element = this._elements[handle];
// Dispose of element subtree
this._disposeTree(element);
}
}
}
})
});
})();
/* File: ListView2.js */
(function ListView2Init() {
"use strict";
var _left = "left";
WinJS.Namespace.define("CommonJS.UI", {
PanHelper: WinJS.Class.define(
function panHelper_ctor (elem, options) {
//constructor
this._elem = elem;
this._rtl = !!(options.rtl);
this._gestureStartCallback = options.gestureStart || function () { };
this._gestureChangeCallback = options.gestureChange || function () { };
this._gestureEndCallback = options.gestureEnd || function () { };
this._gestureTapCallback = options.gestureTap || function () { };
this._gestureMultiTouchCallback = options.gestureMultiTouch || function () { };
this._gestureHoldEdge = options.gestureHoldEdge || function () { };
this._msPointerDownHandler = this._msPointerDown.bind(this);
this._msPointerUpHandler = this._msPointerUp.bind(this);
this._msGestureStartHandler = this._msGestureStart.bind(this);
this._msGestureChangeHandler = this._msGestureChange.bind(this);
this._msGestureTapHandler = this._msGestureTap.bind(this);
this._msGestureEndHandler = this._msGestureEnd.bind(this);
this.leftEdge = options.leftEdgeWidth || options.edgeWidth || CommonJS.UI.PanHelper.SCREENEDGETHRESHOLD;
this.rightEdge = options.rightEdgeWidth || options.edgeWidth || CommonJS.UI.PanHelper.SCREENEDGETHRESHOLD;
}, {
// public:
enable: function enable() {
this._attachHandlers();
this._enabled = true;
},
disable: function disable() {
this._detachHandlers();
this._enabled = false;
},
isEnabled: function isEnabled() {
return this._enabled;
},
reset: function reset() {
if (this._msGesture && this._gesturePointerId) {
// cancel current gesture
this._msGestureGoing = false;
this._gesturePointerId = null;
this._msGesture.stop();
}
},
totalTranslationX: 0,
totalTranslationY: 0,
totalScale: 1,
// private:
_onGestureStart: function _onGestureStart(e) {
e.type = "gestureStart";
if (this._gestureStartCallback(e)) {
this._preventFutureHandle = true;
}
},
_onGestureChange: function _onGestureChange(e) {
e.type = "gestureChange";
if (this._gestureChangeCallback(e)) {
this._preventFutureHandle = true;
}
},
_onGestureEnd: function _onGestureEnd(e) {
e.type = "gestureEnd";
if (this._gestureEndCallback(e)) {
this._preventFutureHandle = true;
}
},
_onGestureTap: function _onGestureTap(e) {
e.type = "gestureTap";
if (this._gestureTapCallback(e)) {
this._preventFutureHandle = true;
}
},
_onGestureMultiTouch: function _onGestureMultiTouch(e) {
e.type = "gestureMultiTouch";
if (this._gestureMultiTouchCallback(e)) {
this._preventFutureHandle = true;
}
},
_msPointerDown: function _msPointerDown(e) {
try {
var leftButton, rightButton,
touchInput = (e.pointerType === CommonJS.UI.PanHelper.TOUCHTYPE);
if (WinJS.Utilities.hasWinRT) {
// xButton is true when you've x-clicked with a mouse or pen. Otherwise it is false.
var currentPoint = Windows.UI.Input.PointerPoint.getCurrentPoint(e.pointerId);
var pointProps = currentPoint.properties;
if (!(touchInput || pointProps.isInverted || pointProps.isEraser || pointProps.isMiddleButtonPressed)) {
rightButton = pointProps.isRightButtonPressed;
leftButton = !rightButton && pointProps.isLeftButtonPressed;
} else {
leftButton = rightButton = false;
}
} else {
// xButton is true when you've x-clicked with a mouse. Otherwise it is false.
leftButton = (e.button === WinJS.UI._LEFT_MSPOINTER_BUTTON);
rightButton = (e.button === WinJS.UI._RIGHT_MSPOINTER_BUTTON);
}
// in special condition, there is no msPointerUp event, and mouse pointerID is always 1. We should avoid add pointerCount twice on mouse
if ((leftButton || touchInput) && this._msGesture) {
if (!this._gesturePointerId) {
// HACKY, listView control will reset gesture on its viewport if there is no action (tap, swipe) enabled.
if (!this._msGesture.target) {
this._msGesture.target = this._elem;
}
// we only cound the first pan, ignore multi-touch pan
this._gesturePointerId = e.pointerId;
this._msGesture.addPointer(e.pointerId);
this._elem.msSetPointerCapture(e.pointerId);
this.totalTranslationX = 0;
this.totalTranslationY = 0;
this.totalScale = 1;
this._onGestureStart({
target: e.target,
clientX: e.clientX,
clientY: e.clientY,
screenX: e.screenX,
screenY: e.screenY,
pointerId: e.pointerId,
isTouch: touchInput
});
} else {
// multi touch
this._onGestureMultiTouch({
target: e.target,
clientX: e.clientX,
clientY: e.clientY,
screenX: e.screenX,
screenY: e.screenY,
pointerId: e.pointerId,
isTouch: touchInput
});
}
}
} catch (e) {
// e.code === 11, "InvalidStateError" happens when touch and click happens at the same time.
}
},
_msPointerUp: function _msPointerUp(e) {
if (this._gesturePointerId === e.pointerId) {
this._onGestureEnd({
clientX: e.clientX,
clientY: e.clientY,
screenX: e.screenX,
screenY: e.screenY,
translationX: this.totalTranslationX,
translationY: this.totalTranslationY,
scale: this.totalScale,
pointerId: e.pointerId
});
this._gesturePointerId = null;
}
if (this._preventFutureHandle) {
e.stopPropagation();
e.preventDefault();
this._preventFutureHandle = false;
}
},
_msGestureStart: function _msGestureStart(e) {
this._msGestureGoing = true;
},
_msGestureChange: function _msGestureChange(e) {
if (this._msGestureGoing) {
this.totalTranslationX += e.translationX;
this.totalTranslationY += e.translationY;
this.totalScale *= e.scale;
if (e.detail & e.MSGESTURE_FLAG_INERTIA) {
this._onGestureEnd({
clientX: e.clientX,
clientY: e.clientY,
screenX: e.screenX,
screenY: e.screenY,
translationX: this.totalTranslationX,
translationY: this.totalTranslationY,
scale: this.totalScale,
pointerId: this._gesturePointerId
});
this._gesturePointerId = null;
this._msGestureGoing = false;
}
else {
if (!this._rtl) {
if (e.clientX > document.body.offsetWidth - this.rightEdge) {
this._onRightEdge = true;
} else if (e.clientX < this.leftEdge) {
this._onLeftEdge = true;
} else {
this._onLeftEdge = false;
this._onRightEdge = false;
}
} else {
if (e.clientX < this.rightEdge) {
this._onRightEdge = true;
} else if (e.clientX > document.body.offsetWidth - this.leftEdge) {
this._onLeftEdge = true;
} else {
this._onLeftEdge = false;
this._onRightEdge = false;
}
}
if (this._onLeftEdge || this._onRightEdge) {
if (!this._onEdge) {
this._onEdge = true;
this._onEdgeCount = 0;
if (!this._holdTimer) {
this._holdTimer = setTimeout(this._holdEdge.bind(this), CommonJS.UI.PanHelper.FRAMEINTERVAL);
}
}
} else {
if (this._onEdge) {
this._onEdge = false;
this._onLeftEdge = false;
this._onRightEdge = false;
this._onEdgeCount = 0;
if (this._holdTimer) {
clearTimeout(this._holdTimer);
this._holdTimer = null;
}
}
}
this._lastGesture = {
clientX: e.clientX,
clientY: e.clientY,
screenX: e.screenX,
screenY: e.screenY,
translationX: this.totalTranslationX,
translationY: this.totalTranslationY,
scale: this.totalScale,
onLeftEdge: this._onLeftEdge,
onRightEdge: this._onRightEdge
};
this._onGestureChange(this._lastGesture);
}
}
},
_msGestureTap: function _msGestureTap(e) {
this._onGestureTap({
clientX: e.clientX,
clientY: e.clientY,
screenX: e.screenX,
screenY: e.screenY,
translationX: this.totalTranslationX,
translationY: this.totalTranslationY,
scale: this.totalScale,
pointerId: this._gesturePointerId
});
},
_msGestureEnd: function _msGestureEnd(e) {
if (this._msGestureGoing && this._gesturePointerId) {
this._onGestureEnd({
clientX: e.clientX,
clientY: e.clientY,
screenX: e.screenX,
screenY: e.screenY,
translationX: this.totalTranslationX,
translationY: this.totalTranslationY,
scale: this.totalScale,
pointerId: this._gesturePointerId
});
this._gesturePointerId = null;
this._msGestureGoing = false;
}
},
_holdEdge: function _holdEdge() {
if (this._onEdge) {
if (this._onEdgeCount < CommonJS.UI.PanHelper.EDGEHOLDDELAY) {
this._onEdgeCount++;
} else {
this._gestureHoldEdge({
clientX: this._lastGesture.clientX,
clientY: this._lastGesture.clientY,
screenX: this._lastGesture.screenX,
screenY: this._lastGesture.screenY,
translationX: this._lastGesture.translationX,
translationY: this._lastGesture.translationY,
onLeftEdge: this._onLeftEdge,
onRightEdge: this._onRightEdge
});
}
this._holdTimer = setTimeout(this._holdEdge.bind(this), CommonJS.UI.PanHelper.FRAMEINTERVAL);
} else if (this._holdTimer) {
clearTimeout(this._holdTimer);
this._holdTimer = null;
}
},
_attachHandlers: function _attachHandlers() {
if (this._elem) {
this._msGesture = new MSGesture();
this._msGesture.target = this._elem;
this._elem.addEventListener("MSPointerDown", this._msPointerDownHandler, true);
this._elem.addEventListener("MSPointerUp", this._msPointerUpHandler, true);
this._elem.addEventListener("MSPointerCancel", this._msPointerUpHandler, true);
this._elem.addEventListener("MSLostPointerCapture", this._msPointerUpHandler, true);
this._elem.addEventListener("MSGestureStart", this._msGestureStartHandler, true);
this._elem.addEventListener("MSGestureChange", this._msGestureChangeHandler, true);
this._elem.addEventListener("MSGestureTap", this._msGestureTapHandler, true);
this._elem.addEventListener("MSGestureEnd", this._msGestureEndHandler, true);
}
},
_detachHandlers: function _detachHandlers() {
if (this._elem) {
this._elem.removeEventListener("MSPointerDown", this._msPointerDownHandler, true);
this._elem.removeEventListener("MSPointerUp", this._msPointerUpHandler, true);
this._elem.removeEventListener("MSPointerCancel", this._msPointerUpHandler, true);
this._elem.removeEventListener("MSLostPointerCapture", this._msPointerUpHandler, true);
this._elem.removeEventListener("MSGestureStart", this._msGestureStartHandler, true);
this._elem.removeEventListener("MSGestureChange", this._msGestureChangeHandler, true);
this._elem.removeEventListener("MSGestureTap", this._msGestureTapHandler, true);
this._elem.removeEventListener("MSGestureEnd", this._msGestureEndHandler, true);
if (this._holdTimer) {
clearTimeout(this._holdTimer);
this._holdTimer = null;
}
// must release msGesture's target, otherwise, there will be a memory leak
if (this._msGesture) {
this._msGesture.target = null;
this._msGesture = null;
}
}
},
_msGesture: null,
_elem: null,
_gestureStartCallback: null,
_gestureChangeCallback: null,
_gestureEndCallback: null,
_gestureTapCallback: null,
_gestureMultiTouchCallback: null,
_gestureHoldEdge: null,
_enabled: false,
_msGestureGoing: false,
_preventFutureHandle: false,
}, {
TOUCHTYPE: 2, // HACKY: copied from WinJS.UI
EDGEHOLDDELAY: 6, // 6 frames time is close to 0.2 second
SCREENEDGETHRESHOLD: 40, // 40 pixel area as screen edge
FRAMEINTERVAL: 33, // 33 microseconds per frame, 30 frames per second
}),
ListView2: WinJS.Class.derive(WinJS.UI.ListView,
function listView2_ctor (element, options) {
// constructor
// set up pre-required properties
options = options || {};
this._list = null;
this._objectList = {};
this._deleteButtonList = {};
this._pinnedItemCount = 0;
this._handleOutOfOrderItems = CommonJS.UI.ListView2.HANDLEOUTOFORDERITEMS.REMOVE;
this._enabledFeatures = CommonJS.UI.ListView2.FEATURES.ALL;
this._dragThreshold = CommonJS.UI.ListView2.DRAGVERTICALTHRESHOLD;
this._disableEditModeOnBlurred = true;
if (options.handleOutOfOrderItems) {
this.handleOutOfOrderItems = options.handleOutOfOrderItems;
}
if (options.enabledFeatures) {
this.enabledFeatures = options.enabledFeatures;
}
// call parent's constructor
WinJS.UI.ListView.call(this, element, options);
WinJS.Utilities.addClass(this.element, "CommonJSListView2");
// set up extension
this._initListView2(options);
}, {
// new defined public interface
controlKey: {
set: function set(value) {
this._controlKey = value;
}
},
friendlyName: {
set: function set(value) {
this._friendlyName = value;
}
},
logContext: {
set: function set(value) {
this._logContext = value;
}
},
enabledFeatures: {
set: function set(value) {
// if user change this value in runtime, should we update the UI? (e.g. add "Add" tile)
this._enabledFeatures = value;
}
},
handleOutOfOrderItems: {
set: function set(value) {
this._handleOutOfOrderItems = value;
}
},
addTileItemTemplate: {
set: function set(value) {
this._addTileItemTemplate = value;
}
},
cancelDrag: function cancelDrag() {
if (this._target && this._draggedElement && this._inDrag) {
this._targetIndex = this._sourceIndex;
this._dropItem();
}
this._panHelper.reset();
},
saveOrder: function saveOrder() {
var result = false;
if (this._persistenceEnabled()) {
if (this._controlKey) {
var sort = JSON.stringify(this.getOrder());
if (sort) {
var localSettingKey = CommonJS.UI.ListView2.LOCALSETTINGSKEY + this._controlKey;
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
localSettings.values[localSettingKey] = sort;
result = true;
}
}
}
return result;
},
getOrder: function getOrder() {
if (this._list) {
return this._list.filter(function (data) {
return !!(data.itemKey);
}).map(function (data) {
return data.itemKey;
});
} else {
return null;
}
},
isEditMode: function isEditMode() {
return !this._nonEditMode;
},
scrollViewport: {
set: function set(value) {
this._scrollViewport = value;
if (this._scrollViewport) {
this._scrollViewport.addEventListener("scroll", this._onSurfaceScrollHandler);
}
}
},
leftEdgeWidth: {
set: function set(value) {
this._leftEdgeWidth = value;
}
},
rightEdgeWidth: {
set: function set(value) {
this._rightEdgeWidth = value;
}
},
edgeWidth: {
set: function set(value) {
this._edgeWidth = value;
}
},
dragThreshold: {
set: function set(value) {
this._dragThreshold = value;
}
},
allowDragOutControl: {
set: function set(value) {
this._dragOutOfControl = value;
}
},
dragAnimationScaling: {
set: function set(value) {
this._dragAnimationScaling = value;
}
},
dragAnimationOffset: {
set: function set(value) {
this._dragAnimationOffset = value;
}
},
isHandlePrevented: function isHandlePrevented() {
return this._panHelper._preventFutureHandle;
},
switchEditMode: function switchEditMode() {
if (this._removeEnabled()) {
this._nonEditMode = !this._nonEditMode;
if (this._nonEditMode) {
// normal mode
if (this._disableEditModeOnBlurred && this.element) {
this.element.removeEventListener("focusout", this._editButtonBlurredBinding);
}
WinJS.Utilities.removeClass(this._editButtonContainer, "inMode");
WinJS.Utilities.removeClass(this._canvas, "listView2EditMode");
WinJS.Utilities.addClass(this._canvas, "listView2NonEditMode");
this._enableStaticMode(false);
} else {
// edit mode
this.selection.clear();
WinJS.Utilities.addClass(this._editButtonContainer, "inMode");
WinJS.Utilities.addClass(this._canvas, "listView2EditMode");
WinJS.Utilities.removeClass(this._canvas, "listView2NonEditMode");
this._enableStaticMode(true);
if (this._disableEditModeOnBlurred) {
this.element.addEventListener("focusout", this._editButtonBlurredBinding);
}
}
}
},
enableEditMode: function enableEditMode() {
if (this._nonEditMode) {
this.switchEditMode();
}
},
disableEditMode: function disableEditMode() {
if (!this._nonEditMode) {
this.switchEditMode();
}
},
disableEditModeOnBlurred:{
set: function set(value){
this._disableEditModeOnBlurred = value;
}
},
draggedElementRender: {
// suggest to use CommonJS.UI.ListView2.defaultDraggedElementRender
set: function set(value){
this._draggedElementRender = value;
}
},
// new added event:
// listreordered
// itemdeleted
// additeminvoked
// overrided public interface
itemDataSource: {
get: function get() {
if (!this._descriptor.itemDataSource) {
this._descriptor.itemDataSource = Object.getOwnPropertyDescriptor(WinJS.UI.ListView.prototype, "itemDataSource");
}
var descriptor = this._descriptor.itemDataSource;
return descriptor.get.call(this);
},
set: function set(value) {
if (!this._descriptor.itemDataSource) {
this._descriptor.itemDataSource = Object.getOwnPropertyDescriptor(WinJS.UI.ListView.prototype, "itemDataSource");
}
var descriptor = this._descriptor.itemDataSource;
// TODO: use IListDataSource interface instead of using _list private member which is a member of WinJS.Binding.List implementation. Its scope covers this whole control
if (this._list && this._list.length) {
this._clearListBinding();
}
var list = value._list;
var that = this;
var item = null;
if (list) {
if (this._addEnabled()) {
// must put this before we sort the list. After the list is sort, there is a short period that ListView can't update the UI if item is inserted.
var length = list.length;
if (length) {
if (!list.getAt(length - 1).isCommonJSAddTile) {
list.push(CommonJS.UI.ListView2.ADDTILEITEMOBJECT);
} else {
list.getAt(length - 1).reusedCommonJSAddTile = true;
}
} else {
list.push(CommonJS.UI.ListView2.ADDTILEITEMOBJECT);
}
}
if (this._persistenceEnabled()) {
var order = this._readOrder();
if (order) {
var orderMap = {};
order.forEach(function (key, index) {
orderMap[key] = index + 1;
});
var defaultOrder = 0;
if (that._handleOutOfOrderItems === CommonJS.UI.ListView2.HANDLEOUTOFORDERITEMS.MOVETOBOTTOM) {
defaultOrder = order.length + 1;
}
list.sort(function (a, b) {
var aOrder = orderMap[a.itemKey] || defaultOrder;
var bOrder = orderMap[b.itemKey] || defaultOrder;
if (a.isCommonJSAddTile === true) {
aOrder = order.length + 2;
}
if (b.isCommonJSAddTile === true) {
bOrder = order.length + 2;
}
return aOrder - bOrder;
});
// Can't use filter function here, since filter will create a different list
if (that._handleOutOfOrderItems === CommonJS.UI.ListView2.HANDLEOUTOFORDERITEMS.REMOVE) {
for (var i = 0; i < list.length; i++) {
item = list.getAt(i);
if (orderMap[item.itemKey] || item.isCommonJSAddTile === true) {
break;
}
}
if (i > 0) {
list.splice(0, i);
}
}
}
}
// ensure all the pinned item is at the top of the list
for (var index = 0, iLen = list.length; index < iLen; index++) {
item = list.getAt(index);
if (item.pinned) {
if (index !== this._pinnedItemCount) {
item.pinned = false;
} else {
this._pinnedItemCount++;
}
}
}
if (this._addEnabled()) {
list.onitemmoved = that._listUpdatedHandler;
list.onitemmutated = that._listUpdatedHandler;
}
list.onitemremoved = function listView2_bindingListItemRemoved (e) {
if (e.detail && e.detail.value && e.detail.value.pinned) {
if (e.detail.index < that._pinnedItemCount) {
that._pinnedItemCount--;
}
}
if (that._addEnabled()) {
that._listUpdatedHandler();
}
if (that._removeEnabled()) {
that._updateEditButton();
}
};
list.oniteminserted = function listView2_bindingListItemInserted (e) {
if (e.detail && e.detail.value && e.detail.value.pinned) {
if (e.detail.index > that._pinnedItemCount) {
var itemData = list.getAt(e.detail.index);
itemData.pinned = false;
} else {
that._pinnedItemCount++;
}
} else if (that._addEnabled()) {
that._listUpdatedHandler();
}
if (that._removeEnabled()) {
that._updateEditButton();
}
that._setSaveOrderTimer(true);
};
this._list = list;
if (this._removeEnabled()) {
this._updateEditButton();
}
return descriptor.set.call(this, list.dataSource);
} else {
return descriptor.set.call(this, value);
}
}
},
itemTemplate: {
get: function get() {
if (!this._descriptor.itemTemplate) {
this._descriptor.itemTemplate = Object.getOwnPropertyDescriptor(WinJS.UI.ListView.prototype, "itemTemplate");
}
var descriptor = this._descriptor.itemTemplate;
return descriptor.get.call(this);
},
set: function set(itemTemplate) {
var that = this;
var newItemTemplate = function listView2_ItemTemplateRenderer (itemPromise) {
// first leavel of template promise wrapper to get the item.
return itemPromise.then(function listView2_ItemTemplateRenderer1stWrapper (item) {
var itemData = item.data;
// check it's a normal tile or an "add" tile
if (that._addEnabled() && item.data.isCommonJSAddTile) {
var renderPromise = null;
if (that._addTileItemTemplate) {
renderPromise = that._getItemTemplatePromiseResult(that._addTileItemTemplate(itemPromise)).then(function (element) {
if (element) {
element.addEventListener("keydown", that._addTileKeydownBinding);
element.addEventListener("click", that._addTileClickedBinding);
that._addTileElement = element;
}
return element;
});
} else {
var addTile = CommonJS.createElement("div", that.element, "ListView2AddTile");
WinJS.Utilities.addClass(addTile, "addTileContainer");
var addTileIcon = CommonJS.createElement("span", addTile, "AddIcon");
WinJS.Utilities.addClass(addTileIcon, "addTileIcon");
addTile.appendChild(addTileIcon);
that._addTileID = addTile.id;
addTile.addEventListener("keydown", that._addTileKeydownBinding);
addTile.addEventListener("click", that._addTileClickedBinding);
that._addTileElement = addTile;
renderPromise = WinJS.Promise.wrap(addTile);
}
return renderPromise;
} else {
var promise = that._getItemTemplatePromiseResult(itemTemplate(itemPromise));
// second level of template promise wrapper to invoke caller's template render, and get the DIV
return promise.then(function listView2_ItemTemplateRenderer2ndWrapper (result) {
// itemDIV is the DIV created by caller's tempalte render
var prepareReorder = function (itemDiv) {
if (!itemDiv) {
return null;
}
if (!itemData || !itemData.itemKey) {
return itemDiv;
}
if (!itemDiv.id) {
CommonJS.setAutomationId(itemDiv, that.element, "ListView2" + itemData.itemKey);
}
WinJS.Utilities.addClass(itemDiv, "ListView2ItemDiv");
if (itemData.pinned) {
WinJS.Utilities.addClass(itemDiv, "listView2PinnedItem");
} else if (that._removeEnabled()) {
// create the delete button which will be hidden when nonEditMode class is applied to canvas element
var deleteButton = null;
if (!itemData.unremovable) {
deleteButton = CommonJS.createElement("button", itemDiv, "DeleteButton");
WinJS.Utilities.addClass(deleteButton, "listView2ItemDeleteButton");
deleteButton.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/Common/Delete"));
deleteButton.title = PlatformJS.Services.resourceLoader.getString("/Common/Delete");
var deleteButtonKeydownHandler = that._handleKeydown(that._removeItemByElement, [itemDiv]);
deleteButton.addEventListener("keydown", deleteButtonKeydownHandler);
itemDiv.appendChild(deleteButton);
that._deleteButtonList[deleteButton.id] = {
key: itemData.itemKey,
keydownHandler: deleteButtonKeydownHandler,
button: deleteButton
};
}
that._objectList[itemDiv.id] = {
id: itemDiv.id,
key: itemData.itemKey,
element: itemDiv,
deleteButton: deleteButton,
data: itemData
};
}
return itemDiv;
};
if (WinJS.Promise.is(result)) {
return result.then(prepareReorder);
} else {
return prepareReorder(result);
}
});
}
});
};
if (!this._descriptor.itemTemplate) {
this._descriptor.itemTemplate = Object.getOwnPropertyDescriptor(WinJS.UI.ListView.prototype, "itemTemplate");
}
var descriptor = this._descriptor.itemTemplate;
descriptor.set.call(this, newItemTemplate);
}
},
_clearListBinding: function _clearListBinding(){
if (this._focusElement && this._focusChangeHandlerCount > 0) {
this._focusElement.removeEventListener("blur", this._focusChangeBinding);
this._focusElement = null;
}
var list = this._list;
if (list) {
list.onitemmoved = null;
list.onitemmutated = null;
list.onitemremoved = null;
list.oniteminserted = null;
var length = list.length;
if (length) {
var lastItem = list.getAt(length - 1);
if (lastItem.isCommonJSAddTile) {
if (lastItem.reusedCommonJSAddTile) {
lastItem.reusedCommonJSAddTile = false;
} else {
list.pop();
}
}
}
}
if (this._addTileElement) {
if (this._addTileKeydownBinding) {
this._addTileElement.removeEventListener("keydown", this._addTileKeydownBinding);
this._addTileKeydownBinding = null;
}
if (this._addTileClickedBinding) {
this._addTileElement.removeEventListener("click", this._addTileClickedBinding);
this._addTileClickedBinding = null;
}
}
if (this._removeEnabled()) {
for (var key in this._deleteButtonList) {
var deleteObject = this._deleteButtonList[key];
if (deleteObject && deleteObject.button && deleteObject.keydownHandler) {
deleteObject.button.removeEventListener("keydown", deleteObject.keydownHandler);
}
this._deleteButtonList[key] = null;
}
}
this._pinnedItemCount = 0;
this._objectList = {};
},
// overrided private function
_dispose: function _dispose() {
try {
if (!this._disposed2) { // ListView use _disposed
this._disposed2 = true;
this.cancelDrag();
this._panHelper.disable();
this._clearListBinding();
if (this._editButtonContainer) {
this._editButtonContainer.onclick = null;
if (this._editButtonContainer && this._editButtonContainerKeydownHandler) {
this._editButtonContainer.removeEventListener("keydown", this._editButtonContainerKeydownHandler);
this._editButtonContainerKeydownHandler = null;
}
}
if (this.element) {
if (this._selectionChangingHandlerBinding) {
this.element.removeEventListener("selectionchanging", this._selectionChangingHandlerBinding);
this._selectionChangingHandlerBinding = null;
}
if (this._selectionChangedHandlerBinding) {
this.element.removeEventListener("selectionchanged", this._selectionChangedHandlerBinding);
this._selectionChangedHandlerBinding = null;
}
if (this._onkeyboardnavigatingBinding) {
this.element.removeEventListener("keyboardnavigating", this._onkeyboardnavigatingBinding);
this._onkeyboardnavigatingBinding = null;
}
if (this._handleArrowKeyNavigationBinding) {
this.element.removeEventListener("keydown", this._handleArrowKeyNavigationBinding);
this._handleArrowKeyNavigationBinding = null;
}
}
if (this._scrollViewport && this._onSurfaceScrollHandler) {
this._scrollViewport.removeEventListener("scroll", this._onSurfaceScrollHandler);
this._onSurfaceScrollHandler = null;
}
if (this._persistenceEnabled()) {
this.saveOrder();
}
}
WinJS.UI.ListView.prototype._dispose.call(this);
} catch (e) {
// WinJS.UI.ListView may throw exception when try to get dataSource on _dispose
}
},
// private
_initListView2: function _initListView2(options) {
// _rtl function of WinJS.UI.ListView doesn't always work.
this._rtl2 = window.getComputedStyle(document.body, null).direction === "rtl";
_left = !this._rtl2 ? "left" : "right";
if (this._canvas) {
WinJS.Utilities.addClass(this._canvas, "noVerticalGesture");
WinJS.Utilities.addClass(this._canvas, "listView2NonEditMode");
this._nonEditMode = true;
}
var PanHelper = this._panHelper = new CommonJS.UI.PanHelper(this._viewport,
{
gestureStart: this._gestureStart.bind(this),
gestureChange: this._gestureChange.bind(this),
gestureEnd: this._gestureEnd.bind(this),
gestureHoldEdge: this._gestureHoldEdge.bind(this),
gestureTap: this._gestureTap.bind(this),
gestureMultiTouch: this._gestureMultiTouch.bind(this),
leftEdgeWidth: this._leftEdgeWidth,
rightEdgeWidth: this._rightEdgeWidth,
edgeWidth: this._edgeWidth,
rtl: this._rtl2
});
PanHelper.enable();
if (this._removeEnabled()) {
this._addEditButton(options);
}
this._addTileClickedBinding = this._addTileClicked.bind(this);
this._addTileKeydownBinding = this._handleKeydown(this._addTileClicked);
this._focusChangeBinding = this._onFocusChange.bind(this);
this._focusChangeHandler = {};
this._focusChangeHandlerCount = 0;
if (!this._dragAnimationScaling) {
this._dragAnimationScaling = CommonJS.UI.ListView2.DRAGANIMATIONSCALING;
}
if (!this._dragAnimationOffset) {
this._dragAnimationOffset = CommonJS.UI.ListView2.DRAGANIMATIONOFFSET;
}
this._draggedElementBlurredBinding = this._draggedElementBlurred.bind(this);
this._editButtonBlurredBinding = this._editButtonBlurred.bind(this);
this._savedTapBehavior = this.tapBehavior;
this._savedSelectionMode = this.selectionMode;
this._staticModeLevel = 0;
this._listUpdatedHandler = this._keepAddTileAtEnd.bind(this);
this._onSurfaceScrollHandler = this._onSurfaceScroll.bind(this);
if (this._addEnabled()) {
this._selectionChangingHandlerBinding = this._selectionChangingHandler.bind(this);
this.element.addEventListener("selectionchanging", this._selectionChangingHandlerBinding);
}
this._selectionChangedHandlerBinding = this._selectionChangedHandler.bind(this);
this.element.addEventListener("selectionchanged", this._selectionChangedHandlerBinding);
this._onkeyboardnavigatingBinding = this._onkeyboardnavigating.bind(this);
this.element.addEventListener("keyboardnavigating", this._onkeyboardnavigatingBinding);
this._handleArrowKeyNavigationBinding = this._handleArrowKeyNavigation.bind(this);
this.element.addEventListener("keydown", this._handleArrowKeyNavigationBinding);
},
_addEnabled: function _addEnabled() {
return !!(this._enabledFeatures & CommonJS.UI.ListView2.FEATURES.ADD);
},
_removeEnabled: function _removeEnabled() {
return !!(this._enabledFeatures & CommonJS.UI.ListView2.FEATURES.REMOVE);
},
_reorderEnabled: function _reorderEnabled() {
return !!(this._enabledFeatures & CommonJS.UI.ListView2.FEATURES.REORDER);
},
_persistenceEnabled: function _persistenceEnabled() {
return !!(this._enabledFeatures & CommonJS.UI.ListView2.FEATURES.PERSISTENCE);
},
_readOrder: function _readOrder() {
var result = null;
if (this._persistenceEnabled()) {
var localSettingKey = CommonJS.UI.ListView2.LOCALSETTINGSKEY + this._controlKey;
var localSettings = Windows.Storage.ApplicationData.current.localSettings;
var sortString = localSettings.values[localSettingKey];
if (sortString) {
result = JSON.parse(sortString);
}
}
return result;
},
_addEditButton: function _addEditButton(options) {
function getTopOffset(elChild, elAncestor) {
var _y = 0;
var el = elChild;
while (el && el !== elAncestor && !isNaN(el.offsetTop)) {
_y += el.offsetTop - el.scrollTop;
el = el.parentNode;
}
return _y;
}
if (this.element && this._viewport) {
this._editButtonContainer = CommonJS.createElement("button", this._viewport, "EditButtonContainer");
this._editButtonContainer.setAttribute("aria-label", PlatformJS.Services.resourceLoader.getString("/Common/Delete"));
WinJS.Utilities.addClass(this._editButtonContainer, "listView2EditButtonContainer");
if (options.editButtonTemplate) {
CommonJS.loadModule(editButtonTemplate.moduleInfo, editButtonTemplate.moduleData, this._editButtonContainer).done();
} else {
var editButton = CommonJS.createElement("div", this._editButtonContainer);
WinJS.Utilities.addClass(editButton, "listView2EditButton");
this._editButtonContainer.appendChild(editButton);
}
var topOffset = getTopOffset(this._viewport, this.element);
this._editButtonContainer.style.top = (topOffset - 27) + "px";
this.element.appendChild(this._editButtonContainer);
var that = this;
this._editButtonContainer.onclick = function () {
var logContext = that._logContext || CommonJS.UI.ListView2.DEFAULTLOGCONTEXT;
var logElement = (that._friendlyName || that._controlKey || "ListView2") + " edit button";
var logActionMethod = that._isTouch ? Platform.Telemetry.UserActionMethod.touch : Platform.Telemetry.UserActionMethod.unknown;
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal,
logContext,
logElement,
Platform.Telemetry.UserActionOperation.click,
logActionMethod,
0);
that.switchEditMode();
};
this._editButtonContainerKeydownHandler = this._handleKeydown(this.switchEditMode);
this._editButtonContainer.addEventListener("keydown", this._editButtonContainerKeydownHandler);
this._updateEditButton();
}
},
_updateEditButton: function _updateEditButton() {
if (this._editButtonContainer && this._list) {
if (this._list.length === 0 || (this._list.length === 1 && this._addEnabled())) {
WinJS.Utilities.addClass(this._editButtonContainer, "platformHide");
} else {
WinJS.Utilities.removeClass(this._editButtonContainer, "platformHide");
}
}
},
_addFocusElement: function _addFocusElement(element) {
element = element || this.element;
if (element && !this._focusElement) {
this._focusElement = CommonJS.createElement("div", this._viewport, "focusElement");
var focusElementStyle = this._focusElement.style;
focusElementStyle.width = 0;
focusElementStyle.height = 0;
focusElementStyle.border = "none";
focusElementStyle.padding = 0;
focusElementStyle.margin = 0;
element.appendChild(this._focusElement);
}
},
_onFocusChange: function _onFocusChange(e) {
var continueFocus = false;
var toBeRemoved = [];
for (var key in this._focusChangeHandler) {
var listener = this._focusChangeHandler[key];
if (listener) {
var result = listener(e);
continueFocus = continueFocus || result;
if (!result) {
toBeRemoved.push(key);
}
}
}
if (continueFocus) {
this._focusElement.focus();
}
for (var i = 0; i < toBeRemoved.length; i++) {
this._removeFocusChangeListener(toBeRemoved[i]);
}
},
_addFocusChangeListener: function _addFocusChangeListener(key, listener) {
var noHandlerRegistered = this._focusChangeHandlerCount === 0;
if (!this._focusChangeHandler[key]) {
this._focusChangeHandlerCount++;
}
this._focusChangeHandler[key] = listener;
if (!this._focusElement) {
this._addFocusElement();
}
if (noHandlerRegistered && this._focusElement) {
this._focusElement.focus();
this._focusElement.addEventListener("blur", this._focusChangeBinding);
}
},
_removeFocusChangeListener: function _removeFocusChangeListener(key) {
if (this._focusChangeHandler[key]) {
this._focusChangeHandler[key] = null;
this._focusChangeHandlerCount--;
}
if (this._focusChangeHandlerCount === 0) {
if (this._focusElement) {
if (this._focusChangeBinding) {
this._focusElement.removeEventListener("blur", this._focusChangeBinding);
this._focusChangeBinding = null;
}
if (this._focusElement.parentElement) {
this._focusElement.parentElement.removeChild(this._focusElement);
}
this._focusElement = null;
}
}
},
_enableStaticMode: function _enableStaticMode(real) {
var oldLevel = this._staticModeLevel;
if (real) {
this._staticModeLevel++;
} else {
this._staticModeLevel--;
}
if (this._staticModeLevel < 0) {
console.warn("ListView2: staticModeLevel < 0");
this._staticModeLevel = 0;
}
if (this._staticModeLevel === 0) {
this.tapBehavior = this._savedTapBehavior;
this.selectionMode = this._savedSelectionMode;
} else {
if (oldLevel === 0 && this._staticModeLevel === 1) {
this._savedTapBehavior = this.tapBehavior;
this._savedSelectionMode = this.selectionMode;
this.tapBehavior = WinJS.UI.TapBehavior.none;
this.selectionMode = WinJS.UI.SelectionMode.none;
}
}
},
_checkScrollViewport: function _checkScrollViewport() {
if (!this._scrollViewport) {
var viewport = this._viewport;
while (viewport) {
if (viewport.offsetWidth < viewport.scrollWidth && viewport.offsetWidth <= document.body.offsetWidth) {
break;
}
viewport = viewport.parentElement;
}
this.scrollViewport = viewport;
}
},
_removeFromEditList: function _removeFromEditList(index) {
var element = this.elementFromIndex(index);
if (element && element.id) {
var deleteButton = element.getElementsByClassName("listView2ItemDeleteButton");
var deleteButtonId = null;
if (deleteButton && deleteButton.length) {
deleteButtonId = deleteButton[0].id;
element.removeChild(deleteButton[0]);
}
this._objectList[element.id] = null;
if (deleteButtonId) {
var deleteObject = this._deleteButtonList[deleteButtonId];
if (deleteObject && deleteObject.button && deleteObject.keydownHandler) {
deleteObject.button.removeEventListener("keydown", deleteObject.keydownHandler);
}
this._deleteButtonList[deleteButtonId] = null;
}
WinJS.Utilities.addClass(element, "listView2PinnedItem");
}
},
_gestureStart: function _gestureStart(e) {
if (this._draggedElement || (this._pointerId && this._pointerId !== e.pointerId)) {
return null;
}
// TODO: there is a bug to use e.target or e.srcElement. After a tap happens, next tap's target or srcElement is something wrong. maybe should call getFocuse after tap.
var element = document.elementFromPoint(e.clientX, e.clientY);
this._targetPosition = {};
this._addTileTapped = false;
this._isTouch = e.isTouch;
if (this._list && (this._removeEnabled() || this._reorderEnabled())) {
while (element) {
if (element === this._addTileElement) {
this._addTileTapped = true;
break;
}
if (this._deleteButtonList[element.id]) {
this._tapKey = this._deleteButtonList[element.id].key;
}
if (this._objectList[element.id]) {
this._target = this._objectList[element.id];
this._sourceIndex = this.indexOfElement(this._target.element);
this._pointerId = e.pointerId;
this._isTouchSelection = this._isTouch;
this._targetPosition.x = parseInt(this._target.element.parentElement.style[_left]);
this._targetPosition.y = parseInt(this._target.element.parentElement.style.top);
this._originalLeft = this._target.element.parentElement.style[_left];
this._originalTop = this._target.element.parentElement.style.top;
this._checkScrollViewport();
this._drageAnimationScalingString = "scale(" + this._dragAnimationScaling + ")";
break;
}
element = element.parentElement;
}
}
},
_gestureChange: function _gestureChange(e) {
// HACKY: I have to use ListView's selection mode to deal with selection status
var mode = this._mode;
if (this._reorderEnabled() && !this._inDrag && this._target) {
this._clearTargetIndex();
var box = {
top: parseInt(this._target.element.parentElement.style.top),
left: parseInt(this._target.element.parentElement.style[_left])
};
box.right = box.left + parseInt(this._target.element.parentElement.style.width);
box.bottom = box.top + parseInt(this._target.element.parentElement.style.height);
this._getItemIndex(box);
if (!this._isTouch) {
// there is no swip selection behavior for mouse or pen
this._inDrag = true;
} else if (!this._detectSelectionAllowed) {
var allowed = mode.selectionAllowed(this._sourceIndex);
this._detectSelectionAllowed = true;
if (!allowed.canSelect) {
// if an item is not selection allowed, there is no swip selection behavior
this._inDrag = true;
}
} else {
// there is a selection behavior, we'll use drag threshold
var x = Math.abs(e.translationX),
y = Math.abs(e.translationY);
if (y > this._dragThreshold) {
this._inDrag = true;
} else if (y > CommonJS.UI.ListView2.SELECTIONTHRESHOLD && x > CommonJS.UI.ListView2.DRAGHORIZONTALTHRESHOLD) {
this._inDrag = true;
}
}
}
if (this._target && this._inDrag && this._canvas && this._list) {
if (!this._draggedElement) {
// cancel current selection mode, 
// and (HACKY) _selectionMode's gesture recognizer is in a bad status, we must force reset it (via releasing current recognizer)
mode.resetPointerDownState();
if (mode.cachedRecognizer) {
try {
msReleaseWinRTObject(mode.cachedRecognizer);
}
catch (e) {
// double-release throws, and in this scenario we are OK with just
// ignoring that
}
mode.cachedRecognizer = null;
}
// HACKY: remove selection tag from the cloned element (_renderSelection is a private function of WinJS.UI.ListView)
this._renderSelection(this._target.element.parentElement, this._target.element, false);
if (this._draggedElementRender) {
this._draggedElement = this._draggedElementRender(this._target.element, this._target.data);
} else {
this._draggedElement = this._target.element.parentElement.cloneNode(true);
}
WinJS.Utilities.addClass(this._target.element.parentElement, "shadowElement");
this.selection.clear();
WinJS.Utilities.removeClass(this._draggedElement, "win-swipe");
WinJS.Utilities.addClass(this._draggedElement, "draggedElement");
this._canvas.appendChild(this._draggedElement);
this._addFocusElement(this._draggedElement);
this._addFocusChangeListener("draggedElementReset", this._draggedElementBlurredBinding);
this._enableStaticMode(true);
this._scrollLimit = {
min: Number.MIN_VALU,
max: Number.MAX_VALU
};
var element, item;
this._dragAffectedElements = [];
for (var index = 0, count = this._list.length; index < count; index++) {
element = this.elementFromIndex(index);
item = this._list.getAt(index);
if (item && element && !item.pinned && index !== this._sourceIndex) {
this._dragAffectedElements.push(element);
}
}
if (this._dragAffectedElements.length) {
this._privateDragSourceStart(this._dragAffectedElements);
}
if (this._dragOutOfControl) {
WinJS.Utilities.addClass(this._viewport, "ListView2ViewportAllowDragOutControl");
if (this.element.parentElement) {
WinJS.Utilities.addClass(this.element.parentElement, "clusterOnTopStock");
}
}
this._isTouchSelection = false; // it's already beyond the drag threshold, there is no selection.
}
this._updateTargetPositionAndScrolling(e);
}
},
_gestureHoldEdge: function _gestureHoldEdge(e) {
if (this._target && this._draggedElement) {
this._updateTargetPositionAndScrolling(e);
}
},
_updateTargetPositionAndScrolling: function _updateTargetPositionAndScrolling(e) {
var scroll = 0;
var leftOffset = 0;
if (this._rtl2) {
leftOffset = this._targetPosition.x - e.translationX;
} else {
leftOffset = this._targetPosition.x + e.translationX;
}
var width = this._itemsTotalWidth || this._canvas.offsetWidth;
if (this._dragOutOfControl || (leftOffset >= 0 && leftOffset + this._draggedElement.offsetWidth <= width)) {
this._draggedElement.style[_left] = leftOffset + "px";
} else if (leftOffset < 0) {
this._draggedElement.style[_left] = "0px";
} else if (leftOffset + this._draggedElement.offsetWidth > width) {
this._draggedElement.style[_left] = (width - this._draggedElement.offsetWidth) + "px";
}
var topOffset = this._targetPosition.y + e.translationY;
var topLimit = 0;
if (this._canvas.offsetParent === this._viewport) {
topLimit = -this._canvas.offsetTop;
} else {
topLimit = this._viewport.offsetTop - this._canvas.offsetTop;
}
topLimit--; // get top limit one less pixel, so that when an item is dragged at the topend, it could be inserted at the top of the column.
if (this._dragOutOfControl || (topOffset >= topLimit && topOffset + this._draggedElement.offsetHeight <= this._viewport.offsetHeight)) {
this._draggedElement.style.top = topOffset + "px";
} else if (topOffset < topLimit) {
this._draggedElement.style.top = topLimit + "px";
} else if (topOffset + this._draggedElement.offsetHeight > this._viewport.offsetHeight) {
this._draggedElement.style.top = (this._viewport.offsetHeight - this._draggedElement.offsetHeight) + "px";
}
if (e.onLeftEdge && this._scrollViewport && this._scrollViewport.scrollLeft > 0) {
scroll = Math.max(-1 * CommonJS.UI.ListView2.SCROLLSTEP, 0 - this._scrollViewport.scrollLeft);
} else if (e.onRightEdge && this._scrollViewport && this._scrollViewport.scrollLeft + this._scrollViewport.offsetWidth < this._scrollViewport.scrollWidth) {
scroll = Math.min(CommonJS.UI.ListView2.SCROLLSTEP, this._scrollViewport.scrollWidth - (this._scrollViewport.scrollLeft + this._scrollViewport.offsetWidth));
}
if (this._scrollViewport && scroll) {
// check if we want to scroll
var allowScroll = true;
if (scroll < 0 && this._scrollViewport.scrollLeft <= this._scrollLimit.min) {
allowScroll = false;
} else if (scroll > 0 && this._scrollViewport.scrollLeft >= this._scrollLimit.max) {
allowScroll = false;
} else {
if ((leftOffset + this._draggedElement.offsetWidth > 0 && scroll < 0) || (leftOffset < width && scroll > 0)) {
allowScroll = true; // do nothing here.
} else if (scroll < 0) {
allowScroll = false;
this._scrollLimit.min = this._scrollViewport.scrollLeft;
} else if (scroll > 0) {
allowScroll = false;
this._scrollLimit.max = this._scrollViewport.scrollLeft;
}
}
if (allowScroll) {
this._scrollViewport.scrollLeft += scroll;
}
}
var oldTargetIndex = this._targetIndex;
var box = {
top: parseInt(this._draggedElement.style.top),
left: parseInt(this._draggedElement.style[_left])
};
box.right = box.left + parseInt(this._draggedElement.style.width);
box.bottom = box.top + parseInt(this._draggedElement.style.height);
this._getItemIndex(box);
if (this._targetIndex !== oldTargetIndex) {
this._moveInAnimation();
}
},
_onSurfaceScroll: function _onSurfaceScroll(e) {
if (this._oldScrollLeft !== null) {
if (this._draggedElement) {
var scroll = this._scrollViewport.scrollLeft - this._oldScrollLeft;
this._targetPosition.x += scroll;
this._draggedElement.style[_left] = (this._targetPosition.x + e.translationX) + "px";
}
}
this._oldScrollLeft = this._scrollViewport.scrollLeft;
},
_gestureEnd: function _gestureEnd(e) {
if (this._target && this._draggedElement) {
var box = {
top: parseInt(this._draggedElement.style.top),
left: parseInt(this._draggedElement.style[_left])
};
box.right = box.left + parseInt(this._draggedElement.style.width);
box.bottom = box.top + parseInt(this._draggedElement.style.height);
this._getItemIndex(box);
this._dropItem();
} else {
this._tapKey = null;
this._target = null;
}
if (this._pointerId === e.pointerId) {
this._pointerId = null;
}
},
_dropItem: function _dropItem() {
this._removeFocusChangeListener("draggedElementReset");
this._draggedElement.parentElement.removeChild(this._draggedElement);
WinJS.Utilities.removeClass(this._target.element.parentElement, "shadowElement");
if (this._animationBoxElement1) {
WinJS.Utilities.removeClass(this._animationBoxElement1.parentElement, "dropMoveAnimationElement");
}
if (this._animationBoxElement2) {
WinJS.Utilities.removeClass(this._animationBoxElement2.parentElement, "dropMoveAnimationElement");
}
this._nextAnimation(this._privateDragSourceEnd, [this._dragAffectedElements]);
if (this._sourceIndex > this._targetIndex) {
this._targetIndex++;
}
if (this._sourceIndex !== this._targetIndex) {
this._list.move(this._sourceIndex, this._targetIndex);
var eventObject = document.createEvent("CustomEvent");
eventObject.initCustomEvent("listreordered", true, true, null);
var element = this.element;
element.dispatchEvent(eventObject);
var logContext = this._logContext || CommonJS.UI.ListView2.DEFAULTLOGCONTEXT;
var logElement = (this._friendlyName || this._controlKey || "ListView2") + " tile";
var logActionMethod = this._isTouch ? Platform.Telemetry.UserActionMethod.touch : Platform.Telemetry.UserActionMethod.unknown;
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal,
logContext,
logElement,
"Drag And Drop",
logActionMethod,
0);
this._setSaveOrderTimer(true);
}
if (this._dragOutOfControl) {
WinJS.Utilities.removeClass(this._viewport, "ListView2ViewportAllowDragOutControl");
if (this.element.parentElement) {
WinJS.Utilities.removeClass(this.element.parentElement, "clusterOnTopStock");
}
}
var targetElement = this.elementFromIndex(this._targetIndex) || this.element;
if (targetElement) {
targetElement.focus();
}
this._draggedElement = null;
this._pointerId = null;
this._target = null;
this._inDrag = false;
this._tapKey = null;
this._addTileTapped = false;
this._enableStaticMode(false);
this._detectSelectionAllowed = false;
},
_addTileClicked: function _addTileClicked(e){
this._target = null;
this.disableEditMode();
this.selection.clear();
var eventObject = document.createEvent("CustomEvent");
eventObject.initCustomEvent("additeminvoked", true, true, null);
var element = this.element;
var that = this;
var logContext = this._logContext || CommonJS.UI.ListView2.DEFAULTLOGCONTEXT;
var logElement = (this._friendlyName || this._controlKey || "ListView2") + " addButton";
var logActionMethod = this._isTouch ? Platform.Telemetry.UserActionMethod.touch : Platform.Telemetry.UserActionMethod.unknown;
msSetImmediate(function () {
element.dispatchEvent(eventObject);
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal,
logContext,
logElement,
Platform.Telemetry.UserActionOperation.click,
logActionMethod,
0);
});
},
_gestureTap: function _gestureTap(e) {
var eventObject = null;
if (this._removeEnabled() && this._tapKey && this._target && this._target.element && this._list) {
this._removeItemByElement(this._target.element);
return true;
}
if (this._addTileTapped) {
return true;
}
if (!(this._target && this._target.element)) {
return false;
} else {
return !this._nonEditMode;
}
},
_gestureMultiTouch: function _gestureMultiTouch(e) {
if (!this._draggedElement) {
// not dragged yet
this._pointerId = null;
this._target = null;
}
},
_removeItemByElement: function _removeItemByElement(element) {
var index = this.indexOfElement(element);
var eventObject = null;
var removedItem = this._list.splice(index, 1);
removedItem = removedItem.length ? removedItem[0] : null;
var elementID = element.id;
var deleteButton = this._objectList[elementID].deleteButton;
if (deleteButton && this._deleteButtonList[deleteButton.id] && this._deleteButtonList[deleteButton.id].keydownHandler) {
deleteButton.removeEventListener("keydown", this._deleteButtonList[deleteButton.id].keydownHandler);
this._deleteButtonList[deleteButton.id] = null;
}
this._objectList[elementID] = null;
this._tapKey = null;
this._pointerId = null;
this._target = null;
this._detectSelectionAllowed = false;
eventObject = document.createEvent("CustomEvent");
eventObject.initCustomEvent("itemdeleted", true, true, removedItem);
this.element.dispatchEvent(eventObject);
this._setSaveOrderTimer(true);
var logContext = this._logContext || CommonJS.UI.ListView2.DEFAULTLOGCONTEXT;
var logElement = (this._friendlyName || this._controlKey || "ListView2") + " tile";
var logActionMethod = this._isTouch ? Platform.Telemetry.UserActionMethod.touch : Platform.Telemetry.UserActionMethod.unknown;
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal,
logContext,
logElement,
"Delete",
logActionMethod,
0);
},
_isOneRow: function _isOneRow() {
return this.layout.maxRows === 1;
},
_getItemIndex: function _getItemIndex(box) {
this._getItemIndexImpl(box);
var targetIndex = this._targetIndex;
if (targetIndex < this._sourceIndex) {
// to drop an item before a pinned item, _sourceIndex must be larger than targetIndex;
targetIndex++;
}
if (targetIndex < this._pinnedItemCount) {
this._targetIndex = this._pinnedItemCount - 1;
}
var maxIndex = this._list.length - 1;
if (this._addEnabled()) {
maxIndex--;
}
if (maxIndex < 0) {
maxIndex = 0;
}
if (targetIndex > maxIndex) {
this._targetIndex = maxIndex;
}
},
_getItemIndexImpl: function _getItemIndexImpl(box) {
function checkBox(b1, b2) {
var y = b1.top;
var x = (b1.left + b1.right) / 2;
return (b2.left < x && b2.right >= x && b2.top < y && b2.bottom >= y);
};
if (this._isOneRow()) {
return this._getItemIndexInOneRow(box);
}
// 1. check if it's still in the original box
if (this._targetBox && checkBox(box, this._targetBox)) {
return this._targetIndex;
}
var targetIndex = this._sourceIndex;
var count = this._list.length;
// 2. if matrix available, check matrix.
if (this._targetBoxMatrix) {
var y = box.top;
var x = (box.left + box.right) / 2;
var sizeX = this._targetBoxMatrix.matrix[this._targetBoxMatrix.matrix.length - 1][0].right;
var sizeY = this._targetBoxMatrix.matrix[0][this._targetBoxMatrix.matrix[0].length - 1].bottom;
var leftIndex = 0;
var topIndex = 0;
var sizeBox = this._targetBoxMatrix.matrix[0][0];
var height = (sizeBox.bottom - sizeBox.top);
var halfHeight = height / 2;
if (x < 0 || x >= sizeX || y < -height || y >= sizeY) {
this._targetIndex = this._sourceIndex;
this._targetBox = null;
} else if (y < 0) {
leftIndex = Math.floor(x / (sizeBox.right - sizeBox.left));
this._targetIndex = leftIndex * this._targetBoxMatrix.Y - 1;
this._targetBox = null;
} else if (y >= (sizeY - height)) {
leftIndex = Math.floor(x / (sizeBox.right - sizeBox.left));
this._targetIndex = (leftIndex + 1) * this._targetBoxMatrix.Y - 1;
this._targetBox = null;
} else {
leftIndex = Math.floor(x / (sizeBox.right - sizeBox.left));
topIndex = Math.floor(y / (sizeBox.bottom - sizeBox.top));
if (leftIndex * this._targetBoxMatrix.Y + topIndex >= count) {
if (this._addEnabled()) {
this._targetIndex = count - 2;
} else {
this._targetIndex = count - 1;
}
this._targetBox = null;
} else {
this._targetIndex = leftIndex * this._targetBoxMatrix.Y + topIndex;
this._targetBox = this._targetBoxMatrix.matrix[leftIndex][topIndex];
}
}
return this._targetIndex;
}
// 3. go through all element, and create matrix if possible.
if (!this.layout.groupInfo || !this.layout.groupInfo.enableCellSpanning) {
this._targetBoxMatrix = {
matrix: [],
lastBox: null,
Y: 0,
};
}
var element = null;
var index = 0;
var margins = WinJS.UI._getMargins(this.elementFromIndex(0).parentElement);
var horizontalMargin = margins.left + margins.right;
var verticalMargin = margins.top + margins.bottom;
for (index = 0; index < count; index++) {
element = this.elementFromIndex(index).parentElement;
var targetBox = {
top: parseInt(element.style.top),
left: parseInt(element.style[_left])
};
targetBox.bottom = targetBox.top + parseInt(element.style.height) + verticalMargin;
targetBox.right = targetBox.left + parseInt(element.style.width) + horizontalMargin;
if (this._targetBoxMatrix) {
if (!this._targetBoxMatrix.lastBox || this._targetBoxMatrix.lastBox.left !== targetBox.left) {
this._targetBoxMatrix.matrix.push([]);
}
this._targetBoxMatrix.matrix[this._targetBoxMatrix.matrix.length - 1].push(targetBox);
this._targetBoxMatrix.lastBox = targetBox;
}
if (checkBox(box, targetBox)) {
targetIndex = index;
this._targetBox = targetBox;
// continue to complete _targetBoxMatrix
}
}
if (this._targetBoxMatrix.Y === 0 && this._targetBoxMatrix.matrix.length >= 1) {
this._targetBoxMatrix.Y = this._targetBoxMatrix.matrix[0].length;
}
this._itemsTotalWidth = targetBox.right;
this._targetIndex = targetIndex;
this._targetIndex = parseInt(this._targetIndex);
return targetIndex;
},
_getItemIndexInOneRow: function _getItemIndexInOneRow(box) {
function checkBox(b1, b2) {
var x = (b1.left + b1.right) / 2;
var y = box.top;
return (b2.left < x && b2.right >= x && b2.top < y && b2.bottom >= y);
};
// 1. check if it's still in the current box.
if (this._targetBox && checkBox(box, this._targetBox)) {
return this._targetIndex;
}
var targetIndex = this._sourceIndex;
var targetBox = {};
var count = this._list.length;
var x = (box.left + box.right) / 2;
var y = box.top;
// 2. if matrix is abailable, check matrix.
if (this._targetBoxMatrix) {
var sizeX = this._targetBoxMatrix.matrix[this._targetBoxMatrix.matrix.length - 1].right;
var leftIndex = 0;
var sizeBox = this._targetBoxMatrix.matrix[0];
var halfWidth = (sizeBox.right - sizeBox.left) / 2;
if (x < 0 || x >= sizeX) {
this._targetIndex = this._sourceIndex;
this._targetBox = null;
} else {
leftIndex = Math.floor(x / (sizeBox.right - sizeBox.left));
if (y < this._targetBoxMatrix.matrix[leftIndex].top || y >= this._targetBoxMatrix.matrix[leftIndex].bottom) {
this._targetIndex = this._sourceIndex;
this._targetBox = null;
return this._targetIndex;
}
if (x < (this._targetBoxMatrix.matrix[leftIndex].left + this._targetBoxMatrix.matrix[leftIndex].right) / 2) {
leftIndex--;
}
if (leftIndex >= 0) {
targetBox.left = (this._targetBoxMatrix.matrix[leftIndex].left + this._targetBoxMatrix.matrix[leftIndex].right) / 2;
targetBox.right = this._targetBoxMatrix.matrix[leftIndex].right;
} else if (leftIndex + 1 < this._targetBoxMatrix.matrix.length) {
targetBox.left = this._targetBoxMatrix.matrix[leftIndex + 1].left;
}
if (leftIndex + 1 < this._targetBoxMatrix.matrix.length) {
targetBox.right = (this._targetBoxMatrix.matrix[leftIndex + 1].left + this._targetBoxMatrix.matrix[leftIndex + 1].right) / 2;
}
this._targetIndex = leftIndex;
this._targetBox = targetBox;
}
return this._targetIndex;
}
// 3. if no matrix, go through all elements, and create matrix if possible.
if (!this.layout.groupInfo || !this.layout.groupInfo.enableCellSpanning) {
this._targetBoxMatrix = {
matrix: []
};
}
var element = null;
var index = 0;
var margins = WinJS.UI._getMargins(this.elementFromIndex(0).parentElement);
var horizontalMargin = margins.left + margins.right;
var verticalMargin = margins.top + margins.bottom;
for (index = 0; index < count; index++) {
element = this.elementFromIndex(index).parentElement;
targetBox = {
left: parseInt(element.style[_left])
};
targetBox.right = targetBox.left + parseInt(element.style.width) + horizontalMargin;
targetBox.top = parseInt(element.style.top) - (parseInt(element.style.height) + verticalMargin);
targetBox.bottom = parseInt(element.style.top) + parseInt(element.style.height) + verticalMargin;
if (this._targetBoxMatrix) {
this._targetBoxMatrix.matrix.push(targetBox);
}
if (checkBox(box, targetBox)) {
targetIndex = index;
if (x < (targetBox.left + targetBox.right) / 2) {
targetIndex--;
this._targetBox = {
left: targetBox.left,
right: (targetBox.left + targetBox.right) / 2,
top: targetBox.top,
bottom: targetBox.bottom
};
} else {
this._targetBox = {
left: (targetBox.left + targetBox.right) / 2,
right: targetBox.right,
top: targetBox.top,
bottom: targetBox.bottom
};
}
// continue to complete _targetBoxMatrix
}
}
this._itemsTotalWidth = targetBox.right;
this._targetIndex = targetIndex;
this._targetIndex = parseInt(this._targetIndex);
return targetIndex;
},
_clearTargetIndex: function _clearTargetIndex() {
this._animationBoxElement1 = null;
this._animationBoxElement2 = null;
this._taretIndex = null;
this._targetBox = null;
this._targetBoxMatrix = null;
},
_setSaveOrderTimer: function _setSaveOrderTimer(result) {
if (this._persistenceEnabled() && result) {
if (!this._saveOrderTimer) {
var that = this;
this._saveOrderTimer = setTimeout(function () {
that.saveOrder();
that._saveOrderTimer = null;
}, 200);
}
}
},
_keepAddTileAtEnd: function _keepAddTileAtEnd() {
var that = this;
var list = this._list;
msSetImmediate(function listView2_keepAddTileAtEnd () {
if (list) {
var length = list.length;
if (length && !(list.getAt(length - 1).isCommonJSAddTile)) {
var sourceIndex = -1;
for (var index = 0; index < length - 1; index++) {
if (list.getAt(index).isCommonJSAddTile === true) {
sourceIndex = index;
break;
}
}
if (sourceIndex < 0) {
list.push(CommonJS.UI.ListView2.ADDTILEITEMOBJECT);
} else {
list.move(sourceIndex, length - 1);
}
}
}
});
},
_moveInAnimation: function _moveInAnimation() {
var index1 = null,
index2 = null;
var boxElement1 = null,
boxElement2 = null;
if (this._list && this._targetIndex !== this._sourceIndex && this._targetIndex !== this._sourceIndex - 1) {
index1 = this._targetIndex;
index2 = this._targetIndex + 1;
var count = this._list.length;
var animationTarget = [];
var targetOffset = [];
if (index1 >= this._pinnedItemCount) {
boxElement1 = this.elementFromIndex(index1);
animationTarget.push(boxElement1);
if (this._isOneRow()) {
var tmpOffset1 = { top: 0 };
// WinJS.UI.executeTransition only support "top" and "left", not support "right" 
tmpOffset1.left = this._rtl2 ? this._dragAnimationOffset : -this._dragAnimationOffset;
targetOffset.push(tmpOffset1);
} else {
targetOffset.push({ top: -this._dragAnimationOffset, left: 0 });
}
} else {
boxElement1 = null;
}
if (index2 < count) {
boxElement2 = this.elementFromIndex(index2);
animationTarget.push(boxElement2);
if (this._isOneRow()) {
var tmpOffset2 = { top: 0 };
// WinJS.UI.executeTransition only support "top" and "left", not support "right" 
tmpOffset2.left = this._rtl2 ? -1 * this._dragAnimationOffset : this._dragAnimationOffset;
targetOffset.push(tmpOffset2);
} else {
targetOffset.push({ top: this._dragAnimationOffset, left: 0 });
}
} else {
boxElement2 = null;
}
}
if (boxElement1 || boxElement2) {
this._nextAnimation(this._dragBetweenInHelper.bind(this), [animationTarget, targetOffset]);
} else {
this._nextAnimation(this._dragBetweenLeaveHelper.bind(this));
}
},
_dragBetweenInHelper: function _dragBetweenInHelper(animationTarget, targetOffset) {
var that = this;
animationTarget.forEach(function (element) {
WinJS.Utilities.addClass(element.parentElement, "dropMoveAnimationElement");
});
return this._dragBetweenLeaveHelper().then(function () {
return that._privateDragBetweenEnter(animationTarget, targetOffset).then(function () {
that._animationBoxElement1 = animationTarget[0] || null;
that._animationBoxElement2 = animationTarget[1] || null;
});
});
},
_dragBetweenLeaveHelper: function _dragBetweenLeaveHelper() {
var animationTarget = [];
var that = this;
if (this._animationBoxElement1) {
WinJS.Utilities.removeClass(this._animationBoxElement1.parentElement, "dropMoveAnimationElement");
animationTarget.push(this._animationBoxElement1);
}
if (this._animationBoxElement2) {
WinJS.Utilities.removeClass(this._animationBoxElement2.parentElement, "dropMoveAnimationElement");
animationTarget.push(this._animationBoxElement2);
}
return this._privateDragBetweenLeave(animationTarget).then(function () {
that._animationBoxElement1 = null;
that._animationBoxElement2 = null;
});
},
_privateDragSourceStart: function _privateDragSourceStart(affected) {
return WinJS.UI.executeTransition(
affected,
{
property: "transform",
delay: 0,
duration: 240,
timing: "cubic-bezier(0.1, 0.9, 0.2, 1)",
to: this._drageAnimationScalingString
});
},
_privateDragSourceEnd: function _privateDragSourceEnd(affected) {
return WinJS.UI.executeTransition(
affected,
{
property: "transform",
delay: 0,
duration: 500,
timing: "cubic-bezier(0.1, 0.9, 0.2, 1)",
to: ""
});
},
_translateCallback: function _translateCallback(offsetArray, suffix) {
suffix = suffix || "";
return function (i, elem) {
var offset = offsetArray[i];
return "translate(" + offset.left + "px, " + offset.top + "px) " + suffix;
};
},
_privateDragBetweenEnter: function _privateDragBetweenEnter(target, offset) {
var that = this;
return WinJS.UI.executeTransition(
target,
{
property: "transform",
delay: 0,
duration: 50,
timing: "cubic-bezier(0.1, 0.9, 0.2, 1)",
to: that._translateCallback(offset, that._drageAnimationScalingString)
});
},
_privateDragBetweenLeave: function _privateDragBetweenLeave(target) {
return WinJS.UI.executeTransition(
target,
{
property: "transform",
delay: 0,
duration: 50,
timing: "cubic-bezier(0.1, 0.9, 0.2, 1)",
to: this._drageAnimationScalingString
});
},
_nextAnimation: function _nextAnimation(command, args) {
var that = this;
if (command) {
this._animationNext = {
command: command,
args: args
};
}
if (!this._inAnimation) {
if (this._animationNext) {
this._inAnimation = true;
var animation = that._animationNext;
that._animationNext = null;
animation.command.apply(null, animation.args).then(function () {
that._inAnimation = false;
that._nextAnimation();
});
}
}
},
_handleKeydown: function _handleKeydown(command, parameters) {
var that = this;
return function listView2_spaceEnterKeysHandler (event) {
if (event.keyCode && (event.keyCode === 32 || event.keyCode === 13)) {
command.apply(that, parameters);
event.stopPropagation();
event.preventDefault();
}
};
},
_selectionChangingHandler: function _selectionChangingHandler(e) {
if (this._list && e && e.detail && e.detail.newSelection && e.detail.preventTapBehavior) {
if (this._inDrag) {
// clear selection during drag
// (since this will prevent selection changing from selected to unselected when drag begins, it will get better performance if app is listing to selection changed)
e.detail.newSelection.clear();
e.detail.preventTapBehavior();
} else {
// prevent add tile to be selected
var addTileIndex = this._list.length - 1;
var indices = e.detail.newSelection.getIndices();
if (indices.indexOf(addTileIndex) !== -1) {
e.detail.newSelection.remove(addTileIndex);
e.detail.preventTapBehavior();
}
}
}
},
_selectionChangedHandler: function _selectionChangedHandler(e) {
if (!this._target) { // otherwise, it's in drag n' drop, the selection changed is caused by selectionCancel.
if (this.selection.count() > 0) {
var logContext = this._logContext || CommonJS.UI.ListView2.DEFAULTLOGCONTEXT;
var logElement = (this._friendlyName || this._controlKey || "ListView2") + " tile";
var logActionMethod = this._isTouchSelection ? Platform.Telemetry.UserActionMethod.touch : Platform.Telemetry.UserActionMethod.unknown;
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal,
logContext,
logElement,
"Selection Changed",
logActionMethod,
0);
}
this._isTouchSelection = false;
}
},
_draggedElementBlurred: function _draggedElementBlurred() {
// in case of sleep/lock/switch, there will be a "blur" event, but the activeElement won't change.
if (document.activeElement === this._focusElement) {
this.cancelDrag();
return false;
}
return true;
},
_editButtonBlurred: function _editButtonBlurred() {
var result = false;
var element = document.activeElement;
while (element) {
if (element === this.element) {
result = true;
break;
}
element = element.parentElement;
}
if (!result) {
// new focus is out of this control
this.disableEditMode();
}
},
_getItemTemplatePromiseResult: function _getItemTemplatePromiseResult(itemTemplateResult){
var promise = null;
if (WinJS.Promise.is(itemTemplateResult)) {
promise = itemTemplateResult;
} else if (itemTemplateResult.element && WinJS.Promise.is(itemTemplateResult.element)) {
promise = itemTemplateResult.element;
} else {
promise = WinJS.Promise.wrap(itemTemplateResult);
}
return promise;
},
_onkeyboardnavigating: function _onkeyboardnavigating(e) {
// we'll get this event before arrow key down. so if there is arrow key down but no keyboard navigating, then we must get to boundary
this._keyboardNavigated = true;
},
_handleArrowKeyNavigation: function _handleArrowKeyNavigation(e) {
var mode = this._mode; // HACKY: use ListView selection mode to detect if this is a key to handle.
var Key = WinJS.Utilities.Key;
var keyCode = e.keyCode;
if (this._keyboardNavigated) {
this._keyboardNavigated = false; // reset the flag
} else if (mode.keyboardNavigationHandlers[keyCode]) {
var direction = null;
var NV = CommonJS.NavigableView;
switch (keyCode) {
case Key.upArrow:
direction = NV.Direction.UP;
break;
case Key.downArrow:
direction = NV.Direction.DOWN;
break;
case Key.leftArrow:
direction = NV.Direction.LEFT;
break;
case Key.rightArrow:
direction = NV.Direction.RIGHT;
break;
}
NV.dispatchBoundaryEvent(this.element, direction);
}
},
_list: null,
_controlKey: null,
_descriptor: {},
_objectList: {},
_deleteButtonList: {},
_target: null,
_targetPosition: {},
_draggedElement: null,
_pointerId: null,
_tapKey: null,
_isTouch: false,
_isTouchSelection: false,
_inDrag: false,
_oldScrollLeft: null,
_detectSelectionAllowed: false,
_scrollViewport: null,
_handleOutOfOrderItems: null,
_enabledFeatures: null,
_inAnimation: false,
_animationNext: null,
// normally we use the same edge width for both left and right. But for Cluster Customization, the ListView2 
// doesn't take the whole screen width, it's even not horizontal centering. There is an "add" button at the right
// of the ListView2, which makes ListView2's right edge is wider than its left edge.
_leftEdgeWidth: 0,
_rightEdgeWidth: 0,
_edgeWidth: 0,
_dragOutOfControl: false,
_pinnedItemCount: 0,
_dragAffectedElements: null,
_dragThreshold: null,
_disposed2: false,
_dragAnimationScaling: null,
_dragAnimationOffset: null,
_scrollLimit: null,
_focusElement: null,
_focusChangeBinding: null,
_focusChangeHandler: null,
_focusChangeHandlerCount: 0,
_disableEditModeOnBlurred: true,
_draggedElementBlurredBinding: null,
_editButtonBlurredBinding: null,
_addTileElement: null,
_addTileClickedBinding: null,
_addTileKeydownBinding: null,
_addTileTapped: false,
_draggedElementRender: null,
_rtl2: false,
_keyboardNavigated: false,
_friendlyName: null,
_logContext: null
}, {
defaultDraggedElementRender: function defaultDraggedElementRender(sourceElement, sourceData) {
var elementPrototype = sourceElement.parentElement;
var element = CommonJS.createElement("div", sourceElement, "ListView2SimpleDraggedElement");
WinJS.Utilities.addClass(element, "ListView2SimpleDraggedElement");
element.style.width = elementPrototype.style.width;
element.style.height = elementPrototype.style.height;
var titleElement = CommonJS.createElement("div", element, "titleDiv");
WinJS.Utilities.addClass(titleElement, "titleDiv");
titleElement.innerText = sourceData.title;
element.appendChild(titleElement);
return element;
},
FEATURES: {
ADD: 1,
REMOVE: 2,
REORDER: 4,
PERSISTENCE: 8,
ALL: 15
},
DRAGVERTICALTHRESHOLD: 190,
SELECTIONTHRESHOLD: 21,
DRAGHORIZONTALTHRESHOLD: 50,
SCROLLSTEP: 10,
LOCALSETTINGSKEY: "LISTVIEW2PERSISTENCE",
ADDTILEITEMOBJECT: { isCommonJSAddTile: true },
HANDLEOUTOFORDERITEMS: {
REMOVE: 0,
MOVETOTOP: 1,
MOVETOBOTTOM: 2
},
DRAGANIMATIONSCALING: 0.86,
DRAGANIMATIONOFFSET: 12,
DEFAULTLOGCONTEXT: "Entity Customization List"
})
});
})();
/* File: LocalPanoManager.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformLocalPanoManagerInit() {
"use strict";
WinJS.Namespace.define("CommonJS.LocalPano", {
LocalPanoManager: WinJS.Class.mix(WinJS.Class.define(function localPanoManager_ctor() {
this._geolocator = new Windows.Devices.Geolocation.Geolocator();
this._market = Platform.Utilities.Globalization.getCurrentMarket().toLocaleLowerCase();
this._localSettingsKeyPrefix = "LocalPano_" + this._market + "_";
},
{
_geolocator: null,
_regionData: null,
_regionDataLastDownloaded: null,
_market: null,
_localSettingsKeyPrefix: null,
_activeRegion: null,
_isPrimed: null,
_isEnabled: null,
getRegionsFeed: function getRegionsFeed(forceReload) {
var that = this;
// TODO: This should use the expiration time of the cache entry on disk.
if (!forceReload && this._regionData && this._regionDataLastDownloaded) {
var regionsMemoryCacheAge = Date.now() - this._regionDataLastDownloaded;
regionsMemoryCacheAge = (regionsMemoryCacheAge / 1000) / 60;
if (regionsMemoryCacheAge <= Platform.Configuration.ConfigurationManager.custom.getDictionary("LocalPano").getInt32("MemoryCacheExpiry", 60)) {
return WinJS.Promise.wrap(that._regionData);
}
}
return this.downloadRegionsFeed();
},
downloadRegionsFeed: function downloadRegionsFeed() {
if (!this.isLocalPanoEnabled) {
this._regionData = null;
this._regionDataLastDownloaded = null;
return WinJS.Promise.wrap(null);
}
var that = this;
var queryService = new PlatformJS.DataService.QueryService("RegionListDataSource");
var options = new Platform.DataServices.QueryServiceOptions();
options.retryCount = 3;
if (!this._market || this._market === "") {
return WinJS.Promise.wrap(null);
}
else {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams["market"] = this._market;
if (PlatformJS.isDebug && Platform.Configuration.ConfigurationManager.instance.previewFeaturedDataSource) {
urlParams["previewparams"] = "/" + PlatformJS.Services.appConfig.getString("CMSPreviewParam");
}
else {
urlParams["previewparams"] = "";
}
return new WinJS.Promise(function (complete, error, progress) {
queryService.downloadDataAsync(urlParams, null, null, options).then(function (response) {
var data = null;
var success = false;
try {
if (response.dataString && response.dataString.length) {
data = JSON.parse(response.dataString);
if (!data || !data.regions || data.regions.length === 0) {
queryService.deleteCacheEntryAsync(urlParams, null);
error(ex);
}
else {
success = true;
}
}
}
catch (ex) {
// The data is invalid. Remove from cache.
queryService.deleteCacheEntryAsync(urlParams, null);
error(ex);
}
if (success) {
that._regionData = data;
that._regionDataLastDownloaded = Date.now();
}
else {
that._regionData = null;
that._regionDataLastDownloaded = null;
}
that._validateActiveRegion();
complete(data);
}, function (response) {
if (PlatformJS.Utilities.isPromiseCanceled(response)) {
return;
}
that._regionData = null;
that._regionDataLastDownloaded = null;
error(response);
}, function (response) {
if (response.statusCode === Platform.DataServices.QueryServiceStatusCode.cachedEntryFound) {
progress(response.cachedResponse);
}
});
});
}
},
//getRegionById: function (id) {
//    return this.getRegionsFeed().then(function (regionData) {
//        if (regionData.regions && regionData.regions.length > 0) {
//            for (var i = 0; i < regionData.regions.length; i++) {
//                var region = regionData.regions[i];
//                if (this.compareRegions(id, region.id)) {
//                    return WinJS.Promise.wrap(region);
//                }
//            }
//        }
//        return WinJS.Promise.wrap(null);
//    });
//},
//getRegionByName: function (name) {
//    return this.getRegionsFeed().then(function (regionData) {
//        if (regionData.regions && regionData.regions.length > 0) {
//            for (var i = 0; i < regionData.regions.length; i++) {
//                var region = regionData.regions[i];
//                if (this.compareRegions(name, region.name)) {
//                    return WinJS.Promise.wrap(region);
//                }
//            }
//        }
//        return WinJS.Promise.wrap(null);
//    });
//},
getRegionByGeoposition: function getRegionByGeoposition(geoposition) {
var that = this;
var regionsFeedPromise = this.getRegionsFeed();
if (geoposition) {
return regionsFeedPromise.then(function getRegionByGeoposition_Complete1(response) {
var regionData = response;
return WinJS.Promise.wrap(that._findRegionByGeoposition(regionData, geoposition));
});
}
else {
return regionsFeedPromise.then(function getRegionByGeoposition_Complete2(response) {
if (that._regionData && that._regionData.regions && that._regionData.regions.length > 0) {
var regionData = response;
return that._geolocator.getGeopositionAsync().then(function getGeoPositionAsync_Complete(geopositionResponse) {
return WinJS.Promise.wrap(that._findRegionByGeoposition(regionData, geopositionResponse));
});                            
}
else {
return WinJS.Promise.wrap(null);
}
});
}
},
refreshRegionByGeoposition: function refreshRegionByGeoposition(geoposition) {
var that = this;
return this.getRegionByGeoposition(geoposition).then(function (region) {
that.activeRegion = region ? region.id : null;
return WinJS.Promise.wrap(that.activeRegion);
},
function () {
that.activeRegion = null;
return WinJS.Promise.wrapError(that.activeRegion);
});
},
isActiveRegion: function isActiveRegion(region) {
if (!region || !this.hasActiveRegion) {
return false;
}
else {
return this.compareRegions(region, this.activeRegion);
}
},
compareRegions: function compareRegions(region1, region2) {
if (region1 === region2) {
return true;
}
else if (!region1 && !region2) {
return true;
}
else if (!region1 || !region2) {
return false;
}
else {
return (region1.toLocaleLowerCase() === region2.toLocaleLowerCase());
}
},
isGeopositionAvailable: {
get: function get() {
return  (this._geolocator.locationStatus !== Windows.Devices.Geolocation.PositionStatus.disabled) &&
(this._geolocator.locationStatus !== Windows.Devices.Geolocation.PositionStatus.notAvailable);
}
},
isLocalPanoEnabled: {
get: function get() {
if (this._isEnabled === null) {
var dataSourceAvailable = false;
var enabled = Platform.Configuration.ConfigurationManager.custom.getDictionary("LocalPano").getBool("Enable", false);
if (enabled) {
var dataSources = PlatformJS.Services.manifest.dataSources;
for (var i = 0; i < dataSources.length; i++) {
var dataSource = dataSources[i];
if (dataSource.id === "RegionListDataSource") {
dataSourceAvailable = true;
break;
}
}
}
this._isEnabled = enabled && dataSourceAvailable;
}
return this._isEnabled;
}
},
isPrimed: {
// Only request location once at startup. This is either FRE or first time after an upgrade.
get: function get() {
if (!this._isPrimed) {
this._isPrimed = Windows.Storage.ApplicationData.current.localSettings.values[this._localSettingsKeyPrefix + "Primed"];
}
return this._isPrimed;
},
set: function set(value) {
this._isPrimed = value;
Windows.Storage.ApplicationData.current.localSettings.values[this._localSettingsKeyPrefix + "Primed"] = value;
}
},
hasRegionData: {
get: function get() {
return this._regionData && this._regionData.regions && this._regionData.regions.length > 0;
}
},
hasActiveRegion: {
get: function get() {
return this.activeRegion ? true : false;
}
},
activeRegion: {
// The activeRegion local settings will be read by the managed code in the Platform.
get: function get() {
if (!this._activeRegion) {
this._activeRegion = Windows.Storage.ApplicationData.current.localSettings.values[this._localSettingsKeyPrefix + "ActiveRegion"];
}
return this._activeRegion;
},
set: function set(newRegion) {
var oldRegion = this.activeRegion;
Windows.Storage.ApplicationData.current.localSettings.values[this._localSettingsKeyPrefix + "ActiveRegion"] = newRegion;
this._activeRegion = newRegion;
if (!this.compareRegions(oldRegion, newRegion)) {
this.dispatchEvent("regionChanged", { oldRegion: oldRegion, newRegion: newRegion });
}
}
},
_validateActiveRegion: function _validateActiveRegion() {
// If a region is removed from the region list, we want to remove that from local settings.
if (this.hasActiveRegion) {
if (this._regionData && this._regionData.regions) {
var regions = this._regionData.regions;
for (var i = 0; i < regions.length; i++) {
if (this.isActiveRegion(regions[i].id)) {
return;
}
}
// The active region is invalid.
this.activeRegion = null;
}
}
},
_findRegionByGeoposition: function _findRegionByGeoposition(regionData, geoposition) {
if (regionData.regions && regionData.regions.length > 0 && geoposition) {
for (var i = 0; i < regionData.regions.length; i++) {
var region = regionData.regions[i];
var distance = Platform.Utilities.GeoUtilities.distance(region.latitude, region.longitude, geoposition.coordinate.latitude, geoposition.coordinate.longitude);
if (distance <= region.radius && distance !== -1) {
return region;
}
}
}
return null;
},
},
{
_instance: null,
instance: {
get: function get() {
if (!CommonJS.LocalPano.LocalPanoManager._instance) {
CommonJS.LocalPano.LocalPanoManager._instance = new CommonJS.LocalPano.LocalPanoManager();
}
return CommonJS.LocalPano.LocalPanoManager._instance;
}
}
}), WinJS.Utilities.eventMixin),
});
})();
/* File: MetroPicker.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsMetroPicker() {
"use strict";
WinJS.Namespace.define("CommonJS",
{
MetroPicker: WinJS.Class.define(function MetroPicker_ctor(element, options) {
var attribute = null,
parentSettings = null,
that = this,
regionLabel = null,
regionSelectorContainer = null,
regionSelect = null,
currentLocation = null;
this.element = element ? element : document.createElement("div");
element.winControl = this;
element.className = "platformSettingsGroup platformHide";
this._regionLabel = regionLabel = document.createElement("div");
regionLabel.setAttribute("id", "RegionTitle");
regionLabel.className = "platformSettingsGroupLabel";
regionLabel.innerText = PlatformJS.Services.resourceLoader.getString("/Common/RegionSelectLabel");
this._regionSelectorContainer = regionSelectorContainer = document.createElement("div");
regionSelectorContainer.setAttribute("id", "RegionList");
regionSelectorContainer.setAttribute("role", "group");
regionSelectorContainer.className = "platformRegionSelectContainer";
this._regionSelect = regionSelect = document.createElement("select");
regionSelect.setAttribute("id", "RegionDropdown");
regionSelect.className = "platformRegionSelect";
// TODO: This string was moved from News to Common UX post-string freeze. The below code retrieves it
// from News resources. For the next release, use /Common/RegionCurrentLocation. Currently only the
// News app uses this control, but if another app chooses to use it, the string must be moved.
var currentLocationString = PlatformJS.Services.resourceLoader.getString("CurrentLocationButtonText");
this._currentLocation = currentLocation = document.createElement("div");
currentLocation.setAttribute("role", "button");
currentLocation.setAttribute("aria-label", currentLocationString);
currentLocation.className = "platformCurrentLocationButton";
currentLocation.title = currentLocationString;
currentLocation.onclick = function () {
that._onCurrentLocationClick();
};
element.appendChild(regionLabel);
regionSelectorContainer.appendChild(regionSelect);
regionSelectorContainer.appendChild(currentLocation);
element.appendChild(regionSelectorContainer);
// Find the parent settings dialog
parentSettings = this.element.parentElement;
while (parentSettings) {
attribute = parentSettings.attributes.getNamedItem("data-win-control");
if (attribute && attribute.nodeValue === "WinJS.UI.SettingsFlyout") {
break;
}
parentSettings = parentSettings.parentElement;
}
if (parentSettings) {
this._parentSettingsFlyout = parentSettings.winControl;
this._parentSettingsFlyout.addEventListener("beforeshow", function (event) {
that._populate();
});
}
this._regionSelect.addEventListener("change", function (event) {
that._onRegionChange();
});
WinJS.UI.setOptions(this, options);
},
{
element: null,
_regionSelect: null,
_regionLabel: null,
_currentLocation: null,
_parentSettingsFlyout: null,
visible: {
get: function get() {
return !WinJS.Utilities.hasClass(this.element, "platformHide");
},
set: function set(makeVisible) {
if (makeVisible) {
if (WinJS.Utilities.hasClass(this.element, "platformHide")) {
WinJS.Utilities.removeClass(this.element, "platformHide");
}
}
else {
if (!WinJS.Utilities.hasClass(this.element, "platformHide")) {
WinJS.Utilities.addClass(this.element, "platformHide");
}
}
}
},
_populate: function _populate() {
var that = this;
while (this._regionSelect.hasChildNodes()) {
this._regionSelect.removeChild(this._regionSelect.firstChild);
}
if (!CommonJS.LocalPano.LocalPanoManager.instance.isLocalPanoEnabled) {
this.visible = false;
}
else {
CommonJS.LocalPano.LocalPanoManager.instance.getRegionsFeed().then(function (regionData) {
if (regionData && regionData.regions && regionData.regions.length > 0) {
var localPanoManager = CommonJS.LocalPano.LocalPanoManager.instance;
for (var i = -1; i < regionData.regions.length; i++) {
var option = document.createElement("option");
if (i === -1) {
option.setAttribute("value", "none");
option.innerText = PlatformJS.Services.resourceLoader.getString("/Common/RegionSelectNone");
if (!localPanoManager.hasActiveRegion) {
option.selected = true;
}
}
else {
var region = regionData.regions[i];
option.setAttribute("value", region.id);
option.innerText = region.name;
if (localPanoManager.isActiveRegion(region.id)) {
option.selected = true;
}
}
that._regionSelect.appendChild(option);
}
}
// The drop-down must have more than just the automatic option to be visible.
that.visible = that._regionSelect.length > 1;
},
function (error) {
that.visible = false;
});
}
},
_selectNone: function _selectNone() {
var localPanoManager = CommonJS.LocalPano.LocalPanoManager.instance;
var oldRegion = localPanoManager.activeRegion;
for (var i = 0; i < this._regionSelect.options.length; i++) {
var option = this._regionSelect.options[i];
option.selected = i === 0;
}
localPanoManager.activeRegion = null;
this._instrumentRegionChange(localPanoManager.activeRegion, oldRegion);
},
_onCurrentLocationClick: function _onCurrentLocationClick() {
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
this._showOfflineError();
}
else {
var that = this;
var localPanoManager = CommonJS.LocalPano.LocalPanoManager.instance;
var oldRegion = localPanoManager.activeRegion;
localPanoManager.refreshRegionByGeoposition().then(function refreshRegionSuccess() {
if (!localPanoManager.hasRegionData) {
that._selectNone();
that._showNoContentError();
}
else {
for (var i = 0; i < that._regionSelect.options.length; i++) {
var option = that._regionSelect.options[i];
option.selected = localPanoManager.isActiveRegion(option.value);
}
}
that._instrumentRegionChange(localPanoManager.activeRegion, oldRegion);
that._instrumentCurrentLocationClick(localPanoManager.activeRegion, oldRegion);
},
function refreshRegionError() {
if (Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
that._selectNone();
that._showNoLocationError();
}
else {
that._showOfflineError();
}
that._instrumentRegionChange(localPanoManager.activeRegion, oldRegion);
that._instrumentCurrentLocationClick(localPanoManager.activeRegion, oldRegion);
});
}
},
_onRegionChange: function _onRegionChange() {
var localPanoManager = CommonJS.LocalPano.LocalPanoManager.instance;
if (!Platform.Networking.NetworkManager.instance.isNetworkAvailable) {
this._showOfflineError();
// Change the selection back to what it was in the first place.
for (var i = 0; i < this._regionSelect.options.length; i++) {
var option = this._regionSelect.options[i];
option.selected = localPanoManager.isActiveRegion(option.value);
}
}
else {
var oldRegion = localPanoManager.activeRegion;
if (this._regionSelect.selectedIndex === 0) {
// Disable regional sources.
localPanoManager.activeRegion = null;
}
else {
// Show regional sources for the given region.
localPanoManager.activeRegion = this._regionSelect.value;
}
this._instrumentRegionChange(localPanoManager.activeRegion, oldRegion);
}
},
_showOfflineError: function _showOfflineError() {
var messageString = PlatformJS.Services.resourceLoader.getString("/platform/feedbackOfflineMessage");
var buttonString = PlatformJS.Services.resourceLoader.getString("/platform/close");
var messageBar = new CommonJS.MessageBar(messageString, { autoHide: false });
messageBar.addButton(buttonString, function () { messageBar.hide(); });
messageBar.show();
},
_showNoContentError: function _showNoContentError() {
var messageString = PlatformJS.Services.resourceLoader.getString("/platform/offline_noContent");
var messageBar = new CommonJS.MessageBar(messageString, { autoHide: true });
messageBar.show();
},
_showNoLocationError: function _showNoLocationError() {
var messageString = PlatformJS.Services.resourceLoader.getString("/Common/LocationCannotBeFound");
var buttonString = PlatformJS.Services.resourceLoader.getString("/platform/close");
var messageBar = new CommonJS.MessageBar(messageString, { autoHide: false });
messageBar.addButton(buttonString, function () { messageBar.hide(); });
messageBar.show();
},
_instrumentRegionChange: function _instrumentRegionChange(newRegion, oldRegion) {
if (newRegion !== oldRegion) {
// Count number of region changes.
// 28524 = lo
PlatformJS.Services.instrumentation.incrementInt32(
Platform.Instrumentation.InstrumentationDataSetId.platform,
28524,
1);
// Report the last selected region.
// 28268 = ln
PlatformJS.Services.instrumentation.setString(
Platform.Instrumentation.InstrumentationDataSetId.platform,
28268,
this._regionSelect.value);
}
},
_instrumentCurrentLocationClick: function _instrumentCurrentLocationClick(newRegion, oldRegion) {
if (newRegion !== oldRegion) {
// Count number of Change Location clicks.
// 28780 = lp
PlatformJS.Services.instrumentation.incrementInt32(
Platform.Instrumentation.InstrumentationDataSetId.platform,
28780,
1);
}
}
})
});
})();
/* File: MarketPicker.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsMarketPicker() {
"use strict";
/// <summary>
/// Market picker is a dropdown to select market.
/// </summary>
var NS = WinJS.Namespace.define("CommonJS",
{
MarketPicker: WinJS.Class.define(function marketPicker_ctor(element, options) {
/// <summary>Market picker control
/// </summary>
/// <param name="element">
/// Host dom element
/// </param>
/// <param name="options">
/// Options
/// </param>
var attribute = null,
parentSettings = null,
that = this;
this.element = element ? element : document.createElement("div");
this.element.winControl = this;
this._marketLabel = document.createElement("div");
this._marketLabel.setAttribute("id", "MarketTitle");
this._marketSelectorContainer = document.createElement("div");
this._marketSelectorContainer.setAttribute("id", "MarketList");
this._marketSelectorContainer.setAttribute("role", "group");
this._marketSelect = document.createElement("select");
this._marketSelect.setAttribute("id", "MarketDropDown");
this._applyPrompt = document.createElement("div");
this._applyPrompt.setAttribute("id", "ApplyPrompt");
// Note: Dynamic Market selection has been removed due to Windows issue
// with CultureInfo.CurrentCulture not getting updated correctly without app restart.
this._promptLabel = document.createElement("div");
this._promptLabel.innerText = PlatformJS.Services.resourceLoader.getString("/platform/marketChangePrompt");
WinJS.Utilities.addClass(this.element, "platformSettingsGroup");
WinJS.Utilities.addClass(this._marketLabel, "platformSettingsGroupLabel");
WinJS.Utilities.addClass(this._marketSelect, "platformMarketSelect");
WinJS.Utilities.addClass(this._applyPrompt, "buttonContainer platformHide"); 
this.element.appendChild(this._marketLabel);
this._marketSelectorContainer.appendChild(this._marketSelect);
this.element.appendChild(this._marketSelectorContainer);
this.element.appendChild(this._applyPrompt);             
this._applyPrompt.appendChild(this._promptLabel);
this._marketLabel.innerText = CommonJS.resourceLoader.getString("/platform/languageSetting"); 
// Find the parent settings dialog
parentSettings = this.element.parentElement;
while (parentSettings) {
attribute = parentSettings.attributes.getNamedItem("data-win-control");
if (attribute && attribute.nodeValue === "WinJS.UI.SettingsFlyout") {
break;
}
parentSettings = parentSettings.parentElement;
}
if (parentSettings) {
this._parentSettingsFlyout = parentSettings.winControl;
this._marketPicker_parentSettingsFlyout_beforeShow = function marketPicker_parentSettingsFlyout_beforeShow(event) {
// Remove the event listener
that._parentSettingsFlyout.removeEventListener("beforeshow", that._marketPicker_parentSettingsFlyout_beforeShow);
// Populate the language list before the settings dialog is shown
that._populate();
};
this._parentSettingsFlyout.addEventListener("beforeshow", this._marketPicker_parentSettingsFlyout_beforeShow);
}
this._marketPicker_onChange = function marketPicker_onChange(event) {
that._marketChanged(event);
};
this._marketSelect.addEventListener("change", this._marketPicker_onChange);
WinJS.UI.setOptions(this, options);
},
{
/// <field>
/// The DOM element container
/// </field>
element: null,
_marketSelect: null,
_marketLabel: null,
_marketSelectorContainer: null,
_parentSettingsFlyout: null,
_currentMarket: null,
_applyPrompt: null,
_promptLabel: null,
_marketPicker_onChange: null,
_marketPicker_parentSettingsFlyout_beforeShow: null,
/// <field>
/// Repopulate the markets when language or location changes.
/// </field>
rePopulate: function rePopulate() {
this._marketSelect.removeEventListener("change", this._marketPicker_onChange);
while (this._marketSelect.hasChildNodes()) {
this._marketSelect.removeChild(this._marketSelect.firstChild);
}
this._populate();
this._marketSelect.addEventListener("change", this._marketPicker_onChange);
},
// Populate the market list
_populate: function _populate() {
var data = Platform.Globalization.Marketization.getMarketsForCurrentLanguageAndLocation();
this._currentMarket = Platform.Globalization.Marketization.getCurrentMarket();
for (var i = 0; i < data.length; i++) {
var option = document.createElement("option");
var item = data[i].valueAsString;
option.setAttribute("value", item);
option.innerText = data[i].displayName + "\n";
if (item === this._currentMarket) {
option.selected = true;
}
this._marketSelect.appendChild(option);
}
},
_marketChanged: function _marketChanged(event) {
/// <summary>
/// When the language is changed
/// </summary>
/// <param name="event">
/// Event
/// </param>
/// on market change, persist value and show close prompt
Platform.Globalization.Marketization.hasPendingUpdate = true;
Platform.Globalization.Marketization.setPersistedMarket(this._marketSelect.value);
this._currentMarket = Platform.Globalization.Marketization.getCurrentMarket();
Platform.Configuration.ConfigurationManager.partnerDataFeedUrl = "";
if (WinJS.Utilities.hasClass(this._applyPrompt, "platformHide")) {
WinJS.Utilities.removeClass(this._applyPrompt, "platformHide");
}
Windows.Storage.ApplicationData.current.localSettings.values["platform_resetHistory"] = true;  
},
})
});
})();
/* File: RealizedList.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsRealizedListInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
RealizedList: WinJS.Class.define(function realizedList_ctor() {
this._realized = [];
}, {
_realized: null,
first: {
get: function get() { return this._realized[0]; }
},
last: {
get: function get() {
var realized = this._realized;
return realized[realized.length - 1];
}
},
length: {
get: function get() { return this._realized.length; }
},
insert: function insert(item, index) {
var realized = this._realized;
var count = realized.length - 1;
if (index > count) {
return this.push(item);
}
var previous = (index > 0) ? realized[index - 1] : null;
var next = (index <= count) ? realized[index] : null;
if (previous) {
previous.next = item;
}
if (next) {
next.previous = item;
}
item.previous = previous;
item.next = next;
realized.splice(index, 0, item);
var node = item;
while (node) {
node.index = index;
node = node.next;
index++;
}
},
push: function push(item) {
var realized = this._realized;
var index = realized.length;
var last = this.last;
if (last) {
last.next = item;
item.previous = last;
}
item.index = index;
realized.push(item);
},
remove: function remove(item) {
if (item) {
return this.removeAt(item.index);
}
},
removeAt: function removeAt(index) {
var realized = this._realized;
if (index < realized.length) {
var item = realized[index];
var previous = item.previous;
var next = item.next;
if (previous) {
previous.next = next;
}
if (next) {
next.previous = previous;
}
realized.splice(index, 1);
while (next) {
next.index = index;
next = next.next;
index++;
}
}
},
addRange: function addRange(range) {
for (var i = 0, ilen = range.length; i < ilen; i++) {
this.push(range[i]);
}
},
fromIndex: function fromIndex(index) {
var realized = this._realized;
return realized.length > index ? realized[index] : null;
},
fromHandle: function fromHandle(handle) {
var item = this.first;
while (item && item.handle !== handle) {
item = item.next;
}
return item;
},
fromKey: function fromKey(key) {
var item = this.first;
while (item && item.key !== key) {
item = item.next;
}
return item;
}
})
});
})();
/* File: Panorama.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function () {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
Panorama: WinJS.Class.mix(WinJS.Class.define(function panorama_ctor(element, options) {
options = options || {};
this._element = element;
CommonJS.setAutomationId(this._element);
element.winControl = this;
Object.defineProperties(this, WinJS.Utilities.createEventProperties("afterFirstView"));
// Fix for bug 527426 (change list # 155565):
// prevent splash screen from dismissing since the panorama control has not loaded yet
CommonJS.Progress.canDestroySplash(false);
// End of fix for bug 527426 (change list # 155565):
this._initialize(options);
}, {
// elements
_element: null,
_watermarkElement: null,
// controls
_panel: null,
header: null,
// private variables
_showInertiaBackground: null,
_watermark: null,
// observers
_virtualizer: null,
_headerController: null,
_surfaceSize: null,
_focusManager: null,
// events
afterFirstView: null,
// properties
currentClusterIndex: {
set: function set(value) {
//TODO: implement
this._virtualizer.setCurrentIndex(value);
}
},
// methods
_initialize: function _initialize(options) {
var element = this._element;
// header
var headerElement = CommonJS.createElement("header", element, "header");
headerElement.id = "header";
var header = this.header = new CommonJS.Immersive.Header(headerElement);
header.onSubTitleClick = this._onHeaderSubtitleClick.bind(this);
element.appendChild(headerElement);
// panel
var panelElement = CommonJS.createElement("div", element, "panel");
var panel = this._panel = new NS.PanoramaPanel(panelElement, options);
panel._createOrchestrator = this._createOrchestrator.bind(this);
CommonJS.setAutomationId(panelElement, element, "panel");
element.appendChild(panelElement);
// focus manager
var focusManager = this._focusManager = new NS.PanoramaFocusManager(panel);
// panel virtualizer
var virtualizer = this._virtualizer = new NS.PanoramaVirtualizer(panel, focusManager, this);
WinJS.UI.setOptions(virtualizer, options);
focusManager.virtualizer = virtualizer;
// surface size observer
this._surfaceSize = new NS.SurfaceSizeObserver(panel, header, virtualizer);
// header controller
this._headerController = new NS.PanoramaHeaderOrchestrator(this, panel, header);
// metrics observer for events
this._metrics = new NS.PanoramaMetricsObserver(panel, virtualizer);
// watermark
var watermarkElement = this._watermarkElement = CommonJS.createElement("div", element, "watermark");
WinJS.Utilities.addClass(watermarkElement, "platformWatermark");
PlatformJS.Utilities.setWatermarkText(this._watermark, this._watermarkElement);
element.appendChild(watermarkElement);
},
_createOrchestrator: function _createOrchestrator(item) { return null; },
_onHeaderSubtitleClick: function _onHeaderSubtitleClick(event) {
var item = this._panel.current;
var panoramaItem = item.data;
if (panoramaItem.onHeaderSelection) {
panoramaItem.onHeaderSelection(panoramaItem.clusterKey, item.index, this);
PlatformJS.Services.instrumentation.incrementInt32(
Platform.Instrumentation.InstrumentationDataSetId.platform,
Platform.Instrumentation.InstrumentationDataPointId.totalClusterHeaderTappedCount,
1);
}
},
getPanoramaState: function getPanoramaState() {
if (this._virtualizer) {
return this._virtualizer.getState();
} else {
console.error("this._virtualizer is null or undefined.");
}
return null;
},
getClusterContentControl: function getClusterContentControl(key) {
var item = this._virtualizer._realized ? this._virtualizer._realized.first : null;
while (item) {
if (item.key === key) {
return item.winControl;
} else {
item = item.next;
}
}
return null;
},
cancelPendingOperations: function cancelPendingOperations() {
this._virtualizer.cancelPendingOperations();
},
refresh: function refresh() {
this._panel.detach();
this._panel.current = null;
this._disposeAllClusters();
this._panel.clear();
this._virtualizer.refresh();
},
// properties for binding begins
panoramaState: {
set: function set(value) {
this._virtualizer.panoramaState = value;
},
},
clusterDataSource: {
set: function set(value) {
this._disposeAllClusters();
this._panel.clear();
this._virtualizer.dataSource = value;
}
},
headerOptions: {
set: function set(value) {
WinJS.UI.setOptions(this.header, value);
}
},
showInertiaBackground: {
get: function get() {
return this._panel.showInertiaBackground;
},
set: function set(value) {
this._panel.showInertiaBackground = value;
},
},
compactPanorama: {
get: function get() {
return this._panel.compactPanorama;
},
set: function set(value) {
this._panel.compactPanorama = value;
},
},
zeroOffset: {
get: function get() {
if (this._surfaceSize) {
return this._surfaceSize.zeroOffset;
}
},
},
zeroOffsetSnapPoints: {
get: function get() {
return this._panel.zeroOffsetSnapPoints;
},
set: function set(value) {
this._panel.zeroOffsetSnapPoints = value;
},
},
title: {
set: function set(value) {
this.header.headerTitleText = value;
}
},
subTitle: {
set: function set(value) {
this.header.headerSubTitleText = value;
}
},
headerImage: {
set: function set(value) {
this.header.headerImage = value;
}
},
watermark: {
get: function get() { return this._watermark; },
set: function set(value) {
this._watermark = value;
PlatformJS.Utilities.setWatermarkText(value, this._watermarkElement);
},
},
current: {
get: function get() {
return this._panel.current;
},
},
addScrollListener: function addScrollListener(scrollBinding) {
if (this._panel) {
this._panel.addEventListener("scroll", scrollBinding);
}
},
removeScrollListener: function removeScrollListener(scrollBinding) {
if (this._panel) {
this._panel.removeEventListener("scroll", scrollBinding);
}
},
// properties for binding ends
onLayoutChange: function onLayoutChange(layoutState) {
this._surfaceSize._onWindowResize(layoutState);
this._headerController._invalidateCheckpoints(layoutState);
var realized = this._virtualizer._realized;
if (realized) {
var item = realized.first;
while (item) {
var winControl = item.winControl;
if (winControl && winControl.onLayoutChange) {
winControl.onLayoutChange(layoutState);
}
item = item.next;
}
}
},
onWindowResize: function onWindowResize() {
var realized = this._virtualizer._realized;
if (realized) {
var item = realized.first;
while (item) {
var winControl = item.winControl;
if (winControl && winControl.onWindowResize) {
winControl.onWindowResize(event);
}
item = item.next;
}
}
},
onOrientationChange: function onOrientationChange(event) {
this._surfaceSize._onWindowResize(event);
this._headerController._invalidateCheckpoints(event);
var realized = this._virtualizer._realized;
if (realized) {
var item = realized.first;
while (item) {
var winControl = item.winControl;
if (winControl && winControl.onOrientationChange) {
winControl.onOrientationChange(event);
}
item = item.next;
}
}
},
onDpiChange: function onDpiChange(event) {
if (this.semanticZoom) {
this.semanticZoom.forceLayout();
}
},
dispose: function dispose() {
//TODO: implement dipose against all panorama controls
if (this._virtualizer) {
this._disposeAllClusters();
// release references
this._virtualizer.dispose();
this._virtualizer = null;
}
this.header.dispose();
this.header = null;
this._surfaceSize.dispose();
this._surfaceSize = null;
this._metrics.dispose();
this._metrics = null;
this._panel.dispose();
this._panel = null;
// release DOM elements
this._element.winControl = null;
this._element = null;
this._headerController = null;
this._focusManager.dispose();
this._focusManager = null;
},
_disposeAllClusters: function _disposeAllClusters() {
var item = (this._virtualizer && this._virtualizer._realized) ? this._virtualizer._realized.first : null;
while (item) {
var winControl = item.winControl;
if (winControl && winControl.dispose) {
winControl.dispose();
}
item.element.onresize = null;
item.element = null;
item = item.next;
}
},
}, {
PanoramaItemMode: {
normal: 0,
fullBleed: 1,
},
}), WinJS.Utilities.eventMixin)
});
})();
/* File: PanoramaFocusManager.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformPanoramaFocusManagerInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
PanoramaFocusManager: WinJS.Class.define(function panoramaFocusManager_ctor(panel) {
var WindowEventManager = CommonJS.WindowEventManager;
this._panel = panel;
this._itemBoundsBinding = this._onItemBounds.bind(this);
this._panel.addEventListener("itembounds", this._itemBoundsBinding);
// Semantic zoom will always set focus on panel AFTER we set our focus
// So we listen to that focus event and then set focus back to our cached target element
if (this._panel._element) {
this._panelFocusBinding = this._onPanelElementFocus.bind(this);
this._panel._element.addEventListener("focus", this._panelFocusBinding);
}
this._windowEventManager = WindowEventManager.getInstance();
this._pageUpBinding = this._onPageUp.bind(this);
this._pageDownBinding = this._onPageDown.bind(this);
this._cursorKeyBinding = this._onCursorKey.bind(this);
this._tabKeyBinding = this._onTabKey.bind(this);
this._windowEventManager.addEventListener(WindowEventManager.Events.PAGEUP, this._pageUpBinding);
this._windowEventManager.addEventListener(WindowEventManager.Events.PAGEDOWN, this._pageDownBinding);
this._windowEventManager.addEventListener(WindowEventManager.Events.CURSORKEY, this._cursorKeyBinding);
this._windowEventManager.addEventListener(WindowEventManager.Events.TAB, this._tabKeyBinding);
}, {
// Private Fields
_windowEventManager: null,
_postZoomFocusTarget: null,
_panel: null,
_itemBoundsBinding: null,
_panelFocusBinding: null,
_pageUpBinding: null,
_pageDownBinding: null,
_cursorKeyBinding: null,
_tabKeyBinding: null,
// Private Methods
// After a semantic zoom, this event will always trigger after any of our setFocusTo methods
// So we must explicitly reset focus back to our cached target element
_onPanelElementFocus: function _onPanelElementFocus(event) {
if (this._postZoomFocusTarget) {
this._postZoomFocusTarget.focus();
this._postZoomFocusTarget = null;
}
},
_onPageUp: function _onPageUp(event) {
var originalEvent = event.detail.originalEvent ? event.detail.originalEvent : event.detail;
if (this._panel && originalEvent.target.isContentEditable !== true && originalEvent.target.tagName.toLowerCase() !== "select") {
var previous = (this._panel._current) ? this._panel._current.previous : null;
if (previous) {
this._panel.scrollToWithAnimation(previous.offset, previous, this);
originalEvent.preventDefault();
originalEvent.stopImmediatePropagation();
}
}
},
_onPageDown: function _onPageDown(event) {
var originalEvent = event.detail.originalEvent ? event.detail.originalEvent : event.detail;
if (this._panel && originalEvent.target.isContentEditable !== true && originalEvent.target.tagName.toLowerCase() !== "select") {
var next = (this._panel._current) ? this._panel._current.next : null;
if (next) {
this._panel.scrollToWithAnimation(next.offset, next, this);
originalEvent.preventDefault();
originalEvent.stopImmediatePropagation();
}
}
},
//When user presses a cursor key we need to move keyboard focus to a navigable cluster item, only IF focus isn't on a navigable item already.
//On inital app load or immediately after page navigation, key event target will be the BODY element.
//If a user alternates between keyboard and pointer input, keyboard focus may get 'lost' when using pointer - 
//the next time we receive a keyboard event, the target could potentially be any element on the page - not just the BODY!
//So we have to check that element isn't a navigable element or role.
_onCursorKey: function _onCursorKey(event) {
var originalEvent = event.detail.originalEvent ? event.detail.originalEvent : event.detail;
if (this._panel && (originalEvent.target.tagName.toLowerCase() === "body" || this._isNavigableElement(originalEvent.target) === false)) {
this.setFocusToFirstInPanel(this._panel._current);
originalEvent.preventDefault();
originalEvent.stopImmediatePropagation();
}
},
_isNavigableElement: function _isNavigableElement(element) {
var tagName = element.tagName.toLowerCase(),
role = element.getAttribute("role");
return this._isNavigableTagName(tagName) || this._isNavigableRole(role);
},
_isNavigableRole: function _isNavigableRole(role) {
var isNavigableRole = false;
switch (role) {
case "button":
case "link":
case "option":
case "menuitem":
case "gridcell":
case "tab":
case "listbox":
case "combobox":
isNavigableRole = true; break;
default: break;
}
return isNavigableRole;
},
_isNavigableTagName: function _isNavigableTagName(tagName) {
var isNavigableTagName = false;
switch (tagName) {
case "button":
case "a":
case "option":
case "input":
case "select":
isNavigableTagName = true; break;
default: break;
}
return isNavigableTagName;
},
_onTabKey: function _onTabKey(event) {
this._checkRealization();
},
_onItemBounds: function _onItemBounds(event) {
var Direction = CommonJS.NavigableView.Direction,
item = event.detail.item,
direction = event.detail.direction,
isLtr = (document.dir !== "rtl");
var hasFocusedNext = false;
this._checkRealization();
while (item && !hasFocusedNext) {
hasFocusedNext = true;
if (event.detail.header) {
switch (direction) {
case Direction.RIGHT:
item = (isLtr) ? item.next : item.previous;
hasFocusedNext = this.setFocusToHeader(item) || this.setFocusToContent(item) || this.setFocusToFirstInCluster(item);
break;
case Direction.DOWN:
this.setFocusToContent(item) || this.setFocusToFirstInCluster(item);
break;
case Direction.LEFT:
item = (isLtr) ? item.previous : item.next;
hasFocusedNext = this.setFocusToHeader(item) || this.setFocusToContent(item) || this.setFocusToFirstInCluster(item);
break;
default:
break;
}
} else {
switch (direction) {
case Direction.UP:
this.setFocusToHeader(item);
break;
case Direction.RIGHT:
item = (isLtr) ? item.next : item.previous;
hasFocusedNext = this.setFocusToContent(item) || this.setFocusToFirstInCluster(item) || this.setFocusToHeader(item);
break;
case Direction.LEFT:
item = (isLtr) ? item.previous : item.next;
hasFocusedNext = this.setFocusToContent(item) || this.setFocusToFirstInCluster(item) || this.setFocusToHeader(item);
break;
default:
break;
}
}
}
},
_checkRealization: function _checkRealization() {
if (this.virtualizer && this.virtualizer._isRealizationRequired()) {
this.virtualizer._realizeNextItems(1);
}
},
// Public Fields
virtualizer: null,
// Public Methods
setFocusTo: function setFocusTo(item, ignoreAnimation) {
return this.setFocusToContent(item, ignoreAnimation) || this.setFocusToHeader(item, ignoreAnimation) || this.setFocusToFirstInCluster(item, ignoreAnimation);
},
// Iterate through clusters until a focussable item is found
setFocusToFirstInPanel: function setFocusToFirstInPanel(item) {
var success = false;
while (item && success === false) {
success = this.setFocusTo(item);
item = item.next;
}
return success;
},
setFocusToFirstInCluster: function setFocusToFirstInCluster(item) {
var success = false;
var firstTabIndexItem = null;
if (item && item.element) {
firstTabIndexItem = item.element.querySelector("*[tabIndex='0']");
if (firstTabIndexItem) {
CommonJS.Immersive.PanoramaFocusManager.animationToContent(firstTabIndexItem);
success = true;
// If we arrived here from semantic zoom, focus will get stolen to panel
// So we must cacche our intended target and reset focus on it later
this._postZoomFocusTarget = firstTabIndexItem;
}
}
return success;
},
setFocusToHeader: function setFocusToHeader(item) {
var success = false;
if (item && item.element) {
var headerElement = item.element.querySelector(".platformClusterHeader");
if (headerElement) {
var headerElementLink = headerElement.querySelector("*[role='link']");
if (headerElementLink && headerElementLink.getAttribute("tabIndex") === "0") {
CommonJS.Immersive.PanoramaFocusManager.animationToContent(headerElementLink);
success = true;
// If we arrived here from semantic zoom, focus will get stolen to panel
// So we must cacche our intended target and reset focus on it later
this._postZoomFocusTarget = headerElementLink;
}
}
}
return success;
},
setFocusToContent: function setFocusToContent(item, ignoreAnimation) {
var success = false;
if (item && item.winControl) {
var content = item.winControl._domElement || item.winControl.element;
if (content) {
var activeDescendant = content.querySelector("[tabIndex='0']");
}
}
if (activeDescendant) {
if (ignoreAnimation) {
activeDescendant.focus();
} else {
CommonJS.Immersive.PanoramaFocusManager.animationToContent(activeDescendant);
}
success = true;
// If we arrived here from semantic zoom, focus will get stolen to panel
// So we must cacche our intended target and reset focus on it later
this._postZoomFocusTarget = activeDescendant;
}
return success;
},
dispose: function dispose() {
this._panel.removeEventListener("itembounds", this._itemBoundsBinding);
if (this._panel._element) {
this._panel._element.removeEventListener("focus", this._panelFocusBinding);
}
this._windowEventManager.removeEventListener(CommonJS.WindowEventManager.Events.PAGEUP, this._pageUpBinding);
this._windowEventManager.removeEventListener(CommonJS.WindowEventManager.Events.PAGEDOWN, this._pageDownBinding);
this._windowEventManager.removeEventListener(CommonJS.WindowEventManager.Events.CURSORKEY, this._cursorKeyBinding);
this._windowEventManager.removeEventListener(CommonJS.WindowEventManager.Events.TAB, this._tabKeyBinding);
this._panel = null;
}
},
{
animationToContent: function animationToContent(nextItem) {
var nextElement = nextItem;
var nextElementOffset = 0;
var nextElementOffsetCounting = 0;
var startCounting = false;
var isRTL = (window.getComputedStyle(document.body, null).direction === "rtl");
var offset = 0;
while (nextElement && !WinJS.Utilities.hasClass(nextElement, "platformPanorama") && !WinJS.Utilities.hasClass(nextElement, "platformMainContent")) {
if (WinJS.Utilities.hasClass(nextElement, "platformClusterContent") || startCounting) {
nextElementOffsetCounting += nextElement.offsetLeft;
startCounting = true;
}
offset = nextElement.offsetLeft;
if (isRTL) {
offset = nextElement.offsetLeft + nextElement.clientWidth;
if (nextElement.offsetParent) {
offset -= nextElement.offsetParent.clientWidth;
}
}
nextElementOffset += offset;
nextElement = nextElement.offsetParent;
}
if (isRTL) {
nextElementOffset = -nextElementOffset;
}
var viewport = document.querySelector(".platformPanoramaViewport");
var surface = document.querySelector(".platformPanoramaSurface");
if (viewport && surface && nextElement) {
var scrollLeft = viewport.scrollLeft;
var viewportWidth = viewport.clientWidth;
var scrollMin = parseInt(viewport.currentStyle.msScrollLimitXMin);
var surfaceOffset = surface.offsetLeft;
if (nextElementOffset <= scrollLeft + viewportWidth && nextElementOffset >= scrollLeft
&& nextElementOffset + nextItem.clientWidth <= scrollLeft + viewportWidth && nextElementOffset + nextItem.clientWidth >= scrollLeft) {
nextItem.focus();
return;
}
var status = "noHero";
if (scrollMin === nextElementOffsetCounting) {
nextElementOffset = viewportWidth;
status = "toHero";
}
var animationOffset = (status === "toHero") ? nextElementOffset - (scrollLeft + viewportWidth) + scrollMin :
(nextElementOffset > scrollLeft ? (nextElementOffset + nextItem.clientWidth) - (scrollLeft + viewportWidth) :
nextElementOffset - scrollLeft );
if (isRTL) {
animationOffset = -animationOffset;
}
var animationDuration = Math.max(Math.min(0.4 * Math.abs(animationOffset) + 60, 300), 100);
var animation = WinJS.UI.executeAnimation(surface,
{
property: "-ms-transform",
delay: 0,
duration: animationDuration,
timing: "ease-out",
from: "translateX(" + (animationOffset) + "px)",
to: "none"
}).then(function () {
nextItem.focus();
});
viewport.scrollLeft = (status === "toHero") ? scrollMin : (nextElementOffset > scrollLeft ? nextElementOffset + nextItem.clientWidth - viewport.clientWidth + 20 : nextElementOffset - 20);
} else {
nextItem.focus();
}
}
})
});
})();
/* File: PanoramaHeaderOrchestrator.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function () {
"use strict";
function findNextFirstInSequence(current, direction) {
if (!current) {
return null;
}
var node = current[direction];
while (node) {
node = node[direction];
}
return node;
}
function show(item) {
if (item) {
var element = item.headerElement;
if (element) {
WinJS.Utilities.removeClass(element, "platformVisibilityNone");
}
}
}
function collapse(item) {
if (item) {
var element = item.headerElement;
if (element) {
WinJS.Utilities.addClass(element, "platformVisibilityNone");
}
}
}
function forward(candidate, zero) {
return candidate.offset && (!candidate.next || (candidate.next && candidate.next.offset > zero));
}
function backwards(candidate, zero) {
return candidate.offset <= zero;
}
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
PanoramaHeaderOrchestrator: WinJS.Class.define(function panoramaHeaderOrchestrator_ctor(panoramaControl, panoramaPanel, panoramaHeader) {
this._panorama = panoramaControl;
this._panel = panoramaPanel;
this._header = panoramaHeader;
this._initialize();
}, {
// controls
_panorama: null,
_panel: null,
_header: null,
// private variables
_lastScroll: 0,
_titleWidth: 0,
_zoomedIn: false,
_direction: true,
_nextClusterPosition: null,
_nextClippingPosition: null,
// methods
_initialize: function _initialize() {
var panorama = this._panorama;
var panel = this._panel;
panorama.addEventListener("zoomchanged", this._onSemanticZoomChanged.bind(this));
// register to scroll & currentchanged event
panel.addEventListener("scroll", this._onScroll.bind(this));
panel.addEventListener("currentchanged", this._onCurrentChanged.bind(this));
// when the surface changes, force reevaluation of the checkpoints
panel.addEventListener("itemrealized", this._invalidateCheckpoints.bind(this));
panel.addEventListener("itemremoved", this._invalidateCheckpoints.bind(this));
panel.addEventListener("itemresized", this._onItemResized.bind(this));
//TODO: register to semantic zoom events and update the header
//      color back to "null" when switching to zoomed-out view.
//      we should switch back to "current color" when zooming
//      back in.
},
dispose: function dispose() {
this._panorama = null;
this._panel = null;
if (this._header && this._header.dispose) {
this._header.dispose();
}
this._header = null;
},
_onCurrentChanged: function _onCurrentChanged(event) {
var header = this._header;
var current = event.detail;
var zoomedIn = this._zoomedIn;
if (current && !zoomedIn) {
// header.headerSubTitleText = current.headerElement.innerHTML || "";
this._invalidateCheckpoints();
this._titleWidth = header.getTitleWidth();
// header.transitionHeader(current.offset, this._nextClippingPosition);
// collapse(current);
this._updateHeader(current);
} else {
// header.headerSubTitleText = "";
// header.transitionHeader(0, Number.POSITIVE_INFINITY);
this._updateHeader(null, Number.POSITIVE_INFINITY);
this._titleWidth = null;
this._nextClusterPosition = null;
this._nextClippingPosition = null;
}
},
_onItemResized: function _onItemResized(event) {
var item = event.detail;
var panel = this._panel;
var current = panel.current;
var delta = item.deltaWidth || 0;
this._invalidateCheckpoints(event);
var direction = delta <= 0; //if item shrunk, same as scrolling to the right
var affectsHeader = item.index < current.index; //only care about items which impact header
if (direction !== this._direction && affectsHeader) {
this._direction = direction;
this._checkForCurrent(panel.scrollPosition, direction);
}
// any item before current as well as the next one have an impact on the header
if (item.index <= (current.index + 1)) {
var header = this._header;
var lastScroll = this._lastScroll;
var zero = panel.scrollPosition;
var nextClipPosition = this._nextClippingPosition;
// header.transitionHeader(zero, nextClipPosition);
//TODO: header color should use a pattern similar to
//      nextClipPosition where we compute the value
//      in advance and only invoke it when the color
//      has actually changed.
//[BEGIN]
this._updateHeader(panel.current, zero);
//[END]
}
},
_invalidateCheckpoints: function _invalidateCheckpoints() {
var panel = this._panel;
var current = panel.current;
// current has not been defined yet
if (!current) {
return;
}
var direction = this._direction;
var operator = direction ? "next" : "previous";
var nextPosition = 0;
var nextClippingPosition = 0;
var nextInSequence = null;
if (direction) {
var nextItem = current[operator];
nextPosition = nextItem ? nextItem.offset : Number.POSITIVE_INFINITY;
nextInSequence = findNextFirstInSequence(current, operator);
nextClippingPosition = nextInSequence ? nextInSequence.offset : Number.POSITIVE_INFINITY;
} else {
nextPosition = current.offset;
nextInSequence = findNextFirstInSequence(current, "next");
nextClippingPosition = nextInSequence ? nextInSequence.offset : Number.POSITIVE_INFINITY;
}
this._nextClusterPosition = nextPosition;
this._nextClippingPosition = nextClippingPosition;
},
_checkForCurrent: function _checkForCurrent(zero, direction) {
var header = this._header;
var panel = this._panel;
var candidate = null;
if (direction !== this._direction) {
this._direction = direction;
this._invalidateCheckpoints();
}
var nextPosition = this._nextClusterPosition;
var nextClipPosition = this._nextClippingPosition;
var check = direction ? function (a, b) { return a >= b; } : function (a, b) { return a < b; };
if (check(zero, nextPosition)) {
candidate = panel.current;
var dir = null;
var operation = null;
// var effect = null;
if (direction) {
dir = "next";
operation = forward;
// effect = collapse;
} else {
dir = "previous";
operation = backwards;
// effect = show;
}
while (candidate) {
if (operation(candidate, zero)) {
break;
} else {
// effect(candidate);
if (candidate[dir]) {
candidate = candidate[dir];
} else {
break;
}
}
}
}
if (candidate) {
panel.current = candidate;
} else {
// header.transitionHeader(zero, nextClipPosition);
//TODO: header color should use a pattern similar to
//      nextClipPosition where we compute the value
//      in advance and only invoke it when the color
//      has actually changed.
//[BEGIN]
this._updateHeader(panel.current, zero);
//[END]
}
},
_onScroll: function _onScroll(evt) {
var header = this._header;
var panel = this._panel;
var lastScroll = this._lastScroll;
var zero = panel.scrollPosition;
var delta = zero - lastScroll;
this._checkForCurrent(zero, delta > 0);
this._lastScroll = zero;
},
_updateHeader: function _updateHeader(current, zero) {
var header = this._header;
var color = null;
var hideTitleText = false;
var titleWidth = header.getTitleWidth();
var zoomedIn = this._zoomedIn;
var activeCluster = current;
// check if user has scrolled out of current cluster. if yes, then make next cluster active
if (this._panel.scrollPosition && current) {
if ((current.width + current.offset - titleWidth) < (this._panel.scrollPosition)) {
activeCluster = current.next;
}
}
if (activeCluster && activeCluster.data && activeCluster.data.hidePanoTitle) {
hideTitleText = true;
}
header.setTitleState(hideTitleText);
if (!zoomedIn && current && current.data) {
if (!zero) {
zero = this._panel.scrollPosition;
}
if (!zero) {
color = current.data.titleClass;
}
else if (current.data.titleClass &&
((current.offset + current.width) - zero) > titleWidth) {
color = current.data.titleClass;
}
}
if (header.fontColor !== color) {
header.fontColor = color;
}
},
_onSemanticZoomChanged: function _onSemanticZoomChanged(event) {
var panel = this._panel;
var header = this._header;
var current = panel.current;
this._zoomedIn = event.detail;
if (event.detail) {
// header.headerSubTitleText = "";
// header.transitionHeader(0, Number.POSITIVE_INFINITY);
this._updateHeader(current, Number.POSITIVE_INFINITY);
//event.detail is true when zooming out
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal,
"",
"",
"zoomout",
Platform.Telemetry.UserActionMethod.unknown, // have no way to detect input method for now.
0);
} else {
this._invalidateCheckpoints();
// header.headerSubTitleText = current.headerElement.innerHTML || "";
// header.transitionHeader(current.offset, this._nextClippingPosition);
// collapse(current);
this._updateHeader(current, panel.scrollPosition);
}
},
}),
});
})();
/* File: PanoramaMetricsObserver.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function () {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
PanoramaMetricsObserver: WinJS.Class.define(function panoramaMetricsObserver_ctor(panoramaPanel, panoramaVirtualizer) {
this._panel = panoramaPanel;
this._virtualizer = panoramaVirtualizer;
this._initialize();
}, {
// controls
_panel: null,
_virtualizer: null,
// context event references
_itemRealizedBinding: null,
_itemRealizedCompleteBinding: null,
// methods
_initialize: function _initialize() {
var panel = this._panel;
this._loadedClusters = [];
this._viewportWidth = document.body.offsetWidth;
var itemRealized = this._itemRealizedBinding = this._onItemRealized.bind(this);
var itemRealizedComplete = this._itemRealizedCompleteBinding = this._onItemRealizedComplete.bind(this);
panel.addEventListener("itemrealizing", itemRealized);
panel.addEventListener("itemrealizedcomplete", itemRealizedComplete);
},
dispose: function dispose() {
var panel = this._panel;
if (panel) {
panel.removeEventListener("itemrealizing", this._itemRealizedBinding);
panel.removeEventListener("itemrealizedcomplete", this._itemRealizedCompleteBinding);
this._panel = null;
}
},
_onItemRealizedComplete: function _onItemRealizedComplete(event) {
var item = event.detail;
var total = this._virtualizer._totalCount();
var clusters = this._loadedClusters;
clusters[item.index] = true; //flag the cluster as loaded
var lastLoaded = this._lastLoaded || 0; //never count the same item in the array more that once
for (var i = lastLoaded, l = clusters.length; i < l; i++) {
if (!clusters[i]) {
break;
}
else {
//store the last loaded cluster
lastLoaded = i;
}
}
var lastItem = item;
if (clusters[lastLoaded] && typeof clusters[lastLoaded].measured === "undefined") {
//find the item by walking the linked list in the right direction
while (lastItem !== null && lastItem.index > lastLoaded) {
lastItem = lastItem.previous;
}
while (lastItem !== null && lastItem.index < lastLoaded) {
lastItem = lastItem.next;
}
if (!lastItem || !lastItem.element) {
//something bad happened, like an index rendered that was removed from the linked list
return;
}
clusters[lastLoaded] = { measured: true };
var screenWidth = this._viewportWidth;
var elemOffset = 0;
if ((lastLoaded === (total - 1)) /*too few items to fill screen*/
|| ((lastItem && lastItem.element) && (elemOffset = lastItem.element.getOffset() > screenWidth)) /*left edge is too far*/
|| ((lastItem && lastItem.element) && (elemOffset + lastItem.element.offsetWidth > screenWidth)) /*right edge too far*/) {
this._writeClusterLoadMark();
}
}
},
_writeClusterLoadMark: function _writeClusterLoadMark() {
Platform.Process.instance.perfTrackWriteStopEventWithMetadata(Platform.PerfTrackScenarioId.panoramaVisibleClustersLoaded, "Panorama Control Visible Clusters Loaded", this._panel._element.id, true);
// Fix for bug 527426 (change list # 155565):
// set the flag to true since the panorama control has loaded
CommonJS.Progress.canDestroySplash(true);
CommonJS.Progress.destroySplash(true);
// End of fix for bug 527426 (change list # 155565):
msWriteProfilerMark("Performance:Panorama:VisibleClustersLoaded:e");
this.dispose(); //don't listen to any more events
},
_onItemRealized: function _onItemRealized(event) {
var item = event.detail;
this._loadedClusters[item.index] = false;
},
})
});
})();
/* File: PanoramaPanel.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function () {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
PanoramaPanel: WinJS.Class.mix(WinJS.Class.define(function panoramaPanel_ctor(element, options) {
options = options || {};
this._element = element;
CommonJS.setAutomationId(element);
// this aria-label creates an unnecessary heirachry in ui inspect, but test team depend on it for now
// please check with MonaG before removing
element.setAttribute("aria-label", element.id);
element.winControl = this;
// create events
Object.defineProperties(this, WinJS.Utilities.createEventProperties("scroll"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemrealized"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemrealizing"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemrealizedcomplete"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemsrealizedcomplete"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemresized"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("itemremoved"));
Object.defineProperties(this, WinJS.Utilities.createEventProperties("currentchanged"));
this._initialize(options);
}, {
// private variables
_scrollTimer: null,
_isScrolling: false,
_verticalScrollDetectTimer: null,
_current: null,
_lastScrollTimeStamp: null,
_scrollPosition: 0,
_showInertiaBackground: false,
_compactPanorama: false,
_zeroOffsetSnapPoints: false,
// DOM elements
_element: null,
_surface: null,
_viewport: null,
_halo: null,
_imageBleed: null,
// properties
current: {
get: function get() { return this._current; },
set: function set(value) {
if (value !== this._current) {
this._current = value;
this.dispatchEvent("currentchanged", value);
}
}
},
scrollPosition: {
get: function get() { return this._scrollPosition; }
},
showInertiaBackground: {
get: function get() { return this._showInertiaBackground; },
set: function set(value) {
this._showInertiaBackground = value;
//if (!value) {
//    WinJS.Utilities.addClass(this._imageBleed, "platformHide");
//} else {
//    WinJS.Utilities.removeClass(this._imageBleed, "platformHide");
//}
},
},
compactPanorama: {
get: function get() { return this._compactPanorama; },
set: function set(value) {
this._compactPanorama = value;
},
},
zeroOffsetSnapPoints: {
get: function get() { return this._zeroOffsetSnapPoints; },
set: function set(value) {
this._zeroOffsetSnapPoints = value;
},
},
// methods
_initialize: function _initialize(options) {
var element = this._element;
WinJS.Utilities.addClass(element, "platformPanorama");
this._setThemeColor();
var viewport = this._viewport = document.createElement("div");
viewport.className = "platformPanoramaViewport";
var appTitle = PlatformJS.Services.resourceLoader.getString("/platform/AppTitle");
viewport.setAttribute("aria-label", appTitle);
viewport.setAttribute("aria-hidden", "false"); // Enables Narrator multi-touch scrolling
viewport.setAttribute("id", appTitle);
element.appendChild(viewport);
viewport.addEventListener("scroll", this._onSurfaceScroll.bind(this));
viewport.addEventListener("wheel", this._onMouseWheel.bind(this));
window.addEventListener("mousewheel", this._onWindowScroll, false);
var header = this._header = document.querySelector(".immersiveHeader");
if (header) {
header.parentNode.removeChild(header);
viewport.appendChild(header);
}
var surface = this._surface = document.createElement("div");
surface.className = "platformPanoramaSurface platformHideableContent";
viewport.appendChild(surface);
var imageBleed = this._imageBleed = document.createElement("div");
imageBleed.className = "platformImageBleed";
var halo = this._halo = document.createElement("div");
halo.className = "platformPanoramaHalo";
},
_setThemeColor: function _setThemeColor() {
var styleSheets = document.styleSheets;
var themeColor = null;
for (var i = 0, l = styleSheets.length; i < l; i++) {
var styleSheet = styleSheets[i];
if (styleSheet.href && styleSheet.href.match("ui-light.css") && !styleSheet.disabled) {
themeColor = "platformPanoramaLight";
} else if (styleSheet.href && styleSheet.href.match("ui-dark.css") && !styleSheet.disabled) {
themeColor = "platformPanoramaDark";
}
}
if (themeColor) {
WinJS.Utilities.addClass(this._element, themeColor);
}
},
dispose: function dispose() {
if (this._scrollTimer) {
clearTimeout(this._scrollTimer);
}
var viewport = this._viewport;
viewport.onscroll = null;
viewport.onwheel = null;
viewport.onkeydown = null;
viewport.MSManipulationStateChanged = null;
this._viewport = null;
this._element = null;
this._surface = null;
this._viewport = null;
this._halo = null;
this._imageBleed = null;
},
realizeItems: function realizeItems(items, scrollPosition) {
var that = this,
promises = [];
this._createElements(items, scrollPosition);
var queue = this._rankRealization(items, scrollPosition);
for (var i = 0, ilen = queue.length; i < ilen; i++) {
var item = queue[i];
var renderPromise = this._realizeItem(item);
(function panoramaPanel_renderPromiseDispatchItemRealizedEvent(item) {
renderPromise.then(function panoramaPanel_renderRealizedItemComplete() {
that.dispatchEvent("itemrealizedcomplete", item);
});
})(item);
promises.push(renderPromise);
}
// scroll to target location
if (scrollPosition) {
this.scrollTo(scrollPosition);
}
return WinJS.Promise.join(promises)
.then(function panoramaPanel_renderPromiseJoinComplete() {
// To avoid image bleed displaying too early and overlapping hero cluster, 
// we wait and show it after all elements are realized
// TODO: investigate a pure CSS solution
that.dispatchEvent("itemsrealizedcomplete", promises.length);
});
},
realizeItem: function realizeItem(item) {
this._createElements([item]);
return this._realizeItem(item);
},
removeItem: function removeItem(item) {
if (item.element) {
item.element.onresize = null;
if (this._surface) {
this._surface.removeChild(item.element);
}
}
item.element = null;
item.contentElement = null;
item.headerElement = null;
this.dispatchEvent("itemremoved", item);
if (item === this.current) {
if (item.next) {
this.current = item.next;
} else if (item.previous) {
this.current = item.previous;
} else {
this.current = null;
}
}
},
_createElements: function _createElements(items) {
var surface = this._surface;
for (var i = 0, ilen = items.length; i < ilen; i++) {
var item = items[i];
if (!item.element) {
var element = this._createElement(item);
if (item.index > (surface.children.length - 2)) {
surface.appendChild(element);
} else {
var sibling = item.next.element;
surface.insertBefore(element, sibling);
}
this.dispatchEvent("itemrealized", item);
}
}
},
_createElement: function _createElement(item) {
var that = this;
var panoramaItem = item.data;
var element = item.element = document.createElement("div");
element.setAttribute("role", "region");
element.id = panoramaItem.clusterContent.contentControl + "_" + item.key;
element.className = "platformCluster";
CommonJS.setAutomationId(element, "platformCluster" + item.key);
if (item.width) {
// we have cached width from last render
element.style.width = item.width + "px";
}
// cluster header
var headerElement = item.headerElement = document.createElement("h2");
headerElement.setAttribute("role", "heading");
if (panoramaItem.titleClass) {
WinJS.Utilities.addClass(headerElement, panoramaItem.titleClass);
}
if (!panoramaItem.clusterKey) {
PlatformJS.Utilities.onError("Cluster Key cannot be null");
return element;
}
CommonJS.setAutomationId(headerElement, element, "", panoramaItem.clusterKey.replace(/ /g, "_"));
element.setAttribute("aria-labelledby", headerElement.id);
WinJS.Utilities.addClass(headerElement, "platformClusterHeader");
if (panoramaItem.hideTitle) {
WinJS.Utilities.addClass(headerElement, "platformClusterNoHeader");
}
//TODO: register onHeaderSelection handler
if (panoramaItem.onHeaderSelection) {
var headerElementLink = document.createElement("div");
//ideally we should use "a" instead of "div", but we would need to create new styles/lessvars
headerElementLink.tabIndex = 0;
headerElementLink.innerText = panoramaItem.clusterTitle || "";
headerElementLink.setAttribute("role", "link");
PlatformJS.Utilities.enablePointerUpDownAnimations(headerElementLink);
headerElement.appendChild(headerElementLink);
var moreIndicator = document.createElement("span");
moreIndicator.className = "platformClusterHeaderMoreIcon";
headerElementLink.appendChild(moreIndicator);
var headerElementLinkClick = function (event) {
panoramaItem.onHeaderSelection(panoramaItem.clusterKey, item.index, that);
PlatformJS.Services.instrumentation.incrementInt32(
Platform.Instrumentation.InstrumentationDataSetId.platform,
Platform.Instrumentation.InstrumentationDataPointId.totalClusterHeaderTappedCount,
1);
};
var headerElementNav = function (event, direction) {
event.stopImmediatePropagation();
event.preventDefault();
var e = document.createEvent("CustomEvent");
e.initCustomEvent("headerbounds", false, false, { item: item, direction: direction });
headerElement.dispatchEvent(e);
};
var headerElementKeyDown = function (event) {
switch (event.keyCode) {
case (WinJS.Utilities.Key.enter): headerElementLinkClick(event); break;
case (WinJS.Utilities.Key.space): headerElementLinkClick(event); break;
case (WinJS.Utilities.Key.upArrow): headerElementNav(event, "up"); break;
case (WinJS.Utilities.Key.downArrow): headerElementNav(event, "down"); break;
case (WinJS.Utilities.Key.leftArrow): headerElementNav(event, "left"); break;
case (WinJS.Utilities.Key.rightArrow): headerElementNav(event, "right"); break;
default: break;
}
};
headerElementLink.addEventListener("click", headerElementLinkClick, false);
headerElementLink.addEventListener("keydown", headerElementKeyDown, false);
}
else {
if (panoramaItem.supressTitleInHeader) {
headerElement.innerText = "";
} else {
headerElement.innerText = panoramaItem.clusterTitle || "";
WinJS.Utilities.addClass(headerElement, "platformClusterHeaderNoLink");
}
}
element.appendChild(headerElement);
// cluster content
var contentElement = item.contentElement = document.createElement("div");
contentElement.id = panoramaItem.clusterContent.contentControl + "_" + (item.key.replace(/ /g, "_")) + ".Content";
contentElement.className = "platformClusterContent";
// support for full-bleed
if (panoramaItem.itemMode === NS.Panorama.PanoramaItemMode.fullBleed) {
WinJS.Utilities.addClass(contentElement, "platformHeroCluster");
}
element.appendChild(contentElement);
if (PlatformJS.isDebug) {
// performance indicator
var indicatorElement = document.createElement("div");
indicatorElement.className = "platformClusterPerformanceIndicator";
element.appendChild(indicatorElement);
}
return element;
},
_rankRealization: function _rankRealization(items, offset) {
if (!offset) {
return items;
}
// the item realization path needs to be re-ordered
// based on the offset.  when we have an offset, we
// we try and render items that would appear on
// screen first.
// sample:
//   - # of items: 6
//   - offset: 3
//   - screen size: 2
// then [1,2,3,4,5,6] => [3,4,2,1,5,6]
var output = [];
var firstIndex = 0;
var i = 0, ilen = 0;
for (i = 0, ilen = items.length; i < ilen; i++) {
var item = items[i];
if (item.offset <= offset && (!item.next || (item.next && item.next.offset > offset))) {
firstIndex = i;
break;
}
}
for (i = firstIndex, ilen = items.length; i < ilen; i++) {
output.push(items[i]);
}
for (i = (firstIndex - 1), ilen = -1; i > ilen; i--) {
output.push(items[i]);
}
return output;
},
_realizeItem: function _realizeItem(item) {
var that = this,
element = item.element,
panoramaItem = item.data,
itemDefinition = panoramaItem.clusterContent,
timestamp = null,
marker = performance.now();
// update current
if (!this.current) {
this.current = item;
}
// register and proxy the resize event
element.onresize = function (evt) {
console.log("onresize");
that.dispatchEvent("itemresized", item);
};
//HACK: here, we grab the content div from the item's element
//      by indexing it's children array.  we are seeing this
//      pattern in many location, which could validate keeping
//      a reference to the div element.
//[BEGIN]
var contentElement = item.contentElement,
headerElement = item.headerElement;
//[END]
// realize content
var renderPromise = null;
if (itemDefinition.contentControl) {
if (!panoramaItem.clusterKey) {
PlatformJS.Utilities.onError("Cluster Key cannot be null");
} else {
element.id = itemDefinition.contentControl + "_" + (panoramaItem.clusterKey.replace(/ /g, "_"));
var orchestrator = this._createOrchestrator(item);
var winControl = item.winControl = PlatformJS.Utilities.createObject(
itemDefinition.contentControl,
contentElement,
itemDefinition.contentOptions,
orchestrator);
if (!winControl) {
if (PlatformJS.isDebug) {
throw "Invalid cluster type metadata";
} else {
winControl = {};
}
}
if (winControl.render) {
renderPromise = WinJS.Promise
.timeout()
.then(function () {
msWriteProfilerMark("Platform:Panorama:realizeItem:" + itemDefinition.contentControl + ":" + marker + ":s");
that.dispatchEvent("itemrealizing", item);
timestamp = new Date().getTime();
return WinJS.Promise.wrap(winControl.render());
});
} else {
timestamp = new Date().getTime();
renderPromise = WinJS.Promise.wrap(null);
}
}
} else {
PlatformJS.Utilities.onError("Require content control property");
}
contentElement.addEventListener("contentbounds", function panoramaPanel_onContentBounds(event) {
if (event.detail.direction) {
that.dispatchEvent("itembounds", { item: item, direction: event.detail.direction, content: true });
}
});
headerElement.addEventListener("headerbounds", function panoramaPanel_onHeaderBounds(event) {
if (event.detail.direction) {
that.dispatchEvent("itembounds", { item: item, direction: event.detail.direction, header: true });
}
});
//if (winControl && !winControl.preventAnimation) {
//    contentElement.style.opacity = 0;
//}
return renderPromise
.then(function panoramaPanel_renderRealizeItemComplete() {
//if (winControl && !winControl.preventAnimation) {
//    WinJS.UI.Animation.enterContent(contentElement);
//}
if (PlatformJS.isDebug && element.children.length > 2) {
var delta = new Date().getTime() - timestamp;
element.children[2].innerText = delta + "ms";
}
msWriteProfilerMark("Platform:Panorama:realizeItem:" + itemDefinition.contentControl + ":" + marker + ":e");
element.style.width = "auto";
});
},
scrollToWithAnimation: function scrollToWithAnimation(offset, item, panoramaFocusManager) {
if (!this._isScrolling && this._viewport) {
var scrollMin = parseInt(this._viewport.currentStyle.msScrollLimitXMin);
if (offset < scrollMin) {
offset = scrollMin;
}
else if (offset > this._viewport.scrollWidth) {
offset = this._viewport.scrollWidth;
}
this._scrollPosition = offset;
var scrollLeft = this._viewport.scrollLeft;
var animationOffset = offset - scrollLeft;
var animationDuration = Math.max(Math.min(0.4 * Math.abs(animationOffset) + 60, 300), 100);
var animation = WinJS.UI.executeAnimation(this._surface,
{
property: "-ms-transform",
delay: 0,
duration: animationDuration,
timing: "ease",
from: "translateX(" + (animationOffset) + "px)",
to: "none"
}).then(function () {
panoramaFocusManager.setFocusToContent(item, true);
});
this._viewport.scrollLeft = offset;
}
},
scrollTo: function scrollTo(offset) {
if (!this._isScrolling && this._viewport) {
if (offset < 0) {
offset = 0;
}
else if (offset > this._viewport.scrollWidth) {
offset = this._viewport.scrollWidth;
}
this._scrollPosition = offset;
this._viewport.scrollLeft = offset;
}
},
detach: function detach() {
var surface = this._surface;
for (var i = 0, ilen = surface.children.length; i < ilen; i++) {
var child = surface.children[i];
child.onresize = null;
}
},
clear: function clear() {
var surface = this._surface;
for (var i = 0, ilen = surface.children.length; i < ilen; i++) {
var child = surface.children[i];
child.onresize = null;
}
surface.innerHTML = "";
surface.appendChild(this._imageBleed);
this.current = null;
},
_startScrolling: function _startScrolling() {
var that = this;
this._isScrolling = true;
//TODO: there is a glitch with the app-bars eating all
//      touch input.  for now, disabling the edgy
//      dismissal when scrolling
//[BEGIN]
//CommonJS.dismissAllEdgies();
//[END]
if (this._scrollTimer) {
clearTimeout(this._scrollTimer);
}
this._scrollTimer = setTimeout(function panoramaPanel_scrollTimer() {
that._isScrolling = false;
that.dispatchEvent("scroll");
}, 250);
},
_onSurfaceScroll: function _onSurfaceScroll(evt) {
// V.1. bug 202530: we can get here with _viewport being null,
// in that case make scroll a no-op; note this is the same
// protection we have in scrollTo(), which is called during
// _onMouseWheel()
if (this._viewport) {
this._startScrolling();
//var lastScrollTimeStamp = this._lastScrollTimeStamp || 0;
//var timeStamp = evt.timeStamp;
//var rate = 33; // 30fps = (1 000 / 30)
this._scrollPosition = this._viewport.scrollLeft;
// throttle the onscroll event at 30fps
//if ((timeStamp - lastScrollTimeStamp) > rate) {
//    this._lastScrollTimeStamp = timeStamp;
this.dispatchEvent("scroll", evt);
//}
}
},
_onWindowScroll: function _onWindowScroll(evt) {
if (!evt.ctrlKey && evt.deltaY !== 0) {
var panoramaPanel = CommonJS.Immersive.PanoramaPanel;
panoramaPanel.isScrollCurrentTargetPanorama = true;
if (this._verticalScrollDetectTimer) {
clearTimeout(this._verticalScrollDetectTimer);
}
var that = this;
this._verticalScrollDetectTimer = setTimeout(function verticalScrollDetectTimer() {
panoramaPanel.isScrollCurrentTargetPanorama = false;
that._verticalScrollDetectTimer = null;
}, panoramaPanel.verticalScrollDelay);
}
},
_onMouseWheel: function _onMouseWheel(evt) {
if (!evt.ctrlKey && evt.deltaY !== 0 && evt.target === this._viewport) {
this.scrollTo(this._scrollPosition + evt.deltaY);
}
}
},
{
leftMargin: 120,
minClusterWidth: 200,
clusterSpacing: 80,
isScrollCurrentTargetPanorama: true,
verticalScrollDelay: 800,
verticalScrollHandler: function verticalScrollHandler(e) {
if (e) {
var panoramaPanel = CommonJS.Immersive.PanoramaPanel;
if (!panoramaPanel.isScrollCurrentTargetPanorama) {
e.stopPropagation();
} else {
e.preventDefault();
var targetElement = e.currentTarget;
if (targetElement) {
var handler = panoramaPanel.verticalScrollHandler;
if (!targetElement.onscroll) {
targetElement.onscroll = handler;
}
targetElement.style.overflowY = "hidden";
targetElement.removeEventListener("mousewheel", handler, false);
if (targetElement.verticalScrollTimer) {
clearTimeout(targetElement.verticalScrollTimer);
}
var that = this;
targetElement.verticalScrollTimer = setTimeout(function verticalScrollDelayTimer() {
targetElement.style.overflowY = "auto";
targetElement.addEventListener("mousewheel", handler, false);
targetElement.verticalScrollTimer = null;
}, panoramaPanel.verticalScrollDelay);
}
}
}
},
}), WinJS.Utilities.eventMixin)
});
})();
/* File: PanoramaVirtualizer.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function () {
"use strict";
var realizationBufferSize = 1;
function isRealizationRequired(virtualizer) {
var panel = virtualizer._panel,
totalCount = virtualizer._totalCount(),
realized = virtualizer._realized,
lastScrollPosition = virtualizer._lastScrollPosition,
list = virtualizer._dataSource && virtualizer._dataSource._list;
if (!panel) {
return false;
}
if (totalCount === 0) {
return false;
}
if (list) {
if (!list.length) {
return false;
}
var last = list.getAt(list.length - 1);
if (realized.fromKey(last.clusterKey)) {
return false;
}
}
var viewportWidth = document.body.offsetWidth,
surfaceWidth = panel._surface.offsetWidth,
scrollPosition = lastScrollPosition;
return surfaceWidth <= scrollPosition + (viewportWidth * (1 + realizationBufferSize));
}
function retainAndDecorateRange(indexes, realized, dataSource) {
var that = this,
promises = [],
pending = [];
for (var i = 0, ilen = indexes.length; i < ilen; i++) {
var itemPromise = dataSource
.fromIndex(indexes[i])
.retain()
.then(function panoramaVirtualizer_retainComplete(listItem) {
if (!listItem) {
return null;
}
var decorated = decorateClusterDefinition(
listItem.index,
listItem.handle,
listItem.data,
"clusterTitle");
pending.push(decorated);
});
promises.push(itemPromise);
}
return WinJS.Promise.join(promises)
.then(function panoramaVirtualizer_joinComplete() {
return WinJS.Promise.wrap(pending);
});
}
function decorateClusterDefinition(index, handle, data) {
return {
index: index,
key: data.clusterKey,
data: data,
handle: handle,
width: null,
offset: null,
// links
previous: null,
next: null,
// references
element: null,
winControl: null,
};
}
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
PanoramaVirtualizer: WinJS.Class.define(function panoramaVirtualizer_ctor(panoramaPanel, focusManager, panoramaEventManager) {
// initialize blank promise
this._operation = WinJS.Promise.wrap({});
this.numClustersToLoad = PlatformJS.Services.configuration.getDictionary("Panorama").getInt32("NumClustersToLoad");
this._panel = panoramaPanel;
this._focusManager = focusManager;
this._panoramaEventManager = panoramaEventManager;
this._initialize();
}, {
// controls
_panel: null,
_focusManager: null,
_panoramaEventManager: null,
// private variables
_realized: null,
_dataSource: null,
_lastScrollPosition: 0,
_isRealizing: false,
_panoramaState: null,
_currentPromise: null,
_scrollLimitMin: 0,
_direction: null,
_nextCheckPoint: null,
_operation: null,
// public variables
numClustersToLoad: 0,
// properties
dataSource: {
set: function set(value) {
var that = this;
if (this._dataSource !== null) {
this._dataSource.release();
}
if (this._operation) {
this._operation.cancel();
this._operation = WinJS.Promise.wrap({});
}
this._realized = new NS.RealizedList();
this._isRealizing = false;
this._direction = null;
this._nextCheckPoint = null;
this._lastScrollPosition = 0;
if (value) {
var list = value._list;
Platform.Process.perfTrackWriteStartEvent(Platform.PerfTrackScenarioId.panoramaVisibleClustersLoaded, "Panorama Control Visible Clusters Loaded", this._panel._element.id);
this._dataSource = value.createListBinding(this);
this._realizeFirstView();
}
}
},
panoramaState: {
set: function set(value) {
this._panoramaState = value;
},
},
// methods
_initialize: function _initialize() {
this._panelScrollBinding = this._onSmartScroll.bind(this);
this._panel.addEventListener("scroll", this._panelScrollBinding);
},
dispose: function dispose() {
// release references
if (this._dataSource) {
this._dataSource.release();
}
if (this._operation) {
this._operation.cancel();
}
if (this._panel && this._panelScrollBinding) {
this._panel.removeEventListener("scroll", this._panelScrollBinding);
}
this._panelScrollBinding = null;
this._panel = null;
this._focusManager = null;
},
_isRunning: false,
_totalCount: function _totalCount() {
var result = 0;
if (this._dataSource && this._dataSource._list) {
result = this._dataSource._list.length || 0;
}
return result;
},
_realizeFirstView: function _realizeFirstView() {
var that = this,
realized = this._realized,
state = this._panoramaState || {},
count = (state.count && state.scrollPosition > 0) ? state.count : 1,
index = 0,
last = Math.min(this._totalCount(), count),
indexes = [],
scrollPosition = state.scrollPosition,
dataSource = this._dataSource;
that._computeRealizationCheckpoint();
this._scrollLimitMin = parseInt(this._panel._viewport.currentStyle.msScrollLimitXMin);
for (; index < last; index++) {
indexes.push(index);
}
var operation = null;
// no cluster metadata to render initial view
if (indexes.length > 0) {
this._isRealizing = true;
this._operation = operation = this._operation
.then(function panoramaVirtualizer_realizeFirstViewOperationComplete() {
return retainAndDecorateRange(indexes, realized, dataSource);
})
.then(function panoramaVirtualizer_realizeFirstViewRetainAndDecorateRangeComplete(panoramaItems) {
that._spliceStateMetadata(panoramaItems);
realized.addRange(panoramaItems);
return that._panel.realizeItems(panoramaItems, scrollPosition);
})
.then(function panoramaVirtualizer_realizeFirstViewRealizeItemsComplete() {
if (isRealizationRequired(that)) {
return that._realizeNextItems(that.numClustersToLoad);
} else {
that._isRealizing = false;
return WinJS.Promise.wrap(null);
}
})
.then(function () {
that._afterRealizeFirstView();
});
}
else {
that._afterRealizeFirstView();
}
return operation;
},
_afterRealizeFirstView: function _afterRealizeFirstView() {
var manager = PlatformJS.mainProcessManager;
if (manager.afterFirstView) {
manager.afterFirstView();
}
if (this._panel && this._panoramaEventManager) {
this._panoramaEventManager.dispatchEvent("afterFirstView");
}
},
_spliceStateMetadata: function _spliceStateMetadata(pendingItems) {
var state = this._panoramaState || {};
for (var i = 0, ilen = pendingItems.length; i < ilen; i++) {
var item = pendingItems[i];
var stateEntry = state[item.key];
if (stateEntry) {
item.offset = stateEntry.offset;
item.width = stateEntry.width;
var options = item.data.clusterContent.contentOptions = (item.data.clusterContent.contentOptions || {});
options.state = stateEntry.state;
}
}
},
setCurrentIndex: function setCurrentIndex(value, ignoreAnimation) {
var that = this;
this._operation = this._operation
.then(function panoramaVirtualizer_setCurrentIndexOperationComplete() {
var realized = that._realized;
var lastIndex = that._getLastRealizedIndex();
if (value <= lastIndex) {
var list = that._dataSource && that._dataSource._list;
var listItem = list && list.getAt(value);
if (listItem) {
var item = realized.fromKey(listItem.clusterKey);
that._computeRealizationCheckpoint();
if (item) {
that._panel.scrollTo(item.offset);
that._focusManager.setFocusTo(item, ignoreAnimation);
}
}
return WinJS.Promise.wrap(null);
} else {
var index = lastIndex + 1;
var length = value + 1;
var indexes = [];
for (; index < length; index++) {
indexes.push(index);
}
that._isRealizing = true;
var dataSource = that._dataSource;
realized = that._realized;
return retainAndDecorateRange(indexes, realized, dataSource)
.then(function panoramaVirtualizer_retainAndDecorateRangeComplete(panoramaItems) {
realized.addRange(panoramaItems);
return that._panel.realizeItems(panoramaItems);
})
.then(function panoramaVirtualizer_realizeItemsComplete() {
if (isRealizationRequired(that)) {
return that._realizeNextItems(1);
} else {
that._isRealizing = false;
that.setCurrentIndex(value);
return;
}
});
}
});
},
cancelPendingOperations: function cancelPendingOperations() {
if (this._operation && this._operation.cancel) {
this._operation.cancel();
this._operation = WinJS.Promise.wrap(null);
}
},
refresh: function refresh() {
this._realized = new NS.RealizedList();
this._realizeFirstView();
},
_bufferSize: 1,
_onSmartScroll: function _onSmartScroll(evt) {
var that = this,
scrollPosition = this._panel.scrollPosition,
delta = scrollPosition - this._lastScrollPosition,
last = this._realized.last;
this._lastScrollPosition = scrollPosition;
if (this._isRealizing) {
return;
}
if (!last || (this._getLastRealizedIndex() >= this._totalCount() - 1)) {
return;
}
// Keyboard users might not scroll through all item elements,
// so we need to set realization point at the start of last item.
var nextRealizationPoint = this._nextCheckPoint - last.width - this._scrollLimitMin;
// we only realize when scrolling to the left
if (delta < 0) {
return;
}
if (scrollPosition >= nextRealizationPoint) {
this._isRealizing = true;
this._computeRealizationCheckpoint();
this._operation = this._operation
.then(function () {
return that._realizeNextItems(1);
});
}
},
_realizeNextItems: function _realizeNextItems(count) {
var that = this;
var realized = this._realized,
dataSource = this._dataSource,
index = that._getLastRealizedIndex() + 1;
var itemCount = Math.min((index + count), that._totalCount());
var indexes = [];
for (; index < itemCount; index++) {
indexes.push(index);
}
return retainAndDecorateRange(indexes, realized, dataSource)
.then(function panoramaVirtualizer_retainAndDecorateRangeComplete(panoramaItems) {
var realized = that._realized;
realized.addRange(panoramaItems);
return that._panel.realizeItems(panoramaItems);
})
.then(function panoramaVirtualizer_realizeItemsComplete() {
if (isRealizationRequired(that)) {
return that._realizeNextItems(count);
} else {
that._isRealizing = false;
return WinJS.Promise.wrap(null);
}
});
},
_getLastRealizedIndex: function _getLastRealizedIndex() {
var result = -1;
var last = this._realized && this._realized.last;
var list = this._dataSource && this._dataSource._list;
while (last && list) {
result = list.indexOfKey(last.handle);
if (result < 0) {
last = last.previous;
} else {
break;
}
}
return result;
},
_computeRealizationCheckpoint: function _computeRealizationCheckpoint() {
if (this._panel && this._panel._viewport) {
var viewportWidth = this._panel._viewport.offsetWidth;
var scrollPosition = this._lastScrollPosition;
this._nextCheckPoint = (scrollPosition + viewportWidth);
}
},
_realizationRequired: function _realizationRequired(listItem) {
var buffer = 2;
//TODO: we should use the viewport's width, but in some scenarios
//      it gets stuck to 0 pixel while we're measuring the children
//[BEGIN]
var screenWidth = document.body.offsetWidth; //this._panel._viewport.offsetWidth;
//[END]
var realized = this._realized;
var last = realized.last;
var scrollPosition = this._lastScrollPosition;
// already realized
if (realized && listItem && realized.fromHandle(listItem.handle)) {
return false;
}
// no items, realize
if (!last) {
return true;
}
// new index < last realized index, realize
if (listItem && (listItem.index <= this._getLastRealizedIndex())) {
return true;
}
// last realized item doesn't fill screen
if (last && ((last.offset + last.width) - scrollPosition) < (screenWidth * (1 + buffer))) {
return true;
}
return false;
},
getState: function getState() {
var realized = this._realized;
if (!realized) {
return null;
}
var first = realized.first;
if (!first) {
return null;
}
var state = {
scrollPosition: this._panel.scrollPosition,
count: realized.length,
};
var item = first;
while (item) {
var itemState = {
index: item.index,
offset: item.offset,
width: item.width
};
// if cluster supports state
if (item.winControl && item.winControl.getState) {
itemState.state = item.winControl.getState();
}
state[item.key] = itemState;
item = item.next;
}
return state;
},
_retainAndDecorateItem: function _retainAndDecorateItem(listItem) {
var that = this;
return listItem
.retain()
.then(function panoramaVirtualizer_retainComplete(listItem) {
var decorated = decorateClusterDefinition(
listItem.index,
listItem.handle,
listItem.data,
"clusterTitle");
return WinJS.Promise.wrap(decorated);
});
},
_realizeItem: function _realizeItem(panoramaItem) {
var that = this;
var realized = this._realized;
var panel = this._panel;
realized.insert(panoramaItem, panoramaItem.index);
return panel.realizeItem(panoramaItem)
.then(function panoramaVirtualizer_realizeItemComplete() {
that._afterRealizeFirstView();
return WinJS.Promise.wrap(null);
});
},
_isRealizationRequired: function _isRealizationRequired() {
return isRealizationRequired(this);
},
// IList implementation begins
beginNotifications: function beginNotifications() { },
endNotifications: function endNotifications() { },
changed: function changed() { },
itemAvailable: function itemAvailable() { },
moved: function moved() { },
countChanged: function countChanged(newCount, oldCount) { },
indexChanged: function indexChanged(handle, newIndex, oldIndex) { },
inserted: function inserted(listItem, previousHandle, nextHandle) {
var that = this;
listItem.retain();
this._operation = this._operation.then(function handleItemInserted() {
that._computeRealizationCheckpoint();
// realization is not required
if (!that._realizationRequired(listItem)) {
return WinJS.Promise.wrap(null);
} else {
return WinJS.Promise.wrap(null)
.then(function panoramaVirtualizer_insertedOperationComplete() {
return that._retainAndDecorateItem(listItem);
})
.then(function panoramaVirtualizer_retainAndDecoratedItemComplete(decorated) {
return that._realizeItem(decorated);
})
.then(function () {
that._computeRealizationCheckpoint();
});
}
});
},
removed: function removed(handle, mirage) {
var that = this;
this._operation = this._operation.then(function handleItemRemoved() {
var realized = that._realized;
var item = realized.fromHandle(handle);
if (item) {
var next = item.next;
realized.removeAt(item.index);
item.next = next;
that._panel.removeItem(item);
that._computeRealizationCheckpoint();
}
});
}
// IList implementation ends
})
});
})();
/* File: SemanticZoomPanorama.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function () {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
SemanticZoomPanorama: WinJS.Class.derive(CommonJS.Immersive.Panorama, function semanticZoomPanorama_ctor(element, options) {
CommonJS.Immersive.Panorama.call(this, element, options);
this._semanticZoomInitialized = false;
this._customizeViewInitialized = false;
this._currentClusters = [];
Object.defineProperties(this, WinJS.Utilities.createEventProperties("zoomchanged"));
}, {
// elements
_semanticZoomElement: null,
_semanticZoomOutElement: null,
_zoomedOutItem: null,
_semanticZoomInitialized: null,
_customizeViewInitialized: false,
_semanticZoomedOutProjection: null,
_customizationList: null,
_customizationCompletedCallback: null,
_customizationAdd: null,
_customizationItemTemplate: null,
_queryClusterData: null,
_currentClusters: null,
_list: null,
_removedCustomizationItems: null,
_viewportInVisibility: null,
_viewportOutVisibility: null,
_customizationAddButtonRenderer: null,
_customizationAddButton: null,
_disposed: false,
_customizationMode: null,
_maxCustomizationClusterCount: -1,
_customizationTitle: null,
_customizationUseVariableSizedTemplates: false,
_actionTray: null,
_enableCustomization: false,
_showCustomizationImmediate: false,
_customizationImageMap: null,
_initializeCustomizationViewInProgress: false,
_onsemanticZoomChangedBinding: null,
_headerParent: null,
_customizationInvisibleList: null,
_customizationInvisibleListMap: null,
_customizationResult: null,
_lastViewState: null,
_restoreCustomizationOnLayoutChange: false,
_initialize: function _initialize(options) {
var element = this._element;
this._disposed = false;
this._lastViewState = Windows.UI.ViewManagement.ApplicationView.value;
// base implementation begins
// header
this._headerElement = CommonJS.createElement("header", element, "header");
var header = this.header = new CommonJS.Immersive.Header(this._headerElement);
header.onSubTitleClick = this._onHeaderSubtitleClick.bind(this);
CommonJS.setAutomationId(this._headerElement, element, "header");
element.appendChild(this._headerElement);
// panel
var panelElement = CommonJS.createElement("div", element, "panel");
var panel = this._panel = new NS.SemanticZoomPanoramaPanel(panelElement, null);
panel._createOrchestrator = this._createOrchestrator.bind(this);
CommonJS.setAutomationId(panelElement, element, "panel");
this._onItemRemovedBinding = this._onItemRemoved.bind(this);
panel.addEventListener("itemremoved", this._onItemRemovedBinding);
// focus manager
var focusManager = this._focusManager = new NS.PanoramaFocusManager(panel);
// panel virtualizer
var virtualizer = this._virtualizer = new NS.SemanticZoomPanoramaVirtualizer(panel, focusManager, this, this);
WinJS.UI.setOptions(virtualizer, options);
panel.zoomableView = virtualizer;
// surface size observer
this._surfaceSize = new NS.SurfaceSizeObserver(panel, header, virtualizer);
// header controller
this._headerController = new NS.PanoramaHeaderOrchestrator(this, panel, header);
//   if (PlatformJS.isDebug) {
this._metrics = new NS.PanoramaMetricsObserver(panel, virtualizer);
//  }
// base implementation ends
var semanticZoomElement = this._semanticZoomElement = CommonJS.createElement("div", element, "semanticZoom");
CommonJS.setAutomationId(semanticZoomElement, element, "semanticZoom");
WinJS.Utilities.addClass(semanticZoomElement, "platformSemanticZoom");
element.appendChild(semanticZoomElement);
var semanticZoomOutElement = this._semanticZoomOutElement = CommonJS.createElement("div", element, "zoomOut");
semanticZoomOutElement.id = "zoomOut";
CommonJS.setAutomationId(semanticZoomOutElement, element, "zoomOut");
WinJS.Utilities.addClass(semanticZoomOutElement, "platformSemanticZoomOut");
semanticZoomOutElement.setAttribute("aria-labelledby", header._platformHeaderTitle.id);
// insert children into semantic zoom element
semanticZoomElement.appendChild(panelElement);
semanticZoomElement.appendChild(semanticZoomOutElement);
// Create the zoomed out listview
this._zoomedOutItem = new WinJS.UI.ListView(semanticZoomOutElement, { selectionMode: "none" });
this._zoomedOutItem.itemTemplate = function (itemPromise) {
return {
element: itemPromise.then(function semanticZoomPanorama_zoomedOutItemPromiseComplete(group) {
var div = document.createElement("div");
WinJS.Utilities.addClass(div, "platformSemanticZoomItem");
if (group.data) {
// if the title is null, its better to have blank innerText than "undefined"
div.innerText = group.data.clusterTitle || "";
}
return div;
})
};
};
// Create the semantic zoom control
var semanticZoom = this.semanticZoom = new WinJS.UI.SemanticZoom(semanticZoomElement, options);
semanticZoom.locked = true;
this._onsemanticZoomChangedBinding = this._onsemanticZoomChanged.bind(this);
semanticZoom.addEventListener("zoomchanged", this._onsemanticZoomChangedBinding);
// watermark
var watermarkElement = this._watermarkElement = CommonJS.createElement("div", element, "watermark");
WinJS.Utilities.addClass(watermarkElement, "platformWatermark");
watermarkElement.innerText = this._watermark || "";
element.appendChild(watermarkElement);
},
dispose: function dispose() {
if (!this._disposed) {
this._disposed = true;
if (this._panel) {
this._panel.removeEventListener("itemremoved", this._onItemRemovedBinding);
}
this.cancelCustomizationView();
if (this._updateCustomizationAddButtonBinding) {
window.removeEventListener("resize", this._updateCustomizationAddButtonBinding);
}
var _semanticZoom = this.semanticZoom;
if (_semanticZoom) {
_semanticZoom._clearTimeout(_semanticZoom._completeZoomTimer);
_semanticZoom._clearTimeout(_semanticZoom._TTFFTimer);
if (this._onsemanticZoomChangedBinding) {
_semanticZoom.removeEventListener("zoomchanged", this._onsemanticZoomChangedBinding);
}
};
if (this._closeCustomizationPromise) {
this._closeCustomizationPromise.cancel();
this._closeCustomizationPromise = null;
}
if (this._animationPromise) {
this._animationPromise.cancel();
this._animationPromise = null;
}
this._unregisterCloseHandler();
if (this._customizationListView2) {
this._customizationListView2 = null;
}
if (this._customizationElement) {
this._customizationElement = null;
}
CommonJS.Immersive.Panorama.prototype.dispose.call(this);
}
},
onLayoutChange: function onLayoutChange(layoutState) {
var lastViewState = this._lastViewState;
this._lastViewState = layoutState;
if (lastViewState !== layoutState) {
if (layoutState === Windows.UI.ViewManagement.ApplicationViewState.snapped && this._customizationViewShown) {
this._restoreCustomizationOnLayoutChange = true;
this.cancelCustomizationView();
} else if (lastViewState === Windows.UI.ViewManagement.ApplicationViewState.snapped && this._restoreCustomizationOnLayoutChange) {
this._restoreCustomizationOnLayoutChange = false;
this.showCustomizationView(true);
}
}
CommonJS.Immersive.Panorama.prototype.onLayoutChange.call(this, layoutState);
},
/// Zoom out list options
zoomOutOptions: {
set: function set(value) {
WinJS.UI.setOptions(this._zoomedOutItem, value);
}
},
zoomOptions: {
set: function set(value) {
if (value) {
this._lockSeZo = value.locked;
if (value.locked) {
WinJS.Utilities.addClass(this._semanticZoomElement, "platformSemanticzoom-disabled");
}
else {
WinJS.Utilities.removeClass(this._semanticZoomElement, "platformSemanticzoom-disabled");
}
}
WinJS.UI.setOptions(this.semanticZoom, value);
}
},
semanticZoomRenderer: {
set: function set(value) {
this._zoomedOutItem.itemTemplate = value;
}
},
clusterDataSource: {
set: function set(value) {
this._disposeAllClusters();
this._list = value.list;
var groupedProjection = this._getGroupedProjection(value);
var zoomedOutGroupedProjection = this._getZoomedOutGroupedProjection(value);
if (!this._semanticZoomInitialized) {
this._semanticZoomedOutProjection = zoomedOutGroupedProjection;
}
// if we are going to show customization, there is no need to show the normal view.
if (!(this._enableCustomization && this._showCustomizationImmediate)) {
if (this._semanticZoomInitialized && this._zoomedOutItem) {
this._zoomedOutItem.itemDataSource = this._getZoomedOutDataSource(zoomedOutGroupedProjection.groups.dataSource);
}
if (this._panel) {
this._panel.clear();
}
if (this._virtualizer) {
this._virtualizer.dataSource = groupedProjection.dataSource;
if (this._focusManager) {
this._focusManager.virtualizer = this._virtualizer;
}
}
} else {
msSetImmediate(this.initializeCustomizationView.bind(this));
}
}
},
initializeSemanticZoom: function initializeSemanticZoom() {
if (!this._semanticZoomInitialized && !this._disposed) {
this._semanticZoomInitialized = true;
var that = this,
options = this._options,
semanticZoomElement = this._semanticZoomElement,
semanticZoomOutElement = this._semanticZoomOutElement;
WinJS.UI.setOptions(this, this._remainingOptions);
if (!this._zoomedOutItem._disposed) {
this._zoomedOutItem.itemDataSource = this._getZoomedOutDataSource(this._semanticZoomedOutProjection.groups.dataSource);
}
}
},
// customization view options
enableCustomization: {
set: function set(value){
this._enableCustomization = value;
}
},
customizationMode: {
set: function set(value) {
this._customizationMode = value;
}
},
customizationTitle: {
set: function set(value) {
this._customizationTitle = value;
}
},
maxCustomizationClusterCount: {
set: function set(value) {
this._maxCustomizationClusterCount = value;
}
},
customizationItemTemplate: {
set: function set(value) {
this._customizationItemTemplate = value;
}
},
customizationAddButtonRenderer: {
set: function set(value) {
this._customizationAddButtonRenderer = value;
}
},
customizationAddCallback: {
set: function set(value) {
this._customizationAdd = value;
}
},
queryClusterData: {
set: function set(value) {
this._queryClusterData = value;
}
},
onCustomizationCompleted: {
set: function set(value) {
this._customizationCompletedCallback = value;
}
},
customizationUseVariableSizedTemplates: {
set: function set(value) {
this._customizationUseVariableSizedTemplates = value;
}
},
customizationAddButtonOptions: {
set: function set(value) {
this._customizationAddButtonOptions = value;
}
},
customizationImageMap: {
get: function get() {
if (this._customizationImageMap) {
return JSON.stringify(this._customizationImageMap);
} else {
return "";
}
},
set: function set(value) {
if (value) {
this._customizationImageMap = JSON.parse(value);
}
}
},
showCustomizationImmediate: {
set: function set(value){
this._showCustomizationImmediate = value;
if (value && !this._customizeViewInitialized) {
WinJS.Utilities.addClass(this._headerElement, "platformHidden");
}
}
},
initializeCustomizationView: function initializeCustomizationView() {
if (this._enableCustomization && !this._customizeViewInitialized && !this._disposed && !this._initializeCustomizationViewInProgress) {
var that = this;
this._initializeCustomizationViewInProgress = true;
if (!this._customizationMode) {
this._customizationMode = CommonJS.Immersive.SemanticZoomPanorama.CUSTOMIZATIONMODE.full;
}
// container
this._customizationElement = CommonJS.createElement("div", this._element, "customization");
WinJS.Utilities.addClass(this._customizationElement, "platformCustomizationView");
// close button
var closeButton = this._customizationCloseButton = document.createElement("button");
closeButton.setAttribute("aria-label", CommonJS.resourceLoader.getString("/platform/GoBack"));
CommonJS.setAutomationId(closeButton, this._customizationElement, "closeButton");
WinJS.Utilities.addClass(closeButton, "win-backbutton immersiveBackButton");
this._customizationElement.appendChild(closeButton);
// title
if (this._customizationTitle) {
var customizationTitle = CommonJS.createElement("h1", this._customizationElement, "title");
WinJS.Utilities.addClass(customizationTitle, "immersiveHeaderTitle");
customizationTitle.textContent = this._customizationTitle;
this._customizationElement.appendChild(customizationTitle);
}
// customization add button
var createActionTray = function () {
if (!that._actionTray) {
that._actionTray = CommonJS.createElement("div", that._customizationViewport, "actionTray");
WinJS.Utilities.addClass(that._actionTray, "customizationActionTray");
that._customizationElement.appendChild(that._actionTray);
}
};
if (this._haveCustomizationAdd()) {
createActionTray();
this._customizationAddButton = CommonJS.createElement("button", this._customizationElement, "addButton");
if (this._customizationAddButtonRenderer) {
this._customizationAddButton.appendChild(this._customizationAddButtonRenderer());
} else {
this._customizationAddButton.appendChild(this._defaultCustomizationAddButtonRenderer());
}
this._customizationAddButton.title = CommonJS.resourceLoader.getString("/platform/add");
this._customizationAddButton.onclick = this._customizationAddClicked.bind(this);
WinJS.Utilities.addClass(this._customizationAddButton, "customizationAddButton");
this._actionTray.appendChild(this._customizationAddButton);
this._updateCustomizationAddButtonBinding = this._updateCustomizationAddButton.bind(this);
window.addEventListener("resize", this._updateCustomizationAddButtonBinding);
}
// ListView2 viewport and container
this._customizationViewport = CommonJS.createElement("div", this._customizationElement, "customizationViewport");
WinJS.Utilities.addClass(this._customizationViewport, "customizationViewport");
var customizationSurface = CommonJS.createElement("div", this._customizationViewport, "customizationSurface");
WinJS.Utilities.addClass(customizationSurface, "customizationSurface");
// ListView2 control
var enabledFeatues = CommonJS.UI.ListView2.FEATURES.REMOVE | CommonJS.UI.ListView2.FEATURES.REORDER;
if (this._customizationMode === CommonJS.Immersive.SemanticZoomPanorama.CUSTOMIZATIONMODE.sort) {
enabledFeatues = CommonJS.UI.ListView2.FEATURES.REORDER;
}
var layout = { type: WinJS.UI.GridLayout, maxRows: 1 };
if (this._customizationUseVariableSizedTemplates) {
enabledFeatues = enabledFeatues | CommonJS.UI.ListView2.FEATURES.ADD; // use an invisible ADD button to work around insert point issue.
layout.groupInfo = {
enableCellSpanning: true,
cellWidth: 60,
cellHeight: 161
};
}
var listView2Element = CommonJS.createElement("div", this._customizationElement, "listview2container");
WinJS.Utilities.addClass(listView2Element, "listview2container");
this._customizationList = new WinJS.Binding.List();
this._customizationListView2 = new CommonJS.UI.ListView2(listView2Element, {
friendlyName: "cluster",
logContext: "Cluster Customization View",
selectionMode: "none",
layout: layout,
enabledFeatures: enabledFeatues,
addTileItemTemplate: this._invisibleCustomizationAddTileItemTemplate.bind(this),
itemTemplate: this._customizationItemTemplate || this._defaultCustomizationItemTemplate.bind(this),
itemDataSource: this._customizationList.dataSource,
onitemdeleted: this._onCustomizationItemRemoved.bind(this),
disableEditModeOnBlurred: false,
onkeyboardnavigating: this._customizationKeyboardScrolling.bind(this)
});
customizationSurface.appendChild(listView2Element);
this._customizationViewport.appendChild(customizationSurface);
this._customizationElement.appendChild(this._customizationViewport);
msSetImmediate(function finishInitializeCustomizationView() {
if (that._customizationElement && !that._disposed) {
var viewport = listView2Element.getElementsByClassName("win-viewport")[0];
if (viewport) {
// HACKY "top = "-50vh" doesn't work in CSS file.
viewport.style.top = "-50vh";
}
that._listView2surface = listView2Element.getElementsByClassName("win-surface")[0];
that._customizeViewInitialized = true;
that._initializeCustomizationViewInProgress = false;
if (that._showCustomizationImmediate) {
that.showCustomizationView(false);
}
}
});
this._element.appendChild(this._customizationElement);
}
},
getCustomizationResult: function getCustomizationResult() {
var list = null;
if (this._customizationViewShown) {
list = this._createCustomizationClustersIncludingInvisible();
} else {
this._updateCurrentClusters();
list = this._currentClusters;
}
return list;
},
showCustomizationView: function showCustomizationView(skipUpdateList) {
if (this._customizeViewInitialized && !this._customizationViewShown &&
(this._customizationAdd || !this._haveCustomizationAdd())) {
this._customizationViewShown = true;
this._closeCustomizationPromise = null;
var eventObject = document.createEvent("CustomEvent");
eventObject.initCustomEvent("samePageNav", true, true, null);
CommonJS.WindowEventManager.getInstance().dispatchEvent("samePageNav", eventObject);
if (!skipUpdateList || !this._customizationList.length) {
this._updateCurrentClusters();
if (this._isListDifferent(this._customizationResult)) {
// if customization is not used yet, or if new cluster is added outside of customization
this._updateCustomizationList();
}
}
this._removedCustomizationItems = [];
this._customizationListView2.enableEditMode();
this._customizationListView2.tapBehavior = "invokeOnly";
this._updateCustomizationAddButton();
if (this._animationPromise) {
this._animationPromise.cancel();
this._animationPromise = null;
}
this._animationPromise = this._showCustomizationViewAnimation();
return true;
} else {
if (!this._customizeViewInitialized) {
this._showCustomizationImmediate = true;
this.initializeCustomizationView();
}
return false;
}
},
_invisibleCustomizationAddTileItemTemplate: function _invisibleCustomizationAddTileItemTemplate(itemPromise) {
var that = this;
return itemPromise.then(function renderInvisibleAddTile() {
var container = document.createElement("div");
WinJS.Utilities.addClass(container, "customizationItemContainer customizationItemContainerSmall invisibleAddTile");
container.style.visibility = "hidden";
return container;
});
},
_defaultCustomizationItemTemplate: function _defaultCustomizationItemTemplate(itemPromise) {
var that = this;
var setDivBackgroundImage = function (div, image) {
div.style.backgroundImage = "url('" + image + "')";
};
return itemPromise.then(function customizationItemTemplateRenderer(result) {
var render = function (item) {
// item root element
var container = document.createElement("div");
var itemData = (item && item.data) || null;
var title = (itemData && itemData.clusterTitle) || undefined;
container.title = title;
// item root element size
var defaultBackground = "/common/images/customizationItemMedium.png";
WinJS.Utilities.addClass(container, "customizationItemContainer");
if (that._customizationUseVariableSizedTemplates) {
var itemSize = (itemData && itemData.itemSize) || null;
switch (itemSize) {
case CommonJS.Immersive.SemanticZoomPanorama.CUSTOMIZATIONTEMPLATESIZE.large:
WinJS.Utilities.addClass(container, "customizationItemContainerLarge");
defaultBackground = "/common/images/customizationItemLarge.png";
break;
case CommonJS.Immersive.SemanticZoomPanorama.CUSTOMIZATIONTEMPLATESIZE.small:
WinJS.Utilities.addClass(container, "customizationItemContainerSmall");
defaultBackground = "/common/images/customizationItemSmall.png";
break;
case CommonJS.Immersive.SemanticZoomPanorama.CUSTOMIZATIONTEMPLATESIZE.medium:
default:
WinJS.Utilities.addClass(container, "customizationItemContainerMedium");
break;
}
} else {
WinJS.Utilities.addClass(container, "customizationItemContainerMedium");
}
// title
var titleElement = document.createElement("div");
WinJS.Utilities.addClass(titleElement, "customizationItemTitle");
titleElement.textContent = title;
container.appendChild(titleElement);
var imageDiv = document.createElement("div");
WinJS.Utilities.addClass(imageDiv, "customizationItemBackgroundImage");
var backgroundImage = null;
if (itemData) {
if (itemData.itemKey && that._customizationImageMap && that._customizationImageMap[itemData.itemKey]) {
backgroundImage = that._customizationImageMap[itemData.itemKey];
} else {
backgroundImage = itemData.customizationItemBackgroundImage;
}
}
if (!backgroundImage) {
if (itemData && itemData.isHero) {
// hero cluster
backgroundImage = itemData.clusterContent && itemData.clusterContent.contentOptions && itemData.clusterContent.contentOptions.backgroundImage && itemData.clusterContent.contentOptions.backgroundImage.lowResolutionUrl;
backgroundImage = backgroundImage || defaultBackground;
} else {
backgroundImage = defaultBackground;
}
}
if (backgroundImage) {
if (!that._customizationImageMap) {
that._customizationImageMap = {};
}
if (itemData && itemData.itemKey) {
that._customizationImageMap[itemData.itemKey] = backgroundImage;
}
if (backgroundImage.indexOf("/") === 0 || backgroundImage.indexOf("ms-appdata://") === 0) {
setDivBackgroundImage(imageDiv, backgroundImage);
} else {
WinJS.Utilities.addClass(imageDiv, "customizationItemOnlineBackgroundImage");
PlatformJS.Cache.CacheService.getInstance("PlatformImageCache")
.findEntry(backgroundImage, { fileNameOnly: true })
.then(function (response) {
if (!response || response.isStale()) {
PlatformJS.Utilities.fetchImage("PlatformImageCache", backgroundImage)
.then(function (fetchResponse) {
setDivBackgroundImage(imageDiv, fetchResponse);
});
} else {
setDivBackgroundImage(imageDiv, response.dataValue);
}
});
}
}
container.appendChild(imageDiv);
return container;
};
if (WinJS.Promise.is(result)) {
return result.then(render);
} else {
return render(result);
}
});
},
_defaultCustomizationAddButtonRenderer: function _defaultCustomizationAddButtonRenderer(){
if (this._customizationAddButtonOptions && this._customizationAddButtonOptions.title) {
var addSectionContainer = CommonJS.createElement("div", this._customizationAddButton, "addSection");
WinJS.Utilities.addClass(addSectionContainer, "customizationAddSectionContainer");
var addSectionTitle = CommonJS.createElement("div", addSectionContainer, "addSectionTitle");
WinJS.Utilities.addClass(addSectionTitle, "customizationAddSectionTitle");
addSectionTitle.textContent = this._customizationAddButtonOptions.title;
addSectionContainer.appendChild(addSectionTitle);
var addSectionContent = CommonJS.createElement("div", addSectionContainer, "addSectionContent");
WinJS.Utilities.addClass(addSectionContent, "customizationAddSectionContent");
addSectionContent.textContent = this._customizationAddButtonOptions.content;
addSectionContainer.appendChild(addSectionContent);
var addSectionButton = CommonJS.createElement("span", addSectionContainer, "addSectionButton");
WinJS.Utilities.addClass(addSectionButton, "customizationAddSectionButton");
addSectionContainer.appendChild(addSectionButton);
return addSectionContainer;
} else {
var addButtonContentdiv = document.createElement("div");
WinJS.Utilities.addClass(addButtonContentdiv, "customizationAddButtonContent");
return addButtonContentdiv;
}
},
_customizationKeyboardScrolling: function _customizationKeyboardScrolling(e) {
if (e && e.detail && e.detail.newFocus !== e.detail.oldFocus) {
var index = e.detail.newFocus;
if (this._customizationUseVariableSizedTemplates && index === this._customizationList.length - 1) {
e.stopPropagation();
e.preventDefault();
} else {
var element = this._customizationListView2.elementFromIndex(index);
var offset = element.parentElement.offsetLeft;
this._customizationViewport.scrollLeft = offset - document.body.clientWidth / 2;
}
}
},
_updateCustomizationAddButton: function _updateCustomizationAddButton() {
if (this._customizationViewShown) {
if (this._customizationListView2 && this._customizationListView2.element && this._actionTray) {
var topOffset = this._customizationListView2.element.offsetTop + this._customizationListView2.element.clientHeight + 40;
this._actionTray.style.top = topOffset + "px";
}
if (this._maxCustomizationClusterCount >= 0 && this._customizationAddButton) {
var length = this._createCustomizationClusters().length;
if (length >= this._maxCustomizationClusterCount) {
this._customizationAddButton.disabled = true;
} else {
this._customizationAddButton.disabled = false;
}
}
}
},
_onCustomizationItemRemoved: function _onCustomizationItemRemoved(e) {
if (this._customizationViewShown) {
if (this._removedCustomizationItems && e && e.detail && e.detail.itemKey) {
this._removedCustomizationItems.push(e.detail.itemKey);
}
this._updateSurfaceSize();
this._updateCustomizationAddButton();
}
},
_closeCustomizationViewHandler: function _closeCustomizationViewHandler(e) {
var isBackButtonClicked = (e.type === "click" || e.type === "samePageNav");
var isBackSpacePressed = (e.keyCode === WinJS.Utilities.Key.backspace || e.keyCode === WinJS.Utilities.Key.escape);
if (isBackButtonClicked || isBackSpacePressed) {
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal,
"Cluster Customization View",
"Back Button",
Platform.Telemetry.UserActionOperation.click,
Platform.Telemetry.UserActionMethod.unknown, // have no way to detect input method for now.
0);
this.closeCustomizationView();
}
},
closeCustomizationView: function closeCustomizationView() {
if (this._customizationViewShown) {
var that = this;
if (!this._closeCustomizationPromise) {
this._closeCustomizationPromise = this._postCustomization().then(that._hideCustomizationShowAnimation.bind(that)).then(function notifyCustomizationCompleted() {
// customization view is already hidden
that._closeCustomizationPromise = null;
if (that._customizationCompletedCallback) {
that._customizationCompletedCallback(that._customizationResult);
}
});
}
}
},
cancelCustomizationView: function cancelCustomizationView() {
if (this._customizationViewShown) {
if (this._animationPromise) {
this._animationPromise.cancel();
this._animationPromise = null;
}
this._animationPromise = this._hideCustomizationShowAnimation();
}
},
_updateSurfaceSize: function _updateSurfaceSize() {
var that = this;
msSetImmediate(function updateCustomizationViewSurfaceSize() {
var listView2Element = that._customizationListView2 && that._customizationListView2.element;
if (that._customizationViewShown && that._listView2surface && that._listView2surface.style.width && listView2Element) {
listView2Element.style.width = that._listView2surface.style.width;
listView2Element.parentNode.style.width = that._listView2surface.style.width;
}
});
},
_disableBottomEdgy: function _disableBottomEdgy(value) {
var appbarDoms = WinJS.Utilities.query(".win-appbar");
for (var i = 0; i < appbarDoms.length; i++) {
var appbarControl = appbarDoms[i].winControl;
if (appbarControl && appbarControl.placement === "bottom") {
appbarControl.disabled = value;
}
}
},
_showCustomizationViewAnimation: function _showCustomizationViewAnimation() {
var that = this;
CommonJS.dismissAllEdgies();
this._disableBottomEdgy(true);
this._viewportInVisibility = this.semanticZoom._viewportIn.style.visibility;
this._viewportOutVisibility = this.semanticZoom._viewportOut.style.visibility;
this._customizationElement.style.visibility = "visible";
WinJS.UI.executeTransition(this._semanticZoomElement,
{
property: "opacity",
delay: 0,
duration: 300,
timing: "ease",
to: "0"
}).then(function hideNormalView() {
if (that.semanticZoom._viewportIn && that.semanticZoom._viewportOut) {
that.semanticZoom._viewportIn.style.visibility = "hidden";
that.semanticZoom._viewportOut.style.visibility = "hidden";
}
if (that._showCustomizationImmediate) {
WinJS.Utilities.removeClass(that._headerElement, "platformHidden");
}
});
return WinJS.UI.executeTransition(this._customizationElement,
{
property: "transform",
delay: 0,
duration: 0,
to: "scale(2)"
}).then(function () {
return WinJS.UI.executeTransition(that._customizationElement,
[{
property: "transform",
delay: 0,
duration: 300,
timing: "ease",
to: "scale(1)"
},
{
property: "opacity",
delay: 0,
duration: 300,
timing: "ease",
to: "1"
}]);
}).then(function postShowCustomizationAnimation() {
that._updateSurfaceSize();
}).then(function registerCloseCustomizationViewHandler() {
that._registerCloseHandler();
});
},
_hideCustomizationShowAnimation: function _hideCustomizationShowAnimation() {
var that = this;
this.semanticZoom._viewportIn.style.visibility = this._viewportInVisibility;
this.semanticZoom._viewportOut.style.visibility = this._viewportOutVisibility;
this._disableBottomEdgy(false);
WinJS.UI.executeTransition(this._semanticZoomElement,
{
property: "opacity",
delay: 0,
duration: 300,
timing: "ease",
to: "1"
});
return WinJS.UI.executeTransition(this._customizationElement,
[{
property: "transform",
delay: 0,
duration: 300,
timing: "ease",
to: "scale(2)"
},
{
property: "opacity",
delay: 0,
duration: 300,
timing: "ease",
to: "0"
}]).then(function () {
return WinJS.UI.executeTransition(that._customizationElement,
{
property: "transform",
delay: 0,
duration: 0,
to: "scale(1)"
});
}).then(function postHideCustomizationViewAnimation() {
if (that._customizationElement) {
that._customizationElement.style.visibility = "hidden";
that._customizationViewShown = false;
}
}).then(function unregisterCloseHandler() {
that._unregisterCloseHandler();
});
},
_postCustomization: function _postCustomization() {
var list = this._customizationResult = this._createCustomizationClustersIncludingInvisible();
// if _showCustomizationImmediate, the normal view is not drawn yet.
if (!this._isListDifferent(list) && !this._showCustomizationImmediate) {
return WinJS.Promise.wrap(false);
}
this._showCustomizationImmediate = false;
var newListOrder = {};
var currentListOrder = {};
list.forEach(function (item, index) {
newListOrder[item] = index + 1;
});
this._currentClusters.forEach(function (item, index) {
currentListOrder[item] = index + 1;
});
var promises = [];
// add new cluster
var that = this;
list.forEach(function customizationUpdateBindingList_Add(item, index) {
if (!currentListOrder[item]) {
var fullData = null;
if (this._queryClusterData) {
fullData = that._queryClusterData(item);
}
if (!fullData) {
fullData = that._customizationList.getAt(index);
}
(function postQueryCluster(queryResult) {
if (queryResult) {
if (WinJS.Promise.is(queryResult)) {
promises.push(queryResult.then(function (data) {
if (data) {
that._list.push(data);
}
}));
} else {
that._list.push(queryResult);
}
}
})(fullData);
}
}, this);
return WinJS.Promise.join(promises).then(function customizationUpdateBindingList_SortRemove() {
// sort
that._list.sort(function customizationUpdateBindingList_Sort(a, b) {
var aOrder = newListOrder[a.clusterKey] || 0;
var bOrder = newListOrder[b.clusterKey] || 0;
return aOrder - bOrder;
});
// remove
for (var index = 0, count = that._list.length; index < count; index++) {
if (newListOrder[that._list.getAt(index).clusterKey]) {
break;
}
}
that._list.splice(0, index);
// can't use refresh here. Since _list.sort will remove all its dataBinding's retain (which doesn't make sense to me, however WinJS does work in this way)
// then since lack of dataBidning, virtualizer can't update its _totalCount, if new list's length is shorter that the old one, virtualize will always try to
// realize not existing items.
// so use reset clusterDataSource to force refresh it.
that.clusterDataSource = that._list.dataSource;
return true;
});
},
_isListDifferent: function _isListDifferent(list) {
if (list && list.length === this._currentClusters.length) {
for (var i = 0, ilen = list.length; i < ilen; i++) {
if (list[i] !== this._currentClusters[i]) {
return true;
}
}
return false;
} else {
return true;
}
},
// create an array based current panorama clusters
_updateCurrentClusters: function _updateCurrentClusters() {
this._currentClusters = [];
for (var index = 0, count = this._list.length; index < count; index++) {
var data = this._list.getAt(index);
this._currentClusters.push(data.clusterKey);
}
},
// create an array based on customization view content
_createCustomizationClusters: function _createCustomizationClusters() {
var list = this._customizationList.filter(function (item) {
return item.itemKey && !item.isCommonJSAddTile;
}).map(function (item) {
return item.itemKey;
});
return list;
},
// create an array based on customization view content, plus invisible clusters
_createCustomizationClustersIncludingInvisible: function _createCustomizationClustersIncludingInvisible() {
var list = this._createCustomizationClusters();
if (this._customizationInvisibleList) {
var count = this._customizationInvisibleList.length;
var wholeListLength = list.length + count;
var previousPoint = 0;
var insertPoint = 0;
for (var index = 0; index < count; index++) {
var invisibleItem = this._customizationInvisibleList[index];
if (invisibleItem && invisibleItem.data) {
insertPoint = invisibleItem.index;
if (insertPoint < 0) {
insertPoint = wholeListLength + invisibleItem.index;
}
if (invisibleItem.data.platformInvisiblePermanentCluster) {
// must insert cluster
if (insertPoint < previousPoint) {
insertPoint = previousPoint;
} else if (insertPoint > list.length) {
insertPoint = list.length;
}
previousPoint = insertPoint;
list.splice(insertPoint, 0, invisibleItem.data.clusterKey);
} else {
if (insertPoint > previousPoint && insertPoint <= list.length) {
previousPoint = insertPoint;
list.splice(insertPoint, 0, invisibleItem.data.clusterKey);
} else {
wholeListLength--;
}
}
} else {
wholeListLength--;
}
}
}
return list;
},
// discard customization view content, refresh the content with current panorama clusters
_updateCustomizationList: function _updateCustomizationList() {
if (this._customizationList.length) {
this._customizationList.splice(0, this._customizationList.length);
}
// can't reset the invisible list, otherwise the removed ads (caused by cluster number reduced) won't be added back if the cluster number is increased again.
if (!this._customizationInvisibleList) {
this._customizationInvisibleList = [];
this._customizationInvisibleListMap = {};
}
var headPart = true;
for (var index = 0, count = this._list.length; index < count; index++) {
var data = this._list.getAt(index);
data.itemKey = data.clusterKey;
data.title = data.clusterTitle;
if (data.isHero) {
data.pinned = true;
}
if (!headPart && index !== count - 1 && data.platformInvisibleCluster) {
data.platformInvisiblePermanentCluster = false;
}
if (data.platformInvisibleCluster) {
var invisibleIndex = index;
if (!data.platformInvisiblePermanentCluster || (data.platformInvisiblePermanentCluster && index === count - 1)) {
invisibleIndex = index - count;
}
if (typeof this._customizationInvisibleListMap[data.clusterKey] !== "number") {
this._customizationInvisibleList.push({
index: invisibleIndex,
key: data.clusterKey,
data: data
});
this._customizationInvisibleListMap[data.clusterKey] = this._customizationInvisibleList.length - 1;
} else {
this._customizationInvisibleList.splice(this._customizationInvisibleListMap[data.clusterKey], 1, {
index: invisibleIndex,
key: data.clusterKey,
data: data
});
}
} else {
this._customizationList.push(data);
}
headPart = headPart && data.pinned;
}
},
_customizationAddClicked: function _customizationAddClicked() {
if (this._customizationAdd) {
this._customizationListView2.cancelDrag();
var list = this._createCustomizationClustersIncludingInvisible();
Platform.Telemetry.FlightRecorder.logUserAction(Platform.Telemetry.LogLevel.normal,
"Cluster Customization View",
"addButton",
Platform.Telemetry.UserActionOperation.click,
Platform.Telemetry.UserActionMethod.unknown, // have no way to detect input method for now.
0);
this._customizationAdd(list, this._removedCustomizationItems, this._postAddUIClosed.bind(this));
}
},
_postAddUIClosed: function _postAddUIClosed(pickedItems) {
if (pickedItems.length) {
var originalLength = this._customizationList.length;
var that = this;
pickedItems.forEach(function (item) {
item.itemKey = item.clusterKey;
that._customizationList.push(item);
});
this._removedCustomizationItems = [];
}
this._updateSurfaceSize();
this._updateCustomizationAddButton();
},
_haveCustomizationAdd: function _haveCustomizationAdd() {
return !(this._customizationMode === CommonJS.Immersive.SemanticZoomPanorama.CUSTOMIZATIONMODE.sort ||
this._customizationMode === CommonJS.Immersive.SemanticZoomPanorama.CUSTOMIZATIONMODE.sortRemove);
},
_registerCloseHandler: function _registerCloseHandler() {
if (!this._closeCustomizationViewHandlerBinding) {
this._closeCustomizationViewHandlerBinding = this._closeCustomizationViewHandler.bind(this);
if (this._customizationCloseButton) {
this._customizationCloseButton.addEventListener("click", this._closeCustomizationViewHandlerBinding);
}
document.addEventListener("keydown", this._closeCustomizationViewHandlerBinding);
CommonJS.WindowEventManager.getInstance().addEventListener("samePageNav", this._closeCustomizationViewHandlerBinding);
}
},
_unregisterCloseHandler: function _unregisterCloseHandler() {
if (this._closeCustomizationViewHandlerBinding) {
CommonJS.WindowEventManager.getInstance().removeEventListener("samePageNav", this._closeCustomizationViewHandlerBinding);
if (this._customizationCloseButton) {
this._customizationCloseButton.removeEventListener("click", this._closeCustomizationViewHandlerBinding);
}
document.removeEventListener("keydown", this._closeCustomizationViewHandlerBinding);
this._closeCustomizationViewHandlerBinding = null;
}
},
_onItemRemoved: function _onItemRemoved(item) {
var itemData = item && item.detail && item.detail.data;
if (itemData.platformInvisiblePermanentCluster && this._customizationInvisibleListMap && typeof this._customizationInvisibleListMap[itemData.clusterKey] === "number") {
this._customizationInvisibleList.splice(this._customizationInvisibleListMap[itemData.clusterKey], 1, null); // if remove the item, the index of other items will be changed, so only replace it with null.
this._customizationInvisibleListMap[itemData.clusterKey] = null;
}
},
//#region Private Helper Function
_isSemanticZoomEnabled: function _isSemanticZoomEnabled(item) {
return !(item.platformInvisibleCluster);
},
_getZoomedOutGroupedProjection: function _getZoomedOutGroupedProjection(dataSource) {
var filteredList = dataSource.list.createFiltered(this._isSemanticZoomEnabled);
return this._getGroupedProjection(filteredList.dataSource);
},
_getGroupedProjection: function _getGroupedProjection(dataSource) {
return dataSource.list.createGrouped(
function (item) {
return item.clusterKey;
},
function (item) {
return item;
},
function (left, right) {
return 0;
}
);
},
// wraps around the itemFromKey function of the out-of-box grouped data source object.
// The zoomed-out data source is different from the zoomed-in (default) data source.
// Hence if current cluster in the zoomed-in view has semanticZoomEnabled set to false, 
// when zooming out, the zoomed-out data source won't be able to locate the cluster. 
// This function helps to curb the problem.
_getZoomedOutDataSource: function _getZoomedOutDataSource(filteredDataSource) {
if (!filteredDataSource.oldItemFromKey) {
filteredDataSource.oldItemFromKey = filteredDataSource.itemFromKey;
filteredDataSource.itemFromKey = function itemFromKeyWrapper(key) {
var that = this;
try {
return this.oldItemFromKey(key);
} catch (error) {
return this.itemFromIndex(0).then(function getFirstItemFromKey(item) {
return that.oldItemFromKey(item.key); // need to retrieve item using this call as it prepares the item differently from itemFromIndex
});
}
};
}
return filteredDataSource;
},
_onsemanticZoomChanged: function _onsemanticZoomChanged(event) {
if (event.detail) {
// zoom out
if (this._headerElement && this._semanticZoomOutElement) {
this._headerParent = this._headerElement.parentNode;
this._semanticZoomOutElement.appendChild(this._headerElement);
}
} else {
// zoom in
if (this._headerElement && this._headerParent) {
this._headerParent.appendChild(this._headerElement);
}
}
this.dispatchEvent("zoomchanged", event.detail);
},
//#endregion Private Helper Function
}, {
CUSTOMIZATIONMODE: {
full: "full",
sortRemove: "sortRemove",
sort: "sort"
},
CUSTOMIZATIONTEMPLATESIZE: {
large: "large",
medium: "medium",
small: "small"
}
})
});
})();
/* File: SurfaceSizeObserver.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function () {
"use strict";
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
SurfaceSizeObserver: WinJS.Class.define(function surfaceSizeObserver_ctor(panoramaPanel, panoramaHeader, panoramaVirtualizer) {
this._panel = panoramaPanel;
this._header = panoramaHeader;
this._virtualizer = panoramaVirtualizer;
this._snapPoints = [];
this._initialize();
}, {
// controls
_panel: null,
_virtualizer: null,
_header: null,
// private variables
_snapPoints: null,
_surfaceMargin: null,
_zeroOffset: null,
_viewportWidth: null,
_surfaceImageBleedMin: null,
_surfaceMinWidth: null,
// context event references
_itemRealizedBinding: null,
_itemResizedBinding: null,
_itemRemovedBinding: null,
// methods
_initialize: function _initialize() {
var panel = this._panel;
this._zeroOffset = this._computeZeroOffset();
this._viewportWidth = document.body.offsetWidth;
var surfaceMargin = this._computeSurfaceMargin();
this._surfaceImageBleedMin = (surfaceMargin === 0 ? (this._viewportWidth + 300) : 0);
var itemRealized = this._itemRealizedBinding = this._onItemRealized.bind(this);
var itemResized = this._itemResizedBinding = this._onItemResized.bind(this);
var itemRemoved = this._itemRemovedBinding = this._onItemRemoved.bind(this);
panel.addEventListener("itemrealized", itemRealized);
panel.addEventListener("itemresized", itemResized);
panel.addEventListener("itemremoved", itemRemoved);
},
dispose: function dispose() {
var panel = this._panel;
panel.removeEventListener("itemrealized", this._itemRealizedBinding);
panel.removeEventListener("itemresized", this._itemResizedBinding);
panel.removeEventListener("itemremoved", this._itemRemovedBinding);
this._panel = null;
this._virtualizer = null;
this._header = null;
},
_computeZeroOffset: function _computeZeroOffset() {
var headerElement = document.querySelector(".immersiveHeaderTitle");
if (headerElement) {
return headerElement.getOffset();
} else {
return 120;
}
},
zeroOffset: {
get: function get() {
if (!this._zeroOffset) {
this._zeroOffset = this._computeZeroOffset();
}
return this._zeroOffset;
}
},
surfaceMargin: {
get: function get() {
if (!this._surfaceMargin) {
this._surfaceMargin = this._computeSurfaceMargin();
}
return this._surfaceMargin;
}
},
_computeSurfaceMargin: function _computeSurfaceMargin() {
var surface = this._panel._surface;
return surface.getMarginLeft();
},
_onItemRealized: function _onItemRealized(event) {
// when a new item is realized, we set the first values
// and update the snap-points accordingly.  this is especially
// useful when re-hydrating a view from existing cache
// where onItemResized might never be called
var item = event.detail;
var element = item.element;
var snapPoints = this._snapPoints;
var zeroOffset = this._zeroOffset;
var snapPointsOffset = 0;
var panel = this._panel;
var offset = 0;
var width = 0;
var delta = 0;
if (element) {
offset = (element.getOffset() - zeroOffset);
width = element.offsetWidth;
delta = WinJS.Utilities.getTotalWidth(element);
item.offset = offset;
item.width = width;
if (panel && panel.zeroOffsetSnapPoints === true) {
snapPointsOffset = zeroOffset;
}
snapPoints.splice(item.index, 0, Math.max(offset + snapPointsOffset, 0) + "px");
}
var next = item.next;
while (next) {
if (next.element) {
next.offset += delta;
snapPoints[next.index] = Math.max(next.offset + snapPointsOffset, 0) + "px";
}
next = next.next;
}
this._updateSnapPoints(item);
this._updateWhitespace(item);
this._cancelScroll(item, delta);
},
_onItemResized: function _onItemResized(evt) {
var item = evt.detail;
var panel = this._panel;
var snapPoints = this._snapPoints;
var delta = null;
var zeroOffset = this._zeroOffset;
var snapPointsOffset = 0;
var element = item.element;
if (element) {
var previous = item.previous;
var offset = item.offset ? item.offset : (previous ? (previous.offset + previous.width) : (element.getOffset() - zeroOffset));
var width = element.offsetWidth;
delta = (width - item.width);
item.deltaWidth = width - (item.width || 0);
item.width = width;
if (panel && panel.zeroOffsetSnapPoints === true) {
snapPointsOffset = zeroOffset;
}
snapPoints[item.index] = Math.max(item.offset + snapPointsOffset, 0) + "px";
var next = item.next;
while (next) {
next.offset += delta;
snapPoints[next.index] = Math.max(next.offset + snapPointsOffset, 0) + "px";
// break on last (used for extra whitespace)
if (next.next) {
next = next.next;
} else {
break;
}
}
next = next || item;
this._updateSnapPoints(next);
this._updateWhitespace(next);
this._cancelScroll(item, delta);
}
},
_onItemRemoved: function _onItemRemoved(evt) {
// when an existing item is removed, we must update
// the snap-points as well as the offset of following
// items to refresh data
var item = evt.detail;
var delta = 0;
var snapPoints = this._snapPoints;
var panel = this._panel;
var zeroOffset = this._zeroOffset;
snapPoints.splice(item.index, 1);
var candidate = item.next;
if (candidate) {
delta = candidate.offset - item.offset;
while (candidate) {
candidate.offset -= delta;
snapPoints[candidate.index] = (candidate.offset - zeroOffset) + "px";
if (candidate.next) {
candidate = candidate.next;
} else {
break;
}
}
}
this._updateSnapPoints(candidate);
this._updateWhitespace(candidate);
this._cancelScroll(item, delta);
},
_onWindowResize: function _onWindowResize(event) {
//TODO: check zero offset from DOM tree against
//      cached value and if different, we've switched
//      to portrait mode.  in that case, invalidate ALL
//      measures and start over
this._viewportWidth = document.body.offsetWidth;
var zeroOffset = this._computeZeroOffset();
var surfaceMargin = this._computeSurfaceMargin();
var delta = 0;
this._surfaceImageBleedMin = (surfaceMargin === 0 ? (this._viewportWidth + 300) : 0);
if (zeroOffset !== this._zeroOffset || surfaceMargin !== this.surfaceMargin) {
delta = zeroOffset - this._zeroOffset;
this._zeroOffset = zeroOffset;
this._surfaceMargin = surfaceMargin;
this._invalidateAllMeasures(delta);
}
var item = this._panel.current;
if (!item) {
return;
}
while (item.next) {
item = item.next;
}
this._updateSnapPoints(item);
this._updateWhitespace(item);
},
_invalidateAllMeasures: function _invalidateAllMeasures(delta) {
//TODO: we leverage a private property from the virtualizer here
//      to return the proper winControl instance.  we might need
//      to expose the private vector to components.
//[BEGIN]
////TODO (eric): need to look for the case when we don't find items list, for now we are returning to avoid the crash
if (!this._virtualizer || !this._virtualizer._realized || !this._virtualizer._realized.first) {
return;
}
var item = this._virtualizer._realized.first;
var zeroOffset = this._zeroOffset;
var snapPoints = [];
while (item) {
var offset = item.offset - delta; //(element.getOffset() - zeroOffset);
//var width = element.offsetWidth;
item.offset = offset;
//item.width = width;
snapPoints.push(Math.max(offset, 0) + "px");
item = item.next;
}
this._snapPoints = snapPoints;
//[END]
},
_timeoutIdentifier: null,
_updateSnapPoints: function _updateSnapPoints(last) {
var that = this;
if (this._timeoutIdentifier) {
clearTimeout(this._timeoutIdentifier);
this._timeoutIdentifier = null;
}
this._timeoutIdentifier = setTimeout(function surfaceSizeObserver_updateSnapPointsTimeout() {
var panel = that._panel;
var snapPoints = that._snapPoints;
//HACK: I have refactored the _updateSnapPointsAndWhitespace logic to the surfaceSizeObserver
//      because it already goes over every item and measures them.  it also
//      allows us to refresh the list of snapPoints only from the last item
//      that changed in size.  however, it requires reading values out of
//      _viewport and _surface which are private variables out of PanoramaPanel.
//[BEGIN]
if (panel) {
var viewport = panel._viewport;
if (viewport && snapPoints) {
// If a scroll limit is applied to viewport, it should also be added as a snap point
// otherwise we can't snap back to page 0 when in mandatory snap mode
var scrollLimitSnap = "";
if (viewport.currentStyle && viewport.currentStyle.msScrollLimitXMin &&
viewport.currentStyle.msScrollLimitXMin !== "auto") {
scrollLimitSnap = viewport.currentStyle.msScrollLimitXMin + ",";
}
viewport.style.msScrollSnapPointsX = "snapList(" + scrollLimitSnap + snapPoints.join(",") + ")";
}
}
//[END]
}, 25);
},
_updateWhitespace: function _updateWhitespace(lastItem) {
var surface = this._panel._surface;
var surfaceMargin = this.surfaceMargin;
var viewportWidth = this._viewportWidth;
//TODO: we should use the viewport's width, but in some scenarios
//      it gets stuck to 0 pixel while we're measuring the children
// Update: not sure if this is still validate. I tried to use document.body.offsetWidth to get the viewport width. but not sure if it's the right way to go.
//[BEGIN]
//var extraSpace = document.body.offsetWidth - surfaceMargin; //viewport.offsetWidth - margin;
var extraSpace = viewportWidth - surfaceMargin;
//[END]
var lastWidth = lastItem ? lastItem.width : 0;
var lastOffset = lastItem ? lastItem.offset : 0;
extraSpace = Math.max(extraSpace, lastWidth);
// Leave enough extraSpace so that previous item can be snapped, but not too much that leaves large white space in the end.
if (lastItem && lastItem.previous && typeof lastItem.previous.offset === "number") {
lastOffset = lastItem.previous.offset;
}
var desired = lastOffset + extraSpace;
// To make the surface scrollable and hide the image bleed when only part of panorama is rendered, surface width has to be atleast _surfaceImageBleedMin
var actual = this._panel.compactPanorama ? this._surfaceImageBleedMin : Math.max(this._surfaceImageBleedMin, desired);
if (this._surfaceMinWidth !== actual) {
surface.style.minWidth = actual + "px";
this._surfaceMinWidth = actual;
}
},
_cancelScroll: function _cancelScroll(item, delta) {
var panel = this._panel;
// if the resized item is before current, adjust scroll offset
if (panel.current && item.index < panel.current.index) {
var scrollPosition = panel.scrollPosition;
scrollPosition += delta;
panel.scrollTo(scrollPosition);
}
},
})
});
})();
/* File: SemanticZoomPanoramaPanel.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsSemanticZoomPanoramaInit(WinJS) {
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
SemanticZoomPanoramaPanel: WinJS.Class.derive(CommonJS.Immersive.PanoramaPanel, function semanticZoomPanoramaPanel_ctor(element, options) {
CommonJS.Immersive.PanoramaPanel.call(this, element, options);
}, {
// private variables
_zoomableView: null,
// properties
zoomableView: {
get: function get() { return this._zoomableView; },
set: function set(value) { this._zoomableView = value; }
},
}),
});
})(WinJS);
/* File: SemanticZoomPanoramaVirtualizer.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformControlsSemanticZoomPanoramaInit(WinJS) {
var NS = WinJS.Namespace.define("CommonJS.Immersive", {
SemanticZoomPanoramaVirtualizer: WinJS.Class.derive(CommonJS.Immersive.PanoramaVirtualizer, function semanticZoomPanoramaVirtualizer_ctor(panoramaPanel, focusManager, panoramaControl, panoramaEventManager) {
CommonJS.Immersive.PanoramaVirtualizer.call(this, panoramaPanel, focusManager, panoramaEventManager);
this._panoramaControl = panoramaControl;
}, {
// private variables
_isZoomedOut: null,
_triggerZoom: null,
_selectedIndex: null,
_panoramaControl: null,
_afterRealizeFirstView: function _afterRealizeFirstView() {
this._panoramaControl.initializeSemanticZoom();
this._panoramaControl.semanticZoom.locked = this._panoramaControl._lockSeZo ? true : false;
this._panoramaControl.initializeCustomizationView();
CommonJS.Immersive.PanoramaVirtualizer.prototype._afterRealizeFirstView.call(this);
},
// IZoomable begins
beginZoom: function beginZoom() {
//var viewport = this._panel._viewport;
//var surface = this._panel._surface;
//var scrollPosition = -viewport.scrollLeft;
//surface.style.left = scrollPosition + "px";
//viewport.style.overflowX = "visible";
//viewport.style.overflowY = "visible";
//surface.setLeft(scrollPosition + "px");
},
configureForZoom: function configureForZoom(isZoomedOut, isCurrentView, triggerZoom, prefectchedPages) {
this._isZoomedOut = isZoomedOut;
this._triggerZoom = triggerZoom;
},
endZoom: function endZoom(isCurrentView, setFocus) {
//var viewport = this._panel._viewport;
//var surface = this._panel._surface;
//viewport.style.overflowX = "auto";
//viewport.style.overflowY = "hidden";
//surface.style.left = "0px";
},
getCurrentItem: function getCurrentItem() {
var panel = this._panel;
if (!panel) {
console.log("getCurrentItem: Null or undefined panel");
} else {
var item = panel.current;
var dataSource = this._dataSource;
if (item) {
var pos = {
left: item.offset,
top: item.element.offsetTop,
width: item.width,
height: item.element.offsetHeight
};
return dataSource.fromIndex(item.index)
.then(function semanticZoomPanoramaVirtualizer_dataSourceFromIndexComplete(listItem) {
return WinJS.Promise.wrap({
item: listItem,
position: pos
});
});
}
}
return null;
},
getPanAxis: function getPanAxis() { return "horizontal"; },
handlePointer: function handlePointer(pointerId) {
// we don't set pointer capture so nested ListView get the proper event routing
},
positionItem: function positionItem(listItem, position) {
var that = this;
return this._getItemIndexAsync(listItem).then(function positionFoundItem(index) {
that.setCurrentIndex(index, true);
that._selectedIndex = index;
return {x: 0,y: 0};
});
},
setCurrentItem: function setCurrentItem(x, y) {
//if (this._realized) {
//    var realized = this._realized;
//    var item = realized.first;
//    while (item) {
//        if (item.offset <= x && (item.offset + item.width) >= x)
//            break;
//        if (item.next)
//            item = item.next;
//        else
//            break;
//    }
//    this._panel.scrollTo(item.offset);
//} else {
//    //TODO: there are bugs reported by the tool
//    //      that identifies either realized as
//    //      being null.  this must be a timing
//    //      issue with the SemanticZoom control
//    //      that invalidates too early.
//    //[BEGIN]
//    debugger;
//    //[END]
//}
},
// IZoomable ends
_getItemIndexAsync: function _getItemIndexAsync(findItem) {
if (!findItem.data) {
return WinJS.Promise.wrap(findItem.firstItemIndexHint);
}
var clusterKey = findItem.data.clusterKey;
var promises = [];
for (var i = 0; i < this._totalCount(); i++) {
promises.push(this._dataSource.fromIndex(i));
}
return WinJS.Promise.join(promises).then(function findMatchIndex(results) {
for (var index = 0; index < results.length; index++) {
if (results[index] && results[index].data && results[index].data.clusterKey === clusterKey) {
return index;
}
}
return findItem.firstItemIndexHint;
});
}
}),
});
})(WinJS);
/* File: CoordsNavigableView.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformCoordsNavigableViewInit() {
"use strict";
var NV = CommonJS.NavigableView;
WinJS.Namespace.define("CommonJS.NavigableView", {
/// <summary>
/// INavigableView implementation that depends on row/col based HTML element data augmentation
/// e.g. data-platform-cnv="{col:0, row:2}"
/// </summary>
CoordsNavigableView: WinJS.Class.define(
function coordsNavigableView_ctor(element, model) {
this._element = element || document.createElement("div");
this._model = model || this._buildModel(element);
var elementData = PlatformJS.Utilities.parseDataAttribute("data-platform-cnv", this.element);
if (elementData) {
this._numCols = elementData.numCols;
this._numRows = elementData.numRows;
}
},
{
_element: null,
_numCols: 0,
_numRows: 0,
_model: [],
_modelIndex: null,
// [BEGIN] INavigableView implementation
element: {
get: function get() {
return this._element;
}
},
isNavigableItem: function isNavigableItem(item) {
/// <summary>
/// Returns true if the given element is an item that should be navigated to with the keyboard.
/// </summary>
return item.getAttribute("data-platform-cnv");
},
getActiveItem: function getActiveItem() {
return this._element.querySelector("[tabIndex='0']");
},
getNavigableItems: function getNavigableItems() {
return this._element.querySelectorAll("[data-platform-cnv]");
},
getNextItem: function getNextItem(eventTarget, eventKeyCode) {
/// <summary>
/// Given an item and a keyCode, returns the next item that should get focus. 
/// </summary>
var currentItemData = PlatformJS.Utilities.parseDataAttribute("data-platform-cnv", eventTarget),
modelIndex = { col: currentItemData.col, row: currentItemData.row };
switch (eventKeyCode) {
case WinJS.Utilities.Key.upArrow: this._moveModelUp(modelIndex); break;
case WinJS.Utilities.Key.rightArrow: this._moveModelRight(modelIndex); break;
case WinJS.Utilities.Key.downArrow: this._moveModelDown(modelIndex); break;
case WinJS.Utilities.Key.leftArrow: this._moveModelLeft(modelIndex); break;
default: break;
}
return this._model[modelIndex.col][modelIndex.row];
},
// [END] INavigableView implementation
// Build a two-dimensional gridModel
_buildModel: function _buildModel(element) {
var model = [],
navigableItems = this.getNavigableItems();
// Fill model with Navigable items.
for (var i = 0, navigableItemsLength = navigableItems.length; i < navigableItemsLength; i++) {
var navigableItem = navigableItems[i],
navigableItemData = PlatformJS.Utilities.parseDataAttribute("data-platform-cnv", navigableItem);
if (navigableItemData) {
var col = navigableItemData.col,
row = navigableItemData.row;
if (!model[col]) {
model[col] = [];
}
model[col][row] = navigableItem;
}
}
return model;
},
_moveModelUp: function _moveModelUp(modelIndex) {
if (modelIndex.row === 0) {
NV.dispatchBoundaryEvent(this._element, NV.Direction.UP);
}
else {
modelIndex.row--;
}
},
_moveModelRight: function _moveModelRight(modelIndex) {
if (modelIndex.col === this._numCols - 1) {
NV.dispatchBoundaryEvent(this._element, NV.Direction.RIGHT);
}
else {
modelIndex.col++;
}
},
_moveModelDown: function _moveModelDown(modelIndex) {
if (modelIndex.row === this._numRows - 1) {
NV.dispatchBoundaryEvent(this._element, NV.Direction.DOWN);
} else {
modelIndex.row++;
}
},
_moveModelLeft: function _moveModelLeft(modelIndex) {
if (modelIndex.col === 0) {
NV.dispatchBoundaryEvent(this._element, NV.Direction.LEFT);
} else {
modelIndex.col--;
}
}
}
)
});
})();
/* File: NavigableView.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformsNavigableViewInit() {
"use strict";
var NV = WinJS.Namespace.define("CommonJS.NavigableView", {
_elementIds: 0,
ItemBounds: {
top: null,
right: null,
bottom: null,
left: null
},
ItemPointers: {
up: null,
right: null,
down: null,
left: null
},
ItemInfo: {
id: null,
bounds: null,
pointers: null
},
BoundaryEventDetail: {
direction: null
},
Direction: {
UP: "up",
RIGHT: "right",
DOWN: "down",
LEFT: "left"
},
createItemId: function createItemId() {
return "pnv_" + NV._elementIds++;
},
createItemInfo: function createItemInfo(id) {
return Object.create(NV.ItemInfo, {
id: {
value: id || NV.createItemId()
},
bounds: {
value: Object.create(NV.ItemBounds)
},
pointers: {
value: Object.create(NV.ItemPointers)
}
});
},
createItemInfos: function createItemInfos(elements) {
var itemInfos = [];
for (var elementKey in elements) {
if (elements[elementKey]) {
var element = elements[elementKey],
itemInfo = this.createItemInfo(element.id);
itemInfos[elementKey] = itemInfo;
}
}
return itemInfos;
},
getItemBounds: function getItemBounds(element) {
return Object.create(NV.ItemBounds, {
top: { value: element.offsetTop },
right: { value: element.offsetLeft + element.offsetWidth },
bottom: { value: element.offsetTop + element.offsetHeight },
left: { value: element.offsetLeft }
});
},
addBoundsMetaDataToItemInfos: function addBoundsMetaDataToItemInfos(itemInfos, elements) {
for (var key in itemInfos) {
var element = elements[key],
itemInfo = itemInfos[key];
if (itemInfo && element) {
var bounds = NV.getItemBounds(element);
itemInfo.bounds.top = bounds.top;
itemInfo.bounds.right = bounds.right;
itemInfo.bounds.bottom = bounds.bottom;
itemInfo.bounds.left = bounds.left;
}
}
},
addPointersMetaDataToItemInfos: function addPointersMetaDataToItemInfos(itemInfos, isLtr, isGridBased) {
for (var key in itemInfos) {
var itemInfo = itemInfos[key];
if (itemInfo && itemInfo.bounds) {
NV.addPointersMetaDataToItemInfo(itemInfo, itemInfos, isLtr, isGridBased);
}
}
},
addPointersMetaDataToItemInfo: function addPointersMetaDataToItemInfo(itemInfo, itemInfos, isLtr, isGridBased) {
var flipHorizontal = (!isLtr && isGridBased) ? true : false;
NV._addTopPointer(itemInfo, itemInfos, isLtr);
NV._addRightPointer(itemInfo, itemInfos, isLtr, flipHorizontal);
NV._addDownPointer(itemInfo, itemInfos, isLtr);
NV._addLeftPointer(itemInfo, itemInfos, isLtr, flipHorizontal);
},
addPnvDataAttributesToElements: function addPnvDataAttributesToElements(itemInfos, elements) {
for (var key in itemInfos) {
var element = elements[key],
itemInfo = itemInfos[key];
if (element && itemInfo && itemInfo.pointers) {
if (!element.id) {
element.id = itemInfo.id;
}
element.setAttribute("data-platform-pnv", JSON.stringify(itemInfo.pointers));
}
}
},
isItemAbove: function isItemAbove(srcBounds, isAboveBounds) {
return isAboveBounds.bottom <= srcBounds.top;
},
isItemToRight: function isItemToRight(srcBounds, isToRightBounds) {
return isToRightBounds.left >= srcBounds.right;
},
isItemBelow: function isItemBelow(srcBounds, isBelowBounds) {
return isBelowBounds.top >= srcBounds.bottom;
},
isItemToLeft: function isItemToLeft(srcBounds, isToLeftBounds) {
return isToLeftBounds.right <= srcBounds.left;
},
dispatchBoundaryEvent: function dispatchBoundaryEvent(element, direction) {
var event = document.createEvent("CustomEvent");
// Focus manager is listening for events on platformClusterContent
// Make sure we always trigger the events from this outer container
while (element && !WinJS.Utilities.hasClass(element, "platformClusterContent")) {
element = element.parentElement;
}
if (element) {
event.initCustomEvent("contentbounds", false, false, Object.create(NV.BoundaryEventDetail, { direction: { value: direction } }));
element.dispatchEvent(event);
}
},
// Conveniance function for creating bounds, pointers & HTML data in correct order
convertElementsToPnvItems: function convertElementsToPnvItems(itemInfos, siblingElements, isLtr, isGridBased) {
NV.addBoundsMetaDataToItemInfos(itemInfos, siblingElements);
NV.addPointersMetaDataToItemInfos(itemInfos, isLtr, isGridBased);
NV.addPnvDataAttributesToElements(itemInfos, siblingElements);
},
_addTopPointer: function _addTopPointer(itemInfo, itemInfos, isLtr) {
if (itemInfo.pointers.up === null) {
var itemAbove = NV._getItemAbove(itemInfo, itemInfos, isLtr);
if (itemAbove) {
itemInfo.pointers.up = itemAbove.id;
// Create link back
if (itemAbove.pointers.down === null) {
itemAbove.pointers.down = itemInfo.id;
}
}
}
},
_addRightPointer: function _addRightPointer(itemInfo, itemInfos, isLtr, horizontalFlip) {
if (itemInfo.pointers.right === null) {
var itemToRight = NV._getItemToRight(itemInfo, itemInfos, isLtr);
if (itemToRight) {
if (!horizontalFlip) {
itemInfo.pointers.right = itemToRight.id;
// Create link back
if (itemToRight.pointers.left === null) {
itemToRight.pointers.left = itemInfo.id;
}
} else {
itemInfo.pointers.left = itemToRight.id;
// Create link back
if (itemToRight.pointers.right === null) {
itemToRight.pointers.right = itemInfo.id;
}
}
}
}
},
_addDownPointer: function _addDownPointer(itemInfo, itemInfos, isLtr) {
if (itemInfo.pointers.down === null) {
var itemBelow = NV._getItemBelow(itemInfo, itemInfos, isLtr);
if (itemBelow) {
itemInfo.pointers.down = itemBelow.id;
// Create link back
if (itemBelow.pointers.up === null) {
itemBelow.pointers.up = itemInfo.id;
}
}
}
},
_addLeftPointer: function _addLeftPointer(itemInfo, itemInfos, isLtr, horizontalFlip) {
if (itemInfo.pointers.left === null) {
var itemToLeft = NV._getItemToLeft(itemInfo, itemInfos, isLtr);
if (itemToLeft) {
if (!horizontalFlip) {
itemInfo.pointers.left = itemToLeft.id;
// Create link back
if (itemToLeft.pointers.right === null) {
itemToLeft.pointers.right = itemInfo.id;
}
} else {
itemInfo.pointers.right = itemToLeft.id;
// Create link back
if (itemToLeft.pointers.left === null) {
itemToLeft.pointers.left = itemInfo.id;
}
}
}
}
},
_getItemAbove: function _getItemAbove(currentItemInfo, itemInfos, isLtr) {
var allItemsAbove = [],
currentItemBounds = currentItemInfo.bounds;
for (var i = 0, otherItemInfo, otherItemBounds, itemInfosLength = itemInfos.length; i < itemInfosLength; i++) {
otherItemInfo = itemInfos[i];
if (otherItemInfo && otherItemInfo !== currentItemInfo && !otherItemInfo.isAd) {
otherItemBounds = otherItemInfo.bounds;
if (NV.isItemAbove(currentItemBounds, otherItemBounds) && !NV.isItemToLeft(currentItemBounds, otherItemBounds) && !NV.isItemToRight(currentItemBounds, otherItemBounds)) {
allItemsAbove.push(otherItemInfo);
}
}
}
allItemsAbove.sort(function navigableView_allItemsAboveCompare(a, b) {
b.bounds.top - a.bounds.top;
});
return allItemsAbove[0] || null;
},
_getItemToRight: function _getItemToRight(currentItemInfo, itemInfos, isLtr) {
var allItemsToRight = [],
currentItemBounds = currentItemInfo.bounds;
for (var i = 0, otherItemInfo, otherItemBounds, itemInfosLength = itemInfos.length; i < itemInfosLength; i++) {
otherItemInfo = itemInfos[i];
if (otherItemInfo && otherItemInfo !== currentItemInfo && !otherItemInfo.isAd) {
otherItemBounds = otherItemInfo.bounds;
if (NV.isItemToRight(currentItemBounds, otherItemBounds) && !NV.isItemAbove(currentItemBounds, otherItemBounds) && !NV.isItemBelow(currentItemBounds, otherItemBounds)) {
allItemsToRight.push(otherItemInfo);
}
}
}
allItemsToRight.sort(function allItemsToRightCompareFunc(a, b) {
a.bounds.left - b.bounds.left;
});
return allItemsToRight[0] || null;
},
_getItemBelow: function _getItemBelow(currentItemInfo, itemInfos, isLtr) {
var allItemsBelow = [],
currentItemBounds = currentItemInfo.bounds;
for (var i = 0, otherItemInfo, otherItemBounds, itemInfosLength = itemInfos.length; i < itemInfosLength; i++) {
otherItemInfo = itemInfos[i];
if (otherItemInfo && otherItemInfo !== currentItemInfo && !otherItemInfo.isAd) {
otherItemBounds = otherItemInfo.bounds;
if (NV.isItemBelow(currentItemBounds, otherItemBounds) && !NV.isItemToLeft(currentItemBounds, otherItemBounds) && !NV.isItemToRight(currentItemBounds, otherItemBounds)) {
allItemsBelow.push(otherItemInfo);
}
}
}
allItemsBelow.sort(function allItemsBelowCompareFunc(a, b) {
a.bounds.top - b.bounds.top;
});
return allItemsBelow[0] || null;
},
_getItemToLeft: function _getItemToLeft(currentItemInfo, itemInfos, isLtr) {
var allItemsToLeft = [],
currentItemBounds = currentItemInfo.bounds;
for (var i = 0, otherItemInfo, otherItemBounds, itemInfosLength = itemInfos.length; i < itemInfosLength; i++) {
otherItemInfo = itemInfos[i];
if (otherItemInfo && otherItemInfo !== currentItemInfo && !otherItemInfo.isAd) {
otherItemBounds = otherItemInfo.bounds;
if (NV.isItemToLeft(currentItemBounds, otherItemBounds) && !NV.isItemAbove(currentItemBounds, otherItemBounds) && !NV.isItemBelow(currentItemBounds, otherItemBounds)) {
allItemsToLeft.push(otherItemInfo);
}
}
}
allItemsToLeft.sort(function allItemsToLeftCompareFunc(a, b) {
a.bounds.left - b.bounds.left;
});
if (isLtr) {
allItemsToLeft.reverse();
}
return allItemsToLeft[0] || null;
},
NavigableViewOrchestrator: WinJS.Class.define(function _NavigableViewOrchestrator_ctor(element, navigableChildren) {
this._navigableView = new CommonJS.NavigableView.PointerNavigableView(element, navigableChildren);
this._focusManager = new CommonJS.NavigableView.RovingTabIndexManager(this._navigableView),
this._orientationManager = new CommonJS.NavigableView.NavigableViewOrientationManager();
this._bindings.onNavigableViewActivated = this._onNavigableViewActivated.bind(this);
this._navigableView.addEventListener("activated", this._bindings.onNavigableViewActivated);
},
{
// PRIVATE
_bindings: {
onNavigableViewActivated: null
},
_navigableView: null, // PointerNavigableView or MatrixNavigableView
_focusManager: null, // RovingTabIndexManager or ActiveDescendentManager
_orientationManager: null,
_onNavigableViewActivated: function _onNavigableViewActivated() {
this._render();
},
_render: function _render(itemInfos) {
if (!this._navigableView) {
return;
}
var elements = this._navigableView.getNavigableItems();
if (itemInfos) {
CommonJS.NavigableView.addPnvDataAttributesToElements(itemInfos, elements);
}
else {
itemInfos = CommonJS.NavigableView.createItemInfos(elements);
CommonJS.NavigableView.convertElementsToPnvItems(itemInfos, elements);
this._orientationManager.itemInfos = itemInfos;
}                             
},
// PUBLIC
navigableView: {
get: function get() {
return this._navigableView;
}
},
onOrientationChange: function onOrientationChange(event) {
if (this._navigableView && this._navigableView.isActivated) {
var itemInfos = this._orientationManager.itemInfos;
this._render(itemInfos);
}
},
onWindowResize: function onWindowResize(event) {
if (this._navigableView && this._navigableView.isActivated) {
this._orientationManager.reset();
this._render();
}
},
dispose: function dispose() {
if (this._navigableView) {
this._navigableView.removeEventListener("activated", this._bindings.onNavigableViewActivated);
}
this._navigableView = null;
this._focusManager.dispose();
this._focusManager = null;
this._orientationManager = null;             
}
},
{
// STATIC MEMBERS
})
});
})();
/* File: PointerNavigableView.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformPointerNavigableViewInit() {
"use strict";
var NV = CommonJS.NavigableView;
WinJS.Namespace.define("CommonJS.NavigableView", {
/// <summary>
/// INavigableView implementation that depends on pointer based HTML element data augmentation
/// e.g. data-platform-pnv="{up:'elementId', right:'elemendId', down:'elementId', left:'elementId'}"
/// </summary>
PointerNavigableView: WinJS.Class.define(
function pointerNavigableView_ctor(element, navigableChildren) {
this._element = element;
this._navigableItems = navigableChildren;
},
{
// [BEGIN] INavigableView implementation
_element: null,
_navigableItems: null,
isActivated: false,
element: {
get: function get() {
return this._element;
}
},
activate: function activate(){
this.isActivated = true;
this.dispatchEvent("activated");
},
isNavigableItem: function isNavigableItem(item) {
/// <summary>
/// Returns true if the given element is an item that should be navigated to with the keyboard.
/// </summary>
return item.getAttribute("data-platform-pnv");
},
getActiveItem: function getActiveItem() {
return this._element.querySelector("div[tabIndex='0']");
},
getNavigableItems: function getNavigableItems() {
return this._navigableItems;
},
getNextItem: function getNextItem(eventTarget, eventKeyCode) {
/// <summary>
/// Given an item and a keyCode, returns the next item that should get focus. 
/// Returning null means there is no next item to get focus. 
/// </summary>
var nextItem = null;
if (PlatformJS.Utilities.isCursorKey(event.keyCode)) {
if (this.isActivated === false) {
this.activate();
}
var pointerData = PlatformJS.Utilities.parseDataAttribute("data-platform-pnv", eventTarget);
switch (eventKeyCode) {
case WinJS.Utilities.Key.upArrow:
nextItem = this._onArrow(pointerData, NV.Direction.UP);
this._setPointer(nextItem, NV.Direction.DOWN, eventTarget.id);
break;
case WinJS.Utilities.Key.rightArrow:
nextItem = this._onArrow(pointerData, NV.Direction.RIGHT);
this._setPointer(nextItem, NV.Direction.LEFT, eventTarget.id);
break;
case WinJS.Utilities.Key.downArrow:
nextItem = this._onArrow(pointerData, NV.Direction.DOWN);
this._setPointer(nextItem, NV.Direction.UP, eventTarget.id);
break;
case WinJS.Utilities.Key.leftArrow:
nextItem = this._onArrow(pointerData, NV.Direction.LEFT);
this._setPointer(nextItem, NV.Direction.RIGHT, eventTarget.id);
break;
default: break;
}
}
return nextItem;
},
// [END] INavigableView implementation
_setPointer: function _setPointer(item, direction, id) {
if (item) {
var pointerData = PlatformJS.Utilities.parseDataAttribute("data-platform-pnv", item);
pointerData[direction] = id;
item.setAttribute("data-platform-pnv", JSON.stringify(pointerData));
}
},
_onArrow: function _onArrow(pointerData, direction) {
var nextItem = null;
if (pointerData !== null) {
var childId = pointerData[direction];
nextItem = document.getElementById(childId) || null; //convert possible undefined to null
}
if (nextItem === null) {
NV.dispatchBoundaryEvent(this._element, direction);
}
return nextItem;
}
}
)
});
WinJS.Class.mix(CommonJS.NavigableView.PointerNavigableView, WinJS.Utilities.eventMixin);
WinJS.Class.mix(CommonJS.NavigableView.PointerNavigableView, WinJS.Utilities.createEventProperties("activated"));
})();
/* File: PointerNavigableViewOrientationManager.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformPointerNavigableViewOrientationManagerInit() {
"use strict";
var NV = CommonJS.NavigableView;
WinJS.Namespace.define("CommonJS.NavigableView", {
// DEPRECATED
PointerNavigableViewOrientationManager: WinJS.Class.define(
function pointerNavigableViewOrientationManager_ctor(navigableView, initialItemInfos, elements, isAssociativeElementArray) {
this._navigableView = navigableView;
//Prime cache with initial orientation infos
this._itemInfos = {
orientation1: initialItemInfos,
orientation2: null
},
this._elements = elements;
// ItemsContainer builds it's elements array with handles rather than indexes
// so we need to support that
this._isAssociativeElementArray = isAssociativeElementArray || false;
},
{
_elements: null,
// Cache for the two different orientations
_itemInfos: {
orientation1: null,
orientation2: null
},
// Orientation 1 is the initial state
_isOrientation1: true,
toggleOrientation: function toggleOrientation() {
if (this._isOrientation1 === true) {
// Calculate infos for alternative orientation
// This will only run the first time we change orientation, after that we use the cached values
if (!this._itemInfos.orientation2) {
var isLtr = window.getComputedStyle(document.body).direction !== "rtl";
this._itemInfos.orientation2 = NV.createItemInfos(this._elements, this._isAssociativeElementArray);
NV.addBoundsMetaDataToItemInfos(this._itemInfos.orientation2, this._elements);
NV.addPointersMetaDataToItemInfos(this._itemInfos.orientation2, isLtr);
}
NV.addPnvDataAttributesToElements(this._itemInfos.orientation2, this._elements);
this._isOrientation1 = false;
}
else {
NV.addPnvDataAttributesToElements(this._itemInfos.orientation1, this._elements);
this._isOrientation1 = true;
}
}
}
),
NavigableViewOrientationManager: WinJS.Class.define(
function navigableViewOrientationManager_ctor() {
this._landscapeItemInfos = null;
this._portraitItemInfos = null;
},
{
// PRIVATE
_landscapeItemInfos: null,
_portraitItemInfos: null,
_isLandscape: function _isLandscape() {
var orientation = Windows.UI.ViewManagement.ApplicationView.value;
return orientation === Windows.UI.ViewManagement.ApplicationViewState.fullScreenLandscape || orientation === Windows.UI.ViewManagement.ApplicationViewState.filled;
},
// PUBLIC
itemInfos: { 
get: function get() {
return (this._isLandscape() === true) ? this._landscapeItemInfos : this._portraitItemInfos;
},
set: function set(itemInfos){
if (this._isLandscape()) {
this._landscapeItemInfos = itemInfos;
}
else {
this._portraitItemInfos = itemInfos;
}
}
},
reset: function reset() {
this._landscapeItemInfos = null;
this._portraitItemInfos = null;
}
}
)
});
})();
/* File: RovingTabIndexManager.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformRovingTabIndexManagerInit() {
"use strict";
var NV = CommonJS.NavigableView;
WinJS.Namespace.define("CommonJS.NavigableView", {
RovingTabIndexManager: WinJS.Class.define(
function rovingTabIndexManager_ctor(navigableView) {
this._navigableView = navigableView;
this._bindings.onKeyDown = this._onKeyDown.bind(this);
if (this._navigableView) {
var navigableItems = this._navigableView.getNavigableItems(),
activeItem = this._navigableView.getActiveItem() || null;
if (activeItem) {
this._disableNavigableItems(navigableItems, activeItem);
this._navigableView.element.addEventListener("keydown", this._bindings.onKeyDown);
}
}
},
{
_bindings: {
onKeyDown: null
},
_navigableView: null,
_disableNavigableItems: function _disableNavigableItems(navigableItems, activeItem) {
for (var key in navigableItems) {
var navigableItem = navigableItems[key];
if (activeItem !== navigableItem) {
navigableItem.tabIndex = -2; // we'll use -2 for temp disabled tab indexes
}
};
},
_onKeyDown: function _onKeyDown(event) {
if (!event.altKey && PlatformJS.Utilities.isCursorKey(event.keyCode)) {
event.stopImmediatePropagation();
event.preventDefault();
var currentItem = event.target,
nextItem = this._navigableView && this._navigableView.getNextItem(currentItem, event.keyCode);
if (nextItem) {
currentItem.tabIndex = -2;
nextItem.tabIndex = 0;
CommonJS.Immersive.PanoramaFocusManager.animationToContent(nextItem);
// TODO: other modules be interested in tabindex changes (e.g. DateSelector)
// var event = document.createEvent("CustomEvent");
// event.initCustomEvent("rovingtabchange", false, false, { activeItem: nextItem });
// navigableView.element.dispatchEvent(event);
}
}
},
dispose: function dispose() {
if (this._navigableView) {
this._navigableView.element.removeEventListener("keydown", this._bindings.onKeyDown);
}
this._bindings.onKeyDown = null;
}
}
)
});
})();
/* File: SequentialNavigableView.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function appexPlatformSequentialNavigableViewInit() {
"use strict";
var NV = CommonJS.NavigableView;
WinJS.Namespace.define("CommonJS.NavigableView", {
/// <summary>
/// Simple INavigableView implementation that uses left/up to go back in dom order, and right/down to go forward in dom order.
/// </summary>
SequentialNavigableView: WinJS.Class.define(
function sequentialNavigableView_ctor(element) {
this.element = element;
NV._elementIds++;
},
{
_element: null,
// [BEGIN] INavigableView implementation
element: {
get: function get() {
return this._element;
},
set: function set(newElement) {
this._element = newElement;
}
},
getNavigableItems: function getNavigableItems() {
var navigableItems = [];
for (var i = 0, childrenLength = this._element.children.length; i < childrenLength; i++) {
var child = this._element.children[i];
if (this.isNavigableItem(child)) {
navigableItems.push(child);
}
}
return navigableItems;
},
isNavigableItem: function isNavigableItem(item) {
/// <summary>
/// Returns true if the given element is an item that should be navigated to with the keyboard.
/// </summary>
return WinJS.Utilities.hasClass(item, "platformClusterItem");
},
getNextItem: function getNextItem(item, keyCode) {
/// <summary>
/// Given an item and a keyCode, returns the next item that should get focus. 
/// Returning null means there is no next item to get focus. 
/// Returning undefined means the keyCode is not relevant to navigation and it should not be handled.
/// </summary>
var nextItem;
var rtl = (document.dir === "rtl");
switch (keyCode) {
case WinJS.Utilities.Key.upArrow:
nextItem = item.previousSibling;
break;
case WinJS.Utilities.Key.downArrow:
nextItem = item.nextSibling;
break;
case WinJS.Utilities.Key.leftArrow:
nextItem = rtl ? item.nextSibling : item.previousSibling;
break;
case WinJS.Utilities.Key.rightArrow:
nextItem = rtl ? item.previousSibling : item.nextSibling;
break;
default:
break;
}
// Recurse until the next Navigable element (or null/undefined) is found.
if (nextItem && !this.isNavigableItem(nextItem)) {
nextItem = this.getNextItem(nextItem, keyCode);
}
return nextItem;
}
// [END] INavigableView implementation
}
)
});
})();
/* File: SeachDialog.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
var DISABLE_ANIMATION = true;
(function appexPlatformControlsSearchDialogInit() {
"use strict";
var NS = WinJS.Namespace.define("CommonJS", {
SearchDialog: WinJS.Class.define(
function searchDialog_ctor(element, options) {
var that = this;
this.element = element ? element : document.createElement("div");
WinJS.Utilities.addClass(this.element, "platformSearchDialog");
this._setupWidthMaintenance();
this._createSearchBox();
this._createStaticContents();
this._createButtons();
this._constructDialog();
this.autoSuggest = new CommonJS.AutoSuggest(this._searchBox);
this.onCloseButtonClick = function () {
that.searchVisible = false;
};
WinJS.UI.setOptions(this, options);
},
{
//#region Private Variables
_prompt: null,
_searchBox: null,
_progress: null,
_message: null,
_closeButton: null,
_addButton: null,
_isVisible: false,
_flyout: null,
_ignoreNextEnter: false,
_appViewType: 0,
//#endregion Private Variables
//#region Public Variables Getters and Setters
element: null,
autoSuggest: null,
flyout: {
set: function set(value) {
var that = this;
this._flyout = value;
value.onHide = function () {
window.removeEventListener("resize", that._orientationChange);
that.autoSuggest.hide();
if (that.onHide) {
that.onHide();
}
};
},
get: function get(value) {
return this._flyout;
}
},
promptText: {
set: function set(value) {
this._prompt.textContent = value;
}
},
closeButtonText: {
set: function set(value) {
this._closeButton.textContent = value;
}
},
message: {
set: function set(value) {
this._message.innerText = value;
}
},
searchVisible: {
set: function set(value) {
if (this._isVisible === value) {
return;
}
this._isVisible = value;
if (!value) {
PlatformJS.Navigation.mainNavigator.enableSearchOnType();
this.flyout.hide();
} else {
// disable search on keyboardinput
PlatformJS.Navigation.mainNavigator.disableSearchOnType();
this.flyout.show();
}
},
get: function get() {
return this._isVisible;
}
},
extraButtons: {
set: function set(value) {
for (var i = 0, len = value.length; i < len; i++) {
var button = value[i];
if (button && button.icon && button.clickHandler) {
var theme = button.theme || "light";
var title = button.title || "untitled";
this._addExtraButton(button.icon, button.clickHandler, button.pressedIcon, theme, title);
}
}
}
},
autoSuggestOptions: {
set: function set(value) {
WinJS.UI.setOptions(this.autoSuggest, value);
}
},
searchPlaceholderText: {
set: function set(value) {
this._overlay.innerText = value;
}
},
maxLength: {
set: function set(value) {
this._searchBox.maxLength = value;
}
},
searchBoxText: {
set: function set(value) {
this._searchBox.value = value;
},
get: function get(value) {
return this._searchBox.value;
}
},
searchSuggestions: {
set: function set(value) {
this.autoSuggest.itemDataSource = value.suggestions;
},
get: function get() {
return this.autoSuggest._itemDataSource;
}
},
//#endregion Public Variables Getters and Setters
//#region Public Events/Eventhandlers
onHide: null,
onCloseButtonClick: {
set: function set(value) {
this._closeButton.onclick = value;
},
get: function get(value) {
return this._closeButton.onclick;
}
},
onSearchTextEntered: {
set: function set(value) {
var that = this;
this.autoSuggest.onItemSelection = function (index, event) {
that._addButton.disabled = true; // Disable ADD button to prevent next click
value({ queryText: that.searchBoxText, src: that, index: index }, that);
if (event && event.key === "Enter") {
that._ignoreNextEnter = true;
}
};
this._searchBox.addEventListener("keydown", function searchDialog_onKeyDown(event) {
if (!that._ignoreNextEnter && event.key === "Enter" && !that.autoSuggest._inIME) {
value({ queryText: that.searchBoxText, src: that }, that);
}
that._ignoreNextEnter = false;
});
this._addButton.addEventListener("click", function searchDialog_onClick() {
var evt = document.createEvent("KeyboardEvent");
evt.initKeyboardEvent("keydown", true, true, window, "Enter", 0, "", false, "");
that._searchBox.dispatchEvent(evt);
that._addButton.disabled = true; // Disable ADD button to prevent next click
});
}
},
onSearchSuggestionsRequested: {
set: function set(value) {
var that = this;
this.autoSuggest.keyup = function (event) {
event.queryText = that.searchBoxText;
// Create a request object to emulate Search Contract
event.request = {
getDeferral: function getDeferral() {
return {
complete: function complete() { }
};
}
};
value(event, that);
};
}
},
//#endregion Public Events/Eventhandlers
//#region Public Functions
showSearchProgress: function showSearchProgress() {
/// <summary> Show progress </summary>
WinJS.Utilities.removeClass(this._progress, "platformHide");
},
hideSearchProgress: function hideSearchProgress() {
/// <summary> Hide progress </summary>
WinJS.Utilities.addClass(this._progress, "platformHide");
},
//#endregion Public Functions
//#region Private Functions
_setupWidthMaintenance: function _setupWidthMaintenance() {
var that = this;
this.element.style.width = (document.body.clientWidth - CommonJS.SearchDialog._edgeOffset) + "px"; //NOTE: could not apply full clientWidth to an absolute positioned div, will cause incorrect window shifting when using key up/down
this._appViewType = Windows.UI.ViewManagement.ApplicationView.value;
this._orientationChange = function (event) {
if (that._appViewType !== Windows.UI.ViewManagement.ApplicationView.value) {
that._appViewType = Windows.UI.ViewManagement.ApplicationView.value;
that.element.style.width = (document.body.clientWidth - CommonJS.SearchDialog._edgeOffset) + "px";
}
};
},
_createSearchBox: function _createSearchBox() {
var that = this;
this._searchBox = document.createElement("input");
this._searchBox.id = "SearchTextBox";
this._searchBox.type = "text";
this._searchBox.addEventListener("focus", function searchDialog_onFocus() {
that.message = "";
that._addButton.disabled = that.searchBoxText.length === 0; // Enable ADD button when searchbox has some value
});
this._searchBox.addEventListener("input", function searchDialog_onInput() {
that._addButton.disabled = that.searchBoxText.length === 0;
if (this.value.length !== 0) {
that._overlay.style.visibility = "hidden";
}
else {
that._overlay.style.visibility = "visible";
}
});
this._searchBox.addEventListener("keydown", function searchDialog_onKeyDown(event) {
if (event.shiftKey && event.keyCode === 9) { //"tab" keyCode is 9
that._setFocusToTextBox();
}
});
WinJS.Utilities.addClass(this._searchBox, "platformSearchDialogSearchBox");
},
_createStaticContents: function _createStaticContents() {
this._message = document.createElement("div");
WinJS.Utilities.addClass(this._message, "platformSearchDialogMessage");
this._message.setAttribute("aria-label", CommonJS.resourceLoader.getString("/platform/progressAlertAriaLabel"));
this._message.setAttribute("role", "alert");
this._message.setAttribute("aria-atomic", "true");
this._progress = document.createElement("progress");
WinJS.Utilities.addClass(this._progress, "platformSearchDialogProgress platformHide");
this._prompt = document.createElement("div");
WinJS.Utilities.addClass(this._prompt, "platformSearchDialogPrompt");
//overlay div to support placeholder text with focus
this._overlay = document.createElement("div");
WinJS.Utilities.addClass(this._overlay, "platformSearchDialogOverlayDiv");
this._overlay.id = "SearchDialog_texboxOverlay";
},
_createButtons: function _createButtons() {
this._extraButtonContainer = document.createElement("div");
WinJS.Utilities.addClass(this._extraButtonContainer, "platformSearchDialogButtonContainer");
this._closeButton = document.createElement("button");
this._closeButton.setAttribute("role", "button");
this._closeButton.id = "SearchDialog_closeButton";
WinJS.Utilities.addClass(this._closeButton, "platformSearchButton rectUIBtnDark platformSearchDialogClose");
this._closeButton.textContent = CommonJS.resourceLoader.getString("/platform/cancel");
this._closeButton.addEventListener("keydown", this._setFocusToTextBox.bind(this)); //loop tab-focus inside SearchDialog
this._addButton = document.createElement("button");
this._addButton.setAttribute("role", "button");
this._addButton.id = "SearchDialog_addButton";
WinJS.Utilities.addClass(this._addButton, "platformSearchButton rectUIBtnDark platformSearchDialogAdd");
this._addButton.textContent = CommonJS.resourceLoader.getString("/platform/add");
this._addButton.disabled = true;
},
_constructDialog: function _constructDialog() {
var searchBoxContainer = document.createElement("div");
WinJS.Utilities.addClass(searchBoxContainer, "platformSearchDialogSearchBoxContainer");
searchBoxContainer.appendChild(this._searchBox);
searchBoxContainer.appendChild(this._overlay);
var mainRowContainer = document.createElement("div");
WinJS.Utilities.addClass(mainRowContainer, "platformSearchDialogMainRow");
mainRowContainer.appendChild(searchBoxContainer);
mainRowContainer.appendChild(this._extraButtonContainer);
mainRowContainer.appendChild(this._addButton);
mainRowContainer.appendChild(this._closeButton);
this.element.appendChild(this._prompt);
this.element.appendChild(mainRowContainer);
this.element.appendChild(this._progress);
this.element.appendChild(this._message);
},
_setFocusToTextBox: function _setFocusToTextBox() {
var that = this;
setTimeout(function searchDialog_setFocusTextBoxTimeout() {
that._searchBox.focus();
}, 50);
},
_addExtraButton: function _addExtraButton(icon, clickHandler, pressedIcon, theme, title) {
/// <summary>Add an extra button</summary>
/// <param name="icon">The icon of the image. This can be one of the values from WinJS.UI.AppBarIcon, a url(""), or a CSS class</param>
/// <param name="clickHandler">The click handler function</param>
var button;
if (pressedIcon) {
button = new CommonJS.Button(null, { icon: icon, theme: theme, title: title, pressedIcon: pressedIcon, onclick: clickHandler });
} else {
button = new CommonJS.Button(null, { icon: icon, theme: theme, title: title, onclick: clickHandler });
}
this._extraButtonContainer.appendChild(button.element);
}
//#endregion Private Functions
},
{
_edgeOffset: 5, // private static variable
show: function show(searchDialogOptions) { 
var domElement = document.createElement("div");
WinJS.Utilities.addClass(domElement, "platformSearchDialogFlyout");
var flyout = new CommonJS.Flyout(domElement, {
closeButtonHidden: true
});
var searchDialog = new CommonJS.SearchDialog(flyout._flyoutContent, searchDialogOptions);
searchDialog.flyout = flyout;
searchDialog.message = "";
searchDialog.searchVisible = true;
window.addEventListener("resize", searchDialog._orientationChange);
searchDialog._setFocusToTextBox();
return searchDialog;
}
})
});
})();
/* File: TempClusterDataProvider.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("CommonJS",
{
TempClusterDataProvider: WinJS.Class.define(function (options) {
WinJS.UI.setOptions(this, options);
},
{
_gettingStartedCache: "GettingStartedCache",
useLocalCache: false,
fetchData: function fetchData(dataSourceId) {
var that = this;
return new WinJS.Promise(function (complete, error, progress) {
if (that.useLocalCache) {
var fileName = that._getCacheFileName();
that.getCachedData(fileName).then(function (response) {
if (response) {
progress(response);
}
});
}
that._downloadData(dataSourceId, complete);
});
},
_downloadData: function _downloadData(dataSourceId, complete) {
var queryService = new PlatformJS.DataService.QueryService(dataSourceId);
var queryParams = PlatformJS.Collections.createStringDictionary();
var market = Platform.Utilities.Globalization.getCurrentMarket();
queryParams.insert("market", market);
var options = new Platform.DataServices.QueryServiceOptions();
options.useCompatDataModel = true;
queryService.downloadDataAsync(queryParams, null, null, options).then(function gsDownloadSuccess(data) {
var result = null;
if (data) {
try {
result = JSON.parse(data.dataString);
} catch (exp) {
console.log("Invalid json response for GettingStarted Cluster.");
}
}
return result;
},
function (error) {
return null;
})
.then(function (result) {
complete(result);
});
},
getCachedData: function getCachedData(fileName) {
return WinJS.Application.local.readText(fileName, null).then(function (dataString) {
try{
return JSON.parse(dataString);
} catch (exp) {
return null;
}
});
},
saveToCache: function saveToCache(fileName, dataObject) {
if (fileName && dataObject) {
var dataString = JSON.stringify(dataObject);
return WinJS.Application.local.writeText(fileName, dataString);
}
},
// Parse server response for 'getting started' cluster.
parseData: function parseData(response) {
var result = {};
if (response
&& response.clusters
&& response.clusters.length > 0) {
var cluster = response.clusters[0];
if (cluster.entityList
&& cluster.entityList.entities
&& cluster.entityList.entities.length > 0) {
var entities = cluster.entityList.entities;
result.clusterTitle = cluster.entityList.categoryName;
result.data = {};
// Set the header text
if (entities[0].content && entities[0].content.abstract) {
result.data.messageText = this._stripHTML(entities[0].content.abstract);
}
result.data.contentSlots = [];
for (var index = 1, len = entities.length; index < len; index++) {
var entity = entities[index];
if (entity && entity.content) {
var slot = {};
// header and message text for each content slot
slot.slotHeaderText = this._stripHTML(entity.content.headline);
slot.slotText = this._stripHTML(entity.content.abstract);
// parse icon char.
var icon = this._parseIcon(entity.content.byline);
slot.icon = icon;
slot.isAppExSymbolChar = icon !== false;
// the destination url
slot.destination = entity.destination;
// parse action. Used to uniquely identify the command and send via telemetry
if (entity.content.source && entity.content.source.name) {
slot.actionId = entity.content.source.name;
}
result.data.contentSlots.push(slot);
}
}
}
}
if (this.isGettingStartedDataValid(result)) {
if (this.useLocalCache) {
var fileName = this._getCacheFileName();
this.saveToCache(fileName, result).then();
}
}
else {
result = {};
}
return result;
},
_stripHTML: function _stripHTML(string) {
if (string) {
return string.replace(/<(?:.|\n)*?>/gm, "");
}
else {
return "";
}
},
_getCacheFileName: function _getCacheFileName() {
var market = Platform.Utilities.Globalization.getCurrentMarket();
return this._gettingStartedCache + "_" + market;
},
_parseIcon: function _parseIcon(iconString) {
if (iconString) {
try {
return "&#" + parseInt(iconString, 16) + ";";
} catch (exp) {
}
}
return false;
},
isGettingStartedDataValid: function isGettingStartedDataValid(clusterData) {
var result = false;
if (clusterData
&& clusterData.data
&& clusterData.data.contentSlots
&& clusterData.data.contentSlots.length > CommonJS.TempClusterDataProvider._minContentCount
&& clusterData.clusterTitle) {
result = true;
}
return result;
},
// Parse server response for 'next steps' cluster.
parseNextStepsData: function parseNextStepsData(response) {
var result = {};
if (response
&& response.clusters
&& response.clusters.length > 0) {
var cluster = response.clusters[0];
if (cluster.entityList
&& cluster.entityList.entities
&& cluster.entityList.entities.length > 0) {
var entities = cluster.entityList.entities;
result.clusterTitle = cluster.entityList.categoryName;
result.data = {};
result.data.contentSlots = [];
for (var index = 0, len = entities.length; index < len; index++) {
var entity = entities[index];
if (entity && entity.content) {
var slot = {};
// text for each content slot
slot.slotHeaderText = this._stripHTML(entity.content.headline);
// the destination url
slot.destination = entity.destination;
// parse the icon char.
var icon = this._parseIcon(entity.content.byline);
if (icon) {
slot.icon = icon;
slot.isAppExSymbolChar = icon !== false;
}
// parse action. Used to uniquely identify the command and sent via telemetry
if (entity.content.source && entity.content.source.name) {
slot.actionId = entity.content.source.name;
}
result.data.contentSlots.push(slot);
}
}
}
}
return result;
}
},
{
// Minimum required content slots
_minContentCount: 1,
})
});
})();
/* File: TempCluster.js */
/********************************************************
* *
* © Microsoft. All rights reserved. *
* *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("CommonJS",
{
TempCluster: WinJS.Class.define(function (domElement, options) {
this._domElement = domElement || document.createElement("div");
// set options
WinJS.UI.setOptions(this, options);
this._updateMessageData();
this._rendererMap = {
// 'Getting Started' new template renderer
gsTemplate: {
renderer: this._gsTemplateRenderer.bind(this),
clusterName: "Getting Started",
navMethodName: "Getting Started Cluster"
},
// 'Next Steps' template renderer
nsTemplate: {
renderer: this._nsTemplateRenderer.bind(this),
clusterName: "Next Steps",
navMethodName: "Next Steps Cluster"
}
};
},
{
_domElement: null,
_removeCluster: null,
_dismissAction: "dismiss",
_renderPromise: null,
_innerPromiseList: null,
// the following fields will be provided by the client.
// uniquely identifies the cluster in pano.
clusterKey: null,
// list of clusters.
clusterList: null,
// The cluster data.
messageData: null,
// The template to render. One of the keys from this._rendererMap.
templateName: "",
// The css class to be applied to the cluster. This is used by 'next steps' cluster to paint a full background.
clusterClass: "",
// The click handler to dismiss the cluster.
removeClusterHandler: {
set: function set(value) {
this._removeCluster = value;
}
},
render: function render() {
var that = this;
var promise = WinJS.Promise.wrap(null);
var config = that._rendererMap[that.templateName];
if (config && config.renderer) {
promise = config.renderer();
}
return promise;
},
_gsTemplateRenderer: function _gsTemplateRenderer() {
return this._renderTemplate("templateGSMessageHost", "slotIcon win-commandring win-commandicon");
},
_nsTemplateRenderer: function _nsTemplateRenderer() {
return this._renderTemplate("templateNSMessageHost", "slotIcon");
},
_renderTemplate: function _renderTemplate(containerClass, iconClass){
var that = this;
return new WinJS.Promise(function TempClusterRenderComplete(complete) {
var host = that._createGSHost(containerClass, that.messageData);
var contentBlock = host.querySelector(".contentBlock");
var contentSlotDataList = that.messageData.contentSlots;
if (contentSlotDataList) {
for (var index = 0, len = contentSlotDataList.length; index < len; index++) {
var contentData = contentSlotDataList[index];
var contentElement = that._createContentSlot(contentData, iconClass, index);
contentBlock.appendChild(contentElement);
}
}
if (that.clusterClass) {
// set additional class on the cluster. (hacky way)
// This is used by next steps cluster, to render full background.
var clusterNode = that._domElement.parentNode;
if (WinJS.Utilities.hasClass(clusterNode, "platformCluster")) {
WinJS.Utilities.addClass(clusterNode, that.clusterClass);
}
}
that._domElement.appendChild(host);
complete();
});
},
_createGSHost: function _createGSHost(containerClassName, data){
var addClass = WinJS.Utilities.addClass;
var host = document.createElement("div");
addClass(host, containerClassName);
var messageContainer = document.createElement("div");
addClass(messageContainer, "messageContainer");
host.appendChild(messageContainer);
if (data.messageText) {
var headerBlock = document.createElement("div");
addClass(headerBlock, "headerBlock");
messageContainer.appendChild(headerBlock);
var headerMessage = document.createElement("div");
addClass(headerMessage, "headerMessage");
headerMessage.innerText = data.messageText;
headerBlock.appendChild(headerMessage);
}
var contentBlock = document.createElement("div");
addClass(contentBlock, "contentBlock");
messageContainer.appendChild(contentBlock);
return host;
},
_createContentSlot: function _createContentSlot(data, iconClass, index){
var addClass = WinJS.Utilities.addClass;
var slotContainer = document.createElement("div");
addClass(slotContainer, "slotContainer");
slotContainer.style.msGridRow = index + 1;
var borderElement = document.createElement("div");
addClass(borderElement, "borderElement");
slotContainer.appendChild(borderElement);
var slotContent = document.createElement("div");
addClass(slotContent, "slotContent");
slotContainer.appendChild(slotContent);
if (index === 0) {
slotContent.tabIndex = 0;
}
this._setClickHandler(slotContainer, data, index);
var slotIconText = document.createElement("div");
addClass(slotIconText, "slotIconText");
slotContent.appendChild(slotIconText);
// create the icon
if (data.icon) {
var slotIcon = document.createElement("div");
addClass(slotIcon, iconClass);
slotIconText.appendChild(slotIcon);
var span = document.createElement("span");
slotIcon.appendChild(span);
if (data.isAppExSymbolChar) {
span.innerHTML = data.icon;
}
else {
WinJS.Utilities.addClass(span, data.icon);
}
}
var slotHeader = document.createElement("div");
addClass(slotHeader, "slotHeader");
slotHeader.innerText = data.slotHeaderText;
slotIconText.appendChild(slotHeader);
if (data.slotText) {
var slotText = document.createElement("div");
addClass(slotText, "slotText");
slotText.innerText = data.slotText;
slotIconText.appendChild(slotText);
}
return slotContainer;
},
_setClickHandler: function _setClickHandler(slotContainer, slotData, index) {
var that = this;
// enable pointer updown animation
var slotContent = WinJS.Utilities.query(".slotContent", slotContainer)[0];
PlatformJS.Utilities.enablePointerUpDownAnimations(slotContent);
// enable click handler
slotContent.onclick = function slotClickHandler() {
that._performAction(slotData, index);
};
slotContent.addEventListener("keydown", function slotContentKeyDown(event) {
var winjsKey = WinJS.Utilities.Key;
switch (event.keyCode) {
case winjsKey.enter:
case winjsKey.space:
that._performAction(slotData, index);
break;
case winjsKey.upArrow:
case winjsKey.downArrow:
that._handleArrowKeys(event);
break;
case winjsKey.leftArrow:
case winjsKey.rightArrow:
that._dispatchBoundaryEvent(event);
break;
}
});
},
_handleArrowKeys: function _handleArrowKeys(event) {
var currentNode = event.currentTarget;
var direction = event.keyCode;
var hasClass = WinJS.Utilities.hasClass;
var slotContainer = currentNode.parentNode;
if (direction === WinJS.Utilities.Key.upArrow) {
slotContainer = slotContainer.previousSibling;
while (slotContainer && !hasClass(slotContainer, "slotContainer")) {
slotContainer = slotContainer.previousSibling;
}
}
else if (direction === WinJS.Utilities.Key.downArrow) {
slotContainer = slotContainer.nextSibling;
while (slotContainer && !hasClass(slotContainer, "slotContainer")) {
slotContainer = slotContainer.nextSibling;
}
}
if (slotContainer) {
var slotContent = WinJS.Utilities.query(".slotContent", slotContainer);
if (slotContent && slotContent.length > 0) {
event.preventDefault();
event.stopImmediatePropagation();
currentNode.tabIndex = -1;
slotContent[0].tabIndex = 0;
slotContent[0].focus();
}
} else {
this._dispatchBoundaryEvent(event);
}
},
_dispatchBoundaryEvent: function _dispatchBoundaryEvent(event){
var Key = WinJS.Utilities.Key;
var keyCode = event.keyCode;
var direction = null;
var NV = CommonJS.NavigableView;
switch (keyCode) {
case Key.upArrow:
direction = NV.Direction.UP;
break;
case Key.downArrow:
direction = NV.Direction.DOWN;
break;
case Key.leftArrow:
direction = NV.Direction.LEFT;
break;
case Key.rightArrow:
direction = NV.Direction.RIGHT;
break;
}
NV.dispatchBoundaryEvent(event.srcElement, direction);
event.preventDefault();
event.stopImmediatePropagation();
},
_closeHandler: function _closeHandler() {
var key = CommonJS.TempCluster._showSettingsKey;
Windows.Storage.ApplicationData.current.localSettings.values[key] = false;
if (this._removeCluster) {
this._removeCluster();
}
},
_performAction: function _performAction(slotData, contentIndex) {
if (!slotData || !slotData.destination) {
return;
}
var clusterName = this._rendererMap[this.templateName].clusterName;
var navMethodName = this._rendererMap[this.templateName].navMethodName;
this._logUserAction(clusterName, slotData.actionId, contentIndex);
var searchString = "pano&panoid=action_";
var url = slotData.destination.toLowerCase();
var index = url.indexOf(searchString);
if (index < 0) {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(navMethodName);
PlatformJS.Navigation.navigateTo(url);
} else {
var action = url.substring(index + searchString.length);
if (action === this._dismissAction) {
this._closeHandler();
}
else {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(navMethodName);
url = url.replace(searchString, "");
PlatformJS.Navigation.navigateTo(url);
}
}
},
_updateMessageData: function _updateMessageData() {
if (this.messageData) {
this.messageData.closeHandler = WinJS.Utilities.markSupportedForProcessing(this._closeHandler.bind(this));
}
},
_logUserAction: function _logUserAction(actionContext, element, itemIndex) {
var telemetry = Platform.Telemetry;
var customAttributes = {
itemIndex: itemIndex
};
telemetry.FlightRecorder.logUserActionWithJsonAttributes(
telemetry.LogLevel.normal,
actionContext,
element,
telemetry.UserActionOperation.click,
telemetry.UserActionMethod.unknown,
0,
JSON.stringify(customAttributes));
},
dispose: function dispose() {
var currentRenderPromise = this._renderPromise;
this._renderPromise = null;
if (currentRenderPromise) {
currentRenderPromise.cancel();
}
var innerPromises = this._innerPromiseList;
if (innerPromises && innerPromises.length > 0) {
for (var index = 0, len = innerPromises.length; index < len; index++) {
var innerPromise = innerPromises[index];
innerPromise.cancel();
innerPromise = null;
}
this._innerPromiseList = null;
}
if (this._domElement) {
this._domElement.winControl = null;
this._domElement = null;
}
}
},
{
_showSettingsKey: "showGettingStartedCluster",
_gettingStartedDataSourceId: "GettingStarted",
_nextStepsDataSourceId: "NextSteps",
_defaultRemoveCluster: function _defaultRemoveCluster(clusterList, clusterKey) {
if (clusterList) {
for (var index = 0, len = clusterList.length; index < len; index++) {
var cluster = clusterList.getAt(index);
if (cluster.clusterKey === clusterKey) {
clusterList.splice(index, 1);
break;
}
}
}
},
// returns true/false depending on whether 'GettingStarted' cluster can be shown or not.
shouldShowTempCluster: function shouldShowTempCluster() {
var hasUserDismissed = Windows.Storage.ApplicationData.current.localSettings.values[CommonJS.TempCluster._showSettingsKey];
var clusterEnabled = PlatformJS.Services.configuration.getBool("EnableGettingStarted");
return (hasUserDismissed !== false) && clusterEnabled;
},
// Apps would call this function to insert the 'Getting Started' cluster.
insertClusterIfAllowed: function insertClusterIfAllowed(clusterList, config) {
var tempCluster = CommonJS.TempCluster;
var key = tempCluster._showSettingsKey;
// note: 'undefined' means should insert!
var showCluster = tempCluster.shouldShowTempCluster();
var promise = WinJS.Promise.wrap(false);
if (showCluster && clusterList && config) {
// Check if data is provided.
if (!config.data) {
// Data is not available. use a provider.
// use the default data provider.
if (!config.dataProvider) {
var options = {
useLocalCache: config.useLocalCache
};
config.dataProvider = new CommonJS.TempClusterDataProvider(options);
}
var dataSourceId = tempCluster._gettingStartedDataSourceId;
var clusterRendered = false;
promise = config.dataProvider.fetchData(dataSourceId).then(function gsFetchDataComplete(serverResponse) {
// if data is available, then parse it and insert the cluster.
// otherwise, dont insert the cluster.
if (serverResponse) {
var clusterData = config.dataProvider.parseData(serverResponse);
if (clusterData && clusterData.data && !clusterRendered) {
config.data = clusterData.data;
config.clusterTitle = clusterData.clusterTitle;
tempCluster._insertCluster(clusterList, config);
return true;
}
}
},
function gsFetchDataError(error){
},
function gsFetchDataProgress(cacheData) {
// progress returns data from cache, which is always valid. 
if (cacheData) {
config.data = cacheData.data;
config.clusterTitle = cacheData.clusterTitle;
tempCluster._insertCluster(clusterList, config);
clusterRendered = true;
}
});
}
else if (config.data) {
// We have the data itself, just render the cluster.
tempCluster._insertCluster(clusterList, config);
promise = WinJS.Promise.wrap(true);
}
}
return promise;
},
// Apps would call this function to insert the 'Next Steps' cluster.
insertNextStepsCluster: function insertNextStepsCluster(clusterList, config) {
var tempCluster = CommonJS.TempCluster;
var clusterEnabled = PlatformJS.Services.configuration.getBool("EnableNextSteps");
var promise = WinJS.Promise.wrap(false);
if (!clusterList || !config) {
console.log("Invalid input parameters to insertNextStepsCluster(). Not inserting the cluster");
return promise;
}
if (!clusterEnabled) {
return promise;
}
config.isLastCluster = true; // this flag allows other clusters to be aware not to insert themselves into the cluster list after the NextSteps cluster
// Check if data is provided.
if (!config.data) {
// Data is not available. use a provider.
// use the default data provider.
if (!config.dataProvider) {
config.dataProvider = new CommonJS.TempClusterDataProvider();
}
var dataSourceId = tempCluster._nextStepsDataSourceId;
promise = config.dataProvider.fetchData(dataSourceId).then(function gsFetchDataSuccess(serverResponse) {
// if data is available, then parse it and insert the cluster.
// otherwise, dont insert the cluster.
if (serverResponse) {
var clusterData = config.dataProvider.parseNextStepsData(serverResponse);
if (clusterData && clusterData.data && clusterData.clusterTitle) {
config.data = clusterData.data;
config.clusterTitle = clusterData.clusterTitle;
config.clusterClass = "nextStepsCluster";
tempCluster._setCompactPanorama();
tempCluster._insertCluster(clusterList, config);
return true;
}
}
});
}
else if (config.data) {
// We have the data itself, just render the cluster.
tempCluster._insertCluster(clusterList, config);
promise = WinJS.Promise.wrap(true);
}
return promise;
},
// Sometimes there is whitespace at the end of panorama, after next steps cluster.
// The requirement is to have the 'next steps' cluster touch the end of page. 
// Setting the 'compactPanorama' property here, so that the whitespace is not added at the end.
_setCompactPanorama: function _setCompactPanorama() {
var panoNode = document.querySelector(".platformPanorama");
if (panoNode && panoNode.winControl) {
panoNode.winControl.compactPanorama = true;
}
},
_insertCluster: function _insertCluster(clusterList, config) {
var clusterKey = config.clusterKey ? config.clusterKey : "tempCluster";
var clusterData = {
clusterKey: clusterKey,
clusterTitle: config.clusterTitle,
clusterPosition: config.clusterPosition,
clusterIndex: config.clusterPosition,
clusterContent: {
contentControl: "CommonJS.TempCluster",
contentOptions: {
clusterKey: clusterKey,
clusterList: clusterList,
messageData: config.data,
templateName: config.templateName,
removeClusterHandler: CommonJS.TempCluster._defaultRemoveCluster.bind(this, clusterList, clusterKey),
clusterClass: config.clusterClass
}
},
isTempCluster: true,
isLastCluster: config.isLastCluster,
platformInvisibleCluster: true,
platformInvisiblePermanentCluster: true
};
if (config.isLastCluster === true) {
config.clusterPosition = clusterList.length;
}
var cluster;
for (var index = clusterList.length - 1; index >= 0; index--) {
cluster = clusterList.getAt(index);
if (cluster && cluster.clusterKey === clusterKey) {
if (index === config.clusterPosition) {
return;
} else {
clusterList.splice(index, 1);
}
}
}
clusterList.splice(config.clusterPosition, 0, clusterData);
},
_stripHTML: function _stripHTML(string) {
if (string) {
return string.replace(/<(?:.|\n)*?>/gm, "");
}
else {
return "";
}
},
htmlConverter: WinJS.Binding.converter(function(htmlInput){
var tempCluster = CommonJS.TempCluster;
return tempCluster._stripHTML(htmlInput);
})
})
});
})();
