﻿/* shared\js\Globalization.js */
/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
'use strict';
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS", {
Globalization: WinJS.Class.define(
function () { }, {
}, {
_data: null,
_countryCodes: null,
_currencyToCultureMap: null,
_indexSymbolToCodeMap: null,
_ccyTotopConvertedCcys: null,
calculatorValues: null,
reload: function reload() {
this._data = null;
this._getData();
},
_setData: function _setData(data) {
this._data = data;
/* Initialize local settings data for PageClusters */
this._data.region = Platform.Globalization.Marketization.getCurrentMarketInfo().geographicRegion || Windows.Globalization.GeographicRegion();
this._applicationData = Windows.Storage.ApplicationData.current;
this._localSettings = this._applicationData.localSettings;
this._composite = new Windows.Storage.ApplicationDataCompositeValue();
return this._data;
},
_getData: function _getData() {
/*   ATTENTION : If Logic of populating homePageIndices is changed here, it has to be changed in TaskFirstSignOn.js as well */
/// <summary>
/// Get globalization Data and load it if unloaded
/// </summary>
if (this._data) { return this._data; }
/// Get the region from the current market
return this._setData(FinanceAppJS.ConfigManager.getCollatedConfigData());
},
getChannels: function getChannels() {
return this._getData().channels;
},
getClusters: function getClusters(pageId) {
return this._getData().clusters[pageId];
},
getBATSDisclaimerFlag: function getBATSDisclaimerFlag() {
return this._getData().showBATSDisclaimer;
},
getMoversExchangeList: function getMoversExchangeList() {
return this._getData().moversExchangeList;
},
getLocalExchanges: function getLocalExchanges() {
return this._getData().localExchangesList;
},
getDefaultExchange: function getDefaultExchange() {
//For ROW markets, the exchange list is empty. So passing "none" in this case (this is what the backend expects) 
var exchanges = this._getData().moversExchangeList;
if (exchanges.length > 0) {
return exchanges[0];
}
else {
return "none";
}
},
getExchangeConfig: function getExchangeConfig(exchange) {
return this._getData().exchangeDef[exchange];
},
getDefaultCurrency: function getDefaultCurrency() {
var defCurr = this._getData().defaultCurrency;
return defCurr ? defCurr : "USD";
},
getCurrencies: function getCurrencies() {
return this._getData().currencies;
},
getIndexCodeForSymbol: function getIndexCodeForSymbol(symbol) {
var map = this._getData().indexSymbolToInfoMap;
if (map[symbol]) {
return map[symbol].code;
}
else {
return symbol;
}
},
// from a partial index symbol get the complete 5 part symbol to enable more information on charts
getCompleteIndexSymbol: function getCompleteIndexSymbol(symbol) {
if (!symbol || symbol === undefined || symbol === "") {
return null;
}
var map = this._getData().indexSymbolToInfoMap;
// try and find the partial or complete symbol (parameter) as one among the entries of the map _indexSymbolToCodeMap
for (var sym in map) {
var pos = sym.indexOf(symbol);
if (pos !== -1) {
return sym;
}
}
return symbol;
},
getHomePageIndices: function getHomePageIndices(field) {
if (!field) {
return this._getData().homePageIndices;
} else {
return this._getData().homePageIndices.map(function (value) { return value[field]; });
}
},
getRegion: function getRegion() {
return this._getData().region;
},
getRegionClassification: function getRegionClassification() {
return this._getData().regionClassification;
},
getLanguage: function getLanguage() {
return Platform.Globalization.Marketization.getQualifiedLanguageString();
},
getMarket: function getMarket() {
return this._getData().market;
},
getDefaultWatchlistSymbols: function getDefaultWatchlistSymbols() {
return this._getData().defaultWatchlistSymbols;
},
getNewsClusterDisplay: function getNewsClusterDisplay() {
return this._getData().newsClusterDisplay;
},
getCodeForCountry: function getCodeForCountry(country) {
var simplifier = function (str) { return str.toLowerCase().replace(/[\W]/g, ""); };
if (!this._countryCodes) {
var data = FinanceAppJS.ConfigManager.getConfigData("CountryCodes");
this._countryCodes = {};
for (var i in data) {
this._countryCodes[simplifier(data[i].country)] = data[i].code;
}
}
var simple = simplifier(country);
if (this._countryCodes.hasOwnProperty(simple)) {
return this._countryCodes[simple];
}
return null;
},
getIndexInfo: function getIndexInfo(idxSym) {
// assume idxSym should be the complete symbol
if (!idxSym) {
return null;
}
var map = this._getData().indexSymbolToInfoMap;
if (map.hasOwnProperty(idxSym)) {
return map[idxSym];
}
return null;
},
getCultureForCurrency: function getCultureForCurrency(currency) {
if (!this._currencyToCultureMap) {
this._currencyToCultureMap = FinanceAppJS.ConfigManager.getConfigData("CurrencyCulture");
}
return this._currencyToCultureMap[currency];
},
getTopConvertedCcysForccy: function getTopConvertedCcysForccy(ccy) {
ccy = ccy.toUpperCase();
if (!this._ccyTotopConvertedCcys) {
this._ccyTotopConvertedCcys = FinanceAppJS.ConfigManager.getConfigData("CurrencyConverterInfo");
}
if (this._ccyTotopConvertedCcys.hasOwnProperty(ccy)) {
return this._ccyTotopConvertedCcys[ccy];
} else {
return this._ccyTotopConvertedCcys["Default"];
}
return null;
},
getCalculatorValues: function getCalculatorValues() {
if (!this.calculatorValues) {
var currentRegion = this._data.region.codeThreeLetter;
var basePath = "/pages/Calculator/DefaultValuesAndRanges/";
var calcValues;
try {
calcValues = FinanceAppJS.ConfigManager._getFile(basePath + currentRegion);
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
calcValues = FinanceAppJS.ConfigManager._getFile(basePath + "ROW");
}
this.calculatorValues = calcValues;
}
return this.calculatorValues;
},
isLocalExchange: function isLocalExchange(exchange) {
return (utils.inArray(this.getLocalExchanges(), exchange) !== -1) ? true : false;
},
isUSMarket: function isUSMarket() {
return (this._getData().regionClassification === "USA");
},
getUSExchanges: function getUSExchanges() {
return this._getData().usExchangeList;
},
isUSExchange: function isUSExchange(exchange) {
return (utils.inArray(this.getUSExchanges(), exchange) !== -1) ? true : false;
},
isSecondaryMarket: function isSecondaryMarket(market) {
if (this._getData().freSecondaryMarketsConfig) {
return this._getData().freSecondaryMarketsConfig[market.toUpperCase()];
}
return false;
},
getPrimaryMarket: function getPrimaryMarket(secondaryMarket) {
return this._getData().freSecondaryMarketsConfig[secondaryMarket.toUpperCase()].PrimaryMarket;
},
isProviderSupported: function isProviderSupported(secondaryMarket, providerName) {
var supportedProviders = this._getData().freSecondaryMarketsConfig[secondaryMarket.toUpperCase()].SupportedProviders;
return supportedProviders.indexOf(providerName) >= 0;
},
getAboutLinks: function getAboutLinks() {
if (!this._getData().aboutLinks) {
this._getData().aboutLinks = FinanceAppJS.ConfigManager.getConfigData("AboutLinks");
}
//Check for market overrides. If present use it, else use language. If language also is not present, fallback to en-us.
var market = this._data && this._data.market;
var result = (this._data && this._data.aboutLinks && this._data.aboutLinks["en-us"]);
if (market) {
market = market.toLowerCase();
var linkBasedOnMarket = this._data.aboutLinks && this._data.aboutLinks[market];
if (linkBasedOnMarket) {
result = linkBasedOnMarket;
}
else {
var lang = this.getLanguage();
if (lang) {
lang = lang.toLowerCase();
var linkBasedOnLang = this._data.aboutLinks && this._data.aboutLinks[lang];
if (linkBasedOnLang) {
result = linkBasedOnLang;
}                                
}                            
}
}
return result;                    
}
}
)
});
})();
/* shared\js\processListener.js */
/********************************************************
*                                                        *
*   © Microsoft. All rights reserved.                    *
*                                                        *
*********************************************************/
(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS", {
ProcessListener: WinJS.Class.define(
/// <summary>
/// Constructor for your IProcessListener class
/// </summary>
function () {
},
{
_searchCharm: {
get: function get() {
return FinanceAppJS.Utilities.SearchCharm.getInstance();
}
},
_shareCharm: {
get: function get() {
return FinanceAppJS.Utilities.ShareContract.getInstance();
}
},
/// <summary>
/// Handles layout change event
/// </summary>
/// <param name="event">
/// Layout change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onLayoutChange: function onLayoutChange(event) {
},
/// <summary>
/// Handles orientation change event
/// </summary>
/// <param name="event">
/// Orientation change event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onOrientationChange: function onOrientationChange(event) {
},
/// <summary>
/// Handles Suspend event
/// </summary>
/// <param name="event">
/// Suspend event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onSuspending: function onSuspending(event) {
if (FinanceAppJS.Utils.ResumeHandler.suspendCallback) {
FinanceAppJS.Utils.ResumeHandler.suspendCallback();
}
},
/// <summary>
/// Handles Checkpoint event
/// </summary>
/// <param name="event">
/// Suspend suspend event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onCheckpoint: function onCheckpoint(event) {
},
/// <summary>
/// Handles Resume event
/// </summary>
/// <param name="event">
/// resume event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onResuming: function onResuming(event) {
//TODO - resume the periodic update once the user comes from a non-market country to market country
if (FinanceAppJS.Utils.ResumeHandler.refreshCallback) {
FinanceAppJS.Utils.ResumeHandler.refreshCallback();
}
FinanceAppJS.ConfigManager.checkConfigUpdate();
},
/// <summary>
/// Handles activation event
/// </summary>
/// <param name="event">
/// activation event
/// </param>
/// <returns>
/// Nothing
/// </returns>
onActivated: function onActivated(event, isRecoverMode) {
try {
FinanceAppJS.Globals.init();
FinanceAppJS.initializeConfigOperation.promise.then(function () {
try {
if (isRecoverMode) {
/* clear data from local settings data */
FinanceAppJS.LocalSettingsManager.instance.clearData("EquityDataProvider");
FinanceAppJS.LocalSettingsManager.instance.clearData("EditorialNewsDataProvider");
FinanceAppJS.LocalSettingsManager.instance.clearData("TempClusterProvider");
}
FinanceAppJS.DataProvider.loadStartupDataProviders();
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
}
});
var market = Platform.Globalization.Marketization.getCurrentMarket();
// Bug fix: 357863 - Work Around
if (market === "ZH-HK") {
var colorGain = "rgb(113, 171, 4);";
var colorLose = "rgb(204, 17, 34);";
// Captured the selectors by searching with @colorGain and @colorLose
var upSelectors = ".CurrencyPage .upIndx1{border-bottom-color:@colorGain}#marketToday .acrossmarket .up *{color:@colorGain}.upIndicatorSmall{border-bottom-color:@colorGain}#worldMarketPanorama .upIndx1{border-bottom-color:@colorGain}.up{color:@colorGain}.upIndx1{border-bottom-color:@colorGain}.lupIndx1{border-bottom-color:@colorGain}.upcolor{color:@colorGain}.financeGlyphUp{color:@colorGain}#FundEntityMainList .gainBar,#ETFEntityMainList .gainBar{background-color:@colorGain}".replace(/@colorGain/g, colorGain);
var downSelectors = ".CurrencyPage .dnIndx1{border-top-color:@colorLose}#marketToday .acrossmarket .down *{color:@colorLose}.dnIndicatorSmall{border-top-color:@colorLose}#worldMarketPanorama .dnIndx1{border-top-color:@colorLose}.dn{color:@colorLose}.dnIndx1{border-top-color:@colorLose}.ldnIndx1{border-top-color:@colorLose}.downcolor{color:@colorLose}.financeGlyphDn{color:@colorLose}#FundEntityMainList .loseBar,#ETFEntityMainList .loseBar{background-color:@colorLose}".replace(/@colorLose/g, colorLose);
utils.createElement(document.body, "style", { type: "text/css", textContent: upSelectors + "\n" + downSelectors });
}
/// PERF: This start the JITing of the BDI Dlls which take up 400ms of CPU Time(although not on the UI Thread)
/// Doing this now when the app is idle.
FinanceAppJS.AppManager.doWhenIdle(function () {
// 'prime' the bdi parser before the first actual hit.
// required to warm up the datacontract serializer.
try {
var newsDict = PlatformJS.Services.appConfig.getDictionary("BdiNewsVersions");
var transformer = new AppEx.Common.NewsBdiTransformer.Transforms.NewsTransformer(newsDict.getString("CurrentVersion"), null);
transformer.primeParser();
} catch (err) {
// ignore
}
});
// initialize finance uri scheme
Platform.Utilities.AppExUri.addScheme("bingfinance");
//handling of activation events from app and secondary tile
if (event.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.search) {
this._searchCharm.handleQuerySubmitted(event.detail);
}
else if (event.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.protocol && !FinanceAppJS.Globals.FRE) {
// Handle protocol activation.
// Example: bingfinance://controller/command#entityId?params
if (!FinanceAppJS.Utilities.NavigationController.defaultControllerAdded) {
FinanceAppJS.Utilities.NavigationController.addDefaultController();
}
FinanceAppJS.Utilities.NavigationController.addShareEntityViews();
PlatformJS.Navigation.addCloudPanoEntityView("CloudPano");
var uri = event.detail.uri.rawUri;
var action = PlatformJS.Navigation.createCommandFromUri(uri);
if (WinJS.Promise.is(action)) {
return action;
} else if (action.channelInfo) {
return { pageInfo: action.channelInfo.pageInfo, state: action.options };
} else {
action();
}
}
else if ((event.type === "activated") && event.detail && (event.detail.tileId !== "App") && event.detail.arguments) {
if (Windows.UI.ViewManagement.ApplicationView.value === Windows.UI.ViewManagement.ApplicationViewState.snapped) {
return activationHelper.navigateToSnappedHomePage();
}
else {
return FinanceAppJS.SecondaryTile.handleActivation(event.detail.arguments);
}
}
/************************************************************************************
*   ATTENTION. DO NOT ADD ANYTHING HERE. THERE WAS A return; ABOVE.
*************************************************************************************/
} catch (ex) {
}
},
startPostActivateWork: function startPostActivateWork() {
FinanceAppJS.AppManager.doWhenIdle(function () {
//Set-up secondary tiles
FinanceAppJS.TopEdgy.instance.initializeEdgy();
FinanceAppJS.Utilities.SearchCharm.getInstance();
FinanceAppJS.Utilities.ShareContract.getInstance();
if (!FinanceAppJS.Utilities.NavigationController.defaultControllerAdded) {
FinanceAppJS.Utilities.NavigationController.addDefaultController();
}
if (!FinanceAppJS.Utilities.NavigationController.shareEntityViewsAdded) {
FinanceAppJS.Utilities.NavigationController.addShareEntityViews();
}
//set-up the live tile
utils.updateLiveTile();
FinanceAppJS.SecondaryTile.updateAllTiles();
PlatformJS.Navigation.addCloudPanoEntityView("CloudPano");
PlatformJS.Navigation.addFeedbackEntityView();
FinanceAppJS.ConfigManager.checkConfigUpdate();
//Prefetching calculators' values
FinanceAppJS.Globalization.getCalculatorValues();
});
},
showSettings: function showSettings() {
CommonJS.Settings.showDefaultSettingsFlyout();
},
onUpdatedConfig: function onUpdatedConfig() {
FinanceAppJS.Globalization.reload();
FinanceAppJS.TopEdgy.instance.reload();
utils.updateLiveTile();
FinanceAppJS.ConfigManager.checkConfigUpdate(true);
//Returning true here as we want to go to home page on config update, as platform expects us to return true if we want to navigate to home.
return true;
},
onUpdatedMarket: function onUpdatedMarket() {
FinanceAppJS.Globalization.reload();
FinanceAppJS.TopEdgy.instance.reload();
utils.updateLiveTile();
FinanceAppJS.ConfigManager.checkConfigUpdate(true);
//TODO: when platform calls onNavigateAway on Market Change remove this line. This line clears out WSJ styles
if (NewsJS.Partners.Theme.onNavigated) {
NewsJS.Partners.Theme.onNavigated();
}
},
onBeforeActivate: function onBeforeActivate(event) {
if (FinanceAppJS.initializeConfigOperation._completed) {
FinanceAppJS.initializeConfigOperation.promise = WinJS.Promise.as();
return null;
} else {
return (FinanceAppJS.initializeConfigOperation.promise = new WinJS.Promise(function (complete, error, progress) {
FinanceAppJS.initializeConfigOperation.complete = complete;
}));
}
},
}
)
});
})();
/* shared\js\AppManager.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.AppManager", {
_appIsIdle: false,
appIsIdle: {
get: function get() { return this._appIsIdle; },
set: function set(value) {
this._appIsIdle = value;
if (value) {
msSetImmediate(this.processIdleTasks.bind(this));
}
}
},
_idleTasks: [],
_idleCheckInterval: null,
doWhenIdle: function doWhenIdle(callback) {
if (this.appIsIdle) {
callback();
} else {
this._idleTasks.push(callback);
}
},
processIdleTasks: function processIdleTasks() {
var task = this._idleTasks.shift();
while (task) {
task();
task = this._idleTasks.shift();
}
}
});
WinJS.Namespace.define("FinanceAppJS.Globals", {
_FRE: null,
_freClickMessageBar: null,
_freMessageBar: null,
_freMessageDisplayedOnce: false,
checkFREClick: function checkFREClick() {
if (FinanceAppJS.Globals.FRE) {
FinanceAppJS.Globals.showFREClickMessage();
return true;
}
return false;
},
showFREMessage: function showFREMessage() {
if (this._freMessageDisplayedOnce) {
return;
}
this._freMessageDisplayedOnce = true;
FinanceAppJS.Globals._freMessageBar = new CommonJS.MessageBar(utils.getResourceString("FREMessage"), { autoHide: false });
FinanceAppJS.Globals._freMessageBar.addButton(utils.getResourceString("/platform/close"), function () { FinanceAppJS.Globals._freMessageBar.hide(); });
FinanceAppJS.Globals._freMessageBar.show();
},
showFREClickMessage: function showFREClickMessage() {
var anchor = document.querySelector('.ImersivePage');
var flyoutDiv = anchor.querySelector('.networkUnavailableFlyoutContainer');
if (!flyoutDiv) {
flyoutDiv = document.createElement('div');
var freFlyout = this._freClickMessageBar = new WinJS.UI.Flyout(flyoutDiv);
WinJS.Utilities.addClass(flyoutDiv, 'networkUnavailableFlyoutContainer');
anchor.appendChild(flyoutDiv);
var moduleInfo = { templateId: "networkUnavailableTemplate", fragmentPath: "/shared/html/templates.html" };
CommonJS.loadModule(moduleInfo, {}, null, null, null).then(function (template) {
flyoutDiv.appendChild(template);
freFlyout.show(anchor, "top", "center");
});
}
else {
flyoutDiv.winControl.show(anchor, "top", "center");
}
utils.vAlignContainerInMiddle(flyoutDiv, anchor);
},
hideFREClickMessage: function hideFREClickMessage() {
if (this._freClickMessageBar) {
this._freClickMessageBar.hide();
}
if (this._freMessageBar) {
this._freMessageBar.hide();
}
},
FRE: {
get: function get() {
if (typeof FinanceAppJS.Globals._FRE === "undefined" || FinanceAppJS.Globals._FRE === null) {
FinanceAppJS.Globals._FRE = Windows.Storage.ApplicationData.current.localSettings.values["FRE"];
if (typeof FinanceAppJS.Globals._FRE === "undefined" || FinanceAppJS.Globals._FRE === null) {
FinanceAppJS.Globals._FRE = true;
}
if (this.internetAvailable) {
this.FRE = false;
}
}
return FinanceAppJS.Globals._FRE;
},
set: function set(value) {
if (FinanceAppJS.Globals._FRE === value) { return; }
Windows.Storage.ApplicationData.current.localSettings.values["FRE"] = FinanceAppJS.Globals._FRE = value;
if (FinanceAppJS.Globals.freStatusChangeCallback) {
FinanceAppJS.Globals.freStatusChangeCallback(value);
}
}
},
freStatusChangeCallback: null,
internetAvailable: false,
loadPageByDefault: "Default",
loadPageByRefresh: "AutoRefresh",
loadPageBySettingsChange: "SettingsChange",
loadPageByInternetChange: "NetworkChangeCallback",
networkChangeCallback: null,
init: function init() {
this.internetAvailable = Platform.Networking.NetworkManager.instance.isNetworkAvailable;
Platform.Networking.NetworkManager.addEventListener("networkstatuschanged", FinanceAppJS.Globals.onNetworkStatusChange);
},
onNetworkStatusChange: function onNetworkStatusChange(event) {
var currentStatus = Platform.Networking.NetworkManager.instance.isNetworkAvailable;
//app goes offline from onlfine.
if (FinanceAppJS.Globals.internetAvailable === true && currentStatus === false) {
// FinanceAppJS.Error.showMessageBar(utils.getResourceString("NoInternetConnection"), true);
FinanceAppJS.Globals.internetAvailable = currentStatus;
//  FinanceAppJS.Utilities.UI.setElementsByInternetAvailability();
}
// app comes online from offline.
if (FinanceAppJS.Globals.internetAvailable === false && currentStatus === true) {
FinanceAppJS.Globals.FRE = false;
FinanceAppJS.Globals.internetAvailable = currentStatus;
utils.enableEdgies();
/* Watchlist L2 needs to be refreshed when use goes from offline to online */
FinanceAppJS.TopEdgy.instance.setupMyStocksL2();
//FinanceAppJS.Utilities.UI.setElementsByInternetAvailability();
if (typeof FinanceAppJS.Globals.networkChangeCallback === 'function') {
try {
FinanceAppJS.Globals.networkChangeCallback();
}
catch (err) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(err.message, err.stack);
console.log(err.description);
}
}
}
}
});
var MAXSIZESTRING = 4095;
WinJS.Namespace.define("FinanceAppJS", {
LocalSettingsManager: WinJS.Class.define(function () {
this._localSettings = Windows.Storage.ApplicationData.current.localSettings;
}, {
_localSettings: null,
getData: function getData(key) {
try {
this._localSettings = Windows.Storage.ApplicationData.current.localSettings;
if (!this._localSettings.containers.hasKey(key)) {
return null;
}
var container = this._localSettings.containers.lookup(key);
/// <disable>JS2005.BuitinTypes</disable>
var str = new Array(container.values.size);
for (var iter = container.values.first() ; iter.hasCurrent; iter.moveNext()) {
str[parseInt(iter.current.key)] = iter.current.value;
}
var merged = "";
str.forEach(function (s) { merged = merged + s; });
return JSON.parse(merged);
}
catch (ex) {
debugger;
return null;
}
},
setData: function setData(key, data) {
try {
if (this._localSettings.containers[key]) {
this._localSettings.deleteContainer(key);
}
var container = this._localSettings.createContainer(key, Windows.Storage.ApplicationDataCreateDisposition.always);
var str = JSON.stringify(data);
for (var i = 0, j = 0; i < str.length; i += MAXSIZESTRING, j++) {
container.values.insert(j, str.substr(i, MAXSIZESTRING));
}
}
catch (ex) {
debugger;
}
},
clearData: function clearData(key) {
try {
if (this._localSettings.containers[key]) {
this._localSettings.deleteContainer(key);
}
} catch (ex) {
debugger;
}
}
}, {
_instance: null,
instance: {
get: function get() {
return FinanceAppJS.LocalSettingsManager._instance || (FinanceAppJS.LocalSettingsManager._instance = new FinanceAppJS.LocalSettingsManager());
}
}
})
});
})();
/* shared\js\utils.js */
(function () {
'use strict';
//This function retrieves the url parameters and retuns an object containing all url params (with parameters as fields )
function getUrlParameters(destinationURL) {
var queryParams = {};
//split the url query and parameters
var urlParts = destinationURL.split("?");
//process url parameters
if (urlParts && urlParts.length > 1) {
var urlParams = urlParts[1].split("&");
//add url parameters to queryParams object
var keyValuePair;
for (var i = 0; i < urlParams.length; i++) {
keyValuePair = urlParams[i].split("=");
if (keyValuePair && keyValuePair.length > 1) {
queryParams[keyValuePair[0]] = keyValuePair[1];
}
}
}
return queryParams;
}
//done
function setHighlight(element) {
WinJS.Utilities.addClass(element, 'active');
}
//done
function removeHighlight(element) {
WinJS.Utilities.removeClass(element, 'active');
}
//done
function inArray(arr, findObj, fromIndex) {
var index;
if (fromIndex === null || typeof fromIndex === 'undefined') {
index = 0;
} else if (fromIndex < 0) {
index = Math.max(0, arr.length + fromIndex);
}
for (var i = index, j = arr.length; i < j; i++) {
if (arr[i] === findObj) {
return i;
}
}
return -1;
}
//done
function formatVolume(value) {
// Return if value is abscent OR null; Otherwise format the value - even if value is zero also.
if ((!value && value !== 0) || (typeof value === undefined)) {
return "-";
}
var append = "";
if (value > 1000000000) {
value /= 1000000000;
append = utils.getResourceString("B");
}
else if (value > 1000000) {
value /= 1000000;
append = utils.getResourceString("M");
}
else if (value > 1000) {
value /= 1000;
append = utils.getResourceString("K");
}
return roundFloat(value, 2) + append;
}
function timeZoneFormatNoLabel(timeZone, offset) {
return utils.addLtrMark("{0} ({1})".format(timeZone, offset));
}
function timeZoneFormat(timeZone, offset) {
return utils.addLtrMark(utils.getResourceString("timeZoneLabel").format(timeZone, offset));
}
function indexChartLinkFormat(obj) {
return utils.addLtrMark(utils.getResourceString("indexChartLinkLabel").format(obj));
}
function getRangeIndicatorPos(start, end, currentPrice) {
return (parseInt((50 / (end - start)) * (currentPrice - start)) - 5) + "px";
}
function formatNumber(val, type, usePositivePrefix, retVal) {
var ret = null;
switch (type) {
case "PERCENT":
ret = FinanceAppJS.Utils.Formatters.percentFormatter(val, 2, usePositivePrefix, retVal);
break;
case "VOLUME":
ret = (typeof val !== 'undefined' && val !== null && isNaN(val) === false && parseInt(val) !== -99999) ? (parseInt(val) === 0 ? val : utils.formatVolume(val)) : (typeof retVal !== 'undefined' ? retVal : "-");
break;
case "FLOAT4":
ret = FinanceAppJS.Utils.Formatters.numberFormatter(val, 4, usePositivePrefix, retVal);
break;
case "FLOAT2":
ret = FinanceAppJS.Utils.Formatters.numberFormatter(val, 2, usePositivePrefix, retVal);
break;
case "FLOAT1":
ret = FinanceAppJS.Utils.Formatters.numberFormatter(val, 1, usePositivePrefix, retVal);
break;
case "FLOAT0":
ret = FinanceAppJS.Utils.Formatters.numberFormatter(val, 0, usePositivePrefix, retVal);
break;
case "P/E":
ret = val < 0 ? utils.getResourceString("NOT_APPLICACBLE") : FinanceAppJS.Utils.Formatters.numberFormatter(val, 2, usePositivePrefix, retVal);
break;
}
return ret;
}
//done
function formatString(val, retVal) {
var ret = null;
if (val && val !== "") {
ret = val;
}
else {
ret = typeof retVal !== 'undefined' ? retVal : "-";
}
return ret;
}
//done
function isValidStock(stock) {
return (stock &&
stock.Cmp && stock.Cmp !== "" &&
stock.E1 && stock.E1 !== "" &&
stock.E2 && stock.E2 !== "" &&
stock.St && stock.St !== "");
}
//done
function isValidFund(fnd) {
return (fnd &&
fnd.Fn && fnd.Fn !== "" &&
fnd.Nav && fnd.Nav !== "" &&
(fnd.Tkr !== "" || fnd.Scid !== ""));
}
//done
function trim(stringToTrim) {
return stringToTrim && stringToTrim.replace(/^\s+|\s+$/g, "");
}
//done
function dateTimeFormat(str) {
if (typeof str === 'undefined') {
return "-";
}
if (!str) {
return "-";
}
var dateObject = dateUtil(str.substring(0, str.indexOf(" ")));
var timeObject = timeUtil(str.substring(str.indexOf(" ") + 1, str.length));
var date = new Date(dateObject.year, dateObject.month, dateObject.date, timeObject.hour, timeObject.minute, 0, 0);
var mydatefmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month.abbreviated day");
var mytimefmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("hour minute");
var mydate = mydatefmt.format(date);
var mytime = mytimefmt.format(date);
return mydate + " " + mytime;
}
//done
function toTitleCase(str) {
if (!str) {
return "-";
}
return str.replace(/\w\S*/g, function (txt) { return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase(); });
}
//done
function formatValue(prepend, value, append) {
if (typeof value === 'undefined') { return "-"; }
if (value === "" || value === null || typeof value === 'undefined') {
return "-";
}
else {
return prepend + value + append;
}
}
//done
function dateFormat(str) {
if (typeof str === 'undefined') {
return "-";
}
if (!str) {
return "-";
}
var dateObject = dateUtil(str);
var date = new Date(dateObject.year, dateObject.month, dateObject.date, 0, 0, 0, 0);
var mydatefmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month day year");
var mydate = mydatefmt.format(date);
return mydate;
}
//done
function dateUtil(str) {
// str is in format dd-mm-yyyy
var dateParts = str.split("-");
return dateParts.length === 3 ? { month: dateParts[1] - 1, date: dateParts[0], year: dateParts[2] } : null;
}
//done
function timeUtil(str) {
// str is in format hh:MM:ss.ssss
var timeParts = str.split(':');
return timeParts.length === 3 ? { hour: timeParts[0], minute: timeParts[1], seconds: timeParts[2].split('.')[0] } : null;
}
//done
function timeFormat(str) {
if (typeof str === 'undefined') {
return "-";
}
if (!str) {
return "-";
}
var timeObject = timeUtil(str);
var date = new Date(0, 0, 0, timeObject.hour, timeObject.minute, 0, 0);
var mytimefmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("hour minute");
var mytime = mytimefmt.format(date);
return mytime;
}
//done
function contractFormat(month, year) {
// Sometimes response may have invalid month and year. So handling the exception and returning "-" in this case
try {
var date = new Date(year, month + 1, 0);
var dateFormat = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month.abbreviated year.abbreviated");
var formattedDate = dateFormat.format(date);
return utils.getResourceString("contract").format(formattedDate);
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
return "-";
}
}
//done
function createElement(parent, tag, attrs) {
// <summary>Helper function to create an Element in the DOM</summary>
// <param name="parent" type="DOMElement">The Parent Element for this Element</param>
// <param name="tag" type="String">The Tag Type for this Element</param>
// <param name="attrs" type="Array">Array of attributes to be applied to this element</param>
// <returns type="DOMElement" >The DOM Element created</returns>
var elem = document.createElement(tag);
if (typeof attrs !== 'undefined') {
for (var attr in attrs) {
if (typeof elem[attr] !== 'undefined') {
elem[attr] = attrs[attr];
} else {
elem.setAttribute(attr, attrs[attr]);
}
}
}
if (parent) {
parent.appendChild(elem);
}
return elem;
}
function roundFloat(num, digit) {
return FinanceAppJS.Utils.Formatters.numberFormatter(num, digit);
}
//done
function createElements(tag, containertag, elementClassValues, wrapperclass, parent) {
/// <summary>Helper function to quickly create DOM Elements from Arrays</summary>
/// <param name="tag" type="String">The Element Type to be created eg: div</param>
/// <param name="containertag" type="String">The Element Type for the container element of all the elements in the array</param>
/// <param name="elementClassValues" type="Array">Dictionary of classNames and innerTexts</param>
/// <param name="wrapperclass" type="String">Attach a classname to the container element (Optional)</param>
/// <param name="parent" type="DOMElement">Insert everyhting as a child of parent (Optional)</param>
var container = document.createElement(containertag);
for (var className in elementClassValues) {
createElement(container, tag, { className: className, innerText: elementClassValues[className] });
}
if (parent) {
parent.appendChild(container);
}
if (wrapperclass) {
container.className = wrapperclass;
}
return container;
}
function getGlobalInstrumentSymbol(data, exchange2) {
var globalExchangeFormat;
if (data) {
if (exchange2 === false) {
globalExchangeFormat = data.E1 + "." + data.St + "." + data.Sym;
}
else {
if (data.Eqsm && data.Sym !== data.Eqsm) {
globalExchangeFormat = data.E1 + "." + data.St + "." + data.Sym + "." + data.E2 + "." + data.Eqsm;
}
else {
globalExchangeFormat = data.E1 + "." + data.St + "." + data.Sym + "." + data.E2;
}
}
}
return globalExchangeFormat;
}
function getIndicesSymbol(data) {
var indicesFormat;
if (data.Eqsm && data.Sym !== data.Eqsm) {
indicesFormat = data.E1 + "." + data.St + "." + data.Sym + "." + data.E2 + "." + data.Eqsm;
}
else {
indicesFormat = data.E1 + "." + data.St + "." + data.Sym;
}
indicesFormat = FinanceAppJS.Globalization.getCompleteIndexSymbol(indicesFormat);
return indicesFormat;
}
function isEODInstrument(symbol) {
// Checks if the current instrument falls under the EOD category
if (symbol && symbol.indexOf) {
var E1 = symbol.substring(0, symbol.indexOf("."));
var EODConfig = FinanceAppJS.ConfigManager.getConfigData("EODandOTCconfig");
if (EODConfig) {
var EODRealTimeExchanges = EODConfig.EODRealTimeExchanges;
if (EODRealTimeExchanges && EODRealTimeExchanges.indexOf && EODRealTimeExchanges.indexOf(E1) >= 0) {
return true;
}
var EODInstrumentExceptions = EODConfig.EODInstrumentExceptions;
if (EODInstrumentExceptions && EODInstrumentExceptions.indexOf && EODInstrumentExceptions.indexOf(symbol) >= 0) {
return true;
}
}
}
return false;
}
function isOTCInstrument(exchange) {
// Checks if the current instrument falls under the OTC category
var OtcConfig = FinanceAppJS.ConfigManager.getConfigData("EODandOTCconfig");
if (OtcConfig) {
var OTCExchanges = OtcConfig.OTCEquityExchanges;
if (OTCExchanges && OTCExchanges.indexOf && OTCExchanges.indexOf(exchange) >= 0) {
return true;
}
}
return false;
}
function getDefaultConfigForIndices() {
// Gets the deafault config for the current market's indices
// Checks if the indices are EOD
// If even one is intraday, it sets shouldRefresh to true
// Checks if default index is EOD, and based on that it sets the default chart duration (ie: 'Day' for intrayday and 'week' for 'EOD')
var shouldRefresh = false;
var homePageIndices = FinanceAppJS.Globalization.getHomePageIndices();
var chartDefaultSymbol = homePageIndices[0].symbol;
var chartDefaultDuration = '1m';
var isEOD = true;
for (var i = 0; i < homePageIndices.length; i++) {
if (!utils.isEODInstrument(homePageIndices[i].symbol)) {
if (i === 0) {
chartDefaultDuration = '1d';
isEOD = false;
}
shouldRefresh = true;
break;
}
}
return { shouldRefresh: shouldRefresh, chartDefaultSymbol: chartDefaultSymbol, chartDefaultDuration: chartDefaultDuration, isEOD: isEOD };
}
function getFundUniqueStatus(data) {
var idConstants = FinanceAppJS.Constants.IDs;
return {
symbol: data.Tkr || data.Scid,
type: (data.Tkr && idConstants.FundTicker) || (data.Scid && idConstants.FundSecid)
};
}
function getResourceString(key) {
var value = null;
var resourceValue = WinJS.Resources.getString(key);
if (resourceValue.empty) {
console.log("MISSING RESOURCE - " + key);
}
value = resourceValue.value;
return value;
}
function getDateRangeButtonName(id) {
var chartButtonConfig =
{
'1d': { resourceName: 'ChartDay' },
'5d': { resourceName: 'ChartWeek' },
'1m': { resourceName: 'ChartMonth' },
'3m': { resourceName: 'Chart3Month' },
'1y': { resourceName: 'Chart1Year' },
'3y': { resourceName: 'Chart3Year' },
'5y': { resourceName: 'Chart5Year' },
'max': { resourceName: 'All' }
};
return this.getResourceString(chartButtonConfig[id].resourceName);
}
function populateMoversData(response, type) {
/*
type should be gainers, losers or active only.
*/
var data = {
headerData: null,
listData: null
};
var isActiveTile = false;
var moversData;
if (response) {
if (type === "gainers" && response.hasOwnProperty("G")) {
moversData = response.G;
}
else if (type === "losers" && response.hasOwnProperty("L")) {
moversData = response.L;
}
else if (type === "active" && response.hasOwnProperty("A")) {
moversData = response.A;
isActiveTile = true;
}
else {
return data;
}
var dataForTemplate = [];
for (var i = 0; i < moversData.length; i++) {
var item = {};
if (utils.isValidStock(moversData[i])) {
item.tileData = WatchListDataProvider.getStockItemTemplateData(moversData[i], true, isActiveTile);
item.tileData.StEx = null; // For market movers do not display exchange code
item.tileData.type = utils.getMarketMoversType(item.tileData.It);
dataForTemplate.push(item);
}
}
return dataForTemplate;
}
else {
return null;
}
}
function getSharedContractDataForStocks(dataProvider, entityObj, exchangeName, lastUpdatedTime, type) {
if (!dataProvider) {
return new WinJS.Promise.wrapError({});
}
var data = dataProvider._lastFetchData;
var frmtLastUpdatedTime = utils.formatLastUpdatedTime(lastUpdatedTime);
var sharedContractDataPromise;
if (data) {
var stockEntityData = {
timeAttribution: utils.getResourceString('ShareContractTimeAttribution').format(frmtLastUpdatedTime),
dataAttribution: utils.getResourceString('ShareContractDataAttribution'),
source: FinanceAppJS.Constants.IDs.Indices,
entities: [{
type: FinanceAppJS.Constants.IDs.Index,
header: exchangeName ? FinanceAppJS.Utils.Formatters.stringFormatter("{0} ({1})", [data.Cmp, exchangeName]) : data.Cmp,
cmp: data.Cmp,
sym: utils.getAutoSuggestDisplaySymbol(data),
lp: data.Lp,
changeStatus: data.changeStatus,
ch: data.Ch,
chp: data.Pc
}]
};
var uriBuilder = new Platform.Utilities.AppExUriBuilder();
uriBuilder.controllerId = "application";
uriBuilder.commandId = "view";
uriBuilder.queryParameters.insert("entitytype", type);
if (type === "stock") {
uriBuilder.queryParameters.insert("sym", entityObj.Sym);
uriBuilder.queryParameters.insert("eqsm", entityObj.Eqsm);
uriBuilder.queryParameters.insert("e1", entityObj.E1);
uriBuilder.queryParameters.insert("e2", entityObj.E2);
uriBuilder.queryParameters.insert("st", entityObj.St);
uriBuilder.queryParameters.insert("cmp", entityObj.Cmp);
if (entityObj.ExSn) {
uriBuilder.queryParameters.insert("exsn", entityObj.ExSn);
}
} else if (type === "index") {
uriBuilder.queryParameters.insert("displayName", entityObj.displayName);
uriBuilder.queryParameters.insert("ExchCode", entityObj.ExchCode);
uriBuilder.queryParameters.insert("indexSymbol", entityObj.indexSymbol);
}
var uri = uriBuilder.toString();
stockEntityData.deepLink = uri;
sharedContractDataPromise = new WinJS.Promise(function (complete, error) {
utils.getShareContractHtmlFormat(stockEntityData).then(function (htmlOutput) {
complete({
title: utils.getShareContractSubject(data.Cmp),
htmlContent: htmlOutput,
textContent: utils.getShareContractTextFormat(stockEntityData),
uri: uri
});
});
});
}
else {
sharedContractDataPromise = new WinJS.Promise.wrapError({});
}
return sharedContractDataPromise;
}
function removeAboutPageFragment() {
var aboutElement = document.getElementById('AboutPageDiv');
if (aboutElement) {
document.body.removeChild(aboutElement);
}
}
// ToDo: Need to move this out of utils.js
function addAboutPageFragment() {
var aboutElement = document.getElementById('AboutPageDiv');
if (!aboutElement) {
aboutElement = document.createElement('div');
aboutElement.id = 'AboutPageDiv';
document.body.appendChild(aboutElement);
}
var promise = new WinJS.Promise(function (complete) {
WinJS.UI.Fragments.renderCopy("/shared/html/AboutSettings.html", aboutElement).then(function (doc) {
var version = Windows.ApplicationModel.Package.current.id.version;
var versionStr = version.major + "." + version.minor + "." + version.build + "." + version.revision;
var aboutLabelText = utils.getResourceString("About");
// App Name and PublisherDisplayName should be retrieved from Platform Configuration Manifest, but they're not populated
var appName = utils.getResourceString("SettingAppName");
var publisherName = utils.getResourceString("SettingCmpName");
var productDetailsURL = FinanceAppJS.Globalization.getAboutLinks();
var productDetailsText = utils.getResourceString("productDetailsText");
var onProductDetailsClick = WinJS.Utilities.markSupportedForProcessing(function () {
// ToDo: Change the onClick Link once we have one.
Windows.System.Launcher.launchUriAsync(new Windows.Foundation.Uri(productDetailsURL));
});
var poweredByText = utils.getResourceString("poweredByText");
var dataPoweredByText = utils.getResourceString("dataPoweredByText");
var quotesText = utils.getResourceString("quotesText");
var quotesProviderText = utils.getResourceString("quotesProviderText");
var ratesText = utils.getResourceString("ratesText");
var ratesProviderText = utils.getResourceString("ratesProviderText");
var aboutPageBackButtonclick = WinJS.Utilities.markSupportedForProcessing(WinJS.UI.SettingsFlyout.show);
var disclaimerText = utils.getResourceString("disclaimerText");
var onDisclaimerClick = WinJS.Utilities.markSupportedForProcessing(function () {
// ToDo: Change the onClick Link once we have one.
Windows.System.Launcher.launchUriAsync(new Windows.Foundation.Uri(productDetailsURL + "#Disclaimer"));
});
var dataContext = {
versionText: utils.getResourceString("Version").format(versionStr),
aboutLabelText: aboutLabelText,
appName: appName,
poweredByText: poweredByText,
dataPoweredByText: dataPoweredByText,
quotesText: quotesText,
quotesProviderText: quotesProviderText,
ratesText: ratesText,
ratesProviderText: ratesProviderText,
publisherName: publisherName,
productDetailsText: productDetailsText,
onProductDetailsClick: onProductDetailsClick,
disclaimerText: disclaimerText,
onDisclaimerClick: onDisclaimerClick,
aboutPageBackButtonclick: aboutPageBackButtonclick
};
WinJS.UI.processAll(aboutElement).then();
WinJS.Binding.processAll(aboutElement, dataContext);
complete(PlatformJS.Utilities.getControl('AboutPage'));
}, function (error) {
complete(null);
});
});
return promise;
}
function getPointForSecondaryTile(element) {
var point = element.getBoundingClientRect();
return {
x: point.left,
y: point.top - 5
};
}
function setWinLabelText(dom, text) {
if (dom) {
dom.winControl.label = utils.getResourceString(text);
dom.winControl.tooltip = utils.getResourceString(text);
}
}
function isNonEmptyNumber(num) {
if (num === null || typeof num === 'undefined') {
return false;
} else {
return true;
}
}
function isNonEmptyString(str) {
if (!str || str === null || str === "") {
return false;
}
else {
return true;
}
}
function isNonEmptyObject(obj) {
return typeof obj === 'object' && obj !== null && typeof obj !== 'undefined' && Object.keys(obj).length > 0;
}
function isNonEmptyArray(obj) {
return typeof obj === 'object' && obj !== null && typeof obj !== 'undefined' && obj.constructor === Array && obj.length > 0;
}
function copyArrayToList(arr, list, start) {
start = start || 0;
for (var i = start; i < list.length && i < arr.length + start; i++) {
utils.deepCopy(arr[i - start], list.getAt(i));
}
}
function deepCopy(p, c, depth) {
/// <summary>Deep copy of values from source to destination</summary>
/// <param name="p" type="Object">Source</param>
/// <param name="c" type="Object">Destination</param>
if (typeof depth === 'undefined') {
depth = -1;
}
var c = c || new p.constructor();
for (var i in p) {
if (p[i] && typeof p[i] === 'object' && depth !== 0) {
c[i] = c[i] || new p[i].constructor();
deepCopy(p[i], c[i], depth - 1);
} else {
if (c[i] !== p[i]) { c[i] = p[i]; }
}
}
return c;
}
function getDeepObject(obj, key) {
if (!key) {
return obj;
}
if (typeof key === 'string') {
return obj[key];
}
if (isNonEmptyArray(key)) {
//if (key.length == 1) {
//    return obj;
//}
key.forEach(function (k) { obj = obj && obj[k]; });
}
return obj;
}
function setDeepObject(obj, key, value) {
if (!key) {
return;
}
if (typeof key === 'string') {
var old = obj[key];
obj[key] = value;
return old;
}
for (var i = 0, k; i < key.length - 1 && (k = key[i]) ; i++) {
obj = obj[k];
}
var old = obj[key[key.length - 1]];
(obj[key[key.length - 1]] = value);
return old;
}
function immediatePromiseCallback(complete, error, progress) {
msSetImmediate(this, complete, error, progress);
}
function createImmediatePromise(func) {
return new WinJS.Promise(immediatePromiseCallback.bind(func));
}
// Fields to hold the cached values of various Less variables
var colorBgGain = null, colorBgSecondary = null, gainColor = null, loseColor = null, equalColor = null, barGraphPimaryColor = null, barGraphSecondaryColor = null, chartBgColor = null, lessProxy = null, dir = null, colorBgMain = null, colorBgSub = null, colorProfit = null, colorLoss = null;
function JsLessVariables(useCachedValue) {
lessProxy = lessProxy ? lessProxy : document.querySelector("#LessProxyContainer");
dir = dir && useCachedValue ? dir : document.dir;
return {
gainColor: gainColor && useCachedValue ? gainColor : gainColor = lessProxy.querySelector(".up").currentStyle.color,
loseColor: loseColor && useCachedValue ? loseColor : loseColor = lessProxy.querySelector(".dn").currentStyle.color,
equalColor: equalColor && useCachedValue ? equalColor : equalColor = lessProxy.querySelector(".equalcolor").currentStyle.color,
barGraphPimaryColor: barGraphPimaryColor && useCachedValue ? barGraphPimaryColor : barGraphPimaryColor = lessProxy.querySelector(".barGraphPimaryColor").currentStyle.color,
barGraphSecondaryColor: barGraphSecondaryColor && useCachedValue ? barGraphSecondaryColor : barGraphSecondaryColor = lessProxy.querySelector(".barGraphSecondaryColor").currentStyle.color,
left: (dir === "rtl") ? "right" : "left",
right: (dir === "rtl") ? "left" : "right",
colorBgSecondary: colorBgSecondary && useCachedValue ? colorBgSecondary : colorBgSecondary = lessProxy.querySelector(".colorBgSecondary").currentStyle.color,
colorBgGain: colorBgGain && useCachedValue ? colorBgGain : colorBgGain = lessProxy.querySelector(".colorBgGain").currentStyle.color,
chartBgColor: chartBgColor && useCachedValue ? chartBgColor : chartBgColor = lessProxy.querySelector(".chartBgColor").currentStyle.color,
colorBgMain: colorBgMain && useCachedValue ? colorBgMain : colorBgMain = lessProxy.querySelector(".colorBgMain").currentStyle.color,
colorBgSub: colorBgSub && useCachedValue ? colorBgSub : colorBgSub = lessProxy.querySelector(".colorBgSub").currentStyle.color,
colorProfit: colorProfit && useCachedValue ? colorProfit : colorProfit = lessProxy.querySelector(".colorProfit").currentStyle.color,
colorLoss: colorLoss && useCachedValue ? colorLoss : colorLoss = lessProxy.querySelector(".colorLoss").currentStyle.color
};
}
//opacity mentioned in percentage.
//colorString can be either #xxyyzz or rgb(x,y,z) or rgba(w,x,y,z)
function getColorWithOpacity(colorString, opacity) {
var result = colorString;
if (colorString.substring(0, 3) === "rgb" && colorString[3] !== 'a') {
var splitStrings = colorString.split(")");
result = splitStrings[0] + "," + opacity / 100 + ")";
result = result.replace("rgb", "rgba");
}
else {
var rHex = colorString[1] + colorString[2];
var gHex = colorString[3] + colorString[4];
var bHex = colorString[5] + colorString[6];
var red = parseInt(rHex, 16);
var green = parseInt(gHex, 16);
var blue = parseInt(bHex, 16);
result = "rgba(" + red + "," + green + "," + blue + "," + opacity / 100 + ")";
}
return result;
}
function normalizeDate(date) {
var normailizedDate = null;
if (date) {
normailizedDate = new Date(date + ((new Date()).getTimezoneOffset()) * 60 * 1000);
}
return normailizedDate;
}
function getShareContractHtmlFormat(data) {
return WinJS.UI.Fragments.render("/shared/html/ShareContractHtmlTemplate.html").then(function (templateFragment) {
return WinJS.UI.processAll(templateFragment).then(function () {
var templateContent = null, newBlock = null, indicatorUp = null, indicatorDown = null;
var JsLessVars = utils.JsLessVariables(true);
var indicatorEqual = templateFragment.querySelector('#indicatorEqual').innerHTML;
indicatorUp = templateFragment.querySelector('#indicatorUp').innerHTML;
indicatorDown = templateFragment.querySelector('#indicatorDown').innerHTML;
var entities = data.entities || [];
var idConstants = FinanceAppJS.Constants.IDs;
if (parseInt(data.source) === idConstants.Currencies || parseInt(data.source) === idConstants.WorldMarket) {
templateContent = templateFragment.querySelector('#ShareContractMailSimpleTemplate').innerHTML;
templateContent = templateContent.replace("<!--[Text]-->", data.text)
.replace("<!--[Headline]-->", data.headline);
}
else if (parseInt(data.source) === idConstants.Indices) {
templateContent = templateFragment.querySelector('#ShareContractMailIndexTemplate').innerHTML;
newBlock = templateFragment.querySelector('#modelRowIndex').innerHTML;
}
else {
templateContent = templateFragment.querySelector('#ShareContractMailEntityTemplate').innerHTML;
newBlock = templateFragment.querySelector('#modelRowEntity').innerHTML;
}
// Respect text alignment for RTL: Replace left & right accordingly by using "center" as intermediate placeholder
if (document.dir === "rtl") {
templateContent = shareContractFormatHelperRTL(templateContent, JsLessVars);
newBlock = newBlock && shareContractFormatHelperRTL(newBlock, JsLessVars);
}
var op = templateContent;
op = op.replace("<!--[timeAttribution]-->", data.timeAttribution)
.replace("<!--[dataAttribution]-->", data.dataAttribution)
.replace("<!--[deepLink]-->", utils.getResourceString("/platform/win8Share").format(utils.getResourceString("/platform/win8Brand"), "<a style='color: color: rgb(0, 130, 153);' href='" + data.deepLink + "'>" + utils.getResourceString("/platform/win8ShareLink").format(utils.getResourceString("AppTitle")) + "</a>"));
for (var i = 0; i < entities.length; i++) {
var chColor = null, chPColor = null, indicatorColor = null, indicatorTemp = null;
if (i > 0) {
op = op.replace("<!--[TR]-->", newBlock + "<!--[TR]-->"); // Replace placeholder TR with empty markup + another place holder (for upcoming row)
}
if (entities[i].header) {
op = op.replace("<!--[header]-->", utils.removeLtrMark(entities[i].header));
}
op = op.replace("<!--[symbol]-->", entities[i].sym)
.replace("<!--[lastPrice]-->", utils.removeLtrMark(entities[i].lp))
.replace("<!--[change]-->", utils.removeLtrMark(entities[i].ch))
.replace("<!--[changePercent]-->", utils.removeLtrMark(entities[i].chp));
if (entities[i].changeStatus < 0) {
indicatorColor = chColor = chPColor = JsLessVars.loseColor;
indicatorTemp = indicatorDown;
}
else if (entities[i].changeStatus > 0) {
indicatorColor = chColor = chPColor = JsLessVars.gainColor;
indicatorTemp = indicatorUp;
}
else {
indicatorColor = chColor = chPColor = JsLessVars.equalColor;
indicatorTemp = indicatorEqual;
}
op = op.replace("color: red", "color: " + chColor).replace("color: red", "color: " + chPColor);
op = op.replace("<!--[chIndicator]-->", indicatorTemp);
op = op.replace("red", indicatorColor);
}
return op;
},
function (error) {
return null;
});
});
}
function getShareContractSubject(itemName) {
return utils.addLtrMark(itemName) + " - " + utils.addLtrMark(utils.getResourceString("BingAppTitle"));
}
function shareContractFormatHelperRTL(content, JsLessVars) {
return content.replace(/left/g, "center").replace(/right/g, JsLessVars.right).replace(/center/g, JsLessVars.left);
}
function getShareContractTextFormat(data) {
var text = "";
var entities = data.entities;
if (entities.length > 1) {
for (var i = 0; i < entities.length; i++) {
text += FinanceAppJS.Utils.Formatters.stringFormatter("{0}, {1}, {2}, {3}", [entities[i].sym, entities[i].lp, entities[i].ch, entities[i].chp]) + "; ";
}
}
else {
text = FinanceAppJS.Utils.Formatters.stringFormatter("{0}, {1}, {2}, {3}, {4}", [entities[0].cmp, entities[0].sym, entities[0].lp, entities[0].ch, entities[0].chp]) + "; ";
}
text += data.timeAttribution + "; " + data.dataAttribution + ";";
return text;
}
function createTileNotificationUri(symbol, type, mode) {
var pullNotification = PlatformJS.Services.appConfig.getDictionary("PullNotification");
var serverUri = pullNotification.getString("AppTileUri");
serverUri = serverUri.replace("\{symbol\}", symbol);
serverUri = serverUri.replace("\{type\}", type);
var lang = FinanceAppJS.Globalization.getMarket();
serverUri = serverUri.replace("\{market\}", lang);
serverUri = serverUri.replace("\{mode\}", mode);
return new Windows.Foundation.Uri(serverUri);
}
function GetNotificationFrequency() {
var pullNotification = PlatformJS.Services.appConfig.getDictionary("PullNotification");
var updateRecurrence = pullNotification.getString("UpdateRecurrence");
return updateRecurrence;
}
/// <summary>
/// This function adds a special zero length character "\u200e" to the begining and end of string. The presence of this char indicates that this string should not be modified in RTL languages while rendering on UI.
/// </summary>
/// <returns>
/// string containing LTR mark character
/// </returns>
function addLtrMark(text) {
return "\u200e" + text + "\u200e";
}
/// <summary>
/// This function removes a special zero length character "\u200e" from the string. For share contract mail content, we no need LTR marks.
/// </summary>
/// <returns>
/// string containing zero length characters
/// </returns>
function removeLtrMark(text) {
return text.replace(/\u200e/g, "");
}
function isStockSymbol(symbol) {
return utils.inArray(FinanceAppJS.Constants.AutoSuggest.StockSymbolTypes, symbol) !== -1;
}
function isEtfSymbol(symbol) {
return utils.inArray(FinanceAppJS.Constants.AutoSuggest.EtfSymbolTypes, symbol) !== -1;
}
function isFundSymbol(symbol) {
return utils.inArray(FinanceAppJS.Constants.AutoSuggest.FundSymbolTypes, symbol) !== -1;
}
function isIndexSymbol(symbol) {
return utils.inArray(FinanceAppJS.Constants.AutoSuggest.IndexSymbolTypes, symbol) !== -1;
}
function getAutoSuggestDisplaySymbol(dataObj, type) {
var idConstants = FinanceAppJS.Constants.IDs;
switch (type) {
case idConstants.Fund:
return dataObj.Tkr || "";
case idConstants.Stock:
case idConstants.Etf:
case idConstants.Index:
default: // Backward compatible. This function originally written for stock/etf. So, sending type is optional.
return dataObj.Eqsm || dataObj.Sym || "";
}
}
function getAutoSuggestDisplayName(dataObj, type) {
var idConstants = FinanceAppJS.Constants.IDs;
switch (type) {
case idConstants.Fund:
return utils.trim(dataObj.Fn) || "";
case idConstants.Stock:
case idConstants.Etf:
case idConstants.Index:
default:
return utils.trim(dataObj.Cmp) || "";
}
}
/// <summary>
/// This function adds a custom aria label to its parent container. It also works for list of parentcontainer objects.
/// </summary>
/// <param name="parentContainerObj" type="object">ParentContainerObject</param>
/// <param name="ntTextClass" type="String">narator text class to identify</param>
function addCustomAriaLabelToTiles(parentContainerObj, ntTextClass) {
try {
if (parentContainerObj) {
if (parentContainerObj.length > 0) {
var listLength = parentContainerObj.length;
var ntClass = '.' + ntTextClass;
for (var index = 0; index < listLength; index++) {
var nartext = null;
var wacthListtile = parentContainerObj[index].querySelector(ntClass);
if (wacthListtile) {
nartext = wacthListtile.innerText;
}
if (nartext) {
parentContainerObj[index].setAttribute("aria-label", nartext);
parentContainerObj[index].setAttribute("role", "link");
}
/* Add Tile. Checking if Add tile */
var addListtile = WinJS.Utilities.hasClass(parentContainerObj[index], "addTileContainer");
if (addListtile) {
nartext = utils.getResourceString("WathcListAddButton_Al");
parentContainerObj[index].setAttribute("aria-label", nartext);
parentContainerObj[index].setAttribute("role", "link");
}
}
}
}
}
catch (err) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(err.message, err.stack);
}
}
function vAlignContainerInMiddle(container) {
var elmHeight = container.currentStyle.height.replace("px", "");
var pElmHeight = screen.height;
container.style.marginTop = ((pElmHeight - elmHeight) / 2) + "px";
}
function formatLastUpdatedTime(lastUpdateTime, isSnapped) {
if (!lastUpdateTime) {
return "-";
}
if (!isSnapped) {
return FinanceAppJS.Utils.Formatters.GetLastUpdatedDate(lastUpdateTime);
}
else {
return FinanceAppJS.Utils.Formatters.shortDateFormatDateObj(lastUpdateTime) + " " + FinanceAppJS.Utils.Formatters.shortTimeFormatDateObj(lastUpdateTime);
}
}
function launchNewsUri(data, source) {
var useArticleReader = FinanceAppJS.Utils.UseArticleReader();
var articleUrl = data.articleUrl;
// if the response from bedrock has article url then open the same in the browser
if (articleUrl) {
if (Platform.Utilities.Globalization.isFeatureEnabled("NewsinIFrame")) {
PlatformJS.Navigation.navigateToChannel("NewsFramePage", { articleUrl: articleUrl });
} else {
var uri = new Windows.Foundation.Uri(articleUrl);
Windows.System.Launcher.launchUriAsync(uri).then();
}
}
else if (data.articleid && useArticleReader) { //if article id is specified and we have the option to use article reader
var articleId = data.articleIndex + "";
var arConfig = utils.getArticleReaderConfig(data);
if (arConfig && arConfig.providerConfiguration && data.articleInfos) {
populateArticleReaderAds(data.articleInfos);
arConfig.providerConfiguration.insert("indexBased", "");
arConfig.providerConfiguration.insert("articleInfos", JSON.stringify({ articleInfos: data.articleInfos }));
}
PlatformJS.Navigation.navigateToChannel('CommonControlsArticleReader', {
providerType: arConfig.providerType,
providerConfiguration: arConfig.providerConfiguration,
initialArticleId: articleId,
market: FinanceAppJS.Globalization.getMarket().toLowerCase(),
source: source,
enableSharing: true,
});
} else if (data.type === "slideshow") {
WinJS.Navigation.navigate({
fragment: "/pages/SlideShow/html/SlideShow.html",
page: "FinanceAppJS.SlideShow"
},
{
slideshowId: data.contentid,
});
}
}
function populateArticleReaderAds(articleInfos) {
var ads2 = PlatformJS.Services.appConfig.getDictionary("Ads2");
if (ads2) {
var applicationId = ads2.getString("ApplicationID");
var articleReaderConfigString = ads2.getString("ArticleReaderAdConfig");
if (articleReaderConfigString) {
var configJSON = JSON.parse(articleReaderConfigString);
if (configJSON) {
/// Note:  This forces a load of all the files in case they arent loaded on Demand
NewsJS.Partners.Config;
PlatformJS.Utilities.annotateArticleInfosWithAdInfo(articleInfos, configJSON, applicationId);
}
}
}
}
function getArticleReaderId(item) {
var articleid = null;
if (item.contentid) {
articleid = item.articleid + '/' + item.contentid;
}
else {
articleid = item.articleid;
}
return articleid;
}
function getArticleReaderConfig(data) {
var providerConfiguration = PlatformJS.Collections.createStringDictionary();
// if it is preview data then call the non caching bedrock article service
if (FinanceAppJS.Settings.cmsPreviewEnabled) {
providerConfiguration.insert("queryServiceId", "RenderProxy_BedrockArticlesNoCache");
providerConfiguration.insert("fallbackQueryServiceId", "BedrockArticlesNoCache");
} else {
providerConfiguration.insert("queryServiceId", "RenderProxy_BedrockArticles");
providerConfiguration.insert("fallbackQueryServiceId", "BedrockArticles");
}
providerConfiguration.insert("market", FinanceAppJS.Globalization.getMarket().toLowerCase());
return {
providerType: "AppEx.Common.ArticleReader.BedrockArticleProvider",
providerConfiguration: providerConfiguration
};
}
function parseMagazineResponse(jsonResponse, collectionsArray, type) {
if (!jsonResponse || !utils.isNonEmptyArray(jsonResponse.clusters)) {
return null;
}
var clusters = jsonResponse.clusters;
collectionsArray = collectionsArray.map(function (item) { return item.toLowerCase(); });
for (var i = 0; i < clusters.length; i++) {
var cluster = clusters[i];
if (!cluster.entityList || (collectionsArray && collectionsArray.indexOf(cluster.entityList.collectionId.trim().toLowerCase()) === -1)) {
continue;
}
var entities = cluster.entityList.entities;
var results = [];
for (var j = 0; j < entities.length; j++) {
results.push(convertCMSArticle(entities[j], type));
}
results = results.filter(function (item) { return item; });
return { native: true, newsItems: results };
}
return null;
}
/* Initialize DOM Parser */
var parser = new DOMParser();
function parseTextContent(string) {
/* setting inside div so that plain text is never passed */
if (parser) {
var tempstr = "<p>" + string + "</p>";
var xml = parser.parseFromString(tempstr, "text/xml");
return xml.documentElement.textContent;
}
else {
return string;
}
}
function getDateString(dateType) {
if (dateType && dateType.utctime) {
var utcTime = dateType.utctime;
// TODO: Let platform know that this may be off an hour, need to investigate
var time = CommonJS.Utils.convertBDITimeToFriendlyTime(utcTime);
return time && time.indexOf("NaN") < 0 ? time : "";
}
else {
return "";
}
}
function convertCMSArticle(result, type) {
var newsArticle = {};
newsArticle.editorial = true;
if (result.heroImage) {
newsArticle.anchorpoint = result.heroImage.anchorPoint;
newsArticle.headerfont = result.heroImage.headerFont;
}
var cmsImage = null;
if (result.heroImage && result.heroImage.image) {
cmsImage = result.heroImage.image;
}
else {
cmsImage = result.image;
}
if (cmsImage) {
newsArticle.imageAttribution = cmsImage.attribution ? cmsImage.attribution : "";
newsArticle.altText = cmsImage.altText ? cmsImage.altText : "";
newsArticle.caption = cmsImage.caption ? toStaticHTML(cmsImage.caption) : "";
var originalImage = null;
for (var i = 0; i < cmsImage.images.length; i++) {
var img = cmsImage.images[i];
if (img.name === "original") {
newsArticle.thumbnail = {};
newsArticle.thumbnail.url = img.url;
newsArticle.thumbnail.width = img.width;
newsArticle.thumbnail.height = img.height;
newsArticle.thumbnail.name = img.name;
}
else if (img.name === "semanticZoom") {
newsArticle.semanticZoomThumbnail = {};
newsArticle.semanticZoomThumbnail.url = img.url;
newsArticle.semanticZoomThumbnail.width = img.width;
newsArticle.semanticZoomThumbnail.height = img.height;
newsArticle.semanticZoomThumbnail.name = img.name;
}
else if (img.name === "lowRes") {
newsArticle.thumbnailLowRes = {};
newsArticle.thumbnailLowRes.url = img.url;
newsArticle.thumbnailLowRes.width = img.width;
newsArticle.thumbnailLowRes.height = img.height;
newsArticle.thumbnailLowRes.name = img.name;
}
}
}
newsArticle.sourceImageUrl = result.content.source && result.content.source.favicon ? result.content.source.favicon : "";
// CMS headline support HTML Fragments
newsArticle.headline = result.content.headline ? toStaticHTML(result.content.headline) : "";
newsArticle.title = newsArticle.headline;
newsArticle.source = result.content.source && result.content.source.name ? result.content.source.name : "";
newsArticle.destination = result.destination;
// The following are the different destination URLs we support (newsArticle.destination)
// OLD FORMAT (To be eventually removed)
//      article://33259198
// NEW FORMAT (various urls below, see data contract spec for full details)
//      //application/view?entitytype=article&pageId=33259198
//      bingfinance://application/view?entitytype=article&pageId=33259198 
//      ///view?entitytype=article&pageId=2123125
//      bingfinance:///view?entitytype=article&pageId=2123125
var appUri = CollectionProvider.FinanceAppExUriManager.instance.createAppUri(newsArticle.destination);
if (appUri) {
if (appUri.entitytype === "article") {
newsArticle.type = appUri.entitytype;
newsArticle.articleid = appUri.pageId && appUri.pageId.length > 0 ? appUri.pageId : null;
newsArticle.contentid = appUri.contentId && appUri.contentId.length > 0 ? appUri.contentId : null;
}
else if (appUri.entitytype === "slideshow") {
newsArticle.type = appUri.entitytype;
newsArticle.contentid = appUri.pageId;
}
else if (appUri.entitytype === "video") {
// Strip HTML tags out as topBarTitle of video Controlonly supports text.
if (newsArticle.thumbnail) {
newsArticle.thumbnail.videoOptions = {
videoSource: appUri.contentId,
source: newsArticle.source,
title: newsArticle.title ? parseTextContent(newsArticle.title) : "",
sourceImageUrl: newsArticle.sourceImageUrl,
thumbnail: newsArticle.thumbnail && newsArticle.thumbnail.url ? newsArticle.thumbnail.url : null
};
}
else {
return null;
}
}
}
// CMS abstract support HTML Fragments
newsArticle.abstract = result.content.abstract ? toStaticHTML(result.content.abstract) : "";
newsArticle.snippet = newsArticle.abstract;
newsArticle.byline = result.content.byline ? parseTextContent(result.content.byline) : "";
if (type === "money") {
newsArticle.kicker = newsArticle.byline; //byline is bound to kicker since EC does not render byline
}
var publicationDate = result.content.publicationDate;
if (publicationDate) {
newsArticle.updatedTime = getDateString(publicationDate.updated);
newsArticle.publishTime = getDateString(publicationDate.published);
}
newsArticle.template = result.template;
newsArticle.templateClass = result.templateClass;
if (!isNonEmptyString(newsArticle.headline)) {
return null;
}
return newsArticle;
}
/// <summary>
/// Blocks non-numerical input to the textbox
/// </summary>
function isNumber(evt) {
var charCode = (evt.which) ? evt.which : event.keyCode;
//charcode 13 - Enter key. It is blocked as it should not submit the page. Enter key should have no functionality
if (charCode === 13) {
return false;
}
//current culture's decimal character's code is allowed
var decimal = CollectionProvider.Formatters.getDecimalCharacter();
if (charCode === decimal.charCodeAt(0)) {
return true;
}
//charcode < 31 are control keys like backspace, del, etc which are allowed. Charcode between 48 and 57 are numbers and hence they are allowed. All other characters are blocked at the entry level
if (charCode > 31 && (charCode < 48 || charCode > 57)) {
return false;
}
return true;
}
function redrawChart(chartElmSelector) {
var chartDiv = document.querySelector(chartElmSelector);
if (chartDiv) {
var chartControl = chartDiv.winControl;
chartDiv.innerHTML = "";
chartControl.redraw();
}
}
//this function finds whether a given url is absolute url or a relative url
//this works on the logic that relative urls will always start with string "//"
function isAbsoluteURL(destinationURL) {
return !(destinationURL.indexOf("//") === 0);
}
function disableEdgies() {
var edgies = document.querySelectorAll(".win-appbar");
for (var i = 0; i < edgies.length; i++) {
edgies[i].winControl.disabled = true;
}
}
function enableEdgies() {
var edgies = document.querySelectorAll(".win-appbar");
for (var i = 0; i < edgies.length; i++) {
edgies[i].winControl.disabled = false;
}
}
function isMarketSupportedForBingNews(mkt) {
var currMkt = mkt;
if (!currMkt) {
currMkt = FinanceAppJS.Globalization.getMarket();
}
var bingNewsMarketList = Platform.Configuration.ConfigurationManager.custom.getDictionary("NewsFromBingMarketList").getString(currMkt.toLowerCase());
if (!bingNewsMarketList || bingNewsMarketList === "") {
return false;
}
else {
return true;
}
}
function makeObjectKeysToLowerCase(obj) {
if (obj) {
for (var key in obj) {
if (obj.hasOwnProperty(key) && typeof key === "string") {
obj[key.toLowerCase()] = obj[key];
}
}
}
return obj;
}
function getCurrencyChangeForReverseRatio(price, change) {
var prevPrice = price - change;
var prevRate = 1 / prevPrice;
var ch = ((1 / price) - prevRate);
var pc = ((ch / prevRate) * 100);
return { Ch: ch, Pc: pc };
}
// This method creates a temporary div apply the color and reads the computed color back 
// to know how a particular color is represented in current Contrast scheme.
// For example:
//          if you pass this function '#ffffff' (white) color in HighContrast Black scheme 
//          this will return black. But if current scheme is HighContrst White it will return white.
function getColorForCurrentContrast(color) {
var tempDiv = document.createElement("DIV");
tempDiv.style.color = color;
return window.getComputedStyle(tempDiv).color;
}
function prefetchCurrencies() {
var currencyParams = FinanceAppJS.CurrencyInfo.getLocalCurrencyParameters();
var queryService = new PlatformJS.DataService.QueryService("GlobalCurrency");
var param = PlatformJS.Collections.createStringDictionary();
param.insert("localCurrency", currencyParams.localCurrency);
var prefetchOptions = new Platform.DataServices.PrefetchQueryServiceOptions();
prefetchOptions.priority = Platform.DataServices.QueryServicePriority.low;
queryService.prefetchData(param, null, null, prefetchOptions, null, null);
}
function getMarketMoversType(moversType) {
moversType = (moversType && moversType.toLowerCase()) || "";
var idConstants = FinanceAppJS.Constants.IDs;
var mmConstants = FinanceAppJS.Constants.MarketMovers;
switch (moversType) {
case mmConstants.EtfType:
return idConstants.Etf;
case mmConstants.EquityType:
default:
return idConstants.Stock;
}
}
function updateLiveTile() {
try{
var notifications = Windows.UI.Notifications;
var recurrence = utils.GetNotificationFrequency();
var tickers = FinanceAppJS.Globalization.getHomePageIndices();
var urisToPoll = tickers.map(function (tkr) { return utils.createTileNotificationUri(tkr.symbol, 0, 0); });
//put the hero image
if (!Platform.Utilities.Globalization.isFeatureEnabled("Unsupported") && Platform.Utilities.Globalization.isFeatureEnabled("NewsFromCMSEditorial")) {
var liveTileUrlString = PlatformJS.Services.appConfig.getDictionary("PullNotification").getString("HeroLiveTileUri");
liveTileUrlString = liveTileUrlString.replace(/\{market\}/g, FinanceAppJS.Globalization.getMarket());
urisToPoll.push(new Windows.Foundation.Uri(liveTileUrlString));
}
notifications.TileUpdateManager.createTileUpdaterForApplication().clear();
notifications.TileUpdateManager.createTileUpdaterForApplication().enableNotificationQueue(true);
notifications.TileUpdateManager.createTileUpdaterForApplication().startPeriodicUpdateBatch(urisToPoll, recurrence);
}
catch (err) {
}
}
function clearCurrencyConfig() {
FinanceAppJS.CurrencyInfo.currencyList = null;
FinanceAppJS.CurrencyInfo.currencyInfo = null;
FinanceAppJS.Controls.CurrencyPopulator.sortedCurrencyList = null;
}
function tryAttachHelpButtonAction() {
var helpButton = document.getElementById("helpButton");
var instrumentId = null;
if (helpButton && helpButton.winControl) {
helpButton.winControl.label = CommonJS.resourceLoader.getString("/platform/HelpLabel");
helpButton.addEventListener("click", function (e) {
if (helpButton.winControl.InstrumentId) {
instrumentId = helpButton.winControl.InstrumentId;
}
navigationHelper.navigateToHelpPage(null, instrumentId);
});
}
}
function showSearchPane() {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.Cluster.CHART, FinanceAppJS.Telemetry.Constants.Location.SEARCHQUOTE, "");
var searchPane = Windows.ApplicationModel.Search.SearchPane.getForCurrentView();
if (!showSearchPane.visible) {
FinanceAppJS.Telemetry.FlightRecorder.LogAppEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.AppUtility.SEARCHCHARM,
"");
searchPane.show("");
}
}
function getIndexDetails(tileData, source) {
var entityItem = {};
switch (source) {
case "ChartData":
entityItem.entity = {
displayName: tileData.displayName,
indexSymbol: tileData.indexSymbol,
ExchCode: tileData.E2,
};
break;
case "WorldMarketData":
entityItem.entity = {
displayName: tileData.nm,
indexSymbol: tileData.idx,
ExchCode: tileData.E2 || {},
};
break;
case "IndexTileData":
entityItem.entity = {
displayName: tileData.Cmp,
indexSymbol: getIndicesSymbol(tileData),
ExchCode: tileData.E2 || {},
};
break;
}
return entityItem;
}
/* used for setting partner pano item attributes. */
function setSourceItemAttributes(channelInfo, sourceItem) {
if (channelInfo && sourceItem) {
channelInfo.panoTitle = sourceItem.displayname;
channelInfo.panoDescription = sourceItem.description ? sourceItem.description : "";
channelInfo.clusterlogo = sourceItem.clusterlogo;
channelInfo.featured_url = sourceItem.featuredurl;
channelInfo.ad_unit_id = sourceItem.adunitid;
channelInfo.instrumentation_id = sourceItem.instrumentationid;
channelInfo.alphaorder = sourceItem.alphaorder;
}
}
function importStyleSheet(styleSheet) {
var link = document.createElement("link");
link.rel = "stylesheet";
link.href = styleSheet;
document.head.appendChild(link);
}
function disposeHeaderControl() {
var header = document.body.querySelector(".immersiveHeader");
if (header && header.winControl) {
header.winControl.dispose();
header.winControl = null;
header.innerHTML = "";
}
}
function getTempClusterConfig() {
return {
clusterTitle: utils.getResourceString("GSClusterHeader"),
clusterKey: "tempCluster",
isTempCluster: true,
clusterControl: "FinanceAppJS.Controls.TempCluster",
dataProviders: {
TempClusterProvider: {isPreview: FinanceAppJS.Settings.cmsPreviewEnabled}
},
validateFunc: function validateFunc() {
return (CommonJS.TempCluster.shouldShowTempCluster() && !Windows.Storage.ApplicationData.current.localSettings.values[FinanceAppJS.DataProvider.TempClusterProvider.clusterValidationLocalKey]);
},
platformInvisibleCluster: true,
platformInvisiblePermanentCluster: true,
postDataValidateFunc: function postDataValidateFunc(data) {
if (Windows.Storage.ApplicationData.current.localSettings.values[FinanceAppJS.DataProvider.TempClusterProvider.clusterValidationLocalKey]) {
return "hide";
}
return data && data.data && data.data.contentSlots;
}
};
}
function getNextStepsClusterConfig() {
return {
clusterTitle: utils.getResourceString("NSClusterHeader"),
clusterKey: "nextStepsCluster",
isLastCluster: true,
isImmersive: true,
platformInvisibleCluster: true,
platformInvisiblePermanentCluster: true,
clusterControl: "FinanceAppJS.Controls.NextStepsCluster",
dataProviders: {
NextStepsClusterProvider: {isPreview: FinanceAppJS.Settings.cmsPreviewEnabled}
},
validateFunc: function validateFunc() {
return PlatformJS.Services.configuration.getBool("EnableNextSteps");
},
postDataValidateFunc: function postDataValidateFunc(data) {
return data && data.data && data.data.contentSlots;
}
};
}
function getHomePageWLDisplayCount() {
/// <summary>
/// No. of Watchlist items to show on Home Page
/// </summary>
var watchListClusterHeight = document.body.offsetHeight - 150; /* see markettoday.css*/
var noOfTilesPerColumn = Math.floor(watchListClusterHeight / 150); // 140 - tile + 10 -margin
var noOfTiles = 4 * noOfTilesPerColumn; // 4 columns
return noOfTiles - 1; // 1- for add tile        
}
function getCompanyProfileDescForCurrentRes(companyProfileDesc) {
var noOfChars = 450;
var screenHeight = document.body.offsetHeight;
if (screenHeight >= 1600) {
noOfChars = 2800;
} else if (screenHeight >= 1440) {
noOfChars = 2400;
} else if (screenHeight >= 1200) {
noOfChars = 1700;
} else if (screenHeight >= 1050) {
noOfChars = 1100;
} else if (screenHeight >= 900) {
noOfChars = 1000;
} else if (screenHeight >= 768) {
noOfChars = 450;
} else {
noOfChars = 350;
}
if (!companyProfileDesc) {
return "";
}
if(companyProfileDesc.length < noOfChars) {
return companyProfileDesc;
}
var descriptionChar = companyProfileDesc[noOfChars];
while (noOfChars > 0 && ((descriptionChar >= 'a' && descriptionChar <= 'z') || (descriptionChar >= 'A' && descriptionChar <= 'Z') || descriptionChar === '-')) {
descriptionChar = companyProfileDesc[--noOfChars];
}
companyProfileDesc = companyProfileDesc.substring(0, noOfChars) +  "&hellip;";
return utils.addLtrMark(companyProfileDesc) ;
}
function showBATSDisclaimerForCountry(countryCode) {
var showDisclaimer = false;
var config = FinanceAppJS.ConfigManager.getConfigData("GlobalizationData");
if (config.hasOwnProperty(countryCode)) {
if (config[countryCode].hasOwnProperty("showBATSDisclaimer")) {
showDisclaimer = config[countryCode].showBATSDisclaimer;
}
}
return showDisclaimer;
}
WinJS.Namespace.define("utils", {
getRangeIndicatorPos: getRangeIndicatorPos,
createElements: createElements,
createElement: createElement,
setHighlight: setHighlight,
removeHighlight: removeHighlight,
inArray: inArray,                    // Search one-dimentional array and returns position if element is found
formatVolume: formatVolume,          // Formats volume field
isValidStock: isValidStock,
isValidFund: isValidFund,
trim: trim,
roundFloat: roundFloat,
dateTimeFormat: dateTimeFormat,
toTitleCase: toTitleCase,
dateFormat: dateFormat,
timeFormat: timeFormat,
dateUtil: dateUtil,
timeUtil: timeUtil,
contractFormat: contractFormat,
getGlobalInstrumentSymbol: getGlobalInstrumentSymbol,
getFundUniqueStatus: getFundUniqueStatus,
formatNumber: formatNumber,
formatString: formatString,
getPointForSecondaryTile: getPointForSecondaryTile,
formatValue: formatValue,
addAboutPageFragment: addAboutPageFragment,
removeAboutPageFragment: removeAboutPageFragment,
getResourceString: getResourceString,
getDateRangeButtonName: getDateRangeButtonName,
populateMoversData: populateMoversData,
getSharedContractDataForStocks: getSharedContractDataForStocks,
getUrlParameters: getUrlParameters,
deepCopy: deepCopy,
copyArrayToList: copyArrayToList,
isNonEmptyNumber: isNonEmptyNumber,
isNonEmptyString: isNonEmptyString,
isNonEmptyObject: isNonEmptyObject,
isNonEmptyArray: isNonEmptyArray,
getDeepObject: getDeepObject,
setDeepObject: setDeepObject,
createImmediatePromise: createImmediatePromise,
getShareContractSubject: getShareContractSubject,
getShareContractHtmlFormat: getShareContractHtmlFormat,
shareContractFormatHelperRTL: shareContractFormatHelperRTL,
getShareContractTextFormat: getShareContractTextFormat,
JsLessVariables: JsLessVariables,
getColorWithOpacity: getColorWithOpacity,
normalizeDate: normalizeDate,
createTileNotificationUri: createTileNotificationUri,
GetNotificationFrequency: GetNotificationFrequency,
addLtrMark: addLtrMark,
removeLtrMark: removeLtrMark,
timeZoneFormat: timeZoneFormat,
timeZoneFormatNoLabel: timeZoneFormatNoLabel,
indexChartLinkFormat: indexChartLinkFormat,
isStockSymbol: isStockSymbol,
isEtfSymbol: isEtfSymbol,
isFundSymbol: isFundSymbol,
isIndexSymbol: isIndexSymbol,
getAutoSuggestDisplaySymbol: getAutoSuggestDisplaySymbol,
getAutoSuggestDisplayName: getAutoSuggestDisplayName,
addCustomAriaLabelToTiles: addCustomAriaLabelToTiles,
setWinLabelText: setWinLabelText,
vAlignContainerInMiddle: vAlignContainerInMiddle,
launchNewsUri: launchNewsUri,
getArticleReaderConfig: getArticleReaderConfig,
parseMagazineResponse: parseMagazineResponse,
convertCMSArticle: convertCMSArticle,
formatLastUpdatedTime: formatLastUpdatedTime,
isNumber: isNumber,
redrawChart: redrawChart,
isAbsoluteURL: isAbsoluteURL,
disableEdgies: disableEdgies,
enableEdgies: enableEdgies,
getArticleReaderId: getArticleReaderId,
isMarketSupportedForBingNews: isMarketSupportedForBingNews,
makeObjectKeysToLowerCase: makeObjectKeysToLowerCase,
getCurrencyChangeForReverseRatio: getCurrencyChangeForReverseRatio,
getColorForCurrentContrast: getColorForCurrentContrast,
prefetchCurrencies: prefetchCurrencies,
getMarketMoversType: getMarketMoversType,
updateLiveTile: updateLiveTile,
clearCurrencyConfig: clearCurrencyConfig,
getIndicesSymbol: getIndicesSymbol,
tryAttachHelpButtonAction: tryAttachHelpButtonAction,
isEODInstrument: isEODInstrument,
getDefaultConfigForIndices: getDefaultConfigForIndices,
getIndexDetails: getIndexDetails,
isOTCInstrument: isOTCInstrument,
setSourceItemAttributes: setSourceItemAttributes,
showSearchPane: showSearchPane,
importStyleSheet: importStyleSheet,
disposeHeaderControl: disposeHeaderControl,
getTempClusterConfig: getTempClusterConfig,
getNextStepsClusterConfig: getNextStepsClusterConfig,
getHomePageWLDisplayCount: getHomePageWLDisplayCount,
getCompanyProfileDescForCurrentRes: getCompanyProfileDescForCurrentRes,
showBATSDisclaimerForCountry: showBATSDisclaimerForCountry
});
WinJS.Namespace.define("FinanceAppJS.CurrencyInfo", {
currencyInfo: null,
currencyList: null,
getCurrencyInfo: function getCurrencyInfo() {
if (!FinanceAppJS.CurrencyInfo.currencyInfo) {
FinanceAppJS.CurrencyInfo.currencyInfo = FinanceAppJS.ConfigManager.getConfigData("CurrencyInfo");
}
return FinanceAppJS.CurrencyInfo.currencyInfo;
},
getCurrencyList: function getCurrencyList() {
if (!FinanceAppJS.CurrencyInfo.currencyList) {
FinanceAppJS.CurrencyInfo.currencyList = FinanceAppJS.ConfigManager.getConfigData("CurrencyList");
}
return FinanceAppJS.CurrencyInfo.currencyList;
},
getLocalCurrencyParameters: function getLocalCurrencyParameters() {
var currency = FinanceAppJS.Globalization.getDefaultCurrency();
var currencyInfo = FinanceAppJS.CurrencyInfo.getCurrencyInfo();
var fallback = FinanceAppJS.CurrencyFlatRates.DefaultCurrency();
if (!currencyInfo[currency]) {
currency = fallback;
}
var convertTo = null;
if (currency === fallback) {
convertTo = FinanceAppJS.CurrencyFlatRates.CurrencyRight();
}
else {
convertTo = fallback;
}
var convertFrom = currency;
var parameters = {};
parameters.localCurrency = currency;
parameters.convertFrom = convertFrom;
parameters.convertTo = convertTo;
return parameters;
}
});
WinJS.Namespace.define('FinanceAppJS.Utils.Logger', {
LogLevels: { Info: 'info', Debug: 'debug', Warning: 'warning', Error: 'error' },
PerformanceMarks: { Start: 's', End: 'e', Progress: 'p' },
_logName: "Finance",
Log: function Log(pageName, functionName, logLevel, message) {
/// <summary>
/// This function logs the messages to console or ETW.
/// example usage: Log("MarketToday.js", "constructor", LogHelper.logLevels.debug, "initialization successful");
/// </summary>
/// <param name="pageName" mayBeNull="false" optional="false" type="String">Page Name</param>
/// <param name="functionName" mayBeNull="false" optional="false" type="String">Function Name</param>
/// <param name="logLevel" mayBeNull="false" optional="false" type="FinanceAppJS.Utils.LogLevels" >
/// logLevels: { Info: 'info', Debug: 'debug', Warning: 'warning', Error: 'error' },
/// </param>
/// <param name="message" mayBeNull="false" optional="false" type="String">Message to be logged</param>
console.log(this._logName + ":" + pageName + ":" + functionName + ":" + logLevel + ":" + message);
},
LogPerformance: function LogPerformance(pageName, functionName, performanceMark) {
/// <summary>
/// this function logs the performance marks to console or ETW.
/// example usage: LogPerformance("MarketToday.js", "constructor", LogHelper.performanceMarks.start);
/// </summary>
/// <param name="pageName" mayBeNull="false" optional="false" type="String">Page Name</param>
/// <param name="functionName" mayBeNull="false" optional="false" type="String">Function Name</param>
/// <param name="performanceMark" mayBeNull="false" optional="false" type="FinanceAppJS.Utils.PerformanceMarks" >
/// Flag to indicate a start event or an end event
/// </param>
/* Uncomment the following lines to get the Time *
this._activatedTime = this._activatedTime || (new Date()).getTime();
var delta = (new Date()).getTime() - this._activatedTime;
console.log("TimeStamp:" + delta + " :: " + this._logName + " : " + pageName + " : " + functionName + " : " + performanceMark);
/* Debug Code END */
msWriteProfilerMark(this._logName + ":" + pageName + ":" + functionName + ":" + performanceMark);
}
});
WinJS.Namespace.define('FinanceAppJS.Utils.ResumeHandler', {
refreshCallback: null,
suspendCallback: null,
setRefreshCallback: function setRefreshCallback(refreshFunction) {
this.refreshCallback = refreshFunction;
},
clearRefreshCallback: function clearRefreshCallback() {
this.refreshCallback = null;
},
setSuspendCallback: function setSuspendCallback(suspendFunction) {
this.suspendCallback = suspendFunction;
},
clearSuspendCallback: function clearSuspendCallback() {
this.suspendCallback = null;
},
});
WinJS.Namespace.define('FinanceAppJS.Utils', {
semanticZoomTile: function semanticZoomTile(clusterTitle) {
var binding = {
clusterTitle: clusterTitle,
moduleInfo: {
fragmentPath: '/shared/html/templates.html',
templateId: 'semanticZoomTile',
}
};
return binding;
},
ShowNewsCluster: function ShowNewsCluster() {
var newsDisplay = FinanceAppJS.Globalization.getNewsClusterDisplay();
var value = newsDisplay.showNewsCluster;
return value;
},
UseArticleReader: function UseArticleReader() {
var newsDisplay = FinanceAppJS.Globalization.getNewsClusterDisplay();
var value = newsDisplay.useArticleReader;
return value;
},
_spriteCountryList: [
"NONE", "AUS", "AUT", "BEL", "CAN", "CHL", "CHN", "CYP", "DNK", "FIN", "FRA",
"DEU", "GRC", "HKG", "ISL", "IND", "IRL", "ITA", "JPN", "KOR", "LUX",
"MYS", "MEX", "NLD", "NZL", "NOR", "PHL", "PRT", "ROU", "RUS", "SGP",
"ZAF", "ESP", "SWE", "CHE", "TWN", "THA", "GBR", "USA", "ARG", "BRA",
"CZE", "DOM", "EGY", "EUR", "HUN", "IDN", "ISR", "PAK", "POL", "TUR",
"ARE", "MAC"],
getCountryFlagClass: function getCountryFlagClass(cntry) {
//If country is Taiwan, never show the flag (Geopol issue)
if (cntry === "TWN") {
return "countryFlagSprite_NONE staticFlagClass";
}
if (!document.getElementById("countryFlagSpriteStyleSheet")) {
var countryList = FinanceAppJS.Utils._spriteCountryList;
var stylesheet = utils.createElement(document.head, "style", { id: "countryFlagSpriteStyleSheet", type: 'text/css' });
var cssRules = [];
var staticRule = ".staticFlagClass { background-color:transparent; width: 32px; height: 31px; overflow:hidden; margin-top:-10px; direction:ltr;}";
cssRules.push(staticRule);
for (var i = 0, country; i < countryList.length && (country = countryList[i]) ; i++) {
var rule = ".countryFlagSprite_" + country + ":before {content:url('/shared/images/FlagSprite.png'); margin-left: " + (-30 - i * 60 + 1).toString() + "px; margin-top:-10px;}";
cssRules.push(rule);
}
var textNode = document.createTextNode(cssRules.join("\n"));
stylesheet.appendChild(textNode);
}
var code = FinanceAppJS.Utils._spriteCountryList.indexOf(cntry) === -1 ? "NONE" : cntry;
return "countryFlagSprite_" + code + " staticFlagClass";
},
//// handles bottom edgy disabling
disableBottomEdgy: function disableBottomEdgy() {
var bottomEdgy = document.querySelector(".entityappbar");
if (bottomEdgy) {
bottomEdgy.disabled = true;
}
},
});
WinJS.Namespace.define('FinanceAppJS.Utils.Navigator', {
arrowKeyDown: function arrowKeyDown(event) {
var keyCode = event.keyCode;
var ne = null;
switch (keyCode) {
case WinJS.Utilities.Key.upArrow:
if (event.srcElement.previousElementSibling) {
for (ne = event.srcElement.previousElementSibling; ne && ne.nodeName !== event.srcElement.nodeName;) {
ne = ne.previousElementSibling;
}
if (ne) {
event.srcElement.tabIndex = -1;
ne.tabIndex = 0;
ne.focus();
}
}
/*we dont want to scroll the screen on first and last instance of rows.*/
event.stopImmediatePropagation();
event.preventDefault();
break;
case WinJS.Utilities.Key.downArrow:
if (event.srcElement.nextElementSibling) {
for (ne = event.srcElement.nextElementSibling; ne && ne.nodeName !== event.srcElement.nodeName;) {
ne = ne.nextElementSibling;
}
if (ne) {
event.srcElement.tabIndex = -1;
ne.tabIndex = 0;
ne.focus();
}
}
/*we dont want to scroll the screen on first and last instance of rows.*/
event.stopImmediatePropagation();
event.preventDefault();
break;
case WinJS.Utilities.Key.leftArrow:
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.LEFT);
event.stopImmediatePropagation();
event.preventDefault();
break;
case WinJS.Utilities.Key.rightArrow:
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.RIGHT);
event.stopImmediatePropagation();
event.preventDefault();
break;
default:
break;
}
},
leftRightArrowKeyDown: function leftRightArrowKeyDown(event, isChartButton, isGeneralPage) {
if (!event) {
return;
}
var keyCode = event.keyCode;
var ne = null;
switch (keyCode) {
case WinJS.Utilities.Key.leftArrow:
{
if (isChartButton) {
if (event.srcElement.previousElementSibling) {
for (ne = event.srcElement.previousElementSibling; ne && ne.nodeName !== event.srcElement.nodeName;) {
ne = ne.previousElementSibling;
}
if (ne) {
event.srcElement.tabIndex = -1;
ne.tabIndex = 0;
ne.focus();
}
}
else {
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.LEFT);
}
}
else if (isGeneralPage) {
if (event.srcElement.previousElementSibling) {
for (ne = event.srcElement.previousElementSibling; ne && ne.nodeName !== event.srcElement.nodeName;) {
ne = ne.previousElementSibling;
}
if (ne) {
event.srcElement.tabIndex = -1;
ne.tabIndex = 0;
ne.focus();
}
}
else {
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement.parentElement, CommonJS.NavigableView.Direction.LEFT);
}
}
}
break;
case WinJS.Utilities.Key.rightArrow:
{
if (isChartButton) {
if (event.srcElement.nextElementSibling) {
for (ne = event.srcElement.nextElementSibling; ne && ne.nodeName !== event.srcElement.nodeName;) {
ne = ne.nextElementSibling;
}
if (ne) {
event.srcElement.tabIndex = -1;
ne.tabIndex = 0;
ne.focus();
}
}
else {
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.RIGHT);
}
}
else if (isGeneralPage) {
if (event.srcElement.nextElementSibling) {
for (ne = event.srcElement.nextElementSibling; ne && ne.nodeName !== event.srcElement.nodeName;) {
ne = ne.nextElementSibling;
}
if (ne) {
event.srcElement.tabIndex = -1;
ne.tabIndex = 0;
ne.focus();
}
}
else {
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement.parentElement, CommonJS.NavigableView.Direction.RIGHT);
}
}
}
break;
case WinJS.Utilities.Key.pageDown:
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.RIGHT);
break;
case WinJS.Utilities.Key.pageUp:
CommonJS.NavigableView.dispatchBoundaryEvent(event.srcElement, CommonJS.NavigableView.Direction.LEFT);
break;
}
},
checkKeyPressed: function checkKeyPressed(keycode) {
if (!keycode) {
return false;
}
if (keycode === WinJS.Utilities.Key.space || keycode === WinJS.Utilities.Key.enter || keycode === WinJS.Utilities.Key.tab || keycode === WinJS.Utilities.Key.shift) {
return true;
}
return false;
},
focusRow: function focusRow(event) {
WinJS.Utilities.addClass(event.srcElement, "tblRowFocus");
},
blurRow: function blurRow(event) {
WinJS.Utilities.removeClass(event.srcElement, "tblRowFocus");
},
linkNavigation: function linkNavigation(event) {
if (event.keyCode === WinJS.Utilities.Key.space || event.keyCode === WinJS.Utilities.Key.enter) {
var uri = new Windows.Foundation.Uri(event.srcElement.href);
//LogAppEvent: function (page, utility, data) {
Windows.System.Launcher.launchUriAsync(uri).then();
}
},
enableArrowKeyAccessibilty: function enableArrowKeyAccessibilty() {
var docObj = document.querySelectorAll(".platformClusterContent");
if (docObj) {
if (docObj.length > 0) {
for (var j = 0; j < docObj.length; j++) {
docObj[j].addEventListener("keydown", FinanceAppJS.Utils.Navigator.arrowKeyDown);
}
}
}
},
});
WinJS.Namespace.define('FinanceAppJS.Utils.Formatters', {
formatVolume: function formatVolume(value) {
if ((!value) || (typeof value === "undefined")) { return; }
var append = "";
if (value > 1000000000) {
value /= 1000000000;
append = utils.getResourceString("B");
}
else if (value > 1000000) {
value /= 1000000;
append = utils.getResourceString("M");
}
else if (value > 1000) {
value /= 1000;
append = utils.getResourceString("K");
}
return roundFloat(value, 2) + append;
},
//Function used by tools. Intended to round of the numbers if bigger than 10 digits
formatOutputNumber: function formatOutputNumber(value) {
if ((!value) || (typeof value === "undefined")) { return; }
var append = "";
var numberFormat = FinanceAppJS.Utils.Formatters.numberFormatter(value,0);
if (numberFormat.toString().length > 10) {
if (value > 1000000000000) {
value /= 1000000000000;
append = utils.getResourceString("T");
}
else if (value > 1000000000) {
value /= 1000000000;
append = utils.getResourceString("B");
}
else if (value > 1000000) {
value /= 1000000;
append = utils.getResourceString("M");
}
return CollectionProvider.Formatters.formatNumber(value, 2) + append;
}
else {
return numberFormat;
}
},
//Function used by tools. Intended to avoid the minus sign when we want to round of numbers like -0.0005
toolsNumberFormatter: function toolsNumberFormatter(num, maxdigit, doNotUseLTRMarker) {
var digits = (maxdigit !== null && !isNaN(maxdigit)) ? maxdigit : 2;
var formattedValue;
var value = 0;
if (typeof num === "string") {
value = parseFloat(num);
}
else {
value = num;
}
if (value > 0) {
formattedValue = CollectionProvider.Formatters.formatNumber(value, digits);
}
else if (value < 0) {
formattedValue = CollectionProvider.Formatters.formatNumber(value * (-1), digits);
if (formattedValue !== "0") {
formattedValue = "-" + formattedValue;
}
}
else {
formattedValue = CollectionProvider.Formatters.formatNumber(value, digits);
}
if (!doNotUseLTRMarker) {
formattedValue = utils.addLtrMark(formattedValue);
}
return formattedValue;
},
isCurrencyDecimalIgnored: function isCurrencyDecimalIgnored(market, exchange) {
if ((exchange && FinanceAppJS.Globalization.isLocalExchange(exchange)) && (market === "JA-JP" || market === "KO-KR")) {
return true;
}
else {
return false;
}
},
/// <summary>Convert a number percent into display in globalized format</summary>
/// <param name="num" type="Float">The percentage value</param>
/// <param name="maxdigit" type="int">number of digits after decimal</param>
/// <param name="usePositivePrefix" type="bool">should the value be prefixed with + sign if value is > 0 </param>
/// <param name="fallbackValue" type="string">fallback string to be sent if the num is undefined</param>
numberFormatter: function numberFormatter(num, maxdigit, usePositivePrefix, fallbackValue, doNotUseLTRMarker) {
if (num === "" || typeof num === "undefined" || num === null || isNaN(num)) {
return (typeof fallbackValue !== "undefined") ? fallbackValue : "-";
}
var value = 0;
if (typeof num === "string") {
value = parseFloat(num);
}
else {
value = num;
}
var digits = maxdigit !== null && !isNaN(maxdigit) ? maxdigit : 2;
var formattedValue;
if (usePositivePrefix) {
if (value > 0) {
formattedValue = CollectionProvider.Formatters.formatNumber(value, digits);
formattedValue = "+" + formattedValue;
}
else if (value < 0) {
formattedValue = CollectionProvider.Formatters.formatNumber(value * (-1), digits);
formattedValue = "-" + formattedValue;
}
else {
formattedValue = CollectionProvider.Formatters.formatNumber(value, digits);
}
}
else {
if (value > 0) {
formattedValue = CollectionProvider.Formatters.formatNumber(value, digits);
}
else if (value < 0) {
formattedValue = CollectionProvider.Formatters.formatNumber(value * (-1), digits);
formattedValue = "-" + formattedValue;
}
else {
formattedValue = CollectionProvider.Formatters.formatNumber(value, digits);
}
}
if (!doNotUseLTRMarker) {
formattedValue = utils.addLtrMark(formattedValue);
}
return formattedValue;
},
/// <summary>Convert a number percent into display in globalized format</summary>
/// <param name="num" type="Float">The percentage value</param>
/// <param name="maxdigit" type="int">number of digits after decimal</param>
/// <param name="usePositivePrefix" type="bool">should the value be prefixed with + sign if value is > 0 </param>
/// <param name="fallbackValue" type="string">fallback string to be sent if the num is undefined</param>
percentFormatter: function percentFormatter(num, maxdigit, usePositivePrefix, fallbackValue, doNotUseLTRMarker) {
if (num === "" || typeof num === "undefined" || num === null || isNaN(num)) {
return (typeof fallbackValue !== "undefined") ? fallbackValue : "-";
}
var value = 0;
if (typeof num === "string") {
value = parseFloat(num);
}
else {
value = num;
}
var digits = (maxdigit !== null && !isNaN(maxdigit)) ? maxdigit : 2;
var formattedValue = CollectionProvider.Formatters.formatPercent(value, digits);
if (usePositivePrefix && value > 0) {
formattedValue = "+" + formattedValue;
}
if (!doNotUseLTRMarker) {
formattedValue = utils.addLtrMark(formattedValue);
}
return formattedValue;
},
/// <summary>Convert a Given Currency amount to display in globalized format</summary>
/// <param name="amount" type="Float">The Amount</param>
/// <param name="currency" type="String" default="USD">currency belongs to</param>
/// <param name="showCurrencySymbol" type="bool" default="false">parameter to specify if the return value should have currency symbol or not</param>
stringFormatter: function stringFormatter(format, args) {
var formattedValue = CollectionProvider.Formatters.formatString(format, args);
return formattedValue;
},
/// <summary>Convert a Given Currency amount to display in globalized format</summary>
/// <param name="amount" type="Float">The Amount</param>
/// <param name="currency" type="String" default="USD">currency belongs to</param>
/// <param name="showCurrencySymbol" type="bool" default="false">parameter to specify if the return value should have currency symbol or not</param>
currencyFormatter: function currencyFormatter(amount, currency, showCurrencySymbol, decimalPlaces) {
var currencyCulture = FinanceAppJS.Globalization.getCultureForCurrency(currency ? currency : "USD");
if (!decimalPlaces && decimalPlaces !== 0) {
decimalPlaces = -1;
}
var formattedValue = CollectionProvider.Formatters.formatCurrency(amount, currencyCulture, showCurrencySymbol, decimalPlaces);
return formattedValue;
},
/// <summary>Gets the Currency Symbol</summary>
getCurrencySymbol: function getCurrencySymbol() {
var currency = currency || FinanceAppJS.Globalization.getDefaultCurrency();
var currencyCulture = FinanceAppJS.Globalization.getCultureForCurrency(currency ? currency : "USD") || "en-US";
return CollectionProvider.Formatters.getCurrencySymbol(currencyCulture);
},
formatString: function formatString(val, retVal) {
// if val is present return the value
// else if retVal is present  ||  empty string , return retVal
// else return -
var ret = null;
if (val && val !== "") {
ret = val;
}
else {
ret = retVal ? retVal : "-";
}
return ret;
},
lastUpdateTimeStampFormatter: function lastUpdateTimeStampFormatter(date, time, zone, retVal) {
var ret = null;
if (!date || !time || !zone) {
ret = retVal ? retVal : "-";
}
else{
ret = FinanceAppJS.Utils.Formatters.shortDateFormat(date) + " " + FinanceAppJS.Utils.Formatters.shortTimeFormat(time) + " " + FinanceAppJS.Utils.Formatters.formatString(zone);
}
return ret;
},
formatValue: function formatValue(prepend, value, append) {
if (typeof value === 'undefined') { return "-"; }
if (value === "" || value === null || value === undefined || value === "-") {
return "-";
}
else {
return prepend + value + append;
}
},
trim: function trim(stringToTrim) {
return stringToTrim.replace(/^\s+|\s+$/g, "");
},
dateFormat: function dateFormat(str) {
if (typeof str === 'undefined') { return "-"; }
if (str === undefined || !str) { return "-"; }
var dateObject = dateUtil(str);
var date = new Date(dateObject.year, dateObject.month, dateObject.date, 0, 0, 0, 0);
var mydatefmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month day year");
var mydate = mydatefmt.format(date);
return mydate;
},
longDateFormat: function longDateFormat(str) {
if (typeof str === 'undefined') { return "-"; }
if (str === undefined || !str) { return "-"; }
var dateObject = dateUtil(str);
//if (dateObject === null) return "-";
if (!dateObject) { return "-"; }
var date = new Date(dateObject.year, dateObject.month, dateObject.date, 0, 0, 0, 0);
return this.longDateFormatDateObj(date);
},
longDateFormatDateObj: function longDateFormatDateObj(date) {
var mydatefmt = Windows.Globalization.DateTimeFormatting.DateTimeFormatter.longDate;
var mydate = mydatefmt.format(date);
return mydate;
},
GetLastUpdatedDate: function GetLastUpdatedDate(date) {
var dateFmt = Windows.Globalization.DateTimeFormatting.DateTimeFormatter.longDate;
var formattedDate = dateFmt.format(date);
var today = new Date();
var todayFmt = dateFmt.format(today);
if (formattedDate.toString() === todayFmt.toString()) {
return FinanceAppJS.Utils.Formatters.shortTimeFormatDateObj(date);
}
else {
var dayMonthYearFmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("day month.full year.full");
var dayMonthYear = dayMonthYearFmt.format(date).toString();
return dayMonthYear;
}
},
shortDateFormat: function shortDateFormat(str) {
if (typeof str === 'undefined') { return "-"; }
if (str === undefined || !str) { return "-"; }
var dateObject = dateUtil(str);
//if (dateObject == null) return "-";
if (!dateObject) { return "-"; }
var date = new Date(dateObject.year, dateObject.month, dateObject.date, 0, 0, 0, 0);
return this.shortDateFormatDateObj(date);
},
shortDateFormatDateObj: function shortDateFormatDateObj(date) {
var mydatefmt = Windows.Globalization.DateTimeFormatting.DateTimeFormatter.shortDate;
var mydate = mydatefmt.format(date);
return mydate;
},
shortTimeFormat: function shortTimeFormat(str) {
if (typeof str === 'undefined') { return "-"; }
if (str === undefined || !str) { return "-"; }
var timeObject = timeUtil(str);
//if (timeObject == null) return "-";
if (!timeObject) { return "-"; }
var date = new Date(0, 0, 0, timeObject.hour, timeObject.minute, timeObject.seconds, 0);
return this.shortTimeFormatDateObj(date);
},
shortTimeFormatDateObj: function shortTimeFormatDateObj(date) {
var timeFormat = Windows.Globalization.DateTimeFormatting.DateTimeFormatter.shortTime;
var time = timeFormat.format(date);
return time;
},
longTimeFormatDateObj: function longTimeFormatDateObj(date) {
var timeFormat = Windows.Globalization.DateTimeFormatting.DateTimeFormatter.longTime;
var time = timeFormat.format(date);
return time;
},
contractFormat: function contractFormat(month, year) {
var date = new Date(year, month + 1, 1);
var mydatefmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month.abbreviated year.abbreviated");
var mydate = mydatefmt.format(date);
return mydate + " " + utils.getResourceString("contract");
},
dateTimeFormat: function dateTimeFormat(str) {
if (typeof str === 'undefined') { return "-"; }
if (str === undefined || !str) { return "-"; }
var dateObject = dateUtil(str.substring(0, str.indexOf(" ")));
var timeObject = timeUtil(str.substring(str.indexOf(" ") + 1, str.length));
var date = new Date(dateObject.year, dateObject.month, dateObject.date, timeObject.hour, timeObject.minute, 0, 0);
var mydatefmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month.abbreviated day");
var mytimefmt = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("hour minute");
var mydate = mydatefmt.format(date);
var mytime = mytimefmt.format(date);
return mydate + " " + mytime;
},
toTitleCase: function toTitleCase(str) {
if (str === undefined || !str) { return "-"; }
return str.replace(/\w\S*/g, function (txt) { return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase(); });
},
getEmptyValue: function getEmptyValue(str) {
/// <summary>Retuns the empty value that need to be displayed in the UX</summary>
/// <param name="str" type="String" >String to be checked</param>
/// <returns type="String">Empty formatted String</returns>
if (typeof str === 'undefined') { return "-"; }
if (str === undefined || !str) { return "-"; }
return str;
},
_decimalFormatter: null,
_percentFormatter: null,
_numberFormattingInfo: null,
parseDouble: function parseDouble(str) {
this._decimalFormatter = this._decimalFormatter || new Windows.Globalization.NumberFormatting.DecimalFormatter();
return this._decimalFormatter.parseDouble(str);
},
parseInt: function parseInt(str) {
this._decimalFormatter = this._decimalFormatter || new Windows.Globalization.NumberFormatting.DecimalFormatter();
return this._decimalFormatter.parseInt(str);
},
parsePercent: function parsePercent(str) {
return CollectionProvider.Formatters.parsePercent(str);
},
});
String.prototype.isEmpty = function () {
var str = this;
return (!str || str.length === 0);
};
})();
(function () {
'use strict';
CommonJS.Immersive.PanoramaPanel.prototype._realizeOffscreenItems = function (queue) {
var promises = [], item = queue.shift();
while (item) {
if (!item.element || !item.data || !item.contentElement) {
item = queue.shift();
continue;
}
promises.push(this._realizeItem(item).then(this.dispatchEvent.bind(this, "itemrealizedcomplete", item)));
item = queue.shift();
}
return WinJS.Promise.join(promises);
};
CommonJS.Immersive.PanoramaPanel.prototype.realizeItems = function (items, scrollPosition) {
this._createElements(items, scrollPosition);
var queue = this._rankRealization(items, scrollPosition), promises = [];
for ( var item = queue.onscreen.shift(); item ; item = queue.onscreen.shift() ) {
if (item.data.clusterContent.contentControl === "PlatformJS.Ads.AdsWrapperControl") {
queue.offscreen.push(item);
continue;
}
promises.push(this._realizeItem(item).then(this.dispatchEvent.bind(this, "itemrealizedcomplete", item)));
}
// scroll to target location
if (scrollPosition) {
this.scrollTo(scrollPosition);
}
// To avoid image bleed displaying too early and overlapping hero cluster, 
// we wait and show it after all elements are realized
// TODO: investigate a pure CSS solution
var finalPromise = WinJS.Promise.join(promises);
finalPromise = queue.offscreen.length ? finalPromise.then(this._realizeOffscreenItems.bind(this, queue.offscreen)) : finalPromise;
return finalPromise.then(this.dispatchEvent.bind(this, "itemsrealizedcomplete", items.length));
};
CommonJS.Immersive.PanoramaPanel.prototype._rankRealization = function (items, offset) {
if (typeof offset !== 'number' || offset < ( items[0].offset + items[0].width / 2)) {
return { onscreen: [items[0]], offscreen: items.slice(1) };
}
// the item realization path needs to be re-ordered
// based on the offset.  when we have an offset, we
// we try and render items that would appear on
// screen first.
// sample:
//   - # of items: 6
//   - offset: 3
//   - screen size: 2
// then [1,2,3,4,5,6] => [3,4,2,1,5,6]
var onscreen = [], offscreen = [];
var firstIndex = 0;
var i = 0, ilen = 0;
var width = this._viewport.clientWidth;
for (i = 0, ilen = items.length; i < ilen; i++) {
var item = items[i];
if (item.offset <= offset && (!item.next || (item.next && item.next.offset > offset))) {
firstIndex = i;
break;
}
}
for (i = firstIndex, ilen = items.length; i < ilen && items[i].offset < offset + width; i++) {
onscreen.push(items[i]);
}
for (var j = firstIndex, k = i; j > 0 || k < ilen; j--, k++) {
if (j > 0) { offscreen.push(items[j - 1]); }
if (k < ilen) { offscreen.push(items[k]); }
}
return { onscreen: onscreen, offscreen: offscreen };
};
})();
/* shared\js\AppConstants.js */
(function () {
"use strict";
WinJS.Namespace.define('FinanceAppJS.Constants.HomePage', {
NewsTextCharCount: function NewsTextCharCount() {
/// <summary>
/// Refresh Interval for Index Chart
/// </summary>
return 260;
},
/// <summary>
/// Max No. of indices to show in the home page for hero image and indices cluster
/// </summary>
MaxIndices: 4,
USAcrossMarketCount: 3,
//Adding new constants
ccClusterLength: 2, //Shouldnt include Rates - Only Currency,Commodity as of march-13
USAcrossMarketRatesMaxCount: 12,
USAcrossMarketCurrencyMaxCount: 7,
USAcrossMarketCommoditiesMaxCount: 7,
NonUSCurrenciesCount: 6,
NonUSCommoditiesCount: 4,
availableOnFREChannelIdList: ["Home", "MortgageCalc", "SavingsCalc", "AutoLoanCalc", "RetirementCalc", "FinancialCalc","Tools"]
});
WinJS.Namespace.define('FinanceAppJS.Constants.Tools', {
CompoundingMonthsList: [1, 3, 6, 12],
CompoundingMonthsNames: ["monthly", "quarterly", "halfYearly", "yearly"],
MinScreenWidthForMaxBars: 1366,
MaxBars: 30,
SecondMaxBars: 18,
AllowableDigitsChars: ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'],
// Allow all ctrl keys whose char value is '' and backspace whose char value is'\b', and tab '\t' and enter '\n'
AllowableCtrlKeyChars: ['', '\b', '\t', '\n'],
Currency_converter_Min_Amount: 1,
Currency_converter_Max_Amount: 1000000,
Currency_converter_Max_Tiles_Count: 25,
Currency_converter_Autosuggest_Max_Length: 30,
Currency_converter_Default_From_Selected_Currency: "USD",
decimalChar: CollectionProvider.Formatters.getDecimalCharacter(),
});
WinJS.Namespace.define('FinanceAppJS.Constants.SnappedHomePage', {
/// <summary>
/// No. of Watchlist items to show in Snap Mode
/// </summary>
ShowWatchlistItemCount: 10
});
WinJS.Namespace.define('FinanceAppJS.Constants.MarketPage', {
/// <summary>
/// No. of Watchlist items to show in Snap Mode
/// </summary>
MoversMaxCount: 9
});
WinJS.Namespace.define('FinanceAppJS.Constants.SearchCharm', {
/// <summary>
/// No. of suggestions to show on Search Charm
/// </summary>
NoOfSuggestions: 5
});
WinJS.Namespace.define('FinanceAppJS.Constants.QuickSearch', {
/// <summary>
/// No. of suggestions to show on Flyout
/// </summary>
NoOfSuggestions: 10
});
WinJS.Namespace.define('FinanceAppJS.Constants.SecurityType', {
/// <summary>
/// Security type to be used with the Indices while creating the index Symbol
/// </summary>
Index: "10",
Stock: "1"
});
WinJS.Namespace.define('FinanceAppJS.Constants.AutoSuggest', {
/// <summary>
/// AutoSuggest config variables
/// </summary>
StockSymbolTypes: ["ST"],
EtfSymbolTypes: ["FE","CE"],
FundSymbolTypes: ["FO", "FC"],
IndexSymbolTypes: ["XI"]
});
WinJS.Namespace.define('FinanceAppJS.Constants.MarketMovers', {
/// <summary>
/// MarketMovers config variables
/// </summary>
CustomCacheKey: "MARKETMOVERS",
EquityType: "equity",
EtfType: "etf"
});
WinJS.Namespace.define('FinanceAppJS.Constants.WorldMarket', {
RealTimeSymbolComponents: 3,
});
WinJS.Namespace.define('FinanceAppJS.Constants.Funds', {
FundStyleMap: function FundStyleMap() {
/// <summary>
/// Refresh Interval for Index Chart
/// </summary>
var map = [];
map.push(utils.getResourceString("ETFLarge Value"), utils.getResourceString("ETFLarge Blend"), utils.getResourceString("ETFLarge Growth"), utils.getResourceString("ETFMedium Value"), utils.getResourceString("ETFMedium Blend"), utils.getResourceString("ETFMedium Growth"), utils.getResourceString("ETFSmall Value"), utils.getResourceString("ETFSmall Blend"), utils.getResourceString("ETFSmall Growth"));
return map;
},
DefaultNoDataDisplayValue: "-"
});
WinJS.Namespace.define('FinanceAppJS.CurrencyFlatRates', {
CurrencyRight: function CurrencyRight() {
return "EUR";
},
DefaultCurrency: function DefaultCurrency() {
return "USD";
},
RemovedCurrency: function RemovedCurrency() {
return "AUD";
},
DefaultAmount: function DefaultAmount() {
return 1000.00;
},
});
WinJS.Namespace.define('FinanceAppJS.Constants.Watchlist', {
/// <summary>
/// Watchlist config variables
/// </summary>
CustomCacheKey: "WATCHLIST"
});
WinJS.Namespace.define('FinanceAppJS.Constants.IDs', {
// We still want to have below IDs for backward compatibility
DefaultRealtimeExchangeId: 126,
DefaultSecurityType: 1,
// Below prefixes are used while storing the watchlist entries. for e.g: We will store watch list entry like "S:126.1.MSFT.NAS"
RoamingStockPrefix: 'S:',
RoamingTickerPrefix: 'T:',
RoamingETFPrefix: 'E:',
RoamingSecIdPrefix: 'M:',
RoamingIndexPrefix: 'I:',
// Below IDs are used just to identify the tile type.
AddTile: -1,
Index: 0,
Stock: 1,
Etf: 2,
Fund: 3,
FundTicker: 4,
FundSecid: 5,
WatchList: 9,
WATCHLIST_SYMBOL: "watchlist",
Indices: 50,
NonIndices: 51,
Currencies: 101,
WorldMarket: 102,
MarketToday: 110,
CurrencyConverter: 200
});
WinJS.Namespace.define('FinanceAppJS.Constants.SecondaryTile', {
Logo: 'ms-appx:///shared/images/logo.png',
WideLogo: 'ms-appx:///shared/images/wide.png',
SmallLogo: 'ms-appx:///shared/images/small.png'
});
WinJS.Namespace.define('FinanceAppJS.Constants.RefreshIntervals', {
/// <summary>
/// This class has the constants for all the refresh intervals in the app. All the times are in "SECONDS"
/// </summary>
EntityPanoProvider: 600, /* Refresh Interval is in seconds */
EditorialNewsDataProvider: 600,
HomePageIndexChart: 60,
/// <summary>
/// Refresh Interval for Equity data on home page
/// </summary>
HomePageEquityData: 60,
HomePageICCBEWM: 60,
WatchList: 60,
SnappedView: 60,
/// <summary>
/// Refresh Interval for Bonds
/// </summary>
Bonds: 60,
StockDetailsRT: 60,
//Wathclist
/// <summary>
/// Refresh Interval for ETFDetailsRT
/// </summary>
ETFDetailsRT: 60,
WorldMarket: 60,
IndexDetailsConstituent: 60,   // one minute
SixtySeconds:60,
});
WinJS.Namespace.define('FinanceAppJS.Constants.Instrumentation', {
/// <summary>
/// This class has the constants for all the Instrumentation constants in the app. 
/// Page Levels Id are the AppEx Instrumentation Ids
/// Cluster level are the call for each cluster
/// </summary>
MarketTodayPage: 25190,
//Market Today Chart
MarketTodayChart: 0,
//Here image call
MarketTodayHero: 1,
//News call
MarketTodayNews: 2,
//CCBE call
MarketTodayCCBE: 3,
//Index,WatchList,Movers
MarketTodayIWM: 11,
//Mutual Funds
MarketTodayFunds: 4,
//Mutual Funds
MarketTodayRates: 5,
//WatchListPage
WatchListPage: 25446,
//Add Ticker
WatchListTicker: 6,
//Stock Details Page
StockDetailsPage: 25958,
//Stock Details RT
StockDetailsRT: 7,
//Stock Details Chart
StockDetailsChart: 8,
//Stock Details Clusters
StockDetailsCluster: 9,
//Rates Pages
RatesECVPage: 27750,
RatesECVCluster: 10,
//Average Rates
RatesAveragePage: 27750,
RatesAverageCluster: 14,
//Movers Page
MoversPage: 25702,
MoversCluster: 15,
//Fund Browse Page
FundBrowsePage: 26726,
FundBrowseCluster: 16, //TODO - How do we do per ecv
//Fund Picks Page
FundPicksPage: 26470,
FundPicksCluster: 12,
//Fund Details Page
FundDetailsPage: 26214,
FundDetailsChart: 13,
FundDetailsCluster: 26,
//ECV Details page
ETFDetailsPage: 29286,
ETFDetailsChart: 17,
ETFDetailsCluster: 18,
ETFRTCluster: 19,
//Mortgage Caluclator
MortgageCaluclatorPage: 29286,
MortageCaluclatorCluster: 20,
//Bonds Page
BondsPage: 27494,
BondsChart: 21,
BondsRT: 22,
BondsEquity: 23,
//Commodity
CommoditiesPage: 27238,
CommondityCluster: 24,
//Currency Page
CurrencyPage: 26982,
CurrencyCluster: 25,
CurrencyConverter: 29798,
CurrencyConverter_Button_Click: 29538,
//Index details page
IndexDetailsPage: 35240,
//Instrumentation Constant
InstrumentationDataSetId: 30,
PanoInstrumentationDataSetId: 7,
PlatformInstrumentationDataSetId: 1,
MarketToday_MyStocksCluster_Click: 25187,
MarketToday_ChartDateClick: 25461,
MarketToday_ChartTap_Click: 25701, // Not Implemented
MarketToday_ChartIndices_Click: 26212,
MarketToday_RatesCluster_Click: 30052,
MarketToday_HeroImageHeadLine_Click: 26984,
MarketToday_MyStocksHeader_Click: 25192,
MarketToday_NewsCluster_Click: 24931,
MarketToday_RatesClusterHeader_Click: 25960, //change
MarketToday_ETFCluster_Click: 44444,//change
MarketToday_NewsClusterHeader_Click: 24936,
MarketToday_AcrossMarket_Currencies_Click: 25699,
MarketToday_AcrossMarket_Commodities_Click: 25955,
MarketToday_AcrossMarket_Bonds_Click: 26211,
MarketToday_AcrossMarket_AverageRates_Click: 28515,
MarketToday_mortgageCluster_Click: 26467,
MarketToday_depositCluster_Click: 26723,
MarketToday_creditcardCluster_Click: 26979,
MarketToday_MarketClusterHeader_Click: 27491,
MarketToday_VideoClusterHeader: 27747,
MarketToday_MyStockAdd_Click: 25958,
Market_WorldMarketClusterHeader_Click: 28259,
Market_CurrenciesClusterHeader_Click: 28771,
Market_CommoditiesClusterHeader_Click: 29027,
Market_BondsClusterHeader_Click: 29283,
Market_FundPicksHeader_Click: 25704,
Market_MarketMoversHeader_Click: 25448,
Market_MarketMoversCluster_Click: 25443,
Market_FundPicksCluster_Click: 27235,
Market_ChartDateClick: 28003,
Market_ChartTap_Click: 25957, // Not Implemented
Market_ChartIndices_Click: 30820,
Market_WorldMarketCluster_Click: 26468,
Market_CurrenciesCluster_Click: 26724,
Market_CommoditiesCluster_Click: 26980,
Market_BondsCluster_Click: 27236,
Market_FundPicksCluster_topperformerytd_Click: 29795,
Market_FundPicksCluster_topperformer5y_Click: 30051,
Market_FundPicksCluster_highestratedcorebondfunds_Click: 30307,
Market_FundPicksCluster_conservativebond_Click: 30563,
News_TopStoriesClusterHeader_Click: 25189,
News_BusinessClusterHeader_Click: 30819,
News_MarketClusterHeader_Click: 31075,
News_TechnologyClusterHeader_Click: 31331,
News_MoneyClusterHeader_Click: 24932,
News_MoreSource_Click: 25188,
NewsPage_TopStoriesCluster_Click: 27492,
NewsPage_BusinessCluster_Click: 27748,
NewsPage_MarketCluster_Click: 28004,
NewsPage_TechnologyCluster_Click: 28260,
NewsPage_MoneyCluster_Click: 28516,
News_TopStoriesCluster_Click: 28772,
News_BusinessCluster_Click: 29028,
News_MarketCluster_Click: 29284,
News_TechnologyCluster_Click: 29540,
News_MoneyCluster_Click: 29796,
NonUsNews_Item_Click: 31076,
Videos_FeaturedClusterHeader_Click: 25444,
Videos_ShowsClusterHeader_Click: 25700,
Videos_MoreSource_Click: 25956,
Today_FeaturedCluster_ItemClick: 25444,
MyStocks_NumberOfStock: 29550,
MyStocks_MyStocks_Click: 29549,
MyStocks_AddTile_Click: 26214,
Index_TodayIndexCluster_Click: 35221,
Index_TodayIndexHero_Click: 35222,
Index_TodayWatchList_Click: 35225,
Index_AddToWatchList_Click: 35228,
Index_RemoveFromWatchList_Click: 35229,
Index_MarketsIndex_Click: 35223,
Index_MarketWMCluster_Click:35224,
Index_WorldMarketMap_Click: 35220,
Index_WorldMarketTable_Click: 35226,
Index_IndexConstituents_Click: 35227,
MarketMovers_Cluster_Click: 28775,
QuoteDetails_NewsCluster_Click: 25457,
QuoteDetails_ChartDateClick: 28023,
QuoteDetails_ChartTap_Click: 26213, // Not Implemented
QuoteDetails_AddToWatchList: 30561,
QuoteDetails_ReadMoreBtn_Click: 30308,
Rates_Cluster_mortgage_Click: 26981,
Rates_Cluster_homequity_Click: 27237,
Rates_Cluster_auto_Click: 27493,
Rates_Cluster_deposit_Click: 27749,
Rates_Cluster_creditcard_Click: 28005,
// Rates ClusterHeader 
Rates_ClusterHeader_mortgage_Click: 28261,
Rates_ClusterHeader_homequity_Click: 28517,
Rates_ClusterHeader_auto_Click: 28773,
Rates_ClusterHeader_deposit_Click: 29029,
Rates_ClusterHeader_creditcard_Click: 29285,
// PageViews
Rates_mortgage_PageView: 28006,
Rates_homequity_PageView: 28262,
Rates_auto_PageView: 28518,
Rates_deposit_PageView: 28774,
Rates_creditcard_PageView: 29030,
Rates_mortgage_LenderClick: 30821,
Rates_homequity_LenderClick: 31077,
Rates_auto_LenderClick: 31333,
Rates_deposit_LenderClick: 24934,
Rates_creditcard_LenderClick: 25190,
//Change-----------
FundDetails_AddToWatchList: 30566,
FundDetails_ChartDateClick: 25702,
FundDetails_ChartTap_Click: 26469, // Not Implemented
ETFDetails_AddToWatchList: 30565,
ETFDetails_ChartDateClick: 25445,
ETFDetails_ChartTap_Click: 26725, // Not Implemented
BestOfWeb_Splurge_Click: 26722,
BestOfWeb_Give_Click: 26978,
// BottomEdgy Instrumentation Clicks
//StockDetails
Remove_Quote_Click: 28518,
PinStart_Quote_Click: 28774,
//FundDetails
Remove_Fund_Click: 29030,
PinStart_Fund_Click: 29286,
// ETFPage
Remove_ETF_Click: 30053,
PinStart_ETF_Click: 30309,
// WatchListPage
Remove_mystock_Click: 26470,
Moveup_mystock_Click: 26726,
Movedown_mystock_Click: 26982,
PinStart_mystock_Click: 27238,
Clear_mystock_Click: 27494,
// TodayPage
Remove_Today_Click: 27750,
PinStart_Today_Click: 28006,
Clear_Today_Click: 28262,
// MarketMovers
Add_Movers_Click: 30822,
PinStart_Movers_Click: 31078,
Clear_Movers_Click: 31334,
// Help button
Help_MarketToday_Click: 24935,
Help_Market_Click: 25191,
Help_Watchlist_Click: 25447,
Help_Quote_Click: 25703,
Help_Fund_Click: 25959,
Help_ETF_Click: 26215,
Help_Currency_Click: 26471,
Help_Commodities_Click: 26727,
Help_Bond_Click: 26983,
Help_WorldMarket_Click: 27239,
Help_AverageRates_Click: 27495,
Help_FundPick_Click: 27751,
Help_RatesEcv_Click: 28007,
Help_FundScreenerEcv_Click: 28263,
Help_News_Click: 28519,
Help_Video_Click: 28775, // FutureUse
Help_Movers_Click: 29031,
Help_ToolsLanding_Click: 30050,
Help_CalucaltorPage_Click: 30306,
//ToolsPano
Tools_MortgageCalculator_Click: 24929,
Tools_LoanPaymentsCalculator_Click: 25441,
Tools_CurrencyConverter_Click: 25185,
Tools_SavingsCalculator_Click: 26209,
Tools_RetirementsCalculator_Click: 25697,
Tools_FinancialCalculator_Click: 25953,
//CalcPages
//Differnent calculators Navigation click
AutoLoan_Nav_Click: 27745,
AutoLease_Nav_Click: 28769,
FinancialTVM_Nav_Click: 30817,
FinancialROI_Nav_Click: 25186,
SavingsCalc_Nav_Click: 26210,
SavingEstimator_Nav_Click: 27234,
SavingsTAG_Nav_Click: 28258,
//Calculate Button
MortgageCalc_Calculate_Click: 26465,
AutoLoan_Calculate_Click: 28001,
AutoLease_Calculate_Click: 29025,
RetirementCalc_Calculate_Click: 29793,
FinancialTVM_Calculate_Click: 31073,
FinancialROI_Calculate_Click: 25442,
SavingsCalc_Calculate_Click: 26466,
SavingEstimator_Calculate_Click: 27490,
SavingsTAG_Calculate_Click: 28514,
//Chart Button
MortgageCalc_Chart_Click: 26721,
AutoLoan_Chart_Click: 28257,
AutoLease_Chart_Click: 29281,
RetirementCalc_Chart_Click: 30049,
FinancialTVM_Chart_Click: 31329,
FinancialROI_Simple_Click: 25698,
FinancialROI_Compound_Click: 25954,
SavingsCalc_Chart_Click: 26722,
SavingEstimator_Chart_Click: 27746,
SavingsTAG_Chart_Click: 28770,
//Schedule Button
MortgageCalc_Schedule_Click: 26977,
AutoLoan_Schedule_Click: 28513,
AutoLease_Schedule_Click: 29537,
RetirementCalc_Schedule_Click: 30305,
FinancialTVM_Schedule_Click: 24930,
SavingsCalc_Schedule_Click: 26978,
SavingEstimator_Schedule_Click: 28002,
SavingsTAG_Schedule_Click: 29026,
// ToolsPages PageView
AutoLoan_PageView_Click: 28775,
AutoLease_PageView_Click: 29031,
RetirementCalc_PageView_Click: 26471,
FinancialTVM_PageView_Click: 29287,
FinancialROI_PageView_Click: 29543,
SavingsCalc_PageView_Click: 29799,
SavingEstimator_PageView_Click: 30055,
SavingsTAG_PageView_Click: 30311,
// Currency Converter Page
From_Currency_Amount_Click: 27233,
To_Currency_List_AddTile_Click: 27489,
BE_Remove_Currency_Button_Click: 29794,
BE_Add_Currency_Button_Click: 30562,
BE_MoveUp_Button_Click: 30562,
BE_MoveDown_Button_Click: 31330,
});
WinJS.Namespace.define("FinanceAppJS.Instrumentation.EntryPoint", {
/*
Values 0, 1 and others are defined in Platform.Instrumentation.InstrumentationArticleEntryPoint.
App specific entry points should start at PlatformReserved + 1 and go up.
See InstrumentationIds.cs for Platform Mapping.
*/
entityVideosPano: Platform.Instrumentation.InstrumentationArticleEntryPoint.platformReserved + 1,
entityNewsPano: Platform.Instrumentation.InstrumentationArticleEntryPoint.platformReserved + 2, /* Entity News Pano */
newsPano: Platform.Instrumentation.InstrumentationArticleEntryPoint.platformReserved + 3 /* non-US News Pano */
});
WinJS.Namespace.define('FinanceAppJS.Constants.Global', {
ChartAndEquitydata_Allowed_TimeDifference: 86400000, // 24 * 60 * 60 * 1000 (24 hours),
MaxIndexConstituentsToBeQueried: 200
});
WinJS.Namespace.define('FinanceAppJS.Constants.NewsCluster', {
NewsResultsCount: "30",
NewsSortByDate: "0",
NewsClusterMaxColumnCount: 3
});
})();
/* shared\js\settingsUI.js */
(function () {
'use strict';
/// <summary>
/// Weather App Settings UI namespace.
/// </summary>
WinJS.Namespace.define("FinanceAppJS.UI.Settings", {
/// <summary>
/// Brings up the about flyout
/// </summary>
onAboutCmd: function onAboutCmd(ev) {
console.log("onAboutCmd");
utils.addAboutPageFragment().then(function (aboutFlyout) {
if (aboutFlyout) {
aboutFlyout.addEventListener("afterhide", function (event) {
utils.removeAboutPageFragment();
});
aboutFlyout.show();
}
});
},
/// <summary>
/// Pushes the setting commands specific to app into the application commands vector.
/// </summary>
onCommandsRequested: function onCommandsRequested(e) {
var appSettings = Windows.UI.ApplicationSettings;
var vector = e.request.applicationCommands;
console.log("onCommandsRequested");
// Add default settings
vector.append(CommonJS.Settings.getDefaultSettingsCommand());
// Add about settings command to the settings charm
var aboutCmd = new appSettings.SettingsCommand("AboutPage", utils.getResourceString("About"), FinanceAppJS.UI.Settings.onAboutCmd);
vector.append(aboutCmd);
CommonJS.Settings.addCommonButtons(vector);
},
/// <summary>
/// Updates the display unit
/// <param>
/// unit - display unit as set in the settings flyout
/// </param>
/// </summary>
//onDisplayUnitSet: function (unit) {
//    WeatherAppJS.Utilities.Instrumentation.incrementInt32("NumTimesDisplayUnitSettingChanged");
//    WeatherAppJS.SettingsManager.setDisplayUnitAsync(unit).then(
//        function () { },
//        function (e) {
//            console.log("Set Display unit failed." + " ErrorCode:" + e.errorcode + " ErrorMessage: " + e.errormessage);
//        }
//    );
//},
});
// On application load, register the application commands and settings callback. When the settings charm comes up in application context, 
// these callbacks populates the setting commands necessary to show up on the setting charm.
var settingsPane = Windows.UI.ApplicationSettings.SettingsPane.getForCurrentView();
settingsPane.oncommandsrequested = FinanceAppJS.UI.Settings.onCommandsRequested;
/// Handler for laying out the application settings and defining the fragments to load for each application command.
WinJS.Application.onsettings = function (e) {
console.log("onsettings");
};
})();
/* shared\js\ActivationHelper.js */
(function () {
'use strict';
function navigateToEntityPage(state, instrumentationId) {
navigationHelper.instrumentNavigation(instrumentationId);
return {
pageInfo: {
fragment: "/pages/StockDetails/html/StockDetailsImmersive.html",
page: "FinanceAppJS.StockDetailsImmersive",
channelId: "StockEntity"
},
state: state
};
}
function navigateToFundDetailsPage(state, instrumentationId) {
navigationHelper.instrumentNavigation(instrumentationId);
return {
pageInfo: {
fragment: "/pages/FundDetails/html/FundDetailsImmersive.html",
page: "FinanceAppJS.FundDetails",
channelId: "FundDetails"
},
state: state
};
}
function navigateToETFDetailsPage(state, instrumentationId) {
navigationHelper.instrumentNavigation(instrumentationId);
return {
pageInfo: {
fragment: "/pages/ETF/html/ETF.html",
page: "FinanceAppJS.ETF",
channelId: "ETF"
},
state: state
};
}
function navigateToMyStocksPage(state, instrumentationId) {
navigationHelper.instrumentNavigation(instrumentationId);
var st = state ? state : {};
return {
pageInfo: {
fragment: "/pages/MyStocks/html/Watchlist.html",
page: "FinanceAppJS.WatchlistPage",
channelId: "MyStocks"
},
state: st
};
}
function navigateToSnappedHomePage() {
return {
pageInfo: {
fragment: "/pages/SnappedView/html/SnappedView.html",
page: "FinanceAppJS.SnappedView"
},
state: {}
};
}
function navigateToWSJPage(state) {
return {
pageInfo: {
fragment: "/wsjshared/wsjCategoryPano.html",
page: "FinanceAppJS.WSJPano"
},
state: state
};
}
function navigateToDynamicPano (state) {
return {
state: {
dynamicInfo: state
},
pageInfo: {
channelId: state && state.id,
fragment: "common/DynamicPano/html/DynamicPano.html",
page: "DynamicPanoJS.DynamicPano"
}
};
}
function navigateToIndexDetailsPage(state, instrumentationId) {
navigationHelper.instrumentNavigation(instrumentationId);
return {
pageInfo: {
fragment: "/pages/IndexDetails/html/IndexDetails.html",
page: "FinanceAppJS.IndexDetails",
channelId: "IndexDetails"
},
state: state
};
}
WinJS.Namespace.define("activationHelper", {
navigateToEntityPage: navigateToEntityPage,
navigateToFundDetailsPage: navigateToFundDetailsPage,
navigateToETFDetailsPage: navigateToETFDetailsPage,
navigateToMyStocksPage: navigateToMyStocksPage,
navigateToSnappedHomePage: navigateToSnappedHomePage,
navigateToWSJPage: navigateToWSJPage,
navigateToDynamicPano: navigateToDynamicPano,
navigateToIndexDetailsPage: navigateToIndexDetailsPage
});
})();
/* shared\js\TopEdgy.js */
(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS", {
TopEdgy: WinJS.Class.define(function () {
this.initialize();
this.binding_watchlistDataAvailable = this._watchlistDataAvailable.bind(this);
this.roaming = new FinanceAppJS.Roaming();
//TodO: try to move to queryservice
this.provider = CommonJS.PanoEx.BaseDataProvider.findOrCreateProvider("TopEdgyWatchlistDataProvider", { wlSymbols: this.roaming.getWatchListSymbols() });
// This will internally trigger getData() so we dont need to call setupWatchListL2 Explicitly.
this.provider.addListeners({ dataavailable: this.binding_watchlistDataAvailable }, "topedgy");
},
{
provider: null,
binding_watchlistDataAvailable: null,
standardChannels: null,
featuredChannels: null,
snappedChannels: null,
RatesECV: null,
tools: null,
EntityVideosPano: null,
EntityNewsPano: null,
EntityMoneyPano: null,
WSJLivePano: null,
MyStocks: null,
financeRoaming: null,
initialize: function initialize() {
var channelManager = PlatformJS.Navigation && PlatformJS.Navigation.mainNavigator && PlatformJS.Navigation.mainNavigator.channelManager;
if (channelManager) {
this.standardChannels = channelManager.standardChannels;
this.featuredChannels = channelManager.featuredChannels;
this.snappedChannels = channelManager.snappedChannels;
var channelData;
var channelProperty;
if (this.standardChannels) {
for (var channelProperty in this.standardChannels) {
var channelData = this.standardChannels[channelProperty];
this._assign(channelData);
}
}
if (this.featuredChannels) {
for (channelProperty in this.featuredChannels) {
channelData = this.featuredChannels[channelProperty];
if (!channelData.toggleButtonColor) {
var color = PlatformJS.Services.appConfig.getDictionary("PartnerToggleButtonColor").getString(channelData.id);
channelData.toggleButtonColor = color;
}
this._assign(channelData);
}
}
}
},
reload: function reload() {
var featuredChannels = [];
this._readConfig(PlatformJS.Services.manifest.featuredChannels, featuredChannels);
var channelManager = PlatformJS.Navigation.mainNavigator.channelManager;
var allFeaturedChannels = channelManager.featuredChannels;
allFeaturedChannels.length = 0;
for (var i = 0; i < featuredChannels.length; i++) {
allFeaturedChannels.push(featuredChannels[i]);
}
/* initialize dynamic L2 contents of FinanceApp . */
this.initialize();
this.initializeEdgy();
this.setupMyStocksL2();
channelManager.channelConfigChanged = true;
Platform.Configuration.ConfigurationManager.instance.hasPendingFeaturesConfigUpdate = true;
channelManager.loadFeaturesConfigAsync(true);
},
_readConfig: function _readConfig(channelsList, channelArray) {
// This code is taken from navigation.js. It's duplicated here because currently, and temporarily, we're
// basing the L2 on the FeaturedChannels configuration which is only for test purposes (that should not be a predefined
// element in the configuration schema, but that's a separate issue).
//
// This code is used when the user changes the market. The configuration gets reloaded. We'll mimic the initial app
// startup logic by reading the config and feeding it back to the top edgy.
var i = 0;
var channelConfig = null;
for (i = 0; i < channelsList.size; i++) {
channelConfig = channelsList[i];
var channel = this._readChannel(channelConfig);
channelArray.push(channel);
}
},
_readChannel: function _readChannel(channelConfig) {
var i = 0,
state = null,
title = null,
images = null;
if (!channelConfig.id) {
PlatformJS.Utilities.onError("Invalid channel Id");
}
if (!channelConfig.title) {
PlatformJS.Utilities.onError("Invalid channel title");
}
if (!channelConfig.fragment) {
PlatformJS.Utilities.onError("Invalid channel fragment");
}
if (!channelConfig.page) {
PlatformJS.Utilities.onError("Invalid channel page object name");
}
var subChannels = [];
for (i = 0; i < channelConfig.subChannels.size; i++) {
var subChannelConfig = channelConfig.subChannels[i];
var subChannel = this._readChannel(subChannelConfig);
subChannels.push(subChannel);
}
try {
state = channelConfig.state.length > 0 ? JSON.parse(channelConfig.state) : null;
} catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
PlatformJS.Utilities.onError(e);
}
try {
images = channelConfig.images && channelConfig.images.length > 0 ? JSON.parse(channelConfig.images) : null;
} catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
PlatformJS.Utilities.onError(e);
}
title = channelConfig.title;
var channel = ({
id: channelConfig.id,
visible: channelConfig.visible === "true",
title: title,
icon: channelConfig.icon,
pressedIcon: channelConfig.pressedIcon,
pageInfo: {
fragment: channelConfig.fragment,
page: channelConfig.page,
channelId: channelConfig.id
},
subChannels: subChannels,
doubleWide: channelConfig.doubleWide,
images: images,
subTitle: channelConfig.subTitle,
state: state
});
return channel;
},
_assign: function _assign(channelData) {
switch (channelData.id) {
case "RatesAverage":
this.RatesECV = channelData;
break;
case "EntityNewsPano":
this.EntityNewsPano = channelData;
break;
case "EntityMoneyPano":
this.EntityMoneyPano = channelData;
break;
case "EntityVideosPano":
this.EntityVideosPano = channelData;
break;
case "MyStocks":
this.MyStocks = channelData;
break;
case "WSJ LIVE":
this.WSJLivePano = channelData;
break;
case "Tools":
this.tools = channelData;
break;
}
},
initializeEdgy: function initializeEdgy() {
var PUG = Platform.Utilities.Globalization;
if (PUG.isFeatureEnabled("US")) {
this.setupRatesL2();
}
if (PUG.isFeatureEnabled("EntityNews")) {
this.setupEntityNewsL2();
}
if (PUG.isFeatureEnabled("EntityMoney")) {
this.setupEntityMoneyL2();
}
if (PUG.isFeatureEnabled("featuredChannelWSJ")) {
this.setupWSJLiveL2();
}
if (PUG.isFeatureEnabled("Tools")) {
this.setupToolsL2();
}
},
setupToolsL2: function setupToolsL2() {
var subChannels = [];
var toolsList = PlatformJS.Services.configuration.getList("ToolsCalculatorCategoryList");
for (var i = 0; i < toolsList.length; i++ ) {
var tool = JSON.parse(toolsList[i].value);
var channel = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel(tool.id);
if (channel) {
var newChannel = {};
utils.deepCopy(channel, newChannel);
newChannel.visible = true;
if (Platform.Utilities.Globalization.isFeatureEnabled(tool.id)) {
subChannels.push(newChannel);
}
}                    
}
this.tools.subChannels = subChannels;
PlatformJS.Navigation.mainNavigator.channelManager.channelConfigChanged = true;
},
setupMyStocksL2: function setupMyStocksL2() {
this.provider.cancelFetch();
// Reset the data provider with new settings when symbols changed
this.provider = CommonJS.PanoEx.BaseDataProvider.findOrCreateProvider("TopEdgyWatchlistDataProvider", { wlSymbols: this.roaming.getWatchListSymbols() });
this.provider.getData();
},
_watchlistDataAvailable: function _watchlistDataAvailable(providerName, data) {
if (PlatformJS.Services.configuration.getBool("DisableWatchlistL2")) {
return;
}
if (data.Wl && data.Wl.length >= 0) {
var wl = data.Wl;
var subChannels = [];
for (var i = 0; i < wl.length; i++) {
var newChannel = this.createChannelForStock(wl[i].tileData, i);
if (newChannel) {
subChannels.push(newChannel);
}
}
this.MyStocks.subChannels = subChannels;
PlatformJS.Navigation.mainNavigator.channelManager.channelConfigChanged = true;
}
},
createChannelForStock: function createChannelForStock(tileData, i) {
var state = null;
var fragmentPath = null;
var page = null;
var display = null;
if (WatchListManage.isFundTile(tileData)) {
state = WatchListManage.getFundDetailsPageInfo(tileData);
fragmentPath = "/pages/FundDetails/html/FundDetailsImmersive.html";
page = "FinanceAppJS.FundDetails";
display = state.entity.Tkr || state.entity.Scid;
}
else if (WatchListManage.isStockTile(tileData)) {
if (tileData.type === FinanceAppJS.Constants.IDs.Stock) {
state = WatchListManage.getEntityPageInfo(tileData);
fragmentPath = "/pages/StockDetails/html/StockDetailsImmersive.html";
page = "FinanceAppJS.StockDetailsImmersive";
}
else {
state = WatchListManage.getETFDetailsPageInfo(tileData);
fragmentPath = "/pages/ETF/html/ETF.html";
page = "FinanceAppJS.ETF";
}
display = utils.getAutoSuggestDisplaySymbol(state.entity);
}
else if (WatchListManage.isIndexTile(tileData)) {
state = utils.getIndexDetails(tileData, "IndexTileData");
fragmentPath = "/pages/IndexDetails/html/IndexDetails.html";
page = "FinanceAppJS.IndexDetails";
display = utils.getAutoSuggestDisplaySymbol(tileData);
}
if (state && fragmentPath && page) {
return FinanceAppJS.TopEdgy.createChannel("WatchListL2-" + i, display, "", [], fragmentPath, page, state);
}
return null;
},
setupEntityNewsL2: function setupEntityNewsL2() {
if (this.EntityNewsPano) {
var subChannels = [];
var EntityNewsCategoryList = PlatformJS.Services.configuration.getList("EntityNewsCategoryList");
for (var i = 0; i < EntityNewsCategoryList.size; i++) {
var category = JSON.parse(EntityNewsCategoryList[i].value);
var newChannel = FinanceAppJS.TopEdgy.createChannel(
"EntityNews_" + category.collectionId, category.clusterTitleKey, "", [], "/pages/EntityCluster/html/EntityPano.html", "FinanceAppJS.EntityCategoryNewsPano",
{
provider: "EntityPanoProvider",
pageTitle: "chNews",
panoId: "EntityNewsPano",
singleCluster: true,
collectionId: category.collectionId
});
subChannels.push(newChannel);
}
this.EntityNewsPano.subChannels = subChannels;
PlatformJS.Navigation.mainNavigator.channelManager.channelConfigChanged = true;
}
},
setupEntityMoneyL2: function setupEntityMoneyL2() {
if (this.EntityMoneyPano) {
var subChannels = [];
var EntityNewsCategoryList = PlatformJS.Services.configuration.getList("EntityMoneyCategoryList");
if (PlatformJS.Services.configuration.getBool("EnableMoneyL2") === false) {
return;
}
for (var i = 0; i < EntityNewsCategoryList.size; i++) {
var category = JSON.parse(EntityNewsCategoryList[i].value);
var newChannel = FinanceAppJS.TopEdgy.createChannel(
"EntityMoney_" + category.collectionId, category.clusterTitleKey, "", [], "/pages/EntityCluster/html/EntityPano.html", "FinanceAppJS.EntityCategoryMoneyPano",
{
provider: "MoneyEntityPanoProvider",
pageTitle: "chMoney",
panoId: "EntityMoneyPano",
singleCluster: true,
collectionId: category.collectionId
});
subChannels.push(newChannel);
}
this.EntityMoneyPano.subChannels = subChannels;
PlatformJS.Navigation.mainNavigator.channelManager.channelConfigChanged = true;
}
},
setupWSJLiveL2: function setupWSJLiveL2() {
var liveFeeds = NewsJS.Partners.Config.getConfig("WSJ", "WSJLiveFeeds", []);
var subChannels = [];
for (var i = 0; i < liveFeeds.length; i++) {
var id = liveFeeds[i].value;
if (id) {
var newChannel = FinanceAppJS.TopEdgy.createChannel("WSJ LIVE_" + id, "/wsj/" + id, "", [], "/wsjshared/wsjCategoryPano.html", "FinanceAppJS.WSJPano",
{
feedType: "WSJ.GetVideoSection", feedIdentifierValue: id, theme: "WSJ"
}
);
subChannels.push(newChannel);
}
}
if (this.WSJLivePano) {
this.WSJLivePano.subChannels = subChannels;
PlatformJS.Navigation.mainNavigator.channelManager.channelConfigChanged = true;
}
},
setupRatesL2: function setupRatesL2() {
if (this.RatesECV) {
var subChannels = [];
var RatesCategories = PlatformJS.Services.configuration.getList("RatesCategoriesList");
for (var i = 0; i < RatesCategories.size; i++) {
var category = RatesCategories[i].value;
var newChannel = FinanceAppJS.TopEdgy.createChannel(
"Rates" + category, utils.getResourceString("RatesTitle_" + category), "", [], "/pages/Rates/html/rates.html", "CommonJS.EntityCollection.Controller",
{
entityView: "",
title: "RatesTitle_" + category, //ECV code look in the resource, so we dont need to again look-up in the resource
class: "FinanceAppJS.Rates.Collection",
key: { label: "Rates" },
state: { category: category }
});
subChannels.push(newChannel);
}
this.RatesECV.subChannels = subChannels;
PlatformJS.Navigation.mainNavigator.channelManager.channelConfigChanged = true;
}
},
},
{
createChannel: function createChannel(id, title, icon, subChannels, fragment, page, state) {
return {
id: id,
title: title,
icon: icon,
subChannels: subChannels,
pageInfo: {
fragment: fragment,
page: page,
channelId: id,
},
state: state,
};
},
_instance: null,
instance: {
get: function get() {
if (!FinanceAppJS.TopEdgy._instance) {
FinanceAppJS.TopEdgy._instance = new FinanceAppJS.TopEdgy();
}
return FinanceAppJS.TopEdgy._instance;
}
}
}),
});
})();
/* shared\js\navigationHelper.js */
(function () {
'use strict';
function instrumentNavigation(instrumentationId) {
/// <summary>Given a the Id (or a array of Ids) increment the instrumentation counters</summary>
FinanceAppJS.Telemetry.FlightRecorder.NavigateFromTo(); // no conditions attached 
}
function navigateToEntityPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("StockEntity", state || {});
instrumentNavigation(instrumentationId);
}
/* Navigate to News/Videos entity cluster pano */
function navigateToEntityNewsPano(state, instrumentationId) {
// channelId needs to be changed to a different one so that navigator thinks this is a different channel.
var channelId = state.collectionId ? "EntityNews_" + state.collectionId : state.panoId;
WinJS.Navigation.navigate({ channelId: channelId, fragment: "/pages/EntityCluster/html/EntityPano.html", page: "FinanceAppJS.EntityNewsPano" }, state);
instrumentNavigation(instrumentationId);
}
function navigateToEntityCategoryNewsPano(state, instrumentationId) {
// channelId needs to be changed to a different one so that navigator thinks this is a different channel.
var channelId = state.collectionId ? "EntityNews_" + state.collectionId : state.panoId;
WinJS.Navigation.navigate({ channelId: channelId, fragment: "/pages/EntityCluster/html/EntityPano.html", page: "FinanceAppJS.EntityCategoryNewsPano" }, state);
instrumentNavigation(instrumentationId);
}
function navigateToEntityCategoryMoneyPano(state, instrumentationId) {
// channelId needs to be changed to a different one so that navigator thinks this is a different channel.
var channelId = state.collectionId ? "EntityMoney_" + state.collectionId : state.panoId;
WinJS.Navigation.navigate({ channelId: channelId, fragment: "/pages/EntityCluster/html/EntityPano.html", page: "FinanceAppJS.EntityCategoryMoneyPano" }, state);
instrumentNavigation(instrumentationId);
}
function navigateToEntityCategoryRatesPano(state, instrumentationId) {
// channelId needs to be changed to a different one so that navigator thinks this is a different channel.
var channelId = state.state.category ? "Rates" + state.state.category : "mortgage";
WinJS.Navigation.navigate({ channelId: channelId, fragment: "/pages/Rates/html/rates.html", page: "CommonJS.EntityCollection.Controller" }, state);
instrumentNavigation(instrumentationId);
}
function navigateToEntityMoneyPano(state, instrumentationId) {
// channelId needs to be changed to a different one so that navigator thinks this is a different channel.
var channelId = state.collectionId ? "EntityMoney_" + state.collectionId : state.panoId;
WinJS.Navigation.navigate({ channelId: channelId, fragment: "/pages/EntityCluster/html/EntityPano.html", page: "FinanceAppJS.EntityMoneyPano" }, state);
instrumentNavigation(instrumentationId);
}
function navigateToEntityVideosPano(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel(state.panoId, state || {});
instrumentNavigation(instrumentationId);
}
function navigateToCustomPano(state) {
PlatformJS.Navigation.navigateToChannel("CustomPano", state || {});
}
function navigateToFundDetailsPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("FundDetails", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToETFDetailsPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("ETF", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToWorldMarketPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("WorldMarket", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToCurrenciesPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("Currencies", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToCommoditiesPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("Commodities", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToBondsPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("Bonds", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToAverageRatesPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("RatesAverage", {});
instrumentNavigation(instrumentationId);
}
function navigateToRatesPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("Rates", {
entityView: "",
title: "RatesTitle_" + state.category, //ECV code look in the resource, so we dont need to again look-up in the resource
class: "FinanceAppJS.Rates.Collection",
key: { label: "Rates" },
state: state
});
instrumentNavigation(instrumentationId);
}
function navigateToFundBrowsePage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("FundBrowse", {
entityView: "FundDetails",
title: state.sid,
class: "FinanceAppJS.FundBrowse.Collection",
key: { label: "FundBrowse", sid: state.sid, category: state.category },
state: state
});
instrumentNavigation(instrumentationId);
}
function navigateToMyStocksPage(state, instrumentationId) {
var st = state ? state : {};
PlatformJS.Navigation.navigateToChannel("MyStocks", st);
instrumentNavigation(instrumentationId);
}
function navigateToMarketMoversPage(state, instrumentationId) {
var st = state ? state : {};
PlatformJS.Navigation.navigateToChannel("MarketMovers", {
entityView: "StockEntity",
title: "MarketMovers",
class: "FinanceAppJS.MarketMovers.Collection",
key: { label: utils.getResourceString("MarketMovers") },
state: st
});
instrumentNavigation(instrumentationId);
}
function navigateToNewsPage(state, instrumentationId) {
var st = state ? state : {};
PlatformJS.Navigation.navigateToChannel("NewsPano", st);
instrumentNavigation(instrumentationId);
}
function navigateToFundPicksPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("FundPicks", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToMarketPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("Market", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToAutoLoanCalculator(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("AutoLoanCalc", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToRetirementCalculator(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("RetirementCalc", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToMortgageCalculator(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("MortgageCalc", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToCurrencyCalculator(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("CurrencyConverter", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToDetailsPage(tag) {
// TODO: chnage tag to JSON object
var autoSuggestVal = tag;
var autoSuggestValArr = autoSuggestVal.split('~');
var sym = utils.trim(autoSuggestValArr[0]);
var secId = autoSuggestValArr[1];
var symbolType = autoSuggestValArr[2];
var E1 = autoSuggestValArr[5];
var St = autoSuggestValArr[7];
var E2 = autoSuggestValArr[3];
var cmp = autoSuggestValArr[4];
var exsn = autoSuggestValArr[8];
var Eqsm = null;
if (autoSuggestValArr[6] !== sym) {
Eqsm = autoSuggestValArr[6];
}
var entityItem = {};
switch (true) {
case utils.isFundSymbol(symbolType):
entityItem = {
entity: {
Tkr: sym,
Fn: cmp,
Scid: secId
}
};
navigationHelper.navigateToFundDetailsPage(entityItem);
break;
case utils.isEtfSymbol(symbolType):
entityItem = {
entity: {
Sym: sym,
Eqsm: Eqsm,
E1: E1,
E2: E2,
St: St,
Cmp: cmp                       
}
};
navigationHelper.navigateToETFDetailsPage(entityItem);
break;
case utils.isStockSymbol(symbolType):
entityItem = {
entity: {
Sym: sym,
Eqsm: Eqsm,
E1: E1,
E2: E2,
St: St,
Cmp: cmp,
ExSn: exsn
}
};
navigationHelper.navigateToEntityPage(entityItem);
break;
case utils.isIndexSymbol(symbolType):
entityItem = utils.getIndexDetails({E1: E1, E2: E1, Sym: sym, St: St, Eqsm: Eqsm, Cmp: cmp}, "IndexTileData");
navigationHelper.navigateToIndexDetailsPage(entityItem);
break;
};
return;
}
function navigateToHelpPage(event, instrumentationId) {
// Ecv Pages will not call in a same way. So we have to get instrumentid from this object.
if (!instrumentationId) {
if (this.instrumentId) {
instrumentationId = this.instrumentId;
}
}
if (instrumentationId) {
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, instrumentationId, 1);
}
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.Location.HELP,
"");
var uri = PlatformJS.Services.appConfig.getString("HelpURL");
if (uri) {
try {
Platform.Telemetry.FlightRecorder.setImpressionNavMethod(Platform.Telemetry.ImpressionNavMethod.appBar);
PlatformJS.Navigation.navigateTo(uri);
} catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
// TODO: Do something with this error later
}
}
}
function navigateToSavingsCalculator(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("SavingsCalc", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToFinancialCalculator(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("FinancialCalc", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToIndexDetailsPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("IndexDetails", state || {});
instrumentNavigation(instrumentationId);
}
function navigateToToolsPage(state, instrumentationId) {
PlatformJS.Navigation.navigateToChannel("Tools", state || {});
instrumentNavigation(instrumentationId);
}
WinJS.Namespace.define("navigationHelper", {
instrumentNavigation: instrumentNavigation,
navigateToEntityPage: navigateToEntityPage,
navigateToFundDetailsPage: navigateToFundDetailsPage,
navigateToCurrenciesPage: navigateToCurrenciesPage,
navigateToCommoditiesPage: navigateToCommoditiesPage,
navigateToBondsPage: navigateToBondsPage,
navigateToRatesPage: navigateToRatesPage,
navigateToAverageRatesPage: navigateToAverageRatesPage,
navigateToMyStocksPage: navigateToMyStocksPage,
navigateToMarketMoversPage: navigateToMarketMoversPage,
navigateToNewsPage: navigateToNewsPage,
navigateToFundPicksPage: navigateToFundPicksPage,
navigateToFundBrowsePage: navigateToFundBrowsePage,
navigateToETFDetailsPage: navigateToETFDetailsPage,
navigateToEntityNewsPano: navigateToEntityNewsPano,
navigateToEntityMoneyPano: navigateToEntityMoneyPano,
navigateToEntityCategoryNewsPano: navigateToEntityCategoryNewsPano,
navigateToEntityCategoryMoneyPano: navigateToEntityCategoryMoneyPano,
navigateToEntityVideosPano: navigateToEntityVideosPano,
navigateToDetailsPage: navigateToDetailsPage,
navigateToCustomPano: navigateToCustomPano,
navigateToWorldMarketPage: navigateToWorldMarketPage,
navigateToMarketPage: navigateToMarketPage,
navigateToHelpPage: navigateToHelpPage,
navigateToSavingsCalculator: navigateToSavingsCalculator,
navigateToMortgageCalculator: navigateToMortgageCalculator,
navigateToFinancialCalculator: navigateToFinancialCalculator,
navigateToAutoLoanCalculator: navigateToAutoLoanCalculator,
navigateToRetirementCalculator: navigateToRetirementCalculator,
navigateToCurrencyCalculator: navigateToCurrencyCalculator,
navigateToIndexDetailsPage: navigateToIndexDetailsPage,
navigateToToolsPage: navigateToToolsPage,
navigateToEntityCategoryRatesPano: navigateToEntityCategoryRatesPano
});
})();
/* shared\js\AutoSuggest.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Utilities", {
AutoSuggest: WinJS.Class.define(
function AutoSuggestCtor () {
this._autoCompleteQueryService = new PlatformJS.DataService.QueryService("GlobalWatchlistAutoComplete");
this._autoCompleteQueryServiceOptions = new Platform.DataServices.QueryServiceOptions();
this._autoCompleteQueryServiceOptions.useCompatDataModel = true;
this._autoCompleteQueryServiceOptions.enableAutoRefresh = false;
this._getSuggestionsPromiseFunc_binding = this._getSuggestionsPromiseFunc.bind(this);
},
{
_autoCompleteQueryService: null,
_autoCompleteQueryServiceOptions: null,
_localeFormatted: null,
_queryText: null,
_noOfSuggestions: null,
release: function release() {
},
_getSuggestionsPromiseFunc: function _getSuggestionsPromiseFunc(complete, error) {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("keyword", encodeURIComponent(this._queryText));
urlParams.insert("locale", encodeURIComponent(this._localeFormatted));
urlParams.insert("count", this._noOfSuggestions);
var dataReqObj = FinanceAppJS.Telemetry.FlightRecorder.LogDataRequest("request", "GlobalWatchlistAutoComplete");
this._autoCompleteQueryService.downloadDataAsync(urlParams, null, null, this._autoCompleteQueryServiceOptions).then(function (response) {
if (response.dataString) {
try {
FinanceAppJS.Telemetry.FlightRecorder.LogDataRequest("service", "GlobalWatchlistAutoComplete", dataReqObj);
var dt = JSON.parse(response.dataString);
if (dt && dt.status && dt.status.code === 0 && dt.data) {
complete(dt.data);
}
}
catch (err) {
complete([]);
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(err.message, err.stack);
}
}
}, function (errData) {
complete([]);
});
},
getSuggestions: function getSuggestions(queryText, noOfSuggestions, locale) {
this._queryText = queryText;
this._noOfSuggestions = noOfSuggestions;
this._localeFormatted = locale || FinanceAppJS.Globalization.getMarket().replace("-", ":");
return new WinJS.Promise(this._getSuggestionsPromiseFunc_binding);
},
buildSuggestion: function buildSuggestion(dataObj, userLang) {
var symbolType = dataObj.OS010;
var symbolTypeText = null;
if (utils.isFundSymbol(symbolType)) {                   
symbolTypeText = utils.getResourceString("FundLabel");
}
else if (utils.isStockSymbol(symbolType)) {
symbolTypeText = utils.getResourceString("StockLabel");
}
else if (utils.isEtfSymbol(symbolType)) {
if (symbolType === "CE") {
symbolTypeText = utils.getResourceString("FundLabel");
}
else {
symbolTypeText = utils.getResourceString("ETFLabel");
}
}
else if (utils.isIndexSymbol(symbolType)) {
symbolTypeText = utils.getResourceString("IndexLabel");
}
else {
return;
}
var equitySymbol = dataObj.OS001;
var securityId = dataObj.SecId;
var exchangeCode = dataObj.LS01Z;
var realTimeExchange = dataObj.RT00E;
var exchangeName = "";
if (exchangeCode) {
if (utils.isOTCInstrument(exchangeCode)) {
exchangeName = dataObj.AC040;
}
else {
exchangeName = utils.getResourceString("Ex_" + exchangeCode);
}
}
var securityType = {};
if (utils.isIndexSymbol(symbolType)) {
securityType = FinanceAppJS.Constants.SecurityType.Index; // some constant for the file 
exchangeName = "";
} else {
securityType = dataObj.RT00T || FinanceAppJS.Constants.SecurityType.Stock;
}
var realTimeSymbolHidden = (!dataObj.RT00S) ? (equitySymbol ? equitySymbol : "") : dataObj.RT00S;
var realTimeSymbolDisplay = realTimeSymbolHidden ? realTimeSymbolHidden : securityId;
var displaySymbol = utils.getAutoSuggestDisplaySymbol({ Eqsm: equitySymbol, Sym: dataObj.RT00S });
// Display CompanyName in local language (either long or short) if available, Otherwise show company name in English
var companyName = dataObj.OS01W;
var symbolLocale = dataObj.OS01V;
var userLangFirstComponent = userLang ? userLang.split("-")[0] : null;
if (symbolLocale) {
symbolLocale = symbolLocale.toLowerCase();
if (userLangFirstComponent !== "en" && (symbolLocale === userLang || symbolLocale === userLangFirstComponent)) {
var cmpNameLocalLangLong = dataObj.OS0LN;
var cmpNameLocalLangShort = dataObj.RT0SN;
companyName = cmpNameLocalLangLong ? cmpNameLocalLangLong : (cmpNameLocalLangShort ? cmpNameLocalLangShort : companyName);
}
}
var suggestion = {
symbolType: symbolType,
symbolTypeText: symbolTypeText,
equitySymbol: equitySymbol,
realTimeSymbol: realTimeSymbolDisplay,
displaySymbol: displaySymbol,
securityId: securityId,
companyName: companyName,
exchangeCode: exchangeCode,
realTimeExchange: realTimeExchange,
exchangeName: exchangeName,
value: realTimeSymbolHidden + '~' + securityId + '~' + symbolType + '~' + exchangeCode + '~' + companyName + '~' + realTimeExchange + '~' + equitySymbol + '~' + securityType + "~" + exchangeName // TODO: do it in JSON.stringify way
};
return suggestion;
}
})
});
})();
/* shared\js\QuickSearch.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Utilities", {
QuickSearch: WinJS.Class.define(
function quicksearchCtor () {
// Initialize AutoSuggest service
this._autoSuggest = new FinanceAppJS.Utilities.AutoSuggest();
this._noOfSuggestions = FinanceAppJS.Constants.QuickSearch.NoOfSuggestions;
this._getSymbolsPromiseFunc_binding = this._getSymbolsPromiseFunc.bind(this);
this._handleQuerySubmittedComplete_binding = this._handleQuerySubmittedComplete.bind(this);
this._handleQuerySubmittedError_binding = this._handleQuerySubmittedError.bind(this);
},
{
_autoSuggest: null,
_noOfSuggestions: null,
_query: null,
_suggPromise: null,
_event: null,
_source: null,
_searchDialog: null,
_listElemWin: null,
_panoName: null,
_isMessageShowing: false,
_addStockPromise: null,
_querySubmitted: false,
_queryResults: null,
release: function release() {
this._listElemWin = null;
this._panoName = null;
this._suggPromise = null;
this._event = null;
this._source = null;
this._autoSuggest.release();
this._wlProvider = null;
this._addStockPromise = null;
},
init: function init(watchList, panoName, provider) {
this._listElemWin = watchList;
this._panoName = panoName;
this._wlProvider = provider;
},
showProgress: function showProgress() {
if (this._searchDialog) {
this._searchDialog.showSearchProgress();
}
CommonJS.dismissAllEdgies();
},
hideProgress: function hideProgress() {
if (this._searchDialog) {
this._searchDialog.hideSearchProgress();
}
CommonJS.dismissAllEdgies();
},
showMessage: function showMessage(msg, isResourceKey) {
if (this._searchDialog) {
this._searchDialog.message = (isResourceKey === false) ? msg : utils.getResourceString(msg);
}
CommonJS.dismissAllEdgies();
},
hideMessage: function hideMessage() {
if (this._searchDialog) {
this._searchDialog.message = "";
}
CommonJS.dismissAllEdgies();
},
showQuickSearch: function showQuickSearch() {
var that = this;
if (this._wlProvider) {
this._wlProvider.cancelFetch();
this._wlProvider.pausePeriodicRefresh();
}
if (!this._searchDialog) {
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
FinanceAppJS.Telemetry.FlightRecorder.LogAppEvent(page,
FinanceAppJS.Telemetry.Constants.AppUtility.FLYOUT,
"");
this._searchDialog = CommonJS.SearchDialog.show({
onSearchTextEntered: function onSearchTextEntered(event) {
var elmWin = event.src;
var query = utils.trim(event.queryText);
if (!query) {
return;
}
that._query = query;
if (FinanceAppJS.Globals.internetAvailable === false) {
that.showMessage("ASOfflineMessage");
return;
}
// Suggestion is chosen from the Auto-Suggest tray
if (typeof event.index !== 'undefined' && event.index >= 0 && elmWin.searchSuggestions && that._panoName && that._listElemWin) {
var suggItem = elmWin.searchSuggestions[event.index];
that._queryResults.selectionIndex = event.index + 1;
that._querySubmitted = true;
FinanceAppJS.Telemetry.FlightRecorder.LogSearchApp("CompleteQuickSearch", that._query, that._queryResults);
if (that._addStockPromise) {
that._addStockPromise.cancel();
}
that._addStockPromise = WatchListAutoSuggest.addStock(that._query, suggItem.value, that._panoName, that._listElemWin);
}
else {
// Didn't wait for suggestion. Type & hit enter too fast. In this case, we query AutoSuggest and then Add the first suggestion to watchlist
// Cancel if there are any existing promises
if (that._suggPromise) {
that._suggPromise.cancel();
}
that.showProgress();
that._autoSuggest.getSuggestions(that._query, 1).then(that._handleQuerySubmittedComplete_binding, that._handleQuerySubmittedError_binding);
}
},
onSearchSuggestionsRequested: this.populateSymbols.bind(this),
autoSuggestOptions: {
selectionMode: CommonJS.AutoSuggest.forceSelection,
},
searchPlaceholderText: utils.getResourceString("AUTOSUGGEST_PLACEHOLDER"),
promptText: utils.getResourceString("AddStockPopupTitle"),
onHide: function onHide() {
if (that._querySubmitted === false) {
var data = that._queryResults ? that._queryResults : "";
FinanceAppJS.Telemetry.FlightRecorder.LogSearchApp("CancelQuickSearch", that._query, data);
}
that._searchDialog = null;
that._wlProvider && (that._wlProvider.resumePeriodicRefresh());
that.resetQueryOnExit();
},
onCloseButtonClick: this.onCancelButtonClick.bind(this)
});
}
CommonJS.dismissAllEdgies();
},
resetQueryOnExit: function resetQueryOnExit() {
this._query = null;
this._querySubmitted = false;
this._queryResults = {};
},
onCancelButtonClick: function onCancelButtonClick() {
/* do the platform task of enabling searchOnType and hide */
this._searchDialog.searchVisible = false;
/* cancel the addStock Promise when user exits the add dialogue */
if (this._addStockPromise) {
this._addStockPromise.cancel();
this._addStockPromise = null;
}
},
hideQuickSearch: function hideQuickSearch() {
if (this._searchDialog) {
// Hide Progressbar
this.hideProgress();
this.setQuickSearchText("");
this.hideMessage();
this._searchDialog.searchVisible = false;
}
CommonJS.dismissAllEdgies();
},
setQuickSearchText: function setQuickSearchText(message) {
if (this._searchDialog) {
this._searchDialog.searchBoxText = message;
}
CommonJS.dismissAllEdgies();
},
populateSymbols: function populateSymbols(event, src) {
// Clear if there are any error messages in previous
this.hideMessage();
var text = utils.trim(event.queryText);
//In offline mode - show empty list and show offline message
if (FinanceAppJS.Globals.internetAvailable === false) {
src.searchSuggestions = { event: event, suggestions: [] };
if (text) {
this.showMessage("ASOfflineMessage");
this._isMessageShowing = true;
}
return;
}
if (this._isMessageShowing) {
this.hideMessage();
this._isMessageShowing = false;
}
switch (event.key) {
case "Enter":
// user has to select from the list... dont accept any other text.
// TODO: display error message
break;
default:
if (!text) {
src.searchSuggestions = { event: event, suggestions: [] };
}
else {
if (this._suggPromise) {
this._suggPromise.cancel();
}
var deferral = event.request.getDeferral();
this._event = event;
this._source = src;
this._suggPromise = this.getSymbols(text).then(function (symbolList) {
src.searchSuggestions = { event: event, suggestions: symbolList };
deferral.complete();
});
}
break;
}
},
getSymbols: function getSymbols(query) {
this._query = query;
return new WinJS.Promise(this._getSymbolsPromiseFunc_binding);
},
_getSymbolsPromiseFunc: function _getSymbolsPromiseFunc(complete, error) {
var that = this;
this._autoSuggest.getSuggestions(this._query, this._noOfSuggestions).then(
function (data) {
var sugg = [];
that._queryResults = {};
for (var cnt = 0; cnt < data.length; cnt++) {
var suggestion = that._autoSuggest.buildSuggestion(data[cnt], FinanceAppJS.Globalization.getLanguage().toLowerCase());
if (!suggestion) { continue; }
var displayedText = (cnt === 0) ? that._query : suggestion.displaySymbol; // Bug#215205: Showing typed query for first entry.
sugg.push({
Symbol: suggestion.displaySymbol,
Title: suggestion.companyName,
Subtitle: suggestion.symbolTypeText,
Comment: suggestion.exchangeName,
displayedText: displayedText,
value: suggestion.value
});
that._queryResults[cnt + 1] = suggestion.value;
}
complete(sugg);
},
function (e) {
error(e);
});
},
_handleQuerySubmittedComplete: function _handleQuerySubmittedComplete(symbolList) {
if (symbolList && symbolList.length) {
var suggestion = this._autoSuggest.buildSuggestion(symbolList[0], FinanceAppJS.Globalization.getLanguage().toLowerCase());
if (suggestion && suggestion.value) {
var data = {};
data.selectionIndex = 1;
for (var i = 0; i < symbolList.length; i++) {
if (i === 0) {
data[i + 1] = suggestion.value;
} else {
data[i + 1] = this._autoSuggest.buildSuggestion(symbolList[i], FinanceAppJS.Globalization.getLanguage().toLowerCase()).value;
}
}
this._querySubmitted = true;
FinanceAppJS.Telemetry.FlightRecorder.LogSearchApp("CompleteQuickSearch", this._query, data );
WatchListAutoSuggest.addStock(this._query, suggestion.value, this._panoName, this._listElemWin);
return;
}
}
this._showError(utils.getResourceString("ASInvalidEntry").format(this._query), false);
},
_handleQuerySubmittedError: function _handleQuerySubmittedError(errData) {
this._showError(utils.getResourceString("ASNoDataAvailable").format(this._query), false);
},
_showError: function _showError(msg, isResourceKey) {
this.hideProgress();
this.showMessage(msg, isResourceKey);
}
},
{
_instance: null,
getInstance: function getInstance() {
var quickSearch = FinanceAppJS.Utilities.QuickSearch;
if (quickSearch._instance === null) {
quickSearch._instance = new FinanceAppJS.Utilities.QuickSearch();
}
return quickSearch._instance;
}
})
});
})();
/* shared\js\SearchCharm.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Utilities", {
SearchCharm: WinJS.Class.define(
function SearchCharmCtor () {
try {
var searchPane = Windows.ApplicationModel.Search.SearchPane.getForCurrentView();
searchPane.placeholderText = utils.getResourceString("AUTOSUGGEST_PLACEHOLDER");
searchPane.addEventListener("querysubmitted", this.handleQuerySubmitted.bind(this), false);
searchPane.addEventListener('suggestionsrequested', this.handleSuggestionsRequested.bind(this), false);
searchPane.addEventListener("resultsuggestionchosen", this.handleResultSuggestionChosen.bind(this), false);
} catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
}
// Initialize AutoSuggest service
this._autoSuggest = new FinanceAppJS.Utilities.AutoSuggest();
this._noOfSuggestions = FinanceAppJS.Constants.SearchCharm.NoOfSuggestions;
this._handleQuerySubmittedComplete_binding = this._handleQuerySubmittedComplete.bind(this);
},
{
_autoSuggest: null,
_noOfSuggestions: null,
_queryText: null,
populateResultSuggestions: function populateResultSuggestions(e) {
//In offline mode - show empty list
if (FinanceAppJS.Globals.internetAvailable === false) {
return;
}
this._queryText = e.queryText;
var request = e.request;
// Indicate that we'll do this asynchronously:
var deferral = request.getDeferral();
var that = this;
this._autoSuggest.getSuggestions(this._queryText, this._noOfSuggestions).then(function (data) {
var qryResults = [];
for (var cnt = 0; cnt < data.length; cnt++) {
var suggestion = that._autoSuggest.buildSuggestion(data[cnt], FinanceAppJS.Globalization.getLanguage().toLowerCase());
if (!suggestion) { continue; }
var imageSourceUrl = null;
if (utils.isFundSymbol(suggestion.symbolType)) {
imageSourceUrl = "ms-appx:///shared/images/fundIconNew.png";
}
else if (utils.isStockSymbol(suggestion.symbolType)) {
imageSourceUrl = "ms-appx:///shared/images/stockIconNew.png";
}
else if (utils.isEtfSymbol(suggestion.symbolType)) {
imageSourceUrl = "ms-appx:///shared/images/etfIconNew.png";
}
else if (utils.isIndexSymbol(suggestion.symbolType)) {
// TODO: Add the proper image for index once we get the image from the designers.
imageSourceUrl = "ms-appx:///shared/images/stockIconNew.png";
}
var txtHighlight = detailText = null;
var detailText = suggestion.companyName;
if (suggestion.displaySymbol) {
txtHighlight = suggestion.displaySymbol;
}
else {
txtHighlight = suggestion.companyName;
detailText = "";
}
qryResults[cnt + 1] = suggestion.value;
var text = suggestion.exchangeName ? FinanceAppJS.Utils.Formatters.stringFormatter("{0} - {1}: {2}", [txtHighlight, suggestion.symbolTypeText, suggestion.exchangeName]) : FinanceAppJS.Utils.Formatters.stringFormatter("{0} - {1}", [txtHighlight, suggestion.symbolTypeText]);
var imageUri = new Windows.Foundation.Uri(imageSourceUrl);
var imageSource = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(imageUri);
if (imageSource) {
request.searchSuggestionCollection.appendResultSuggestion(text, detailText, suggestion.value, imageSource, suggestion.symbolTypeText);
}
}
FinanceAppJS.Telemetry.FlightRecorder.SetSearchCharmQueryDisplay(qryResults);
deferral.complete(); // Indicate we're done supplying suggestions.
}, function (e) {
});
},
handleQuerySubmitted: function handleQuerySubmitted(e) {
var queryText = utils.trim(e.queryText);
if (!queryText) {
return;
}
this._queryText = queryText;
//In offline mode - show offline message
if (FinanceAppJS.Globals.internetAvailable === false) {
// TODO: sugu log offline App Action
var msgbar = new CommonJS.MessageBar(utils.getResourceString("ASOfflineMessage"), { autoHide: true });
msgbar.show();
return;
}
this._autoSuggest.getSuggestions(this._queryText, 1).then(this._handleQuerySubmittedComplete_binding);
},
_handleQuerySubmittedComplete: function _handleQuerySubmittedComplete(symbolList) {
if (symbolList.length > 0) {
var suggestion = this._autoSuggest.buildSuggestion(symbolList[0], FinanceAppJS.Globalization.getLanguage().toLowerCase());
var data = {};
data.selectionIndex = 1;
for (var i = 0; i < symbolList.length; i++) {
if (i === 0) {
data[i + 1] = suggestion.value;
} else {
data[i + 1] = this._autoSuggest.buildSuggestion(symbolList[i], FinanceAppJS.Globalization.getLanguage().toLowerCase()).value;
}
}
var tag = suggestion.value;
if (tag) {
FinanceAppJS.Telemetry.FlightRecorder.LogSearchApp("CompleteSearchCharm", this._queryText, data);
FinanceAppJS.Telemetry.FlightRecorder.SetNavigationMethod(FinanceAppJS.Telemetry.Constants.NavMethod.SEARCHCHARM);
navigationHelper.navigateToDetailsPage(tag);
}
}
else {
PlatformJS.Navigation.navigateToChannel("NoSearchResults", { query: this._queryText });
}
},
// Register to Handle Suggestion Request
handleSuggestionsRequested: function handleSuggestionsRequested(e) {
this.populateResultSuggestions(e);
},
// Handle the selection of a Result Suggestion
handleResultSuggestionChosen: function handleResultSuggestionChosen(e) {
var data = { selectionIndex: 1 };
data[0] = e.tag;
FinanceAppJS.Telemetry.FlightRecorder.LogSearchApp("CompleteSearchCharm", this._queryText, data);
FinanceAppJS.Telemetry.FlightRecorder.SetNavigationMethod(FinanceAppJS.Telemetry.Constants.NavMethod.SEARCHCHARM);
navigationHelper.navigateToDetailsPage(e.tag);
}
},
{
_instance: null,
getInstance: function getInstance() {
var searchCharm = FinanceAppJS.Utilities.SearchCharm;
if (searchCharm._instance === null) {
searchCharm._instance = new FinanceAppJS.Utilities.SearchCharm();
}
return searchCharm._instance;
}
})
});
})();
/* shared\js\ShareContract.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Utilities", {
ShareContract: WinJS.Class.define(
function ShareContractCtor () {
try {
// Obtain the Data Transfer object and register for handling search while running as the main application
this._dataTransfer = Windows.ApplicationModel.DataTransfer;
var dataTransferManager = this._dataTransfer.DataTransferManager.getForCurrentView();
dataTransferManager.addEventListener("datarequested", this.handleDataRequested.bind(this));
this.handleDataRequestedComplete_binding = this.handleDataRequestedComplete.bind(this);
this.handleDataRequestedError_binding = this.handleDataRequestedError.bind(this);
} catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
}
},
{
_dataTransfer: null,
_request: null,
_deferral: null,
handleDataRequested: function handleDataRequested(e) {
if (Platform.Configuration.ConfigurationManager.instance.isEmbargoed) {
// do not share
return;
}
this._request = e.request;
var currentIPage = PlatformJS.Navigation.currentIPage;
if (currentIPage && typeof currentIPage.getSharedContractData === 'function') {
this._deferral = this._request.getDeferral();
var sharedContractDataPromise = PlatformJS.Navigation.currentIPage.getSharedContractData();
if (sharedContractDataPromise && WinJS.Promise.is(sharedContractDataPromise)) {
sharedContractDataPromise.then(this.handleDataRequestedComplete_binding, this.handleDataRequestedError_binding);
}
}
else if (currentIPage && typeof currentIPage.handleShareRequest === 'function') {
// Sharing for Article Reader
currentIPage.handleShareRequest(this._request);
}
else {
this._request.failWithDisplayText(utils.getResourceString("ShareCharmDataNotAvailable"));
}
},
handleDataRequestedComplete: function handleDataRequestedComplete(sharedData) {
if (sharedData) {
this._request.data.properties.title = sharedData.title;
var htmlFormat = this._dataTransfer.HtmlFormatHelper.createHtmlFormat(sharedData.htmlContent);
this._request.data.setHtmlFormat(htmlFormat);
var uriString = sharedData.uri || Platform.Utilities.AppExUri.default.uri;
var uri = new Windows.Foundation.Uri(uriString);
this._request.data.setText(sharedData.textContent + "\n\n" + uriString);
this._request.data.setUri(uri);
}
else {
this._request.failWithDisplayText(utils.getResourceString("ShareCharmDataNotAvailable"));
}
this._deferral.complete();
},
handleDataRequestedError: function handleDataRequestedError(error) {
if (error.msg) {
this._request.failWithDisplayText(error.msg);
}
this._deferral.complete();
}
},
{
_instance: null,
getInstance: function getInstance() {
var shareContract = FinanceAppJS.Utilities.ShareContract;
if (shareContract._instance === null) {
shareContract._instance = new FinanceAppJS.Utilities.ShareContract();
}
return shareContract._instance;
}
})
});
})();
/* shared\js\SecondaryTile.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.SecondaryTile", {
getTileID: function getTileID(symbolDesc, type) {
var tileID = type + "_" + symbolDesc;
tileID = tileID.replace(/\,/g, "-");
tileID = tileID.replace(/ /g, "-");
tileID = tileID.replace(/\//g, "-");
tileID = tileID.replace(/!/g, "-");
return tileID;
},
isTilePinned: function isTilePinned(symbolDesc, type) {
var isPinned;
var tileID = FinanceAppJS.SecondaryTile.getTileID(symbolDesc, type);
try {
isPinned = Windows.UI.StartScreen.SecondaryTile.exists(tileID);
} catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
isPinned = false;
}
return isPinned;
},
pinTile: function pinTile(symbolDesc, type, pageState, displayName, shortName, complete, point, instrumentId) {
//
// Verify if we are still under the maximum pinned locations limit.
//
complete = complete || function () { };
var state = {};
switch (type) {
case FinanceAppJS.Constants.IDs.Fund:
shortName = pageState.entity.Tkr || pageState.entity.Fn;
displayName = pageState.entity.Fn;
state.entity = pageState.entity;
break;
case FinanceAppJS.Constants.IDs.Etf:
case FinanceAppJS.Constants.IDs.Stock:
shortName = utils.getAutoSuggestDisplaySymbol(pageState.entity);
displayName = pageState.entity.Cmp;
state.entity = pageState.entity;
break;
case FinanceAppJS.Constants.IDs.Index:
shortName = pageState.entity.displayName;
displayName = pageState.entity.displayName;
state.entity = pageState.entity;
break;
default:
shortName = utils.getResourceString("PinTile_WatchList");
displayName = shortName;
break;
}
displayName = utils.trim(displayName);
displayName = displayName || shortName;
// Adding instrumentation for PinUnpin bottomedgy
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, instrumentId, 1);
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
var data = { ButtonType: "Pin", tileId: FinanceAppJS.SecondaryTile.getTileID(symbolDesc, type), Name: displayName };
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.BtmEdgyBtn.PIN,
data);
var that = FinanceAppJS.SecondaryTile;
return Windows.UI.StartScreen.SecondaryTile.findAllAsync().then(function (tiles) {
if (tiles) {
var MAX_PINNED_LOCATIONS = PlatformJS.Services.appConfig.getInt32("MaxSecondaryTileLimit");
if (tiles.size < MAX_PINNED_LOCATIONS) {
var secTileConst = FinanceAppJS.Constants.SecondaryTile;
var tileID = FinanceAppJS.SecondaryTile.getTileID(symbolDesc, type);
var uriLogo = new Windows.Foundation.Uri(secTileConst.Logo);
var uriWideLogo = new Windows.Foundation.Uri(secTileConst.WideLogo);
var uriSmallLogo = new Windows.Foundation.Uri(secTileConst.SmallLogo);
var data = { Symbol: symbolDesc, Name: shortName };
FinanceAppJS.Telemetry.FlightRecorder.LogAppEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.AppUtility.PINTILEMSG,
data);
var tile = new Windows.UI.StartScreen.SecondaryTile(tileID,
shortName,
displayName,
JSON.stringify({ tileType: type, pageState: state }),
Windows.UI.StartScreen.TileOptions.showNameOnWideLogo + Windows.UI.StartScreen.TileOptions.showNameOnLogo + Windows.UI.StartScreen.TileOptions.copyOnDeployment,
uriLogo,
uriWideLogo);
tile.smallLogo = uriSmallLogo;
tile.requestCreateAsync(point).then(function (isCreated) {
if (isCreated) {
// this is the place where actually pining has been done 
var data = { Symbol: symbolDesc, TotalPinCount: tiles.size };
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.AppUtility.PINTILEMSG,
FinanceAppJS.Telemetry.Constants.AppUtility.PINTILESTART,
data);
// The tile was successfully created. Start periodic updates.
that.startPeriodicUpdateForSecondaryTile(symbolDesc, type);
}
complete();
}
);
} else {
(new CommonJS.MessageBar(utils.getResourceString("ErrMsg_MaxPinTile"), { autoHide: true })).show();
}
}
});
},
unpinTile: function unpinTile(symbolDesc, type, complete, point, instrumentId) {
var tileID = FinanceAppJS.SecondaryTile.getTileID(symbolDesc, type);
var tile = Windows.UI.StartScreen.SecondaryTile(tileID);
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
var data = { ButtonType: "UnPin", TileId: tileID };
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.BtmEdgyBtn.UNPIN,
data);
return tile.requestDeleteAsync(point).then(function (isDeleted) {
complete();
});
},
createWatchListPollUrls: function createWatchListPollUrls(tickers, type) {
var urisToPoll = [];
if (tickers !== null && tickers.length > 0) {
for (var i = tickers.length - 1; i >= 0; i--) {
urisToPoll.push(utils.createTileNotificationUri(tickers[i], type, 1));
}
}
return urisToPoll;
},
startPeriodicUpdateForSecondaryTile: function startPeriodicUpdateForSecondaryTile(symbolDesc, type) {
try {
var tileID = FinanceAppJS.SecondaryTile.getTileID(symbolDesc, type);
var recurrence = utils.GetNotificationFrequency();
var notifications = Windows.UI.Notifications;
if (type !== FinanceAppJS.Constants.IDs.WatchList) {
var serverUriForTile = utils.createTileNotificationUri(symbolDesc, type, 0);
notifications.TileUpdateManager.createTileUpdaterForSecondaryTile(tileID).startPeriodicUpdate(serverUriForTile, recurrence);
}
else {
//get the users watchlist
var financeRoaming = new FinanceAppJS.Roaming();
var watchlistList = financeRoaming.getWatchListSymbols(5);
var tileUpdater = notifications.TileUpdateManager.createTileUpdaterForSecondaryTile(tileID);
var stockUris = FinanceAppJS.SecondaryTile.createWatchListPollUrls(watchlistList.stocksOnly, FinanceAppJS.Constants.IDs.Stock);
var etfUri = FinanceAppJS.SecondaryTile.createWatchListPollUrls(watchlistList.etfOnly, FinanceAppJS.Constants.IDs.Etf);
var fundUri = FinanceAppJS.SecondaryTile.createWatchListPollUrls(watchlistList.mftOnly, FinanceAppJS.Constants.IDs.Fund);
var idxUri = FinanceAppJS.SecondaryTile.createWatchListPollUrls(watchlistList.idxOnly, FinanceAppJS.Constants.IDs.Index);
var urisToPoll = stockUris.concat(etfUri, fundUri, idxUri);
tileUpdater.clear();
if (urisToPoll.length > 0) {
tileUpdater.enableNotificationQueue(true);
tileUpdater.startPeriodicUpdateBatch(urisToPoll, recurrence);
}
}
} catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
}
},
handleActivation: function handleActivation(Arguments) {
var args = JSON.parse(Arguments);
if (args.tileType !== null && args.tileType !== undefined) {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(
FinanceAppJS.Telemetry.Constants.Page.MOGOSCREEN,
"",
FinanceAppJS.Telemetry.Constants.Location.PINNEDTILE,
args.pageState.entity);
FinanceAppJS.Telemetry.FlightRecorder.LogSearchApp(
FinanceAppJS.Telemetry.Constants.Location.PINNEDTILE,
"",
args.pageState.entity);
FinanceAppJS.Telemetry.FlightRecorder.SetNavigationMethod(FinanceAppJS.Telemetry.Constants.NavMethod.PINTILE);
switch (args.tileType) {
case FinanceAppJS.Constants.IDs.Stock:
return activationHelper.navigateToEntityPage(args.pageState);
case FinanceAppJS.Constants.IDs.Fund:
return activationHelper.navigateToFundDetailsPage(args.pageState);
case FinanceAppJS.Constants.IDs.Etf:
return activationHelper.navigateToETFDetailsPage(args.pageState);
case FinanceAppJS.Constants.IDs.WatchList:
return activationHelper.navigateToMyStocksPage(args.pageState);
case FinanceAppJS.Constants.IDs.Index:
return activationHelper.navigateToIndexDetailsPage(args.pageState);
}
}
else {
//The args.target will always have the value WSJJS.CategoryPano. We are not overriding this
//Here we don't have to replace WSJJS.CategoryPano with FinanceAppJS.WSJPano, as this is just an identifier for the WSJ pano
//Navigate to WSJ page will anyways navigate to FinanceAppJS.WSJPano
if (args.target && args.target === "WSJJS.CategoryPano") {
return activationHelper.navigateToWSJPage(args.data);
} else if (args.target === "DynamicPano.DynamicCategoryPano") {
args.data.entrypoint = "secondarytile";
return activationHelper.navigateToDynamicPano(args.data);
}
}
},
updateAllTiles: function updateAllTiles() {
Windows.UI.StartScreen.SecondaryTile.findAllAsync().done(function (tiles) {
if (tiles) {
var secTileConst = FinanceAppJS.Constants.SecondaryTile;
var logo = secTileConst.Logo;
var smallLogo = secTileConst.SmallLogo;
var wideLogo = secTileConst.WideLogo;
tiles.forEach(function (tile) {
try {
var args = JSON.parse(tile.arguments);
if (args.tileType !== null && args.tileType !== undefined && args.pageState) {
var tileType = args.tileType;
var pageState = args.pageState;
var symbolDesc = null;
var argsToUpdate = null;
if (tileType === FinanceAppJS.Constants.IDs.Fund && pageState.entity) {
symbolDesc = pageState.entity.Tkr && pageState.entity.Tkr !== "" ? pageState.entity.Tkr : pageState.entity.Scid;
} else if ((tileType === FinanceAppJS.Constants.IDs.Etf || tileType === FinanceAppJS.Constants.IDs.Stock) && pageState.entity) {
symbolDesc = utils.getGlobalInstrumentSymbol(pageState.entity);
} else if (tileType === FinanceAppJS.Constants.IDs.Index && pageState.entity) {
symbolDesc = pageState.entity.indexSymbol;
}
// Backward compatible for secondary tile to work; Remove this block after NEXT release
else if (tileType === FinanceAppJS.Constants.IDs.Stock && pageState.items) {
symbolDesc = utils.getGlobalInstrumentSymbol(pageState.items[0]);
var entityItem = {
entity: pageState.items[0]
};
argsToUpdate = JSON.stringify({ tileType: tileType, pageState: entityItem });
} else if (tileType === FinanceAppJS.Constants.IDs.WatchList) {
symbolDesc = FinanceAppJS.Constants.IDs.WATCHLIST_SYMBOL;
}
if (symbolDesc) {
FinanceAppJS.SecondaryTile.startPeriodicUpdateForSecondaryTile(symbolDesc, args.tileType);
//Updating image location. Location of the image might have changed from previous install.
var tileToUpdate = new Windows.UI.StartScreen.SecondaryTile(tile.tileId);
tileToUpdate.smallLogo = new Windows.Foundation.Uri(smallLogo);
tileToUpdate.wideLogo = new Windows.Foundation.Uri(wideLogo);
tileToUpdate.logo = new Windows.Foundation.Uri(logo);
if (argsToUpdate) {
tileToUpdate.arguments = argsToUpdate;
}
tileToUpdate.updateAsync();
}
}
} catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
} // The tile some how became corrupt. Can't update it
});
}
});
}
});
})();
/* shared\js\converters.js */
(function () {
'use strict';
var dir;
function appender(convert, separator) {
return function (source, sourceProperties, dest, destProperties) {
var initialValue = dest;
var src = source;
sourceProperties.forEach(function (item) { src = src[item]; });
destProperties.forEach(function (item) { initialValue = initialValue[item]; });
var bindObj = dest;
for (var i = 0, key; i < destProperties - 1 && (key = destProperties[i]) ; i++) {
bindObj = bindObj[key];
}
initialValue = bindObj[destProperties[destProperties.length - 1] + "_initial"] || initialValue;
var newValue = initialValue + separator + convert(src);
bindObj[destProperties[destProperties.length - 1] + "_initial"] = initialValue;
function convertwrapper(newvalue) {
return initialValue + separator + convert(newvalue);
}
var converter = WinJS.Binding.converter(convertwrapper);
return converter(source, sourceProperties, dest, destProperties);
};
}
function timeZoneFormat(data) {
return utils.timeZoneFormat(data.tz, data.utcOff);
}
function timeZoneFormatNoLabel(data) {
return utils.timeZoneFormatNoLabel(data.tz, data.utcOff);
}
function ColorConverter(chng) {
var clr;
if (chng > 0) {
clr = utils.JsLessVariables(true).gainColor;
}
else if (chng < 0) {
clr = utils.JsLessVariables(true).loseColor;
}
else {
clr = utils.JsLessVariables(true).equalColor;
}
return clr;
}
function StockClassConverter(item) {
if (!item) { return ""; }
var cls = "";
if (item.type === 'F') {
cls =  stockClassConverterSmall(item.Odr, true);
} else {
cls =  stockClassConverterSmall(item.Ch, true);
}
return cls;
}
function stockClassConverterSmall(chng, addMediumGlyph) {
if (chng === null) {
return;
}
var cls = "";
if (chng > 0) {
cls = "financeGlyphUp";
}
else if (chng < 0) {
cls = "financeGlyphDn";
}
else if (Number(chng) === 0) {
if (addMediumGlyph) {
cls = "mediumGlyph financeGlyphEq";
}
else {
cls = "financeGlyphEq";
}
}
return cls;
}
function IndicesClassConverter(item) {
// if (!item) return;
var cls = stockClassConverterSmall(item, true);
return cls;
}
function IndicesClassConverterSmall(item) {
var cls = stockClassConverterSmall(item, false);
return cls;
}
function NumberFormator(number) {
return FinanceAppJS.Utils.Formatters.numberFormatter(number, null, true);
}
function NumberFormatorWithoutSign(number) {
return FinanceAppJS.Utils.Formatters.numberFormatter(number, null);
}
//change functionname based on precision later
function NumberFormatorWithoutSign5(number) {
return FinanceAppJS.Utils.Formatters.numberFormatter(number, 4);
}
function NumberFormator5(number) {
return FinanceAppJS.Utils.Formatters.numberFormatter(number, 5, true);
}
function dateFormat(str) {
if (str) {
var mon = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
var hyphenIndex = str.indexOf("-");
var date = str.substring(0, hyphenIndex);
var hyphenIndex2 = str.indexOf("-", hyphenIndex + 1);
var month = parseInt(str.substring(hyphenIndex + 1, hyphenIndex2));
var year = str.substring(hyphenIndex2 + 1, str.length);
return mon[month - 1] + " " + date + ", " + year;
}
else {
return '';
}
}
function dateTimeFormat(str) {
return utils.dateTimeFormat(str);
}
function longDateFormat(str) {
return FinanceAppJS.Utils.Formatters.longDateFormat(str);
}
function shortDateFormat(str) {
return FinanceAppJS.Utils.Formatters.shortDateFormat(str);
}
function shortTimeFormat(str) {
return FinanceAppJS.Utils.Formatters.shortTimeFormat(str);
}
function timeFormat(str) {
if (str) {
return utils.timeFormat(str);
/*
var colonIndex = str.indexOf(":");
var hour = str.substring(str.indexOf(" "), colonIndex);
var append;
if (parseInt(hour) > 12) {
append = " PM";
hour = parseInt(hour) - 12;
if (hour.toString().length < 2)
hour = "0" + hour;
}
else {
append = " AM";
}
var minute = str.substring(colonIndex + 1, str.indexOf(":", colonIndex + 1));
return hour + ":" + minute + append;
*/
}
else {
return '';
}
}
function visibilityConverter(obj) {
if (!(obj === null || typeof obj === 'undefined' || obj === "")) {
return 'visible';
}
else {
return 'hidden';
}
}
function nullOrEmptyVisibilityConverter(obj) {
return obj && obj !== "" ? "visible" : "hidden";
}
function nonWhiteSpaceVisibilityConverter(obj) {
return obj && obj !== "" && obj.replace(/ /, "") !== "" ? "visible" : "hidden";
}
function styleDisplayConverter(obj) {
if (typeof obj === "undefined" || obj === null || obj === "" || obj === false) {
return 'none';
}
else {
return 'block';
}
}
function newsAbstractDisplayConverter(obj) {
if (obj) {
return 'newsText ctrlHeight';
}
else {
return 'newsText';
}
}
/// <summary>Special Currency Formatter to enclose the Currency code with a span tag for formatting</summary>
/// <param name="obj" type="integer">The currency amount</param>
/// Currently no Symbol is added to string
function currencyFormatterSpecial(obj) {       
var currencyStr = currencyFormatterWithNoSymbol(obj);
var currencyCode = FinanceAppJS.Utils.Formatters.getCurrencySymbol();
currencyStr = currencyStr.replace(currencyCode, "<span class=currencyCode>" + currencyCode + "</span>");
return currencyStr;
}
function currencyFormatter(obj) {
/// <summary>Format the amount into the localized currency</summary>
return FinanceAppJS.Utils.Formatters.currencyFormatter(obj, FinanceAppJS.Globalization.getDefaultCurrency(), true);
}
function currencyFormatterWithNoSymbol(obj) {
/// <summary>Format the amount into the localized currency and display numbers without signs</summary>
return FinanceAppJS.Utils.Formatters.currencyFormatter(obj, FinanceAppJS.Globalization.getDefaultCurrency(), false);
}
function formatValueAndAddCcySymbol(obj) {
var formattedValue = CollectionProvider.Formatters.formatNumber(obj,0);
return addCurrencySymbol(formattedValue);
}
function addCurrencySymbol(obj) {
var formattedValue = obj;
if (!formattedValue) {
return "";
}
var currencyFormat = CollectionProvider.Formatters.getCurrencyPositivePattern();
var currencySymbol = CollectionProvider.Formatters.getCurrencySymbol();
switch (currencyFormat) {
case 0:
formattedValue = currencySymbol + formattedValue;
break;
case 1:
formattedValue += currencySymbol;
break;
case 2:
formattedValue = currencySymbol + " " + formattedValue;
break;
case 3:
formattedValue += " " + currencySymbol;
break;
}
return formattedValue;
}
function rateNoPercentFormat(obj) {
return FinanceAppJS.Utils.Formatters.numberFormatter(obj, 3, false, "");
}
function rateNoPercentFormatWithoutLTR(obj) {
return FinanceAppJS.Utils.Formatters.numberFormatter(obj, 3, false, "", true);
}
function rateFormat(obj) {
return FinanceAppJS.Utils.Formatters.percentFormatter(obj, 3, false, "-", true); 
}
function rate2Format(obj) {
return FinanceAppJS.Utils.Formatters.percentFormatter(obj, 2, true, "");
}
function rate3Format(obj) {
return FinanceAppJS.Utils.Formatters.percentFormatter(obj, 2, true);
}
function ValueChangeIndicatorCSSClass(obj) {
if (typeof obj === 'undefined' || obj === null) { return ""; }
if (typeof obj !== 'number') { obj = parseFloat(obj); }
if (obj === 0) { return "financeGlyphEq"; }
if (obj < 0) { return "financeGlyphDn"; }
if (obj > 0) { return "financeGlyphUp"; }
}
function ValueChangeColorCSSClass(obj) {
if (typeof obj === 'undefined' || obj === null) { return ""; }
if (typeof obj !== 'number') { obj = parseFloat(obj); }
if (obj === 0) { return ""; }
if (obj < 0) { return "dn"; }
if (obj > 0) { return "up"; }
}
// including the EQ glyph symbol under a class to help style the element
function ValueChangeColorCSSClassIncludeEq(obj) {
var objVal = obj;
if (typeof obj === 'undefined' || obj === null || obj === "") { return ""; }
if (typeof obj !== 'number') { objVal = parseFloat(obj); }
if (objVal === 0) { return "eq"; }
if (objVal < 0) { return "dn"; }
if (objVal > 0) { return "up"; }
}
function seTrendImageDisplayConverter(obj) {
if (typeof obj === "undefined" || obj === null) {
return 'none';
}
else {
return 'inline';
}
}
function seTrendImageSource(obj) {
if (typeof obj === 'undefined' || obj === null) {
return "";
}
var mkt = FinanceAppJS.Globalization.getMarket();
if (typeof obj !== 'number') { obj = parseFloat(obj); }
if (obj === 0) { return "/shared/images/noChange.png"; }
if (obj < 0) { return "/shared/images/sell.png"; }
if (obj > 0) { return "/shared/images/buy.png"; }
}
function seTrendMessage(obj) {
if (!obj) { return "-"; }
if (typeof obj !== 'number') { obj = parseFloat(obj); }
if (obj === 0) { return utils.getResourceString("No Change"); }
if (obj < 0) { return utils.getResourceString("SELLING"); }
if (obj > 0) { return utils.getResourceString("BUYING"); }
}
function AverageRatesAnimatedConverter(data, source, elem, dest) {
if (!source || !dest || !elem || !data.items || !source[0] || !dest[0]) { return true; }
if (!data.index) { data.index = 0; }
elem[dest[0]] = data.items[data.index][source[0]];
return true;
}
function CountryFlagCSSClass(obj) {
return FinanceAppJS.Utils.getCountryFlagClass(obj);
}
function CountryFlagCSSClassIfExists(obj) {
var flagClass = FinanceAppJS.Utils.getCountryFlagClass(obj);
if (flagClass.indexOf('countryFlagSprite_NONE') !== -1) {
return "hide";
}
return flagClass;
}
function MarketOpenIndicator(obj) {
var className = "mktBar ";
className += (obj === 1) ? "marketOpen" : "marketClose";
return className;
}
function BarIndicatorClass(objHeight) {
var className = "hide ";
if (objHeight === "0px") {
return className;
}
}
function rangeConverter(low, high) {
if (!low && !high) {
return "-";
}
if (!low) {
low = "-";
}
if (!high) {
high = "-";
}
var left = low, right = high;
dir = dir ? dir : document.dir;
if (dir === "rtl") {
left = high;
right = low;
}
return FinanceAppJS.Utils.Formatters.stringFormatter("{0} - {1}", [FinanceAppJS.Utils.Formatters.numberFormatter(left, null), FinanceAppJS.Utils.Formatters.numberFormatter(right, null)]);
}
function indexChartLinkFormat(obj) {
return utils.indexChartLinkFormat(obj);
}
function dayRangeConverter(obj) {
return rangeConverter(obj.Dl, obj.Dh);
}
function yearRangeConverter(obj) {
return rangeConverter(obj.Yl, obj.Yh);
}
function vBarHeight(val) {
return val * 40 + "px";
}
function appendClass(obj) {
return obj || "";
}
function valueOverflowConverter(value) {
if (value <= 999) {
return "fontBig1";
}
else if (value > 999 && value < (1e+8)) {
return "fontBig2";
}
else if (value >= (1e+8) && value < (1e+11)) {
return "fontBig3";
}
else if (value >= (1e+11) && value < (1e+35)) {
return "fontBig4";
}
else {
return "fontBig1"; //if nothing is passed. This is for default case.
}
}
WinJS.Namespace.define('FinanceAppJS.ConverterFunc', {
indexChartLinkFormat: (indexChartLinkFormat),
StockClassConverter: (StockClassConverter),
stockClassConverterSmall: (stockClassConverterSmall),
NumberFormator: (NumberFormator),
NumberFormatorWithoutSign: (NumberFormatorWithoutSign),
NumberFormator5: (NumberFormator5),
NumberFormatorWithoutSign5: (NumberFormatorWithoutSign5),
ColorConverter: (ColorConverter),
visibilityConverter: (visibilityConverter),
nullOrEmptyVisibilityConverter: (nullOrEmptyVisibilityConverter),
nonWhiteSpaceVisibilityConverter: (nonWhiteSpaceVisibilityConverter),
styleDisplayConverter: (styleDisplayConverter),
newsAbstractDisplayConverter: (newsAbstractDisplayConverter),
AverageRatesAnimatedConverter: AverageRatesAnimatedConverter,
currencyFormatterSpecial: (currencyFormatterSpecial),
currencyFormatter: (currencyFormatter),
dateFormat: (dateFormat),
shortDateFormat: (shortDateFormat),
shortTimeFormat: (shortTimeFormat),
longDateFormat: (longDateFormat),
currencyFormatterWithNoSymbol: (currencyFormatterWithNoSymbol),
timeFormat: (timeFormat),
dateTimeFormat: (dateTimeFormat),
resourceString: (utils.getResourceString),
rateFormat: (rateFormat),
rate2Format: (rate2Format),
rate3Format: (rate3Format),
rateNoPercentFormat: (rateNoPercentFormat),
rateNoPercentFormatWithoutLTR: (rateNoPercentFormatWithoutLTR),
ValueChangeIndicatorCSSClass: (ValueChangeIndicatorCSSClass),
ValueChangeColorCSSClass: (ValueChangeColorCSSClass),
ValueChangeColorCSSClassIncludeEq: (ValueChangeColorCSSClassIncludeEq),
CountryFlagCSSClass: (CountryFlagCSSClass),
CountryFlagCSSClassIfExists: (CountryFlagCSSClassIfExists),
MarketOpenIndicator: (MarketOpenIndicator),
timeZoneFormat: (timeZoneFormat),
timeZoneFormatNoLabel: (timeZoneFormatNoLabel),
seTrendMessage: (seTrendMessage),
seTrendImageSource: (seTrendImageSource),
seTrendImageDisplayConverter: (seTrendImageDisplayConverter),
IndicesClassConverter: (IndicesClassConverter),
IndicesClassConverterSmall: (IndicesClassConverterSmall),
BarIndicatorClass: (BarIndicatorClass),
rangeConverter: (rangeConverter),
dayRangeConverter: (dayRangeConverter),
yearRangeConverter: (yearRangeConverter),
vBarHeight: (vBarHeight),
valueOverflowConverter: (valueOverflowConverter),
addCurrencySymbol: (addCurrencySymbol),
formatValueAndAddCcySymbol: (formatValueAndAddCcySymbol)
});
WinJS.Namespace.define('converters', {
indexChartLinkFormat: WinJS.Binding.converter(indexChartLinkFormat),
StockClassConverter: WinJS.Binding.converter(StockClassConverter),
stockClassConverterSmall: WinJS.Binding.converter(stockClassConverterSmall),
NumberFormator: WinJS.Binding.converter(NumberFormator),
NumberFormatorWithoutSign: WinJS.Binding.converter(NumberFormatorWithoutSign),
NumberFormator5: WinJS.Binding.converter(NumberFormator5),
NumberFormatorWithoutSign5: WinJS.Binding.converter(NumberFormatorWithoutSign5),
ColorConverter: WinJS.Binding.converter(ColorConverter),
visibilityConverter: WinJS.Binding.converter(visibilityConverter),
nullOrEmptyVisibilityConverter: WinJS.Binding.converter(nullOrEmptyVisibilityConverter),
nonWhiteSpaceVisibilityConverter: WinJS.Binding.converter(nonWhiteSpaceVisibilityConverter),
styleDisplayConverter: WinJS.Binding.converter(styleDisplayConverter),
newsAbstractDisplayConverter: WinJS.Binding.converter(newsAbstractDisplayConverter),
AverageRatesAnimatedConverter: AverageRatesAnimatedConverter,
currencyFormatterSpecial: WinJS.Binding.converter(currencyFormatterSpecial),
currencyFormatter: WinJS.Binding.converter(currencyFormatter),
dateFormat: WinJS.Binding.converter(dateFormat),
shortDateFormat: WinJS.Binding.converter(shortDateFormat),
shortTimeFormat: WinJS.Binding.converter(shortTimeFormat),
longDateFormat: WinJS.Binding.converter(longDateFormat),
currencyFormatterWithNoSymbol: WinJS.Binding.converter(currencyFormatterWithNoSymbol),
timeFormat: WinJS.Binding.converter(timeFormat),
dateTimeFormat: WinJS.Binding.converter(dateTimeFormat),
resourceString: WinJS.Binding.converter(utils.getResourceString),
rateFormat: WinJS.Binding.converter(rateFormat),
rate2Format: WinJS.Binding.converter(rate2Format),
rate3Format: WinJS.Binding.converter(rate3Format),
rateNoPercentFormat: WinJS.Binding.converter(rateNoPercentFormat),
rateNoPercentFormatWithoutLTR: WinJS.Binding.converter(rateNoPercentFormatWithoutLTR),
ValueChangeIndicatorCSSClass: WinJS.Binding.converter(ValueChangeIndicatorCSSClass),
ValueChangeColorCSSClass: WinJS.Binding.converter(ValueChangeColorCSSClass),
ValueChangeColorCSSClassIncludeEq: WinJS.Binding.converter(ValueChangeColorCSSClassIncludeEq),
CountryFlagCSSClass: WinJS.Binding.converter(CountryFlagCSSClass),
CountryFlagCSSClassIfExists:WinJS.Binding.converter(CountryFlagCSSClassIfExists),
MarketOpenIndicator: WinJS.Binding.converter(MarketOpenIndicator),
timeZoneFormat: WinJS.Binding.converter(timeZoneFormat),
timeZoneFormatNoLabel: WinJS.Binding.converter(timeZoneFormatNoLabel),
seTrendMessage: WinJS.Binding.converter(seTrendMessage),
seTrendImageSource: WinJS.Binding.converter(seTrendImageSource),
seTrendImageDisplayConverter: WinJS.Binding.converter(seTrendImageDisplayConverter),
IndicesClassConverter: WinJS.Binding.converter(IndicesClassConverter),
IndicesClassConverterSmall: WinJS.Binding.converter(IndicesClassConverterSmall),
BarIndicatorClass: WinJS.Binding.converter(BarIndicatorClass),
rangeConverter: WinJS.Binding.converter(rangeConverter),
dayRangeConverter: WinJS.Binding.converter(dayRangeConverter),
yearRangeConverter: WinJS.Binding.converter(yearRangeConverter),
vBarHeight: WinJS.Binding.converter(vBarHeight),
valueOverflowConverter: WinJS.Binding.converter(valueOverflowConverter),
formatValueAndAddCcySymbol: WinJS.Binding.converter(formatValueAndAddCcySymbol),
addCurrencySymbol: WinJS.Binding.converter(addCurrencySymbol)
});
WinJS.Namespace.define('Appenders', {
StockClassConverter: appender(StockClassConverter, " "),
stockClassConverterSmall: appender(stockClassConverterSmall, " "),
ValueChangeIndicatorCSSClass: appender(ValueChangeIndicatorCSSClass, " "),
ValueChangeColorCSSClass: appender(ValueChangeColorCSSClass, " "),
ValueChangeColorCSSClassIncludeEq: appender(ValueChangeColorCSSClassIncludeEq, " "),
CountryFlagCSSClass: appender(CountryFlagCSSClass, " "),
CountryFlagCSSClassIfExists: appender(CountryFlagCSSClassIfExists, " "),
IndicesClassConverter: appender(IndicesClassConverter, " "),
IndicesClassConverterSmall: appender(IndicesClassConverterSmall, " "),
BarIndicatorClass: appender(BarIndicatorClass, " "),
appendClass: appender(appendClass, " "),
});
WinJS.Utilities.markSupportedForProcessing(Appenders.StockClassConverter);
WinJS.Utilities.markSupportedForProcessing(Appenders.stockClassConverterSmall);
WinJS.Utilities.markSupportedForProcessing(Appenders.ValueChangeIndicatorCSSClass);
WinJS.Utilities.markSupportedForProcessing(Appenders.ValueChangeColorCSSClass);
WinJS.Utilities.markSupportedForProcessing(Appenders.ValueChangeColorCSSClassIncludeEq);
WinJS.Utilities.markSupportedForProcessing(Appenders.CountryFlagCSSClass);
WinJS.Utilities.markSupportedForProcessing(Appenders.CountryFlagCSSClassIfExists);
WinJS.Utilities.markSupportedForProcessing(Appenders.IndicesClassConverter);
WinJS.Utilities.markSupportedForProcessing(Appenders.IndicesClassConverterSmall);
WinJS.Utilities.markSupportedForProcessing(Appenders.appendClass);
})();
/* shared\js\Settings.js */
(function () {
"use strict";
WinJS.Namespace.define("FinanceAppJS.Settings", {
cmsPreviewEnabled: false,
});
WinJS.Namespace.define("FinanceAppJS.WSJ", {
wsjCssLoaded: false
});
})();
/* shared\js\NavigationController.js */
// AppEx Finance navigation
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS.Utilities", {
NavigationController: WinJS.Class.define(
function () {
},
{
applicationController: {
default: function (args) {
return PlatformJS.Navigation.createNavigationAction("Home");
},
search: function search(args) {
if (!args) {
return;
}
args = utils.makeObjectKeysToLowerCase(args);
args.q = utils.trim(args.q);
if (!args.q) {
return;
}
// Initialize AutoSuggest service
this._autoSuggest = new FinanceAppJS.Utilities.AutoSuggest(),
this._handleQuerySubmittedComplete_binding = this._handleQuerySubmittedComplete.bind(this),
this._handleQuerySubmittedError_binding = this._handleQuerySubmittedError.bind(this),
this.handleQuerySubmitted(args);
},
view: function view(args) {
if (!args) {
return;
}
args = utils.makeObjectKeysToLowerCase(args);
if (args.entitytype) {
var entityType = args.entitytype;
if (PlatformJS.Navigation.entityViewMap[entityType]) {
return PlatformJS.Navigation.entityViewMap[entityType](args);
}
}
return PlatformJS.Navigation.createCommand("application", "default");
},
back: function back(args) {
if (!PlatformJS.Navigation || PlatformJS.Navigation.canGoBack) {
WinJS.Navigation.back();
}
},
handleQuerySubmitted: function handleQuerySubmitted(args) {
args = utils.makeObjectKeysToLowerCase(args);
this._args = args;
this._args.q = utils.trim(this._args.q);
if (!this._args.q) {
return;
}
this._args.redirurl = (this._args.redirurl && decodeURIComponent(this._args.redirurl)) || null;
var locale = this._args.locale || null;
this._autoSuggest.getSuggestions(this._args.q, 1, locale).then(this._handleQuerySubmittedComplete_binding, this._handleQuerySubmittedError_binding);
},
_handleQuerySubmittedComplete: function _handleQuerySubmittedComplete(symbolList) {
if (symbolList && symbolList.length > 0) {
var suggestion = this._autoSuggest.buildSuggestion(symbolList[0], FinanceAppJS.Globalization.getLanguage().toLowerCase());
if (suggestion && suggestion.value) {
var qLowerCase = this._args.q.toLowerCase();
if ((suggestion.equitySymbol && suggestion.equitySymbol.toLowerCase() === qLowerCase) ||
(suggestion.realTimeSymbol && suggestion.realTimeSymbol.toLowerCase() === qLowerCase)) {
navigationHelper.navigateToDetailsPage(suggestion.value);
return;
}
}
}
var moreinfo = null;
if (this._args.redirurl) {
moreinfo = utils.getResourceString("MSNMoneyRedirMessage").format("<a href='" + this._args.redirurl + "'>" + utils.getResourceString("MSNMoneySite") + "</a>");
}
var stateObj = {
query: this._args.q,
moreinfo: moreinfo,
};
PlatformJS.Navigation.navigateToChannel("NoSearchResults", stateObj);
},
_handleQuerySubmittedError: function _handleQuerySubmittedError() {
return PlatformJS.Navigation.createCommand("application", "default");
},
}
},
{
worldMarketsController: {
default: function (args) {
return PlatformJS.Navigation.createNavigationAction("WorldMarket");
}
},
watchlistController: {
default: function (args) {
return PlatformJS.Navigation.createNavigationAction("MyStocks");
}
},
currenciesController: {
default: function (args) {
return PlatformJS.Navigation.createNavigationAction("Currencies");
}
},
currencyConverterController: {
default: function (args) {
return PlatformJS.Navigation.createNavigationAction("CurrencyConverter");
}
},
partnerPanoEntityView: function partnerPanoEntityView(args) { /* This function returns state/channelInfo in a promise not a command */
var idToUse = null;
if (args.sourceid) { idToUse = args.sourceid; }
if (args.panoid) { idToUse = args.panoid; }
if (idToUse) {
return new WinJS.Promise(function (complete, error, progress) {
var channel = PlatformJS.Navigation.mainNavigator.channelManager.resolveChannel(idToUse, "featured");
if (channel) {
complete(channel);
}
else {
/* Match the sourceid in the sources data and create the dynamicInfo accordingly */
var sources = new PlatformJS.FeaturedPartners.Sources();
sources.getSourceForID(idToUse).then(function (sourceItem) {
if (sourceItem) {
complete(sourceItem);
}
else {
complete(null);
}
}, function (value) {
complete(null);
});
}
});
}
return PlatformJS.Navigation.createCommand("application", "default");
},
articleEntityView: function articleEntityView(args) {
var arConfig = utils.getArticleReaderConfig(args);
var articleId = null;
if (args.articleid) {   // Editorial link
articleId = args.articleid;
}
else if (args.pageid) { // Share charm link
articleId = args.pageid + (args.contentid ? "/" + args.contentid : "");
}
if (args.market) {
arConfig.providerConfiguration.market = args.market;
}
if (articleId) {
var entityItem = {
providerType: arConfig.providerType,
providerConfiguration: arConfig.providerConfiguration,
initialArticleId: articleId,
market: args.market,
source: 2,
enableSharing: true
};
return PlatformJS.Navigation.createNavigationAction("CommonControlsArticleReader", entityItem);
}
return PlatformJS.Navigation.createCommand("application", "default");
},
readerEntityView: function readerEntityView(args) {
var entityItem = {
articleUrl: args.articleUrl
};
return PlatformJS.Navigation.createNavigationAction("NewsFramePage", entityItem);
},
searchEntityView: function searchEntityView(args) {
return utils.showSearchPane;
},
stockEntityView: function stockEntityView(args) {
var entityItem = {
entity: {
Sym: args.sym,
Eqsm: args.eqsm,
E1: args.e1,
E2: args.e2,
St: args.st,
Cmp: args.cmp,
ExSn: args.exsn
}
};
return PlatformJS.Navigation.createNavigationAction("StockEntity", entityItem);
},
indexEntityView: function indexEntityView(args) {
var entityItem = {
entity: {
displayName: args.displayName,
ExchCode: args.ExchCode,
indexSymbol: args.indexSymbol,
}
};
return PlatformJS.Navigation.createNavigationAction("IndexDetails", entityItem);
},
etfEntityView: function etfEntityView(args) {
var entityItem = {
entity: {
Sym: args.sym,
Eqsm: args.eqsm,
E1: args.e1,
E2: args.e2,
St: args.st,
Cmp: args.cmp
}
};
return PlatformJS.Navigation.createNavigationAction("ETF", entityItem);
},
fundEntityView: function fundEntityView(args) {
var entityItem = {
entity: {
Tkr: args.sym,
Fn: args.cmp,
Scid: args.secid
}
};
return PlatformJS.Navigation.createNavigationAction("FundDetails", entityItem);
},
shareEntityViewsAdded: false,
startClusterEntityViewsAdded: false,
pageEntityViewsAdded: false,
defaultControllerAdded: false,
addShareEntityViews: function addShareEntityViews() {
var controller = FinanceAppJS.Utilities.NavigationController;
PlatformJS.Navigation.addController("worldmarkets", controller.worldMarketsController);
PlatformJS.Navigation.addController("watchlist", controller.watchlistController);
PlatformJS.Navigation.addController("currencies", controller.currenciesController);
PlatformJS.Navigation.addEntityView("article", controller.articleEntityView);
PlatformJS.Navigation.addEntityView("reader", controller.readerEntityView);
PlatformJS.Navigation.addEntityView("stock", controller.stockEntityView);
PlatformJS.Navigation.addEntityView("index", controller.indexEntityView);
PlatformJS.Navigation.addEntityView("etf", controller.etfEntityView);
PlatformJS.Navigation.addEntityView("fund", controller.fundEntityView);
PlatformJS.Navigation.addEntityView("partnerPano", controller.partnerPanoEntityView);
PlatformJS.Navigation.addEntityView("pano", controller.partnerPanoEntityView);
PlatformJS.Navigation.addController("currencyconverter", controller.currencyConverterController);
PlatformJS.Navigation.addHelpEntityView();
FinanceAppJS.Utilities.NavigationController.shareEntityViewsAdded = true;
},
addStartClusterEntityViews: function addStartClusterEntityViews() {
PlatformJS.Navigation.addEntityView("watchlist", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "MyStocks", {}));
PlatformJS.Navigation.addTourEntityView();
FinanceAppJS.Utilities.NavigationController.startClusterEntityViewsAdded = true;
},
navigateToNewsL2Command: function navigateToNewsL2Command(state) {
return navigationHelper.navigateToEntityCategoryNewsPano.bind(navigationHelper, state);
},
navigateToMoneyL2Command: function navigateToMoneyL2Command(state) {
return navigationHelper.navigateToEntityCategoryMoneyPano.bind(navigationHelper, state);
},
navigateToRatesL2Command: function navigateToRatesL2Command(state) {
return navigationHelper.navigateToEntityCategoryRatesPano.bind(navigationHelper, state);
},
addPageEntityViews: function addPageEntityViews() {
PlatformJS.Navigation.addEntityView("watchlist", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "MyStocks", {}));
PlatformJS.Navigation.addEntityView("worldmarket", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "WorldMarket", {}));
PlatformJS.Navigation.addEntityView("commodities", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "Commodities", {}));
PlatformJS.Navigation.addEntityView("currencies", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "Currencies", {}));
PlatformJS.Navigation.addEntityView("bonds", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "Bonds", {}));
PlatformJS.Navigation.addEntityView("bestofweb", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "BestOfWeb"));
PlatformJS.Navigation.addEntityView("rates", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "RatesAverage", {}));
PlatformJS.Navigation.addEntityView("market", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "Market", {}));
PlatformJS.Navigation.addEntityView("newspano", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "NewsPano", {}));
PlatformJS.Navigation.addEntityView("rssnews", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "NewsRSSSources", {}));
PlatformJS.Navigation.addEntityView("entitynewspano", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "EntityNewsPano", { provider: "EntityPanoProvider", pageTitle: "chNews", panoId: "EntityNewsPano", singleCluster: false, collectionId: null }));
PlatformJS.Navigation.addEntityView("entitymoneypano", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "EntityMoneyPano", { provider: "MoneyEntityPanoProvider", pageTitle: "chMoney", panoId: "EntityMoneyPano", singleCluster: false, collectionId: null }));
PlatformJS.Navigation.addEntityView("movers", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "MarketMovers", { entityView: "StockEntity", title: "MarketMovers", class: "FinanceAppJS.MarketMovers.Collection", key: "MarketMovers" }));
PlatformJS.Navigation.addEntityView("fundpicks", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "FundPicks", {}));
PlatformJS.Navigation.addEntityView("videos", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "EntityVideosPano", { provider: "VideoEntityPanoProvider", pageTitle: "Videos" }));
PlatformJS.Navigation.addEntityView("mortgagecalc", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "MortgageCalc", {}));
PlatformJS.Navigation.addEntityView("currencyconverter", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "CurrencyConverter", {}));
PlatformJS.Navigation.addEntityView("financialcalculator", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "FinancialCalc", {}));
PlatformJS.Navigation.addEntityView("savingscalculator", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "SavingsCalc", {}));
PlatformJS.Navigation.addEntityView("retirementcalculator", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "RetirementCalc", {}));
PlatformJS.Navigation.addEntityView("autoloan", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "AutoLoanCalc", {}));
PlatformJS.Navigation.addEntityView("tools", PlatformJS.Navigation.createNavigationAction.bind(PlatformJS.Navigation, "Tools", {}));
PlatformJS.Navigation.addEntityView("article", FinanceAppJS.Utilities.NavigationController.articleEntityView);
PlatformJS.Navigation.addEntityView("search", FinanceAppJS.Utilities.NavigationController.searchEntityView);
PlatformJS.Navigation.addFeedbackEntityView();
PlatformJS.Navigation.addHelpEntityView();
PlatformJS.Navigation.addTourEntityView();
var EntityNewsCategoryList = PlatformJS.Services.configuration.getList("EntityNewsCategoryList");
for (var i = 0; i < EntityNewsCategoryList.size; i++) {
var category = JSON.parse(EntityNewsCategoryList[i].value);
PlatformJS.Navigation.addEntityView("news" + category.deepLinkId, FinanceAppJS.Utilities.NavigationController.navigateToNewsL2Command.bind(FinanceAppJS.Utilities.NavigationController, {
provider: "EntityPanoProvider",
pageTitle: "chNews",
panoId: "EntityNewsPano",
singleCluster: true,
collectionId: category.collectionId
}));
}
var EntityMoneyCategoryList = PlatformJS.Services.configuration.getList("EntityMoneyCategoryList");
if (EntityMoneyCategoryList) {
for (var i = 0; i < EntityMoneyCategoryList.size; i++) {
var category = JSON.parse(EntityMoneyCategoryList[i].value);
PlatformJS.Navigation.addEntityView("money" + category.deepLinkId, FinanceAppJS.Utilities.NavigationController.navigateToMoneyL2Command.bind(FinanceAppJS.Utilities.NavigationController, {
provider: "MoneyEntityPanoProvider",
pageTitle: "chMoney",
panoId: "EntityMoneyPano",
singleCluster: true,
collectionId: category.collectionId
}));
}
}
var RatesCategories = PlatformJS.Services.configuration.getList("RatesCategoriesList");
for (var i = 0; i < RatesCategories.size; i++) {
var category = RatesCategories[i].value;
PlatformJS.Navigation.addEntityView("rates" + category, FinanceAppJS.Utilities.NavigationController.navigateToRatesL2Command.bind(FinanceAppJS.Utilities.NavigationController, {
entityView: "",
title: "RatesTitle_" + category, //ECV code look in the resource, so we dont need to again look-up in the resource
class: "FinanceAppJS.Rates.Collection",
key: { label: "Rates" },
state: { category: category }
}));
}
FinanceAppJS.Utilities.NavigationController.pageEntityViewsAdded = true;
},
instance: null,
addDefaultController: function addDefaultController() {
if (FinanceAppJS.Utilities.NavigationController.instance === null) {
FinanceAppJS.Utilities.NavigationController.instance = new FinanceAppJS.Utilities.NavigationController();
}
PlatformJS.Navigation.addController("application", FinanceAppJS.Utilities.NavigationController.instance.applicationController);
FinanceAppJS.Utilities.NavigationController.defaultControllerAdded = true;
}
})
});
})();
/* shared\js\PinUnpinManager.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS", {
PinUnpinManager: WinJS.Class.define(
function (bottomEdgy, pageState, type, symbol, displayName, ticker, pinCallback, unpinCallback, instrumentationID) {
var bottomEdgyElm = (typeof bottomEdgy === "string") ? document.querySelector(bottomEdgy) : bottomEdgy;
//Constructor
this._pinElem = bottomEdgyElm.querySelector("#pinTileButton");
this._unPinElem = bottomEdgyElm.querySelector("#unpinTileButton");
this._bottomEdgyControl = bottomEdgyElm.winControl;
this._pageState = pageState;
this._instrumentId = null;
if (instrumentationID) {
this._instrumentId = instrumentationID;
}
if (this._pinElem && this._unPinElem) {
this.onPinClickHandler_binding = this.onPinClickHandler.bind(this);
this.onUnPinClickHandler_binding = this.onUnPinClickHandler.bind(this);
this._pinElem.addEventListener("click", this.onPinClickHandler_binding);
this._unPinElem.addEventListener("click", this.onUnPinClickHandler_binding);
}
if (pageState && (type !== null || type !== undefined) && symbol) {
this.initializeBottomSwipyPinButtons(pageState, type, symbol, displayName, ticker, pinCallback, unpinCallback);
}
},
{
//member Functions and variables
_bottomEdgyControl: null,
_pageState: null,
_symbol: null,
_type: null,
_ticker: null,
_displayName: null,
_pinElem: null,
_unPinElem: null,
_pinCallback: null,
_unPinCallback: null,
defaultTileCallBack: function defaultTileCallBack() {
this._bottomEdgyControl.sticky = false;
this._bottomEdgyControl.hide();
},
initializeBottomSwipyPinButtons: function initializeBottomSwipyPinButtons(pageState, type, symbol, displayName, ticker, pinCallback, unpinCallback) {
this._pageState = pageState;
this._type = type;
this._symbol = symbol;
this._displayName = displayName;
this._ticker = ticker;
this._pinCallback = pinCallback || this.defaultTileCallBack;
this._unPinCallback = unpinCallback || this.defaultTileCallBack;
this.updateBottomSwipyPinButtons(this._symbol, this._type);
},
onPinUpdateCallback: function onPinUpdateCallback() {
this.updateBottomSwipyPinButtons(this._symbol, this._type);
this._pinCallback();
},
onUnPinUpdateCallback: function onUnPinUpdateCallback() {
this.updateBottomSwipyPinButtons(this._symbol, this._type);
this._unPinCallback();
},
onPinClickHandler: function onPinClickHandler() {
var point = utils.getPointForSecondaryTile(this._pinElem);
// Bottom edgy needs to be sticky when the pin/unpin watchlist tile is on the screen.
this._bottomEdgyControl.sticky = true;
FinanceAppJS.SecondaryTile.pinTile(this._symbol, this._type, this._pageState, this._displayName, this._ticker, this.onPinUpdateCallback.bind(this), point, this._instrumentId);
},
onUnPinClickHandler: function onUnPinClickHandler() {
var point = utils.getPointForSecondaryTile(this._unPinElem);
// Bottom edgy needs to be sticky when the pin/unpin watchlist tile is on the screen.
this._bottomEdgyControl.sticky = true;
FinanceAppJS.SecondaryTile.unpinTile(this._symbol, this._type, this.onUnPinUpdateCallback.bind(this), point, this._instrumentId);
},
updateBottomSwipyPinButtons: function updateBottomSwipyPinButtons(symbol, type) {
var pinElem = this._pinElem;
var unPinElem = this._unPinElem;
if (!(this._pinElem && this._unPinElem)) {
return;
}
var pinElemStyle = pinElem.style;
var unPinElemStyle = unPinElem.style;
if (!symbol || symbol === "") {
pinElemStyle.display = "none";
unPinElemStyle.display = "none";
return;
}
var isPinned = FinanceAppJS.SecondaryTile.isTilePinned(symbol, type);
/*
Set the text of pin/unpin button based on the context of the initialization.
will be pin to watchlist if type === FinanceAppJS.Constants.IDs.WatchList otherwise pin/unpin tile
*/
if (isPinned && (pinElemStyle.display !== "none" || unPinElemStyle.display !== "block")) {
utils.setWinLabelText(unPinElem, "/Platform/unpinFromStart");
pinElemStyle.display = "none";
unPinElemStyle.display = "block";
} else if (!isPinned && (pinElemStyle.display !== "block" || unPinElemStyle.display !== "none")) {
pinElemStyle.display = "block";
utils.setWinLabelText(pinElem, "/Platform/pinToStart");
unPinElemStyle.display = "none";
}
},
clean: function clean() {
if (this._pinElem && this._unPinElem) {
this._pinElem.removeEventListener("click", this.onPinClickHandler_binding);
this._unPinElem.removeEventListener("click", this.onUnPinClickHandler_binding);
}
this._bottomEdgyControl = null;
this._pageState = null;
this._pinElem = null;
this._unPinElem = null;
this._pinCallback = null;
this._unPinCallback = null;
}
}, {
//Static Functions and variables
})
});
})();
/* shared\js\watchListRoamingInterface.js */
(function () {
'use strict';
WinJS.Namespace.define("FinanceAppJS", {
Roaming: WinJS.Class.define(
function (data) {
this.data = data;
this.applicationData = Windows.Storage.ApplicationData.current;
this.roamingSettings = this.applicationData.roamingSettings;
this.roamingFolder = this.applicationData.roamingFolder;
this.composite = new Windows.Storage.ApplicationDataCompositeValue();
this.init();
},
{
data: '',
readWatchlistModeFromRoaming: function readWatchlistModeFromRoaming() {
var mode = this.roamingSettings.values["watchlistmode"];
return mode ? mode : "list";
},
writeWatchlistModeToRoaming: function writeWatchlistModeToRoaming(mode) {
this.roamingSettings.values["watchlistmode"] = mode;
this.applicationData.signalDataChanged();
},
writeToRoaming: function writeToRoaming(dt) {
if (!dt) {
return;
}
if (typeof dt === "object") {
this.data = JSON.stringify(dt);
}
var content = {
Page: FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
Symbol: this.data,
Count: dt.length,
};
FinanceAppJS.Telemetry.FlightRecorder.LogContentView(content);
this.composite["Sym"] = this.data;
this.roamingSettings.values["MyStocks"] = this.composite;
this.applicationData.signalDataChanged();
FinanceAppJS.TopEdgy.instance.setupMyStocksL2();
//update watchlist secondary tile
if (FinanceAppJS.SecondaryTile.isTilePinned(FinanceAppJS.Constants.IDs.WATCHLIST_SYMBOL, FinanceAppJS.Constants.IDs.WatchList)) {
FinanceAppJS.SecondaryTile.startPeriodicUpdateForSecondaryTile(FinanceAppJS.Constants.IDs.WATCHLIST_SYMBOL, FinanceAppJS.Constants.IDs.WatchList);
}
},
clearWatchList: function clearWatchList() {
this.composite["Sym"] = "";
this.roamingSettings.values["MyStocks"] = this.composite;
this.applicationData.signalDataChanged();
if (FinanceAppJS.SecondaryTile.isTilePinned(FinanceAppJS.Constants.IDs.WATCHLIST_SYMBOL, FinanceAppJS.Constants.IDs.WatchList)) {
FinanceAppJS.SecondaryTile.startPeriodicUpdateForSecondaryTile(FinanceAppJS.Constants.IDs.WATCHLIST_SYMBOL, FinanceAppJS.Constants.IDs.WatchList);
}
},
isWatchListEmpty: function isWatchListEmpty() {
return (!this.roamingSettings.values["MyStocks"]);
},
isDefaultWatchListAdded: function isDefaultWatchListAdded() {
return !this.isWatchListEmpty();
},
isWatchListEntryExists: function isWatchListEntryExists(newSym, type) {
if (this.isWatchListEmpty()) {
return false;
}
else {
var rSettings = this.readFromRoaming();
var dt = rSettings.data;
newSym = this.getWatchListRoamingPrefix(type) + newSym;
if (!dt) {
return false;
}
else if (utils.inArray(dt, newSym) !== -1) {
return true;
}
else {
return false;
}
}
},
getWatchListRoamingPrefix: function getWatchListRoamingPrefix(type) {
var prefix = "";
var idConstants = FinanceAppJS.Constants.IDs;
switch (type) {
case idConstants.Stock:
prefix = idConstants.RoamingStockPrefix;
break;
case idConstants.FundSecid:
prefix = idConstants.RoamingSecIdPrefix;
break;
case idConstants.FundTicker:
prefix = idConstants.RoamingTickerPrefix;
break;
case idConstants.Etf:
prefix = idConstants.RoamingETFPrefix;
break;
case idConstants.Index:
prefix = idConstants.RoamingIndexPrefix;
break;
default:
prefix = idConstants.RoamingStockPrefix;
break;
}
return prefix;
},
readFromRoaming: function readFromRoaming() {
var dt = this.roamingSettings.values["MyStocks"];
var data = null;
var isDefault = false;
if (typeof dt === "object" && dt["Sym"]) {
try {
isDefault = false;
data = JSON.parse(dt["Sym"]);
}
catch (e) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(e.message, e.stack);
data = null;
isDefault = false;
}
} else {
// return default watchlist so that it is never saved to roaming without other user intervention.
data = FinanceAppJS.Globalization.getDefaultWatchlistSymbols();
if (data && data.length > 0) {
data = utils.deepCopy(data);
}
isDefault = true;
}
return { data: data, isDefault: isDefault };
},
init: function init() {
},
/* max is used in pinning Watchlist Top 5 */
getWatchListSymbols: function getWatchListSymbols(max) {
var rSettings = this.readFromRoaming();
var symbols = rSettings.data;
if (max && symbols && symbols.length > max) {
symbols = symbols.slice(0, max);
}
var stockSymbols = [];
var etfSymbols = [];
var fundSymbols = [];
var fundSecIds = [];
var indexSymbols = [];
var rtSymbols = [];
var wlSymbols = { es: null, mft: null, mfs: null, all: null, rt: null, isDefault: rSettings.isDefault, stocksOnly:[], etfOnly:[], mftOnly:[], mfsOnly:[], idxOnly:[]  };
if (symbols !== null) {
// Seperate symbols for funds and stocks
if (symbols.length > 0) {
var defaultStockSymbolPrefix = FinanceAppJS.Constants.IDs.DefaultRealtimeExchangeId + "." + FinanceAppJS.Constants.IDs.DefaultSecurityType + ".";
for (var cnt = 0; cnt < symbols.length; cnt++) {
if (symbols[cnt]) {
if (symbols[cnt].indexOf(FinanceAppJS.Constants.IDs.RoamingStockPrefix) !== -1) {
if (this.isNonGlobalWatchlistEntry(symbols[cnt])) { //Backward compatibility
stockSymbols.push(symbols[cnt].replace(FinanceAppJS.Constants.IDs.RoamingStockPrefix, defaultStockSymbolPrefix));
}
else {
stockSymbols.push(symbols[cnt].replace(FinanceAppJS.Constants.IDs.RoamingStockPrefix, ""));
}
}
else if (symbols[cnt].indexOf(FinanceAppJS.Constants.IDs.RoamingETFPrefix) !== -1) {
if (this.isNonGlobalWatchlistEntry(symbols[cnt])) { //Backward compatibility
etfSymbols.push(symbols[cnt].replace(FinanceAppJS.Constants.IDs.RoamingETFPrefix, defaultStockSymbolPrefix));
}
else {
etfSymbols.push(symbols[cnt].replace(FinanceAppJS.Constants.IDs.RoamingETFPrefix, ""));
}
}
else if (symbols[cnt].indexOf(FinanceAppJS.Constants.IDs.RoamingTickerPrefix) !== -1) {
fundSymbols.push(symbols[cnt].replace(FinanceAppJS.Constants.IDs.RoamingTickerPrefix, ""));
}
else if (symbols[cnt].indexOf(FinanceAppJS.Constants.IDs.RoamingSecIdPrefix) !== -1) {
fundSecIds.push(symbols[cnt].replace(FinanceAppJS.Constants.IDs.RoamingSecIdPrefix, ""));
}
else if (symbols[cnt].indexOf(FinanceAppJS.Constants.IDs.RoamingIndexPrefix) !== -1) {
indexSymbols.push(symbols[cnt].replace(FinanceAppJS.Constants.IDs.RoamingIndexPrefix, ""));
}
}
}
wlSymbols.es = stockSymbols.concat(etfSymbols).concat(indexSymbols).sort().join(',');
wlSymbols.mft = fundSymbols.sort().join(',');
wlSymbols.mfs = fundSecIds.sort().join(',');
wlSymbols.stocksOnly = stockSymbols;
wlSymbols.etfOnly = etfSymbols;
wlSymbols.mftOnly = fundSymbols;
wlSymbols.mfsOnly = fundSecIds;
wlSymbols.idxOnly = indexSymbols;
}
wlSymbols.all = symbols.join(",");
}
/* Even when there are no watchlist symbols populate rt, rtNames */
/* should check what happens when we set null in watchlist roaming data object in releases prior to march */
wlSymbols.rt = FinanceAppJS.Globalization.getHomePageIndices("symbol");
wlSymbols.rtNames = FinanceAppJS.Globalization.getHomePageIndices("displayName");
return wlSymbols;
},
getWatchListDataSorted: function getWatchListDataSorted(wlSymbols, wlResonseObj) {
// Return stocks/funds in the same order as defined by the roaming
var watchList = [];
if (wlSymbols.all) { //if the watch list is empty, then don't do any post processing
var allSymbols = wlSymbols.all.split(',');
var index = 0;
var pos = null;
if (wlResonseObj.MFs &&
typeof wlResonseObj.MFs === 'object' &&
wlResonseObj.MFs.length > 0) {
for (index = 0; index < wlResonseObj.MFs.length; index++) {
if (wlResonseObj.MFs[index]) {
// Search for position
pos = utils.inArray(allSymbols, wlResonseObj.MFs[index].Tkr ? FinanceAppJS.Constants.IDs.RoamingTickerPrefix + wlResonseObj.MFs[index].Tkr : FinanceAppJS.Constants.IDs.RoamingSecIdPrefix + wlResonseObj.MFs[index].Scid);
watchList[pos] = wlResonseObj.MFs[index];
watchList[pos].type = FinanceAppJS.Constants.IDs.Fund;
}
}
}
if (wlResonseObj.Stks &&
typeof wlResonseObj.Stks === 'object' &&
wlResonseObj.Stks.length > 0) {
for (index = 0; index < wlResonseObj.Stks.length; index++) {
if (wlResonseObj.Stks[index]) {
// Search for position
var StkType = wlResonseObj.Stks[index].St;
if (StkType === FinanceAppJS.Constants.SecurityType.Index) {
pos = utils.inArray(allSymbols, FinanceAppJS.Constants.IDs.RoamingIndexPrefix + utils.getIndicesSymbol(wlResonseObj.Stks[index]));
if (pos !== -1) {
watchList[pos] = wlResonseObj.Stks[index];
watchList[pos].type = FinanceAppJS.Constants.IDs.Index;
}
} else {
pos = utils.inArray(allSymbols, FinanceAppJS.Constants.IDs.RoamingStockPrefix + utils.getGlobalInstrumentSymbol(wlResonseObj.Stks[index]));
if (pos === -1) {
pos = utils.inArray(allSymbols, FinanceAppJS.Constants.IDs.RoamingETFPrefix + utils.getGlobalInstrumentSymbol(wlResonseObj.Stks[index]));
if (pos !== -1) {
watchList[pos] = wlResonseObj.Stks[index];
watchList[pos].type = FinanceAppJS.Constants.IDs.Etf;
}
}
else {
watchList[pos] = wlResonseObj.Stks[index];
watchList[pos].type = FinanceAppJS.Constants.IDs.Stock;
}
}
}
}
}
}
return watchList;
},
setDefaultWatchList: function setDefaultWatchList(wlStocks) {
var defaultSymbols = [];
if (wlStocks && wlStocks.length > 0) {
for (var index = 0; index < wlStocks.length; index++) {
if (wlStocks[index]) {
defaultSymbols.push(FinanceAppJS.Constants.IDs.RoamingStockPrefix + utils.getGlobalInstrumentSymbol(wlStocks[index]));
// Set the type                  
wlStocks[index].type = FinanceAppJS.Constants.IDs.Stock;
}
}
if (defaultSymbols.length > 0) {
this.setWatchlist(defaultSymbols);
}
return wlStocks;
}
else {
return [];
}
},
setWatchlist: function setWatchlist(data) {
this.writeToRoaming(data);
},
// items is array of objects and each object has the form { symbol: ___ , type: ___ }
removeFromWatchlist: function removeFromWatchlist(items) {
if (!items) {
return false;
}
if (items.length > 0) {
var symbs = [];
// Prefix entity type for each symbol if TYPE exists
for (var k = 0; k < items.length; k++) {
if (items[k].type !== null && items[k].type !== undefined) {
var prefix = this.getWatchListRoamingPrefix(items[k].type);
symbs[k] = prefix + items[k].symbol;
}
else {
symbs[k] = items[k].symbol;
}
}
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.WatchListAction.WATCHLISTMODIFY,
FinanceAppJS.Telemetry.Constants.WatchListAction.REMOVE,
symbs.join("|"));
var rSettings = this.readFromRoaming();
var existingSymbols = rSettings.data;
if (existingSymbols === null) {
return true;
}
else {
for (var k = 0; k < symbs.length; k++) {
var pos = existingSymbols.indexOf(symbs[k]);
if (pos !== -1) {
existingSymbols.splice(pos, 1);
}
}
this.writeToRoaming(existingSymbols);
}
}
return true;
},
isNonGlobalWatchlistEntry: function isNonGlobalWatchlistEntry(symbol) {
if (!symbol) {
return false;
}
if (symbol.split(":")[1].indexOf(".") !== -1) {
return false;
}
else {
return true;
}
},
// items is array of objects and each object has the form { symbol: ___ , type: ___ }
addToWatchlist: function addToWatchlist(items, isAddToFront) {
if (!items) {
return false;
}
if (items.length > 0) {
var symbs = [];
// Prefix entity type for each symbol if TYPE exists
for (var k = 0; k < items.length; k++) {
var prefix = this.getWatchListRoamingPrefix(items[k].type);
symbs[k] = prefix + items[k].symbol;
}
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(
FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId),
FinanceAppJS.Telemetry.Constants.WatchListAction.WATCHLISTMODIFY,
FinanceAppJS.Telemetry.Constants.WatchListAction.ADD,
symbs.join("|"));
var rSettings = this.readFromRoaming();
var existingSymbols = rSettings.data;
if (existingSymbols === null) {
this.writeToRoaming(symbs);
}
else {
var symbsUnique = [];
for (var i = 0; i < symbs.length; i++) {
if (existingSymbols.indexOf(symbs[i]) === -1) {
symbsUnique.push(symbs[i]);
}
}
if (symbsUnique.length > 0) {
if (isAddToFront) {
this.writeToRoaming(symbsUnique.concat(existingSymbols));
}
else {
this.writeToRoaming(existingSymbols.concat(symbsUnique));
}
}
}
return true;
}
},
getWatchlistCount: function getWatchlistCount() {
var data = (this.readFromRoaming()).data;
return data ? data.length : 0;
}
})
});
})();
/* pages\MyStocks\js\WatchListAutoSuggest.js */
(function () {
'use strict';
var watchListFlyout = null;
// TODO: There are saperate handlers for fund and stocks. Optimize it better.
function addStock(query, suggItem, pageSource, paramListElmWin) {
/// <summary>This function will be called from AutoSuggest drop-down. This function will add NEW stock/etf/fund tile to watchlist panorama.</summary>
/// <param name="event" type="EventObject">Auto Seuugest selection event</param>
/// <param name="event" type="EventObject">Supported Values as of now FinanceAppJS.Constants.IDs.MarketToday and FinanceAppJS.Constants.IDs.WatchList</param>
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistAutoSuggest.js", "AddToWatchList", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);
var quickSearch = FinanceAppJS.Utilities.QuickSearch.getInstance();
var invalidEntryMessage = utils.getResourceString("ASInvalidEntry").format(query);
if (suggItem && paramListElmWin && pageSource) {
// Show progress
quickSearch.showProgress();
var idConstants = FinanceAppJS.Constants.IDs;
var clusterHeaderElm = (pageSource === idConstants.MarketToday) ? WatchListManage.getWatchlistHeaderElement() : null;
var autoSuggestValArr = suggItem.split('~');
var sym = utils.trim(autoSuggestValArr[0]);
var eqSym = utils.trim(autoSuggestValArr[6]);
var sec = autoSuggestValArr[1];
var symbolType = autoSuggestValArr[2];
var E1 = autoSuggestValArr[5];
var St = autoSuggestValArr[7];
var E2 = autoSuggestValArr[3];
var cmp = autoSuggestValArr[4];
var globalInstrument = {};
if (utils.isIndexSymbol(symbolType)) {
globalInstrument = utils.getIndicesSymbol({ E1: E1, Sym: sym, St: St, E2: E1, Eqsm: eqSym });
} else {
globalInstrument = E1 + '.' + St + '.' + sym + '.' + E2;
if (eqSym !== sym) {
globalInstrument += '.' + eqSym;
}
}
if (globalInstrument !== "" || sec !== "") {
var FinanceRoaming = new FinanceAppJS.Roaming();
var listElmWin = paramListElmWin;
var watchListQueryService = new PlatformJS.DataService.QueryService("HomepageEquityData");
var options = new Platform.DataServices.QueryServiceOptions();
options.customCacheKey = WatchListDataProvider.getWatchlistCustomCacheKey();
options.enableAutoRefresh = false;
options.useCompatDataModel = true;
var isStock = utils.isStockSymbol(symbolType);
var isEtf = utils.isEtfSymbol(symbolType);
var isIdx = utils.isIndexSymbol(symbolType);
var isFund = utils.isFundSymbol(symbolType);
if (isStock || isEtf || isIdx || isFund) {
var urlParamToAppend = null;
var responseProperty = null;
var tickerToAppend = null;
var stockType = "";
if (isStock &&
FinanceRoaming.isWatchListEntryExists(globalInstrument, idConstants.Stock) === false) {
stockType = idConstants.Stock;
urlParamToAppend = "stockSymbol";
tickerToAppend = globalInstrument;
responseProperty = "Stks";
}
else if (isEtf &&
FinanceRoaming.isWatchListEntryExists(globalInstrument, idConstants.Etf) === false) {
stockType = idConstants.Etf;
urlParamToAppend = "stockSymbol";
tickerToAppend = globalInstrument;
responseProperty = "Stks";
}
else if (isIdx &&
FinanceRoaming.isWatchListEntryExists(globalInstrument, idConstants.Index) === false) {
stockType = idConstants.Index;
urlParamToAppend = "stockSymbol";
tickerToAppend = globalInstrument;
responseProperty = "Stks";
} else if (isFund && sym !== "" && FinanceRoaming.isWatchListEntryExists(sym, idConstants.FundTicker) === false) {
stockType = idConstants.FundTicker;
urlParamToAppend = "fundSymbol";
tickerToAppend = sym;
responseProperty = "MFs";
}
else if (isFund && sym === "" && sec !== "" && FinanceRoaming.isWatchListEntryExists(sec, idConstants.FundSecid) === false) {
stockType = idConstants.FundSecid;
urlParamToAppend = "fundSecId";
tickerToAppend = sec;
responseProperty = "MFs";
}
var roamingSymbolToAdd = FinanceRoaming.getWatchListRoamingPrefix(stockType) + tickerToAppend;
if (stockType !== "") {
var roamingSymbols = FinanceRoaming.getWatchListSymbols();
roamingSymbols.all = roamingSymbolToAdd;
var urlParams = getQuickSearchQueryParams(urlParamToAppend, tickerToAppend, roamingSymbols);
var showNoDataMsg = false;
var dataReqObj = FinanceAppJS.Telemetry.FlightRecorder.LogDataRequest("request", "WatchListCollection");
return watchListQueryService.downloadDataAsync(urlParams, null, null, options).then(function (e) {
if (e.dataString !== "") {
try {
FinanceAppJS.Telemetry.FlightRecorder.LogDataRequest("service", "WatchListCollection", dataReqObj);
var dt = JSON.parse(e.dataString);
if (dt.Wl[responseProperty] && dt.Wl[responseProperty].length > 0) {
var processedData = FinanceRoaming.getWatchListDataSorted(roamingSymbols, dt.Wl);
var isValidTicker = null;
if (responseProperty === "MFs") {
isValidTicker = utils.isValidFund(processedData[0]);
} else if (responseProperty === "Stks") {
isValidTicker = utils.isValidStock(processedData[0]);
}
if (isValidTicker) {
var newData = processedData[0];
newData.type = stockType;
var newTmplData = {};
switch (stockType) {
case idConstants.Index:
newTmplData.tileData = WatchListDataProvider.getIndexItemTemplateData(newData);
break;
case idConstants.FundTicker:
case idConstants.FundSecid:
newData.type = idConstants.Fund; //getFundItemTemplate requires type to be fund.
newTmplData.tileData = WatchListDataProvider.getFundItemTemplateData(newData);
break;
case idConstants.Stock:
default:
newTmplData.tileData = WatchListDataProvider.getStockItemTemplateData(newData);
break;
}
newTmplData.itemKey = WatchListManage.getSymbolFromTileData(newTmplData.tileData);
var ds = listElmWin.itemDataSource;
if (pageSource === idConstants.MarketToday) {
var currCount = listElmWin._cachedCount;
var maxShow = utils.getHomePageWLDisplayCount();
if (listElmWin && (currCount >= (maxShow + 1))) {
ds.itemFromIndex(maxShow - 1).then(function (item) {
ds.remove(item.key);
});
}
}
ds.insertAtStart(null, newTmplData).then(function (item) {
listElmWin.ensureVisible(0);
if (pageSource === idConstants.MarketToday) {
// Just add the stock / idx to roaming.
WatchListManage.addSymbolToWatchlist(tickerToAppend, stockType, FinanceRoaming);
var wlCount = FinanceRoaming.getWatchlistCount();
if (wlCount > utils.getHomePageWLDisplayCount()) {
WatchListManage.updateWatchlistClusterTitle(WatchListManage.getTitleWithCount(wlCount), clusterHeaderElm);
}
}
else {
WatchListManage.addStocksToWatchlist(listElmWin, FinanceRoaming);
// Enable pin/unpin button panel when pano is not empty i.e. when there is atleast one item.
if (listElmWin.itemDataSource.list.length === 2) {
var mode = FinanceRoaming.readWatchlistModeFromRoaming();
WatchListManage.handlePinButtonsVisibility(true);
if (mode === "tile") {
WatchListManage.handleSortButtonsVisibility(true);
}
}
}
// Close Auto Suggest
quickSearch.hideQuickSearch();
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistAutoSuggest.js", "AddToWatchList", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
}, function (err) {
});
}
else {
showNoDataMsg = true;
}
}
else {
showNoDataMsg = true;
}
if (showNoDataMsg) {
WatchListAutoSuggest.showError(quickSearch, utils.getResourceString("ASNoDataAvailable").format(cmp), false);
}
}
catch (err) {
FinanceAppJS.Telemetry.FlightRecorder.LogErrorEvent(err.message, err.stack);
return err;
}
}
else {
}
},
function (errData) {
WatchListAutoSuggest.showError(quickSearch, utils.getResourceString("ASNoDataAvailable").format(cmp), false);
});
}
else {
WatchListAutoSuggest.showError(quickSearch, "StockItemExists");
}
}
}
else {
WatchListAutoSuggest.showError(quickSearch, invalidEntryMessage, false);
}
}
else {
WatchListAutoSuggest.showError(quickSearch, invalidEntryMessage, false);
}
return null;
}
function showError(quickSearch, message, isResourceKey) {
if (!quickSearch) {
return;
}
quickSearch.hideProgress();
quickSearch.showMessage(message, isResourceKey);
}
function getQuickSearchQueryParams(appendToParam, tickerToAppend, roamingSymbols) {
var urlParams = PlatformJS.Collections.createStringDictionary();
urlParams.insert("lang", FinanceAppJS.Globalization.getLanguage());
var symbolsToPass = {
stockSymbol: roamingSymbols.stocksOnly.concat(roamingSymbols.etfOnly).concat(roamingSymbols.idxOnly),
fundSymbol: roamingSymbols.mftOnly,
fundSecId: roamingSymbols.mfsOnly,
};
symbolsToPass[appendToParam].push(tickerToAppend);
urlParams.insert("stockSymbol", encodeURIComponent(symbolsToPass.stockSymbol.sort().join(",")));
urlParams.insert("fundSymbol", encodeURIComponent(symbolsToPass.fundSymbol.sort().join(",")));
urlParams.insert("fundSecId", encodeURIComponent(symbolsToPass.fundSecId.sort().join(",")));
urlParams.insert("rtSymbol", encodeURIComponent(roamingSymbols.rt));
urlParams.insert("rtexchangeIds", encodeURIComponent(FinanceAppJS.Globalization.getDefaultExchange().code));
urlParams.insert("mvrCnt", encodeURIComponent(FinanceAppJS.Constants.MarketPage.MoversMaxCount));
urlParams.insert("lang", FinanceAppJS.Globalization.getLanguage());
urlParams.insert("locale", FinanceAppJS.Globalization.getMarket().toLowerCase());
return urlParams;
}
WinJS.Namespace.define("WatchListAutoSuggest", {
addStock: addStock,
showError: showError
});
})();
/* pages\MyStocks\js\WatchListManage.js */
(function () {
'use strict';
function addStocksToWatchlist(listElmWin, FinanceRoaming) {
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistManage.js", "saveWatchListToRoaming", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);
var symbols = [];
var stkTilesLength = parseInt(listElmWin._cachedCount);
for (var cnt = 0; cnt < stkTilesLength; cnt++) {
var binding = listElmWin.itemDataSource.createListBinding();
binding.fromIndex(cnt).then(function (item) {
var symbol = getRoamingSymbolForTile(item);
if (utils.isNonEmptyString(symbol)) {
symbols.push(symbol);
}
binding.release();
});
}
if (symbols.length) {
FinanceRoaming.setWatchlist(symbols);
}
else {
FinanceRoaming.clearWatchList();
}
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistManage.js", "saveWatchListToRoaming", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
}
function getRoamingSymbolForTile(item) {
var tileData = item.data.isCommonJSAddTile ? item.data : item.data.tileData;
return getSymbolFromTileData(tileData);
}
function getSymbolFromTileData(tileData) {
var roamingSymbol = "";
var idConstants = FinanceAppJS.Constants.IDs;
if (WatchListManage.isStockTile(tileData) || WatchListManage.isIndexTile(tileData)) {
switch (tileData.type) {
case idConstants.Stock:
roamingSymbol = idConstants.RoamingStockPrefix + utils.getGlobalInstrumentSymbol(tileData);
break;
case idConstants.Etf:
roamingSymbol = idConstants.RoamingETFPrefix + utils.getGlobalInstrumentSymbol(tileData);
break;
case idConstants.Index:
roamingSymbol = idConstants.RoamingIndexPrefix + utils.getIndicesSymbol(tileData);
break;
}
}
else if (WatchListManage.isFundTile(tileData)) {
// If Tickr available
if (tileData.Tkr) {
roamingSymbol = idConstants.RoamingTickerPrefix + tileData.Tkr;
}
else if (tileData.Scid) {
roamingSymbol = idConstants.RoamingSecIdPrefix + tileData.Scid;
}
}
else if (tileData.isCommonJSAddTile) {
roamingSymbol = "";
}
return roamingSymbol;
}
function isAddTile(tileData) {
if (!tileData) {
return false;
}
if (tileData.type === FinanceAppJS.Constants.IDs.AddTile) {
return true;
}
else {
return false;
}
}
function isStockTile(tileData) {
if (!tileData) {
return false;
}
if ((tileData.type === FinanceAppJS.Constants.IDs.Stock || tileData.type === FinanceAppJS.Constants.IDs.Etf) &&
tileData.Sym) {
return true;
}
else {
return false;
}
}
function isFundTile(tileData) {
if (!tileData) {
return false;
}
if (tileData.type === FinanceAppJS.Constants.IDs.Fund &&
(tileData.Tkr || tileData.Scid)) {
return true;
}
else {
return false;
}
}
function isIndexTile(tileData) {
if (!tileData) {
return false;
}
if (tileData.type === FinanceAppJS.Constants.IDs.Index && tileData.Sym) {
return true;
}
else {
return false;
}
}
function reArrangeTilesForNewTile(listViewDataSource, stkTilesLength) {
var newTileIndex = -1;
var binding = listViewDataSource.createListBinding();
for (var cnt = 0; cnt < stkTilesLength; cnt++) {
binding.fromIndex(cnt).then(function (item) {
if (isAddTile(item)) {
newTileIndex = item.key;
}
});
}
binding.release();
if (newTileIndex !== -1) {
listViewDataSource.beginEdits();
listViewDataSource.moveToEnd(newTileIndex);
listViewDataSource.endEdits();
}
}
function preventDefaultForAddTile(listViewDataSource, selectedIndices, eventInfo) {
var selectedItemCountCurrent = 0;
if (selectedIndices.length > 0) {
for (var cnt = 0; cnt < selectedIndices.length; cnt++) {
var binding = listViewDataSource.createListBinding();
binding.fromIndex(selectedIndices[cnt]).then(function (item) {
var tileData = item.data.tileData;
if (isAddTile(tileData)) {
eventInfo.preventDefault();
}
else {
++selectedItemCountCurrent;
}
binding.release();
});
}
}
_gSelectedItemCountCurrent = selectedItemCountCurrent;
}
function deleteSelection(listViewDataSource, indexes) {
if (indexes.length > 0) {
// Create a list binding and use it to obtain the items.
var binding = listViewDataSource.createListBinding();
// Item manipulation requires a key, so convert the indexes to keys.
// Form an array of all the item keys and promises.
var keys = [], promises = [];
for (var i = 0, count = indexes.length; i < count; i++) {
// Wrap the loop variable in a function to ensure
// the correct value when the async binding actually occurs.
(function (j) {
// Get the items based on the index (async).
promises[j] = binding.fromIndex(indexes[j]).then(function (item) {
if (!isAddTile(item)) {
keys[j] = item.key;
}
});
})(i);
}
// The fromIndex method is async, so use a join to wait for them all to be complete.
WinJS.Promise.join(promises).then(function () {
// We're done with the binding so it can be released.
binding.release();
// Start a batch for the edits.
listViewDataSource.beginEdits();
// To remove the items, call the IListDataSource object's
// remove method and pass it the item's key.
for (var i = 0, count = keys.length; i < count; i++) {
listViewDataSource.remove(keys[i]);
}
// End the batch of edits.
listViewDataSource.endEdits();
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistManage.js", "DeleteSelection", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
});
}
}
function removeStockFromWatchList(listElmWin, selectedIndices, roaming, pageType) {
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistManage.js", "DeleteSelection", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);
if (listElmWin && selectedIndices && (selectedIndices.length > 0) && roaming) {
var ds = listElmWin.itemDataSource;
var promises = [];
var symbols = [];
for (var i = 0, len = selectedIndices.length; i < len; i++) {
promises[i] = ds.itemFromIndex(selectedIndices[i]).then(function (item) {
symbols.push({ symbol: getRoamingSymbolForTile(item) });
});
}
if (promises.length) {
WinJS.Promise.join(promises).then(function () {
if (symbols.length) {
roaming.removeFromWatchlist(symbols);
deleteSelection(ds, selectedIndices);
}
});
}
}
}
function addSymbolToWatchlist(symbol, type, roaming) {
roaming.addToWatchlist([{ symbol: symbol, type: type }], true);
}
function getEntityPageInfo(tileData) {
var entityItem = {};
entityItem.entity = {
Sym: tileData.Sym,
Eqsm: tileData.Eqsm,
E1: tileData.E1,
E2: tileData.E2,
St: tileData.St,
Cmp: tileData.Cmp,
ExSn: tileData.ExSn
};
return entityItem;
}
function getFundDetailsPageInfo(tileData) {
var entityItem = {};
entityItem.entity = {
Tkr: tileData.Tkr,
Fn: tileData.Fn,
Scid: tileData.Scid
};
return entityItem;
}
function getETFDetailsPageInfo(tileData) {
var entityItem = {};
entityItem.entity = {
Sym: tileData.Sym,
Eqsm: tileData.Eqsm,
E1: tileData.E1,
E2: tileData.E2,
St: tileData.St,
Cmp: tileData.Cmp
};
return entityItem;
}
function listSelectionHandler(eventInfo, data) {
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistManage.js", "ListSelectionHandler", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);        
var btnClearSelection = data.btnClearSelection;        
var reorderButtons = data.btnReorderButtons;
var listElmWin = data.listElmWin;
var ipage = data.ipage;
var enablePinWatchlist = data.enablePinWatchlist;
var watchListActionEdgyControl = ipage.bottomEdgy.winControl;
if (reorderButtons) {
WatchListManage.handleMoveButtonsVisibility(listElmWin, reorderButtons);
}
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
var cluster = "";
if (page === FinanceAppJS.Telemetry.Constants.Page.MARKETTODAY) {
cluster = FinanceAppJS.Telemetry.Constants.Cluster.WATCHLIST;
}
if (data.currCount === 0) {            
WinJS.Utilities.addClass(btnClearSelection, "hide");
if (reorderButtons) {
WinJS.Utilities.addClass(reorderButtons, "hide");
}
var secTileCallback = function () {
watchListActionEdgyControl.sticky = false;
watchListActionEdgyControl.hide();
};
if (enablePinWatchlist) {
ipage.pinUnpinManager.initializeBottomSwipyPinButtons(ipage.getPageState(), FinanceAppJS.Constants.IDs.WatchList, FinanceAppJS.Constants.IDs.WATCHLIST_SYMBOL, null, null,
null, null);
}
else {
ipage.pinUnpinManager.initializeBottomSwipyPinButtons(null, null, "");
}
// Reset the bottom pin to have pin to WatchList
if (!watchListActionEdgyControl.hidden) {
secTileCallback();
}
}
else if (data.currCount === 1) {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page,
cluster,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
"Tile Selection - " + data.currCount);
/*reorder buttons are displayed when more than one stock.*/
var stkTilesLength = parseInt(listElmWin._cachedCount) - 1;
if (stkTilesLength > 1) {
if (reorderButtons) {
WinJS.Utilities.removeClass(reorderButtons, "hide");
}
}
WinJS.Utilities.addClass(btnClearSelection, "hide");
listElmWin.selection.getItems().then(function (items) {
var item;
if (!items || !items.length || !(item = items[0].data.tileData)) {
return;
}
var tileData;
if (WatchListManage.isFundTile(item)) {
tileData = {
entityObj: WatchListManage.getFundDetailsPageInfo(item),
type: FinanceAppJS.Constants.IDs.Fund,
symbol: item.Tkr || item.Scid,
displayName: item.Fn,
shortName: item.Tkr || item.Scid,
};
}
else if (WatchListManage.isStockTile(item)) {
if (item.type === FinanceAppJS.Constants.IDs.Stock) {
tileData = {
entityObj: WatchListManage.getEntityPageInfo(item),
type: FinanceAppJS.Constants.IDs.Stock,
symbol: utils.getGlobalInstrumentSymbol(item),
displayName: item.Cmp,
shortName: item.Sym
};
}
else {
tileData = {
entityObj: WatchListManage.getETFDetailsPageInfo(item),
type: FinanceAppJS.Constants.IDs.Etf,
symbol: utils.getGlobalInstrumentSymbol(item),
displayName: item.Cmp,
shortName: item.Sym
};
}
}
else if (WatchListManage.isIndexTile(item)) {
tileData = {
entityObj: utils.getIndexDetails(item, "IndexTileData"),
type: FinanceAppJS.Constants.IDs.Index,
symbol: utils.getIndicesSymbol(item),
displayName: item.Cmp,
shortName: item.Sym
};
}
ipage.pinUnpinManager.initializeBottomSwipyPinButtons(tileData.entityObj, tileData.type, tileData.symbol, tileData.displayName, tileData.shortName,
function () { watchListActionEdgyControl.hide(); listElmWin.selection.clear(); },
function () { watchListActionEdgyControl.hide(); listElmWin.selection.clear(); }
);
});
} else {
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page,
cluster,
FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM,
"Tile Selection - " + data.currCount);
ipage.pinUnpinManager.initializeBottomSwipyPinButtons(null, null, "");
}
if (data.prevCount < data.currCount) { // item selected            
if (data.currCount > 1) {
WinJS.Utilities.removeClass(btnClearSelection, "hide");
}
if (watchListActionEdgyControl.hidden) {
watchListActionEdgyControl.sticky = true;
watchListActionEdgyControl.show();
}
}
/*Set Focus to bottom edgy Icon so that action can be performed.*/
/*UPDATE: Focus removed due to #328617 */
msSetImmediate(
function () {
if (data.currCount > 1) {
btnClearSelection.tabIndex = 0;
}
});
data.prevCount = data.currCount;
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistManage.js", "ListSelectionHandler", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
}
function itemInvokedHandler(itemInfo, listElmWin, pageId) {
if (itemInfo && listElmWin) {
var binding = listElmWin.itemDataSource.createListBinding();
var instrumentID = null;
var page = null;
var location = null;
var cluster = "";
// Add instrumentation of tile click.
if (pageId) {
if (pageId === FinanceAppJS.Constants.IDs.MarketToday) {
instrumentID = FinanceAppJS.Constants.Instrumentation.MarketToday_MyStocksCluster_Click;
page = FinanceAppJS.Telemetry.Constants.Page.MARKETTODAY;
cluster = FinanceAppJS.Telemetry.Constants.Cluster.WATCHLISTITEMS;
location = FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM;
}
else if (pageId === FinanceAppJS.Constants.IDs.WatchList) {
instrumentID = FinanceAppJS.Constants.Instrumentation.MyStocks_MyStocks_Click;
page = FinanceAppJS.Telemetry.Constants.Page.WATCHLIST;
location = FinanceAppJS.Telemetry.Constants.Location.CLUSTERITEM;
}
}
binding.fromIndex(itemInfo.detail.itemIndex).then(function (item) {
var tileData = item.data.tileData;
if (tileData && tileData.isBlankTile) {
binding.release();
return;
}
if (WatchListManage.isFundTile(tileData)) {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page, cluster, location, tileData.Fn);
navigationHelper.navigateToFundDetailsPage(WatchListManage.getFundDetailsPageInfo(tileData), instrumentID);
}
else if (WatchListManage.isStockTile(tileData)) {
if (tileData.type === FinanceAppJS.Constants.IDs.Stock) {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page, cluster, location, tileData.Cmp);
navigationHelper.navigateToEntityPage(WatchListManage.getEntityPageInfo(tileData), instrumentID);
}
else {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page, cluster, location, tileData.Cmp);
navigationHelper.navigateToETFDetailsPage(WatchListManage.getETFDetailsPageInfo(tileData), instrumentID);
}
}
else if (WatchListManage.isIndexTile(tileData)) {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page, cluster, location, tileData.Cmp);
navigationHelper.navigateToIndexDetailsPage(utils.getIndexDetails(tileData, "IndexTileData"), instrumentID);
}
else if (WatchListManage.isAddTile(tileData)) {
// add a different instrumentation for add tile click.
if (pageId === FinanceAppJS.Constants.IDs.MarketToday) {
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.MarketToday_MyStockAdd_Click, 1);
}
else if (pageId === FinanceAppJS.Constants.IDs.WatchList) {
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.MyStocks_AddTile_Click, 1);
}
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page, cluster, location, "AddTile");
FinanceAppJS.Utilities.QuickSearch.getInstance().showQuickSearch();
}
binding.release();
});
}
}
function prepareShareContractData(item) {
var tileData = item.data.tileData;
var entity = {
type: tileData.type,
header: (tileData.E2) ? FinanceAppJS.Utils.Formatters.stringFormatter("{0} ({1})", [tileData.instrumentFullName, utils.getResourceString("Ex_" + tileData.E2)]) : tileData.instrumentFullName,
cmp: tileData.instrumentFullName,
sym: tileData.instrumentDisplaySymbol,
lp: tileData.instrumentLastPrice,
changeStatus: (tileData.type === FinanceAppJS.Constants.IDs.Fund) ? tileData.Odr : tileData.Ch,
ch: tileData.instrumentChangeOrVolume,
chp: tileData.instrumentChangePercent
};
return entity;
}
function handleMoveButtonsVisibility(listElmWin, reorderButtons) {
if (listElmWin && reorderButtons) {
var itemsCount = listElmWin._cachedCount - 1;
var selectedItems = listElmWin.selection.getIndices();
if (itemsCount > 1 && selectedItems && selectedItems.length > 0) {
WinJS.Utilities.removeClass(reorderButtons, "hide");
} else {
WinJS.Utilities.addClass(reorderButtons, "hide");
}
}
}
function handleMoveOne(listElmWin, fromIndex, direction) {
if (listElmWin && typeof fromIndex !== 'undefined' && fromIndex !== null && (direction === "down" || direction === "up")) {
var toIndex, count = listElmWin._cachedCount - 1;
var ds = listElmWin.itemDataSource;
if (direction === "up") {
toIndex = fromIndex - 1;
toIndex = toIndex < 0 ? 0 : toIndex;
}
else {
toIndex = fromIndex + 1;
++toIndex;
toIndex = toIndex > count ? count : toIndex;
}
if (toIndex !== fromIndex) {
return ds.itemFromIndex(fromIndex).then(function (sourceItem) {
ds.itemFromIndex(toIndex).then(function (targetItem) {
ds.moveBefore(sourceItem.key, targetItem.key);
});
});
}
}
return WinJS.Promise.wrap({});
}
function handleMove(listElmWin, direction) {
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistManage.js", "ReorderWatchlist", FinanceAppJS.Utils.Logger.PerformanceMarks.Start);
var selectedItems = listElmWin.selection.getIndices();
var selectedCount = selectedItems.length;
var returnPromise = WinJS.Promise.wrap({});
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
if (direction === "up") {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.BtmEdgyBtn.MOVEUP,
listElmWin.selection.getIndices());
for (var i = 0; i < selectedCount; i++) {
// Skip the selected items in the top
if (selectedItems[i] === i) {
continue;
}
returnPromise = returnPromise.then(function () {
handleMoveOne(listElmWin, selectedItems[i], direction);
});
}
}
else {
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.BtmEdgyBtn.MOVEDOWN,
listElmWin.selection.getIndices());
var lastSelectedIndex = selectedCount - 1;
var lastItemIndex = listElmWin._cachedCount - 2; /* one for the add index tile and one for the start with 0 */
for (var j = selectedCount - 1; j >= 0; j--) {
// Skip the selected items if they are in the end.
if ((selectedItems[j] - lastItemIndex) === (j - lastSelectedIndex)) {
continue;
}
returnPromise = returnPromise.then(function () {
handleMoveOne(listElmWin, selectedItems[j], direction);
});
}
}
return returnPromise.then(function () {
FinanceAppJS.Utils.Logger.LogPerformance("WatchlistManage.js", "ReorderWatchlist", FinanceAppJS.Utils.Logger.PerformanceMarks.End);
});
}
function handlePinButtonsVisibility(shouldShow) {
var btnPinUnPin = document.querySelector(".pinUnpinBtn");
if (shouldShow) {
WinJS.Utilities.removeClass(btnPinUnPin, "hide");
}
else {
WinJS.Utilities.addClass(btnPinUnPin, "hide");
}
}
function handleSortButtonsVisibility(shouldShow) {
var btnSortUnPin = document.querySelector(".edgySortButton");
if (shouldShow) {
WinJS.Utilities.removeClass(btnSortUnPin, "hide");
}
else {
WinJS.Utilities.addClass(btnSortUnPin, "hide");
}
}
function attachBasicListEvents(ipage, listElm, btnClearSelection, enablePinWatchlist, btnReorderButtons, optionItem, showBottomEdgy) {
var that = ipage;
var listElmWin = listElm.view;
if (showBottomEdgy) {
listElm.addEventListener("selectionchanging", function (eventInfo) {
if (eventInfo) {
WatchListManage.preventDefaultForAddTile(listElmWin.itemDataSource, eventInfo.detail.newSelection.getIndices(), eventInfo);
}
});
listElm.addEventListener("selectionchanged", function (eventInfo) {
var data = {
currCount: _gSelectedItemCountCurrent,
prevCount: _gSelectedItemCountPrev,
listElmWin: that.listElmWin,                    
btnReorderButtons: btnReorderButtons,
btnClearSelection: btnClearSelection,                    
ipage: that,
enablePinWatchlist: enablePinWatchlist
};
WatchListManage.listSelectionHandler(eventInfo, data);
_gSelectedItemCountCurrent = data.currCount;
_gSelectedItemCountPrev = data.prevCount;
});
}
listElm.addEventListener("iteminvoked", function (itemInfo) {
if (itemInfo && listElmWin && !(listElmWin.isHandlePrevented && listElmWin.isHandlePrevented())) {
WatchListManage.itemInvokedHandler(itemInfo, listElmWin, optionItem.PageId);
}
});
}
function attachBasicPageEvents(ipage, listElm, btnClearSelection, enablePinWatchlist, btnReorderButtons, optionItem) {
var that = ipage;
// Attach event to Clear Selection
btnClearSelection.addEventListener("click", function () {
var listElmWin = ipage.listElmWin;
if (optionItem.PageId === FinanceAppJS.Constants.IDs.MarketToday) {
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.Clear_Today_Click, 1);
}
else if (optionItem.PageId === FinanceAppJS.Constants.IDs.WatchList) {
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.Clear_mystock_Click, 1);
}
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page,
FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY,
FinanceAppJS.Telemetry.Constants.BtmEdgyBtn.CLEARSEL,
listElmWin.selection.getIndices());
listElmWin.selection.clear();
}, false);
if (that.bottomEdgy) {
that.bottomEdgy.addEventListener("beforehide", function () {
var listElmWin = ipage.listElmWin;
if (listElmWin) {
listElmWin.selection.clear();
}
}, false);
}
var noOfTiles = (ipage.listElmWin._cachedCount - 1);
PlatformJS.Services.instrumentation.setInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, FinanceAppJS.Constants.Instrumentation.MyStocks_NumberOfStock, noOfTiles);
}
function getTitleWithCount(count) {
return FinanceAppJS.Utils.Formatters.stringFormatter("{0} ({1})", [utils.getResourceString("chMyStocks"), count]);
}
function updateWatchlistClusterTitle(newTitle, headerElm) {
// TODO: This is DOM manipulation now. Change the approach once header is bindable
headerElm = headerElm || WatchListManage.getWatchlistHeaderElement();
if (headerElm) {
headerElm.nodeValue = newTitle;
}
}
function getWatchlistHeaderElement() {
var headerMoreElm = document.querySelector(".platformCluster #myStocksCluster .platformClusterHeaderMoreIcon");
return (headerMoreElm && headerMoreElm.previousSibling) || null;
}
function addOrRemoveFromEdgy(options) {
var symbol = options.symbol || null;
var type = options.type || (options.type === 0 ? 0 : null);
var isEntryExists = options.isEntryExists || false;
var elm = options.elm || null;
var actionBar = options.actionBar || null;
var fnRoaming = options.roamingObj || null;
var name = options.name || null;
var idConstants = FinanceAppJS.Constants.IDs;
var instrumentationID = null;
var msg = "";
var entryExists = isEntryExists;
var page = FinanceAppJS.Telemetry.FlightRecorder.GetPageId(WinJS.Navigation.location.channelId);
var location = "";
var cluster = FinanceAppJS.Telemetry.Constants.Cluster.BOTTOMEDGY;
var data = {};
var FR_Msg = "";
if (elm.value === "add") {
fnRoaming.addToWatchlist([{ symbol: symbol, type: type }], true);
msg = utils.getResourceString("MSG_ADD_STOCK");
FR_Msg = "MSG_ADD_STOCK";
entryExists = true;
if (type === idConstants.Stock) {
instrumentationID = FinanceAppJS.Constants.Instrumentation.QuoteDetails_AddToWatchList;
}
else if (type === idConstants.FundTicker || type === idConstants.FundSecid) {
instrumentationID = FinanceAppJS.Constants.Instrumentation.FundDetails_AddToWatchList;
}
else if (type === idConstants.Etf) {
instrumentationID = FinanceAppJS.Constants.Instrumentation.ETFDetails_AddToWatchList;
}
else if (type === idConstants.Index) {
instrumentationID = FinanceAppJS.Constants.Instrumentation.Index_AddToWatchList_Click;
}
data = { operation: "Add", Sym: symbol, Name: name };
location = FinanceAppJS.Telemetry.Constants.BtmEdgyBtn.ADD;
}
else {
fnRoaming.removeFromWatchlist([{ symbol: symbol, type: type }]);
msg = utils.getResourceString("MSG_REMOVE_STOCK");
FR_Msg = "MSG_REMOVE_STOCK";
entryExists = false;
if (type === idConstants.Stock) {
instrumentationID = FinanceAppJS.Constants.Instrumentation.Remove_Quote_Click;
}
else if (type === idConstants.FundTicker || type === idConstants.FundSecid) {
instrumentationID = FinanceAppJS.Constants.Instrumentation.Remove_Fund_Click;
}
else if (type === idConstants.Etf) {
instrumentationID = FinanceAppJS.Constants.Instrumentation.Remove_ETF_Click;
}
else if (type === idConstants.Index) {
instrumentationID = FinanceAppJS.Constants.Instrumentation.Index_RemoveFromWatchList_Click;
}
data = { operation: "Remove", Sym: symbol, Name: name };
location = FinanceAppJS.Telemetry.Constants.BtmEdgyBtn.REMOVE;
}
FinanceAppJS.Telemetry.FlightRecorder.LogUserEvent(page, cluster, location, data);
if (instrumentationID) {
PlatformJS.Services.instrumentation.incrementInt32(FinanceAppJS.Constants.Instrumentation.InstrumentationDataSetId, instrumentationID, 1);
}
if (typeof name !== 'undefined') {
msg = msg.format(name);
}
FinanceAppJS.Telemetry.FlightRecorder.LogAppEvent(page,
FinanceAppJS.Telemetry.Constants.AppUtility.TOPBARMSG,
FR_Msg);
var msgbar = new CommonJS.MessageBar(msg, { autoHide: true });
actionBar.winControl.hide();
document.querySelector("#platformNavigationBar").winControl.hide();
msgbar.show();
WatchListManage.toggleEdgyLabels(entryExists, elm);
}
function toggleEdgyLabels(isEntryExists, elm) {
var btnVal = "";
var btnIcon = "";
var btnLbl = "";
if (isEntryExists) {
btnVal = "remove";
btnIcon = "\uE018";
btnLbl = "BTN_LABEL_REMOVE_FROM_WATCHLIST";
}
else {
btnVal = "add";
btnIcon = "\uE017";
btnLbl = "BTN_LABEL_ADD_TO_WATCHLIST";
}
utils.setWinLabelText(elm, btnLbl);
elm.winControl.icon = btnIcon;
elm.value = btnVal;
}
WinJS.Namespace.define("WatchListManage", {
isStockTile: isStockTile,
isFundTile: isFundTile,
isIndexTile: isIndexTile,
isAddTile: isAddTile,
getRoamingSymbolForTile: getRoamingSymbolForTile,
addStocksToWatchlist: addStocksToWatchlist,
deleteSelection: deleteSelection,
reArrangeTilesForNewTile: reArrangeTilesForNewTile,
preventDefaultForAddTile: preventDefaultForAddTile,
addSymbolToWatchlist: addSymbolToWatchlist,
removeStockFromWatchList: removeStockFromWatchList,
getETFDetailsPageInfo: getETFDetailsPageInfo,
getFundDetailsPageInfo: getFundDetailsPageInfo,
getEntityPageInfo: getEntityPageInfo,
listSelectionHandler: listSelectionHandler,
itemInvokedHandler: itemInvokedHandler,
prepareShareContractData: prepareShareContractData,
handleMove: handleMove,
handleMoveButtonsVisibility: handleMoveButtonsVisibility,
handlePinButtonsVisibility: handlePinButtonsVisibility,
attachBasicListEvents: attachBasicListEvents,
getTitleWithCount: getTitleWithCount,
getWatchlistHeaderElement: getWatchlistHeaderElement,
updateWatchlistClusterTitle: updateWatchlistClusterTitle,
addOrRemoveFromEdgy: addOrRemoveFromEdgy,
toggleEdgyLabels: toggleEdgyLabels,
handleSortButtonsVisibility: handleSortButtonsVisibility,
getSymbolFromTileData: getSymbolFromTileData,
attachBasicPageEvents: attachBasicPageEvents
});
})();
(function () {
'use strict';
/*
* <summary>
* This function returns headers to Market Movers cluster in MarketToday Home Page.
* </summary>
*/
function getMoversHeaderTemplate(type) {
var item = {};
item.firstClass = "smallGlyph arrowPaddingBottom";
if (type === "gainers") {
item.firstClass += " financeGlyphUp";
item.secondClass = "hide";
item.headerText = utils.getResourceString("Gainers");
}
else if (type === "losers") {
item.firstClass += " financeGlyphDn";
item.secondClass = "hide";
item.headerText = utils.getResourceString("Losers");
}
else if (type === "active") {
item.firstClass += " financeGlyphUp";
item.secondClass = "smallGlyph arrowPaddingBottom financeGlyphDn";
item.headerText = utils.getResourceString("Active");
}
return item;
}
/* Process common data to Stocks ETF and Funds */
function addAdditionalItemTemplateData(data) {
data.V = data.V ? data.V : null;
data.instrumentVolumes = {
vol: data.V ? utils.formatVolume(data.V) : "",
avgVol: data.avgV ? utils.formatVolume(data.avgV) + " " + utils.getResourceString("AVG_STR") : ""
};
data.Pe < 0 && (data.Pe = null);
data.FpEPS < 0 && (data.FpEPS = null);
data.instrumentPE = utils.formatNumber(data.Pe, "P/E", false, "");
data.instrumentFPE = utils.formatNumber(data.FpEPS, "P/E", false, "");
data.instrumentYield = FinanceAppJS.Utils.Formatters.numberFormatter(data.Dy, null, false, "");
data.instrumentMarketcap = utils.formatNumber(data.Mc, "VOLUME", false, "");
data.instrumentEPS = FinanceAppJS.Utils.Formatters.numberFormatter(data.Eps, null, false, "");
data.instrumentPricebook = FinanceAppJS.Utils.Formatters.numberFormatter(data.PtB, null, false, "");
data.instrumentRPMonth = FinanceAppJS.Utils.Formatters.numberFormatter(data.Rm, null, false, "");
data.instrumentRPYear = FinanceAppJS.Utils.Formatters.numberFormatter(data.Ry, null, false, "");
data.instrumentBeta = FinanceAppJS.Utils.Formatters.numberFormatter(data.Beta, null, false, "");
}
/**
* <summary>
*   This function is invoked for every stock item object and transfors the object into format that comply with the template
* </summary>
* <returns>
*   Transformed Object
* </returns>
*
**/
function getStockItemTemplateData(data, isMovers, isActiveMover, isSnappedMode) {
var item = {};
var status;
var chClass = "chPri";
var chpClass = "chPer";
data.statusIndicator = "largeGlyph";
if (data.Ch > 0) {
status = "up";
data.statusIndicator += " financeGlyphUp";
}
else if (data.Ch < 0) {
status = "dn";
data.statusIndicator += " financeGlyphDn";
}
else if(utils.isNonEmptyNumber(data.Ch)) {
data.statusIndicator = "mediumGlyph financeGlyphEq";
}
if (data.Lp > 9999) {
data.lpClass = "lpSmall";
}
else {
data.lpClass = "lp";
}
data.chpClass = status + " " + chpClass;
data.instrumentDisplaySymbol = utils.getAutoSuggestDisplaySymbol(data, data.type);
data.instrumentExchangeName = data.E2 && FinanceAppJS.Globalization.isLocalExchange(data.E2) ? null : ((data.E2 && utils.getResourceString("Ex_" + data.E2)) || "");
data.ExSn = data.ExSn ? data.ExSn : "";
if (utils.isOTCInstrument(data.E2) && data.ExSn) {
data.instrumentExchangeName = data.ExSn;
}
data.instrumentFullName = utils.getAutoSuggestDisplayName(data);
data.instrumentChangePercent = FinanceAppJS.Utils.Formatters.percentFormatter(data.Chp, null, true);
data.instrumentListIndicator = "indicator " + FinanceAppJS.ConverterFunc.ValueChangeIndicatorCSSClass(data.Ch);
data.instrumentSortLp = data.Lp;
data.instrumentSortChp = data.Chp;
data.instrumentSortCh = data.Ch;
var market = Platform.Utilities.Globalization.getCurrentMarket() || "";
var isCurrencyDecimalIgnored = FinanceAppJS.Utils.Formatters.isCurrencyDecimalIgnored(market, data.E2);
var precision;
if (isCurrencyDecimalIgnored) {
precision = 0;
}
else if (data.Lp < 1) {
precision = 4;
}
else {
precision = 2;
}
if (!isActiveMover) {
data.instrumentChangeOrVolume = FinanceAppJS.Utils.Formatters.numberFormatter(data.Ch, precision, true);
data.chOrVClass = status + " " + chClass;
}
else {
data.instrumentChangeOrVolume = utils.formatNumber(data.V, "VOLUME");
data.chOrVClass = "fontMB";
}
data.instrumentLastPrice = FinanceAppJS.Utils.Formatters.numberFormatter(data.Lp, precision, false);
data.instrumentYearValues = {
value: data.Lp,
min: FinanceAppJS.Utils.Formatters.numberFormatter(data.Yl, precision, false),
max: FinanceAppJS.Utils.Formatters.numberFormatter(data.Yh, precision, false)
};
data.instrumentDayValues = {
value: data.Lp,
min: FinanceAppJS.Utils.Formatters.numberFormatter(data.Dl, precision, false),
max: FinanceAppJS.Utils.Formatters.numberFormatter(data.Dh, precision, false)
};
data.instrumentYearRangeClass = data.Lp && data.Yl && data.Yh ? "visible" : "hidden";
data.instrumentDayRangeClass = data.Lp && data.Dl && data.Dh ? "visible" : "hidden";
addAdditionalItemTemplateData(data);
item = data;
//Add Aria-label narrator text for watchlist tile.
var exchName;
if (!item.instrumentExchangeName || item.instrumentExchangeName === "") {
exchName = "local"; //Mention local as exchange name.
}
else {
exchName = item.instrumentExchangeName;
}
/* update narrator Text*/
var naText;
naText = utils.getResourceString("TileContent_Al").format(item.instrumentFullName, item.instrumentLastPrice, exchName); //Add Company, Price and exchange
// update change value.
if (item.instrumentChangeOrVolume) {
naText = naText + ". " + utils.getResourceString("Changetext_Al") + item.instrumentChangeOrVolume;
}
// Update change percent
if (item.instrumentChangePercent) {
naText = naText + ". " + utils.getResourceString("PercentText_Al") + item.instrumentChangePercent;
}
if (naText) {
item.naratorText = naText;
}
isMovers === true ? item.instrumentTypeClass = 'watchListTile moversTile' : item.instrumentTypeClass = 'watchListTile';
item.instrumentNameFieldClass = "nameField msGridRowColumn";
return item;
}
function getFundItemTemplateData(data, isSnappedMode) {
var item = {};
var status;
var chClass = "chPri";
var chpClass = "chPer";
var naratorText = "";
data.statusIndicator = "largeGlyph";
if (data.Odr > 0) {
status = "up";
data.statusIndicator += " financeGlyphUp";
}
else if (data.Odr < 0) {
status = "dn";
data.statusIndicator += " financeGlyphDn";
}
else if(utils.isNonEmptyNumber(data.Odr)) {
data.statusIndicator = "mediumGlyph financeGlyphEq";
}
if (data.Nav > 9999) {
data.lpClass = "lpSmall";
}
else {
data.lpClass = "lp";
}
data.chOrVClass = chClass + " " + status;
data.chpClass = chpClass + " " + status;
data.instrumentDisplaySymbol = utils.getAutoSuggestDisplaySymbol(data, FinanceAppJS.Constants.IDs.Fund);
data.instrumentFullName = utils.getAutoSuggestDisplayName(data, FinanceAppJS.Constants.IDs.Fund);
data.instrumentExchangeName = null;
data.instrumentLastPrice = FinanceAppJS.Utils.Formatters.numberFormatter(data.Nav, null, false);
data.instrumentChangePercent = FinanceAppJS.Utils.Formatters.percentFormatter(data.Odr, null, true);
data.instrumentChangeOrVolume = FinanceAppJS.Utils.Formatters.numberFormatter(data.ChOdr, null, true);
data.instrumentListIndicator = "indicator " + FinanceAppJS.ConverterFunc.ValueChangeIndicatorCSSClass(data.ChOdr);
data.instrumentSortLp = data.Nav;
data.instrumentSortChp = data.Odr;
data.instrumentSortCh = data.ChOdr;
// Funds do not have day/year ranges
data.instrumentDayRangeClass = "hidden";
data.instrumentYearRangeClass = "hidden";
data.instrumentDayValues = data.instrumentYearValues = {
value: 0,
min: 0,
max: 0
};
data.ExSn = data.ExSn ? data.ExSn : "";
addAdditionalItemTemplateData(data);
item = data;
//Add Aria-label narrator text for watchlist tile.
/* update narrator Text*/
var naText;
naText = item.instrumentFullName;
naText = utils.getResourceString("TileContent_Al").format(item.instrumentLastPrice, item.instrumentDisplaySymbol); //Add Price and symbol
// update change value.
if (item.instrumentChangeOrVolume) {
naText = naText + ". " + utils.getResourceString("Changetext_Al") + item.instrumentChangeOrVolume;
}
// Update change percent
if (item.instrumentChangePercent) {
naText = naText + ". " + utils.getResourceString("PercentText_Al") + item.instrumentChangePercent;
}
if (naText) {
item.naratorText = naText;
}
item.instrumentTypeClass = "fundTile";
item.instrumentNameFieldClass = "nameField msGridRowColumn";
return item;
}
function getIndexItemTemplateData(data, isSnappedMode) {
var item = {};
var status;
var chClass = "chPri";
var chpClass = "chPer";
var naratorText = "";
data.statusIndicator = "largeGlyph";
if (data.Ch > 0) {
status = "up";
data.statusIndicator += " financeGlyphUp";
}
else if (data.Ch < 0) {
status = "dn";
data.statusIndicator += " financeGlyphDn";
}
else if(utils.isNonEmptyNumber(data.Ch)) {
data.statusIndicator = "mediumGlyph financeGlyphEq";
}
if (data.Lp > 9999) {
data.lpClass = "lpSmall";
}
else {
data.lpClass = "lp";
}
data.chpClass = status + " " + chpClass;
data.instrumentChangeOrVolume = FinanceAppJS.Utils.Formatters.numberFormatter(data.Ch, null, true);
data.chOrVClass = status + " " + chClass;
data.instrumentDisplaySymbol = utils.getAutoSuggestDisplaySymbol(data, data.type);
data.instrumentExchangeName = null;
data.instrumentFullName = utils.getAutoSuggestDisplayName(data);
data.instrumentLastPrice = FinanceAppJS.Utils.Formatters.numberFormatter(data.Lp, null, false);
data.instrumentChangePercent = FinanceAppJS.Utils.Formatters.percentFormatter(data.Chp, null, true);
data.instrumentListIndicator = "indicator " + FinanceAppJS.ConverterFunc.ValueChangeIndicatorCSSClass(data.Ch);
data.instrumentSortLp = data.Lp;
data.instrumentSortChp = data.Chp;
data.instrumentSortCh = data.Ch;
data.ExSn = data.ExSn ? data.ExSn : "";
data.instrumentYearValues = {
value: data.Lp,
min: FinanceAppJS.Utils.Formatters.numberFormatter(data.Yl, null, false),
max: FinanceAppJS.Utils.Formatters.numberFormatter(data.Yh, null, false)
};
data.instrumentDayValues = {
value: data.Lp,
min: FinanceAppJS.Utils.Formatters.numberFormatter(data.Dl, null, false),
max: FinanceAppJS.Utils.Formatters.numberFormatter(data.Dh, null, false)
};
data.instrumentYearRangeClass = data.Lp && data.Yl && data.Yh ? "visible" : "hidden";
data.instrumentDayRangeClass = data.Lp && data.Dl && data.Dh ? "visible" : "hidden";
addAdditionalItemTemplateData(data);
item = data;
/* update narrator Text*/
var naText;
naText = utils.getResourceString("IndexTileNarrator_Al").format(item.instrumentFullName, item.instrumentLastPrice); //Add Index name and price
// update change value.
if (item.instrumentChangeOrVolume) {
naText = naText + ". " + utils.getResourceString("Changetext_Al") + item.instrumentChangeOrVolume;
}
// Update change percent
if (item.instrumentChangePercent) {
naText = naText + ". " + utils.getResourceString("PercentText_Al") + item.instrumentChangePercent;
}
if (naText) {
item.naratorText = naText;
}      
item.instrumentTypeClass = 'watchListTile indexTile';
item.instrumentNameFieldClass = "nameField msGridRowColumn indexName";
return item;
}
function getWatchListTemplates(shouldIncludeAddTileTemplate) {
return WinJS.UI.Fragments.render("/shared/html/WatchListItemTemplate.html").then(templateFragmentSelector_S.bind(null, shouldIncludeAddTileTemplate));
}
function templateFragmentSelector_S(shouldIncludeAddTileTemplate, templateFragment) {
return WinJS.UI.processAll(templateFragment).then(templateSelector_S.bind(null, shouldIncludeAddTileTemplate, templateFragment));
}
function templateSelector_S(shouldIncludeAddTileTemplate, templateFragment) {
var templates = {};
templates['stockItemTemplate'] = templateFragment.querySelector('#stockItemTemplate');
templates['listModeTemplate'] = templateFragment.querySelector('#listModeTemplate');
templates['listModeHeaderTemplate'] = templateFragment.querySelector('#listModeHeaderTemplate');
if (shouldIncludeAddTileTemplate !== false) {
templates['addTileTemplate'] = templateFragment.querySelector('#addTileTemplate');
templates['addTileTemplateWide'] = templateFragment.querySelector('#addTileTemplateWide');
}
return templates;
}
function watchListTemplateSelector(itemData, templates, mode, cols, useRenderer) {
var itemTemplate = null;
var idConstants = FinanceAppJS.Constants.IDs;
var data = itemData.tileData;
var isSupportedType = data.type === idConstants.Stock || data.type === idConstants.Fund ||
data.type === idConstants.Etf || data.type === idConstants.Index;
if (mode === "list") {
if (isSupportedType) {
itemTemplate = templates.listModeTemplate;
var container = document.createElement("div");
container.id = itemData.itemKey;
itemTemplate.winControl.render(itemData, container);
if (cols && cols.length > 0) {
cols.forEach(function (item) {
var elem = container.querySelector("." + item.id);
if (item.showorhide === "hide") {
elem.style.display = "none";
} else {
elem.style.width = item.innerWidth + "px";
}
});
}
return container;
} else {
itemTemplate = templates.addTileTemplateWide;
return itemTemplate;
}
} else {
// Select either add template OR stockTemplate
if (isSupportedType) {
if (useRenderer) {
var container = watchlistTileRenderer(itemData);
container.id = itemData.itemKey;
return container;
} else {
itemTemplate = templates.stockItemTemplate;
var container = document.createElement("div");
container.id = itemData.itemKey;
itemTemplate.winControl.render(itemData, container);
return container;
}
}
else {
itemTemplate = templates.addTileTemplate;
return itemTemplate;
}
}
}
function watchlistTileRenderer(item) {
var base = PlatformJS.Utilities.createElements(
{
tag: "div", attrs: { tabindex: "-1" }, children: [
{ tag: "div", attrs: { className: "sym", tabindex: "-1", ariaDisabled: "true" } },
{ tag: "div", attrs: { className: "info", tabindex: "-1" }, children: [
{ tag: "div", attrs: { className: "cmp" }},
{ tag: "div", attrs: { className: "ex" }}
]},
{ tag: "div", attrs: { className: "indx", tabindex: "-1" }, children: [
{ tag: "div" },
{ tag: "div", attrs: { className: "lp" }}
]},
{ tag: "div", attrs: { dir:"ltr", className: "ch" }, children: [
{ tag: "div", attrs: { tabindex: "-1" } },
{ tag: "div", attrs: { tabindex: "-1" } }
]}
]
}
);
var narrator = PlatformJS.Utilities.createElements({ tag: "div", attrs: { className: "naratorText hide" } });
var container = document.createElement("div");
container.appendChild(narrator);
container.appendChild(base);
var sym = base.children[0];
var info = base.children[1];
var indx = base.children[2];
var ch = base.children[3];
item.tileData.bind("naratorText", function (newValue) {
narrator.textContent = newValue;
});
item.tileData.bind("instrumentTypeClass", function (newValue) {
base.className = newValue;
});
item.tileData.bind("instrumentDisplaySymbol", function (newValue) {
sym.textContent = newValue;
});
item.tileData.bind("instrumentFullName", function (newValue) {
info.children[0].textContent = newValue;
});
item.tileData.bind("instrumentExchangeName", function (newValue) {
info.children[1].textContent = newValue;
});
item.tileData.bind("statusIndicator", function (newValue) {
indx.children[0].className = newValue;
});
item.tileData.bind("instrumentLastPrice", function (newValue) {
indx.children[1].textContent = newValue;
});
item.tileData.bind("lpClass", function (newValue) {
indx.children[1].className = newValue;
});
item.tileData.bind("instrumentChangeOrVolume", function (newValue) {
ch.children[0].textContent = newValue;
});
item.tileData.bind("chOrVClass", function (newValue) {
ch.children[0].className = newValue;
});
item.tileData.bind("instrumentChangePercent", function (newValue) {
ch.children[1].textContent = newValue;
});
item.tileData.bind("chpClass", function (newValue) {
ch.children[1].className = newValue;
});
return container;
}
function processWatchListData(watchList, isAddTileRequiredIfEmpty, isSnapped) {
//  Return stocks/funds in the same order as defined by the roaming
var myStocksData = [];
if (!isAddTileRequiredIfEmpty) {
if (watchList.length === 0) {
return myStocksData;
}
}
var item;
for (var i = 0; i < watchList.length; i++) {
if (!watchList[i]) {
continue;
}
item = {};
if (watchList[i].type === FinanceAppJS.Constants.IDs.Stock ||
watchList[i].type === FinanceAppJS.Constants.IDs.Etf) {
item.tileData = WatchListDataProvider.getStockItemTemplateData(watchList[i], false, false, isSnapped);
}
else if (watchList[i].type === FinanceAppJS.Constants.IDs.Fund) {
item.tileData = WatchListDataProvider.getFundItemTemplateData(watchList[i], isSnapped);
}
else if (watchList[i].type === FinanceAppJS.Constants.IDs.Index) {
item.tileData = WatchListDataProvider.getIndexItemTemplateData(watchList[i], isSnapped);
}
myStocksData.push(item);
}
return myStocksData;
}
function getWatchlistCustomCacheKey() {
return "watchlistData_" + FinanceAppJS.Globalization.getMarket().toLowerCase();
}
function getWatchListDragElement ( sourceElement, sourceData) {
var elementPrototype = sourceElement.parentElement;
var element = document.createElement("div");
WinJS.Utilities.addClass(element, "ListView2SimpleDraggedElement");
element.style.width = elementPrototype.style.width;
element.style.height = elementPrototype.style.height;
var titleElement = document.createElement("div");
WinJS.Utilities.addClass(titleElement, "titleDiv");
WinJS.Utilities.addClass(titleElement, "watchListTitle");
titleElement.textContent = sourceData.tileData.instrumentDisplaySymbol;
element.appendChild(titleElement);
return element;
}
WinJS.Namespace.define("WatchListDataProvider", {
getStockItemTemplateData: getStockItemTemplateData,
getMoversHeaderTemplate: getMoversHeaderTemplate,
getFundItemTemplateData: getFundItemTemplateData,
getIndexItemTemplateData: getIndexItemTemplateData,
watchListTemplates: getWatchListTemplates,
watchListTemplateSelector: watchListTemplateSelector,
processWatchListData: processWatchListData,
getWatchListDragElement: getWatchListDragElement,
getWatchlistCustomCacheKey: getWatchlistCustomCacheKey
});
})();
